
//  ------------------------------------------------------------------
//  The Goldware Utilities. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gbuild.cpp,v 1.2 1998/11/22 15:32:26 odinn Exp $
//  ------------------------------------------------------------------
//  Utility to build make-related files.
//  ------------------------------------------------------------------


#include <iostream.h>
#include <fstream.h>
#include <gstrall.h>


//  ------------------------------------------------------------------

char* strtrim(char* p) {

  _strtrim_wht(p);
  return p;
}

char* strskip_wht(char* p) {
  _strskip_wht(p);
  return p;
}

char* strskip_txt(char* p) {
  _strskip_txt(p);
  return p;
}

const char* striinc(const char* str1, const char* str2) {

  int max = strlen(str1);
  for(const char* p=str2; *p; p++)
    if(!strnicmp(str1,p,max))
      return p;
  return NULL;
}


//  ------------------------------------------------------------------
//  args: <listfile> <outputfile> <path> <type:inc/lib/pack> <format> <criteria>
//        1          2            3      4                   5        6

int main(int argc, char** argv) {

  if(argc >= 6) {

    const int type_inc = 1;
    const int type_lib = 2;
    const int type_link = 3;
    const int type_pack = 4;

    const int format_borland = 1;
    const int format_watcom = 2;
    const int format_gnu = 3;
    const int format_dos = 4;

    const char* listfile = argv[1];
    const char* outputfile = argv[2];
    const char* path = argv[3];
    const char* types = argv[4];
    const char* formats = argv[5];
    int data_arg = 6;

    int type = 0;
    if(strieql(types, "inc"))
      type = type_inc;
    else if(strieql(types, "lib"))
      type = type_lib;
    else if(strieql(types, "link"))
      type = type_link;
    else if(strieql(types, "pack"))
      type = type_pack;

    int format = 0;
    if(strieql(formats, "borland"))
      format = format_borland;
    else if(strieql(formats, "watcom"))
      format = format_watcom;
    else if(strieql(formats, "gnu") or strieql(formats, "unix"))
      format = format_gnu;
    else if(strieql(formats, "dos"))
      format = format_dos;

    ifstream ifp;
    ofstream ofp;

    ifp.open(listfile);
    if(ifp) {

      ofp.open(outputfile);
      if(ofp) {

        int count = 0;

        if(type == type_inc)
          ofp << "objs = " << (format == format_watcom ? '&' : '\\') << endl;

        if((type == type_link) and (format == format_watcom))
          ofp << "path " << path << endl;

        char buf[256];
        while(!ifp.eof()) {
          ifp.getline(buf, sizeof(buf));
          strtrim(buf);
          if(isalpha(*buf)) {
            char* ptr = buf;
            const char* filename = buf;
            ptr = strskip_txt(ptr);
            *ptr++ = NUL;
            ptr = strskip_wht(ptr);
            const char* extension = ptr;
            ptr = strskip_txt(ptr);
            *ptr++ = NUL;
            if(*extension == '.')
              extension++;
            ptr = strskip_wht(ptr);
            const char* data = ptr;

            bool okay = true;
            bool borland_dos16 = false;
            for(int n=data_arg; n<argc; n++) {
              const char* argp = argv[n];
              if(not striinc(argp, data)) {
                okay = false;
                break;
              }
              if(strieql(argp, "bcd"))
                borland_dos16 = true;
            }

            if(okay) {
              switch(type) {
                case type_inc:
                  ofp
                    << (format == format_gnu ? "\t" : "  ")
                    << path
                    << (format == format_gnu ? '/' : '\\')
                    << filename
                    << '.'
                    << (format == format_gnu ? "o" : "obj")
                    << ' '
                    << (format == format_watcom ? '&' : '\\')
                    << endl;
                  break;
                case type_lib:
                  ofp
                    << "-+"
                    << path
                    << '\\'
                    << filename
                    << ".obj"
                    << (format == format_borland ? " &" : "")
                    << endl;
                  break;
                case type_link:
                  if(format == format_borland) {
                    ofp
                      << (count ? "+\n" : "")
                      << (borland_dos16 ? (striinc("ovl", data) ? "/o+ " : "/o- ") : "")
                      << path
                      << '\\'
                      << filename;
                    count++;
                  }
                  else if(format == format_watcom) {
                    ofp << "file " << filename << endl;
                  }
                  break;
                case type_pack:
                  ofp
                    << path
                    << (*path ? (format == format_dos ? "\\" : "/") : "")
                    << filename
                    << (*extension ? "." : "")
                    << extension
                    << endl;
                  break;
              }
            }
          }
        }

        if(type == type_link and format == format_borland)
          ofp << endl;

        ofp.close();
      }
      ifp.close();
    }

    return 0;
  }

  cout << "Parameters: <inputfile> <outputfile> <path> <type> <format> <criteria>" << endl;

  return -1;
}


//  ------------------------------------------------------------------

