
//  ------------------------------------------------------------------
//  The Goldware Utilities. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmanual.cpp,v 1.1.1.1 1998/11/21 19:38:01 odinn Exp $
//  ------------------------------------------------------------------
//  Manual Compiler.
//  ------------------------------------------------------------------


//#pragma option -Od


//  ------------------------------------------------------------------

#include <ctype.h>
#include <stdlib.h>
#include <gstrall.h>
#include <gstroman.h>
#include <gfile.h>


//  ------------------------------------------------------------------

class gmanual {

public:

  Path  manualfile;
  Path  tocfile;
  Path  tmpfile;
  gfile tocfp;
  gfile tmpfp;
  int   pagelength;
  int   pagewidth;
  int   leftmargin;
  int   rightmargin;
  int   linenumber;
  char  pagenumber[10];
  char  chapternumber[10];
  int   chapterfirst;
  char  chapter[137];
  char  heading[10][137];
  int   headinglines;
  char  header[5][137];
  int   headerlines;
  char  footer[5][137];
  int   footerlines;
  int   intoc;
  char  tocchapter[137];
  char  tocline[137];
  int   tocindent;
  char  tocpagenumber[10];
  int   toclinenumber;

  gmanual();
  ~gmanual();

  void run(char* inputfile);

  char* translate(char* buf, char* text);

  void end_page(int end=false);
  void end_tocpage(int end=false);

  void preprocess(char* inputfile);
};


//  ------------------------------------------------------------------

gmanual::gmanual() {

  *manualfile = NUL;
  *tocfile = NUL;
  *tmpfile = NUL;
  pagelength = 0;
  pagewidth = 0;
  leftmargin = 0;
  rightmargin = 0;
  linenumber = 0;
  *pagenumber = NUL;
  *chapternumber = NUL;
  chapterfirst = true;
  headinglines = 0;
  headerlines = 0;
  footerlines = 0;
  *tocline = NUL;
  tocindent = 0;
  *tocchapter = NUL;
  *tocpagenumber = NUL;
  toclinenumber = 0;
  intoc = false;
}


//  ------------------------------------------------------------------

gmanual::~gmanual() {

}


//  ------------------------------------------------------------------

void gmanual::run(char* name) {

  tocfp.fopen("toc.tmp", "wt");
  tocfp.setvbuf(NULL, _IOFBF, 16384);
  tmpfp.fopen("pre.tmp", "wt");
  tmpfp.setvbuf(NULL, _IOFBF, 16384);

  preprocess(name);

  printf("Writing %s ...\n", manualfile);

  gfile fp;
  fp.fopen(manualfile, "wt");
  if(fp.isopen()) {
    fp.setvbuf(NULL, _IOFBF, 16384);
    tocfp.fclose();
    tocfp.fopen("toc.tmp", "rt");
    tocfp.setvbuf(NULL, _IOFBF, 16384);
    tmpfp.fclose();
    tmpfp.fopen("pre.tmp", "rt");
    tmpfp.setvbuf(NULL, _IOFBF, 16384);
    char buf[512];
    int outpage = 0;
    while(tmpfp.fgets(buf, sizeof(buf))) {
      if(strnieql(buf, "#toc", 4)) {
        while(tocfp.fgets(buf, sizeof(buf))) {
          fp.printf("%s", buf);
          if(strchr(buf, '\xC'))
            printf("Page %u\r", ++outpage);
        }
      }
      else {
        fp.printf("%s", buf);
        if(strchr(buf, '\xC'))
          printf("Page %u\r", ++outpage);
      }
    }
    fp.fclose();
  }

  tocfp.fclose();
  tmpfp.fclose();

  remove("toc.tmp");
  remove("pre.tmp");

  printf("\nDone!\n");
}


//  ------------------------------------------------------------------

char* gmanual::translate(char* buf, char* text) {

  char ibuf[512];
  strcpy(ibuf, text);
  int linewidth = pagewidth - leftmargin - rightmargin;
  memset(buf, ' ', pagewidth-rightmargin);
  buf[pagewidth-rightmargin] = NUL;
  char* optr = buf + leftmargin;
  strischg(ibuf, "<page>", intoc ? tocpagenumber : pagenumber);
  strischg(ibuf, "<chapter>", intoc ? tocchapter : chapter);
  char* iptr = ibuf;
  while(*iptr) {
    if(*iptr == '<') {
      if(strnieql(iptr, "<center>", 8)) {
        iptr += 8;
        int len = strlen(iptr);
        optr = buf + leftmargin + ((linewidth/2)-(len/2));
      }
      else if(strnieql(iptr, "<right>", 7)) {
        iptr += 7;
        int len = strlen(iptr);
        optr = buf + leftmargin + linewidth - len;
      }
    }
    else
      *optr++ = *iptr++;
  }

  return buf;
}


//  ------------------------------------------------------------------

void gmanual::end_page(int end) {

  if(linenumber) {
    while(linenumber <= (pagelength-footerlines)) {
      tmpfp.printf("\n");
      linenumber++;
    }
    char buf[512];
    for(int n=0; n<footerlines; n++)
      tmpfp.printf("%s\n", translate(buf, footer[n]));
    tmpfp.printf("");
    printf("Page %s\r", pagenumber);
    if(NOT end)
      for(int n=0; n<headerlines; n++)
        tmpfp.printf("%s\n", translate(buf, header[n]));
    if(*pagenumber) {
      if(isdigit(*pagenumber))
        sprintf(pagenumber, "%u", atoi(pagenumber)+1);
      else if(islower(*pagenumber))
        sprintf(pagenumber, "%s", i_to_roman(buf, roman_to_i(pagenumber)+1));
      else
        (*pagenumber)++;
    }
  }
}


//  ------------------------------------------------------------------

void gmanual::end_tocpage(int end) {

  if(toclinenumber) {
    intoc = true;
    while(toclinenumber <= (pagelength-footerlines)) {
      tocfp.printf("\n");
      toclinenumber++;
    }
    char buf[512];
    for(int n=0; n<footerlines; n++)
      tocfp.printf("%s\n", translate(buf, footer[n]));
    tocfp.printf("");
    if(NOT end)
      for(int n=0; n<headerlines; n++)
        tocfp.printf("%s\n", translate(buf, header[n]));
    if(*tocpagenumber) {
      if(isdigit(*tocpagenumber))
        sprintf(tocpagenumber, "%u", atoi(tocpagenumber)+1);
      else if(islower(*tocpagenumber))
        sprintf(tocpagenumber, "%s", i_to_roman(buf, roman_to_i(tocpagenumber)+1));
      else
        (*tocpagenumber)++;
    }
    intoc = false;
  }
}


//  ------------------------------------------------------------------

void gmanual::preprocess(char* inputfile) {

  gfile fp;

  fp.fopen(inputfile, "rt");
  if(fp.isopen()) {
    fp.setvbuf(NULL, _IOFBF, 16384);

    printf("Processing %s ...\n", inputfile);

    char buf[512];
    char tmpbuf[512];
    int inputline = 0;

    while(fp.fgets(buf, sizeof(buf))) {

      char* nl = strchr(buf, '\n');
      if(nl)
        *nl = NUL;

      inputline++;

      if(*buf == '#') {
        char* key = buf + 1;
        char* prm = strchr(buf, ' ');
        if(prm)
          *prm++ = NUL;
        if(strieql(key, "manualfile")) {
          strcpy(manualfile, prm);
        }
        else if(strieql(key, "pagelength")) {
          pagelength = atoi(prm) - 2;
        }
        else if(strieql(key, "pagewidth")) {
          pagewidth = atoi(prm);
        }
        else if(strieql(key, "leftmargin")) {
          leftmargin = atoi(prm);
        }
        else if(strieql(key, "rightmargin")) {
          rightmargin = atoi(prm);
        }
        else if(strieql(key, "page")) {
          end_page();
          linenumber = 0;
        }
        else if(strieql(key, "header")) {
          strcpy(header[headerlines++], prm ? prm : "");
        }
        else if(strieql(key, "noheader")) {
          headerlines = 0;
        }
        else if(strieql(key, "heading")) {
          strcpy(heading[headinglines++], prm ? prm : "");
        }
        else if(strieql(key, "noheading")) {
          headinglines = 0;
        }
        else if(strieql(key, "footer")) {
          strcpy(footer[footerlines++], prm ? prm : "");
        }
        else if(strieql(key, "nofooter")) {
          footerlines = 0;
        }
        else if(strieql(key, "pagenumber")) {
          strcpy(pagenumber, prm);
        }
        else if(strieql(key, "tocpagenumber")) {
          strcpy(tocpagenumber, prm);
        }
        else if(strieql(key, "chapter")) {
          if(chapterfirst)
            chapterfirst = false;
          else {
            if(*chapternumber) {
              if(isdigit(*chapternumber))
                sprintf(chapternumber, "%u", atoi(chapternumber)+1);
              else if(islower(*chapternumber))
                sprintf(chapternumber, "%s", i_to_roman(buf, roman_to_i(chapternumber)+1));
              else
                (*chapternumber)++;
            }
          }
          strcpy(intoc ? tocchapter : chapter, prm);
          for(int n=0; n<headinglines; n++) {
            if(intoc) {
              tocfp.printf("%s\n", translate(tmpbuf, heading[n]));
              toclinenumber++;
            }
            else {
              tmpfp.printf("%s\n", translate(tmpbuf, heading[n]));
              linenumber++;
            }
          }
          if(NOT intoc) {
            int pagenumberlen = strlen(pagenumber);
            int toclen = pagewidth - leftmargin - rightmargin - tocindent - pagenumberlen - 2;
            int chapterlen = strlen(chapter);
            int chapterpart = MinV(chapterlen, toclen);
            int toclinepart = toclen - chapterpart;
            if(toclinepart < 0)
              toclinepart = 0;
            sprintf(tmpbuf, "%s.", chapternumber);
            tocfp.printf("\n%*s" "%-*s" "%*.*s" " %*.*s " "%s\n",
              leftmargin, "",
              tocindent, tmpbuf,
              chapterpart, chapterpart, chapter,
              toclinepart, toclinepart, tocline,
              pagenumber
            );
            toclinenumber += 2;
            if(toclinenumber > (pagelength-footerlines)) {
              end_tocpage();
              toclinenumber = 0;
            }
          }
          if(*chapternumber)
            printf("%s. %s\n", chapternumber, chapter);
        }
        else if(strieql(key, "toc")) {
          tmpfp.printf("\n#toc\n");
        }
        else if(strieql(key, "tocbegin")) {
          intoc = true;
        }
        else if(strieql(key, "tocend")) {
          intoc = false;
          for(int n=0; n<headerlines; n++)
            tmpfp.printf("%s\n", translate(tmpbuf, header[n]));
        }
        else if(strieql(key, "end")) {
          end_page(true);
          end_tocpage(true);
        }
        else if(strieql(key, "tocindent")) {
          tocindent = atoi(prm);
        }
        else if(strieql(key, "tocline")) {
          strcpy(tocline, prm);
        }
        else if(strieql(key, "chapternumber")) {
          strcpy(chapternumber, prm);
          chapterfirst = true;
        }
        else if(strieql(key, "include")) {
          preprocess(prm);
        }
        else if(strieql(key, "rem")) {
          // This line is a REMark
        }
        else if(strieql(key, "seealso")) {
          // not implemented yet.
        }
        else if(isalpha(*key)) {
          printf("%s:%u Unknown keyword: \"%s\"\n", inputfile, inputline, key);
        }
      }
      else {
        char* begin = strchr(buf, '\x1');
        if(begin) {
          *begin = NUL;
          char* end = strchr(begin+1, '\x1');
          if(end) {
            *end = NUL;
            int pagenumberlen = strlen(pagenumber);
            int toclen = pagewidth - leftmargin - rightmargin - tocindent - pagenumberlen - 2;
            int chapterlen = strlen(begin+1);
            int chapterpart = MinV(chapterlen, toclen);
            int toclinepart = toclen - chapterpart;
            if(toclinepart < 0)
              toclinepart = 0;
            tocfp.printf("%*s" "%*s" "%*.*s" " %*.*s " "%s\n",
              leftmargin, "",
              tocindent, "",
              chapterpart, chapterpart, begin+1,
              toclinepart, toclinepart, tocline,
              pagenumber
            );
            toclinenumber++;
            if(toclinenumber > (pagelength-footerlines)) {
              end_tocpage();
              toclinenumber = 0;
            }
            tmpfp.printf("%*s%s%s%s\n", leftmargin, "", buf, begin+1, end+1);
            linenumber++;
          }
          else {
            printf("\a%s:%u Begin mark without end mark\n", inputfile, inputline);
          }
        }
        else {
          tmpfp.printf("%*s%s\n", leftmargin, "", buf);
          linenumber++;
        }

        if(linenumber > (pagelength-footerlines)) {
          end_page();
          linenumber = 0;
        }
      }
    }

    fp.fclose();
  }
}


//  ------------------------------------------------------------------

int main(int argc, char** argv) {

  printf(
  // 12345678901234567890123456789012345678901234567890123456789012345678901234567890
    "\n"
    "Goldware Manual Compiler 1.02.             Copyright (C) 1995-97 Odinn Sorensen\n"
    "-------------------------------------------------------------------------------\n"
    "\n"
  );

  if(argc < 2) {
    printf(
      "Syntax: GMANUAL <inputfile> [inputfile] [..]\n"
      "\n"
      "Makes one manual per inputfile.\n"
    );
  }

  for(int n=1; n<argc; n++) {
    gmanual manual;
    manual.run(argv[n]);
  }

  return 0;
}


//  ------------------------------------------------------------------


