
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gcalst.cpp,v 1.2 1999/01/01 17:59:45 damware Exp $
//  ------------------------------------------------------------------
//  Arealist configuration and other one-time stuff.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include "gmoezyc.h"
#include "gmofido.h"
#include "gmogold.h"
#include "gmohuds.h"
#include "gmojamm.h"
#include "gmosqsh.h"
#include "gmopcbd.h"
#include "gmowcat.h"
#include "gmoxbbs.h"


//  ------------------------------------------------------------------

class SepArea : public gmo_area {

public:

  SepArea() { }
  virtual ~SepArea() { }

  //  ----------------------------------------------------------------
  //  Messagebase member functions
  
  void open() { }
  void close() { }

  void suspend() { }
  void resume() { }

  void lock() { }
  void unlock() { }

  void scan() { }
  void scan_area() { }
  void scan_area_pm() { }

  int load_hdr(gmsg*) { return 0; }
  int load_msg(gmsg*) { return 0; }

  void save_hdr(int, gmsg*) { }
  void save_msg(int, gmsg*) { }

  void del_msg(gmsg*) { }

  void new_msgno(gmsg*) { }
  char* user_lookup(char*) { return NULL; }
  int renumber() { return false; }

  void update_timesread(gmsg*) { }

  Line* make_dump_msg(Line*&, gmsg*) { return NULL; }
};


//  ------------------------------------------------------------------
//  Arealist constructor

AreaList::AreaList() {

  total = 0;
  index = NULL;
  current = 0;
  pointer = NULL;
  *sortspec = NUL;

  strcpy(alistselections[0], "Mark Areas with New Mail");

  for(uint i = 1; i < 16; i++)
    strcpy(alistselections[i], "(unused)");

}


//  ------------------------------------------------------------------
//  Arealist destructor

AreaList::~AreaList() {

  Reset();
}


//  ------------------------------------------------------------------
//  Arealist deallocate and reset data

void AreaList::Reset() {

  ListScan.Reset();
  register Area** i = index;
  while(total) {
    register Area* a = *i++;
    delete a;
    total--;
  }
  throw_release(index);
}


//  ------------------------------------------------------------------
//  Arealist save arealist data

void AreaList::Save(gfile& fp) {

  ListScan.Save(fp);
  fp.fwrite(&total, sizeof(int));
  fp.fwrite(&msgbases, sizeof(uint));
  if(First()) {
    do {
      fp.fwrite(&pointer->area->cfg, sizeof(AreaCfgBase));
      pointer->area->cfg2.Save(fp);
    } while(Next());
  }
}


//  ------------------------------------------------------------------
//  Return a new'ed area of the specified format

Area* AreaList::NewArea(int msgbase) {

  gmo_area* ap = NULL;
  switch(msgbase) {
    case MT_SEPARATOR:  ap = new SepArea;     break;
    case MT_EZYCOM:     ap = new EzycomArea;  break;
    case MT_FTS1:       ap = new FidoArea;    break;
    case MT_OPUS:       ap = new FidoArea;    break;
    case MT_GOLDBASE:   ap = new GoldArea;    break;
    case MT_HUDSON:     ap = new HudsArea;    break;
    case MT_JAM:        ap = new JamArea;     break;
    case MT_PCBOARD:    ap = new PcbArea;     break;
    case MT_SQUISH:     ap = new SquishArea;  break;
    case MT_WILDCAT:    ap = new WCatArea;    break;
    case MT_ADEPTXBBS:  ap = new XbbsArea;    break;
  }
  return new Area(ap);
}


//  ------------------------------------------------------------------
//  Arealist load arealist data

void AreaList::Load(gfile& fp) {

  ListScan.Load(fp);
  fp.fread(&total, sizeof(int));
  fp.fread(&msgbases, sizeof(uint));
  index = (Area**)throw_calloc(total, sizeof(Area*));
  AreaCfgBase cfg;
  for(uint n=0; n<total; n++) {
    fp.fread(&cfg, sizeof(AreaCfgBase));
    index[n] = NewArea(cfg.msgbase);
    throw_new(index[n]);
    memcpy(&index[n]->area->cfg, &cfg, sizeof(AreaCfgBase));
    index[n]->area->cfg2.Load(fp);
  }
}


//  ------------------------------------------------------------------
//  Write lastreads for the next session

void AreaList::WriteGoldLast() {

  word GOLDLAST_VER = CUR_GOLDLAST_VER;
  gfile fp;
  ggoldlast entry;
  Path lst;

  strcpy(lst, AddPath(CFG->goldpath, "goldlast.lst"));

  fp.fopen(lst, "wb", CFG->sharemode);
  if(fp.isopen()) {

    fp.setvbuf(NULL, _IOFBF, 8192);
    fp.fwrite(&GOLDLAST_VER, sizeof(word));
    fp.fwrite(AL.alistselections, sizeof(AL.alistselections));

    for(uint n=0; n<total; n++) {
      Area* ap = index[n];
      if(ap->isscanned and not ap->isseparator()) {

        // Write fixed header
        entry.crcechoid    = strCrc32(ap->echoid(), false);
        entry.lastread     = ap->lastread();
        entry.msgncount    = ap->Msgn.Count();
        entry.unread       = ap->unread;
        entry.marks        = ap->marks;
        entry.flags        = 0;
        if(ap->isscanned)
          entry.flags |= 1;
        if(ap->isvalidchg)
          entry.flags |= 2;
        if(ap->isunreadchg)
          entry.flags |= 4;

        fp.fwrite(&entry, sizeof(entry));

        // Write variable length extensions
        ap->Mark.Save(fp);
        ap->PMrk.Save(fp);
      }
    }

    fp.fclose();
  }
}


//  ------------------------------------------------------------------
//  Read the lastreads from the last session

void AreaList::ReadGoldLast() {

  word GOLDLAST_VER;
  gfile fp;
  ggoldlast entry;
  
  fp.fopen(AddPath(CFG->goldpath, "goldlast.lst"), "rb", CFG->sharemode);
  if(fp.isopen()) {

    fp.setvbuf(NULL, _IOFBF, 8192);
    fp.fread(&GOLDLAST_VER, sizeof(word));

    if(GOLDLAST_VER != CUR_GOLDLAST_VER) {
      fp.close();
      return;
    }

    fp.fread(AL.alistselections, sizeof(AL.alistselections));

    while(fp.fread(&entry, sizeof(entry))) {

      bool found = false;
      
      for(uint n=0; n<total; n++) {
        if(strCrc32(index[n]->echoid(), false) == entry.crcechoid) {
          Area* ap = index[n];

          ap->set_lastread(entry.lastread);
          ap->Msgn.count  = entry.msgncount;
          ap->unread      = entry.unread;
          ap->marks       = entry.marks;
          ap->isscanned   = (entry.flags & 1) ? true : false;
          ap->isvalidchg  = (entry.flags & 2) ? true : false;
          ap->UpdateAreadata();
          ap->isunreadchg = (entry.flags & 4) ? true : false;

          ap->Mark.Load(fp);
          ap->PMrk.Load(fp);
          
          found = true;
          break;
        }
      }
      
      if(not found) {
        // skip stored message marks
        dword dw;
        fp.fread(&dw, sizeof(dword));
        fp.fseek(dw*sizeof(dword), SEEK_CUR);
        fp.fread(&dw, sizeof(dword));
        fp.fseek(dw*sizeof(dword), SEEK_CUR);
      }
    }

    fp.fclose();
  }
}


//  ------------------------------------------------------------------
//  Write all areas to GOLDAREA.INC in AREADEF format

void AreaList::WriteAreaDef(const char* file) {

  uint n;
  int tmp;
  FILE* fp;
  Area* aa;
  Path path;
  char groupid[10], echoid[sizeof(Echo)+2];
  int maxechoid=0, maxdesc=0, maxgroupid=0, maxpath=0, maxaddr=0, maxattr=0;
  char desc[sizeof(Desc)+2], type[6], msgbase[7], addr[40], attr[150], origin[sizeof(Orig)+2];

  fp = fsopen(file, "wt", CFG->sharemode);
  if(fp) {

    for(n=0; n<total; n++) {
      aa = index[n];
      tmp = strlen(aa->echoid());
      if(strchr(aa->echoid(), ' '))
        tmp += 2;
      maxechoid = MaxV(maxechoid, tmp);
      tmp = strlen(aa->desc())+2;
      maxdesc = MaxV(maxdesc, tmp);
      tmp = strlen(aa->path());
      if(strchr(aa->path(), ' '))
        tmp += 2;
      maxpath = MaxV(maxpath, tmp);
      if(aa->groupid() & 0x8000u)
        sprintf(groupid, "#%u", aa->groupid()&0x7FFF);
      else if(isupper(aa->groupid()))
        *groupid = (char)aa->groupid(), groupid[1] = NUL;
      else
        *groupid = '0', groupid[1] = NUL;
      tmp = strlen(groupid);
      maxgroupid = MaxV(maxgroupid, tmp);
      if(memcmp(&aa->aka(), &CFG->aka[0], sizeof(Addr)))
        aa->aka().make_string(addr);
      else
        strcpy(addr, ".");
      tmp = strlen(addr);
      maxaddr = MaxV(maxaddr, tmp);
      tmp = strlen(MakeAttrStr(attr, &aa->attr()));
      maxattr = MaxV(maxattr, tmp+2);
    }

    for(n=0; n<total; n++) {
      aa = index[n];
      sprintf(desc, "\"%s\"", aa->desc());
      if(aa->groupid() & 0x8000u)
        sprintf(groupid, "#%u", aa->groupid()&0x7FFF);
      else if(isupper(aa->groupid()))
        *groupid = (char)aa->groupid(), groupid[1] = NUL;
      else
        *groupid = '0', groupid[1] = NUL;
      if(aa->isemail())
        strcpy(type, "EMail");
      else if(aa->isnewsgroup())
        strcpy(type, "News ");
      else if(aa->isnet())
        strcpy(type, "Net  ");
      else if(aa->isecho())
        strcpy(type, "Echo ");
      else if(aa->islocal())
        strcpy(type, "Local");
      switch(aa->msgbase()) {
        case MT_OPUS:      strcpy(msgbase, "Opus  "); break;
        case MT_FTS1:      strcpy(msgbase, "Fts1  "); break;
        case MT_HUDSON:    strcpy(msgbase, "Hudson"); break;
        case MT_GOLDBASE:  strcpy(msgbase, "Gold  "); break;
        case MT_SQUISH:    strcpy(msgbase, "Squish"); break;
        case MT_EZYCOM:    strcpy(msgbase, "Ezycom"); break;
        case MT_JAM:       strcpy(msgbase, "Jam   "); break;
        case MT_PCBOARD:   strcpy(msgbase, "Pcb   "); break;
        case MT_WILDCAT:   strcpy(msgbase, "WCat  "); break;
        case MT_ADEPTXBBS: strcpy(msgbase, "XBBS  "); break;
      }
      if(strchr(aa->echoid(), ' '))
        sprintf(echoid, "\"%s\"", aa->echoid());
      else
        strcpy(echoid, aa->echoid());
      if(aa->isseparator()) {
        fprintf(fp, "AREASEP %-*s %-*s %*s %s\n",
          maxechoid,  echoid,
          maxdesc,    desc,
          maxgroupid, groupid,
          strtrim(type)
        );
      }
      else {
        if(strchr(aa->path(), ' '))
          sprintf(path, "\"%s\"", aa->path());
        else
          strcpy(path, aa->path());
        if(memcmp(&aa->aka(), &CFG->aka[0], sizeof(Addr)))
          aa->aka().make_string(addr);
        else
          strcpy(addr, ".");
        *attr = '(';  /*)*/
        MakeAttrStr(attr+1, &aa->attr());
        strcat(attr, /*(*/ ")");
        if(aa->originno())
          sprintf(origin, "\"%s\"", CFG->origin[aa->originno()]);
        else
          *origin = NUL;
        fprintf(fp, "AREADEF %-*s %-*s %*s %s %s %-*s %-*s %-*s %s\n",
          maxechoid,  echoid,
          maxdesc,    desc,
          maxgroupid, groupid,
          type,
          msgbase,
          maxpath, path,
          maxaddr, addr,
          maxattr, attr,
          origin
        );
      }
    }

    fclose(fp);
  }
}


//  ------------------------------------------------------------------

void SetAreaDesc(char* echoid, char* desc) {

  AL.SetAreaDesc(echoid, desc);
}


//  ------------------------------------------------------------------

void AreaList::SetAreaDesc(char* echoid, char* desc) {

  for(uint n=0; n<total; n++) {
    if(strieql(echoid, index[n]->echoid())) {
      index[n]->set_desc(desc);
      break;
    }
  }
}


//  ------------------------------------------------------------------

void PcbAdjustArea(uint rec, const char* msgfile) {

  for(uint n=0; n<AL.total; n++) {
    Area* a = AL[n];
    if(a->ispcboard()) {
      if((a->board() == rec) AND (*a->path() == NUL)) {
        a->set_path(msgfile);
        break;
      }
      else if(strieql(a->path(), msgfile)) {
        a->set_board(rec);
        break;
      }
    }
  }
}


//  ------------------------------------------------------------------

