
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gcarea.cpp,v 1.3 1999/01/20 19:13:00 damware Exp $
//  ------------------------------------------------------------------
//  Common code for the area compile.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

const char* AddStamp(const char* file, int filecheck) {

  stamp = (Stamp*)throw_realloc(stamp, (stamps+2)*sizeof(Stamp));
  strcpy(stamp[stamps].fn, file);
  stamp[stamps].ft = GetFiletime(file);
  stamp[stamps].fc = (byte)filecheck;
  stamps++;

  return file;
}


//  ------------------------------------------------------------------
//  Add stamp unless told not to

void ChkAddStamp(const char* file, char* options) {

  if(!striinc("/NOCHK", options) AND !striinc("-NOCHK", options))
    AddStamp(file);
}


//  ------------------------------------------------------------------
//  Fix up the echoid and make default if none specified

void FixEchoid(char* echoid, int type) {

  static uint __FAR netnum = 1;
  static uint __FAR echonum = 1;
  static uint __FAR localnum = 1;

  //strupr(echoid);
  //strchg(echoid, ' ', '_');

  if(*echoid == NUL) {
    char* t = CFG->areaautoid == AUTOID_LONG ? "MAIL" : "";
    if(type & AT_NET)
      sprintf(echoid, "NET%s%03u", t, netnum++);
    else if(type & AT_ECHO)
      sprintf(echoid, "ECHO%s%03u", t, echonum++);
    else if(type & AT_LOCAL)
      sprintf(echoid, "LOCAL%03u", localnum++);
  }
}


//  ------------------------------------------------------------------
//  Sort a group of areas

void AreaList::SortAreaGroup(const char* options, int beginarea, int endarea) {

  if(beginarea != endarea) {
    const char* ptr = striinc("-S", options);
    if(ptr == NULL)
      ptr = striinc("/S", options);
    if(ptr) {
      ptr += 2;
      if(*ptr == '=')
        ptr++;
      Sort(ptr, beginarea, endarea);
    }
  }
}


//  ------------------------------------------------------------------
//  Rename an echoid

void RenameArea(char* echoid) {
  
  for(int n=0; n<CFG->arearenames; n++) {
    if(strieql(echoid, CFG->arearename[n].from)) {
      strcpy(echoid, CFG->arearename[n].to);
      break;
    }
  }
}


//  ------------------------------------------------------------------
//  Check if the echoid is on the exclusion list

int CheckExcl(char* echoid) {

  for(int n=0; n<CFG->areaexcls; n++) {
    if(strwild(echoid, CFG->areaexcl[n])) {

      // Found excl, now check for incl
      for(int x=0; x<CFG->areaincls; x++)
        if(strwild(echoid, CFG->areaincl[x]))
          return(1);

      return(0);   // Echoid is excluded
    }
  }
  return(1);
}


//  ------------------------------------------------------------------
//  Check if the echo is email or news

void CheckEMailOrNews(char* echoid, uint& type) {

  for(int e=0; e<CFG->areaisemails; e++) {
    if(strwild(echoid, CFG->areaisemail[e])) {
      type = AT_EMAIL | AT_NET;
      break;
    }
  }

  for(int n=0; n<CFG->areaisnewss; n++) {
    if(strwild(echoid, CFG->areaisnews[n])) {
      type = AT_NEWSGROUP | AT_ECHO;
      break;
    }
  }
}


//  ------------------------------------------------------------------
//  Add or update area data

void AddNewArea(AreaCfg* aa) {

  AL.AddNewArea(aa);
}


//  ------------------------------------------------------------------
//  Add or update area data

void AddNewArea(AreaCfg& aa) {

  AL.AddNewArea(&aa);
}


//  ------------------------------------------------------------------
//  Add or update area data

void AreaList::AddNewArea(AreaCfg* aa) {

  if(veryverbose)
    printf("\n  fmt=%u, eid=\"%s\", pth=\"%s\", brd=%u, gid=%u, aka=%u:%u/%u.%u",
      aa->msgbase,
      aa->echoid,
      aa->path,
      aa->board,
      aa->groupid,
      aa->aka.zone,
      aa->aka.net,
      aa->aka.node,
      aa->aka.point
    );

  Desc desc;
  char buf[256];
  int newarea = YES;
  static int __FAR serial = 0;
  static int __FAR net1st = FALSE;

  *desc = NUL;

  // Make sure the path field is 100% correct for the msgbase
  switch(aa->msgbase) {
    case MT_HUDSON:
      if((aa->board < 1) OR (aa->board > 200))  // Ignore areas with invalid numbers
        return;
      sprintf(aa->path, "%u", aa->board);
      break;
    case MT_GOLDBASE:
      #if defined(__BORLANDC_) and defined(__MSDOS__)
      return;
      #endif
      if((aa->board < 1) OR (aa->board > 500))  // Ignore areas with invalid numbers
        return;
      sprintf(aa->path, "%u", aa->board);
      break;
    case MT_EZYCOM:
      #if defined(__BORLANDC_) and defined(__MSDOS__)
      return;
      #endif
      // Ignore areas with invalid numbers
      if((aa->board < 1) OR (aa->board > 1536))
        return;
      sprintf(aa->path, "%u", aa->board);
      break;
    case MT_FTS1:
    case MT_OPUS:
      if(*aa->path == NUL)
        return;
      MapPath(aa->path);
      AddBackslash(aa->path);
      strschg_environ(aa->path);
      break;
    case MT_WILDCAT:
      #if defined(__BORLANDC_) and defined(__MSDOS__)
      return;
      #endif
      if(*aa->path == NUL)
        return;
      MapPath(aa->path);
      StripBackslash(aa->path);
      strschg_environ(aa->path);
      break;
    case MT_ADEPTXBBS:
      #if not defined(__OS2__)
      return;
      #else
      if(*aa->path == NUL)
        return;
      MapPath(aa->path);
      StripBackslash(aa->path);
      strschg_environ(aa->path);
      break;
      #endif
    case MT_SQUISH:
    case MT_JAM:
      if(*aa->path == NUL)
        return;
      MapPath(aa->path);
      StripBackslash(aa->path);
      strschg_environ(aa->path);
      break;
    case MT_PCBOARD:
      #if defined(__BORLANDC_) and defined(__MSDOS__)
      return;
      #endif
      MapPath(aa->path);
      StripBackslash(aa->path);
      strschg_environ(aa->path);
      break;
    case MT_SEPARATOR:
      break;
    default:
      return;
  }

  // Note msgbase type
  msgbases |= aa->msgbase;

  // Things to do for real areas only
  if(NOT aa->isseparator()) {

    // If the echoid is missing, try to find it in the echolist
    if(strblank(aa->echoid))
      AFILE->echolist.FindEcho(aa->echoid, aa->path, aa->desc);

    // Fix up the echoid
    FixEchoid(aa->echoid, aa->type);

    // Rename the echoid if necessary
    RenameArea(aa->echoid);

    // Do not accept excluded areas
    if(NOT CheckExcl(aa->echoid))
      return;

    // Fix description
    if(strblank(aa->desc))
      strxcpy(aa->desc, aa->echoid, sizeof(aa->desc));
  }

  // Check if it's email or news
  CheckEMailOrNews(aa->echoid, aa->type);

  // Give each area a unique serial number
  aa->areaid = serial++;

  // Check if we already have the area (dup echoid or path)
  int _currarea = 0;
  for(uint n=0; n<total; n++) {
    if(NOT index[n]->isseparator()) {
      int eq_echoid  = strieql(aa->echoid, index[n]->echoid());
      int eq_path    = strieql(aa->path, index[n]->path());
      int eq_board   = (aa->board == index[n]->board());
      int eq_msgbase = (aa->msgbase == index[n]->msgbase());
      int eq_isfido  = (aa->isfido() AND index[n]->isfido());
      if(eq_echoid OR (eq_path AND eq_board AND (eq_msgbase OR eq_isfido))) {
        // We had it already, so override with the new data
        _currarea = n;
        newarea = NO;
        if(strblank(index[n]->desc()))
          strcpy(desc, aa->desc);
        break;
      }
    }
  }

  // If first netmail area, set default for AREAFREQTO and AREAREPLYTO
  if((aa->isnet() and not aa->isseparator()) and not net1st) {
    net1st = TRUE;
    if(NOT *CFG->areafreqto)
      strcpy(CFG->areafreqto, aa->echoid);
    if(NOT *CFG->areareplyto)
      strcpy(CFG->areareplyto, aa->echoid);
  }

  // Increase area index size if new area
  if(newarea) {
    if((total % 50) == 0)  // Allocate 50 at a time to speed up
      index = (Area**)throw_realloc(index, (total+55)*sizeof(Area*));
    index[total] = NewArea(aa->msgbase);
    throw_new(index[total]);
    _currarea = total++;
  }
  else {
    delete index[_currarea];
    index[_currarea] = NewArea(aa->msgbase);
    throw_new(index[_currarea]);
  }

  // Add aka if not found
  if(aa->aka.net) {
    int found = NO;
    for(uint n=0; n<CFG->akas; n++) {
      if(aa->aka.zone == CFG->aka[n].addr.zone) {
        if(aa->aka.net == CFG->aka[n].addr.net) {
          if(aa->aka.node == CFG->aka[n].addr.node) {
            if(aa->aka.point == CFG->aka[n].addr.point) {
              found = YES;
              break;
            }
          }
        }
      }
    }
    if(NOT found) {   // Then add it
      CfgAddress(aa->aka.make_string(buf));  // Add the aka
    }
  }
  else {
    // If address is missing, use main address
    if(CFG->akas)
      aa->aka = CFG->aka[0].addr;
  }

  // Copy the data
  Area* ap = index[_currarea];
  if(newarea) {
    ap->set_echoid(aa->echoid);
    ap->set_desc(aa->desc);
    ap->set_path(aa->path);
  }
  else {
    ap->set_echoid(aa->echoid);
    ap->set_desc(strblank(desc) ? aa->desc : desc);
    ap->set_path(aa->path);
  }
  ap->set_areaid(aa->areaid);
  ap->set_groupid(aa->groupid);
  ap->set_type(aa->type);
  ap->set_msgbase(aa->msgbase);
  ap->set_board(aa->board);
  ap->set_aka(aa->aka);
  ap->set_originno(aa->originno);
  ap->set_attr(aa->attr);
  ap->set_scan(aa->scan);
  ap->set_scanexcl(aa->scanexcl);
  ap->set_scanincl(aa->scanincl);
  ap->set_pmscan(aa->pmscan);
  ap->set_pmscanexcl(aa->pmscanexcl);
  ap->set_pmscanincl(aa->pmscanincl);
}


//  ------------------------------------------------------------------
//  Get areas from other programs

void AreaList::GetAreafile(char* value) {

  const word CRC_ECHOLIST = 0xCE70;

  word crcval;
  char* keyword;
  char* options;
  int beginarea, endarea;

  crcval = getkeyvalcrc(&keyword, &value);
  options = throw_strdup(value);

  beginarea = total;

  if(crcval == CRC_ECHOLIST)
    ReadEcholist(value);
  else {
    AFILE->quiet = quiet;
    AFILE->sharemode = CFG->sharemode;
    AFILE->fidomsgtype = CFG->fidomsgtype;
    AFILE->ra2usersbbs = CFG->ra2usersbbs;
    AFILE->squishuserno = CFG->squishuserno;

    if(CFG->akas)
      AFILE->primary_aka = CFG->aka[0].addr;
    else
      AFILE->primary_aka = 0;

    AFILE->attribsnet = CFG->attribsnet;
    AFILE->attribsecho = CFG->attribsecho;
    AFILE->attribsnews = CFG->attribsnews;
    AFILE->attribsemail = CFG->attribsemail;
    AFILE->attribslocal = CFG->attribslocal;

    AFILE->areapath = CFG->areapath;
    AFILE->adeptxbbspath = CFG->adeptxbbspath;
    AFILE->jampath = CFG->jampath;
    AFILE->squishuserpath = CFG->squishuserpath;
    AFILE->hudsonpath = CFG->hudsonpath;
    AFILE->goldbasepath = CFG->goldbasepath;
    AFILE->pcboardpath = CFG->pcboardpath;
    AFILE->ezycom_msgbasepath = CFG->ezycom.msgbasepath;
    AFILE->ezycom_userbasepath = CFG->ezycom.userbasepath;
    AFILE->fidolastread = CFG->fidolastread;

    AFILE->ReadAreafile(crcval, value);

    CFG->ra2usersbbs = AFILE->ra2usersbbs;
    CFG->squishuserno = AFILE->squishuserno;
  }

  endarea = total;
  SortAreaGroup(options, beginarea, endarea);

  throw_free(options);
}


//  ------------------------------------------------------------------
//  Get area definition

void AreaList::GetArea(char* def) {

  AreaCfg aa;
  char* ptr;
  char* tag;
  char* desc;
  char* base;
  char* loc;
  char* attr;

  if(strnieql(def, "-FILE ", 6))  // Check for alternate formats
    GetAreafile(def+6);
  else {
    tag = def;
    desc = strpbrk(tag, " \t");
    *desc = NUL;

    aa.reset();

    // Split up the string definition
    desc = SkipWht(desc+1);
    switch(*desc) {
      case '\"':
        base = strchr(desc+1, '\"')+1;
        break;
      case '\'':
        base = strchr(desc+1, '\'')+1;
        break;
      default:
        base = strpbrk(desc, " \t");
    }
    *base = NUL;
    base = SkipWht(base+1);
    loc = strpbrk(base, " \t");
    *loc = NUL;
    loc = SkipWht(loc+1);
    attr = strpbrk(loc, " \t");
    if(attr AND *attr) {
      *attr = NUL;
      attr = SkipWht(attr+1);
    }

    // Parse the individual parts

    aa.setechoid(tag);              // Copy the echotag

    StripQuotes(desc);              // Copy description, stripping quotes ("")
    aa.setdesc(desc);

    switch(toupper(*base)) {        // Store area info

      case '-':
        aa.msgbase = MT_SEPARATOR;
        break;

      case 'M':
        aa.msgbase = MT_SQUISH;
        aa.setpath(loc);
        break;

      case 'H':
      case 'R':
      case 'Q':
        aa.msgbase = MT_HUDSON;
        aa.board = (uint) atoi(loc);
        break;

      case 'G':
        aa.msgbase = MT_GOLDBASE;
        aa.board = (uint) atoi(loc);
        break;

      case 'E':
        aa.msgbase = MT_EZYCOM;
        aa.board = (uint) atoi(loc);
        break;

      case 'J':
        aa.msgbase = MT_JAM;
        aa.setpath(loc);
        break;

      case 'P':
        aa.msgbase = MT_PCBOARD;
        aa.setpath(loc);
        break;

      case 'W':
        aa.msgbase = MT_WILDCAT;
        aa.setpath(loc);
        break;

      case 'X':
        aa.msgbase = MT_ADEPTXBBS;
        aa.setpath(loc);
        break;

      case 'S':
        aa.msgbase = MT_FTS1;
        aa.setpath(loc);
        break;

      case 'F':
      case 'O':
      default:
        aa.msgbase = MT_OPUS;
        aa.setpath(loc);
    }

    aa.type = AT_ECHO;
    aa.attr = CFG->attribsecho;
    strupr(base);
    if(strchr(base, 'N')) {
      aa.type = AT_NET;
      aa.attr = CFG->attribsnet;
    }
    if(strchr(base, 'L')) {
      aa.type = AT_LOCAL;
      aa.attr = CFG->attribslocal;
    }
    if(strchr(base, 'E')) {
      aa.type = AT_ECHO;
      aa.attr = CFG->attribsecho;
    }
    if(attr) {                      // Get attributes and AKAs
      GetAttribstr(&aa.attr, attr);
      if((ptr = strpbrk(attr, "0123456789")) != NULL) {
        int n = atoi(ptr);
        if((n < 0) or ((uint) n >= CFG->akas))
          n = 0;
        if(CFG->akas)
          aa.aka = CFG->aka[n].addr;   // Found AKA number
      }
    }

    if(NOT aa.aka.net and CFG->akas)
      aa.aka = CFG->aka[0].addr;

    AddNewArea(&aa);
  }
}


//  ------------------------------------------------------------------
//  Get area definition (new)
//
//  Syntax of AREADEF keyword:
//  
//    AREADEF <echoid> <"desc"> <group> <type> <msgbase> <path/board> <aka> <(attrs)> ["origin"]
//  
//  Examples:
//  
//    AREADEF NET.ALL  "Netmail, Line 1" N Net   Opus   R:\NETMAIL\             2:231/77 (PVT K/S)
//    AREADEF TEST     "Testing..."      D Echo  Hudson 67                      .        ()
//    AREADEF 1LOCAL   "Sysop <-> Users" L Local Squish R:\MAX\MSG\LOC\1LOCAL   .        (PVT)     "Your Sysop * The Goldware BBS Line 1"
//    AREADEF DANEINFO "DaneNet Info"    D Echo  Squish R:\MAX\MSG\231\DANEINFO .        (R/O)

bool AreaList::GetAreaFirstPart(AreaCfg& aa, char*& key, char*& val) {

  const word CRC_NET      = 0xEC5E;
  const word CRC_ECHO     = 0xC2D1;
  const word CRC_LOCAL    = 0x4CD5;
  const word CRC_EMAIL    = 0x9C64;
  const word CRC_NEWS     = 0x61F1;

  // Get echoid
  getkeyval(&key, &val);
  aa.setechoid(key);

  // Get description
  getkeyval(&key, &val);
  aa.setdesc(key);

  // Get group (letter)
  getkeyval(&key, &val);
  if(*key == '#')
    aa.groupid = atoi(key+1)+0x8000u;
  else
    aa.groupid = (isupper(*key) ? *key : 0);

  // Get type
  word crc = getkeyvalcrc(&key, &val);
  switch(crc) {
    case CRC_NET:
      aa.type = AT_NET;
      aa.attr = CFG->attribsnet;
      break;
    case CRC_ECHO:
      aa.type = AT_ECHO;
      aa.attr = CFG->attribsecho;
      break;
    case CRC_LOCAL:
      aa.type = AT_LOCAL;
      aa.attr = CFG->attribslocal;
      break;
    case CRC_EMAIL:
      aa.type = AT_EMAIL | AT_NET;
      aa.attr = CFG->attribsnet;
      break;
    case CRC_NEWS:
      aa.type = AT_NEWSGROUP | AT_ECHO;
      aa.attr = CFG->attribsecho;
      break;
    default:
      return false;
  }

  return true;
}


//  ------------------------------------------------------------------

void AreaList::GetAreaSep(char* val) {

  char* key;
  AreaCfg aa;

  aa.reset();
  aa.msgbase = MT_SEPARATOR;

  if(!GetAreaFirstPart(aa, key, val))
    return;

  AddNewArea(&aa);
}


//  ------------------------------------------------------------------

void AreaList::GetAreaDef(char* val) {

  const word CRC_SEP      = 0x2FC6;
  const word CRC_FIDO     = 0x3AC8;
  const word CRC_OPUS     = 0x1254;
  const word CRC_SDMSG    = 0xAEC0;
  const word CRC_FTS1     = 0xEE2A;
  const word CRC_FTSC     = 0xEE58;
  const word CRC_QBBS     = 0x175B;
  const word CRC_HUDSON   = 0xBAB1;
  const word CRC_SQUISH   = 0xFCF6;
  const word CRC_EZYCOM   = 0xC81B;
  const word CRC_JAM      = 0xA8C3;
  const word CRC_GOLD     = 0x6134;
  const word CRC_GOLDBASE = 0x560D;
  const word CRC_PCB      = 0x19B7;
  const word CRC_PCBOARD  = 0x84EC;
  const word CRC_WCAT     = 0xAEDB;
  const word CRC_WILDCAT  = 0x7F3A;
  const word CRC_XBBS     = 0xADC3;

  char* key;
  AreaCfg aa;

  aa.reset();

  if(!GetAreaFirstPart(aa, key, val))
    return;

  // Get format
  word crc = getkeyvalcrc(&key, &val);
  getkeyval(&key, &val);        // Get path/board
  switch(crc) {
    case CRC_SEP:
      aa.msgbase = MT_SEPARATOR;
      break;
    case CRC_FIDO:
    case CRC_OPUS:
    case CRC_SDMSG:
      aa.msgbase = MT_OPUS;
      aa.setpath(key);
      break;
    case CRC_FTS1:
    case CRC_FTSC:
      aa.msgbase = MT_FTS1;
      aa.setpath(key);
      break;
    case CRC_QBBS:
    case CRC_HUDSON:
      aa.msgbase = MT_HUDSON;
      aa.board = (uint) atoi(key);
      if((aa.board < 1) OR (aa.board > 200)) {
        return;
      }
      break;
    case CRC_GOLD:
    case CRC_GOLDBASE:
      aa.msgbase = MT_GOLDBASE;
      aa.board = atoi(key);
      if((aa.board < 1) OR (aa.board > 500)) {
        return;
      }
      break;
    case CRC_SQUISH:
      aa.msgbase = MT_SQUISH;
      aa.setpath(key);
      break;
    case CRC_EZYCOM:
      aa.msgbase = MT_EZYCOM;
      aa.board = atoi(key);
      break;
    case CRC_JAM:
      aa.msgbase = MT_JAM;
      aa.setpath(key);
      break;
    case CRC_PCB:
    case CRC_PCBOARD:
      aa.msgbase = MT_PCBOARD;
      aa.setpath(key);
      break;
    case CRC_WCAT:
    case CRC_WILDCAT:
      aa.msgbase = MT_WILDCAT;
      aa.setpath(key);
      break;
    case CRC_XBBS:
      aa.msgbase = MT_ADEPTXBBS;
      aa.setpath(key);
      break;
    default:
      return;
  }

  // Get aka
  getkeyval(&key, &val);
  if(CFG->akas)                        // Seed aka with main address
    aa.aka = CFG->aka[0].addr;

  aa.aka.set(key);

  // Get attributes
  getkeyval(&key, &val);
  GetAttribstr(&aa.attr, key);

  // Get origin (optional)
  getkeyval(&key, &val);
  aa.setorigin(key);

  // Add to arealist
  AddNewArea(&aa);
}


//  ------------------------------------------------------------------

void ReadEcholist(char* val) {

  AL.ReadEcholist(val);
}


//  ------------------------------------------------------------------

void AreaList::ReadEcholist(char* val) {

  FILE* fp;
  char* key;
  Path file;
  char buf[256], options[80];

  // Get filename or options
  *file = NUL;
  *options = NUL;
  while(*val) {
    getkeyval(&key, &val);
    if((*key == '-') OR (*key == '/')) {
      // Get option
      strcat(options, key);
      strcat(options, " ");
      continue;
    }
    strcpy(file, key);
  }

  if(*file) {

    fp = fsopen(file, "rt", CFG->sharemode);
    if(fp) {

      if(NOT quiet)
        printf("\n* Reading %s", file);
      ChkAddStamp(file, options);

      while(fgets((val=buf), sizeof(buf), fp)) {

        // Get echoid
        getkeyval(&key, &val);
        if(isalnum(*key)) {
          for(uint n=0; n<total; n++) {
            if(strieql(key, index[n]->echoid())) {
              index[n]->set_desc(val);
              break;
            }
          }
        }
      }

      fclose(fp);
    }
  }
}


//  ------------------------------------------------------------------

void AreaList::GetAreaDesc(char* val) {

  uint n;
  char* key;
  Area* aa = NULL;

  // Get echoid and find area
  getkeyval(&key, &val);
  for(n=0; n<total; n++) {
    if(strieql(key, index[n]->echoid())) {
      aa = index[n];
      break;
    }
  }
  if(n == total)   // Not found, ignore
    return;

  // Get description
  getkeyval(&key, &val);
  aa->set_desc(key);

  // Get group letter (optional)
  getkeyval(&key, &val);
  if(*key) {
    if(*key != '-') {
      if(*key == '#')
        aa->set_groupid(atoi(key+1)+0x8000u);
      else
        aa->set_groupid(toupper(*key));
    }

    // Get aka (optional)
    getkeyval(&key, &val);
    if(*key) {

      if(*key != '-') {
        ftn_address aka;
        if(CFG->akas)          // Seed aka with main address
          aka = CFG->aka[0].addr;

        aka.set(key);
        aa->set_aka(aka);
      }

      // Get attributes (optional)
      getkeyval(&key, &val);
      if(*key) {

        if(*key != '-') {
          Attr attr;
          GetAttribstr(&attr, key);
          aa->set_attr(attr);
        }

        // Get origin (optional)
        getkeyval(&key, &val);
        if(*key)
          aa->set_origin(key);
      }
    }
  }
}


//  ------------------------------------------------------------------
//  Reset area data

void AreaCfg::reset() {

  *echoid = *desc = *path = *origin = NUL;
  areaid = 0;
  groupid = 0;
  originno = 0;
  board = 0;
  msgbase = 0;
  aka.reset();
  type = 0;
  scan = 0;
  scanexcl = 0;
  scanincl = 0;
  pmscan = 0;
  pmscanexcl = 0;
  pmscanincl = 0;
  attr.reset();
}


//  ------------------------------------------------------------------
//  Set area description

const char* AreaCfg::setdesc(const char* _desc) {

  return strxcpy(desc, _desc, sizeof(Desc));
}


//  ------------------------------------------------------------------
//  Set area echoid

const char* AreaCfg::setautoid(const char* _echoid) {

  return strxcpy(echoid, CFG->areaautoid ? "" : _echoid, sizeof(Echo));
}


//  ------------------------------------------------------------------
//  Set area echoid

const char* AreaCfg::setechoid(const char* _echoid) {

  return strxcpy(echoid, _echoid, sizeof(Echo));
}


//  ------------------------------------------------------------------
//  Set area path/filename

const char* AreaCfg::setpath(const char* _path) {

  return StripQuotes(strxcpy(path, _path, sizeof(Path)));
}


//  ------------------------------------------------------------------
//  Set area origin

int AreaCfgBase::setorigin(char* _origin) {

  if(NOT strblank(_origin)) {

    // Check if it already exists
    for(int n=0; n<CFG->origins; n++)
      if(streql(CFG->origin[n], _origin))
        return originno = n;

    // Not found, so add it
    CfgOrigin(_origin);
    originno = CFG->origins-1;
  }

  return originno;
}


//  ------------------------------------------------------------------

Area::Area(gmo_area* a) {

  throw_new(a);

  area = a;
  adat = NULL;
  bookmark = 0;
  marks = 0;
  unread = 0;
  isvalidchg = false;
  isunreadchg = false;
  isreadmark = false;
  isreadpm = false;
  isscanned = false;
  ispmscanned = false;
  istossed = false;
}


//  ------------------------------------------------------------------

Area::~Area() {

  // Nothing to do
}


//  ------------------------------------------------------------------
#if 0
void Area::set_marked(bool value, word bit) {

  if(not bit)
//    bit = marks_mask;
    bit = 1;

  marks = (word)(value ? marks|bit : marks&~bit);
}
#endif

//  ------------------------------------------------------------------

char* MapPath(char* map, bool reverse) {

  Path buf,cmap;

  int a = reverse ? 1 : 0;
  int b = reverse ? 0 : 1;

  strcpy(cmap, map);
  if(reverse)
    strchg(cmap, GOLD_WRONG_SLASH_CHR, GOLD_SLASH_CHR);

  for(uint i=0; i < CFG->mappaths; i++) {
    char* p = CFG->mappath[i*2+a];
    char* q = CFG->mappath[i*2+b];
    if(strnieql(cmap, p, strlen(p))) {
      strxcpy(buf, map, sizeof(Path));
      strcpy(map, q);
      strcat(map, &buf[strlen(p)]);

      char sl1, sl2;
      char* ptr;

      ptr = strpbrk(p, "/\\");
      sl1 = ptr ? *ptr : NUL;
      ptr = strpbrk(q, "/\\");
      sl2 = ptr ? *ptr : NUL;

      if(sl1 and sl2)
        strchg(map, sl1, sl2);

      #if defined(__UNIX__)
      strlwr(map);
      #endif

      break;
    }
  }
  return map;
}


//  ------------------------------------------------------------------

