
//  ------------------------------------------------------------------
//  GoldED Source Code.
//  Configuration functions, part 0.
//  ------------------------------------------------------------------
//  $Id: gccfgg0.cpp,v 1.4 1999/02/02 02:09:40 damware Exp $
//  ------------------------------------------------------------------
//  The contents of this file are subject to the Goldware Source
//  License Version 1.0 (the "License"); you may not use this file
//  except in compliance with the License.
//
//  Software distributed under the License is distributed on an "AS
//  IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
//  implied. See the License for the specific language governing
//  rights and limitations under the License.
//
//  The Initial Developer of this code is Odinn Sorensen. Portions
//  created by Odinn Sorensen are Copyright (C) 1998 Odinn Sorensen.
//  All Rights Reserved.
//  ------------------------------------------------------------------


#include "golded.h"
#include "gccfgg.h"
#include "gcprot.h"


//  ------------------------------------------------------------------

int cfgingroup = FALSE;
int cfggroups = 0;
gfile grpfp;
char* val;


//  ------------------------------------------------------------------

struct tglobalbool {
  word crc_token;
};

// it must be sorted by value
tglobalbool __FAR globalbool[gswitches_last] = {
  { CRC_INTERNETLOOKUP           }, // 0x0E22;
  { CRC_KLUDGECHRS               }, // 0x0EEF;
  { CRC_MOUSE                    }, // 0x1B1B;
  { CRC_DISPLOCALHIGH            }, // 0x1D8B;
  { CRC_AREALISTGROUPID          }, // 0x1F75;
  { CRC_ASKDELORIG               }, // 0x26DD;
  { CRC_LOOKUPLOCAL              }, // 0x34E7;
  { CRC_DISPSTATUSLINE           }, // 0x3FCF;
  { CRC_SCREENUSEBIOS            }, // 0x43DE;
  { CRC_HIGHLIGHTUNREAD          }, // 0x45DA;
  { CRC_AKAMATCHLOCAL            }, // 0x4AC4;
  { CRC_STYLECODES               }, // 0x4CB0;
  { CRC_FIDONULLFIX              }, // 0x4F38;
  { CRC_DISPAUTONEXT             }, // 0x52D9;
  { CRC_LOOKUPUSERBASE           }, // 0x5852;
  { CRC_AREAKEEPLAST             }, // 0x5876;
  { CRC_FILELISTPAGEBAR          }, // 0x58AA;
  { CRC_PRINTFORMFEED            }, // 0x59FB;
  { CRC_SCREENELIMSNOW           }, // 0x5E4D;
  { CRC_AREALISTNOS              }, // 0x5FD7;
  { CRC_DISPPAGEBAR              }, // 0x60A4;
  { CRC_INTERNETREPLY            }, // 0x6253;
  { CRC_AREALISTPAGEBAR          }, // 0x6C37;
  { CRC_USEMSGID                 }, // 0x6FDB;
  { CRC_LOOKUPNET                }, // 0x7359;
  { CRC_SQUISHDIRECT             }, // 0x7C35;
  { CRC_FIDOHWMARKS              }, // 0x7E07;
  { CRC_MSGLISTPAGEBAR           }, // 0x8041;
  { CRC_BEEPNOISES               }, // 0x84D5;
  { CRC_SCREENSHADOWS            }, // 0x8543;
  { CRC_DISPSOFTCR               }, // 0x85E9;
  { CRC_EMPTYTEARLINE            }, // 0x8D0A;
  { CRC_MENUDROPMSG              }, // 0x9645;
  { CRC_BEEPLOCALMSG             }, // 0x984C;
  { CRC_STATUSLINECLOCK          }, // 0x9C8A;
  { CRC_TWITTO                   }, // 0x9DFE;
  { CRC_KEYBDEFAULTS             }, // 0x9FAE;
  { CRC_DISPLISTWRAP             }, // 0xB36D;
  { CRC_QUOTESPACING             }, // 0xB403;
  { CRC_LOOKUPECHO               }, // 0xB787;
  { CRC_KEYBEXT                  }, // 0xC48A;
  { CRC_BEEPCOMMENT              }, // 0xC49E;
  { CRC_QUOTEBLANK               }, // 0xD3BB;
  { CRC_KEYBCLEAR                }, // 0xD407;
  { CRC_DOSPROMPT                }, // 0xD983;
  { CRC_AREACATCHUPREAD          }, // 0xDAFF;
  { CRC_USEFLAGS                 }, // 0xE2B6;
  { CRC_JAMHARDDELETE            }, // 0xE2D5;
  { CRC_DISPREALMSGNO            }, // 0xE30F;
  { CRC_FRQWAZOO                 }, // 0xE49C;
  { CRC_MSGLISTVIEWSUBJ          }, // 0xED92;
  { CRC_AKAMATCHECHO             }, // 0xF0C1;
  { CRC_AREAFILEGROUPS           }, // 0xF0E7;
  { CRC_AREAAUTONEXT             }, // 0xF589;
  { CRC_TIMEOUTSAVEMSG           }, // 0xF644;
  { CRC_NODELISTWARN             }, // 0xF818;
  { CRC_NODELISTPAGEBAR          }, // 0xFA40;
  { CRC_AKAMATCHNET              }, // 0xFD6E;
};

bool gswitches::handle(word crc, const char* value) {

  uint left,right,middle;
  word cur;

  left = 0;
  right = gswitches_last - 1;

  if((crc < globalbool[left].crc_token) or (crc > globalbool[right].crc_token))
    return false;

  do {
    middle = (left + right)/2;

    cur = globalbool[middle].crc_token;

    if(crc < cur) {
      right = middle - 1;
    } else if(crc > cur) {
      left  = middle + 1;
    }
    else {
      cfg[middle] = (bool) GetYesno(value);
      return true;
    }
  } while(left <= right);

  return false;
}

//  ------------------------------------------------------------------

int SwitchCfg(word crc, char k, char* v) {

  val = v;
  int found = TRUE;

  if(CFG->switches.handle(crc,v))
    return true;

  // Switch the first letter of the keyword
  switch(toupper(k)) {
    case 'A': goto SwitchA;
    case 'B': goto SwitchB;
    case 'C': goto SwitchC;
    case 'D': goto SwitchD;
    case 'E': goto SwitchE;
    case 'F': goto SwitchF;
    case 'G': goto SwitchG;
    case 'H': goto SwitchH;
    case 'I': goto SwitchI;
    case 'J': goto SwitchJ;
    case 'K': goto SwitchK;
    case 'L': goto SwitchL;
    case 'M': goto SwitchM;
    case 'N': goto SwitchN;
    case 'O': goto SwitchO;
    case 'P': goto SwitchP;
    case 'Q': goto SwitchQ;
    case 'R': goto SwitchR;
    case 'S': goto SwitchS;
    case 'T': goto SwitchT;
    case 'U': goto SwitchU;
    case 'V': goto SwitchV;
    case 'W': goto SwitchW;
    case 'X': goto SwitchX;
    case 'Y': goto SwitchY;
    case 'Z': goto SwitchZ;
    default:  found = FALSE;
  }
  goto End;

SwitchA:
  switch(crc) {
    case CRC_ADDRESS          :  CfgAddress          ();  break;
    case CRC_ADDRESSBOOKADD   :  CfgAddressbookadd   ();  break;
    case CRC_ADDRESSMACRO     :  CfgAddressmacro     ();  break;
    case CRC_ADEPTXBBSPATH    :  CfgAdeptxbbspath    ();  break;
    case CRC_ADEPTXBBSUSERNO  :  CfgAdeptxbbsuserno  ();  break;
    case CRC_AKA              :  CfgAddress          ();  break;
    case CRC_AKAMATCH         :  CfgAkamatch         ();  break;
    case CRC_AKAMATCHING      :  CfgAkamatching      ();  break;
    case CRC_AREA             :  CfgArea             ();  break;
    case CRC_AREAAUTOID       :  CfgAreaautoid       ();  break;
    case CRC_AREACOPYADDID    :  CfgAreacopyaddid    ();  break;
    case CRC_AREACOPYDIRECT   :  CfgAreacopydirect   ();  break;
    case CRC_AREACOPYTO       :  CfgAreacopyto       ();  break;
    case CRC_AREADEF          :  CfgAreadef          ();  break;
    case CRC_AREADESC         :  CfgAreadesc         ();  break;
    case CRC_AREAEXCL         :  CfgAreaexcl         ();  break;
    case CRC_AREAFREQDIRECT   :  CfgAreafreqdirect   ();  break;
    case CRC_AREAFREQTO       :  CfgAreafreqto       ();  break;
    case CRC_AREAINCL         :  CfgAreaincl         ();  break;
    case CRC_AREAISEMAIL      :  CfgAreaisemail      ();  break;
    case CRC_AREAISNEWS       :  CfgAreaisnews       ();  break;
    case CRC_AREALISTECHOMAX  :  CfgArealistechomax  ();  break;
    case CRC_AREALISTFORMAT   :  CfgArealistformat   ();  break;
    case CRC_AREALISTSCAN     :  CfgArealistscan     ();  break;
    case CRC_AREALISTSORT     :  CfgArealistsort     ();  break;
    case CRC_AREALISTTYPE     :  CfgArealisttype     ();  break;
    case CRC_AREAPATH         :  CfgAreapath         ();  break;
    case CRC_AREAPMSCAN       :  CfgAreapmscan       ();  break;
    case CRC_AREAPMSCANEXCL   :  CfgAreapmscanexcl   ();  break;
    case CRC_AREAPMSCANINCL   :  CfgAreapmscanincl   ();  break;
    case CRC_AREAREADONLY     :  CfgAreareadonly     ();  break;
    case CRC_AREARENAME       :  CfgArearename       ();  break;
    case CRC_AREAREPLYDIRECT  :  CfgAreareplydirect  ();  break;
    case CRC_AREAREPLYTO      :  CfgAreareplyto      ();  break;
    case CRC_AREASCAN         :  CfgAreascan         ();  break;
    case CRC_AREASCANEXCL     :  CfgAreascanexcl     ();  break;
    case CRC_AREASCANINCL     :  CfgAreascanincl     ();  break;
    case CRC_AREASCANSORT     :  CfgAreascansort     ();  break;
    case CRC_AREASEP          :  CfgAreasep          ();  break;
    case CRC_AREASTART        :  CfgAreastart        ();  break;
    case CRC_AREATYPEORDER    :  CfgAreatypeorder    ();  break;
    case CRC_AREAYOUWROTETO   :  CfgAreayouwroteto   ();  break;
    case CRC_ATTACHPATH       :  CfgAttachpath       ();  break;
    case CRC_ATTRIBSATTACH    :  CfgAttribsattach    ();  break;
    case CRC_ATTRIBSCC        :  CfgAttribscc        ();  break;
    case CRC_ATTRIBSCFM       :  CfgAttribscfm       ();  break;
    case CRC_ATTRIBSECHO      :  CfgAttribsecho      ();  break;
    case CRC_ATTRIBSEMAIL     :  CfgAttribsemail     ();  break;
    case CRC_ATTRIBSFRQ       :  CfgAttribsfrq       ();  break;
    case CRC_ATTRIBSLOCAL     :  CfgAttribslocal     ();  break;
    case CRC_ATTRIBSNET       :  CfgAttribsnet       ();  break;
    case CRC_ATTRIBSNEWS      :  CfgAttribsnews      ();  break;
    case CRC_ATTRIBUTES       :  CfgAttributes       ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchB:
  switch(crc) {
    case CRC_BEEPFACTOR       :  CfgBeepfactor       ();  break;
    case CRC_BEEPYOURMAIL     :  CfgBeepyourmail     ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchC:
  switch(crc) {
    case CRC_CARBONCOPYLIST   :  CfgCarboncopylist   ();  break;
    case CRC_COLOR            :  CfgColor            ();  break;
    case CRC_COLORSET         :  CfgColorset         ();  break;
    case CRC_COLOUR           :  CfgColor            ();  break;
    case CRC_CONFIRMFILE      :  CfgConfirmfile      ();  break;
    case CRC_CONFIRMRESPONSE  :  CfgConfirmresponse  ();  break;
    case CRC_COOKIEFILE       :  CfgCookiefile       ();  break;
    case CRC_COOKIEPATH       :  CfgCookiepath       ();  break;
    case CRC_CROSSPOSTLIST    :  CfgCrosspostlist    ();  break;
    case CRC_CTRLINFO         :  CfgCtrlinfo         ();  break;
    case CRC_CTRLINFOECHO     :  CfgCtrlinfoecho     ();  break;
    case CRC_CTRLINFOEMAIL    :  CfgCtrlinfoemail    ();  break;
    case CRC_CTRLINFOLOCAL    :  CfgCtrlinfolocal    ();  break;
    case CRC_CTRLINFONET      :  CfgCtrlinfonet      ();  break;
    case CRC_CTRLINFONEWS     :  CfgCtrlinfonews     ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchD:
  switch(crc) {
    case CRC_DISPAREANO       :  CfgDispareano       ();  break;
    case CRC_DISPATTACHSIZE   :  CfgDispattachsize   ();  break;
    case CRC_DISPHDRDATESET   :  CfgDisphdrdateset   ();  break;
    case CRC_DISPHDRNAMESET   :  CfgDisphdrnameset   ();  break;
    case CRC_DISPHDRNODESET   :  CfgDisphdrnodeset   ();  break;
    case CRC_DISPLISTCURSOR   :  CfgDisplistcursor   ();  break;
    case CRC_DISPMARGIN       :  CfgDispmargin       ();  break;
    case CRC_DISPMSGSIZE      :  CfgDispmsgsize      ();  break;
    case CRC_DISPTABSIZE      :  CfgDisptabsize      ();  break;
    case CRC_DOSSWAP          :  CfgDosswap          ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchE:
  switch(crc) {
    case CRC_EDITAUTOATTACH   :  CfgEditautoattach   ();  break;
    case CRC_EDITAUTOSAVE     :  CfgEditautosave     ();  break;
    case CRC_EDITCHANGEDATE   :  CfgEditchangedate   ();  break;
    case CRC_EDITCHARPARA     :  CfgEditcharpara     ();  break;
    case CRC_EDITCHARSPACE    :  CfgEditcharspace    ();  break;
    case CRC_EDITCOMMENT      :  CfgEditcomment      ();  break;
    case CRC_EDITCOMPLETION   :  CfgEditcompletion   ();  break;
    case CRC_EDITCRLFTERM     :  CfgEditcrlfterm     ();  break;
    case CRC_EDITFIELDCLEAR   :  CfgEditfieldclear   ();  break;
    case CRC_EDITHARDLINE     :  CfgEdithardline     ();  break;
    case CRC_EDITHARDLINES    :  CfgEdithardlines    ();  break;
    case CRC_EDITHARDTERM     :  CfgEdithardterm     ();  break;
    case CRC_EDITHDRNAMESET   :  CfgEdithdrnameset   ();  break;
    case CRC_EDITHDRNODESET   :  CfgEdithdrnodeset   ();  break;
    case CRC_EDITHEADERATTRS  :  CfgEditheaderattrs  ();  break;
    case CRC_EDITHEADERFIRST  :  CfgEditheaderfirst  ();  break;
    case CRC_EDITINTERNAL     :  CfgEditinternal     ();  break;
    case CRC_EDITMARGIN       :  CfgEditmargin       ();  break;
    case CRC_EDITMENU         :  CfgEditmenu         ();  break;
    case CRC_EDITMIXCASE      :  CfgEditmixcase      ();  break;
    case CRC_EDITMSGSIZE      :  CfgEditmsgsize      ();  break;
    case CRC_EDITOR           :  CfgEditor           ();  break;
    case CRC_EDITORFILE       :  CfgEditorfile       ();  break;
    case CRC_EDITQUOTEMARGIN  :  CfgEditquotemargin  ();  break;
    case CRC_EDITREPLYRE      :  CfgEditreplyre      ();  break;
    case CRC_EDITSAVEMENU     :  CfgEditsavemenu     ();  break;
    case CRC_EDITSAVEUTIL     :  CfgEditsaveutil     ();  break;
    case CRC_EDITSOFTCRXLAT   :  CfgEditsoftcrxlat   ();  break;
    case CRC_EDITSPELLCHECK   :  CfgEditspellcheck   ();  break;
    case CRC_EDITUNDELETE     :  CfgEditundelete     ();  break;
    case CRC_ENDGROUP         :  CfgEndgroup         ();  break;
    case CRC_EVENT            :  CfgEvent            ();  break;
    case CRC_EXCLUDENODES     :  CfgExcludenodes     ();  break;
    case CRC_EXTERNOPTIONS    :  CfgExternoptions    ();  break;
    case CRC_EXTERNUTIL       :  CfgExternutil       ();  break;
    case CRC_EZYCOMMSGBASE    :  CfgEzycommsgbase    ();  break;
    case CRC_EZYCOMUSERBASE   :  CfgEzycomuserbase   ();  break;
    case CRC_EZYCOMUSERNO     :  CfgEzycomuserno     ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchF:
  switch(crc) {
    case CRC_FIDOLASTREAD     :  CfgFidolastread     ();  break;
    case CRC_FIDOMSGTYPE      :  CfgFidomsgtype      ();  break;
    case CRC_FIDOUSERLIST     :  CfgFidouserlist     ();  break;
    case CRC_FIDOUSERNO       :  CfgFidouserno       ();  break;
    case CRC_FILEALIAS        :  CfgFilealias        ();  break;
    case CRC_FILECHECK        :  CfgFilecheck        ();  break;
    case CRC_FILECHECKCALL    :  CfgFilecheckcall    ();  break;
    case CRC_FORCETEMPLATE    :  CfgForcetemplate    ();  break;
    case CRC_FRQEXT           :  CfgFrqext           ();  break;
    case CRC_FRQNODEMAP       :  CfgFrqnodemap       ();  break;
    case CRC_FRQOPTIONS       :  CfgFrqoptions       ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchG:
  switch(crc) {
    case CRC_GIF              :  CfgGif              ();  break;
    case CRC_GIFPATH          :  CfgGifpath          ();  break;
    case CRC_GOLDBASEPATH     :  CfgGoldbasepath     ();  break;
    case CRC_GOLDBASESYSPATH  :  CfgGoldbasesyspath  ();  break;
    case CRC_GOLDBASEUSERNO   :  CfgGoldbaseuserno   ();  break;
    case CRC_GOLDPATH         :  CfgGoldpath         ();  break;
    case CRC_GROUP            :  CfgGroup            ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchH:
  switch(crc) {
    case CRC_HAPPYBIRTHDAY    :  CfgHappybirthday    ();  break;
    case CRC_HUDSONPATH       :  CfgHudsonpath       ();  break;
    case CRC_HUDSONSIZEWARN   :  CfgHudsonsizewarn   ();  break;
    case CRC_HUDSONSYSPATH    :  CfgHudsonsyspath    ();  break;
    case CRC_HUDSONUSERNO     :  CfgHudsonuserno     ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchI:
  switch(crc) {
    case CRC_IMPORTBEGIN      :  CfgImportbegin      ();  break;
    case CRC_IMPORTEND        :  CfgImportend        ();  break;
    case CRC_INBOUNDPATH      :  CfgInboundpath      ();  break;
    case CRC_INCLUDENODES     :  CfgIncludenodes     ();  break;
    case CRC_INPUTFILE        :  CfgInputfile        ();  break;
    case CRC_INTENSECOLORS    :  CfgIntensecolors    ();  break;
    case CRC_INTERNETADDRESS  :  CfgInternetaddress  ();  break;
    case CRC_INTERNETDOMAIN   :  CfgInternetdomain   ();  break;
    case CRC_INTERNETGATE     :  CfgInternetgate     ();  break;
    case CRC_INTERNETMSGID    :  CfgInternetmsgid    ();  break;
    case CRC_INTERNETRFCBODY  :  CfgInternetrfcbody  ();  break;
    case CRC_INTERNETSERVER   :  CfgInternetserver   ();  break;
    case CRC_INVALIDATE       :  CfgInvalidate       ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchJ:
  switch(crc) {
    case CRC_JAMPATH          :  CfgJampath          ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchK:
  switch(crc) {
    case CRC_KEYBMODE         :  CfgKeybmode         ();  break;
    case CRC_KEYBSTACK        :  CfgKeybstack        ();  break;
    case CRC_KLUDGE           :  CfgKludge           ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchL:
  switch(crc) {
    case CRC_LOADLANGUAGE     :  CfgLoadlanguage     ();  break;
    case CRC_LOGFILE          :  CfgLogfile          ();  break;
    case CRC_LOGFORMAT        :  CfgLogformat        ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchM:
  switch(crc) {
    case CRC_MAILINGLIST      :  CfgMailinglist      ();  break;
    case CRC_MAILTOSS         :  CfgMailtoss         ();  break;
    case CRC_MAPPATH          :  CfgMappath          ();  break;
    case CRC_MEMBER           :  CfgMember           ();  break;
    case CRC_MENUMARKED       :  CfgMenumarked       ();  break;
    case CRC_MSGLISTDATE      :  CfgMsglistdate      ();  break;
    case CRC_MSGLISTFAST      :  CfgMsglistfast      ();  break;
    case CRC_MSGLISTFIRST     :  CfgMsglistfirst     ();  break;
    case CRC_MSGLISTHEADER    :  CfgMsglistheader    ();  break;
    case CRC_MSGLISTWIDESUBJ  :  CfgMsglistwidesubj  ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchN:
  switch(crc) {
    case CRC_NAMESFILE        :  CfgNamesfile        ();  break;
    case CRC_NETNAME          :  CfgNetname          ();  break;
    case CRC_NICKNAME         :  CfgNickname         ();  break;
    case CRC_NODELIST         :  CfgNodelist         ();  break;
    case CRC_NODEPATH         :  CfgNodepath         ();  break;
    case CRC_NODEPATHFD       :  CfgNodepathfd       ();  break;
    case CRC_NODEPATHIM       :  CfgNodepathfd       ();  break;
    case CRC_NODEPATHV7       :  CfgNodepathv7       ();  break;
    case CRC_NODEV7FLAGS      :  CfgNodev7flags      ();  break;
    case CRC_NODEV7MODEM      :  CfgNodev7modem      ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchO:
  switch(crc) {
    case CRC_ORGANIZATION     :  CfgOrganization     ();  break;
    case CRC_ORIGIN           :  CfgOrigin           ();  break;
    case CRC_OUTBOUNDPATH     :  CfgOutboundpath     ();  break;
    case CRC_OUTPUTFILE       :  CfgOutputfile       ();  break;
    case CRC_OVERLAY          :  CfgOverlay          ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchP:
  switch(crc) {
    case CRC_PCBOARDMAILER    :  CfgPcboardmailer    ();  break;
    case CRC_PCBOARDPATH      :  CfgPcboardpath      ();  break;
    case CRC_PCBOARDUSERNO    :  CfgPcboarduserno    ();  break;
    case CRC_PERSONALMAIL     :  CfgPersonalmail     ();  break;
    case CRC_PLAY             :  CfgPlay             ();  break;
    case CRC_PRINTDEVICE      :  CfgPrintdevice      ();  break;
    case CRC_PRINTINIT        :  CfgPrintinit        ();  break;
    case CRC_PRINTLENGTH      :  CfgPrintlength      ();  break;
    case CRC_PRINTMARGIN      :  CfgPrintmargin      ();  break;
    case CRC_PRINTRESET       :  CfgPrintreset       ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchQ:
  switch(crc) {
    case CRC_QUOTEBUFFILE     :  CfgQuotebuffile     ();  break;
    case CRC_QUOTEBUFMODE     :  CfgQuotebufmode     ();  break;
    case CRC_QUOTECHARS       :  CfgQuotechars       ();  break;
    case CRC_QUOTEMARGIN      :  CfgQuotemargin      ();  break;
    case CRC_QUOTESTRING      :  CfgQuotestring      ();  break;
    case CRC_QUOTEWRAPHARD    :  CfgQuotewraphard    ();  break;
    case CRC_QWKBADMSGS       :  CfgQwkbadmsgs       ();  break;
    case CRC_QWKCONFMAP       :  CfgQwkconfmap       ();  break;
    case CRC_QWKEXPORTPATH    :  CfgQwkexportpath    ();  break;
    case CRC_QWKIMPORTPATH    :  CfgQwkimportpath    ();  break;
    case CRC_QWKOPTIONS       :  CfgQwkoptions       ();  break;
    case CRC_QWKREPLYLINKER   :  CfgQwkreplylinker   ();  break;
    case CRC_QWKTOSSLOG       :  CfgQwktosslog       ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchR:
  switch(crc) {
    case CRC_RA2USERSBBS      :  CfgRa2usersbbs      ();  break;
    case CRC_REGISTERKEY      :                           break;
    case CRC_REGISTERNAME     :                           break;
    case CRC_REPLYLINK        :  CfgReplylink        ();  break;
    case CRC_REPLYLINKLIST    :  CfgReplylinklist    ();  break;
    case CRC_ROBOTNAME        :  CfgRobotname        ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchS:
  switch(crc) {
    case CRC_SCREENBLANKER    :  CfgScreenblanker    ();  break;
    case CRC_SCREENMAXCOL     :  CfgScreenmaxcol     ();  break;
    case CRC_SCREENMAXROW     :  CfgScreenmaxrow     ();  break;
    case CRC_SCREENPALETTE    :  CfgScreenpalette    ();  break;
    case CRC_SCREENSIZE       :  CfgScreensize       ();  break;
    case CRC_SEARCHFOR        :  CfgSearchfor        ();  break;
    case CRC_SEMAPHORE        :  CfgSemaphore        ();  break;
    case CRC_SERIALNO         :  CfgSerialno         ();  break;
    case CRC_SHAREMODE        :  CfgSharemode        ();  break;
    case CRC_SOUNDPATH        :  CfgSoundpath        ();  break;
    case CRC_SOUPBADMSGS      :  CfgSoupbadmsgs      ();  break;
    case CRC_SOUPEMAIL        :  CfgSoupemail        ();  break;
    case CRC_SOUPEXPORTMARGIN :  CfgSoupexportmargin ();  break;
    case CRC_SOUPEXPORTPATH   :  CfgSoupexportpath   ();  break;
    case CRC_SOUPIMPORTPATH   :  CfgSoupimportpath   ();  break;
    case CRC_SOUPNEWSRCFILE   :  CfgSoupnewsrcfile   ();  break;
    case CRC_SOUPREPLYLINKER  :  CfgSoupreplylinker  ();  break;
    case CRC_SOUPREPLYTO      :  CfgSoupreplyto      ();  break;
    case CRC_SOUPTOSSLOG      :  CfgSouptosslog      ();  break;
    case CRC_SQUISHRECYCLE    :  CfgSquishrecycle    ();  break;
    case CRC_SQUISHSCAN       :  CfgSquishscan       ();  break;
    case CRC_SQUISHUSERNO     :  CfgSquishuserno     ();  break;
    case CRC_SQUISHUSERPATH   :  CfgSquishuserpath   ();  break;
    case CRC_STATUSLINEHELP   :  CfgStatuslinehelp   ();  break;
    case CRC_STYLECODEPUNCT   :  CfgStylecodepunct   ();  break;
    case CRC_STYLECODESTOPS   :  CfgStylecodestops   ();  break;
    case CRC_SWAPPATH         :  CfgSwappath         ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchT:
  switch(crc) {
    case CRC_TAGLINE          :  CfgTagline          ();  break;
    case CRC_TAGLINECHAR      :  CfgTaglinechar      ();  break;
    case CRC_TAGLINEFILE      :  CfgTaglinefile      ();  break;
    case CRC_TAGLINESUPPORT   :  CfgTaglinesupport   ();  break;
    case CRC_TEARLINE         :  CfgTearline         ();  break;
    case CRC_TEMPLATE         :  CfgTemplate         ();  break;
    case CRC_TEMPLATEMATCH    :  CfgTemplatematch    ();  break;
    case CRC_TEMPLATEPATH     :  CfgTemplatepath     ();  break;
    case CRC_TEMPPATH         :  CfgTemppath         ();  break;
    case CRC_TIMEOUT          :  CfgTimeout          ();  break;
    case CRC_TIMEZONEOFFSET   :  CfgTimezoneoffset   ();  break;
    case CRC_TWITMODE         :  CfgTwitmode         ();  break;
    case CRC_TWITNAME         :  CfgTwitname         ();  break;
    case CRC_TWITSUBJ         :  CfgTwitsubj         ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchU:
  switch(crc) {
    case CRC_USEFWD           :  CfgUsefwd           ();  break;
    case CRC_USEINTL          :  CfgUseintl          ();  break;
    case CRC_USERLIST         :  CfgUserlist         ();  break;
    case CRC_USERLISTFILE     :  CfgUserlistfile     ();  break;
    case CRC_USERNAME         :  CfgUsername         ();  break;
    case CRC_UUDECODEPATH     :  CfgUudecodepath     ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchV:
  switch(crc) {
    case CRC_VIEWHIDDEN       :  CfgViewhidden       ();  break;
    case CRC_VIEWKLUDGE       :  CfgViewkludge       ();  break;
    case CRC_VIEWQUOTE        :  CfgViewquote        ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchW:
  switch(crc) {
    case CRC_WHOTO            :  CfgWhoto            ();  break;
    case CRC_WILDCATUSERNO    :  CfgWildcatuserno    ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchX:
  switch(crc) {
    case CRC_XLATCHARSET      :  CfgXlatcharset      ();  break;
    case CRC_XLATESCSET       :  CfgXlatescset       ();  break;
    case CRC_XLATEXPORT       :  CfgXlatexport       ();  break;
    case CRC_XLATIMPORT       :  CfgXlatimport       ();  break;
    case CRC_XLATLOCALSET     :  CfgXlatlocalset     ();  break;
    case CRC_XLATPATH         :  CfgXlatpath         ();  break;
    default                   :  found = FALSE;
  }
  goto End;

SwitchY:
  switch(crc) {
    default                   :  found = FALSE;
  }
  goto End;

SwitchZ:
  switch(crc) {
    case CRC_ZONEGATING       :  CfgZonegating       ();  break;
    default                   :  found = FALSE;
  }
  goto End;

End:
  if(found)
    return TRUE;
  //found = TRUE;

  if(cmdlineoldkeyw == FALSE)
    return FALSE;

  // Obsolete keywords
  switch(crc) {
    case CRC_AREAAUTOFREQ     :  CfgAreafreqto       ();  break;
    case CRC_AREABADMSGS      :  CfgSoupbadmsgs      ();  break;
    case CRC_AREASORT         :  CfgArealistsort     ();  break;
    case CRC_ASSIGNTO         :  CfgMember           ();  break;
    case CRC_AUTOATTACH       :  CfgEditautoattach   ();  break;
    case CRC_BLANKTIME        :  CfgScreenblanker    ();  break;
    case CRC_CCATTRIB         :  CfgAttribscc        ();  break;
    case CRC_CCLIST           :  CfgCarboncopylist   ();  break;
    case CRC_CFMATTRIB        :  CfgAttribscfm       ();  break;
    case CRC_CHANGEDATE       :  CfgEditchangedate   ();  break;
    case CRC_CHARSET          :  CfgXlatcharset      ();  break;
    case CRC_CRLFTERM         :  CfgEditcrlfterm     ();  break;
    case CRC_DISPMSGLIST      :  CfgMsglistfirst     ();  break;
    case CRC_DISPMSGLISTFAST  :  CfgMsglistfast      ();  break;
    case CRC_ECHOATTRIB       :  CfgAttribsecho      ();  break;
    case CRC_ECHOINFO         :  CfgCtrlinfoecho     ();  break;
    case CRC_EDITORVERSION    :  CfgEditorversion    ();  break;
    case CRC_ESCSET           :  CfgXlatescset       ();  break;
    case CRC_EXCLAREA         :  CfgAreaexcl         ();  break;
    case CRC_EXPORTCHARSET    :  CfgXlatexport       ();  break;
    case CRC_FIDOLASTREADNO   :  CfgFidouserno       ();  break;
    case CRC_FIELDCLEAR       :  CfgEditfieldclear   ();  break;
    case CRC_FREEAREA         :  CfgFreearea         ();  break;
    case CRC_FREETEAR         :  CfgFreearea         ();  break;
    case CRC_GOLDEDCFM        :  CfgConfirmfile      ();  break;
    case CRC_GOLDEDLOG        :  CfgLogfile          ();  break;
    case CRC_GOLDEDLST        :  CfgUserlistfile     ();  break;
    case CRC_GOLDEDMSG        :  CfgEditorfile       ();  break;
    case CRC_GOLDEDNAM        :  CfgNamesfile        ();  break;
    case CRC_GOLDEDPRN        :  CfgOutputfile       ();  break;
    case CRC_GOLDHELP         :  CfgGoldhelp         ();  break;
    case CRC_GOLDKEYS         :  CfgGoldkeys         ();  break;
    case CRC_GOLDLANG         :  CfgGoldlang         ();  break;
    case CRC_GOLDRAND         :  CfgGoldrand         ();  break;
    case CRC_GOLDXLAT         :  CfgGoldxlat         ();  break;
    case CRC_HARDLINE         :  CfgEdithardline     ();  break;
    case CRC_HARDLINES        :  CfgEdithardlines    ();  break;
    case CRC_INCLAREA         :  CfgAreaincl         ();  break;
    case CRC_INTERNALEDITOR   :  CfgEditinternal     ();  break;
    case CRC_LASTREAD         :  CfgFidolastread     ();  break;
    case CRC_LASTREADUSER     :  CfgFidouserno       ();  break;
    case CRC_LOCALATTRIB      :  CfgAttribslocal     ();  break;
    case CRC_LOCALCHARSET     :  CfgXlatimport       ();  break;
    case CRC_LOCALINFO        :  CfgCtrlinfolocal    ();  break;
    case CRC_MAPDRIVE         :  CfgMapdrive         ();  break;
    case CRC_MAXCOLS          :  CfgScreenmaxcol     ();  break;
    case CRC_MAXMSGSIZE       :  CfgEditmsgsize      ();  break;
    case CRC_MAXROWS          :  CfgScreenmaxrow     ();  break;
    case CRC_MIXCASE          :  CfgEditmixcase      ();  break;
    case CRC_MULTIQBBS        :  CfgQbbsincrescan    ();  break;
    case CRC_NETATTRIB        :  CfgAttribsnet       ();  break;
    case CRC_NETINFO          :  CfgCtrlinfonet      ();  break;
    case CRC_NETTEAR          :  CfgNettear          ();  break;
    case CRC_NOISEFACTOR      :  CfgBeepfactor       ();  break;
    case CRC_OVERLAYEMS       :  CfgOverlayems       ();  break;
    case CRC_OVERLAYEXT       :  CfgOverlayext       ();  break;
    case CRC_QBBSINCRESCAN    :  CfgQbbsincrescan    ();  break;
    case CRC_QBBSPATH         :  CfgHudsonpath       ();  break;
    case CRC_QBBSREBUILD      :  CfgQbbsrebuild      ();  break;
    case CRC_QBBSSCAN         :  CfgQbbsscan         ();  break;
    case CRC_QMSGPATH         :  CfgHudsonpath       ();  break;
    case CRC_REBUILD          :  CfgQbbsrebuild      ();  break;
    case CRC_RENAREA          :  CfgArearename       ();  break;
    case CRC_REPLYRE          :  CfgEditreplyre      ();  break;
    case CRC_RIGHTMARGIN      :  CfgDispmargin       ();  break;
    case CRC_SAVEMENU         :  CfgEditsavemenu     ();  break;
    case CRC_SAVETIME         :  CfgEditautosave     ();  break;
    case CRC_SCANAREA         :  CfgAreascan         ();  break;
    case CRC_SCREENUSEANSI    :  CfgScreenuseansi    ();  break;
    case CRC_SHARE            :  CfgSharemode        ();  break;
    case CRC_SHOWTWITS        :  CfgTwitmode         ();  break;
    case CRC_SOUNDDEVICE      :  CfgSounddevice      ();  break;
    case CRC_SPELLCHECKER     :  CfgEditspellcheck   ();  break;
    case CRC_STACKKEYS        :  CfgKeybstack        ();  break;
    case CRC_STARTECHO        :  CfgAreastart        ();  break;
    case CRC_SWAPALL          :  CfgDosswap          ();  break;
    case CRC_SYSOP            :  CfgUsername         ();  break;
    case CRC_TABSIZE          :  CfgDisptabsize      ();  break;
    case CRC_TIMESLICE        :  CfgTimeslice        ();  break;
    case CRC_UNDELETELINES    :  CfgEditundelete     ();  break;
    case CRC_XPLIST           :  CfgCrosspostlist    ();  break;
    default                   :  return FALSE;
  }

  return TRUE;
}
               

//  ------------------------------------------------------------------

static int do_if(char* val) {

  if(strieql(val, "OS/2") OR strieql(val, "OS2")) {
    #ifdef __OS2__
    return TRUE;
    #else
    return FALSE;
    #endif
  }
  else if(strieql(val, "NT") OR strieql(val, "W32") OR strieql(val, "WIN32")) {
    #ifdef __WIN32__
    return TRUE;
    #else
    return FALSE;
    #endif
  }
  else if(strieql(val, "386")) {
    #if defined(__MSDOS__) and defined(__386__)
    return TRUE;
    #else
    return FALSE;
    #endif
  }
  else if(strieql(val, "DOS")) {
    #ifdef __MSDOS__
    return TRUE;
    #else
    return FALSE;
    #endif
  }
  else if(strieql(val, "LINUX") or strieql(val, "UNIX")) {
    #ifdef __UNIX__
    return TRUE;
    #else
    return FALSE;
    #endif
  }
  else if(strieql(val, "INOS2")) {
    #ifdef __OS2__
    return TRUE;
    #else
    return _osmajor >= 10;
    #endif
  }
  else if(strieql(val, "FIREBIRD")) {
    return TRUE;
  }
  else if(strieql(val, "YES") OR strieql(val, "TRUE") OR strieql(val, "ON"))
    return TRUE;
  return !!atoi(val);
}


//  ------------------------------------------------------------------
//  Display a "twirly"

#if 0
     
#define TWIRLY_FACTOR 25
#define TWIRLY(n) if((((n)%TWIRLY_FACTOR)==0) AND quiet) twirly()

void twirly() {

  static int n = 0;

  switch((n++)%4) {
    case 0:  fputs("|\b", stderr);  break;
    case 1:  fputs("/\b", stderr);  break;
    case 2:  fputs("-\b", stderr);  break;
    case 3:  fputs("\\\b",stderr);  break;
  }
}
#endif


//  ------------------------------------------------------------------
//  Compile a GoldED text config file

int ReadCfg(const char* cfgfile, int ignoreunknown) {

  const word CRC_APP      = 0x08B5;

  FILE* fp;
  Path cfg;
  char buf[1024];
  static int __FAR inuse = 0;
  static int __FAR in_if = FALSE;
  static int __FAR in_else = FALSE;
  static int __FAR if_status = FALSE;
  char* val;
  char* key;
  char* cfgname;
  word crc;
  int cfgignore=NO, line=0;

  // Open the file
  strcpy(cfg, cfgfile);
  MakePathname(cfg, CFG->goldpath, cfg);

  fp = fsopen(cfg, "rt", CFG->sharemode);
  if(fp) {

    AddStamp(cfg);

    cfgname = strrchr(cfg, '\\');
    cfgname = cfgname ? cfgname+1 : cfg;
    if(inuse == 0) {

      // Working ...
      printf(" ... ");
    }
    inuse++;

    if(NOT quiet)
      printf(NL "* Reading %s", cfg);

    // Assign file buffer
    setvbuf(fp, NULL, _IOFBF, 8192);

    // Read each line
    while(fgets((val=buf), sizeof(buf), fp)) {

      #if 0
      TWIRLY(line);
      #endif

      line++;

      // Replace TABs with SPACEs
      strchg(val, '\t', ' ');

      fastSkipWht(val);

      if(isalpha(*val)) {

        // Extract key and val
        crc = getkeyvalcrc(&key, &val);

        int _gotcond = TRUE;
        switch(crc) {
          case CRC_IF:
            if(in_if) {
              printf(NL "* %s: Misplaced IF at line %u. IF's cannot be nested.", cfgname, line);
              cfgerrors++;
            }
            in_if = TRUE;
            if_status = do_if(val);
            cfgignore = !if_status;
            break;
          case CRC_ELIF:
          case CRC_ELSEIF:
            if((NOT in_if) OR in_else) {
              printf(NL "* %s: Misplaced ELIF/ELSEIF at line %u.", cfgname, line);
              cfgerrors++;
            }
            if(if_status)
              cfgignore = TRUE;
            else {
              if_status = do_if(val);
              cfgignore = !if_status;
            }
            break;
          case CRC_ELSE:
            if((NOT in_if) OR in_else) {
              printf(NL "* %s: Misplaced ELSE at line %u.", cfgname, line);
              cfgerrors++;
            }
            in_else = TRUE;
            if_status ^= TRUE;
            cfgignore = !if_status;
            break;
          case CRC_ENDIF:
            if(NOT in_if) {
              printf(NL "* %s: Misplaced ENDIF at line %u.", cfgname, line);
              cfgerrors++;
            }
            cfgignore = FALSE;
            in_if = in_else = FALSE;
            break;
          case CRC_IGNORE:
            cfgignore ^= TRUE;
            break;
          default:
            _gotcond = FALSE;
        }

        // Tell the world what we found
        if(veryverbose)
          printf(NL " %c%04u: %s %s", cfgignore ? '-' : '+', line, key, val);

        // Call switch function to act on the key
        if(NOT cfgignore) {
          if(NOT _gotcond) {
            switch(crc) {
              case CRC_INCLUDE:
                if(NOT quiet)
                  printf(NL "* Including %s", val);
                ReadCfg(val);          // NOTE! This is a recursive call!
                if(NOT quiet)
                  printf(NL "* Resuming %s", cfg);
                break;
              case CRC_AREAFILE:
                if(NOT quiet)
                  printf(NL "* Handling %s %s", key, val);
                AL.GetAreafile(val);
                if(NOT quiet)
                  printf(NL "* Resuming %s", cfg);
                break;
              case CRC_APP:
                // Ignore 3rd party application lines
                break;
              case CRC_REM:
                // Ignore remark lines
                break;
              default:
                if(NOT SwitchCfg(crc, *key, val)) {
                  if(NOT ignoreunknown) {
                    printf(NL "* %s: Unknown keyword \"%s\" at line %u.", cfgname, key, line);
                    if(NOT disablesound)
                      beep();
                    cfgerrors++;
                  }
                }
            }
          }
        }
      }
    }

    // GOLDRAND.CFG is no longer read separately, so if there were
    // no groups and a GOLDRAND.CFG exists, include it from here
    // after reading the main configuration.
    if(inuse == 1) {
      MakePathname(CFG->randcfg.fn, CFG->goldpath, CFG->randcfg.fn);
      if(NOT cfggroups AND fexist(CFG->randcfg.fn)) {
        ReadCfg(CFG->randcfg.fn);   // NOTE! This is a recursive call!
        cfggroups = -1;             // Tell us that we've been here ...
      }
    }

    // Close and return all OK
    fclose(fp);
    inuse--;

    // When the final cfg is compiled
    if(inuse == 0) {

      // sort the group items and close the file
      GRP->Sort();
      grpfp.fclose();

      // Mark all areas listed in the NEWSRC file as newsgroups
      gfile fp;
      fp.fopen(CFG->soupnewsrcfile, "rt");
      if(fp.isopen()) {
        AddStamp(CFG->soupnewsrcfile);
        char buf[512];
        while(fp.fgets(buf, sizeof(buf))) {
          char* ptr = strpbrk(buf, ":! ");
          if(ptr) {
            *ptr = NUL;
            Area* ap = AL.AreaEchoToPtr(buf);
            if(ap)
              ap->set_type(ap->isnet() ? AT_SOUP|AT_EMAIL|AT_NET : AT_SOUP|AT_NEWSGROUP|AT_ECHO);
          }
        }
        fp.fclose();
      }

      if(*CFG->soupemail) {
        Area* ap = AL.AreaEchoToPtr(buf);
        if(ap)
          ap->set_type(AT_SOUP|AT_EMAIL|AT_NET);
      }

      for(int n=0; n<CFG->mailinglists; n++) {
        Area* ap = AL.AreaEchoToPtr(CFG->mailinglist[n].echoid);
        if(ap)
          ap->set_type(AT_SOUP|AT_EMAIL|AT_NET);
      }

      // Mark all QWK areas
      if(QWK->FirstBBS()) {
        do {
          if(QWK->FirstConf()) {
            do {
              Area* ap = AL.AreaEchoToPtr(buf);
              if(ap)
                ap->set_type(ap->type() | AT_QWK);
            } while(QWK->NextConf());
          }
        } while(QWK->NextBBS());
      }
    }

    return 1;
  }
  else {

    //printf(NL "* Unable to open %s.", cfg);
    LOG.ErrOpen();
    LOG.printf("! Unable to configuration file.");
    LOG.printf(": %s", cfg);
    LOG.ErrOSInfo();
    OpenErrorExit();
  }

  // Error opening file
  return 0;
}


//  ------------------------------------------------------------------

