
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gccfgg4.cpp,v 1.1.1.1 1998/11/21 19:38:01 odinn Exp $
//  ------------------------------------------------------------------
//  Configuration functions, part 4.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include "gcprot.h"


//  ------------------------------------------------------------------

extern char* val;


//  ------------------------------------------------------------------

void CfgExcludenodes() {

  // Only used by GoldNODE
}

//  ------------------------------------------------------------------
              
void CfgExternoptions() {

  const word CRC_CLS            = 0x34F4;
  const word CRC_SWAP           = 0x30AE;
  const word CRC_CURSOR         = 0x0D67;
  const word CRC_RELOAD         = 0xDEC0;
  const word CRC_PAUSE          = 0x3926;
  const word CRC_PAUSEONERROR   = 0xD99A;
  const word CRC_WIPE           = 0x1E80;
  const word CRC_KEEPCTRL       = 0x0738;
  const word CRC_NOCLS          = 0x2FAA;
  const word CRC_NOSWAP         = 0xCDF4;
  const word CRC_NOCURSOR       = 0x9345;
  const word CRC_NORELOAD       = 0x40E2;
  const word CRC_NOPAUSE        = 0x5D94;
  const word CRC_NOPAUSEONERROR = 0xDBAA;
  const word CRC_NOWIPE         = 0xE3DA;
  const word CRC_NOKEEPCTRL     = 0x38FC;

  // Handle options
  while(*val) {

    if((*val == '-') OR (*val == '/'))
      val++;
    
    char* _key;
    word _crc = getkeyvalcrc(&_key, &val);

    switch(_crc) {

      case CRC_CLS:
        CFG->externoptions |= EXTUTIL_CLS;
        break;
      case CRC_SWAP:
        CFG->externoptions |= EXTUTIL_SWAP;
        break;
      case CRC_CURSOR:
        CFG->externoptions |= EXTUTIL_CURSOR;
        break;
      case CRC_RELOAD:
        CFG->externoptions |= EXTUTIL_RELOAD;
        break;
      case CRC_PAUSE:
        CFG->externoptions |= EXTUTIL_PAUSE;
        break;
      case CRC_PAUSEONERROR:
        CFG->externoptions |= EXTUTIL_PAUSEONERROR;
        break;
      case CRC_WIPE:
        CFG->externoptions |= EXTUTIL_WIPE;
        break;
      case CRC_KEEPCTRL:
        CFG->externoptions |= EXTUTIL_KEEPCTRL;
        break;
      case CRC_NOCLS:
        CFG->externoptions &= ~EXTUTIL_CLS;
        break;
      case CRC_NOSWAP:
        CFG->externoptions &= ~EXTUTIL_SWAP;
        break;
      case CRC_NOCURSOR:
        CFG->externoptions &= ~EXTUTIL_CURSOR;
        break;
      case CRC_NORELOAD:
        CFG->externoptions &= ~EXTUTIL_RELOAD;
        break;
      case CRC_NOPAUSE:
        CFG->externoptions &= ~EXTUTIL_PAUSE;
        break;
      case CRC_NOPAUSEONERROR:
        CFG->externoptions &= ~EXTUTIL_PAUSEONERROR;
        break;
      case CRC_NOWIPE:
        CFG->externoptions &= ~EXTUTIL_WIPE;
        break;
      case CRC_NOKEEPCTRL:
        CFG->externoptions &= ~EXTUTIL_KEEPCTRL;
        break;
    }
  }
}


//  ------------------------------------------------------------------

void CfgExternutil() {
             
  // Allocate space for new utility data
  CFG->externutil = (ExtUtil*)throw_realloc(CFG->externutil, (CFG->externutils+1)*sizeof(ExtUtil));

  // Pointer to the new util data
  ExtUtil* _extutil = CFG->externutil + CFG->externutils;
  memset(_extutil, 0, sizeof(ExtUtil));

  // Get util number
  char* _key;
  char* _val = val;
  getkeyval(&_key, &_val);
  _extutil->utilno = atoi(_key);

  // Get options
  int _optbak = CFG->externoptions;
  while(strchr("/-", *_val)) {
    getkeyval(&_key, &_val);
    val = _key;
    CfgExternoptions();
  }
  _extutil->options = CFG->externoptions;
  CFG->externoptions = _optbak;

  // Get commandline
  strxcpy(_extutil->cmdline, _val, sizeof(_extutil->cmdline));

  // Count it
  CFG->externutils++;
}

//  ------------------------------------------------------------------

void CfgEzycommsgbase() {

  PathCopy(CFG->ezycom.msgbasepath, val);
}

//  ------------------------------------------------------------------

void CfgEzycomuserbase() {

  PathCopy(CFG->ezycom.userbasepath, val);
}

//  ------------------------------------------------------------------

void CfgEzycomuserno() {

  CFG->ezycomuserno = atoi(val);
}

//  ------------------------------------------------------------------

void CfgFidolastread() {

  strcpy(CFG->fidolastread, val);
}

//  ------------------------------------------------------------------

void CfgFidomsgtype() {

  if(striinc("FTS", val))
    CFG->fidomsgtype = MT_FTS1;
  else
    CFG->fidomsgtype = MT_OPUS;
}

//  ------------------------------------------------------------------

void CfgFidouserlist() {

  strcpy(CFG->fidouserlist, val);
}

//  ------------------------------------------------------------------

void CfgFidouserno() {

  CFG->fidouserno = atoi(val);
}

//  ------------------------------------------------------------------

void CfgFilealias() {

  CFG->filealias = (FileAlias*)throw_realloc(CFG->filealias, (CFG->filealiases+1)*sizeof(FileAlias));
  char* ptr = val;
  ptr = SkipTxt(ptr);
  *ptr++ = NUL;
  ptr = SkipWht(ptr);
  strxcpy(CFG->filealias[CFG->filealiases].alias, val, sizeof(CFG->filealias[CFG->filealiases].alias));
  strxcpy(CFG->filealias[CFG->filealiases].file, ptr, GMAXPATH);
  CFG->filealiases++;
}

//  ------------------------------------------------------------------

void CfgFilecheck() {

  AddStamp(val);
}

//  ------------------------------------------------------------------

void CfgFilecheckcall() {

  strcpy(CFG->filecheckcall, val);
}

//  ------------------------------------------------------------------

void CfgForcetemplate() {

  int _flag = GetYesno(val);
  if(cfgingroup)
    GRP->AddItm(GRP_FORCETEMPLATE, &_flag, sizeof(int));
  else
    CFG->forcetemplate = _flag;
}

//  ------------------------------------------------------------------

void CfgFreearea() {

  // Obsolete
}

//  ------------------------------------------------------------------

void CfgFrqext(char* v)  {

  char* ptr = val = throw_strdup(v);
  CfgFrqext();
  throw_free(ptr);
}

void CfgFrqext() {

  val = strtok(val, ", \t");
  while(val) {

    // Do we have it already?
    for(int n=0; n<CFG->frqexts; n++)
      if(strieql(val, CFG->frqext[n]))
        return;

    // Nope, so add it
    CFG->frqext = (FrqExt*)throw_realloc(CFG->frqext, (CFG->frqexts+1)*sizeof(FrqExt));
    strxcpy(CFG->frqext[CFG->frqexts], val, sizeof(FrqExt));
    CFG->frqexts++;

    val = strtok(NULL, ", \t");
  }
}

//  ------------------------------------------------------------------

void CfgFrqnodemap() {

  #if 0
  GTok t;
  if(t.First(val)) {
    CFG->frqnodemap.to_node(t.Token());
    while(t.Next())
      CFG->frqnodemap.from_node(t.Token());
  }
  #endif

  char* key;
  FrqNodeMap fnm;
  getkeyval(&key, &val);
  fnm.to.reset(key);
  getkeyval(&key, &val);
  fnm.from.reset(key);

  CFG->frqnodemap = (FrqNodeMap*)throw_realloc(CFG->frqnodemap, (CFG->frqnodemaps+1)*sizeof(FrqNodeMap));
  CFG->frqnodemap[CFG->frqnodemaps] = fnm;
  CFG->frqnodemaps++;
}

//  ------------------------------------------------------------------

void CfgFrqoptions() {

  if(striinc("SORT", val))
    CFG->frqoptions |= FREQ_SORT;
  if(striinc("FROMTOP", val))
    CFG->frqoptions |= FREQ_FROMTOP;
  if(striinc("NOFILES", val))
    CFG->frqoptions |= FREQ_NOFILES;
  if(striinc("FAST", val))
    CFG->frqoptions |= FREQ_FAST;
  if(striinc("NOWAZOOMSG", val))
    CFG->frqoptions |= FREQ_NOWAZOOMSG;
  if(striinc("NOTFROMTOP", val))
    CFG->frqoptions &= ~FREQ_FROMTOP;
}

//  ------------------------------------------------------------------

void CfgGif() {

  Gifname buf;
  strxcpy(buf, val, sizeof(buf));
  char* ptr = strchr(buf, '.');
  if(ptr)
    *ptr = NUL;
  if(cfgingroup)
    GRP->AddItm(GRP_GIF, buf, strlen(buf)+1);
  else {
    CFG->gif = (Gifname*)throw_realloc(CFG->gif, (CFG->gifs+1)*sizeof(Gifname));
    strxcpy(CFG->gif[CFG->gifs], val, sizeof(Gifname));
    CFG->gifs++;
  }
}

//  ------------------------------------------------------------------

void CfgGifpath() {

  PathCopy(CFG->gifpath, val);
}

//  ------------------------------------------------------------------

void CfgGoldbasepath() {

  PathCopy(CFG->goldbasepath, val);
}

//  ------------------------------------------------------------------

void CfgGoldbasesyspath() {

  PathCopy(CFG->goldbasesyspath, val);
}

//  ------------------------------------------------------------------

void CfgGoldbaseuserno() {

  CFG->goldbaseuserno = atoi(val);
}

//  ------------------------------------------------------------------

void CfgGoldhelp() {

  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  _splitpath(val, drive, dir, file, ext);
  _makepath(CFG->helpcfg.fn, drive, dir, file, ext);
  _makepath(CFG->helpged, drive, dir, file, __gver_cfgext__);
}

//  ------------------------------------------------------------------

void CfgGoldkeys() {

  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  _splitpath(val, drive, dir, file, ext);
  _makepath(CFG->keyscfg.fn, drive, dir, file, ext);
}

//  ------------------------------------------------------------------

void CfgGoldlang() {

  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  _splitpath(val, drive, dir, file, ext);
  _makepath(CFG->langcfg.fn, drive, dir, file, ext);
  _makepath(CFG->langged, drive, dir, file, __gver_cfgext__);
}

//  ------------------------------------------------------------------

void CfgGoldpath() {

  PathCopy(CFG->goldpath, val);
}

//  ------------------------------------------------------------------

void CfgGoldrand() {

  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  _splitpath(val, drive, dir, file, ext);
  _makepath(CFG->randcfg.fn, drive, dir, file, ext);
  _makepath(CFG->randged, drive, dir, file, __gver_cfgext__);
}

//  ------------------------------------------------------------------

void CfgGoldxlat() {

  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  _splitpath(val, drive, dir, file, ext);
  _makepath(CFG->xlatged, drive, dir, file, __gver_cfgext__);
}

//  ------------------------------------------------------------------

void CfgGroup() {

  char* _key;
  getkeyval(&_key, &val);
  GRP->AddGrp(_key);
  cfgingroup = TRUE;
  cfggroups++;

  // Open the GOLDRAND.GE? file when the first group is found
  if((cfggroups == 1) AND NOT grpfp.isopen()) {
    grpfp.fopen(AddPath(CFG->goldpath, CFG->randged), "wb", CFG->sharemode);
    if(grpfp.isopen()) {
      GRP->SetFp(grpfp);
    }
  }
}

//  ------------------------------------------------------------------

