
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gccfgg5.cpp,v 1.4 1999/01/29 03:06:47 damware Exp $
//  ------------------------------------------------------------------
//  Configuration functions, part 5.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include "gccfgg.h"
#include "gcprot.h"


//  ------------------------------------------------------------------

extern char* val;


//  ------------------------------------------------------------------

void CfgHappybirthday() {

  if(strieql(val, "FRIEND"))
    CFG->happybirthday = TRUE;
}

//  ------------------------------------------------------------------

void CfgHudsonpath() {

  PathCopy(CFG->hudsonpath, val);
}

//  ------------------------------------------------------------------

void CfgHudsonsizewarn() {

  CFG->hudsonsizewarn = atol(val);
}

//  ------------------------------------------------------------------

void CfgHudsonsyspath() {

  PathCopy(CFG->hudsonsyspath, val);
}

//  ------------------------------------------------------------------

void CfgHudsonuserno() {

  CFG->hudsonuserno = atoi(val);
}

//  ------------------------------------------------------------------

void CfgImportbegin() {

  strxcpy(CFG->importbegin, StripQuotes(val), sizeof(CFG->importbegin));
}

//  ------------------------------------------------------------------

void CfgImportend() {

  strxcpy(CFG->importend, StripQuotes(val), sizeof(CFG->importend));
}

//  ------------------------------------------------------------------

void CfgInboundpath() {

  PathCopy(CFG->inboundpath, val);
}

//  ------------------------------------------------------------------

void CfgIncludenodes() {

  // Only used by GoldNODE
}

//  ------------------------------------------------------------------

void CfgInputfile() {

  Path buf;
  strxcpy(buf, val, sizeof(buf));
  if(cfgingroup)
    GRP->AddItm(GRP_INPUTFILE, buf, strlen(buf)+1);
  else
    strcpy(CFG->inputfile, buf);
}

//  ------------------------------------------------------------------

void CfgIntensecolors() {

  CFG->intensecolors = GetYesno(val);
  if(CFG->intensecolors)
    memcpy(CFG->color, gold_color2, sizeof(CFG->color));
}

//  ------------------------------------------------------------------

void CfgInternetaddress() {

  IAdr buf;
  strxcpy(buf, val, sizeof(buf));
  if(cfgingroup)
    GRP->AddItm(GRP_INTERNETADDRESS, buf, strlen(buf)+1);
  else
    strcpy(CFG->internetaddress, buf);
}

//  ------------------------------------------------------------------

void CfgInternetdomain() {

  strxcpy(CFG->internetdomain, val, sizeof(CFG->internetdomain));
}

//  ------------------------------------------------------------------

void CfgInternetgate() {

  Node tmp;
  memset(&tmp, 0, sizeof(Node));
  char* ptr = strchr(val, ',');
  if(ptr) {
    *ptr++ = NUL;
    strxcpy(tmp.name, strtrim(val), sizeof(Name));
  }
  else {
    ptr = val;
  }
  if(CFG->akas)
    tmp.addr = CFG->aka[0].addr;
  tmp.addr.set(SkipWht(ptr));
  if(cfgingroup)
    GRP->AddItm(GRP_INTERNETGATE, &tmp, sizeof(Node));
  else
    CFG->internetgate = tmp;
}

//  ------------------------------------------------------------------

void CfgInternetmsgid() {

  int _flag = GetYesno(val);
  if(cfgingroup)
    GRP->AddItm(GRP_INTERNETMSGID, &_flag, sizeof(int));
  else
    CFG->internetmsgid = _flag;
}

//  ------------------------------------------------------------------

void CfgInternetrfcbody() {

  int _flag = GetYesno(val);
  if(cfgingroup)
    GRP->AddItm(GRP_INTERNETRFCBODY, &_flag, sizeof(int));
  else
    CFG->internetrfcbody = _flag;
}

//  ------------------------------------------------------------------

void CfgInternetserver() {

  GTok t;
  if(t.First(val)) {
    int server = 0;
    if(strieql(t.Token(), "NNTP"))
      server = 1;
    else if(strieql(t.Token(), "SMTP"))
      server = 2;
    else if(strieql(t.Token(), "POP3"))
      server = 3;
    int parameter = 1;
    while(t.Next()) {
      if(server == 1)
        strxcpy(CFG->internetserver.nntp.server, t.Token(), sizeof(CFG->internetserver.nntp.server));
      else if(server == 2)
        strxcpy(CFG->internetserver.smtp.server, t.Token(), sizeof(CFG->internetserver.smtp.server));
      else if(server == 3) {
        if(parameter == 1)
          strxcpy(CFG->internetserver.pop3.server, t.Token(), sizeof(CFG->internetserver.pop3.server));
        else if(parameter == 2)
          strxcpy(CFG->internetserver.pop3.username, t.Token(), sizeof(CFG->internetserver.pop3.username));
        else if(parameter == 3)
          strxcpy(CFG->internetserver.pop3.password, t.Token(), sizeof(CFG->internetserver.pop3.password));
      }
      parameter++;
    }
  }
}

//  ------------------------------------------------------------------

void CfgInvalidate() {

  const word CRC_SEENBY = 0x01D0;

  char* key;

  switch(getkeyvalcrc(&key, &val)) {
    case CRC_TEARLINE:
      getkeyval(&key, &val);
      strxcpy(CFG->invalidate.tearline[0], StripQuotes(key), sizeof(*CFG->invalidate.tearline));
      strxcpy(CFG->invalidate.tearline[1], StripQuotes(val), sizeof(*CFG->invalidate.tearline));
      break;
    case CRC_ORIGIN:
      getkeyval(&key, &val);
      strxcpy(CFG->invalidate.origin[0], StripQuotes(key), sizeof(*CFG->invalidate.origin));
      strxcpy(CFG->invalidate.origin[1], StripQuotes(val), sizeof(*CFG->invalidate.origin));
      break;
    case CRC_SEENBY:
      getkeyval(&key, &val);
      strxcpy(CFG->invalidate.seenby[0], StripQuotes(key), sizeof(*CFG->invalidate.seenby));
      strxcpy(CFG->invalidate.seenby[1], StripQuotes(val), sizeof(*CFG->invalidate.seenby));
      break;
  }
}

//  ------------------------------------------------------------------

void CfgJampath() {

  PathCopy(CFG->jampath, val);
}

//  ------------------------------------------------------------------

void CfgKeybmode() {

  if(strnieql(val, "Poll", 4))
    CFG->keybmode = KEYB_POLL;
  else if(strnieql(val, "Block", 5))
    CFG->keybmode = KEYB_BLOCK;
}

//  ------------------------------------------------------------------

void CfgKeybstack() {

  strcat(CFG->keybstack, val);
}

//  ------------------------------------------------------------------

void CfgKludge() {

  StripQuotes(val);
  if(*val == CTRL_A)
    val++;
  CFG->kludge = (Kludge*)throw_realloc(CFG->kludge, (CFG->kludges+1)*sizeof(Kludge));
  strxcpy(CFG->kludge[CFG->kludges], val, sizeof(Kludge));
  CFG->kludges++;
}

//  ------------------------------------------------------------------

void CfgLoadlanguage() {

  Path buf;
  strxcpy(buf, val, sizeof(buf));
  if(cfgingroup)
    GRP->AddItm(GRP_LOADLANGUAGE, buf, strlen(buf)+1);
  else
    strcpy(CFG->loadlanguage, buf);
}

//  ------------------------------------------------------------------

void CfgLogfile() {

  strcpy(CFG->logfile, val);
}

//  ------------------------------------------------------------------

void CfgLogformat() {

  if(strieql(val, "FD"))
    CFG->logformat = GLOG_FD;
  else if(strieql(val, "MAX"))
    CFG->logformat = GLOG_MAX;
  else if(strieql(val, "BINK"))
    CFG->logformat = GLOG_BINK;
  else if(strieql(val, "QBBS"))
    CFG->logformat = GLOG_QBBS;
  else if(strieql(val, "DB"))
    CFG->logformat = GLOG_DB;
}

//  ------------------------------------------------------------------

void CfgMailinglist() {

  char* key;
  CFG->mailinglist = (MailList*)throw_realloc(CFG->mailinglist, (CFG->mailinglists+1)*sizeof(MailList));
  MailList* mlstp = CFG->mailinglist + CFG->mailinglists++;
  mlstp->sender_is_pattern = false;
  getkeyval(&key, &val);
  strxcpy(mlstp->echoid, key, sizeof(Echo));
  getkeyval(&key, &val);
  strxcpy(mlstp->sender, key, sizeof(IAdr));
  getkeyval(&key, &val);
  strxcpy(mlstp->contribution, key, sizeof(IAdr));
}

//  ------------------------------------------------------------------

void CfgMailtoss() {

  char* key;
  CFG->mailinglist = (MailList*)throw_realloc(CFG->mailinglist, (CFG->mailinglists+1)*sizeof(MailList));
  MailList* mlstp = CFG->mailinglist + CFG->mailinglists++;
  mlstp->sender_is_pattern = true;
  getkeyval(&key, &val);
  strxcpy(mlstp->echoid, key, sizeof(Echo));
  getkeyval(&key, &val);
  strxcpy(mlstp->contribution, key, sizeof(IAdr));
  strxcpy(mlstp->sender, val, sizeof(IAdr));
}

//  ------------------------------------------------------------------

void CfgMapdrive() {

  CfgMappath();
}

//  ------------------------------------------------------------------

void CfgMappath() {

  char* key;

  getkeyval(&key, &val);

  CFG->mappath = (MapPathE*)throw_realloc(CFG->mappath, ((CFG->mappaths+1)*2)*sizeof(MapPathE));
  strxcpy(CFG->mappath[CFG->mappaths*2  ], key, sizeof(MapPathE));
  strxcpy(CFG->mappath[CFG->mappaths*2+1], val, sizeof(MapPathE));

  CFG->mappaths++;
}

//  ------------------------------------------------------------------

void CfgMember() {

  if(cfgingroup) {
    val = strtok(val, ", \t");
    while(val) {
      GRP->AddMbr(val);
      val = strtok(NULL, ", \t");
    }
  }
}

//  ------------------------------------------------------------------

void CfgMenumarked() {

  if(strieql(val, "Marked"))
    CFG->menumarked = MODE_MARKED;
  else if(strieql(val, "Current"))
    CFG->menumarked = MODE_CURRENT;
  else if(strieql(val, "Previous"))
    CFG->menumarked = MODE_PREVIOUS;
  else
    CFG->menumarked = MODE_DONT;
}

//  ------------------------------------------------------------------

void CfgMsglistdate() {

  int flag;
  if(strieql(val, "Written"))
    flag = MSGLISTDATE_WRITTEN;
  else if(strieql(val, "Arrived"))
    flag = MSGLISTDATE_ARRIVED;
  else if(strieql(val, "Received"))
    flag = MSGLISTDATE_RECEIVED;
  else
    flag = GetYesno(val) ? MSGLISTDATE_WRITTEN : MSGLISTDATE_NONE;
  if(cfgingroup)
    GRP->AddItm(GRP_MSGLISTDATE, &flag, sizeof(int));
  else
    CFG->msglistdate = flag;
}

//  ------------------------------------------------------------------

void CfgMsglistfast() {

  int _flag = GetYesno(val);
  if(cfgingroup)
    GRP->AddItm(GRP_MSGLISTFAST, &_flag, sizeof(int));
  else
    CFG->msglistfast = _flag;
}

//  ------------------------------------------------------------------

