
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gectrl.cpp,v 1.5 1999/01/29 20:49:04 damware Exp $
//  ------------------------------------------------------------------
//  Random system and control info handling.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include <gutlcode.h>


//  ------------------------------------------------------------------

int _use_fwd = true;


//  ------------------------------------------------------------------

char* MakeOrigin(GMsg* msg, const char* orig, const char* netn) {

  int len;
  char buf[256];
  char origin[100];
  char netname[sizeof(Netn)+1];

  strcpy(origin, orig);
  strcpy(netname, netn);

  if(msg->orig.net)
    msg->orig.make_string(buf);
  else
    AA->Aka().addr.make_string(buf);

  strbtrim(netname);
  if(*netname)
    strcat(netname, " ");
  len = 11 + strlen(origin) + 1 + 1 + strlen(netname) + strlen(buf) + 1;
  if(len > 79) {
    len -= strlen(netname);
    strsetsz(origin, strlen(origin)-(len-79));  // Chop the originline to max 79 chars
    strtrim(origin);
    *netname = NUL;
  }

  if(*origin == '@')
    GetRandomLine(origin, sizeof(origin), origin+1);

  sprintf(msg->origin, "%s (%s%s)", origin, netname, buf);
  return msg->origin;
}


//  ------------------------------------------------------------------

char* MakeTearline(GMsg* msg, char* buf) {

  if(*msg->tearline == '@')
    GetRandomLine(msg->tearline, sizeof(msg->tearline), msg->tearline+1);

  strcpy(buf, "--- ");
  strcat(buf, strbtrim(msg->tearline));

  return strtrim(buf);
}


//  ------------------------------------------------------------------

void MakeFlags(GMsg* msg, Line** line, char* buf) {

  // The FrontDoor FLAGS netmail kludge
  if(CFG->switches.get(useflags)) {
    sprintf(buf, "\001FLAGS ");
    if(AA->ishudson()) {
      if(msg->attr.hld())
        strcat(buf, "HLD ");
      if(msg->attr.frq())
        strcat(buf, "FRQ ");
    }
    if(msg->attr.imm())
      strcat(buf, "IMM ");
    if(msg->attr.dir())
      strcat(buf, "DIR ");
    if(msg->attr.tfs())
      strcat(buf, "TFS ");
    if(msg->attr.kfs())
      strcat(buf, "KFS ");
    if(msg->attr.lok())
      strcat(buf, "LOK ");
    if(msg->attr.a_s())
      strcat(buf, "A/S ");
    if(msg->attr.zon())
      strcat(buf, "ZON ");
    if(msg->attr.hub())
      strcat(buf, "HUB ");
    if(msg->attr.xma())
      strcat(buf, "XMA ");
    if(msg->attr.cfm())
      strcat(buf, "CFM ");
    if(msg->attr.hir())
      strcat(buf, "HIR ");
    if(msg->attr.cov())
      strcat(buf, "COV ");
    if(msg->attr.sig())
      strcat(buf, "SIG ");
    if(msg->attr.let())
      strcat(buf, "LET ");
    if(msg->attr.fax())
      strcat(buf, "FAX ");
    if(strlen(buf) > 7) {
      buf[strlen(buf)-1] = 0;
      *line = AddKludge(*line, buf);
      (*line)->kludge = GKLUD_FLAGS;
    }
  }
}


//  ------------------------------------------------------------------

char* mime_header_encode(char* dest, const char* source) {

  bool inmime = false;
  char* bp = dest;

  const char* lp = source + strlen(source) - 1;

  while((lp>source) and in_range((char)*(lp-1), (char)' ', (char)'\x7F'))
    lp--;

  for(const char* ptr = source; *ptr and (ptr<lp); ptr++) {
    if((*ptr < ' ') or (*ptr > '\x7F') or (inmime and strchr(" =", *ptr))) {
      if(not inmime) {
        strcpy(bp, "=?iso-8859-1?Q?");
        bp += 15;
        inmime = true;
      }
      sprintf(bp, "=%Xc", *ptr);
      bp += 3;
    }
    else
      *bp++ = *ptr;
  }

  if(inmime) {
    strcpy(bp, "?=");
    bp += 2;
  }

  return strcpy(bp, lp);
}


//  ------------------------------------------------------------------

void DoKludges(int mode, GMsg* msg, bool attronly) {

  char* buf = (char*)throw_malloc(4096);
  char buf2[256];
  Line* line = msg->lin;
  Line* newline;
  
  // Insert empty line at the top for practical purposes

  newline = (Line*)throw_xcalloc(1, sizeof(Line));
  newline = line = InsertLine(newline, line, DIR_PREV);

  // Strip all the kludges we insert ourselves

  while(line) {

    int stripkludges = attronly ? GKLUD_FLAGS : (GKLUD_RFC|GKLUD_FWD|GKLUD_INTL|GKLUD_FMPT|GKLUD_TOPT|GKLUD_FLAGS|GKLUD_AREA|GKLUD_MSGID|GKLUD_REPLY|GKLUD_PID|GKLUD_CHARSET|GKLUD_GIF|(mode==MODE_FORWARD?(GKLUD_PATH|GKLUD_SEENBY):0));

    if(line->kludge & stripkludges) {
      int waswrapped;
      do {
        waswrapped = !!(line->type & GLINE_WRAP);
        line = DeleteLine(line);
      } while(line AND waswrapped);
    }
    else {
      line = line->next;
    }
  }

  line = newline;

  if(attronly) {
    if(AA->isnet())
      MakeFlags(msg, &line, buf);
  }
  else {

    if(not AA->isnet() and msg->areakludgeid) {
      sprintf(buf, "\001AREA:%s", msg->areakludgeid);
      line = AddKludge(line, buf);
      line->kludge = GKLUD_AREA;
    }

    if(AA->isnet()) {

      // The INTL kludge for zone crossing
      if(CFG->useintl AND (CFG->useintl == YES OR (msg->dest.zone != msg->orig.zone))) {
        sprintf(buf, "\001INTL %u:%u/%u %u:%u/%u",
          msg->dest.zone ? msg->dest.zone : AA->Aka().addr.zone,
          msg->dest.net, msg->dest.node,
          msg->orig.zone ? msg->orig.zone : AA->Aka().addr.zone,
          msg->orig.net, msg->orig.node
        );
        line = AddKludge(line, buf);
        line->kludge = GKLUD_INTL;
      }

      // The FMPT and TOPT kludges for point addressing
      if(msg->dest.point) {
        sprintf(buf, "\001TOPT %u", msg->dest.point);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_TOPT;
      }
      if(msg->orig.point AND msg->orig.net == msg->oorig.net AND msg->orig.node == msg->oorig.node) {
        sprintf(buf, "\001FMPT %u", msg->orig.point);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_FMPT;
      }

      MakeFlags(msg, &line, buf);
    }

    // The MSGID: kludge for Origination identification
    if(AA->Internetmsgid() AND *msg->iorig AND AA->isinternet()) {
      // User configured Internetdomain, use it
      if(*CFG->internetdomain) {
        strcpy(buf, CFG->internetdomain);
      }
      else {
        // Fallback method. Yes, its "Crosspoint"-style. Yes, it's not
        // Son-Of-RFC1036-compatible. Yes, I don't care.
        strcpy(buf, msg->iorig);
        strchg(buf, '@', '%');
      }
      sprintf(buf2, "<GED%08lX@%s>", gtime(NULL)+(msgcount++), buf);
      throw_release(msg->messageid);
      msg->messageid = throw_strdup(buf2);
      CvtMessageIDtoMSGID(buf2, buf, AA->echoid(), "MSGID");
      strcpy(msg->msgids, buf+8);
    }
    else {
      msg->orig.make_string(buf2, msg->odom);
      sprintf(msg->msgids, "%s %08lx", buf2, gtime(NULL)+(msgcount++));        
    }
    if(CFG->switches.get(usemsgid) and not AA->ispcboard()) {
      sprintf(buf, "\001MSGID: %s", msg->msgids);
      line = AddKludge(line, buf);
      line->kludge = GKLUD_MSGID;
    }

    // The REPLY: kludge for Receiver identification
    if(CFG->switches.get(usemsgid) AND *msg->replys) {
      sprintf(buf, "\001REPLY: %s", msg->replys);
      line = AddKludge(line, buf);
      line->kludge = GKLUD_REPLY;
    }

    // The PID: (Product ID code) kludge
    strcpy(msg->pid, __gver_shortpid__);

    if(CFG->switches.get(emptytearline) or not (striinc(__gver_longpid__, msg->tearline))) {

      sprintf(buf, "\001PID: %s", msg->pid);
      line = AddKludge(line, strtrim(buf));
      line->kludge = GKLUD_PID;
    }

    // The CHARSET kludge
    if(*msg->charset) {
      *buf = NUL;
      if(strieql(msg->charset, "I51"))
        strcpy(buf, "\001I51");
      else if(NOT strieql(msg->charset, "COMPOSED"))
        sprintf(buf, "\001%s: %s", CFG->switches.get(kludgechrs) ? "CHRS" : "CHARSET", msg->charset);
      if(*buf) {
        line = AddKludge(line, buf);
        line->kludge = GKLUD_CHARSET;
      }
    }

    // The GIF kludge
    if(*msg->gif) {
      sprintf(buf, "%s.GIF", AddPath(CFG->gifpath, msg->gif));
      if(fexist(buf)) {
        sprintf(buf, "\001GIF: %s", msg->gif);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_GIF;
      }
    }

    // The FWD* kludges
    if(_use_fwd) {
      if(*msg->fwdfrom) {
        sprintf(buf, "\001FWDFROM %s", msg->fwdfrom);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_FWD;
      }
      if(msg->fwdorig.net) {
        sprintf(buf, "\001FWDORIG %s", msg->fwdorig.make_string(buf2));
        line = AddKludge(line, buf);
        line->kludge = GKLUD_FWD;
      }
      if(*msg->fwdto) {
        sprintf(buf, "\001FWDTO %s", msg->fwdto);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_FWD;
      }
      if(msg->fwddest.net) {
        sprintf(buf, "\001FWDDEST %s", msg->fwddest.make_string(buf2));
        line = AddKludge(line, buf);
        line->kludge = GKLUD_FWD;
      }
      if(*msg->fwdsubj) {
        sprintf(buf, "\001FWDSUBJ %s", msg->fwdsubj);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_FWD;
      }
      if(*msg->fwdarea) {
        sprintf(buf, "\001FWDAREA %s", msg->fwdarea);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_FWD;
      }
      if(*msg->fwdmsgid) {
        sprintf(buf, "\001FWDMSGID %s", msg->fwdmsgid);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_FWD;
      }
    }

    if(AA->isinternet()) {

      char* rfc = AA->Internetrfcbody() ? "" : "\001";

      char to_buf[256];
      *to_buf = NUL;

      if(*msg->idest OR strchr(msg->to, '@')) {
        char* ptr = *msg->idest ? msg->idest : msg->to;
        sprintf(to_buf, "%s%sTo: %s", rfc, AA->isnewsgroup() ? "X-" : "", ptr);
        if(NOT strieql(ptr, msg->realto) AND *msg->realto) {
          mime_header_encode(buf, msg->realto);
          sprintf(buf2, " (%s)", buf);
          strcat(to_buf, buf2);
        }
      }

      if(*to_buf AND AA->isemail()) {
        line = AddKludge(line, to_buf);
        line->kludge = GKLUD_RFC;
      }

      if(*msg->iorig) {
        mime_header_encode(buf2, msg->By());
        sprintf(buf, "%sFrom: %s (%s)", rfc, msg->iorig, buf2);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      if(AA->isnewsgroup()) {
        sprintf(buf, "%sNewsgroups: %s", rfc, strlwr(strcpy(buf2, AA->echoid())));
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      if(*msg->re) {
        mime_header_encode(buf2, msg->re);
        sprintf(buf, "%sSubject: %s", rfc, buf2);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      Tm* tm = glocaltime(&msg->written);
        sprintf(buf, "%sDate: %s, %02d %s %04d %02d:%02d:%02d %c%04d", rfc,
        __gsweekday[tm->tm_wday],
        tm->tm_mday, __gsmonth[tm->tm_mon], 1900+tm->tm_year,
        tm->tm_hour, tm->tm_min, tm->tm_sec,
        (CFG->timezoneoffset < 0) ? '-' : '+', AbsV(CFG->timezoneoffset)
      );
      line = AddKludge(line, buf);
      line->kludge = GKLUD_RFC;

      if(msg->messageid) {
        sprintf(buf, "%sMessage-ID: %s", rfc, msg->messageid);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      if(msg->references) {
        sprintf(buf, "%sReferences: %s", rfc, msg->references);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      if(msg->inreplyto) {
        sprintf(buf, "%sIn-Reply-To: %s", rfc, msg->inreplyto);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      if(*msg->organization) {
        sprintf(buf, "%sOrganization: %s", rfc, msg->organization);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      if(striinc("LATIN-1", msg->charset) OR striinc("LATIN1QP", msg->charset) OR striinc("ASCII", msg->charset)) {
        sprintf(buf, "%sMIME-Version: 1.0", rfc);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
        sprintf(buf, "%sContent-Type: text/plain; charset=%s", rfc, striinc("ASCII", msg->charset) ? "us-ascii" : "iso-8859-1");
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
        sprintf(buf, "%sContent-Transfer-Encoding: %s", rfc, striinc("LATIN1QP", msg->charset) ? "quoted-printable" : striinc("ASCII", msg->charset) ? "7bit" : "8bit");
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      if(*to_buf AND AA->isnewsgroup()) {
        line = AddKludge(line, to_buf);
        line->kludge = GKLUD_RFC;
      }

      if(*msg->iorig) {
        mime_header_encode(buf2, msg->By());
        sprintf(buf, "%sSender: %s (%s)", rfc, msg->iorig, buf2);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      if((*msg->ireplyto or *CFG->soupreplyto) and (not streql(*msg->ireplyto ? msg->ireplyto : CFG->soupreplyto, msg->iorig))) {
        sprintf(buf, "%sReply-To: %s", rfc, *msg->ireplyto ? msg->ireplyto : CFG->soupreplyto);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      if(striinc("MNEMONIC", msg->charset)) {
        sprintf(buf, "%sX-Charset: ISO_8859-1", rfc);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
        sprintf(buf, "%sX-Char-Esc: 29", rfc);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      if(AA->isnewsgroup() OR AA->isemail()) {
        sprintf(buf, "%sX-%sreader: %s", rfc, AA->isnewsgroup() ? "News" : "Mail", __gver_longpid__);
        line = AddKludge(line, buf);
        line->kludge = GKLUD_RFC;
      }

      if(AA->Internetrfcbody() AND line->next AND NOT strblank(line->next->text)) {
        line = AddKludge(line, "");
        line->kludge = GKLUD_RFC;
      }
    }

    if(AA->isnet()) {
      Line* firstline = FirstLine(line);
      firstline = firstline->next;
                              // 123456789012345678901234567
      if(strneql(firstline->text, "-----BEGIN PGP MESSAGE-----", 27)) {
        line = AddKludge(line, "\001ENC: PGP");
        //line->kludge = GKLUD_MISC;
      }

      if(*msg->iaddr and not AA->isinternet()) {
        sprintf(buf, "To: %s\r", msg->iaddr);
        line = AddKludge(line, buf);
        //line->kludge = GKLUD_MISC;
      }
    }
  }

  // Reset line pointer
  msg->lin = DeleteLine(FirstLine(line));

  MsgLineReIndex(msg, YES, YES, YES);

  throw_free(buf);
}


//  ------------------------------------------------------------------

void DoTearorig(int mode, GMsg* msg) {

  uint  ctrlinfo;
  char  buf[256];
  char* ptr;
  char* ptr2;
  Line* line = msg->lin;
  Line* newline;
  Orig  origin;
  Netn  netname;

  strcpy(origin, AA->Origin());
  strcpy(netname, AA->Netname());

  if(AA->Taglinesupport()) {
    if(*msg->tagline == '@')
      GetRandomLine(msg->tagline, sizeof(msg->tagline), msg->tagline+1);
    TokenXlat(mode, msg->tagline, msg, msg, CurrArea);
    strbtrim(msg->tagline);
  }

  TokenXlat(mode, msg->tearline, msg, msg, CurrArea);

  ctrlinfo = AA->Ctrlinfo();

  if(NOT *msg->origin)
    MakeOrigin(msg, origin, netname);

  // Delete current tag-, tear- and originlines
  int deltypes = GLINE_TEAR|GLINE_ORIG;
  if(AA->Taglinesupport())
    deltypes |= GLINE_TAGL;
  while(line) {
    if(line->type & deltypes) {
      if(line == msg->lin)
        msg->lin = NULL;
      newline = line;
      Line* nextline = newline->next;
      line = DeleteLine(line);
      if(nextline == NULL)
        line = NULL;
    }
    else
      line = line->next;
  }
  
  // Make sure there is a blank line at the bottom, just before the tearline
  // Unless the line is a tagline (..., ___ etc).
  line = LastLine(msg->lin);
  if(line == NULL)
    msg->lin = line = AddLine(NULL, "");
  else {
    ptr = line->text;
    if(NOT strblank(ptr))
      if(NOT ((ptr[0] == ptr[1]) AND (ptr[1] == ptr[2])))
        line = AddLine(line, "");
  }

  // Check and fix originline
  if(*msg->origin) {
    ptr = strrchr(msg->origin, '(' /*)*/ );
    if(ptr) {

      if(ptr != msg->origin)
        *((ptr++)-1) = NUL;
      else
        *ptr++ = NUL;

      strcpy(origin, msg->origin);

      ptr2 = ptr;
      while(NOT isdigit(*ptr) AND *ptr != /*(*/ ')' AND *ptr)
        ptr++;
      if(isdigit(*ptr))
        *ptr = NUL;
      else
        *ptr2 = NUL;

      strcpy(netname, ptr2);
    }
    else {
      strcpy(origin, msg->origin);
      *netname = NUL;
    }
  }
  MakeOrigin(msg, origin, netname);

  TokenXlat(mode, msg->tagline, msg, msg, CurrArea);
  TokenXlat(mode, msg->tearline, msg, msg, CurrArea);
  TokenXlat(mode, msg->origin, msg, msg, CurrArea);

  // Add the tagline, tearline and origin as defined
  if(AA->Taglinesupport() and *msg->tagline) {
    sprintf(buf, "%c%c%c %s", AA->Taglinechar(), AA->Taglinechar(), AA->Taglinechar(), msg->tagline);
    strtrim(buf);
    line = AddLine(line, buf);
    line->type |= GLINE_TAGL;
  }
  if(ctrlinfo & CI_TEAR) {
    line = AddLine(line, MakeTearline(msg, buf));
    line->type |= GLINE_TEAR;
  }
  if(ctrlinfo & CI_ORIG) {
    sprintf(buf, " * Origin: %s", msg->origin);
    line = AddLine(line, buf);
    line->type |= GLINE_ORIG;
  }
}


//  ------------------------------------------------------------------

