
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gedoit.cpp,v 1.1.1.1 1998/11/21 19:38:02 odinn Exp $
//  ------------------------------------------------------------------
//  Various major job handling funcs
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include "gutlclip.h"

//  ------------------------------------------------------------------

static FILE* __FAR prnfp;
static int   __FAR prnfh;
static int   __FAR prnacc;
static int   __FAR prnmargin;
static int   __FAR prnheader;


//  ------------------------------------------------------------------

void SaveLines(int mode, const char* savefile, GMsg* msg) {

  int prn=NO;
  char fnam[GMAXPATH];
  
  if(mode == MODE_SAVE or mode == MODE_SAVENOCTRL)
    prnacc = O_CREAT|O_WRONLY|O_TRUNC|O_TEXT;

  strcpy(fnam, "PRN");
  if(mode == MODE_WRITE AND streql(savefile, "\001PRN")) {
    prn = YES;
    #if defined(__UNIX__)
    if(prnfp == NULL)
      prnfp = popen(fnam, "w");
    #else
    if(prnfh == -1)
      prnfh = sopen(fnam, prnacc, CFG->sharemode, S_IWRITE);
    #endif
  }
  else {
    strcpy(fnam, savefile);
    strschg_environ(fnam);
    prnfh = sopen(fnam, prnacc, CFG->sharemode, S_STDRW);
  }
  int lines=0;
  if(prnfh != -1 or prnfp) {
    if(mode == MODE_WRITE) {
      if(prnheader)
        DispHeader(msg, prnfh, prnfp, prnmargin);
      if(prn)
        lines = 6;
    }
    int n = 0;
    Line** lin = msg->line;
    if(lin) {
      Line* line = lin[n];
      while(line) {
        if(line->text) {
          uint lineisctrl = line->type & (GLINE_TEAR|GLINE_ORIG|GLINE_HIDD|GLINE_KLUD);
          if(NOT ((mode == MODE_SAVENOCTRL) AND lineisctrl)) {
            char* ptr = line->text;
            while(*ptr) {
              if(mode == MODE_WRITE) {
                // Replace control codes, except the ANSI escape code
                if(*ptr < ' ') {
                  // only allow ESC in file write
                  if(prn OR (*ptr != '\x1B')) {
                    *ptr = (*ptr == CTRL_A) ? '@' : '.';
                  }
                }
              }
              ptr++;
            }
            ptr = line->text;
            #if defined(__UNIX__)
            if(prn)
              fwrite(ptr, strlen(ptr), 1, prnfp);
            else
              write(prnfh, ptr, strlen(ptr));
            #else
            write(prnfh, ptr, strlen(ptr));
            #endif
            if(mode == MODE_NEW) {
              if(EDIT->HardLines()) {
                if(line->type & GLINE_HARD) {
                  if(NOT((line->type & (GLINE_TEAR|GLINE_ORIG|GLINE_KLUD|GLINE_HIDD|GLINE_QUOT)) OR strblank(ptr))) {
                    #if defined(__UNIX__)
                    if(prn)
                      fwrite(EDIT->HardLine(), strlen(EDIT->HardLine()), 1, prnfp);
                    else
                      write(prnfh, EDIT->HardLine(), strlen(EDIT->HardLine()));
                    #else
                    write(prnfh, EDIT->HardLine(), strlen(EDIT->HardLine()));
                    #endif
                  }
                }
              }
            }
            #if defined(__UNIX__)
            if(prn)
              fwrite("\r\n", 2, 1, prnfp);
            else
              write(prnfh, "\n", 1);
            #else
            write(prnfh, "\n", 1);
            #endif
            if(prn) {
              lines++;
              if(lines%CFG->printlength == 0 AND CFG->switches.get(printformfeed)) {
                #if defined(__UNIX__)
                if(prn)
                  fwrite("\f", 1, 1, prnfp);
                else
                  write(prnfh, "\f", 1);
                #else
                write(prnfh, "\f", 1);
                #endif
              }
            }
          }
        }
        line = lin[++n];
      }
    }
    // Add an empty line and formfeed at the bottom
    if(mode == MODE_WRITE) {
      #if defined(__UNIX__)
      if(prn)
        fwrite("\r\n", 2, 1, prnfp);
      else
        write(prnfh, "\n", 1);
      #else
      write(prnfh, "\n", 1);
      #endif
    }
    if(prn) {
      if(CFG->switches.get(printformfeed)) {          // Add formfeed
        #if defined(__UNIX__)
        fwrite("\f", 1, 1, prnfp);
        #else
        write(prnfh, "\f", 1);
        #endif
      }
    }
    else {
      #if defined(__UNIX__)
      if(prn)
        pclose(prnfp);
      else
        close(prnfh);
      #else
      close(prnfh);
      #endif
    }
  }
  else {
    char buf[256];
    sprintf(buf, LNG->CouldNotOpen, fnam);
    w_info(buf);
    waitkeyt(10000);
    w_info(NULL);
  }
}


//  ------------------------------------------------------------------

void WriteMsgs(GMsg* msg) {

  if(AA->Msgn.Tags() == 0)
    return;

  GFTRK("WriteMsgs");

  char buf[256];

  int overwrite = false;

  prnfp = NULL;
  prnfh = -1;
  prnacc = 0;

  GMenuDomarks MenuDomarks;
  int source = AA->Mark.Count() ? MenuDomarks.Run(LNG->Write) : WRITE_CURRENT;

  if(source != WRITE_QUIT) {

    GMenuWriteMsg MenuWriteMsg;
    int target = MenuWriteMsg.Run();

    if(target & WRITE_CLIPBRD)
      source = WRITE_CURRENT;

    if(target != WRITE_QUIT) {

      prnheader = !(target & WRITE_NO_HEADER);

      if(target & WRITE_PRINTER)
        prnmargin = CFG->printmargin;
      else
        prnmargin = 79;

      if(source == WRITE_MARKED) {
        if(target & WRITE_FILE) {
          do {
            overwrite = false;
            strcpy(CFG->outputfile, AA->Outputfile());
            if(NOT edit_pathname(CFG->outputfile, sizeof(Path), LNG->WriteMsgs, H_WriteMessage))
              goto Finish;
            if(is_dir(CFG->outputfile)) {
              AddBackslash(CFG->outputfile);
              strcat(CFG->outputfile, "golded.txt");
            }
            AA->SetOutputfile(CFG->outputfile);
            w_infof(" %s ", AA->Outputfile());
            if(stricmp(AA->Outputfile(), "PRN") AND strnicmp(AA->Outputfile(), "LPT", 3))
              if(fexist(AA->Outputfile())) {
                GMenuOverwrite MenuOverwrite;
                overwrite = MenuOverwrite.Run();
              }
          } while(overwrite == -1);
        }
        else if(target & WRITE_PRINTER) {
          #if defined(__UNIX__)
          prnfp = popen(CFG->printdevice, "w");
          if(prnfp)
            fwrite(CFG->printinit+1, CFG->printinit[0], 1, prnfp);
          #else
          prnfh = sopen(CFG->printdevice, O_WRONLY|O_TEXT, CFG->sharemode, S_IWRITE);
          if(prnfh != -1)
            write(prnfh, &(CFG->printinit[1]), CFG->printinit[0]);
          #endif
        }
        w_info(NULL);
        w_progress(MODE_NEW, C_INFOW, 0, AA->Mark.Count(), LNG->Writing);
        for(uint n=0; n<AA->Mark.Count(); n++) {
          if(overwrite AND n)
            overwrite = NO;     // Overwrite only the first time
          if(kbxhit()) {
            if(kbxget() == Key_Esc) {
              HandleGEvent(EVTT_JOBFAILED);
              break;
            }
          }
          update_statuslinef(LNG->WritingMsg, n+1, AA->Mark.Count());
          w_progress(MODE_UPDATE, C_INFOW, n+1, AA->Mark.Count(), LNG->Writing);
          if(target & WRITE_PRINTER) {
            AA->LoadMsg(msg, AA->Mark[n], prnmargin);
            #if defined(__UNIX__)
            if(prnfp)
            #else
            if(prnfh != -1)
            #endif
              SaveLines(MODE_WRITE, "\001PRN", msg);
          }
          else {
            AA->LoadMsg(msg, AA->Mark[n], prnmargin);
            if(overwrite)
              prnacc = O_CREAT|O_WRONLY|O_TRUNC|O_TEXT;
            else
              prnacc = O_CREAT|O_WRONLY|O_APPEND|O_TEXT;
            SaveLines(MODE_WRITE, AA->Outputfile(), msg);
          }
        }
        #if defined(__UNIX__)
        if(prnfp)
          fwrite(CFG->printreset+1, CFG->printreset[0], 1, prnfp);
        #else
        if(prnfh != -1)
          write(prnfh, &(CFG->printreset[1]), CFG->printreset[0]);
        #endif
        w_progress(MODE_QUIT, 0, 0, 0, NULL);
      }
      else if(source == WRITE_CURRENT) {
        if(target & WRITE_FILE) {
          do {
            overwrite = NO;
            strcpy(CFG->outputfile, AA->Outputfile());
            if(edit_pathname(CFG->outputfile, sizeof(Path), LNG->WriteMsgs, H_WriteMessage)) {
              if(is_dir(CFG->outputfile)) {
                AddBackslash(CFG->outputfile);
                strcat(CFG->outputfile, "golded.txt");
              }
              AA->SetOutputfile(CFG->outputfile);
              w_infof(" %s ", AA->Outputfile());
              if(stricmp(AA->Outputfile(), "PRN") AND strnicmp(AA->Outputfile(), "LPT", 3)) {
                if(fexist(AA->Outputfile())) {
                  GMenuOverwrite MenuOverwrite;
                  overwrite = MenuOverwrite.Run();
                  if(overwrite == -1)   // Escape was hit
                    continue;
                }
              }
              sprintf(buf, LNG->WritingFile, AA->Outputfile());
              w_info(buf);
              AA->LoadMsg(msg, msg->msgno, prnmargin);
              if(overwrite)
                prnacc = O_CREAT|O_WRONLY|O_TRUNC|O_TEXT;
              else
                prnacc = O_CREAT|O_WRONLY|O_APPEND|O_TEXT;
              SaveLines(MODE_WRITE, AA->Outputfile(), msg);
              w_info(NULL);
            }
          } while(overwrite == -1);
        }
        else if(target & WRITE_PRINTER) {
          w_info(LNG->WritingPRN);
          AA->LoadMsg(msg, msg->msgno, prnmargin);
          #if defined(__UNIX__)
          prnfp = popen(CFG->printdevice, "w");
          if(prnfp) {
            fwrite(CFG->printinit+1, CFG->printinit[0], 1, prnfp);
            SaveLines(MODE_WRITE, "\001PRN", msg);
            fwrite(CFG->printreset+1, CFG->printreset[0], 1, prnfp);
          }
          #else
          prnfh = sopen(CFG->printdevice, O_WRONLY|O_TEXT, CFG->sharemode, S_IWRITE);
          if(prnfh != -1) {
            write(prnfh, &(CFG->printinit[1]), CFG->printinit[0]);
            SaveLines(MODE_WRITE, "\001PRN", msg);
            write(prnfh, &(CFG->printreset[1]), CFG->printreset[0]);
          }
          #endif
          w_info(NULL);
        }
        else if(target & WRITE_CLIPBRD) {
          w_info(LNG->Wait);

          char fname[GMAXPATH];

          strcpy(fname, AddPath(CFG->goldpath, "goldclip.tmp"));

          AA->LoadMsg(msg, msg->msgno, prnmargin);
          prnacc = O_CREAT|O_WRONLY|O_TRUNC|O_TEXT;
          SaveLines(MODE_WRITE, fname, msg);

          gclipbrd clipbrd;
          gfile fp;

          if(fp.fopen(fname, "rb")) {

            long len = fp.filelength();
            char* buf = (char*) throw_malloc(len+1);
            buf[len] = NUL;
            fp.fread(buf, 1, len);

            clipbrd.writeclipbrd(buf);

            throw_free(buf);

            fp.close();
          }

          remove(fname);
          w_info(NULL);
        }
      }
    }
  }

Finish:

  w_info(NULL);

  #if defined(__UNIX__)
  if(prnfp) {
    pclose(prnfp);
    prnfp = NULL;
  }
  #else
  if(prnfh != -1) {
    close(prnfh);
    prnfh = -1;
  }
  #endif

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void WriteMsg(GMsg* msg) {

  if(AA->Msgn.Count())
    WriteMsgs(msg);
}


//  ------------------------------------------------------------------
//  Get name of current quotebuffer filename

char* GetCurrQuotebuf(char* quotebuf) {

  if(*AA->Quotebuffile()) {
    strcpy(quotebuf, AA->Quotebuffile());
    MakePathname(quotebuf, CFG->goldpath, quotebuf);
  }
  else {
    if(AA->isezycom())
      sprintf(quotebuf, "%sgld%05u.qbf", CFG->ezycom.msgbasepath, AA->board());
    else if(AA->isfido())
      sprintf(quotebuf, "%s%s", AA->path(), "golded.qbf");
    else if(AA->isgoldbase())
      sprintf(quotebuf, "%sgoldg%03u.qbf", CFG->goldbasepath, AA->board());
    else if(AA->ishudson())
      sprintf(quotebuf, "%sgoldh%03u.qbf", CFG->hudsonpath, AA->board());
    else
      sprintf(quotebuf, "%s%s", AA->path(), ".qbf");
  }

  return quotebuf;
}


//  ------------------------------------------------------------------

void QuoteBuf(GMsg* msg) {

  if(AA->Msgn.Count()) {

    Path quotebuf;
    char openmode[4];

    TemplateToText(MODE_QUOTEBUF, msg, msg, AA->Tpl(), CurrArea);
    msg->attr.tou1();
    msg->TextToLines(-CFG->quotemargin);
    msg->attr.tou0();
    msg->charsetlevel = LoadCharset(CFG->xlatlocalset, CFG->xlatlocalset);
    msg->LinesToText();

    GetCurrQuotebuf(quotebuf);
    w_infof(" %s ", quotebuf);

    switch(CFG->quotebufmode) {

      case QBUF_ASK:
        *openmode = NUL;
        if(fexist(quotebuf)) {
          GMenuOverwrite MenuOverwrite;
          switch(MenuOverwrite.Run()) {
            case TRUE:
              strcpy(openmode, "wt");
              break;
            case FALSE:
              strcpy(openmode, "at");
              break;
            default:
              *openmode = NUL;
          }
        }
        else {
          strcpy(openmode, "wt");
        }
        break;

      case QBUF_APPEND:
        strcpy(openmode, "at");
        break;

      case QBUF_OVERWRITE:
        strcpy(openmode, "wt");
        break;
    }

    if(*openmode) {

      FILE* fp = fsopen(quotebuf, openmode, CFG->sharemode);
      if(fp) {
        strchg(msg->txt, 0x0D, 0x0A);
        fputs(msg->txt, fp);
        fclose(fp);
      }

      HandleGEvent(EVTT_JOBDONE);
      waitkeyt(1000);
    }
    w_info(NULL);
  }
}


//  ------------------------------------------------------------------

