
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geedit.cpp,v 1.1.1.1 1998/11/21 19:38:02 odinn Exp $
//  ------------------------------------------------------------------
//  The Internal Editor (IE), part 1.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include "geedit.h"


//  ------------------------------------------------------------------
//  Globals

int CFG__editquotewrap = YES;

Line* Edit__killbuf  = NULL;
Line* Edit__pastebuf = NULL;

Path Edit__exportfilename = {""};


//  ------------------------------------------------------------------

#ifndef NDEBUG

void IEclass::debugtest(char* __test, int __a, int __b, char* __file, int __line, int __values) {

  #if defined(GFTRK_ENABLE)
  int _tmp = __gftrk_on;
  __gftrk_on = FALSE;
  #endif

  savefile(MODE_UPDATE);

  #if defined(GFTRK_ENABLE)
  __gftrk_on = _tmp;
  #endif

  if(__values)
    w_infof(" Range check: (%s) <%i,%i> [%s,%u] ", __test, __a, __b, CleanFilename(__file), __line);
  else
    w_infof(" Range check: (%s) [%s,%u] ", __test, CleanFilename(__file), __line);
  update_statusline(LNG->EscOrContinue);
  if(NOT disablesound)
    beep();
  while(kbxhit())
    kbxget();
  Key _key = kbxget();
  w_info(NULL);
  if(_key == Key_Esc) {
    LOG.errtest(__file, __line);
    LOG.printf("! An internal editor range check failed.");
    if(__values)
      LOG.printf(": Details: (%s) <%i,%i>.", __test, __a, __b);
    else
      LOG.printf(": Details: (%s).", __test);
    LOG.printf(": Details: r%u,c%u,mr%u,mc%u,i%u,dm%u,qm%u,eqm%u.",
      row, col, maxrow, maxcol, insert,
      CFG->dispmargin, CFG->quotemargin, EDIT->QuoteMargin()
    );
    LOG.printf("+ Advice: Report to the Author.");
    TestErrorExit();
  }
  else if(_key == Key_F12) {
    char* crash = NULL;
    *crash = NUL;
  }
}

#endif


//  ------------------------------------------------------------------
//  Make sure line type is correct

void IEclass::setlinetype(Line* __line) {

  _test_halt(__line == NULL);
  _test_halt(__line->text == NULL);

  __line->type &= ~(GLINE_HIDD|GLINE_KLUD|GLINE_QUOT);
  __line->type |= is_quote(__line->text) ? GLINE_QUOT : (*__line->text == CTRL_A) ? GLINE_HIDD : 0;
}


//  ------------------------------------------------------------------
//  Insert character in string at position

void IEclass::strinschr(char** __string, char __ch, uint __position) {

  _test_halt(__string == NULL);

  //  0123456789  _position = 2
  //  1234567890  length = 4
  //  helo<       < = nul
  //  he lo<
  //  hello<

  // Get string length
  uint _length = strlen(*__string);

  // Is position beyond the nul-terminator in the string?
  _test_haltab(__position > _length, __position, _length);

  // Reallocate string to make room for the char (3=nul+newchar+possible space)
  *__string = (char*)throw_realloc(*__string, _length+3);

  // Make room for the char to insert
  memmove(*__string+__position+1, *__string+__position, _length-__position+1);
  THROW_CHECKPTR(*__string);

  // Put in the char
  (*__string)[__position] = __ch;
  THROW_CHECKPTR(*__string);
  if(((*__string)[__position+1] == NUL) AND (__ch != ' ')) {
    (*__string)[__position+1] = ' ';
    (*__string)[__position+2] = NUL;
    THROW_CHECKPTR(*__string);
  }
}


//  ------------------------------------------------------------------
//  Delete character in string at position

void IEclass::strdelchr(char* __string, uint __position) {

  _test_halt(__string == NULL);

  //  0123456789  _position = 1
  //  1234567890  length = 5
  //  hello<      < = nul
  //  hllo<

  uint _length = strlen(__string);

  // Is position at or beyond the nul-terminator in the string?
  _test_haltab(__position >= _length, __position, _length);

  // Delete the character
  memmove(__string+__position, __string+__position+1, _length-__position);
  THROW_CHECKPTR(__string);
}


//  ------------------------------------------------------------------
//  Zero-based

inline int IEclass::dispchar(char __ch, int attr) {

  if(__ch != '\n') {
    if(__ch == ' ')
      __ch = EDIT->CharSpace();
  }
  else {
    __ch = EDIT->CharPara();
  }

  int atr;
  char chr;
  editwin.getc(crow, ccol, &atr, &chr);
  editwin.printc(crow, ccol, attr == -1 ? atr : attr, __ch);
  return atr;
}


//  ------------------------------------------------------------------

void IEclass::cursoroff() {

  vcurhide();
}


//  ------------------------------------------------------------------

void IEclass::cursoron() {

  vcurshow();
}


//  ------------------------------------------------------------------
//  Zero-based

inline void IEclass::scrollup(int __scol, int __srow, int __ecol, int __erow, int __lines) {

  editwin.scroll_box_up(__srow, __scol, __erow, __ecol, __lines);
}


//  ------------------------------------------------------------------
//  Zero-based

inline void IEclass::scrolldown(int __scol, int __srow, int __ecol, int __erow, int __lines) {

  editwin.scroll_box_down(__srow, __scol, __erow, __ecol, __lines);
}


//  ------------------------------------------------------------------

Line* IEclass::findfirstline() {

  GFTRK("Editfindfirstline");

  // Rewind to the first line
  Line* _firstline = currline;
  while(_firstline->prev)
    _firstline = _firstline->prev;

  GFTRK(NULL);

  return _firstline;
}


//  ------------------------------------------------------------------
//  Find out what number the current line is and put it in "thisrow"

void IEclass::getthisrow(Line* __currline) {

  GFTRK("Editgetthisrow");

  Line* _templine = findfirstline();

  thisrow = 0;
  while((_templine != __currline) AND _templine->next) {
    _templine = _templine->next;
    thisrow++;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------
//  Zero-based

void IEclass::gotorowcol(uint __col, uint __row) {

  GFTRK("Editgotorowcol");

  _test_haltab(__col > maxcol, __col, maxcol);
  _test_haltab(__row > maxrow, __row, maxrow);

  editwin.move_cursor(__row, __col);

  ccol = __col;
  crow = __row;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------
//  Zero-based

extern IEclass* _editor;
void stylecode_prints(int wrow, int wcol, int atr, char* str) { _editor->editwin.prints(wrow,wcol,atr,str); }

void IEclass::dispstring(char* __string, uint __row, int attr) {

  GFTRK("Editdispstring");

  _test_halt(__string == NULL);
  _test_haltab(__row > maxrow, __row, maxrow);

  // Get string length
  uint _length = strlen(__string);

  // String longer than window width?
  _test_haltab(_length > (maxcol+1), _length, (maxcol+1));
  _length = MinV(_length, (maxcol+1));

  // Buffer for translation to visual representation
  char _buf[EDIT_BUFLEN];

  // Space-pad and nul-terminate the buffer
  memset(_buf, ' ', maxcol+1);
  _buf[maxcol+1] = NUL;

  // Copy/translate string into buffer
  if(attr == -1) {
    char* _bufptr = _buf;
    uint _position = 0;
    char* __str = __string;
    while(_position < _length) {
      switch(*__str) {
        case ' ':   *_bufptr = EDIT->CharSpace();  break;
        case '\n':  *_bufptr = EDIT->CharPara();   break;
        default:    *_bufptr = *__str;
      }
      _position++;
      _bufptr++;
      __str++;
    }
  }
  else {
    memcpy(_buf, __string, _length);
  }

  // Print the string
  editwin.prints(__row, mincol, attr, _buf);
  if(CFG->switches.get(stylecodes))
    StyleCodeHighlight(__string, __row, mincol, false, attr, (VidPutStrCP)stylecode_prints);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::setcolor(Line* __line) {

  // Set window attribute
  if(__line->type & GLINE_BLOK)
    editwin.text_color(C_READA);
  else if(__line->type & GLINE_HIDD)
    editwin.text_color(C_READK);
  else if(__line->type & GLINE_KLUD)
    editwin.text_color(C_READK);
  else if(__line->type & GLINE_TAGL)
    editwin.text_color(C_READG);
  else if(__line->type & GLINE_TEAR)
    editwin.text_color(C_READT);
  else if(__line->type & GLINE_ORIG)
    editwin.text_color(C_READO);
  else if(__line->type & GLINE_QUOT)
    editwin.text_color(quotecolor(__line->text));
  else if(__line->type & GLINE_SIGN)
    editwin.text_color(C_READS);
  else
    editwin.text_color(C_READW);

}


//  ------------------------------------------------------------------
//  Zero-based

void IEclass::displine(Line* __line, uint __row) {

  GFTRK("Editdispline");

  _test_halt(__line == NULL);
  _test_halt(__line->text == NULL);
  _test_haltab(__row > maxrow, __row, maxrow);

  // Display line
  setcolor(__line);
  dispstring(__line->text, __row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------
//  Zero-based

void IEclass::clreol(int __col, int __row) {

  GFTRK("Editclreol");

  if(__col == -1)
    __col = ccol;

  if(__row == -1)
    __row = crow;

  if((uint)__col <= maxcol)
    editwin.fill(__row, __col, __row, maxcol, ' ', C_READW);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------
//  Zero-based

void IEclass::refresh(Line* __currline, uint __row) {

  GFTRK("Editrefresh");

  _test_halt(__currline == NULL);

  cursoroff();

  // Display as many lines as we can
  while(__currline AND (__row <= maxrow)) {
    displine(__currline, __row++);
    __currline = __currline->next;
  }

  // If we ran out of lines, blank the rest
  if(__row <= maxrow) {
    char buf[256];
    memset(buf, _box_table[W_BREAD][1], maxcol+1);
    buf[maxcol+1] = NUL;
    dispstring(buf, __row++, C_READB|ACSET);
    while(__row <= maxrow)
      dispstring("", __row++);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

Line* IEclass::insertlinebelow(Line* __currline, char* __text) {

  GFTRK("Editinsertlinebelow");

  Line* _nextline = (Line*)throw_xcalloc(1, sizeof(Line));

  if(__currline) {

    _nextline->text = __text ? throw_strdup(__text) : (char*)NULL;
    _nextline->prev = __currline;
    _nextline->next = __currline->next;
    if(_nextline->next)
      _nextline->next->prev = _nextline;
    __currline->next = _nextline;
  }

  GFTRK(NULL);

  return _nextline;
}


//  ------------------------------------------------------------------
//  Zero-based

int IEclass::downoneline(uint __row) {

  GFTRK("Editdownoneline");

  _test_haltab(__row > maxrow, __row, maxrow);

  thisrow++;

  if(__row == maxrow)
    scrollup(mincol, minrow, maxcol, maxrow);
  else
    __row++;

  gotorowcol(mincol, __row);

  GFTRK(NULL);

  return __row;
}


//  ------------------------------------------------------------------

void IEclass::GoEOL() {

  GFTRK("EditGoEOL");

  // Move cursor to the last char on the line
  col = strlen(currline->text);
  if(col)
    col--;

  #if 0
  // If the last char is not a linefeed
  if(currline->text[col] != '\n') {

    // If there is room at the end of the line
    if(col < maxcol) {

      // Move the cursor one char beyond the last char
      col++;
    }
  }
  #endif

  // String must not be longer than the window width
  _test_haltab(col > maxcol, col, maxcol);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::GoUp() {

  GFTRK("EditGoUp");

  _test_haltab(row < minrow, row, minrow);

  if(currline->prev) {

    currline = currline->prev;
    thisrow--;
    
    if(row == minrow) {
      scrolldown(mincol, row, maxcol, maxrow);
      displine(currline, row);
    }
    else {
      row--;
    }

    if((col+1) > strlen(currline->text))
      GoEOL();
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::GoDown() {

  GFTRK("EditGoDown");

  _test_haltab(row > maxrow, row, maxrow);

  if(currline->next) {

    currline = currline->next;
    thisrow++;

    if(row == maxrow) {
      scrollup(mincol, minrow, maxcol, maxrow);
      displine(currline,row);
    }
    else {
      row++;
    }

    if((col+1) > strlen(currline->text))
      GoEOL();
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::GoLeft() {

  GFTRK("EditGoLeft");

  _test_haltab(col < mincol, col, mincol);

  if(col == mincol) {
    if(currline->prev) {
      GoUp();
      GoEOL();
    }
    col++;  // reverse of col-- below...
  }

  col--;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::GoRight() {

  GFTRK("EditGoRight");

  _test_haltab(col > maxcol, col, maxcol);

  char _cursorchar = currline->text[col];

  if((col == maxcol) OR (_cursorchar == '\n') OR (_cursorchar == NUL)) {
    if(currline->next != NULL) {
      GoDown();
      col = mincol;
    }
    col--;  // reverse of col++ below
  }

  col++;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

Line* IEclass::wrapit(Line** __currline, uint* __curr_col, uint* __curr_row, int __display) {

  _test_halt(__currline == NULL);
  _test_halt(*__currline == NULL);

  uint  _quotelen;
  char  _quotebuf[100];

  uint  _curscol = *__curr_col;
  uint  _cursrow = *__curr_row;

  uint  _thisrow = *__curr_row;
  Line* _thisline = *__currline;
  Line* _lastadded = _thisline;

  int  _wrapped_above = NO;

  // Start wrapping from the current line onwards
  while(_thisline) {

    // Length of this line
    uint _thislen = strlen(_thisline->text);

    uint _wrapmargin = (_thisline->type & GLINE_QUOT) ? marginquotes : margintext;

    // Does this line need wrapping?
    if(_thislen > _wrapmargin) {

      // Reset quote string length
      _quotelen = 0;

      // Is this line quoted?
      if((_thisline->type & GLINE_QUOT) AND CFG__editquotewrap) {

        // Get quote string and length
        GetQuotestr(_thisline->text, _quotebuf, &_quotelen);
      }

      // wrapmargin = 40
      //                   1111111111222222222233333333334444444444
      //         01234567890123456789012345678901234567890123456789
      //         --------------------------------------------------
      //                                                v- wrapptr
      // Case 1: this is another test with a bit of text to wrap.
      // Case 2: this is a test with a line that need wrapping.
      // Case 3: thisxisxaxtestxwithxaxlinexthatxneedxwrapping.
      // Case 4: >thisxisxaxtestxwithxaxlinexthatxneedxwrapping.

      // Point to the last char inside the margin
      char* _wrapptr = _thisline->text + _wrapmargin - 1;

      // Locate the word to be wrapped

      // Did we find a space?
      if(*_wrapptr == ' ') {

        // Case 1: A space was found as the last char inside the margin
        //
        // Now we must locate the first word outside the margin.
        // NOTE: Leading spaces to this word will be nulled out!

        // Begin at the first char outside the margin
        _wrapptr++;

        #if 0
        // Search forward until a non-space or NUL is found
        while(*_wrapptr == ' ' AND *_wrapptr) {

          // Nul spaces along the way
          *_wrapptr++ = NUL;
          THROW_CHECKPTR(_thisline->text);
        }
        #endif
      }
      else {

        // Case 2: A non-space was found as the last char inside the margin
        //
        // Now we must locate the beginning of the word we found.

        // Keep copy of original pointer
        char* _atmargin = _wrapptr;

        // Search backwards until a space or the beginning of the line is found
        while((_wrapptr > _thisline->text) AND (*(_wrapptr-1) != ' '))
          _wrapptr--;

        // Check if we hit leading spaces
        char* _spaceptr = _wrapptr;
        if(_spaceptr > _thisline->text) {
          _spaceptr--;
          while((_spaceptr > _thisline->text) AND (*_spaceptr == ' '))
            _spaceptr--;
        }

        // Did we search all the way back to the beginning of the line?
        //if((_wrapptr == _thisline->text) OR (_wrapptr == (_thisline->text+_quotelen)) OR (strlen(_wrapptr) > _wrapmargin) OR (*_spaceptr == ' '))
        if((_wrapptr == _thisline->text) OR (_wrapptr == (_thisline->text+_quotelen)) OR (*_spaceptr == ' ')) {

          // Case 3: There are no spaces within the margin or we hit leading spaces

          // We have to break it up at the margin
          _wrapptr = _atmargin;
        }
      }

      // The wrapptr now points to the location to be wrapped or NUL

      // Is the line hard-terminated?
      if(strchr(_wrapptr, '\n')) {

        // The line is hard-terminated.
        //
        // The wrapped part must be placed on a new line below.

        // Allocate a new line
        Line* _wrapline = (Line*)throw_xcalloc(1, sizeof(Line));

        // Allocate space for the new line
        _wrapline->text = (char*)throw_malloc(_quotelen + strlen(_wrapptr) + 1);

        // Copy the quote string, if any, to the new line first
        if(_quotelen) {
          strcpy(_wrapline->text, _quotebuf);
          THROW_CHECKPTR(_wrapline->text);
        }
        else {
          *_wrapline->text = NUL;
          THROW_CHECKPTR(_wrapline->text);
        }

        // Copy/append the wrapped part to the new line
        strcat(_wrapline->text, _wrapptr);
        THROW_CHECKPTR(_wrapline->text);

        // Make sure the type of the line is correct
        setlinetype(_wrapline);

        // Link the new line in between this line and the next
        _wrapline->prev = _thisline;
        _wrapline->next = _thisline->next;
        _thisline->next = _wrapline;
        if(_wrapline->next)
          _wrapline->next->prev = _wrapline;
        _lastadded = _wrapline;

        if(__display) {

          // Is there at least one line below this one?
          if(_thisrow < maxrow) {

            // Scroll the lines below to make room for the new line
            scrolldown(mincol, _thisrow+1, maxcol, maxrow);

            // Display the new line
            if(strlen(_wrapline->text) <= (maxcol+1))
              displine(_wrapline, _thisrow+1);
          }
        }
      }
      else {

        // The line is not hard-terminated
        //
        // The wrapped part must be inserted into the next line

        // Indicate that one or more lines were wrapped in this way
        _wrapped_above = YES;

        // Pointer to the next line
        Line* _nextline = _thisline->next;

        // Flag to indicate if a new line was added below
        int _line_added_below = NO;

        // Is there no next line or is the next line quoted?
        if((_nextline == NULL) OR (_nextline->type & GLINE_QUOT)) {

          // The wrapped part must be placed on a new line below

          // Allocate a new line
          Line* _newline = (Line*)throw_xcalloc(1, sizeof(Line));
          _newline->text = throw_strdup("");

          // Link the new line to this line
          //
          //          thisline->next Ŀ                   thisline->next Ŀ
          //                             \\\    
          //     >>>    prev<-newline->next Ŀ
          //                             ///    
          //  prev<-nextline->next              prev<-nextline->next

          _newline->prev = _thisline;
          _thisline->next = _newline;
          if(_nextline) {
            _newline->next = _nextline;
            _nextline->prev = _newline;
          }
          _nextline = _newline;
          _lastadded = _newline;

          _line_added_below = YES;
        }

        // Get length of the wrapped part
        uint _wraplen = strlen(_wrapptr);

        // Get length of the text of next line
        uint _nextlen = _nextline->text ? strlen(_nextline->text) : 0;

        // Reallocate next line's text to make room for the wrapped part
        _nextline->text = (char*)throw_realloc(_nextline->text, _quotelen + _wraplen + _nextlen + 1);

        // Move the next line's text to make room for the wrapped part and the quote string, if any
        memmove(_nextline->text+_quotelen+_wraplen, _nextline->text, _nextlen+1);
        THROW_CHECKPTR(_nextline->text);

        // Copy the wrapped part
        memmove(_nextline->text+_quotelen, _wrapptr, _wraplen);
        THROW_CHECKPTR(_nextline->text);

        // Was this line quoted?
        if(_quotelen) {

          // Copy the quote string
          memmove(_nextline->text, _quotebuf, _quotelen);
          THROW_CHECKPTR(_nextline->text);
        }

        // Make sure the type of the line is correct
        setlinetype(_nextline);

        if(__display) {

          // Is there at least one line below this one?
          if(_line_added_below AND (_thisrow < maxrow)) {

            // Scroll the lines below to make room for the new line
            scrolldown(mincol, _thisrow+1, maxcol, maxrow);
          }

          // Display the new/wrapped line
          if(((_thisrow+1) <= maxrow) AND (strlen(_nextline->text) <= (maxcol+1)))
            displine(_nextline, _thisrow+1);
        }
      }

      // Nul-terminate at the wrapping location
      *_wrapptr = NUL;
      THROW_CHECKPTR(_thisline->text);

      // Was this line quoted?
      if(_quotelen) {

        // Trim spaces off the end of the line
        char* _trimptr = _thisline->text + strlen(_thisline->text) - 1;
        while(*_trimptr == ' ')
          *_trimptr-- = NUL;
        THROW_CHECKPTR(_thisline->text);

        // Append a new linefeed
        strcat(_thisline->text, "\n");
        THROW_CHECKPTR(_thisline->text);
      }

      // Make sure the line type still is correct
      setlinetype(_thisline);

      if(__display) {

        // Display this line after wrapping
        if(_thisrow <= maxrow)
          displine(_thisline, _thisrow);
      }

      // If we are on the cursor line, check if the cursor char was wrapped
      if(_thisrow == *__curr_row) {

        // Pointer to the cursor char
        char* _cursptr = _thisline->text + *__curr_col;

        // If the cursor char was wrapped, move the cursor coords
        if(_wrapptr <= _cursptr) {
          _curscol = (int)(_cursptr - _wrapptr) + _quotelen;
          _cursrow++;
        }
      }
    }

    // If this line is hard-terminated, we have finished wrapping
    // Unless the next line has grown too large
    if(strchr(_thisline->text, '\n')) {
      if(_thisline->next == NULL)
        break;
      _wrapmargin = (_thisline->next->type & GLINE_QUOT) ? marginquotes : margintext;
      if(strlen(_thisline->next->text) <= _wrapmargin)
        break;
    }

    // Go to the next line
    _thisline = _thisline->next;
    _thisrow++;
  }

  if(__display) {

    // Display the current line after wrapping
    if(*__curr_row <= maxrow)
      displine(*__currline, *__curr_row);

    // Was the line or lines above wrapped?
    if(_wrapped_above) {
      
      // Display the last line in the paragraph
      if((_thisrow <= maxrow) AND _thisline)
        displine(_thisline, _thisrow);
    }
  }

  // Move to the next line if the cursor was wrapped
  if(_cursrow != *__curr_row) {
    *__currline = (*__currline)->next;
    if(_cursrow > maxrow) {
      _cursrow = maxrow;
      scrollup(mincol, minrow, maxcol, maxrow);
      displine(*__currline, row);
    }
  }

  // Update cursor position
  *__curr_row = _cursrow;
  *__curr_col = _curscol;
  //thisrow = _cursrow;

  GFTRK(NULL);

  return _lastadded;
}


//  ------------------------------------------------------------------

Line* IEclass::wrapdel(Line** __currline, uint* __curr_col, uint* __curr_row, int __display) {

  GFTRK("Editwrapdel");

  return wrapit(__currline, __curr_col, __curr_row, __display);
} 


//  ------------------------------------------------------------------

Line* IEclass::wrapins(Line** __currline, uint* __curr_col, uint* __curr_row, int __display) {

  GFTRK("Editwrapins");

  return wrapit(__currline, __curr_col, __curr_row, __display);
}


//  ------------------------------------------------------------------

void IEclass::insertchar(char __ch) {

  GFTRK("Editinsertchar");

  // Insert or overwrite the char
  if(insert) {
    strinschr(&currline->text, __ch, col);
  }
  else {
    uint _currline_len = strlen(currline->text);
    _test_haltab(col > _currline_len, col, _currline_len);
    if((currline->text[col] != '\n') AND (currline->text[col] != NUL)) {
      currline->text[col] = __ch;
      THROW_CHECKPTR(currline->text);
    }
    else
      strinschr(&currline->text, __ch, col);
  }

  // Make sure the line type still is correct
  setlinetype(currline);

  // Move cursor
  col++;

  // Insert a line below if necessary
  #if 0
  if((col > maxcol) AND (currline->text[col] == NUL)) {
    Line* _newline = insertlinebelow(currline);
    _newline->text = throw_strdup("");
    refresh(currline, row);
  }
  #endif

  wrapins(&currline, &col, &row);

  // Adjust cursor position and display if necessary
  if(col > maxcol) {
    if(currline->next) {
      currline = currline->next;
      col = mincol;
      row++;
      if(row > maxrow) {
        row = maxrow;
        scrollup(mincol, minrow, maxcol, maxrow);
        displine(currline, row);
      }
    }
    else {
      col = maxcol;
    }
  }

  gotorowcol(col, row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::DelChar() {

  GFTRK("EditDelChar");

  Line* _thisline = currline;
  Line* _nextline = currline->next;
  uint  _thislen = strlen(_thisline->text);

  // Cannot delete at or beyond the nul-terminator
  if(col < _thislen)
    strdelchr(_thisline->text, col);

  // Did we delete the last char on the line or
  // was the cursor at or beyond the nul-terminator?
  // And is there a next line at all?
  if(((col+1) >= _thislen) AND _nextline) {

    // Join the next line to this line

    // Is the next line quoted?
    // And is the cursor column non-zero?
    uint _quotelen = 0;
    if((_nextline->type & GLINE_QUOT) AND CFG__editquotewrap AND col) {

      // Get quote string length
      char _dummybuf[100];
      GetQuotestr(_nextline->text, _dummybuf, &_quotelen);
    }

    // Reallocate this line's text to make room for the next
    _thisline->text = (char*)throw_realloc(_thisline->text, strlen(_thisline->text) + strlen(_nextline->text) - _quotelen + 1);

    // Copy the next line's text to this line
    // Skip past the next line's quote string and blanks, if any
    char* _nextptr = _nextline->text+_quotelen;
    if(NOT((_nextline->type & GLINE_QUOT) AND (col == 0))) {
      while(*_nextptr == ' ')
        _nextptr++;
    }
    strcat(_thisline->text, _nextptr);
    THROW_CHECKPTR(_thisline->text);

    // Relink this line
    _thisline->next = _nextline->next;
    if(_thisline->next)
      _thisline->next->prev = _thisline;

    // Remove the next line from memory
    throw_release(_nextline->text);
    throw_xfree(_nextline);
  }

  // Make sure the line type still is correct
  setlinetype(_thisline);

  // Rewrap this line
  wrapdel(&currline, &col, &row);

  refresh(currline, row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::DelLeft() {

  GFTRK("EditDelLeft");

  // Cannot backspace from the first column on the first line in the msg
  if(currline->prev == NULL)
    if(col == mincol)
      return;

  // Go left(/up) and delete the character there
  GoLeft();
  DelChar();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::GoWordLeft() {

  GFTRK("EditGoWordLeft");

  if(col == 0) {
    if(currline->prev) {
      GoUp();
      GoEOL();
    }
    GFTRK(NULL);
    return;
  }
  
  col--;

  if(!isxalnum(currline->text[col])) {
    while(!isxalnum(currline->text[col]) AND (col > 0))
      col--;
    while(isxalnum(currline->text[col]) AND (col > 0))
      col--;
  }
  else {
    while(isxalnum(currline->text[col]) AND (col > 0))
      col--;
  }

  if(col != 0)
    col++;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::GoWordRight() {

  GFTRK("EditGoWordRight");

  if((currline->text[col] == NUL) OR (currline->text[col] == '\n')) {
    if(currline->next) {
      GoDown();
      col = 0;
    }
    GFTRK(NULL);
    return;
  }

  if(!isxalnum(currline->text[col])) {
    while(!isxalnum(currline->text[col]) AND ((col+1) <= strlen(currline->text)))
      col++;
  }
  else {
    while(isxalnum(currline->text[col]) AND ((col+1) <= strlen(currline->text)))
      col++;
    while(!isxalnum(currline->text[col]) AND ((col+1) <= strlen(currline->text)))
      col++;
  }

  if(currline->text[col-1] == '\n')
    col--;

  if(currline->text[col] == NUL) {
    if(currline->next) {
      GoDown();
      col = 0;
    }
    else {
      col--;
    }
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::Newline() {

  GFTRK("EditNewline");

  char* _text = currline->text;

  // If the line is not hard-terminated, make room for it
  if(NOT strchr(_text, '\n') AND (_text[col] == NUL))
    _text = currline->text = (char*)throw_realloc(_text, strlen(_text)+3);

  // Pointer to the split position
  char* _splitptr = _text + col;

  // Buffer for the second part of the split line
  char* _splitbuf = (char*)throw_malloc(EDIT_BUFLEN);
  *_splitbuf = NUL;

  // If the split line was quoted, get the quotestring
  // But do not get it if the cursor points to a linefeed or is
  uint _quotelen = 0;
  if(is_quote(_text)) {
    GetQuotestr(_text, _splitbuf, &_quotelen);
    THROW_CHECKPTR(_splitbuf);
  }

  // Eliminate the quotestring if
  // - the cursor points to a linefeed or
  // - the cursor is located inside the quotestring
  if(_quotelen AND ((*_splitptr == '\n') OR (*_splitptr == NUL) OR (col < _quotelen)))
    *_splitbuf = NUL;

  // Append the second part to the split buffer
  strcat(_splitbuf, _splitptr);
  THROW_CHECKPTR(_splitbuf);

  // Copy linefeed+nul to the split position
  strcpy(_splitptr, "\n");
  THROW_CHECKPTR(currline->text);

  // Re-type and display the split line
  setlinetype(currline);
  displine(currline, row);

  // Insert a new line below
  currline = insertlinebelow(currline);

  // Set the line text to the split-off part
  currline->text = throw_strdup(_splitbuf);
  setlinetype(currline);
  throw_free(_splitbuf);

  // Move down the cursor
  col = 0;
  row = downoneline(row);
  gotorowcol(col, row);

  // Scroll the remaining lines if necessary
  if(row < maxrow)
    scrolldown(mincol, row, maxcol, maxrow);

  // Rewrap the split-off line
  wrapdel(&currline, &col, &row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::CopyAboveChar() {

  GFTRK("EditCopyAboveChar");

  char _ch = ' ';
  if(currline->prev) {
    char* _ptr = currline->prev->text;
    uint _len = strlen(_ptr);
    if(_len AND _len > col)
      _ch = _ptr[col];
    if(_ch == '\n')
      _ch = ' ';
  }
  insertchar(_ch);
  
  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::DupLine() {

  GFTRK("EditDupLine");

  Line* _nextline;
  _nextline = insertlinebelow(currline);
  _nextline->text   = throw_strdup(currline->text);
  _nextline->type   = currline->type;
  _nextline->color  = currline->color;
  _nextline->kludge = currline->kludge;
  refresh(currline, row);
  GoDown();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------
//  PageUp behavior:
//
//  ... The top line becomes the bottom line.
//  ... Always remain at cursor row, except if we can't page up.
//  ... If we can't page up, move the cursor row to the top line.

void IEclass::GoPgUp() {

  GFTRK("EditGoPgUp");

  // Not at the first line in msg?
  if(currline->prev) {

    // Count lines
    int _count = row;

    // Move to the top line currently displayed
    Line* _topline = currline;
    while(_count AND _topline->prev) {
      _topline = _topline->prev;
      _count--;
    }

    // The count must be zero at this point!
    _test_haltab(_count, _count, _count);

    // At we already fully at the top?
    if(_topline->prev == NULL) {

      // Yes, so move the cursor row to the top line
      row = minrow;
      currline = _topline;
    }
    else {

      // We are not at the top, so continue with paging

      // Move a full page of lines, if possible
      _count = maxrow;
      while(_count AND _topline->prev) {
        _topline = _topline->prev;
        _count--;
      }

      // Refresh display
      refresh(_topline, minrow);

      // Set the current line
      _count = row;
      currline = _topline;
      while(_count--)
        currline = currline->next;
    }
  }
  else {
    GoTopMsg();
  }

  if((col+1) > strlen(currline->text))
    GoEOL();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------
//  PageDown behavior:
//
//  ... The bottom line becomes the top line.
//  ... Always remain at cursor row, except if at the last line.
//  ... If at the last line, move cursor to the last window line and
//      display the lines above.
//  ... If there are too few lines to display after the page down, the
//      rest are displayed blank.

void IEclass::GoPgDn() {

  GFTRK("EditGoPgDn");

  // Not at the last line in the msg?
  if(currline->next) {

    // Go down to the last displayed line in the window
    uint _newrow = row;
    while((_newrow < maxrow) AND currline->next) {
      currline = currline->next;
      _newrow++;
    }

    // If there are more lines after the last line, start displaying from the top
    if(currline->next) {
      refresh(currline, minrow);

      // Set current line
      _newrow = 0;
      while((_newrow < row) AND currline->next) {
        currline = currline->next;
        _newrow++;
      }

      // Move cursor row if necessary
      if(_newrow < row)
        row = _newrow;
    }
    else {
      row = _newrow;
    }
  }
  else {
    GoBotMsg();
  }

  if((col+1) > strlen(currline->text))
    GoEOL();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::Tab() {

  GFTRK("EditTab");

  int tabsz = CFG->disptabsize ? CFG->disptabsize : 1;

  // Move to the next tab position
  do {
    if(insert)
      insertchar(' ');
    else if(currline->text[col] != '\n')
      GoRight();
    else
      break;
  } while(col % tabsz);


  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::ReTab() {

  GFTRK("EditTabReverse")

  int tabsz = CFG->disptabsize ? CFG->disptabsize : 1;

  // Move to the next tab position
  do {
    if(not col)
      break;

    if(insert)
      DelLeft();
    else
      GoLeft();
  } while(col % tabsz);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::DeleteEOL() {

  GFTRK("EditDeleteEOL");

  int _has_linefeed = strchr(currline->text,'\n') ? YES : NO;

  currline->text[col] = NUL;
  THROW_CHECKPTR(currline->text);

  if(_has_linefeed) {
    strcat(currline->text, "\n");
    THROW_CHECKPTR(currline->text);
  }

  clreol();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::deleteline(int zapquotesbelow) {

  GFTRK("Editdeleteline");

  bool done = false;

  while(not done) {

    if(zapquotesbelow AND NOT ((currline->type & GLINE_QUOT) OR strblank(currline->text)))
      break;

    // Pointer to the deleted line
    Line* _deletedline = currline;

    // Pointers to the previous and next lines
    Line* _prevline = currline->prev;
    Line* _nextline = currline->next;

    // Set the new current line to the next line (which may be NULL!)
    currline = _nextline;

    // If all lines were deleted, allocate a blank one
    if(currline == NULL) {
      currline = (Line*)throw_xcalloc(1, sizeof(Line));
      currline->text = throw_strdup("");
      done = true;
    }

    // Link the new current line to the previous line
    currline->prev = _prevline;

    // Link the previous line to this line
    if(_prevline)
      _prevline->next = currline;

    // Link the deleted line to the killbuffer
    if(Edit__killbuf) {
      Edit__killbuf->next = _deletedline;
      _deletedline->prev = Edit__killbuf;
      Edit__killbuf = _deletedline;
    }
    else {
      Edit__killbuf = _deletedline;
      Edit__killbuf->prev = NULL;
    }
    Edit__killbuf->next = NULL;

    // Move the cursor to EOL if necessary
    if((col+1) > strlen(currline->text))
      GoEOL();

    if(NOT zapquotesbelow)
      break;
  }

  // Refresh display from cursor row
  refresh(currline, row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::UnDelete() {

  GFTRK("EditUnDelete");

  // If there are deleted lines
  if(Edit__killbuf) {

    Line* _prevline = Edit__killbuf->prev;

    Edit__killbuf->prev = currline->prev;
    Edit__killbuf->next = currline;
    if(currline->prev)
      currline->prev->next = Edit__killbuf;
    currline->prev = Edit__killbuf;
    currline = Edit__killbuf;
    Edit__killbuf = _prevline;
    if(Edit__killbuf)
      Edit__killbuf->next = NULL;

    // Move the cursor to EOL if necessary
    if((col+1) > strlen(currline->text))
      GoEOL();

    refresh(currline, row);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::ZapQuoteBelow() {

  GFTRK("ZapQuoteBelow");

  deleteline(TRUE);
  GoUp();
  GoEOL();
  Newline();
}


//  ------------------------------------------------------------------

Line* IEclass::findtopline() {

  GFTRK("Editfindtopline");

  uint  _toprow  = row;
  Line* _topline = currline;

  while(_topline->prev AND (_toprow > minrow)) {
    _topline = _topline->prev;
    _toprow--;
  }

  GFTRK(NULL);

  return _topline;
}


//  ------------------------------------------------------------------

void IEclass::savefile(int __status) {

  Subj statbak;

  GFTRK("Editsavefile");

  // Turn off cursor and put up a wait window
  int wascursoron = NOT vcurhidden();
  cursoroff();

  strcpy(statbak, information);
  update_statusline(LNG->Wait+1);

  // Open the save file
  const char* editorfile = AddPath(CFG->goldpath, EDIT->File());
  remove(editorfile);
  FILE* _fp = fsopen(editorfile, "wb", CFG->sharemode);
  if(_fp) {

    // Find the first line
    Line* _saveline = findfirstline();

    // First save the "unfinished" identifier
    if(__status == MODE_UPDATE) {
      fputs(unfinished, _fp);
      fputs("\r\n", _fp);
    }

    // Save as whole paragraphs
    while(_saveline) {

      // Copy the line to a buffer
      char _buf[EDIT_BUFLEN];
      strcpy(_buf, _saveline->text);

      // If a LF was found, replace it with a CR/LF combo
      char* _lfptr = strchr(_buf, '\n');
      if(_lfptr)
        strcpy(_lfptr, "\r\n");

      // Save the line
      fputs(_buf, _fp);

      // Continue with the next line
      _saveline = _saveline->next;
    }

    // Close save file and remove wait window
    fclose(_fp);
  }

  update_statusline(statbak);

  if(wascursoron)
    cursoron();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::SaveFile() {

  GFTRK("EditSaveFile");

  savefile(MODE_UPDATE);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::SaveMsg() {

  GFTRK("EditSaveMsg");

  done = MODE_SAVE;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

int IEclass::isempty(Line* __line) {

  if(__line == NULL)
    __line = currline;

  return (*__line->text == '\n') OR (*__line->text == NUL);
}


//  ------------------------------------------------------------------

int IEclass::reflowok(char* __qstr) {

  // Stop reflow if there is no next line
  if(currline->next == NULL)
    return FALSE;

  // Stop reflow if the next line is empty
  if(isempty(currline->next))
    return FALSE;

  // Stop reflow if the next line is a control line
  if(currline->next->type & (GLINE_HIDD|GLINE_KLUD|GLINE_TEAR|GLINE_ORIG|GLINE_TAGL))
    return FALSE;

  // Stop reflow if the quotestring on the next line is not the same
  uint _qlen2;
  char _qstr2[100];
  GetQuotestr(currline->next->text, _qstr2, &_qlen2);
  if(NOT strieql(__qstr, _qstr2))
    return FALSE;

  return TRUE;
}


//  ------------------------------------------------------------------

void IEclass::Reflow() {

  GFTRK("EditReflow");

  // Skip empty lines
  while(isempty()) {
    if(currline->next)
      GoDown();
    else
      return;
  }

  // Get the first quotestring
  uint _qlen1;
  char _qstr1[100];
  GetQuotestr(currline->text, _qstr1, &_qlen1);

  // Strip leading spaces from the first line
  char* ptr = SkipWht(currline->text + _qlen1);
  memmove(currline->text+_qlen1, ptr, strlen(ptr)+1);

  // Perform the reflow
  while(reflowok(_qstr1)) {

    // Work on the current line until it is done
    Line* _thisline = currline;
    while(_thisline == currline) {

      // Stop reflow?
      if(NOT reflowok(_qstr1))
        break;

      // Go to the EOL, insert a space and delete the LF
      GoEOL();
      if(col+1 < maxcol) {
        insertchar(' ');
        DelChar();
      }
      else {
        GoDown();
        col = mincol;
      }
    }
  }

  // Go to the next line
  displine(currline,row);
  GoDown();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::ExitMsg() {

  GFTRK("EditExitMsg");

  done = MODE_QUIT;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::DelLine() {

  GFTRK("EditDelLine");

  cursoroff();
  deleteline();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::ToUpper() {

  GFTRK("EditToUpper");

  currline->text[col] = to_upper(currline->text[col]);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::ToLower() {

  GFTRK("EditToLower");

  currline->text[col] = to_lower(currline->text[col]);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::ToggleCase() {

  GFTRK("EditToggleCase");

  if(is_upper(currline->text[col]))
    currline->text[col] = to_lower(currline->text[col]);
  else
    currline->text[col] = to_upper(currline->text[col]);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::LookupCursor() {

  GFTRK("EditLookupCursor");

  LookupNode(msgptr, currline->text+col, LOOK_NAME);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::LookupDest() {

  GFTRK("EditLookupDest");

  LookupNode(msgptr, "", LOOK_DEST);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::LookupOrig() {

  GFTRK("EditLookupOrig");

  LookupNode(msgptr, "", LOOK_ORIG);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::Soundkill() {

  HandleGEvent(EVTT_STOPVOICE);
  #if defined(GFTRK_ENABLE)
  //__gftrk_log();
  #endif
}


//  ------------------------------------------------------------------

void IEclass::statusline() {

  if(chartyped) {
    if(EDIT->Completion.First()) {
      do {
        register const char* trig = EDIT->Completion.Trigger();
        uint tlen = strlen(trig);
        if(col >= tlen) {
          if(strneql(trig, currline->text+col-tlen, tlen)) {
            uint n;
            for(n=0; n<tlen; n++)
              DelLeft();
            const char* cptr = EDIT->Completion.Text();
            uint clen = strlen(cptr);
            for(n=0; n<clen; n++)
              insertchar(*cptr++);
            HandleGEvent(EVTT_EDITCOMPLETION);
            break;
          }
        }
      } while(EDIT->Completion.Next());
    }
  }

  char _buf[EDIT_BUFLEN];
  *_buf = NUL;

  if(EDIT->Comment.First()) {
    do {
      register const char* trig = EDIT->Comment.Trigger();
      uint tlen = strlen(trig);
      if(col >= tlen) {
        if(strnieql(trig, currline->text+col-tlen, tlen)) {
          strcpy(_buf, EDIT->Comment.Text());
          break;
        }
      }
    } while(EDIT->Comment.Next());
  }

  update_statuslinef(LNG->EditStatus, 1+thisrow, 1+col, _buf);
  if(*_buf and CFG->switches.get(beepcomment)) {
    HandleGEvent(EVTT_EDITCOMMENT);
  }
}


//  ------------------------------------------------------------------

int IEclass::handlekey(Key __key) {

  switch(__key) {
    case KK_EditAbort:            Abort();              break;
    case KK_EditAskExit:          AskExit();            break;
    case KK_EditClearDeleteBuf:   ClearDeleteBuf();     break;
    case KK_EditClearPasteBuf:    ClearPasteBuf();      break;
    case KK_EditCopyAboveChar:    CopyAboveChar();      break;
    case KK_EditDelChar:          DelChar();            break;
    case KK_EditDeleteEOL:        DeleteEOL();          break;
    case KK_EditDelLeft:          DelLeft();            break;
    case KK_EditDelLine:          DelLine();            break;
    case KK_EditDelLtWord:        DelLtWord();          break;
    case KK_EditDelRtWord:        DelRtWord();          break;
    case KK_EditDosShell:         DosShell();           break;
    case KK_EditDupLine:          DupLine();            break;
    case KK_EditExitMsg:          ExitMsg();            break;
    case KK_EditExportText:       ExportText();         break;
    case KK_EditGoBegLine:        GoBegLine();          break;
    case KK_EditGoBotLine:        GoBotLine();          break;
    case KK_EditGoBotMsg:         GoBotMsg();           break;
    case KK_EditGoDown:           GoDown();             break;
    case KK_EditGoEOL:            GoEOL();              break;
    case KK_EditGoLeft:           GoLeft();             break;
    case KK_EditGoPgDn:           GoPgDn();             break;
    case KK_EditGoPgUp:           GoPgUp();             break;
    case KK_EditGoRight:          GoRight();            break;
    case KK_EditGoTopLine:        GoTopLine();          break;
    case KK_EditGoTopMsg:         GoTopMsg();           break;
    case KK_EditGoUp:             GoUp();               break;
    case KK_EditGoWordLeft:       GoWordLeft();         break;
    case KK_EditGoWordRight:      GoWordRight();        break;
    case KK_EditHeader:           Header();             break;
    case KK_EditImportQuotebuf:   ImportQuotebuf();     break;
    case KK_EditImportText:       ImportText();         break;
    case KK_EditLoadFile:         LoadFile();           break;
    case KK_EditLookupCursor:     LookupCursor();       break;
    case KK_EditLookupDest:       LookupDest();         break;
    case KK_EditLookupOrig:       LookupOrig();         break;
    case KK_EditNewline:          Newline();            break;
    case KK_EditQuitNow:          QuitNow();            break;
    case KK_EditReflow:           Reflow();             break;
    case KK_EditSaveFile:         SaveFile();           break;
    case KK_EditSaveMsg:          SaveMsg();            break;
    case KK_EditSoundkill:        Soundkill();          break;
    case KK_EditSpellCheck:       SpellCheck();         break;
    case KK_EditTab:              Tab();                break;
    case KK_EditTabReverse:       ReTab();              break;
    case KK_EditToggleCase:       ToggleCase();         break;
    case KK_EditToggleInsert:     ToggleInsert();       break;
    case KK_EditToLower:          ToLower();            break;
    case KK_EditToUpper:          ToUpper();            break;
    case KK_EditUndefine:                               break;
    case KK_EditUnDelete:         UnDelete();           break;
    case KK_EditZapQuoteBelow:    ZapQuoteBelow();      break;

    // Block functions
    case KK_EditAnchor:           BlockAnchor();        break;
    case KK_EditCopy:             BlockCopy();          break;
    case KK_EditCut:              BlockCut();           break;
    case KK_EditPaste:            BlockPaste();         break;

    default:
      if(NOT PlayMacro(__key, KT_E))
        return FALSE;
  }

  return TRUE;
}


//  ------------------------------------------------------------------

int IsMacro(Key key, int type);

int IEclass::Start(int __mode, uint* __position, GMsg* __msg) {

  GFTRK("EditStart");

  thisrow = 0;
  quitnow = NO;
  col = mincol;
  row = minrow;
  msgptr = __msg;
  msgmode = __mode;
  currline = __msg->lin;

  if(AA->isinternet() and CFG->soupexportmargin <= CFG->dispmargin)
    margintext = CFG->soupexportmargin;
  else
    margintext = CFG->dispmargin;

  marginquotes = EDIT->QuoteMargin();
  if(marginquotes > margintext)
    marginquotes = margintext;

  whelppcat(H_Editor);

  if(currline == NULL) {
    currline = (Line*)throw_xcalloc(1, sizeof(Line));
    currline->text = throw_strdup("\n");
  }

  // Check if there is an unfinished backup message
  FILE* _fp = fsopen(AddPath(CFG->goldpath, EDIT->File()), "rt", CFG->sharemode);
  if(_fp) {
    char _buf[EDIT_BUFLEN];
    fgets(_buf, sizeof(_buf), _fp);
    fclose(_fp);
    if(striinc(unfinished, _buf)) {
      w_info(LNG->UnfinishedMsg);
      update_statusline(LNG->LoadUnfinished);
      HandleGEvent(EVTT_ATTENTION);
      Key _ch = getxch();
      w_info(NULL);
      if(_ch != Key_Esc) {
        LoadFile();
        *__position = 0;
        remove(AddPath(CFG->goldpath, EDIT->File()));
      }
    }
  }

  if(*__position) {
    for(uint _posrow=0; _posrow < *__position; _posrow++) {
      if(currline->next) {
        currline = currline->next;
        row++;
      }
    }
    thisrow = row;
  }

  done = NO;

  // If the starting line is outside the first screenful
  if(*__position >= (maxrow+1)) {
    refresh(currline->prev, minrow);
    row = 1;
  }
  else {
    refresh(findfirstline(), minrow);
  }

  gotorowcol(mincol, minrow);
  dispins();

  Time _lasttime = gtime(NULL);

  while(NOT done) {

    statusline();

    gotorowcol(col, row);

    int backattr = dispchar(currline->text[col], C_READC);
    gotorowcol(col, row);

    cursoron();
    if(insert)
      vcursmall();
    else
      vcurlarge();

    Key _ch;

    do {
      _ch = getxchtick();

      if(EDIT->AutoSave()) {
        Time _thistime = gtime(NULL);
        if(_thistime >= (_lasttime+EDIT->AutoSave())) {
          _lasttime = _thistime;
          SaveFile();
        }
      }

      if(_ch == Key_Tick)
        CheckTick(KK_EditQuitNow);

    } while(_ch == Key_Tick);

    int ismacro = FALSE;
    Key _kk = SearchKey(_ch, EditKey, EditKeys);
    if(_kk) {
      _ch = _kk;
    }
    else {
      ismacro = IsMacro(_ch, KT_E);
    }

    dispchar(currline->text[col], backattr);

    chartyped = FALSE;
    if((_ch < KK_Commands) AND (_ch & 0xFF) AND NOT ismacro) {
      chartyped = TRUE;
      _ch &= 0xFF;
      insertchar((char)_ch);
    }
    else if(handlekey(_ch)) {
      getthisrow(currline);
    }
  }

  cursoroff();

  msgptr->lin = findfirstline();
  savefile(quitnow ? MODE_UPDATE : MODE_SAVE);
  whelpop();

  // Prune killbuffer
  if(Edit__killbuf) {
    currline = Edit__killbuf;

    int _count = EDIT->UnDelete();
    while(currline AND _count--)
      currline = currline->prev;

    if(currline)
      if(currline->next)
        currline->next->prev = NULL;

    while(currline) {
      throw_release(currline->text);
      if(currline->prev) {
        currline = currline->prev;
        throw_xrelease(currline->next);
      }
      else
        throw_xrelease(currline);
    }
  }

  *__position = 1 + thisrow;

  GFTRK(NULL);

  return done;
}


//  ------------------------------------------------------------------

