
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geedit.h,v 1.1.1.1 1998/11/21 19:38:02 odinn Exp $
//  ------------------------------------------------------------------
//  The Internal Editor (IE).
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Screen coordinate models
//  ------------------------------------------------------------------
//
//  00    01
//  01   Msg# :                     02
//  02   From :                     03
//  03   To   :                     04
//  04   Subj :                     05
//  05    06
//  06  00  EDIT_MINROW             07  EDIT_MIN_ROW
//  07  01                          08
//  08  02                          09
//  09  03                          10
//  10  04                          11
//  11  05                          12
//  12  06                          13
//  13  07                          14
//  14  08                          15
//  15  09                          16
//  16  10                          17
//  17  11                          18
//  18  12                          19
//  19  13                          10
//  10  14                          21
//  21  15                          22
//  22  16                          23
//  23  17  EDIT_MAXROW             24  EDIT_MAX_ROW
//  24  GoldED xxxx (statusline)   25
//                                 
//                                  one-based old-style coord
//      editor window corrdinates
//    actual screen coordinates
//
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Defines

#define EDIT_BUFLEN       256
#define EDIT_PARABUFLEN   2048


//  ------------------------------------------------------------------
//  Globals

extern int CFG__editquotewrap;

extern Line* Edit__killbuf;
extern Line* Edit__pastebuf;

extern Path Edit__exportfilename;


//  ------------------------------------------------------------------
//  Reality check define and function

#ifndef NDEBUG
#define _test_halt(__t)             if(__t) { debugtest(#__t,   0,   0, __FILE__, __LINE__, FALSE); }
#define _test_haltab(__t, __a, __b) if(__t) { debugtest(#__t, __a, __b, __FILE__, __LINE__, TRUE);  }
#endif


//  ------------------------------------------------------------------
//  Internal Editor Class

class IEclass {

public:

  gwindow editwin;       // window

protected:

  //  ----------------------------------------------------------------
  //  Editor window data

  int    win_mincol;     // First column
  int    win_minrow;     // First row
  int    win_maxcol;     // Last column
  int    win_maxrow;     // Last row
  int    win_border;     // Border type
  int    win_hasborder;  // 1 == window has a border, 0  == no border


  //  ----------------------------------------------------------------
  //  Window-relative minimum/maximum values

  uint  mincol;
  uint  minrow;
  uint  maxcol;
  uint  maxrow;


  //  ----------------------------------------------------------------
  //  Cursor coordinates

  uint col;
  uint row;
  uint ccol;
  uint crow;


  //  ----------------------------------------------------------------
  //  Misc.

  int  chartyped;
  Line* currline;
  int  done;
  int  insert;
  int  marginquotes;
  int  margintext;
  int  msgmode;
  GMsg* msgptr;
  int  quitnow;
  uint  thisrow;
  char*  unfinished;


  //  ----------------------------------------------------------------
  //  Internal helper functions

  void  clreol          (int __col=-1, int __row=-1);
  void  cursoroff       ();
  void  cursoron        ();
  void  deleteline      (int zapquotesbelow=FALSE);
  int   dispchar        (char __ch, int attr=-1);
  void  dispins         ();
  void  displine        (Line* __line, uint __row);
  void  dispstring      (char* __string, uint __row, int attr=-1);
  int  downoneline     (uint __row);
  void  export          (Line* __exportline, int __endat);
  Line* findanchor      ();
  Line* findfirstline   ();
  Line* findtopline     ();
  void  getthisrow      (Line* __currline);
  void  gotorowcol      (uint __col, uint __row);
  int  handlekey       (Key __key);
  void  import          (Line* __line, char* __filename, bool imptxt = false);
  void  imptxt          (char* __filename, bool imptxt = false);
  void  insertchar      (char __ch);
  Line* insertlinebelow (Line* __currline, char* __text=NULL);
  int  isempty         (Line* __line=NULL);
  void  killkillbuf     ();
  void  killpastebuf    ();
  int  reflowok        (char* __qstr);
  void  refresh         (Line* __currline, uint __row);
  void  savefile        (int __status);
  void  scrolldown      (int __scol, int __srow, int __ecol, int __erow, int __lines=1);
  void  scrollup        (int __scol, int __srow, int __ecol, int __erow, int __lines=1);
  void  setcolor        (Line* __line);
  void  setlinetype     (Line* __line);
  void  statusline      ();
  void  strdelchr       (char* __string, uint __position);
  void  strinschr       (char** __string, char __ch, uint __position);
  void  windowclose     ();
  void  windowopen      ();
  Line* wrapit          (Line** __currline, uint* __curr_col, uint* __curr_row, int __display=YES);
  Line* wrapdel         (Line** __currline, uint* __curr_col, uint* __curr_row, int __display=YES);
  Line* wrapins         (Line** __currline, uint* __curr_col, uint* __curr_row, int __display=YES);

  #ifndef NDEBUG
  void  debugtest       (char* __test, int __a, int __b, char* __file, int __line, int __values);
  #endif


public:

  //  ----------------------------------------------------------------
  //  Constructor and destructor

  IEclass(int __scol, int __ecol, int __srow, int __erow, int __border);
  ~IEclass();
  

  //  ----------------------------------------------------------------
  //  Function to start the editor

  int Start(int __mode, uint* __position, GMsg* __msg);


  //  ----------------------------------------------------------------
  //  User key functions

  void Abort          ();
  void AskExit        ();
  void BlockAnchor    ();
  void BlockCopy      ();
  void BlockCut       ();
  void BlockPaste     ();
  void ClearDeleteBuf ();
  void ClearPasteBuf  ();
  void CopyAboveChar  ();
  void DelChar        ();
  void DeleteEOL      ();
  void DelLeft        ();
  void DelLine        ();
  void DelLtWord      ();
  void DelRtWord      ();
  void DosShell       ();
  void DupLine        ();
  void ExitMsg        ();
  void ExportText     ();
  void GoBegLine      ();
  void GoBotLine      ();
  void GoBotMsg       ();
  void GoDown         ();
  void GoEOL          ();
  void GoLeft         ();
  void GoPgDn         ();
  void GoPgUp         ();
  void GoRight        ();
  void GoTopLine      ();
  void GoTopMsg       ();
  void GoUp           ();
  void GoWordLeft     ();
  void GoWordRight    ();
  void Header         ();
  void ImportQuotebuf ();
  void ImportText     ();
  void LoadFile       ();
  void LookupCursor   ();
  void LookupDest     ();
  void LookupOrig     ();
  void Newline        ();
  void QuitNow        ();
  void Reflow         ();
  void ReTab          ();
  void SaveFile       ();
  void SaveMsg        ();
  void Soundkill      ();
  void SpellCheck     ();
  void Tab            ();
  void ToggleCase     ();
  void ToggleInsert   ();
  void ToLower        ();
  void ToUpper        ();
  void UnDelete       ();
  void ZapQuoteBelow  ();


  //  ----------------------------------------------------------------
};


//  ------------------------------------------------------------------

