
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geedit2.cpp,v 1.1.1.1 1998/11/21 19:38:02 odinn Exp $
//  ------------------------------------------------------------------
//  The Internal Editor (IE), part 2.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include "geedit.h"
#include "gutlcode.h"
#include "gutlclip.h"


//  ------------------------------------------------------------------
//  Constructor

IEclass::IEclass(int __scol, int __ecol, int __srow, int __erow, int __border) {

  win_mincol = __scol;
  win_minrow = __srow;
  win_maxcol = __ecol;
  win_maxrow = __erow;
  win_border = __border;

  win_hasborder = (win_border == 5) ? 0 : 1;

  mincol  = 0;
  minrow  = 0;
  maxcol  = win_maxcol - win_mincol - (2*win_hasborder);
  maxrow  = win_maxrow - win_minrow - (2*win_hasborder);

  col  = 0;
  row  = 0;
  ccol = 0;
  crow = 0;

  chartyped    = FALSE;
  currline     = NULL;
  done         = NO;
  insert       = YES;
  marginquotes = 0;
  margintext   = 0;
  msgmode      = 0;
  msgptr       = NULL;
  quitnow      = NO;
  thisrow      = 0;
  unfinished   = "+$!$+ GoldED Internal Editor: Unfinished Message!";

  windowopen();
}


//  ------------------------------------------------------------------
//  Destructor

IEclass::~IEclass() {

  windowclose();
}


//  ------------------------------------------------------------------

void IEclass::windowopen() {

  // Open editor window without clearing the window area
  #define STYLE_NOCLEAR -1
  int _tmp = gwin.style;
  gwin.style = STYLE_NOCLEAR;
  editwin.open(win_minrow, win_mincol, win_maxrow, win_maxcol, 5, C_READB, C_READW, C_READPB);
  gwin.style = _tmp;
}


//  ------------------------------------------------------------------

void IEclass::windowclose() {

  // Close editor window without removing the window itself
  editwin.unlink();
}


//  ------------------------------------------------------------------

void Edit__killpastebuf() {

  while(Edit__pastebuf) {
    throw_release(Edit__pastebuf->text);
    if(Edit__pastebuf->next) {
      Edit__pastebuf = Edit__pastebuf->next;
      throw_xrelease(Edit__pastebuf->prev);
    }
    else {
      throw_xrelease(Edit__pastebuf);
    }
  }
}


//  ------------------------------------------------------------------

inline void IEclass::killpastebuf() {

  Edit__killpastebuf();
}


//  ------------------------------------------------------------------

void Edit__killkillbuf() {

  while(Edit__killbuf) {
    throw_release(Edit__killbuf->text);
    if(Edit__killbuf->prev) {
      Edit__killbuf = Edit__killbuf->prev;
      throw_xrelease(Edit__killbuf->next);
    }
    else {
      throw_xrelease(Edit__killbuf);
    }
  }
}


//  ------------------------------------------------------------------

inline void IEclass::killkillbuf() {

  Edit__killkillbuf();
}


//  ------------------------------------------------------------------

void FreePastebuf() {

  Edit__killpastebuf();
  Edit__killkillbuf();
}


//  ------------------------------------------------------------------

void IEclass::ClearDeleteBuf() {

  GFTRK("EditClearDeleteBuf");

  killkillbuf();
  HandleGEvent(EVTT_JOBDONE);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::ClearPasteBuf() {

  GFTRK("EditClearPasteBuf");

  killpastebuf();
  HandleGEvent(EVTT_JOBDONE);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::GoBegLine() {

  GFTRK("EditGoBegLine");

  col = mincol;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::DelLtWord() {

  GFTRK("EditDelLtWord");

  char* _ptr;
  char* _ptr2;
  char* _bptr;

  if(col == 0) {
    DelLeft();
    GFTRK(NULL);
    return;
  }

  _bptr = currline->text;
  _ptr = _ptr2 = currline->text + col;
  _ptr--;

  // test test test test     test test test test
  // test test test test ,   test test test test
  // test test test test ,,  test test test test

  if(*_ptr == ' ') {
    while((*_ptr == ' ') AND (_bptr < _ptr))
      _ptr--;
    if(*_ptr != ' ')
      _ptr++;
  }
  else if(isxalnum(*_ptr)) {
    while(isxalnum(*_ptr) AND (_bptr < _ptr))
      _ptr--;
    while((*_ptr == ' ') AND (_bptr < _ptr))
      _ptr--;
    if((*_ptr != ' ') AND (_bptr < _ptr))
      _ptr++;
  }
  else {
    DelLeft();
    GFTRK(NULL);
    return;
  }

  col -= (int)(_ptr2-_ptr);

  memmove(_ptr, _ptr2, strlen(_ptr2)+1);
  THROW_CHECKPTR(currline->text);

  wrapdel(&currline, &col, &row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::DelRtWord() {

  GFTRK("EditDelRtWord");

  char* _ptr;
  char* _ptr2;

  if((currline->text[col+1] == NUL) OR (currline->text[col+1] == '\n')) {
    DelChar();
    GFTRK(NULL);
    return;
  }
  
  _ptr = _ptr2 = currline->text + col;

  // test test test test,   test test test test
  // test test test test,   test test test test

  if(*_ptr == ' ') {
    while((*_ptr == ' ') AND (*_ptr != '\n') AND *_ptr)
      _ptr++;
  }
  else if(isxalnum(*_ptr)) {
    // Delete word
    while(isxalnum(*_ptr) AND (*_ptr != '\n') AND *_ptr)
      _ptr++;
    // Delete spaces after word
    while((*_ptr == ' ') AND (*_ptr != '\n') AND *_ptr)
      _ptr++;
  }
  else {
    DelChar();
    GFTRK(NULL);
    return;
  }

  memmove(_ptr2, _ptr, strlen(_ptr2)+1);
  THROW_CHECKPTR(currline->text);

  wrapdel(&currline, &col, &row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::GoTopMsg() {

  GFTRK("EditGoTopMsg");

  currline = findfirstline();
  refresh(currline, minrow);
  col = mincol;
  row = minrow;
  thisrow = 0;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::GoBotMsg() {

  GFTRK("EditGoBotMsg");

  col = mincol;
  thisrow = 0;
  currline = findfirstline();

  // Go to the last line in the msg
  while(currline->next) {
    currline = currline->next;
    thisrow++;
  }

  // Pointer to the line to display at the top of the window
  Line* _topline = currline;

  // The new cursor row
  row = MinV(thisrow, maxrow);

  // How many lines to go back to get the top line
  int _count = row;

  // Go back to get the top line
  while(_count-- AND _topline->prev)
    _topline = _topline->prev;

  // Refresh the display
  refresh(_topline, minrow);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::GoTopLine() {

  GFTRK("EditGoTopLine");

  int _count = row;

  while(_count-- AND currline->prev) {
    currline = currline->prev;
    thisrow--;
  }

  col = mincol;
  row = minrow;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::GoBotLine() {

  GFTRK("EditGoBotLine");

  while((row < maxrow) AND currline->next) {
    currline = currline->next;
    thisrow++;
    row++;
  }

  col = mincol;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

Line* IEclass::findanchor() {

  GFTRK("Editfindanchor");

  // Rewind to the first line
  Line* _anchor = findfirstline();

  // Search all lines to find the anchor (a line with a block mark)
  while(NOT (_anchor->type & GLINE_BLOK) AND _anchor->next)
    _anchor = _anchor->next;

  GFTRK(NULL);

  // Return pointer to the anchor line or NULL if no anchor was found
  return (_anchor->type & GLINE_BLOK) ? _anchor : (Line*)NULL;
}


//  ------------------------------------------------------------------

void IEclass::BlockAnchor() {

  GFTRK("EditBlockAnchor");

  Line* _anchor = findanchor();

  // Is there an anchor already?
  if(_anchor) {

    // Yes, so replace it with the current line

    // Remove block mark
    _anchor->type &= ~GLINE_BLOK;

    // Is the old anchor different from the current line?
    if(_anchor != currline) {

      // Set the new anchor
      _anchor = currline;
      _anchor->type |= GLINE_BLOK;

      // Find the line at the top
      Line* _topline = findtopline();

      // Refresh display to remove the block color on the old anchor line
      // Just in case the old anchor line is visible
      refresh(_topline, minrow);
    }

    // Remove the old contents of the paste buffer
    killpastebuf();
  }
  else {

    // There was no anchor, so mark the current line as the new anchor
    currline->type |= GLINE_BLOK;
  }

  displine(currline, row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::BlockCopy() {

  GFTRK("EditBlockCopy");

  // Find the anchor, if any
  Line* _anchor = findanchor();

  // Did we find the anchor?
  if(_anchor) {

    Line* _firstcopyline = currline;
    Line* _lastcopyline  = currline;

    // Search below to find the anchor line
    while(_lastcopyline->next AND (_lastcopyline != _anchor))
      _lastcopyline = _lastcopyline->next;

    // Was the anchor line above or on the current line?
    if(_lastcopyline != _anchor) {

      // The last copy line is the current line
      _lastcopyline = currline;

      // Search above to find the anchor line
      while(_firstcopyline->prev AND (_firstcopyline != _anchor))
        _firstcopyline = _firstcopyline->prev;
    }

    // The _firstcopyline and _lastcopyline pointers
    // are now pointing where they should

    // Remove the old paste buffer
    killpastebuf();

    // Pointer to the previous line in the paste buffer
    Line* _prevline = NULL;

    // Copy lines to the paste buffer
    while(1) {

      // Allocate a new line
      Line* _copyline = (Line*)throw_xcalloc(1, sizeof(Line));

      // Copy text and type
      _copyline->text = throw_strdup(_firstcopyline->text);
      _copyline->type = _firstcopyline->type;

      // Link in the new line
      _copyline->prev = _prevline;
      if(_prevline)
        _prevline->next = _copyline;

      // Point the paste buffer to the first line of the copy
      if(Edit__pastebuf == NULL)
        Edit__pastebuf = _copyline;

      // Break out of the loop if the last line was copied
      if(_firstcopyline == _lastcopyline)
        break;

      // Keep pointer to the new line
      _prevline = _copyline;

      // Continue with the next line
      _firstcopyline = _firstcopyline->next;
    } 
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::BlockCut() {

  GFTRK("EditBlockCut");

  // Find the anchor, if any
  Line* _anchor = findanchor();

  // Did we find the anchor?
  if(_anchor) {

    Line* _firstcutline = currline;
    Line* _lastcutline  = currline;
    uint _firstcutlinerow = row;
    uint _lastcutlinerow  = row;
    //int _currrow = row;

    // Search below to find the anchor line
    while(_lastcutline->next AND (_lastcutline != _anchor)) {
      _lastcutline = _lastcutline->next;
      _lastcutlinerow++;
    }

    // Was the anchor line above or on the current line?
    if(_lastcutline != _anchor) {

      // The last cut line is the current line
      _lastcutline = currline;
      _lastcutlinerow = row;

      // Search above to find the anchor line
      while(_firstcutline->prev AND (_firstcutline != _anchor)) {
        _firstcutline = _firstcutline->prev;
        if(_firstcutlinerow)
          _firstcutlinerow--;
      }
    }

    // The _firstcutline and _lastcutline pointers
    // are now pointing where they should

    // Is the first cut line NOT the first line in the msg?
    if(_firstcutline->prev) {

      // Link the previous line of the first cut line
      // to the next line of the last cut line
      _firstcutline->prev->next = _lastcutline->next;
    }

    // If there is a next line for the last cut line
    if(_lastcutline->next) {

      // Set the current line to it
      currline = _lastcutline->next;
      row = _lastcutlinerow + 1;

      // Link the next line of the last cut line
      // to the previous line of the first cut line
      _lastcutline->next->prev = _firstcutline->prev;
    }
    else {

      // Set the current line to the previous line for the first cut line
      currline = _firstcutline->prev;
      row = _firstcutlinerow - 1;
    }

    // Remove the previous/next links from the cut lines
    _firstcutline->prev = _lastcutline->next = NULL;

    // If the current line was lost, allocate a blank one
    if(currline == NULL) {
      currline = (Line*)throw_xcalloc(1, sizeof(Line));
      currline->text = throw_strdup("\n");
      row = 0;
    }

    // Remove the old paste buffer and point to the new cut
    killpastebuf();
    Edit__pastebuf = _firstcutline;

    // Refresh the display
    if(NOT RngV(row, minrow, maxrow)) {
      row = minrow;
      refresh(currline, minrow);
    }
    else {
      row = MaxV(_firstcutlinerow, minrow);
      Line* _topline = findtopline();
      refresh(_topline, minrow);
    }
    col = mincol;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::BlockPaste() {

  GFTRK("EditBlockPaste");

  if(Edit__pastebuf) {

    Line* _newcurrline = NULL;
    Line* _pasteline   = Edit__pastebuf;
    Line* _prevline    = currline->prev;

    // For each of the lines in the paste buffer
    while(_pasteline) {

      // Allocate a new line
      Line* _templine = (Line*)throw_xcalloc(1, sizeof(Line));

      // Set the new current line to the first pasted line
      if(_newcurrline == NULL)
        _newcurrline = _templine;

      // Link in the pasted line
      currline->prev = _templine;
      _templine->next = currline;
      _templine->prev = _prevline;
      if(_prevline)
        _prevline->next = _templine;

      // Duplicate the text and type of the pasted line
      _templine->text = throw_strdup(_pasteline->text);
      _templine->type = _pasteline->type & ~GLINE_BLOK;

      // Store pointer to the new pasted line
      _prevline = _templine;

      // Continue with the next line in the paste buffer
      _pasteline = _pasteline->next;
    }

    // Set the new current line
    if(_newcurrline)
      currline = _newcurrline;

    // Refresh the display
    Line* _topline = findtopline();
    refresh(_topline, minrow);
    col = mincol;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::LoadFile() {

  GFTRK("EditLoadFile");

  // Open the file to load
  FILE* _fp = fsopen(AddPath(CFG->goldpath, EDIT->File()), "rb", CFG->sharemode);
  if(_fp) {

    // Pop up a wait window
    cursoroff();
    w_info(LNG->Wait);

    // Find the first line
    Line* _line = findfirstline();

    // Remove all lines
    while(_line) {
      Line* _nextline = _line->next;
      throw_release(_line->text);
      throw_xfree(_line);
      _line = _nextline;
    }

    // Remove message text and reset pointers
    throw_release(msgptr->txt);
    currline = msgptr->lin = NULL;

    // Allocate space for new message text
    msgptr->txt = (char*)throw_calloc(1, (uint)(fsize(_fp)+256));

    // Eat the backup marking line
    char _buf[EDIT_BUFLEN];
    fgets(_buf, sizeof(_buf), _fp);
    if(NOT striinc(unfinished, _buf))
      rewind(_fp);

    // Load the file and close it
    fread(msgptr->txt, 1, (uint)fsize(_fp), _fp);
    fclose(_fp);

    // Index message text
    msgptr->TextToLines(margintext-1);
    _line = currline = msgptr->lin;

    // Change lines to internal editor format
    while(_line) {
      strtrimspace(_line->text);
      if(_line->type & GLINE_HARD) {
        strcat(_line->text, "\n");
        THROW_CHECKPTR(_line->text);
      }
      else {
        strcat(_line->text, " ");
        THROW_CHECKPTR(_line->text);
      }
      _line = _line->next;
    }

    // Remove the wait window
    w_info(NULL);

    // Display the loaded message
    Line* l = findfirstline();
    uint i;

    for(i=0; l->next and i<thisrow-row; i++)
      l = l->next;

    refresh(l, minrow);

    for(i=0; l->next and i<row; i++)
      l = l->next;

    currline = l;
    col = mincol;

  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

inline char uuencode_enc(int c) { return (char)(c ? (c & 077) + ' ': '`'); }

void IEclass::import(Line* __line, char* __filename, bool imptxt) {

  GFTRK("Editimport");

  // Save the unfinished msg first of all
  savefile(MODE_UPDATE);

  update_statusline(LNG->ImportFile);

  // Set initial import filename or wildcards
  if(__filename) {
    AA->SetInputfile(__filename);
  }
  else {
    if(*AA->Inputfile() == NUL)
      AA->SetInputfile("*");
  }

  strcpy(CFG->inputfile, AA->Inputfile());

  int ImportMode;

  if(imptxt)
    ImportMode = 0;
  else {

    GMenuImportTxt MenuImportTxt;

    ImportMode = MenuImportTxt.Run();
  }

  Path filenamebuf;
  char* _filenameptr = filenamebuf;
  char* _selectedfile = NULL;

  // Should the imported text be quoted or uuencoded?
  #define quoteit    (ImportMode == 1)
  #define uuencode   (ImportMode == 2)
  #define base64     (ImportMode == 3)
  #define getclip    (ImportMode == 4)
  #define binary     (uuencode or base64)

  if(in_range(ImportMode, 0, 3) and edit_pathname(CFG->inputfile, sizeof(Path), LNG->ImportWhichFile, H_ImportFile)) {

    AA->SetInputfile(CFG->inputfile);

    // Pointer to the filename string
    _filenameptr = strcpy(filenamebuf, AA->Inputfile());

    // Check for wildcards
    // Is the filename a directory?
    if(is_dir(_filenameptr)) {

      // Does the filename contain wildcards?
      if(NOT strpbrk(_filenameptr, "*?")) {

        // Add match-all wildcards
        strcat(AddBackslash(_filenameptr), "*");
      }
    }

    // Pointer to selected filename or NULL if no file is selected
    _selectedfile = _filenameptr;

    // Does the filename contain wildcards?
    if(strpbrk(_filenameptr, "*?")) {

      // Set selection window title and statusline
      set_title(LNG->ImportTitle, TCENTER, C_MENUT);
      update_statuslinef(LNG->ImportStatus, _filenameptr);

      // Copy filename with wildcards to a temp buffer
      Path _filenamebuf;
      strcpy(_filenamebuf, _filenameptr);

      // Start the file picker
      _selectedfile = wpickfile(
        win_minrow,
        win_mincol,
        win_maxrow,
        win_maxcol,
        W_BMENU,
        C_MENUB,
        C_MENUW,
        C_MENUS,
        NO,
        _filenamebuf,
        _filenameptr,
        maketitle
      );

      // If a file was selected, copy the filename
      if(_selectedfile)
        strcpy(_filenameptr, _selectedfile);
    }
  }

  if(_selectedfile or getclip) {

    // Open the file/clipboard
    FILE* fp = NULL;
    gclipbrd clipbrd;

    if(getclip)
      _filenameptr = strcpy(filenamebuf, CLIP_NAME);

    if(getclip ? clipbrd.openread() :
       (fp = fsopen(_filenameptr, binary ? "rb" : "rt", CFG->sharemode))!=NULL) {

      update_statuslinef(LNG->ImportStatus, _filenameptr);

      // Allocate paragraph read buffer
      char* _parabuf = (char*)throw_malloc(EDIT_PARABUFLEN);

      // Add import begin text, if any
      if(*CFG->importbegin) {
        sprintf(_parabuf, "%s\n", CFG->importbegin);
        strischg(_parabuf, "@file", CleanFilename(_filenameptr));
        __line = insertlinebelow(__line, _parabuf);
        setlinetype(__line);
      }

      if(uuencode) {

        sprintf(_parabuf, "begin 644 %s\n", CleanFilename(_filenameptr));
        __line = insertlinebelow(__line, _parabuf);
        setlinetype(__line);

        while(1) {
          char ibuf[80];
          char* iptr = ibuf;
          char* optr = _parabuf;
          int n = fread(ibuf, 1, 45, fp);
          if(n < 45)
            memset(ibuf+n, 0, 45-n);
          *optr++ = uuencode_enc(n);
          for(int i=0; i<n; i+=3,iptr+=3) {
            *optr++ = uuencode_enc(*iptr >> 2);
            *optr++ = uuencode_enc(((*iptr << 4) & 060) | ((iptr[1] >> 4) & 017));
            *optr++ = uuencode_enc(((iptr[1] << 2) & 074) | ((iptr[2] >> 6) & 03));
            *optr++ = uuencode_enc(iptr[2] & 077);
          }
          *optr++ = '\n';
          *optr = NUL;
          __line = insertlinebelow(__line, _parabuf);
          // set type to text
          __line->type &= ~(GLINE_HIDD|GLINE_KLUD|GLINE_QUOT);
          if(n <= 0)
            break;
        }

        __line = insertlinebelow(__line, "end\n");
        setlinetype(__line);
      }
      else if(base64) {

        base64_engine b64;

        sprintf(_parabuf, "Content-type: application/octet-stream; name=\"%s\"\n", CleanFilename(_filenameptr));
        __line = insertlinebelow(__line, _parabuf);
        setlinetype(__line);
        sprintf(_parabuf, "Content-transfer-encoding: base64\n");
        __line = insertlinebelow(__line, _parabuf);
        setlinetype(__line);

        sprintf(_parabuf, "\n");
        __line = insertlinebelow(__line, _parabuf);
        setlinetype(__line);

        for(;;) {
          char ibuf[80];
          char* optr = _parabuf;
          int n = fread(ibuf, 1, 54, fp);
          optr = b64.encode(optr, ibuf, n);
          *optr++ = '\n';
          *optr = NUL;
          __line = insertlinebelow(__line, _parabuf);
          setlinetype(__line);
          if(n <= 0)
            break;
        }
      }
      else {

        int level = LoadCharset(AA->Xlatimport(), CFG->xlatlocalset);
        char* buf = (char*) throw_malloc(EDIT_PARABUFLEN);

        // Read paragraphs
        while(getclip ? clipbrd.read(_parabuf, EDIT_PARABUFLEN-6) : fgets(_parabuf, EDIT_PARABUFLEN-6, fp)) {

          XlatStr(buf, _parabuf, level, CharTable);

          // Invalidate tearline
          if(*CFG->invalidate.tearline[0])
            doinvalidate(buf, CFG->invalidate.tearline[0], CFG->invalidate.tearline[1]);

          // Invalidate originline
          if(*CFG->invalidate.origin[0])
            doinvalidate(buf, CFG->invalidate.origin[0], CFG->invalidate.origin[1]);

          // Invalidate SEEN-BY's
          if(*CFG->invalidate.seenby[0])
            doinvalidate(buf, CFG->invalidate.seenby[0], CFG->invalidate.seenby[1]);

          // Invalidate kludge chars
          //strchg(_parabuf, CTRL_A, '@');

          // Insert a quotestring if asked
          if(quoteit)
            strins(" > ", buf, 0);

          // Replace tabs with single spaces
          strchg(buf, HT, ' ');

          // Translate softcr to configured char
          if(EDIT->SoftCrXlat())
            strchg(buf, SOFTCR, EDIT->SoftCrXlat());

          // Copy the paragraph to the new line and retype it
          Line* _newline = __line = insertlinebelow(__line, buf);
          setlinetype(_newline);

          // If the paragraph is longer than one line
          uint _wrapmargin = (_newline->type & GLINE_QUOT) ? marginquotes : margintext;
          if(strlen(_newline->text) >= _wrapmargin) {

            // Wrap it
            uint _tmpcol = 0;
            uint _tmprow = 0;
            _newline = wrapins(&_newline, &_tmpcol, &_tmprow, NO);
          }

          __line = _newline;
        }

        throw_free(buf);
      }

      // Add import end text, if any
      if(*CFG->importend OR *CFG->importbegin) {
        sprintf(_parabuf, "%s\n", *CFG->importend ? CFG->importend : CFG->importbegin);
        strischg(_parabuf, "@file", CleanFilename(_filenameptr));
        __line = insertlinebelow(__line, _parabuf);
        setlinetype(__line);
      }

      throw_free(_parabuf);

      if(getclip)
        clipbrd.close();
      else
        fclose(fp);
    }
    else {
      w_infof(LNG->CouldNotOpen, _filenameptr);
      waitkeyt(10000);
      w_info(NULL);
    }
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::imptxt(char* __filename, bool imptxt) {

  GFTRK("Editimptxt");

  msgptr->lin = findfirstline();
  import(currline, __filename, imptxt);
  refresh(currline, row);
  col = mincol;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::ImportText() {

  GFTRK("EditImportText");

  imptxt(NULL);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::ImportQuotebuf() {

  GFTRK("EditImportQuotebuf");

  Path _quotebuf;

  GetCurrQuotebuf(_quotebuf);
  imptxt(_quotebuf, true);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::export(Line* __exportline, int __endat) {

  GFTRK("Editexport");

  update_statusline(LNG->ExportFile);

  if(edit_string(Edit__exportfilename, sizeof(Path), LNG->ExportWhatFile, H_ExportFile)) {

    // Pointer to export filename
    char* _filenameptr = Edit__exportfilename;

    // Is append requested?
    if(*_filenameptr == '+')
      _filenameptr++;

    FILE* _fp = fsopen(_filenameptr, (*Edit__exportfilename == '+') ? "at" : "wt", CFG->sharemode);
    if(_fp) {

      update_statuslinef(LNG->ExportStatus, Edit__exportfilename);
      fputc('\n', _fp);

      while((__endat ? __exportline != currline : 1) AND __exportline) {
        fputs(__exportline->text, _fp);
        if(NOT strchr(__exportline->text, '\n'))
          fputc('\n', _fp);
        __exportline = __exportline->next;
      }

      fclose(_fp);
    }
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::SpellCheck() {

  GFTRK("EditSpellCheck");

  char _buf[EDIT_BUFLEN];
  char _buf2[EDIT_BUFLEN];

  savefile(MODE_SAVE);
  strcpy(_buf, EDIT->SpellChecker());
  strischg(_buf, "@File", AddPath(CFG->goldpath, EDIT->File()));
  sprintf(_buf2, LNG->SpellChecker, _buf);
  ShellToDos(_buf, _buf2, LGREY|_BLACK, YES);
  LoadFile();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::ExportText() {

  GFTRK("EditExportText");

  if(Edit__pastebuf) {
    export(Edit__pastebuf, NO);
    GFTRK(NULL);
    return;
  }

  Line* _anchor = findanchor();
  if(_anchor) {
    export(_anchor, YES);
    GFTRK(NULL);
    return;
  }

  export(findfirstline(), NO);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::DosShell() {

  GFTRK("EditDosShell");

  ShellToDos(getenv(GOLD_SHELL_ENV), LNG->DOS_Shell, LGREY|_BLACK, YES);
  cursoron();
  cursoroff();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::dispins() {

  GFTRK("Editdispins");

  if(insert)
    HeaderView.window.prints(5, MAXCOL-6, C_HEADT, LNG->Ins);
  else {
    char _lbuf[6];
    memset(_lbuf, _box_table[W_BHEAD][1], 5);
    _lbuf[5] = NUL;
    HeaderView.window.prints(5, MAXCOL-6, C_HEADB|ACSET, _lbuf);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::ToggleInsert() {

  GFTRK("EditToggleInsert");

  insert = !insert;
  dispins();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::Header() {

  GFTRK("EditHeader");

  windowclose();
  //EditHeaderinfo(msgmode, msgptr);
  EditHeaderinfo(msgmode, HeaderView);
  windowopen();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::Abort() {

  GFTRK("EditAbort");

  cursoroff();
  GMenuDropmsg MenuDropmsg;
  if(MenuDropmsg.Run()) {
    done = MODE_QUIT;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::AskExit() {

  GFTRK("EditAskExit");

  cursoroff();
  GMenuQuit MenuQuit;
  gkbd.quitall = MenuQuit.Run();
  if(gkbd.quitall) {
    GMenuDropmsg MenuDropmsg;
    if(MenuDropmsg.Run())
      done = MODE_QUIT;
    else
      done = MODE_SAVE;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void IEclass::QuitNow() {

  GFTRK("EditQuitNow");

  quitnow = CFG->switches.get(timeoutsavemsg) ? NO : YES;
  done = CFG->switches.get(timeoutsavemsg) ? MODE_SAVE : MODE_QUIT;
  gkbd.quitall = YES;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

IEclass* _editor = NULL;

int EditMsg(int __mode, uint* __position, GMsg* __msg) {

  _editor = new IEclass(0, MAXCOL-1, MINROW, MAXROW-2, 5);
  throw_new(_editor);
  int retval = _editor->Start(__mode, __position, __msg);
  delete _editor;
  return retval;
}


//  ------------------------------------------------------------------

