
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gefind.cpp,v 1.1.1.1 1998/11/21 19:38:02 odinn Exp $
//  ------------------------------------------------------------------
//  Search functions.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//#define OLD_SEARCH
#define NEW_SEARCH


#ifdef OLD_SEARCH
#include <gbmh.h>
#else
#include "gesrch.h"
#endif


//  ------------------------------------------------------------------

#ifdef OLD_SEARCH
BmhArg* find_pattern = NULL;
BmhPat* find_patstr = NULL;
int     find_pats = 0;
#endif

//  ------------------------------------------------------------------

#ifdef OLD_SEARCH
static int searchpatterns(char* str) {

  uint len = strlen(str);
  for(int n=0; n<find_pats; n++)
    if(strbmhsearch(str, len, &find_pattern[n]))
      return true;
  return false;
}
#endif


//  ------------------------------------------------------------------

#ifdef OLD_SEARCH
void SearchExit() {

  throw_release(find_pattern);
  throw_release(find_patstr);
}
#else
void SearchExit() { }
#endif


//  ------------------------------------------------------------------

#ifdef OLD_SEARCH
bool FindString(GMsg* msg, const char* prompt, int what) {

  GFTRK("FindString");

  static ulong __FAR lastfound = 0;

  if(find_pattern == NULL) {
    find_pattern = (BmhArg*)throw_calloc(MAX_FIND, sizeof(BmhArg));
    find_patstr = (BmhPat*)throw_calloc(MAX_FIND, sizeof(BmhPat));
  }

  char* ptr;
  char buf[100], buf2[100];
  int direction = DIR_NEXT;
  int reverse = false;
  int caseignore = true;
  uint last = AA->lastread();

  int what2 = 0;
  while(*prompt and strchr("!-+<>:=", *prompt)) {
    switch(*prompt) {
      case '!':
        reverse = true;
        break;
      case '-':
        direction = DIR_PREV;
        break;
      case '+':
        direction = DIR_NEXT;
        break;
      case '<':
        what2 |= GFIND_FROM;
        break;
      case '>':
        what2 |= GFIND_TO;
        break;
      case ':':
        what2 |= GFIND_SUBJECT;
        break;
      case '=':
        caseignore = false;
        break;
    }
    prompt++;
  }

  if(what2) {
    if(what & GFIND_HDR)
      what &= ~GFIND_HDR;
    what |= what2;
  }

  find_pats = 0;
  strcpy(buf2, prompt);
  ptr = strtok(buf2, "|;&");
  while(ptr AND (find_pats<MAX_FIND)) {
    if(*ptr) {
      memset(&find_pattern[find_pats], 0, sizeof(BmhArg));
      memset(find_patstr[find_pats], 0, BmhMAXPAT);
      strcpy(find_patstr[find_pats], ptr);
      strbmhinit(find_patstr[find_pats], &find_pattern[find_pats], caseignore);
      find_pats++;
    }
    ptr = strtok(NULL, "|;&");
  }

  lastfound = AA->Msgn.CvtReln(AA->lastread());
  sprintf(buf, "%s \"%s\" ", LNG->SearchingFor, prompt);
  w_progress(MODE_NEW, C_INFOW, AA->lastread(), AA->Msgn.Count(), buf);

  ulong tmpmsgno;

  do {

    update_statuslinef(LNG->ReadingMsg, AA->lastread(), AA->Msgn.Count());
    w_progress(MODE_UPDATE, C_INFOW, AA->lastread(), AA->Msgn.Count(), buf);

    if(AA->LoadMsg(msg, AA->Msgn.CvtReln(AA->lastread()), CFG->dispmargin-(int)CFG->switches.get(disppagebar))) {

      if(what & GFIND_FROM)
        if(searchpatterns(msg->By()))
          msg->foundwhere |= GFIND_FROM;

      if(what & GFIND_TO)
        if(searchpatterns(msg->to))
          msg->foundwhere |= GFIND_TO;

      if(what & GFIND_SUBJECT)
        if(searchpatterns(msg->re))
          msg->foundwhere |= GFIND_SUBJECT;

      if(what & GFIND_BODY) {
        msg->foundtopline = -1;
        for(int l=0; l<msg->lines; l++) {
          Line* line = msg->line[l];
          if(searchpatterns(line->text)) {
            msg->foundwhere |= GFIND_BODY;
            line->type |= GLINE_HIGH;
            if(msg->foundtopline == -1)
              msg->foundtopline = l;
          }
        }
      }

      if((reverse ? NOT msg->foundwhere : msg->foundwhere) AND (lastfound != msg->msgno)) {
        bool istwitto, istwitsubj;
        if(MsgIsTwit(msg, istwitto, istwitsubj) != TWIT_SKIP) {
          HandleGEvent(EVTT_SEARCHSUCCESS);
          lastfound = msg->msgno;
          AA->set_lastread(AA->Msgn.ToReln(msg->msgno));
          w_progress(MODE_QUIT, 0, 0, 0, NULL);
          GFTRK(NULL);
          return true;
        }
      }
    }

    if(kbxhit()) {
      if(kbxget() == Key_Esc) {
        HandleGEvent(EVTT_SEARCHFAILED);
        AA->set_lastread(last);
        w_progress(MODE_QUIT, 0, 0, 0, NULL);
        GFTRK(NULL);
        return false;
      }
    }

    tmpmsgno = msg->msgno;
    switch(direction) {
      case DIR_PREV:
        if(AA->Msgn.ToReln(msg->msgno) > 1)
          msg->msgno = AA->Msgn.CvtReln(AA->Msgn.ToReln(msg->msgno)-1);
        else
          msg->msgno = 0;
        break;
      case DIR_NEXT:
        if(AA->Msgn.ToReln(msg->msgno) < AA->Msgn.Count())
          msg->msgno = AA->Msgn.CvtReln(AA->Msgn.ToReln(msg->msgno)+1);
        else
          msg->msgno = 0;
        break;
    }
    if(msg->msgno != 0)
      AA->set_lastread(AA->Msgn.ToReln(msg->msgno));

  } while((msg->msgno != 0) AND (msg->msgno != tmpmsgno));

  HandleGEvent(EVTT_SEARCHFAILED);
  AA->set_lastread(last);
  w_progress(MODE_QUIT, 0, 0, 0, NULL);
  HandleGEvent(EVTT_BREAKLOOP);

  GFTRK(NULL);
  return false;
}
#endif


//  ------------------------------------------------------------------

void FindAll(GMsg* msg, int& topline, int& keyok) {

  if(AA->Msgn.Count()) {
    topline = 0;
    AA->attr().hex0();
    strcpy(CFG->searchfor, AA->Searchfor());
    if(edit_string(CFG->searchfor, sizeof(INam), LNG->HeaderText, H_FindString)) {
      AA->SetSearchfor(CFG->searchfor);
      if(FindString(msg, AA->Searchfor(), GFIND_HDRTXT))
        keyok = true;
    }
  }
}


//  ------------------------------------------------------------------

void FindHdr(GMsg* msg, int& topline, int& keyok) {

  if(AA->Msgn.Count()) {
    topline = 0;
    AA->attr().hex0();
    strcpy(CFG->searchfor, AA->Searchfor());
    if(edit_string(CFG->searchfor, sizeof(INam), LNG->HeaderOnly, H_FindString)) {
      AA->SetSearchfor(CFG->searchfor);
      if(FindString(msg, AA->Searchfor(), GFIND_HDR))
        keyok = true;
    }
  }
}


//  ------------------------------------------------------------------

#ifdef OLD_SEARCH
bool SearchHighlight(const Line* line, int row, int /*width*/, int highlight_color) {

  bool found = false;

  // Handle highlighting
  if(line->type & GLINE_HIGH) {
    int len = strlen(line->text);
    for(int n=0; n<find_pats; n++) {
      char *patstr = find_patstr[n];
      int patlen = strlen(patstr);
      if(len >= patlen) {
        int endpos = len - patlen;
        const char *ptr = line->text;
        for(int x=0; x<=endpos; x++,ptr++) {
          if(strnieql(patstr, ptr, patlen)) {
            BmhPat highstr;
            strxcpy(highstr, ptr, patlen+1);
            found = true;
            vputs(row, (uint)(ptr-line->text), highlight_color, highstr);
          }
        }
      }
    }
  }

  return found;
}
#else
bool SearchHighlight(const Line*, int, int, int) { return false; }
#endif


//  ------------------------------------------------------------------

#ifdef NEW_SEARCH
bool FindString(GMsg* msg, const char* prompt, int what) {

  GFTRK("FindString");

  static ulong __FAR lastfound = 0;

  golded_search_manager srchmgr;
  srchmgr.prepare_from_string(prompt, what);

  uint last = AA->lastread();

  #define LNG__AdvancedSearch " Advanced Search "

  lastfound = AA->Msgn.CvtReln(AA->lastread());
  w_progress(MODE_NEW, C_INFOW, AA->lastread(), AA->Msgn.Count(), LNG__AdvancedSearch);

  ulong tmpmsgno;

  do {

    update_statuslinef(LNG->ReadingMsg, AA->lastread(), AA->Msgn.Count());
    w_progress(MODE_UPDATE, C_INFOW, AA->lastread(), AA->Msgn.Count(), LNG__AdvancedSearch);

    if(AA->LoadMsg(msg, AA->Msgn.CvtReln(AA->lastread()), CFG->dispmargin-(int)CFG->switches.get(disppagebar))) {

      bool success;
      // If hit, search again current mail without shortcircuit evaluation
      success = srchmgr.search(msg, false, true);
      if(success)
        srchmgr.search(msg, false, false);

      if((srchmgr.reverse ? not success : success) and (lastfound != msg->msgno)) {
        bool istwitto, istwitsubj;
        if(MsgIsTwit(msg, istwitto, istwitsubj) != TWIT_SKIP) {
          HandleGEvent(EVTT_SEARCHSUCCESS);
          lastfound = msg->msgno;
          AA->set_lastread(AA->Msgn.ToReln(msg->msgno));
          w_progress(MODE_QUIT, 0, 0, 0, NULL);
          GFTRK(NULL);
          return true;
        }
      }
    }

    if(kbxhit()) {
      if(kbxget() == Key_Esc) {
        HandleGEvent(EVTT_SEARCHFAILED);
        AA->set_lastread(last);
        w_progress(MODE_QUIT, 0, 0, 0, NULL);
        GFTRK(NULL);
        return false;
      }
    }

    tmpmsgno = msg->msgno;
    switch(srchmgr.direction) {
      case DIR_PREV:
        if(AA->Msgn.ToReln(msg->msgno) > 1)
          msg->msgno = AA->Msgn.CvtReln(AA->Msgn.ToReln(msg->msgno)-1);
        else
          msg->msgno = 0;
        break;
      case DIR_NEXT:
        if(AA->Msgn.ToReln(msg->msgno) < AA->Msgn.Count())
          msg->msgno = AA->Msgn.CvtReln(AA->Msgn.ToReln(msg->msgno)+1);
        else
          msg->msgno = 0;
        break;
    }
    if(msg->msgno != 0)
      AA->set_lastread(AA->Msgn.ToReln(msg->msgno));

  } while((msg->msgno != 0) AND (msg->msgno != tmpmsgno));

  HandleGEvent(EVTT_SEARCHFAILED);
  AA->set_lastread(last);
  w_progress(MODE_QUIT, 0, 0, 0, NULL);
  HandleGEvent(EVTT_BREAKLOOP);

  #define LNG__NoMoreMatches " No more matches found "
  w_info(LNG__NoMoreMatches);
  waitkeyt(5000);
  w_info(NULL);

  GFTRK(NULL);
  return false;
}
#endif


//  ------------------------------------------------------------------

