
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gehdre.cpp,v 1.2 1999/01/01 17:59:45 damware Exp $
//  ------------------------------------------------------------------
//  Header edit.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <gwinput.h>


//  ------------------------------------------------------------------

class GMsgHeaderEdit : public gwinput2 {

public:

  enum {
    id_from_name,
    id_from_addr,
    id_to_name,
    id_to_addr,
    id_subject
  };

  GMsgHeaderView &view;
  //gwindow &window;

  GMsgHeaderEdit(GMsgHeaderView &v) : gwinput2(v.window), view(v) { msg = view.msg; }

  GMsg* msg;
  bool lookup;

  bool handle_other_keys(uint& key);
  bool validate();
};


//  ------------------------------------------------------------------

bool GMsgHeaderEdit::handle_other_keys(uint& key) {

  switch(key) {
    case Key_F10:
      vcurhide();
      ChgAttrs(NO, msg);
      lookup_addressbook(msg, get_field(id_to_name)->buf, get_field(id_to_addr)->buf, true);
      ChgAttrs(YES, msg);
      get_field(id_to_name)->draw();
      get_field(id_to_addr)->draw();
      current->update();
      vcurshow();
      break;
    case Key_S_F10:
      {
        Addr addr,addr2;
        INam buf;
        field* p;

        addr.set_all(65535u);
        addr2.set_all(65535u);
        addr.set(get_field(id_to_addr)->buf);
        strcpy(buf, strtrim(get_field(id_to_name)->buf));

        if(addr != addr2)
          addr.make_string(buf);

        addr2 = addr;

        Lookup(msg, &addr, buf, -6, LNG->SelectDestNode);
        vcurshow();

        if(addr != addr2 and addr.valid()) {
          p = get_field(id_to_addr);

          addr.make_string(p->buf);
          p->buf_end_pos = strlen(buf);
          p->buf_pos = p->buf_end_pos;

          if(current->id == id_to_addr)
            p->update();
          else
            p->draw();

          p = get_field(id_to_name);
          strcpy(p->buf, buf);

          if(current->id == id_to_name)
            p->update();
          else
            p->draw();
        }
      }
      return true;
  }
  return false;
}


//  ------------------------------------------------------------------

bool GMsgHeaderEdit::validate() {

  if(current->id == id_to_name) {

    field* from_addr = get_field(id_from_addr);
    field* to_addr = get_field(id_to_addr);
    field* subject = get_field(id_subject);

    INam buf, buf1, buf2;
    char* name = current->buf;
    char* ptr = name;
    strcpy(buf, ptr);

    if(CFG->addressmacros) {
      for(int n=0; n<CFG->addressmacros; n++) {
        if(strieql(buf, CFG->addressmacro[n].macro)) {
          ptr = strcpy(buf1, CFG->addressmacro[n].name);
          if(*buf1 == '@') {
            // UUCP/INTERNET addressing
            ptr = strchr(buf1, '/');
            if(ptr) {
              *ptr++ = NUL;
              strcpy(msg->idest, ptr);
              strcpy(msg->realto, buf1+1);
              if(*AA->Internetgate().name) {
                strcpy(msg->iaddr, ptr);
                ptr = strcpy(buf1, AA->Internetgate().name);
              }
              else
                strcpy(buf1, ptr);
            }
            else {
              strcpy(msg->idest, buf1+1);
              if(*AA->Internetgate().name) {
                strcpy(msg->iaddr, buf1+1);
                ptr = strcpy(buf1, AA->Internetgate().name);
              }
              else {
                strcpy(buf2, buf1+1);
                ptr = strcpy(buf1, buf2);
              }
            }
          }
          else if(AA->isinternet()) {
            ParseInternetAddr(buf1, buf2, buf);
            ptr = buf2;
            strcpy(buf1, buf2);
            strcpy(msg->idest, buf);
          }
          strcpy(name, ptr);
          current->update();
          strcpy(msg->to, buf1);
          if(AA->isinternet()) {
            strcpy(to_addr->buf, msg->idest);
            if(CFG->addressmacro[n].addr.net)
              msg->dest = msg->odest = CFG->addressmacro[n].addr;
            else
              msg->dest = msg->odest = AA->Internetgate().addr;
          }
          else {
            CFG->addressmacro[n].addr.make_string(to_addr->buf);
          }
          to_addr->update();
          if(*CFG->addressmacro[n].subj) {
            strcpy(subject->buf, CFG->addressmacro[n].subj);
            subject->update();
          }

          AttrAdd(&msg->attr, &CFG->addressmacro[n].attr);
          char bot2[200];
          MakeAttrStr(bot2, &msg->attr);
          strsetsz(bot2, EDIT->HdrNodeLen());
          window.prints(1, EDIT->HdrNodePos(), C_HEADW, bot2);

          if(lookup or AA->isnet())
            go_next_field();

          return true;
        }
      }
    }

    if(lookup_addressbook(msg, current->buf, to_addr->buf, false)) {
      if(lookup or AA->isnet())
        go_next_field();
      return true;
    }

    // Internet gating
    if(not AA->isinternet()) {
      if(strchr(buf, '@') AND AA->Internetgate().addr.net) {
        if(*AA->Internetgate().name) {
          strcpy(msg->iaddr, buf);
          strcpy(msg->to, AA->Internetgate().name);
          strcpy(name, msg->to);
          current->update();
        }
        AA->Internetgate().addr.make_string(to_addr->buf);
        to_addr->update();
        if(lookup or AA->isnet())
          go_next_field();
        return true;
      }

      if(lookup) {
        strcpy(buf1, name);
        strcpy(buf2, msg->to);
        if(strblank(to_addr->buf) OR strblank(name) OR strcmp(buf1, buf2)) {
          if(strblank(name))
            strcpy(buf, to_addr->buf);
          else
            strcpy(buf, name);
          strtrim(buf);
          if(!*buf)
            strcpy(buf, " ");

          if(*buf) {
            Addr addr;
            addr.reset(from_addr->buf);   // Get the orig address to base lookup on
            Lookup(msg, &addr, buf, 0, LNG->SelectDestNode);
            vcurshow();
            if(striinc("Sysop", buf) OR strieql(AA->Whoto(), buf)) {
              if(addr.net == 0) {
                if(NOT AA->isnet())
                  go_next_field();
                return true;
              }
            }
            int robotchk = FALSE;
            for(int n=0; n<CFG->robotnames; n++) {
              if(striinc(CFG->robotname[n], buf)) {
                robotchk = TRUE;
                break;
              }
            }
            if(robotchk) {
              EDIT->HardLines(FALSE);
              if(addr.net == 0)
                return true;
            }
            if(addr.net) {
              addr.make_string(to_addr->buf);
              to_addr->update();
              strcpy(msg->to, buf);
              strcpy(name, buf);
              current->update();
              go_next_field();
            }
            else {
              HandleGEvent(EVTT_SEARCHFAILED);
            }
          }
        }
      }
      else {
        if(strpbrk(buf, "*?")) {
          if(CFG->switches.get(lookupuserbase)) {
            if(AA->UserLookup(buf)) {
              strcpy(msg->to, buf);
              current->update();
              return true;
            }
          }
        }
      }
    }
  }
  else if(current->id == id_subject) {
    CheckSubject(msg, current->buf);
    current->update();
  }

  return true;
}


//  ------------------------------------------------------------------

int EditHeaderinfo(int mode, GMsgHeaderView &view) {

  GMsgHeaderEdit hedit(view);
  GMsg *msg = view.msg;

  gstring from_name, to_name, subject;
  gstring from_addr, to_addr, orig_from_addr;

  from_name = msg->By();
  if(AA->isinternet()) {
    from_addr = msg->iorig;
  }
  else {
    if(msg->orig.net)
      msg->orig.make_string(from_addr, msg->odom);
  }
  orig_from_addr = from_addr;
  if(AA->isinternet()) {
    to_name = msg->To();
    to_addr = msg->idest;
  }
  else {
    to_name = (*msg->iaddr) ? msg->iaddr : msg->to;
    if(msg->dest.net)
      msg->dest.make_string(to_addr, msg->ddom);
  }
  subject = msg->re;

  if(AA->isnet())
    hedit.lookup = CFG->switches.get(lookupnet);
  else if(AA->isecho())
    hedit.lookup = CFG->switches.get(lookupecho);
  else
    hedit.lookup = CFG->switches.get(lookuplocal);

  int from_name_pos = EDIT->HdrNamePos();
  int from_name_len = EDIT->HdrNameLen();
  int from_addr_pos = EDIT->HdrNodePos();
  int from_addr_len = EDIT->HdrNodeLen();
  int to_name_pos = from_name_pos;
  int to_name_len = from_name_len;
  int to_addr_pos = from_addr_pos;
  int to_addr_len = hedit.lookup or AA->isnet() ? from_addr_len : 0;
  int subject_pos = 8;
  int subject_len = gvid.numcols - subject_pos;
  int name_cvt = AA->Editmixcase() ? GMsgHeaderEdit::cvt_mixedcase : GMsgHeaderEdit::cvt_none;
  int addr_cvt = GMsgHeaderEdit::cvt_none;
  int subj_cvt = GMsgHeaderEdit::cvt_none;

  hedit.setup(C_HEADW, C_HEADW, C_HEADE, _box_table[W_BHEAD][13], true);

  hedit.add_field(GMsgHeaderEdit::id_from_name, 2, from_name_pos, from_name_len, from_name, sizeof(INam), name_cvt);
  hedit.add_field(GMsgHeaderEdit::id_from_addr, 2, from_addr_pos, from_addr_len, from_addr, sizeof(IAdr), addr_cvt);
  hedit.add_field(GMsgHeaderEdit::id_to_name,   3,   to_name_pos,   to_name_len,   to_name, sizeof(INam), name_cvt);
  hedit.add_field(GMsgHeaderEdit::id_to_addr,   3,   to_addr_pos,   to_addr_len,   to_addr, sizeof(IAdr), addr_cvt);
  hedit.add_field(GMsgHeaderEdit::id_subject,   4,   subject_pos,   subject_len,   subject, sizeof(ISub), subj_cvt);

  hedit.start_id = GMsgHeaderEdit::id_to_name;
  switch(mode) {
    case MODE_REPLYCOMMENT:
      if(NOT (msg->dest.net OR NOT AA->isnet()))
        break;
      // else drop through ...
    case MODE_REPLY:
    case MODE_QUOTE:
    case MODE_CHANGE:
      hedit.start_id = GMsgHeaderEdit::id_subject;
      break;
  }

  int help = gwin.help;

  ChgAttrs(TRUE, msg);

  whelpdef(CFG->helpged, Key_F1, C_HELPB, C_HELPW, C_HELPQ, C_HELPS, NULL);
  vcurshow();
  if(not (hedit.lookup or AA->isnet())) {
    char date2[25] = {""};
    strsetsz(date2, view.width - CFG->disphdrdateset.pos);
    view.window.prints(3, CFG->disphdrdateset.pos, view.to_color, date2);
  }

  #define LNG__HeaderEditHelp1 " #F10: Nodelist "
  #define LNG__HeaderEditHelp2 " F10: Addressbook "

  view.window.prints(5, view.width-strlen(LNG__HeaderEditHelp1)-1, view.title_color, LNG__HeaderEditHelp1);
  view.window.prints(5, view.width-strlen(LNG__HeaderEditHelp1)-strlen(LNG__HeaderEditHelp2)-3, view.title_color, LNG__HeaderEditHelp2);

  hedit.run(H_Header);
  vcurhide();

  ChgAttrs(FALSE, msg);

  whelpcat(help);

  if(NOT hedit.dropped AND NOT gkbd.quitall) {

    strcpy(msg->by, from_name.c_str());
    strcpy(msg->re, subject.c_str());

    INam tmp_to_name;
    strxcpy(tmp_to_name, to_name.c_str(), sizeof(INam));
    lookup_addressbook(msg, tmp_to_name, NULL);
    to_name = tmp_to_name;

    if(AA->isinternet()) {
      strcpy(msg->to, to_name.c_str());
      strcpy(msg->realby, msg->by);
      strcpy(msg->realto, msg->to);
      strcpy(msg->iorig, from_addr.c_str());
      strcpy(msg->idest, to_addr.c_str());
      strcpy(msg->ifrom, msg->iorig);
      strcpy(msg->ito, msg->idest);
      if(msg->orig.net == 0)
        msg->orig = msg->oorig = AA->Aka().addr;
      if(msg->dest.net == 0)
        msg->dest = msg->odest = AA->Internetgate().addr;
    }
    else {
      if(strchr(to_name.c_str(), '@')) {
        if(*AA->Internetgate().name)
          strcpy(msg->iaddr, to_name.c_str());
        else {
          if(strlen(to_name.c_str()) > 34) {
            strcpy(msg->to, "UUCP");
            strcpy(msg->iaddr, to_name.c_str());
          }
          else
            strcpy(msg->to, to_name.c_str());
        }
      }
      else
        strcpy(msg->to, to_name.c_str());

      Addr address;
      address = AA->Aka().addr;

      if(from_addr.length()) {
        address.set(from_addr.c_str(), msg->odom);
        msg->orig.zone  = address.zone;
        msg->orig.net   = address.net;
        msg->orig.node  = address.node;
        msg->orig.point = address.point;
      }

      if(to_addr.empty()) {
        // Set destination to yourself or the Boss node
        address = msg->orig;
        address.point = 0;
        address.make_string(to_addr);
      }

      address = AA->Aka().addr;
      const char* domain = address.set(to_addr);
      msg->odest.zone  = msg->dest.zone  = address.zone;
      msg->odest.net   = msg->dest.net   = address.net;
      msg->odest.node  = msg->dest.node  = address.node;
      msg->odest.point = msg->dest.point = address.point;

      if(*domain) {
        address.reset();
        address.zone  = msg->orig.zone;
        address.set(domain);
        if(address.net) {
          msg->odest.zone  = address.zone;
          msg->odest.net   = address.net;
          msg->odest.node  = address.node;
          msg->odest.point = address.point;
        }
        else {
          msg->odest.zone  = msg->dest.zone;
          msg->odest.net   = msg->dest.net;
          msg->odest.node  = msg->dest.node;
          msg->odest.point = msg->dest.point;
        }
      }

      // Try to match akas with the dest address, but only if the orig address was NOT changed
      uint n;
      for(n=0; n<CFG->akas; n++) {
        if(memcmp(&msg->orig, &CFG->aka[n], sizeof(Addr)) == 0)
          break;  // Found one of our own akas.
      }

      // Try to match akas with the dest address, but only if the orig address was NOT changed
      if((n < CFG->akas)) {
        // ... and only if the orig aka was already one of our own
        if(orig_from_addr.ieql(from_addr)) {
          // ... and only if we did NOT change aka manually
          if(AA->Aka().addr.equals(AA->aka())) {

            // Do aka matching
            if(AA->Akamatching()) {
              strcpy(msg->odom, CFG->aka[AkaMatch(&msg->orig, &msg->dest)].domain);
            }
          }
        }
      }
    }
    if(EDIT->SoftCrXlat()) {
      strchg(msg->by, SOFTCR, EDIT->SoftCrXlat());
      strchg(msg->to, SOFTCR, EDIT->SoftCrXlat());
      strchg(msg->realby, SOFTCR, EDIT->SoftCrXlat());
      strchg(msg->realto, SOFTCR, EDIT->SoftCrXlat());
      strchg(msg->re, SOFTCR, EDIT->SoftCrXlat());
    }
  }
  else {
    for(int n=0; n<fspecs; n++)
      throw_release(fspec[n].fblk);
    fspecs = specfiles = 0;
  }

  view.Paint();

  return hedit.dropped ? W_ESCPRESS : W_NOERROR;
}


//  ------------------------------------------------------------------

