
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geinit.cpp,v 1.6 1999/02/05 22:58:59 damware Exp $
//  ------------------------------------------------------------------
//  System initializing.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#if defined(__WIN32__)
#include <windows.h>
#endif

#include <gmoprot.h>


//  ------------------------------------------------------------------
//  Handle commandline parameters

#if defined(GFTRK_ENABLE)
static int __FAR gftrk_set_max = -1;
extern int __gftrk_statusline;
#endif

#if defined(__OS2__)
extern bool gmem_check_overrun;
#endif

static char __FAR keybuf[80];

       Path __FAR cmdlinecfg        = {""};
       int  __FAR cmdlinedebughg    = false;
static int  __FAR cmdlineforce      = false;
static int  __FAR cmdlinehelp       = false;
       bool __FAR cmdlinenoscan     = false;
       bool __FAR cmdlineexportsoup = false;
       int  __FAR cmdlineimportsoup = false;
static int  __FAR cmdlineinstall    = false;
static Path __FAR cmdlineinstpath   = {""};
       int  __FAR cmdlineoldkeyw    = true;
       bool __FAR cmdlinepriority   = false;
       int  __FAR cmdlinesharemode  = SH_DENYNO;
       int  __FAR cmdlinetimeout    = -1;
static int  __FAR cmdlinewriteareas = false;

static void InitCmdline(char* val) {

  char* key;
  char* trueval;
  
  if(strchr("-/", *val)) {
    val++;
    getkeyvaleql(&key, &val, TRUE);
    trueval = val;
    if(*val == NUL)
      val = key+1;
    switch(toupper(*key)) {
      case '2':
        //__gforceos2slice = *val != '-';
        break;
      case 'C':                   // Use another Configfile
        strcpy(cmdlinecfg, val);
        break;
      case 'D':
        cmdlineoldkeyw = *val == '-';
        break;
      case 'E':
        if(strieql(key, "EXPORTSOUP"))
          cmdlineexportsoup = true;
        else
          strxcpy(stecho, val, sizeof(stecho));
        break;
      case 'F':
        cmdlineforce = *val != '-';
        if(toupper(key[1]) == 'F')
          cmdlineforce++;
        break;
      case '?':
      case 'H':
        cmdlinehelp = TRUE;
        break;
      case 'I':
        if(strieql(key, "INSTALL")) {
          cmdlineinstall = YES;
          cmdlineforce = 1;
          if(*trueval)
            PathCopy(cmdlineinstpath, trueval);
        }
        else if(strieql(key, "IMPORTSOUP"))
          cmdlineimportsoup = true;
        break;
      case 'M':
        disablesound = *val != '-';
        break;
      case 'N':
        if(strieql(key, "NOSCAN"))
          cmdlinenoscan = true;
        else
          cmdlinesharemode = 0;
        break;
      case 'P':
        cmdlinepriority = *val != '-';
        break;
      case 'Q':
        quiet = YES;
        break;
      case 'S':
        strcpy(AL.sortspec, val);
        break;
      case 'T':
        cmdlinetimeout = atoi(val);
        break;
      case 'V':
        quiet = NO;
        veryverbose = (toupper(key[1]) == 'V' ? YES : NO);
        break;
      case 'W':
        cmdlinewriteareas = *val != '-';
        break;
      case 'Y':
        cmdlinedebughg = *val != '-';
        break;
      #if defined(__OS2__)
      case 'X':
        gmem_check_overrun = *val != '-';
        break;
      #else
      #if defined(GFTRK_ENABLE)
      case 'X':
        __gftrk_statusline = *val != '-';
        break;
      case 'Z':
        gftrk_set_max = atoi(val);
        break;
      #endif
      #endif
    }
  }
  else {
    strcat(keybuf, val);
    strcat(keybuf, " ");
  }
}


//  ------------------------------------------------------------------

static void ReadEcholists() {

  Echo* echoin = (Echo*)throw_calloc(1, sizeof(Echo));

  // Read the import taglist
  FILE* fp = fsopen(AddPath(CFG->areapath, CFG->semaphore.importlist), "rt", CFG->sharemode);

  if(fp) {
    char buf[256];
    int echonums = 0;
    update_statusline(LNG->ReadingEcholist);
    while(fgets(buf, sizeof(buf), fp)) {
      if(NOT strblank(buf)) {
        echonums++;
        echoin = (Echo*)throw_realloc(echoin, (echonums+2)*sizeof(Echo));
        strcpy(echoin[echonums-1], strtrim(strsetsz(buf, sizeof(Echo)-1)));
      }
    }
    *echoin[echonums] = 0;   // Mark end
    fclose(fp);
  }

  // Mark the areas from the import taglist
  for(uint n=0; n<AL.total; n++) {
    char buf[256];
    strcpy(buf, AL[n]->echoid());
    int x = SearchTaglist(echoin, buf);
    if(*echoin[x])
      AL[n]->set_marked(true);
  }

  throw_free(echoin);
}


//  ------------------------------------------------------------------

static void ReadEscsets() {

  FILE* fp;

  for(int n=0; n<CFG->xlatescsets; n++) {
    if(strieql(CFG->xlatescset[n].import, "Composed")) {
      fp = fsopen(AddPath(CFG->goldpath, CFG->xlatged), "rb", CFG->sharemode);
      if(fp) {
        CompTable = (Esc*)throw_realloc(CompTable, sizeof(Esc));
        fseek(fp, ((long)CFG->xlatcharsets*(long)sizeof(Chs)) + ((long)n*(long)sizeof(Esc)), SEEK_SET);
        fread(CompTable, sizeof(Esc), 1, fp);
        CompTP = CompTable->t;
        fclose(fp);
      }
    }
    else if(strieql(CFG->xlatescset[n].import, "I51")) {
      fp = fsopen(AddPath(CFG->goldpath, CFG->xlatged), "rb", CFG->sharemode);
      if(fp) {
        I51Table = (Esc*)throw_realloc(I51Table, sizeof(Esc));
        fseek(fp, ((long)CFG->xlatcharsets*(long)sizeof(Chs)) + ((long)n*(long)sizeof(Esc)), SEEK_SET);
        fread(I51Table, sizeof(Esc), 1, fp);
        I51TP = I51Table->t;
        fclose(fp);
      }
    }
    else if(strieql(CFG->xlatescset[n].import, "MNEMONIC")) {
      fp = fsopen(AddPath(CFG->goldpath, CFG->xlatged), "rb", CFG->sharemode);
      if(fp) {
        MNETable = (Esc*)throw_realloc(MNETable, sizeof(Esc));
        fseek(fp, ((long)CFG->xlatcharsets*(long)sizeof(Chs)) + ((long)n*(long)sizeof(Esc)), SEEK_SET);
        fread(MNETable, sizeof(Esc), 1, fp);
        MNETP = MNETable->t;
        fclose(fp);
      }
    }
  }
}


//  ------------------------------------------------------------------

static void ReadAddrMacros() {

  FILE* fp;
  char* ptr;
  char buf[256], path[GMAXPATH];

  ptr = getenv("FD");
  if(ptr)
    AddBackslash(strcpy(path, ptr));
  else
    strcpy(path, CFG->goldpath);

  MakePathname(CFG->namesfile, path, CFG->namesfile);

  if(fexist(CFG->namesfile)) {
    fp = fsopen(CFG->namesfile, "rt", CFG->sharemode);
    if(fp) {
      update_statusline(LNG->ReadingAddrMacros);
      while(fgets(buf, sizeof(buf), fp)) {
        strbtrim(buf);
        if(*buf != ';' AND *buf)
          CfgAddressmacro(buf);
      }
      fclose(fp);
    }
  }
}


//  ------------------------------------------------------------------

static void kbputc_(char c) {

  kbput((Key)((scancode(c) << 8) | c));
}


//  ------------------------------------------------------------------

static void kbputstr(char* buf) {

  Key xkey;
  register int n,x;

  x = strlen(buf);

  for(n=0; n<x; n++) {
    switch(buf[n]) {
      case HT:
      case ' ':
        break;
      case '!':
        clearkeys();
        kbclear();
        break;
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
        kbput((Key)atoi(&buf[n]));
        for(; n<x; n++) {
          if(buf[n] == ' ')
            break;
        }
        break;
      case '~':
      case '^':
        xkey = (Key)(toupper(buf[++n]) - '@');
        if(xkey)
          kbput(xkey);
        break;
      case '@':
        xkey = (Key)atoi(&buf[++n]);
        if(xkey == 0)
          xkey = scancode(buf[n]);
        kbput((Key)((xkey << 8) & 0xFF00));
        for(; n<x; n++) {
          if(buf[n] == ' ')
            break;
        }
        break;
      case '\"':
        for(n+=1; n<x; n++) {
          if(buf[n] == '\"')
            break;
          kbputc_(buf[n]);
        }
        break;
      case '\'':
        for(n+=1; n<x; n++) {
          if(buf[n] == '\'')
            break;
          kbputc_(buf[n]);
        }
        break;
      default:
        kbputc_(buf[n]);
    }
  }
}


//  ------------------------------------------------------------------

static void w_back() {

  char fillch = _box_table[W_BBRAG][14];

  W_STAT = wopen(MAXROW-1,0,MAXROW-1,MAXCOL-1,5,C_STATW,C_STATW);
  update_statusline("");
  wfillch(fillch);
  W_BACK = wopen(0,0,MAXROW-2,MAXCOL-1,5,C_BACKB,C_BACKW|ACSET);
  wfillch(' ');
}


//  ------------------------------------------------------------------

static void w_brag() {

  char buf[200];
  char* logo[6];

  if(W_BBRAG == 7) {
    logo[0] = strdup("                    **     **            **  ");
    logo[1] = strdup("      ****** ****** ** ****** ****** ******  ");
    logo[2] = strdup("      **  ** **  ** ** **  ** ****** **  **  ");
    logo[3] = strdup("      ****** ****** ** ****** ****** ******  ");
    logo[4] = strdup("  **      **                                 ");
    logo[5] = strdup("  **********                                 ");
  }
  else {
    #if defined(__UNIX__)
    if(gvid_xterm) {
      logo[0] = strdup("                    ڿ     ڿ            ڿ  ");
      logo[1] = strdup("      ¿ ¿  Ĵ ¿ Ĵ  ");
      logo[2] = strdup("                     ");
      logo[3] = strdup("      Ĵ       ");
      logo[4] = strdup("  ڿ                                       ");
      logo[5] = strdup("                                   ");
    }
    else {
    #endif
      logo[0] = strdup("                    ɻ     ɻ            ɻ  ");
      logo[1] = strdup("      ˻ ˻  ͹ ˻ ͹  ");
      logo[2] = strdup("                ʼ     ");
      logo[3] = strdup("      ͹ ʼ ȼ ʼ ʼ ʼ  ");
      logo[4] = strdup("  ɻ                                       ");
      logo[5] = strdup("  ʼ                                 ");
    #if defined(__UNIX__)
    }
    gvid_boxcvt(logo[0]);
    gvid_boxcvt(logo[1]);
    gvid_boxcvt(logo[2]);
    gvid_boxcvt(logo[3]);
    gvid_boxcvt(logo[4]);
    gvid_boxcvt(logo[5]);
    #endif
  }

  W_READ = wopen_(1, 2, MAXROW-4, MAXCOL-5, W_BBRAG, C_BRAGB, C_BRAGW);
  w_shadow();

  wprints(0, 0, C_BRAGT|ACSET, logo[0]);
  wprints(1, 0, C_BRAGT|ACSET, logo[1]);
  wprints(2, 0, C_BRAGT|ACSET, logo[2]);
  wprints(3, 0, C_BRAGT|ACSET, logo[3]);
  wprints(4, 0, C_BRAGT|ACSET, logo[4]);
  wprints(5, 0, C_BRAGT|ACSET, logo[5]);

  for(int n=0; n<6; n++)
    free(logo[n]);

  if((__gver_minor__ & 1))
    wprints(5, 17, C_BRAGW, "THIS IS A \"HACKER\" RELEASE");

  wprints(4, 43-strlen(__gver_longpid__), C_BRAGW, __gver_longpid__);

  wprints(0, 46, C_BRAGW, "   The GoldED Msg Editor   ");
  wprints(1, 46, C_BRAGW, "     By Odinn Sorensen     ");
  wprints(2, 46, C_BRAGW, "      Dirk A. Mueller      ");
  wprints(3, 46, C_BRAGW, "        (core team)        ");
  sprintf(buf, "  Copyright (C) 1990-%s  ",__gver_date__+7);
  wprints(5, 46, C_BRAGW, buf);

  whline(6, 0, MAXCOL-5, W_BBRAG, C_BRAGB);
  wvline(0, 45, 7, W_BBRAG, C_BRAGB);

  sprintf(buf, "---*-*-*** %s ***-*-*---", __gver_releasename__);
  wcenters(8, C_BRAGW, buf);

  wcenters(10, C_BRAGW, "This program is free software; it is licensed under the");
  wcenters(11, C_BRAGW, "GNU General Public License version 2. You're welcome to");
  wcenters(12, C_BRAGW, "redistribute the program or any parts hereof under cer-");
  wcenters(13, C_BRAGW, "tain conditions. See the LICENSE.TXT for more details.");

  wcenters(MAXROW-10, C_BRAGW, "This executable is distributed by");
  sprintf(buf, "%s (Fido: %s) - <%s>", __gver_vendor_name__,
                                       __gver_vendor_fido__,
                                       __gver_vendor_email__);
  wcenters(MAXROW-9, C_BRAGW, buf);
  sprintf(buf, "Compiled on %s %s", __gver_date__, __gver_time__);
  wcenters(MAXROW-8, C_BRAGW, buf);

  update_statusline(LNG->Initializing);
}


//  ------------------------------------------------------------------

int ExistCfg(char* path, char* file) {

  int found = fexist(AddPath(path, file));
  if(found)
    strcat(path, file);
  return found;
}


//  ------------------------------------------------------------------

#if defined(__WIN32__)
BOOL WINAPI GoldedCtrlHandler(DWORD dwCtrlType) {

  switch(dwCtrlType) {
    case CTRL_BREAK_EVENT:
      return TRUE;
    case CTRL_CLOSE_EVENT:
    case CTRL_LOGOFF_EVENT:
    case CTRL_SHUTDOWN_EVENT:
      break;
    default:
      return FALSE;
  }

  Cleanup();
  ExitProcess(errorlevel);
  return TRUE;
}
#endif


//  ------------------------------------------------------------------
//  Initialize defaults and generally get the system up and running

void Initialize(int argc, char* argv[]) {

  char* ptr;
  char* ptr2;
  int found=NO, dbedit=NO, compiled;
  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT], buf[256];

  // First inits
  #ifndef __GNUC__
  _fmode = O_BINARY;
  #endif
  ggetosversion();
  *keybuf = NUL;
  #if defined(__BORLANDC__)
  _timezone = 0;
  _daylight = 0;
  tzset();
  #endif
  goldmark = ' ';
  *information = NUL;
  srand((unsigned)time(NULL));
  #if defined(__WATCOMC__) OR defined(__UNIX__) OR defined(__DJGPP__)
  setvbuf(stdout, NULL, _IONBF, 0);
  #endif

  // Check environment commandline
  ptr = getenv("GEDCMD");
  if(ptr AND *ptr) {
    ptr = strcpy(buf, ptr);
    while(*ptr) {
      fastSkipWht(ptr);
      ptr2 = ptr;
      fastSkipTxt(ptr);
      if(*ptr)
        *ptr++ = NUL;
      InitCmdline(ptr2);
    }
  }

  // Check if we have been renamed to replace the D'Bridge Editor
  if(striinc("DBEDIT.EXE", argv[0])) {
    for(int a=1; a<argc; a++) {
      if(striinc("INTERNAL", argv[a])) {
        dbedit = YES;   // Do not accept any commandline arguments
        break;
      }
    }
  }

  // Check the commandline arguments
  if(argc > 1 AND NOT dbedit) {
    for(int a=1; a<argc; a++) {
      ptr = strcpy(buf, argv[a]);
      InitCmdline(ptr);
    }
  }

  #if defined(__WIN32__)
  if(cmdlinepriority)
    SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
  #endif

  // Set global product identifier strings
  sprintf(__gver_longpid__, "%s%s%s%s %s%i.%i.%i%s",
    __gver_prename__,
    __gver_name__,
    __gver_postname__,
    __gver_platform__,
    __gver_preversion__,
    __gver_major__,
    __gver_minor__,
    __gver_release__,
    __gver_postversion__
  );
  sprintf(__gver_shortpid__, "%s%s%s%s %s%i.%i.%i%s",
    __gver_preshortname__,
    __gver_shortname__,
    __gver_postshortname__,
    __gver_shortplatform__,
    __gver_preversion__,
    __gver_major__,
    __gver_minor__,
    __gver_release__,
    __gver_postversion__
  );

  // Print commandline help and exit if requested
  if(cmdlinehelp) {

    printf(                                     // 123456789012345678901234567890123456
       // 34567890123456789012345678901234567890123456789012345678901234567890123456789
       "%-41.41s"                                 "Copyright (C) 1990-%s Odinn Sorensen" NL
       "-------------------------------------------------------------------------------" NL
       "" NL
       #if defined(__OS2__)
       "Commandline syntax: GED2"
       #elif defined(__WIN32__)
       "Commandline syntax: GEDW32"
       #elif defined(__MSDOS__) and defined(__386__)
       "Commandline syntax: GED386"
       #elif defined(__MSDOS__)
       "Commandline syntax: GOLDED"
       #elif defined(__linux__)
       "Commandline syntax: gedlnx"
       #endif
       " [-options] [keystacking]" NL
       "" NL
       "-C<configfile>  Use a different configuration file." NL
       "-D              Disable old obsolete configuration keywords." NL
       "-E<echoid>      Start directly in the specified mail area." NL
       "-EXPORTSOUP     Export SOUP packets during startup." NL
       "-F  or  -FF     Force recompile of most (or all with -FF) configuration files." NL
       "-INSTALL[=path] Start the quick install procedure. Look in path, if given." NL
       "-IMPORTSOUP     Import SOUP packets during startup." NL
       "-M              Mute sounds. Disables all noises in GoldED." NL
       "-N              Disable share-compatible file opens during startup." NL
       "-NOSCAN         Temporarily disable area scan during startup." NL
       #if defined(__BORLANDC__) and defined(__MSDOS__)
       "-O<ovlsize>     Set overlay size. Larger is faster, but requires more memory." NL
       #endif
       #if defined(__WIN32__)
       "-P              Increase program priority to run faster." NL
       #endif
       "-S<sortspec>    Sorts all mail areas according to the sort specs." NL
       "-T<seconds>     Set a timeout value. GoldED will auto-exit after timeout." NL
       "-V  or  -VV     Verbose or Very verbose (-VV) config compile. Use -VV to debug." NL
       "-W              Write a GOLDAREA.INC file with AREADEF's of all mail areas." NL
       "-X,  -Y,  -Z    Reserved for debugging purposes." NL
       "" NL
       "Any non-option parameter is stuffed into the keyboard buffer." NL
       "" NL, 
       __gver_longpid__,
       __gver_date__+7
    );

    exit(0);
  }

  throw_init();
  #if defined(GTHROW_LOG)
  throw_log = &LOG;
  #endif
  CfgInit();
  GFTrkInit(gftrk_set_max);

  // Register exit cleanup functions
  atexit(Cleanup);
  signal(SIGINT, SIG_IGN);

  #if defined(__WIN32__)
  SetConsoleCtrlHandler(GoldedCtrlHandler, TRUE);
  #endif

  if(*cmdlinecfg)
    found = YES;

  if(NOT found) {

    #if defined(__OS2__)
    ptr = getenv("GED2");   // Look for configfilename in the environment
    #elif defined(__WIN32__)
    ptr = getenv("GEDW32");   // Look for configfilename in the environment
    #else
    ptr = NULL;
    #endif
    if(NOT(ptr AND *ptr))
      ptr = getenv("GOLDED");   // Look for configfilename in the environment
    if(NOT(ptr AND *ptr))
      ptr = getenv("GED");

    // Get it from environment variables
    if(ptr AND *ptr) {
      AddBackslash(strcpy(cmdlinecfg, ptr));
      #if defined(__OS2__)
      if(NOT found)
        found = ExistCfg(cmdlinecfg, "ged2.cfg");
      #elif defined(__WIN32__)
      if(NOT found)
        found = ExistCfg(cmdlinecfg, "gedw32.cfg");
      #endif
      if(NOT found)
        found = ExistCfg(cmdlinecfg, "golded.cfg");
    }
    
    // Get it in current directory
    if(NOT found) {
      getcwd(cmdlinecfg, GMAXPATH-1);
      AddBackslash(cmdlinecfg);
      #if defined(__OS2__)
      if(NOT found)
        found = ExistCfg(cmdlinecfg, "ged2.cfg");
      #elif defined(__WIN32__)
      if(NOT found)
        found = ExistCfg(cmdlinecfg, "gedw32.cfg");
      #endif
      if(NOT found)
        found = ExistCfg(cmdlinecfg, "golded.cfg");
    }

    // Get it where the the .EXE file is
    if(NOT found) {
      strcpy(cmdlinecfg, argv[0]);

      // There must be a full path in argv[0]
      if((ptr = strrchr(cmdlinecfg, GOLD_SLASH_CHR)) != NULL)
        ptr[1] = NUL;
      else
        *cmdlinecfg = NUL;
    }
  }

  //strupr(cmdlinecfg);
  _splitpath(cmdlinecfg, drive, dir, file, ext);

  #ifdef GOLD_HAS_DRIVES
  if(*drive == NUL) {
    Path cwd;
    getcwd(cwd, sizeof(Path));
    sprintf(drive, "%c:", *cwd);
  }
  if(*dir == NUL) {
    getcurdir(*drive-'@', buf);
    sprintf(dir, "%s%s\\", *buf ? "\\" : "", buf);
  }
  #endif
  if(*file == NUL)
    strcpy(file, "golded");
  if(*ext == NUL)
    strcpy(ext, ".cfg");

  // Make filenames
  _makepath(cmdlinecfg, drive, dir, file, ext);

  strcpy(CFG->goldcfg, cmdlinecfg);
  _makepath(CFG->goldged, drive, dir, file, __gver_cfgext__);
  _makepath(CFG->goldpath, drive, dir, "", "");

  // Call install detect procedure
  if(cmdlineinstall)
    InstallDetect(cmdlineinstpath);

  #if defined(__WIN32__)
  SetConsoleTitle(__gver_longpid__);
  #endif

  // Display startup banner
  printf("\r%s", __gver_longpid__);

  if(NOT fexist(cmdlinecfg)) {
    printf(NL "*** Cannot start: %s not found! ***" NL, cmdlinecfg);
    errorlevel = EXIT_NONAME;
    exit(0);
  }

  // Read/compile the config files
  compiled = ReadGoldedCfg(cmdlineforce);

  // Call install finish procedure
  if(cmdlineinstall) {
    if(InstallFinish()) {
      printf(NL "*** INSTALL NOT COMPLETED ***" NL);
      remove(cmdlinecfg);
      errorlevel = EXIT_NONAME;
      exit(0);
    }
  }

  // Overlay swapping
  #if defined(__BORLANDC__) and defined(__MSDOS__)
  switch(CFG->overlay) {
    case OVL_EMS:
      _OvrInitEms(0,0,0);
      break;
    case OVL_EXT:
      _OvrInitExt(0,0);
      break;
  }
  #endif

  // Mouse support
  #ifdef GOLD_MOUSE
  if(CFG->mouse) {
    gmou.Init();
    gmou.SetLevel(GMOU_LEVEL_CURS);
  }
  #endif

  // Start the log
  LOG.open(CFG->logfile, __gver_longpid__, __gver_shortlogname__, CFG->logformat);

  // Read/compile various configs
  compiled += ReadLangCfg(cmdlineforce);
  compiled += ReadKeysCfg(cmdlineforce);
  compiled += ReadHelpCfg(cmdlineforce);

  // Initialize sound system
  InitSound();

  // Handle extended keyboard
  if(not CFG->switches.get(keybext))
    gkbd.extkbd = CFG->switches.get(keybext);

  // Report detected multitasker
  if(NOT quiet) {
    if(gmtsk.detected)
      printf(NL "* Running under %s.", gmtsk.name);
  }

  if(cfgerrors) {
    printf(NL "* ");
    printf("Total CFG errors found: %i. ", cfgerrors);
    printf("Press almost any key to continue.");
    clearkeys();
    kbclear();
    kbxget();
  }

  if(CFG->switches.get(keybclear)) {
    clearkeys();
    kbclear();
    if(*CFG->keybstack)             // The config keys
      kbputstr(CFG->keybstack);
  }
  else if(*keybuf)                  // The commandline keys
    kbputstr(keybuf);
  else if(*CFG->keybstack)          // The config keys
    kbputstr(CFG->keybstack);

  fieldupd = EDIT->FieldClear() ? 2 : 1;

  printf(NL);

  vposget(&gvid.orig.cursor.row, &gvid.orig.cursor.column);

  oldscreen = vsave();

  if(CFG->screensize > 0xFF) {
    gvid.setmode(CFG->screensize >> 8);    // Set video mode
  }
  else if(CFG->screensize) {
    gvid.setrows(CFG->screensize);         // Just set rows
  }

  gvid.setintensity(CFG->intensecolors);

  if(C_BACKB != 0)
    gvid.setoverscan(C_BACKB);

  vcurhide();

  if(CFG->screenmaxrow >= 10)
    gvid.numrows = CFG->screenmaxrow;

  if(CFG->screenmaxcol >= 80)
    gvid.numcols = CFG->screenmaxcol;

  CFG->dispmargin = CFG->cfgdispmargin;
  if(CFG->dispmargin <= 0)
    CFG->dispmargin += gvid.numcols;

  if(CFG->dispmargin > gvid.numcols)
    CFG->dispmargin = gvid.numcols;

  if(EDIT->Margin() <= 0)
    EDIT->Margin(EDIT->Margin()+gvid.numcols);

  CFG->quotemargin = CFG->cfgquotemargin;
  if(CFG->quotemargin <= 0)
    CFG->quotemargin += gvid.numcols;

  if(EDIT->QuoteMargin() <= 0)
    EDIT->QuoteMargin(EDIT->QuoteMargin()+gvid.numcols);

  #if defined(__MSDOS__)
  if(CFG->switches.get(screenusebios))
    gvid.setdevice(GVID_BIO);
  else if(CFG->switches.get(screenelimsnow))
    gvid.setdevice(GVID_CGA);
  else
    gvid.setdevice(GVID_DMA);
  #endif

  // Set palette if changes were specified
  if(CFG->screenpalette[16])
    gvid.setpalette(CFG->screenpalette);
  
  // Set window opening style
  wsetstyle(STYLE_NORMAL);

  // If keyboard polling is enabled
  if(CFG->keybmode == KEYB_POLL) {

    // Switch to keyboard polling instead of blocking
    gkbd.polling = TRUE;
    gkbd.tickinterval = 500;   // Tick twice per second
    gkbd.tickvalue = gclock();  // Reset starting tick
    kbdsettickfunc(update_statuslines);
  }

  inforow = ((MAXROW-1)/2)+6;

  bool areasdefined = false;

  // Do checking for an area and unknown aka's in area
  // config at once. 
  if(AL.First()) {
    do {
      if(not AL.pointer->isseparator()) 
        areasdefined = true;

      if(CFG->akas and (not AL.pointer->aka().valid()))
        AL.pointer->set_aka(CFG->aka[0].addr);
    
    } while(AL.Next());
  }

  if(not areasdefined) {
    LOG.ErrConfig();
    LOG.printf("! There do not seem to be any mail areas defined.");
    LOG.printf("+ Advice: Check your setup of mail areas.");
    ConfigErrorExit();
  }

  if(CFG->usernames == 0) {
    LOG.ErrConfig();
    LOG.printf("! There do not seem to be any USERNAME's defined.");
    LOG.printf("+ Advice: Check your setup of USERNAME's.");
    ConfigErrorExit();
  }

  if(CFG->cmdkeys == 0) {
    LOG.ErrConfig();
    LOG.printf("! There do not seem to be any command keys defined.");
    LOG.printf("+ Advice: Check your key setup in GOLDKEYS.CFG.");
    ConfigErrorExit();
  }

  if(CFG->origins == 0)
    CfgOrigin("");

  w_back();     // Open a nice background window
  w_brag();     // Display Brag Window
    
  HandleGEvent(EVTT_STARTUP);
         
  // Write the binary config if any of the cfgs were compiled.
  if(compiled)
    WriteGoldGed();
  throw_release(stamp);
  stamps = 0;

  // Override timeout from commandline
  if(cmdlinetimeout != -1)
    CFG->timeout = cmdlinetimeout;

  // Adjust header item sizes if negative
  if(CFG->disphdrnameset.pos < 0)    CFG->disphdrnameset.pos += MAXCOL;
  if(CFG->disphdrnodeset.pos < 0)    CFG->disphdrnodeset.pos += MAXCOL;
  if(CFG->disphdrdateset.pos < 0)    CFG->disphdrdateset.pos += MAXCOL;
  if(CFG->disphdrnameset.len < 0)    CFG->disphdrnameset.len += MAXCOL;
  if(CFG->disphdrnodeset.len < 0)    CFG->disphdrnodeset.len += MAXCOL;
  if(CFG->disphdrdateset.len < 0)    CFG->disphdrdateset.len += MAXCOL;

  if(EDIT->HdrNamePos() < 0)  EDIT->HdrNamePos(EDIT->HdrNamePos()+MAXCOL);
  if(EDIT->HdrNameLen() < 0)  EDIT->HdrNameLen(EDIT->HdrNameLen()+MAXCOL);
  if(EDIT->HdrNodePos() < 0)  EDIT->HdrNodePos(EDIT->HdrNodePos()+MAXCOL);
  if(EDIT->HdrNodeLen() < 0)  EDIT->HdrNodeLen(EDIT->HdrNodeLen()+MAXCOL);

  // Read the lastreads from the last session
  if(CFG->switches.get(areakeeplast))
    AL.ReadGoldLast();

  // Setup start echo
  if(NOT *stecho)
    strcpy(stecho, CFG->areastart);

  // Sort the areas
  if(*AL.sortspec)
    strcpy(CFG->arealistsort, AL.sortspec);
  AL.Sort();

  // Write GOLDAREA.INC if told
  if(cmdlinewriteareas)
    AL.WriteAreaDef(AddPath(CFG->goldpath, "GOLDAREA.INC"));

  // Get the global max length of echoids (for the arealist)
  extern int arealistnumgrps;
  extern int areaswithgroupid;
  for(uint n=0; n<AL.total; n++) {
    int _tmp = strlen(AL[n]->echoid());
    if(echoid_width < _tmp)
      echoid_width = _tmp;
    if(AL[n]->groupid()) {
      areaswithgroupid++;
      if(AL[n]->groupid() & 0x8000u)
        arealistnumgrps = TRUE;
    }
  }
  if(CFG->arealistechomax) {
    if(CFG->arealistechomax > (int)(sizeof(Echo)-1))
      CFG->arealistechomax = sizeof(Echo)-1;
    if(CFG->arealistechomax < 0)
      echoid_width += CFG->arealistechomax;
    else
      echoid_width = CFG->arealistechomax;
  }
  if(areaswithgroupid and CFG->switches.get(arealistgroupid))
    groupid_width = arealistnumgrps ? 3 : 1;

  int spaces = 0;
  bool area_found = false;
  bool marked_found = false;
  bool desc_found = false;
  bool count_found = false;
  bool pmark_found = false;
  bool unread_found = false;
  bool changed_found = false;
  bool echoid_found = false;
  bool groupid_found = false;
  for(int pass=1; pass<=2; pass++) {
    if(pass == 2) {
      if(!area_found)     area_width = 0;
      if(!marked_found)   marked_width = 0;
      if(!desc_found)     desc_width = 0;
      if(!count_found)    count_width = 0;
      if(!pmark_found)    pmark_width = 0;
      if(!unread_found)   unread_width = 0;
      if(!changed_found)  changed_width = 0;
      if(!echoid_found)   echoid_width = 0;
      if(!groupid_found)  groupid_width = 0;
      if(desc_width == -1) {
        desc_width =
          MAXCOL - 2 - spaces - area_width - marked_width -
          count_width - pmark_width - unread_width - changed_width -
          echoid_width - groupid_width;
      }
    }
    int pos = 0;
    char* p = CFG->arealistformat;
    while(*p) {
      char c = (char)toupper(*p);
      char d = *(++p);
      int w = atoi(p);
      while(isdigit(*p))
        p++;
      if(pass == 1) {
        if(isalpha(c)) {
          switch(c) {
            case 'A':  area_found = true;     if(isdigit(d)) area_width = w;     break;
            case 'M':  marked_found = true;   if(isdigit(d)) marked_width = w;   break;
            case 'D':  desc_found = true;     if(isdigit(d)) desc_width = w;     break;
            case 'C':  count_found = true;    if(isdigit(d)) count_width = w;    break;
            case 'P':  pmark_found = true;    if(isdigit(d)) pmark_width = w;    break;
            case 'U':  unread_found = true;   if(isdigit(d)) unread_width = w;   break;
            case 'N':  changed_found = true;  if(isdigit(d)) changed_width = w;  break;
            case 'E':  echoid_found = true;   if(isdigit(d)) echoid_width = w;   break;
            case 'G':  groupid_found = true;  if(isdigit(d)) groupid_width = w;  break;
          }
        }
        else {
          spaces++;
        }
      }
      else {
        switch(c) {
          case 'A':  area_pos = pos;     pos += area_width;     break;
          case 'M':  marked_pos = pos;   pos += marked_width;   break;
          case 'D':  desc_pos = pos;     pos += desc_width;     break;
          case 'C':  count_pos = pos;    pos += count_width;    break;
          case 'P':  pmark_pos = pos;    pos += pmark_width;    break;
          case 'U':  unread_pos = pos;   pos += unread_width;   break;
          case 'N':  changed_pos = pos;  pos += changed_width;  break;
          case 'E':  echoid_pos = pos;   pos += echoid_width;   break;
          case 'G':  groupid_pos = pos;  pos += groupid_width;  break;
          default:   pos++;
        }
      }
    }
  }

  //update_statuslines();
  update_statuslines();
  ReadEcholists();            // Read the Confmail compatible echotag lists
  update_statuslines();
  ReadAddrMacros();           // Read the address macro file
  update_statuslines();
  ReadEscsets();              // Read the escape tables
  update_statuslines();

  #if defined(__BORLANDC__) and defined(__DOS__)
  if(coreleft()/1024L <= 50L) {
    whelpcat(H_EMemoryWarning);
    call_help();
    whelpcat(H_General);
  }
  #endif
  
  // Initialize the messagebases
  update_statuslinef(LNG->LockShareCap, LNG->Checking);
  WideLog = &LOG;
  WideDebug = cmdlinedebughg;
  WideMsgSize = EDIT->MsgSize();
  WideCanLock = CFG->sharemode ? TestLockPath(CFG->goldpath) : FALSE;
  WideSharemode = CFG->sharemode;
  WideUsernames = CFG->usernames;
  WideUsername = new const char*[WideUsernames];
  for(int w=0; w<WideUsernames; w++)
    WideUsername[w] = CFG->username[w].name;
  WidePersonalmail = CFG->personalmail;
  WideDispsoftcr = CFG->switches.get(dispsoftcr);
  if(AL.msgbases & MT_EZYCOM) {
    update_statuslinef("%s Ezycom", LNG->Checking);
    EzycomInit(CFG->ezycom.msgbasepath, CFG->ezycom.userbasepath, CFG->ezycomuserno);
  }
  if(AL.msgbases & MT_FIDO) {
    update_statuslinef("%s Fido", LNG->Checking);
    FidoInit(CFG->fidolastread, CFG->switches.get(fidohwmarks), CFG->switches.get(fidonullfix), CFG->fidouserno, CFG->squishuserpath);
  }
  if(AL.msgbases & MT_GOLDBASE) {
    update_statuslinef("%s Goldbase", LNG->Checking);
    GoldInit(CFG->goldbasepath, CFG->goldbasesyspath, CFG->goldbaseuserno);
  }
  if(AL.msgbases & MT_HUDSON) {
    update_statuslinef("%s Hudson", LNG->Checking);
    HudsInit(CFG->hudsonpath, CFG->hudsonsyspath, CFG->hudsonuserno, CFG->hudsonsizewarn, CFG->ra2usersbbs);
  }
  if(AL.msgbases & MT_JAM) {
    update_statuslinef("%s JAM", LNG->Checking);
    JamInit(CFG->jampath, CFG->switches.get(jamharddelete));
  }
  if(AL.msgbases & MT_PCBOARD) {
    update_statuslinef("%s PCBoard", LNG->Checking);
    PcbInit(CFG->pcboardpath, CFG->pcboarduserno);
  }
  if(AL.msgbases & MT_SQUISH) {
    update_statuslinef("%s Squish", LNG->Checking);
    SquishInit(CFG->squishuserpath, CFG->squishuserno, CFG->switches.get(squishdirect), CFG->squishrecycle, CFG->squishscan);
  }
  if(AL.msgbases & MT_WILDCAT) {
    update_statuslinef("%s WildCat!", LNG->Checking);
    WCatInit(CFG->wildcatuserno);
  }
  if(AL.msgbases & MT_ADEPTXBBS) {
    update_statuslinef("%s AdeptXBBS", LNG->Checking);
    XbbsInit(CFG->adeptxbbspath, CFG->adeptxbbsuserno);
  }
  update_statuslinef("...");

  // Delete the D'Bridge "mail waiting" semaphore files
  if(dbedit) {
    remove(AddPath(CFG->areapath, "DBRIDGE.NMW"));
    remove(AddPath(CFG->areapath, "DBRIDGE.EMW"));
  }

  // NO! I'm fed up. This is now commented out. (dam)
  #if 0
  #define CFG__logowait 5000
  // only if not (-NOSCAN or any keys are given on commandline)
  if(not (cmdlinenoscan or (*keybuf)))
    waitkeyt(CFG__logowait);
  #endif

  // Unlink windows
  wunlink(W_READ);

  THROW_CHECK();
}


//  ------------------------------------------------------------------

