
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gekeys.h,v 1.2 1999/02/02 02:09:43 damware Exp $
//  ------------------------------------------------------------------
//  Keyboard system.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//
//  The keyboard system in GoldED
//
//  Keycodes in the range 0x0000-0xFDFF are normal keys.
//  Keycodes in the range 0xFE00-0xFE7F are macro numbers.
//  Keycodes in the range 0xFE80-0xFFFE are command numbers.
//
//  The global variable cmdkeys holds a list of all commands and
//  the keys or macro numbers, sorted by key/macro.
//
//  The global variable macros holds all the macros, sorted by key.
//  The global variable totmacros contains the total number of macros.
//
//  The macro list is searched first, then the command list, thus
//  allowing macros that override commands.
//
//  ------------------------------------------------------------------

const Key KK_Auto                 = 0xFD00;
const Key KK_Macro                = 0xFE00;
const Key KK_AreaMacro            = 0xFE01;
const Key KK_EditMacro            = 0xFE02;
const Key KK_FileMacro            = 0xFE03;
const Key KK_ListMacro            = 0xFE04;
const Key KK_NodeMacro            = 0xFE05;
const Key KK_ReadMacro            = 0xFE06;

const Key KK_Commands             = 0xFE80;
const Key KK_ExternUtil01         = 0xFE81;
const Key KK_ExternUtil02         = 0xFE82;
const Key KK_ExternUtil03         = 0xFE83;
const Key KK_ExternUtil04         = 0xFE84;
const Key KK_ExternUtil05         = 0xFE85;
const Key KK_ExternUtil06         = 0xFE86;
const Key KK_ExternUtil07         = 0xFE87;
const Key KK_ExternUtil08         = 0xFE88;
const Key KK_ExternUtil09         = 0xFE89;
const Key KK_ExternUtil10         = 0xFE8A;
const Key KK_ExternUtil11         = 0xFE8B;
const Key KK_ExternUtil12         = 0xFE8C;
const Key KK_ExternUtil13         = 0xFE8D;
const Key KK_ExternUtil14         = 0xFE8E;
const Key KK_ExternUtil15         = 0xFE8F;
const Key KK_ExternUtil16         = 0xFE90;
const Key KK_ExternUtil17         = 0xFE91;
const Key KK_ExternUtil18         = 0xFE92;
const Key KK_ExternUtil19         = 0xFE93;
const Key KK_ExternUtil20         = 0xFE94;
const Key KK_ExternUtil21         = 0xFE95;
const Key KK_ExternUtil22         = 0xFE96;
const Key KK_ExternUtil23         = 0xFE97;
const Key KK_ExternUtil24         = 0xFE98;
const Key KK_ExternUtil25         = 0xFE99;

const Key KK_AreaAbort            = 0xFEA0;
const Key KK_AreaAskExit          = 0xFEA1;
const Key KK_AreaBoardnos         = 0xFEA2;
const Key KK_AreaCatchUp          = 0xFEA3;
const Key KK_AreaDosShell         = 0xFEA4;
const Key KK_AreaDropMsgMarks     = 0xFEA5;
const Key KK_AreaGotoFirst        = 0xFEA6;
const Key KK_AreaGotoLast         = 0xFEA7;
const Key KK_AreaGotoNext         = 0xFEA8;
const Key KK_AreaGotoPrev         = 0xFEA9;
const Key KK_AreaHeat             = 0xFEAA;
const Key KK_AreaInfo             = 0xFEAB;
const Key KK_AreaJump             = 0xFEAC;
const Key KK_AreaJumpNextMatch    = 0xFEAD;
const Key KK_AreaMainMenu         = 0xFEAE;
const Key KK_AreaMenu             = 0xFEAF;
const Key KK_AreaQuitNow          = 0xFEB0;
const Key KK_AreaScan             = 0xFEB1;
const Key KK_AreaScanPM           = 0xFEB2;
const Key KK_AreaSelect           = 0xFEB3;
const Key KK_AreaSelectMarks      = 0xFEB4;
const Key KK_AreaSoundkill        = 0xFEB5;
const Key KK_AreaToggle           = 0xFEB6;
const Key KK_AreaTouchNetscan     = 0xFEB7;
const Key KK_AreaWriteGoldlast    = 0xFEB8;
const Key KK_AreaZap              = 0xFEB9;

const Key KK_EditAbort            = 0xFEC0;
const Key KK_EditAnchor           = 0xFEC1;
const Key KK_EditAskExit          = 0xFEC2;
const Key KK_EditClearDeleteBuf   = 0xFEC3;
const Key KK_EditClearPasteBuf    = 0xFEC4;
const Key KK_EditCopy             = 0xFEC5;
const Key KK_EditCopyAbove        = 0xFEC6;
const Key KK_EditCopyAboveChar    = 0xFEC7;
const Key KK_EditCut              = 0xFEC8;
const Key KK_EditDelChar          = 0xFEC9;
const Key KK_EditDeleteEOL        = 0xFECA;
const Key KK_EditDelLeft          = 0xFECB;
const Key KK_EditDelLine          = 0xFECC;
const Key KK_EditDelLtWord        = 0xFECD;
const Key KK_EditDelRtWord        = 0xFECE;
const Key KK_EditDosShell         = 0xFECF;
const Key KK_EditDupLine          = 0xFED0;
const Key KK_EditExitMsg          = 0xFED1;
const Key KK_EditExportText       = 0xFED2;
const Key KK_EditGoBegLine        = 0xFED3;
const Key KK_EditGoBotLine        = 0xFED4;
const Key KK_EditGoBotMsg         = 0xFED5;
const Key KK_EditGoDown           = 0xFED6;
const Key KK_EditGoEOL            = 0xFED7;
const Key KK_EditGoLeft           = 0xFED8;
const Key KK_EditGoPgDn           = 0xFED9;
const Key KK_EditGoPgUp           = 0xFEDA;
const Key KK_EditGoRight          = 0xFEDB;
const Key KK_EditGoTopLine        = 0xFEDC;
const Key KK_EditGoTopMsg         = 0xFEDD;
const Key KK_EditGoUp             = 0xFEDE;
const Key KK_EditGoWordLeft       = 0xFEDF;
const Key KK_EditGoWordRight      = 0xFEE0;
const Key KK_EditHeader           = 0xFEE1;
const Key KK_EditImportQuotebuf   = 0xFEE2;
const Key KK_EditImportText       = 0xFEE3;
const Key KK_EditKillQuotes       = 0xFEE4;
const Key KK_EditLoadFile         = 0xFEE5;
const Key KK_EditLookupCursor     = 0xFEE6;
const Key KK_EditLookupDest       = 0xFEE7;
const Key KK_EditLookupOrig       = 0xFEE8;
const Key KK_EditMainMenu         = 0xFEE9;
const Key KK_EditMenu             = 0xFEEA;
const Key KK_EditNewline          = 0xFEEB;
const Key KK_EditPaste            = 0xFEEC;
const Key KK_EditQuitNow          = 0xFEED;
const Key KK_EditReflow           = 0xFEEE;
const Key KK_EditSaveFile         = 0xFEEF;
const Key KK_EditSaveMsg          = 0xFEF0;
const Key KK_EditSoundkill        = 0xFEF1;
const Key KK_EditSpellCheck       = 0xFEF2;
const Key KK_EditTab              = 0xFEF3;
const Key KK_EditTabReverse       = 0xFEF4;
const Key KK_EditToggleCase       = 0xFEF5;
const Key KK_EditToggleInsert     = 0xFEF6;
const Key KK_EditToLower          = 0xFEF7;
const Key KK_EditToUpper          = 0xFEF8;
const Key KK_EditUnDelete         = 0xFEF9;
const Key KK_EditZapQuoteBelow    = 0xFEFA;

const Key KK_FileAbort            = 0xFF00;
const Key KK_FileAskExit          = 0xFF01;
const Key KK_FileDosShell         = 0xFF02;
const Key KK_FileGotoFirst        = 0xFF03;
const Key KK_FileGotoLast         = 0xFF04;
const Key KK_FileGotoNext         = 0xFF05;
const Key KK_FileGotoPrev         = 0xFF06;
const Key KK_FileMark             = 0xFF07;
const Key KK_FileMarkAll          = 0xFF08;
const Key KK_FileMenu             = 0xFF09;
const Key KK_FileQuitNow          = 0xFF0A;
const Key KK_FileSelect           = 0xFF0B;
const Key KK_FileToggleMark       = 0xFF0C;
const Key KK_FileToggleMarkAll    = 0xFF0D;
const Key KK_FileUnMark           = 0xFF0E;
const Key KK_FileUnMarkAll        = 0xFF0F;

const Key KK_ListAbort            = 0xFF10;
const Key KK_ListAskExit          = 0xFF11;
const Key KK_ListDosShell         = 0xFF12;
const Key KK_ListGotoBookMark     = 0xFF13;
const Key KK_ListGotoFirst        = 0xFF14;
const Key KK_ListGotoLast         = 0xFF15;
const Key KK_ListGotoNext         = 0xFF16;
const Key KK_ListGotoPrev         = 0xFF17;
const Key KK_ListMainMenu         = 0xFF18;
const Key KK_ListMarkingOptions   = 0xFF19;
const Key KK_ListMenu             = 0xFF1A;
const Key KK_ListQuitNow          = 0xFF1B;
const Key KK_ListSelect           = 0xFF1C;
const Key KK_ListToggleBookMark   = 0xFF1D;
const Key KK_ListToggleDate       = 0xFF1E;
const Key KK_ListToggleMark       = 0xFF1F;
const Key KK_ListToggleWideSubj   = 0xFF20;

const Key KK_NodeAbort            = 0xFF30;
const Key KK_NodeAskExit          = 0xFF31;
const Key KK_NodeDosShell         = 0xFF32;
const Key KK_NodeGotoFirst        = 0xFF33;
const Key KK_NodeGotoLast         = 0xFF34;
const Key KK_NodeGotoNext         = 0xFF35;
const Key KK_NodeGotoPrev         = 0xFF36;
const Key KK_NodeMainMenu         = 0xFF37;
const Key KK_NodeMenu             = 0xFF38;
const Key KK_NodeQuitNow          = 0xFF39;
const Key KK_NodeSelect           = 0xFF3A;

const Key KK_ReadAddressbookAdd   = 0xFF40;
const Key KK_ReadAskExit          = 0xFF41;
const Key KK_ReadChangeAka        = 0xFF42;
const Key KK_ReadChangeAttrs      = 0xFF43;
const Key KK_ReadChangeCharsIn    = 0xFF44;
const Key KK_ReadChangeCharsOut   = 0xFF45;
const Key KK_ReadChangeMsg        = 0xFF46;
const Key KK_ReadChangeOrigin     = 0xFF47;
const Key KK_ReadChangeTagline    = 0xFF48;
const Key KK_ReadChangeTemplate   = 0xFF49;
const Key KK_ReadChangeUsername   = 0xFF4A;
const Key KK_ReadChangeXlatImport = 0xFF4B;
const Key KK_ReadCommentMsg       = 0xFF4C;
const Key KK_ReadCopyMoveForward  = 0xFF4D;
const Key KK_ReadDecreaseMargin   = 0xFF4E;
const Key KK_ReadDeleteMsg        = 0xFF4F;
const Key KK_ReadDosShell         = 0xFF50;
const Key KK_ReadExternUtilMenu   = 0xFF51;
const Key KK_ReadFidoRenumber     = 0xFF52;
const Key KK_ReadFileRequest      = 0xFF53;
const Key KK_ReadFindAll          = 0xFF54;
const Key KK_ReadFindHeader       = 0xFF55;
const Key KK_ReadGotoBookMark     = 0xFF56;
const Key KK_ReadGotoFirstMsg     = 0xFF57;
const Key KK_ReadGotoLastMsg      = 0xFF58;
const Key KK_ReadGotoMsgno        = 0xFF59;
const Key KK_ReadGotoNextArea     = 0xFF5A;
const Key KK_ReadGotoNextMsg      = 0xFF5B;
const Key KK_ReadGotoNextUnread   = 0xFF5C;
const Key KK_ReadGotoPrevArea     = 0xFF5D;
const Key KK_ReadGotoPrevMsg      = 0xFF5E;
const Key KK_ReadGotoPrevUnread   = 0xFF5F;
const Key KK_ReadGotoReplies      = 0xFF60;
const Key KK_ReadGotoReply1st     = 0xFF61;
const Key KK_ReadGotoReplyNext    = 0xFF62;
const Key KK_ReadGotoReplyPrev    = 0xFF63;
const Key KK_ReadIncreaseMargin   = 0xFF64;
const Key KK_ReadLookupDest       = 0xFF65;
const Key KK_ReadLookupOrig       = 0xFF66;
const Key KK_ReadMainMenu         = 0xFF67;
const Key KK_ReadMakeUserlist     = 0xFF68;
const Key KK_ReadMarkingOptions   = 0xFF69;
const Key KK_ReadMenu             = 0xFF6A;
const Key KK_ReadMessageList      = 0xFF6B;
const Key KK_ReadMakePathreport   = 0xFF6C;
const Key KK_ReadMoveCommentMsg   = 0xFF6D;
const Key KK_ReadMoveQuoteMsg     = 0xFF6E;
const Key KK_ReadMsgContinue      = 0xFF6F;
const Key KK_ReadMsgEnd           = 0xFF70;
const Key KK_ReadMsgHome          = 0xFF71;
const Key KK_ReadMsgLineDown      = 0xFF72;
const Key KK_ReadMsgLineUp        = 0xFF73;
const Key KK_ReadMsgPgDn          = 0xFF74;
const Key KK_ReadMsgPgUp          = 0xFF75;
const Key KK_ReadNewArea          = 0xFF76;
const Key KK_ReadNewMsg           = 0xFF77;
const Key KK_ReadQuitNow          = 0xFF78;
const Key KK_ReadQuoteBuf         = 0xFF79;
const Key KK_ReadQuoteMsg         = 0xFF7A;
const Key KK_ReadReplyMsg         = 0xFF7B;
const Key KK_ReadSearch           = 0xFF7C;
const Key KK_ReadSoundkill        = 0xFF7D;
const Key KK_ReadStealTagline     = 0xFF7E;
const Key KK_ReadThreadtree       = 0xFF7F;
const Key KK_ReadToggleBookMark   = 0xFF80;
const Key KK_ReadToggleHexdump    = 0xFF81;
const Key KK_ReadToggleHidden     = 0xFF82;
const Key KK_ReadToggleHiddKlud   = 0xFF83;
const Key KK_ReadToggleKludge     = 0xFF84;
const Key KK_ReadToggleMark       = 0xFF85;
const Key KK_ReadToggleMarkRead   = 0xFF86;
const Key KK_ReadTogglePageBar    = 0xFF87;
const Key KK_ReadToggleQuote      = 0xFF88;
const Key KK_ReadToggleRealMsgno  = 0xFF89;
const Key KK_ReadToggleROT13      = 0xFF8A;
const Key KK_ReadToggleStyles     = 0xFF8B;
const Key KK_ReadToggleTwits      = 0xFF8C;
const Key KK_ReadTouchNetscan     = 0xFF8D;
const Key KK_ReadTouchSemaphore   = 0xFF8E;
const Key KK_ReadUserbase         = 0xFF8F;
const Key KK_ReadUUDecode         = 0xFF90;
const Key KK_ReadWriteMsg         = 0xFF91;

const Key KK_AreaUndefine         = 0xFFA0;
const Key KK_EditUndefine         = 0xFFA1;
const Key KK_FileUndefine         = 0xFFA2;
const Key KK_ListUndefine         = 0xFFA3;
const Key KK_NodeUndefine         = 0xFFA4;
const Key KK_ReadUndefine         = 0xFFA5;

const Key KK_ZzzzLast             = 0xFFA6;


//  ------------------------------------------------------------------
//  Key groups (type of key)

const int KT_A =  1;   // Arealist
const int KT_E =  2;   // Editor
const int KT_G =  4;   // General (not used)
const int KT_M =  8;   // Msg lister
const int KT_N = 16;   // Node browser
const int KT_R = 32;   // Reader                  (+ external utils)
const int KT_F = 64;   // File browser


//  ------------------------------------------------------------------
//  Key command

struct CmdKey {
  Key  key;        // Associated key or macro number
  Key  cmd;        // Command number
  int  type;       // Type of key
};


//  ------------------------------------------------------------------
//  Macro

struct Macro {
  int  type;       // Type of key
  Key  key;        // Associated key
  Key  buf[128];   // Actual keys/commands
};


//  ------------------------------------------------------------------

struct tglobalkey {
  word crc_token;
  Key  keyval;
  word keytype;
};

//  ------------------------------------------------------------------

