
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geline.cpp,v 1.6 1999/01/31 19:22:59 damware Exp $
//  ------------------------------------------------------------------
//  Conversion of a raw message to a linked list of lines.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include <gstrmail.h>
#include <gutlcode.h>
#include <ghdrmime.h>


//  ------------------------------------------------------------------

#if defined(__UNIX__)
#define GOLDMARK '\xB2'
#else
#define GOLDMARK '\xFD'
#endif


//  ------------------------------------------------------------------

const int BODYLINE = 0;
const int HEADERLINE = 1;
const int USERDEFINED = 2;
const int RFC_X = 3;

enum {
  MASK_FTS = 0x1000,
  FTS_AREA,
  FTS_INTL,
  FTS_FMPT,
  FTS_TOPT,
  FTS_MSGID,
  FTS_REPLY,
  FTS_SEENBY,
  FTS_PATH,
  FTS_ZZZZ
};

enum {
  MASK_FSC = 0x2000,
  FSC_CHARSET,
  FSC_CHRC,
  FSC_CHRS,
  FSC_DOMAIN,
  FSC_EID,
  FSC_ENC,
  FSC_ENCLFILE,
  FSC_FLAGS,
  FSC_FWDFROM,
  FSC_FWDORIG,
  FSC_FWDTO,
  FSC_FWDDEST,
  FSC_FWDSUBJ,
  FSC_FWDAREA,
  FSC_FWDMSGID,
  FSC_I51,
  FSC_MSGTO,
  FSC_PID,
  FSC_PTH,
  FSC_REPLYADDR,
  FSC_REPLYTO,
  FSC_SPLIT,
  FSC_SPTH,
  FSC_TID,
  FSC_ZZZZ
};

enum {
  MASK_XXX = 0x8000,
  XXX_ACUPDATE,
  XXX_CODEPAGE,
  XXX_DESTADDR,
  XXX_ENCRYPTION,
  XXX_EOT,
  XXX_GATECHK,
  XXX_GID,
  XXX_GIF,
  XXX_GMD,
  XXX_GROUP,
  XXX_MOOD,
  XXX_MSGSEQ,
  XXX_NOTE,
  XXX_ORIGID,
  XXX_ORIGINAL,
  XXX_ORIGREF,
  XXX_RECD,
  XXX_RFC,
  XXX_RFD,
  XXX_RID,
  XXX_SN,
  XXX_SOT,
  XXX_TCL1,
  XXX_TCL2,
  XXX_TZUTCINFO,
  XXX_TZUTC,
  XXX_TZ,
  XXX_VIA,
  XXX_XID,
  XXX_ZZZZ
};

enum {
  MASK_RFC = 0x4000,
  RFC_ALSO_CONTROL,
  RFC_APPARENTLY_TO,
  RFC_APPROVED,
  RFC_ARTICLE_NAMES,
  RFC_ARTICLE_UPDATES,
  RFC_BCC,
  RFC_CC,
  RFC_COMMENT,
  RFC_COMMENTS,
  RFC_CONTENT_DESCRIPTION,
  RFC_CONTENT_DISPOSITION,
  RFC_CONTENT_ID,
  RFC_CONTENT_LENGTH,
  RFC_CONTENT_TRANSFER_ENCODING,
  RFC_CONTENT_TYPE,
  RFC_CONTROL,
  RFC_DATE,
  RFC_DELIVERED_TO,
  RFC_DELIVERY_DATE,
  RFC_DISTRIBUTION,
  RFC_ENCRYPTED,
  RFC_ERRORS_TO,
  RFC_EXPIRES,
  RFC_FOLLOWUP_TO,
  RFC_FROM,
  RFC_FROMX,
  RFC_IN_REPLY_TO,
  RFC_KEYWORDS,
  RFC_LINES,
  RFC_MAILING_LIST,                   // This one is actually _not_ RFC
  RFC_MESSAGE_ID,
  RFC_MIME_VERSION,
  RFC_NEWSGROUPS,
  RFC_NEWS_SOFTWARE,
  RFC_NNTP_POSTING_HOST,
  RFC_NNTP_POSTING_USER,
  RFC_OLD_DATE,
  RFC_ORGANIZATION,
  RFC_ORIGINATOR,
  RFC_PATH,
  RFC_PRECEDENCE,
  RFC_PRIORITY,
  RFC_RECEIVED,
  RFC_REFERENCES,
  RFC_REPLY_TO,
  RFC_RETURN_PATH,
  RFC_RETURN_RECEIPT_TO,
  RFC_SEE_ALSO,
  RFC_SENDER,
  RFC_STATUS,
  RFC_SUBJECT,
  RFC_SUMMARY,
  RFC_SUPERSEDES,
  RFC_TO,
  RFC_VERSION,
  RFC_XREF,
  RFC_X_CHARSET,
  RFC_X_CHAR_ESC,
  RFC_X_FTN_TO,
  RFC_X_MAILREADER,
  RFC_X_NEWSREADER,
  RFC_X_TO,
  RFC_RNEWS,
  RFC_ZZZZ
};

#define MASK_ALL (MASK_FTS|MASK_FSC|MASK_RFC|MASK_XXX)


//  ------------------------------------------------------------------

struct Kludges {
  char* key;
  uint  num;
  byte  req;
};


//  ------------------------------------------------------------------

const byte KCRQ_NONE  = 0x0000;
const byte KCRQ_COLON = 0x0001;
const byte KCRQ_CASE  = 0x0002;


//  ------------------------------------------------------------------

static Kludges __FAR fts_list[] = {

  { "AREA"                       , FTS_AREA                      , KCRQ_CASE  },
  { "INTL"                       , FTS_INTL                      , KCRQ_CASE  },
  { "FMPT"                       , FTS_FMPT                      , KCRQ_CASE  },
  { "TOPT"                       , FTS_TOPT                      , KCRQ_CASE  },
  { "MSGID"                      , FTS_MSGID                     , KCRQ_CASE  },
  { "REPLY"                      , FTS_REPLY                     , KCRQ_CASE  },
  { "SEEN-BY"                    , FTS_SEENBY                    , KCRQ_CASE  },
  { "PATH"                       , FTS_PATH                      , KCRQ_CASE  },
  { ""                           , FTS_ZZZZ                      , KCRQ_NONE  },
};


//  ------------------------------------------------------------------

static Kludges __FAR fsc_list[] = {

  { "CHARSET"                    , FSC_CHARSET                   , KCRQ_CASE  },
  { "CHRC"                       , FSC_CHRC                      , KCRQ_CASE  },
  { "CHRS"                       , FSC_CHRS                      , KCRQ_CASE  },
  { "DOMAIN"                     , FSC_DOMAIN                    , KCRQ_CASE  },
  { "EID"                        , FSC_EID                       , KCRQ_CASE  },
  { "ENC"                        , FSC_ENC                       , KCRQ_CASE  },
  { "ENCLFILE"                   , FSC_ENCLFILE                  , KCRQ_CASE  },
  { "FLAGS"                      , FSC_FLAGS                     , KCRQ_CASE  },
  { "FWDFROM"                    , FSC_FWDFROM                   , KCRQ_CASE  },
  { "FWDORIG"                    , FSC_FWDORIG                   , KCRQ_CASE  },
  { "FWDTO"                      , FSC_FWDTO                     , KCRQ_CASE  },
  { "FWDDEST"                    , FSC_FWDDEST                   , KCRQ_CASE  },
  { "FWDSUBJ"                    , FSC_FWDSUBJ                   , KCRQ_CASE  },
  { "FWDAREA"                    , FSC_FWDAREA                   , KCRQ_CASE  },
  { "FWDMSGID"                   , FSC_FWDMSGID                  , KCRQ_CASE  },
  { "I51"                        , FSC_I51                       , KCRQ_CASE  },
  { "MSGTO"                      , FSC_MSGTO                     , KCRQ_CASE  },
  { "PID"                        , FSC_PID                       , KCRQ_CASE  },
  { "PTH"                        , FSC_PTH                       , KCRQ_CASE  },
  { "REPLYADDR"                  , FSC_REPLYADDR                 , KCRQ_CASE  },
  { "REPLYTO"                    , FSC_REPLYTO                   , KCRQ_CASE  },
  { "SPLIT"                      , FSC_SPLIT                     , KCRQ_CASE  },
  { "SPTH"                       , FSC_SPTH                      , KCRQ_CASE  },
  { "TID"                        , FSC_TID                       , KCRQ_CASE  },
  { ""                           , FSC_ZZZZ                      , KCRQ_NONE  },
};


//  ------------------------------------------------------------------

static Kludges __FAR xxx_list[] = {

  { "ACUPDATE"                   , XXX_ACUPDATE                  , KCRQ_CASE  },
  { "CODEPAGE"                   , XXX_CODEPAGE                  , KCRQ_CASE  },
  { "DESTADDR"                   , XXX_DESTADDR                  , KCRQ_CASE  },
  { "ENCRYPTION"                 , XXX_ENCRYPTION                , KCRQ_CASE  },
  { "EOT"                        , XXX_EOT                       , KCRQ_CASE  },
  { "GATECHK"                    , XXX_GATECHK                   , KCRQ_CASE  },
  { "GID"                        , XXX_GID                       , KCRQ_CASE  },
  { "GIF"                        , XXX_GIF                       , KCRQ_CASE  },
  { "GMD"                        , XXX_GMD                       , KCRQ_CASE  },
  { "GROUP"                      , XXX_GROUP                     , KCRQ_CASE  },
  { "MOOD"                       , XXX_MOOD                      , KCRQ_CASE  },
  { "MSGSEQ"                     , XXX_MSGSEQ                    , KCRQ_CASE  },
  { "NOTE"                       , XXX_NOTE                      , KCRQ_CASE  },
  { "ORIGID"                     , XXX_ORIGID                    , KCRQ_CASE  },
  { "Original"                   , XXX_ORIGINAL                  , KCRQ_NONE  },
  { "ORIGREF"                    , XXX_ORIGREF                   , KCRQ_CASE  },
  { "Recd"                       , XXX_RECD                      , KCRQ_CASE  },
  { "RFC"                        , XXX_RFC                       , KCRQ_CASE  },
  { "RFD"                        , XXX_RFD                       , KCRQ_CASE  },
  { "RID"                        , XXX_RID                       , KCRQ_CASE  },
  { "SN"                         , XXX_SN                        , KCRQ_CASE  },
  { "SOT"                        , XXX_SOT                       , KCRQ_CASE  },
  { "TCL1"                       , XXX_TCL1                      , KCRQ_CASE  },
  { "TCL2"                       , XXX_TCL2                      , KCRQ_CASE  },
  { "TZUTCINFO"                  , XXX_TZUTCINFO                 , KCRQ_CASE  },
  { "TZUTC"                      , XXX_TZUTC                     , KCRQ_CASE  },
  { "TZ"                         , XXX_TZ                        , KCRQ_CASE  },
  { "Via"                        , XXX_VIA                       , KCRQ_NONE  },
  { "XID"                        , XXX_XID                       , KCRQ_CASE  },
  { ""                           , XXX_ZZZZ                      , KCRQ_NONE  }
};


//  ------------------------------------------------------------------

static Kludges __FAR rfc_list[] = {

  { "Also-Control"               , RFC_ALSO_CONTROL              , KCRQ_COLON },
  { "Apparently-To"              , RFC_APPARENTLY_TO             , KCRQ_COLON },
  { "Approved"                   , RFC_APPROVED                  , KCRQ_COLON },
  { "Article-Names"              , RFC_ARTICLE_NAMES             , KCRQ_COLON },
  { "Article-Updates"            , RFC_ARTICLE_UPDATES           , KCRQ_COLON },
  { "Bcc"                        , RFC_BCC                       , KCRQ_COLON },
  { "Cc"                         , RFC_CC                        , KCRQ_COLON },
  { "Comment"                    , RFC_COMMENT                   , KCRQ_COLON },
  { "Comments"                   , RFC_COMMENTS                  , KCRQ_COLON },
  { "Content-Description"        , RFC_CONTENT_DESCRIPTION       , KCRQ_COLON },
  { "Content-Disposition"        , RFC_CONTENT_DISPOSITION       , KCRQ_COLON },
  { "Content-ID"                 , RFC_CONTENT_ID                , KCRQ_COLON },
  { "Content-Length"             , RFC_CONTENT_LENGTH            , KCRQ_COLON },
  { "Content-Transfer-Encoding"  , RFC_CONTENT_TRANSFER_ENCODING , KCRQ_COLON },
  { "Content-Type"               , RFC_CONTENT_TYPE              , KCRQ_COLON },
  { "Control"                    , RFC_CONTROL                   , KCRQ_COLON },
  { "Date"                       , RFC_DATE                      , KCRQ_COLON },
  { "Delivered-To"               , RFC_DELIVERED_TO              , KCRQ_COLON },
  { "Delivery-Date"              , RFC_DELIVERY_DATE             , KCRQ_COLON },
  { "Distribution"               , RFC_DISTRIBUTION              , KCRQ_COLON },
  { "Encrypted"                  , RFC_ENCRYPTED                 , KCRQ_COLON },
  { "Errors-To"                  , RFC_ERRORS_TO                 , KCRQ_COLON },
  { "Expires"                    , RFC_EXPIRES                   , KCRQ_COLON },
  { "Followup-To"                , RFC_FOLLOWUP_TO               , KCRQ_COLON },
  { "From"                       , RFC_FROM                      , KCRQ_COLON },
  { "From"                       , RFC_FROMX                     , KCRQ_NONE  },
  { "In-Reply-To"                , RFC_IN_REPLY_TO               , KCRQ_COLON },
  { "Keywords"                   , RFC_KEYWORDS                  , KCRQ_COLON },
  { "Lines"                      , RFC_LINES                     , KCRQ_COLON },
  { "Message-ID"                 , RFC_MESSAGE_ID                , KCRQ_COLON },
  { "Mailing-List"               , RFC_MAILING_LIST              , KCRQ_COLON },
  { "MIME-Version"               , RFC_MIME_VERSION              , KCRQ_COLON },
  { "Newsgroups"                 , RFC_NEWSGROUPS                , KCRQ_COLON },
  { "News-Software"              , RFC_NEWS_SOFTWARE             , KCRQ_COLON },
  { "NNTP-Posting-Host"          , RFC_NNTP_POSTING_HOST         , KCRQ_COLON },
  { "NNTP-Posting-User"          , RFC_NNTP_POSTING_USER         , KCRQ_COLON },
  { "Old-Date"                   , RFC_OLD_DATE                  , KCRQ_COLON },
  { "Organization"               , RFC_ORGANIZATION              , KCRQ_COLON },
  { "Originator"                 , RFC_ORIGINATOR                , KCRQ_COLON },
  { "Path"                       , RFC_PATH                      , KCRQ_COLON },
  { "Precedence"                 , RFC_PRECEDENCE                , KCRQ_COLON },
  { "Priority"                   , RFC_PRIORITY                  , KCRQ_COLON },
  { "Received"                   , RFC_RECEIVED                  , KCRQ_COLON },
  { "References"                 , RFC_REFERENCES                , KCRQ_COLON },
  { "Reply-To"                   , RFC_REPLY_TO                  , KCRQ_COLON },
  { "Return-Path"                , RFC_RETURN_PATH               , KCRQ_COLON },
  { "Return-Receipt-To"          , RFC_RETURN_RECEIPT_TO         , KCRQ_COLON },
  { "See-Also"                   , RFC_SEE_ALSO                  , KCRQ_COLON },
  { "Sender"                     , RFC_SENDER                    , KCRQ_COLON },
  { "Status"                     , RFC_STATUS                    , KCRQ_COLON },
  { "Subject"                    , RFC_SUBJECT                   , KCRQ_COLON },
  { "Summary"                    , RFC_SUMMARY                   , KCRQ_COLON },
  { "Supersedes"                 , RFC_SUPERSEDES                , KCRQ_COLON },
  { "To"                         , RFC_TO                        , KCRQ_COLON },
  { "Version"                    , RFC_VERSION                   , KCRQ_COLON },
  { "Xref"                       , RFC_XREF                      , KCRQ_COLON },
  { "X-Charset"                  , RFC_X_CHARSET                 , KCRQ_COLON },
  { "X-Char-Esc"                 , RFC_X_CHAR_ESC                , KCRQ_COLON },
  { "X-FTN-To"                   , RFC_X_FTN_TO                  , KCRQ_COLON },
  { "X-Mailreader"               , RFC_X_MAILREADER              , KCRQ_COLON },
  { "X-Newsreader"               , RFC_X_NEWSREADER              , KCRQ_COLON },
  { "X-To"                       , RFC_X_TO                      , KCRQ_COLON },
  { "#!"                         , RFC_RNEWS                     , KCRQ_NONE  },
  { ""                           , RFC_ZZZZ                      , KCRQ_NONE  },
};


char* mime_header_decode(char* decoded, const char* encoded) {

  char dbuf[200], cbuf[100], ebuf[50], tbuf[200];
  char* dptr = decoded;
  const char* eptr = encoded;
  while(*eptr) {
    if(*eptr == '=') {
      const char* mptr = mime_crack_encoded_word(eptr, cbuf, ebuf, tbuf);
      if(mptr) { // and strieql(cbuf, "ISO-8859-1")) {
        bool okay = false;
        strchg(tbuf, '_', ' ');
        if(strieql(ebuf, "Q")) {
          quoted_printable_engine qb;
          qb.decode(dbuf, tbuf);
          dptr = stpcpy(dptr, dbuf);
          okay = true;
        }
        else if(strieql(ebuf, "B")) {
          base64_engine b64;
          b64.decode(dbuf, tbuf);
          dptr = stpcpy(dptr, dbuf);
          okay = true;
        }
        if(okay) {
          eptr = mptr;
          mptr = strskip_lwsp(mptr);
          if(mime_crack_encoded_word(mptr, cbuf, ebuf, tbuf))
            eptr = mptr;
          continue;
        }
      }
    }
    *dptr++ = *eptr++;
  }
  *dptr = NUL;

  return decoded;
}


//  ------------------------------------------------------------------

char* strxmimecpy(char* dest, const char* source, int level, int size) {

  char buf[500];
  char buf2[500];

  strxcpy(buf, source, sizeof(buf));
  mime_header_decode(buf2, buf);
  XlatStr(buf, buf2, level, CharTable);
  strxcpy(dest, buf, size);
  return dest;
}


//  ------------------------------------------------------------------

void KludgeAREA(GMsg* msg, char* echoid) {

  Area* ap = AL.AreaEchoToPtr(echoid);
  if(ap)
    msg->areakludgeid = ap->echoid();
}


//  ------------------------------------------------------------------

void KludgeINTL(GMsg* msg, char* ptr) {

  char buf1[201], buf2[201];
  word fmpt = msg->orig.point;
  word topt = msg->dest.point;
  sscanf(ptr, "%s %s", buf1, buf2);
  msg->dest.set(buf1);
  msg->orig.set(buf2);
  msg->orig.point = fmpt;
  msg->dest.point = topt;
}


//  ------------------------------------------------------------------

void KludgeFMPT(GMsg* msg, char* ptr) {

  msg->orig.point = atow(ptr);
}


//  ------------------------------------------------------------------

void KludgeTOPT(GMsg* msg, char* ptr) {

  msg->dest.point = atow(ptr);
}


//  ------------------------------------------------------------------

void KludgeMSGID(GMsg* msg, char* ptr) {

  strxcpy(msg->msgids, ptr, sizeof(msg->msgids));
  msg->msgid.reset(msg->msgids, msg->odom);
}


//  ------------------------------------------------------------------

void KludgeREPLY(GMsg* msg, char* ptr) {

  strxcpy(msg->replys, ptr, sizeof(msg->replys));
}


//  ------------------------------------------------------------------

void KludgeDOMAIN(GMsg* msg, char* ptr) {

  char buf1[201], buf2[201];
  sscanf(ptr, "%s %s %s %s", msg->ddom, buf1, msg->odom, buf2);
  msg->dest.reset(buf1);
  msg->orig.reset(buf2);
}


//  ------------------------------------------------------------------

void KludgeFLAGS(GMsg* msg, char* ptr) {

  GetAttribstr(&msg->attr, ptr);
}


//  ------------------------------------------------------------------

void KludgeMSGTO(GMsg* msg, char* ptr) {

  msg->dest.reset(ptr, msg->ddom);
}


//  ------------------------------------------------------------------

void KludgePID(GMsg* msg, char* ptr) {

  strxcpy(msg->pid, ptr, sizeof(msg->pid));

  // Recognize another GoldED msg
  if(striinc(__gver_name__, ptr) or striinc(__gver_shortname__, ptr))
    goldmark = GOLDMARK;
}


//  ------------------------------------------------------------------

void KludgeREPLYADDR(GMsg* msg, char* ptr) {

  strcpy(msg->iaddr, ptr);
}


//  ------------------------------------------------------------------

void KludgeREPLYTO(GMsg* msg, char* ptr) {

  strcpy(msg->igate, ptr);
}


//  ------------------------------------------------------------------

void KludgeFROM(GMsg* msg, char* ptr) {

  INam fromname;
  IAdr fromaddr;
  strxmimecpy(msg->ifrom, ptr, msg->charsetlevel, sizeof(INam));
  ParseInternetAddr(msg->ifrom, fromname, fromaddr);
  if(*fromaddr)
    strcpy(msg->iorig, fromaddr);
  if(*fromname)
    strxcpy(msg->realby, fromname, sizeof(msg->realby));
}


//  ------------------------------------------------------------------

void KludgeTO(GMsg* msg, char* ptr) {

  INam _toname;
  IAdr _toaddr;
  char* buf = throw_strdup(ptr);
  strxcpy(msg->ito, buf, sizeof(msg->ito));
  ParseInternetAddr(buf, _toname, _toaddr);
  throw_free(buf);
  if(*_toaddr)
    strcpy(msg->idest, _toaddr);
  if(*_toname)
    strxcpy(msg->realto, _toname, sizeof(msg->realto));
}


//  ------------------------------------------------------------------

void KludgeBCC(GMsg* msg, char* ptr) {

  char* buf = (char*)throw_malloc(strlen(msg->ibcc) + strlen(ptr) + 3);
  strcpy(buf, msg->ibcc);
  strcat(buf, *buf ? ", " : "");
  strcat(buf, ptr);
  strxcpy(msg->ibcc, buf, sizeof(msg->ibcc));
  throw_free(buf);
}


//  ------------------------------------------------------------------

void KludgeCC(GMsg* msg, char* ptr) {

  char* buf = (char*)throw_malloc(strlen(msg->icc) + strlen(ptr) + 3);
  strcpy(buf, msg->icc);
  strcat(buf, *buf ? ", " : "");
  strcat(buf, ptr);
  strxcpy(msg->icc, buf, sizeof(msg->icc));
  throw_free(buf);
}


//  ------------------------------------------------------------------

void KludgeREPLY_TO(GMsg* msg, char* ptr) {

  INam _rtname;
  IAdr _rtaddr;
  ParseInternetAddr(ptr, _rtname, _rtaddr);
  if(*_rtaddr)
    strcpy(msg->ireplyto, _rtaddr);
}


//  ------------------------------------------------------------------

void KludgeSUBJECT(GMsg* msg, char* ptr) {

  char* buf = throw_strdup(ptr);
  if(not msg->attr.att())
    strxcpy(msg->re, buf, sizeof(msg->re));
  throw_free(buf);
}


//  ------------------------------------------------------------------

void KludgeDATE(GMsg* msg, char* ptr) {

  // RFC822 Date: BNF
  //
  // date-time   =  [ day "," ] date time        ; dd mm yy
  //                                             ;  hh:mm:ss zzz
  //
  // day         =  "Mon"  / "Tue" /  "Wed"  / "Thu"
  //             /  "Fri"  / "Sat" /  "Sun"
  //
  // date        =  1*2DIGIT month 2DIGIT        ; day month year
  //                                             ;  e.g. 20 Jun 82
  //
  // month       =  "Jan"  /  "Feb" /  "Mar"  /  "Apr"
  //             /  "May"  /  "Jun" /  "Jul"  /  "Aug"
  //             /  "Sep"  /  "Oct" /  "Nov"  /  "Dec"
  //
  // time        =  hour zone                    ; ANSI and Military
  //
  // hour        =  2DIGIT ":" 2DIGIT [":" 2DIGIT]
  //                                             ; 00:00:00 - 23:59:59
  //
  // zone        =  "UT"  / "GMT"                ; Universal Time
  //                                             ; North American : UT
  //             /  "EST" / "EDT"                ;  Eastern:  - 5/ - 4
  //             /  "CST" / "CDT"                ;  Central:  - 6/ - 5
  //             /  "MST" / "MDT"                ;  Mountain: - 7/ - 6
  //             /  "PST" / "PDT"                ;  Pacific:  - 8/ - 7
  //             /  1ALPHA                       ; Military: Z = UT;
  //                                             ;  A:-1; (J not used)
  //                                             ;  M:-12; N:+1; Y:+12
  //             / ( ("+" / "-") 4DIGIT )        ; Local differential
  //                                             ;  hours+min. (HHMM)

  bool date_ok = false;
  int year=0, month=0, day=0;
  int hour=0, minute=0, second=0;

  ptr = strskip_wht(ptr);
  if(not isdigit(*ptr)) {
    // Skip past weekday string
    ptr = strskip_wht(strskip_txt(ptr));
  }
  if(*ptr) {
    if(isdigit(*ptr)) {
      day = atoi(ptr);
      ptr = strskip_wht(strskip_txt(ptr));
      if(isalpha(*ptr)) {
        month = str2mon(ptr);
        if(month) {
          ptr = strskip_wht(strskip_txt(ptr));
          if(isdigit(*ptr)) {
            year = atoi(ptr);
            ptr = strskip_wht(strskip_txt(ptr));
            if(isdigit(*ptr)) {
              hour = atoi(ptr);
              ptr = strskip_digits(ptr);
              if(*ptr and isdigit(ptr[1])) {
                minute = atoi(++ptr);
                date_ok = true;
                // The seconds part is optional
                ptr = strskip_digits(ptr);
                if(*ptr and isdigit(ptr[1])) {
                  second = atoi(++ptr);
                }
                // The timezone field is ignored for now ...
              }
            }
          }
        }
      }
    }
  }

  if(date_ok) {
    Tm t;
    t.tm_year  = (year < 80) ? (year+100) : year;
    t.tm_mon   = month - 1;
    t.tm_mday  = day;
    t.tm_hour  = hour;
    t.tm_min   = minute;
    t.tm_sec   = second;
    t.tm_isdst = 0;
    msg->written = gmktime(&t);
  }
}


//  ------------------------------------------------------------------

void KludgeMESSAGE_ID(GMsg* msg, char* ptr) {

  char buf[201];
  throw_free(msg->messageid);
  msg->messageid = throw_strdup(ptr);
  CvtMessageIDtoMSGID(ptr, buf, AA->echoid(), "MSGID");
  strcpy(msg->msgids, buf+8);
}


//  ------------------------------------------------------------------

void KludgeREFERENCES(GMsg* msg, char* ptr) {

  throw_free(msg->references);
  msg->references = throw_strdup(ptr);
}


//  ------------------------------------------------------------------

void KludgeIN_REPLY_TO(GMsg* msg, char* ptr) {

  throw_free(msg->inreplyto);
  msg->inreplyto = throw_strdup(ptr);
}


//  ------------------------------------------------------------------

void KludgeGIF(GMsg* msg, char* ptr) {

  strxcpy(msg->gif, ptr, 9);
}


//  ------------------------------------------------------------------

void KludgeORGANIZATION(GMsg* msg, char* ptr) {

  strxcpy(msg->organization, ptr, sizeof(msg->organization));
}


//  ------------------------------------------------------------------

void KludgeX_FTN_TO(GMsg* msg, char* ptr) {

  strxcpy(msg->realto, ptr, sizeof(msg->realto));
}


//  ------------------------------------------------------------------

void _fast KludgeFWDFROM(GMsg* msg, char* ptr) {

  strxcpy(msg->fwdfrom, ptr, sizeof(msg->fwdfrom));
}


//  ------------------------------------------------------------------

void _fast KludgeFWDORIG(GMsg* msg, char* ptr) {

  msg->fwdorig.reset(ptr);
}


//  ------------------------------------------------------------------

void _fast KludgeFWDTO(GMsg* msg, char* ptr) {

  strxcpy(msg->fwdto, ptr, sizeof(msg->fwdto));
}


//  ------------------------------------------------------------------

void _fast KludgeFWDDEST(GMsg* msg, char* ptr) {

  msg->fwddest.reset(ptr);
}


//  ------------------------------------------------------------------

void _fast KludgeFWDSUBJ(GMsg* msg, char* ptr) {

  strxcpy(msg->fwdsubj, ptr, sizeof(msg->fwdsubj));
}


//  ------------------------------------------------------------------

void _fast KludgeFWDAREA(GMsg* msg, char* ptr) {

  strxcpy(msg->fwdarea, ptr, sizeof(msg->fwdarea));
}


//  ------------------------------------------------------------------

void _fast KludgeFWDMSGID(GMsg* msg, char* ptr) {

  strxcpy(msg->fwdmsgid, ptr, sizeof(msg->fwdmsgid));
}


//  ------------------------------------------------------------------

char* UnwrapLine(Line* line, char* ptr, int addspace) {

  if(line->type & GLINE_WRAP) {
    uint len = strlen(ptr);
    char* uptr = throw_strdup(ptr);
    while(line AND (line->type & GLINE_WRAP)) {
      if(line->next) {
        uint nextlen = strlen(line->next->text);
        uptr = (char*)throw_realloc(uptr, len+nextlen+2);
        if(addspace and len and (uptr[len-1] != ' ')) {
          strcat(uptr, " ");
          len++;
        }
        strcpy(uptr+len, line->next->text);
        len += nextlen;
      }
      line = line->next;
    }
    return uptr;
  }

  return NULL;
}


//  ------------------------------------------------------------------

int HandleKludges(GMsg* msg, Line* line, int kludgenum, char* ptr, int getvalue) {

  switch(kludgenum) {

    case FTS_AREA:
      line->kludge = GKLUD_AREA;
      if(getvalue)
        KludgeAREA(msg, ptr);
      return TRUE;

    case FTS_INTL:
      line->kludge = GKLUD_INTL;
      if(getvalue)
        KludgeINTL(msg, ptr);
      return TRUE;

    case FTS_FMPT:
      line->kludge = GKLUD_FMPT;
      if(getvalue)
        KludgeFMPT(msg, ptr);
      return TRUE;

    case FTS_TOPT:
      line->kludge = GKLUD_TOPT;
      if(getvalue)
        KludgeTOPT(msg, ptr);
      return TRUE;

    case FTS_MSGID:
      line->kludge = GKLUD_MSGID;
      if(getvalue) {
        char* tmp = UnwrapLine(line, ptr, TRUE);
        KludgeMSGID(msg, tmp ? tmp : ptr);
        if(tmp)
          throw_free(tmp);
      }
      return TRUE;

    case FTS_REPLY:
      line->kludge = GKLUD_REPLY;
      if(getvalue) {
        char* tmp = UnwrapLine(line, ptr, TRUE);
        KludgeREPLY(msg, tmp ? tmp : ptr);
        if(tmp)
          throw_free(tmp);
      }
      return TRUE;

    case FTS_SEENBY:
      line->kludge = GKLUD_SEENBY;
      return TRUE;

    case FTS_PATH:
      line->kludge = GKLUD_PATH;
      return TRUE;

    ////////////////////////////////////////////////////////////

    case FSC_CHARSET:
      line->kludge = GKLUD_CHARSET;
      return TRUE;

    case FSC_CHRS:
      line->kludge = GKLUD_CHARSET;
      return TRUE;

    case FSC_DOMAIN:
      //line->kludge = GKLUD_DOMAIN;
      if(getvalue)
        KludgeDOMAIN(msg, ptr);
      return TRUE;

    case FSC_ENCLFILE:
      if(getvalue)
        strxcpy(msg->re, ptr, sizeof(msg->re));
      return TRUE;

    case FSC_FLAGS:
      line->kludge = GKLUD_FLAGS;
      if(getvalue)
        KludgeFLAGS(msg, ptr);
      return TRUE;

    case FSC_I51:
      line->kludge = GKLUD_CHARSET;
      msg->i51 = true;
      return TRUE;

    case FSC_MSGTO:
      //line->kludge = GKLUD_MSGTO;
      if(getvalue)
        KludgeMSGTO(msg, ptr);
      return TRUE;

    case FSC_PID:
      line->kludge = GKLUD_PID;
      if(getvalue)
        KludgePID(msg, ptr);
      return TRUE;

    case FSC_REPLYADDR:
      line->kludge = GKLUD_REPLYADDR;
      if(getvalue)
        KludgeREPLYADDR(msg, ptr);
      return TRUE;

    case FSC_REPLYTO:
      line->kludge = GKLUD_REPLYTO;
      if(getvalue)
        KludgeREPLYTO(msg, ptr);
      return TRUE;

    case FSC_FWDFROM:
      line->kludge = GKLUD_FWD;
      if(getvalue)
        KludgeFWDFROM(msg, ptr);
      return TRUE;

    case FSC_FWDORIG:
      line->kludge = GKLUD_FWD;
      if(getvalue)
        KludgeFWDORIG(msg, ptr);
      return TRUE;

    case FSC_FWDTO:
      line->kludge = GKLUD_FWD;
      if(getvalue)
        KludgeFWDTO(msg, ptr);
      return TRUE;

    case FSC_FWDDEST:
      line->kludge = GKLUD_FWD;
      if(getvalue)
        KludgeFWDDEST(msg, ptr);
      return TRUE;

    case FSC_FWDSUBJ:
      line->kludge = GKLUD_FWD;
      if(getvalue)
        KludgeFWDSUBJ(msg, ptr);
      return TRUE;

    case FSC_FWDAREA:
      line->kludge = GKLUD_FWD;
      if(getvalue)
        KludgeFWDAREA(msg, ptr);
      return TRUE;

    case FSC_FWDMSGID:
      line->kludge = GKLUD_FWD;
      if(getvalue)
        KludgeFWDMSGID(msg, ptr);
      return TRUE;

    case FSC_CHRC:
    case FSC_EID:
    case FSC_ENC:
    case FSC_PTH:
    case FSC_SPLIT:
    case FSC_SPTH:
    case FSC_TID:
      // Recognized but not processed
      return TRUE;

    ////////////////////////////////////////////////////////////

    case XXX_DESTADDR:
      //line->kludge = GKLUD_DESTADDR;
      if(getvalue)
        KludgeMSGTO(msg, ptr);
      return TRUE;

    case XXX_GIF:
      line->kludge = GKLUD_GIF;
      if(getvalue)
        KludgeGIF(msg, ptr);
      return TRUE;

    case XXX_ACUPDATE:
    case XXX_CODEPAGE:
    case XXX_ENCRYPTION:
    case XXX_EOT:
    case XXX_GATECHK:
    case XXX_GID:
    case XXX_GMD:
    case XXX_GROUP:
    case XXX_MOOD:
    case XXX_MSGSEQ:
    case XXX_NOTE:
    case XXX_ORIGID:
    case XXX_ORIGINAL:
    case XXX_ORIGREF:
    case XXX_RECD:
    case XXX_RFC:
    case XXX_RFD:
    case XXX_RID:
    case XXX_SN:
    case XXX_SOT:
    case XXX_TCL1:
    case XXX_TCL2:
    case XXX_TZUTCINFO:
    case XXX_TZUTC:
    case XXX_TZ:
    case XXX_VIA:
    case XXX_XID:
      // Recognized but not processed
      return TRUE;
  }

  return FALSE;
}


//  ------------------------------------------------------------------

int HandleRFCs(GMsg* msg, Line* line, int kludgenum, char* ptr, int getvalue) {

  switch(kludgenum) {

    case RFC_FROM:
      line->kludge = GKLUD_RFC;
      if(getvalue) {
        char* tmp = UnwrapLine(line, ptr, TRUE);
        KludgeFROM(msg, tmp ? tmp : ptr);
        if(tmp)
          throw_free(tmp);
      }
      return TRUE;

    case RFC_TO:
    case RFC_X_TO:
      line->kludge = GKLUD_RFC;
      if(getvalue) {
        char* tmp = UnwrapLine(line, ptr, TRUE);
        KludgeTO(msg, tmp ? tmp : ptr);
        if(tmp)
          throw_free(tmp);
      }
      return TRUE;

    case RFC_BCC:
      line->kludge = GKLUD_RFC;
      if(getvalue) {
        char* tmp = UnwrapLine(line, ptr, TRUE);
        KludgeBCC(msg, tmp ? tmp : ptr);
        if(tmp)
          throw_free(tmp);
      }
      return TRUE;

    case RFC_CC:
      line->kludge = GKLUD_RFC;
      if(getvalue) {
        char* tmp = UnwrapLine(line, ptr, TRUE);
        KludgeCC(msg, tmp ? tmp : ptr);
        if(tmp)
          throw_free(tmp);
      }
      return TRUE;

    case RFC_DATE:
      line->kludge = GKLUD_RFC;
      if(getvalue)
        KludgeDATE(msg, ptr);
      return TRUE;

    case RFC_SUBJECT:
      line->kludge = GKLUD_RFC;
      if(getvalue) {
        char* tmp = UnwrapLine(line, ptr, TRUE);
        KludgeSUBJECT(msg, tmp ? tmp : ptr);
        if(tmp)
          throw_free(tmp);
      }
      return TRUE;

    case RFC_REPLY_TO:
      line->kludge = GKLUD_RFC;
      if(1 or getvalue) {
        char* tmp = UnwrapLine(line, ptr, TRUE);
        KludgeREPLY_TO(msg, tmp ? tmp : ptr);
        if(tmp)
          throw_free(tmp);
      }
      return TRUE;

    case RFC_MESSAGE_ID:
      line->kludge = GKLUD_RFC;
      if(getvalue) {
        char* tmp = UnwrapLine(line, ptr, FALSE);
        KludgeMESSAGE_ID(msg, tmp ? tmp : ptr);
        if(tmp)
          throw_free(tmp);
      }
      return TRUE;

    case RFC_REFERENCES:
      line->kludge = GKLUD_RFC;
      if(getvalue) {
        char* tmp = UnwrapLine(line, ptr, TRUE);
        KludgeREFERENCES(msg, tmp ? tmp : ptr);
        if(tmp)
          throw_free(tmp);
      }
      return TRUE;

    case RFC_IN_REPLY_TO:
      line->kludge = GKLUD_RFC;
      if(getvalue) {
        char* tmp = UnwrapLine(line, ptr, TRUE);
        KludgeIN_REPLY_TO(msg, tmp ? tmp : ptr);
        if(tmp)
          throw_free(tmp);
      }
      return TRUE;

    case RFC_ORGANIZATION:
      line->kludge = GKLUD_RFC;
      if(getvalue)
        KludgeORGANIZATION(msg, ptr);
      return TRUE;

    case RFC_X_FTN_TO:
      line->kludge = GKLUD_RFC;
      if(getvalue)
        KludgeX_FTN_TO(msg, ptr);
      return TRUE;

    case RFC_X_MAILREADER:
      line->kludge = GKLUD_RFC;
      if(getvalue)
        KludgePID(msg, ptr);
      return TRUE;

    case RFC_X_NEWSREADER:
      line->kludge = GKLUD_RFC;
      if(getvalue)
        KludgePID(msg, ptr);
      return TRUE;

    case RFC_NEWSGROUPS:
    case RFC_SENDER:
    case RFC_CONTENT_TRANSFER_ENCODING:
    case RFC_CONTENT_TYPE:
    case RFC_MIME_VERSION:
    case RFC_X_CHARSET:
    case RFC_X_CHAR_ESC:
      // Mark RFC's that we add ourselves
      line->kludge = GKLUD_RFC;
      return TRUE;

    case RFC_ALSO_CONTROL:
    case RFC_APPARENTLY_TO:
    case RFC_APPROVED:
    case RFC_ARTICLE_NAMES:
    case RFC_ARTICLE_UPDATES:
    case RFC_COMMENT:
    case RFC_COMMENTS:
    case RFC_CONTENT_DESCRIPTION:
    case RFC_CONTENT_DISPOSITION:
    case RFC_CONTENT_ID:
    case RFC_CONTENT_LENGTH:
    case RFC_CONTROL:
    case RFC_DELIVERED_TO:
    case RFC_DELIVERY_DATE:
    case RFC_DISTRIBUTION:
    case RFC_ENCRYPTED:
    case RFC_ERRORS_TO:
    case RFC_EXPIRES:
    case RFC_FOLLOWUP_TO:
    case RFC_FROMX:
    case RFC_KEYWORDS:
    case RFC_LINES:
    case RFC_MAILING_LIST:
    case RFC_NEWS_SOFTWARE:
    case RFC_NNTP_POSTING_HOST:
    case RFC_NNTP_POSTING_USER:
    case RFC_OLD_DATE:
    case RFC_ORIGINATOR:
    case RFC_PATH:
    case RFC_PRECEDENCE:
    case RFC_PRIORITY:
    case RFC_RECEIVED:
    case RFC_RETURN_PATH:
    case RFC_RETURN_RECEIPT_TO:
    case RFC_SEE_ALSO:
    case RFC_STATUS:
    case RFC_SUMMARY:
    case RFC_SUPERSEDES:
    case RFC_VERSION:
    case RFC_XREF:
    case RFC_RNEWS:
      // Recognized but not processed
      return TRUE;
  }

  return FALSE;
}


//  ------------------------------------------------------------------

int ScanCtrlList(Kludges* k, char* kludge, char endchar) {

  while(*k->key) {
    if((k->req & KCRQ_CASE) ? streql(kludge, k->key) : strieql(kludge, k->key)) {
      if(k->req & KCRQ_COLON) {
        if(endchar == ':')
          return k->num;
      }
      else {
        return k->num;
      }
    }
    k++;
  }

  return 0;
}


//  ------------------------------------------------------------------

int ScanLine(GMsg* msg, Line* line, char* ptr, int getvalue, int mask) {

  // Kludge number
  int kludgenum = 0;

  // Pointer to kludge id
  char* kludge = ptr;

  // Skip past "RFC" string, if any
  if(strnieql(kludge, "RFC", 3) AND (kludge[3] != ':')) {
    kludge += 3;
    if(NOT isalpha(*kludge))
      kludge++;
  }

  // Keep copy of id terminating char
  while((*ptr != ' ') AND (*ptr != ':') AND *ptr)
    ptr++;
  char endchar = *ptr;
  *ptr = NUL;

  // Search for it in the known kludges list
  if(*kludge) {
    Kludges* k;
    while(1) {
      if(mask & MASK_FTS) {
        k = fts_list;
        kludgenum = ScanCtrlList(k, kludge, endchar);
        if(kludgenum)
          break;
      }
      if(mask & MASK_FSC) {
        k = fsc_list;
        kludgenum = ScanCtrlList(k, kludge, endchar);
        if(kludgenum)
          break;
      }
      if(mask & MASK_RFC) {
        k = rfc_list;
        kludgenum = ScanCtrlList(k, kludge, endchar);
        if(kludgenum)
          break;
      }
      if(mask & MASK_XXX) {
        k = xxx_list;
        kludgenum = ScanCtrlList(k, kludge, endchar);
        if(kludgenum)
          break;
      }
      break;
    }
  }

  // Restore terminating char
  *ptr = endchar;
  if(*ptr != ' ')
    ptr++;

  if(kludgenum) {
    fastSkipWht(ptr);
    line->type |= GLINE_KLUD;
    if(kludgenum & (MASK_FTS|MASK_FSC|MASK_XXX))
      HandleKludges(msg, line, kludgenum, ptr, getvalue);
    else if(kludgenum & MASK_RFC)
      HandleRFCs(msg, line, kludgenum, ptr, getvalue);
  }
  else {
    for(int k=0; k<CFG->kludges; k++) {
      if(strnieql(kludge, CFG->kludge[k], strlen(CFG->kludge[k]))) {
        line->type |= GLINE_KLUD;
        kludgenum = USERDEFINED;
        break;
      }
    }
    if(NOT(line->type & GLINE_KLUD))
      if((strnieql(kludge, "X-", 2) OR strnieql(kludge, "Resent-", 7)) AND (endchar == ':') AND (mask & MASK_RFC)) {
        line->type |= GLINE_KLUD;
        kludgenum = RFC_X;
      }
    if(NOT(line->type & GLINE_KLUD))
      kludgenum = (endchar == ':') ? HEADERLINE : BODYLINE;
  }

  return kludgenum;
}


//  ------------------------------------------------------------------

void ScanKludges(GMsg* msg, int getvalue) {

  char* ptr;
  int tearlineno = INT_MAX;
  int originlineno = INT_MAX;
  int gotorig=NO, gottear=NO, gottag=NO;

  // Scan for kludge-, tear- and originlines

  if(getvalue)
    goldmark = ' ';   // Reset the "recognizer" code

  int lineno = 0;
  Line* line = LastLine(msg->lin);

  do {
    ptr = line->text;
    if(*ptr == CTRL_A) {

      // Set kludge/hidden color
      line->color = C_READK;

      int kludgenum = ScanLine(msg, line, ptr+1, getvalue, MASK_ALL);
      if((kludgenum == BODYLINE) OR (kludgenum == HEADERLINE))
        line->type |= GLINE_HIDD;
    }
    else {

      if(strneql(ptr, "AREA:", 5) AND ((line->prev == NULL) OR line->prev->text[0] == CTRL_A)) {
        line->color = C_READK;
        line->kludge = GKLUD_AREA;
        line->type |= GLINE_KLUD;
        char* areakludgeptr = ptr+5;   // Extract echoid from kludge
        fastSkipWht(areakludgeptr);
        Area* ap = AL.AreaEchoToPtr(areakludgeptr);
        if(ap)
          msg->areakludgeid = ap->echoid();
      }

      if(NOT(line->type & GLINE_KLUD)) {

        // Check if it is a tagline
        if(strneql("...", ptr, 3) OR strneql("___", ptr, 3)) {

          // It's only a tagline if it's just above the tearline or origin.
          // Or if it's the last line in the message.
          if(!lineno OR ((lineno-1) == tearlineno) OR ((lineno-1) == originlineno)) {
            if(NOT gottag) {
              gottag = YES;
              line->type |= GLINE_TAGL;
              line->color = C_READG;
              if(AA->Taglinesupport())
                strbtrim(strcpy(msg->tagline, ptr+3));
            }
          }
        }

        // Check if it's a tearline
        if(strneql("---", ptr, 3) AND (ptr[3] == ' ' OR ptr[3] == NUL)) {

          // Found Tearline
          if(NOT gottear) {
            gottear = YES;
            tearlineno = lineno;
            line->type |= GLINE_TEAR;
            line->color = C_READT;
            strbtrim(strcpy(msg->tearline, ptr+3));
          }
          if(getvalue) {
            char* tearid[] = {
              "GoldED",
              " GED ",
              " GED2 ",
              " GED3 ",
              " GED386 ",
              " GED/2 ",
              " GED/386 ",
              " GEDP16 ",
              " GEDP32 ",
              " GEDW32 ",
              " GEDLNX ",
              " GED/W32 ",
              NULL
            };
            int n = 0;
            while(tearid[n]) {
              if(striinc(tearid[n], ptr)) {
                goldmark = GOLDMARK;          // Recognize another GoldED msg
                break;
              }
              n++;
            }
          }
        }

        // Check if it's an originline
        if(strneql(" * Origin: ", ptr, 11)) {

          // Found Origin line
          if(not gotorig and (not (gottear or gottag))) {
            gotorig = YES;
            originlineno = lineno;
            line->type |= GLINE_ORIG;
            line->color = C_READO;
            strcpy(msg->origin, line->text+11);
            if(line->next AND line->next->text) {  // Get the next line too
              if(*line->next->text != CTRL_A AND strncmp(line->next->text, "SEEN-BY:", 8)) {
                strcat(msg->origin, line->next->text);
                line->next->color = C_READO;
                line->next->type |= GLINE_ORIG;     // Mark next line as Origin too
              }
            }
          }
        }

        if(strneql(ptr, "SEEN-BY:", 8)) {
          line->kludge = GKLUD_SEENBY;
          line->color = C_READK;
          line->type |= GLINE_KLUD;
        }

        // Check if it's a signature indicator
        if(AA->isinternet()) {
          if(streql(ptr, "-- ")) {
            for(Line* q = line; (q != NULL); q = q->next) {
              if((q->type & (GLINE_KLUD|GLINE_ORIG|GLINE_TEAR)) == 0) {
                q->color = C_READS;
                q->type |= GLINE_SIGN;
                if(q != line)
                  q->type |= GLINE_HARD;
              }
            }
          }
        }
      }
    }
    if(line->type & GLINE_WRAP) {
      Line* linep = line;
      while(linep AND (linep->type & GLINE_WRAP)) {
        if(linep->next)
          linep->next->type |= (linep->type & (GLINE_HIDD|GLINE_KLUD));
        linep = linep->next;
      }
    }

    lineno++;

  } while((line = line->prev) != NULL);

  if(NOT gottag)
    *msg->tagline = NUL;
  if(NOT gottear)
    *msg->tearline = NUL;
  if(NOT gotorig)
    *msg->origin = NUL;

  if(getvalue) {
    // This is the old code (<= 3.00.Beta4)
    #if 0
    if(msg->msgid.net)
      msg->orig = msg->msgid;
    else if(AA->isecho() or (NOT(msg->orig.net))) {
      if((ptr = strrchr(msg->origin, '(' /*)*/ )) != NULL) {
        while(!isdigit(*ptr) AND *ptr)
          ptr++;
        msg->orig.reset(ptr);
      }
    }
    #else
    // This is the new code (experimental)
    // It looks for an Origin before taking MSGID 
    // Trust msg->orig if valid and we're in netmail area.
    // (msg->orig is already merged with INTL/FMPT/TOPT)

    if(not (AA->isnet() and msg->orig.valid())) { 
      if((ptr = strrchr(msg->origin, '(' /*)*/ )) != NULL) {
        while(!isdigit(*ptr) AND *ptr)
          ptr++;
        msg->orig.reset(ptr);
      }
      else if(msg->msgid.valid())
        msg->orig = msg->msgid;
    }
    #endif

    if(msg->orig.zone == 0)
      msg->orig.zone = AA->Aka().addr.zone;

    if(msg->dest.zone == 0)
      msg->dest.zone = msg->orig.zone;
  }
}


//  ------------------------------------------------------------------

char* XlatStr(char* dest, char* src, int level, Chs* chrtbl, int qpencoded, bool i51) {

  if(!chrtbl)
    return stpcpy(dest, src);

  register uint n;
  register int clen;
  register int translated;
  register char* tptr;
  register char* escp;
  register char* sptr = src;
  register char* dptr = dest;
  register char dochar;
  register ChsTab* chrs = chrtbl ? chrtbl->t : (ChsTab*)NULL;

  while(*sptr) {
    switch(*sptr) {
      case 0x02:
        if(i51 and I51Table) {
          for(n=0; n<I51Table->size; n++) {
            tptr = (char*)I51TP[n];
            if(*(sptr+1) == tptr[0]) {
              if(*(sptr+2) == tptr[1]) {
                escp = &tptr[2];
                if(*escp) {
                  *dptr++ = *escp++;
                  if(*escp) {
                    *dptr++ = *escp++;
                    if(*escp) {
                      *dptr++ = *escp;
                    }
                  }
                }
                sptr += 2;
                n = (uint)-1;
                break;
              }
            }
          }
          if(n != (uint)-1)   // I51 char not found, use fallback method
            sptr++;
        }
        sptr++;
        break;

      case 29:
        if(MNETable) {
          for(n=0; n<MNETable->size; n++) {
            tptr = (char*)MNETP[n];
            if(*(sptr+1) == tptr[0]) {
              if(*(sptr+2) == tptr[1]) {
                escp = &tptr[2];
                if(*escp) {
                  *dptr++ = *escp++;
                  if(*escp) {
                    *dptr++ = *escp++;
                    if(*escp) {
                      *dptr++ = *escp;
                    }
                  }
                }
                sptr += 2;
                n = (uint)-1;
                break;
              }
            }
          }
          if(n != (uint)-1)   // MNEMONIC char not found, use fallback method
            sptr++;
        }
        sptr++;
        break;

      case SOFTCR:
        translated = FALSE;
        if(CompTable and not CFG->switches.get(dispsoftcr)) {
          if(sptr > src) {
            if(NOT(isspace(*(sptr-1)) OR isspace(*(sptr+1)))) {
              for(n=0; n<CompTable->size; n++) {
                tptr = (char*)CompTP[n];
                if(*(sptr-1) == tptr[0]) {
                  if(*(sptr+1) == tptr[1]) {
                    escp = &tptr[2];
                    if(*escp) {
                      dptr--;
                      *dptr++ = *escp++;
                      if(*escp) {
                        *dptr++ = *escp++;
                        if(*escp) {
                          *dptr++ = *escp;
                          translated = TRUE;
                        }
                      }
                    }
                    sptr += 2;
                    break;
                  }
                }
              }
            }
          }
        }
        if(NOT translated)
          *dptr++ = *sptr++;
        break;

      case '=':
        if(qpencoded) {
          if(isxdigit(sptr[1]) AND isxdigit(sptr[2])) {
            dochar = (char)((xtoi(sptr[1]) << 4) | xtoi(sptr[2]));
            sptr += 3;
            goto chardo;
          }
        }
        goto defaultchardo;

      default:
      defaultchardo:
        dochar = *sptr++;
      chardo:
        if((level&3) and chrs) {   // Translate level 1 and 2
          tptr = (char*)chrs[(byte)dochar];
          clen = *tptr++;
          while(clen--)
            *dptr++ = *tptr++;
        }
        else {
          *dptr++ = dochar;
        }
    }
  }
  *dptr = NUL;
  return dptr;
}


//  ------------------------------------------------------------------

static int cmp_quotes(char* q1, char* q2) {

  q1--;
  q2--;

  do {
    do {
      q1++;
    } while(isspace(*q1) or (*q1 == SOFTCR and not CFG->switches.get(dispsoftcr)));
    do {
      q2++;
    } while(isspace(*q2) or (*q2 == SOFTCR and not CFG->switches.get(dispsoftcr)));
    if(*q2 == 0)
      return YES;
    if(*q1 != *q2)
      return NO;
  } while(*q1 AND *q2);

  return *q1 == *q2;
}


//  ------------------------------------------------------------------

void GMsg::TextToLines(int __line_width, bool header_recode) {

  line_width = __line_width;

  MakeLineIndex(this, line_width, header_recode);
}


//  ------------------------------------------------------------------

void MakeLineIndex(GMsg* msg, int margin, bool header_recode) {

  uint idx=0;
  register uint len;
  register int level=0;
  register uint n;
  register char ch, chln = 0, dochar;
  Line* line;
  Line* nextline=NULL;
  Line* prevline=NULL;
  char* bp;
  char* btmp=NULL;
  char* tptr;
  char* escp;
  char* bptr;
  char buf[256], qbuf[500], qbuf2[500], chsbuf[100];
  char* ptr;
  char* qptr;
  char* tmp=NULL;
  char* linetmp=NULL;
  uint qlen=0, qlen2=0;
  int wraps=0, para=0, reflow=NO, quoteflag=NO, chslev;
  int getvalue = NOT msg->attr.tou();
  int quotewraphard = AA->Quotewraphard();
  int qpencoded = strieql(AA->Xlatimport(), "LATIN1QP") ? TRUE : FALSE;
  int gotmime = FALSE;
  int firstemptyline = FALSE;

  *buf = *qbuf = *qbuf2 = NUL;

  if(margin < 0) {
    margin = -margin;
    quoteflag = YES;
  }

  // Free all previously allocated lines
  line = msg->lin;
  while(line) {
    nextline = line->next;
    throw_release(line->text);
    throw_xfree(line);
    line = nextline;
  }

  msg->lines = 0;
  msg->lin = NULL;

  if(AA->attr().hex()) {

    // Make a complete hexdump as a list of message lines
    w_info(LNG->GenHexdump);
    throw_release(msg->txt);
    line = AA->MakeDumpMsg(msg);
    if(line AND msg->txt) {
      line = AddLine(line, "");
      line = AddLine(line, LNG->Hexdumptext);
      line = AddLine(line, "");
      ptr = msg->txt;
      uint _size = strlen(msg->txt);
      for(idx=0; idx < _size; ptr+=16,idx+=16) {
        sprintf(buf, "%04X   ", idx+(AA->isfido()?190:0));
        HexDump16(buf+7, ptr, 16, HEX_DUMP2);
        line = AddLine(line, buf);
      }
    }

    w_info(NULL);
  }
  else {  // Convert the message text to a list of separately allocated lines

    char prev_ptr[3] = {"\xFF\xFF"};
    //char* prev_ptr = "\xFF\xFF";

    ptr = msg->txt;

    // Set default conversion table for area
    if(getvalue) {
      if(NOT strieql(AA->Xlatimport(), CFG->xlatlocalset)) {
        memset(msg->charset, 0, sizeof(msg->charset));
        strcpy(msg->charset, AA->Xlatimport());
        level = msg->charsetlevel = LoadCharset(msg->charset, CFG->xlatlocalset);
      }
    }

    line = msg->lin = (Line*)throw_xcalloc(1, sizeof(Line));
      
    if(CFG->switches.get(dispsoftcr)) {
      while((*ptr == LF) AND *ptr)
        ptr++;
    }
    else {
      while((*ptr == LF OR *ptr == SOFTCR) AND *ptr)
        ptr++;
    }

    if(*ptr == NUL) {
      throw_xrelease(msg->lin);
    }
    else {
      // Alloc space for one line
      linetmp = (char*)throw_calloc(1, margin+512);
      while(*ptr) {

        bptr = linetmp;
        bp = bptr;
        len = 0;

        // Link previous line to this one

        if(line->prev)
          line->prev->next = line;

        // Reflow quotes

        if(reflow) {
          len = qlen;
          qptr = qbuf;
          reflow = NO;
          // Insert previous quotestring
          for(n=0; n<qlen; n++) {
            *(++bp) = *qptr++;
          }
          if(CFG->switches.get(dispsoftcr)) {
            while((*ptr == LF) AND *ptr)
              ptr++;
          }
          else {
            while((*ptr == LF OR *ptr == SOFTCR) AND *ptr)
              ptr++;
          }
          line->type |= GLINE_QUOT | GLINE_HARD;
        }

        // Get type of line

        if(wraps == 0) {
          para = 0;
          if(*ptr == CTRL_A) {              // Found kludge/hidden line
            #if 1
            line->type |= GLINE_HARD;
            if(getvalue) {
              tptr = ptr;
              char* kludge = ptr + 1;
              if(strnieql(kludge, "RFC", 3)) {
                kludge += 3;
                if(NOT isalpha(*kludge))
                  kludge++;
              }
              while((*ptr != ' ') AND (*ptr != ':') AND *ptr)
                ptr++;
              char endchar = *ptr;
              *ptr = NUL;
              int kludgetype = -1;
              if(strieql(kludge, "I51"))
                kludgetype = FSC_I51;
              else if(strieql(kludge, "CHRS") OR strieql(kludge, "CHARSET"))
                kludgetype = FSC_CHARSET;
              else if(strieql(kludge, "Content-Type"))
                kludgetype = RFC_CONTENT_TYPE;
              else if(strieql(kludge, "Content-Transfer-Encoding"))
                kludgetype = RFC_CONTENT_TRANSFER_ENCODING;
              else if(strieql(kludge, "X-Charset"))
                kludgetype = RFC_X_CHARSET;
              else if(strieql(kludge, "X-Char-Esc"))
                kludgetype = RFC_X_CHAR_ESC;
              *ptr = endchar;
              if(*ptr != ' ')
                ptr++;
              fastSkipWht(ptr);
              char* keptr = strpbrk(ptr, "\r\n");
              if(keptr) {
                endchar = *keptr;
                *keptr = NUL;
              }
              if(kludgetype == FSC_I51) {
                msg->i51 = true;
                if(getvalue) {
                  // Convert FSC-0051.003 to FSC-0054.003
                  strcpy(chsbuf, "LATIN-1");
                  chslev = LoadCharset(chsbuf, CFG->xlatlocalset);
                  if(chslev) {
                    level = msg->charsetlevel = chslev;
                    strcpy(msg->charset, chsbuf);
                  }
                }
              }
              else if(kludgetype == FSC_CHARSET) {
                if(getvalue) {
                  *chsbuf = NUL;
                  qpencoded = striinc("LATIN1QP", ptr) ? TRUE : FALSE;
                  strxcpy(chsbuf, qpencoded ? "LATIN-1" : ptr, sizeof(chsbuf));
                  chslev = LoadCharset(chsbuf, CFG->xlatlocalset);
                  if(chslev) {
                    level = msg->charsetlevel = chslev;
                    strcpy(msg->charset, chsbuf);
                  }
                }
              }
              else if(kludgetype == RFC_CONTENT_TYPE) {
                if(getvalue) {
                  if(striinc("iso-8859-1", ptr)) {
                    strcpy(chsbuf, "LATIN-1");
                    chslev = LoadCharset(chsbuf, CFG->xlatlocalset);
                    if(chslev) {
                      level = msg->charsetlevel = chslev;
                      strcpy(msg->charset, chsbuf);
                    }
                    gotmime = TRUE;
                  }
                }
              }
              else if(kludgetype == RFC_CONTENT_TRANSFER_ENCODING) {
                if(getvalue) {
                  if(striinc("quoted-printable", ptr)) {
                    qpencoded = TRUE;
                    msg->charsetencoding |= GCHENC_QP;
                    if(striinc("LATIN-1", msg->charset)) {
                      strcpy(chsbuf, "LATIN1QP");
                      chslev = LoadCharset("LATIN-1", CFG->xlatlocalset);
                      if(chslev) {
                        level = msg->charsetlevel = chslev;
                        strcpy(msg->charset, chsbuf);
                      }
                    }
                  }
                }
              }
              else if(kludgetype == RFC_X_CHARSET) {
                if(getvalue) {
                  if(NOT gotmime) {
                    strcpy(chsbuf, (striinc("8859-1", ptr) OR striinc("Latin1", ptr)) ? "LATIN-1" : CFG->xlatlocalset);
                    chslev = LoadCharset(chsbuf, CFG->xlatlocalset);
                    if(chslev) {
                      level = msg->charsetlevel = chslev;
                      strcpy(msg->charset, chsbuf);
                    }
                  }
                }
              }
              else if(kludgetype == RFC_X_CHAR_ESC) {
                if(getvalue)
                  if(NOT gotmime)
                    msg->charsetencoding |= GCHENC_MNE;
              }
              if(keptr)
                *keptr = endchar;
              ptr = tptr;
            }
            #endif
          }
          else if(is_quote(ptr)) {
            para = GLINE_QUOT;
            line->type |= GLINE_QUOT | GLINE_HARD;
            GetQuotestr(ptr, qbuf, &qlen);
          }
        }

        // Get one line

        ch = 0;
        tmp = NULL;
        while(*ptr AND (len < (uint)margin)) {
          switch(*ptr) {
            case CR:
              do_cr:
              ptr++;
              ch = CR;
              if(CFG->switches.get(dispsoftcr)) {
                while((*ptr == LF) AND *ptr)
                  ptr++;
              }
              else {
                while((*ptr == LF OR *ptr == SOFTCR) AND *ptr)
                  ptr++;
              }
              if(wraps) {
                if(para != GLINE_QUOT) {
                  if(quoteflag) {
                    //while((*ptr == ' ') AND *ptr)
                      //ptr++;
                    if(prevline and prevline->text[0] == CTRL_A and prevline->type & GLINE_WRAP) {
                      wraps = 0;
                      break;
                    }
                    if(*ptr == CR OR is_quote(ptr) OR *ptr == CTRL_A) {
                      wraps = 0;
                      break;
                    }
                    if(((ptr[0] == ptr[1]) AND (ptr[0] == ptr[2])) OR strneql(ptr, " * Origin: ", 11)) {
                      wraps = 0;
                      break;
                    }
                    if(strneql(ptr, "SEEN-BY:", 8)) {
                      wraps = 0;
                      break;
                    }
                    if((ptr[0] == prev_ptr[0]) AND (ptr[1] == prev_ptr[1])) {
                      wraps = 0;
                      break;
                    }
                    char* lp = ptr;
                    while(*lp == ' ')
                      lp++;
                    if(*lp == CR) {
                      wraps = 0;
                      break;
                    }
                    else {
                      ptr = lp;
                    }
                    if(*ptr != ' ' AND *ptr != HT AND *bp != ' ') {
                      *(++bp) = ' ';
                      len++;
                    }
                    ch = 0;
                    continue;
                  }
                  else {
                    wraps=0;
                    break;
                  }
                }
                else {
                  GetQuotestr(ptr, qbuf2, &qlen2);
                  if(*ptr == CR) {
                    wraps = 0;
                    para = 0;
                    break;
                  }
                  else if(cmp_quotes(qbuf2, qbuf)) {
                    char* lp = ptr + qlen2;
                    if(is_quote(lp)) {
                      wraps = 0;
                      para = 0;
                      break;
                    }
                    else {
                      ptr = lp;
                      para = GLINE_QUOT;
                      if(*ptr != ' ' AND *bp != ' ') {
                        *(++bp) = ' ';
                        len++;
                      }
                      ch = 0;
                      continue;
                    }
                  }
                  else {
                    wraps = 0;
                    para = 0;
                    break;
                  }
                }
              }
              break;
            case 0x02:  // The I51 escape character
              if(msg->i51 and I51Table) {
                for(n=0; n<I51Table->size; n++) {
                  tptr = (char*)I51TP[n];
                  if(*(ptr+1) == tptr[0]) {
                    if(*(ptr+2) == tptr[1]) {
                      escp = &tptr[2];
                      if(*escp) {
                        *(++bp) = *escp++;
                        if(*escp) {
                          *(++bp) = *escp++;
                          len++;
                          if(*escp) {
                            *(++bp) = *escp;
                            len++;
                          }
                        }
                      }
                      ptr += 2;
                      n = (uint)-1;
                      break;
                    }
                  }
                }
                if(n != (uint)-1)   // I51 char not found, use fallback method
                  ptr++;
              }
              ptr++;
              break;
            case 29:  // The MNE escape character
              if(MNETable) {
                for(n=0; n<MNETable->size; n++) {
                  tptr = (char*)MNETP[n];
                  if(*(ptr+1) == tptr[0]) {
                    if(*(ptr+2) == tptr[1]) {
                      escp = &tptr[2];
                      if(*escp) {
                        *(++bp) = *escp++;
                        if(*escp) {
                          *(++bp) = *escp++;
                          len++;
                          if(*escp) {
                            *(++bp) = *escp;
                            len++;
                          }
                        }
                      }
                      ptr += 2;
                      n = (uint)-1;
                      break;
                    }
                  }
                }
                if(n != (uint)-1)   // MNE char not found, use fallback method
                  ptr++;
              }
              ptr++;
              break;
            case SOFTCR:
              if(CFG->switches.get(dispsoftcr)) {
                *(++bp) = *ptr++;
                ++len;
              }
              else {
                if(CompTable) {
                  if(NOT(isspace(*(ptr-1)) OR isspace(*(ptr+1)))) {
                    for(n=0; n<CompTable->size; n++) {
                      tptr = (char*)CompTP[n];
                      if(*(ptr-1) == tptr[0]) {
                        if(*(ptr+1) == tptr[1]) {
                          escp = &tptr[2];
                          if(*escp) {
                            *(bp) = *escp++;
                            if(*escp) {
                              *(++bp) = *escp++;
                              len++;
                              if(*escp) {
                                *(++bp) = *escp;
                                len++;
                              }
                            }
                          }
                          ptr++;
                          break;
                        }
                      }
                    }
                  }
                  else if(para == GLINE_QUOT) {
                    *ptr-- = CR;   // Fake a CR
                  }
                }
                else if(para == GLINE_QUOT) {
                  *ptr-- = CR;   // Fake a CR
                }
                ptr++;
              }
              break;
            case LF:
              ptr++;
              break;
            case CTRL_A:
              *(++bp) = *ptr++;
              ++len;
              break;
            case HT:
              ptr++;
              do_ht:
              if(CFG->disptabsize) {
                int llen = (int)(bp-linetmp);
                for(n=0; n<(CFG->disptabsize-(llen%CFG->disptabsize)); n++) {
                  *(++bp) = ' ';
                  len++;
                }
              }
              else {
                *(++bp) = ' ';
                len++;
              }
              break;
            case '=':
              if(qpencoded) {
                if(isxdigit(ptr[1]) AND isxdigit(ptr[2])) {
                  // Decode the character
                  dochar = (char)((xtoi(ptr[1]) << 4) | xtoi(ptr[2]));
                  ptr += 3;
                  //if((dochar == 0x20) AND (*ptr == CR))   // Undocumented soft line break?
                  //  ptr++;
                  if(dochar == HT)
                    goto do_ht;
                  else if(dochar == CR)
                    goto do_cr;
                  goto chardo;
                }
                else if((ptr[1] == CR) OR (ptr[1] == LF)) {
                  // Skip soft line break
                  ptr++;
                  while((*ptr == CR) OR (*ptr == LF))
                    ptr++;
                  break;
                }
              }
              goto defaultchardo;
            case ' ':
              if(len >= qlen) {
                tmp = ptr;
                btmp = bp;
              }
            default:
            defaultchardo:
              dochar = *ptr++;
            chardo:
              if((level&3) and ChsTP) {   // Translate level 1 and 2
                tptr = (char*)ChsTP[(byte)dochar];
                chln = *tptr++;
                while(chln--) {
                  *(++bp) = *tptr++;
                  ++len;
                }
              }
              else {
                *(++bp) = dochar;
                ++len;
              }
              break;
          }
          if(ch == CR)
            break;
        }

        *(bp+1) = NUL;

        if(len == (uint)margin) {
          if(*ptr == CR) {
            line->type |= GLINE_HARD;
            wraps = 0;
            ptr++;
          }
          else {
            wraps++;
            if(para == GLINE_QUOT)
              reflow = quotewraphard;
            line->type |= GLINE_WRAP;
            if(CFG->switches.get(dispsoftcr)) {
              while((*ptr == LF OR *ptr == SOFTCR) AND *ptr)
                ptr++;
              if(*bp == ' ' OR *ptr == ' ' OR *ptr == SOFTCR) {
                while((*ptr == ' ' OR *ptr == SOFTCR) AND *ptr)
                  ptr++;
              }
              else {
                if(tmp) {
                  bp = btmp;
                  *(bp+1) = NUL;
                  ptr = tmp+1;
                }
              }
            }
            else {
              while((*ptr == LF) AND *ptr)
                ptr++;
              if(*bp == ' ' OR *ptr == ' ') {
                while((*ptr == ' ') AND *ptr)
                  ptr++;
              }
              else {
                if(tmp) {
                  bp = btmp;
                  *(bp+1) = NUL;
                  ptr = tmp+1;
                }
              }
            }
          }
        }
        else {
          line->type |= GLINE_HARD;
        }

        // Get line length
        uint tmplinelength = (uint)((long)bp-(long)bptr);
        if(tmplinelength > (uint)(margin + 512)) {
          LOG.ErrPointer();
          LOG.printf("! A message line length (%u bytes) exceeded an internal buffer limit of %u bytes", tmplinelength, margin+512);
          LOG.printf(": Message line text: %s", bptr+1);
          PointerErrorExit();
        }

        // Strip trailing spaces if they exceed margin
        #if 0
        while((tmplinelength > margin) AND (*bp == ' ')) {
          tmplinelength--;
          *(bp--) = NUL;
        }
        #endif

        // Store line
        line->text = (char*)throw_malloc(tmplinelength+10);
        memcpy(line->text, linetmp+1, tmplinelength+5);
        prev_ptr[0] = line->text[0];
        prev_ptr[1] = line->text[1];

        // Set line color and type
        if((line->type & GLINE_QUOT) AND NOT is_quote(line->text))
          line->type ^= GLINE_QUOT;
        if(line->type & GLINE_QUOT)
          line->color = quotecolor(line->text);
        else
          line->color = C_READW;

        // Scan msg body top for RFC headerlines
        #if 1
        if((*line->text == NUL) AND NOT firstemptyline) {
          firstemptyline = TRUE;
          if(AA->Internetrfcbody()) {
            Line* linep = FirstLine(line);
            int headerlines = 0;
            while(linep) {
              char* tptr = linep->text;
              if(*tptr) {
                if(*tptr != CTRL_A) {
                  int kludgetype = ScanLine(msg, linep, tptr, getvalue, MASK_RFC);
                  if(kludgetype) {
                    tptr = strchr(tptr, ' ');
                    if(tptr) {
                      fastSkipWht(tptr);
                      if(kludgetype == RFC_CONTENT_TYPE) {
                        if(getvalue) {
                          if(striinc("iso-8859-1", tptr)) {
                            strcpy(chsbuf, "LATIN-1");
                            chslev = LoadCharset(chsbuf, CFG->xlatlocalset);
                            if(chslev) {
                              level = msg->charsetlevel = chslev;
                              strcpy(msg->charset, chsbuf);
                            }
                            gotmime = TRUE;
                          }
                        }
                      }
                      else if(kludgetype == RFC_CONTENT_TRANSFER_ENCODING) {
                        if(getvalue) {
                          if(striinc("quoted-printable", tptr)) {
                            qpencoded = TRUE;
                            msg->charsetencoding |= GCHENC_QP;
                            if(striinc("LATIN-1", msg->charset)) {
                              strcpy(chsbuf, "LATIN1QP");
                              chslev = LoadCharset("LATIN-1", CFG->xlatlocalset);
                              if(chslev) {
                                level = msg->charsetlevel = chslev;
                                strcpy(msg->charset, chsbuf);
                              }
                            }
                          }
                        }
                      }
                      else if(kludgetype == RFC_X_CHARSET) {
                        if(getvalue) {
                          if(NOT gotmime) {
                            strcpy(chsbuf, (striinc("8859-1", tptr) OR striinc("Latin1", tptr)) ? "LATIN-1" : CFG->xlatlocalset);
                            chslev = LoadCharset(chsbuf, CFG->xlatlocalset);
                            if(chslev) {
                              level = msg->charsetlevel = chslev;
                              strcpy(msg->charset, chsbuf);
                            }
                          }
                        }
                      }
                      else if(kludgetype == RFC_X_CHAR_ESC) {
                        if(getvalue)
                          if(NOT gotmime)
                            msg->charsetencoding |= GCHENC_MNE;
                      }
                      else if(kludgetype == HEADERLINE) {
                        linep->type |= GLINE_HIDD;
                      }
                    }
                    headerlines++;
                    linep->color = C_READK;
                    if(linep->next) {
                      char lwsp = *linep->next->text;
                      if((lwsp == ' ') OR (lwsp == '\t')) {
                        linep = linep->next;
                        linep->type |= linep->prev->type;
                      }
                    }
                  }
                  else
                    break;
                }
              }
              else {
                if(headerlines) {
                  linep->type |= GLINE_KLUD;
                  linep->color = C_READK;
                }
                break;
              }
              if(linep->type & GLINE_WRAP) {
                while(linep AND (linep->type & GLINE_WRAP))
                  linep = linep->next;
              }
              if(linep)
                linep = linep->next;
            }
          }
        }
        #endif

        prevline = line;
        line = (Line*)throw_xcalloc(1, sizeof(Line));
        line->prev = prevline;

        if(CFG->switches.get(dispsoftcr)) {
          while((*ptr == LF) AND *ptr)
            ptr++;
        }
        else {
          while((*ptr == LF OR *ptr == SOFTCR) AND *ptr)
            ptr++;
        }
      }

      throw_release(linetmp);
      throw_xfree(line);

      // Scan for kludge-, tear- and originlines
      ScanKludges(msg, getvalue);

      // Charset translate header fields
      if(header_recode) {
        strxmimecpy(msg->realby, msg->realby, level, sizeof(INam));
        strxmimecpy(msg->realto, msg->realto, level, sizeof(INam));
        strxmimecpy(msg->by    , msg->by    , level, sizeof(INam));
        strxmimecpy(msg->to    , msg->to    , level, sizeof(INam));

        if(NOT (msg->attr.frq() or msg->attr.att() or msg->attr.urq()))
          strxmimecpy(msg->re  , msg->re    , level, sizeof(ISub));
      }
    }
  }

  // Make the index to the line index as allowed by config
  MsgLineReIndex(msg);
}


//  ------------------------------------------------------------------

void MsgLineReIndex(GMsg* msg, int viewhidden, int viewkludge, int viewquote) {

  if(viewhidden == -1)
    viewhidden = AA->Viewhidden();
  if(viewkludge == -1)
    viewkludge = AA->Viewkludge();
  if(viewquote == -1)
    viewquote = AA->Viewquote();

  register int x;
  Line* line;

  throw_xrelease(msg->line);
  line = msg->lin;
  msg->lines = 0;
  while(line) {
    msg->lines++;
    if(line->next) {
      if(line->type & GLINE_QUOT) {
        if(NOT(line->next->type & GLINE_QUOT) AND NOT strblank(line->next->text)) {
          if(CFG->switches.get(quotespacing)) {
            line = AddLine(line, "");
            msg->lines++;
          }
        }
      }
      else {
        if(msg->attr.pos())
          if(strischg(line->text, "@position", ""))
            line->type |= GLINE_POSI;
        if((line->next->type & GLINE_QUOT) AND NOT strblank(line->text)) {
          if(CFG->switches.get(quotespacing)) {
            line = AddLine(line, "");
            msg->lines++;
          }
        }
      }
    }
    line = line->next;
  }
  msg->line = (Line**)throw_xcalloc(msg->lines+2, sizeof(Line*));

  x = 0;
  msg->lines = 0;
  line = msg->lin;

  char qbuf[50];
  char qbuf0[50];
  uint qlen = 0;
  int qmatches = 0;

  *qbuf0 = NUL;
  while(line) {
    if(line->type & GLINE_KLUD) {
      *qbuf0 = NUL;
      qmatches = 0;
      if(NOT viewkludge) {
        line = line->next;
        continue;
      }
    }
    else if(line->type & GLINE_HIDD) {
      *qbuf0 = NUL;
      qmatches = 0;
      if(NOT viewhidden) {
        line = line->next;
        continue;
      }
    }
    else if(line->type & GLINE_QUOT) {
      if(NOT viewquote) {
        GetQuotestr(line->text, qbuf, &qlen);
        strtrim(qbuf);
        if(strieql(qbuf0, qbuf)) {
          if(strpbrk(line->text+qlen, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"))
            qmatches++;
        }
        else {
          strcpy(qbuf0, qbuf);
          if(strpbrk(line->text+qlen, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"))
            qmatches = 1;
          else
            qmatches = 0;
        }
        if(qmatches != 1) {
          line = line->next;
          continue;
        }
      }
    }
    else {
      *qbuf0 = NUL;
      qmatches = 0;
    }
    msg->line[x++] = line;
    msg->lines++;
    line = line->next;
  }

  msg->line[x] = NULL;  // Mark end of index
}


//  ------------------------------------------------------------------

int LoadCharset(const char* import, const char* export) {

  FILE* fp;
  register int n;

  throw_release(CharTable);
  ChsTP = NULL;

  // Find and load charset table
  for(n=0; n<CFG->xlatcharsets; n++) {
    if(striinc(CFG->xlatcharset[n].import, import) AND striinc(CFG->xlatcharset[n].export, export)) {
      fp = fsopen(AddPath(CFG->goldpath, CFG->xlatged), "rb", CFG->sharemode);
      if(fp) {
        CharTable = (Chs*)throw_calloc(1, sizeof(Chs));
        fseek(fp, ((long)n*(long)sizeof(Chs)), SEEK_SET);
        fread(CharTable, sizeof(Chs), 1, fp);
        fclose(fp);
        ChsTP = CharTable->t;

        // Disable softcr translation unless DISPSOFTCR is enabled
        if(NOT CFG->switches.get(dispsoftcr)) {
          register char* tptr = (char*)ChsTP[SOFTCR];
          *tptr++ = 1;
          *tptr = SOFTCR;
        }
        return CharTable->level;
      }
    }
  }

  // No matching table found
  return 0;
}


//  ------------------------------------------------------------------

Line* DeleteLine(Line* line) {

  Line* nextline = NULL;

  // Link next and previous lines and release this line

  if(line) {
    if(line->prev) {
      line->prev->next = line->next;
      nextline = line->prev;
    }
    if(line->next) {
      line->next->prev = line->prev;
      nextline = line->next;
    }
    throw_release(line->text);
    throw_xfree(line);
  }

  return nextline;
}


//  ------------------------------------------------------------------

Line* InsertLine(Line* newline, Line* oldline, int pos) {

  if(oldline) {
    if(pos >= DIR_BELOW) {
      newline->prev = oldline;
      newline->next = oldline->next;
      if(oldline->next)
        oldline->next->prev = newline;
      oldline->next = newline;
    }
    else {
      newline->prev = oldline->prev;
      newline->next = oldline;
      if(oldline->prev)
        oldline->prev->next = newline;
      oldline->prev = newline;
    }
  }
  return newline;
}


//  ------------------------------------------------------------------

Line* FirstLine(Line* line) {

  if(line)
    while(line->prev)
      line = line->prev;
  return line;
}


//  ------------------------------------------------------------------

Line* LastLine(Line* line) {

  if(line)
    while(line->next)
      line = line->next;
  return line;
}


//  ------------------------------------------------------------------

Line* AddLine(Line* line, char* buf, int where) {

  Line* newline;

  newline = (Line*)throw_xcalloc(1, sizeof(Line));
  newline->type = GLINE_HARD;
  newline->text = (char*)throw_malloc(strlen(buf)+10);
  newline->color = C_READW;
  strcpy(newline->text, buf);

  return InsertLine(newline, line, where);
}


//  ------------------------------------------------------------------

Line* AddLine(Line* line, char* buf) {

  return AddLine(line, buf, DIR_BELOW);
}


//  ------------------------------------------------------------------

Line* AddKludge(Line* line, char* buf, int where) {

  Line* newline;

  newline = (Line*)throw_xcalloc(1, sizeof(Line));
  newline->type = GLINE_HARD | GLINE_KLUD;
  newline->text = (char*)throw_malloc(strlen(buf)+10);
  newline->color = C_READK;
  strcpy(newline->text, buf);

  return InsertLine(newline, line, where);
}


//  ------------------------------------------------------------------

Line* _stdc AddLineF(Line*& line, const char* format, ...) {

  char buf[256];
  va_list argptr;
  va_start(argptr, format);
  vsprintf(buf, format, argptr);
  va_end(argptr);
  line = AddLine(line, buf);
  return line;
}


//  ------------------------------------------------------------------

Line* AddHexdump(Line*& line, void* data, size_t datalen) {

  char buf[256];
  register uint pos = 0;
  register char* ptr = (char*)data;

  while(pos < datalen) {

    register uint dataleft = datalen - pos;
    sprintf(buf, "%04X   ", pos);
    HexDump16(buf+7, ptr, (dataleft < 16) ? dataleft : 16, HEX_DUMP2);
    line = AddLine(line, buf);
    ptr += 16;
    pos += 16;
  }

  return line;
}


//  ------------------------------------------------------------------

char* ParseInternetAddr(char* __string, char* __name, char* __addr) {

  *__name = *__addr = NUL;
  char* name = __name;
  char* addr = __addr;
  char* commaptr = NULL;
  
  if(strchr(__string, ',')) {
    bool inquotes = false;
    commaptr = __string;
    while(commaptr) {
      if(*commaptr == '\"')
        inquotes ^= true;
      else if((*commaptr == ',') and not inquotes) {
        *commaptr = NUL;
        break;
      }
      if(not *commaptr)
        commaptr = NULL;
      else
        commaptr++;
    }
  }

  char* p;
  if((p = strrchr(__string, '>')) != NULL)
    *(++p) = NUL;

  char* endchar = __string + strlen(__string) - 1;
  if(*endchar == /*(*/ ')') {
    char* begchar = endchar;
    int pcnt = 0;
    while(begchar > __string) {
      if(*begchar == /*(*/ ')')
        pcnt++;
      else if(*begchar == '(' /*)*/)
        pcnt--;
      if(pcnt == 0)
        break;
      begchar--;
    }
    if(*begchar == '(' /*)*/)
      begchar++;
    if(NOT strchr(__string, '@') AND strchr(begchar, '@')) {
      name = __addr;
      addr = __name;
    }
    strbtrim(strxcpy(name, begchar, (uint)(endchar-begchar)+1));
    strbtrim(strxcpy(addr, __string, (uint)(begchar-__string)));
  }
  else if(*endchar == '>') {
    char* endaddr = endchar;
    while(*endchar != '<' AND endchar > __string)
      endchar--;
    char* begaddr = endchar;
    if(*endchar == '<') {
      begaddr++;
      endchar--;
    }
    fastSkipWht(__string);
    if(NOT strchr(begaddr, '@') AND strchr(__string, '@')) {
      name = __addr;
      addr = __name;
    }
    strbtrim(strxcpy(name, __string, (uint)(endchar-__string)+1));
    strbtrim(strxcpy(addr, begaddr, (uint)(endaddr-begaddr)+1));
  }
  else {
    strcpy(__addr, __string);
  }

  if(*__addr == '@') {
    char* ptr = strchr(__addr, ':');
    if(ptr)
      memmove(__addr, ptr+1, strlen(ptr));
  }

  if(commaptr)
    *commaptr = ',';

  StripQuotes(__name);

  return __name;
}


//  ------------------------------------------------------------------

