
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gelmsg.cpp,v 1.2 1999/02/06 02:56:31 damware Exp $
//  ------------------------------------------------------------------
//  Load message or header. Save header.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif


//  ------------------------------------------------------------------

void ResetMsg(GMsg* msg) {

  throw_free(msg->references);
  throw_free(msg->inreplyto);
  throw_free(msg->messageid);
  throw_free(msg->txt);
  throw_xfree(msg->line);
  Line* line = msg->lin;
  while(line) {
    Line* nextline = line->next;
    throw_free(line->text);
    throw_xfree(line);
    line = nextline;
  }
  memset(msg, 0, sizeof(GMsg));
}


//  ------------------------------------------------------------------

int Area::LoadHdr(GMsg* msg, ulong msgno) {

  ResetMsg(msg);
  msg->msgno = msgno;
  int retval = area->load_hdr(msg);

  // Don't translate charsets if we don't know charset
  // Currently, it only mime-decodes, so it's okay.
  #if 1
  if(retval) {
    // Charset translate header fields
    strxmimecpy(msg->realby, msg->realby, msg->charsetlevel, sizeof(INam));
    strxmimecpy(msg->realto, msg->realto, msg->charsetlevel, sizeof(INam));
    strxmimecpy(msg->by    , msg->by    , msg->charsetlevel, sizeof(INam));
    strxmimecpy(msg->to    , msg->to    , msg->charsetlevel, sizeof(INam));

    if(NOT (msg->attr.frq() or msg->attr.att() or msg->attr.urq()))
      strxmimecpy(msg->re  , msg->re    , msg->charsetlevel, sizeof(ISub));
  }
  #endif
  return retval;
}


//  ------------------------------------------------------------------

int Area::LoadMsg(GMsg* msg, ulong msgno, int margin, int mode) {

  ResetMsg(msg);
  msg->msgno = msgno;
  if(msgno AND area->load_msg(msg)) {
    if(mode & (GMSG_COPY|GMSG_MOVE)) {
      if(NOT((mode & GMSG_MOVE) AND (mode & GMSG_UNS_NOT_RCV)))
        return TRUE;
      if(NOT (msg->attr.uns() AND NOT msg->attr.rcv()))
        return TRUE;
    }
    else {
      // We can't rely on the destination address unless we are in netmail
      if(NOT AA->isnet()) {
        msg->dest.reset_fast();
        msg->odest.reset_fast();
      }
    }
    msg->attr.tou0();
    msg->TextToLines(margin);
    return TRUE;
  }
  return FALSE;
}


//  ------------------------------------------------------------------

void Area::SaveHdr(int mode, GMsg* msg) {

  area->save_hdr(mode, msg);
  UpdateAreadata();
}


//  ------------------------------------------------------------------

