
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gepost.cpp,v 1.5 1999/02/06 02:56:31 damware Exp $
//  ------------------------------------------------------------------
//  Mega-functions to handle creation of msgs :-(
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include "gstrblks.h"


//  ------------------------------------------------------------------

int* post_xparea;
uint position;
ulong msgcount = 0;


//  ------------------------------------------------------------------

const char* get_subject_re_info(const char* s, ulong& n) {

  if(toupper(*s) == 'R' and tolower(s[1]) == 'e') {

    if(s[2] == ':') {
      n = 1;
      return s + 3;
    }
    else if(s[2] == '^' and isdigit(s[3])) {
      const char* d = s + 4;
      while(isdigit(*d))
        d++;
      if(*d == ':') {
        n = (ulong)atol(s + 3);
        return d + 1;
      }
    }
  }

  n = 0;
  return NULL;
}


//  ------------------------------------------------------------------

void CheckSubject(GMsg* msg, char* subj) {

  for(int n=0; n<fspecs; n++)
    throw_release(fspec[n].fblk);
  fspecs = specfiles = 0;
  fspec = (FileSpec*)throw_realloc(fspec, sizeof(FileSpec));

  if(AA->isnet()) {
    if(EDIT->AutoAttach()) {
      // If the subject was changed
      if(NOT strieql(subj, msg->re)) {
        // Check if the subj begins with drive spec
        if(NOT(msg->attr.frq() OR msg->attr.att() OR msg->attr.urq())) {
          char* ptr = subj;
          if(*ptr == '^')
            ptr++;
          if((in_range((char)tolower(*ptr), (char)'c', (char)'z')  AND (ptr[1] == ':')) OR ((ptr[0] == '\\') AND (ptr[1] == '\\'))) {
            // Autoattach
            msg->attr.att1();
            AttrAdd(&msg->attr, &CFG->attribsattach);
            DispHeadAttrs(msg);
          }
        }
      }
    }

    if(msg->attr.frq()) {
      // Handle File Requests
      if(*subj) {
        ISub buf;
        strcpy(buf, subj);
        char* ptr = strtok(buf, " ");
        while(ptr) {
          if(*ptr == '!') {   // Store password
            if(fspecs)
              strcpy(fspec[fspecs-1].password, ptr);
          }
          else {
            fspec = (FileSpec*)throw_realloc(fspec, (fspecs+1)*sizeof(FileSpec));
            memset(fspec+fspecs, 0, sizeof(FileSpec));
            fspec[fspecs].files = 1;
            strcpy(fspec[fspecs].path, ptr);
            specfiles++;
            fspecs++;
          }
          ptr = strtok(NULL, " ");
        }
      }
    }
    else if(msg->attr.att()) {
      // Handle File Attaching
      ISub buf;
      if(strblank(subj)) {
        if(*CFG->attachpath)
          strcpy(subj, CFG->attachpath);
        else
          strcpy(subj, CFG->goldpath);
      }
      strcpy(buf, subj);
      char* ptr = strtok(buf, " ");
      while(ptr) {
        fspec = (FileSpec*)throw_realloc(fspec, (fspecs+1)*sizeof(FileSpec));
        memset(fspec+fspecs, 0, sizeof(FileSpec));
        if(*ptr == '^') {
          fspec[fspecs].delsent = YES;
          ptr++;
        }
        strcpy(fspec[fspecs].path, ptr);
        FileSelect(msg, LNG->AttachFiles, &fspec[fspecs]);
        specfiles += fspec[fspecs].files;
        if(fspec[fspecs].files)
          fspecs++;
        ptr = strtok(NULL, " ");
      }
    }
    else if(msg->attr.urq()) {
      // Handle Update Requests
      ISub buf;
      if(NOT *subj)
        strcpy(subj, ".");
      strcpy(buf, subj);
      char* ptr = strtok(buf, " ");
      while(ptr) {
        if(*ptr == '!') {   // Store password
          if(fspecs)
            strcpy(fspec[fspecs-1].password, ptr);
        }
        else {
          fspec = (FileSpec*)throw_realloc(fspec, (fspecs+1)*sizeof(FileSpec));
          memset(fspec+fspecs, 0, sizeof(FileSpec));
          strcpy(fspec[fspecs].path, ptr);
          FileSelect(msg, LNG->UpdreqFiles, &fspec[fspecs]);
          specfiles += fspec[fspecs].files;
          if(fspec[fspecs].files)
            fspecs++;
        }
        ptr = strtok(NULL, " ");
      }
    }

    // Generate the (first) processed subject line
    if(specfiles) {
      ISub buf;
      *buf = NUL;
      for(int x=0,n=0; x<fspecs; x++) {
        for(int m=0; m<fspec[x].files; m++) {
          ISub subject;
          sprintf(subject, "%s%s%s%s%s ", fspec[x].delsent ? "^" : "", ReMapPath(fspec[x].path), fspec[x].fblk ? (fspec[x].fblk[m].name ? fspec[x].fblk[m].name : "") : "", *fspec[x].password ? " " : "", fspec[x].password);
          strupr(subject);
          if((strlen(buf) + strlen(subject)) > 71) {
            n++;
            break;
          }
          else
            strcat(buf, subject);
        }
        if(n)
          break;
      }
      strtrim(strcpy(subj, buf));
    }
  }
}


//  ------------------------------------------------------------------

static void MakeMsg3(int& mode, GMsg* msg) {

  register int n;
  uint u;

  msg->charsetlevel = 0;
  throw_release(CharTable);
  if(*AA->Xlatexport()) {
    msg->charsetlevel = LoadCharset(CFG->xlatlocalset, AA->Xlatexport());
    if(msg->charsetlevel)
      sprintf(msg->charset, "%s %d", AA->Xlatexport(), msg->charsetlevel);
    else
      sprintf(msg->charset, "%s 2", CFG->xlatlocalset);
  }

  // Do Timefields
  if(msg->attr.fmu()) {
    if(AA->isjam() or AA->iswildcat())
      msg->received = gtime(NULL);
    else
      msg->arrived = gtime(NULL);
  }

  GMsg* carbon=NULL;
  int cc = DoCarboncopy(msg, &carbon);

  if(AA->isecho())
    DoTearorig(mode, msg);

  // Do FakeNets
  msg->oorig = msg->orig;
  if(AA->isnet() AND msg->orig.point) {
    for(u=0; u<CFG->akas; u++) {
      if(!memcmp(&CFG->aka[u], &msg->orig, sizeof(Addr))) {
        // Use fakenet to everybody
        if(CFG->aka[u].pointnet) {
          msg->oorig.net   = CFG->aka[u].pointnet;  // Create fake address
          msg->oorig.node  = msg->orig.point;
          msg->oorig.point = 0;
        }
        break;
      }
    }
  }

  DoKludges(mode, msg);

  if(cc) {
    update_statuslinef(LNG->StatusCC, msg->to,
      msg->dest.zone, msg->dest.net,
      msg->dest.node, msg->dest.point
    );
  }

  msg->LinesToText();

  if(AA->isnet() AND (msg->attr.frq() OR msg->attr.att() OR msg->attr.urq()) AND specfiles >= 1)
    CreateFileMsgs(mode, msg);
  else {
    HeaderView.Use(AA, msg);
    HeaderView.Paint();

    // Do zoneGating
    if(AA->isnet() AND CFG->zonegating AND (msg->dest.zone != msg->orig.zone)) {
      if(NOT (msg->attr.dir() or msg->attr.cra() or msg->attr.hld())) {
        GMenuZonegate MenuZonegate;
        if(CFG->zonegating == YES OR MenuZonegate.Run())
          ZonegateIt(msg->odest, msg->orig, msg->dest);
      }
    }

    if(msg->odest.net == 0)
      msg->odest = msg->dest;

    HeaderView.Use(AA, msg);
    HeaderView.Paint();
    w_info(LNG->Wait);
    AA->SaveMsg((mode == MODE_CHANGE) ? GMSG_UPDATE : GMSG_NEW, msg);
    if(not (AA->isecho() and mode == MODE_REPLYCOMMENT))
      update_addressbook(msg);
    w_info(NULL);
  }

  #if 1
  if(AL.AreaEchoToNo(CFG->areayouwroteto)>=0) {
    Area* A = AL.AreaEchoToPtr(CFG->areayouwroteto);
    if(AA != A and (mode != MODE_CHANGE)) {
      msg->link.reset();
      char* ptr = msg->txt + (*msg->txt == CTRL_A);
      if(NOT strnieql(ptr, "AREA:", 5)) {
        uint elen = 6 + strlen(AA->echoid()) + 1;
        uint mlen = strlen(msg->txt)+1;
        msg->txt = (char*)throw_realloc(msg->txt, elen+mlen);
        memmove(msg->txt+elen, msg->txt, mlen);
        sprintf(msg->txt, "\001AREA:%s", AA->echoid());
        msg->txt[elen-1] = CR;
      }
      AA->Close();
      A->Open();
      ulong oldmsgno = msg->msgno;
      A->NewMsgno(msg);
      A->SaveMsg(GMSG_NEW|GMSG_NOLSTUPD, msg);
      A->Close();
      AA->Open();
      msg->msgno = oldmsgno;
    }
  }
  #endif

  if(cc) {
    for(n=0; n<cc; n++) {
      GMsg* cmsg = &carbon[n];
      cmsg->txt   = msg->txt;
      cmsg->lin   = msg->lin;
      cmsg->line  = msg->line;
      cmsg->lines = msg->lines;
      cmsg->messageid = msg->messageid;
      cmsg->inreplyto = msg->inreplyto;
      cmsg->references = msg->references;
      AA->NewMsgno(cmsg);
      update_statuslinef(LNG->StatusCC, cmsg->to,
        cmsg->dest.zone, cmsg->dest.net,
        cmsg->dest.node, cmsg->dest.point
      );

      // Do aka matching
      if(AA->Akamatching()) {
        int newaka = AkaMatch(&cmsg->orig, &cmsg->dest);
        cmsg->orig = CFG->aka[newaka].addr;
      }

      // Do FakeNets
      cmsg->oorig = cmsg->orig;
      if(AA->isnet() AND cmsg->orig.point) {
        for(u=0; u<CFG->akas; u++) {
          if(!memcmp(&CFG->aka[u], &cmsg->orig, sizeof(Addr))) {
            // Use fakenet to everybody
            if(CFG->aka[u].pointnet) {
              cmsg->oorig.net   = CFG->aka[u].pointnet;  // Create fake address
              cmsg->oorig.node  = cmsg->orig.point;
              cmsg->oorig.point = 0;
            }
            break;
          }
        }
      }

      DoKludges(mode, cmsg);

      cmsg->LinesToText();
      msg->txt   = cmsg->txt;
      msg->lin   = cmsg->lin;
      msg->line  = cmsg->line;
      msg->lines = cmsg->lines;
      msg->messageid  = cmsg->messageid;
      msg->inreplyto  = cmsg->inreplyto;
      msg->references = cmsg->references;
      if(AA->isnet() AND (cmsg->attr.frq() OR cmsg->attr.att() OR cmsg->attr.urq())) {
        CreateFileMsgs(MODE_NEW, cmsg);
        msg->txt   = cmsg->txt;
        msg->lin   = cmsg->lin;
        msg->line  = cmsg->line;
        msg->lines = cmsg->lines;
        msg->messageid  = cmsg->messageid;
        msg->inreplyto  = cmsg->inreplyto;
        msg->references = cmsg->references;
      }
      else {
        HeaderView.Use(AA, cmsg);
        HeaderView.Paint();

        // Do zoneGating
        if(AA->isnet() AND CFG->zonegating AND (cmsg->dest.zone != cmsg->orig.zone)) {
          if(NOT cmsg->attr.dir()) {
            GMenuZonegate MenuZonegate;
            if(CFG->zonegating == YES OR MenuZonegate.Run())
              ZonegateIt(cmsg->odest, cmsg->orig, cmsg->dest);
          }
        }

        if(cmsg->odest.net == 0)
          cmsg->odest = cmsg->dest;

        HeaderView.Use(AA, cmsg);
        HeaderView.Paint();
        w_info(LNG->Wait);
        AA->SaveMsg(GMSG_NEW, cmsg);
        w_info(NULL);
        update_addressbook(cmsg);
        msg->txt   = cmsg->txt;
        msg->lin   = cmsg->lin;
        msg->line  = cmsg->line;
        msg->lines = cmsg->lines;
        msg->messageid  = cmsg->messageid;
        msg->inreplyto  = cmsg->inreplyto;
        msg->references = cmsg->references;
      }
    }
    throw_release(carbon);
  }
}


//  ------------------------------------------------------------------

static void MakeMsg2(int& mode, int& status, int& forwstat, int& topline, int& crosspost, int& xps, GMsg* msg, GMsg* oldmsg, GMsg* cmpmsg) {

  register uint n;
  register int x;
  char* ptr;
  char buf[256], buf2[256];
  Line* line;
  Line* newline;
  Line* oldline;
  int quotes, nonquotes, xphide;

  if(status == MODE_CHANGE OR (mode == MODE_FORWARD AND status == MODE_SAVE)) {
    if(mode == MODE_CHANGE) {
      if(stricmp(msg->By(), AA->Username().name)) {
        for(n=0; n<CFG->usernames; n++) {
          if(strieql(CFG->username[n].name, msg->By())) {
            n = 0;
            break;
          }
        }
        if(n)
          status = MODE_SAVE;
      }
    }
    w_info(LNG->Wait);
    if(mode != MODE_CHANGE) {
      if(not CFG->switches.get(emptytearline))
        strcpy(msg->tearline, AA->Tearline());

      if(AA->Taglinesupport())
        strcpy(msg->tagline, AA->Tagline());

      DoTearorig(mode, msg);

      strcpy(msg->gif, AA->Gif());
      strcpy(msg->organization, AA->Organization());
    }
    if(mode != MODE_CHANGE OR status == MODE_SAVE) {
      if(TemplateToText(mode, msg, oldmsg, AA->Tpl(), OrigArea)) {
        HeaderView.Use(AA, msg);
        HeaderView.Paint();
      }
      msg->attr.tou1();  // Ignore any kludge address found
      msg->attr.pos1();
      msg->TextToLines(CFG->dispmargin-1);
      msg->attr.tou0();
      msg->attr.pos0();
    }
    uint lineno = position = reader_topline+1;
    line = msg->lin;
    while(line) {
      if(NOT(line->type & GLINE_QUOT))
        if(line->type & GLINE_POSI)
          position = lineno;
      lineno++;
      line = line->next;
    }
    if(*EDIT->External() AND NOT EDIT->Internal()) {
      SaveLines(MODE_SAVE, AddPath(CFG->goldpath, EDIT->File()), msg);
    }
    int loop = 0;
    w_info(NULL);
    do {
      if(loop) {
        status = MODE_SAVE;
        forwstat = NO;
      }
      if(mode == MODE_FORWARD)
        status = MODE_SAVE;
      if(*EDIT->External() AND forwstat == NO AND NOT EDIT->Internal()) {
        strcpy(buf, EDIT->External());
        itoa(position, buf2, 10);
        strischg(buf, "@line", buf2);
        strischg(buf, "@file", AddPath(CFG->goldpath, EDIT->File()));
        long ftbefore = GetFiletime(AddPath(CFG->goldpath, EDIT->File()));
        sprintf(buf2, LNG->EditCmd, buf);
        ShellToDos(buf, buf2, LGREY|_BLACK, YES);
        long ftafter = GetFiletime(AddPath(CFG->goldpath, EDIT->File()));
        if(status != MODE_SAVE) {
          status = MODE_SAVE;
          if(ftbefore == ftafter) {
            if(streql(msg->by, cmpmsg->by))
              if(streql(msg->to, cmpmsg->to))
                if(streql(msg->re, cmpmsg->re))
                  if(!memcmp(&msg->attr, &cmpmsg->attr, sizeof(Attr)))
                    status = MODE_QUIT;
          }
        }
      }
      else if(forwstat == NO AND (EDIT->Internal() OR NOT *EDIT->External())) {
        w_info(LNG->Wait);
        status = MODE_SAVE;
        newline = line = msg->lin;
        while(line) {
          newline = line;
          if(((line->type & GLINE_HIDD) AND NOT AA->Viewhidden()) OR ((line->type & GLINE_KLUD) AND NOT AA->Viewkludge()))
            newline = line = DeleteLine(line);
          else {
            strtrimspace(line->text);
            if(NOT(line->type & GLINE_HARD))
              strcat(line->text, " ");
            else
              strcat(line->text, "\n");
            line = line->next;
          }
        }
        msg->lin = FirstLine(newline);

        w_info(NULL);
        if(position)
          position--;
        if(NOT savedirect)
          status = EditMsg(mode, &position, msg);
        _in_editor = NO;
        w_info(LNG->Wait);

        if(status != MODE_QUIT) {
          line = msg->lin;
          while(line) {
            if(line->text) {
              if(strchr(line->text, LF))
                line->type = GLINE_HARD;
              else if(line->next AND (line->next->type & GLINE_QUOT))
                line->type = GLINE_HARD;
              else
                line->type = 0;
              strtrimspace(line->text);
              if(AA->isinternet()) {
                // Check for signature indicator
                if(streql(line->text, "--")) {
                  throw_free(line->text);
                  line->text = throw_strdup("-- ");
                  line->type = GLINE_HARD;
                }
              }
            }
            line = line->next;
          }
          LoadCharset("N/A", "N/A");
          strcpy(msg->charset, CFG->xlatlocalset);
          strcat(msg->charset, " 2");
          msg->charsetlevel = 2;
          msg->LinesToText();
        }
        w_info(NULL);
      }
      if(status == MODE_SAVE) {
        w_info(LNG->Wait);
        if(*EDIT->External() AND NOT EDIT->Internal())
          LoadText(msg, AddPath(CFG->goldpath, EDIT->File()));
        if(mode == MODE_FORWARD)
          msg->attr.pos1();
        msg->attr.tou1();  // ignore any kludge address found
        msg->TextToLines(CFG->dispmargin-1);
        msg->attr.tou0();
        msg->attr.pos0();
        quotes=0;
        nonquotes=0;
        Line tmpline;
        line = msg->lin;
        while(line) {
          if(NOT(line->type & (GLINE_KLUD | GLINE_HIDD | GLINE_TEAR | GLINE_ORIG))) {
            if(strnieql(line->text, "XC:", 3) OR strnieql(line->text, "XP:", 3)) {

              gstrarray xposts;
              tokenize(xposts, line->text+3 ,", ");

              oldline = line;
              x = 0;

              for(int i = 0; i < xposts.size(); i++) {
                ptr = strcpy(buf2, xposts[i]);
                if(*ptr == '#') {
                  xposts.set(i, ++ptr);
                  xphide = true;
                }
                else
                  xphide = false;

                if(*ptr == '@') {
                  FILE* fp = fsopen(AddPath(CFG->goldpath, ++ptr), "rt", CFG->sharemode);
                  if(fp) {
                    gstrarray fc;

                    while(fgets(buf, sizeof(buf), fp))
                      tokenize(fc, buf, ", \n");

                    fclose(fp);

                    for(int j = 0; j < fc.size(); j++) {
                      if(*fc[j] != '@') {  // we don't like nesting!
                        if(*xposts[i] == '@')
                          xposts.set(i,fc[j]);
                        else
                          xposts.add(fc[j]);
                      }
                    }
                  }
                }
                for(uint j=0; j < AL.total; j++) {
                  strcpy(buf2, AL[j]->echoid());
                  strcpy(buf, xposts[i]);
                  if(strwild(buf2, buf) and not AL[j]->isseparator()) {
                    if(CFG->crosspostlist == CC_NAMES and not xphide) {
                      sprintf(buf, LNG->Crosspostedin, buf2);
                      line = AddLine(line, buf);
                      nonquotes += strlen(line->text);
                    }
                    x++;
                    if(not strieql(AA->echoid(), buf2)) {
                      post_xparea[xps++] = j;
                      crosspost = true;
                    }
                  }
                }
              }

              if(x and CFG->crosspostlist != CC_KEEP) {
                Line* nxtline;
                if(oldline == msg->lin)
                  nxtline = msg->lin = DeleteLine(oldline);
                else
                  nxtline = DeleteLine(oldline);
                if(oldline == line) {
                  tmpline.next = nxtline;
                  line = &tmpline;
                }
              }
            }
            else {
              n = strlen(line->text);
              nonquotes += n;
              if(line->type & GLINE_QUOT)
                quotes += n;
            }
          }
          line = line->next;
        }
        MsgLineReIndex(msg);
        msg->quotepct = 100-Pct(nonquotes, quotes);

        w_info(NULL);
        if(NOT savedirect) {
          HeaderView.Use(AA, msg);
          HeaderView.Paint();
          BodyView.Use(AA, msg, topline);
          BodyView.Paint();
          do {
            if(gkbd.quitall)
              status = MODE_SAVE;
            else {
              GMenuEditfile MenuEditfile;
              status = MenuEditfile.Run(msg);
            }
            if(status == MODE_VIEW) {
              BodyView.Use(AA, msg, topline);
              BodyView.Paint();
              ViewMessage();
            }
          } while(status == MODE_VIEW);
        }
      }
      loop++;
      if(status == MODE_CHANGE) {
        _in_editor = YES;
        HeaderView.Use(AA, msg);
        HeaderView.Paint();
      }
    } while(status == MODE_CHANGE);
  }
}


//  ------------------------------------------------------------------

static void GetLastLink(GMsg* msg, ulong& msgno) {

  GMsg* uplink = (GMsg*)throw_calloc(1, sizeof(GMsg));

  uplink->msgno = msg->msgno;
  uplink->link.first_set(msg->link.first());

  if(uplink->link.first()) {
    while(AA->Msgn.ToReln(uplink->link.first())) {
      if(NOT AA->LoadHdr(uplink, uplink->link.first()))
        uplink->msgno = 0;
    }
  }

  // Return msgno of the last link
  msgno = uplink->msgno;

  ResetMsg(uplink);
  throw_free(uplink);
}


//  ------------------------------------------------------------------

void MakeMsg(int mode, GMsg* omsg) {

  GFTRK("MakeMsg");

  int xparea[512];
  post_xparea = xparea;

  // Tell DispHeader not to show msg size
  _in_editor = YES;

  // Allocate some msgs
  GMsg* msg = (GMsg*)throw_calloc(1, sizeof(GMsg));
  GMsg* reply = (GMsg*)throw_calloc(1, sizeof(GMsg));
  GMsg* cmpmsg = (GMsg*)throw_calloc(1, sizeof(GMsg));
  ulong reply_msgno = 0;

  // Keep copy of original aka for later restoration
  Addr origaka = AL.AreaIdToPtr(OrigArea)->aka();

  // Force template choice if necessary
  if(AA->Forcetemplate() AND (mode != MODE_CHANGE))
    ChangeTemplate();

  // Random System
  AA->RandomizeData();

  // Prepare confirmation receipts
  int confirm = NO;
  if(mode == MODE_CONFIRM) {
    confirm = YES;
    mode = MODE_REPLY;
  }

  // Prepare crossposting
  int crosspost = NO;
  int xps = 0;

  do {
    if(xps) {
      mode = MODE_COPY;
      AL.SetActiveAreaNo(xparea[xps-1]);
      if(CurrArea != OrigArea) {
        AA->Open();
        AA->RandomizeData(mode);
      }

      msg->attr.tou0();
      msg->TextToLines(CFG->dispmargin-1);
      msg->orig = AA->Aka().addr;
      msg->charsetlevel = LoadCharset(CFG->xlatlocalset, AA->Xlatexport());
      if(CharTable AND NOT strblank(CharTable->export))
        strcpy(msg->charset, CharTable->export);
      strcpy(msg->odom, CFG->aka[AkaMatch(&msg->orig, &AA->Aka().addr)].domain);
      if(AA->isecho())
        DoTearorig(mode, msg);
      DoKludges(mode, msg);
      msg->LinesToText();
      xps--;
    }

    int status = MODE_SAVE;
    int hardlines = EDIT->HardLines();
    int topline = 0;
    fspecs = specfiles = 0;
    fspec = (FileSpec*)throw_realloc(fspec, sizeof(FileSpec));
    memset(fspec, 0, sizeof(FileSpec));

    // Get msg number and reset/copy msg data
    if(mode != MODE_COPY)
      ResetMsg(msg);
    switch(mode) {
      case MODE_FORWARD:
        memcpy(msg, omsg, sizeof(GMsg));
        msg->txt = (char*)throw_strdup(msg->txt);
        msg->lin = NULL;
        msg->line = NULL;
        msg->messageid = NULL;
        msg->inreplyto = NULL;
        msg->references = NULL;
        msg->attr.tou0();
        msg->TextToLines(CFG->dispmargin-1, false);
        *msg->iorig = NUL;
        *msg->idest = NUL;
        *msg->ireplyto = NUL;
        *msg->iaddr = NUL;
        *msg->igate = NUL;
        *msg->ifrom = NUL;
        *msg->ito = NUL;
        *msg->organization = NUL;
        *msg->realby = NUL;
        *msg->realto = NUL;
        *msg->origin = NUL;
        // Drop through ...
      case MODE_COPY:
        AA->NewMsgno(msg);
        msg->link.reset();
        break;
      case MODE_CHANGE:
        memcpy(msg, omsg, sizeof(GMsg));
        msg->txt = (char*)throw_strdup(msg->txt);
        msg->lin = NULL;
        msg->line = NULL;
        msg->messageid = NULL;
        msg->inreplyto = NULL;
        msg->references = NULL;
        msg->attr.tou0();
        msg->TextToLines(CFG->dispmargin-1, false);
        break;
      case MODE_NEW:
        wfill(MINROW, 0, MAXROW-2, MAXCOL-1, ' ', C_READW);
      case MODE_QUOTE:
      case MODE_REPLY:
      case MODE_REPLYCOMMENT:
        AA->NewMsgno(msg);
        msg->timesread = 1;
    }

    msg->oorig = msg->orig;
    msg->board = AA->board();
    msg->odest = msg->dest;

    if(mode != MODE_COPY) {

      // Set msg date and time
      int dochgdate = TRUE;
      if(mode == MODE_CHANGE) {
        dochgdate = EDIT->ChangeDate();
        if((dochgdate == TRUE) AND NOT msg->attr.fmu())
          dochgdate = FALSE;
      }
      if(dochgdate) {
        msg->written = gtime(NULL);
        msg->arrived = msg->received = 0;
      }

      // Prepare msg header

      if(mode != MODE_CHANGE)
        msg->attr = AA->Attributes();   // Set default attributes

      switch(mode) {
        case MODE_QUOTE:
        case MODE_REPLYCOMMENT:
          omsg->attr.tou0();
          omsg->TextToLines(-CFG->quotemargin, false);
          if(omsg->attr.rot())
            Rot13(omsg);
          // Drop through
        case MODE_REPLY:
          if(confirm)
            msg->attr = CFG->attribscfm;
          if(omsg->attr.pvt())
            msg->attr.pvt1();    // Set Pvt attr in response to Pvt in orig
          if(omsg->attr.frq() AND (mode == MODE_REPLY) AND (NOT confirm)) {
            msg->attr = CFG->attribsfrq;
            msg->attr.frq1();   // Special case for the file request function
          }

          // Change header data if replying to a forwarded message
          if((mode == MODE_REPLYCOMMENT) AND omsg->fwdorig.net) {
            strcpy(omsg->by, omsg->fwdfrom);
            strcpy(omsg->realby, omsg->fwdfrom);
            omsg->oorig = omsg->orig = omsg->fwdorig;
            strcpy(omsg->to, omsg->fwdto);
            strcpy(omsg->realto, omsg->fwdto);
            omsg->odest = omsg->dest = omsg->fwddest;
            strcpy(omsg->re, omsg->fwdsubj);
            *omsg->ireplyto = *omsg->iorig = NUL;
          }

          // Do aka matching
          {
            if(AA->Akamatching()) {
              // ... but only if we did NOT change aka manually
              if(AA->Aka().addr.equals(AA->aka())) {
                Addr aka_addr = AA->Aka().addr;
                AkaMatch(&aka_addr, &omsg->orig);
                AA->SetAka(aka_addr);
              }
            }
          }

          if((mode == MODE_REPLYCOMMENT) AND NOT omsg->fwdorig.net) {
            strcpy(msg->to, omsg->to);
            strcpy(msg->realto, omsg->realto);
            msg->dest = omsg->dest;
            strcpy(msg->idest, omsg->idest);
          }
          else {
            strcpy(msg->to, omsg->By());
            strcpy(msg->realto, omsg->realby);
            msg->dest = omsg->orig;
            strcpy(msg->idest, *omsg->ireplyto ? omsg->ireplyto : omsg->iorig);
          }
          strcpy(msg->re, omsg->re);
          
          if(AA->Replyre() or AA->isinternet()) {

            ulong number;
            const char* r = get_subject_re_info(msg->re, number);
            if(r) {
              if(AA->Replyre() == REPLYRE_NUMERIC and not AA->isinternet()) {
                ISub subj;
                sprintf(subj, "Re^%lu:%s", ((number+1) > number) ? (number+1) : number, r);
                strcpy(msg->re, subj);
              }
            }
            else {
              ISub subj;
              sprintf(subj, "%s%s", (msg->attr.frq() OR msg->attr.urq()) ? "" : "Re: ", msg->re);
              strcpy(msg->re, subj);
            }
          }
          else {
            char* r;
            ulong number;
            char* ptr = msg->re;
            do {
              r = (char*)get_subject_re_info(ptr, number);
              if(r)
                ptr = strskip_wht(r);
            } while(r);
            strcpy(msg->re, ptr);
          }
          strcpy(msg->ddom, omsg->odom);
          if((mode == MODE_REPLYCOMMENT) AND omsg->fwdorig.net)
            strcpy(msg->replys, omsg->fwdmsgid);
          else
            strcpy(msg->replys, omsg->msgids);
          if(AA->isnet()) {
            if(omsg->messageid) {
              throw_free(msg->inreplyto);
              msg->inreplyto = throw_strdup(omsg->messageid);
            }
            if(CFG->switches.get(internetreply) or (*omsg->iaddr AND (strlen(omsg->iaddr) >= sizeof(Name)))) {
              strcpy(msg->iaddr, omsg->iaddr);
              strcpy(msg->igate, omsg->igate);
              if(*msg->igate) {
                msg->dest.set(msg->igate);
                char* ptr = strchr(msg->igate, ' ');
                if(ptr) {
                  strcpy(msg->realto, msg->to);
                  strcpy(msg->to, SkipWht(ptr));
                }
              }
            }
            else if(*omsg->iaddr AND (strlen(omsg->iaddr) < sizeof(Name))) {
              strcpy(msg->realto, msg->to);
              strcpy(msg->to, omsg->iaddr);
            }
          }
          else {
            int reflen = omsg->references ? strlen(omsg->references) : 0;
            int midlen = omsg->messageid ? strlen(omsg->messageid) : 0;
            msg->references = (char*)throw_calloc(1, reflen+1+midlen+1);
            if(omsg->references)
              strcpy(msg->references, omsg->references);
            if(omsg->messageid) {
              if(reflen)
                strcat(msg->references, " ");
              strcat(msg->references, omsg->messageid);
            }
            if(*msg->references == NUL)
              throw_release(msg->references);
          }
          if(CurrArea == OrigArea) {
            if((CFG->replylink == REPLYLINK_DIRECT) OR AA->isjam())
              reply_msgno = omsg->msgno;
            else if(CFG->replylink == REPLYLINK_CHAIN)
              GetLastLink(omsg, reply_msgno);
            msg->link.to_set(reply_msgno);
            if(msg->link.to() == omsg->msgno)
              omsg->link.first_set(msg->msgno);
          }

          ///////////
          /////////// Drop through
          ///////////

        case MODE_FORWARD:
          if(mode == MODE_FORWARD) {
            if(msg->fwdorig.net == 0) {
              strcpy(msg->fwdfrom, msg->By());
              msg->fwdorig = msg->orig;
              strcpy(msg->fwdto, msg->To());
              msg->fwddest = msg->dest;
              strxcpy(msg->fwdsubj, msg->re, sizeof(msg->fwdsubj));
              Area* fwdarea = AL.AreaIdToPtr(OrigArea);
              strcpy(msg->fwdarea, fwdarea->isecho() ? fwdarea->echoid() : "");
              strcpy(msg->fwdmsgid, msg->msgids);
            }
          }

          ///////////
          /////////// Drop through
          ///////////

        case MODE_NEW:
          strcpy(msg->by, AA->Username().name);
          msg->attr.nwm1();
          if(NOT AA->islocal())
            msg->attr.uns1();
          strcpy(msg->odom, CFG->aka[AkaMatch(&msg->orig, &AA->Aka().addr)].domain);
          break;

        case MODE_CHANGE:
          if(NOT AA->islocal())
            msg->attr.uns1();
          msg->attr.snt0();
          msg->attr.rcv0();
          break;
      }

      // Get random items

      switch(mode) {
        case MODE_NEW:
        case MODE_FORWARD:
          *msg->replys = NUL;
          throw_release(msg->references);
          throw_release(msg->inreplyto);
          if(AA->isnet()) {
            strcpy(msg->to, "");
            msg->dest.reset_fast();
            msg->odest.reset_fast();
          }
          if(NOT AA->isnet() OR (AA->isemail() AND strchr(AA->Whoto(), '@')))
            strcpy(msg->to, AA->Whoto());

          ///////
          /////// Drop through
          ///////

        case MODE_REPLYCOMMENT:
        case MODE_QUOTE:
        case MODE_REPLY:
        case MODE_CHANGE:
          AA->RandomizeData(mode);
          if(mode != MODE_CHANGE) {
            msg->orig = AA->Aka().addr;
            if(AA->isinternet())
              strcpy(msg->iorig, AA->Internetaddress());
            if(NOT strblank(AA->Username().name))
              strcpy(msg->by, AA->Username().name);
            if(CFG->switches.get(emptytearline))
              *msg->tearline = NUL;
          }
          break;
      }

      int forwstat = NO;
      if(msg->odest.net == 0)
        msg->odest = msg->dest;
      status = NO;
      if(NOT confirm) {
        strcpy(cmpmsg->by, msg->by);
        strcpy(cmpmsg->to, msg->to);
        strcpy(cmpmsg->re, msg->re);
        cmpmsg->attr = msg->attr;
        if(savedirect) {
          CheckSubject(msg, msg->re);
        }
        else {
          GMenuEditHeader MenuEditHeader;
          while((status = MenuEditHeader.Run(mode, msg)) == YES);
        }
      }

      if(status == NO) {
        if(mode == MODE_CHANGE OR mode == MODE_FORWARD) {
          status = MODE_SAVE;
          if(mode == MODE_FORWARD)
            forwstat = YES;
        }
        else {
          if(confirm) {
            throw_release(msg->txt);
            FILE* fp = fsopen(AddPath(CFG->goldpath, CFG->confirmfile), "rt", CFG->sharemode);
            if(fp) {
              LoadText(msg, AddPath(CFG->goldpath, CFG->confirmfile));
              fclose(fp);
            }
            if(msg->txt == NULL)
              msg->txt = throw_strdup("\r\rConfirmation Receipt\r\r");
            TokenXlat(mode, msg->txt, msg, omsg, CurrArea);
          }
          else
            CreateFileAddr(msg);
          msg->TextToLines(CFG->dispmargin-1);
          status = MODE_SAVE;
        }
      }
      MakeMsg2(mode, status, forwstat, topline, crosspost, xps, msg, omsg, cmpmsg);
    }

    if(status != MODE_SAVE)
      xps = crosspost = NO;

    if(status == MODE_SAVE) {

      if(*msg->iorig AND (strlen(msg->iorig) < sizeof(Name))) {
        strcpy(msg->realby, msg->by);
        strcpy(msg->by, msg->iorig);
      }

      if(mode == MODE_COPY)
        AA->SaveMsg(GMSG_NEW, msg);
      else
        MakeMsg3(mode, msg);

      // If message is a reply, update the links on the original
      if(CurrArea == OrigArea AND (mode == MODE_QUOTE OR mode == MODE_REPLYCOMMENT OR mode == MODE_REPLY)) {
        if(AA->Msgn.ToReln(reply_msgno)) {
          if(AA->LoadHdr(reply, reply_msgno)) {
            ulong replynext;
            int ok2save = FALSE;
            if(AA->issquish()) {
              if(reply->link.first()) {
                for(int r=0; r<reply->link.list_max()-1; r++) {
                  if(reply->link.list(r) == 0) {
                    reply->link.list_set(r, msg->msgno);
                    ok2save = TRUE;
                    break;
                  }
                }
              }
              else {
                reply->link.first_set(msg->msgno);
                ok2save = TRUE;
              }
            }
            else if(AA->isjam()) {
              if(reply->link.first()) {
                replynext = reply->link.first();
                do {
                  reply_msgno = replynext;
                  if(NOT AA->LoadHdr(reply, reply_msgno))
                    break;
                  replynext = reply->link.next();
                } while(reply->link.next());
                if(reply->link.next() == 0) {
                  reply->link.next_set(msg->msgno);
                  ok2save = TRUE;
                }
              }
              else {
                reply->link.first_set(msg->msgno);
                ok2save = TRUE;
              }
            }
            else {
              reply->link.first_set(msg->msgno);
              ok2save = TRUE;
            }
            if(ok2save) {
              reply->pcboard.reply_written = msg->written;
              AA->SaveHdr(GMSG_UPDATE, reply);
            }
          }
        }
      }

      // Ask if the original msg should be deleted while we're at it
      if(CurrArea == OrigArea and CFG->switches.get(askdelorig) and not confirm) {
        switch(mode) {
          case MODE_REPLY:
          case MODE_QUOTE:
          case MODE_REPLYCOMMENT:
            if(NOT AA->isecho()) {
              topline = 0;
              omsg->TextToLines(CFG->dispmargin-1);
              if(omsg->attr.rot())
                Rot13(omsg);
              HeaderView.Use(AA, omsg);
              HeaderView.Paint();
              BodyView.Use(AA, omsg, topline);
              BodyView.Paint();
              GMenuDelorig MenuDelorig;
              if(MenuDelorig.Run()) {
                AA->Mark.Del(omsg->msgno);
                AA->PMrk.Del(omsg->msgno);
                AA->DeleteMsg(omsg, DIR_PREV);
              }
            }
            break;
        }
      }
    }

    if(CurrArea != OrigArea) {
      AA->UpdateAreadata();
      AA->Close();
      AL.SetActiveAreaId(OrigArea);
      AA->RandomizeData();
    }
    AA->UpdateAreadata();
    EDIT->HardLines(hardlines);
    for(int n=0; n<fspecs; n++)
      throw_release(fspec[n].fblk);
    fspecs = specfiles = 0;
    throw_release(fspec);

    if(crosspost) {
      if(xps) {
        update_statuslinef(LNG->Crossposting, AL[xparea[xps-1]]->echoid());
      }
    }
  } while(xps);

  _in_editor = NO;

  CFG->tplno = 0;

  // Restore original aka
  AA->SetAka(origaka);

  ResetMsg(omsg);
  ResetMsg(cmpmsg);
  ResetMsg(reply);
  ResetMsg(msg);

  throw_free(cmpmsg);
  throw_free(reply);
  throw_free(msg);

  LoadLanguage(AA->Loadlanguage());

  GFTRK(NULL);
}
    

//  ------------------------------------------------------------------

