
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geqwks.cpp,v 1.1.1.1 1998/11/21 19:38:02 odinn Exp $
//  ------------------------------------------------------------------
//  QWK packet handling.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include "gs_qwk.h"


//  ------------------------------------------------------------------

static void ReadGldFile() {

  if(QWK->FoundBBS()) {

    gfile fp;
    Path gldfile;

    QWK->ResetConfNo();

    sprintf(gldfile, "%s%s.GLD", CFG->goldpath, QWK->BbsID());
    fp.fopen(gldfile, "rt");
    if(fp.isopen()) {
      char* key;
      char* val;
      char buf[256];
      while(fp.fgets(buf, sizeof(buf))) {
        val = strtrim(buf);
        getkeyval(&key, &val);
        strtrim(StripQuotes(val));
        if(QWK->FindEcho(val))
          QWK->ConfNo(atoi(key));
      }
      fp.fclose();
    }
  }
}


//  ------------------------------------------------------------------

int ImportQWK() {

  if(NOT *QWK->ImportPath())
    return 0;

  gfile fp;
  gfile fpb;   // For BBSID.GLD
  Path file;
  Path gldfile;
  char bbsid[9] = {""};

  // Parse the control file
  strcpy(file, AddPath(QWK->ImportPath(), "CONTROL.DAT"));
  fp.fopen(file, "rt");
  if(fp.isopen()) {
    char buf[256];
    int line = 0;
    int confno = 0;
    int confcnt = 0;
    int confnos = 0;
    while(fp.fgets(buf, sizeof(buf))) {

      line++;
      strtrim(buf);

      if((line >= 12) AND (confcnt < confnos)) {
        if(line % 2) {
          if(fpb.isopen())
            fpb.printf("%u \"%s\"\n", confno, buf);
          confcnt++;
        }
        else
          confno = atoi(buf);
      }
      else if(line == 5) {
        char* ptr = strchr(buf, ',');
        if(ptr) {
          strxcpy(bbsid, SkipWht(ptr+1), 9);
          sprintf(gldfile, "%s%s.GLD", CFG->goldpath, bbsid);
          fpb.fopen(gldfile, "wt");
        }
      }
      else if(line == 11)
        confnos = atoi(buf) + 1;
    }
    if(fpb.isopen())
      fpb.fclose();
    fp.fclose();
    remove(file);
  }

  if(strblank(bbsid))
    return 0;

  QWK->FindBBS(bbsid);
  ReadGldFile();

  int imported = 0;

  OrigArea = CurrArea = -1;

  strcpy(file, AddPath(QWK->ImportPath(), "MESSAGES.DAT"));
  fp.fopen(file, "rb");
  if(fp.isopen()) {

    // Skip past product info header
    fp.fseekset(sizeof(QWKHdr));

    QWKHdr hdr;

    GMsg* msg = (GMsg*)throw_calloc(1, sizeof(GMsg));

    // Read each message, header first
    int tosstobadmsgs = FALSE;
    int lastconfno = -1;
    int more;
    do {

      ResetMsg(msg);

      memset(&hdr, 0, sizeof(QWKHdr));
      more = 1 == fp.fread(&hdr, sizeof(QWKHdr));
      if(more) {

        char blocks[7];
        strxcpy(blocks, hdr.blocks, 7);
        uint msglen = atoi(blocks);
        if(msglen == 0) {
          //more = FALSE;
          break;
        }
        msglen = (msglen-1)*128;

        if(hdr.confno != lastconfno) {
          lastconfno = hdr.confno;
          if(QWK->FindEcho(hdr.confno)) {
            int areano = AL.AreaEchoToNo(QWK->EchoID());
            if(areano != -1) {
              CurrArea = AL.AreaNoToId(areano);
              tosstobadmsgs = FALSE;
            }
          }
          else {
            int areano = AL.AreaEchoToNo(QWK->BadMsgs());
            if(areano != -1) {
              CurrArea = AL.AreaNoToId(areano);
              tosstobadmsgs = TRUE;
            }
            else {
              tosstobadmsgs = -1;
              fp.fseek(msglen, SEEK_CUR);
            }
          }
        }

        if(tosstobadmsgs != -1) {

          if(OrigArea != CurrArea) {
            if(AA->isopen()) {
              AA->Unlock();
              AA->Close();
            }
            AL.SetActiveAreaId(CurrArea);
            OrigArea = CurrArea;
            AA->Open();
            AA->Lock();
            AA->RandomizeData();
          }

          // Convert header
          msg->orig = msg->oorig = AA->aka();
          msg->dest = msg->odest = AA->aka();
          msg->attr.pvt(hdr.status == '*' OR hdr.status == '+');
          msg->attr.rcv(hdr.status == '+' OR hdr.status == '-' OR hdr.status == '`' OR hdr.status == '^' OR hdr.status == '#');
          msg->attr.del(hdr.activestatus == 226);
          strtrim(strxcpy(msg->by, hdr.from, 26));
          strtrim(strxcpy(msg->to, hdr.to, 26));
          strtrim(strxcpy(msg->re, hdr.subject,  26));
          int _year, _month, _day, _hour, _minute;
          sscanf(hdr.date, "%d%*c%d%*c%2d", &_month, &_day, &_year);
          sscanf(hdr.time, "%d%*c%2d", &_hour, &_minute);
          Tm _tm;
          _tm.tm_year  = (_year < 80) ? (_year+100) : _year;
          _tm.tm_mon   = _month - 1;
          _tm.tm_mday  = _day;
          _tm.tm_hour  = _hour;
          _tm.tm_min   = _minute;
          _tm.tm_sec   = 0;
          _tm.tm_isdst = 0;
          msg->written = gmktime(&_tm);
          msg->arrived = gtime(NULL);

          // Read message text
          char* txtptr = msg->txt = (char*)throw_calloc(1, msglen+128);
          if(tosstobadmsgs) {
            sprintf(msg->txt, "AREA:%s_%u\r", bbsid, hdr.confno);
            txtptr += strlen(msg->txt);
          }
          fp.fread(txtptr, msglen);
          strtrim(txtptr);
          strchg(txtptr, 0xE3, 0x0D);

          imported++;
          AA->istossed = TRUE;
          update_statuslinef("%s: %u", AA->echoid(), imported);

          msg->attr.tou0();
          msg->TextToLines(CFG->dispmargin-1);
          if(msg->messageid OR msg->references OR msg->inreplyto OR *msg->ifrom) {
            char kbuf[256];
            uint txtlen = strlen(msg->txt);
            if(*msg->ifrom) {
              INam _fromname;
              IAdr _fromaddr;
              ParseInternetAddr(msg->ifrom, _fromname, _fromaddr);
              if(CFG->internetgate.addr.net) {
                char abuf[40];
                sprintf(kbuf, "\x1""REPLYTO %s %s\r""\x1""REPLYADDR %s\r",
                  CFG->internetgate.addr.make_string(abuf),
                  *CFG->internetgate.name ? CFG->internetgate.name : "UUCP",
                  _fromaddr
                );
                uint addlen = strlen(kbuf);
                msg->txt = (char*)throw_realloc(msg->txt, txtlen+addlen+1);
                memmove(msg->txt+addlen, msg->txt, txtlen+1);
                memcpy(msg->txt, kbuf, addlen);
                txtlen += addlen;
              }
            }
            if(msg->references) {
              char* mptr = msg->references;
              char* sptr = mptr;
              while(*mptr) {
                if(NOT strnieql(mptr, "<NOMSGID_", 9)) {
                  sptr = strpbrk(mptr, " ,");
                  if(sptr == NULL)
                    sptr = mptr + strlen(mptr);
                  while(((*sptr == ' ') OR (*sptr == ',')) AND *sptr)
                    *sptr++ = NUL;
                  CvtMessageIDtoMSGID(mptr, kbuf, AA->echoid(), "REPLY");
                  strcat(kbuf, "\r");
                  uint addlen = strlen(kbuf);
                  msg->txt = (char*)throw_realloc(msg->txt, txtlen+addlen+1);
                  memmove(msg->txt+addlen, msg->txt, txtlen+1);
                  memcpy(msg->txt, kbuf, addlen);
                  txtlen += addlen;
                }
                mptr = sptr;
              }
            }
            if(msg->inreplyto) {
              if(NOT strnieql(msg->inreplyto, "<NOMSGID_", 9)) {
                CvtMessageIDtoMSGID(msg->inreplyto, kbuf, AA->echoid(), "REPLY");
                strcat(kbuf, "\r");
                uint addlen = strlen(kbuf);
                msg->txt = (char*)throw_realloc(msg->txt, txtlen+addlen+1);
                memmove(msg->txt+addlen, msg->txt, txtlen+1);
                memcpy(msg->txt, kbuf, addlen);
                txtlen += addlen;
              }
            }
            if(msg->messageid) {
              if(NOT strnieql(msg->messageid, "<NOMSGID_", 9)) {
                CvtMessageIDtoMSGID(msg->messageid, kbuf, AA->echoid(), "MSGID");
                strcat(kbuf, "\r");
                uint addlen = strlen(kbuf);
                msg->txt = (char*)throw_realloc(msg->txt, txtlen+addlen+1);
                memmove(msg->txt+addlen, msg->txt, txtlen+1);
                memcpy(msg->txt, kbuf, addlen);
                //txtlen += addlen;
              }
            }
          }

          if(*msg->realto AND (strieql(msg->to, "ALL") OR NOT *msg->to))
            strcpy(msg->to, msg->realto);

          AA->SaveMsg(GMSG_NEW, msg);
        }
      }
    } while(more);

    if(AA->isopen()) {
      AA->Unlock();
      AA->Close();
    }

    ResetMsg(msg);
    throw_free(msg);

    fp.fclose();
    remove(file);

    if(*QWK->TossLog()) {
      fp.fopen(QWK->TossLog(), "at");
      if(fp.isopen()) {
        uint na = 0;
        while(na < AL.total) {
          if(AL[na]->istossed) {
            AL[na]->istossed = FALSE;
            AL[na]->isunreadchg = true;
            fp.printf("%s\n", AL[na]->echoid());
          }
          na++;
        }
        fp.fclose();
      }
    }

    if(imported AND *QWK->ReplyLinker()) {
      char buf[256];
      sprintf(buf, LNG->Replylinker, QWK->ReplyLinker());
      ShellToDos(QWK->ReplyLinker(), buf, LGREY|_BLACK, YES);
    }
  }

  if(imported)
    startupscan_success = true;

  return imported;
}


//  ------------------------------------------------------------------

int ExportQwkMsg(GMsg* msg, gfile& fp, int confno, int& pktmsgno) {

  // Prepare for Return Receipt Request
  char msg__re[26];
  if(QWK->ReceiptAllowed() AND msg->attr.rrq())
    sprintf(msg__re, "RRR%-22.22s", msg->re);
  else
    strxcpy(msg__re, msg->re, 26);

  // Build QWK header
  QWKHdr hdr;
  char buf[26];
  int tolen = strlen(msg->to);  tolen = MinV(25,tolen);
  int bylen = strlen(msg->by);  bylen = MinV(25,bylen);
  int relen = strlen(msg__re);  relen = MinV(25,relen);
  memset(&hdr, ' ', sizeof(QWKHdr));
  hdr.status = msg->attr.pvt() ? '*' : ' ';
  itoa(confno, buf, 10);
  memcpy(hdr.msgno, buf, strlen(buf));
  Tm* _tm = glocaltime(&msg->written);
  int _year = (_tm->tm_year >= 100) ? (_tm->tm_year-100) : _tm->tm_year;
  sprintf(buf, "%02d-%02d-%02d", _tm->tm_mon+1, _tm->tm_mday, _year);
  memcpy(hdr.date, buf, 8);
  sprintf(buf, "%02d:%02d", _tm->tm_hour, _tm->tm_min);
  memcpy(hdr.time, buf, 5);
  strxcpy(buf, msg->to, tolen+1);
  if(NOT QWK->MixCaseAllowed())
    strupr(buf);
  memcpy(hdr.to, buf, tolen);
  strxcpy(buf, msg->by, bylen+1);
  if(NOT QWK->MixCaseAllowed())
    strupr(buf);
  memcpy(hdr.from, buf, bylen);
  memcpy(hdr.subject, msg__re, relen);
  hdr.activestatus = '\xE1';
  hdr.confno = (word)confno;
  hdr.pktmsgno = (word)++pktmsgno;

  // Write preliminary header
  fp.fwrite(&hdr, sizeof(QWKHdr));

  // Write body
  #if 0
  strchg(msg->txt, 0x0D, 0xE3);
  fp.fwrite(msg->txt, msglen);
  if(endlen) {
    memset(&hdr, ' ', sizeof(QWKHdrT));
    fp.fwrite(&hdr, 128-endlen);
  }
  strchg(msg->txt, 0xE3, 0x0D);
  #endif

  int level = 0;
  if(CharTable)
    level = CharTable->level ? CharTable->level : 2;

  char mbuf[512];
  uint msglen = 0;

  if(msg->charsetencoding & GCHENC_MNE) {
    if(NOT striinc("MNEMONIC", CharTable->export))
      LoadCharset(CFG->xlatlocalset, "MNEMONIC");
  }
  else if(striinc("LATIN1QP", msg->charset)) {
    if(NOT striinc("LATIN1QP", CharTable->export))
      LoadCharset(CFG->xlatlocalset, "LATIN1QP");
  }
  else if(striinc("LATIN-1", msg->charset)) {
    if(NOT striinc("LATIN-1", CharTable->export))
      LoadCharset(CFG->xlatlocalset, "LATIN-1");
  }

  char qwkterm = '\xE3';
  //int lineno = 0;

  // Process kludges and write header lines
  Line* line = msg->lin;
  while(line) {
    if(line->type & GLINE_KLUDGE) {
      if(AA->isinternet()) {
        if((line->kludge == GKLUD_RFC) OR (line->kludge == 0)) {
          XlatStr(mbuf, line->text, level, CharTable);
          msglen += fp.printf("%s%c", mbuf, qwkterm);
          //lineno++;
        }
        else if(line->type & GLINE_WRAP) {
          while(line->next AND (line->type & GLINE_WRAP))
            line = line->next;
        }
      }
      else {
        if((line->type & GLINE_KLUDGE) AND QWK->KludgesAllowed()) {
          XlatStr(mbuf, line->text, level, CharTable);
          msglen += fp.printf("%s%c", mbuf, qwkterm);
          //lineno++;
        }
      }
    }
    line = line->next;
  }

  // Write blank line after header lines
  if(AA->Internetrfcbody()) {
    msglen += fp.printf("%c", qwkterm);
    //lineno++;
  }

  // Write all message lines
  line = msg->lin;
  while(line) {
    if(NOT(line->type & GLINE_KLUDGE)) {
      XlatStr(mbuf, line->text, level, CharTable);
      msglen += fp.printf("%s%c", mbuf, qwkterm);
      //lineno++;
    }
    line = line->next;
  }

  // Calculate blocks
  uint endlen = msglen % 128;
  uint blocks = 1+(msglen/128)+(endlen?1:0);
  itoa(blocks, buf, 10);
  memcpy(hdr.blocks, buf, strlen(buf));

  // Write padding spaces at the end if necessary
  if(endlen) {
    char padding[128];
    memset(padding, ' ', 128);
    fp.fwrite(padding, 128-endlen);
  }

  // Re-write the header
  fp.fseek(-(blocks*128), SEEK_CUR);
  fp.fwrite(&hdr, sizeof(QWKHdr));
  fp.fseek((blocks-1)*128, SEEK_CUR);

  // Mark msg as sent
  msg->attr.snt1();
  msg->attr.scn1();
  msg->attr.uns0();
  AA->SaveHdr(GMSG_UPDATE, msg);

  return 1;
}


//  ------------------------------------------------------------------

int ExportQwkArea(int areano, gfile& fp, int confno, int& pktmsgno) {

  int exported = 0;

  AL.SetActiveAreaNo(areano);

  AA->Open();
  AA->Lock();
  AA->RandomizeData();

  GMsg* msg = (GMsg*)throw_calloc(1, sizeof(GMsg));

  for(uint n=0; n<AA->Expo.Count(); n++) {
    if(AA->LoadMsg(msg, AA->Expo[n], 80)) {
      if(msg->attr.uns()) {
        exported += ExportQwkMsg(msg, fp, confno, pktmsgno);
        update_statuslinef("%s: %u", AA->echoid(), exported);
      }
    }
  }

  ResetMsg(msg);
  throw_free(msg);

  AA->Unlock();
  AA->Close();

  AA->Expo.ResetAll();

  return exported;
}


//  ------------------------------------------------------------------

int ExportQWK() {

  if(NOT *QWK->ExportPath())
    return 0;

  int exported = 0;

  gfile fp;
  Path scanfile;

  // Get the scan list
  strcpy(scanfile, AddPath(CFG->goldpath, "GOLDQWK.LST"));
  fp.fopen(scanfile, "rt");
  if(fp.isopen()) {
    char buf[256];
    while(fp.fgets(buf, sizeof(buf))) {
      char* ptr = strchr(buf, ' ');
      if(ptr) {
        *ptr++ = NUL;
        int a = AL.AreaEchoToNo(buf);
        if(a != -1 AND AL[a]->isqwk())
          AL[a]->Expo.Add(atol(ptr));
      }
    }
    fp.fclose();
  }

  // Export from the QWK areas
  if(QWK->FirstBBS()) {
    do {
      ReadGldFile();
      Path replyfile;
      int pktmsgno = 0;
      if(QWK->FirstConf()) {
        sprintf(replyfile, "%s%s.MSG", QWK->ExportPath(), QWK->BbsID());
        fp.fopen(replyfile, "wb");
        if(fp.isopen()) {
          char firstrec[128];
          memset(firstrec, ' ', 128);
          memcpy(firstrec, QWK->BbsID(), strlen(QWK->BbsID()));
          fp.fwrite(firstrec, 128);
          pktmsgno = 0;
        }
        do {
          int a = AL.AreaEchoToNo(QWK->EchoID());
          if(AL[a]->Expo.Count()) {
            if(QWK->ConfNo() != -1)
              exported += ExportQwkArea(a, fp, QWK->ConfNo(), pktmsgno);
          }
        } while(QWK->NextConf());
      }
      if(fp.isopen()) {
        fp.fclose();
        if(pktmsgno == 0)
          remove(replyfile);
      }
    } while(QWK->NextBBS());
  }

  // Delete the scanfile
  remove(scanfile);

  return exported;
}


//  ------------------------------------------------------------------

Qwk::Qwk() {

  bbs = bbsp = NULL;
  mapp = NULL;
  bbss = 0;
}


//  ------------------------------------------------------------------

Qwk::~Qwk() {

  Reset();
}


//  ------------------------------------------------------------------

void Qwk::Reset() {

  for(int n=0; n<bbss; n++)
    throw_release(bbs[n].map);
  throw_release(bbs);
  bbsp = NULL;
  mapp = NULL;
  bbss = 0;
}


//  ------------------------------------------------------------------

void Qwk::Save(gfile& fp) {

  fp.fwrite(&cfg, sizeof(QwkCfg));
  fp.fwrite(&bbss, sizeof(int));
  fp.fwrite(bbs, sizeof(QwkBbs), bbss);
  bbsp = bbs;
  for(int n=0; n<bbss; n++,bbsp++)
    fp.fwrite(bbsp->map, sizeof(QwkMap), bbsp->maps);
}


//  ------------------------------------------------------------------

void Qwk::Load(gfile& fp) {

  fp.fread(&cfg, sizeof(QwkCfg));
  fp.fread(&bbss, sizeof(int));
  if(bbss) {
    bbs = (QwkBbs*)throw_calloc(bbss, sizeof(QwkBbs));
    fp.fread(bbs, sizeof(QwkBbs), bbss);
    bbsp = bbs;
    int n;
    for(n=0; n<bbss; n++,bbsp++)
      bbsp->map = NULL;
    bbsp = bbs;
    for(n=0; n<bbss; n++,bbsp++) {
      bbsp->map = (QwkMap*)throw_calloc(bbsp->maps, sizeof(QwkMap));
      fp.fread(bbsp->map, sizeof(QwkMap), bbsp->maps);
    }
  }
}


//  ------------------------------------------------------------------

void Qwk::AddBBS(char* bbsid) {

  bbs = (QwkBbs*)throw_realloc(bbs, (bbss+1)*sizeof(QwkBbs));
  bbsp = bbs + bbss++;
  strxcpy(bbsp->bbsid, bbsid, sizeof(bbsp->bbsid));
  bbsp->kludges = FALSE;
  bbsp->mixcase = FALSE;
  bbsp->receipt = FALSE;
  bbsp->maxlines = -1;
  bbsp->map = NULL;
  bbsp->maps = 0;
  mapp = NULL;
}


//  ------------------------------------------------------------------

void Qwk::AddMap(char* bbsid, char* echoid, char* confname, int confno) {

  FindAddBBS(bbsid);
  bbsp->map = (QwkMap*)throw_realloc(bbsp->map, (bbsp->maps+1)*sizeof(QwkMap));
  mapp = bbsp->map + bbsp->maps++;
  strxcpy(mapp->confname, confname, sizeof(mapp->confname));
  strxcpy(mapp->echoid, echoid, sizeof(mapp->echoid));
  mapp->confno = confno;
}


//  ------------------------------------------------------------------

int Qwk::FindBBS(char* bbsid) {

  bbsp = bbs;
  for(int n=0; n<bbss; n++,bbsp++) {
    if(strieql(bbsid, bbsp->bbsid)) {
      mapp = bbsp->map;
      return TRUE;
    }
  }
  mapp = NULL;
  return FALSE;
}


//  ------------------------------------------------------------------

void Qwk::FindAddBBS(char* bbsid) {

  if(NOT FindBBS(bbsid))
    AddBBS(bbsid);
}


//  ------------------------------------------------------------------

int Qwk::FirstBBS() {

  if(bbs) {
    bbsp = bbs;
    bbsn = 1;
    return TRUE;
  }

  bbsp = NULL;
  bbsn = 0;
  return FALSE;
}


//  ------------------------------------------------------------------

int Qwk::NextBBS() {

  if(bbsp AND (bbsn < bbss)) {
    bbsp++;
    bbsn++;
    return TRUE;
  }

  bbsp = NULL;
  bbsn = 0;
  return FALSE;
}


//  ------------------------------------------------------------------

int Qwk::FindConf(char* echoid) {

  if(bbsp) {
    mapp = bbsp->map;
    for(int n=0; n<bbsp->maps; n++,mapp++)
      if(strieql(echoid, mapp->echoid))
        return TRUE;
  }

  mapp = NULL;
  return FALSE;
}


//  ------------------------------------------------------------------

int Qwk::FindEcho(char* confname) {
           
  if(bbsp) {
    mapp = bbsp->map;
    for(int n=0; n<bbsp->maps; n++,mapp++)
      if(strieql(confname, mapp->confname))
        return TRUE;
  }

  mapp = NULL;
  return FALSE;
}


//  ------------------------------------------------------------------

int Qwk::FindEcho(int confno) {

  if(bbsp) {
    mapp = bbsp->map;
    for(int n=0; n<bbsp->maps; n++,mapp++)
      if(confno == mapp->confno)
        return TRUE;
  }

  mapp = NULL;
  return FALSE;
}


//  ------------------------------------------------------------------

int Qwk::FirstConf() {

  if(bbsp AND bbsp->map) {
    mapp = bbsp->map;
    mapn = 1;
    return TRUE;
  }

  mapp = NULL;
  mapn = 0;
  return FALSE;
}


//  ------------------------------------------------------------------

int Qwk::NextConf() {

  if(mapp AND (mapn < bbsp->maps)) {
    mapp++;
    mapn++;
    return TRUE;
  }

  mapp = NULL;
  mapn = 0;
  return FALSE;
}


//  ------------------------------------------------------------------

char* Qwk::BbsID() {

  return bbsp ? bbsp->bbsid : "";
}


//  ------------------------------------------------------------------

char* Qwk::ConfName() {

  return mapp ? mapp->confname : "";
}


//  ------------------------------------------------------------------

int Qwk::ConfNo(int set) {

  if(mapp) {
    if(set != -2)
      mapp->confno = set;
    return mapp->confno;
  }
  return -1;
}


//  ------------------------------------------------------------------

char* Qwk::EchoID() {

  return mapp ? mapp->echoid : "";
}


//  ------------------------------------------------------------------

int Qwk::KludgesAllowed(int set) {

  if(bbsp) {
    if(set != -1)
      bbsp->kludges = set;
    return bbsp->kludges;
  }

  return FALSE;
}


//  ------------------------------------------------------------------

int Qwk::MixCaseAllowed(int set) {

  if(bbsp) {
    if(set != -1)
      bbsp->mixcase = set;
    return bbsp->mixcase;
  }

  return FALSE;
}


//  ------------------------------------------------------------------

int Qwk::ReceiptAllowed(int set) {

  if(bbsp) {
    if(set != -1)
      bbsp->receipt = set;
    return bbsp->receipt;
  }

  return FALSE;
}


//  ------------------------------------------------------------------

int Qwk::MaxLines(int set) {

  if(bbsp) {
    if(set != -1)
      bbsp->maxlines = set;
    return bbsp->maxlines;
  }

  return 0;
}


//  ------------------------------------------------------------------

void Qwk::ResetConfNo(int set) {

  if(bbsp) {
    mapp = bbsp->map;
    for(int n=0; n<bbsp->maps; n++,mapp++)
      mapp->confno = set;
  }
}


//  ------------------------------------------------------------------

char* Qwk::BadMsgs(char* set) {

  if(set)
    strxcpy(cfg.badmsgs, set, sizeof(Echo));

  return cfg.badmsgs;
}


//  ------------------------------------------------------------------

char* Qwk::ExportPath(char* set) {

  if(set)
    PathCopy(cfg.exportpath, set);

  return cfg.exportpath;
}


//  ------------------------------------------------------------------

char* Qwk::ImportPath(char* set) {

  if(set)
    PathCopy(cfg.importpath, set);

  return cfg.importpath;
}


//  ------------------------------------------------------------------

char* Qwk::ReplyLinker(char* set) {

  if(set)
    strxcpy(cfg.replylinker, set, sizeof(Path));

  return cfg.replylinker;
}


//  ------------------------------------------------------------------

char* Qwk::TossLog(char* set) {

  if(set)
    strxcpy(cfg.tosslog, set, sizeof(Path));

  return cfg.tosslog;
}


//  ------------------------------------------------------------------

