
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gerand.cpp,v 1.2 1999/02/02 02:09:44 damware Exp $
//  ------------------------------------------------------------------
//  Random system and control info handling.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

char* GetRandomLine(char* __buf, size_t __bufsize, const char* __file) {
  
  Path textfile, indexfile;
  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  strcpy(textfile, __file);
  _splitpath(textfile, drive, dir, file, ext);
  strcpy(ext, ".sdx");
  _makepath(indexfile, drive, dir, file, ext);

  FILE* fp = fsopen(AddPath(CFG->goldpath, textfile), "rb", CFG->sharemode);
  if(fp) {

    // Check if index exists or if it is older than the textfile
    const char* idxfile = AddPath(CFG->goldpath, indexfile);
    int idxexist = fexist(idxfile);
    if(idxexist)
      if(FiletimeCmp(textfile, idxfile) > 0)
        idxexist = FALSE;

    FILE* fpi = fsopen(idxfile, idxexist ? "rb" : "wb+", CFG->sharemode);
    if(fpi) {

      // Create index if one was missing
      if(NOT idxexist) {
        setvbuf(fp, NULL, _IOFBF, 32000);
        setvbuf(fpi, NULL, _IOFBF, 16000);
        long fpos = 0;
        char buf[512];
        while(fgets(buf, sizeof(buf), fp)) {
          fwrite(&fpos, sizeof(long), 1, fpi);
          fpos += strlen(buf);
        }
      }

      // Get random line if there is at least one
      int _lines = (int)(fsize(fpi)/sizeof(long));
      if(_lines > 0) {

        // Select a random line
        long _offset = 0;
        long _ourline = rand() % _lines;
        fseek(fpi, _ourline*(long)sizeof(long), SEEK_SET);
        fread(&_offset, sizeof(long), 1, fpi);

        // Get it
        char buf[512];
        fseek(fp, _offset, SEEK_SET);
        fgets(buf, sizeof(buf), fp);
        StripQuotes(strbtrim(buf));
        strxcpy(__buf, buf, __bufsize);
      }
      fclose(fpi);
    }
    fclose(fp);
  }

  return __buf;
}


//  ------------------------------------------------------------------

void Area::InitData() {

  // Set defaults
  adat->aka = CFG->aka[GetAkaNo(aka())];
  adat->akamatching = (CFG->switches.get(akamatchnet) and isnet()) or
                      (CFG->switches.get(akamatchecho) and isecho()) or
                      (CFG->switches.get(akamatchlocal) and islocal());
  adat->attributes = attr();
  adat->areacopyaddid = CFG->areacopyaddid;
  adat->areacopydirect = CFG->areacopydirect;
  strcpy(adat->areacopyto, CFG->areacopyto);
  adat->areafreqdirect = CFG->areafreqdirect;
  strcpy(adat->areafreqto, CFG->areafreqto);
  adat->areareplydirect = CFG->areareplydirect;
  strcpy(adat->areareplyto, CFG->areareplyto);
  if(isemail())
    adat->ctrlinfo = CFG->ctrlinfoemail;
  else if(isnewsgroup())
    adat->ctrlinfo = CFG->ctrlinfonews;
  else if(isnet())
    adat->ctrlinfo = CFG->ctrlinfonet;
  else if(isecho())
    adat->ctrlinfo = CFG->ctrlinfoecho;
  else
    adat->ctrlinfo = CFG->ctrlinfolocal;
  adat->edithardterm = EDIT->HardTerm();
  adat->editmixcase = EDIT->MixCase();
  adat->forcetemplate = CFG->forcetemplate;
  strcpy(adat->gif, CFG->gifs ? CFG->gif[CFG->gifno] : "");
  strcpy(adat->inputfile, CFG->inputfile);
  strcpy(adat->internetaddress, CFG->internetaddress);
  memcpy(&adat->internetgate, &CFG->internetgate, sizeof(Node));
  adat->internetmsgid = CFG->internetmsgid;
  adat->internetrfcbody = CFG->internetrfcbody;
  strcpy(adat->loadlanguage, CFG->loadlanguage);
  adat->msglistdate = CFG->msglistdate;
  adat->msglistfast = CFG->msglistfast;
  adat->msglistfirst = CFG->msglistfirst;
  adat->msglistheader = CFG->msglistheader;
  adat->msglistwidesubj = CFG->msglistwidesubj;
  strcpy(adat->netname, CFG->netname);
  strcpy(adat->organization, CFG->organization);
  strcpy(adat->origin, CFG->origins ? CFG->origin[CFG->originno] : "");
  strcpy(adat->outputfile, CFG->outputfile);
  memcpy(&adat->play, &CFG->play, sizeof(GPlay));
  strcpy(adat->quotebuffile, CFG->quotebuffile);
  strcpy(adat->quotechars, CFG->quotechars);
  strcpy(adat->quotestring, CFG->quotestring);
  adat->quotewraphard = CFG->quotewraphard;
  adat->replyre = EDIT->ReplyRe();
  if(*CFG->searchfor)
    strcpy(adat->searchfor, CFG->searchfor);
  strcpy(adat->tagline, CFG->taglines ? CFG->tagline[CFG->taglineno] : "");
  adat->taglinechar = CFG->taglinechar;
  strcpy(adat->taglinefile, CFG->taglinefile);
  adat->taglinesupport = CFG->taglinesupport;
  strcpy(adat->tearline, CFG->tearline);
  strcpy(adat->tpl, (CFG->tplno and (CFG->tplno < CFG->tpls)) ? CFG->tpl[CFG->tplno].file : "");
  adat->templatematch = CFG->templatematch;
  adat->twitmode = CFG->twitmode;
  adat->usefwd = CFG->usefwd;
  strcpy(adat->username.name, CFG->usernames ? CFG->username[CFG->usernameno].name : "");
  adat->username.addr = adat->aka.addr;
  adat->viewhidden = CFG->viewhidden;
  adat->viewkludge = CFG->viewkludge;
  adat->viewquote = CFG->viewquote;
  strcpy(adat->nickname, CFG->nickname);
  strcpy(adat->whoto, CFG->whoto);
  strcpy(adat->xlatexport, CFG->xlatexport);
  strcpy(adat->xlatimport, CFG->xlatimport);

  LoadLanguage(adat->loadlanguage);
}


//  ------------------------------------------------------------------
//  Get data from the Random System

void Area::RandomizeData(int mode) {

  if(NOT adat) {
    adat = (AreaData*)throw_calloc(1, sizeof(AreaData));
    InitData();
  }
  else {       // reset to global defaults
    strcpy(adat->tpl, (CFG->tplno and (CFG->tplno < CFG->tpls)) ? CFG->tpl[CFG->tplno].file : "");
  }

  char _groupid[10];
  if(CFG->areafilegroups AND groupid()) {
    if(groupid() & 0x8000u)
      sprintf(_groupid, "#%u", groupid() & 0x7FFF);
    else {
      _groupid[0] = (char)(isalpha(groupid()) ? groupid() : NUL);
      _groupid[1] = NUL;
    }
  }
  else {
    *_groupid = NUL;
  }

  Echo _echoid;
  strcpy(_echoid, echoid());

  const char* found = GRP->SetGrp(_echoid);
  int curgrp = GRP->curgrp;
  if(*_groupid) {
    found = GRP->SetGrp(_groupid);
    if(found AND (strieql(found, "*") OR ((curgrp >= 0) AND (curgrp < GRP->curgrp))))
      found = NULL;
    if(NOT found)
      found = GRP->SetGrp(_echoid);
  }

  if(found) {

    gfile fp;
    fp.fopen(CFG->randged, "rb", CFG->sharemode);
    if(fp.isopen()) {

      GRP->SetFp(fp);

      char buf[256];

      if(GRP->GetItm(GRP_AKA, &adat->aka, sizeof(adat->aka)))
        SetAka(adat->aka.addr);

      GRP->GetItm(GRP_AKAMATCHING, &adat->akamatching, sizeof(adat->akamatching));

      if(GRP->GetItm(GRP_ATTRIBUTES, buf, sizeof(buf))) {
        if(isnet())
          adat->attributes = CFG->attribsnet;
        else if(isecho())
          adat->attributes = CFG->attribsecho;
        else
          adat->attributes = CFG->attribslocal;
        GetAttribstr(&adat->attributes, buf);
      }

      GRP->GetItm(GRP_AREACOPYADDID, &adat->areacopyaddid, sizeof(adat->areacopyaddid));
      GRP->GetItm(GRP_AREACOPYDIRECT, &adat->areacopydirect, sizeof(adat->areacopydirect));
      GRP->GetItm(GRP_AREACOPYTO, adat->areacopyto, sizeof(adat->areacopyto));

      GRP->GetItm(GRP_AREAFREQDIRECT, &adat->areafreqdirect, sizeof(adat->areafreqdirect));
      GRP->GetItm(GRP_AREAFREQTO, adat->areafreqto);

      GRP->GetItm(GRP_AREAREPLYDIRECT, &adat->areareplydirect, sizeof(adat->areareplydirect));
      GRP->GetItm(GRP_AREAREPLYTO, adat->areareplyto, sizeof(adat->areareplyto));
    
      GRP->GetItm(GRP_CTRLINFO, &adat->ctrlinfo, sizeof(adat->ctrlinfo));

      GRP->GetItm(GRP_EDITHARDTERM, &adat->edithardterm, sizeof(adat->edithardterm));
      GRP->GetItm(GRP_EDITMIXCASE, &adat->editmixcase, sizeof(adat->editmixcase));
      GRP->GetItm(GRP_EDITREPLYRE, &adat->replyre, sizeof(adat->replyre));

      GRP->GetItm(GRP_FORCETEMPLATE, &adat->forcetemplate, sizeof(adat->forcetemplate));

      if(GRP->GetItm(GRP_GIF, buf, sizeof(buf))) {
        if(*buf == '@')
          GetRandomLine(buf, sizeof(buf), buf+1);
        strxcpy(adat->gif, buf, sizeof(Gifname));
      }

      GRP->GetItm(GRP_INPUTFILE, adat->inputfile, sizeof(adat->inputfile));

      GRP->GetItm(GRP_INTERNETADDRESS, adat->internetaddress, sizeof(adat->internetaddress));
      GRP->GetItm(GRP_INTERNETGATE, &adat->internetgate, sizeof(adat->internetgate));
      GRP->GetItm(GRP_INTERNETMSGID, &adat->internetmsgid, sizeof(adat->internetmsgid));
      GRP->GetItm(GRP_INTERNETRFCBODY, &adat->internetrfcbody, sizeof(adat->internetrfcbody));

      if(GRP->GetItm(GRP_LOADLANGUAGE, adat->loadlanguage, sizeof(adat->loadlanguage)))
        LoadLanguage(adat->loadlanguage);

      GRP->GetItm(GRP_MSGLISTDATE, &adat->msglistdate, sizeof(adat->msglistdate));
      GRP->GetItm(GRP_MSGLISTFAST, &adat->msglistfast, sizeof(adat->msglistfast));
      GRP->GetItm(GRP_MSGLISTFIRST, &adat->msglistfirst, sizeof(adat->msglistfirst));
      GRP->GetItm(GRP_MSGLISTHEADER, &adat->msglistheader, sizeof(adat->msglistheader));
      GRP->GetItm(GRP_MSGLISTWIDESUBJ, &adat->msglistwidesubj, sizeof(adat->msglistwidesubj));

      if(GRP->GetItm(GRP_NETNAME, buf, sizeof(buf))) {
        if(*buf == '@')
          GetRandomLine(buf, sizeof(buf), buf+1);
        strxcpy(adat->netname, buf, sizeof(adat->netname));
      }

      GRP->GetItm(GRP_ORGANIZATION, adat->organization, sizeof(adat->organization));

      if(GRP->GetItm(GRP_ORIGIN, buf, sizeof(buf)))
        strxcpy(adat->origin, buf, sizeof(adat->origin));

      GRP->GetItm(GRP_OUTPUTFILE, adat->outputfile, sizeof(adat->outputfile));

      GRP->GetItm(GRP_QUOTEBUFFILE, adat->quotebuffile, sizeof(adat->quotebuffile));

      GRP->GetItm(GRP_QUOTECHARS, adat->quotechars, sizeof(adat->quotechars));

      GRP->GetItm(GRP_QUOTESTRING, adat->quotestring, sizeof(adat->quotestring));

      GRP->GetItm(GRP_QUOTEWRAPHARD, &adat->quotewraphard, sizeof(adat->quotewraphard));

      if(GRP->GetItm(GRP_PLAY, buf, sizeof(buf))) {
        if(*buf == '@')
          GetRandomLine(buf, sizeof(buf), buf+1);
        FileToGPlay(&adat->play, buf);
      }

      GRP->GetItm(GRP_SEARCHFOR, adat->searchfor, sizeof(adat->searchfor));

      if(GRP->GetItm(GRP_TAGLINE, buf, sizeof(buf)))
        strxcpy(adat->tagline, buf, sizeof(adat->tagline));
      
      GRP->GetItm(GRP_TAGLINECHAR, &adat->taglinechar, sizeof(adat->taglinechar));

      GRP->GetItm(GRP_TAGLINEFILE, adat->taglinefile, sizeof(adat->taglinefile));

      GRP->GetItm(GRP_TAGLINESUPPORT, &adat->taglinesupport, sizeof(adat->taglinesupport));

      if(GRP->GetItm(GRP_TEARLINE, buf, sizeof(buf)))
        strxcpy(adat->tearline, buf, sizeof(adat->tearline));
      
      if(NOT adat->forcetemplate) {
        if(CFG->tplno and (CFG->tplno < CFG->tpls))
          strxcpy(adat->tpl, CFG->tpl[CFG->tplno].file, sizeof(adat->tpl));
        else
          if(GRP->GetItm(GRP_TEMPLATE, buf, sizeof(buf))) {
            if(*buf == '@')
              GetRandomLine(buf, sizeof(buf), buf+1);
            strxcpy(adat->tpl, buf, sizeof(adat->tpl));
          }
      }

      GRP->GetItm(GRP_TEMPLATEMATCH, &adat->templatematch, sizeof(adat->templatematch));

      GRP->GetItm(GRP_TWITMODE, &adat->twitmode, sizeof(adat->twitmode));

      GRP->GetItm(GRP_USEFWD, &adat->usefwd, sizeof(adat->usefwd));

      if(GRP->GetItm(GRP_USERNAME, buf, sizeof(buf))) {
        if(*buf == '@')
          GetRandomLine(buf, sizeof(buf), buf+1);
        Addr tmp;
        tmp.reset();
        switch(mode) {
          case MODE_NEW:
          case MODE_QUOTE:
          case MODE_REPLY:
          case MODE_REPLYCOMMENT:
          case MODE_FORWARD:
            char* aptr = strrchr(buf, ',');
            if(aptr) {
              *aptr++ = NUL;
              fastSkipWht(aptr);
            }
            else {
              aptr = strrchr(buf, ' ');
              if(aptr) {
                aptr++;
                tmp.set(aptr);
                if(tmp.net == 0)
                  aptr = NULL;
                else
                  *aptr++ = NUL;
              }
            }
            if(aptr) {
              tmp = adat->aka.addr;
              tmp.set(aptr);
              if(tmp.net != 0)
                *aptr = NUL;
              //else
              //  aptr = NULL;
            }
            strbtrim(buf);
            break;
        }
        if(tmp.net == 0)
          tmp = adat->aka.addr;
        strxcpy(adat->username.name, buf, sizeof(adat->username.name));
        adat->username.addr = tmp;
        SetAka(tmp);
      }

      if(!mode) {
        GRP->GetItm(GRP_VIEWHIDDEN, &adat->viewhidden, sizeof(adat->viewhidden));
        GRP->GetItm(GRP_VIEWKLUDGE, &adat->viewkludge, sizeof(adat->viewkludge));
        GRP->GetItm(GRP_VIEWQUOTE, &adat->viewquote, sizeof(adat->viewquote));
      }

      if(GRP->GetItm(GRP_NICKNAME, buf, sizeof(buf)))
        strxcpy(adat->nickname, buf, sizeof(adat->nickname));

      if(GRP->GetItm(GRP_WHOTO, buf, sizeof(buf))) {
        if(*buf == '@')
          GetRandomLine(buf, sizeof(buf), buf+1);
        strxcpy(adat->whoto, buf, sizeof(adat->whoto));
      }

      GRP->GetItm(GRP_XLATEXPORT, adat->xlatexport, sizeof(adat->xlatexport));

      GRP->GetItm(GRP_XLATIMPORT, adat->xlatimport, sizeof(adat->xlatimport));

      fp.fclose();
    }
  }

  int n = 0;
  while(n < CFG->mailinglists) {
    if(strieql(echoid(), CFG->mailinglist[n].echoid)) {
      strcpy(adat->whoto, *CFG->mailinglist[n].contribution ? CFG->mailinglist[n].contribution : CFG->mailinglist[n].sender);
      break;
    }
    n++;
  }
}


//  ------------------------------------------------------------------

