
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geusrbse.cpp,v 1.6 1999/02/03 23:01:45 leonid Exp $
//  ------------------------------------------------------------------
//  Userbase (Addressbook) functions.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include "geusrbse.h"


//  ------------------------------------------------------------------

guserbase::guserbase() {

  long tries = 0;

  strcpy(fname, AddPath(CFG->goldpath, "golduser.lst"));

  do {
    usrbase.open(fname, O_RDWR|O_CREAT|O_BINARY, SH_DENYNO, S_STDRW);
    if(not usrbase) {
      if((errno != EACCES) or (not PopupLocked(++tries, false, fname))) {
        WideLog->ErrOpen();
        WideLog->printf("! GoldED's Addressbook cannot be opened.");
        WideLog->printf(": %s", fname);
        WideLog->ErrOSInfo();
        OpenErrorExit();
      }
    }
  } while(not usrbase);

  if(tries)
    PopupLocked(0, 0, NULL);

  if((uint) usrbase.filelength() < sizeof(gusrbaseheader) + sizeof(gusrbaseentry)) {

    memset(&header, 0, sizeof(gusrbaseheader));
    memset(&entry, 0, sizeof(gusrbaseentry));

    strcpy(entry.macro, "os");
    strcpy(entry.name, "Odinn Sorensen");
    entry.fidoaddr.zone = 2;
    entry.fidoaddr.net  = 236;
    entry.fidoaddr.node = 77;
    strcpy(entry.iaddr, "odinn@goldware.dk");
    strcpy(entry.organisation, "Goldware International");
    strcpy(entry.snail1, "Ellebjergvej 21st.th.");
    strcpy(entry.snail2, "DK-2450 Copenhagen SV");
    strcpy(entry.snail3, "Denmark, Europe");
    strcpy(entry.dataphone, "+45-36139496");
    strcpy(entry.voicephone, "+45-36139495");
    strcpy(entry.faxphone, "+45-36139496");
    strcpy(entry.homepage, "http://www.goldware.dk");
    strcpy(entry.comment1, "Author of GoldED! :-)");

    usrbase.lseek(0, SEEK_SET);
    usrbase.write(&header, sizeof(gusrbaseheader));
    usrbase.write(&entry, sizeof(gusrbaseentry));
  }

  index = 0;
  read_time.l = 0;  // Indicate that we're should reread timestamp
  refresh_maximum_index();
  need_update = false;
}


//  ------------------------------------------------------------------

guserbase::~guserbase() {

  usrbase.close();
}


//  ------------------------------------------------------------------

void guserbase::refresh_maximum_index() {

  // Are we doing it for the first time?
  if(!read_time.l) {
    usrbase.getftime(&read_time);
    need_update = true;
  }
  else {
    GFTime tmp;
    usrbase.getftime(&tmp);
    if(read_time.l != tmp.l) {
      read_time.l = tmp.l;
      need_update = true;
    }
  }
  if(need_update)
    maximum_index = (usrbase.filelength()-sizeof(gusrbaseheader)) / sizeof(gusrbaseentry) - 1;
  if(index > maximum_index)
    index = maximum_index;
}


//  ------------------------------------------------------------------

void guserbase::lock() {

  if(WideCanLock) {

    long tries = 0;

    do {
      usrbase.lock(0, 1);
      if(not usrbase.okay()) {
        if(not PopupLocked(++tries, false, fname)) {
          WideLog->ErrLock();
          WideLog->printf("! GoldED's Addressbook could not be locked.");
          WideLog->printf(": %s", fname);
          WideLog->ErrOSInfo();
          LockErrorExit();
        }
      }
    } while(not usrbase.okay());
    if(tries)
      PopupLocked(0, 0, NULL);
  }
}


//  ------------------------------------------------------------------

void guserbase::unlock() {

  if(WideCanLock)
    usrbase.unlock(0, 1);
}


//  ------------------------------------------------------------------

void guserbase::open() {

  window.openxy(ypos, xpos, ylen+2, xlen+2,  btype, battr, 7);
  cwidth = (xlen-28) / 2;

  window.message(LNG__UserHeaderName, TP_BORD, 2, tattr);
  window.message(LNG__UserHeaderOrg,  TP_BORD, 3+cwidth, tattr);
  window.message(LNG__UserHeaderAka,  TP_BORD, 4+cwidth*2, tattr);

  center(CFG->displistcursor);
}


//  ------------------------------------------------------------------

void guserbase::close() {

  window.close();
}


//  ------------------------------------------------------------------

void guserbase::do_delayed() {

  wscrollbar(W_VERT, maximum_index+1, maximum_index, index);
  update_statuslinef(LNG__UserStatusline, index+1, maximum_index+1, maximum_index-index);
}


//  ------------------------------------------------------------------

void guserbase::print_line(uint idx, uint pos, bool isbar) {

  char buf[200];
  char buf2[100];

  read_entry(idx);

  *buf2 = NUL;

  if(AA->isinternet()) {
    if(*entry.iaddr) {
      strcat(buf2, "<");
      strcat(buf2, entry.iaddr);
      strcat(buf2, ">");
    }
  }
  else {
    if(entry.fidoaddr.valid()) {
      *buf2 = '(';
      entry.fidoaddr.make_string(buf2+1);
      strcat(buf2, ")");
    }
  }

  sprintf(buf, "%c %-*.*s %-*.*s %s ",
    entry.is_deleted ? 'D' : ' ',
    cwidth, (int)cwidth, entry.name,
    cwidth, (int)cwidth, entry.organisation,
    buf2);

  strsetsz(buf, xlen);
  window.prints(pos, 0, isbar ? sattr : wattr, buf);
}


//  ------------------------------------------------------------------

addressbook_form::~addressbook_form() { }
addressbook_form::addressbook_form(gwindow& w) : gwinput2(w) { };
void addressbook_form::after() { gwinput2::after(); };
void addressbook_form::before() { gwinput2::before(); };


//  ------------------------------------------------------------------

bool addressbook_form::validate() {

  if(current->id == id_name) {
    if(g->find_entry(current->buf)) {
      LoadForm();
      reload_all();
      go_next_field();
    }
  }
  return true;
}


//  ------------------------------------------------------------------

void addressbook_form::LoadForm() {

  gusrbaseentry& entry = g->entry;

  entry.fidoaddr.make_string(fidoaddr);
  name      = entry.name;
  macro     = entry.macro;
  pseudo    = entry.pseudo;
  iaddr     = entry.iaddr;
  organisation = entry.organisation;
  voicephone= entry.voicephone;
  faxphone  = entry.faxphone;
  dataphone = entry.dataphone;
  snail1    = entry.snail1;
  snail2    = entry.snail2;
  snail3    = entry.snail3;
  comment1  = entry.comment1;
  comment2  = entry.comment2;
  comment3  = entry.comment3;
  homepage  = entry.homepage;
}


//  ------------------------------------------------------------------

void addressbook_form::SaveForm() {

  gusrbaseentry& entry = g->entry;
  entry.fidoaddr.set(fidoaddr);
  strcpy(entry.name, name.c_str());
  strcpy(entry.macro, macro.c_str());
  strcpy(entry.pseudo, pseudo.c_str());
  strcpy(entry.iaddr, iaddr.c_str());
  strcpy(entry.organisation, organisation.c_str());
  strcpy(entry.voicephone, voicephone.c_str());
  strcpy(entry.faxphone, faxphone.c_str());
  strcpy(entry.dataphone, dataphone.c_str());
  strcpy(entry.snail1, snail1.c_str());
  strcpy(entry.snail2, snail2.c_str());
  strcpy(entry.snail3, snail3.c_str());
  strcpy(entry.comment1, comment1.c_str());
  strcpy(entry.comment2, comment2.c_str());
  strcpy(entry.comment3, comment3.c_str());
  strcpy(entry.homepage, homepage.c_str());
}


//  ------------------------------------------------------------------

bool guserbase::edit_entry(uint idx) {

  gwindow window;
  char tbuf[50];

  const int width = 75;
  const int height = 16;

  window.openxy((gvid.numrows-height)/2, (gvid.numcols-width)/2, height, width, btype, battr, wattr);
  window.shadow(C_SHADOW);

  sprintf(tbuf, " Record <%d> ", idx+1);
  window.title(tbuf, tattr);

  window.prints( 0, 1, wattr, "Full Name :");
  window.prints( 1, 1, wattr, "Macro Name:");
  window.prints( 1,34, wattr, "Nick Name :");
  window.prints( 2, 1, wattr, "Fidonet   :");
  window.prints( 2,34, wattr, "Internet  :");
  window.prints( 3, 1, wattr, "Organisat.:");
  window.horizontal_line(4, 0, width-2, btype, battr);

  window.prints( 5, 1, wattr, "Voice Num.:");
  window.prints( 6, 1, wattr, "Fax Number:");
  window.prints( 7, 1, wattr, "Data Num. :");
  window.prints( 5,34, wattr, "Address(1):");
  window.prints( 6,34, wattr, "Address(2):");
  window.prints( 7,34, wattr, "Address(3):");
  window.prints( 8, 1, wattr, "Group     :");
  window.prints( 8,34, wattr, "Homepage  :");
  window.prints( 9, 1, wattr, "Comment(1):");
  window.prints(10, 1, wattr, "Comment(2):");
  window.prints(11, 1, wattr, "Comment(3):");
  window.horizontal_line(12, 0, width-2, btype, battr);

  window.prints(13, 1, wattr, "First Used:");
  window.prints(13, 27, wattr, "Last Used:");
  window.prints(13, 53, wattr, "Times Used:");

  Time dt = entry.firstdate;
  if(dt) {
    Tm* tmp = glocaltime(&dt);
    char dbuf[12];
    sprintf(dbuf, "%02d %s %02d", tmp->tm_mday, gsmonth[tmp->tm_mon], (tmp->tm_year >= 100) ? (tmp->tm_year-100) : tmp->tm_year);
    window.prints(13, 13, wattr, dbuf);
  }
  dt = entry.lastdate;
  if(dt) {
    Tm* tmp = glocaltime(&dt);
    char dbuf[12];
    sprintf(dbuf, "%02d %s %02d", tmp->tm_mday, gsmonth[tmp->tm_mon], (tmp->tm_year >= 100) ? (tmp->tm_year-100) : tmp->tm_year);
    window.prints(13, 38, wattr, dbuf);
  }

  char dbuf[12];
  sprintf(dbuf, "%8ld", entry.times);
  window.prints(13, width-11, wattr, dbuf);

  addressbook_form form(window);
  form.g = this;
  form.setup(C_HEADW, C_HEADW, C_HEADE, _box_table[W_BHEAD][13], true);

  /*
    id_prefer_internet,
    id_firstdate,
    id_lastdate,
    id_times,
  */

  read_entry(idx);
  form.LoadForm();

  form.add_field(addressbook_form::id_name,         0, 13, 59, form.name, sizeof(entry.name));
  form.add_field(addressbook_form::id_macro,        1, 13, 20, form.macro, sizeof(entry.macro));
  form.add_field(addressbook_form::id_pseudo,       1, 46, 26, form.pseudo, sizeof(entry.pseudo));
  form.add_field(addressbook_form::id_fidoaddr,     2, 13, 20, form.fidoaddr, 24);
  form.add_field(addressbook_form::id_iaddr,        2, 46, 26, form.iaddr, sizeof(entry.iaddr));
  form.add_field(addressbook_form::id_organisation, 3, 13, 59, form.organisation, sizeof(entry.organisation));
  form.add_field(addressbook_form::id_voicephone,   5, 13, 20, form.voicephone, sizeof(entry.voicephone));
  form.add_field(addressbook_form::id_faxphone,     6, 13, 20, form.faxphone, sizeof(entry.faxphone));
  form.add_field(addressbook_form::id_dataphone,    7, 13, 20, form.dataphone, sizeof(entry.dataphone));
  form.add_field(addressbook_form::id_snail1,       5, 46, 26, form.snail1, sizeof(entry.snail1));
  form.add_field(addressbook_form::id_snail2,       6, 46, 26, form.snail2, sizeof(entry.snail2));  
  form.add_field(addressbook_form::id_snail3,       7, 46, 26, form.snail3, sizeof(entry.snail3));
  form.add_field(addressbook_form::id_homepage ,    8, 46, 26, form.homepage, sizeof(entry.homepage));
  //form.add_field(addressbook_form::id_group,        8, 13, 20, entry.group, sizeof(entry.group));
  form.add_field(addressbook_form::id_comment1,     9, 13, 59, form.comment1, sizeof(entry.comment1));
  form.add_field(addressbook_form::id_comment2,    10, 13, 59, form.comment2, sizeof(entry.comment2));
  form.add_field(addressbook_form::id_comment3,    11, 13, 59, form.comment3, sizeof(entry.comment3));

  form.run(H_EditAdrEntry);
  window.close();

  if(not form.dropped)
    form.SaveForm();

  return not form.dropped;
}


//  ------------------------------------------------------------------

bool guserbase::find_entry(char* name, bool lookup) {

  gusrbaseentry ent;

  if(not strblank(name)) {
    refresh_maximum_index();
    usrbase.lseek(sizeof(gusrbaseheader), SEEK_SET);
    for(uint i=0; i<=maximum_index; i++) {
      usrbase.read(&ent, sizeof(gusrbaseentry));

      if(strieql(name, ent.name) or (lookup and strieql(name, ent.macro))) {
        memcpy(&entry, &ent, sizeof(gusrbaseentry));
        index = i;
        return true;
      }
    }
  }

  return false;
}


//  ------------------------------------------------------------------

void guserbase::write_entry(uint idx, bool updateit) {

  if(updateit and not entry.is_deleted) {
    entry.lastdate = gtime(NULL);
    if(not entry.firstdate)
      entry.firstdate = entry.lastdate;

    entry.times++;
  }

  usrbase.lseek(sizeof(gusrbaseheader) + sizeof(gusrbaseentry)*idx, SEEK_SET);
  usrbase.write(&entry, sizeof(entry));
}

//  ------------------------------------------------------------------

bool guserbase::read_entry(uint idx) {

  refresh_maximum_index();
  if(idx > maximum_index) {
    memset(&entry, 0, sizeof(gusrbaseentry));
    return false;
  }
  else {                  
    usrbase.lseek(idx*sizeof(gusrbaseentry)+sizeof(gusrbaseheader), SEEK_SET);
    usrbase.read(&entry, sizeof(gusrbaseentry));
    return true;
  }
}


//  ------------------------------------------------------------------

#if 0
// This is obsolete code, we have pack_addressbook()
// It's likely that I'll remove it. 
void guserbase::remove_entry(uint idx) {

  long tries = 0;
 
  lock();
  refresh_maximum_index();

  for(uint i = idx; i < maximum_index; i++) {
    if(read_entry(i+1))
      write_entry(i);
  }

  // At least one record should present
  if(maximum_index)
    --maximum_index;
  usrbase.chsize((maximum_index + 1) * sizeof(gusrbaseentry) + sizeof(gusrbaseheader));
  usrbase.close();

  do {
    usrbase.open(fname, O_RDWR|O_CREAT|O_BINARY, SH_DENYNO, S_STDRW);
    if(not usrbase) {
      if((errno != EACCES) or (not PopupLocked(++tries, false, fname))) {
        WideLog->ErrOpen();
        WideLog->printf("! GoldED's Addressbook cannot be opened.");
        WideLog->printf(": %s", fname);
        WideLog->ErrOSInfo();
        OpenErrorExit();
      }
    }
  } while(not usrbase);

  if(tries)
    PopupLocked(0, 0, NULL);

  unlock();               
}
#endif


//  ------------------------------------------------------------------

void guserbase::pack_addressbook() {

  long tries = 0;

  lock();
  refresh_maximum_index();
  uint nidx = 0;
  uint nindex = index;
  for(uint idx = 0; idx <= maximum_index; idx++) {
    read_entry(idx);
    if(not entry.is_deleted) {
      if(nidx != idx)
        write_entry(nidx);
      ++nidx;
    }
    else if(idx < index)
      --nindex;
  }
  index = nindex;
  // zap
  maximum_index = nidx;
  // At least one record should present
  if(maximum_index)
    --maximum_index;
  usrbase.chsize((maximum_index + 1) * sizeof(gusrbaseentry) + sizeof(gusrbaseheader));
  usrbase.close();

  do {
    usrbase.open(fname, O_RDWR|O_CREAT|O_BINARY, SH_DENYNO, S_STDRW);
    if(not usrbase) {
      if((errno != EACCES) or (not PopupLocked(++tries, false, fname))) {
        WideLog->ErrOpen();
        WideLog->printf("! GoldED's Addressbook cannot be opened.");
        WideLog->printf(": %s", fname);
        WideLog->ErrOSInfo();
        OpenErrorExit();
      }
    }
  } while(not usrbase);

  if(tries)
    PopupLocked(0, 0, NULL);

  unlock();
}


//  ------------------------------------------------------------------

void guserbase::update_screen(bool force) {

  refresh_maximum_index();
  if(force || need_update) {
    if(position > index)
      center(CFG->displistcursor);
    update();
    do_delayed();
    need_update = false;
  }
}
                                   

//  ------------------------------------------------------------------

bool guserbase::handle_key() {

  switch(key) {
    case Key_Esc:
      aborted = true;
      return false;
    case Key_Ins:           // Add new entry
      {
        memset(&entry, 0, sizeof(entry));
        lock();
        refresh_maximum_index();
        uint nidx = maximum_index + 1;
        if(edit_entry(nidx)) {
          if(nidx == maximum_index + 1) {
            maximum_index++;
            if(maximum_position < ylen-1)
              maximum_position = maximum_index;
          }
          write_entry(nidx);
          index = nidx;
        }
        unlock();  
        center(CFG->displistcursor);
      }
      break;
    case Key_Ent:          // Select/Edit entry
      if(not select_list) {
        if(not entry.is_deleted) {
          lock();
          if(edit_entry(index)) {
            write_entry(index);
            display_bar();
          }
          unlock();
        }
        break;
      }
      else {
        aborted = false;
        return false;
      }
    case Key_Del:          // Soft-Delete Entry       
      lock();
      refresh_maximum_index();
      read_entry(index);
      entry.is_deleted = not entry.is_deleted;
      write_entry(index);
      unlock();
      update_screen();
      break;
    case Key_A_P:          // Pack Addressbook
      pack_addressbook();
      index = position = 0;
      update_screen();
      break;

    case Key_Tick:
      //update_screen(false);
      CheckTick(0);    // KK_UserQuitNow ???
      break;
    default:
      SayBibi();
  }
  return true;
}


//  ------------------------------------------------------------------

bool guserbase::run(GMsg* msg, bool selectonly) {

  select_list = selectonly;

  ypos    = selectonly ? 6 : 1;
  xpos    = 0;
  ylen    = MAXROW-3-ypos;
  xlen    = MAXCOL-2;
  btype   = W_BMENU;
  battr   = C_MENUB;
  wattr   = C_MENUW;
  tattr   = C_MENUT;
  sattr   = C_MENUS;
  hattr   = C_MENUQ;
  sbattr  = C_MENUPB;
  helpcat = H_Userbase;
  listwrap  = CFG->switches.get(displistwrap);
  maximum_position = MinV(maximum_index, ylen - 1);

  if(not select_list)
    find_entry(msg->By(), false);

  run_picker();

  if(not aborted) {
    read_entry(index);

    // other stuff not yet implemented
  }

  return not aborted;
}


//  ------------------------------------------------------------------

void guserbase::update_addressbook(GMsg* msg, bool reverse, bool force) {

  IAdr iaddr;
  Addr fidoaddr; 
  INam name;

  strcpy(name, (reverse ? msg->By() : msg->To()));
  strcpy(iaddr, (reverse ? msg->iorig : msg->idest));
  memcpy(&fidoaddr, (reverse ? &msg->orig : &msg->dest), sizeof(fidoaddr));

  if(not strblank(name)) {
    if(not find_entry(name)) {

      // Don't automatically add the user if...
      // 1. It's not allowed
      if(CFG->addressbookadd == NO or (CFG->addressbookadd == YES and not (AA->isnet() or AA->isemail())))
        return;

      // 2. It's a pseudo name
      if(strieql(name, entry.pseudo))
        return;

      // 3. It's a robotname
      for(int n=0; n<CFG->robotnames; n++)
        if(striinc(CFG->robotname[n], name))
          return;

      // 4. It's a username
      for(uint u=0; u<CFG->usernames; u++)
        if(striinc(CFG->username[u].name, name))
          return;

      // 5. It's a WHOTO name
      if(strieql(AA->Whoto(), name))
        return;

      // 6. Address/aka is unknown
      if((AA->isinternet() and strblank(iaddr)) or (not AA->isinternet() and not fidoaddr.valid()))
        return;

      // Ok, let's add it...
      memset(&entry, 0, sizeof(entry));

      strxcpy(entry.name, name, sizeof(entry.name));
      if(AA->isinternet())
        strxcpy(entry.iaddr, iaddr, sizeof(entry.iaddr));
      else
        memcpy(&entry.fidoaddr, &fidoaddr, sizeof(entry.fidoaddr));

      refresh_maximum_index();
      index = ++maximum_index;
    }
    lock();
    write_entry(index, not force);
    unlock();
  }
}


//  ------------------------------------------------------------------

bool guserbase::lookup_addressbook(GMsg* msg, char* name, char* aka, bool browser) {

  if(not strblank(name) or browser) {
    w_info(LNG__UserWait);

    if((find_entry(name, true) and not entry.is_deleted) or browser) {

      if(browser) {
        if(not run(msg, true)) {
          w_info(NULL);
          return false;
        }
      }

      strcpy(name, entry.name);
      if(not strblank(entry.pseudo))
        strcpy(msg->pseudoto, entry.pseudo);

      if(aka) {
        *aka = NUL;
        if(AA->isinternet())
          strcpy(aka, entry.iaddr);
        else {
          entry.fidoaddr.make_string(aka);
          if(strblank(aka) and not strblank(entry.iaddr)) {
            // do UUCP addressing
            strcpy(msg->realto, entry.name);
            strcpy(name, entry.iaddr);
            strcpy(msg->iaddr, entry.iaddr);
          }
        }
      }
    }
    w_info(NULL);
    return aka ? not strblank(aka) : true;
  }
  return false;
}


//  ------------------------------------------------------------------

void guserbase::build_pseudo(GMsg* msg, char* name, char* aka, bool direction) {

  strcpy(direction ? msg->pseudoto : msg->pseudofrom, strlword(name));

  for(uint u=0; u<CFG->usernames; u++)
    if(strieql(CFG->username[u].name, name)) {
      if(*AA->Nickname())
        strcpy(direction ? msg->pseudoto : msg->pseudofrom, strlword(AA->Nickname()));
      return;
    }

  if(find_entry(name, true) and not entry.is_deleted) {

    if(not strblank(entry.pseudo)) {

      if(AA->isinternet()) {
        if(strcmp(aka, entry.iaddr))
          return;
      }
      else {
        Addr AKA = aka;
        if(entry.fidoaddr != AKA)
          return;
      }

      strcpy(direction ? msg->pseudoto : msg->pseudofrom, entry.pseudo);

    }
  }
}


//  ------------------------------------------------------------------

void update_addressbook(GMsg* msg, bool reverse, bool force) {

  guserbase p;

  p.update_addressbook(msg, reverse, force);
}


//  ------------------------------------------------------------------

bool lookup_addressbook(GMsg* msg, char* name, char* aka, bool browser) {

  guserbase p;

  bool result = p.lookup_addressbook(msg, name, aka, browser);

  return result;
}

 
//  ------------------------------------------------------------------

void build_pseudo(GMsg* msg, bool direction) {

  guserbase p;
  char buf[128];

  if(direction)
    p.build_pseudo(msg, strbtrim(msg->To()), msg->dest.make_string(buf));
  else
    p.build_pseudo(msg, strbtrim(msg->By()), msg->orig.make_string(buf), false);
}                                  


//  ------------------------------------------------------------------

void edit_addressbook(GMsg* msg) {

  guserbase p;

  p.run(msg, false);

}


//  ------------------------------------------------------------------

