
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geutil.cpp,v 1.3 1999/02/06 02:56:31 damware Exp $
//  ------------------------------------------------------------------
//  Global utility functions (not overlayed in 16-bit DOS).
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

extern bool in_arealist;
extern GPickArealist* PickArealist;


//  ------------------------------------------------------------------

void update_statuslines() {

  char buf[200];
  static int __FAR called = NO;

  HandleGEvent(EVTT_REMOVEVOCBUF);

  if((CFG->switches.get(dispstatusline) or not called) /* AND NOT gkbd.inhelp */) {

    called = YES;

    #if defined(__UNIX__)
    char sep = '|';
    #else
    char sep = _box_table[W_BSTAT][3];
    #endif
    char help[200], meminfo[200], clkinfo[200];
    *clkinfo = NUL;
    *meminfo = NUL;
    *help = NUL;

    #if defined(__MSDOS__) AND defined(__BORLANDC__)
    long mem = coreleft()/1024L;
    sprintf(meminfo, " %c %luk", sep, mem);
    #elif defined(__WATCOMC__) and defined(__386__)
    #endif

    if(CFG->switches.get(statuslineclock)) {
      Time t = gtime();
      strftimei(help, 40, LNG->StatusLineTimeFmt, glocaltime(&t));
      sprintf(clkinfo, " %c %s", sep, help);
    }

    if(CFG->statuslinehelp == -1)
      *help = NUL;
    else if(CFG->statuslinehelp)
      sprintf(help, "%s %c ", LNG->StatusLineHelp, sep);
    else
      sprintf(help, "%s%s%s%s%c%s%i.%i.%i%s %c ",
        __gver_prename__,
        __gver_name__,
        __gver_postname__,
        __gver_platform__,
        goldmark,
        __gver_preversion__,
        __gver_major__,
        __gver_minor__,
        __gver_release__,
        __gver_postversion__,
        sep
      );
    
    int len = gvid.numcols-strlen(help)-strlen(meminfo)-strlen(clkinfo)-2;
    sprintf(buf, " %s%-*.*s%s%s ", help, len, len, information, meminfo, clkinfo);

    gmou.GetStatus();
    if(gmou.Row() == MAXROW-1)
      gmou.HideCursor();
    int row, col;
    vposget(&row, &col);
    wwprintstr(W_STAT, 0,0, C_STATW, buf);
    vposset(row, col);
    if(gmou.Row() == MAXROW-1)
      gmou.ShowCursor();
  }
}


//  ------------------------------------------------------------------

void update_statusline(const char* info) {

  strxcpy(information, info, sizeof(Subj));
  update_statuslines();
}


//  ------------------------------------------------------------------

void _stdc update_statuslinef(const char* format, ...) {

  va_list argptr;
  va_start(argptr, format);
  vsprintf(information, format, argptr);
  va_end(argptr);
  update_statuslines();
}


//  ------------------------------------------------------------------

void w_shadow() {

  if(CFG->switches.get(screenshadows))
    wshadow(C_SHADOW);
}


//  ------------------------------------------------------------------

void w_info(const char* info) {

  static int __FAR wh=-1;
  static int __FAR srow;
  static int __FAR scol;
  static int __FAR erow;
  static int __FAR ecol;
  static int __FAR len;
  static char __FAR buf[150] = { "" };

  int prev_wh = whandle();
  if(wh != -1)
    wactiv_(wh);

  if(info) {
    int tmp = strlen(info);
    if(wh == -1) {
      len = tmp;
      srow = inforow;
      erow = srow+3-1;
      scol = ((MAXCOL-len)/2)-1;
      ecol = scol+len+1;
      wh = wopen(srow, scol, erow, ecol, W_BINFO, C_INFOB, C_INFOW);
      w_shadow();
    }
    else {
      if(len != tmp) {
        len = tmp;
        scol = ((MAXCOL-len)/2)-1;
        ecol = scol+len+1;
        wclose();
        wh = wopen(srow, scol, erow, ecol, W_BINFO, C_INFOB, C_INFOW);
        w_shadow();
      }
    }
    if(NOT streql(buf, info)) {
      strcpy(buf, info);
      wprints(0, 0, C_INFOW, buf);
    }
  }
  else {
    if(wh != -1) {
      *buf = NUL;
      wclose();
      wh = -1;
    }
  }

  wactiv_(prev_wh);
}


//  ------------------------------------------------------------------


void _stdc w_infof(const char* format, ...) {

  char winfobuf[150];
  va_list argptr;
  va_start(argptr, format);
  vsprintf(winfobuf, format, argptr);
  va_end(argptr);
  w_info(winfobuf);
}


//  ------------------------------------------------------------------

void w_progress(int mode, int attr, long pos, long size, const char* title) {

  static int __FAR wh = -1;

  int prev_wh = whandle();
  if(wh != -1)
    wactiv_(wh);

  switch(mode) {
    case MODE_NEW:
      oops:
      if(wh == -1) {
        wh = wopen_(inforow,((MAXCOL-63)/2)-1,3,63,W_BINFO,C_INFOB,C_INFOW);
        set_title(title,TCENTER,C_INFOT);
        title_shadow();
      }
    case MODE_UPDATE:
      if(wh == -1)
        goto oops;  // Oops, someone forgot to open the window..
      wpropbar(PROP_BARGRAPH,1,0,-59,1,attr,pos,size);
      break;
    case MODE_QUIT:
      if(wh != -1) {
        wclose();
        wunlink(wh);
        wh = -1;
      }
      break;
  }

  wactiv_(prev_wh);
}


//  ------------------------------------------------------------------

void maketitle() {

  wtitle(m_title, m_titlepos, m_titleattr);
}


//  ------------------------------------------------------------------

void set_title(const char* t, int p, int a) {

  strcpy(m_title, t);
  m_titlepos  = p;
  m_titleattr = a;
}


//  ------------------------------------------------------------------

void title_shadow() {

  maketitle();
  w_shadow();
}


//  ------------------------------------------------------------------

int IsQuoteChar(const char* s) {

  if(*s) {
    if(*s == '>')
      return TRUE;
    if(*AA->Quotechars())
      if(strchr(AA->Quotechars(), *s))
        return TRUE;
  }
  return FALSE;
}


//  ------------------------------------------------------------------

int is_quote(const char* ptr) {

  register const char* endptr = ptr + 7;

  // Skip leading whitespace
  while((*ptr == ' ') OR (*ptr == LF) OR (*ptr == SOFTCR and not CFG->switches.get(dispsoftcr)))
    ptr++;

  // Check for empty string
  if(*ptr == NUL)
    return FALSE;

  // Check for userdefined quotechars after first whitespace
  if(*AA->Quotechars())
    if(strchr(AA->Quotechars(), *ptr))
      return TRUE;

  while((ptr < endptr) AND *ptr) {

    if(*ptr == LF OR (*ptr == SOFTCR and not CFG->switches.get(dispsoftcr))) {
      // Ignore LF's and SOFTCR's and extend check zone if found
      endptr++;
    }
    else if(*ptr == '>') {
      // Found true quote char
      return TRUE;
    }
    else if((((byte)*ptr) < ' ') OR strchr("<\"\'-", *ptr)) {
      // Found a char that cannot occur in a quotestring
      return FALSE;
    }
    ptr++;
  }

  return FALSE;
}


//  ------------------------------------------------------------------

int quotecolor(const char* line) {

  char buf[100];
  uint len;

  GetQuotestr(line, buf, &len);
  uint qc = 0;

  for(uint i=0; i<len; i++)
    if(IsQuoteChar(&buf[i]))
      qc++;

  return (qc & 1) ? C_READQ : C_READQ2;
}


//  ------------------------------------------------------------------

int GetQuotestr(const char* ptr, char* qbuf, uint* qlen) {

  if(is_quote(ptr)) {

    char* qptr;
    const char* tmp;
    const char* lp = ptr;
    register int n, x;

    MoreQuotes:   // Naughty goto-point for quotestring skipping

    // Skip leading spaces
    while(*lp == ' ' OR *lp == LF OR (*lp == SOFTCR and not CFG->switches.get(dispsoftcr)))
      lp++;
    if(IsQuoteChar(lp)) {      // Type 1 : ">xxxx>" and ">xxxx:"
      lp++;
      while(*lp == ' ' OR *lp == LF OR (*lp == SOFTCR and not CFG->switches.get(dispsoftcr)))
        lp++;
      if(is_quote(lp))
        goto MoreQuotes;
      if(NOT(IsQuoteChar(lp-1) OR (*(lp-1) == ':'))) {
        while(NOT IsQuoteChar(lp))
          lp--;
        lp++;
      }
    }
    else {                // Type 2: "xxxx>"
      while(NOT(IsQuoteChar(lp) AND (NOT IsQuoteChar(lp+1))) AND *lp != CR AND *lp)
        ++lp;
      if(is_quote(lp))
        goto MoreQuotes;
      if(*lp)
        lp++;
    }

    // lp now points to the character after the quotestring

    *qlen = (int)((dword)lp - (dword)ptr);
    if(*lp == ' ' OR *lp == LF OR (*lp == SOFTCR and not CFG->switches.get(dispsoftcr)))
      (*qlen)++;

    for(x=*qlen,n=0,tmp=ptr,qptr=qbuf; n<x AND n<40; n++,tmp++) {
      if(CFG->switches.get(dispsoftcr)) {
        if(*tmp != LF)
          *qptr++ = *tmp;
        else
          (*qlen)--;
      }
      else {
        if(*tmp != LF AND *tmp != SOFTCR)
          *qptr++ = *tmp;
        else
          (*qlen)--;
      }
    }
    *qptr = NUL;
    if(*qlen > 40)
      *qlen = 40;
  }
  else {
    *qbuf = NUL;
    *qlen = 0;
  }

  return *qlen;
}


//  ------------------------------------------------------------------

int _stdc CmdKeyCmp(const CmdKey* a, const CmdKey* b) {

  int cmp = CmpV(a->type, b->type);
  if(cmp)
    return cmp;
  return CmpV(a->key, b->key);
}


//  ------------------------------------------------------------------

static int KeyCmp(const Key* a, const Key* b) {

  return CmpV(*a, *b);
}


//  ------------------------------------------------------------------

Key SearchKey(Key key, CmdKey* keys, int totkeys) {

  char* kmin;
  char* probe;
  int again = 0;
  register int i, j, tkeys=totkeys;

  do {
    kmin = (char*)keys;
    while(tkeys > 0) {
      i = tkeys >> 1;
      probe = kmin + i * sizeof(CmdKey);
      j = KeyCmp(&key, (Key*)probe);
      if(j == 0)
        return(((CmdKey*)probe)->cmd);
      else if(j < 0)
        tkeys = i;
      else {
        kmin = probe + sizeof(CmdKey);
        tkeys = tkeys - i - 1;
      }
    }

    // Key not found. Try again, this time without the scancode.
    key &= 0x00FF;
    tkeys = totkeys;
  } while(NOT again++ AND key);

  return 0;
}


//  ------------------------------------------------------------------

static void call_func_geutil(VfvCP func) {

  struct _menu_t* menu;
  int row, col;

  bool hidden = vcurhidden();
  vposget(&row, &col);
  menu = gwin.cmenu;
  (*func)();
  gwin.cmenu = menu;
  vposset(row, col);
  if(hidden)
    vcurhide();
  else
    vcurshow();
}

  
//  ------------------------------------------------------------------

void call_help() {

  // search through onkey linked list for a
  // matching defined onkey.  if one is found,
  // then save the current environment, call the
  // onkey's function, and restore the environment.

  KBnd* onkey = gkbd.onkey;
  while(onkey != NULL) {
    if(onkey->keycode == Key_F1) {
      call_func_geutil(onkey->func);
      break;
    }
    onkey = onkey->prev;
  }
}


//  ------------------------------------------------------------------

void CheckTick(Key quitkey) {

  Clock idle_secs = (gkbd.tickvalue - gkbd.tickpress) / 1000L;

  if(CFG->timeout) {
    if(idle_secs >= CFG->timeout) {
      kbput(quitkey);
      return;
    }
  }

  IdleCheckSemaphores();

  if(CFG->screenblanker) {
    if(idle_secs >= CFG->screenblanker) {

      blanked = TRUE;
      ScreenBlankIdle();
      kbdsettickfunc(ScreenBlankIdle);

      getxch();

      blanked = FALSE;
      ScreenBlankIdle();
      kbdsettickfunc(update_statuslines);

      //maybe we've scanned areas while in screenblanker, so
      //update screen
      if(in_arealist) {
        PickArealist->update();
        PickArealist->do_delayed();
      }
    }
  }
}


//  ------------------------------------------------------------------

void IdleCheckSemaphores() {

  // I don't like this solution either... :(
  static Clock last_secs = 0;
  Clock idle_secs = (gkbd.tickvalue - gkbd.tickpress) / 1000L;

  // Make sure the stuff below is only run once in a second
  if(not idle_secs or (idle_secs - last_secs == 0))
    return ;

  if(in_arealist) {
    if(CFG->semaphore.idletime) {
      if((idle_secs % CFG->semaphore.idletime) == 0)
        CheckSemaphores();
    }
  }

  last_secs = idle_secs;
}


//  ------------------------------------------------------------------

char* strtmp(const char* str) {

  static __FAR INam tmp;
  return strxcpy(tmp, str, sizeof(tmp));
}


//  ------------------------------------------------------------------

