
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geutil2.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Global utility functions.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include <gwinput.h>


//  ------------------------------------------------------------------

int edit_string(char* buf, int buf_size, char* title, int helpcat) {

  gwindow window;

  window.openxy(inforow,((MAXCOL-63)/2)-1,3,63,W_BASK,C_ASKB,C_ASKW);
  window.title(title, C_ASKT);
  if(CFG->switches.get(screenshadows))
    window.shadow(C_SHADOW);

  gwinput2 iform(window);

  gstring buf2 = buf;
  iform.setup(C_ASKW, C_ASKW, C_ASKQ, _box_table[W_BASK][13], true);
  iform.add_field(0, 0, 1, 59, buf2, buf_size, gwinput::cvt_none, fieldupd);
  vcurshow();
  iform.run(helpcat);
  vcurhide();

  window.close();

  if(iform.dropped)
    return FALSE;

  buf2.trim();
  strcpy(buf, buf2.c_str());
  return *buf ? TRUE : FALSE;
}


//  ------------------------------------------------------------------

bool edit_pathname(char* buf, int buf_size, char* title, int helpcat) {

  if(NOT edit_string(buf, buf_size, title, helpcat))
    return false;
      
  for(int z=0; z<CFG->filealiases; z++) {
    if(strieql(buf, CFG->filealias[z].alias)) {
      strcpy(buf, CFG->filealias[z].file);
      break;
    }
  }

  strschg_environ(buf);
  return true;
}


//  ------------------------------------------------------------------

int PopupLocked(long tries, int isopen, const char* file) {

  // Close popup window if requested
  if(tries == 0) {
    w_info(NULL);
    return TRUE;
  }

  // Tell the world
  char buf[200];
  if((uint) strlen(file) > (uint) (MAXCOL-25))
    sprintf(buf, "[..]%*.*s", MAXCOL-30, MAXCOL-30, file);
  else
    strcpy(buf, file);
  w_infof(LNG->WaitLocked, buf);
  update_statuslinef(LNG->RetryOrESC, isopen ? LNG->RetryLock : LNG->RetryOpen , tries);

  // Check for keypress
  if(kbxhit()) {

    // Return fail value if Escape was pressed
    Key keyval = kbxget();
    if(keyval == Key_Esc) {
      GMenuLockLoop MenuLockLoop;
      if(MenuLockLoop.Run()) {
        w_info(NULL);
        return FALSE;
      }
    }
  }

  // Pause approx 1 second before trying again
  Pause(1000);

  // Try again
  return TRUE;
}


//  ------------------------------------------------------------------
//  Build zonegate address from orig/dest address

void ZonegateIt(ftn_address& gate, ftn_address& orig, ftn_address& dest) {

  gate.zone  = orig.zone;  // Send to the local zoneGate
  gate.net   = orig.zone;
  gate.node  = dest.zone;
  gate.point = 0;
}


//  ------------------------------------------------------------------

int GetAkaNo(const ftn_address& aka) {

  int akano = 0;
  while(akano < CFG->akas) {
    if(CFG->aka[akano].addr.equals(aka))
      return akano;
    akano++;
  }
  return -1;
}


//  ------------------------------------------------------------------

void doinvalidate(char* text, const char* find, const char* replace) {

  if(strnieql(text, find, strlen(find))) {
    char buf[256];

    strcpy(buf, text);
    strcpy(text, replace);
    strcat(text, &buf[strlen(find)]);
  }
}


//  ------------------------------------------------------------------

int AkaMatchOne(const ftn_address* mask, const ftn_address* addr) {

  int match = 0;

  // Count matching parts
  if((mask->zone == 0) OR (addr->zone == 0) OR (mask->zone == GFTN_ALL) OR (mask->zone == addr->zone)) {
    match++;
    if((mask->net == GFTN_ALL) OR (mask->net == addr->net)) {
      match++;
      if((mask->node == GFTN_ALL) OR (mask->node == addr->node)) {
        match++;
        if((mask->point == GFTN_ALL) OR (mask->point == addr->point)) {
          match++;
        }
      }
    }
  }

  return match;
}


//  ------------------------------------------------------------------

int AkaMatch(ftn_address* match, const ftn_address* addr) {

  int aka;

  for(aka=0; aka<CFG->akamatches; aka++) {
    if(addr->match(CFG->akamatch[aka].mask)) {
      int akano = GetAkaNo(CFG->akamatch[aka].aka);
      if(akano != -1) {
        *match = CFG->aka[akano].addr;
        return akano;
      }
    }
  }

  int bestaka = -1;
  int bestmatch = 0;
  int matchaka = 0;

  for(aka=0; aka<CFG->akas; aka++) {

    if(match->equals(CFG->aka[aka].addr))
      matchaka = aka;

    // Ignore fakenet addresses
    if(CFG->aka[aka].addr.net != CFG->aka[aka].pointnet) {

      int match = AkaMatchOne(&CFG->aka[aka].addr, addr);

      // Note if we found a better match
      if(match > bestmatch) {
        bestmatch = match;
        bestaka = aka;
      }
    }
  }

  // If there was no matching aka, use the aka
  if(bestaka == -1)
    bestaka = matchaka;
  else {
    // If the current address matches at least as good as the best match, don't change it
    if(bestmatch <= AkaMatchOne(match, addr)) {
      int akano = GetAkaNo(*match);
      if(akano != -1)
        return akano;
    }
  }

  *match = CFG->aka[bestaka].addr;
  return bestaka;
}


//  ------------------------------------------------------------------

void ScreenBlankIdle() {

  static int __FAR whb = -1;
  static int __FAR whh = -1;
  static int __FAR wascurhid = FALSE;
  static Clock __FAR lastmoved = 0;
  int windowheight = 4;

  IdleCheckSemaphores();

  char blankmsg1[80];
  char blankmsg2[80];
  Time t = gtime();
  strftimei(blankmsg2, 40, LNG->StatusLineTimeFmt, glocaltime(&t));
  sprintf(blankmsg1, " %s %s ", __gver_longpid__, blankmsg2);
  sprintf(blankmsg2, " %s ", LNG->BlankMsg);
  if(strblank(blankmsg2)) {
    *blankmsg2 = NUL;
    windowheight--;
  }
  register int b1 = strlen(blankmsg1);
  register int b2 = strlen(blankmsg2);
  int blankmsglen = MaxV(b1,b2);
  int ry = rand() % (gvid.numrows-windowheight);
  int rx = rand() % ((gvid.numcols-blankmsglen)-2);

  if(blanked AND (whb == -1)) {
    wascurhid = vcurhidden();
    vcurhide();
    if(C_BACKB != 0)
      gvid.setoverscan(0);
    whb = wopen(0,0,gvid.numrows-1,gvid.numcols-1, 5, 7, 7);
    if(CFG->screenblankertype == BLANK_SLIDEWIN)
      whh = wopen_(ry, rx, windowheight, blankmsglen+2, W_BINFO, C_INFOB, C_INFOW);
    lastmoved = gkbd.tickvalue;
  }

  if(NOT blanked AND (whb != -1)) {
    if(CFG->screenblankertype == BLANK_SLIDEWIN)  {
      wactiv_(whh);
      wclose();
      whh = -1;
    }
    wactiv_(whb);
    wclose();
    whb = -1;
    if(C_BACKB != 0)
      gvid.setoverscan(C_BACKB);
    if(wascurhid)
      vcurhide();
    else
      vcurshow();
    return;
  }

  if(gkbd.tickvalue > (lastmoved+5000L)) {
    lastmoved = gkbd.tickvalue;
    if(CFG->screenblankertype == BLANK_SLIDEWIN)
      wslide(ry, rx);
  }

  if(CFG->screenblankertype == BLANK_SLIDEWIN) {
    wcenters(0, C_INFOW, blankmsg1);
    if(*blankmsg2)
      wcenters(1, C_INFOW, blankmsg2);
  }

  // If timed out, exit screenblanker
  if(CFG->timeout)
    if(gkbd.tickvalue >= (gkbd.tickpress+(CFG->timeout*1000L)))
      kbput(Key_Tick);
}


//  ------------------------------------------------------------------

