
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gcalst.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Arealist configuration and other one-time stuff
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------
//  Save arealist data

void garealist::save(gfile& fp) {

  gvector_save(fp, listscan);
  fp << list.size();
  fp << msgbases;
  for(int n=0; n<list.size(); n++)
    fp << *list[n];
}


//  ------------------------------------------------------------------
//  Load arealist data

void garealist::load(gfile& fp) {

  gvector_load(fp, listscan);
  int total;
  fp >> total;
  fp >> msgbases;
  list.resize(total);
  for(int n=0; n<list.size(); n++) {
    list[n] = new garealistdata;
    throw_new(list[n]);
    fp >> *list[n];
  }
}


//  ------------------------------------------------------------------
//  Write lastreads etc. for the next session

void garealist::write_goldlast() {

  gfile fp;

  gstring bak2(AddPath(CFG->goldpath, "goldlast.bk2"));
  gstring bak1(AddPath(CFG->goldpath, "goldlast.bk1"));
  gstring lst(AddPath(CFG->goldpath, "goldlast.lst"));

  remove(bak2.c_str());
  rename(bak1.c_str(), bak2.c_str());
  rename(lst.c_str(), bak1.c_str());

  fp.fopen(lst.c_str(), "wt", CFG->sharemode);
  if(fp.isopen()) {

    fp.setvbuf(NULL, _IOFBF, 8192);

    gstring echoid;

    for(int n=0; n<list.size(); n++) {
      garealistdata* ap = list[n];
      if(not ap->isseparator()) {
        if(strchr(ap->echoid().c_str(), ' '))
          echoid = "\"" + ap->echoid() + "\"";
        else
          echoid = ap->echoid();
        fp.printf("%s %d %d %d %d %d %d\n",
          echoid.c_str(),
          ap->lastread(),
          ap->scancount(),
          ap->unread(),
          ap->scanned() ? ap->scans() : -1,
          ap->changed(),
          ap->pmscancount()
        );
      }
    }

    fp.fclose();
  }
}


//  ------------------------------------------------------------------
//  Read the lastreads from the last session

void garealist::read_goldlast() {

  gfile fp;

  fp.fopen(AddPath(CFG->goldpath, "goldlast.lst"), "rt", CFG->sharemode);
  if(fp.isopen()) {

    fp.setvbuf(NULL, _IOFBF, 8192);

    char* key;
    char* val;
    char buf[256];
    while(fp.fgets((val=buf), sizeof(buf))) {

      getkeyval(&key, &val);

      for(int n=0; n<list.size(); n++) {
        if(list[n]->echoid().ieql(key)) {
          garealistdata* ap = list[n];
          if(not ap->isseparator()) {
            getkeyval(&key, &val);
            ap->set_lastread(atoi(key));
            getkeyval(&key, &val);
            ap->set_scancount(atoi(key));
            getkeyval(&key, &val);
            ap->set_unread(atoi(key));
            getkeyval(&key, &val);
            ap->set_scans(atoi(key));
            ap->set_scanned(ap->scans() != -1);
            ap->update_data();
            getkeyval(&key, &val);
            if(*key)
              ap->set_changed(atoi(key));
            getkeyval(&key, &val);
            if(*key)
              ap->set_pmscancount(atoi(key));
            break;
          }
        }
      }
    }

    fp.fclose();
  }
}


//  ------------------------------------------------------------------
//  Write all areas to GOLDAREA.INC in AREADEF format

void garealist::write_areadef(const char* file) {

  gfile fp;
  Path path;
  int n, tmp;
  char groupid[10], echoid[sizeof(Echo)+2];
  int maxechoid=0, maxdesc=0, maxgroupid=0, maxpath=0, maxaddr=0, maxattr=0;
  char desc[sizeof(Desc)+2], type[6], msgbase[7], addr[40], origin[sizeof(Orig)+2];
  gstring attr;

  fp.fopen(file, "wt");
  if(fp.isopen()) {

    for(n=0; n<list.size(); n++) {
      garealistdata* aa = list[n];
      tmp = aa->echoid().length();
      if(strchr(aa->echoid().c_str(), ' '))
        tmp += 2;
      maxechoid = maximum_of_two(maxechoid, tmp);
      tmp = aa->desc().length()+2;
      maxdesc = maximum_of_two(maxdesc, tmp);
      tmp = aa->path().length();
      if(strchr(aa->path().c_str(), ' '))
        tmp += 2;
      maxpath = maximum_of_two(maxpath, tmp);
      if(aa->groupid() & 0x8000u)
        sprintf(groupid, "#%u", aa->groupid()&0x7FFF);
      else if(isupper(aa->groupid()))
        *groupid = (char)aa->groupid(), groupid[1] = NUL;
      else
        *groupid = '0', groupid[1] = NUL;
      tmp = strlen(groupid);
      maxgroupid = maximum_of_two(maxgroupid, tmp);
      if(CFG->address[0] != aa->aka())
        aa->aka().make_string(addr);
      else
        strcpy(addr, ".");
      tmp = strlen(addr);
      maxaddr = maximum_of_two(maxaddr, tmp);
      tmp = aa->attr().make_string(attr).length();
      maxattr = maximum_of_two(maxattr, tmp)+2;
    }

    for(n=0; n<list.size(); n++) {
      garealistdata* aa = list[n];
      sprintf(desc, "\"%s\"", aa->desc().c_str());
      if(aa->groupid() & 0x8000u)
        sprintf(groupid, "#%u", aa->groupid()&0x7FFF);
      else if(isupper(aa->groupid()))
        *groupid = (char)aa->groupid(), groupid[1] = NUL;
      else
        *groupid = '0', groupid[1] = NUL;
      if(aa->isemail())
        strcpy(type, "EMail");
      else if(aa->isnewsgroup())
        strcpy(type, "News ");
      else if(aa->isnet())
        strcpy(type, "Net  ");
      else if(aa->isecho())
        strcpy(type, "Echo ");
      else if(aa->islocal())
        strcpy(type, "Local");
      switch(aa->msgbase()) {
        case GMB_OPUS:      strcpy(msgbase, "Opus  "); break;
        case GMB_FTS1:      strcpy(msgbase, "Fts1  "); break;
        case GMB_HUDSON:    strcpy(msgbase, "Hudson"); break;
        case GMB_GOLDBASE:  strcpy(msgbase, "Gold  "); break;
        case GMB_SQUISH:    strcpy(msgbase, "Squish"); break;
        case GMB_EZYCOM:    strcpy(msgbase, "Ezycom"); break;
        case GMB_JAM:       strcpy(msgbase, "Jam   "); break;
        case GMB_PCBOARD:   strcpy(msgbase, "Pcb   "); break;
        case GMB_WILDCAT:   strcpy(msgbase, "WCat  "); break;
        case GMB_ADEPTXBBS: strcpy(msgbase, "XBBS  "); break;
      }
      if(strchr(aa->echoid().c_str(), ' '))
        sprintf(echoid, "\"%s\"", aa->echoid().c_str());
      else
        strcpy(echoid, aa->echoid().c_str());
      if(aa->isseparator()) {
        fp.printf("AREASEP %-*s %-*s %*s %s\n",
          maxechoid,  echoid,
          maxdesc,    desc,
          maxgroupid, groupid,
          strtrim(type)
        );
      }
      else {
        if(strchr(aa->path().c_str(), ' '))
          sprintf(path, "\"%s\"", aa->path().c_str());
        else
          strcpy(path, aa->path().c_str());
        if(CFG->address[0] != aa->aka())
          aa->aka().make_string(addr);
        else
          strcpy(addr, ".");
        gstring tmpattr;
        aa->attr().make_string(tmpattr);
        attr = '(' + tmpattr + ')';
        if(aa->originno())
          sprintf(origin, "\"%s\"", CFG->origin[aa->originno()]);
        else
          *origin = NUL;
        fp.printf("AREADEF %-*s %-*s %*s %s %s %-*s %-*s %-*s %s\n",
          maxechoid,  echoid,
          maxdesc,    desc,
          maxgroupid, groupid,
          type,
          msgbase,
          maxpath, path,
          maxaddr, addr,
          maxattr, attr.c_str(),
          origin
        );
      }
    }

    fp.fclose();
  }
}


//  ------------------------------------------------------------------

void SetAreaDesc(char* echoid, char* desc) {

  AL.set_areadesc(echoid, desc);
}


//  ------------------------------------------------------------------

void garealist::set_areadesc(const char* echoid, const char* desc) {

  for(int n=0; n<list.size(); n++) {
    if(list[n]->echoid().ieql(echoid)) {
      list[n]->set_desc(desc);
      break;
    }
  }
}


//  ------------------------------------------------------------------

void PcbAdjustArea(int rec, const char* msgfile) {

  for(int n=0; n<AL.areas(); n++) {
    garealistdata* a = AL[n];
    if(a->ispcboard()) {
      if((a->board() == rec) and a->path().empty()) {
        a->set_path(msgfile);
        break;
      }
      else if(a->path().ieql(msgfile)) {
        a->set_board(rec);
        break;
      }
    }
  }
}


//  ------------------------------------------------------------------

