
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gcarea.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Common code for the area compile.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

garealistdata::garealistdata() {

  is_marked = is_changed = is_scanned = is_pmscanned = false;
  scans_count = scan_count = pmscan_count = lastread_count = 0;
}


//  ------------------------------------------------------------------

garealistdata::~garealistdata() {

}


//  ------------------------------------------------------------------

const char* AddStamp(const char* file, int filecheck) {

  stamp = (Stamp*)throw_realloc(stamp, (stamps+2)*sizeof(Stamp));
  stamp[stamps].fn = file;
  stamp[stamps].ft = GetFiletime(file);
  stamp[stamps].fc = (byte)filecheck;
  stamps++;

  return file;
}


//  ------------------------------------------------------------------
//  Add stamp unless told not to

void ChkAddStamp(const char* file, char* options) {

  if(!striinc("/NOCHK", options) and !striinc("-NOCHK", options))
    AddStamp(file);
}


//  ------------------------------------------------------------------
//  Fix up the echoid and make default if none specified

void FixEchoid(char* echoid, int type) {

  static uint __FAR netnum = 1;
  static uint __FAR echonum = 1;
  static uint __FAR localnum = 1;

  //strupr(echoid);
  //strchg(echoid, ' ', '_');

  if(*echoid == NUL) {
    char* t = CFG->areaautoid == AUTOID_LONG ? "MAIL" : "";
    if(type & GMB_NET)
      sprintf(echoid, "NET%s%03u", t, netnum++);
    else if(type & GMB_ECHO)
      sprintf(echoid, "ECHO%s%03u", t, echonum++);
    else if(type & GMB_LOCAL)
      sprintf(echoid, "LOCAL%03u", localnum++);
  }
}


//  ------------------------------------------------------------------
//  Sort a group of areas

void garealist::sort_areagroup(const char* options, int beginarea, int endarea) {

  if(beginarea != endarea) {
    const char* ptr = striinc("-S", options);
    if(ptr == NULL)
      ptr = striinc("/S", options);
    if(ptr) {
      ptr += 2;
      if(*ptr == '=')
        ptr++;
      sort(ptr, beginarea, endarea);
    }
  }
}


//  ------------------------------------------------------------------
//  Rename an echoid

void RenameArea(char* echoid) {
  
  for(int n=0; n<CFG->arearename.size(); n++) {
    if(CFG->arearename[n].from.ieql(echoid)) {
      strcpy(echoid, CFG->arearename[n].to.c_str());
      break;
    }
  }
}


//  ------------------------------------------------------------------
//  Check if the echoid is on the exclusion list

bool CheckExcl(const char* echoid) {

  for(int n=0; n<CFG->areaexcl.size(); n++) {
    if(strwild(echoid, CFG->areaexcl[n])) {

      // Found excl, now check for incl
      for(int x=0; x<CFG->areaincl.size(); x++)
        if(strwild(echoid, CFG->areaincl[x]))
          return true;

      return false;   // Echoid is excluded
    }
  }
  return true;
}


//  ------------------------------------------------------------------
//  Check if the echo is email or news

void CheckEMailOrNews(const char* echoid, uint& type) {

  for(int e=0; e<CFG->areaisemail.size(); e++) {
    if(strwild(echoid, CFG->areaisemail[e])) {
      type = GMB_EMAIL | GMB_NET;
      return;
    }
  }

  for(int n=0; n<CFG->areaisnews.size(); n++) {
    if(strwild(echoid, CFG->areaisnews[n])) {
      type = GMB_NEWSGROUP | GMB_ECHO;
      return;
    }
  }
}


//  ------------------------------------------------------------------
//  Add or update area data

void AddNewArea(AreaCfg& aa) {

  AL.add(aa);
}


//  ------------------------------------------------------------------
//  Add or update area data

void garealist::add(AreaCfg& aa) {

  if(veryverbose)
    printf("\n  fmt=%u, eid=\"%s\", pth=\"%s\", brd=%u, gid=%u, aka=%u:%u/%u.%u",
      aa.msgbase,
      aa.echoid,
      aa.path,
      aa.board,
      aa.groupid,
      aa.aka.zone,
      aa.aka.net,
      aa.aka.node,
      aa.aka.point
    );

  gstring desc;
  char buf[256];
  bool newarea = true;
  static int __FAR serial = 0;
  static bool __FAR net1st = false;

  // Make sure the path field is 100% correct for the msgbase
  switch(aa.msgbase) {
    case GMB_HUDSON:
      // Ignore areas with invalid numbers
      if((aa.board < 1) or (aa.board > 200))
        return;
      sprintf(aa.path, "%u", aa.board);
      break;
    case GMB_GOLDBASE:
      // Ignore areas with invalid numbers
      if((aa.board < 1) or (aa.board > 500))
        return;
      sprintf(aa.path, "%u", aa.board);
      break;
    case GMB_EZYCOM:
      // Ignore areas with invalid numbers
      if((aa.board < 1) or (aa.board > 1536))
        return;
      sprintf(aa.path, "%u", aa.board);
      break;
    case GMB_FTS1:
    case GMB_OPUS:
      if(*aa.path == NUL)
        return;
      AddBackslash(aa.path);
      strschg_environ(aa.path);
      break;
    case GMB_SQUISH:
    case GMB_JAM:
    case GMB_WILDCAT:
    case GMB_ADEPTXBBS:
      if(*aa.path == NUL)
        return;
      StripBackslash(aa.path);
      strschg_environ(aa.path);
      break;
    case GMB_PCBOARD:
      StripBackslash(aa.path);
      strschg_environ(aa.path);
      break;
    case GMB_SEPARATOR:
      break;
    default:
      return;
  }

  // Note msgbase type
  msgbases |= aa.msgbase;

  // Things to do for real areas only
  if(not aa.isseparator()) {

    // If the echoid is missing, try to find it in the echolist
    if(strblank(aa.echoid))
      AFILE->echolist.FindEcho(aa.echoid, aa.path, aa.desc);

    // Fix up the echoid
    FixEchoid(aa.echoid, aa.type);

    // Rename the echoid if necessary
    RenameArea(aa.echoid);

    // Do not accept excluded areas
    if(not CheckExcl(aa.echoid))
      return;

    // Fix description
    if(strblank(aa.desc))
      strcpy(aa.desc, aa.echoid);
  }

  // Check if it's email or news
  CheckEMailOrNews(aa.echoid, aa.type);

  // Give each area a unique serial number
  aa.areaid = serial++;

  // Check if we already have the area (dup echoid or path)
  garealistdata* ap;
  for(int n=0; n<list.size(); n++) {
    ap = list[n];
    if(not ap->isseparator()) {
      bool eq_echoid  = ap->echoid().ieql(aa.echoid);
      bool eq_path    = ap->path().ieql(aa.path);
      bool eq_board   = (aa.board == ap->board());
      bool eq_msgbase = (aa.msgbase == ap->msgbase());
      bool eq_isfido  = (aa.isfido() and ap->isfido());
      if(eq_echoid or (eq_path and eq_board and (eq_msgbase or eq_isfido))) {
        // We had it already, so override with the new data
        newarea = false;
        if(ap->desc().blank())
          desc = aa.desc;
        break;
      }
    }
  }

  // If first netmail area, set default for AREAFREQTO and AREAREPLYTO
  if(aa.isnet() and not net1st) {
    net1st = true;
    if(not *CFG->areafreqto)
      CFG->areafreqto = aa.echoid;
    if(not *CFG->areareplyto)
      CFG->areareplyto = aa.echoid;
  }

  // Increase area index size if new area
  if(newarea) {
    ap = new garealistdata;
    throw_new(ap);
    list.add(ap);
  }

  // Add aka if not found
  if(aa.aka.net) {
    bool found = false;
    for(int n=0; n<CFG->address.size(); n++) {
      if(CFG->address[n] == aa.aka) {
        found = true;
        break;
      }
    }
    if(not found) {   // Then add it
      CfgAddress(aa.aka.make_string(buf));  // Add the aka
    }
  }
  else {
    // If address is missing, use main address
    if(CFG->address.size())
      aa.aka = CFG->address[0];
  }

  // Check origin
  if(not strblank(aa.origin)) {

    // Check if it already exists
    bool found = false;
    for(int n=0; n<CFG->origin.size(); n++) {
      if(streql(CFG->origin[n], aa.origin)) {
        aa.originno = n;
        found = true;
        break;
      }
    }

    // Not found, so add it
    if(not found) {
      CfgOrigin(aa.origin);
      aa.originno = CFG->origin.size() - 1;
    }
  }

  // Copy the data
  ap->set_areaid(aa.areaid);
  ap->set_groupid(aa.groupid);
  ap->set_board(aa.board);
  ap->set_type(aa.type);
  ap->set_msgbase(aa.msgbase);
  ap->set_aka(aa.aka);
  ap->set_originno(aa.originno);
  ap->set_attr(aa.attr);
  ap->set_echoid(aa.echoid);
  ap->set_desc(newarea or desc.blank() ? aa.desc : desc.c_str());
  ap->set_path(aa.path);
}


//  ------------------------------------------------------------------
//  Get areas from other programs

void garealist::get_areafile(char* value) {

  const word CRC_ECHOLIST = 0xCE70;

  word crcval;
  char* keyword;
  char* options;
  int beginarea, endarea;

  crcval = getkeyvalcrc(&keyword, &value);
  options = throw_strdup(value);

  beginarea = list.size();

  if(crcval == CRC_ECHOLIST)
    ReadEcholist(value);
  else {
    AFILE->quiet = quiet;
    AFILE->sharemode = CFG->sharemode;
    AFILE->fidomsgtype = CFG->fidomsgtype;
    AFILE->ra2usersbbs = CFG->ra2usersbbs;
    AFILE->squishuserno = CFG->squishuserno;

    AFILE->primary_aka = CFG->address[0];

    AFILE->attribsnet = CFG->attribsnet;
    AFILE->attribsecho = CFG->attribsecho;
    AFILE->attribsnews = CFG->attribsnews;
    AFILE->attribsemail = CFG->attribsemail;
    AFILE->attribslocal = CFG->attribslocal;

    AFILE->areapath = CFG->areapath;
    AFILE->adeptxbbspath = CFG->adeptxbbspath;
    AFILE->jampath = CFG->jampath;
    AFILE->squishuserpath = CFG->squishuserpath;
    AFILE->hudsonpath = CFG->hudsonpath;
    AFILE->goldbasepath = CFG->goldbasepath;
    AFILE->pcboardpath = CFG->pcboardpath;
    AFILE->ezycom_msgbasepath = CFG->ezycommsgbasepath;
    AFILE->ezycom_userbasepath = CFG->ezycomuserbasepath;
    AFILE->fidolastread = CFG->fidolastread;

    AFILE->ReadAreafile(crcval, value);

    CFG->ra2usersbbs = AFILE->ra2usersbbs;
    CFG->squishuserno = AFILE->squishuserno;
  }

  endarea = list.size();
  sort_areagroup(options, beginarea, endarea);

  throw_free(options);
}


//  ------------------------------------------------------------------
//  Get area definition

void garealist::get_area(char* def) {

  AreaCfg aa;
  char* ptr;
  char* tag;
  char* desc;
  char* base;
  char* loc;
  char* attr;

  if(strnieql(def, "-FILE ", 6))  // Check for alternate formats
    get_areafile(def+6);
  else {
    tag = def;
    desc = strpbrk(tag, " \t");
    *desc = NUL;

    aa.reset();

    // Split up the string definition
    desc = strskip_wht(desc+1);
    switch(*desc) {
      case '\"':
        base = strchr(desc+1, '\"')+1;
        break;
      case '\'':
        base = strchr(desc+1, '\'')+1;
        break;
      default:
        base = strpbrk(desc, " \t");
    }
    *base = NUL;
    base = strskip_wht(base+1);
    loc = strpbrk(base, " \t");
    *loc = NUL;
    loc = strskip_wht(loc+1);
    attr = strpbrk(loc, " \t");
    if(attr and *attr) {
      *attr = NUL;
      attr = strskip_wht(attr+1);
    }

    // Parse the individual parts

    aa.setechoid(tag);              // Copy the echotag

    StripQuotes(desc);              // Copy description, stripping quotes ("")
    aa.setdesc(desc);

    switch(toupper(*base)) {        // Store area info

      case '-':
        aa.msgbase = GMB_SEPARATOR;
        break;

      case 'M':
        aa.msgbase = GMB_SQUISH;
        aa.setpath(loc);
        break;

      case 'H':
      case 'R':
      case 'Q':
        aa.msgbase = GMB_HUDSON;
        aa.board = atoi(loc);
        break;

      case 'G':
        aa.msgbase = GMB_GOLDBASE;
        aa.board = atoi(loc);
        break;

      case 'E':
        aa.msgbase = GMB_EZYCOM;
        aa.board = atoi(loc);
        break;

      case 'J':
        aa.msgbase = GMB_JAM;
        aa.setpath(loc);
        break;

      case 'P':
        aa.msgbase = GMB_PCBOARD;
        aa.setpath(loc);
        break;

      case 'W':
        aa.msgbase = GMB_WILDCAT;
        aa.setpath(loc);
        break;

      case 'X':
        #ifdef __OS2__
        aa.msgbase = GMB_ADEPTXBBS;
        aa.setpath(loc);
        break;
        #else
        return;
        #endif

      case 'S':
        aa.msgbase = GMB_FTS1;
        aa.setpath(loc);
        break;

      case 'F':
      case 'O':
      default:
        aa.msgbase = GMB_OPUS;
        aa.setpath(loc);
    }

    aa.type = GMB_ECHO;
    aa.attr = CFG->attribsecho;
    strupr(base);
    if(strchr(base, 'N')) {
      aa.type = GMB_NET;
      aa.attr = CFG->attribsnet;
    }
    if(strchr(base, 'L')) {
      aa.type = GMB_LOCAL;
      aa.attr = CFG->attribslocal;
    }
    if(strchr(base, 'E')) {
      aa.type = GMB_ECHO;
      aa.attr = CFG->attribsecho;
    }
    if(attr) {                      // Get attributes and AKAs
      aa.attr.get(attr);
      if((ptr = strpbrk(attr, "0123456789")) != NULL)
        aa.aka = CFG->aka[atoi(ptr)].addr;   // Found AKA number
    }

    if(not aa.aka.net)
      aa.aka = CFG->aka[0].addr;

    add(aa);
  }
}


//  ------------------------------------------------------------------
//  Get area definition (new)
//
//  Syntax of AREADEF keyword:
//  
//    AREADEF <echoid> <"desc"> <group> <type> <msgbase> <path/board> <aka> <(attrs)> ["origin"]
//  
//  Examples:
//  
//    AREADEF NET.ALL  "Netmail, Line 1" N Net   Opus   R:\NETMAIL\             2:231/77 (PVT K/S)
//    AREADEF TEST     "Testing..."      D Echo  Hudson 67                      .        ()
//    AREADEF 1LOCAL   "Sysop <-> Users" L Local Squish R:\MAX\MSG\LOC\1LOCAL   .        (PVT)     "Your Sysop * The Goldware BBS Line 1"
//    AREADEF DANEINFO "DaneNet Info"    D Echo  Squish R:\MAX\MSG\231\DANEINFO .        (R/O)

bool garealist::get_area_firstpart(AreaCfg& aa, char*& key, char*& val) {

  const word CRC_NET      = 0xEC5E;
  const word CRC_ECHO     = 0xC2D1;
  const word CRC_LOCAL    = 0x4CD5;
  const word CRC_EMAIL    = 0x9C64;
  const word CRC_NEWS     = 0x61F1;

  // Get echoid
  getkeyval(&key, &val);
  aa.setechoid(key);

  // Get description
  getkeyval(&key, &val);
  aa.setdesc(key);

  // Get group (letter)
  getkeyval(&key, &val);
  if(*key == '#')
    aa.groupid = atoi(key+1)+0x8000u;
  else
    aa.groupid = (isupper(*key) ? *key : 0);

  // Get type
  word crc = getkeyvalcrc(&key, &val);
  switch(crc) {
    case CRC_NET:
      aa.type = GMB_NET;
      aa.attr = CFG->attribsnet;
      break;
    case CRC_ECHO:
      aa.type = GMB_ECHO;
      aa.attr = CFG->attribsecho;
      break;
    case CRC_LOCAL:
      aa.type = GMB_LOCAL;
      aa.attr = CFG->attribslocal;
      break;
    case CRC_EMAIL:
      aa.type = GMB_EMAIL | GMB_NET;
      aa.attr = CFG->attribsnet;
      break;
    case CRC_NEWS:
      aa.type = GMB_NEWSGROUP | GMB_ECHO;
      aa.attr = CFG->attribsecho;
      break;
    default:
      return false;
  }

  return true;
}


//  ------------------------------------------------------------------

void garealist::get_areasep(char* val) {

  char* key;
  AreaCfg aa;

  aa.reset();
  aa.msgbase = GMB_SEPARATOR;

  if(!get_area_firstpart(aa, key, val))
    return;

  add(aa);
}


//  ------------------------------------------------------------------

void garealist::get_areadef(char* val) {

  const word CRC_SEP      = 0x2FC6;
  const word CRC_FIDO     = 0x3AC8;
  const word CRC_OPUS     = 0x1254;
  const word CRC_SDMSG    = 0xAEC0;
  const word CRC_FTS1     = 0xEE2A;
  const word CRC_FTSC     = 0xEE58;
  const word CRC_QBBS     = 0x175B;
  const word CRC_HUDSON   = 0xBAB1;
  const word CRC_SQUISH   = 0xFCF6;
  const word CRC_EZYCOM   = 0xC81B;
  const word CRC_JAM      = 0xA8C3;
  const word CRC_GOLD     = 0x6134;
  const word CRC_GOLDBASE = 0x560D;
  const word CRC_PCB      = 0x19B7;
  const word CRC_PCBOARD  = 0x84EC;
  const word CRC_WCAT     = 0xAEDB;
  const word CRC_WILDCAT  = 0x7F3A;
  #if defined(__OS2__)
  const word CRC_XBBS     = 0xADC3;
  #endif

  char* key;
  AreaCfg aa;

  aa.reset();

  if(!get_area_firstpart(aa, key, val))
    return;

  // Get format
  word crc = getkeyvalcrc(&key, &val);
  getkeyval(&key, &val);        // Get path/board
  switch(crc) {
    case CRC_SEP:
      aa.msgbase = GMB_SEPARATOR;
      break;
    case CRC_FIDO:
    case CRC_OPUS:
    case CRC_SDMSG:
      aa.msgbase = GMB_OPUS;
      aa.setpath(key);
      break;
    case CRC_FTS1:
    case CRC_FTSC:
      aa.msgbase = GMB_FTS1;
      aa.setpath(key);
      break;
    case CRC_QBBS:
    case CRC_HUDSON:
      aa.msgbase = GMB_HUDSON;
      aa.board = atoi(key);
      if((aa.board < 1) or (aa.board > 200)) {
        return;
      }
      break;
    case CRC_GOLD:
    case CRC_GOLDBASE:
      aa.msgbase = GMB_GOLDBASE;
      aa.board = atoi(key);
      if((aa.board < 1) or (aa.board > 500)) {
        return;
      }
      break;
    case CRC_SQUISH:
      aa.msgbase = GMB_SQUISH;
      aa.setpath(key);
      break;
    case CRC_EZYCOM:
      aa.msgbase = GMB_EZYCOM;
      aa.board = atoi(key);
      break;
    case CRC_JAM:
      aa.msgbase = GMB_JAM;
      aa.setpath(key);
      break;
    case CRC_PCB:
    case CRC_PCBOARD:
      aa.msgbase = GMB_PCBOARD;
      aa.setpath(key);
      break;
    case CRC_WCAT:
    case CRC_WILDCAT:
      aa.msgbase = GMB_WILDCAT;
      aa.setpath(key);
      break;
    #ifdef __OS2__
    case CRC_XBBS:
      aa.msgbase = GMB_ADEPTXBBS;
      aa.setpath(key);
      break;
    #endif
    default:
      return;
  }

  // Get aka
  getkeyval(&key, &val);
  aa.aka = CFG->aka[0].addr;         // Seed aka with main address
  aa.aka.set(key);

  // Get attributes
  getkeyval(&key, &val);
  aa.attr.get(key);

  // Get origin (optional)
  getkeyval(&key, &val);
  aa.setorigin(key);

  // Add to arealist
  add(aa);
}


//  ------------------------------------------------------------------

void ReadEcholist(char* val) {

  AL.get_echolist(val);
}


//  ------------------------------------------------------------------

void garealist::get_echolist(char* val) {

  FILE* fp;
  char* key;
  Path file;
  char buf[256], options[80];

  // Get filename or options
  *file = NUL;
  *options = NUL;
  while(*val) {
    getkeyval(&key, &val);
    if((*key == '-') or (*key == '/')) {
      // Get option
      strcat(options, key);
      strcat(options, " ");
      continue;
    }
    strcpy(file, key);
  }

  if(*file) {

    fp = fsopen(file, "rt", CFG->sharemode);
    if(fp) {

      if(not quiet)
        printf("\n* Reading %s", file);
      ChkAddStamp(file, options);

      while(fgets((val=buf), sizeof(buf), fp)) {

        // Get echoid
        getkeyval(&key, &val);
        if(isalnum(*key)) {
          for(int n=0; n<list.size(); n++) {
            if(list[n]->echoid().ieql(key)) {
              list[n]->set_desc(val);
              break;
            }
          }
        }
      }

      fclose(fp);
    }
  }
}


//  ------------------------------------------------------------------

void garealist::get_areadesc(char* val) {

  int n;
  char* key;
  garealistdata* aa;

  // Get echoid and find area
  getkeyval(&key, &val);
  for(n=0; n<list.size(); n++) {
    if(list[n]->echoid().ieql(key)) {
      aa = list[n];
      break;
    }
  }
  if(n == list.size())   // Not found, ignore
    return;

  // Get description
  getkeyval(&key, &val);
  aa->set_desc(key);

  // Get group letter (optional)
  getkeyval(&key, &val);
  if(*key) {
    if(*key != '-') {
      if(*key == '#')
        aa->set_groupid(atoi(key+1)+0x8000u);
      else
        aa->set_groupid(toupper(*key));
    }

    // Get aka (optional)
    getkeyval(&key, &val);
    if(*key) {

      if(*key != '-') {
        ftn_addr aka(CFG->aka[0].addr);  // Seed aka with main addres
        aka.set(key);
        aa->set_aka(aka);
      }

      // Get attributes (optional)
      getkeyval(&key, &val);
      if(*key) {

        if(*key != '-') {
          ftn_attr attr;
          attr.get(key);
          aa->set_attr(attr);
        }

        // Get origin (optional)
        getkeyval(&key, &val);
        if(*key)
          aa->set_origin(key);
      }
    }
  }
}


//  ------------------------------------------------------------------

void garealistcfg::set_origin(const gstring& s) {

  if(not strblank(s.c_str())) {

    // Check if it already exists
    bool found = false;
    for(int n=0; n<CFG->origin.size(); n++) {
      if(streql(CFG->origin[n], s.c_str())) {
        cfg_originno = n;
        found = true;
        break;
      }
    }

    // Not found, so add it
    if(not found) {
      CfgOrigin(s.c_str());
      cfg_originno = CFG->origin.size() - 1;
    }
  }
}


//  ------------------------------------------------------------------

char* MapDrive(char* map) {

  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  _splitpath(map, drive, dir, file, ext);
  if(*drive) {
    if(isalpha(*drive))
      *drive = CFG->mapdrive[toupper(*drive) - 'A'];
    _makepath(map, drive, dir, file, ext);
  }
  return(map);
}


//  ------------------------------------------------------------------

