
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gccfgg1.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Configuration functions, part 1.
//  ------------------------------------------------------------------


#include "gecfgg.h"


//  ------------------------------------------------------------------

void golded_cfg::handle_address() {

  if(not strblank(val)) {

    gftnaddrs addr;
    uint pointnet = 0;

    char* ptr = strchr(val, ',');
    if(ptr) {
      pointnet = atow(ptr+1);
      *ptr = NUL;
    }
    if(address.size())
      addr = address[0];
    addr.set(val);

    if(group_active)
      group.add_address(addr);

    if(not find_address(addr)) {
      address.push_back(addr);
      addresspointnet.push_back(pointnet);
    }
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_addressmacro() {

  addressmacro.add(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_adeptxbbspath() {

  set_path(adeptxbbspath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_adeptxbbsuserno() {

  set_int(adeptxbbsuserno);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_akamatch() {

  next_keyval();

  ftn_akamatch tmp;
  tmp.mask.set_all(ftn::wildcard_all);
  tmp.mask.set(key);
  tmp.aka = val;

  akamatch.push_back(tmp);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_akamatchecho() {

  set_bool(akamatchecho);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_akamatching() {

  bool flag = get_bool(val);
  group.add_akamatching(flag);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_akamatchlocal() {

  set_bool(akamatchlocal);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_akamatchnet() {

  set_bool(akamatchnet);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_area() {

  arealist->get_area(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areaautoid() {

  gstrvaltable table[] = {
    { "VERBOSE", AUTOID_VERBOSE },
    { "SHORT",   AUTOID_SHORT },
    { "LONG",    AUTOID_LONG  }
  };

  set_int_table(areaautoid, table, sizeof(table));
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areaautonext() {

  set_bool(areaautonext);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areacopyaddid() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_areacopyaddid(flag);
  else
    areacopyaddid = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areacopydirect() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_areacopydirect(flag);
  else
    areacopydirect = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areacopyto() {

  gstring tmp = val;
  if(group_active)
    group.add_areacopyto(tmp);
  else
    areacopyto = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areadef() {

  arealist->get_areadef(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areadesc() {

  arealist->get_areadesc(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areaexcl() {

  tokenize(areaexcl, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areafilegroups() {

  set_bool(areafilegroups);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areafreqdirect() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_areafreqdirect(flag);
  else
    areafreqdirect = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areafreqto() {

  gstring tmp = val;
  if(group_active)
    group.add_areafreqto(tmp);
  else
    areafreqto = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areaincl() {

  tokenize(areaincl, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areaisemail() {

  tokenize(areaisemail, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areaisnews() {

  tokenize(areaisnews, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areakeeplast() {

  set_bool(areakeeplast);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_arealistechomax() {

  set_int(arealistechomax);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_arealistformat() {

  strstripquotes(val);
  set_string(arealistformat);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_arealistgroupid() {

  set_bool(arealistgroupid);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_arealistnos() {

  set_bool(arealistnos);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_arealistpagebar() {

  set_bool(arealistpagebar);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_arealistscan() {

  next_keyval();

  glistscan tmp;
  tmp.menutext = strstripquotes(key);
  tmp.file = val;

  arealist->listscan.push_back(tmp);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_arealistsort() {

  set_string(arealistsort);
  arealistsort.upper();
}

//  ------------------------------------------------------------------

void golded_cfg::handle_arealisttype() {

  gstrvaltable table[] = {
    { "NEW",  AL_TOTNEW },
    { "LAST", AL_TOTLST }
  };

  set_int_table(arealisttype, table, sizeof(table));
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areapath() {

  set_path(areapath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areapmscan() {

  tokenize(areapmscan, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areapmscanexcl() {

  tokenize(areapmscanexcl, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areapmscanincl() {

  tokenize(areapmscanincl, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areareadonly() {
  
  gstrvaltable table[] = {
    { "SOFT", READONLY_SOFT },
    { "HARD", READONLY_HARD }
  };

  set_int_table(areareadonly, table, sizeof(table));
}

//  ------------------------------------------------------------------

void golded_cfg::handle_arearename() {

  next_keyval();

  garearename tmp;
  tmp.from = key;
  tmp.to = strtrim(val);

  arearename.push_back(tmp);
}

//  ------------------------------------------------------------------

