
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gccfgg2.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Configuration functions, part 2.
//  ------------------------------------------------------------------


#include "gecfgg.h"


//  ------------------------------------------------------------------

void golded_cfg::handle_areareplydirect() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_areareplydirect(flag);
  else
    areareplydirect = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areareplyto() {

  gstring tmp = val;
  if(group_active)
    group.add_areareplyto(tmp);
  else
    areareplyto = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areascan() {

  tokenize(areascan, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areascanexcl() {

  tokenize(areascanexcl, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areascanincl() {

  tokenize(areascanincl, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areascansort() {

  set_string(areascansort);
  areascansort.upper();
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areasep() {

  arealist->get_areasep(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areastart() {

  set_string(areastart);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_areatypeorder() {
  
  const word CRC_NET   = 0xEC5E;
  const word CRC_ECHO  = 0xC2D1;
  const word CRC_LOCAL = 0x4CD5;
  const word CRC_EMAIL = 0x9C64;
  const word CRC_NEWS  = 0x61F1;

  int order = 1;
  gtokenizer token(val);
  while(token.get()) {
    switch(gcrc16u(token.c_str())) {
      case CRC_NET:    areatypeorder[GMB_NET]                = order;  break;
      case CRC_EMAIL:  areatypeorder[GMB_NET|GMB_EMAIL]      = order;  break;
      case CRC_ECHO:   areatypeorder[GMB_ECHO]               = order;  break;
      case CRC_NEWS:   areatypeorder[GMB_ECHO|GMB_NEWSGROUP] = order;  break;
      case CRC_LOCAL:  areatypeorder[GMB_LOCAL]              = order;  break;
      default:         error = error_unknown_value;                    return;
    }
    order++;
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_askdelorig() {

  set_yesno(askdelorig);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_attribsattach() {

  set_attrib(attribsattach);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_attribscc() {

  set_attrib(attribscc);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_attribscfm() {

  set_attrib(attribscfm);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_attribsecho() {

  set_attrib(attribsecho);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_attribsemail() {

  set_attrib(attribsemail);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_attribsfrq() {

  set_attrib(attribsfrq);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_attribslocal() {

  set_attrib(attribslocal);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_attribsnet() {

  set_attrib(attribsnet);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_attribsnews() {

  set_attrib(attribsnews);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_attributes() {

  ftn_attr tmp = val;
  group.add_attributes(tmp);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_baseon() {

  strarray tmp;
  tokenize(tmp, val);
  for(int n=0; n<tmp.size(); n++)
    group.inherit_from(tmp[n]);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_beepcomment() {

  set_bool(beepcomment);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_beepfactor() {

  set_int(beepfactor);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_beeplocalmsg() {

  set_bool(beeplocalmsg);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_beepnoises() {

  set_bool(beepnoises);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_beepyourmail() {

  set_bool(beepyourmail);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_carboncopylist() {

  gstrvaltable table[] = {
    { "REMOVE",  CC_REMOVE },
    { "HIDDEN",  CC_HIDDEN },
    { "VISIBLE", CC_VISIBLE },
    { "NAMES",   CC_NAMES },
    { "KEEP",    CC_KEEP },
    { "TRUE",    CC_KEEP },
    { "YES",     CC_KEEP },
    { "NO",      CC_REMOVE },
    { "FALSE",   CC_REMOVE }
  };

  set_int_table(carboncopylist, table, sizeof(table));
}

//  ------------------------------------------------------------------

void golded_cfg::handle_color() {

  get_colors();
}

//  ------------------------------------------------------------------

void golded_cfg::handle_colorset() {

  gstrvaltable table[] = {
    { "NORMAL",  COLORSET_NORMAL },
    { "MONO",    COLORSET_MONO },
    { "INTENSE", COLORSET_INTENSE }
  };

  int colorset = 0;
  set_int_table(colorset, table, sizeof(table));

  if(not error) {
    switch(colorset) {
      case COLORSET_NORMAL:
        memcpy(color, gold_color1, sizeof(gcolor)*16);
        break;
      case COLORSET_MONO:
        memcpy(color, gold_mono1, sizeof(gcolor)*16);
        break;
      case COLORSET_INTENSE:
        intensecolors = true;
        memcpy(color, gold_color2, sizeof(gcolor)*16);
        break;
    }
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_confirmfile() {

  set_file(confirmfile);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_confirmresponse() {

  set_yesno(confirmresponse);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_cookiepath() {

  set_path(cookiepath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_crosspostlist() {

  gstrvaltable table[] = {
    { "NONE",    CC_REMOVE },
    { "VERBOSE", CC_NAMES },
    { "RAW",     CC_KEEP },
    { "TRUE",    CC_NAMES },
    { "YES",     CC_NAMES },
    { "NO",      CC_REMOVE },
    { "FALSE",   CC_REMOVE }
  };

  set_int_table(crosspostlist, table, sizeof(table));
}

//  ------------------------------------------------------------------

void golded_cfg::handle_ctrlinfo() {

  int ctrlinfo;
  if(get_ctrlinfo(ctrlinfo))
    group.add_ctrlinfo(ctrlinfo);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_ctrlinfoecho() {

  get_ctrlinfo(ctrlinfoecho);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_ctrlinfoemail() {

  get_ctrlinfo(ctrlinfoemail);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_ctrlinfolocal() {

  get_ctrlinfo(ctrlinfolocal);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_ctrlinfonet() {

  get_ctrlinfo(ctrlinfonet);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_ctrlinfonews() {

  get_ctrlinfo(ctrlinfonews);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_dispareano() {

  set_bool(dispareano);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_dispattachsize() {

  gstrvaltable table[] = {
    { "KBYTES", ATTACH_KBYTES },
    { "BYTES",  ATTACH_BYTES },
    { "NO",     false }
  };

  set_int_table(dispattachsize, table, sizeof(table));
}

//  ------------------------------------------------------------------

void golded_cfg::handle_dispautonext() {

  set_bool(dispautonext);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_disphdrdateset() {

  next_keyval();
  disphdrdateset.pos = atoi(key);
  disphdrdateset.len = atoi(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_disphdrnameset() {

  next_keyval();
  disphdrnameset.pos = atoi(key);
  disphdrnameset.len = atoi(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_disphdrnodeset() {

  next_keyval();
  disphdrnodeset.pos = atoi(key);
  disphdrnodeset.len = atoi(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_displistcursor() {

  gstrvaltable table[] = {
    { "TOP",        LIST_TOP },
    { "NEARTOP",    LIST_NEARTOP },
    { "MIDDLE",     LIST_MIDDLE },
    { "NEARBOTTOM", LIST_NEARBOTTOM },
    { "BOTTOM",     LIST_BOTTOM }
  };

  set_int_table(displistcursor, table, sizeof(table));
}

//  ------------------------------------------------------------------

