
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gccfgg3.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Configuration functions, part 3.
//  ------------------------------------------------------------------


#include "gecfgg.h"


//  ------------------------------------------------------------------

void golded_cfg::handle_displistwrap() {

  set_bool(displistwrap);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_displocalhigh() {

  set_bool(displocalhigh);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_dispmargin() {

  set_int(dispmargin);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_dispmsgsize() {

  gstrvaltable table[] = {
    { "KBYTES", DISPMSGSIZE_KBYTES },
    { "BYTES",  DISPMSGSIZE_BYTES },
    { "LINES",  DISPMSGSIZE_LINES },
    { "NO",     false }
  };

  set_int_table(dispmsgsize, table, sizeof(table));
}

//  ------------------------------------------------------------------

void golded_cfg::handle_disppagebar() {

  set_bool(disppagebar);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_disprealmsgno() {

  set_bool(disprealmsgno);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_dispsoftcr() {

  set_bool(dispsoftcr);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_dispstatusline() {

  set_bool(dispstatusline);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_disptabsize() {

  set_int(disptabsize, 0, 8);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_dosprompt() {

  set_bool(dosprompt);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_dosswap() {

  #if defined(USE_SPAWNO) or defined(USE_EXECSWAP)

  set_inf(dosswap);
  if(dosswap == 0) {
    gtokenizer token(",. \t", val);
    while(token.get()) {
      if(strieql(token.c_str(), "YES")) {
        dosswap |= GSWAP_ALL;
      }
      else if(strieql(token.c_str(), "NO")) {
        dosswap = GSWAP_NONE;
        break;
      }
      else if(strieql(token.c_str(), "XMS")) {
        dosswap |= GSWAP_XMS;
        if(not (dosswap & GSWAP_EMS))
          dosswap |= GSWAP_XMS_FIRST;
      }
      else if(strieql(token.c_str(), "EMS")) {
        dosswap |= GSWAP_EMS;
        if(not (dosswap & GSWAP_XMS))
          dosswap |= GSWAP_EMS_FIRST;
      }
      else if(strieql(token.c_str(), "Disk")) {
        dosswap |= GSWAP_DISK;
      }
      else if(strieql(token.c_str(), "Hide")) {
        dosswap |= GSWAP_HIDE_FILE;
      }
      else if(strieql(token.c_str(), "NoPreAlloc")) {
        dosswap |= GSWAP_NO_PREALLOC;
      }
      else if(strieql(token.c_str(), "CheckNet")) {
        dosswap |= GSWAP_CHECK_NET;
      }
    }
  }

  #endif
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editautoattach() {

  set_bool(editautoattach);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editautosave() {

  set_int(editautosave);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editchangedate() {

  set_bool(editchangedate);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editcharpara() {

  char ch = *strstripquotes(val);
  if(ch)
    editcharpara = ch;
  else
    error = error_invalid_value;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editcharspace() {

  char ch = *strstripquotes(val);
  if(ch)
    editcharspace = ch;
  else
    error = error_invalid_value;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editcomment() {

  next_keyval();

  gedittrigger e;
  e.trigger = strstripquotes(key);
  e.text = strstripquotes(val);

  editcomment.push_back(e);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editcompletion() {

  next_keyval();

  gedittrigger e;
  e.trigger = strstripquotes(key);
  e.text = strstripquotes(val);

  editcompletion.push_back(e);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editcrlfterm() {

  set_bool(editcrlfterm);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editfieldclear() {

  set_bool(editfieldclear);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_edithardline() {

  strstripquotes(val);
  set_string(edithardline);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_edithardlines() {

  set_bool(edithardlines);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_edithardterm() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_edithardterm(flag);
  else
    edithardterm = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_edithdrnameset() {

  next_keyval();
  edithdrnamepos = atoi(key);
  edithdrnamelen = atoi(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_edithdrnodeset() {

  next_keyval();
  edithdrnodepos = atoi(key);
  edithdrnodelen = atoi(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editheaderattrs() {

  set_bool(editheaderattrs);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editheaderfirst() {

  int v = EDITHEADERFIRST_NO;
  gtokenizer token(val);
  while(token.get()) {
    if(strieql("NEW", token.c_str()))
      v |= EDITHEADERFIRST_NEW;
    else if(strieql("CHANGES", token.c_str()))
      v |= EDITHEADERFIRST_CHANGES;
    else if(strieql("REPLIES", token.c_str()))
      v |= EDITHEADERFIRST_REPLIES;
    else if(strieql("FORWARDS", token.c_str()))
      v |= EDITHEADERFIRST_FORWARDS;
    else
      v = get_yesno(token.c_str()) ? EDITHEADERFIRST_YES : EDITHEADERFIRST_NO;
  }
  editheaderfirst = v;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editinternal() {

  set_bool(editinternal);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editmargin() {

  set_int(editmargin);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_Editmenu() {

  set_bool(editmenu);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editmixcase() {

  bool tmp = get_bool(val);
  if(group_active)
    group.add_editmixcase(tmp);
  else
    editmixcase = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editmsgsize() {

  set_long(editmsgsize);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editor() {

  set_string(editexternal);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editorfile() {

  set_file(editfile);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editquotemargin() {

  set_int(editquotemargin);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editreplyre() {

  int tmp;
  if(strieql(val, "NUMERIC"))
    tmp = REPLYRE_NUMERIC;
  else
    tmp = get_bool(val) ? REPLYRE_YES : REPLYRE_NO;

  if(group_active)
    group.add_editreplyre(tmp);
  else
    editreplyre = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editsavemenu() {

  set_bool(editsavemenu);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editsaveutil() {

  next_keyval();

  geditsaveutil e;
  e.number = atoi(key);
  e.text = strstripquotes(val);

  editsaveutil.push_back(e);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editsoftcrxlat() {

  editsoftcrxlat = *val;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editspellcheck() {

  set_string(editspellchecker);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_editundelete() {

  set_int(editundelete);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_emptytearline() {

  set_bool(emptytearline);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_endgroup() {

  if(group_active)
    group_active = false;
  else
    error = error_misplaced_endgroup;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_event() {

  word x = get_keyvalcrc(key, val);

  int n;
  for(n=0; n<event.size(); n++)
    if(x == event[n].type)
      break;

  if(n == event.size()) {
    gevent e;
    event.push_back(e);
  }

  event[n].type = x;

  next_keyval();

  if(strieql(key, "PLAY")) {
    event[n].command = EVTC_PLAY;
    event[n].play.get_file(val);
  }
}

//  ------------------------------------------------------------------

