
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gccfgg4.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Configuration functions, part 4.
//  ------------------------------------------------------------------


#include "gecfgg.h"


//  ------------------------------------------------------------------

void golded_cfg::handle_excludenodes() {

  // Only used by GoldNODE
}

//  ------------------------------------------------------------------
              
void golded_cfg::handle_externoptions() {

  const word CRC_CLS            = 0x34F4;
  const word CRC_SWAP           = 0x30AE;
  const word CRC_CURSOR         = 0x0D67;
  const word CRC_RELOAD         = 0xDEC0;
  const word CRC_PAUSE          = 0x3926;
  const word CRC_PAUSEONERROR   = 0xD99A;
  const word CRC_WIPE           = 0x1E80;
  const word CRC_KEEPCTRL       = 0x0738;
  const word CRC_NOCLS          = 0x2FAA;
  const word CRC_NOSWAP         = 0xCDF4;
  const word CRC_NOCURSOR       = 0x9345;
  const word CRC_NORELOAD       = 0x40E2;
  const word CRC_NOPAUSE        = 0x5D94;
  const word CRC_NOPAUSEONERROR = 0xDBAA;
  const word CRC_NOWIPE         = 0xE3DA;
  const word CRC_NOKEEPCTRL     = 0x38FC;

  // Handle options
  while(*val) {

    if((*val == '-') or (*val == '/'))
      val++;
    
    word crc = get_keyvalcrc(key, val);

    switch(crc) {
      case CRC_CLS:            externoptions |= gexternutil::cls;           break;
      case CRC_SWAP:           externoptions |= gexternutil::swap;          break;
      case CRC_CURSOR:         externoptions |= gexternutil::cursor;        break;
      case CRC_RELOAD:         externoptions |= gexternutil::reload;        break;
      case CRC_PAUSE:          externoptions |= gexternutil::pause;         break;
      case CRC_PAUSEONERROR:   externoptions |= gexternutil::pauseonerror;  break;
      case CRC_WIPE:           externoptions |= gexternutil::wipe;          break;
      case CRC_KEEPCTRL:       externoptions |= gexternutil::keepctrl;      break;
      case CRC_NOCLS:          externoptions &= ~gexternutil::cls;          break;
      case CRC_NOSWAP:         externoptions &= ~gexternutil::swap;         break;
      case CRC_NOCURSOR:       externoptions &= ~gexternutil::cursor;       break;
      case CRC_NORELOAD:       externoptions &= ~gexternutil::reload;       break;
      case CRC_NOPAUSE:        externoptions &= ~gexternutil::pause;        break;
      case CRC_NOPAUSEONERROR: externoptions &= ~gexternutil::pauseonerror; break;
      case CRC_NOWIPE:         externoptions &= ~gexternutil::wipe;         break;
      case CRC_NOKEEPCTRL:     externoptions &= ~gexternutil::keepctrl;     break;
    }
  }
}


//  ------------------------------------------------------------------

void golded_cfg::handle_externutil() {

  char* val2 = val;
  get_keyval(key, val2);

  gexternutil extutil;
  extutil.utilno = atoi(key);

  int optbak = externoptions;
  while(strchr("/-", *val2)) {
    get_keyval(key, val2);
    val = key;
    handle_externoptions();
  }
  extutil.options = externoptions;
  externoptions = optbak;

  extutil.cmdline = val2;

  externutil.push_back(extutil);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_ezycommsgbase() {

  set_path(ezycommsgbasepath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_ezycomuserbase() {

  set_path(ezycomuserbasepath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_ezycomuserno() {

  set_int(ezycomuserno);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_fidohwmarks() {

  set_bool(fidohwmarks);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_fidolastread() {

  set_string(fidolastread);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_fidomsgtype() {

  if(striinc("FTS", val))
    fidomsgtype = GMB_FTS1;
  else
    fidomsgtype = GMB_OPUS;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_fidonullfix() {

  set_bool(fidonullfix);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_fidouserlist() {

  set_file(fidouserlist);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_fidouserno() {

  set_int(fidouserno);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_filealias() {

  next_keyval();

  gfilealias f;
  f.alias = key;
  f.file = val;

  filealias.push_back(f);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_filelistpagebar() {

  set_bool(filelistpagebar);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_followupto() {

  gstring tmp = val;
  group.add_followupto(tmp);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_forcetemplate() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_forcetemplate(flag);
  else
    forcetemplate = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_frqext() {

  gtokenizer token(val);
  while(token.get()) {

    // Do we have it already?
    for(int n=0; n<frqext.size(); n++)
      if(strieql(token.c_str(), frqext[n]))
        return;

    // Nope, so add it
    frqext.add(token.c_str());
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_frqnodemap() {
  
  gfrqnodemap tmp;
  next_keyval();
  tmp.to = key;
  next_keyval();
  tmp.from = key;
  frqnodemap.push_back(tmp);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_Frqoptions() {

  if(striinc("SORT", val))
    frqoptions |= FREQ_SORT;
  if(striinc("FROMTOP", val))
    frqoptions |= FREQ_FROMTOP;
  if(striinc("NOFILES", val))
    frqoptions |= FREQ_NOFILES;
  if(striinc("FAST", val))
    frqoptions |= FREQ_FAST;
  if(striinc("NOWAZOOMSG", val))
    frqoptions |= FREQ_NOWAZOOMSG;
  if(striinc("NOTFROMTOP", val))
    frqoptions &= ~FREQ_FROMTOP;
  if(striinc("USEFLOWFILE", val))
    frqoptions |= FREQ_USEFLOWFILE;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_frqwazoo() {

  set_bool(frqwazoo);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_gif() {

  gstring buf = val;
  const char* ptr = buf.chr('.');
  if(ptr)
    buf.erase(ptr);
  if(group_active)
    group.add_gif(buf);
  else
    gif.add(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_gifpath() {

  set_path(gifpath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_goldbasepath() {

  set_path(goldbasepath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_goldbasesyspath() {

  set_path(goldbasesyspath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_goldbaseuserno() {

  set_int(goldbaseuserno);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_goldpath() {

  set_path(goldpath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_group() {

  next_keyval();

  group.add_group(key);
  group_active = true;

  groups++;
}

//  ------------------------------------------------------------------

