
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gccfgg5.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Configuration functions, part 5.
//  ------------------------------------------------------------------


#include "gecfgg.h"


//  ------------------------------------------------------------------

void golded_cfg::handle_highlightunread() {

  set_bool(highlightunread);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_hudsonpath() {

  set_path(hudsonpath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_hudsonsizewarn() {

  set_long(hudsonsizewarn);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_hudsonsyspath() {

  set_path(hudsonsyspath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_hudsonuserno() {

  set_int(hudsonuserno);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_importbegin() {

  strstripquotes(val);
  set_string(importbegin);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_importend() {

  strstripquotes(val);
  set_string(importend);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_inboundpath() {

  set_path(inboundpath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_includenodes() {

  // Only used by GoldNODE
}

//  ------------------------------------------------------------------

void golded_cfg::handle_inputfile() {

  gstring buf = val;
  if(group_active)
    group.add_inputfile(buf);
  else
    inputfile = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_intensecolors() {

  set_bool(intensecolors);
  if(intensecolors)
    memcpy(color, gold_color2, sizeof(color));
}

//  ------------------------------------------------------------------

void golded_cfg::handle_internetaddress() {

  gstring buf;
  if(group_active)
    group.add_internetaddress(buf);
  else
    internetaddress = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_internetgate() {

  gnode tmp;
  char* ptr = strchr(val, ',');
  if(ptr) {
    *ptr++ = NUL;
    tmp.name = strtrim(val);
  }
  else {
    ptr = val;
  }
  if(address.size())
    tmp.addr = address[0];
  tmp.addr.set(strskip_wht(ptr));
  if(group_active)
    group.add_internetgate(tmp);
  else
    internetgate = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_internetlookup() {

  set_bool(internetlookup);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_internetmsgid() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_internetmsgid(flag);
  else
    internetmsgid = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_internetreply() {

  set_bool(internetreply);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_internetrfcbody() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_internetrfcbody(flag);
  else
    internetrfcbody = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_invalidate() {

  const word CRC_TEARLINE = 0xB385;
  const word CRC_ORIGIN   = 0x4CE5;
  const word CRC_SEENBY   = 0x01D0;

  switch(get_keyvalcrc(key, val)) {
    case CRC_TEARLINE:
      next_keyval();
      invalidate.tearline[0] = strstripquotes(key);
      invalidate.tearline[1] = strstripquotes(val);
      break;
    case CRC_ORIGIN:
      next_keyval();
      invalidate.origin[0] = strstripquotes(key);
      invalidate.origin[1] = strstripquotes(val);
      break;
    case CRC_SEENBY:
      next_getkeyval();
      invalidate.seenby[0] = strstripquotes(key);
      invalidate.seenby[1] = strstripquotes(val);
      break;
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_jamharddelete() {

  set_bool(jamharddelete);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_jampath() {

  set_path(jampath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_jamskipdeleted() {

  set_bool(jamskipdeleted);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_keybclear() {

  set_bool(keybclear);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_keybdef() {

  if(not keyboard.add(val))
    error = error_unknown_keyboard_keyword;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_keybdefaults() {

  keyboard.defaults = get_bool(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_keybext() {

  set_bool(keybext);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_keybmode() {

  if(strieql(val, "POLL"))
    keybmode = KEYB_POLL;
  else if(strieql(val, "BLOCK"))
    keybmode = KEYB_BLOCK;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_keybstack() {

  keybstack += val;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_kludge() {

  strstripquotes(val);
  if(*val == CTRL_A)
    val++;
  kludge.add(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_kludgechrs() {

  set_bool(kludgechrs);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_langdef() {

  next_keyvalcrc();
  if(not language.switch_language(crc, val))
    error = error_unknown_language_keyword;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_loadlanguage() {

  gstring buf = val;
  if(group_active)
    group.add_loadlanguage(buf);
  else
    loadlanguage = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_logfile() {

  set_file(logfile);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_logformat() {

  if(strieql(val, "FD"))
    logformat = GLOG_FD;
  else if(strieql(val, "MAX"))
    logformat = GLOG_MAX;
  else if(strieql(val, "BINK"))
    logformat = GLOG_BINK;
  else if(strieql(val, "QBBS"))
    logformat = GLOG_QBBS;
  else if(strieql(val, "DB"))
    logformat = GLOG_DB;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_lookupecho() {

  set_bool(lookupecho);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_lookuplocal() {

  set_bool(lookuplocal);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_lookupnet() {

  set_bool(lookupnet);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_lookupuserbase() {

  set_bool(lookupuserbase);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_mailinglist() {

  gmailinglist m;
  next_keyval();
  m.echoid = key;
  next_keyval();
  m.sender = key;
  next_keyval();
  m.contribution = key;
  mailinglist.push_back(m);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_mapdrive() {

  strupr(val);
  if(isalpha(*val)) {
    char* ptr = val;
    ptr = strskip_txt(ptr);
    ptr = strskip_wht(ptr);
    if(isalpha(*ptr))
      mapdrive[*val-'A'] = *ptr;
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_member() {

  gtokenizer token(val);
  while(token.get())
    group.add_member(token.c_str());
}

//  ------------------------------------------------------------------

void golded_cfg::handle_menudropmsg() {

  set_bool(menudropmsg);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_menumarked() {

  if(strieql(val, "Marked"))
    menumarked = MODE_MARKED;
  else if(strieql(val, "Current"))
    menumarked = MODE_CURRENT;
  else if(strieql(val, "Previous"))
    menumarked = MODE_PREVIOUS;
  else
    menumarked = MODE_DONT;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_mood() {

  gstring buf = val;
  if(group_active)
    group.add_mood(buf);
  else
    mood.add(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_mouse() {

  set_bool(mouse);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_msglistdate() {

  int flag;
  if(strieql(val, "Written"))
    flag = MSGLISTDATE_WRITTEN;
  else if(strieql(val, "Arrived"))
    flag = MSGLISTDATE_ARRIVED;
  else if(strieql(val, "Received"))
    flag = MSGLISTDATE_RECEIVED;
  else
    flag = get_bool(val) ? MSGLISTDATE_WRITTEN : MSGLISTDATE_NONE;
  if(group_active)
    group.add_msglistdate(flag);
  else
    msglistdate = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_msglistfast() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_msglistfast(flag);
  else
    msglistfast = flag;
}

//  ------------------------------------------------------------------

