
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gccfgg6.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Configuration functions, part 6.
//  ------------------------------------------------------------------


#include "gecfgg.h"


//  ------------------------------------------------------------------

void golded_cfg::handle_msglistfirst() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_msglistfirst(flag);
  else
    msglistfirst = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_msglistheader() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_msglistheader(flag);
  else
    msglistheader = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_msglistpagebar() {

  set_bool(msglistpagebar);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_msglistviewsubj() {

  set_bool(msglistviewsubj);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_msglistwidesubj() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_msglistwidesubj(flag);
  else
    msglistwidesubj = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_namesfile() {

  set_file(namesfile);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_netname() {

  gstring buf = strstripquotes(val);
  if(group_active)
    group.add_netname(buf);
  else
    netname = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_newsgroups() {

  gstring tmp = val;
  group.add_newsgroups(tmp);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_nodelist() {

  // Only used by GoldNODE
}

//  ------------------------------------------------------------------

void golded_cfg::handle_nodelistpagebar() {

  set_bool(nodelistpagebar);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_nodelistwarn() {

  set_bool(nodelistwarn);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_nodepath() {

  set_path(nodepath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_nodepathfd() {

  set_path(nodepathfd);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_nodepathv7() {

  set_path(nodepathv7);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_nodev7flags() {

  gtokenizer token(val);
  token.get();
  const char* flagstring = token.c_str();
  token.get();
  const char* flagvals = token.c_str();
  uint flagval;
  if(tolower(*flagvals) == 'b')
    flagval = atoi(flagvals+1);
  else {
    flagval = atoi(flagvals);
    switch(flagval) {
      case    32: flagval =  5; break;
      case    64: flagval =  6; break;
      case   128: flagval =  7; break;
      case   256: flagval =  8; break;
      case   512: flagval =  9; break;
      case  1024: flagval = 10; break;
      case  2048: flagval = 11; break;
      case  8192: flagval = 13; break;
      case 16384: flagval = 14; break;
      default:    flagval = 0;
    }
  }
  if(flagval)
    sprintf(nodev7flags[flagval], "%s,", flagstring);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_nodev7modem() {

  gtokenizer token(val);
  token.get();
  const char* modemstring = token.c_str();
  const char* modemvals = token.c_str();
  int modemval;
  if(tolower(*modemvals) == 'b')
    modemval = atoi(modemvals+1);
  else {
    modemval = atoi(modemvals);
    switch(modemval) {
      case   1: modemval = 0; break;
      case   2: modemval = 1; break;
      case   4: modemval = 2; break;
      case   8: modemval = 3; break;
      case  16: modemval = 4; break;
      case  32: modemval = 5; break;
      case  64: modemval = 6; break;
      case 128: modemval = 7; break;
      default:  modemval = -1;
    }
  }
  if(modemval >= 0)
    sprintf(nodev7modem[modemval], "%s,", modemstring);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_organization() {

  gstring buf = val;
  if(group_active)
    group.add_organization(buf);
  else
    organization = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_origin() {

  if(not strblank(val)) {
    strstripquotes(val);
    gstring buf = val;
    if(group_active)
      group.add_origin(buf);
    else
      origin.add(val);
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_outboundpath() {

  set_path(outboundpath, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_outputfile() {

  gstring buf = val;
  if(group_active)
    group.add_outputfile(buf);
  else
    outputfile = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_pcboardmailer() {

  if(strieql(val, "PCBOARD"))
    pcboardmailer = PCBOARDMAILER_PCBOARD;
  else
    pcboardmailer = PCBOARDMAILER_OTHER;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_pcboardpath() {
  
  set_path(pcboardpath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_pcboarduserno() {

  set_int(pcboarduserno);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_personalmail() {

  if(striinc("STARTUP", val))
    personalmail |= PM_STARTUP;
  if(striinc("NOTSTARTUP", val))
    personalmail &= ~PM_STARTUP;
  if(striinc("ALLNAMES", val))
    personalmail |= PM_ALLNAMES;
  if(striinc("NOTALLNAMES", val))
    personalmail &= ~PM_ALLNAMES;
  if(striinc("LISTONLY", val))
    personalmail |= PM_LISTONLY;
  if(striinc("NOTLISTONLY", val))
    personalmail &= ~PM_LISTONLY;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_play() {

  gstring tmp = val;
  if(group_active)
    group.add_play(tmp);
  else
    play.get_file(tmp);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_printdevice() {

  set_string(printdevice);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_printformfeed() {

  set_bool(printformfeed);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_printinit() {

  CvtPrnstr(printinit, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_printlength() {

  set_int(printlength);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_printmargin() {

  set_int(printmargin);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_printreset() {

  CvtPrnstr(printreset, val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_quoteblank() {

  set_bool(quoteblank);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_quotebuffile() {

  gstring buf = val;
  if(group_active)
    group.add_quotebuffile(buf);
  else
    quotebuffile = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_quotebufmode() {

  if(strieql(val, "ASK"))
    quotebufmode = QBUF_ASK;
  else if(strieql(val, "APPEND"))
    quotebufmode = QBUF_APPEND;
  else if(strieql(val, "OVERWRITE"))
    quotebufmode = QBUF_OVERWRITE;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_quotechars() {

  gstring buf = strstripquotes(val);
  if(group_active)
    group.add_quotechars(buf);
  else
    quotechars = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_quotemargin() {

  set_int(quotemargin);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_quotespacing() {

  set_bool(quotespacing);
}

//  ------------------------------------------------------------------

