
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gccfgg7.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Configuration functions, part 7.
//  ------------------------------------------------------------------


#include "gecfgg.h"


//  ------------------------------------------------------------------

void golded_cfg::handle_quotestring() {

  gstring buf = strstripquotes(val);
  if(buf.empty())
    buf = " > ";
  else {
    if(buf.lastchar() != ' ')
      buf += " ";
  }
  gstring tmp = buf;
  if(group_active)
    group.add_quotestring(tmp);
  else
    quotestring = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_quotewraphard() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_quotewraphard(flag);
  else
    quotewraphard = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_qwkbadmsgs() {

  qwk->BadMsgs(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_qwkconfmap() {

  char* bbsid;
  get_keyval(bbsid, val);

  char* confname;
  get_keyval(confname, val);
  strtrim(strstripquotes(confname));

  char* echoid;
  get_keyval(echoid, val);

  qwk->AddMap(bbsid, echoid, confname);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_qwkexportpath() {

  qwk->ExportPath(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_qwkimportpath() {

  qwk->ImportPath(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_qwkoptions() {

  char* bbsid;
  get_keyval(bbsid, val);
  qwk->FindAddBBS(bbsid);
  gtokenizer token(val);
  while(token.get()) {
    if(strieql(token.c_str(), "KLUDGES"))
      qwk->KludgesAllowed(true);
    else if(strieql(token.c_str(), "NOKLUDGES"))
      qwk->KludgesAllowed(false);
    else if(strieql(token.c_str(), "MIXCASE"))
      qwk->MixCaseAllowed(true);
    else if(strieql(token.c_str(), "NOMIXCASE"))
      qwk->MixCaseAllowed(false);
    else if(strieql(token.c_str(), "RECEIPT"))
      qwk->ReceiptAllowed(true);
    else if(strieql(token.c_str(), "NORECEIPT"))
      qwk->ReceiptAllowed(false);
    else if(strnieql(token.c_str(), "MAXLINES=", 9))
      qwk->MaxLines(atoi(token.c_str()+9));
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_qwkreplylinker() {

  qwk->ReplyLinker(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_qwktosslog() {

  qwk->TossLog(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_ra2usersbbs() {

  ra2usersbbs = get_bool(val) ? 2 : 1;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_replylink() {

  if(strieql(val, "Chain"))
    replylink = REPLYLINK_CHAIN;
  else if(strieql(val, "Direct"))
    replylink = REPLYLINK_DIRECT;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_replylinklist() {

  if(strieql(val, "Fast"))
    replylinklist = REPLYLINKLIST_FAST;
  else if(strieql(val, "Full"))
    replylinklist = REPLYLINKLIST_FULL;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_replyto() {

  gstring tmp = val;
  if(group_active)
    group.add_replyto(tmp);
  else
    replyto = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_robotname() {

  gtokenizer token(val);
  while(token.get()) {

    // Do we have it already?
    for(int n=0; n<robotname.size(); n++)
      if(strieql(token.c_str(), robotname[n]))
        return;

    // Nope, so add it
    robotname.add(token.c_str());
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_screenblanker() {

  gtokenizer token(val);
  while(token.get()) {
    if(isdigit(*token.c_str()))
      screenblanker = atoi(token.c_str());
    else if(striinc("BLACK", token.c_str()))
      screenblankertype = BLANK_ALLBLACK;
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_screenelimsnow() {

  set_bool(screenelimsnow);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_screenmaxcol() {

  set_int(screenmaxcol);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_screenmaxrow() {

  set_int(screenmaxrow);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_screenpalette() {

  //  Syntax:
  //
  //    SCREENPALETTE <reg> <val>   or
  //    SCREENPALETTE <reg> (red green blue)

  next_keyval();

  int palreg = atoi(key);
  if(palreg >= 0 and palreg <= 15) {
    if(*val != '(')
      screenpalette[palreg] = atoi(val);
    else {
      val = strskip_wht(val+1);
      int redval = atoi(val);         // 0 .. 3
      val = strskip_txt(val);
      val = strskip_wht(val);
      int greenval = atoi(val);       // 0 .. 3
      val = strskip_txt(val);
      val = strskip_wht(val);
      int blueval = atoi(val);        // 0 .. 3

      // Construct the rgbRGB color val (nuts!)
      screenpalette[palreg] =
        ((redval   & 2) ? 0x04 : 0x00)|
        ((redval   & 1) ? 0x20 : 0x00)|
        ((greenval & 2) ? 0x02 : 0x00)|
        ((greenval & 1) ? 0x10 : 0x00)|
        ((blueval  & 2) ? 0x01 : 0x00)|
        ((blueval  & 1) ? 0x08 : 0x00);
    }
    screenpalette[16] = true;    // Indicate that changes were made
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_screenshadows() {

  set_bool(screenshadows);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_screensize() {

  set_int(screensize);
  if(screensize == 4350) {
    if(gvid.adapter & V_EGA)
      screensize = 43;
    else
      screensize = 50;
  }
  else if(screensize == 0) {
    if(strnieql(val, "Mode", 4)) {
      sscanf(val+4, " %x", &screensize);
      screensize <<= 8;
    }
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_screenusebios() {

  set_bool(screenusebios);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_searchfor() {
  
  gstring buf = strstripquotes(val);
  if(group_active)
    group.add_searchfor(buf);
  else
    searchfor = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_semaphore() {

  next_keyval();
  if(strieql(key, "NETSCAN"))
    set_file(semaphore.netscan);
  else if(strieql(key, "ECHOSCAN"))
    set_file(semaphore.echoscan);
  else if(strieql(key, "EXPORTLIST"))
    set_file(semaphore.exportlist);
  else if(strieql(key, "IMPORTLIST"))
    set_file(semaphore.importlist);
  else if(strieql(key, "SCANALL"))
    set_file(semaphore.scanall);
  else if(strieql(key, "SCANTHIS"))
    set_file(semaphore.scanthis);
  else if(strieql(key, "SCANNETMAIL"))
    set_file(semaphore.scannetmail);
  else if(strieql(key, "PMSCANALL"))
    set_file(semaphore.pmscanall);
  else if(strieql(key, "PMSCANTHIS"))
    set_file(semaphore.pmscanthis);
  else if(strieql(key, "PMSCANNETMAIL"))
    set_file(semaphore.pmscannetmail);
  else if(strieql(key, "QWKIMPORT"))
    set_file(semaphore.qwkimport);
  else if(strieql(key, "QWKEXPORT"))
    set_file(semaphore.qwkexport);
  else if(strieql(key, "SOUPIMPORT"))
    set_file(semaphore.soupimport);
  else if(strieql(key, "SOUPEXPORT"))
    set_file(semaphore.soupexport);
  else if(strieql(key, "IDLETIME"))
    semaphore.idletime = atoi(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_sender() {

  gstring tmp = val;
  if(group_active)
    group.add_sender(tmp);
  else
    sender = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_serialno() {

  // Just a placeholder
}

//  ------------------------------------------------------------------

void golded_cfg::handle_soundpath() {

  set_path(soundpath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_soupbadmsgs() {

  set_string(soupbadmsgs);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_soupemail() {
  
  set_string(soupemail);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_soupexportpath() {

  set_path(soupexportpath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_soupimportpath() {

  set_path(soupimportpath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_soupnewsrcfile() {

  set_file(soupnewsrcfile);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_soupreplylinker() {

  set_string(soupreplylinker);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_souptosslog() {

  set_file(souptosslog);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_squishdirect() {

  set_bool(squishdirect);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_squishscan() {

  const int SQS_API   = 1;
  const int SQS_QUICK = 2;

  if(strieql(val, "API"))
    squishscan = SQS_API;
  else if(strieql(val, "QUICK"))
    squishscan = SQS_QUICK;
}

//  ------------------------------------------------------------------

