
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gccfgg8.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Configuration functions, part 8.
//  ------------------------------------------------------------------


#include "gecfgg.h"


//  ------------------------------------------------------------------

void golded_cfg::handle_squishuserno() {

  set_int(squishuserno);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_squishuserpath() {

  set_path(squishuserpath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_statuslineclock() {

  set_bool(statuslineclock);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_statuslinehelp() {

  statuslinehelp = strieql(val, "NOLOGO") ? -1 : get_bool(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_stylecodes() {

  set_bool(stylecodes);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_stylecodepunct() {

  next_keyval();
  stylecodepunct = strcvtc(key);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_stylecodestops() {

  next_keyval();
  stylecodestops = strcvtc(key);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_swappath() {

  set_path(swappath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_tagline() {

  gstring buf = val;
  if(group_active)
    group.add_tagline(buf);
  else
    tagline.add(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_taglinechar() {

  strstripquotes(val);
  if(group_active)
    group.add_taglinechar(*val);
  else
    taglinechar = *val;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_taglinefile() {

  gstring buf = val;
  if(group_active)
    group.add_taglinefile(buf);
  else
    taglinefile = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_taglinesupport() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_taglinesupport(flag);
  else
    taglinesupport = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_tearline() {

  gstring buf = strstripquotes(val);
  if(group_active)
    group.add_tearline(buf);
  else
    tearline = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_template() {

  if(group_active) {
    gstring buf = val;
    group.add_template(buf);
  }
  else {
    gtpl tp;
    next_keyval();
    tp.file = key;
    if((*val == '\'') or (*val == '\"')) {
      next_keyval();
      tp.name = key;
      if(*val)
        tp.match.set(val);
    }
    else if(*val)
      tp.match.set(val);
    if(tp.name.empty())
      tp.name = tp.file;
    tpl.push_back(tp);
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_templatematch() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_templatematch(flag);
  else
    templatematch = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_templatepath() {

  set_path(templatepath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_temppath() {

  set_path(temppath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_timeout() {

  set_int(timeout);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_timeoutsavemsg() {

  set_bool(timeoutsavemsg);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_timezoneoffset() {

  set_int(timezoneoffset);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_twitmode() {

  int tmp = TWIT_SHOW;

  if(strieql(val, "Skip"))          tmp = TWIT_SKIP;
  else if(strieql(val, "Blank"))    tmp = TWIT_BLANK;
  else if(strieql(val, "Show"))     tmp = TWIT_SHOW;
  else if(strieql(val, "Ignore"))   tmp = TWIT_IGNORE;
  else if(strieql(val, "Kill"))     tmp = TWIT_KILL;
  else tmp = get_bool(val) ? TWIT_SHOW : TWIT_BLANK;

  if(group_active)
    group.add_twitmode(tmp);
  else
    twitmode = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_twitname() {

  gnode tmp;
  tmp.addr.zone  = ftn::wildcard_all;
  tmp.addr.net   = ftn::wildcard_all;
  tmp.addr.node  = ftn::wildcard_all;
  tmp.addr.point = ftn::wildcard_all;
  char* ptr = strrchr(val, ' ');
  if(ptr == NULL)
    ptr = val;
  ptr = strskip_wht(ptr);
  if(isdigit(*ptr)) {
    if(address.size())
      tmp.addr = address[0];
    tmp.addr.set(ptr);
    if(tmp.addr.valid()) {
      *ptr = NUL;
      strbtrim(val);
    }
  }
  tmp.name = val;
  twitname.push_back(tmp);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_twitsubj() {

  twitsubj.add(val);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_twitto() {

  set_bool(twitto);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_usefwd() {

  bool flag = get_bool(val);
  if(group_active)
    group.add_usefwd(flag);
  else
    usefwd = flag;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_userlist() {

  // Only used by GoldNODE
}

//  ------------------------------------------------------------------

void golded_cfg::handle_userlistfile() {

  set_file(userlistfile);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_username() {

  strstripquotes(val);
  if(not strblank(val)) {

    gnode tmp;
    char* aptr = strrchr(val, ',');
    if(aptr) {
      *aptr++ = NUL;
      aptr = strskip_wht(aptr);
    }
    else {
      aptr = strrchr(val, ' ');
      if(aptr) {
        tmp.addr = aptr+1;
        if(tmp.addr.invalid())
          aptr = NULL;
        else
          *aptr++ = NUL;
      }
    }

    tmp.addr = ftn::wildcard_all;
    if(aptr) {
      if(CFG->address.size())
        tmp.addr = CFG->address[0];
      tmp.addr.set(aptr);
    }

    tmp.name = strbtrim(val);

    if(group_active) {
      group.add_username(tmp);
    }
    else {

      // Check if we have it already
      for(int u=0; u<CFG->username.size(); u++)
        if(tmp.name.ieql(CFG->username[u].name))
          return;

      username.push_back(tmp);
    }
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_uudecodepath() {

  set_path(uudecodepath);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_viewhidden() {

  bool tmp = get_bool(val);
  if(group_active)
    group.add_viewhidden(tmp);
  else
    viewhidden = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_viewkludge() {

  bool tmp = get_bool(val);
  if(group_active)
    group.add_viewkludge(tmp);
  else
    viewkludge = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_viewquote() {

  bool tmp = get_bool(val);
  if(group_active)
    group.add_viewquote(tmp);
  else
    viewquote = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_whoto() {

  gstring buf = val;
  if(group_active)
    group.add_whoto(buf);
  else
    whoto = buf;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_wildcatuserno() {

  set_int(wildcatuserno);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_xlatcharset() {

  gtokenizer token(val);
  if(token.get()) {
    char* import = token.str();
    if(token.get()) {
      char* export = token.str();
      if(token.get()) {
        char* file = token.str();
        strchg(strupr(import), '_', ' ');
        strchg(strupr(export), '_', ' ');
        if(xlat->path.empty())
          xlat->path = goldpath;
        gstring mapfile = file;
        MakePathname(mapfile, xlat->path, mapfile);
        xlat->add_charset(import, export, mapfile.c_str());
      }
    }
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_xlatescset() {

  gtokenizer token(val);
  if(token.get()) {
    char* import = token.str();
    if(token.get()) {
      char* export = token.str();
      if(token.get()) {
        char* file = token.str();
        strchg(strupr(import), '_', ' ');
        strchg(strupr(export), '_', ' ');
        if(xlat->path.empty())
          xlat->path = goldpath;
        gstring mapfile = file;
        MakePathname(mapfile, xlat->path, mapfile);
        xlat->add_escset(import, export, mapfile.c_str());
      }
    }
  }
}

//  ------------------------------------------------------------------

void golded_cfg::handle_xlatexport() {

  gedxlat_name buf;
  strchg(strupr(strxcpy(buf, val, sizeof(buf))), '_', ' ');
  gstring tmp = buf;
  if(group_active)
    group.add_xlatexport(tmp);
  else
    xlat->export = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_xlatimport() {

  gedxlat_name buf;
  strchg(strupr(strxcpy(buf, val, sizeof(buf))), '_', ' ');
  gstring tmp = buf;
  if(group_active)
    group.add_xlatimport(tmp);
  else
    xlat->import = tmp;
}

//  ------------------------------------------------------------------

void golded_cfg::handle_xlatlocalset() {

  set_string(xlat->localset);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_xlatpath() {

  set_path(xlat->path);
}

//  ------------------------------------------------------------------

void golded_cfg::handle_zonegating() {

  set_bool(zonegating);
}

//  ------------------------------------------------------------------

