
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gcinst.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Install functions.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif


//  ------------------------------------------------------------------

static int EnterString(char* prompt, char* string, uint length) {

  printf("\n%s\n> ", prompt);

  *string = NUL;
  char* ptr = string;
  uint pos = 0;
  Key k;
  while(1) {
    k = kbxget();
    if(k == Key_BS) {
      if(pos) {
        fputc('\b', stdout);
        fputc(' ',  stdout);
        fputc('\b', stdout);
        pos--;
        *(--ptr) = NUL;
      }
    }
    else if(k == Key_Esc) {
      *string = NUL;
      return -1;
    }
    else if(k == Key_Ent) {
      *ptr = NUL;
      break;
    }
    else {
      if(pos < length) {
        char c = (char)k;
        if(c) {
          fputc(c, stdout);
          *ptr++ = c;
          pos++;
        }
      }
    }
  }
  if(*string == NUL)
    return 1;
  return 0;
}


//  ------------------------------------------------------------------

void InstallDetect(char* path) {

  // Create GOLDED.BAK file if there is an existing GOLDED.CFG
  if(fexist(cmdlinecfg)) {
    Path cmdlinecfgbak;
    char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];
    _splitpath(cmdlinecfg, drive, dir, file, ext);
    _makepath(cmdlinecfgbak, drive, dir, file, "BAK");
    remove(cmdlinecfgbak);
    rename(cmdlinecfg, cmdlinecfgbak);
    printf("\nWARNING: Existing GOLDED.CFG renamed to GOLDED.BAK!!!");
  }

  printf("\nPlease wait while GoldED is detecting your software.");

  FILE* fp = fopen(cmdlinecfg, "wt");
  if(fp) {

    if(*path) {
      PathCopy(CFG->areapath, path);
      fprintf(fp, "AREAPATH %s\n", path);
    }

    gstring pth;
    const char* ptr;
    int gotareasbbs = false;
    int detected = 0;

    // Detect D'Bridge
    pth = CFG->areapath;
    ptr = getenv("DBRIDGE");
    if(ptr)
      PathCopy(pth, ptr);
    ptr = getenv("DB");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "DBRIDGE.PRM"))) {
      fprintf(fp, "AREAFILE D'Bridge %s\n", pth.c_str());
      printf("\nFound D'Bridge.");
      detected++;
    }

    // Detect Dutchie
    pth = CFG->areapath;
    ptr = getenv("DUTCHIE");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "DUTCHIE.ARE"))) {
      fprintf(fp, "AREAFILE Dutchie %s\n", pth.c_str());
      printf("\nFound Dutchie.");
      detected++;
    }

    // Detect Ezycom
    pth = CFG->areapath;
    ptr = getenv("EZY");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "CONSTANT.EZY"))) {
      fprintf(fp, "AREAFILE Ezycom %s\n", pth.c_str());
      printf("\nFound Ezycom.");
      detected++;
    }

    // Detect FastEcho
    pth = CFG->areapath;
    ptr = getenv("FASTECHO");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "FASTECHO.CFG"))) {
      fprintf(fp, "AREAFILE FastEcho %s\n", pth.c_str());
      printf("\nFound FastEcho.");
      detected++;
    }

    // Detect FidoPCB
    pth = CFG->areapath;
    ptr = getenv("FIDOPCB");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "FIDOPCB.CFG"))) {
      fprintf(fp, "AREAFILE FidoPCB %s\n", pth.c_str());
      printf("\nFound FidoPCB.");
      detected++;
    }

    // Detect FMail
    pth = CFG->areapath;
    ptr = getenv("FMAIL");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "FMAIL.CFG"))) {
      fprintf(fp, "AREAFILE FMail %s\n", pth.c_str());
      printf("\nFound FMail.");
      detected++;
    }

    // Detect FrontDoor / TosScan
    pth = CFG->areapath;
    ptr = getenv("FD");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "SETUP.FD"))) {
      fprintf(fp, "AREAFILE FrontDoor %s\n", pth.c_str());
      printf("\nFound FrontDoor.");
      detected++;
    }
    else if(fexist(AddPath(pth, "FD.SYS")) and fexist(AddPath(pth, "FD.EXE"))) {
      fprintf(fp, "AREAFILE FrontDoor %s\n", pth.c_str());
      printf("\nFound FrontDoor.");
      detected++;
    }
    if(fexist(AddPath(pth, "AREAFILE.FD"))) {
      fprintf(fp, "AREAFILE TosScan %s\n", pth.c_str());
      printf("\nFound TosScan.");
      detected++;
    }

    // Detect GEcho
    pth = CFG->areapath;
    ptr = getenv("GE");
    if(ptr) {
      pth = ptr;
      ptr = pth.chr(' ');
      if(ptr)
        pth.erase(ptr);
      AddBackslash(pth);
    }
    if(fexist(AddPath(pth, "SETUP.GE"))) {
      fprintf(fp, "AREAFILE GEcho %s\n", pth.c_str());
      printf("\nFound GEcho.");
      detected++;
    }

    // Detect IMAIL
    pth = CFG->areapath;
    ptr = getenv("IMAIL");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "IMAIL.CF"))) {
      fprintf(fp, "AREAFILE IMAIL %s\n", pth.c_str());
      printf("\nFound IMAIL.");
      detected++;
    }

    // Detect InterMail
    pth = CFG->areapath;
    ptr = getenv("IM");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "IM.EXE")) or fexist(AddPath(pth, "INTRECHO.EXE"))) {
      fprintf(fp, "AREAFILE InterMail %s\n", pth.c_str());
      printf("\nFound InterMail and/or InterEcho.");
      detected++;
    }

    // Detect LoraBBS
    pth = CFG->areapath;
    ptr = getenv("LORA");
    if(ptr)
      PathCopy(pth, ptr);
    ptr = getenv("LORABBS");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "SYSMSG.DAT"))) {
      fprintf(fp, "AREAFILE LoraBBS %s\n", pth.c_str());
      printf("\nFound LoraBBS.");
      detected++;
    }

    // Detect Maximus
    pth = CFG->areapath;
    ptr = getenv("MAXIMUS");
    if(ptr) {
      if(is_dir(ptr))
        PathCopy(pth, ptr);
      else {
        ptr = pth.chr(GOLD_SLASH_CHR);
        if(ptr)
          pth.erase(ptr+1);
      }
    }
    int gotsquish = false;
    if(fexist(AddPath(pth, "MAX.PRM"))) {
      fprintf(fp, "AREAFILE Maximus %s\n", pth.c_str());
      printf("\nFound Maximus.");
      detected++;
    }
    if(fexist(AddPath(pth, "SQUISH.CFG"))) {
      fprintf(fp, "AREAFILE Squish %s\n", pth.c_str());
      printf("\nFound Squish.");
      gotsquish = true;
      detected++;
    }

    // Detect ME2
    pth = CFG->areapath;
    if(fexist(AddPath(pth, "AREADESC.ME2"))) {
      fprintf(fp, "AREAFILE ME2 %sAREADESC.ME2 %sAREAS.BBS\n", pth.c_str(), pth.c_str());
      printf("\nFound ME2.");
      gotareasbbs = true;
      detected++;
    }

    // Detect Opus
    #if 0
    pth = CFG->areapath;
    ptr = getenv("OPUS");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "SYSMSG.DAT"))) {
      fprintf(fp, "AREAFILE Opus %s\n", pth.c_str());
      printf("\nFound Opus.");
      detected++;
    }
    #endif

    // Detect PCBoard
    pth = CFG->areapath;
    ptr = getenv("PCBOARD");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "PCBOARD.DAT"))) {
      fprintf(fp, "AREAFILE PCBoard %s\n", pth.c_str());
      printf("\nFound PCBoard.");
      detected++;
    }

    // Detect Portal
    pth = CFG->areapath;
    ptr = getenv("POPCMDLINE");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "PORTAL.ARE"))) {
      fprintf(fp, "AREAFILE Portal %s\n", pth.c_str());
      printf("\nFound Portal.");
      detected++;
    }

    // Detect ProBoard
    pth = CFG->areapath;
    ptr = getenv("RA");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "CONFIG.PRO"))) {
      fprintf(fp, "AREAFILE ProBoard %s\n", pth.c_str());
      printf("\nFound ProBoard.");
      detected++;
    }

    // Detect QFront
    pth = CFG->areapath;
    ptr = getenv("QFRONT");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "QECHOS.DAT"))) {
      fprintf(fp, "AREAFILE QFront %s\n", pth.c_str());
      printf("\nFound QFront.");
      detected++;
    }

    // Detect QuickBBS
    pth = CFG->areapath;
    ptr = getenv("QUICKBBS");
    if(ptr)
      PathCopy(pth, ptr);
    ptr = getenv("QBBS");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "QUICKCFG.DAT")) or fexist(AddPath(pth, "CONFIG.BBS"))) {
      fprintf(fp, "AREAFILE QuickBBS %s\n", pth.c_str());
      printf("\nFound QuickBBS.");
      detected++;
    }

    // Detect RA-ECHO
    pth = CFG->areapath;
    ptr = getenv("RAECHO");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "AREAS.RAE"))) {
      fprintf(fp, "AREAFILE RA-ECHO %s\n", pth.c_str());
      printf("\nFound RA-ECHO.");
      detected++;
    }

    // Detect RemoteAccess
    pth = CFG->areapath;
    ptr = getenv("RA");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "CONFIG.RA"))) {
      fprintf(fp, "AREAFILE RemoteAccess %s\n", pth.c_str());
      printf("\nFound RemoteAccess.");
      detected++;
    }

    // Detect Squish
    if(not gotsquish) {
      pth = CFG->areapath;
      ptr = getenv("SQUISH");
      if(ptr)
        PathCopy(pth, ptr);
      if(fexist(pth)) {
        fprintf(fp, "AREAFILE Squish\n");
        printf("\nFound Squish.");
        detected++;
      }
      else if(fexist(AddPath(pth, "SQUISH.CFG"))) {
        fprintf(fp, "AREAFILE Squish %s\n", pth.c_str());
        printf("\nFound Squish.");
        detected++;
      }
    }

    // Detect SuperBBS
    pth = CFG->areapath;
    ptr = getenv("SUPERBBS");
    if(ptr)
      PathCopy(pth, ptr);
    ptr = getenv("SBBS");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "SCONFIG.BBS"))) {
      fprintf(fp, "AREAFILE SuperBBS %s\n", pth.c_str());
      printf("\nFound SuperBBS.");
      detected++;
    }

    // Detect timEd
    pth = CFG->areapath;
    if(fexist(AddPath(pth, "TIMED.CFG"))) {
      fprintf(fp, "AREAFILE timEd %s\n", pth.c_str());
      printf("\nFound timEd.");
      detected++;
    }

    // Detect TMAIL
    pth = CFG->areapath;
    if(fexist(AddPath(pth, "TMAIL.CFG"))) {
      fprintf(fp, "AREAFILE TMAIL %s\n", pth.c_str());
      printf("\nFound TMAIL.");
      detected++;
    }

    // Detect WMail
    pth = CFG->areapath;
    ptr = getenv("WMAIL");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "WMAIL.PRM"))) {
      fprintf(fp, "AREAFILE WMail %s\n", pth.c_str());
      printf("\nFound WMail.");
      detected++;
    }

    // Detect XMail
    pth = CFG->areapath;
    ptr = getenv("XM");
    if(ptr)
      PathCopy(pth, ptr);
    if(fexist(AddPath(pth, "XMAIL.CFG"))) {
      fprintf(fp, "AREAFILE XMail %s\n", pth.c_str());
      printf("\nFound XMail.");
      detected++;
    }

    // Detect AREAS.BBS
    if(not gotareasbbs) {
      pth = CFG->areapath;
      if(fexist(AddPath(pth, "AREAS.BBS"))) {
        fprintf(fp, "AREAFILE AreasBBS %sAREAS.BBS\n", pth.c_str());
        printf("\nFound AREAS.BBS.");
        detected++;
      }
    }

    if(not detected)
      printf("\nSorry, could not find any supported software. Try another path.");

    fclose(fp);
  }

  printf("\n");
}


//  ------------------------------------------------------------------

int InstallFinish() {

  FILE* fp = fopen(cmdlinecfg, "at");
  if(fp) {

    char buf[77];

    // Check what we have

    if(CFG->username.empty()) {
      if(EnterString("Please enter your name:", buf, sizeof(buf)))
        return -1;
      fprintf(fp, "USERNAME %s\n", buf);
      CfgUsername(buf);
    }
    if(CFG->address.empty()) {
      if(EnterString("Please enter your FidoNet address:", buf, sizeof(buf)))
        return -1;
      fprintf(fp, "ADDRESS %s\n", buf);
      CfgAddress(buf);
    }
    if(AL.msgbases == 0) {
      if(EnterString("Please enter the path to your *.MSG netmail area:", buf, sizeof(buf)))
        return -1;
      fprintf(fp, "AREADEF NETMAIL \"Netmail\" 0 Net Opus %s\n", buf);
      AreaCfg aa;
      aa.reset();
      aa.setdesc("Netmail");
      aa.setechoid("NETMAIL");
      aa.msgbase = GMB_OPUS;
      aa.type = GMB_NET;
      aa.setpath(buf);
      aa.attr = CFG->attribsnet;
      AL.add(aa);
      if(EnterString("Please enter the path *and* filename of your AREAS.BBS file:", buf, sizeof(buf)))
        return -1;
      fprintf(fp, "AREAFILE AreasBBS %s\n", buf);
      AFILE->ReadAreasBBS(buf);
    }
    if((AL.msgbases & GMB_HUDSON) and (*CFG->hudsonpath == NUL)) {
      if(EnterString("Please enter the path to your Hudson msgbase files:", buf, sizeof(buf)))
        return -1;
      fprintf(fp, "HUDSONPATH %s\n", buf);
      PathCopy(CFG->hudsonpath, buf);
    }
    if((AL.msgbases & GMB_GOLDBASE) and (*CFG->goldbasepath == NUL)) {
      if(EnterString("Please enter the path to your Goldbase msgbase files:", buf, sizeof(buf)))
        return -1;
      fprintf(fp, "GOLDBASEPATH %s\n", buf);
      PathCopy(CFG->goldbasepath, buf);
    }
    if((AL.msgbases & GMB_JAM) and (*CFG->jampath == NUL)) {
      if(EnterString("Please enter the path where NET/ECHOMAIL.JAM can be placed:", buf, sizeof(buf)))
        return -1;
      fprintf(fp, "JAMPATH %s\n", buf);
      PathCopy(CFG->jampath, buf);
    }
    if((AL.msgbases & GMB_PCBOARD) and (*CFG->pcboardpath == NUL)) {
      if(EnterString("Please enter the path to PCBoard:", buf, sizeof(buf)))
        return -1;
      fprintf(fp, "PCBOARDPATH %s\n", buf);
      PathCopy(CFG->pcboardpath, buf);
    }
    
    fclose(fp);
  }

  return 0;
}


//  ------------------------------------------------------------------

