
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gcload.cpp,v 1.1.1.1 1998/11/21 19:38:03 odinn Exp $
//  ------------------------------------------------------------------
//  Configuration loading.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif


//  ------------------------------------------------------------------

bool CheckStamp(const Stamp& fntm) {

  return GetFiletime(fntm.fn) != fntm.ft;
}


//  ------------------------------------------------------------------
//  Read GOLDED.CFG

int Readgolded_cfg(int& force) {

  gfile fp;
  register int n;
  CfgVer vers;
  uint compile=YES;
  //int filecheck = false;

  // Check if the binary file needs recompile
  fp.fopen(AddPath(CFG->goldpath, CFG->goldged), "rb", CFG->sharemode);
  if(fp.isopen()) {

    fp.setvbuf(NULL, _IOFBF, 8192);
    fp.fread(&vers, sizeof(CfgVer));

    if(not force) {

      // If version is not the same, make sure we recompile everything (force=YES+YES)

      if(not streql(vers.date, CFG->vers.date))
        force = YES + YES;
      else if(not streql(vers.time, CFG->vers.time))
        force = YES + YES;
      else if(vers.vers != CFG->vers.vers)
        force = YES + YES;
    }

    compile = force ? true : CheckStamp(vers.cfg);

    // Check if any files needs compiling
    if(not compile) {
      gvector_load(fp, CFG->stamp);
      for(n=0; n<CFG->stamp.size() and not compile; n++) {
        compile = CheckStamp(CFG->stamp[n]);
        //if(compile and not filecheck)
          //filecheck = stamp[n].fc;
      }
    }

    // Read the complete file if no compile needed
    if(not compile) {

      fp.rewind();

      CFG->load(fp);
      GRP->Load(fp);
      QWK->Load(fp);
      XLAT->load(fp);
      AL.load(fp);

    }
    fp.fclose();
  }

  // Call the general config reader to do the work
  if(compile) {

    CfgInit2();

    CFG->stamp.resize(0);

    //if(filecheck)
      //ShellToDos(CFG->filecheckcall, "", LGREY|_BLACK, true, false, false);

    if(ReadCfg(AddPath(CFG->goldpath, CFG->goldcfg))) {

      CFG->vers.cfg.fn = AddPath(CFG->goldpath, CFG->goldcfg);
      CFG->vers.cfg.ft = GetFiletime(AddPath(CFG->goldpath, CFG->vers.cfg.fn));

      // Unallocate echolist
      #ifdef GOLDED_AREAFILE
      AFILE->echolist.FreeAll();
      #endif

      // Set default paths

      if(CFG->temppath.empty())      CFG->temppath     = CFG->goldpath;
      if(CFG->swappath.empty())       CFG->swappath     = CFG->temppath;
      if(CFG->areapath.empty())       CFG->areapath     = CFG->goldpath;
      if(CFG->inboundpath.empty())    CFG->inboundpath  = CFG->goldpath;
      if(CFG->soundpath.empty())      CFG->soundpath    = CFG->goldpath;
      if(CFG->cookiepath.empty())     CFG->cookiepath   = CFG->goldpath;
      if(CFG->templatepath.empty())   CFG->templatepath = CFG->goldpath;

      if(CFG->squishuserpath.empty()) {
        char* ptr = getenv("MAXIMUS");
        if(ptr) {
          if(is_dir(ptr))
            PathCopy(CFG->squishuserpath, ptr);
        }
        else {
          ptr = getenv("SQUISH");
          if(ptr)
            PathCopy(CFG->squishuserpath, ptr);
          else
            CFG->squishuserpath = CFG->areapath;
        }
      }

      if(CFG->hudsonpath.empty())       CFG->hudsonpath      = CFG->goldpath;
      if(CFG->hudsonsyspath.empty())    CFG->hudsonsyspath   = CFG->hudsonpath;
      if(CFG->goldbasepath.empty())     CFG->goldbasepath    = CFG->hudsonpath;
      if(CFG->goldbasesyspath.empty())  CFG->goldbasesyspath = CFG->goldbasepath;
      if(CFG->jampath.empty())          CFG->jampath         = CFG->hudsonpath;

      MakePathname(CFG->goldcfg,    CFG->goldpath, CFG->goldcfg);
      MakePathname(CFG->helpcfg.fn, CFG->goldpath, CFG->helpcfg.fn);
      MakePathname(CFG->keyscfg.fn, CFG->goldpath, CFG->keyscfg.fn);
      MakePathname(CFG->langcfg.fn, CFG->goldpath, CFG->langcfg.fn);
      MakePathname(CFG->randcfg.fn, CFG->goldpath, CFG->randcfg.fn);

      MakePathname(CFG->goldged, CFG->goldpath, CFG->goldged);
      MakePathname(CFG->helpged, CFG->goldpath, CFG->helpged);
      MakePathname(CFG->langged, CFG->goldpath, CFG->langged);
      MakePathname(CFG->randged, CFG->goldpath, CFG->randged);
      MakePathname(XLAT->xlatfile, CFG->goldpath, XLAT->xlatfile);

      MakePathname(CFG->confirmfile, CFG->goldpath, CFG->confirmfile);
      MakePathname(CFG->logfile, CFG->goldpath, CFG->logfile);
      MakePathname(CFG->userlistfile, CFG->nodepath, CFG->userlistfile);
      MakePathname(CFG->outputfile, CFG->goldpath, CFG->outputfile);
      MakePathname(CFG->inputfile, CFG->goldpath, CFG->inputfile);

      if(CFG->souptosslog)
        MakePathname(CFG->souptosslog, CFG->goldpath, CFG->souptosslog);

      for(n=0; n<CFG->tpl.size(); n++)
        MakePathname(CFG->tpl[n].file, CFG->templatepath, CFG->tpl[n].file);

      MakePathname(CFG->semaphore.importlist, CFG->areapath, CFG->semaphore.importlist);
      MakePathname(CFG->semaphore.exportlist, CFG->areapath, CFG->semaphore.exportlist);
      MakePathname(CFG->semaphore.echoscan,   CFG->areapath, CFG->semaphore.echoscan);
      MakePathname(CFG->semaphore.netscan,    CFG->areapath, CFG->semaphore.netscan);

      MakePathname(CFG->semaphore.scanall,    CFG->goldpath, CFG->semaphore.scanall);
      MakePathname(CFG->semaphore.scanthis,   CFG->goldpath, CFG->semaphore.scanthis);
      MakePathname(CFG->semaphore.pmscanall,  CFG->goldpath, CFG->semaphore.pmscanall);
      MakePathname(CFG->semaphore.pmscanthis, CFG->goldpath, CFG->semaphore.pmscanthis);

      MakePathname(CFG->semaphore.qwkimport,  CFG->goldpath, CFG->semaphore.qwkimport);
      MakePathname(CFG->semaphore.qwkexport,  CFG->goldpath, CFG->semaphore.qwkexport);
      MakePathname(CFG->semaphore.soupimport, CFG->goldpath, CFG->semaphore.soupimport);
      MakePathname(CFG->semaphore.soupexport, CFG->goldpath, CFG->semaphore.soupexport);

      strschg_environ(CFG->jampath);
      strschg_environ(CFG->semaphore.importlist);
      strschg_environ(CFG->semaphore.exportlist);
      strschg_environ(CFG->semaphore.echoscan);
      strschg_environ(CFG->semaphore.netscan);

      if(CFG->semaphore.exportlist.ieql(AddPath(CFG->jampath, "ECHOMAIL.JAM"))) {
        printf("\n* Warning: SEMAPHORE EXPORTLIST must not be the same as ECHOMAIL.JAM!");
        if(not disablesound)
          beep();
        cfgerrors++;
      }
    }

    if(XLAT->import.empty())
      XLAT->import = XLAT->localset;
    XLAT->compile_tables();
    XLAT->charset_file.reset();
    XLAT->escset_file.reset();
  }

  AreaCfg::autoid = CFG->areaautoid;

  return compile;
}


//  ------------------------------------------------------------------

