
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geall.h,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Definition of system-wide sizes and structures.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Compiled config extension

#if defined(__OS2__)
#  define GCFG_EXT ".geo"
#elif defined(__WIN32__)
#  define GCFG_EXT ".gew"
#elif defined(__linux__)
#  define GCFG_EXT ".geu"
#elif defined(__MSDOS__) and defined(__386__)
#  define GCFG_EXT ".gex"
#elif defined(__MSDOS__)
#  define GCFG_EXT ".ged"
#endif


//  ------------------------------------------------------------------
//  Header attach size types

const int ATTACH_BYTES  = 1;
const int ATTACH_KBYTES = 2;


//  ------------------------------------------------------------------
//  Overlay types

const int OVL_DISK = 0;
const int OVL_EMS  = 1;
const int OVL_EXT  = 2;


//  ------------------------------------------------------------------
//  Misc mode types

const int SAVELINES_NEW        = 0;
const int SAVELINES_SAVE       = 1;
const int SAVELINES_SAVENOCTRL = 2;
const int SAVELINES_WRITE      = 3;

const int CMF_COPY    = 1;
const int CMF_MOVE    = 2;
const int CMF_FORWARD = 3;

const int MODE_QUOTE         = 0x0001;
const int MODE_REPLY         = 0x0002;
const int MODE_COMMENT       = 0x0004;
const int MODE_CONFIRM       = 0x0008;
const int MODE_NEW           = 0x0010;
const int MODE_CHANGE        = 0x0020;
const int MODE_FORWARD       = 0x0040;
const int MODE_QUOTEBUF      = 0x0080;
const int MODE_REPLYING      = MODE_QUOTE|MODE_COMMENT|MODE_CONFIRM|MODE_REPLY;
const int MODE_QUOTING       = MODE_QUOTE|MODE_COMMENT|MODE_CONFIRM|MODE_QUOTEBUF;

const int STATUS_SAVE        = 0x0001;
const int STATUS_VIEW        = 0x0002;
const int STATUS_UPDATE      = 0x0004;
const int STATUS_CONTINUE    = 0x0008;
const int STATUS_EDIT        = 0x0010;
const int STATUS_DROP        = 0x4000;
const int STATUS_QUIT        = 0x8000;

const int PROGRESS_NEW       = 1;
const int PROGRESS_UPDATE    = 2;
const int PROGRESS_QUIT      = -1;

const int MODE_PREVIOUS      = 2;
const int MODE_MARKED        = 1;
const int MODE_CURRENT       = 0;
const int MODE_DONT          = -1;


//  ------------------------------------------------------------------
//  Scanning constants

const int SCAN_STARTUP       =  0;
const int SCAN_ALL           =  1;
const int SCAN_MARKED        =  2;
const int SCAN_CURRENT       =  3;
const int SCAN_MATCHING      =  4;
const int SCAN_UNSCANNED     =  5;
const int SCAN_EXPORTQWK     =  6;
const int SCAN_IMPORTQWK     =  7;
const int SCAN_EXPORTSOUP    =  8;
const int SCAN_IMPORTSOUP    =  9;
const int SCAN_LIST          = 10;
const int SCAN_GROUP         = 11;
const int SCAN_NETMAIL       = 12;
const int SCAN_QUIT          = 99;


//  ------------------------------------------------------------------
//  Exit errorcodes

const int EXIT_OK          = 0x00;
const int EXIT_LOCAL       = 0x01;
const int EXIT_NET         = 0x02;
const int EXIT_ECHO        = 0x04;
const int EXIT_NONAME      = 0x20;
const int EXIT_CFGERR      = 0x21;
const int EXIT_ERRORS      = 0x22;


//  ------------------------------------------------------------------
//  Direction defines

const int DIR_PREV  = -1;
const int DIR_NEXT  =  1;

const int DIR_ABOVE = DIR_PREV;
const int DIR_BELOW = DIR_NEXT;


//  ------------------------------------------------------------------
//  Message marking

const char MMRK_MARK = '\x10';
const char MMRK_BOOK = '\x11';


//  ------------------------------------------------------------------
//  Twit options

const int NOT_TWIT    = 0;
const int TWIT_SHOW   = 1;
const int TWIT_BLANK  = 2;
const int TWIT_SKIP   = 3;
const int TWIT_IGNORE = 4;
const int TWIT_KILL   = 5;


//  ------------------------------------------------------------------
//  CC list options

const int CC_REMOVE  = 0;
const int CC_HIDDEN  = 1;
const int CC_VISIBLE = 2;
const int CC_NAMES   = 3;
const int CC_KEEP    = 4;


//  ------------------------------------------------------------------
//  Arealist types

const int AL_TOTLST = 0;
const int AL_TOTNEW = 1;


//  ------------------------------------------------------------------
//  Area R/O modes

const int READONLY_SOFT = 1;
const int READONLY_HARD = 2;


//  ------------------------------------------------------------------
//  Colorset types

const int CS_NORMAL   = 0;
const int CS_INTENSE  = 1;
const int CS_MONO     = 2;


//  ------------------------------------------------------------------
//  Lookup types

const int LOOK_DEST = 0;
const int LOOK_ORIG = 1;
const int LOOK_NAME = 2;


//  ------------------------------------------------------------------
//  Quotebuffer modes

const int QBUF_ASK       = 1;
const int QBUF_APPEND    = 2;
const int QBUF_OVERWRITE = 3;


//  ------------------------------------------------------------------
//  Screenblanker types

const int BLANK_ALLBLACK = 0;
const int BLANK_SLIDEWIN = 1;


//  ------------------------------------------------------------------
//  DISPMSGSIZE types

const int DISPMSGSIZE_BYTES  = 1;
const int DISPMSGSIZE_KBYTES = 2;
const int DISPMSGSIZE_LINES  = 3;


//  ------------------------------------------------------------------
//  Replylink types

const int REPLYLINK_CHAIN  = 0;
const int REPLYLINK_DIRECT = 1;

const int REPLYLINKLIST_FAST = 0;
const int REPLYLINKLIST_FULL = 1;


//  ------------------------------------------------------------------

#define GSWAP_ENABLED       0x8000
#define GSWAP_DISK          (GSWAP_ENABLED|USE_FILE)
#define GSWAP_EMS           (GSWAP_ENABLED|USE_EMS)
#define GSWAP_XMS           (GSWAP_ENABLED|USE_XMS)
#define GSWAP_ALL           (GSWAP_ENABLED|USE_ALL)
#define GSWAP_NONE          0
#define GSWAP_EMS_FIRST     EMS_FIRST
#define GSWAP_XMS_FIRST     XMS_FIRST
#define GSWAP_HIDE_FILE     HIDE_FILE
#define GSWAP_NO_PREALLOC   NO_PREALLOC
#define GSWAP_CHECK_NET     CHECK_NET


//  ------------------------------------------------------------------

const int LIST_BOTTOM      = -2;
const int LIST_NEARBOTTOM  = -1;
const int LIST_MIDDLE      = 0;
const int LIST_NEARTOP     = 1;
const int LIST_TOP         = 2;


//  ------------------------------------------------------------------

const int KEYB_BLOCK = 0;
const int KEYB_POLL  = 1;


//  ------------------------------------------------------------------
//  Registration key valid OS

const int GKEY_VALID_DOS = 0x0001;
const int GKEY_VALID_OS2 = 0x0002;
const int GKEY_VALID_W32 = 0x0004;
const int GKEY_VALID_LNX = 0x0008;
const int GKEY_VALID_WIN = 0x0010;
const int GKEY_VALID_OPM = 0x0020;
const int GKEY_VALID_XWN = 0x0040;


//  ------------------------------------------------------------------
//  File request options

const int FREQ_SORT        = 0x0001;
const int FREQ_FROMTOP     = 0x0002;
const int FREQ_NOFILES     = 0x0004;
const int FREQ_FAST        = 0x0008;
const int FREQ_NOWAZOOMSG  = 0x0010;
const int FREQ_USEFLOWFILE = 0x0020;


//  ------------------------------------------------------------------
//  Automatic echoid names

const int AUTOID_VERBOSE = 0;
const int AUTOID_SHORT   = 1;
const int AUTOID_LONG    = 2;


//  ------------------------------------------------------------------
//  Values for EDITHEADERFIRST

const int EDITHEADERFIRST_NO       = 0x0000;
const int EDITHEADERFIRST_NEW      = 0x0001;
const int EDITHEADERFIRST_CHANGES  = 0x0002;
const int EDITHEADERFIRST_REPLIES  = 0x0004;
const int EDITHEADERFIRST_FORWARDS = 0x0008;
const int EDITHEADERFIRST_YES      = 0x000F;


//  ------------------------------------------------------------------
//  Write constants

const int WRITE_CURRENT       = 0;
const int WRITE_MARKED        = 1;
const int WRITE_QUIT          = -1;
const int WRITE_FILE          = 1;
const int WRITE_PRINTER       = 2;
const int WRITE_NO_HEADER     = 4;
const int WRITE_FILE_NOHDR    = WRITE_NO_HEADER | WRITE_FILE;
const int WRITE_PRINTER_NOHDR = WRITE_NO_HEADER | WRITE_PRINTER;


//  ------------------------------------------------------------------
//  Values for PCBOARDMAILER

const int PCBOARDMAILER_PCBOARD = 0;
const int PCBOARDMAILER_OTHER   = 1;


//  ------------------------------------------------------------------
//  Configuration version stamp

struct CfgVer {
  char  date[12];   // Compiled file date
  char  time[9];    // Compiled file time
  word  vers;       // Configuration version
  gstamp cfg;        // Filename and timestamp of original config file
};


//  ------------------------------------------------------------------
//  Enums for the marking menu (GEMENU etc)

enum {
  TAG_MARKYOURMAIL = 100,
  TAG_MARKHEADER,
  TAG_MARKTXTHDR,
  TAG_MARKTHREAD,
  TAG_MARKNEWMSGS,
  TAG_MARKOLDMSGS,
  TAG_MARKALLMSGS,
  TAG_MARKUNMARK,
  TAG_MARKRANGE,
  TAG_MARKTOGGLE,
  TAG_MARKQUIT
};

   
//  ------------------------------------------------------------------
#if 0
class GGrp : public Grp {

public:

  void AddItm(int type, int data)            { Grp::AddItm(type, &data, sizeof(data)); }
  void AddItm(int type, char data)           { Grp::AddItm(type, &data, sizeof(data)); }
  void AddItm(int type, const gstring& data) { Grp::AddItm(type, data); }
  void AddItm(int type, const gnode& data);
  void AddItm(int type, const gplay& data);
  void AddItm(int type, const ftn_addr& data);
  void AddItm(int type, const ftn_attr& data);

  int GetItm(int type, gnode& data);
  int GetItm(int type, gplay& data);
  int GetItm(int type, ftn_addr& data);
  int GetItm(int type, ftn_attr& data);

};
#endif

//  ------------------------------------------------------------------

class header_display {

public:

  gstring description;
  gstring echoid;

  gstring msgnos;

  gstring from_name;
  gstring from_address;
  gstring date_written;

  gstring to_name;
  gstring to_address;
  gstring date_arrived;

  gstring lngsubj;
  gstring subject;

};


//  ------------------------------------------------------------------

class address_macro {

public:

  address_macro(const char* s) { operator=(s); }

  reset();

  address_macro& operator=(const char* s);

  gstring macro;
  gstring name;
  gstring address;
  gstring subject;
  gstring attribs;
};


//  ------------------------------------------------------------------
//  Window areas

#define GC_BACK_    CFG->color[0]
#define GC_STAT_    CFG->color[1]
#define GC_BRAG_    CFG->color[2]
#define GC_AREA_    CFG->color[3]
#define GC_READ_    CFG->color[4]
#define GC_HEAD_    CFG->color[5]
#define GC_ASK__    CFG->color[6]
#define GC_MENU_    CFG->color[7]
#define GC_HELP_    CFG->color[8]
#define GC_CFG_     CFG->color[9]
#define GC_INFO_    CFG->color[10]
#define GC_CFGB_    CFG->color[11]
#define GC_STYLE_   CFG->color[12]
#define GC_SHAD_    CFG->color[13]


//  ------------------------------------------------------------------
//  Window parts

#define _Window    c[0]
#define _Border    c[1]
#define _Title     c[2]
#define _Cursor    c[2]
#define _Selector  c[3]
#define _Quote     c[3]
#define _Subject   c[3]
#define _Highlight c[4]
#define _Noselect  c[5]
#define _Kludge    c[5]
#define _Tearline  c[6]
#define _Edit      c[6]
#define _Block     c[7]
#define _Input     c[7]
#define _Shadow    c[7]
#define _Origin    c[8]
#define _To        c[8]
#define _From      c[9]
#define _Tagline   c[9]
#define _Hidden    c[10]
#define _Pagebar   c[11]

#define _WindowUnsent     c[6]
#define _WindowUnread     c[7]
#define _HighlightUnsent  c[8]
#define _HighlightUnread  c[9]

#define _Bold                         c[1]
#define _Italic                       c[2]
#define _BoldItalic                   c[3]
#define _Underscore                   c[4]
#define _BoldUnderscore               c[5]
#define _ItalicUnderscore             c[6]
#define _BoldItalicUnderscore         c[7]
#define _Reverse                      c[8]
#define _ReverseBold                  c[9]
#define _ReverseItalic                c[10]
#define _ReverseBoldItalic            c[11]
#define _ReverseUnderscore            c[12]
#define _ReverseBoldUnderscore        c[13]
#define _ReverseItalicUnderscore      c[14]
#define _ReverseBoldItalicUnderscore  c[15]


//  ------------------------------------------------------------------
//  Window part colors

#define C_BACKW   GC_BACK_._Window
#define C_BACKB   GC_BACK_._Border

#define C_STATW   GC_STAT_._Window
#define C_STATB   GC_STAT_._Border

#define C_BRAGW   GC_BRAG_._Window
#define C_BRAGB   GC_BRAG_._Border
#define C_BRAGT   GC_BRAG_._Title
#define C_BRAGH   GC_BRAG_._Highlight
#define C_BRAGA   GC_BRAG_._Block

#define C_AREAW   GC_AREA_._Window
#define C_AREAB   GC_AREA_._Border
#define C_AREAT   GC_AREA_._Title
#define C_AREAS   GC_AREA_._Selector
#define C_AREAQ   GC_AREA_._Highlight
#define C_AREAPB  GC_AREA_._Pagebar

#define C_READW   GC_READ_._Window
#define C_READB   GC_READ_._Border
#define C_READQ   GC_READ_._Quote
#define C_READK   GC_READ_._Kludge
#define C_READKH  GC_READ_._Hidden
#define C_READC   GC_READ_._Cursor
#define C_READA   GC_READ_._Block
#define C_READH   GC_READ_._Highlight
#define C_READG   GC_READ_._Tagline
#define C_READT   GC_READ_._Tearline
#define C_READO   GC_READ_._Origin
#define C_READPB  GC_READ_._Pagebar

#define C_HEADW   GC_HEAD_._Window
#define C_HEADB   GC_HEAD_._Border
#define C_HEADT   GC_HEAD_._Title
#define C_HEADE   GC_HEAD_._Edit
#define C_HEADI   GC_HEAD_._Input
#define C_HEADQ   GC_HEAD_._Highlight
#define C_HEADBY  GC_HEAD_._From
#define C_HEADTO  GC_HEAD_._To
#define C_HEADRE  GC_HEAD_._Subject

#define C_ASKW    GC_ASK__._Window
#define C_ASKB    GC_ASK__._Border
#define C_ASKT    GC_ASK__._Title
#define C_ASKS    GC_ASK__._Selector
#define C_ASKQ    GC_ASK__._Highlight
#define C_ASKN    GC_ASK__._Noselect
#define C_ASKPB   GC_ASK_._Pagebar

#define C_MENUW   GC_MENU_._Window
#define C_MENUB   GC_MENU_._Border
#define C_MENUT   GC_MENU_._Title
#define C_MENUS   GC_MENU_._Selector
#define C_MENUQ   GC_MENU_._Highlight
#define C_MENUN   GC_MENU_._Noselect
#define C_MENUPB  GC_MENU_._Pagebar
#define C_MENUW_UNREAD      GC_MENU_._WindowUnread
#define C_MENUQ_UNREADHIGH  GC_MENU_._HighlightUnread
#define C_MENUW_UNSENT      GC_MENU_._WindowUnsent
#define C_MENUQ_UNSENTHIGH  GC_MENU_._HighlightUnsent

#define C_HELPW   GC_HELP_._Window
#define C_HELPB   GC_HELP_._Border
#define C_HELPT   GC_HELP_._Title
#define C_HELPS   GC_HELP_._Selector
#define C_HELPQ   GC_HELP_._Highlight
#define C_HELPPB  GC_HELP_._Pagebar

#define C_INFOW   GC_INFO_._Window
#define C_INFOB   GC_INFO_._Border
#define C_INFOT   GC_INFO_._Title
#define C_INFOPB  GC_INFO_._Pagebar

#define C_CFGW    GC_CFG_._Window
#define C_CFGB    GC_CFG_._Border
#define C_CFGT    GC_CFG_._Title
#define C_CFGS    GC_CFG_._Selector
#define C_CFGQ    GC_CFG_._Highlight
#define C_CFGN    GC_CFG_._Noselect
#define C_CFGPB   GC_CFG_._Pagebar

#define C_CFGBW   GC_CFGB_._Window
#define C_CFGBB   GC_CFGB_._Border
#define C_CFGBT   GC_CFGB_._Title
#define C_CFGBS   GC_CFGB_._Selector
#define C_CFGBQ   GC_CFGB_._Highlight
#define C_CFGBN   GC_CFGB_._Noselect
#define C_CFGBPB  GC_CFGB_._Pagebar

#define C_SHADOW  GC_SHAD_._Shadow

#define C_STYLE     GC_STYLE_.c
#define C_STYLE___B GC_STYLE_._Bold
#define C_STYLE__I_ GC_STYLE_._Italic
#define C_STYLE__IB GC_STYLE_._BoldItalic
#define C_STYLE_U__ GC_STYLE_._Underscore
#define C_STYLE_U_B GC_STYLE_._BoldUnderscore
#define C_STYLE_UI_ GC_STYLE_._ItalicUnderscore
#define C_STYLE_UIB GC_STYLE_._BoldItalicUnderscore
#define C_STYLER___ GC_STYLE_._Reverse
#define C_STYLER__B GC_STYLE_._ReverseBold
#define C_STYLER_I_ GC_STYLE_._ReverseItalic
#define C_STYLER_IB GC_STYLE_._ReverseBoldItalic
#define C_STYLERU__ GC_STYLE_._ReverseUnderscore
#define C_STYLERU_B GC_STYLE_._ReverseBoldUnderscore
#define C_STYLERUI_ GC_STYLE_._ReverseItalicUnderscore
#define C_STYLERUIB GC_STYLE_._ReverseBoldItalicUnderscore


//  ------------------------------------------------------------------
//  Window handles

#define W_BACK    GC_BACK_.w
#define W_STAT    GC_STAT_.w
#define W_BRAG    GC_BRAG_.w
#define W_AREA    GC_AREA_.w
#define W_READ    GC_READ_.w
#define W_HEAD    GC_READ_.w
#define W_ASK     GC_ASK__.w
#define W_MENU    GC_MENU_.w
#define W_INFO    GC_INFO_.w


//  ------------------------------------------------------------------
//  Window offsets

#define W_XBACK   GC_BACK_.x
#define W_XSTAT   GC_STAT_.x
#define W_XBRAG   GC_BRAG_.x
#define W_XAREA   GC_AREA_.x
#define W_XREAD   GC_READ_.x
#define W_XHEAD   GC_READ_.x
#define W_XASK    GC_ASK__.x
#define W_XMENU   GC_MENU_.x
#define W_XINFO   GC_INFO_.x

#define W_YBACK   GC_BACK_.y
#define W_YSTAT   GC_STAT_.y
#define W_YBRAG   GC_BRAG_.y
#define W_YAREA   GC_AREA_.y
#define W_YREAD   GC_READ_.y
#define W_YHEAD   GC_READ_.y
#define W_YASK    GC_ASK__.y
#define W_YMENU   GC_MENU_.y
#define W_YINFO   GC_INFO_.y


//  ------------------------------------------------------------------
//  Window border types

#define W_BBACK   GC_BACK_.btype
#define W_BSTAT   GC_STAT_.btype
#define W_BBRAG   GC_BRAG_.btype
#define W_BAREA   GC_AREA_.btype
#define W_BREAD   GC_READ_.btype
#define W_BHEAD   GC_READ_.btype
#define W_BASK    GC_ASK__.btype
#define W_BMENU   GC_MENU_.btype
#define W_BHELP   GC_HELP_.btype
#define W_BINFO   GC_INFO_.btype
#define W_BCFG    GC_CFG_.btype
#define W_BCFGB   GC_CFGB_.btype


//  ------------------------------------------------------------------
//  Include various GoldED structures

#include "gehelp.h"
#include "gekeys.h"
#include "gelang.h"
#include "genodo.h"
#include "geplay.h"
#include "gmarea.h"
#include "gecfgg.h"


//  ------------------------------------------------------------------

