
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gealst.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Arealist functions.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

garealist AL;


//  ------------------------------------------------------------------

int AreaTypeOrder[17] = {
  0,
  1,      // GMB_NET
  2,      // GMB_EMAIL
  3,      // GMB_NET | GMB_EMAIL
  4,      // GMB_ECHO
  5,
  6,
  7,
  8,      // GMB_NEWSGROUP
  9,
  10,
  11,
  12,     // GMB_ECHO | GMB_NEWSGROUP
  13,
  14,
  15,
  16      // GMB_LOCAL
};


//  ------------------------------------------------------------------

int garealist::compare(int aa, int bb) {

  const garealistdata* a = list[aa];
  const garealistdata* b = list[bb];
  const garealistdata* A = a;
  const garealistdata* B = b;
  register int cmp = 0;
  register int aunread;
  register int bunread;

  bool rev = false;
  const char* ptr = AL.sortspec.c_str();
  while(*ptr) {
    switch(toupper(*ptr)) {
      case '-':
        rev = true;
        A = b;  B = a;
        break;
      case '+':
        rev = false;
        A = a;  B = b;
        break;
      case 'A':
        if((cmp = A->aka().compare(B->aka())) != 0)
          return cmp;
        break;
      case 'B':
        if((cmp = compare_two(A->board(), B->board())) != 0)
          return cmp;
        break;
      case 'D':
        if((cmp = A->desc().icmp(B->desc())) != 0)
          return cmp;
        break;
      case 'E':
        if((cmp = A->echoid().icmp(B->echoid())) != 0)
          return cmp;
        break;
      case 'F':
        if(*area_maybe) {
          if(striinc(area_maybe, A->echoid().c_str()) and not striinc(area_maybe, B->echoid().c_str()))
            return -1;
          if(striinc(area_maybe, B->echoid().c_str()) and not striinc(area_maybe, A->echoid().c_str()))
            return 1;
        }
        break;
      case 'G':
        {
          int ga = A->groupid();
          int gb = B->groupid();
          if(ga == 0)
            ga = INT_MAX;
          if(gb == 0)
            gb = INT_MAX;
          if((cmp = compare_two(ga, gb)) != 0)
            return cmp;
          if((cmp = compare_two(B->isseparator(), A->isseparator())) != 0)
            return cmp;
        }
        break;
      case 'M':
        if(A->marked() and not B->marked())
          return -1;
        if(B->marked() and not A->marked())
          return 1;
        break;
      case 'P':
        if((cmp = compare_two(B->pmscancount()?1:0, A->pmscancount()?1:0)) != 0)
          return cmp;
        break;
      case 'O':
        if((cmp = compare_two(A->areaid(), B->areaid())) != 0)
          return cmp;
        break;
      case 'T':
        if((cmp = compare_two(CFG->areatypeorder[A->type()&0xFF], CFG->areatypeorder[B->type()&0x00FF])) != 0)
          return cmp;
        break;
      case 'U':
        aunread = A->scancount() - A->lastread();
        bunread = B->scancount() - B->lastread();
        if(rev)
          cmp = compare_two(aunread, bunread);
        else
          cmp = compare_two(aunread ? aunread : INT_MAX, bunread ? bunread : INT_MAX);
        if(cmp != 0)
          return cmp;
        break;
      case 'X':
        if((cmp = compare_two(A->msgbase(), B->msgbase())) != 0)
          return cmp;
        break;
      case 'Y':
        aunread = A->scancount() - A->lastread();
        bunread = B->scancount() - B->lastread();
        if(aunread and not bunread)
          return -1;
        if(bunread and not aunread)
          return 1;
        break;
      case 'Z':
        if((cmp = A->path().icmp(B->path())) != 0)
          return cmp;
        break;
    }
    ptr++;
  }

  return cmp;
}


//  ------------------------------------------------------------------

void garealist::swap(int a, int b) {

  garealistdata* t = list[a];
  list[a] = list[b];
  list[b] = t;
}


//  ------------------------------------------------------------------

void garealist::qsort(int first, int last) {

  int stack[40];
  int* sp = stack;
  int i, j;
  int base = first;
  uint thresh = 7;
  int limit = base + last - first;
  for(;;) {
    if(limit - base > thresh) {
      swap(((limit-base)/2)+base, base);
      i = base + 1;
      j = limit - 1;
      if(compare(i, j) > 0)
        swap(i, j);
      if(compare(base, j) > 0)
        swap(base, j);
      if(compare(i, base) > 0)
        swap(i, base);
      for(;;) {
        do {
          i++;
        } while(compare(i, base) < 0);
        do {
          j--;
        } while(compare(j, base) > 0);
        if(i > j)
          break;
        swap(i, j);
      }
      swap(base, j);
      if(j - base > limit - i) {
        sp[0] = base;
        sp[1] = j;
        base = i;
      }
      else {
        sp[0] = i;
        sp[1] = limit;
        limit = j;
      }
      sp += 2;
    }
    else {
      for(j=base, i=j+1; i<limit; j=i, i++) {
        for(; compare(j, j+1) > 0; j--) {
          swap(j, j+1);
          if(j == base)
            break;
        }
      }
      if(sp != stack) {
        sp -= 2;
        base = sp[0];
        limit = sp[1];
      }
      else {
        break;
      }
    }
  }
}


//  ------------------------------------------------------------------

void garealist::sort(const char* specs, int first, int last) {

  if(specs)
    sortspec = specs;
  else
    sortspec = CFG->arealistsort;
  if(last == -1)
    last = list.size();
  if(sortspec.length())
    qsort(first, last);
}


//  ------------------------------------------------------------------

int garealist::echoid_to_number(const gstring& echoid) {

  for(int n=0; n<list.size(); n++)
    if(echoid.ieql(list[n]->echoid()))
      return n;
  return -1;
}


//  ------------------------------------------------------------------

int garealist::echoid_to_id(const gstring& echoid) {

  for(int n=0; n<list.size(); n++)
    if(echoid.ieql(list[n]->echoid()))
      return list[n]->areaid();
  return -1;
}


//  ------------------------------------------------------------------

garealistdata* garealist::echoid_to_pointer(const gstring& echoid) {

  for(int n=0; n<list.size(); n++)
    if(echoid.ieql(list[n]->echoid()))
      return list[n];
  return NULL;
}


//  ------------------------------------------------------------------

int garealist::number_to_id(int areano) {

  return list[areano]->areaid();
}


//  ------------------------------------------------------------------

garealistdata* garealist::number_to_pointer(int areano) {

  return list[areano];
}


//  ------------------------------------------------------------------

int garealist::id_to_number(int areaid) {

  for(int n=0; n<list.size(); n++) {
    if(list[n]->areaid() == areaid)
      return n;
  }
  return -1;
}


//  ------------------------------------------------------------------

garealistdata* garealist::id_to_pointer(int areaid) {

  int areano = id_to_number(areaid);
  if(areano != -1)
    return list[areano];
  return NULL;
}


//  ------------------------------------------------------------------
//  Update exportlist scanning file

void garealist::update_exportlist(gactivearea* area) {

  gfile fp;
  char buf[256];

  if(area->islocal()) {
    errorlevel |= EXIT_LOCAL;
    locpost++;
  }
  else if(area->isnet()) {
    errorlevel |= EXIT_NET;
    netpost++;
  }
  else {
    errorlevel |= EXIT_ECHO;
    echopost++;

    // Search for echotag in the taglist
    strcpy(buf, area->echoid().c_str());
    int n = SearchTaglist(echoout, buf);

    // Add area to taglist if not found
    if(*echoout[n] == 0) {

      echoout = (Echo*)throw_realloc(echoout, (n+2)*sizeof(Echo));

      strcpy(echoout[n], buf);
      *echoout[n+1] = NUL;

      fp.fopen(AddPath(CFG->areapath, CFG->semaphore.exportlist), "wt");
      if(fp.isopen()) {
        n = 0;
        while(*echoout[n]) {
          fp.fputs(echoout[n]);
          fp.fputs("\n");
          n++;
        }
        fp.fclose();
      }
    }
  }
}


//  ------------------------------------------------------------------

void RenumberArea(gactivearea* area) {

  if(not area->renumber()) {
    HandleGEvent(EVTT_JOBFAILED);
    update_statusline(CFG->language[LNG_NoRenum]);
    waitkeyt(5000);
    reader_keyok = YES;
  }
  else {

    area->markset.reset();

    // Touch the netmail rescan semaphore
    if(area->isnet())
      TouchFile(AddPath(CFG->areapath, CFG->semaphore.netscan));

    // Tell user we are finished
    update_statuslinef("%u %s", area->msgnset.size(), CFG->language[LNG_Renumbered]);
    waitkeyt(5000);
  }
}


//  ------------------------------------------------------------------

void gactivearea::open() {

  randomize();

  area.open();

  if(area.open_references() == 1) {
    area.scan();
    set_scanned();
    update_data();
  }
}


//  ------------------------------------------------------------------

void gactivearea::close() {

  if(readpersonal()) {
    set_lastread(msgnset.absolute_to_relative(area.lastreadentry));
    set_readpersonal(false);
  }

  set_readmarked(false);

  update_data();

  if(area.open_references() == 1) {
    markset.reset();
    area.save_lastread();
    area.close();
  }
}


//  ------------------------------------------------------------------

void FreqWaZOO(const gstring& files, const gftnaddrs& dest, const ftn_attr& attr) {

  gstring buf;
  buf = files;
  Path filename, outbound, tmp;

  StripBackslash(strcpy(outbound, CFG->outboundpath.c_str()));
  strcpy(filename, outbound);

  if(dest.zone != CFG->address[0].zone) {
    sprintf(tmp, ".%03X", dest.zone);
    strcat(filename, tmp);
    if(not is_dir(filename))
      mkdir(filename, S_IWUSR);
  }

  strcat(filename, "\\");
  sprintf(tmp, "%04X", dest.net);
  strcat(filename, tmp);
  sprintf(tmp, "%04X", dest.node);
  strcat(filename, tmp);

  if(dest.point) {
    strcat(filename, ".PNT");
    if(not is_dir(filename))
      mkdir(filename, S_IWUSR);
    strcat(filename, "\\");
    sprintf(tmp, "%08X", dest.point);
    strcat(filename, tmp);
  }

  strcpy(tmp, filename);

  // filename now contains everything but the extension, and we are sure
  // that the directory exists

  strcat(tmp, ".REQ");

  int i = 0;
  while(buf[i]) {
    if(buf[i] == ' ') {
      if(buf[i+1] != '!')
        buf[i] = '\n';
    }
    i++;
  }

  FILE* fcs = fopen(tmp, "at");
  if(fcs) {
    fprintf(fcs, "%s\n", buf.c_str());
    fclose(fcs);
  }

  if(CFG->frqoptions & FREQ_USEFLOWFILE) {

    strcpy(tmp, filename);

    if(attr.imm())
      strcat(tmp, ".ILO");
    else if(attr.cra())
      strcat(tmp, ".CLO");
    else if(attr.hld())
      strcat(tmp, ".HLO");
    else
      strcat(tmp, ".FLO");

    if(*tmp)
      TouchFile(tmp);
  }
}
 

//  ------------------------------------------------------------------

void gactivearea::savemsg(int mode, GMsg& msg) {

  if(CFG->frqwazoo and msg.attr.frq()) {
    FreqWaZOO(msg.subject, msg.destaddr, msg.attr);
    if(CFG->frqoptions & FREQ_NOWAZOOMSG)
      return;
    msg.attr.frq0();
  }

/////////  area.save_msg(mode, msg);

  if(not (mode & GMSG_NOLSTUPD) or msg.attr.uns()) {
    update_data();
    if(msg.attr.loc())
      list.update_exportlist(this);
    if(msg.attr.uns()) {
      if(isqwk()) {
        gfile fp;
        fp.fopen(AddPath(CFG->goldpath, "GOLDQWK.LST"), "at");
        if(fp.isopen()) {
          fp.printf("%s %lu\n", echoid().c_str(), msg.msgno);
          fp.fclose();
        }
      }
      else if(isinternet()) {
        gfile fp;
        fp.fopen(AddPath(CFG->goldpath, "GOLDSOUP.LST"), "at");
        if(fp.isopen()) {
          fp.printf("%s %lu\n", echoid().c_str(), msg.msgno);
          fp.fclose();
        }
      }
    }
  }
}


//  ------------------------------------------------------------------

gfile& operator<<(gfile& fp, const garealistcfg& a) {

  fp << a.cfg_areaid << a.cfg_groupid << a.cfg_board << a.cfg_type << a.cfg_msgbase;
  fp << a.cfg_aka << a.cfg_attr;
  fp << a.cfg_echoid << a.cfg_desc << a.cfg_path;
  fp << a.cfg_scanbits;
  return fp;
}


//  ------------------------------------------------------------------

gfile& operator>>(gfile& fp, garealistcfg& a) {

  fp >> a.cfg_areaid >> a.cfg_groupid >> a.cfg_board >> a.cfg_type >> a.cfg_msgbase;
  fp >> a.cfg_aka >> a.cfg_attr;
  fp >> a.cfg_echoid >> a.cfg_desc >> a.cfg_path;
  fp >> a.cfg_scanbits;
  return fp;
}


//  ------------------------------------------------------------------

