
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gealst.h,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------


#ifndef __GEALST_H
#define __GEALST_H


#include <gtag.h>
//#include <gedacfg.h>
#include <gftnadrs.h>
#include <gmbarea.h>
#include <gvector.h>

#include "gemlst.h"
#include "geplay.h"


//  ------------------------------------------------------------------

class garealist;


//  ------------------------------------------------------------------

class gedmsg : public gmsg {

public:

  gedmsg() { }
  ~gedmsg() { }

};


//  ------------------------------------------------------------------

class garealistcfg : public gareacfg {

protected:

  enum {
    SCAN       = 0x01,
    SCANEXCL   = 0x02,
    SCANINCL   = 0x04,
    PMSCAN     = 0x10,
    PMSCANEXCL = 0x20,
    PMSCANINCL = 0x40
  };

  int      cfg_groupid;
  int      cfg_originno;
  ftn_addr cfg_aka;
  ftn_attr cfg_attr;
  gstring  cfg_desc;
  int      cfg_scanbits;

public:

  garealistcfg()                    { cfg_scanbits = 0; cfg_groupid = cfg_originno = 0; }
  ~garealistcfg()                   { }

  int             groupid() const   { return cfg_groupid; }
  int             originno() const  { return cfg_originno; }
  const ftn_addr& aka() const       { return cfg_aka; }
  const ftn_attr& attr() const      { return cfg_attr; }
  const gstring&  desc() const      { return cfg_desc; }

  bool scan() const                 { return !!(cfg_scanbits & SCAN); }
  bool scanexcl() const             { return !!(cfg_scanbits & SCANEXCL); }
  bool scanincl() const             { return !!(cfg_scanbits & SCANINCL); }
  bool pmscan() const               { return !!(cfg_scanbits & PMSCAN); }
  bool pmscanexcl() const           { return !!(cfg_scanbits & PMSCANEXCL); }
  bool pmscanincl() const           { return !!(cfg_scanbits & PMSCANINCL); }

  void set_groupid(int g)           { cfg_groupid = g; }
  void set_board(uint b)            { cfg_board = b; }
  void set_originno(int o)          { cfg_originno = o; }
  void set_aka(const ftn_addr& a)   { cfg_aka = a; }
  void set_attr(const ftn_attr& a)  { cfg_attr = a; }
  void set_echoid(const gstring& s) { cfg_echoid = s; }
  void set_desc(const gstring& s)   { cfg_desc = s; }
  void set_path(const gstring& s)   { cfg_path = s; }
  void set_origin(const gstring& s);

  void set_scan(bool s=true)        { if(s) cfg_scanbits |= SCAN;       else cfg_scanbits &= ~SCAN; }
  void set_scanexcl(bool s=true)    { if(s) cfg_scanbits |= SCANEXCL;   else cfg_scanbits &= ~SCANEXCL; }
  void set_scanincl(bool s=true)    { if(s) cfg_scanbits |= SCANINCL;   else cfg_scanbits &= ~SCANINCL; }
  void set_pmscan(bool s=true)      { if(s) cfg_scanbits |= PMSCAN;     else cfg_scanbits &= ~PMSCAN; }
  void set_pmscanexcl(bool s=true)  { if(s) cfg_scanbits |= PMSCANEXCL; else cfg_scanbits &= ~PMSCANEXCL; }
  void set_pmscanincl(bool s=true)  { if(s) cfg_scanbits |= PMSCANINCL; else cfg_scanbits &= ~PMSCANINCL; }

  friend gfile& operator<<(gfile& fp, const garealistcfg& a);
  friend gfile& operator>>(gfile& fp, garealistcfg& a);

};


//  ------------------------------------------------------------------
//  Class to contain the "passive" (non-config) data in the arealist

class garealistdata : public garealistcfg {

protected:

  bool is_marked;
  bool is_changed;
  bool is_scanned;
  bool is_pmscanned;

  uint unread_count;
  uint scans_count;
  uint scan_count;
  uint pmscan_count;
  uint lastread_count;

public:

  gtag markset;

  garealistdata();
  ~garealistdata();

  bool marked() const              { return is_marked; }
  bool changed() const             { return is_changed; }
  bool scanned() const             { return is_scanned; }
  bool pmscanned() const           { return is_pmscanned; }

  uint unread() const              { return unread_count; }
  uint scans() const               { return scans_count; }
  uint lastread() const            { return lastread_count; }
  uint scancount() const           { return scan_count; }
  uint pmscancount() const         { return pmscan_count; }

  void toggle_marked()             { is_marked ^= true; }

  void set_marked(bool f=true)     { is_marked = f; }
  void set_changed(bool f=true)    { is_changed = f; }
  void set_scanned(bool f=true)    { is_scanned = f; }
  void set_pmscanned(bool f=true)  { is_pmscanned = f; }

  void inc_scans()                 { scans_count++; }

  void set_unread(uint n)          { unread_count = n; }
  void set_scans(uint n)           { scans_count = n; }
  void set_lastread(uint n)        { lastread_count = n; }
  void set_scancount(uint n)       { scan_count = n; }
  void set_pmscancount(uint n)     { pmscan_count = n; }

  void update_data();

};


//  ------------------------------------------------------------------

class gareagroupdata {

public:

  gstring  address;
  bool     akamatching;
  bool     areacopyaddid;
  bool     areacopydirect;
  gstring  areacopyto;
  bool     areafreqdirect;
  gstring  areafreqto;
  bool     areareplydirect;
  gstring  areareplyto;
  ftn_attr attr;
  int      ctrlinfo;
  bool     edithardterm;
  bool     editmixcase;
  bool     editreplyre;
  gstring  followupto;
  bool     forcetemplate;
  gstring  gif;
  gstring  inputfile;
  gstring  internetaddress;
  gftnnode internetgate;
  bool     internetmsgid;
  bool     internetrfcbody;
  gstring  loadlanguage;
  gstring  mood;
  int      msglistdate;
  bool     msglistfast;
  bool     msglistfirst;
  bool     msglistheader;
  bool     msglistwidesubj;
  gstring  netname;
  gstring  newsgroups;
  gstring  organization;
  gstring  origin;
  gstring  outputfile;
  gplay    play;
  gstring  quotebuffile;
  gstring  quotechars;
  gstring  quotestring;
  bool     quotewraphard;
  gstring  replyto;
  gstring  searchfor;
  gstring  sender;
  gstring  signature;
  gstring  tagline;
  char     taglinechar;
  gstring  taglinefile;
  bool     taglinesupport;
  gstring  tearline;
  bool     templatematch;
  gstring  tpl;
  int      twitmode;
  bool     usefwd;
  gftnnode username;
  bool     viewhidden;
  bool     viewkludge;
  bool     viewquote;
  gstring  whoto;
  gstring  xlatexport;
  gstring  xlatimport;

};


//  ------------------------------------------------------------------
//  Class to work with an active area

class gactivearea {

protected:

  gmb_area&      area;
  garealistdata& data;
  garealist&     list;
  gareagroupdata group;

  ulong info_bookmark;
  bool  mode_readmarked;
  bool  mode_readpersonal;

public:

  gactivearea(gmb_area& a, garealistdata& d, garealist& l);
  ~gactivearea();

  void initialize();
  void randomize(bool posting=false);

  gtag& msgnset;
  gtag& pmrkset;
  gtag& markset;

  gmb_area*       area_pointer() const        { return &area; }

  bool            isopen()                    { return area.isopen(); }
  bool            islocked()                  { return area.islocked(); }

  int             areaid() const              { return data.areaid(); }
  int             groupid() const             { return data.groupid(); }
  uint            type() const                { return data.type(); }
  uint            msgbase() const             { return data.msgbase(); }
  uint            board() const               { return data.board(); }
  const gstring&  echoid() const              { return data.echoid(); }
  const gstring&  desc() const                { return data.desc(); }
  const gstring&  path() const                { return data.path(); }
  const ftn_addr& orig_aka() const            { return data.aka(); }
  const ftn_attr& orig_attr() const           { return data.attr(); }

  bool            isfts1() const              { return data.isfts1(); }
  bool            isopus() const              { return data.isopus(); }
  bool            isezycom() const            { return data.isezycom(); }
  bool            isfido() const              { return data.isfido(); }
  bool            isgoldbase() const          { return data.isgoldbase(); }
  bool            ishudson() const            { return data.ishudson(); }
  bool            isjam() const               { return data.isjam(); }
  bool            ispcboard() const           { return data.ispcboard(); }
  bool            issquish() const            { return data.issquish(); }
  bool            iswildcat() const           { return data.iswildcat(); }
  bool            isadeptxbbs() const         { return data.isadeptxbbs(); }
  bool            isseparator() const         { return data.isseparator(); }

  bool            isnet() const               { return data.isnet(); }
  bool            isecho() const              { return data.isecho(); }
  bool            islocal() const             { return data.islocal(); }
  bool            isemail() const             { return data.isemail(); }
  bool            isnewsgroup() const         { return data.isnewsgroup(); }
  bool            isinternet() const          { return data.isinternet(); }
  bool            isqwk() const               { return data.isqwk(); }
  bool            issoup() const              { return data.issoup(); }

  bool            scan() const                { return data.scan(); }
  bool            scanexcl() const            { return data.scanexcl(); }
  bool            scanincl() const            { return data.scanincl(); }
  bool            pmscan() const              { return data.pmscan(); }
  bool            pmscanexcl() const          { return data.pmscanexcl(); }
  bool            pmscanincl() const          { return data.pmscanincl(); }

  bool            marked() const              { return data.marked(); }
  bool            unread() const              { return data.unread(); }
  bool            changed() const             { return data.changed(); }
  bool            scanned() const             { return data.scanned(); }
  bool            pmscanned() const           { return data.pmscanned(); }

  uint            lastread() const            { return data.lastread(); }
  uint            scancount() const           { return data.scancount(); }
  uint            pmscancount() const         { return data.pmscancount(); }

  ulong           bookmark() const            { return info_bookmark; }
  bool            readmarked() const          { return mode_readmarked; }
  bool            readpersonal() const        { return mode_readpersonal; }

  const gstring&  address() const             { return group.address; }
  bool            akamatching() const         { return group.akamatching; }
  bool            areacopyaddid() const       { return group.areacopyaddid; }
  bool            areacopydirect() const      { return group.areacopydirect; }
  const gstring&  areacopyto() const          { return group.areacopyto; }
  bool            areafreqdirect() const      { return group.areafreqdirect; }
  const gstring&  areafreqto() const          { return group.areafreqto; }
  bool            areareplydirect() const     { return group.areareplydirect; }
  const gstring&  areareplyto() const         { return group.areareplyto; }
  const ftn_attr& attr() const                { return group.attr; }
  int             ctrlinfo() const            { return group.ctrlinfo; }
  bool            edithardterm() const        { return group.edithardterm; }
  bool            editmixcase() const         { return group.editmixcase; }
  bool            editreplyre() const         { return group.editreplyre; }
  const gstring&  followupto() const          { return group.followupto; }
  bool            forcetemplate() const       { return group.forcetemplate; }
  const gstring&  gif() const                 { return group.gif; }
  const gstring&  inputfile() const           { return group.inputfile; }
  const gstring&  internetaddress() const     { return group.internetaddress; }
  const gftnnode& internetgate() const        { return group.internetgate; }
  bool            internetmsgid() const       { return group.internetmsgid; }
  bool            internetrfcbody() const     { return group.internetrfcbody; }
  const gstring&  loadlanguage() const        { return group.loadlanguage; }
  const gstring&  mood() const                { return group.mood; }
  int             msglistdate() const         { return group.msglistdate; }
  bool            msglistfast() const         { return group.msglistfast; }
  bool            msglistfirst() const        { return group.msglistfirst; }
  bool            msglistheader() const       { return group.msglistheader; }
  bool            msglistwidesubj() const     { return group.msglistwidesubj; }
  const gstring&  netname() const             { return group.netname; }
  const gstring&  newsgroups() const          { return group.newsgroups; }
  const gstring&  organization() const        { return group.organization; }
  const gstring&  origin() const              { return group.origin; }
  const gstring&  outputfile() const          { return group.outputfile; }
  const gplay&    play() const                { return group.play; }
  const gstring&  quotebuffile() const        { return group.quotebuffile; }
  const gstring&  quotechars() const          { return group.quotechars; }
  const gstring&  quotestring() const         { return group.quotestring; }
  bool            quotewraphard() const       { return group.quotewraphard; }
  const gstring&  replyto() const             { return group.replyto; }
  const gstring&  searchfor() const           { return group.searchfor; }
  const gstring&  sender() const              { return group.sender; }
  const gstring&  tagline() const             { return group.tagline; }
  char            taglinechar() const         { return group.taglinechar; }
  const gstring&  taglinefile() const         { return group.taglinefile; }
  bool            taglinesupport() const      { return group.taglinesupport; }
  const gstring&  tearline() const            { return group.tearline; }
  bool            templatematch() const       { return group.templatematch; }
  const gstring&  tpl() const                 { return group.tpl; }
  int             twitmode() const            { return group.twitmode; }
  bool            usefwd() const              { return group.usefwd; }
  const gftnnode& username() const            { return group.username; }
  bool            viewhidden() const          { return group.viewhidden; }
  bool            viewkludge() const          { return group.viewkludge; }
  bool            viewquote() const           { return group.viewquote; }
  const gstring&  whoto() const               { return group.whoto; }
  const gstring&  xlatexport() const          { return group.xlatexport; }
  const gstring&  xlatimport() const          { return group.xlatimport; }

  ftn_attr&       set_attr()                  { return group.attr; }

  void inc_scans()                            { data.inc_scans(); }

  void set_marked(bool f=true)                { data.set_marked(f); }
  void set_unread(bool f=true)                { data.set_unread(f); }
  void set_changed(bool f=true)               { data.set_changed(f); }
  void set_scanned(bool f=true)               { data.set_scanned(f); }
  void set_pmscanned(bool f=true)             { data.set_pmscanned(f); }

  void set_lastread(uint n)                   { data.set_lastread(n); area.lastread = n; }
  void set_scancount(uint n)                  { data.set_scancount(n); }
  void set_pmscancount(uint n)                { data.set_pmscancount(n); }

  void set_bookmark(uint m)                   { info_bookmark = msgnset.relative_to_absolute(m); }
  void set_readmarked(bool f=true)            { mode_readmarked = f; }
  void set_readpersonal(bool f=true)          { mode_readpersonal = f; }

  void set_address(const gstring& a)          { group.address = a; }
  void set_akamatching(bool a)                { group.akamatching = a; }
  void set_areacopyaddid(bool a)              { group.areacopyaddid = a; }
  void set_areacopydirect(bool a)             { group.areacopydirect = a; }
  void set_areacopyto(const gstring& s)       { group.areacopyto = s; }
  void set_areafreqdirect(bool a)             { group.areafreqdirect = a; }
  void set_areafreqto(const gstring& s)       { group.areafreqto = s; }
  void set_areareplydirect(bool a)            { group.areareplydirect = a; }
  void set_areareplyto(const gstring& s)      { group.areareplyto = s; }
  void set_attr(const ftn_attr& a)            { group.attr = a; }
  void set_ctrlinfo(int i)                    { group.ctrlinfo = i; }
  void set_edithardterm(bool a)               { group.edithardterm = a; }
  void set_editmixcase(bool a)                { group.editmixcase = a; }
  void set_editreplyre(bool a)                { group.editreplyre = a; }
  void set_followupto(const gstring& s)       { group.followupto = s; }
  void set_forcetemplate(bool a)              { group.forcetemplate = a; }
  void set_gif(const gstring& s)              { group.gif = s; }
  void set_inputfile(const gstring& s)        { group.inputfile = s; }
  void set_internetaddress(const gstring& s)  { group.internetaddress = s; }
  void set_internetgate(const gftnnode& s)    { group.internetgate = s; }
  void set_internetmsgid(bool a)              { group.internetmsgid = a; }
  void set_internetrfcbody(bool a)            { group.internetrfcbody = a; }
  void set_loadlanguage(const gstring& s)     { group.loadlanguage = s; }
  void set_mood(const gstring& s)             { group.mood = s; }
  void set_msglistdate(int i)                 { group.msglistdate = i; }
  void set_msglistfast(bool a)                { group.msglistfast = a; }
  void set_msglistfirst(bool a)               { group.msglistfirst = a; }
  void set_msglistheader(bool a)              { group.msglistheader = a; }
  void set_msglistwidesubj(bool a)            { group.msglistwidesubj = a; }
  void set_netname(const gstring& s)          { group.netname = s; }
  void set_newsgroups(const gstring& s)       { group.newsgroups = s; }
  void set_organization(const gstring& s)     { group.organization = s; }
  void set_origin(const gstring& s)           { group.origin = s; }
  void set_outputfile(const gstring& s)       { group.outputfile = s; }
  void set_play(const gplay& s)               { group.play = s; }
  void set_quotebuffile(const gstring& s)     { group.quotebuffile = s; }
  void set_quotechars(const gstring& s)       { group.quotechars = s; }
  void set_quotestring(const gstring& s)      { group.quotestring = s; }
  void set_quotewraphard(bool a)              { group.quotewraphard = a; }
  void set_replyto(const gstring& s)          { group.replyto = s; }
  void set_searchfor(const gstring& s)        { group.searchfor = s; }
  void set_sender(const gstring& s)           { group.sender = s; }
  void set_tagline(const gstring& s)          { group.tagline = s; }
  void set_taglinechar(char c)                { group.taglinechar = c; }
  void set_taglinefile(const gstring& s)      { group.taglinefile = s; }
  void set_taglinesupport(bool a)             { group.taglinesupport = a; }
  void set_tearline(const gstring& s)         { group.tearline = s; }
  void set_templatematch(bool a)              { group.templatematch = a; }
  void set_tpl(const gstring& s)              { group.tpl = s; }
  void set_twitmode(int i)                    { group.twitmode = i; }
  void set_usefwd(bool a)                     { group.usefwd = a; }
  void set_username(const gftnnode& s)        { group.username = s; }
  void set_viewhidden(bool a)                 { group.viewhidden = a; }
  void set_viewkludge(bool a)                 { group.viewkludge = a; }
  void set_viewquote(bool a)                  { group.viewquote = a; }
  void set_whoto(const gstring& s)            { group.whoto = s; }
  void set_xlatexport(const gstring& s)       { group.xlatexport = s; }
  void set_xlatimport(const gstring& s)       { group.xlatimport = s; }

  void toggle_readmarked()                    { mode_readmarked ^= true; }
  void toggle_readpersonal()                  { mode_readpersonal ^= true; }

  int  toggle_msglistwidesubj()               { group.msglistwidesubj ^= true; return group.msglistwidesubj; }
  int  toggle_viewhidden()                    { group.viewhidden ^= true; return group.viewhidden; }
  int  toggle_viewkludge()                    { group.viewkludge ^= true; return group.viewkludge; }
  int  toggle_viewquote()                     { group.viewquote ^= true; return group.viewquote; }

  int  next_msglistdate()                     { group.msglistdate++; if(group.msglistdate > MSGLISTDATE_RECEIVED) group.msglistdate = MSGLISTDATE_NONE; return group.msglistdate; }


  void update_data();

  void open();
  void close();

  void suspend();
  void resume();

  void lock();
  void unlock();

  void scanarea();
  void scanareapm();

  int  loadhdr(gedmsg& msg, ulong msgno);
  int  loadmsg(gedmsg& msg, ulong msgno, int margin, int mode=0);

  void savehdr(int mode, gedmsg& msg);
  void savemsg(int mode, gedmsg& msg);

  void delmsg(gedmsg& msg);

  void newmsgno(gedmsg& msg);

  bool userlookup(gstring& lookfor);

  int  renumber();

  void updatetimesread(gedmsg& msg);

  void hexdumpmsg(gedmsg& msg, gstring& dump);

  void sethighwatermark();
  void resethighwatermark();

  void deletemsg(gedmsg& msg, int direction);

};


//  ------------------------------------------------------------------

inline void  gactivearea::lock()                                { area.lock(); }
inline void  gactivearea::unlock()                              { area.unlock(); }
inline void  gactivearea::suspend()                             { area.suspend(); }
inline void  gactivearea::resume()                              { area.resume(); }
inline void  gactivearea::delmsg(gedmsg& msg)                     { area.del_msg(msg); }
inline void  gactivearea::newmsgno(gedmsg& msg)                   { area.new_msgno(msg); }
inline bool  gactivearea::userlookup(gstring& lookfor)          { return area.user_lookup(lookfor); }
inline int   gactivearea::renumber()                            { return area.renumber(); }
inline void  gactivearea::updatetimesread(gedmsg& msg)            { area.update_timesread(msg); }
inline void  gactivearea::hexdumpmsg(gedmsg& msg, gstring& dump)  { area.hexdump_msg(msg, dump); }
inline void  gactivearea::sethighwatermark()                    { area.set_highwater_mark(); }
inline void  gactivearea::resethighwatermark()                  { area.reset_highwater_mark(); }


//  ------------------------------------------------------------------

class glistscan {

public:

  glistscan& operator=(const glistscan& o) { menutext=o.menutext; file=o.file; return *this; }

  gstring menutext;
  gstring file;
  void save(gfile& f) const;
  void load(gfile& f);
};

typedef gvector<glistscan> garealistscan;


//  ------------------------------------------------------------------

class garealist {

protected:

  gvectp<garealistdata> list;

  gmb_adeptxbbs_manager* adeptxbbs_manager;
  gmb_ezycom_manager*    ezycom_manager;
  gmb_fido_manager*      fido_manager;
  gmb_goldbase_manager*  goldbase_manager;
  gmb_hudson_manager*    hudson_manager;
  gmb_jam_manager*       jam_manager;
  gmb_pcboard_manager*   pcboard_manager;
  gmb_squish_manager*    squish_manager;
  gmb_wildcat_manager*   wildcat_manager;

  gactivearea* get_active(int areano);

  void swap(int a, int b);
  int  compare(int aa, int bb);
  void qsort(int first, int last);

public:

  uint           msgbases;
  gstring        sortspec;
  garealistscan  listscan;

  garealist();
  ~garealist();

  void           initialize();
  void           reset();

  gactivearea*   activate(int areaid);
  gactivearea*   activate_no(int areano)         { return get_active(areano); }
  void           deactivate(gactivearea* area);

  uint           areas() const                   { return list.size(); }

  garealistdata* operator[](uint i) const        { return list[i]; }

  int            id_to_number(int id);
  garealistdata* id_to_pointer(int id);

  int            number_to_id(int no);
  garealistdata* number_to_pointer(int no);

  int            echoid_to_id(const gstring& id);
  int            echoid_to_number(const gstring& id);
  garealistdata* echoid_to_pointer(const gstring& id);

  void           add(AreaCfg& aa);

  void           get_area(char* val);
  bool           get_area_firstpart(AreaCfg& aa, char*& key, char*& val);
  void           get_areadef(char* val);
  void           get_areasep(char* val);
  void           get_areafile(char* val);

  void           get_areadesc(char* val);
  void           get_echolist(char* val);
  void           set_areadesc(const char* echoid, const char* desc);

  void           sort(const char* specs=NULL, int first=0, int last=-1);
  void           sort_areagroup(const char* options, int beginarea, int endarea);

  void           write_areadef(const char* file);

  void           read_goldlast();
  void           write_goldlast();

  int            scan_areas(int mode, int currno, int pmscan, int& pmails, int& pmareas, const char* file=NULL);

  void           update_exportlist(gactivearea* area);

  void           wide_open();
  void           wide_close();

  void           adeptxbbs_init(const gstring& path, int userno);
  void           ezycom_init(const gstring& msgbasepath, const gstring& userbasepath, int userno);
  void           fido_init(const gstring& lastread, int hwmarks, int nullfix, int userno, const gstring& userpath);
  void           goldbase_init(const gstring& path, const gstring& syspath, int userno);
  void           hudson_init(const gstring& path, const gstring& syspath, int userno, long sizewarn, int ra2usersbbs);
  void           jam_init(const gstring& path, int harddelete);
  void           pcboard_init(const gstring& path, int userno);
  void           squish_init(const gstring& userpath, int userno, int direct, int squishscan);
  void           wildcat_init(int userno);

  void           save(gfile& fp);
  void           load(gfile& fp);

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

