
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gearea2.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Area initializing.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif


//  ------------------------------------------------------------------

extern bool cmdlinewriteareas;


//  ------------------------------------------------------------------

void AreaInit() {

  // Read the lastreads from the last session
  if(CFG->areakeeplast)
    AL.read_goldlast();

  // Setup start echo
  if(startecho.length() == 0)
    startecho = CFG->areastart;
  nextarea = CFG->areaautonext;

  // Sort the areas
  if(AL.sortspec.length())
    CFG->arealistsort = AL.sortspec;
  AL.sort();

  // Write GOLDAREA.INC if told
  if(cmdlinewriteareas)
    AL.write_areadef(AddPath(CFG->goldpath, "goldarea.inc"));

  // Get the global max length of echoids (for the arealist)
  extern int arealistnumgrps;
  extern int areaswithgroupid;
  for(int n=0; n<AL.areas(); n++) {
    int _tmp = AL[n]->echoid().length();
    if(echoid_width < _tmp)
      echoid_width = _tmp;
    if(AL[n]->groupid()) {
      areaswithgroupid++;
      if(AL[n]->groupid() & 0x8000u)
        arealistnumgrps = true;
    }
  }
  if(CFG->arealistechomax) {
    if(CFG->arealistechomax > (int)(sizeof(Echo)-1))
      CFG->arealistechomax = sizeof(Echo)-1;
    if(CFG->arealistechomax < 0)
      echoid_width += CFG->arealistechomax;
    else
      echoid_width = CFG->arealistechomax;
  }
  if(areaswithgroupid and CFG->arealistgroupid)
    groupid_width = arealistnumgrps ? 3 : 1;

  int spaces = 0;
  bool area_found = false;
  bool marked_found = false;
  bool desc_found = false;
  bool count_found = false;
  bool pmark_found = false;
  bool unread_found = false;
  bool changed_found = false;
  bool echoid_found = false;
  bool groupid_found = false;

  for(int pass=1; pass<=2; pass++) {
    if(pass == 2) {
      if(!area_found)     area_width = 0;
      if(!marked_found)   marked_width = 0;
      if(!desc_found)     desc_width = 0;
      if(!count_found)    count_width = 0;
      if(!pmark_found)    pmark_width = 0;
      if(!unread_found)   unread_width = 0;
      if(!changed_found)  changed_width = 0;
      if(!echoid_found)   echoid_width = 0;
      if(!groupid_found)  groupid_width = 0;
      if(desc_width == -1) {
        desc_width =
          gvid.numcols - 2 - spaces - area_width - marked_width -
          count_width - pmark_width - unread_width - changed_width -
          echoid_width - groupid_width;
      }
    }
    int pos = 0;
    const char* p = CFG->arealistformat.c_str();
    while(*p) {
      char c = (char)toupper(*p);
      char d = *(++p);
      int w = atoi(p);
      while(isdigit(*p))
        p++;
      if(pass == 1) {
        if(isalpha(c)) {
          switch(c) {
            case 'A':  area_found = true;     if(isdigit(d)) area_width = w;     break;
            case 'M':  marked_found = true;   if(isdigit(d)) marked_width = w;   break;
            case 'D':  desc_found = true;     if(isdigit(d)) desc_width = w;     break;
            case 'C':  count_found = true;    if(isdigit(d)) count_width = w;    break;
            case 'P':  pmark_found = true;    if(isdigit(d)) pmark_width = w;    break;
            case 'U':  unread_found = true;   if(isdigit(d)) unread_width = w;   break;
            case 'N':  changed_found = true;  if(isdigit(d)) changed_width = w;  break;
            case 'E':  echoid_found = true;   if(isdigit(d)) echoid_width = w;   break;
            case 'G':  groupid_found = true;  if(isdigit(d)) groupid_width = w;  break;
          }
        }
        else {
          spaces++;
        }
      }
      else {
        switch(c) {
          case 'A':  area_pos = pos;     pos += area_width;     break;
          case 'M':  marked_pos = pos;   pos += marked_width;   break;
          case 'D':  desc_pos = pos;     pos += desc_width;     break;
          case 'C':  count_pos = pos;    pos += count_width;    break;
          case 'P':  pmark_pos = pos;    pos += pmark_width;    break;
          case 'U':  unread_pos = pos;   pos += unread_width;   break;
          case 'N':  changed_pos = pos;  pos += changed_width;  break;
          case 'E':  echoid_pos = pos;   pos += echoid_width;   break;
          case 'G':  groupid_pos = pos;  pos += groupid_width;  break;
          default:   pos++;
        }
      }
    }
  }
}


//  ------------------------------------------------------------------

