
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gecarb.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Carbon Copy.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

void carboncopy_helper(const char* ptr, GMsg& msg) {

  gstring name = ptr;
  name.trim();

  msg.destaddr = 0;
  msg.destname = name;
  msg.attr.nwm1();

  HeaderView.Use(msg.area, &msg);
  HeaderView.Paint();

  gftnaddrs addr = msg.area->address();
  if(msg.area->isnet()) {
    addr = msg.origaddr;
    Lookup(msg, addr, name, 6, CFG->language[LNG_DestinationCC], msg.area);
  }

  update_statusline(CFG->language[LNG_ProcessCC]);

  if(addr.net or msg.area->islocal()) {
    // Create a CC if either address or name is different than the original.
    if((addr != msg.destaddr) or name.icmp(msg.destname)) {
      name += " (";
      name += addr.str();
      name += ")";
      msg.kludge.cc += name;
    }
  }
}


//  ------------------------------------------------------------------

int DoCarboncopy(GMsg& msg, gactivearea* area) {

  if(area->isecho())
    return 0;

  char buf[256];
  char buf2[256];

  ftn_attr  backup_attr = msg.attr;
  gstring   backup_destname = msg.destname;
  gftnaddrs backup_destaddr = msg.destaddr;

  msg.kludge.cc.reset();
  
  // Deal with carbon copies
  int ccline = 0;
  bool ignorecc = false;
  for(int i=0; i<msg.linearray.size(); i++) {
    const char* lptr = msg.linearray[i];
    if(strnieql(lptr, "CC:", 3)) {
      if(not ignorecc) {
        if(msg.kludge.cc.size() == 0) {
          msg.attr.nwm1();
          msg.attr.add(CFG->attribscc);
          HeaderView.Use(area, &msg);
          HeaderView.Paint();
          GMenuCarbon MenuCarbon;
          ignorecc = !MenuCarbon.Run(msg);
          if(ignorecc)                    // Do not process carcon copies
            break;
          ccline = i;
        }
        update_statusline(CFG->language[LNG_ProcessCC]);
        char* ptr = strcpy(buf2, lptr+3);
        if(CFG->carboncopylist != CC_KEEP)
          msg.linearray.erase(i);
        while(ptr) {
          ptr = strskip_wht(ptr);
          char* ptr2 = strchr(ptr, ',');
          if(ptr2)
            *ptr2++ = NUL;
          if(*ptr == '@') {
            char* tptr2 = ptr2;
            ptr++;
            gfile fp;
            fp.fopen(AddPath(CFG->goldpath, ptr), "rt", CFG->sharemode);
            if(fp.isopen()) {
              while(fp.fgets(buf, sizeof(buf))) {
                ptr = buf;
                do {
                  ptr = strskip_wht(ptr);
                  if(*ptr == NUL)
                    break;
                  ptr2 = strchr(ptr, ',');
                  if(ptr2)
                    *ptr2++ = NUL;
                  if(*ptr != '@')    // No nesting, please!
                    carboncopy_helper(ptr, msg);
                  ptr = ptr2;
                } while(ptr2);
              }
              fp.fclose();
            }
            ptr2 = tptr2;
          }
          else {
            carboncopy_helper(ptr, msg);
          }
          ptr = ptr2;
        }
      }
    }
  }

  // Fix the CC list in the message
  if(msg.kludge.cc.size()) {
    int n;
    gstring ccdest;
    ccdest = msg.destname + " (";
    ccdest += msg.destaddr.str();
    ccdest += ")";
    switch(CFG->carboncopylist) {
      case CC_REMOVE:
        // No list
        break;
      case CC_KEEP:
        // Keep list as it was entered
        break;
      case CC_HIDDEN:
        strcpy(buf2, "\001CC: %s");
        sprintf(buf, buf2, ccdest.c_str());
        msg.linearray.insert(ccline++, buf, gline::hard|gline::kludge);
        for(n=0; n<msg.kludge.cc.size(); n++) {
          if(msg.kludge.cc[n][0] != '#') {
            sprintf(buf, buf2, strskip_wht(msg.kludge.cc[n]+1));
            msg.linearray.insert(ccline++, buf, gline::hard|gline::kludge);
          }
        }
        break;
      case CC_VISIBLE:
        strcpy(buf2, "CC: %s");
        sprintf(buf, buf2, ccdest.c_str());
        msg.linearray.insert(ccline++, buf, gline::hard);
        for(n=0; n<msg.kludge.cc.size(); n++) {
          if(msg.kludge.cc[n][0] != '#') {
            sprintf(buf, buf2, strskip_wht(msg.kludge.cc[n]+1));
            msg.linearray.insert(ccline++, buf, gline::hard);
          }
        }
        break;
      case CC_NAMES:
        sprintf(buf, "CC: %s", msg.destname.c_str());
        for(n=0; n<msg.kludge.cc.size(); n++) {
          if(msg.kludge.cc[n][0] != '#') {
            int x = strlen(buf) + msg.kludge.cc.length(n) + 2;
            if(x < CFG->dispmargin) {
              if(strlen(buf) > 4)
                strcat(buf, ", ");
              strcat(buf, msg.kludge.cc[n]);
            }
            else {
              msg.linearray.insert(ccline++, buf, gline::hard);
              strcpy(buf, "CC: ");
              strcat(buf, msg.kludge.cc[n]);
            }
          }
        }
        msg.linearray.insert(ccline, buf, gline::hard);
        break;
    }
  }

  if(msg.kludge.cc.size() == 0)
    msg.kludge.cc.reset();

  // Restore original data
  msg.attr = backup_attr;
  msg.destname = backup_destname;
  msg.destaddr = backup_destaddr;

  return msg.kludge.cc.size();
}


//  ------------------------------------------------------------------

