
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gecfgg.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Configuration functions.
//  ------------------------------------------------------------------


#include <gcrcall.h>

#include "gccfgg.h"
#include "gecfgg.h"


//  ------------------------------------------------------------------

void golded_cfg::next_keyval() {

  get_keyval(key, val);
}


//  ------------------------------------------------------------------

word golded_cfg::next_keyvalcrc() {

  crc = get_keyvalcrc(key, val);
  return crc;
}


//  ------------------------------------------------------------------

void golded_cfg::set_int(int& a, int minval, int maxval) {

  int v = atoi(val);
  if(not in_range(v, minval, maxval))
    error = error_value_out_of_range;
  else
    a = v;
}


//  ------------------------------------------------------------------

void golded_cfg::set_int(int& a) {

  a = atoi(val);
}


//  ------------------------------------------------------------------

void golded_cfg::set_long(long& a) {

  a = atol(val);
}


//  ------------------------------------------------------------------

void golded_cfg::set_bool(bool& a) {

  a = get_bool(val);
}


//  ------------------------------------------------------------------

void golded_cfg::set_file(gstring& a) {

  a = val;
}


//  ------------------------------------------------------------------

void golded_cfg::set_path(gstring& a) {

  gpathcopy(a, val);
}


//  ------------------------------------------------------------------

void golded_cfg::set_yesno(int& a) {

  a = get_yesno(val);
}


//  ------------------------------------------------------------------

void golded_cfg::set_string(gstring& a) {

  a = val;
}


//  ------------------------------------------------------------------

void golded_cfg::set_attrib(ftn_attr& a) {

  a = val;
}


//  ------------------------------------------------------------------

void golded_cfg::set_int_table(int& a, const gstrvaltable* table, int tablesize) {

  if(not gtable2value(val, a, table, tablesize))
    error = error_unknown_value;
}


//  ------------------------------------------------------------------

bool golded_cfg::get_ctrlinfo(int& ctrlinfo) {

  const int CTRLINFO_TEARLINE = 3;
  const int CTRLINFO_ORIGIN   = 2;
  const int CTRLINFO_YES      = 1;
  const int CTRLINFO_NO       = 0;

  gstrvaltable table[] = {
    { "TEARLINE",  CTRLINFO_TEARLINE },
    { "ORIGIN",    CTRLINFO_ORIGIN },
    { "TRUE",      CTRLINFO_YES },
    { "YES",       CTRLINFO_YES },
    { "NO",        CTRLINFO_NO },
    { "FALSE",     CTRLINFO_NO }
  };

  ctrlinfo = CI_NONE;
  gtokenizer token(val);
  while(token.get()) {
    int tmp;
    gtable2value(token.c_str(), tmp, table, sizeof(table));
    switch(tmp) {
      case CTRLINFO_TEARLINE:
        ctrlinfo |= CI_TEAR;
        break;
      case CTRLINFO_ORIGIN:
        ctrlinfo |= CI_ORIG;
        break;
      case CTRLINFO_YES:
        ctrlinfo = CI_TEAR|CI_ORIG;
        break;
      case CTRLINFO_NO:
        ctrlinfo = CI_NONE;
        break;
      default:
        error = error_unknown_value;
        return false;
    }
  }

  return true;
}


//  ------------------------------------------------------------------

bool golded_cfg::switch_group_keywords() {

  error = no_error;

  switch(crc) {
    case CRC_ADDRESS         : if(group_active) handle_address         (); break;
    case CRC_AKAMATCHING     : if(group_active) handle_akamatching     (); break;
    case CRC_AREACOPYADDID   : if(group_active) handle_areacopyaddid   (); break;
    case CRC_AREACOPYDIRECT  : if(group_active) handle_areacopydirect  (); break;
    case CRC_AREACOPYTO      : if(group_active) handle_areacopyto      (); break;
    case CRC_AREAFREQDIRECT  : if(group_active) handle_areafreqdirect  (); break;
    case CRC_AREAFREQTO      : if(group_active) handle_areafreqto      (); break;
    case CRC_AREAREPLYDIRECT : if(group_active) handle_areareplydirect (); break;
    case CRC_AREAREPLYTO     : if(group_active) handle_areareplyto     (); break;
    case CRC_ATTRIBUTES      : if(group_active) handle_attributes      (); break;
    case CRC_EDITHARDTERM    : if(group_active) handle_edithardterm    (); break;
    case CRC_EDITMIXCASE     : if(group_active) handle_editmixcase     (); break;
    case CRC_EDITREPLYRE     : if(group_active) handle_editreplyre     (); break;
    case CRC_FOLLOWUPTO      : if(group_active) handle_followupto      (); break;
    case CRC_FORCETEMPLATE   : if(group_active) handle_forcetemplate   (); break;
    case CRC_GIF             : if(group_active) handle_gif             (); break;
    case CRC_INPUTFILE       : if(group_active) handle_inputfile       (); break;
    case CRC_INTERNETADDRESS : if(group_active) handle_internetaddress (); break;
    case CRC_INTERNETGATE    : if(group_active) handle_internetgate    (); break;
    case CRC_INTERNETMSGID   : if(group_active) handle_internetmsgid   (); break;
    case CRC_INTERNETRFCBODY : if(group_active) handle_internetrfcbody (); break;
    case CRC_LOADLANGUAGE    : if(group_active) handle_loadlanguage    (); break;
    case CRC_MOOD            : if(group_active) handle_mood            (); break;
    case CRC_MSGLISTDATE     : if(group_active) handle_msglistdate     (); break;
    case CRC_MSGLISTFAST     : if(group_active) handle_msglistfast     (); break;
    case CRC_MSGLISTFIRST    : if(group_active) handle_msglistfirst    (); break;
    case CRC_MSGLISTHEADER   : if(group_active) handle_msglistheader   (); break;
    case CRC_MSGLISTWIDESUBJ : if(group_active) handle_msglistwidesubj (); break;
    case CRC_NETNAME         : if(group_active) handle_netname         (); break;
    case CRC_NEWSGROUPS      : if(group_active) handle_newsgroups      (); break;
    case CRC_ORGANIZATION    : if(group_active) handle_organization    (); break;
    case CRC_ORIGIN          : if(group_active) handle_origin          (); break;
    case CRC_OUTPUTFILE      : if(group_active) handle_outputfile      (); break;
    case CRC_PLAY            : if(group_active) handle_play            (); break;
    case CRC_QUOTEBUFFILE    : if(group_active) handle_quotebuffile    (); break;
    case CRC_QUOTECHARS      : if(group_active) handle_quotechars      (); break;
    case CRC_QUOTESTRING     : if(group_active) handle_quotestring     (); break;
    case CRC_QUOTEWRAPHARD   : if(group_active) handle_quotewraphard   (); break;
    case CRC_REPLYTO         : if(group_active) handle_replyto         (); break;
    case CRC_SEARCHFOR       : if(group_active) handle_searchfor       (); break;
    case CRC_SENDER          : if(group_active) handle_sender          (); break;
    case CRC_SIGNATURE       : if(group_active) handle_signature       (); break;
    case CRC_TAGLINE         : if(group_active) handle_tagline         (); break;
    case CRC_TAGLINECHAR     : if(group_active) handle_taglinechar     (); break;
    case CRC_TAGLINEFILE     : if(group_active) handle_taglinefile     (); break;
    case CRC_TAGLINESUPPORT  : if(group_active) handle_taglinesupport  (); break;
    case CRC_TEARLINE        : if(group_active) handle_tearline        (); break;
    case CRC_TEMPLATE        : if(group_active) handle_template        (); break;
    case CRC_TEMPLATEMATCH   : if(group_active) handle_templatematch   (); break;
    case CRC_TWITMODE        : if(group_active) handle_twitmode        (); break;
    case CRC_USEFWD          : if(group_active) handle_usefwd          (); break;
    case CRC_USERNAME        : if(group_active) handle_username        (); break;
    case CRC_VIEWHIDDEN      : if(group_active) handle_viewhidden      (); break;
    case CRC_VIEWKLUDGE      : if(group_active) handle_viewkludge      (); break;
    case CRC_VIEWQUOTE       : if(group_active) handle_viewquote       (); break;
    case CRC_WHOTO           : if(group_active) handle_whoto           (); break;
    case CRC_XLATEXPORT      : if(group_active) handle_xlatexport      (); break;
    case CRC_XLATIMPORT      : if(group_active) handle_xlatimport      (); break;

    case CRC_BASEON          : if(group_active) handle_baseon          (); break;
    case CRC_MEMBER          : if(group_active) handle_member          (); break;
    case CRC_ENDGROUP        : if(group_active) handle_endgroup        (); break;

    default                  : error = error_unknown_keyword;
  }

  return error != error_unknown_keyword;
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_a() {

  switch(crc) {
    case CRC_ADDRESS         :  if(not group_active) handle_address         ();  break;
    case CRC_ADDRESSMACRO    :  if(not group_active) handle_addressmacro    ();  break;
    case CRC_ADEPTXBBSPATH   :  if(not group_active) handle_adeptxbbspath   ();  break;
    case CRC_ADEPTXBBSUSERNO :  if(not group_active) handle_adeptxbbsuserno ();  break;
    case CRC_AKA             :  if(not group_active) handle_address         ();  break;
    case CRC_AKAMATCH        :  if(not group_active) handle_akamatch        ();  break;
    case CRC_AKAMATCHECHO    :  if(not group_active) handle_akamatchecho    ();  break;
    case CRC_AKAMATCHLOCAL   :  if(not group_active) handle_akamatchlocal   ();  break;
    case CRC_AKAMATCHNET     :  if(not group_active) handle_akamatchnet     ();  break;
    case CRC_AREA            :  if(not group_active) handle_area            ();  break;
    case CRC_AREAAUTOID      :  if(not group_active) handle_areaautoid      ();  break;
    case CRC_AREAAUTONEXT    :  if(not group_active) handle_areaautonext    ();  break;
    case CRC_AREACOPYADDID   :  if(not group_active) handle_areacopyaddid   ();  break;
    case CRC_AREACOPYDIRECT  :  if(not group_active) handle_areacopydirect  ();  break;
    case CRC_AREACOPYTO      :  if(not group_active) handle_areacopyto      ();  break;
    case CRC_AREADEF         :  if(not group_active) handle_areadef         ();  break;
    case CRC_AREADESC        :  if(not group_active) handle_areadesc        ();  break;
    case CRC_AREAEXCL        :  if(not group_active) handle_areaexcl        ();  break;
    case CRC_AREAFILEGROUPS  :  if(not group_active) handle_areafilegroups  ();  break;
    case CRC_AREAFREQDIRECT  :  if(not group_active) handle_areafreqdirect  ();  break;
    case CRC_AREAFREQTO      :  if(not group_active) handle_areafreqto      ();  break;
    case CRC_AREAINCL        :  if(not group_active) handle_areaincl        ();  break;
    case CRC_AREAISEMAIL     :  if(not group_active) handle_areaisemail     ();  break;
    case CRC_AREAISNEWS      :  if(not group_active) handle_areaisnews      ();  break;
    case CRC_AREAKEEPLAST    :  if(not group_active) handle_areakeeplast    ();  break;
    case CRC_AREALISTECHOMAX :  if(not group_active) handle_arealistechomax ();  break;
    case CRC_AREALISTFORMAT  :  if(not group_active) handle_arealistformat  ();  break;
    case CRC_AREALISTGROUPID :  if(not group_active) handle_arealistgroupid ();  break;
    case CRC_AREALISTNOS     :  if(not group_active) handle_arealistnos     ();  break;
    case CRC_AREALISTPAGEBAR :  if(not group_active) handle_arealistpagebar ();  break;
    case CRC_AREALISTSCAN    :  if(not group_active) handle_arealistscan    ();  break;
    case CRC_AREALISTSORT    :  if(not group_active) handle_arealistsort    ();  break;
    case CRC_AREALISTTYPE    :  if(not group_active) handle_arealisttype    ();  break;
    case CRC_AREAPATH        :  if(not group_active) handle_areapath        ();  break;
    case CRC_AREAPMSCAN      :  if(not group_active) handle_areapmscan      ();  break;
    case CRC_AREAPMSCANEXCL  :  if(not group_active) handle_areapmscanexcl  ();  break;
    case CRC_AREAPMSCANINCL  :  if(not group_active) handle_areapmscanincl  ();  break;
    case CRC_AREAREADONLY    :  if(not group_active) handle_areareadonly    ();  break;
    case CRC_AREARENAME      :  if(not group_active) handle_arearename      ();  break;
    case CRC_AREAREPLYDIRECT :  if(not group_active) handle_areareplydirect ();  break;
    case CRC_AREAREPLYTO     :  if(not group_active) handle_areareplyto     ();  break;
    case CRC_AREASCAN        :  if(not group_active) handle_areascan        ();  break;
    case CRC_AREASCANEXCL    :  if(not group_active) handle_areascanexcl    ();  break;
    case CRC_AREASCANINCL    :  if(not group_active) handle_areascanincl    ();  break;
    case CRC_AREASCANSORT    :  if(not group_active) handle_areascansort    ();  break;
    case CRC_AREASEP         :  if(not group_active) handle_areasep         ();  break;
    case CRC_AREASTART       :  if(not group_active) handle_areastart       ();  break;
    case CRC_AREATYPEORDER   :  if(not group_active) handle_areatypeorder   ();  break;
    case CRC_ASKDELORIG      :  if(not group_active) handle_askdelorig      ();  break;
    case CRC_ATTRIBSATTACH   :  if(not group_active) handle_attribsattach   ();  break;
    case CRC_ATTRIBSCC       :  if(not group_active) handle_attribscc       ();  break;
    case CRC_ATTRIBSCFM      :  if(not group_active) handle_attribscfm      ();  break;
    case CRC_ATTRIBSECHO     :  if(not group_active) handle_attribsecho     ();  break;
    case CRC_ATTRIBSEMAIL    :  if(not group_active) handle_attribsemail    ();  break;
    case CRC_ATTRIBSFRQ      :  if(not group_active) handle_attribsfrq      ();  break;
    case CRC_ATTRIBSLOCAL    :  if(not group_active) handle_attribslocal    ();  break;
    case CRC_ATTRIBSNET      :  if(not group_active) handle_attribsnet      ();  break;
    case CRC_ATTRIBSNEWS     :  if(not group_active) handle_attribsnews     ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_b() {

  switch(crc) {
    case CRC_BEEPCOMMENT     :  if(not group_active) handle_beepcomment     ();  break;
    case CRC_BEEPFACTOR      :  if(not group_active) handle_beepfactor      ();  break;
    case CRC_BEEPLOCALMSG    :  if(not group_active) handle_beeplocalmsg    ();  break;
    case CRC_BEEPNOISES      :  if(not group_active) handle_beepnoises      ();  break;
    case CRC_BEEPYOURMAIL    :  if(not group_active) handle_beepyourmail    ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_c() {

  switch(crc) {
    case CRC_CARBONCOPYLIST  :  if(not group_active) handle_carboncopylist  ();  break;
    case CRC_COLOR           :  if(not group_active) handle_color           ();  break;
    case CRC_COLORSET        :  if(not group_active) handle_colorset        ();  break;
    case CRC_COLOUR          :  if(not group_active) handle_color           ();  break;
    case CRC_CONFIRMFILE     :  if(not group_active) handle_confirmfile     ();  break;
    case CRC_CONFIRMRESPONSE :  if(not group_active) handle_confirmresponse ();  break;
    case CRC_COOKIEFILE      :  if(not group_active) handle_cookiefile      ();  break;
    case CRC_COOKIEPATH      :  if(not group_active) handle_cookiepath      ();  break;
    case CRC_CROSSPOSTLIST   :  if(not group_active) handle_crosspostlist   ();  break;
    case CRC_CTRLINFOECHO    :  if(not group_active) handle_ctrlinfoecho    ();  break;
    case CRC_CTRLINFOEMAIL   :  if(not group_active) handle_ctrlinfoemail   ();  break;
    case CRC_CTRLINFOLOCAL   :  if(not group_active) handle_ctrlinfolocal   ();  break;
    case CRC_CTRLINFONET     :  if(not group_active) handle_ctrlinfonet     ();  break;
    case CRC_CTRLINFONEWS    :  if(not group_active) handle_ctrlinfonews    ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_d() {

  switch(crc) {
    case CRC_DISPAREANO      :  if(not group_active) handle_dispareano      ();  break;
    case CRC_DISPATTACHSIZE  :  if(not group_active) handle_dispattachsize  ();  break;
    case CRC_DISPAUTONEXT    :  if(not group_active) handle_dispautonext    ();  break;
    case CRC_DISPHDRDATESET  :  if(not group_active) handle_disphdrdateset  ();  break;
    case CRC_DISPHDRNAMESET  :  if(not group_active) handle_disphdrnameset  ();  break;
    case CRC_DISPHDRNODESET  :  if(not group_active) handle_disphdrnodeset  ();  break;
    case CRC_DISPLISTCURSOR  :  if(not group_active) handle_displistcursor  ();  break;
    case CRC_DISPLISTWRAP    :  if(not group_active) handle_displistwrap    ();  break;
    case CRC_DISPLOCALHIGH   :  if(not group_active) handle_displocalhigh   ();  break;
    case CRC_DISPMARGIN      :  if(not group_active) handle_dispmargin      ();  break;
    case CRC_DISPMSGSIZE     :  if(not group_active) handle_dispmsgsize     ();  break;
    case CRC_DISPPAGEBAR     :  if(not group_active) handle_disppagebar     ();  break;
    case CRC_DISPREALMSGNO   :  if(not group_active) handle_disprealmsgno   ();  break;
    case CRC_DISPSOFTCR      :  if(not group_active) handle_dispsoftcr      ();  break;
    case CRC_DISPSTATUSLINE  :  if(not group_active) handle_dispstatusline  ();  break;
    case CRC_DISPTABSIZE     :  if(not group_active) handle_disptabsize     ();  break;
    case CRC_DOSPROMPT       :  if(not group_active) handle_dosprompt       ();  break;
    case CRC_DOSSWAP         :  if(not group_active) handle_dosswap         ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_e() {

  switch(crc) {
    case CRC_EDITAUTOATTACH  :  if(not group_active) handle_editautoattach  ();  break;
    case CRC_EDITAUTOSAVE    :  if(not group_active) handle_editautosave    ();  break;
    case CRC_EDITCHANGEDATE  :  if(not group_active) handle_editchangedate  ();  break;
    case CRC_EDITCHARPARA    :  if(not group_active) handle_editcharpara    ();  break;
    case CRC_EDITCHARSPACE   :  if(not group_active) handle_editcharspace   ();  break;
    case CRC_EDITCOMMENT     :  if(not group_active) handle_editcomment     ();  break;
    case CRC_EDITCOMPLETION  :  if(not group_active) handle_editcompletion  ();  break;
    case CRC_EDITCRLFTERM    :  if(not group_active) handle_editcrlfterm    ();  break;
    case CRC_EDITFIELDCLEAR  :  if(not group_active) handle_editfieldclear  ();  break;
    case CRC_EDITHARDLINE    :  if(not group_active) handle_edithardline    ();  break;
    case CRC_EDITHARDLINES   :  if(not group_active) handle_edithardlines   ();  break;
    case CRC_EDITHARDTERM    :  if(not group_active) handle_edithardterm    ();  break;
    case CRC_EDITHDRNAMESET  :  if(not group_active) handle_edithdrnameset  ();  break;
    case CRC_EDITHDRNODESET  :  if(not group_active) handle_edithdrnodeset  ();  break;
    case CRC_EDITHEADERATTRS :  if(not group_active) handle_editheaderattrs ();  break;
    case CRC_EDITHEADERFIRST :  if(not group_active) handle_editheaderfirst ();  break;
    case CRC_EDITINTERNAL    :  if(not group_active) handle_editinternal    ();  break;
    case CRC_EDITMARGIN      :  if(not group_active) handle_editmargin      ();  break;
    case CRC_EDITMENU        :  if(not group_active) handle_editmenu        ();  break;
    case CRC_EDITMIXCASE     :  if(not group_active) handle_editmixcase     ();  break;
    case CRC_EDITMSGSIZE     :  if(not group_active) handle_editmsgsize     ();  break;
    case CRC_EDITOR          :  if(not group_active) handle_editor          ();  break;
    case CRC_EDITORFILE      :  if(not group_active) handle_editorfile      ();  break;
    case CRC_EDITQUOTEMARGIN :  if(not group_active) handle_editquotemargin ();  break;
    case CRC_EDITREPLYRE     :  if(not group_active) handle_editreplyre     ();  break;
    case CRC_EDITSAVEMENU    :  if(not group_active) handle_editsavemenu    ();  break;
    case CRC_EDITSAVEUTIL    :  if(not group_active) handle_editsaveutil    ();  break;
    case CRC_EDITSOFTCRXLAT  :  if(not group_active) handle_editsoftcrxlat  ();  break;
    case CRC_EDITSPELLCHECK  :  if(not group_active) handle_editspellcheck  ();  break;
    case CRC_EDITUNDELETE    :  if(not group_active) handle_editundelete    ();  break;
    case CRC_EMPTYTEARLINE   :  if(not group_active) handle_emptytearline   ();  break;
    case CRC_EVENT           :  if(not group_active) handle_event           ();  break;
    case CRC_EXCLUDENODES    :  if(not group_active) handle_excludenodes    ();  break;
    case CRC_EXTERNOPTIONS   :  if(not group_active) handle_externoptions   ();  break;
    case CRC_EXTERNUTIL      :  if(not group_active) handle_externutil      ();  break;
    case CRC_EZYCOMMSGBASE   :  if(not group_active) handle_ezycommsgbase   ();  break;
    case CRC_EZYCOMUSERBASE  :  if(not group_active) handle_ezycomuserbase  ();  break;
    case CRC_EZYCOMUSERNO    :  if(not group_active) handle_ezycomuserno    ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_f() {

  switch(crc) {
    case CRC_FIDOHWMARKS     :  if(not group_active) handle_fidohwmarks     ();  break;
    case CRC_FIDOLASTREAD    :  if(not group_active) handle_fidolastread    ();  break;
    case CRC_FIDOMSGTYPE     :  if(not group_active) handle_fidomsgtype     ();  break;
    case CRC_FIDONULLFIX     :  if(not group_active) handle_fidonullfix     ();  break;
    case CRC_FIDOUSERLIST    :  if(not group_active) handle_fidouserlist    ();  break;
    case CRC_FIDOUSERNO      :  if(not group_active) handle_fidouserno      ();  break;
    case CRC_FILEALIAS       :  if(not group_active) handle_filealias       ();  break;
    case CRC_FILECHECK       :  if(not group_active) handle_filecheck       ();  break;
    case CRC_FILECHECKCALL   :  if(not group_active) handle_filecheckcall   ();  break;
    case CRC_FILELISTPAGEBAR :  if(not group_active) handle_filelistpagebar ();  break;
    case CRC_FORCETEMPLATE   :  if(not group_active) handle_forcetemplate   ();  break;
    case CRC_FRQEXT          :  if(not group_active) handle_frqext          ();  break;
    case CRC_FRQNODEMAP      :  if(not group_active) handle_frqnodemap      ();  break;
    case CRC_FRQOPTIONS      :  if(not group_active) handle_frqoptions      ();  break;
    case CRC_FRQWAZOO        :  if(not group_active) handle_frqwazoo        ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_g() {

  switch(crc) {
    case CRC_GIF             :  if(not group_active) handle_gif             ();  break;
    case CRC_GIFPATH         :  if(not group_active) handle_gifpath         ();  break;
    case CRC_GOLDBASEPATH    :  if(not group_active) handle_goldbasepath    ();  break;
    case CRC_GOLDBASESYSPATH :  if(not group_active) handle_goldbasesyspath ();  break;
    case CRC_GOLDBASEUSERNO  :  if(not group_active) handle_goldbaseuserno  ();  break;
    case CRC_GOLDPATH        :  if(not group_active) handle_goldpath        ();  break;
    case CRC_GROUP           :  if(not group_active) handle_group           ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_h() {

  switch(crc) {
    case CRC_HAPPYBIRTHDAY   :  if(not group_active) handle_happybirthday   ();  break;
    case CRC_HIGHLIGHTUNREAD :  if(not group_active) handle_highlightunread ();  break;
    case CRC_HUDSONPATH      :  if(not group_active) handle_hudsonpath      ();  break;
    case CRC_HUDSONSIZEWARN  :  if(not group_active) handle_hudsonsizewarn  ();  break;
    case CRC_HUDSONSYSPATH   :  if(not group_active) handle_hudsonsyspath   ();  break;
    case CRC_HUDSONUSERNO    :  if(not group_active) handle_hudsonuserno    ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_i() {

  switch(crc) {
    case CRC_IMPORTBEGIN     :  if(not group_active) handle_importbegin     ();  break;
    case CRC_IMPORTEND       :  if(not group_active) handle_importend       ();  break;
    case CRC_INBOUNDPATH     :  if(not group_active) handle_inboundpath     ();  break;
    case CRC_INCLUDENODES    :  if(not group_active) handle_includenodes    ();  break;
    case CRC_INPUTFILE       :  if(not group_active) handle_inputfile       ();  break;
    case CRC_INTENSECOLORS   :  if(not group_active) handle_intensecolors   ();  break;
    case CRC_INTERNETADDRESS :  if(not group_active) handle_internetaddress ();  break;
    case CRC_INTERNETGATE    :  if(not group_active) handle_internetgate    ();  break;
    case CRC_INTERNETLOOKUP  :  if(not group_active) handle_internetlookup  ();  break;
    case CRC_INTERNETMSGID   :  if(not group_active) handle_internetmsgid   ();  break;
    case CRC_INTERNETREPLY   :  if(not group_active) handle_internetreply   ();  break;
    case CRC_INTERNETRFCBODY :  if(not group_active) handle_internetrfcbody ();  break;
    case CRC_INVALIDATE      :  if(not group_active) handle_invalidate      ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_j() {

  switch(crc) {
    case CRC_JAMHARDDELETE   :  if(not group_active) handle_jamharddelete   ();  break;
    case CRC_JAMPATH         :  if(not group_active) handle_jampath         ();  break;
    case CRC_JAMSKIPDELETED  :  if(not group_active) handle_jamskipdeleted  ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_k() {

  switch(crc) {
    case CRC_KEYBCLEAR       :  if(not group_active) handle_keybclear       ();  break;
    case CRC_KEYBDEF         :  if(not group_active) handle_keybdef         ();  break;
    case CRC_KEYBDEFAULTS    :  if(not group_active) handle_keybdefaults    ();  break;
    case CRC_KEYBEXT         :  if(not group_active) handle_keybext         ();  break;
    case CRC_KEYBMODE        :  if(not group_active) handle_keybmode        ();  break;
    case CRC_KEYBSTACK       :  if(not group_active) handle_keybstack       ();  break;
    case CRC_KLUDGE          :  if(not group_active) handle_kludge          ();  break;
    case CRC_KLUDGECHRS      :  if(not group_active) handle_kludgechrs      ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_l() {

  switch(crc) {
    case CRC_LANGDEF         :  if(not group_active) handle_langdef         ();  break;
    case CRC_LOADLANGUAGE    :  if(not group_active) handle_loadlanguage    ();  break;
    case CRC_LOGFILE         :  if(not group_active) handle_logfile         ();  break;
    case CRC_LOGFORMAT       :  if(not group_active) handle_logformat       ();  break;
    case CRC_LOOKUPECHO      :  if(not group_active) handle_lookupecho      ();  break;
    case CRC_LOOKUPLOCAL     :  if(not group_active) handle_lookuplocal     ();  break;
    case CRC_LOOKUPNET       :  if(not group_active) handle_lookupnet       ();  break;
    case CRC_LOOKUPUSERBASE  :  if(not group_active) handle_lookupuserbase  ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_m() {

  switch(crc) {
    case CRC_MAILINGLIST     :  if(not group_active) handle_mailinglist     ();  break;
    case CRC_MAPDRIVE        :  if(not group_active) handle_mapdrive        ();  break;
    case CRC_MENUDROPMSG     :  if(not group_active) handle_menudropmsg     ();  break;
    case CRC_MENUMARKED      :  if(not group_active) handle_menumarked      ();  break;
    case CRC_MOOD            :  if(not group_active) handle_mood            ();  break;
    case CRC_MOUSE           :  if(not group_active) handle_mouse           ();  break;
    case CRC_MSGLISTDATE     :  if(not group_active) handle_msglistdate     ();  break;
    case CRC_MSGLISTFAST     :  if(not group_active) handle_msglistfast     ();  break;
    case CRC_MSGLISTFIRST    :  if(not group_active) handle_msglistfirst    ();  break;
    case CRC_MSGLISTHEADER   :  if(not group_active) handle_msglistheader   ();  break;
    case CRC_MSGLISTPAGEBAR  :  if(not group_active) handle_msglistpagebar  ();  break;
    case CRC_MSGLISTVIEWSUBJ :  if(not group_active) handle_msglistviewsubj ();  break;
    case CRC_MSGLISTWIDESUBJ :  if(not group_active) handle_msglistwidesubj ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_n() {

  switch(crc) {
    case CRC_NAMESFILE       :  if(not group_active) handle_namesfile       ();  break;
    case CRC_NETNAME         :  if(not group_active) handle_netname         ();  break;
    case CRC_NODELIST        :  if(not group_active) handle_nodelist        ();  break;
    case CRC_NODELISTPAGEBAR :  if(not group_active) handle_nodelistpagebar ();  break;
    case CRC_NODELISTWARN    :  if(not group_active) handle_nodelistwarn    ();  break;
    case CRC_NODEPATH        :  if(not group_active) handle_nodepath        ();  break;
    case CRC_NODEPATHFD      :  if(not group_active) handle_nodepathfd      ();  break;
    case CRC_NODEPATHIM      :  if(not group_active) handle_nodepathfd      ();  break;
    case CRC_NODEPATHV7      :  if(not group_active) handle_nodepathv7      ();  break;
    case CRC_NODEV7FLAGS     :  if(not group_active) handle_nodev7flags     ();  break;
    case CRC_NODEV7MODEM     :  if(not group_active) handle_nodev7modem     ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_o() {

  switch(crc) {
    case CRC_ORGANIZATION    :  if(not group_active) handle_organization    ();  break;
    case CRC_ORIGIN          :  if(not group_active) handle_origin          ();  break;
    case CRC_OUTBOUNDPATH    :  if(not group_active) handle_outboundpath    ();  break;
    case CRC_OUTPUTFILE      :  if(not group_active) handle_outputfile      ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_p() {

  switch(crc) {
    case CRC_PCBOARDMAILER   :  if(not group_active) handle_pcboardmailer   ();  break;
    case CRC_PCBOARDPATH     :  if(not group_active) handle_pcboardpath     ();  break;
    case CRC_PCBOARDUSERNO   :  if(not group_active) handle_pcboarduserno   ();  break;
    case CRC_PERSONALMAIL    :  if(not group_active) handle_personalmail    ();  break;
    case CRC_PLAY            :  if(not group_active) handle_play            ();  break;
    case CRC_PRINTDEVICE     :  if(not group_active) handle_printdevice     ();  break;
    case CRC_PRINTFORMFEED   :  if(not group_active) handle_printformfeed   ();  break;
    case CRC_PRINTINIT       :  if(not group_active) handle_printinit       ();  break;
    case CRC_PRINTLENGTH     :  if(not group_active) handle_printlength     ();  break;
    case CRC_PRINTMARGIN     :  if(not group_active) handle_printmargin     ();  break;
    case CRC_PRINTRESET      :  if(not group_active) handle_printreset      ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_q() {

  switch(crc) {
    case CRC_QUOTEBLANK      :  if(not group_active) handle_quoteblank      ();  break;
    case CRC_QUOTEBUFFILE    :  if(not group_active) handle_quotebuffile    ();  break;
    case CRC_QUOTEBUFMODE    :  if(not group_active) handle_quotebufmode    ();  break;
    case CRC_QUOTECHARS      :  if(not group_active) handle_quotechars      ();  break;
    case CRC_QUOTEMARGIN     :  if(not group_active) handle_quotemargin     ();  break;
    case CRC_QUOTESPACING    :  if(not group_active) handle_quotespacing    ();  break;
    case CRC_QUOTESTRING     :  if(not group_active) handle_quotestring     ();  break;
    case CRC_QUOTEWRAPHARD   :  if(not group_active) handle_quotewraphard   ();  break;
    case CRC_QWKBADMSGS      :  if(not group_active) handle_qwkbadmsgs      ();  break;
    case CRC_QWKCONFMAP      :  if(not group_active) handle_qwkconfmap      ();  break;
    case CRC_QWKEXPORTPATH   :  if(not group_active) handle_qwkexportpath   ();  break;
    case CRC_QWKIMPORTPATH   :  if(not group_active) handle_qwkimportpath   ();  break;
    case CRC_QWKOPTIONS      :  if(not group_active) handle_qwkoptions      ();  break;
    case CRC_QWKREPLYLINKER  :  if(not group_active) handle_qwkreplylinker  ();  break;
    case CRC_QWKTOSSLOG      :  if(not group_active) handle_qwktosslog      ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_r() {

  switch(crc) {
    case CRC_RA2USERSBBS     :  if(not group_active) handle_ra2usersbbs     ();  break;
    case CRC_REGISTERKEY     :                                                   break;
    case CRC_REGISTERNAME    :                                                   break;
    case CRC_REPLYLINK       :  if(not group_active) handle_replylink       ();  break;
    case CRC_REPLYLINKLIST   :  if(not group_active) handle_replylinklist   ();  break;
    case CRC_REPLYTO         :  if(not group_active) handle_replyto         ();  break;
    case CRC_ROBOTNAME       :  if(not group_active) handle_robotname       ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_s() {

  switch(crc) {
    case CRC_SCREENBLANKER   :  if(not group_active) handle_screenblanker   ();  break;
    case CRC_SCREENELIMSNOW  :  if(not group_active) handle_screenelimsnow  ();  break;
    case CRC_SCREENMAXCOL    :  if(not group_active) handle_screenmaxcol    ();  break;
    case CRC_SCREENMAXROW    :  if(not group_active) handle_screenmaxrow    ();  break;
    case CRC_SCREENPALETTE   :  if(not group_active) handle_screenpalette   ();  break;
    case CRC_SCREENSHADOWS   :  if(not group_active) handle_screenshadows   ();  break;
    case CRC_SCREENSIZE      :  if(not group_active) handle_screensize      ();  break;
    case CRC_SCREENUSEBIOS   :  if(not group_active) handle_screenusebios   ();  break;
    case CRC_SEARCHFOR       :  if(not group_active) handle_searchfor       ();  break;
    case CRC_SEMAPHORE       :  if(not group_active) handle_semaphore       ();  break;
    case CRC_SENDER          :  if(not group_active) handle_sender          ();  break;
    case CRC_SERIALNO        :  if(not group_active) handle_serialno        ();  break;
    case CRC_SIGNATURE       :  if(not group_active) handle_signature       ();  break;
    case CRC_SOUNDPATH       :  if(not group_active) handle_soundpath       ();  break;
    case CRC_SOUPBADMSGS     :  if(not group_active) handle_soupbadmsgs     ();  break;
    case CRC_SOUPEMAIL       :  if(not group_active) handle_soupemail       ();  break;
    case CRC_SOUPEXPORTPATH  :  if(not group_active) handle_soupexportpath  ();  break;
    case CRC_SOUPIMPORTPATH  :  if(not group_active) handle_soupimportpath  ();  break;
    case CRC_SOUPNEWSRCFILE  :  if(not group_active) handle_soupnewsrcfile  ();  break;
    case CRC_SOUPREPLYLINKER :  if(not group_active) handle_soupreplylinker ();  break;
    case CRC_SOUPTOSSLOG     :  if(not group_active) handle_souptosslog     ();  break;
    case CRC_SQUISHDIRECT    :  if(not group_active) handle_squishdirect    ();  break;
    case CRC_SQUISHSCAN      :  if(not group_active) handle_squishscan      ();  break;
    case CRC_SQUISHUSERNO    :  if(not group_active) handle_squishuserno    ();  break;
    case CRC_SQUISHUSERPATH  :  if(not group_active) handle_squishuserpath  ();  break;
    case CRC_STATUSLINECLOCK :  if(not group_active) handle_statuslineclock ();  break;
    case CRC_STATUSLINEHELP  :  if(not group_active) handle_statuslinehelp  ();  break;
    case CRC_STYLECODES      :  if(not group_active) handle_stylecodes      ();  break;
    case CRC_STYLECODEPUNCT  :  if(not group_active) handle_stylecodepunct  ();  break;
    case CRC_STYLECODESTOPS  :  if(not group_active) handle_stylecodestops  ();  break;
    case CRC_SWAPPATH        :  if(not group_active) handle_swappath        ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_t() {

  switch(crc) {
    case CRC_TAGLINE         :  if(not group_active) handle_tagline         ();  break;
    case CRC_TAGLINECHAR     :  if(not group_active) handle_taglinechar     ();  break;
    case CRC_TAGLINEFILE     :  if(not group_active) handle_taglinefile     ();  break;
    case CRC_TAGLINESUPPORT  :  if(not group_active) handle_taglinesupport  ();  break;
    case CRC_TEARLINE        :  if(not group_active) handle_tearline        ();  break;
    case CRC_TEMPLATE        :  if(not group_active) handle_template        ();  break;
    case CRC_TEMPLATEMATCH   :  if(not group_active) handle_templatematch   ();  break;
    case CRC_TEMPLATEPATH    :  if(not group_active) handle_templatepath    ();  break;
    case CRC_TEMPPATH        :  if(not group_active) handle_temppath        ();  break;
    case CRC_TIMEOUT         :  if(not group_active) handle_timeout         ();  break;
    case CRC_TIMEOUTSAVEMSG  :  if(not group_active) handle_timeoutsavemsg  ();  break;
    case CRC_TIMEZONEOFFSET  :  if(not group_active) handle_timezoneoffset  ();  break;
    case CRC_TWITMODE        :  if(not group_active) handle_twitmode        ();  break;
    case CRC_TWITNAME        :  if(not group_active) handle_twitname        ();  break;
    case CRC_TWITSUBJ        :  if(not group_active) handle_twitsubj        ();  break;
    case CRC_TWITTO          :  if(not group_active) handle_twitto          ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_u() {

  switch(crc) {
    case CRC_USEFWD          :  if(not group_active) handle_usefwd          ();  break;
    case CRC_USERLIST        :  if(not group_active) handle_userlist        ();  break;
    case CRC_USERLISTFILE    :  if(not group_active) handle_userlistfile    ();  break;
    case CRC_USERNAME        :  if(not group_active) handle_username        ();  break;
    case CRC_UUDECODEPATH    :  if(not group_active) handle_uudecodepath    ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_v() {

  switch(crc) {
    case CRC_VIEWHIDDEN      :  if(not group_active) handle_viewhidden      ();  break;
    case CRC_VIEWKLUDGE      :  if(not group_active) handle_viewkludge      ();  break;
    case CRC_VIEWQUOTE       :  if(not group_active) handle_viewquote       ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_w() {

  switch(crc) {
    case CRC_WHOTO           :  if(not group_active) handle_whoto           ();  break;
    case CRC_WILDCATUSERNO   :  if(not group_active) handle_wildcatuserno   ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_x() {

  switch(crc) {
    case CRC_XLATCHARSET     :  if(not group_active) handle_xlatcharset     ();  break;
    case CRC_XLATESCSET      :  if(not group_active) handle_xlatescset      ();  break;
    case CRC_XLATEXPORT      :  if(not group_active) handle_xlatexport      ();  break;
    case CRC_XLATIMPORT      :  if(not group_active) handle_xlatimport      ();  break;
    case CRC_XLATLOCALSET    :  if(not group_active) handle_xlatlocalset    ();  break;
    case CRC_XLATPATH        :  if(not group_active) handle_xlatpath        ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_y() {

  switch(crc) {
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keywords_z() {

  switch(crc) {
    case CRC_ZONEGATING      :  if(not group_active) handle_zonegating      ();  break;
    default                  :  error = error_unknown_keyword;
  }
}


//  ------------------------------------------------------------------

bool golded_cfg::switch_obsolete_keywords() {

  bool allowed = handle_obsolete and not group_active;

  error = no_error;

  switch(crc) {
    case CRC_AREAAUTOFREQ    :  if(allowed) handle_areafreqto      ();  break;
    case CRC_AREABADMSGS     :  if(allowed) handle_soupbadmsgs     ();  break;
    case CRC_AREASORT        :  if(allowed) handle_arealistsort    ();  break;
    case CRC_ASSIGNTO        :  if(allowed) handle_member          ();  break;
    case CRC_AUTOATTACH      :  if(allowed) handle_editautoattach  ();  break;
    case CRC_BLANKTIME       :  if(allowed) handle_screenblanker   ();  break;
    case CRC_BOARDNOS        :  if(allowed) handle_arealistnos     ();  break;
    case CRC_CCATTRIB        :  if(allowed) handle_attribscc       ();  break;
    case CRC_CCLIST          :  if(allowed) handle_carboncopylist  ();  break;
    case CRC_CFMATTRIB       :  if(allowed) handle_attribscfm      ();  break;
    case CRC_CHANGEDATE      :  if(allowed) handle_editchangedate  ();  break;
    case CRC_CHANGEPROMPT    :  if(allowed) handle_dosprompt       ();  break;
    case CRC_CHARSET         :  if(allowed) handle_xlatcharset     ();  break;
    case CRC_CLEARKEYS       :  if(allowed) handle_keybclear       ();  break;
    case CRC_COMMENTNOISE    :  if(allowed) handle_beepcomment     ();  break;
    case CRC_CRLFTERM        :  if(allowed) handle_editcrlfterm    ();  break;
    case CRC_DELORIG         :  if(allowed) handle_askdelorig      ();  break;
    case CRC_DISPMSGLIST     :  if(allowed) handle_msglistfirst    ();  break;
    case CRC_DISPMSGLISTFAST :  if(allowed) handle_msglistfast     ();  break;
    case CRC_DISPSTYLECODES  :  if(allowed) handle_stylecodes      ();  break;
    case CRC_ECHOATTRIB      :  if(allowed) handle_attribsecho     ();  break;
    case CRC_ECHOINFO        :  if(allowed) handle_ctrlinfoecho    ();  break;
    case CRC_EDITORVERSION   :  if(allowed) handle_editorversion   ();  break;
    case CRC_ELIMSNOW        :  if(allowed) handle_screenelimsnow  ();  break;
    case CRC_ESCSET          :  if(allowed) handle_xlatescset      ();  break;
    case CRC_EXCLAREA        :  if(allowed) handle_areaexcl        ();  break;
    case CRC_EXPORTCHARSET   :  if(allowed) handle_xlatexport      ();  break;
    case CRC_EXTKEYS         :  if(allowed) handle_keybext         ();  break;
    case CRC_FIDOLASTREADNO  :  if(allowed) handle_fidouserno      ();  break;
    case CRC_FIELDCLEAR      :  if(allowed) handle_editfieldclear  ();  break;
    case CRC_FREEAREA        :  if(allowed) handle_freearea        ();  break;
    case CRC_FREETEAR        :  if(allowed) handle_freearea        ();  break;
    case CRC_GOLDEDCFM       :  if(allowed) handle_confirmfile     ();  break;
    case CRC_GOLDEDLOG       :  if(allowed) handle_logfile         ();  break;
    case CRC_GOLDEDLST       :  if(allowed) handle_userlistfile    ();  break;
    case CRC_GOLDEDMSG       :  if(allowed) handle_editorfile      ();  break;
    case CRC_GOLDEDNAM       :  if(allowed) handle_namesfile       ();  break;
    case CRC_GOLDEDPRN       :  if(allowed) handle_outputfile      ();  break;
    case CRC_GOLDHELP        :  if(allowed) handle_goldhelp        ();  break;
    case CRC_GOLDKEYS        :  if(allowed) handle_goldkeys        ();  break;
    case CRC_GOLDLANG        :  if(allowed) handle_goldlang        ();  break;
    case CRC_GOLDRAND        :  if(allowed) handle_goldrand        ();  break;
    case CRC_GOLDXLAT        :  if(allowed) handle_goldxlat        ();  break;
    case CRC_HARDLINE        :  if(allowed) handle_edithardline    ();  break;
    case CRC_HARDLINES       :  if(allowed) handle_edithardlines   ();  break;
    case CRC_HWMARKS         :  if(allowed) handle_fidohwmarks     ();  break;
    case CRC_INCLAREA        :  if(allowed) handle_areaincl        ();  break;
    case CRC_INTERNALEDITOR  :  if(allowed) handle_editinternal    ();  break;
    case CRC_LASTREAD        :  if(allowed) handle_fidolastread    ();  break;
    case CRC_LASTREADUSER    :  if(allowed) handle_fidouserno      ();  break;
    case CRC_LISTWRAP        :  if(allowed) handle_displistwrap    ();  break;
    case CRC_LOCALATTRIB     :  if(allowed) handle_attribslocal    ();  break;
    case CRC_LOCALCHARSET    :  if(allowed) handle_xlatimport      ();  break;
    case CRC_LOCALHIGHLIGHT  :  if(allowed) handle_displocalhigh   ();  break;
    case CRC_LOCALINFO       :  if(allowed) handle_ctrlinfolocal   ();  break;
    case CRC_LOCALNOISE      :  if(allowed) handle_beeplocalmsg    ();  break;
    case CRC_MATCHAKA        :  if(allowed) handle_akamatchnet     ();  break;
    case CRC_MAXCOLS         :  if(allowed) handle_screenmaxcol    ();  break;
    case CRC_MAXMSGSIZE      :  if(allowed) handle_editmsgsize     ();  break;
    case CRC_MAXROWS         :  if(allowed) handle_screenmaxrow    ();  break;
    case CRC_MIXCASE         :  if(allowed) handle_editmixcase     ();  break;
    case CRC_MULTIQBBS       :  if(allowed) handle_qbbsincrescan   ();  break;
    case CRC_NETATTRIB       :  if(allowed) handle_attribsnet      ();  break;
    case CRC_NETINFO         :  if(allowed) handle_ctrlinfonet     ();  break;
    case CRC_NETTEAR         :  if(allowed) handle_nettear         ();  break;
    case CRC_NEXTAREA        :  if(allowed) handle_areaautonext    ();  break;
    case CRC_NEXTMSGS        :  if(allowed) handle_dispautonext    ();  break;
    case CRC_NOISEFACTOR     :  if(allowed) handle_beepfactor      ();  break;
    case CRC_OVERLAY         :  if(allowed) handle_overlay         ();  break;
    case CRC_OVERLAYEMS      :  if(allowed) handle_overlay         ();  break;
    case CRC_OVERLAYEXT      :  if(allowed) handle_overlay         ();  break;
    case CRC_PAGEBAR         :  if(allowed) handle_disppagebar     ();  break;
    case CRC_QBBSINCRESCAN   :  if(allowed) handle_qbbsincrescan   ();  break;
    case CRC_QBBSPATH        :  if(allowed) handle_hudsonpath      ();  break;
    case CRC_QBBSREBUILD     :  if(allowed) handle_qbbsrebuild     ();  break;
    case CRC_QBBSSCAN        :  if(allowed) handle_qbbsscan        ();  break;
    case CRC_QMSGPATH        :  if(allowed) handle_hudsonpath      ();  break;
    case CRC_REALMSGNO       :  if(allowed) handle_disprealmsgno   ();  break;
    case CRC_REBUILD         :  if(allowed) handle_qbbsrebuild     ();  break;
    case CRC_RENAREA         :  if(allowed) handle_arearename      ();  break;
    case CRC_REPLYRE         :  if(allowed) handle_editreplyre     ();  break;
    case CRC_RIGHTMARGIN     :  if(allowed) handle_dispmargin      ();  break;
    case CRC_SAVEMENU        :  if(allowed) handle_editsavemenu    ();  break;
    case CRC_SAVETIME        :  if(allowed) handle_editautosave    ();  break;
    case CRC_SAYBIBI         :  if(allowed) handle_beepnoises      ();  break;
    case CRC_SCANAREA        :  if(allowed) handle_areascan        ();  break;
    case CRC_SCREENUSEANSI   :  if(allowed) handle_screenuseansi   ();  break;
    case CRC_SHADOWS         :  if(allowed) handle_screenshadows   ();  break;
    case CRC_SHARE           :  if(allowed) handle_sharemode       ();  break;
    case CRC_SHAREMODE       :  if(allowed) handle_sharemode       ();  break;
    case CRC_SHOWTWITS       :  if(allowed) handle_twitmode        ();  break;
    case CRC_SIGNALFILE      :  if(allowed) handle_semaphore       ();  break;
    case CRC_SOUNDDEVICE     :  if(allowed) handle_sounddevice     ();  break;
    case CRC_SPACEQUOTES     :  if(allowed) handle_quotespacing    ();  break;
    case CRC_SPELLCHECKER    :  if(allowed) handle_editspellcheck  ();  break;
    case CRC_STACKKEYS       :  if(allowed) handle_keybstack       ();  break;
    case CRC_STARTECHO       :  if(allowed) handle_areastart       ();  break;
    case CRC_SWAPALL         :  if(allowed) handle_dosswap         ();  break;
    case CRC_SYSOP           :  if(allowed) handle_username        ();  break;
    case CRC_TABSIZE         :  if(allowed) handle_disptabsize     ();  break;
    case CRC_TIMESLICE       :  if(allowed) handle_timeslice       ();  break;
    case CRC_TIMESREAD       :  if(allowed) handle_highlightunread ();  break;
    case CRC_UNDELETELINES   :  if(allowed) handle_editundelete    ();  break;
    case CRC_USEBIOS         :  if(allowed) handle_screenusebios   ();  break;
    case CRC_USEFLAGS        :  if(allowed) handle_useflags        ();  break;
    case CRC_USEINTL         :  if(allowed) handle_useintl         ();  break;
    case CRC_USEMSGID        :  if(allowed) handle_usemsgid        ();  break;
    case CRC_XPLIST          :  if(allowed) handle_crosspostlist   ();  break;
    default                  :  error = error_unknown_keyword;
  }

  if(not handle_obsolete and (error != error_unknown_keyword))
    error = error_obsolete_keyword;

  return error != error_unknown_keyword;
}


//  ------------------------------------------------------------------

bool golded_cfg::switch_keywords() {

  error = no_error;

  switch(toupper(*key)) {
    case 'A':  switch_keywords_a();  break;
    case 'B':  switch_keywords_b();  break;
    case 'C':  switch_keywords_c();  break;
    case 'D':  switch_keywords_d();  break;
    case 'E':  switch_keywords_e();  break;
    case 'F':  switch_keywords_f();  break;
    case 'G':  switch_keywords_g();  break;
    case 'H':  switch_keywords_h();  break;
    case 'I':  switch_keywords_i();  break;
    case 'J':  switch_keywords_j();  break;
    case 'K':  switch_keywords_k();  break;
    case 'L':  switch_keywords_l();  break;
    case 'M':  switch_keywords_m();  break;
    case 'N':  switch_keywords_n();  break;
    case 'O':  switch_keywords_o();  break;
    case 'P':  switch_keywords_p();  break;
    case 'Q':  switch_keywords_q();  break;
    case 'R':  switch_keywords_r();  break;
    case 'S':  switch_keywords_s();  break;
    case 'T':  switch_keywords_t();  break;
    case 'U':  switch_keywords_u();  break;
    case 'V':  switch_keywords_v();  break;
    case 'W':  switch_keywords_w();  break;
    case 'X':  switch_keywords_x();  break;
    case 'Y':  switch_keywords_y();  break;
    case 'Z':  switch_keywords_z();  break;
    default :  error = error_unknown_keyword;
  }

  return error != error_unknown_keyword;
}


//  ------------------------------------------------------------------

void golded_cfg::switch_keyword() {

  if(group_active) {
    if(not switch_group_keywords())
      if(switch_keywords() or switch_obsolete_keywords())
        error = error_misplaced_global_keyword;
  }
  else {
    if(not switch_keywords() and not switch_obsolete_keywords())
      if(switch_group_keywords())
        error = error_misplaced_group_keyword;
  }
}


//  ------------------------------------------------------------------

