
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gecfgg.h,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  The GoldED configuration.
//  ------------------------------------------------------------------


#include <gstrblks.h>
#include <gvector.h>

#include "gecfgs.h"
#include "gegrps.h"
#include "gekeys.h"
#include "gelang.h"
#include "geplay.h"
#include "geqwks.h"
#include "gexlat.h"
#include "gealst.h"


//  ------------------------------------------------------------------

class golded_cfg {

public:

  //  ----------------------------------------------------------------
  //  Errors

  enum {
    no_error,
    error_group_active,
    error_group_not_active,
    error_obsolete_keyword,
    error_obsolete_has_no_effect,
    error_misplaced_endgroup,
    error_misplaced_group_keyword,
    error_misplaced_global_keyword,
    error_unknown_keyboard_keyword,
    error_unknown_language_keyword,
    error_unknown_keyword,
    error_unknown_option,
    error_unknown_value,
    error_value_out_of_range,
    error_invalid_value,
    error_end
  };

  int error;


  //  ----------------------------------------------------------------
  //  Runtime data - not written to file

  struct {
    int   dispmargin;
    int   editquotemargin;
    int   quotemargin;
  } runtime;


  //  ----------------------------------------------------------------
  //  Internal data - written to file

  struct {
    gstrarray             files_to_check;
    long                  help_offset;
    long                  group_offset;
  } internal;


  //  ----------------------------------------------------------------
  //  Internal status and helpers

  bool handle_obsolete;
  bool group_active;

  word  crc;
  char* key;
  char* val;

  void next_keyval();
  word next_keyvalcrc();

  void set_int(int& a);
  void set_int(int& a, int minval, int maxval);
  void set_long(long& a);
  void set_bool(bool& a);
  void set_file(gstring& a);
  void set_path(gstring& a);
  void set_yesno(int& a);
  void set_string(gstring& a);
  void set_attrib(ftn_attr& a);

  void set_int_table(int& a, const gstrvaltable* table, int tablesize);

  void get_colors();
  bool get_ctrlinfo(int& a);

  void switch_keywords_a();
  void switch_keywords_b();
  void switch_keywords_c();
  void switch_keywords_d();
  void switch_keywords_e();
  void switch_keywords_f();
  void switch_keywords_g();
  void switch_keywords_h();
  void switch_keywords_i();
  void switch_keywords_j();
  void switch_keywords_k();
  void switch_keywords_l();
  void switch_keywords_m();
  void switch_keywords_n();
  void switch_keywords_o();
  void switch_keywords_p();
  void switch_keywords_q();
  void switch_keywords_r();
  void switch_keywords_s();
  void switch_keywords_t();
  void switch_keywords_u();
  void switch_keywords_v();
  void switch_keywords_w();
  void switch_keywords_x();
  void switch_keywords_y();
  void switch_keywords_z();

  bool switch_group_keywords();
  bool switch_obsolete_keywords();
  bool switch_keywords();
  void switch_keyword();


  //  ----------------------------------------------------------------
  //  Configuration data - written to file

  gvector<gftnaddrs>      address;
  gstrarray               addressmacro;
  gvector<uint>           addresspointnet;
  gstring                 adeptxbbspath;
  int                     adeptxbbsuserno;
  gvector<ftn_akamatch>   akamatch;
  bool                    akamatchecho;
  bool                    akamatchlocal;
  bool                    akamatchnet;
  int                     areaautoid;
  bool                    areaautonext;
  bool                    areacopyaddid;
  bool                    areacopydirect;
  gstring                 areacopyto;
  gstrarray               areaexcl;
  bool                    areafilegroups;
  bool                    areafreqdirect;
  gstring                 areafreqto;
  gstrarray               areaincl;
  gstrarray               areaisemail;
  gstrarray               areaisnews;
  bool                    areakeeplast;
  int                     arealistechomax;
  gstring                 arealistformat;
  bool                    arealistgroupid;
  bool                    arealistnos;
  bool                    arealistpagebar;
  gstring                 arealistsort;
  int                     arealisttype;
  gstring                 areapath;
  gstrarray               areapmscan;
  gstrarray               areapmscanexcl;
  gstrarray               areapmscanincl;
  int                     areareadonly;
  gvector<garearename>    arearename;
  bool                    areareplydirect;
  gstring                 areareplyto;
  gstrarray               areascan;
  gstrarray               areascanexcl;
  gstrarray               areascanincl;
  gstring                 areascansort;
  gstring                 areastart;
  int                     areatypeorder[17];
  bool                    askdelorig;
  ftn_attr                attribsattach;
  ftn_attr                attribscc;
  ftn_attr                attribscfm;
  ftn_attr                attribsecho;
  ftn_attr                attribsemail;
  ftn_attr                attribsfrq;
  ftn_attr                attribslocal;
  ftn_attr                attribsnet;
  ftn_attr                attribsnews;

  int                     beepcomment;
  int                     beepfactor;
  int                     beeplocalmsg;
  int                     beepnoises;
  int                     beepyourmail;

  int                     carboncopylist;
  gcolor                  color[16];
  gstring                 confirmfile;
  int                     confirmresponse;
  gstring                 cookiepath;
  int                     crosspostlist;
  uint                    ctrlinfoecho;
  uint                    ctrlinfoemail;
  uint                    ctrlinfolocal;
  uint                    ctrlinfonet;
  uint                    ctrlinfonews;

  int                     dispareano;
  int                     dispattachsize;
  int                     dispautonext;
  gposlen                 disphdrdateset;
  gposlen                 disphdrnameset;
  gposlen                 disphdrnodeset;
  int                     displistcursor;
  int                     displistwrap;
  int                     displocalhigh;
  int                     dispmargin;
  int                     dispmsgsize;
  int                     disppagebar;
  int                     disprealmsgno;
  int                     dispsoftcr;
  int                     dispstatusline;
  int                     disptabsize;
  int                     dosprompt;
  int                     dosswap;

  int                     editautoattach;
  int                     editautosave;
  int                     editchangedate;
  char                    editcharpara;
  char                    editcharspace;
  gvector<gedittrigger>   editcomment;
  gvector<gedittrigger>   editcompletion;
  int                     editcrlfterm;
  gstring                 editexternal;
  int                     editfieldclear;
  gstring                 editfile;
  gstring                 edithardline;
  int                     edithardlines;
  int                     edithardterm;
  int                     edithdrnamelen;
  int                     edithdrnamepos;
  int                     edithdrnodelen;
  int                     edithdrnodepos;
  int                     editheaderattrs;
  int                     editheaderfirst;
  int                     editinternal;
  int                     editmargin;
  int                     editmenu;
  int                     editmixcase;
  long                    editmsgsize;
  int                     editquotemargin;
  int                     editreplyre;
  int                     editsavemenu;
  gvector<geditsaveutil>  editsaveutil;
  char                    editsoftcrxlat;
  gstring                 editspellchecker;
  int                     editundelete;
  int                     emptytearline;
  gvector<gevent>         event;
  int                     externoptions;
  gvector<gexternutil>    externutil;
  gstring                 ezycommsgbasepath;
  gstring                 ezycomuserbasepath;
  int                     ezycomuserno;

  int                     fidohwmarks;
  gstring                 fidolastread;
  int                     fidomsgtype;
  int                     fidonullfix;
  gstring                 fidouserlist;
  int                     fidouserno;
  gvector<gfilealias>     filealias;
  int                     filelistpagebar;
  int                     forcetemplate;
  gstrarray               frqext;
  gvector<gfrqnodemap>    frqnodemap;
  int                     frqoptions;
  int                     frqwazoo;

  gstrarray               gif;
  int                     gifno;
  gstring                 gifpath;
  gstring                 goldbasepath;
  gstring                 goldbasesyspath;
  int                     goldbaseuserno;
  gstring                 goldpath;

  int                     highlightunread;
  gstring                 hudsonpath;
  long                    hudsonsizewarn;
  gstring                 hudsonsyspath;
  int                     hudsonuserno;

  gstring                 importbegin;
  gstring                 importend;
  gstring                 inboundpath;
  gstring                 inputfile;
  int                     intensecolors;
  gstring                 internetaddress;
  gftnnode                internetgate;
  int                     internetlookup;
  int                     internetmsgid;
  int                     internetreply;
  int                     internetrfcbody;
  ginvalidate             invalidate;

  int                     jamharddelete;
  gstring                 jampath;
  bool                    jamskipdeleted;

  int                     keybclear;
  bool                    keybdefaults;
  int                     keybext;
  int                     keybmode;
  gstring                 keybstack;
  gstrarray               kludge;
  int                     kludgechrs;

  gstring                 loadlanguage;
  gstring                 logfile;
  int                     logformat;
  int                     lookupecho;
  int                     lookuplocal;
  int                     lookupnet;
  int                     lookupuserbase;

  gvector<gmailinglist>   mailinglist;
  char                    mapdrive[27];
  int                     menudropmsg;
  int                     menumarked;
  gstrarray               mood;
  int                     moodno;
  int                     mouse;
  int                     msglistdate;
  int                     msglistfast;
  int                     msglistfirst;
  int                     msglistheader;
  int                     msglistpagebar;
  int                     msglistviewsubj;
  int                     msglistwidesubj;

  gstring                 namesfile;
  gstring                 netname;
  int                     nodelistpagebar;
  int                     nodelistwarn;
  gstring                 nodepath;
  gstring                 nodepathfd;
  gstring                 nodepathv7;
  char                    nodev7flags[16][9];
  char                    nodev7modem[8][9];

  gstring                 organization;
  gstrarray               origin;
  int                     originno;
  gstring                 outboundpath;
  gstring                 outputfile;
  int                     overlay;

  int                     pcboardmailer;
  gstring                 pcboardpath;
  int                     pcboarduserno;
  int                     personalmail;
  gplay                   play;
  gstring                 printdevice;
  int                     printformfeed;
  char                    printinit[80];
  int                     printlength;
  int                     printmargin;
  char                    printreset[80];

  int                     quoteblank;
  gstring                 quotebuffile;
  int                     quotebufmode;
  gstring                 quotechars;
  int                     quotemargin;
  int                     quotespacing;
  gstring                 quotestring;
  int                     quotewraphard;

  int                     ra2usersbbs;
  int                     replylink;
  int                     replylinklist;
  gstring                 replyto;
  gstrarray               robotname;

  int                     screenblanker;
  int                     screenblankertype;
  int                     screenelimsnow;
  int                     screenmaxcol;
  int                     screenmaxrow;
  int                     screenpalette[17];
  int                     screenshadows;
  int                     screensize;
  int                     screenuseansi;
  int                     screenusebios;
  gstring                 searchfor;
  gsemaphore              semaphore;
  gstring                 sender;
  int                     sharemode;
  gstring                 signature;
  gstring                 soundpath;
  gstring                 soupbadmsgs;
  gstring                 soupemail;
  gstring                 soupexportpath;
  gstring                 soupimportpath;
  gstring                 soupnewsrcfile;
  gstring                 soupreplylinker;
  gstring                 souptosslog;
  int                     squishdirect;
  int                     squishscan;
  int                     squishuserno;
  gstring                 squishuserpath;
  int                     statuslineclock;
  int                     statuslinehelp;
  gstring                 stylecodepunct;
  int                     stylecodes;
  gstring                 stylecodestops;
  gstring                 swappath;

  gstrarray               tagline;
  char                    taglinechar;
  gstring                 taglinefile;
  int                     taglineno;
  int                     taglinesupport;
  gstring                 tearline;
  int                     templatematch;
  gstring                 templatepath;
  gstring                 temppath;
  int                     timeout;
  int                     timeoutsavemsg;
  int                     timezoneoffset;
  gvector<gtpl>           tpl;
  int                     tplno;
  int                     twitmode;
  gvector<gftnnode>       twitname;
  gstrarray               twitsubj;
  int                     twitto;

  int                     useflags;
  int                     usefwd;
  int                     useintl;
  int                     usemsgid;
  gstring                 userlistfile;
  gvector<gftnnode>       username;
  int                     usernameno;
  gstring                 uudecodepath;

  int                     viewhidden;
  int                     viewkludge;
  int                     viewquote;

  gstring                 whoto;
  int                     wildcatuserno;

  int                     zonegating;


  //  ----------------------------------------------------------------
  //  Special handling

  golded_grp              group;
  golded_kbd              keyboard;
  golded_lng              language;


  //  ----------------------------------------------------------------
  //  Access to external classes

  gqwk*                   qwk;
  garealist*              arealist;
  golded_xlat*            xlat;


  //  ----------------------------------------------------------------
  //  Keyword handlers

  void handle_address();
  void handle_addressmacro();
  void handle_adeptxbbspath();
  void handle_adeptxbbsuserno();
  void handle_akamatch();
  void handle_akamatchecho();
  void handle_akamatching();
  void handle_akamatchlocal();
  void handle_akamatchnet();
  void handle_area();
  void handle_areaautoid();
  void handle_areaautonext();
  void handle_areacopyaddid();
  void handle_areacopydirect();
  void handle_areacopyto();
  void handle_areadef();
  void handle_areadesc();
  void handle_areaexcl();
  void handle_areafilegroups();
  void handle_areafreqdirect();
  void handle_areafreqto();
  void handle_areaincl();
  void handle_areaisemail();
  void handle_areaisnews();
  void handle_areakeeplast();
  void handle_arealistechomax();
  void handle_arealistformat();
  void handle_arealistgroupid();
  void handle_arealistnos();
  void handle_arealistpagebar();
  void handle_arealistscan();
  void handle_arealistsort();
  void handle_arealisttype();
  void handle_areapath();
  void handle_areapmscan();
  void handle_areapmscanexcl();
  void handle_areapmscanincl();
  void handle_areareadonly();
  void handle_arearename();
  void handle_areareplydirect();
  void handle_areareplyto();
  void handle_areascan();
  void handle_areascanexcl();
  void handle_areascanincl();
  void handle_areascansort();
  void handle_areasep();
  void handle_areastart();
  void handle_areatypeorder();
  void handle_askdelorig();
  void handle_attribsattach();
  void handle_attribscc();
  void handle_attribscfm();
  void handle_attribsecho();
  void handle_attribsemail();
  void handle_attribsfrq();
  void handle_attribslocal();
  void handle_attribsnet();
  void handle_attribsnews();
  void handle_attributes();

  void handle_baseon();
  void handle_beepcomment();
  void handle_beepfactor();
  void handle_beeplocalmsg();
  void handle_beepnoises();
  void handle_beepyourmail();

  void handle_carboncopylist();
  void handle_color();
  void handle_colorset();
  void handle_confirmfile();
  void handle_confirmresponse();
  void handle_cookiefile();
  void handle_cookiepath();
  void handle_crosspostlist();
  void handle_ctrlinfo();
  void handle_ctrlinfoecho();
  void handle_ctrlinfoemail();
  void handle_ctrlinfolocal();
  void handle_ctrlinfonet();
  void handle_ctrlinfonews();

  void handle_dispareano();
  void handle_dispattachsize();
  void handle_dispautonext();
  void handle_disphdrdateset();
  void handle_disphdrnameset();
  void handle_disphdrnodeset();
  void handle_displistcursor();
  void handle_displistwrap();
  void handle_displocalhigh();
  void handle_dispmargin();
  void handle_dispmsgsize();
  void handle_disppagebar();
  void handle_disprealmsgno();
  void handle_dispsoftcr();
  void handle_dispstatusline();
  void handle_disptabsize();
  void handle_dosprompt();
  void handle_dosswap();

  void handle_editautoattach();
  void handle_editautosave();
  void handle_editchangedate();
  void handle_editcharpara();
  void handle_editcharspace();
  void handle_editcomment();
  void handle_editcompletion();
  void handle_editcrlfterm();
  void handle_editfieldclear();
  void handle_edithardline();
  void handle_edithardlines();
  void handle_edithardterm();
  void handle_edithdrnameset();
  void handle_edithdrnodeset();
  void handle_editheaderattrs();
  void handle_editheaderfirst();
  void handle_editinternal();
  void handle_editmargin();
  void handle_editmenu();
  void handle_editmixcase();
  void handle_editmsgsize();
  void handle_editor();
  void handle_editorfile();
  void handle_editquotemargin();
  void handle_editreplyre();
  void handle_editsavemenu();
  void handle_editsaveutil();
  void handle_editsoftcrxlat();
  void handle_editspellcheck();
  void handle_editundelete();
  void handle_emptytearline();
  void handle_endgroup();
  void handle_event();
  void handle_excludenodes();
  void handle_externoptions();
  void handle_externutil();
  void handle_ezycommsgbase();
  void handle_ezycomuserbase();
  void handle_ezycomuserno();

  void handle_fidohwmarks();
  void handle_fidolastread();
  void handle_fidomsgtype();
  void handle_fidonullfix();
  void handle_fidouserlist();
  void handle_fidouserno();
  void handle_filealias();
  void handle_filecheck();
  void handle_filecheckcall();
  void handle_filelistpagebar();
  void handle_followupto();
  void handle_forcetemplate();
  void handle_frqext();
  void handle_frqnodemap();
  void handle_frqoptions();
  void handle_frqwazoo();

  void handle_gif();
  void handle_gifpath();
  void handle_goldbasepath();
  void handle_goldbasesyspath();
  void handle_goldbaseuserno();
  void handle_goldpath();
  void handle_group();

  void handle_happybirthday();
  void handle_highlightunread();
  void handle_hudsonpath();
  void handle_hudsonsizewarn();
  void handle_hudsonsyspath();
  void handle_hudsonuserno();

  void handle_importbegin();
  void handle_importend();
  void handle_inboundpath();
  void handle_includenodes();
  void handle_inputfile();
  void handle_intensecolors();
  void handle_internetaddress();
  void handle_internetgate();
  void handle_internetlookup();
  void handle_internetmsgid();
  void handle_internetreply();
  void handle_internetrfcbody();
  void handle_invalidate();
  void handle_jamharddelete();
  void handle_jampath();
  void handle_jamskipdeleted();

  void handle_keybclear();
  void handle_keybdef();
  void handle_keybdefaults();
  void handle_keybext();
  void handle_keybmode();
  void handle_keybstack();
  void handle_kludge();
  void handle_kludgechrs();

  void handle_langdef();
  void handle_loadlanguage();
  void handle_logfile();
  void handle_logformat();
  void handle_lookupecho();
  void handle_lookuplocal();
  void handle_lookupnet();
  void handle_lookupuserbase();

  void handle_mailinglist();
  void handle_mapdrive();
  void handle_member();
  void handle_menudropmsg();
  void handle_menumarked();
  void handle_mood();
  void handle_mouse();
  void handle_msglistdate();
  void handle_msglistfast();
  void handle_msglistfirst();
  void handle_msglistheader();
  void handle_msglistpagebar();
  void handle_msglistviewsubj();
  void handle_msglistwidesubj();

  void handle_namesfile();
  void handle_netname();
  void handle_newsgroups();
  void handle_nodelist();
  void handle_nodelistpagebar();
  void handle_nodelistwarn();
  void handle_nodepath();
  void handle_nodepathfd();
  void handle_nodepathv7();
  void handle_nodev7flags();
  void handle_nodev7modem();

  void handle_organization();
  void handle_origin();
  void handle_outboundpath();
  void handle_outputfile();
  void handle_overlay();

  void handle_pcboardmailer();
  void handle_pcboardpath();
  void handle_pcboarduserno();
  void handle_personalmail();
  void handle_play();
  void handle_printdevice();
  void handle_printformfeed();
  void handle_printinit();
  void handle_printlength();
  void handle_printmargin();
  void handle_printreset();

  void handle_quoteblank();
  void handle_quotebuffile();
  void handle_quotebufmode();
  void handle_quotechars();
  void handle_quotemargin();
  void handle_quotespacing();
  void handle_quotestring();
  void handle_quotewraphard();
  void handle_qwkbadmsgs();
  void handle_qwkconfmap();
  void handle_qwkexportpath();
  void handle_qwkimportpath();
  void handle_qwkoptions();
  void handle_qwkreplylinker();
  void handle_qwktosslog();

  void handle_ra2usersbbs();
  void handle_replylink();
  void handle_replylinklist();
  void handle_replyto();
  void handle_robotname();

  void handle_screenblanker();
  void handle_screenelimsnow();
  void handle_screenmaxcol();
  void handle_screenmaxrow();
  void handle_screenpalette();
  void handle_screenshadows();
  void handle_screensize();
  void handle_screenusebios();
  void handle_searchfor();
  void handle_semaphore();
  void handle_sender();
  void handle_serialno();
  void handle_sharemode();
  void handle_signature();
  void handle_sounddevice();
  void handle_soundpath();
  void handle_soupbadmsgs();
  void handle_soupemail();
  void handle_soupexportpath();
  void handle_soupimportpath();
  void handle_soupnewsrcfile();
  void handle_soupreplylinker();
  void handle_souptosslog();
  void handle_squishdirect();
  void handle_squishscan();
  void handle_squishuserno();
  void handle_squishuserpath();
  void handle_statuslineclock();
  void handle_statuslinehelp();
  void handle_stylecodes();
  void handle_stylecodepunct();
  void handle_stylecodestops();
  void handle_swappath();

  void handle_tagline();
  void handle_taglinechar();
  void handle_taglinefile();
  void handle_taglinesupport();
  void handle_tearline();
  void handle_template();
  void handle_templatematch();
  void handle_templatepath();
  void handle_temppath();
  void handle_timeout();
  void handle_timeoutsavemsg();
  void handle_timesread();
  void handle_timezoneoffset();
  void handle_twitmode();
  void handle_twitname();
  void handle_twitsubj();
  void handle_twitto();

  void handle_useflags();
  void handle_usefwd();
  void handle_useintl();
  void handle_usemsgid();
  void handle_userlist();
  void handle_userlistfile();
  void handle_username();
  void handle_uudecodepath();

  void handle_viewhidden();
  void handle_viewkludge();
  void handle_viewquote();

  void handle_whoto();
  void handle_wildcatuserno();

  void handle_xlatcharset();
  void handle_xlatescset();
  void handle_xlatexport();
  void handle_xlatimport();
  void handle_xlatlocalset();
  void handle_xlatpath();

  void handle_zonegating();


  //  ----------------------------------------------------------------

  void handle_editorversion();
  void handle_freearea();
  void handle_goldhelp();
  void handle_goldkeys();
  void handle_goldlang();
  void handle_goldrand();
  void handle_goldxlat();
  void handle_qbbsincrescan();
  void handle_nettear();
  void handle_qbbsrebuild();
  void handle_qbbsscan();
  void handle_screenuseansi();
  void handle_timeslice();


  //  ----------------------------------------------------------------
  //  Keyword utility members

  int address_find(const gftnaddrs& find);


  //  ----------------------------------------------------------------
  //  General

  bool check_version();
  bool check_files();
  void read_configs();

  void save(gfile& f);
  void load(gfile& f);

};


//  ------------------------------------------------------------------

