
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gecmfd.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Copy, Move, Forward and Delete.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

void gactivearea::deletemsg(GMsg& msg, int direction) {

  ulong replyto=0, reply1st=0, lread;

  GMsg uplink;
  GMsg downlink;

  if(msg.msgno) {

    // Fix the lastread pointer
    lread = msgnset.relative_to_absolute(lastread());
    if(msg.msgno == lread) {
      uint l = lastread();
      if(direction == DIR_PREV) {
        if(l-1) {
          l--;
        }
        else if((l+1) <= msgnset.size())
          l++;
        else
          l = 0;
      }
      else {
        if((l+1) <= msgnset.size())
          l++;
        else if(l-1)
          l--;
        else
          l = 0;
      }
      set_lastread(l);
      lread = msgnset.relative_to_absolute(lastread());
    }

    // Get reply links
    if(msgnset.absolute_to_relative(msg.link.to())) {
      replyto = msg.link.to();
    }
    if(msgnset.absolute_to_relative(msg.link.first())) {
      reply1st = msg.link.first();
    }
    if(replyto) {
      if(not loadhdr(downlink, replyto))
        downlink.link.first_set(0);
    }
    if(reply1st) {
      if(not loadhdr(uplink, reply1st))
        uplink.link.to_set(0);
    }

    //  --------------------------------------------------------------
    //  Diagram of splicing the links of a deleted msg
    //  --------------------------------------------------------------
    //               Reply1st    Replyto
    //                  Ŀ
    //  Downlink --> Deleted --> Uplink
    //      
    //  Reply1st     Replyto                            Confused ? :-)
    //  --------------------------------------------------------------

    // Only update the up/downlinks if they are pointing to this msg
    if(replyto) {
      if(downlink.link.first() == msg.msgno) {
        downlink.link.first_set(reply1st);
        savehdr(GMSG_UPDATE, downlink);
      }
      else {
        replyto = 0;
      }
    }
    if(reply1st) {
      if(uplink.link.to() == msg.msgno) {
        uplink.link.to_set(replyto);
        savehdr(GMSG_UPDATE, uplink);
      }
    }

    // Delete the message
    msg.attr.del1();
    delmsg(msg);

    // Update scanning files
    if(isnet() and isfido())
      TouchNetscan(false);

    // Remove message from internal table
    msgnset.delete_absolute(msg.msgno);

    // Update lastreads
    if(msgnset.size())
      set_lastread(msgnset.absolute_to_relative(lread));
    else
      set_lastread(0);
    msg.msgno = lread;
    update_data();
  }
}


//  ------------------------------------------------------------------

void DelMsg(GMsg& msg, int direction, gactivearea* area) {

  GFTRK("DelMsg");

  if(area->msgnset.size()) {

    register uint n;
    register uint x;
    int topline=0, delask=YES;

    n = 0;
    if(area->markset.size()) {
      GMenuDomarks MenuDomarks;
      n = MenuDomarks.Run(CFG->language[LNG_Delete], area);
      if(n == 1) {
        HandleGEvent(EVTT_MSGDELETING);
        w_progress(PROGRESS_NEW, C_INFOW, 0, area->markset.size(), CFG->language[LNG_Deleting]);
        area->lock();
        int escaped = false;
        for(n=0; n<area->markset.size(); n++) {
          x = n;
          if(kbxhit()) {
            if(kbxget() == Key_Esc) {
              HandleGEvent(EVTT_JOBFAILED);
              escaped = true;
              break;
            }
          }
          update_statuslinef(CFG->language[LNG_DeletingMsg], x+1, area->markset.size());
          w_progress(PROGRESS_UPDATE, C_INFOW, x+1, area->markset.size(), CFG->language[LNG_Deleting]);
          ulong msgno = area->markset[x];
          if(area->msgnset.absolute_to_relative(msgno)) {
            if(area->loadhdr(msg, msgno)) {
              if(delask) {
                if(msg.attr.uns() and not msg.attr.rcv()) {
                  area->loadmsg(msg, msgno, CFG->dispmargin-CFG->disppagebar);
                  w_progress(PROGRESS_QUIT, 0, 0, 0, NULL);
                  HeaderView.Use(area, &msg);
                  HeaderView.Paint();
                  BodyView.Use(area, &msg, topline);
                  BodyView.Paint();
                  GMenuDelete MenuDelete;
                  switch(MenuDelete.Run(YES, msg)) {
                    case YES:       // Yes, delete
                      break;
                    case NO:        // No, dont delete
                      continue;
                    default:        // Delete without asking
                      delask = NO;
                  }
                }
              }
              area->deletemsg(msg, DIR_PREV);
              area->pmrkset.delete_absolute(msg.msgno);
            }
          }
        }
        area->unlock();
        if(not escaped) {
          area->set_readmarked(false);
          area->markset.reset();
        }
        w_progress(PROGRESS_QUIT, 0, 0, 0, NULL);
      }
    }
    if(n == 0) {
      area->markset.delete_absolute(msg.msgno);
      area->pmrkset.delete_absolute(msg.msgno);
      GMenuDelete MenuDelete;
      if(MenuDelete.Run(NO, msg)) {
        HandleGEvent(EVTT_MSGDELETING);
        area->deletemsg(msg, direction);
      }
    }

    HandleGEvent(EVTT_BREAKLOOP);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void TwitDeleteMsg(GMsg& msg, int direction, gactivearea* area) {

  GFTRK("TwitDeleteMsg");

  HandleGEvent(EVTT_MSGDELETING);
  area->markset.delete_absolute(msg.msgno);
  area->pmrkset.delete_absolute(msg.msgno);
  area->deletemsg(msg, direction);
  HandleGEvent(EVTT_BREAKLOOP);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void CmfMsgs(GMsg& msg, gactivearea* area) {

  // Select action
  GMenuCMF MenuCMF;
  int cmf = MenuCMF.Run();
  if(cmf == -1)
    return;
  
  // Set language strings
  char* pickstr = NULL;
  char* markstr = NULL;
  char* progstr = NULL;
  char* statstr = NULL;
  int loadmode = GMSG_UNS_NOT_RCV;
  switch(cmf) {
    case CMF_COPY:
      GFTRK("CopyMsgs");
      pickstr = CFG->language[LNG_CopyArea];
      markstr = CFG->language[LNG_Copy];
      progstr = CFG->language[LNG_Copying];
      statstr = CFG->language[LNG_CopyingMsg];
      loadmode |= GMSG_COPY;
      break;
    case CMF_MOVE:
      GFTRK("MoveMsgs");
      pickstr = CFG->language[LNG_MoveArea];
      markstr = CFG->language[LNG_Move];
      progstr = CFG->language[LNG_Moving];
      statstr = CFG->language[LNG_MovingMsg];
      loadmode |= GMSG_MOVE;
      break;
    case CMF_FORWARD:
      GFTRK("ForwardMsgs");
      pickstr = CFG->language[LNG_ForwardArea];
      break;
  }

  // Do with current or marked msgs?
  int do_mode = MODE_CURRENT;
  if(cmf != CMF_FORWARD) {
    if(area->markset.size()) {
      GMenuDomarks MenuDomarks;
      do_mode = MenuDomarks.Run(markstr, area);
      if(do_mode == MODE_DONT) {
        GFTRK(NULL);
        return;
      }
    }
  }

  // Pick the destination area
  int destareaid = area->areaid();
  const char* cmfptr = cmf == CMF_FORWARD ? area->areareplyto().c_str() : area->areacopyto().c_str();
  if(*cmfptr)
    destareaid = AL.echoid_to_id(cmfptr);
  if(not ((cmf == CMF_FORWARD) ? area->areareplydirect() : area->areacopydirect()))
    destareaid = AreaPick(pickstr, 6, &destareaid);
  if(destareaid == -1) {
    GFTRK(NULL);
    return;
  }

  // Get destination area
  gactivearea* destarea = AL.activate(destareaid);

  // Is it readonly?
  if(destarea->attr().r_o()) {
    GMenuReadonly MenuReadonly;
    if(!MenuReadonly.Run()) {
      AL.deactivate(destarea);
      GFTRK(NULL);
      return;
    }
  }

  // Open destination area and lock it
  destarea->open();
  destarea->lock();

  // Handle a forward
  if(cmf == CMF_FORWARD) {
    extern int _use_fwd;
    _use_fwd = area->usefwd();
    if(_use_fwd == ASK) {
      GMenuForward MenuForward;
      _use_fwd = MenuForward.Run();
    }
    MakeMsg(MODE_FORWARD, msg, destarea, area);
    destarea->unlock();
    destarea->close();
    AL.deactivate(destarea);
    GFTRK(NULL);
    return;
  }        

  // Popup wait window
  w_info(progstr);

  // Lock original area
  area->lock();

  // Setup some variables for the loop
  const char* echoid = destarea->echoid().c_str();
  ulong loadmsgno = msg.msgno;
  ulong* mrkp = area->markset.tag;
  int  mrks = area->markset.size();
  int  mrk = 0;

  // Copy/move loop
  do {

    // Check for escape key
    if(kbxhit()) {
      if(kbxget() == Key_Esc) {
        HandleGEvent(EVTT_JOBFAILED);
        break;
      }
    }

    // Show progress and load the marked msg
    if(do_mode == MODE_MARKED) {
      update_statuslinef(statstr, mrk+1, mrks, echoid);
      loadmsgno = *mrkp++;
    }

    int mode;
    ulong msgno;
    if(area->loadmsg(msg, loadmsgno, CFG->dispmargin-CFG->disppagebar, loadmode)) {

      // Handle unsent msgs
      mode = cmf;
      if((cmf == CMF_MOVE) and (loadmode & GMSG_UNS_NOT_RCV) and (msg.attr.uns() and not msg.attr.rcv())) {

        // Axe the popup wait window
        w_info(NULL);

        // Display header and message
        int top = 0;
        HeaderView.Use(area, &msg);
        HeaderView.Paint();
        BodyView.Use(area, &msg, top);
        BodyView.Paint();

        // Ask if it should be deleted
        GMenuDelete MenuDelete;
        switch(MenuDelete.Run(YES, msg)) {
          case YES:  break;
          case NO:   mode = CMF_COPY;  break;
          default:   loadmode &= ~GMSG_UNS_NOT_RCV;
        }

        // Put up the wait window again
        w_info(progstr);
      }

      // Change things in the header to match the destination area
      msg.attr.del0();   // Allows deleted msgs to be undeleted
      msg.link.reset();
      msg.area = destarea;

      if(area->areacopyaddid()) {
        if(not strnieql(msg.txt, "AREA:", 5)) {
          uint elen = 5 + area->echoid().length() + 1;
          uint mlen = strlen(msg.txt)+1;
          msg.txt = (char*)throw_realloc(msg.txt, elen+mlen);
          memmove(msg.txt+elen, msg.txt, mlen);
          sprintf(msg.txt, "AREA:%s", area->echoid().c_str());
          msg.txt[elen-1] = CR;
        }
      }

      // Save the new msg to the destination area
      msgno = msg.msgno;
      destarea->savemsg(GMSG_NEW|GMSG_NOLSTUPD, msg);
    }

    // Delete original msg if moved
    if(mode == CMF_MOVE) {
      msg.msgno = msgno;
      area->deletemsg(msg, DIR_PREV);
      area->pmrkset.delete_absolute(msg.msgno);
    }

  } while((do_mode == MODE_MARKED) and ((++mrk) < mrks));

  // Unlock original area
  area->unlock();

  // Unlock and close destination area
  destarea->update_data();
  destarea->unlock();
  destarea->close();
  AL.deactivate(destarea);

  if(do_mode == MODE_MARKED) {
    if(cmf == CMF_MOVE)
      area->markset.reset();
  }
  else {
    if(cmf == CMF_MOVE) {
      area->markset.delete_absolute(msg.msgno);
      area->pmrkset.delete_absolute(msg.msgno);
    }
  }

  w_info(NULL);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void CopyMoveForward(GMsg& msg, gactivearea* area) {

  ulong lastread = msg.msgno;

  area->set_attr().hex0();
  if(area->msgnset.size())
    CmfMsgs(msg, area);

  area->msgnset.absolute_to_relative(lastread, area->lastread());
}


//  ------------------------------------------------------------------

