
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gectrl.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Random system and control info handling.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


#if 0

//  ------------------------------------------------------------------

static ulong GMsg::msgcount = 0;


//  ------------------------------------------------------------------

void GMsg::make_tagline(gstring& buf) {

  char c = area->taglinechar();
  buf = gstrf("%c%c%c %s", c, c, c, msg.tagline.c_str());
  buf.trim();
}


//  ------------------------------------------------------------------

void GMsg::make_tearline(gstring& buf) {

  buf = gstrf("--- %s", CFG->emptytearline ? "" : msg.tearline.c_str());
  buf.trim();
}


//  ------------------------------------------------------------------

void GMsg::make_originline(gstring& buf) {

  gstring orig = origin;
  gstring netn = netname;
  gstring addr = msg.origaddr;

  netn.btrim();
  if(netn)
    netn += " ";
  int len = 11 + orig.length() + 1 + 1 + netn.length() + addr.length() + 1;
  if(len > 79) {
    len -= netn.length();
    orig.resize(orig.length()-(len-79), ' ');
    orig.trim();
    netn = "";
  }
  buf = " * Origin: " + origin + " (" + netn + addr + ")";
}


//  ------------------------------------------------------------------

void GMsg::make_header_message_id(gstring& buf) {

  //                     123456781234567890
  //                     yyyymmddHHMMSShhhc
  // Message-ID: <GoldED.199706012354560071.odinn@goldware.dk>

  GTime u;
  gtimems(&u);

  buf = gstrf("<GoldED.%04d%02d%02d%02d%02d%02d%03d%lu.%s>",
    u.year, u.month,  u.day,
    u.hour, u.minute, u.second,
    u.msecond, msgcount, msg.daddress.c_str()
  );

  msgcount++;
}


//  ------------------------------------------------------------------

void GMsg::make_kludge_msgid(gstring& buf) {

  if(area->internetmsgid() and area->isinternet())
    buf = header.message_id;
  else
    buf = oaddress;

  buf += gstrf(" %08lx", gtime(NULL)+msgcount);
  msgcount++;
}


//  ------------------------------------------------------------------

void GMsg::make_kludge_pid(gstring& buf) {

  gstring tearid = gstrf("%s %s", __gpid__, gversion);
  buf = (CFG->emptytearline or not msg.tearline.iinc(tearid)) ? __pid__ : "";
}


//  ------------------------------------------------------------------

void GMsg::make_kludge_flags(gstring& buf) {

  if(area->ishudson()) {
    if(msg.attr.hld())  buf += "HLD ";
    if(msg.attr.frq())  buf += "FRQ ";
  }
  if(msg.attr.imm())  buf += "IMM ";
  if(msg.attr.dir())  buf += "DIR ";
  if(msg.attr.tfs())  buf += "TFS ";
  if(msg.attr.kfs())  buf += "KFS ";
  if(msg.attr.lok())  buf += "LOK ";
  if(msg.attr.a_s())  buf += "A/S ";
  if(msg.attr.zon())  buf += "ZON ";
  if(msg.attr.hub())  buf += "HUB ";
  if(msg.attr.xma())  buf += "XMA ";
  if(msg.attr.cfm())  buf += "CFM ";
  if(msg.attr.hir())  buf += "HIR ";
  if(msg.attr.cov())  buf += "COV ";
  if(msg.attr.sig())  buf += "SIG ";
  if(msg.attr.let())  buf += "LET ";
  if(msg.attr.fax())  buf += "FAX ";

  buf.trim();
}


//  ------------------------------------------------------------------

void insert_kludge(glineref& line, const char* name, bool present, uint type) {

  if(present) {
    gstring kludge = gstrf("\x1%s", name);
    line.insert(kludge.c_str(), GLINE_HARD|GLINE_KLUD, type);
    line++;
  }
}


//  ------------------------------------------------------------------

void insert_kludge(glineref& line, const char* name, const gstring& content, uint type) {

  if(content) {
    gstring kludge = gstrf("\x1%s ", name);
    kludge += content;
    line.insert(kludge.c_str(), GLINE_HARD|GLINE_KLUD, type);
    line++;
  }
}


//  ------------------------------------------------------------------

void insert_header(glineref& line, const char* name, const gstring& content) {

  if(content) {
    const char* prefix = area->internetrfcbody() ? "" : "\x1RFC-";
    gstring header = gstrf("%s%s: ", prefix, name);
    header += content;
    line.insert(header.c_str(), GLINE_HARD|GLINE_KLUD, GKLUD_RFC);
    line++;
  }
}


//  ------------------------------------------------------------------

void replace_controllines(int mode, GMsg& msg, gactivearea* area) {

  if(area->isecho() {

    int deltypes = GLINE_TEAR|GLINE_ORIG;
    if(area->taglinesupport())
      deltypes |= GLINE_TAGL;

    glineref line = msg.linearray;
    while(line.valid()) {
      if(line.type() & deltypes)
        line.erase();
      else
        line++;
    }

    if(not msg.tagline)
      msg.tagline = area->tagline();

    if(not msg.tearline)
      msg.tagline = area->tearline();

    if(not msg.origin) {
      msg.origin = area->origin();
      msg.netname = area->netname();
    }

    TokenXlat(mode, msg.tagline, msg, msg, area);
    TokenXlat(mode, msg.tearline, msg, msg, area);
    TokenXlat(mode, msg.origin, msg, msg, area);
    TokenXlat(mode, msg.netname, msg, msg, area);

    gstring buf;

    if(msg.tagline and area->taglinesupport()) {
      msg.make_tagline(buf);
      msg.linearray.add(buf, GLINE_HARD|GLINE_TAGL);
    }

    if(area->ctrlinfo() & CI_TEAR) {
      msg.make_tearline(buf);
      msg.linearray.add(buf.c_str(), GLINE_HARD|GLINE_TEAR);
    }

    if(area->ctrlinfo() & CI_ORIG) {
      msg.make_originline(buf);
      msg.linearray.add(buf.c_str(), GLINE_HARD|GLINE_ORIG);
    }
  }

  glineref line = msg.linearray;
  while(line.valid()) {
    if(line.kludge()) {
      bool waswrapped;
      do {
        waswrapped = line.iswrapped();
        line.erase();
      } while(line.valid() and waswrapped);
    }
    else {
      line++;
    }
  }

  line.go_first();

  insert_kludge(line, "INTL",       msg.kludge.intl,      GKLUD_INTL);
  insert_kludge(line, "TOPT",       msg.kludge.topt,      GKLUD_TOPT);
  insert_kludge(line, "FMPT",       msg.kludge.fmpt,      GKLUD_FMPT);
  insert_kludge(line, "FLAGS",      msg.kludge.flags,     GKLUD_FLAGS);
  insert_kludge(line, "SPLIT:",     msg.kludge.split,     GKLUD_SPLIT);
  insert_kludge(line, "MSGID:",     msg.kludge.msgid,     GKLUD_MSGID);
  insert_kludge(line, "REPLY:",     msg.kludge.reply,     GKLUD_REPLY);
  insert_kludge(line, "CHARSET:",   msg.kludge.charset,   GKLUD_CHARSET);
  insert_kludge(line, "CHRS:",      msg.kludge.chrs,      GKLUD_CHRS);
  insert_kludge(line, "REPLYADDR:", msg.kludge.replyaddr, GKLUD_REPLYADDR);
  insert_kludge(line, "REPLYTO:",   msg.kludge.replyto,   GKLUD_REPLYTO);
  insert_kludge(line, "PID:",       msg.kludge.pid,       GKLUD_PID);
  insert_kludge(line, "GIF:",       msg.kludge.gif,       GKLUD_GIF);
  insert_kludge(line, "MOOD:",      msg.kludge.mood,      GKLUD_MOOD);
  insert_kludge(line, "FWDFROM",    msg.kludge.fwdfrom,   GKLUD_FWD);
  insert_kludge(line, "FWDORIG",    msg.kludge.fwdorig,   GKLUD_FWD);
  insert_kludge(line, "FWDTO",      msg.kludge.fwdto,     GKLUD_FWD);
  insert_kludge(line, "FWDDEST",    msg.kludge.fwddest,   GKLUD_FWD);
  insert_kludge(line, "FWDSUBJ",    msg.kludge.fwdsubj,   GKLUD_FWD);
  insert_kludge(line, "FWDAREA",    msg.kludge.fwdarea,   GKLUD_FWD);
  insert_kludge(line, "FWDMSGID",   msg.kludge.fwdmsgid,  GKLUD_FWD);
  insert_kludge(line, "I51",        msg.kludge.i51,       GKLUD_I51);

  insert_header(line, "To",                        msg.header.to);
  insert_header(line, "From",                      msg.header.from);
  insert_header(line, "Subject",                   msg.header.subject);
  insert_header(line, "Date",                      msg.header.date);
  insert_header(line, "Message-ID",                msg.header.message_id);
  insert_header(line, "References",                msg.header.references);
  insert_header(line, "In-Reply-To",               msg.header.in_reply_to);
  insert_header(line, "Newsgroups",                msg.header.newsgroups);
  insert_header(line, "Followup-To",               msg.header.followup_to);
  insert_header(line, "Organization",              msg.header.organization);
  insert_header(line, "MIME-Version",              msg.header.mime_version);
  insert_header(line, "Content-Type",              msg.header.content_type);
  insert_header(line, "Content-Transfer-Encoding", msg.header.content_transfer_encoding);
  insert_header(line, "Sender",                    msg.header.sender);
  insert_header(line, "Reply-To",                  msg.header.reply_to);
  insert_header(line, "X-To",                      msg.header.x_to);
  insert_header(line, "X-Charset",                 msg.header.x_charset);
  insert_header(line, "X-Char-Esc",                msg.header.x_char_esc);
  insert_header(line, "X-Mailreader",              msg.header.x_mailreader);
  insert_header(line, "X-Newsreader",              msg.header.x_newsreader);

  if(area->internetrfcbody() and line.more() and not strblank(line.next().text())) {
    line.insert("", GLINE_HARD|GLINE_KLUD, GKLUD_RFC);
    line++;
  }

  // SEEN-BY and PATH lines
  if(msg.seenby.size()) {
    for(int n=0; n<msg.seenby.size(); n++)
      msg.linearray.add(msg.seenby[n], GLINE_HARD|GLINE_KLUD, GKLUD_SEENBY);
  }
  if(msg.path.size()) {
    for(int n=0; n<msg.path.size(); n++)
      msg.linearray.add(msg.path[n], GLINE_HARD|GLINE_KLUD, GKLUD_PATH);
  }

  msg.reindex_lines(false, true, true, true);

  throw_free(buf);
}

#endif

//  ------------------------------------------------------------------

