
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gedoss.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  DOS Shell, cleanup, errorhandling etc.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include "gmbprot.h"


//  ------------------------------------------------------------------
//  Clean up the screen, memory and files before exiting to DOS

void _stdc Cleanup(void) {

  if(CFG) {

    if(CFG->areakeeplast and startupscan_success)
      AL.write_goldlast();

    // Free area data
    AL.reset();

    delete gmb_wide_username;

    // Free misc data
    throw_release(echoout);
    throw_release(echoin);
    throw_release(find_pattern);
    throw_release(find_patstr);

    throw_delete(XLAT);
    throw_delete(GRP);
    throw_delete(QWK);

    if(errorlevel != EXIT_CFGERR) {
      if(netpost)
        TouchFile(AddPath(CFG->areapath, CFG->semaphore.netscan));
      if(echopost)
        TouchFile(AddPath(CFG->areapath, CFG->semaphore.echoscan));
    }

    // Reset border color
    if(C_BACKB != 0)
      gvid.setoverscan(gvid.orig.color.overscan);

    wcloseall();        // Close all windows
    a_closeall();       // Unlink hidden area windows
    whelpundef();       // Disengage the help system
    kbclear();          // Clear keyboard buffer
    freonkey();         // Free all onkeys (macros)
    FreePastebuf();     // Free the internal editor cut'n'paste buffer

    if(CFG->screenpalette[16])
      gvid.setpalette(gvid.orig.color.palette);

    if(gvid.curr.screen.mode != gvid.orig.screen.mode)
      gvid.setmode(gvid.orig.screen.mode);
    if(gvid.curr.screen.rows != gvid.orig.screen.rows)
      gvid.setrows(gvid.orig.screen.rows);

    if(oldscreen)
      vrestore(oldscreen);
    if(CFG->intensecolors)
      gvid.setintensity(gvid.orig.color.intensity);
    vcurset(gvid.orig.cursor.start, gvid.orig.cursor.end);
    vcurshow();

    vposset(gvid.orig.cursor.row, 0);
    vputx(gvid.orig.cursor.row, 0, gvid.orig.color.textattr, ' ', gvid.orig.screen.columns);
    vposset(gvid.orig.cursor.row-1, 0);
  }

  int smax = minimum_of_two((int)GLOG_STORELINES, LOG.storelines);
  if(smax > 0)
    printf("\n");
  for(int s=0; s<smax; s++)
    printf("%s\n", LOG.storeline[s]);
  if(LOG.storelines > GLOG_STORELINES)
    printf("(See also %s)\n", CFG->logfile.c_str());
  if(errorlevel > EXIT_NONAME)
    MakeNoise(SND_S_O_S);

  // Back to default Ctrl-Break handler
  signal(SIGINT, SIG_DFL);
}


//  ------------------------------------------------------------------
//  Multipurpose DOS shell function

int ShellToDos(char* command, char* message, int cls, int cursor, int swap, int pause, gactivearea* area, bool wait) {

  // Put up a wait window
  if(wait)
    w_info(CFG->language[LNG_Wait]);

  // Close msgbase files
  int _wasopen = false;
  if(area) {
    _wasopen = area->isopen();
    if(_wasopen)
      area->suspend();    // NOTE: Should call a manager->suspend() !!!
  }

  HandleGEvent(EVTT_DOSSHELL);

  // Change the prompt
  static char __FAR prompt[256];
  static char __FAR oldprompt[256];

  #ifdef __DJGPP__
  char* environ[2] = { "", NULL };
  #endif

  if(CFG->dosprompt) {
    int envn = 0;
    while(environ[envn] and *environ[envn]) {
      if(strnieql(environ[envn], "PROMPT=", 7)) {
        strcpy(oldprompt, environ[envn]);
        break;
      }
      envn++;
    }
    sprintf(prompt, "PROMPT=%s%s", CFG->language[LNG_Prompt], *oldprompt ? oldprompt+7 : "");
    environ[envn] = prompt;
  }

  // Store the screen
  word* scrnbuf = vsave();

  // Store current drive/dir
  Path orgdir;
  getcwd(orgdir, sizeof(Path));

  // Set cursor position
  if(gvid.curr.screen.rows != gvid.orig.screen.rows)
    gvid.setrows(gvid.orig.screen.rows);
  if(gvid.curr.screen.mode != gvid.orig.screen.mode)
    gvid.setmode(gvid.orig.screen.mode);

  // Clear screen
  if(cls)
    vclrscr(cls);

  // Reset border color
  if(C_BACKB != 0)
    gvid.setoverscan(gvid.orig.color.overscan);

  // Turn on the blinking attributes
  if(CFG->intensecolors)
    gvid.setintensity(gvid.orig.color.intensity);

  // Restore original palette during the shell
  if(CFG->screenpalette[16])
    gvid.setpalette(gvid.orig.color.palette);

  // Write message on screen
  if(*message)
    puts(message);

  // Turn on cursor
  int yy, xx;
  if(cursor) {
    vposget(&yy, &xx);
    vcurset(gvid.orig.cursor.start, gvid.orig.cursor.end);
    vcurshow();
  }
  
  HandleGEvent(EVTT_BREAKLOOP);

  // Get executable and parameters
  char _cmd[256];
  strcpy(_cmd, command);
  char* _pars;
  char* _xfn = _cmd;
  if(strnieql(_xfn, "/c", 2)) {
    _xfn = "";      // Call command interpreter
    _pars = strskip_wht(_cmd+2);
  }
  else {
    _pars = strpbrk(_xfn, " \t");
    if(_pars) {
      *_pars++ = NUL;
      _pars = strskip_wht(_pars);
    }
    else
     _pars = "";
  }

  // Shell return value
  int status = -1;

  if(swap and not CFG->dosswap)
    swap = false;

  #if defined(USE_EXECSWAP)

  if(swap) {

    // Shell using the EXEC 3.3 swapper by Thomas Wagner

    // Remove Ctrl-Break handler
    signal(SIGINT, SIG_DFL);

    // Call the swapper
    status = do_exec(_xfn, _pars, CFG->dosswap&~GSWAP_ENABLED, 0xFFFF, environ);

    // Log and report any error condition
    if(status > 0xFF) {
      char errbuf[200];
      char* errtxt = errbuf;
      *errtxt = NUL;
      switch(status) {
        case 0x0101:  errtxt = "Error preparing for swap: no space for swapping";  break;
        case 0x0102:  errtxt = "Error preparing for swap: program too low in memory";  break;
        case 0x0200:  errtxt = "Program file not found";  break;
        case 0x0201:  errtxt = "Program file: Invalid drive";  break;
        case 0x0202:  errtxt = "Program file: Invalid path";  break;
        case 0x0203:  errtxt = "Program file: Invalid name";  break;
        case 0x0204:  errtxt = "Program file: Invalid drive letter";  break;
        case 0x0205:  errtxt = "Program file: Path too long";  break;
        case 0x0206:  errtxt = "Program file: Drive not ready";  break;
        case 0x0207:  errtxt = "Batchfile/COMMAND: COMMAND.COM not found";  break;
        case 0x0208:  errtxt = "Error allocating temporary buffer";  break;
        case 0x0400:  errtxt = "Error allocating environment buffer";  break;
        case 0x0500:  errtxt = "Swapping requested, but prep_swap has not been called or returned an error.";  break;
        case 0x0501:  errtxt = "MCBs don't match expected setup";  break;
        case 0x0502:  errtxt = "Error while swapping out";  break;
        case 0x0600:  errtxt = "Redirection syntax error";  break;
        default:
          if((status & 0xFF00) == 0x0300)
            sprintf(errtxt, "DOS error %02Xh calling EXEC", status & 0xFF);
          else if((status & 0xFF00) == 0x0600)
            sprintf(errtxt, "DOS error %02Xh on redirection", status & 0xFF);
          else
            errtxt = "Uknown error???";
      }
      LOG.printf("? For commandline \"%s\"", command);
      LOG.printf("? The swapper returned errorcode %04X:", status);
      LOG.printf("? %s", errtxt);
      printf("\nFor commandline \"%s\"\n", command);
      printf("The swapper returned errorcode %04X:\n", status);
      printf("%s\n", errtxt);
      printf("This has been logged in %s", CFG->logfile);
      printf("\nPress a key to continue.\n");
      getxch();
    }

    // Re-install Ctrl-Break handler
    signal(SIGINT, SIG_IGN);
  }

  #elif defined(USE_SPAWNO)

  if(swap) {
    init_SPAWNO(CFG->swappath, CFG->dosswap&~GSWAP_ENABLED);
    char* _arg_v[3];
    _arg_v[0] = _xfn;
    _arg_v[1] = _pars;
    _arg_v[2] = NULL;
    errno = 0;
    _doserrno = 0;
    status = spawnvpeo(___spawn_swap_dirs, _xfn, _arg_v, environ);
    if(status != 0)
      swap = false;
  }

  #else

  if(swap)
    swap = false;

  #endif

  // Shell using the regular RTL function
  if(not swap) {
    #if defined(__WATCOMC__)
    const char* _arg_v[3];
    #else
    char* _arg_v[3];
    #endif
    _arg_v[0] = _xfn;
    _arg_v[1] = _pars;
    _arg_v[2] = NULL;
    #if defined(__WATCOMC__)
    status = spawnvpe(P_WAIT, _xfn, _arg_v, (const char * const *)environ);
    #elif defined(__BORLANDC__) and defined(__OS2__)
    char sbuf[256];
    sprintf(sbuf, "%s %s", _xfn, _pars);
    status = system(sbuf);
    if(status != -1)
      status = 0;
    #else
    status = spawnvpe(P_WAIT, _xfn, _arg_v, environ);
    #endif
  }

  if(pause) {
    if((pause > 0) or (status != 0))
      kbxget();
  }

  // Restore current directory
  gchdir(orgdir);

  // Restore video mode and rows
  if(CFG->screensize > 0xFF)
    gvid.setmode(CFG->screensize >> 8);
  else if(CFG->screensize)
    gvid.setrows(CFG->screensize);

  // Restore cursor position and form
  if(cursor) {
    vposset(yy+1, xx);
    vcurhide();
  }

  // Restore screen
  if(scrnbuf)
    vrestore(scrnbuf);
  else
    vclrscr();

  // Restore screen intensity
  if(CFG->intensecolors)
    gvid.setintensity(true);

  // Restore border color
  if(C_BACKB != 0)
    gvid.setoverscan(C_BACKB);

  // Set palette if changes were specified
  if(CFG->screenpalette[16])
    gvid.setpalette(CFG->screenpalette);
  
  // Restore prompt
  if(CFG->dosprompt) {
    int envn = 0;
    while(environ[envn] and *environ[envn]) {
      if(strnieql(environ[envn], "PROMPT=", 7))
        break;
      envn++;
    }
    environ[envn] = oldprompt;
  }

  // Re-open msgbase
  if(_wasopen)
    area->resume();

  // Remove the wait window
  if(wait)
    w_info(NULL);

  // Reset tick values to avoid triggering screenblanker or timeout
  gkbdtickpressreset();
  gkbdtickvaluereset();

  return status;
}


//  ------------------------------------------------------------------
//  Error exit function

void ErrorExit(int type) {

  static in_error_exit = false;

  if(not in_error_exit++) {

    error_exit = type;

    if(type) {

      HandleGEvent(EVTT_ERRORFATAL);

      if(type == ERR_MEMO) {

        #if 0
        whelpcat(H_EOutOfMemory);
        call_help();
        whelpcat(H_General);
        #endif
      }

      if(_in_editor) {
        LOG.printf("+ TIP: If you were writing a msg and want to recover it,");
        LOG.printf("+ try looking in the %s file.", AddPath(CFG->goldpath, CFG->editfile));
      }

      // Dump the function tracker log
      #if defined(GFTRK_ENABLE)
      __gftrk_log();
      #endif

      if(type != 9)
        errorlevel = EXIT_ERRORS;
      else if(type == 5)
        errorlevel = 100;
      else
        errorlevel = EXIT_CFGERR;

      exit(errorlevel);
    }

    exit(EXIT_OK);
  }
}


//  ------------------------------------------------------------------

