
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gedxlat.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Character translation.
//  ------------------------------------------------------------------


#include <gutlmisc.h>
#include <gedxlat.h>


//  ------------------------------------------------------------------

static int _stdc gedxlat_esc_cmp(const char* a, const char* b) {

  return strncmp(a, b, 5);
}


//  ------------------------------------------------------------------

void gedxlat_chs::reset() {

  *import = *export = NUL;
  id = version = level = 0;
  memset(t, 0, 256*4);
  for(int n=0; n<256; n++) {
    t[n][0] = 1;
    t[n][1] = (byte)n;
  }
}


//  ------------------------------------------------------------------

void gedxlat_esc::reset() {

  *export = NUL;
  tablesize = 0;
  memset(t, 0, 256*5);
}


//  ------------------------------------------------------------------

char* gedxlat_set::translate(char* dest, const gstring& src, bool dispsoftcr, bool qpencoded) {

  return translate(dest, src.c_str(), dispsoftcr, qpencoded);
}


//  ------------------------------------------------------------------

char* gedxlat_set::translate(char* dest, const char* src, bool dispsoftcr, bool qpencoded) {

  if(not charset.active())
    return stpcpy(dest, src);

  register int n;
  register int clen;
  register bool translated;
  register const char* tptr;
  register const char* escp;
  register const char* sptr = src;
  register char* dptr = dest;
  register char dochar;
  int level = charset.level;

  while(*sptr) {
    switch(*sptr) {
      case 0x02:
        if(i51.active()) {
          for(n=0; n<i51.size(); n++) {
            tptr = i51.chars(n);
            if(*(sptr+1) == tptr[0]) {
              if(*(sptr+2) == tptr[1]) {
                escp = &tptr[2];
                if(*escp) {
                  *dptr++ = *escp++;
                  if(*escp) {
                    *dptr++ = *escp++;
                    if(*escp) {
                      *dptr++ = *escp;
                    }
                  }
                }
                sptr += 2;
                n = -1;
                break;
              }
            }
          }
          if(n != -1)   // I51 char not found, use fallback method
            sptr++;
        }
        sptr++;
        break;

      case 29:
        if(mnemonic.active()) {
          for(n=0; n<mnemonic.size(); n++) {
            tptr = mnemonic.chars(n);
            if(*(sptr+1) == tptr[0]) {
              if(*(sptr+2) == tptr[1]) {
                escp = &tptr[2];
                if(*escp) {
                  *dptr++ = *escp++;
                  if(*escp) {
                    *dptr++ = *escp++;
                    if(*escp) {
                      *dptr++ = *escp;
                    }
                  }
                }
                sptr += 2;
                n = -1;
                break;
              }
            }
          }
          if(n != -1)   // MNEMONIC char not found, use fallback method
            sptr++;
        }
        sptr++;
        break;

      case SOFTCR:
        translated = false;
        if(composed.active() and not dispsoftcr) {
          if(sptr > src) {
            if(not (isspace(*(sptr-1)) or isspace(*(sptr+1)))) {
              for(n=0; n<composed.size(); n++) {
                tptr = composed.chars(n);
                if(*(sptr-1) == tptr[0]) {
                  if(*(sptr+1) == tptr[1]) {
                    escp = &tptr[2];
                    if(*escp) {
                      dptr--;
                      *dptr++ = *escp++;
                      if(*escp) {
                        *dptr++ = *escp++;
                        if(*escp) {
                          *dptr++ = *escp;
                          translated = true;
                        }
                      }
                    }
                    sptr += 2;
                    break;
                  }
                }
              }
            }
          }
        }
        if(not translated)
          *dptr++ = *sptr++;
        break;

      case '=':
        if(qpencoded) {
          if(isxdigit(sptr[1]) and isxdigit(sptr[2])) {
            dochar = (char)((xtoi(sptr[1]) << 4) | xtoi(sptr[2]));
            sptr += 3;
            goto chardo;
          }
        }
        goto defaultchardo;

      default:
      defaultchardo:
        dochar = *sptr++;
      chardo:
        if(level&3) {   // Translate level 1 and 2
          tptr = charset.chars(dochar);
          clen = *tptr++;
          while(clen--)
            *dptr++ = *tptr++;
        }
        else {
          *dptr++ = dochar;
        }
    }
  }

  *dptr = NUL;

  return dptr;
}


//  ------------------------------------------------------------------

gedxlat_map::gedxlat_map() { }
gedxlat_map::~gedxlat_map() { }


//  ------------------------------------------------------------------

gedxlat_map& gedxlat_map::operator=(const gedxlat_map& x) {

  import = x.import;
  export = x.export;
  return *this;
}


//  ------------------------------------------------------------------

void gedxlat_map::save(gfile& f) {

  f << import << export;
}


//  ------------------------------------------------------------------

void gedxlat_map::load(gfile& f) {

  f >> import >> export;
}


//  ------------------------------------------------------------------

void gedxlat::compile_charset(gfile& ifp, gedxlat_chs& chs) {

  int ch = 0;
  int line = 1;
  char buf[256];
  while(ifp.fgets(buf, sizeof(buf))) {
    char* ptr = buf;
    if(*ptr != ';' and not strblank(ptr)) {
      char* ptr2 = strchr(ptr+2, ';');
      if(ptr2)
        *ptr2 = NUL;
      strtrim(ptr);
      if(line < 6) {
        switch(line) {
          case 1:
            chs.id = atol(ptr);
            break;
          case 2:
            chs.version = atoi(ptr);
            break;
          case 3:
            chs.level = atoi(ptr);
            if((chs.level == 1) or (chs.id > 65535L))
              ch = 0;
            else
              ch = 128;
            break;
          case 4:
            strcpy(chs.import, strbtrim(ptr));
            break;
          case 5:
            if(chs.level)
              sprintf(chs.export, "%s %d", strbtrim(ptr), chs.level);
            else
              strcpy(chs.export, strbtrim(ptr));
            break;
        }
      }
      else {
        int x = 0;
        chs.t[ch][0] = (byte)x;
        ptr = strskip_wht(ptr);
        if((ch > 255) or strieql(ptr, "END"))
          break;
        while(*ptr and (x < 3)) {
          if(*ptr == '\\') {
            ptr++;
            switch(*ptr) {
              case '\\':
                chs.t[ch][++x] = (byte)'\\';
                chs.t[ch][0] = (byte)x;
                ptr = strskip_txt(ptr);
                break;
              case 'd':
                chs.t[ch][++x] = (byte)atoi(ptr+1);
                chs.t[ch][0] = (byte)x;
                ptr = strskip_txt(ptr);
                break;
              case 'x':
                chs.t[ch][++x] = (byte)atox(ptr+1);
                chs.t[ch][0] = (byte)x;
                ptr = strskip_txt(ptr);
                break;
              case '0':
                ptr = strskip_txt(ptr);
                break;
            }
          }
          else {
            chs.t[ch][++x] = (byte)*ptr++;
            chs.t[ch][0] = (byte)x;
          }
          ptr = strskip_wht(ptr);
        }
        ch++;
      }
      line++;
    }
  }
}


//  ------------------------------------------------------------------

void gedxlat::compile_escset(gfile& ifp, gedxlat_esc& esc) {

  int line = 1;
  char buf[256];
  while(ifp.fgets(buf, sizeof(buf))) {
    char* ptr = buf;
    if(*ptr != ';' and not strblank(ptr)) {
      char* ptr2 = strchr(ptr+2, ';');
      if(ptr2)
        *ptr2 = NUL;
      strtrim(ptr);
      if(line < 2) {
        switch(line) {
          case 1:
            strcpy(esc.export, strbtrim(ptr));
            break;
        }
      }
      else {
        int n = esc.size();
        if(n > 255 or strieql(ptr, "END"))   // End of table reached
          break;
        esc.t[n][0] = ptr[0];
        esc.t[n][1] = ptr[1];
        int x = 2;
        ptr += 2;
        ptr = strskip_wht(ptr);
        while(*ptr and (x < 5)) {
          if(*ptr == '\\') {
            ptr++;
            switch(*ptr) {
              case '\\':
                esc.t[n][x++] = '\\';
                ptr = strskip_txt(ptr);
                break;
              case 'd':
                esc.t[n][x++] = (uchar)atoi(ptr+1);
                ptr = strskip_txt(ptr);
                break;
              case 'x':
                esc.t[n][x++] = (uchar)atox(ptr+1);
                ptr = strskip_txt(ptr);
                break;
              case '0':
                esc.t[n][x++] = NUL;
                ptr = strskip_txt(ptr);
                break;
            }
          }
          else {
            esc.t[n][x++] = *ptr++;
          }
          ptr = strskip_wht(ptr);
        }
        esc.tablesize++;
      }
      line++;
    }
  }

  qsort(esc.t, esc.size(), 5, (StdCmpCP)gedxlat_esc_cmp);
}


//  ------------------------------------------------------------------

void gedxlat::compile_tables() {

  if(charset_file.size() or escset_file.size()) {

    int count;

    gfile ofp(xlatfile.c_str(), "wb");
    if(ofp) {

      for(count=0; count<charset_file.size(); count++) {

        gedxlat_chs charset;

        const char* ifile = charset_file[count];
        gfile ifp(ifile, "rb");

        if(ifp) {
          show_progress(ifile);
          compile_charset(ifp, charset);
          ifp.fclose();
        }

        ofp.fwrite(&charset, sizeof(gedxlat_chs));
      }

      for(count=0; count<escset_file.size(); count++) {

        gedxlat_esc escset;

        const char* ifile = escset_file[count];
        gfile ifp(ifile, "rb");

        if(ifp) {
          show_progress(ifile);
          compile_escset(ifp, escset);
          ifp.fclose();
        }

        ofp.fwrite(&escset, sizeof(gedxlat_esc));
      }

      ofp.fclose();
    }
  }
}


//  ------------------------------------------------------------------

void gedxlat::save(gfile& f) {

  int n;
  f << xlatfile;

  f << charset.size();
  for(n=0; n<charset.size(); n++)
    charset[n].save(f);

  f << escset.size();
  for(n=0; n<charset.size(); n++)
    escset[n].save(f);
}


//  ------------------------------------------------------------------

void gedxlat::load(gfile& f) {

  int size, n;
  f >> xlatfile;

  f >> size;
  charset.resize(size);
  for(n=0; n<size; n++)
    charset[n].load(f);

  f >> size;
  escset.resize(size);
  for(n=0; n<size; n++)
    escset[n].load(f);
}


//  ------------------------------------------------------------------

void gedxlat::add_charset(const char* import, const char* export, const char* mapfile) {

  gedxlat_map map;
  map.import = import;
  map.export = export;
  charset.push_back(map);
  charset_file.add(mapfile);
}


//  ------------------------------------------------------------------

void gedxlat::add_escset(const char* import, const char* export, const char* mapfile) {

  gedxlat_map map;
  map.import = import;
  map.export = export;
  escset.push_back(map);
  escset_file.add(mapfile);
}


//  ------------------------------------------------------------------

bool gedxlat::load_charset(gedxlat_chs& chs, const gstring& import, const gstring& export, bool disable_softcr) {

  chs.reset();

  for(int n=0; n<charset.size(); n++) {
    if(import.iinc(charset[n].import) and export.iinc(charset[n].export)) {
      gfile fp(xlatfile.c_str(), "rb");
      if(fp) {
        fp.fseekset(n, sizeof(gedxlat_chs));
        fp.fread(&chs, sizeof(gedxlat_chs));
        fp.fclose();
        if(disable_softcr) {
          char* tptr = chs.t[SOFTCR];
          *tptr++ = 1;
          *tptr = SOFTCR;
        }
        return true;
      }
    }
  }

  return false;
}


//  ------------------------------------------------------------------

bool gedxlat::load_escset(gedxlat_esc& esc, const gstring& import) {

  esc.reset();

  for(int n=0; n<escset.size(); n++) {
    if(import.ieql(escset[n].import)) {
      gfile fp(xlatfile.c_str(), "rb");
      if(fp) {
        fp.fseekset(charset.size(), sizeof(gedxlat_chs));
        fp.fseek(n, sizeof(gedxlat_esc), SEEK_CUR);
        fp.fread(&esc, sizeof(gedxlat_esc));
        fp.fclose();
        return true;
      }
    }
  }

  return false;
}


//  ------------------------------------------------------------------

gstring& gedxlat_set::translate(gstring& str, bool qpencoded) {

  char* buf = new char[str.length()*2];
  translate(buf, str.c_str(), true, qpencoded);
  str = buf;
  delete[] buf;
  return str;
}


//  ------------------------------------------------------------------

