
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gedxlat.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Character translation.
//  ------------------------------------------------------------------


#ifndef __GEDXLAT_H
#define __GEDXLAT_H


//  ------------------------------------------------------------------

#include <gvector.h>
#include <gfile.h>
#include <gstrblks.h>
#include <gstrgstr.h>


//  ------------------------------------------------------------------

typedef char gedxlat_name[17];
//typedef byte gedxlat_chstab[4];
//typedef byte gedxlat_esctab[5];


//  ------------------------------------------------------------------

class gedxlat_chs {

protected:

  char         t[256][4];       // Translation table

public:

  gedxlat_chs()                 { reset(); }

  void         reset();

  bool         active()         { return *export; }
  const char*  chars(uint ch)   { return t[ch]; }

  long         id;
  int          version;
  int          level;

  gedxlat_name import;          // From charset
  gedxlat_name export;          // To charset

  friend class gedxlat;

};


//  ------------------------------------------------------------------

class gedxlat_esc {

protected:

  //byte           esc;         // Escape character
  //int            pos1;        // Relative position of first character
  //int            pos2;        // Relative position of last character
  int            tablesize;     // Number of escaped chars in table
  char           t[256][5];     // The Escaped Characters Table

public:

  gedxlat_esc()                 { reset(); }

  void         reset();

  int          size()           { return tablesize; }
  bool         active()         { return *export; }
  const char*  chars(uint ch)   { return t[ch]; }

  gedxlat_name export;          // To charset

  friend class gedxlat;

};


//  ------------------------------------------------------------------

class gedxlat_set {

public:

  gedxlat_chs charset;
  gedxlat_esc i51;
  gedxlat_esc mnemonic;
  gedxlat_esc composed;

  gstring& translate(gstring& str, bool qpencoded=false);
  char* translate(char* dest, const gstring& src, bool dispsoftcr, bool qpencoded=false);
  char* translate(char* dest, const char* src, bool dispsoftcr, bool qpencoded=false);

};


//  ------------------------------------------------------------------

class gedxlat_map {

public:

  gedxlat_map();
  ~gedxlat_map();

  gstring import;
  gstring export;

  gedxlat_map& operator=(const gedxlat_map& x);

  void save(gfile& f);
  void load(gfile& f);

};


//  ------------------------------------------------------------------

typedef gvector<gedxlat_map> gedxlat_map_vector;


//  ------------------------------------------------------------------

class gedxlat {

public:

  gedxlat() { }
  virtual ~gedxlat() { }

  virtual void       show_progress(const char*) { }

  gstring            xlatfile;

  gstring            localset;

  gedxlat_map_vector charset;
  gstrarray          charset_file;

  gedxlat_map_vector escset;
  gstrarray          escset_file;

  void               compile_charset(gfile& ifp, gedxlat_chs& chs);
  void               compile_escset(gfile& ifp, gedxlat_esc& esc);

  void               compile_tables();

  void               add_charset(const char* import, const char* export, const char* mapfile);
  void               add_escset(const char* import, const char* export, const char* mapfile);

  bool               load_charset(gedxlat_chs& charset, const gstring& import, const gstring& export, bool disable_softcr=false);
  bool               load_escset(gedxlat_esc& escset, const gstring& import);

  void               save(gfile& f);
  void               load(gfile& f);

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

