
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geedit.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  The Internal Editor (IE).
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include "geedit.h"


//  ------------------------------------------------------------------
//  Globals

bool CFG__editquotewrap = true;

glinearray ieditor::killbuf;
glinearray ieditor::pastebuf;

gstring Edit__exportfilename;


//  ------------------------------------------------------------------

#ifndef NDEBUG

void ieditor::debugtest(char* test, int a, int b, char* file, int line, int values) {

  #if defined(GFTRK_ENABLE)
  int tmp = __gftrk_on;
  __gftrk_on = false;
  #endif

  savefile(STATUS_UPDATE);

  #if defined(GFTRK_ENABLE)
  __gftrk_on = tmp;
  #endif

  if(values)
    w_infof(" Range check: (%s) <%u,%u> [%s,%u] ", test, a, b, CleanFilename(file), line);
  else
    w_infof(" Range check: (%s) [%s,%u] ", test, CleanFilename(file), line);
  update_statusline(CFG->language[LNG_EscOrContinue]);
  if(not disablesound)
    beep();
  while(kbxhit())
    kbxget();
  Key key = kbxget();
  w_info(NULL);
  if(key == Key_Esc) {
    LOG.errtest(file, line);
    LOG.printf("! An internal editor range check failed.");
    if(values)
      LOG.printf(": Details: (%s) <%u,%u>.", test, a, b);
    else
      LOG.printf(": Details: (%s).", test);
    LOG.printf(": Details: r%u,c%u,mr%u,mc%u,i%u,dm%u,qm%u,eqm%u.",
      cursor_row, cursor_col, window_maxrow, window_maxcol, insert,
      CFG->dispmargin, CFG->quotemargin, CFG->editquotemargin
    );
    LOG.printf("+ Advice: Report to the Author.");
    TestErrorExit();
  }
}

#endif


//  ------------------------------------------------------------------
//  Make sure line type is correct

void ieditor::setlinetype(glineref& line) {

  line.type() &= ~(gline::hidden|gline::kludge|gline::quote);
  line.type() |= quoting.is_quote(line.text()) ? gline::quote : (*line.text() == CTRL_A) ? gline::hidden : 0;
}


//  ------------------------------------------------------------------
//  Insert character in string at position

void ieditor::strinschr(char* str, char ch, int position) {

  _test_halt(str == NULL);

  //  0123456789  position = 2
  //  1234567890  length = 4
  //  helo<       < = nul
  //  he lo<
  //  hello<

  // Get string length
  int length = strlen(str);

  // Is position beyond the nul-terminator in the string?
  _test_haltab(position > length, position, length);

  // Make room for the char to insert
  memmove(str+position+1, str+position, length-position+1);

  // Put in the char
  str[position] = ch;
  if((str[position+1] == NUL) and (ch != ' ')) {
    str[position+1] = ' ';
    str[position+2] = NUL;
  }
}


//  ------------------------------------------------------------------
//  Delete character in string at position

void ieditor::strdelchr(char* str, int position) {

  _test_halt(str == NULL);

  //  0123456789  position = 1
  //  1234567890  length = 5
  //  hello<      < = nul
  //  hllo<

  int length = strlen(str);

  // Is position at or beyond the nul-terminator in the string?
  _test_haltab(position >= length, position, length);

  // Delete the character
  memmove(str+position, str+position+1, length-position+1);
}


//  ------------------------------------------------------------------

inline int ieditor::dispchar(char ch, int attr) {

  if(ch != '\n') {
    if(ch == ' ')
      ch = CFG->editcharspace;
  }
  else {
    ch = CFG->editcharpara;
  }

  int atr;
  char chr;
  editwin.getc(cursor_crow, cursor_ccol, &atr, &chr);
  editwin.printc(cursor_crow, cursor_ccol, attr == -1 ? atr : attr, ch);
  return atr;
}


//  ------------------------------------------------------------------

void ieditor::cursoroff() {

  vcurhide();
}


//  ------------------------------------------------------------------

void ieditor::cursoron() {

  vcurshow();
}


//  ------------------------------------------------------------------

inline void ieditor::scrollup(int scol, int srow, int ecol, int erow, int lines) {

  editwin.scroll_box_up(srow, scol, erow, ecol, lines);
}


//  ------------------------------------------------------------------

inline void ieditor::scrolldown(int scol, int srow, int ecol, int erow, int lines) {

  editwin.scroll_box_down(srow, scol, erow, ecol, lines);
}


//  ------------------------------------------------------------------

void ieditor::gotorowcol(int col, int row) {

  GFTRK("Editgotorowcol");

  _test_haltab(col > window_maxcol, col, window_maxcol);
  _test_haltab(row > window_maxrow, row, window_maxrow);

  editwin.move_cursor(row, col);

  cursor_ccol = col;
  cursor_crow = row;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

extern ieditor* editor;

void stylecode_prints(int wrow, int wcol, int atr, char* str) {

  editor->editwin.prints(wrow,wcol,atr,str);
}


//  ------------------------------------------------------------------

void ieditor::dispstring(const char* str, int row) {

  GFTRK("Editdispstring");

  _test_halt(str == NULL);
  _test_haltab(row > window_maxrow, row, window_maxrow);

  // Get string length
  int length = strlen(str);

  // String longer than window width?
  _test_haltab(length > (window_maxcol+1), length, (window_maxcol+1));
  length = minimum_of_two(length, (window_maxcol+1));

  // Buffer for translation to visual representation
  char buf[EDIT_BUFLEN];

  // Space-pad and nul-terminate the buffer
  memset(buf, ' ', window_maxcol+1);
  buf[window_maxcol+1] = NUL;

  // Copy/translate string into buffer
  char* bufptr = buf;
  int position = 0;
  const char* sptr = str;
  while(position < length) {
    switch(*sptr) {
      case ' ':   *bufptr = CFG->editcharspace;  break;
      case '\n':  *bufptr = CFG->editcharpara;   break;
      default:    *bufptr = *sptr;
    }
    position++;
    bufptr++;
    sptr++;
  }

  // Print the string
  editwin.prints(row, window_mincol, -1, buf);

  // Stylecode highlightning
  if(CFG->stylecodes)
    StyleCodeHighlight(str, row, window_mincol, (VidPutStrCP)stylecode_prints);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::setcolor(const glineref& line) {

  // Set window attribute
  if(line.isblock())
    editwin.text_color(C_READA);
  else if(line.ishidden())
    editwin.text_color(C_READK);
  else if(line.iskludge())
    editwin.text_color(C_READK);
  else if(line.istagline())
    editwin.text_color(C_READG);
  else if(line.istearline())
    editwin.text_color(C_READT);
  else if(line.isorigin())
    editwin.text_color(C_READO);
  else if(line.isquote())
    editwin.text_color(C_READQ);
  else
    editwin.text_color(C_READW);
}


//  ------------------------------------------------------------------

void ieditor::displine(const glineref& line, int row) {

  GFTRK("Editdispline");

  _test_haltab(row > window_maxrow, row, window_maxrow);

  // Display line
  setcolor(line);
  dispstring(line.text(), row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::clreol(int col, int row) {

  GFTRK("Editclreol");

  if(col == -1)
    col = cursor_ccol;

  if(row == -1)
    row = cursor_crow;

  if((int)col <= window_maxcol)
    editwin.fill(row, col, row, window_maxcol, ' ', C_READW);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::refresh(glineref currline, int row) {

  GFTRK("Editrefresh");

  cursoroff();

  // Display as many lines as we can
  while((row <= window_maxrow) and currline.valid()) {
    displine(currline, row++);
    currline++;
  }

  // If we ran out of lines, blank the rest
  while(row <= window_maxrow)
    dispstring("", row++);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

int ieditor::downoneline(int row) {

  GFTRK("Editdownoneline");

  _test_haltab(row > window_maxrow, row, window_maxrow);

  if(row == window_maxrow)
    scrollup(window_mincol, window_minrow, window_maxcol, window_maxrow);
  else
    row++;

  gotorowcol(window_mincol, row);

  GFTRK(NULL);

  return row;
}


//  ------------------------------------------------------------------

void ieditor::GoEOL() {

  GFTRK("EditGoEOL");

  // Move cursor to the last char on the line
  cursor_col = cursor_line.length();
  if(cursor_col)
    cursor_col--;

  // String must not be longer than the window width
  _test_haltab(cursor_col > window_maxcol, cursor_col, window_maxcol);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::GoUp() {

  GFTRK("EditGoUp");

  _test_haltab(cursor_row < window_minrow, cursor_row, window_minrow);

  if(cursor_line) {

    cursor_line--;
    
    if(cursor_row == window_minrow) {
      scrolldown(window_mincol, cursor_row, window_maxcol, window_maxrow);
      displine(cursor_line, cursor_row);
    }
    else {
      cursor_row--;
    }

    if((cursor_col+1) > cursor_line.length())
      GoEOL();
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::GoDown() {

  GFTRK("EditGoDown");

  _test_haltab(cursor_row > window_maxrow, cursor_row, window_maxrow);

  if(cursor_line.more()) {

    cursor_line++;

    if(cursor_row == window_maxrow) {
      scrollup(window_mincol, window_minrow, window_maxcol, window_maxrow);
      displine(cursor_line, cursor_row);
    }
    else {
      cursor_row++;
    }

    if((cursor_col+1) > cursor_line.length())
      GoEOL();
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::GoLeft() {

  GFTRK("EditGoLeft");

  _test_haltab(cursor_col < window_mincol, cursor_col, window_mincol);

  if(cursor_col == window_mincol) {
    if(cursor_line) {
      GoUp();
      GoEOL();
    }
    cursor_col++;  // reverse of cursor_col-- below...
  }

  cursor_col--;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::GoRight() {

  GFTRK("EditGoRight");

  _test_haltab(cursor_col > window_maxcol, cursor_col, window_maxcol);

  char cursorchar = cursor_line.text()[cursor_col];

  if((cursor_col == window_maxcol) or (cursorchar == '\n') or (cursorchar == NUL)) {
    if(cursor_line.more()) {
      GoDown();
      cursor_col = window_mincol;
    }
    cursor_col--;  // reverse of cursor_col++ below
  }

  cursor_col++;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

int ieditor::wrapit(glineref& currline, int& curr_col, int& curr_row, int display) {

  int quotelen;
  char quotebuf[100];

  int curscol = curr_col;
  int cursrow = curr_row;
  int thisrow = curr_row;

  glineref thisline = currline;
  glineref lastadded = thisline;

  bool wrapped_above = false;

  // Start wrapping from the current line onwards
  while(thisline.valid()) {

    // Length of this line
    int thislen = thisline.length();

    int wrapmargin = thisline.isquote() ? marginquotes : margintext;

    // Does this line need wrapping?
    if(thislen > wrapmargin) {

      // Reset quote string length
      quotelen = 0;

      // Is this line quoted?
      if(thisline.isquote() and CFG__editquotewrap) {

        // Get quote string and length
        quoting.get_quotestring(thisline.text(), quotebuf, quotelen);
      }

      // wrapmargin = 40
      //                   1111111111222222222233333333334444444444
      //         01234567890123456789012345678901234567890123456789
      //         --------------------------------------------------
      //                                                v- wrapptr
      // Case 1: this is another test with a bit of text to wrap.
      // Case 2: this is a test with a line that need wrapping.
      // Case 3: thisxisxaxtestxwithxaxlinexthatxneedxwrapping.
      // Case 4: >thisxisxaxtestxwithxaxlinexthatxneedxwrapping.

      char text[256];
      strcpy(text, thisline.text());

      // Point to the last char inside the margin
      char* wrapptr = text + wrapmargin - 1;

      // Locate the word to be wrapped

      // Did we find a space?
      if(*wrapptr == ' ') {

        // Case 1: A space was found as the last char inside the margin
        //
        // Now we must locate the first word outside the margin.
        // NOTE: Leading spaces to this word will be nulled out!

        // Begin at the first char outside the margin
        wrapptr++;
      }
      else {

        // Case 2: A non-space was found as the last char inside the margin
        //
        // Now we must locate the beginning of the word we found.

        // Keep copy of original pointer
        char* atmargin = wrapptr;

        // Search backwards until a space or the beginning of the line is found
        while((wrapptr > text) and (*(wrapptr-1) != ' '))
          wrapptr--;

        // Check if we hit leading spaces
        char* spaceptr = wrapptr;
        if(spaceptr > text) {
          spaceptr--;
          while((spaceptr > text) and (*spaceptr == ' '))
            spaceptr--;
        }

        // Did we search all the way back to the beginning of the line?
        if((wrapptr == text) or (wrapptr == (text+quotelen)) or (*spaceptr == ' ')) {

          // Case 3: There are no spaces within the margin or we hit leading spaces

          // We have to break it up at the margin
          wrapptr = atmargin;
        }
      }

      // The wrapptr now points to the location to be wrapped or NUL

      // Is the line hard-terminated?
      if(strchr(wrapptr, '\n')) {

        // The line is hard-terminated.
        //
        // The wrapped part must be placed on a new line below.

        char wraptext[256];

        // Copy the quote string, if any, to the new line first
        if(quotelen)
          strcpy(wraptext, quotebuf);
        else
          *wraptext = NUL;

        // Copy/append the wrapped part to the new line
        strcat(wraptext, wrapptr);

        // Insert the new line
        glineref wrapline = thisline + 1;
        wrapline.insert(wraptext);

        // Make sure the type of the line is correct
        setlinetype(wrapline);

        // Link the new line in between this line and the next
        lastadded = wrapline;

        if(display) {

          // Is there at least one line below this one?
          if(thisrow < window_maxrow) {

            // Scroll the lines below to make room for the new line
            scrolldown(window_mincol, thisrow+1, window_maxcol, window_maxrow);

            // Display the new line
            if(wrapline.length() <= (window_maxcol+1))
              displine(wrapline, thisrow+1);
          }
        }
      }
      else {

        // The line is not hard-terminated
        //
        // The wrapped part must be inserted into the next line

        // Indicate that one or more lines were wrapped in this way
        wrapped_above = true;

        // Pointer to the next line
        glineref nextline = thisline + 1;

        // Flag to indicate if a new line was added below
        bool line_added_below = false;

        // Is there no next line or is the next line quoted?
        if(nextline.beyond_last() or nextline.isquote()) {

          // The wrapped part must be placed on a new line below

          // Allocate a new line
          glineref newline = thisline + 1;
          newline.insert("");

          nextline = newline;
          lastadded = newline;

          line_added_below = true;
        }

        // Get length of the wrapped part
        int wraplen = strlen(wrapptr);

        // Get length of the text of next line
        int nextlen = nextline.length();

        char nexttext[256];
        strcpy(nexttext, nextline.text());

        // Move the next line's text to make room for the wrapped part and the quote string, if any
        memmove(nexttext+quotelen+wraplen, nexttext, nextlen+1);

        // Copy the wrapped part
        memmove(nexttext+quotelen, wrapptr, wraplen);

        // Was this line quoted?
        if(quotelen) {

          // Copy the quote string
          memmove(nexttext, quotebuf, quotelen);
        }

        nextline.set_text(nexttext);

        // Make sure the type of the line is correct
        setlinetype(nextline);

        if(display) {

          // Is there at least one line below this one?
          if(line_added_below and (thisrow < window_maxrow)) {

            // Scroll the lines below to make room for the new line
            scrolldown(window_mincol, thisrow+1, window_maxcol, window_maxrow);
          }

          // Display the new/wrapped line
          if(((thisrow+1) <= window_maxrow) and (nextline.length() <= (window_maxcol+1)))
            displine(nextline, thisrow+1);
        }
      }

      // Nul-terminate at the wrapping location
      *wrapptr = NUL;

      // Was this line quoted?
      if(quotelen) {

        // Trim spaces off the end of the line
        char* trimptr = text + strlen(text) - 1;
        while(*trimptr == ' ')
          *trimptr-- = NUL;

        // Append a new linefeed
        strcat(text, "\n");
      }

      thisline.set_text(text);

      // Make sure the line type still is correct
      setlinetype(thisline);

      if(display) {

        // Display this line after wrapping
        if(thisrow <= window_maxrow)
          displine(thisline, thisrow);
      }

      // If we are on the cursor line, check if the cursor char was wrapped
      if(thisrow == curr_row) {

        // Pointer to the cursor char
        char* cursptr = text + curr_col;

        // If the cursor char was wrapped, move the cursor coords
        if(wrapptr <= cursptr) {
          curscol = (int)(cursptr - wrapptr) + quotelen;
          cursrow++;
        }
      }
    }

    // If this line is hard-terminated, we have finished wrapping
    // Unless the next line has grown too large
    if(strchr(thisline.text(), '\n')) {
      if(thisline.at_last())
        break;
      glineref nextline = thisline + 1;
      wrapmargin = nextline.isquote() ? marginquotes : margintext;
      if(nextline.length() <= wrapmargin)
        break;
    }

    // Go to the next line
    thisline++;
    thisrow++;
  }

  if(display) {

    // Display the current line after wrapping
    if(curr_row <= window_maxrow)
      displine(currline, curr_row);

    // Was the line or lines above wrapped?
    if(wrapped_above) {
      
      // Display the last line in the paragraph
      if(thisrow <= window_maxrow)
        displine(thisline, thisrow);
    }
  }

  // Move to the next line if the cursor was wrapped
  if(cursrow != curr_row) {
    currline++;
    thisrow++;
    if(cursrow > window_maxrow) {
      cursrow = window_maxrow;
      scrollup(window_mincol, window_minrow, window_maxcol, window_maxrow);
      displine(currline, cursor_row);
    }
  }

  // Update cursor position
  curr_row = cursrow;
  curr_col = curscol;

  GFTRK(NULL);

  return lastadded;
}


//  ------------------------------------------------------------------

int ieditor::wrapdel(glineref& currline, int& curr_col, int& curr_row, int display) {

  GFTRK("Editwrapdel");

  return wrapit(currline, curr_col, curr_row, display);
} 


//  ------------------------------------------------------------------

int ieditor::wrapins(glineref& currline, int& curr_col, int& curr_row, int display) {

  GFTRK("Editwrapins");

  return wrapit(currline, curr_col, curr_row, display);
}


//  ------------------------------------------------------------------

void ieditor::insertchar(char ch) {

  GFTRK("Editinsertchar");

  // Insert or overwrite the char
  if(insert) {
    char buf[256];
    strcpy(buf, cursor_line.text());
    strinschr(buf, ch, cursor_col);
    cursor_line.set_text(buf);
  }
  else {
    int currline_len = cursor_line.length();
    _test_haltab(cursor_col > currline_len, cursor_col, currline_len);
    if((cursor_line.text()[cursor_col] != '\n') and (cursor_line.text()[cursor_col] != NUL)) {
      cursor_line.set_text()[cursor_col] = ch;
    }
    else {
      char buf[256];
      strcpy(buf, cursor_line.text());
      strinschr(buf, ch, cursor_col);
      cursor_line.set_text(buf);
    }
  }

  // Make sure the line type still is correct
  setlinetype(cursor_line);

  // Move cursor
  cursor_col++;

  // Wrap
  wrapins(cursor_line, cursor_col, cursor_row);

  // Adjust cursor position and display if necessary
  if(cursor_col > window_maxcol) {
    if(cursor_line.more()) {
      cursor_line++;
      cursor_col = window_mincol;
      cursor_row++;
      if(cursor_row > window_maxrow) {
        cursor_row = window_maxrow;
        scrollup(window_mincol, window_minrow, window_maxcol, window_maxrow);
        displine(cursor_line, cursor_row);
      }
    }
    else {
      cursor_col = window_maxcol;
    }
  }

  gotorowcol(cursor_col, cursor_row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::DelChar() {

  GFTRK("EditDelChar");

  char buf[256];
  glineref thisline = cursor_line;
  glineref nextline = cursor_line + 1;
  int thislen = thisline.length();

  // Cannot delete at or beyond the nul-terminator
  if(cursor_col < thislen) {
    strcpy(buf, thisline.text());
    strdelchr(buf, cursor_col);
    thisline.set_text(buf);
  }

  // Did we delete the last char on the line or
  // was the cursor at or beyond the nul-terminator?
  // And is there a next line at all?
  if(((cursor_col+1) >= thislen) and nextline.more()) {

    // Join the next line to this line

    // Is the next line quoted?
    // And is the cursor column non-zero?
    int quotelen = 0;
    if(nextline.isquote() and CFG__editquotewrap and cursor_col) {

      // Get quote string length
      char dummybuf[100];
      quoting.get_quotestring(nextline.text(), dummybuf, quotelen);
    }

    // Copy the next line's text to this line
    // Skip past the next line's quote string and blanks, if any
    strcpy(buf, thisline.text());
    const char* nextptr = nextline.text() + quotelen;
    if(not (nextline.isquote() and (cursor_col == 0))) {
      while(*nextptr == ' ')
        nextptr++;
    }
    strcat(buf, nextptr);
    thisline.set_text(buf);

    // Remove the next line
    nextline.erase();
  }

  // Make sure the line type still is correct
  setlinetype(thisline);

  // Rewrap this line
  wrapdel(cursor_line, cursor_col, cursor_row);

  refresh(cursor_line, cursor_row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::DelLeft() {

  GFTRK("EditDelLeft");

  // Cannot backspace from the first column on the first line in the msg
  if(cursor_line == 0)
    if(cursor_col == window_mincol)
      return;

  // Go left(/up) and delete the character there
  GoLeft();
  DelChar();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::GoWordLeft() {

  GFTRK("EditGoWordLeft");

  if(cursor_col == 0) {
    if(cursor_line) {
      GoUp();
      GoEOL();
    }
    GFTRK(NULL);
    return;
  }
  
  cursor_col--;

  if(!isxalnum(cursor_line.text()[cursor_col])) {
    while(!isxalnum(cursor_line.text()[cursor_col]) and (cursor_col > 0))
      cursor_col--;
    while(isxalnum(cursor_line.text()[cursor_col]) and (cursor_col > 0))
      cursor_col--;
  }
  else {
    while(isxalnum(cursor_line.text()[cursor_col]) and (cursor_col > 0))
      cursor_col--;
  }

  if(cursor_col != 0)
    cursor_col++;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::GoWordRight() {

  GFTRK("EditGoWordRight");

  if((cursor_line.text()[cursor_col] == NUL) or (cursor_line.text()[cursor_col] == '\n')) {
    if(cursor_line.more()) {
      GoDown();
      cursor_col = 0;
    }
    GFTRK(NULL);
    return;
  }

  if(!isxalnum(cursor_line.text()[cursor_col])) {
    while(!isxalnum(cursor_line.text()[cursor_col]) and ((cursor_col+1) <= cursor_line.length()))
      cursor_col++;
  }
  else {
    while(isxalnum(cursor_line.text()[cursor_col]) and ((cursor_col+1) <= cursor_line.length()))
      cursor_col++;
    while(!isxalnum(cursor_line.text()[cursor_col]) and ((cursor_col+1) <= cursor_line.length()))
      cursor_col++;
  }

  if(cursor_line.text()[cursor_col-1] == '\n')
    cursor_col--;

  if(cursor_line.text()[cursor_col] == NUL) {
    if(cursor_line.more()) {
      GoDown();
      cursor_col = 0;
    }
    else {
      cursor_col--;
    }
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::Newline() {

  GFTRK("EditNewline");

  char text[256];
  strcpy(text, cursor_line.text());

  // Pointer to the split position
  char* splitptr = text + cursor_col;

  // Buffer for the second part of the split line
  char splitbuf[256] = { "" };

  // If the split line was quoted, get the quotestring
  // But do not get it if the cursor points to a linefeed or is
  int quotelen = 0;
  if(quoting.is_quote(text))
    quoting.get_quotestring(text, splitbuf, quotelen);

  // Eliminate the quotestring if
  // - the cursor points to a linefeed or
  // - the cursor is located inside the quotestring
  if(quotelen and ((*splitptr == '\n') or (*splitptr == NUL) or (cursor_col < quotelen)))
    *splitbuf = NUL;

  // Append the second part to the split buffer
  strcat(splitbuf, splitptr);

  // Copy linefeed+nul to the split position
  strcpy(splitptr, "\n");

  // Re-type and display the split line
  cursor_line.set_text(text);
  setlinetype(cursor_line);
  displine(cursor_line, cursor_row);

  // Insert a new line below
  // Set the line text to the split-off part
  cursor_line.insert_below(splitbuf);
  setlinetype(cursor_line);

  // Move down the cursor
  cursor_col = 0;
  cursor_row = downoneline(cursor_row);
  gotorowcol(cursor_col, cursor_row);

  // Scroll the remaining lines if necessary
  if(cursor_row < window_maxrow)
    scrolldown(window_mincol, cursor_row, window_maxcol, window_maxrow);

  // Rewrap the split-off line
  wrapdel(cursor_line, cursor_col, cursor_row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::CopyAboveChar() {

  GFTRK("EditCopyAboveChar");

  char ch = ' ';
  if(cursor_line) {
    glineref above = cursor_line - 1;
    const char* ptr = above.text();
    int len = strlen(ptr);
    if(len and len > cursor_col)
      ch = ptr[cursor_col];
    if(ch == '\n')
      ch = ' ';
  }
  insertchar(ch);
  
  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::DupLine() {

  GFTRK("EditDupLine");

  cursor_line.next().insert(cursor_line);
  GoDown();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------
//  PageUp behavior:
//
//  ... The top line becomes the bottom line.
//  ... Always remain at cursor row, except if we can't page up.
//  ... If we can't page up, move the cursor row to the top line.

void ieditor::GoPgUp() {

  GFTRK("EditGoPgUp");

  // Not at the first line in msg?
  if(cursor_line) {

    // Count lines
    int count = cursor_row;

    // Move to the top line currently displayed
    glineref topline = cursor_line;
    while(count and topline) {
      topline--;
      count--;
    }

    // Are we already fully at the top?
    if(topline == 0) {

      // Yes, so move the cursor row to the top line
      cursor_row = window_minrow;
      cursor_line = topline;
    }
    else {

      // We are not at the top, so continue with paging

      // Move a full page of lines, if possible
      count = window_maxrow;
      while(count and topline) {
        topline--;
        count--;
      }

      // Refresh display
      refresh(topline, window_minrow);

      // Set the current line
      count = cursor_row;
      cursor_line = topline;
      while(count--)
        cursor_line++;
    }
  }
  else {
    GoTopMsg();
  }

  if((cursor_col+1) > cursor_line.length())
    GoEOL();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------
//  PageDown behavior:
//
//  ... The bottom line becomes the top line.
//  ... Always remain at cursor row, except if at the last line.
//  ... If at the last line, move cursor to the last window line and
//      display the lines above.
//  ... If there are too few lines to display after the page down, the
//      rest are displayed blank.

void ieditor::GoPgDn() {

  GFTRK("EditGoPgDn");

  // Not at the last line in the msg?
  if(cursor_line.more()) {

    // Go down to the last displayed line in the window
    int newrow = cursor_row;
    while((newrow < window_maxrow) and cursor_line.more()) {
      cursor_line++;
      newrow++;
    }

    // If there are more lines after the last line, start displaying from the top
    if(cursor_line.more()) {
      refresh(cursor_line, window_minrow);

      // Set current line
      newrow = 0;
      while((newrow < cursor_row) and cursor_line.more()) {
        cursor_line++;
        newrow++;
      }

      // Move cursor row if necessary
      if(newrow < cursor_row)
        cursor_row = newrow;
    }
    else {
      cursor_row = newrow;
    }
  }
  else {
    GoBotMsg();
  }

  if((cursor_col+1) > cursor_line.length())
    GoEOL();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::Tab() {

  GFTRK("EditTab");

  int tabsz = CFG->disptabsize ? CFG->disptabsize : 1;

  // If at a tab position, move one position beyond it
  if((cursor_col % tabsz) == 0) {
    if(insert)
      insertchar(' ');
    else
      GoRight();
  }

  // Move to the next tab position
  while(cursor_col % tabsz) {
    if(insert)
      insertchar(' ');
    else if(cursor_line.text()[cursor_col] != '\n')
      GoRight();
    else
      break;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::DeleteEOL() {

  GFTRK("EditDeleteEOL");

  bool has_linefeed = strchr(cursor_line.text(),'\n') ? true : false;

  char text[256];
  strcpy(text, cursor_line.text());
  text[cursor_col] = NUL;

  if(has_linefeed)
    strcat(text, "\n");

  cursor_line.set_text(text);

  clreol();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::deleteline(int zapquotesbelow) {

  GFTRK("Editdeleteline");

  while(1) {

    if(zapquotesbelow and not (cursor_line.isquote() or strblank(cursor_line.text())))
      break;

    // Add the deleted line to the killbuffer
    killbuf.add(cursor_line);

    // Delete the line
    cursor_line.erase();

    // If all lines were deleted, allocate a blank one
    if(msg.linearray.lines() == 0)
      msg.linearray.add("");

    // Move the cursor to EOL if necessary
    if((cursor_col+1) > cursor_line.length())
      GoEOL();

    if(not zapquotesbelow)
      break;
  }

  // Refresh display from cursor row
  refresh(cursor_line, cursor_row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::UnDelete() {

  GFTRK("EditUnDelete");

  // If there are deleted lines
  int lastkill = killbuf.lines();
  if(lastkill) {

    glineref undeleteline = killbuf;
    undeleteline += killbuf.lines() - 1;

    cursor_line.insert(undeleteline);

    undeleteline.erase();

    // Move the cursor to EOL if necessary
    if((cursor_col+1) > cursor_line.length())
      GoEOL();

    refresh(cursor_line, cursor_row);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::ZapQuoteBelow() {

  GFTRK("ZapQuoteBelow");

  deleteline(true);
  Newline();
  GoUp();
}


//  ------------------------------------------------------------------

glineref ieditor::findtopline() {

  GFTRK("Editfindtopline");

  int toprow = cursor_row;
  glineref topline = cursor_line;

  while(topline and (toprow > window_minrow)) {
    topline--;
    toprow--;
  }

  GFTRK(NULL);

  return topline;
}


//  ------------------------------------------------------------------

void ieditor::savefile(int status) {

  GFTRK("Editsavefile");

  // Open the save file
  const char* editorfile = AddPath(CFG->goldpath, CFG->editfile);
  remove(editorfile);
  gfile fp;
  fp.fopen(editorfile, "wb", CFG->sharemode);
  if(fp.isopen()) {

    // Turn off cursor and put up a wait window
    bool wascursoron = not vcurhidden();
    cursoroff();
    w_info(CFG->language[LNG_Wait]);

    // First save the "unfinished" identifier
    if(status == STATUS_UPDATE) {
      fp.fputs(unfinished);
      fp.fputs("\r\n");
    }

    // Save as whole paragraphs
    glineref saveline = msg.linearray;
    while(saveline.valid()) {

      // Copy the line to a buffer
      char buf[EDIT_BUFLEN];
      strcpy(buf, saveline.text());

      // If a LF was found, replace it with a CR/LF combo
      char* lfptr = strchr(buf, '\n');
      if(lfptr)
        strcpy(lfptr, "\r\n");

      // Save the line
      fp.fputs(buf);

      saveline++;
    }

    // Close save file and remove wait window
    fp.fclose();
    w_info(NULL);
    if(wascursoron)
      cursoron();
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::SaveFile() {

  GFTRK("EditSaveFile");

  savefile(STATUS_UPDATE);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::SaveMsg() {

  GFTRK("EditSaveMsg");

  done = STATUS_SAVE;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

bool ieditor::isempty(const glineref& line) {

  return (*line.text() == '\n') or (*line.text() == NUL);
}


//  ------------------------------------------------------------------

bool ieditor::reflowok(char* qstr) {

  // Stop reflow if there is no next line
  if(cursor_line.at_last())
    return false;

  glineref nextline = cursor_line + 1;

  // Stop reflow if the next line is empty
  if(isempty(nextline))
    return false;

  // Stop reflow if the next line is a control line
  if(nextline.type() & gline::control)
    return false;

  // Stop reflow if the quotestring on the next line is not the same
  int qlen2;
  char qstr2[100];
  quoting.get_quotestring(nextline.text(), qstr2, qlen2);
  if(not strieql(qstr, qstr2))
    return false;

  return true;
}


//  ------------------------------------------------------------------

void ieditor::Reflow() {

  GFTRK("EditReflow");

  // Skip empty lines
  while(isempty(cursor_line)) {
    if(cursor_line.more())
      GoDown();
    else
      return;
  }

  // Get the first quotestring
  int qlen1;
  char qstr1[100];
  quoting.get_quotestring(cursor_line.text(), qstr1, qlen1);

  // Strip leading spaces from the first line
  char text[256];
  strcpy(text, cursor_line.text());
  char* ptr = strskip_wht(text + qlen1);
  memmove(text+qlen1, ptr, strlen(ptr)+1);
  cursor_line.set_text(text);

  // Perform the reflow
  while(reflowok(qstr1)) {

    // Work on the current line until it is done
    int thisline = cursor_line;
    while(thisline == cursor_line) {

      // Stop reflow?
      if(not reflowok(qstr1))
        break;

      // Go to the EOL, insert a space and delete the LF
      GoEOL();
      insertchar(' ');
      DelChar();
    }
  }

  // Go to the next line
  displine(cursor_line, cursor_row);
  GoDown();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::ExitMsg() {

  GFTRK("EditExitMsg");

  done = STATUS_DROP;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::DelLine() {

  GFTRK("EditDelLine");

  cursoroff();
  deleteline();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::ToUpper() {

  GFTRK("EditToUpper");

  cursor_line.set_text()[cursor_col] = to_upper(cursor_line.text()[cursor_col]);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::ToLower() {

  GFTRK("EditToLower");

  cursor_line.set_text()[cursor_col] = to_lower(cursor_line.text()[cursor_col]);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::ToggleCase() {

  GFTRK("EditToggleCase");

  if(is_upper(cursor_line.text()[cursor_col]))
    cursor_line.set_text()[cursor_col] = to_lower(cursor_line.text()[cursor_col]);
  else
    cursor_line.set_text()[cursor_col] = to_upper(cursor_line.text()[cursor_col]);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::LookupCursor() {

  GFTRK("EditLookupCursor");

  gstring name = cursor_line.text() + cursor_col;
  LookupNode(msg, name, LOOK_NAME, area);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::LookupDest() {

  GFTRK("EditLookupDest");

  gstring name = "";
  LookupNode(msg, name, LOOK_DEST, area);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::LookupOrig() {

  GFTRK("EditLookupOrig");

  gstring name = "";
  LookupNode(msg, name, LOOK_ORIG, area);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::Soundkill() {

  HandleGEvent(EVTT_STOPVOICE);
}


//  ------------------------------------------------------------------

void ieditor::statusline() {

  if(chartyped) {
    for(int i=0; i<CFG->editcompletion.size(); i++) {
      const gstring& trig = CFG->editcompletion[i].trigger;
      int tlen = trig.length();
      if(cursor_col >= tlen) {
        if(trig.nieql(cursor_line.text()+cursor_col-tlen, tlen)) {
          int n;
          for(n=0; n<tlen; n++)
            DelLeft();
          const char* cptr = CFG->editcompletion[i].text.c_str();
          int clen = CFG->editcompletion[i].text.length();
          for(n=0; n<clen; n++)
            insertchar(*cptr++);
          break;
        }
      }
    }
  }

  gstring buf;

  for(int i=0; i<CFG->editcomment.size(); i++) {
    const gstring& trig = CFG->editcomment[i].trigger;
    int tlen = trig.length();
    if(cursor_col >= tlen) {
      if(trig.nieql(cursor_line.text()+cursor_col-tlen, tlen)) {
        buf = CFG->editcomment[i].text.c_str();
        break;
      }
    }
  }

                            // L1:20, C17
                            // Line 1/20, 17.
                            // Line 1 of 20, col 17.
                            // Line 1 of 20, column 17.
  #define LNG__EditStatusline "Line %i of %i, col %i. %s"
  update_statuslinef(LNG__EditStatusline, 1+cursor_line.lineno(), msg.linearray.lines(), 1+cursor_col, buf.c_str());
  if(buf and CFG->beepcomment) {
    HandleGEvent(EVTT_EDITCOMMENT);
  }
}


//  ------------------------------------------------------------------

int ieditor::handlekey(Key key) {

  switch(key) {
    case KK_EditAbort:            Abort();              break;
    case KK_EditAskExit:          AskExit();            break;
    case KK_EditClearDeleteBuf:   ClearDeleteBuf();     break;
    case KK_EditClearPasteBuf:    ClearPasteBuf();      break;
    case KK_EditCopyAboveChar:    CopyAboveChar();      break;
    case KK_EditDelChar:          DelChar();            break;
    case KK_EditDeleteEOL:        DeleteEOL();          break;
    case KK_EditDelLeft:          DelLeft();            break;
    case KK_EditDelLine:          DelLine();            break;
    case KK_EditDelLtWord:        DelLtWord();          break;
    case KK_EditDelRtWord:        DelRtWord();          break;
    case KK_EditDosShell:         DosShell();           break;
    case KK_EditDupLine:          DupLine();            break;
    case KK_EditExitMsg:          ExitMsg();            break;
    case KK_EditExportText:       ExportText();         break;
    case KK_EditGoBegLine:        GoBegLine();          break;
    case KK_EditGoBotLine:        GoBotLine();          break;
    case KK_EditGoBotMsg:         GoBotMsg();           break;
    case KK_EditGoDown:           GoDown();             break;
    case KK_EditGoEOL:            GoEOL();              break;
    case KK_EditGoLeft:           GoLeft();             break;
    case KK_EditGoPgDn:           GoPgDn();             break;
    case KK_EditGoPgUp:           GoPgUp();             break;
    case KK_EditGoRight:          GoRight();            break;
    case KK_EditGoTopLine:        GoTopLine();          break;
    case KK_EditGoTopMsg:         GoTopMsg();           break;
    case KK_EditGoUp:             GoUp();               break;
    case KK_EditGoWordLeft:       GoWordLeft();         break;
    case KK_EditGoWordRight:      GoWordRight();        break;
    case KK_EditHeader:           Header();             break;
    case KK_EditImportQuotebuf:   ImportQuotebuf();     break;
    case KK_EditImportText:       ImportText();         break;
    case KK_EditLoadFile:         LoadFile();           break;
    case KK_EditLookupCursor:     LookupCursor();       break;
    case KK_EditLookupDest:       LookupDest();         break;
    case KK_EditLookupOrig:       LookupOrig();         break;
    case KK_EditNewline:          Newline();            break;
    case KK_EditQuitNow:          QuitNow();            break;
    case KK_EditReflow:           Reflow();             break;
    case KK_EditSaveFile:         SaveFile();           break;
    case KK_EditSaveMsg:          SaveMsg();            break;
    case KK_EditSoundkill:        Soundkill();          break;
    case KK_EditSpellCheck:       SpellCheck();         break;
    case KK_EditTab:              Tab();                break;
    case KK_EditToggleCase:       ToggleCase();         break;
    case KK_EditToggleInsert:     ToggleInsert();       break;
    case KK_EditToLower:          ToLower();            break;
    case KK_EditToUpper:          ToUpper();            break;
    case KK_EditUndefine:                               break;
    case KK_EditUnDelete:         UnDelete();           break;
    case KK_EditZapQuoteBelow:    ZapQuoteBelow();      break;

    // Block functions
    case KK_EditAnchor:           BlockAnchor();        break;
    case KK_EditCopy:             BlockCopy();          break;
    case KK_EditCut:              BlockCut();           break;
    case KK_EditPaste:            BlockPaste();         break;

    default:
      if(not PlayMacro(key, KT_E))
        return false;
  }

  return true;
}


//  ------------------------------------------------------------------

int IsMacro(Key key, int type);

int ieditor::Start(int mode, int& position, GMsg& msgref, gactivearea* areaptr) {

  GFTRK("EditStart");

  msg = msgref;
  area = areaptr;
  msgmode = mode;
  quitnow = false;
  cursor_col = window_mincol;
  cursor_row = window_minrow;
  cursor_line = msg.linearray;

  margintext = CFG->dispmargin;
  marginquotes = CFG->editquotemargin;
  if(marginquotes > margintext)
    marginquotes = margintext;

  whelppcat(H_Editor);

  if(msg.linearray.size() == 0)
    msg.linearray.add("");

  cursor_line = 0;
  while(cursor_line.valid()) {
    if((cursor_line.ishidden() and not area->viewhidden()) or (cursor_line.iskludge() and not area->viewkludge()))
      cursor_line.erase();
    else {
      char text[256];
      strtrimspace(strcpy(text, cursor_line.text()));
      if(not cursor_line.ishard())
        strcat(text, " ");
      else
        strcat(text, "\n");
      cursor_line.set_text(text);
      cursor_line++;
    }
  }

  // Check if there is an unfinished backup message
  gfile fp;
  fp.fopen(AddPath(CFG->goldpath, CFG->editfile), "rt", CFG->sharemode);
  if(fp.isopen()) {
    char buf[EDIT_BUFLEN];
    fp.fgets(buf, sizeof(buf));
    fp.fclose();
    if(striinc(unfinished, buf)) {
      w_info(CFG->language[LNG_UnfinishedMsg]);
      update_statusline(CFG->language[LNG_LoadUnfinished]);
      HandleGEvent(EVTT_ATTENTION);
      Key ch = getxch();
      w_info(NULL);
      if(ch != Key_Esc) {
        LoadFile();
        position = 0;
        remove(AddPath(CFG->goldpath, CFG->editfile));
      }
    }
  }

  if(position) {
    for(int posrow=0; posrow < position; posrow++) {
      if(cursor_line.more()) {
        cursor_line++;
        cursor_row++;
      }
    }
  }

  done = 0;

  // If the starting line is outside the first screenful
  if(position >= (window_maxrow+1)) {
    refresh(cursor_line-1, window_minrow);
    cursor_row = 1;
  }
  else {
    glineref zeroline = msg.linearray;
    refresh(zeroline, window_minrow);
  }

  gotorowcol(window_mincol, window_minrow);
  dispins();

  Time lasttime = gtime(NULL);

  while(not done) {

    statusline();

    gotorowcol(cursor_col, cursor_row);

    int backattr = dispchar(cursor_line.text()[cursor_col], C_READC);

    cursoron();
    if(insert)
      vcursmall();
    else
      vcurlarge();

    Key ch, cht;

    do {
      ch = getxchtick();
      cht = ch;

      if(CFG->editautosave) {
        Time thistime = gtime(NULL);
        if(thistime >= (lasttime+CFG->editautosave)) {
          lasttime = thistime;
          SaveFile();
        }
      }

      if(ch == Key_Tick)
        CheckTick(KK_EditQuitNow);

    } while(ch == Key_Tick);

    int ismacro = false;
    Key kk = SearchKey(cht, EditKey, EditKeys);
    if(kk) {
      ch = kk;
    }
    else {
      Key mkey = ch;
      if(mkey & 0xFF)
        mkey = (Key)(mkey & 0xFF);
      ismacro = IsMacro(mkey, KT_E);
      if(ismacro)
        ch = mkey;
    }

    if((ch < KK_Commands) and not ismacro) {
      if(cht & 0xFF) {
        ch &= 0xFF;
      }
    }

    dispchar(cursor_line.text()[cursor_col], backattr);

    chartyped = false;
    if((ch < KK_Commands) and (ch & 0xFF) and not ismacro) {
      chartyped = true;
      ch &= 0xFF;
      insertchar((char)ch);
    }
    else {
      handlekey(ch);
    }
  }

  cursoroff();

  savefile(quitnow ? STATUS_UPDATE : STATUS_SAVE);
  whelpop();

  w_info(CFG->language[LNG_Wait]);

  position = cursor_line.lineno();

  // Prune killbuffer
  if(killbuf.lines()) {
    int count = CFG->editundelete;
    if(killbuf.lines() > count)
      killbuf.erase(0, killbuf.lines()-count);
  }

  cursor_line = 0;
  while(cursor_line.valid()) {
    if(strchr(cursor_line.text(), '\n'))
      cursor_line.type() = gline::hard;
    else if(cursor_line.more() and cursor_line.next().isquote())
      cursor_line.type() = gline::hard;
    else
      cursor_line.type() = 0;
    char text[256];
    strtrimspace(strcpy(text, cursor_line.text()));
    cursor_line.set_text(text);
    cursor_line++;
  }

  GFTRK(NULL);

  return done;
}


//  ------------------------------------------------------------------

