
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geedit.h,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  The Internal Editor (IE).
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Screen coordinate models
//  ------------------------------------------------------------------
//
//  00    01
//  01   Msg# :                     02
//  02   From :                     03
//  03   To   :                     04
//  04   Subj :                     05
//  05    06
//  06  00  EDIT_MINROW             07  EDIT_MIN_ROW
//  07  01                          08
//  08  02                          09
//  09  03                          10
//  10  04                          11
//  11  05                          12
//  12  06                          13
//  13  07                          14
//  14  08                          15
//  15  09                          16
//  16  10                          17
//  17  11                          18
//  18  12                          19
//  19  13                          10
//  10  14                          21
//  21  15                          22
//  22  16                          23
//  23  17  EDIT_MAXROW             24  EDIT_MAX_ROW
//  24  GoldED xxxx (statusline)   25
//                                 
//                                  one-based old-style coord
//      editor window corrdinates
//    actual screen coordinates
//
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Defines

#define EDIT_BUFLEN       256
#define EDIT_PARABUFLEN   2048


//  ------------------------------------------------------------------
//  Globals

extern bool CFG__editquotewrap;

extern gstring Edit__exportfilename;


//  ------------------------------------------------------------------
//  Reality check define and function

#ifndef NDEBUG
#define _test_halt(t)         if(t) { debugtest(#t, 0, 0, __FILE__, __LINE__, false); }
#define _test_haltab(t, a, b) if(t) { debugtest(#t, a, b, __FILE__, __LINE__, true);  }
#endif


//  ------------------------------------------------------------------
//  Internal Editor Class

class ieditor {

public:

  gwindow editwin;       // window

  //  ----------------------------------------------------------------

  static glinearray killbuf;
  static glinearray pastebuf;


protected:

  //  ----------------------------------------------------------------
  //  Editor window data

  int    win_mincol;     // First column
  int    win_minrow;     // First row
  int    win_maxcol;     // Last column
  int    win_maxrow;     // Last row
  int    win_border;     // Border type
  int    win_hasborder;  // 1 == window has a border, 0  == no border


  //  ----------------------------------------------------------------
  //  Window-relative minimum/maximum values

  int  window_mincol;
  int  window_minrow;
  int  window_maxcol;
  int  window_maxrow;


  //  ----------------------------------------------------------------
  //  Cursor coordinates

  int cursor_col;
  int cursor_row;
  int cursor_ccol;
  int cursor_crow;


  //  ----------------------------------------------------------------
  //  Block coordinates

  bool column_block;
  int  block_begin_row;
  int  block_begin_col;
  int  block_end_row;
  int  block_end_col;


  //  ----------------------------------------------------------------
  //  Misc.

  gmsg_quote quoting;
  int    chartyped;
  glineref cursor_line;
  glineref anchor;
  int    done;
  bool   insert;
  int    marginquotes;
  int    margintext;
  int    msgmode;
  GMsg&  msg;
  bool   quitnow;
  char*  unfinished;
  gactivearea* area;


  //  ----------------------------------------------------------------
  //  Internal helper functions

  void   clreol          (int col=-1, int row=-1);
  void   cursoroff       ();
  void   cursoron        ();
  void   deleteline      (int zapquotesbelow=false);
  int    dispchar        (char ch, int attr=-1);
  void   dispins         ();
  void   displine        (const glineref& line, int row);
  void   dispstring      (const char* string, int row);
  int    downoneline     (int row);
  void   export          (glineref exportline, int endat);
  int    findanchor      ();
  int    findfirstline   ();
  glineref findtopline   ();
  void   getthisrow      (int currline);
  void   gotorowcol      (int col, int row);
  int    handlekey       (Key key);
  void   import          (glineref line, gstring& filename);
  void   imptxt          (gstring& filename);
  void   insertchar      (char ch);
  int    insertlinebelow (int currline, char* text=NULL);
  bool   isempty         (const glineref& line);
  bool   reflowok        (char* qstr);
  void   refresh         (glineref currline, int row);
  void   savefile        (int status);
  void   scrolldown      (int scol, int srow, int ecol, int erow, int lines=1);
  void   scrollup        (int scol, int srow, int ecol, int erow, int lines=1);
  void   setcolor        (const glineref& line);
  void   setlinetype     (glineref& line);
  void   statusline      ();
  void   strdelchr       (char* string, int position);
  void   strinschr       (char* string, char ch, int position);
  void   windowclose     ();
  void   windowopen      ();
  int    wrapit          (glineref& currline, int& curr_col, int& curr_row, int display=YES);
  int    wrapdel         (glineref& currline, int& curr_col, int& curr_row, int display=YES);
  int    wrapins         (glineref& currline, int& curr_col, int& curr_row, int display=YES);

  #ifndef NDEBUG
  void   debugtest       (char* test, int a, int b, char* file, int line, int values);
  #endif


public:

  //  ----------------------------------------------------------------
  //  Constructor and destructor

  ieditor(GMsg& msgref, int scol, int ecol, int srow, int erow, int border);
  ~ieditor();
  

  //  ----------------------------------------------------------------
  //  Function to start the editor

  int Start(int mode, int& position, GMsg& msgref, gactivearea* areaptr);


  //  ----------------------------------------------------------------
  //  User key functions

  void Abort          ();
  void AskExit        ();
  void BlockAnchor    ();
  void BlockCopy      ();
  void BlockCut       ();
  void BlockPaste     ();
  void ClearDeleteBuf ();
  void ClearPasteBuf  ();
  void CopyAboveChar  ();
  void DelChar        ();
  void DeleteEOL      ();
  void DelLeft        ();
  void DelLine        ();
  void DelLtWord      ();
  void DelRtWord      ();
  void DosShell       ();
  void DupLine        ();
  void ExitMsg        ();
  void ExportText     ();
  void GoBegLine      ();
  void GoBotLine      ();
  void GoBotMsg       ();
  void GoDown         ();
  void GoEOL          ();
  void GoLeft         ();
  void GoPgDn         ();
  void GoPgUp         ();
  void GoRight        ();
  void GoTopLine      ();
  void GoTopMsg       ();
  void GoUp           ();
  void GoWordLeft     ();
  void GoWordRight    ();
  void Header         ();
  void ImportQuotebuf ();
  void ImportText     ();
  void LoadFile       ();
  void LookupCursor   ();
  void LookupDest     ();
  void LookupOrig     ();
  void Newline        ();
  void QuitNow        ();
  void Reflow         ();
  void SaveFile       ();
  void SaveMsg        ();
  void Soundkill      ();
  void SpellCheck     ();
  void Tab            ();
  void ToggleCase     ();
  void ToggleInsert   ();
  void ToLower        ();
  void ToUpper        ();
  void UnDelete       ();
  void ZapQuoteBelow  ();


  //  ----------------------------------------------------------------
};


//  ------------------------------------------------------------------

