
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geedit2.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  The Internal Editor (IE).
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include "geedit.h"


//  ------------------------------------------------------------------
//  Constructor

ieditor::ieditor(GMsg& msgref, int scol, int ecol, int srow, int erow, int border)

  : msg(msgref), quoting(msgref.area->quotechars().c_str())

{

  win_mincol = scol;
  win_minrow = srow;
  win_maxcol = ecol;
  win_maxrow = erow;
  win_border = border;

  win_hasborder = (win_border == 5) ? 0 : 1;

  window_mincol  = 0;
  window_minrow  = 0;
  window_maxcol  = win_maxcol - win_mincol - (2*win_hasborder);
  window_maxrow  = win_maxrow - win_minrow - (2*win_hasborder);

  cursor_col  = 0;
  cursor_row  = 0;
  cursor_ccol = 0;
  cursor_crow = 0;

  cursor_line  = msg.linearray;
  anchor       = msg.linearray;

  chartyped    = false;
  done         = 0;
  insert       = true;
  marginquotes = 0;
  margintext   = 0;
  msgmode      = 0;
  quitnow      = false;

  unfinished   = "+$!$+ GoldED Internal Editor: Unfinished Message!";

  windowopen();
}


//  ------------------------------------------------------------------
//  Destructor

ieditor::~ieditor() {

  windowclose();
}


//  ------------------------------------------------------------------

void ieditor::windowopen() {

  // Open editor window without clearing the window area
  #define STYLE_NOCLEAR -1
  int tmp = gwin.style;
  gwin.style = STYLE_NOCLEAR;
  editwin.open(win_minrow, win_mincol, win_maxrow, win_maxcol, 5, C_READB, C_READW, C_READPB);
  gwin.style = tmp;
}


//  ------------------------------------------------------------------

void ieditor::windowclose() {

  // Close editor window without removing the window itself
  editwin.unlink();
}


//  ------------------------------------------------------------------

void Edit__killpastebuf() {

  ieditor::pastebuf.reset();
}


//  ------------------------------------------------------------------

void Edit__killkillbuf() {

  ieditor::killbuf.reset();
}


//  ------------------------------------------------------------------

void FreePastebuf() {

  Edit__killpastebuf();
  Edit__killkillbuf();
}


//  ------------------------------------------------------------------

void ieditor::ClearDeleteBuf() {

  GFTRK("EditClearDeleteBuf");

  killbuf.reset();
  HandleGEvent(EVTT_JOBDONE);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::ClearPasteBuf() {

  GFTRK("EditClearPasteBuf");

  pastebuf.reset();
  HandleGEvent(EVTT_JOBDONE);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::GoBegLine() {

  GFTRK("EditGoBegLine");

  cursor_col = window_mincol;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::DelLtWord() {

  GFTRK("EditDelLtWord");

  if(cursor_col == 0) {
    DelLeft();
    GFTRK(NULL);
    return;
  }

  char buf[256];
  strcpy(buf, cursor_line.text());

  char* bptr = buf;
  char* ptr  = bptr + cursor_col;
  char* ptr2 = ptr--;

  // test test test test     test test test test
  // test test test test ,   test test test test
  // test test test test ,,  test test test test

  if(*ptr == ' ') {
    while((*ptr == ' ') and (bptr < ptr))
      ptr--;
    if(*ptr != ' ')
      ptr++;
  }
  else if(isxalnum(*ptr)) {
    while(isxalnum(*ptr) and (bptr < ptr))
      ptr--;
    while((*ptr == ' ') and (bptr < ptr))
      ptr--;
    if((*ptr != ' ') and (bptr < ptr))
      ptr++;
  }
  else {
    DelLeft();
    GFTRK(NULL);
    return;
  }

  cursor_col -= (int)(ptr2-ptr);

  memmove(ptr, ptr2, strlen(ptr2)+1);
  cursor_line.set_text(buf);

  wrapdel(cursor_line, cursor_col, cursor_row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::DelRtWord() {

  GFTRK("EditDelRtWord");

  if((cursor_line.text()[cursor_col+1] == NUL) or (cursor_line.text()[cursor_col+1] == '\n')) {
    DelChar();
    GFTRK(NULL);
    return;
  }

  char buf[256];
  strcpy(buf, cursor_line.text());
  
  char* ptr = buf + cursor_col;
  char* ptr2 = ptr;

  // test test test test,   test test test test
  // test test test test,   test test test test

  if(*ptr == ' ') {
    while((*ptr == ' ') and (*ptr != '\n') and *ptr)
      ptr++;
  }
  else if(isxalnum(*ptr)) {
    // Delete word
    while(isxalnum(*ptr) and (*ptr != '\n') and *ptr)
      ptr++;
    // Delete spaces after word
    while((*ptr == ' ') and (*ptr != '\n') and *ptr)
      ptr++;
  }
  else {
    DelChar();
    GFTRK(NULL);
    return;
  }

  memmove(ptr2, ptr, strlen(ptr2)+1);
  cursor_line.set_text(buf);

  wrapdel(cursor_line, cursor_col, cursor_row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::GoTopMsg() {

  GFTRK("EditGoTopMsg");

  cursor_line = 0;
  refresh(cursor_line, window_minrow);
  cursor_col = window_mincol;
  cursor_row = window_minrow;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::GoBotMsg() {

  GFTRK("EditGoBotMsg");

  cursor_col = window_mincol;

  // Go to the last line in the msg
  cursor_line.go_last();

  // Pointer to the line to display at the top of the window
  glineref topline = cursor_line;

  // The new cursor row
  cursor_row = minimum_of_two(cursor_line.lineno(), window_maxrow);

  // How many lines to go back to get the top line
  int count = cursor_row;

  // Go back to get the top line
  while(count-- and topline)
    topline--;

  // Refresh the display
  refresh(topline, window_minrow);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::GoTopLine() {

  GFTRK("EditGoTopLine");

  int count = cursor_row;

  while(count-- and cursor_line)
    cursor_line--;

  cursor_col = window_mincol;
  cursor_row = window_minrow;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::GoBotLine() {

  GFTRK("EditGoBotLine");

  while((cursor_row < window_maxrow) and cursor_line.more()) {
    cursor_line++;
    cursor_row++;
  }

  cursor_col = window_mincol;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

bool ieditor::findanchor() {

  GFTRK("Editfindanchor");

  anchor = 0;

  // Search all lines to find the anchor (a line with a block mark)
  while(not anchor.isblock() and anchor.more())
    anchor++;

  GFTRK(NULL);

  return anchor.isblock();
}


//  ------------------------------------------------------------------

void ieditor::BlockAnchor() {

  GFTRK("EditBlockAnchor");

  // Is there an anchor already?
  if(findanchor()) {

    // Yes, so replace it with the current line

    // Remove block mark
    anchor.type() &= ~gline::block;

    // Is the old anchor different from the current line?
    if(anchor != cursor_line) {

      // Set the new anchor
      anchor = cursor_line;
      anchor.type() |= gline::block;

      // Find the line at the top
      glineref topline = findtopline();

      // Refresh display to remove the block color on the old anchor line
      // Just in case the old anchor line is visible
      refresh(topline, window_minrow);
    }

    // Remove the old contents of the paste buffer
    pastebuf.reset();
  }
  else {

    // There was no anchor, so mark the current line as the new anchor
    cursor_line.type() |= gline::block;
  }

  displine(cursor_line, cursor_row);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::BlockCopy() {

  GFTRK("EditBlockCopy");

  // Is there an anchor?
  if(findanchor()) {

    glineref first = cursor_line < anchor ? cursor_line : anchor;
    glineref last  = cursor_line < anchor ? anchor : cursor_line;
    int      lines = 1 + last.lineno() - first.lineno();

    // Copy lines to the paste buffer
    pastebuf.reset();
    pastebuf.add(first, lines);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::BlockCut() {

  GFTRK("EditBlockCut");

  // Find the anchor
  if(findanchor()) {

    glineref firstcutline = cursor_line;
    glineref lastcutline = cursor_line;
    int firstcutlinerow = cursor_row;
    int lastcutlinerow = cursor_row;

    // Search below to find the anchor line
    while(lastcutline.more() and (lastcutline != anchor)) {
      lastcutline++;
      lastcutlinerow++;
    }

    // Was the anchor line above or on the current line?
    if(lastcutline != anchor) {

      // The last cut line is the current line
      lastcutline = cursor_line;
      lastcutlinerow = cursor_row;

      // Search above to find the anchor line
      while(firstcutline and (firstcutline != anchor)) {
        firstcutline--;
        if(firstcutlinerow)
          firstcutlinerow--;
      }
    }

    // The firstcutline and lastcutline pointers
    // are now pointing where they should

    int lines = 1 + lastcutline.lineno() - firstcutline.lineno();

    // If there is a next line for the last cut line
    if(lastcutline.more()) {

      // Set the current line to it
      cursor_line = firstcutline;
      cursor_row = lastcutlinerow + 1;
    }
    else {

      // Set the current line to the previous line for the first cut line
      cursor_line = firstcutline;
      cursor_line--;
      cursor_row = firstcutlinerow ? firstcutlinerow - 1 : 0;
    }

    // Remove the old paste buffer and point to the new cut
    pastebuf.reset();
    pastebuf.add(firstcutline, lines);
    firstcutline.erase(lines);

    // Refresh the display
    if(not in_range(cursor_row, window_minrow, window_maxrow)) {
      cursor_row = window_minrow;
      refresh(cursor_line, window_minrow);
    }
    else {
      cursor_row = maximum_of_two(firstcutlinerow, window_minrow);
      glineref topline = findtopline();
      refresh(topline, window_minrow);
    }
    cursor_col = window_mincol;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::BlockPaste() {

  GFTRK("EditBlockPaste");

  if(pastebuf.lines()) {

    glineref pasteline = pastebuf;
    glineref nextline = cursor_line + 1;
    nextline.insert(pasteline);

    glineref topline = findtopline();
    refresh(topline, window_minrow);
    cursor_col = window_mincol;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::LoadFile() {

  GFTRK("EditLoadFile");

  cursoroff();
  w_info(CFG->language[LNG_Wait]);

  msg.load_textfile(AddPath(CFG->goldpath, CFG->editfile), margintext-1, true);

  cursor_line = 0;
  if(streql(unfinished, cursor_line.text()))
    cursor_line.erase();

  char buf[256];
  while(cursor_line.valid()) {
    strcpy(buf, cursor_line.text());
    strtrimspace(buf);
    if(cursor_line.ishard())
      strcat(buf, "\n");
    else
      strcat(buf, " ");
    cursor_line.set_text(buf);
    cursor_line++;
  }

  w_info(NULL);

  cursor_line = 0;
  cursor_col = window_mincol;
  cursor_row = window_minrow;
  refresh(cursor_line, window_minrow);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

inline char uuencode_enc(int c) { return (char)(c ? (c & 077) + ' ': '`'); }

void ieditor::import(glineref line, gstring& filename) {

  GFTRK("Editimport");

  // Save the unfinished msg first of all
  savefile(STATUS_UPDATE);

  update_statusline(CFG->language[LNG_ImportFile]);

  // Set initial import filename or wildcards
  if(filename) {
    area->set_inputfile(filename);
  }
  else {
    if(area->inputfile().empty())
      area->set_inputfile(GOLD_WILDCARD_ALL_FILES);
  }

  CFG->inputfile = area->inputfile();
  gstring inputfile = CFG->inputfile;

  if(edit_string(inputfile, sizeof(Path), CFG->language[LNG_ImportWhichFile], H_ImportFile)) {

    CFG->inputfile = inputfile.c_str();
    area->set_inputfile(CFG->inputfile);

    // Pointer to the filename string
    Path filenamebuf;
    char* filenameptr = strcpy(filenamebuf, area->inputfile().c_str());

    // Should the imported text be quoted or uuencoded?
    bool quoteit = false;
    bool uuencode = false;
    if(*filenameptr == '>') {
      filenameptr++;
      quoteit = true;
    }
    else if(*filenameptr == '#') {
      filenameptr++;
      uuencode = true;
    }

    // Check file aliases
    for(int z=0; z<CFG->filealias.size(); z++) {
      if(CFG->filealias[z].alias.ieql(filenameptr)) {
        strcpy(filenameptr, CFG->filealias[z].file.c_str());
        break;
      }
    }

    // Check for wildcards

    // Is the filename a directory?
    if(is_dir(filenameptr)) {

      // Does the filename contain wildcards?
      if(not strpbrk(filenameptr, "*?")) {

        // Add match-all wildcards
        strcat(AddBackslash(filenameptr), GOLD_WILDCARD_ALL_FILES);
      }
    }

    // Pointer to selected filename or NULL if no file is selected
    char* selectedfile = filenameptr;

    // Does the filename contain wildcards?
    if(strpbrk(filenameptr, "*?")) {

      // Set selection window title and statusline
      set_title(CFG->language[LNG_ImportTitle], TCENTER, C_MENUT);
      update_statuslinef(CFG->language[LNG_ImportStatus], filenameptr);

      // Copy filename with wildcards to a temp buffer
      Path filenamebuf;
      strcpy(filenamebuf, filenameptr);

      // Start the file picker
      selectedfile = wpickfile(
        win_minrow,
        win_mincol,
        win_maxrow,
        win_maxcol,
        W_BMENU,
        C_MENUB,
        C_MENUW,
        C_MENUS,
        false,
        filenamebuf,
        filenameptr,
        maketitle
      );
    }

    if(selectedfile) {

      // Open the file
      gfile fp;
      fp.fopen(filenameptr, uuencode ? "rb" : "rt", CFG->sharemode);
      if(fp.isopen()) {

        update_statuslinef(CFG->language[LNG_ImportStatus], filenameptr);

        // Allocate paragraph read buffer
        char* parabuf = (char*)throw_malloc(EDIT_PARABUFLEN);

        // Add import begin text, if any
        if(*CFG->importbegin) {
          sprintf(parabuf, "%s\n", CFG->importbegin.c_str());
          strischg(parabuf, "@file", CleanFilename(filenameptr));
          line.insert_below(parabuf);
          setlinetype(line);
        }

        if(uuencode) {

          sprintf(parabuf, "begin 644 %s\n", CleanFilename(filenameptr));
          line.insert_below(parabuf);
          setlinetype(line);

          while(1) {
            char ibuf[80];
            char* iptr = ibuf;
            char* optr = parabuf;
            int n = fp.fread(ibuf, 1, 45);
            if(n < 45)
              memset(ibuf+n, 0, 45-n);
            *optr++ = uuencode_enc(n);
            for(int i=0; i<n; i+=3,iptr+=3) {
              *optr++ = uuencode_enc(*iptr >> 2);
              *optr++ = uuencode_enc(((*iptr << 4) & 060) | ((iptr[1] >> 4) & 017));
              *optr++ = uuencode_enc(((iptr[1] << 2) & 074) | ((iptr[2] >> 6) & 03));
              *optr++ = uuencode_enc(iptr[2] & 077);
            }
            *optr++ = '\n';
            *optr = NUL;
            line.insert_below(parabuf);
            setlinetype(line);
            if(n <= 0)
              break;
          }

          line.insert_below("end\n");
          setlinetype(line);
        }
        else {

          // Read paragraphs
          while(fp.fgets(parabuf, EDIT_PARABUFLEN-3)) {

            // Invalidate tearline
            if(*CFG->invalidate.tearline[0])
              strischg(parabuf, CFG->invalidate.tearline[0].c_str(), CFG->invalidate.tearline[1].c_str());

            // Invalidate originline
            if(*CFG->invalidate.origin[0])
              strischg(parabuf, CFG->invalidate.origin[0].c_str(), CFG->invalidate.origin[1].c_str());

            // Invalidate SEEN-BY's
            if(*CFG->invalidate.seenby[0])
              strischg(parabuf, CFG->invalidate.seenby[0].c_str(), CFG->invalidate.seenby[1].c_str());

            // Invalidate kludge chars
            //strchg(parabuf, CTRL_A, '@');

            // Insert a quotestring if asked
            if(quoteit)
              strins(" > ", parabuf, 0);

            // Replace tabs with single spaces
            strchg(parabuf, HT, ' ');

            // Translate softcr to configured char
            if(CFG->editsoftcrxlat)
              strchg(parabuf, SOFTCR, CFG->editsoftcrxlat);

            // Copy the paragraph to the new line and retype it
            line.insert_below(parabuf);
            setlinetype(line);

            // If the paragraph is longer than one line
            uint wrapmargin = line.isquote() ? marginquotes : margintext;
            if(line.length() >= wrapmargin) {

              // Wrap it
              int tmpcol = 0;
              int tmprow = 0;
              line = wrapins(line, tmpcol, tmprow, false);
            }
          }
        }

        // Add import end text, if any
        if(*CFG->importend or *CFG->importbegin) {
          sprintf(parabuf, "%s\n", CFG->importend ? CFG->importend.c_str() : CFG->importbegin.c_str());
          strischg(parabuf, "@file", CleanFilename(filenameptr));
          line.insert_below(parabuf);
          setlinetype(line);
        }

        throw_free(parabuf);

        fp.fclose();
      }
      else {
        w_infof(CFG->language[LNG_CouldNotOpen], filenameptr);
        waitkeyt(10000);
        w_info(NULL);
      }
    }
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::imptxt(gstring& filename) {

  GFTRK("Editimptxt");

  import(cursor_line, filename);
  refresh(cursor_line, cursor_row);
  cursor_col = window_mincol;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::ImportText() {

  GFTRK("EditImportText");

  gstring tmp;
  imptxt(tmp);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::ImportQuotebuf() {

  GFTRK("EditImportQuotebuf");

  gstring quotebuf;

  GetQuotebufName(quotebuf, area);
  imptxt(quotebuf);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::export(glineref exportline, int endat) {

  GFTRK("Editexport");

  update_statusline(CFG->language[LNG_ExportFile]);

  if(edit_string(Edit__exportfilename, sizeof(Path), CFG->language[LNG_ExportWhatFile], H_ExportFile)) {

    // Pointer to export filename
    const char* filenameptr = Edit__exportfilename.c_str();

    // Is append requested?
    if(*filenameptr == '+')
      filenameptr++;

    gfile fp;
    fp.fopen(filenameptr, (Edit__exportfilename[0] == '+') ? "at" : "wt", CFG->sharemode);
    if(fp.isopen()) {

      update_statuslinef(CFG->language[LNG_ExportStatus], Edit__exportfilename.c_str());
      fp.fputc('\n');

      while(exportline.valid() and (endat ? exportline != cursor_line : 1)) {
        fp.fputs(exportline.text());
        if(not strchr(exportline.text(), '\n'))
          fp.fputc('\n');
        exportline++;
      }

      fp.fclose();
    }
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::SpellCheck() {

  GFTRK("EditSpellCheck");

  char buf[EDIT_BUFLEN];
  char buf2[EDIT_BUFLEN];

  savefile(STATUS_SAVE);
  strcpy(buf, CFG->editspellchecker.c_str());
  strischg(buf, "@File", AddPath(CFG->goldpath, CFG->editfile));
  sprintf(buf2, CFG->language[LNG_SpellChecker], buf);
  ShellToDos(buf, buf2, LGREY|_BLACK, true, true, false, area);
  LoadFile();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::ExportText() {

  GFTRK("EditExportText");

  if(pastebuf.lines()) {
    export(pastebuf, false);
    GFTRK(NULL);
    return;
  }

  if(findanchor()) {
    export(anchor, true);
    GFTRK(NULL);
    return;
  }

  glineref firstline = msg.linearray;
  export(firstline, false);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::DosShell() {

  GFTRK("EditDosShell");

  ShellToDos(getenv("COMSPEC"), CFG->language[LNG_DOS_Shell], LGREY|_BLACK, YES, true, false, area);
  cursoron();
  cursoroff();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::dispins() {

  GFTRK("Editdispins");

  if(insert)
    HeaderView.window.prints(5, gvid.numcols-6, C_HEADT, CFG->language[LNG_Ins]);
  else {
    char lbuf[6];
    memset(lbuf, _box_table[W_BHEAD][1], 5);
    lbuf[5] = NUL;
    HeaderView.window.prints(5, gvid.numcols-6, C_HEADB, lbuf);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::ToggleInsert() {

  GFTRK("EditToggleInsert");

  insert = !insert;
  dispins();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::Header() {

  GFTRK("EditHeader");

  windowclose();
  EditHeaderinfo(msgmode, HeaderView);
  windowopen();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::Abort() {

  GFTRK("EditAbort");

  cursoroff();
  GMenuDropmsg MenuDropmsg;
  if(MenuDropmsg.Run())
    done = STATUS_DROP;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::AskExit() {

  GFTRK("EditAskExit");

  cursoroff();
  GMenuQuit MenuQuit;
  gkbd.quitall = MenuQuit.Run();
  if(gkbd.quitall) {
    GMenuDropmsg MenuDropmsg;
    if(MenuDropmsg.Run())
      done = STATUS_DROP;
    else
      done = STATUS_SAVE;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ieditor::QuitNow() {

  GFTRK("EditQuitNow");

  quitnow = CFG->timeoutsavemsg ? false : true;
  done = CFG->timeoutsavemsg ? STATUS_SAVE : STATUS_DROP;
  gkbd.quitall = true;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

ieditor* editor = NULL;

int EditMsg(int mode, int& position, GMsg& msg, gactivearea* area) {

  editor = new ieditor(msg, 0, CFG->dispmargin-1, 6, gvid.numrows-2, 5);
  throw_new(editor);
  int retval = editor->Start(mode, position, msg, area);
  delete editor;
  return retval;
}


//  ------------------------------------------------------------------

