
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gefile.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  File handling.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif


//  ------------------------------------------------------------------

int fspecs = 0;
int specfiles = 0;
gfilespec* fspec = NULL;
gactivearea* fs_area = NULL;


//  ------------------------------------------------------------------

char CFG__frqinvfilechars[40] = { "\"()+,.\\/:;<=>[]| " };
char CFG__frqskipwordchars[40] = { "0123456789-[" };

inline char* invalidfilechar(char ch) { return strchr(CFG__frqinvfilechars, ch); }


//  ------------------------------------------------------------------

void CreateAutoMsg(GMsg& msg, gactivearea* area) {

  // Create message
  char* _txt;
  if(msg.attr.att())
    _txt = CFG->language[LNG_AutoAttachMsg];
  else if(msg.attr.frq())
    _txt = CFG->language[LNG_AutoRequestMsg];
  else if(msg.attr.urq())
    _txt = CFG->language[LNG_AutoUpdreqMsg];
  else
    _txt = CFG->language[LNG_EmptyMsg];
  msg.txt = (char*)throw_realloc(msg.txt, strlen(_txt)+256);
  strcpy(msg.txt, _txt);
  TokenXlat(MODE_NEW, msg.txt, msg, msg, area);
}


//  ------------------------------------------------------------------

void CreateFileMsgs(int mode, GMsg& msg, gactivearea* area) {

  register int m, n, x;
  int modex, msgsdone=0;
  gstring buf;
  gstring subj;

  if(specfiles > 1) {
    if(msg.attr.att())
      w_progress(PROGRESS_NEW, C_INFOW, 1, specfiles, CFG->language[LNG_File_Attach]);
    else if(msg.attr.frq())
      w_progress(PROGRESS_NEW, C_INFOW, 1, specfiles, CFG->language[LNG_File_Request]);
    else if(msg.attr.urq())
      w_progress(PROGRESS_NEW, C_INFOW, 1, specfiles, CFG->language[LNG_File_Updreq]);
  }

  // Do ZoneGating
  do_zonegating(msg, area);

  if(msg.odest.net == 0)
    msg.odest = msg.dest;

  char* safe_copy = throw_strdup(msg.txt);   // Keep a copy of the original
  modex = (mode == MODE_CHANGE) ? GMSG_UPDATE : GMSG_NEW;
  for(n=0,x=0; x<fspecs; x++) {
    for(m=0; m<fspec[x].files; m++,n++) {
      if(msg.attr.att()) {
        if(specfiles > 1) {
          w_progress(PROGRESS_UPDATE, C_INFOW, n+1, specfiles, CFG->language[LNG_File_Attach]);
          update_statuslinef(CFG->language[LNG_FileAttaching], n+1, specfiles,
            msg.dest.zone, msg.dest.net, msg.dest.node, msg.dest.point
          );
        }
      }
      else if(msg.attr.frq()) {
        if(specfiles > 1) {
          w_progress(PROGRESS_UPDATE, C_INFOW, n+1, specfiles, CFG->language[LNG_File_Request]);
          update_statuslinef(CFG->language[LNG_FileRequesting], n+1, specfiles,
            msg.dest.zone, msg.dest.net, msg.dest.node, msg.dest.point
          );
        }
      }
      else if(msg.attr.urq()) {
        if(specfiles > 1) {
          w_progress(PROGRESS_UPDATE, C_INFOW, n+1, specfiles, CFG->language[LNG_File_Updreq]);
          update_statuslinef(CFG->language[LNG_FileUpdreqing], n+1, specfiles,
            msg.dest.zone, msg.dest.net, msg.dest.node, msg.dest.point
          );
        }
      }
      subj  = fspec[x].delsent ? "^" : "";
      subj += fspec[x].path;
      subj += fspec[x].fblk ? (fspec[x].fblk[m].name ? fspec[x].fblk[m].name : "") : "";
      subj += *fspec[x].password ? " " : "";
      subj += fspec[x].password;
      subj += " ";
      subj.upper();
      if((buf.length() + subj.length()) > 71) {
        if(msgsdone) {
          modex = GMSG_NEW;
          area->newmsgno(msg);

          // Create message

          throw_release(msg.txt);
          CreateAutoMsg(msg, area);
          msg.text_to_lines(CFG->dispmargin);
        }

        replace_controllines(mode, msg, area);

        msg.re = buf.trim();

        HeaderView.Use(area, &msg);
        HeaderView.Paint();
        msg.lines_to_text();
        area->savemsg(modex, msg);
        msgsdone++;
        buf = subj;
      }
      else {
        buf += subj;
      }
    }
  }
  if(not buf.blank()) {
    if(msgsdone) {
      modex = GMSG_NEW;
      area->newmsgno(msg);

      // Create message

      throw_release(msg.txt);
      CreateAutoMsg(msg, area);
      msg.text_to_lines(CFG->dispmargin);
    }

    replace_controllines(mode, msg, area);

    msg.re = buf.trim();

    HeaderView.Use(area, &msg);
    HeaderView.Paint();
    msg.lines_to_text();
    area->savemsg(modex, msg);
  }

  throw_release(msg.txt);
  msg.txt = safe_copy;
  msg.text_to_lines(CFG->dispmargin);
  if(specfiles > 1)
    w_progress(PROGRESS_QUIT, 0, 0, 0, NULL);
}


//  ------------------------------------------------------------------

static int _stdc FreqCmp(const char** a, const char** b) {

  return stricmp(*a, *b);
}


//  ------------------------------------------------------------------

static bool frqchkdesc(gstring& desc) {

  if(desc.length() == 0)
    desc = " ";
  else {
    const char* ptr = desc.c_str();
    while(*ptr) {
      if(isalnum(*ptr))
        break;
      ptr++;
    }
    if(*ptr == NUL) {
      desc = "";
      return true;
    }
  }
  return false;
}


//  ------------------------------------------------------------------

static bool frqgetfile(gstring& file, gstring& desc, char* txt) {

  char* ptr = txt;

  // Scan filename for first invalid char (.extension)
  while(*ptr and not invalidfilechar(*ptr))
    ptr++;

  // General file extension finder
  if(*ptr++ == '.') {
    if(not invalidfilechar(*ptr)) {

      // Find end of extension
      while(*ptr and (*ptr != ' '))
        ptr++;

      // Strip invalid chars from the end of the extension
      while(invalidfilechar(*(ptr-1)))
        ptr--;

      // Copy the filename
      file.copyn(txt, (uint)(ptr-txt));

      // Get description
      if(*ptr) {
        if(isspace(*ptr)) {
          ptr = strskip_wht(ptr);
        }
        else {
          ptr = strskip_txt(ptr);
          ptr = strskip_wht(ptr);
        }
        while(strchr(CFG__frqskipwordchars, *ptr) and *ptr) {
          ptr = strskip_txt(ptr);
          ptr = strskip_wht(ptr);
        }
        desc = ptr;
      }
      return frqchkdesc(desc);
    }
  }

  return false;
}


//  ------------------------------------------------------------------

void FileRequest(GMsg& msg, gactivearea* area) {

  GFTRK("FileRequest");

  int oldtopline = reader_topline;
  ulong msgno = msg.msgno;

  if(area->msgnset.size() and msg.display.size()) {

    char buf[256];
    char buf2[256];
    char* ptr;
    char* ptr1;
    char* ptr2;
    char* txtptr;
    char** freqfile = NULL;
    int gotticket = false;
    int getnextdesc = false;
    uint freqfiles = 0;
    int esc = YES;
    uint n;
    gstring file;
    gstring desc;

    w_info(CFG->language[LNG_Wait]);

    // Get a freqable file from the GIF kludge
    if(msg.gif.length()) {
      file = msg.gif + ".GIF";
      desc = msg.by_name();
      freqfile = (char**)throw_realloc(freqfile, (freqfiles+3)*sizeof(char*));
      sprintf(buf, " %-12.12s  %-60.60s ", file.c_str(), desc.c_str());
      freqfile[freqfiles] = throw_strdup(buf);
      desc = file = "";
      freqfiles++;
    }

    // Scan the current msg for file announcements
    uint tline = reader_topline;
    if(CFG->frqoptions & FREQ_FROMTOP)
      tline = 0;
    for(n=tline; n<msg.display.size(); n++) {

      // Don't look in control info for files
      if(msg.linearray.data(msg.display[n]).type & (GLINE_HIDD|GLINE_KLUD|GLINE_TEAR|GLINE_ORIG))
        continue;

      ptr = txtptr = strcpy(buf2, msg.linearray[msg.display[n]]);

      // Skip past whitespace and highbit junk
      while(((*ptr < '!') or (*ptr > '\x7F')) and *ptr)
        ptr++;

      // Get description if Ticket was detected in the previous line
      if(gotticket) {
        desc = strskip_wht(strskip_txt(ptr));
        if(desc.length() == 0)
          desc = " ";
        gotticket = false;
      }

      // Get description from file of previous line
      if(getnextdesc) {
        desc = ptr;
        getnextdesc = frqchkdesc(desc);
      }

      // Get filename from typical announcement formats
      if((file.length() == 0) and (strnieql(ptr, "File", 4) or strnieql(ptr, "Name", 4))) {
        ptr1 = strskip_txt(ptr);
        ptr2 = strskip_wht(ptr1);
        if((*(ptr1-1) == ':') or (*ptr2 == ':')) {
          if(*ptr2 == ':')
            ptr1 = strskip_wht(ptr2+1);
          else
            ptr1 = ptr2;
          ptr2 = strskip_txt(ptr1);
          *ptr2 = NUL;
          frqgetfile(file, desc, ptr1);
          desc = "";  // Description never comes before filename
          continue;
        }
      }

      // Get description from typical announcement formats
      if((desc.length() == 0) and (strnieql(ptr, "Desc", 4) or strnieql(ptr, "Note", 4))) {
        ptr1 = strskip_txt(ptr);
        ptr2 = strskip_wht(ptr1);
        if((*(ptr1-1) == ':') or (*ptr2 == ':')) {
          if(*ptr2 == ':')
            ptr1 = strskip_wht(ptr2+1);
          else
            ptr1 = ptr2;
          desc = ptr1;
          getnextdesc = frqchkdesc(desc);
        }
      }

      // Check for announcement format "Area:" and skip it if found
      if(strnieql(ptr, "Area", 4)) {
        ptr1 = strskip_txt(ptr);
        ptr2 = strskip_wht(ptr1);
        if((*(ptr1-1) == ':') or (*ptr2 == ':')) {
          //if(*ptr2 == ':')
          //  ptr1 = strskip_wht(ptr2+1);
          continue;
        }
      }

      // Get filename from Ticket announcement if detected
      if((file.length() == 0) and (strlen(txtptr) >= 54)) {
        if(strneql(txtptr+45, " Origin: ", 9)) {
          if(/*(*/ strinc(" Bytes) ", txtptr+25)) {
            if(txtptr[16] == '/' and txtptr[19] == '/' and txtptr[24] == '(' /*)*/) {
              ptr1 = strskip_wht(txtptr);
              ptr2 = strskip_txt(ptr1);
              *ptr2 = NUL;
              frqgetfile(file, desc, ptr1);
              desc = "";  // Description never comes before filename
              gotticket = true;
              continue;
            }
          }
        }
      }

      // Handle blank lines
      if(strblank(ptr)) {
        if(file.length())
          desc = " ";
      }

      // Get file based on extension (general algorithm)
      if(file.length() == 0) {
        if((ptr-txtptr) <= 4)
          getnextdesc = frqgetfile(file, desc, ptr);
      }

      // Get file based on FRQEXT list
      if(file.length() == 0) {
        bool gotone = false;
        ptr2 = strchr(ptr, '.');
        while(ptr2) {
          for(int e=0; e<CFG->frqext.size(); e++) {
            if(strnicmpw(CFG->frqext[e], ptr2, minimum_of_two(strlen(CFG->frqext[e]), strlen(ptr2))) == 0) {
              // Find beginning of filename
              char* ptr3 = ptr2-1;
              while((ptr3 > ptr) and not invalidfilechar(*ptr3))
                ptr3--;
              if(invalidfilechar(*ptr3))
                ptr3++;
              getnextdesc = frqgetfile(file, desc, ptr3);
              if(file.length())
                gotone = true;
              break;
            }
          }
          if(gotone)
            break;
          ptr2 = strchr(ptr2+1, '.');
        }
      }

      // Do we have a complete announcement?
      if(file.length() and desc.length()) {

        // Yes, so add it to the list
        freqfile = (char**)throw_realloc(freqfile, (freqfiles+3)*sizeof(char*));
        sprintf(buf, " %-12s  %s", file.c_str(), desc.c_str());
        strsetsz(buf, 76);
        freqfile[freqfiles] = throw_strdup(buf);
        desc = file = "";
        freqfiles++;
      }
    }

    // Add the filelist
    if(not (CFG->frqoptions & FREQ_NOFILES)) {
      file = "FILES";
      desc = gstrf(CFG->language[LNG_FilelistFrom], msg.by_name().c_str());
      freqfile = (char**)throw_realloc(freqfile, (freqfiles+3)*sizeof(char*));
      sprintf(buf, " %-12.12s  %-60.60s ", file.c_str(), desc.c_str());
      freqfile[freqfiles] = throw_strdup(buf);
      desc = file = "";
      freqfiles++;
    }

    w_info(NULL);

    // Let user select the file(s) to be requested
    if(freqfile) {

      // NULL terminate list
      freqfile[freqfiles] = NULL;

      // Sort list if requested
      if(CFG->frqoptions & FREQ_SORT)
        qsort(freqfile, freqfiles, sizeof(char*), (StdCmpCP)FreqCmp);

      // Run the picker
      int items = minimum_of_two(freqfiles, (gvid.numrows-10));
      set_title(CFG->language[LNG_FreqMenuTitle], TCENTER, C_ASKT);
      update_statusline(CFG->language[LNG_FreqStat]);
      whelppcat(H_FileRequest);
      wpickstr_tag = true;
      int crsr = wpickstr(6, 0, 6+items+1, -1, W_BASK, C_ASKB, C_ASKW, C_ASKS, freqfile, 0, title_shadow);
      wpickstr_tag = false;
      int freqs = 0;
      esc = ((crsr == -1) and (gwin.werrno == W_ESCPRESS));
      whelpop();

      if(not esc) {

        // Open a FILES.BBS in the INBOUNDPATH
        const char* fbfn = AddPath(CFG->inboundpath, "FILES.BBS");
        int fh = sopen(fbfn, O_RDWR|O_CREAT|O_APPEND|O_TEXT, CFG->sharemode, S_IREAD|S_IWRITE);
        if(fh == -1) {
          w_infof(CFG->language[LNG_CouldNotOpen], fbfn);
          waitkeyt(10000);
          w_info(NULL);
        }

        // Handle picked files
        msg.re = "";
        for(n=0; n<freqfiles; n++) {
          if(*freqfile[n] != ' ') {
            msg.attr.frq1();
            ptr = freqfile[n]+1;      //  01234567890123456
            ptr2 = strskip_txt(ptr);
            *ptr2++ = NUL;
            ptr2 = strskip_wht(ptr2);
            msg.re += ptr;
            msg.re += " ";
            if(fh != -1) {
              sprintf(buf, "%-12s %s\n", ptr, ptr2);
              write(fh, buf, strlen(buf));
            }
            freqs++;
          }
        }
        if(freqs == 0) {
          // AARRRGGGHH!!  More bloody duplicate code :-((
          msg.attr.frq1();
          ptr = freqfile[crsr]+1;      //  01234567890123456
          ptr2 = strskip_txt(ptr);
          *ptr2++ = NUL;
          ptr2 = strskip_wht(ptr2);
          msg.re += ptr;
          msg.re += " ";
          if(fh != -1) {
            sprintf(buf, "%-12s %s\n", ptr, ptr2);
            write(fh, buf, strlen(buf));
          }
          freqs++;
        }
        msg.re.trim();

        // Close the FILES.BBS
        if(fh != -1)
          close(fh);
      }
    }
    else {
      w_info(CFG->language[LNG_FreqInfoNoFiles]);
      Key key = waitkeyt(10000);
      w_info(NULL);
      if(key == Key_Ent) {
        msg.attr.frq1();
        esc = false;
      }
    }

    if(not esc) {

      // Pick area and put the msg in it
      int destareaid = area->areaid();
      reader_topline = 0;
      area->set_attr().hex0();
      if(area->areafreqto().size()) {
        for(n=0; n<AL.areas(); n++) {
          if(AL[n]->echoid().ieql(area->areafreqto())) {
            destareaid = AL[n]->areaid();
            break;
          }
        }
      }
      if(not area->areafreqdirect())
        destareaid = AreaPick(CFG->language[LNG_FreqArea], 6, &destareaid);
      if(destareaid != -1) {
        gactivearea* destarea = AL.activate(destareaid);
        destarea->open();
        for(n=0; n<CFG->frqnodemaps; n++) {
          if(CFG->frqnodemap[n].from.equals(msg.orig)) {
            msg.orig = CFG->frqnodemap[n].to;
            break;
          }
        }
        ReplyMsg(msg, destarea);
        destarea->close();
        AL.deactivate(destarea);
      }
    }

    if(freqfile) {
      for(n=0; n<freqfiles; n++)
        throw_release(freqfile[n]);
      throw_free(freqfile);
    }

    if(gkbd.quitall)
      QuitNow();
  }

  msg.msgno = msgno;
  LoadMessage(msg, CFG->dispmargin-CFG->disppagebar, area);
  reader_topline = oldtopline;
  reader_keyok = YES;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

