
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gefind.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Search functions.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

const int MAX_FIND =  10;        // Max number of search strings


//  ------------------------------------------------------------------

static int searchpatterns(const char* str) {

  uint len = strlen(str);
  for(int n=0; n<find_pats; n++)
    if(strbmhsearch(str, len, &find_pattern[n]))
      return true;
  return false;
}


//  ------------------------------------------------------------------

int FindString(GMsg& msg, const char* prompt, int what, gactivearea* area) {

  GFTRK("FindString");

  static ulong __FAR lastfound = 0;

  if(find_pattern == NULL) {
    find_pattern = (BmhArg*)throw_calloc(MAX_FIND, sizeof(BmhArg));
    find_patstr = (BmhPat*)throw_calloc(MAX_FIND, sizeof(BmhPat));
  }

  char* ptr;
  char buf[100], buf2[100];
  int direction = DIR_NEXT;
  int reverse = false;
  int caseignore = true;
  uint last = area->lastread();

  int what2 = 0;
  while(strchr("!-+<>:=", *prompt)) {
    switch(*prompt) {
      case '!':
        reverse = true;
        break;
      case '-':
        direction = DIR_PREV;
        break;
      case '+':
        direction = DIR_NEXT;
        break;
      case '<':
        what2 |= GFIND_FROM;
        break;
      case '>':
        what2 |= GFIND_TO;
        break;
      case ':':
        what2 |= GFIND_SUBJECT;
        break;
      case '=':
        caseignore = false;
        break;
    }
    prompt++;
  }

  if(what2) {
    if(what & GFIND_HDR)
      what &= ~GFIND_HDR;
    what |= what2;
  }

  find_pats = 0;
  strcpy(buf2, prompt);
  ptr = strtok(buf2, "|;&");
  while(ptr and (find_pats<MAX_FIND)) {
    if(*ptr) {
      memset(&find_pattern[find_pats], 0, sizeof(BmhArg));
      memset(find_patstr[find_pats], 0, BmhMAXPAT);
      strcpy(find_patstr[find_pats], ptr);
      strbmhinit(find_patstr[find_pats], &find_pattern[find_pats], caseignore);
      find_pats++;
    }
    ptr = strtok(NULL, "|;&");
  }

  lastfound = area->msgnset.relative_to_absolute(area->lastread());
  sprintf(buf, "%s \"%s\" ", CFG->language[LNG_SearchingFor], prompt);
  w_progress(PROGRESS_NEW, C_INFOW, area->lastread(), area->msgnset.size(), buf);

  ulong tmpmsgno;

  do {

    update_statuslinef(CFG->language[LNG_ReadingMsg], area->lastread(), area->msgnset.size());
    w_progress(PROGRESS_UPDATE, C_INFOW, area->lastread(), area->msgnset.size(), buf);

    if(area->loadmsg(msg, area->msgnset.relative_to_absolute(area->lastread()), CFG->dispmargin-CFG->disppagebar)) {

      if(what & GFIND_FROM)
        if(searchpatterns(msg.by_name().c_str()))
          msg.foundwhere |= GFIND_FROM;

      if(what & GFIND_TO)
        if(searchpatterns(msg.to.c_str()))
          msg.foundwhere |= GFIND_TO;

      if(what & GFIND_SUBJECT)
        if(searchpatterns(msg.re.c_str()))
          msg.foundwhere |= GFIND_SUBJECT;

      if(what & GFIND_BODY) {
        msg.foundtopline = -1;
        for(int l=0; l<msg.display.size(); l++) {
          int n = msg.display[l];
          if(searchpatterns(msg.linearray[n])) {
            msg.foundwhere |= GFIND_BODY;
            msg.linearray.data(n).type |= GLINE_HIGH;
            if(msg.foundtopline == -1)
              msg.foundtopline = l;
          }
        }
      }

      if((reverse ? not msg.foundwhere : msg.foundwhere) and (lastfound != msg.msgno)) {
        bool istwitto, istwitsubj;
        if(MsgIsTwit(msg, istwitto, istwitsubj, area) != TWIT_SKIP) {
          HandleGEvent(EVTT_SEARCHSUCCESS);
          lastfound = msg.msgno;
          area->set_lastread(area->msgnset.absolute_to_relative(msg.msgno));
          w_progress(PROGRESS_QUIT, 0, 0, 0, NULL);
          GFTRK(NULL);
          return true;
        }
      }
    }

    if(kbxhit()) {
      if(kbxget() == Key_Esc) {
        HandleGEvent(EVTT_SEARCHFAILED);
        area->set_lastread(last);
        w_progress(PROGRESS_QUIT, 0, 0, 0, NULL);
        GFTRK(NULL);
        return false;
      }
    }

    tmpmsgno = msg.msgno;
    switch(direction) {
      case DIR_PREV:
        if(area->msgnset.absolute_to_relative(msg.msgno) > 1)
          msg.msgno = area->msgnset.relative_to_absolute(area->msgnset.absolute_to_relative(msg.msgno)-1);
        else
          msg.msgno = 0;
        break;
      case DIR_NEXT:
        if(area->msgnset.absolute_to_relative(msg.msgno) < area->msgnset.size())
          msg.msgno = area->msgnset.relative_to_absolute(area->msgnset.absolute_to_relative(msg.msgno)+1);
        else
          msg.msgno = 0;
        break;
    }
    if(msg.msgno != 0)
      area->set_lastread(area->msgnset.absolute_to_relative(msg.msgno));

  } while((msg.msgno != 0) and (msg.msgno != tmpmsgno));

  HandleGEvent(EVTT_SEARCHFAILED);
  area->set_lastread(last);
  w_progress(PROGRESS_QUIT, 0, 0, 0, NULL);
  HandleGEvent(EVTT_BREAKLOOP);

  GFTRK(NULL);
  return false;
}


//  ------------------------------------------------------------------

void FindAll(GMsg& msg, int& topline, int& keyok, gactivearea* area) {

  if(area->msgnset.size()) {
    topline = 0;
    area->set_attr().hex0();
    strcpy(CFG->searchfor, area->searchfor().c_str());
    gstring searchfor = CFG->searchfor;
    if(edit_string(searchfor, sizeof(BmhPat), CFG->language[LNG_HeaderText], H_FindString)) {
      strcpy(CFG->searchfor, searchfor.c_str());
      area->set_searchfor(CFG->searchfor);
      if(FindString(msg, area->searchfor().c_str(), GFIND_HDRTXT, area))
        keyok = true;
    }
  }
}


//  ------------------------------------------------------------------

void FindHdr(GMsg& msg, int& topline, int& keyok, gactivearea* area) {

  if(area->msgnset.size()) {
    topline = 0;
    area->set_attr().hex0();
    strcpy(CFG->searchfor, area->searchfor().c_str());
    gstring searchfor = CFG->searchfor;
    if(edit_string(searchfor, sizeof(BmhPat), CFG->language[LNG_HeaderOnly], H_FindString)) {
      strcpy(CFG->searchfor, searchfor.c_str());
      area->set_searchfor(CFG->searchfor);
      if(FindString(msg, area->searchfor().c_str(), GFIND_HDR, area))
        keyok = true;
    }
  }
}


//  ------------------------------------------------------------------

