
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gehdre.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Header edit.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <gwinput.h>


//  ------------------------------------------------------------------

class GMsgHeaderEdit : public winput2 {

public:

  enum {
    id_from_name,
    id_from_addr,
    id_to_name,
    id_to_addr,
    id_subject
  };

  bool lookup;

  GMsg& msg;
  gactivearea* area;
  GMsgHeaderView &view;

  GMsgHeaderEdit(GMsgHeaderView &v) : msg(*v.msg), view(v), winput2(v.window) { area = view.area; lookup = false; }

  int handle_other_keys(uint& key);
  int validate();
};


//  ------------------------------------------------------------------

int GMsgHeaderEdit::handle_other_keys(uint&) {

  return false;
}


//  ------------------------------------------------------------------

int GMsgHeaderEdit::validate() {

  if(current->id == id_to_name) {

    field* from_addr = get_field(id_from_addr);
    field* to_addr   = get_field(id_to_addr);
    field* subject   = get_field(id_subject);

    gstring buf1, buf2;
    char* name = current->buf;
    const char* ptr = name;
    gstring buf = current->buf;

    if(CFG->addressmacro.size()) {
      for(int n=0; n<CFG->addressmacro.size(); n++) {
        address_macro amac = CFG->addressmacro[n];
        if(buf.ieql(amac.macro)) {

          gstring aname = amac.name;
          const char* ptr = aname.c_str();
          if(aname[0] == '@') {
            // UUCP/INTERNET addressing
            ptr = strchr(aname.c_str(), '/');
            if(ptr) {
              msg.kludge.replyaddr = ptr+1;
              msg.header.to = ptr+1;
              aname.erase(ptr);
              ptr = aname.c_str()+1;
            }
            else {
              msg.kludge.replyaddr = aname.c_str()+1;
              msg.header.to = aname.c_str()+1;
              aname = "UUCP";
              ptr = aname.c_str();
            }
          }
          strcpy(name, ptr);
          current->update();
          msg.destname = aname;

          strcpy(to_addr->buf, amac.address.c_str());
          to_addr->update();

          if(amac.subject) {
            strcpy(subject->buf, amac.subject);
            subject->update();
          }

          if(amac.attribs) {
            msg.attr.add(amac.attribs);
            gstring attrs = msg.attr;
            attrs.resize(CFG->edithdrnodelen, ' ');
            window.prints(1, CFG->edithdrnodepos, C_HEADW, attrs);
          }

          if(lookup)
            go_next_field();

          return true;
        }
      }
    }

    // Internet gating
    if(not area->isinternet()) {
      if(strchr(buf.c_str(), '@') and area->area->internetgate().addr.net) {
        msg.header.to = buf;
        if(area->internetgate().name) {
          msg.kludge.replyaddr = buf;
          msg.destname = area->internetgate().name;
          strcpy(name, msg.destname.c_str());
          current->update();
        }
        area->internetgate().addr.make_string(to_addr->buf);
        to_addr->update();
        if(lookup)
          go_next_field();
        return true;
      }

      if(lookup) {
        buf1 = name;
        buf2 = msg.destname;
        if(strblank(to_addr->buf) or strblank(name) or (buf1 != buf2)) {
          if(strblank(name))
            buf = to_addr->buf;
          else
            buf = name;
          buf.trim();
          if(buf.length() == 0)
            buf = " ";

          if(buf.length()) {
            ftn_addr addr;
            addr = from_addr->buf;   // Get the orig address to base lookup on
            Lookup(msg, addr, buf, 0, CFG->language[LNG_SelectDestNode], area);
            vcurshow();
            if(striinc("Sysop", buf.c_str()) or area->whoto().ieql(buf)) {
              if(addr.net == 0) {
                if(not area->isnet())
                  go_next_field();
                return true;
              }
            }
            bool robotchk = false;
            for(int n=0; n<CFG->robotname.size(); n++) {
              if(striinc(CFG->robotname[n], buf.c_str())) {
                robotchk = true;
                break;
              }
            }
            if(robotchk) {
              CFG->edithardlines = false;
              if(addr.net == 0)
                return true;
            }
            if(addr.net) {
              addr.make_string(to_addr->buf);
              to_addr->update();
              msg.destname = buf;
              strcpy(name, buf.c_str());
              current->update();
              go_next_field();
            }
            else {
              HandleGEvent(EVTT_SEARCHFAILED);
            }
          }
        }
      }
      else {
        if(strpbrk(buf.c_str(), "*?")) {
          if(CFG->lookupuserbase) {
            if(area->userlookup(buf)) {
              msg.destname = buf;
              current->update();
              return true;
            }
          }
        }
      }
    }
  }
  else if(current->id == id_subject) {
    gstring buf = current->buf;
    CheckSubject(msg, buf, area);
    strcpy(current->buf, buf.c_str());
    current->update();
  }

  return true;
}


//  ------------------------------------------------------------------

int EditHeaderinfo(int mode, GMsgHeaderView &view) {

  GMsgHeaderEdit hedit(view);
  GMsg& msg = *view.msg;
  gactivearea* area = view.area;

  gstring from_name  = msg.oname;
  gstring from_addr  = msg.oaddress;
  gstring to_name    = msg.dname;
  gstring to_addr    = msg.daddress;
  gstring subject    = msg.subject;
  gstring orig_from_addr = from_addr;

  if(not area->isinternet()) {
    if(area->isnet())
      hedit.lookup = CFG->lookupnet;
    else if(area->isecho())
      hedit.lookup = CFG->lookupecho;
    else
      hedit.lookup = CFG->lookuplocal;
  }

  int from_name_pos = CFG->edithdrnamepos;
  int from_name_len = CFG->edithdrnamelen;
  int from_addr_pos = CFG->edithdrnodepos;
  int from_addr_len = CFG->edithdrnodelen;
  int to_name_pos = from_name_pos;
  int to_name_len = from_name_len;
  int to_addr_pos = from_addr_pos;
  int to_addr_len = hedit.lookup ? from_addr_len : 0;
  int subject_pos = 8;
  int subject_len = gvid.numcols - subject_pos;
  int name_cvt = area->editmixcase() ? GMsgHeaderEdit::cvt_mixedcase : GMsgHeaderEdit::cvt_none;
  int addr_cvt = GMsgHeaderEdit::cvt_none;
  int subj_cvt = GMsgHeaderEdit::cvt_none;

  hedit.setup(C_HEADW, C_HEADW, C_HEADE, '\xB0');

  hedit.add_field(GMsgHeaderEdit::id_from_name, 2, from_name_pos, from_name_len, from_name,  256, name_cvt);
  hedit.add_field(GMsgHeaderEdit::id_from_addr, 2, from_addr_pos, from_addr_len, from_addr,  256, addr_cvt);
  hedit.add_field(GMsgHeaderEdit::id_to_name,   3,   to_name_pos,   to_name_len,   to_name,  256, name_cvt);
  hedit.add_field(GMsgHeaderEdit::id_to_addr,   3,   to_addr_pos,   to_addr_len,   to_addr,  256, addr_cvt);
  hedit.add_field(GMsgHeaderEdit::id_subject,   4,   subject_pos,   subject_len,   subject, 2048, subj_cvt);

  hedit.start_id = GMsgHeaderEdit::id_to_name;

  switch(mode) {
    case MODE_COMMENT:
      if(not (msg.dest.net or not area->isnet()))
        break;
      // else drop through ...
    case MODE_REPLY:
    case MODE_QUOTE:
    case MODE_CHANGE:
      hedit.start_id = GMsgHeaderEdit::id_subject;
      break;
  }

  ChgAttrs(true, msg, area);

  whelpdef(CFG->helpged, Key_F1, C_HELPB, C_HELPW, C_HELPQ, C_HELPS, NULL);
  hedit.run(H_Header);

  ChgAttrs(false, msg, area);

  if(not hedit.dropped and not gkbd.quitall) {

    from_name.trim();
    from_addr.trim();
    to_name.trim();
    to_addr.trim();
    subject.trim();

    msg.oname    = from_name;
    msg.oaddress = from_addr;
    msg.dname    = to_name;
    msg.daddress = to_addr;
    msg.subject  = subject;

    if(area->isinternet()) {
      if(to_addr)
        msg.header.to = to_name + " <" + to_addr + ">";
      else
        msg.header.to = to_name;
    }
    else {
      msg.origaddr = from_addr;
      msg.destaddr = to_addr;
    }

    gftnaddrs address = area->aka();

    if(from_addr)
      msg.origaddr = from_addr;

    if(not to_addr) {
      // Set destination to yourself or the Boss node
      address = msg.origaddr;
      address.point = 0;
      to_addr = address.str();
    }

    address = area->aka();
    address.set(to_addr);
    msg.destaddr = address;
    gstring domain = address.domain;

    if(*domain) {
      address.reset();
      address.zone  = msg.origaddr.zone;
      address.set(domain);
      if(address.net)
        msg.destaddr = address;
    }

    // Try to match akas with the dest address, but only if the orig address was not changed
    int n;
    for(n=0; n<CFG->akas; n++) {
      address = CFG->aka[n];
      if(msg.origaddr == address)
        break;  // Found one of our own akas.
    }

    // Try to match akas with the dest address, but only if the orig address was not changed
    if((n < CFG->akas)) {
      // ... and only if the orig aka was already one of our own
      if(orig_from_addr.ieql(from_addr)) {
        // ... and only if we did not change aka manually
        if(area->aka() == area->orig_aka()) {

          // Do aka matching
          if(area->akamatching()) {
            msg.origaddr.domain = CFG->aka[AkaMatch(msg.origaddr, msg.destaddr)].domain;
          }
        }
      }
    }
  }
  else {
    for(int n=0; n<fspecs; n++)
      throw_release(fspec[n].fblk);
    fspecs = specfiles = 0;
  }

  view.Paint();

  return hedit.dropped ? W_ESCPRESS : W_NOERROR;
}


//  ------------------------------------------------------------------

void CheckSubject(GMsg& msg, gstring& subj, gactivearea* area) {

  for(int n=0; n<fspecs; n++)
    throw_release(fspec[n].fblk);
  fspecs = specfiles = 0;
  fspec = (FileSpec*)throw_realloc(fspec, sizeof(FileSpec));

  if(area->isnet()) {
    if(CFG->editautoattach) {
      // If the subject was changed
      if(not subj.ieql(msg.subject)) {
        // Check if the subj begins with drive spec
        if(not (msg.attr.frq() or msg.attr.att() or msg.attr.urq())) {
          const char* ptr = subj.c_str();
          if(*ptr == '^')
            ptr++;
          if((isalpha(*ptr) and (ptr[1] == ':')) or ((ptr[0] == '\\') and (ptr[1] == '\\'))) {
            // Autoattach
            msg.attr.att1();
            msg.attr.add(CFG->attribsattach);
            DispHeadAttrs(msg);
          }
        }
      }
    }

    if(msg.attr.frq()) {
      // Handle File Requests
      if(subj.length()) {
        gstring buf = subj;
        char* ptr = strtok(buf._c_str(), " ");
        while(ptr) {
          if(*ptr == '!') {   // Store password
            if(fspecs)
              strcpy(fspec[fspecs-1].password, ptr);
          }
          else {
            fspec = (FileSpec*)throw_realloc(fspec, (fspecs+1)*sizeof(FileSpec));
            memset(fspec+fspecs, 0, sizeof(FileSpec));
            fspec[fspecs].files = 1;
            strcpy(fspec[fspecs].path, ptr);
            specfiles++;
            fspecs++;
          }
          ptr = strtok(NULL, " ");
        }
      }
    }
    else if(msg.attr.att()) {
      // Handle File Attaching
      if(subj.length() == 0)
        subj = ".";
      gstring buf = subj;
      char* ptr = strtok(buf._c_str(), " ");
      while(ptr) {
        fspec = (FileSpec*)throw_realloc(fspec, (fspecs+1)*sizeof(FileSpec));
        memset(fspec+fspecs, 0, sizeof(FileSpec));
        if(*ptr == '^') {
          fspec[fspecs].delsent = YES;
          ptr++;
        }
        strcpy(fspec[fspecs].path, ptr);
        FileSelect(CFG->language[LNG_AttachFiles], &fspec[fspecs], area);
        specfiles += fspec[fspecs].files;
        if(fspec[fspecs].files)
          fspecs++;
        ptr = strtok(NULL, " ");
      }
    }
    else if(msg.attr.urq()) {
      // Handle Update Requests
      if(subj.length() == 0)
        subj = ".";
      gstring buf = subj;
      char* ptr = strtok(buf._c_str(), " ");
      while(ptr) {
        if(*ptr == '!') {   // Store password
          if(fspecs)
            strcpy(fspec[fspecs-1].password, ptr);
        }
        else {
          fspec = (FileSpec*)throw_realloc(fspec, (fspecs+1)*sizeof(FileSpec));
          memset(fspec+fspecs, 0, sizeof(FileSpec));
          strcpy(fspec[fspecs].path, ptr);
          FileSelect(CFG->language[LNG_UpdreqFiles], &fspec[fspecs], area);
          specfiles += fspec[fspecs].files;
          if(fspec[fspecs].files)
            fspecs++;
        }
        ptr = strtok(NULL, " ");
      }
    }

    // Generate the (first) processed subject line
    if(specfiles) {
      gstring buf;
      for(int x=0,n=0; x<fspecs; x++) {
        for(int m=0; m<fspec[x].files; m++) {
          gstring subject;
          subject  = fspec[x].delsent ? "^" : "";
          subject += fspec[x].path;
          subject += fspec[x].fblk ? (fspec[x].fblk[m].name ? fspec[x].fblk[m].name : "") : "";
          subject += *fspec[x].password ? " " : "";
          subject += fspec[x].password;
          subject += " ";
          subject.upper();
          if((buf.length() + subject.length()) > 71) {
            n++;
            break;
          }
          else {
            buf += subject;
          }
        }
        if(n)
          break;
      }
      subj = buf.trim();
    }
  }
}


//  ------------------------------------------------------------------

