
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gehdrs.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Header display.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

char* strconv(char* str, char*) {

  char* s = str;
  register char* p = str;

  while(*str) {
    if(iscntrl(*str) and (*str != '\n'))  // Control codes
      *p++ = '.';
    else
      *p++ = *str;
    str++;
  }
  return(s);
}


//  ------------------------------------------------------------------

void build_header_display(GMsg& msg, header_display& display, int width, gactivearea* area) {

  int namewidth = CFG->disphdrnodeset.pos - CFG->disphdrnameset.pos;
  int nodewidth = CFG->disphdrdateset.pos - CFG->disphdrnodeset.pos;
  int datewidth = width - CFG->disphdrdateset.pos;

  // Generate from-name
  if(area->isinternet()) {
    if(msg.oname)
      display.from_name = msg.oname + " <" + msg.oaddress + ">";
    else
      display.from_name = msg.oaddress;
  }
  else {
    display.from_name = msg.oname ? msg.oname : msg.origname;
  }
  display.from_name.resize(area->isinternet() ? (namewidth+nodewidth) : namewidth, ' ');

  // Generate to-name
  if(area->isinternet()) {
    if(msg.dname)
      display.to_name = msg.dname + " <" + msg.daddress + ">";
    else
      display.to_name = msg.daddress;
  }
  else {
    display.to_name = msg.dname ? msg.dname : msg.destname;
  }
  display.to_name.resize(area->isinternet() ? (namewidth+nodewidth) : namewidth, ' ');

  // Generate orig address
  display.from_address = msg.oaddress;
  display.from_address.resize(nodewidth, ' ');

  // Generate dest address
  display.to_address = msg.daddress;
  if(not area->isinternet()) {
    ftn_addr daddress = msg.daddress;
    if((daddress.net != msg.destaddr.net) or (daddress.node != msg.destaddr.node))
      display.to_address += gstrf(" %s %u/%u", CFG->language[LNG_Via], msg.destaddr.net, msg.destaddr.node);
  }
  display.to_address.resize(nodewidth, ' ');

  // Generate written date
  display.date_written = msg.date.written.format(CFG->language[LNG_DateTimeFmt]);
  display.date_written.resize(datewidth, ' ');

  // Generate arrived date
  display.date_arrived = msg.date.arrived.format(CFG->language[LNG_DateTimeFmt]);
  display.date_arrived.resize(datewidth, ' ');

  // Generate subjectline
  display.lngsubj = (msg.attr.att() or msg.attr.frq() or msg.attr.urq()) ? CFG->language[LNG_File] : CFG->language[LNG_Subj];
  display.subject = msg.subject;
  display.subject.resize(width-display.lngsubj.length(), ' ');

  // Generate description
  gstring desc = area->desc();
  if((CFG->dispareano == ALWAYS) or (CFG->dispareano and area->board()))
    display.description = gstrf(" [%u] ", area->board());
  else
    display.description = " ";
  display.description += desc;
  display.description.trim();
  desc = area->aka();
  display.description += " (" + desc + ") ";

  // Generate echoid
  display.echoid = " " + area->echoid() + " ";
  if(msg.kludge.area)
    display.echoid += "(" + msg.kludge.area + ") ";

  // Generate message attributes string
  gstring attrs = msg.attr;
  int len2 = display.attrs.length();
  if(len2 > width-CFG->disphdrnodeset.pos) {
    len2 = width-CFG->disphdrnodeset.pos;
    display.attrs.resize(len2, ' ');
  }

  // Generate message number and reply links string
  int list_max = msg.link.list_max();
  ulong* replies = (ulong*)throw_xcalloc(list_max+1, sizeof(ulong));
  ulong replyto, replynext;
  if(CFG->disprealmsgno) {
    display.msgnos = gstrf(" %-5.5s: #%lu [%u]", CFG->language[LNG_Msg], msg.msgno, area->msgnset.size()+(msg.attr.nwm() ? 1 : 0));
    replyto = msg.link.to();
    replies[0] = msg.link.first();
    replynext = msg.link.next();
    for(int replyn=1; replyn<list_max+1; replyn++)
      replies[replyn] = msg.link.list(replyn-1);
  }
  else {
    uint active = area->msgnset.size() + (msg.attr.nwm() ? 1 : 0);
    display.msgnos = gstrf(" %-5.5s: %u %s %u", CFG->language[LNG_Msg], msg.attr.nwm() ? active : area->msgnset.absolute_to_relative(msg.msgno), CFG->language[LNG_of], active);
    replyto = area->msgnset.absolute_to_relative(msg.link.to());
    replies[0] = area->msgnset.absolute_to_relative(msg.link.first());
    replynext = area->msgnset.absolute_to_relative(msg.link.next());
    for(int replyn=1; replyn<list_max+1; replyn++)
      replies[replyn] = area->msgnset.absolute_to_relative(msg.link.list(replyn-1));
  }
  if(replyto)
    display.msgnos += gstrf(" -%lu", replyto);
  for(int replyn=0,plus=0; replyn<list_max+1; replyn++)
    if(replies[replyn])
      display.msgnos += gstrf(" %s%lu", (plus++?"":"+"), replies[replyn]);
  if(replynext)
    display.msgnos += gstrf(" *%lu", replynext);
  int len1 = display.msgnos.length()-8;
  if((CFG->disphdrnameset.pos + len1) > CFG->disphdrnodeset.pos) {
    if(8 + len1 + len2 > width) {
      display.msgnos.resize(width-len2-1, ' ');
      display.msgnos.trim();
    }
    display.msgnos += " " + attrs;
  }
  else {
    display.msgnos.resize(namewidth+8, ' ');
    display.msgnos += attrs;
  }
  display.msgnos.resize(width, ' ');
  throw_xfree(replies);
}


//  ------------------------------------------------------------------

void PrintHeader(GMsg& msg, int fh, int width, gactivearea* area) {

  header_display display;

  build_header_display(msg, display, width, area);

  char headerline[200];
  memset(headerline, _box_table[W_BHEAD][1], width);
  headerline[width] = NUL;

  // Write the total header to a file
  strcpy(buf, headerline);
  strcat(buf, "\n");
  strncpy(buf+1, display.description.c_str(), display.description.length());
  strncpy(buf+width-display.echoid.length()-1, display.echoid, display.echoid.length());
  strconv(buf, tconv);
  write(fh, buf, strlen(buf));

  strcpy(buf, display.msgnos.c_str());
  strtrim(buf);
  strcat(buf, "\n");
  strconv(buf, tconv);
  write(fh, buf, strlen(buf));

  sprintf(buf, "%s%s%s%s",
    CFG->language[LNG_From], display.from_name.c_str(),
    not area->isinternet() ? display.from_address.c_str() : "",
    display.date_written.c_str()
  );
  strtrim(buf);
  strcat(buf, "\n");
  strconv(buf, tconv);
  write(fh, buf, strlen(buf));

  sprintf(buf, "%s%s%s%s", CFG->language[LNG_To], display.to_name.c_str(),
    not area->isinternet() ? display.to_address.c_str() : "",
    display.date_arrived
  );
  strtrim(buf);
  strcat(buf, "\n");
  strconv(buf, tconv);
  write(fh, buf, strlen(buf));

  strcpy(buf, display.lngsubj.c_str());
  strcat(buf, display.subject.c_str());
  strtrim(buf);
  strcat(buf, "\n");
  strconv(buf, tconv);
  write(fh, buf, strlen(buf));

  strcpy(buf, headerline);
  strcat(buf, "\n");
  strconv(buf, tconv);
  write(fh, buf, strlen(buf));
}


//  ------------------------------------------------------------------

