
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geinit.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  System initializing.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <gmbprot.h>


//  ------------------------------------------------------------------

#if defined(__OS2__)
char* __golded_shortname = "GED2";
#elif defined(__WIN32__)
char* __golded_shortname = "GEDW";
#elif defined(__MSDOS__) and defined(__386__)
char* __golded_shortname = "GEDX";
#elif defined(__linux__)
char* __golded_shortname = "GEDL";
#elif defined(__MSDOS__)
char* __golded_shortname = "GEDD";
#endif
char __golded_longname[40];


//  ------------------------------------------------------------------
//  Handle commandline parameters

#if defined(GFTRK_ENABLE)
static int __FAR gftrk_set_max = -1;
extern int __gftrk_statusline;
#endif

static char __FAR keybuf[80];

       Path __FAR cmdlinecfg        = {""};
       int  __FAR cmdlinedebughg    = false;
static int  __FAR cmdlineforce      = false;
static int  __FAR cmdlinehelp       = false;
       int  __FAR cmdlineimportsoup = false;
static int  __FAR cmdlineinstall    = false;
static Path __FAR cmdlineinstpath   = {""};
       int  __FAR cmdlineoldkeyw    = true;
       int  __FAR cmdlinetimeout    = -1;
       bool __FAR cmdlinewriteareas = false;
       int  __FAR cmdlinesharemode  = SH_DENYNO;

static void InitCmdline(char* val) {

  char* key;
  char* trueval;
  
  if(strchr("-/", *val)) {
    val++;
    getkeyvaleql(&key, &val, true);
    trueval = val;
    if(*val == NUL)
      val = key+1;
    switch(toupper(*key)) {
      case '2':
        //__gforceos2slice = *val != '-';
        break;
      case 'C':                   // Use another Configfile
        strcpy(cmdlinecfg, val);
        break;
      case 'D':
        cmdlineoldkeyw = *val == '-';
        break;
      case 'E':
        startecho = val;
        break;
      case 'F':
        cmdlineforce = *val != '-';
        if(toupper(key[1]) == 'F')
          cmdlineforce++;
        break;
      case '?':
      case 'H':
        cmdlinehelp = true;
        break;
      case 'I':
        if(strieql(key, "INSTALL")) {
          cmdlineinstall = YES;
          cmdlineforce = 1;
          if(*trueval)
            PathCopy(cmdlineinstpath, trueval);
        }
        else if(strieql(key, "IMPORTSOUP"))
          cmdlineimportsoup = true;
        break;
      case 'M':
        disablesound = *val != '-';
        break;
      case 'N':
        cmdlinesharemode = 0;
        break;
      case 'Q':
        quiet = YES;
        break;
      case 'S':
        AL.sortspec = val;
        break;
      case 'T':
        cmdlinetimeout = atoi(val);
        break;
      case 'V':
        quiet = NO;
        veryverbose = (toupper(key[1]) == 'V' ? YES : NO);
        break;
      case 'W':
        cmdlinewriteareas = *val != '-';
        break;
      case 'Y':
        cmdlinedebughg = *val != '-';
        break;
      case 'X':
        break;
      #if defined(GFTRK_ENABLE)
      case 'Z':
        gftrk_set_max = atoi(val);
        break;
      #endif
    }
  }
  else {
    strcat(keybuf, val);
    strcat(keybuf, " ");
  }
}


//  ------------------------------------------------------------------

static void ReadEcholists() {

  FILE* fp;
  register uint n;
  char buf[256];
  Echo* echotags;
  int echonums, x;

  echoin = (Echo*)throw_calloc(1, sizeof(Echo));
  echoout = (Echo*)throw_calloc(1, sizeof(Echo));

  // Read the two taglists

  for(n=0; n<2; n++) {
    if(n) {
      fp = fsopen(AddPath(CFG->areapath, CFG->semaphore.exportlist), "rt", CFG->sharemode);
      echotags = echoout;
    }
    else {
      fp = fsopen(AddPath(CFG->areapath, CFG->semaphore.importlist), "rt", CFG->sharemode);
      echotags = echoin;
    }
    if(fp) {
      update_statusline(CFG->language[LNG_ReadingEcholist]);
      echonums = 0;
      while(fgets(buf, sizeof(buf), fp)) {
        if(not strblank(buf)) {
          echonums++;
          echotags = (Echo*)throw_realloc(echotags, (echonums+2)*sizeof(Echo));
          strcpy(echotags[echonums-1], strtrim(strsetsz(buf, sizeof(Echo)-1)));
        }
      }
      *echotags[echonums] = 0;   // Mark end
      fclose(fp);
      if(n) {
        echoout  = echotags;
      }
      else {
        echoin  = echotags;
      }
    }
  }

  // Mark the areas from the import taglist
  for(n=0; n<AL.areas(); n++) {
    strcpy(buf, AL[n]->echoid().c_str());
    x = SearchTaglist(echoin, buf);
    if(*echoin[x])
      AL[n]->set_marked();
  }
}


//  ------------------------------------------------------------------

void golded_xlat::load_escsets() {

  load_escset(set.i51,      "I51");
  load_escset(set.composed, "COMPOSED");
  load_escset(set.mnemonic, "MNEMONIC");
}


//  ------------------------------------------------------------------

static void kbputc_(char c) {

  kbput((Key)((scancode(c) << 8) | c));
}


//  ------------------------------------------------------------------

static void kbputstr(char* buf) {

  Key xkey;
  register int n,x;

  x = strlen(buf);

  for(n=0; n<x; n++) {
    switch(buf[n]) {
      case HT:
      case ' ':
        break;
      case '!':
        clearkeys();
        kbclear();
        break;
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
        kbput((Key)atoi(&buf[n]));
        for(; n<x; n++) {
          if(buf[n] == ' ')
            break;
        }
        break;
      case '~':
      case '^':
        xkey = (Key)(toupper(buf[++n]) - '@');
        if(xkey)
          kbput(xkey);
        break;
      case '@':
        xkey = (Key)atoi(&buf[++n]);
        if(xkey == 0)
          xkey = scancode(buf[n]);
        kbput((Key)((xkey << 8) & 0xFF00));
        for(; n<x; n++) {
          if(buf[n] == ' ')
            break;
        }
        break;
      case '\"':
        for(n+=1; n<x; n++) {
          if(buf[n] == '\"')
            break;
          kbputc_(buf[n]);
        }
        break;
      case '\'':
        for(n+=1; n<x; n++) {
          if(buf[n] == '\'')
            break;
          kbputc_(buf[n]);
        }
        break;
      default:
        kbputc_(buf[n]);
    }
  }
}


//  ------------------------------------------------------------------

static void w_back() {

  W_STAT = wopen(gvid.numrows-1,0,gvid.numrows-1,gvid.numcols-1,5,C_STATW,C_STATW);
  update_statusline("");
  wfillch('');
  W_BACK = wopen(0,0,gvid.numrows-2,gvid.numcols-1,5,C_BACKB,C_BACKW);
  wfillch(' ');
}


//  ------------------------------------------------------------------

#if 0
static void w_logo(int m1, int m2) {

  char buf[50], buf2[50];
  static int w1=0, w2=0;

  if(m1 == 1) {

    #if defined(__MSDOS__)
      i86 cpu;
      cpu.r.w.ax = 0x6601;
      cpu.genint(0x21);
      int codepage = cpu.r.w.bx;
    #elif defined(__OS2__)
      USHORT codepage=437;
      VioGetCp(0, &codepage, 0);
    #elif defined(__WIN32__)
      UINT codepage = GetConsoleCP();
    #endif

    w1 = wopen(1,2,7,31,5,C_BRAGA,C_BRAGA);
    w_shadow();

    if(W_BBRAG == 7) {
      wprints(0, 1,C_BRAGA, "----------------------------");
      wprints(1, 1,C_BRAGA, "------------             ---");
      wprints(2, 1,C_BRAGA, "----------             -----");
      wprints(3, 1,C_BRAGA, "--------             -------");
      wprints(4, 1,C_BRAGA, "------             ---------");
      wprints(5, 1,C_BRAGA, "----             -----------");
      wprints(6, 1,C_BRAGA, "----------------------------");
                          // 123456789012345678901234567890
      wprints(1,14,C_BRAGT,              "G O L D E D");
      wprints(2,12,C_BRAGT,            "G O L D E D");
      wprints(3,10,C_BRAGT,          "G O L D E D");
      wprints(4, 8,C_BRAGT,        "G O L D E D");
      wprints(5, 6,C_BRAGT,      "G O L D E D");
    }
    else {
      wprints(0, 1,C_BRAGA, "                     ");
      wprints(1, 1,C_BRAGA, "                      ");
      wprints(2, 1,C_BRAGA, "                      ");
      wprints(3, 1,C_BRAGA, "                      ");
      wprints(4, 1,C_BRAGA, "                     ");
      wprints(5, 1,C_BRAGA, "                      ");
      wprints(6, 1,C_BRAGA, "                      ");

      wprints(0, 9,C_BRAGH,         "");
      wprints(1, 7,C_BRAGH,       "");
      wprints(1,16,C_BRAGH,                "");
      wprints(1,20,C_BRAGH,                    "");
      wprints(2, 5,C_BRAGH,     "");
      wprints(2,20,C_BRAGH,                    "");
      wprints(3, 3,C_BRAGH,   "");
      wprints(3,20,C_BRAGH,                    "");
      wprints(4, 1,C_BRAGH, "");
      wprints(4,20,C_BRAGH,                    "");
      wprints(5, 3,C_BRAGH,   "");
      wprints(5,11,C_BRAGH,           "");
      wprints(6, 5,C_BRAGH,     "");

      switch(codepage) {
        case 437:
        case 860:
        case 861:
        case 862:
        case 863:
        case 865:
          wprints(1,12,C_BRAGT,            " ");
          wprints(1,18,C_BRAGT,                  "");
          wprints(2, 8,C_BRAGT,        "ֶֶַַַ");
          wprints(3, 8,C_BRAGT,        "ǽ");
          wprints(4, 8,C_BRAGT,        "Ӷӽӽ");
          wprints(5, 7,C_BRAGT,       "Ľ");
          break;
        default:
          wprints(1,12,C_BRAGT,            " ");
          wprints(1,18,C_BRAGT,                  "");
          wprints(2, 8,C_BRAGT,        "ڿڿڴڿڴ");
          wprints(3, 8,C_BRAGT,        "ٳ");
          wprints(4, 8,C_BRAGT,        "");
          wprints(5, 7,C_BRAGT,       "");
      }
    }
  }
  else if(m1 == -1 and w1) {
    wunlink(w1);
  }

  if(m2 == 1) {
    w2 = wopen(1,gvid.numcols+48-80,7,gvid.numcols+76-80,W_BBRAG,C_BRAGB,C_BRAGW);
    w_shadow();

    strcpy(buf2, CFG->ver);
    if(CFG->ver[4])
      buf2[4] = 0;
    sprintf(buf, " Version %s, %s", buf2, CFG->vers.date);
    strschg(buf, "+", "");
    strsetsz(buf, 27);
    wprints(0,0,C_BRAGW, " The GoldED Message Editor ");
    wprints(1,0,C_BRAGW, buf);
    wprints(2,0,C_BRAGB, W_BBRAG == 7 ? " ------------------------- " : "  ");
    wprints(3,0,C_BRAGW, " Copyright (C) 1990 - 1997 ");
    wprints(4,0,C_BRAGW, "     By Odinn Sorensen     ");
  }
  else if(m2 == -1 and w2) {
    wunlink(w2);
  }
}
#endif


//  ------------------------------------------------------------------

static void w_brag() {

  char* logo[6];

  if(W_BBRAG == 7) {
    logo[0] = "                    **     **            **  ";
    logo[1] = "      ****** ****** ** ****** ****** ******  ";
    logo[2] = "      **  ** **  ** ** **  ** ****** **  **  ";
    logo[3] = "      ****** ****** ** ****** ****** ******  ";
    logo[4] = "  **      **                                 ";
    logo[5] = "  **********                                 ";
  }
  else {
    logo[0] = "                    ɻ     ɻ            ɻ  ";
    logo[1] = "      ˻ ˻  ͹ ˻ ͹  ";
    logo[2] = "                ʼ     ";
    logo[3] = "      ͹ ʼ ȼ ʼ ʼ ʼ  ";
    logo[4] = "  ɻ                                       ";
    logo[5] = "  ʼ                                 ";
  }

  W_READ = wopen_(1, 2, gvid.numrows-4, gvid.numcols-5, W_BBRAG, C_BRAGB, C_BRAGW);
  w_shadow();

  wprints(0, 0, C_BRAGT, logo[0]);
  wprints(1, 0, C_BRAGT, logo[1]);
  wprints(2, 0, C_BRAGT, logo[2]);
  wprints(3, 0, C_BRAGT, logo[3]);
  wprints(4, 0, C_BRAGT, logo[4]);
  wprints(5, 0, C_BRAGT, logo[5]);

  char buf[200];
  sprintf(buf, "%s %s", __gpid__, gversion);
  wprints(4, 43-strlen(buf), C_BRAGW, buf);

  if(isalpha(CFG->ver[4])) {
    if(CFG->ver[4] == 'a')
      strcpy(buf, "THIS IS AN ALPHA RELEASE");
    else if(CFG->ver[4] == 'b')
      strcpy(buf, "THIS IS A BETA RELEASE");
    else if(CFG->ver[4] == 'g')
      strcpy(buf, "THIS IS A GAMMA RELEASE");
    wprints(5, 43-strlen(buf), C_BRAGW, buf);
  }

  wprints(1, 46, C_BRAGW, "   The GoldED Msg Editor   ");
  wprints(2, 46, C_BRAGW, "  Copyright (C) 1990-1997  ");
  wprints(3, 46, C_BRAGW, "     By Odinn Sorensen     ");
  wprints(4, 46, C_BRAGW, "    All Rights Reserved    ");

  whline(6, 0, gvid.numcols-5, W_BBRAG, C_BRAGB);
  wvline(0, 45, 7, W_BBRAG, C_BRAGB);

  //wcenters(8, C_BRAGW, "---===*F*I*R*E*B*I*R*D**CHRISTMAS**RELEASE*===---");
  wcenters(8, C_BRAGW, "---===*F*I*R*E*B*I*R*D*===--- \"PATH TO OTHERWHERE\"");

  wgotoxy(10,0);
  update_statusline(CFG->language[LNG_Initializing]);
}


//  ------------------------------------------------------------------

char* MakePID(char* __buf) {

  int number = 0;
  switch(__gver__[4]) {
    case 'a':  number = __golded_rev__;  break;
    case 'b':  number = __golded_rev__;   break;
    case 'g':  number = __golded_rev__;  break;
  }

  if(__gver__[4]) {
    // PID: GED[2,3] <major>.<minor>.<a,b,g><releasenumber>
    sprintf(__buf, "%s %d.%d.%c%d",
      __shortgpid__, __golded_major__, __golded_mipid__, __gver__[4], number
    );
  }
  else {
    // PID: GED[2,3] major.minor
    sprintf(__buf, "%s %d.%d",
      __shortgpid__, __golded_major__, __golded_mipid__
    );
  }

  return __buf;
}


//  ------------------------------------------------------------------

int ExistCfg(char* path, char* file) {

  int found = fexist(AddPath(path, file));
  if(found)
    strcat(path, file);
  return found;
}


//  ------------------------------------------------------------------

void HelpInit() {

  // Init the help system
  whelpfp = CFG->runtime.bincfgfp;
  whelpoffset = CFG->internal.helpoffset;
  whelpdef("", Key_F1, C_HELPB, C_HELPW, C_HELPQ, C_HELPS, NULL);
  whelpwin(0, 0, gvid.numrows-2, gvid.numcols-1, W_BHELP, false);
  whelpcat(H_General);
}


//  ------------------------------------------------------------------

void Initialize(int argc, char* argv[]) {

  char* ptr;
  char* ptr2;
  register uint n;
  int found=NO, dbedit=NO, compiled;
  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT], buf[256];

  // First inits
  _fmode = O_BINARY;
  ggetosversion();
  *keybuf = NUL;
  goldmark = ' ';
  *information = NUL;
  srand((unsigned)time(NULL));
  #if defined(__WATCOMC__) or defined(__GNUC__)
  setvbuf(stdout, NULL, _IONBF, BUFSIZ);
  #endif

  // Check environment commandline
  ptr = getenv("GEDCMD");
  if(ptr and *ptr) {
    ptr = strcpy(buf, ptr);
    while(*ptr) {
      ptr = strskip_wht(ptr);
      ptr2 = ptr;
      ptr = strskip_txt(ptr);
      if(*ptr)
        *ptr++ = NUL;
      InitCmdline(ptr2);
    }
  }

  // Check if we have been renamed to replace the D'Bridge Editor
  if(striinc("DBEDIT.EXE", argv[0])) {
    for(int a=1; a<argc; a++) {
      if(striinc("INTERNAL", argv[a])) {
        dbedit = YES;   // Do not accept any commandline arguments
        break;
      }
    }
  }

  // Check the commandline arguments
  if(argc > 1 and not dbedit) {
    for(int a=1; a<argc; a++) {
      ptr = strcpy(buf, argv[a]);
      InitCmdline(ptr);
    }
  }

  // Set the alpha revision number
  if(__golded_rev__ == 0)
    __golded_rev__ = (str2mon(__DATE__) * 100) + atoi(__DATE__+4);

  // Set program version string for betas
  strcpy(gversion, __gver__);
  if(isalpha(__gver__[4])) {
    if(*__grev__) {
      sprintf(gversion, "%4.4s.%s", __gver__, __grev__);
    }
    else {
      sprintf(gversion, "%4.4s.%c%02d%02d",
        __gver__, toupper(__gver__[4]),
        str2mon(__date__), atoi(&__date__[4])
      );
    }
  }

  // Print commandline help and exit if requested
  if(cmdlinehelp) {

    char gverbuf[80];
    sprintf(gverbuf, "%s %s, Mail Editor.", __gpid__, gversion);

    printf(                                     // 123456789012345678901234567890123456
       // 34567890123456789012345678901234567890123456789012345678901234567890123456789
       "%-41.41s"                               "Copyright (C) 1990-1997 Odinn Sorensen\n"
       "-------------------------------------------------------------------------------\n"
       "\n"
       #if defined(__OS2__)
       "Commandline syntax: GED2"
       #elif defined(__WIN32__)
       "Commandline syntax: GEDW32"
       #elif defined(__MSDOS__) and defined(__386__)
       "Commandline syntax: GED386"
       #elif defined(__MSDOS__) or defined(__linux__)
       "Commandline syntax: GOLDED"
       #endif
       " [-options] [keystacking]\n"
       "\n"
       "-C<configfile>  Use a different configuration file.\n"
       "-D              Disable old obsolete configuration keywords.\n"
       "-E<echoid>      Start directly in the specified mail area.\n"
       "-F  or  -FF     Force recompile of most (or all with -FF) configuration files.\n"
       "-INSTALL[=path] Start the quick install procedure. Look in path, if given.\n"
       "-IMPORTSOUP     Import SOUP packets during startup.\n"
       "-M              Mute sounds. Disables all noises in GoldED.\n"
       "-N              Disable share-compatible file opens during startup.\n"
       #if defined(__BORLANDC__) and defined(__MSDOS__)
       "-O<ovlsize>     Set overlay size. Larger is faster, but requires more memory.\n"
       "-O<type>        Set overlay type: EMS (default), EXT or DISK.
       #endif
       "-S<sortspec>    Sorts all mail areas according to the sort specs.\n"
       "-T<seconds>     Set a timeout value. GoldED will auto-exit after timeout.\n"
       "-V  or  -VV     Verbose or Very verbose (-VV) config compile. Use -VV to debug.\n"
       "-W              Write a GOLDAREA.INC file with AREADEF's of all mail areas.\n"
       "-X,  -Y,  -Z    Reserved for debugging purposes.\n"
       "\n"
       "Any non-option parameter is stuffed into the keyboard buffer.\n"
       "\n",
       gverbuf
    );

    exit(0);
  }

  throw_init();
  throw_log = &LOG;
  CfgInit();
  GFTrkInit(gftrk_set_max);

  // Register exit cleanup functions
  atexit(Cleanup);
  signal(SIGINT, SIG_IGN);

  if(*cmdlinecfg)
    found = YES;

  if(not found) {

    #if defined(__OS2__)
    ptr = getenv("GED2");   // Look for configfilename in the environment
    #elif defined(__WIN32__)
    ptr = getenv("GEDW32");   // Look for configfilename in the environment
    #else
    ptr = NULL;
    #endif
    if(not (ptr and *ptr))
      ptr = getenv("GOLDED");   // Look for configfilename in the environment
    if(not (ptr and *ptr))
      ptr = getenv("GED");

    // Get it from environment variables
    if(ptr and *ptr) {
      PathCopy(cmdlinecfg, ptr));
      #if defined(__OS2__)
      if(not found)
        found = ExistCfg(cmdlinecfg, "ged2.cfg");
      #elif defined(__WIN32__)
      if(not found)
        found = ExistCfg(cmdlinecfg, "gedw32.cfg");
      #endif
      if(not found)
        found = ExistCfg(cmdlinecfg, "golded.cfg");
    }
    
    // Get it in current directory
    if(not found) {
      getcwd(cmdlinecfg, GMAXPATH-1);
      AddBackslash(cmdlinecfg);
      #if defined(__OS2__)
      if(not found)
        found = ExistCfg(cmdlinecfg, "ged2.cfg");
      #elif defined(__WIN32__)
      if(not found)
        found = ExistCfg(cmdlinecfg, "gedw32.cfg");
      #endif
      if(not found)
        found = ExistCfg(cmdlinecfg, "golded.cfg");
    }

    // Get it where the the .EXE file is
    if(not found) {
      strcpy(cmdlinecfg, argv[0]);
      // There must be a full path in argv[0]
      if((ptr = strrchr(cmdlinecfg, GOLD_SLASH_CHR)) != NULL)
        ptr[1] = NUL;
      else
        *cmdlinecfg = NUL;
    }
  }

  _splitpath(cmdlinecfg, drive, dir, file, ext);

  #ifdef GOLD_HAS_DRIVES
  if(*drive == NUL) {
    Path cwd;
    getcwd(cwd);
    sprintf(drive, "%c:", *cwd);
  }
  if(*dir == NUL) {
    getcurdir(*drive-'@', buf);
    sprintf(dir, "%s%s\\", *buf ? "\\" : "", buf);
  }
  #endif
  if(*file == NUL)
    strcpy(file, "golded");
  if(*ext == NUL)
    strcpy(ext, ".cfg");

  // Make filenames
  _makepath(cmdlinecfg, drive, dir, file, ext);

  strcpy(CFG->goldcfg, strupr(cmdlinecfg));
  _makepath(CFG->goldged, drive, dir, file, GCFG_EXT);
  strupr(CFG->goldged);
  _makepath(CFG->goldpath, drive, dir, "", "");
  strupr(CFG->goldpath);

  // Call install detect procedure
  if(cmdlineinstall)
    InstallDetect(cmdlineinstpath);

  // Display startup banner
  printf("%s %s", __gpid__, gversion);

  if(not fexist(cmdlinecfg)) {
    printf("\n*** Cannot start: %s not found! ***\n", cmdlinecfg);
    errorlevel = EXIT_NONAME;
    exit(0);
  }

  // Read/compile the config files
  compiled = Readgolded_cfg(cmdlineforce);

  // Call install finish procedure
  if(cmdlineinstall) {
    if(InstallFinish()) {
      printf("\n*** INSTALL not COMPLETED ***\n");
      remove(cmdlinecfg);
      errorlevel = EXIT_NONAME;
      exit(0);
    }
  }

  // Mouse support
  #ifdef GOLD_MOUSE
  if(CFG->mouse) {
    gmou.Init();
    gmou.SetLevel(GMOU_LEVEL_CURS);
  }
  #endif

  // Check registration
  CheckReg();
  MakePID(__pid__);

  // Start the log
  sprintf(__golded_longname, "%s %s", __gpid__, gversion);
  LOG.open(CFG->logfile, __golded_longname, __golded_shortname, CFG->logformat);

  // Read/compile various configs
  compiled += ReadLangCfg(cmdlineforce);
  compiled += ReadKeysCfg(cmdlineforce);
  compiled += ReadHelpCfg(cmdlineforce);

  // Initialize sound system
  InitSound();

  // Report detected multitasker
  if(not quiet) {
    if(gmtsk.detected)
      printf("\n* Running under %s.", gmtsk.name);
  }

  // Handle extended keyboard
  if(CFG->keybext != -1)
    gkbd.extkbd = CFG->keybext;

  if(cfgerrors) {
    printf("\n* ");
    printf("Total CFG errors found: %i. ", cfgerrors);
    printf("Press almost any key to continue.");
    clearkeys();
    kbclear();
    kbxget();
  }

  if(AL.areas() == 0) {
    LOG.ErrConfig();
    LOG.printf("! There do not seem to be any mail areas defined.");
    LOG.printf("+ Advice: Check your setup of mail areas.");
    ConfigErrorExit();
  }

  if(CFG->usernames == 0) {
    LOG.ErrConfig();
    LOG.printf("! There do not seem to be any USERNAME's defined.");
    LOG.printf("+ Advice: Check your setup of USERNAME's.");
    ConfigErrorExit();
  }

  if(CFG->cmdkeys == 0) {
    LOG.ErrConfig();
    LOG.printf("! There do not seem to be any command keys defined.");
    LOG.printf("+ Advice: Check your key setup in GOLDKEYS.CFG.");
    ConfigErrorExit();
  }

  if(CFG->keybclear) {
    clearkeys();
    kbclear();
    if(*CFG->keybstack)             // The config keys
      kbputstr(CFG->keybstack);
  }
  else if(*keybuf)                  // The commandline keys
    kbputstr(keybuf);
  else if(*CFG->keybstack)          // The config keys
    kbputstr(CFG->keybstack);

  // If keyboard polling is enabled
  if(CFG->keybmode == KEYB_POLL) {

    // Switch to keyboard polling instead of blocking
    gkbd.polling = true;
    gkbd.tickinterval = 1000;   // Tick once per second
    gkbd.tickvalue = gclock();  // Reset starting tick
    kbdsettickfunc(update_statuslines);
  }

  #if defined(__MSDOS__)
  if(CFG->screenusebios)
    gvid.setdevice(GVID_BIO);
  else if(CFG->screenelimsnow)
    gvid.setdevice(GVID_CGA);
  else
    gvid.setdevice(GVID_DMA);
  #endif

  // Set palette if changes were specified
  if(CFG->screenpalette[16])
    gvid.setpalette(CFG->screenpalette);
  
  printf("\n");

  vposget(&gvid.orig.cursor.row, &gvid.orig.cursor.column);

  oldscreen = vsave();

  if(CFG->screensize > 0xFF)
    gvid.setmode(CFG->screensize >> 8);    // Set video mode
  else if(CFG->screensize) {
    gvid.setrows(CFG->screensize);         // Just set rows
  if(CFG->intensecolors)
    gvid.setintensity(true);
  if(C_BACKB != 0)
    gvid.setoverscan(C_BACKB);

  vcurhide();

  if(CFG->screenmaxrow >= 10)
    gvid.numrows = CFG->screenmaxrow;
  if(CFG->screenmaxcol >= 80)
    gvid.numcols = CFG->screenmaxcol;

  CFG->dispmargin = CFG->cfgdispmargin;
  if(CFG->dispmargin <= 0)
    CFG->dispmargin += gvid.numcols;

  if(CFG->editmargin <= 0)
    CFG->editmargin += gvid.numcols;

  CFG->quotemargin = CFG->cfgquotemargin;
  if(CFG->quotemargin <= 0)
    CFG->quotemargin += gvid.numcols;

  if(CFG->editquotemargin <= 0)
    CFG->editquotemargin += gvid.numcols;

  fieldupd = CFG->editfieldclear ? 2 : 1;

  inforow = ((gvid.numrows-1)/2)+6;

  w_back();     // Open a nice background window
  w_brag();     // Display Brag Window

  HandleGEvent(EVTT_STARTUP);

  if(CFG->origin.size() == 0)
    CfgOrigin("");

  // Write the binary config if any of the cfgs were compiled.
  if(compiled)
    WriteGoldGed();

  // Override timeout from commandline
  if(cmdlinetimeout != -1)
    CFG->timeout = cmdlinetimeout;

  // Init help system
  HelpInit();

  // Adjust header item sizes if negative
  if(CFG->disphdrnameset.pos < 0)    CFG->disphdrnameset.pos += gvid.numcols;
  if(CFG->disphdrnodeset.pos < 0)    CFG->disphdrnodeset.pos += gvid.numcols;
  if(CFG->disphdrdateset.pos < 0)    CFG->disphdrdateset.pos += gvid.numcols;
  if(CFG->disphdrnameset.len < 0)    CFG->disphdrnameset.len += gvid.numcols;
  if(CFG->disphdrnodeset.len < 0)    CFG->disphdrnodeset.len += gvid.numcols;
  if(CFG->disphdrdateset.len < 0)    CFG->disphdrdateset.len += gvid.numcols;

  if(CFG->edithdrnamepos < 0)  CFG->edithdrnamepos += gvid.numcols;
  if(CFG->edithdrnamelen < 0)  CFG->edithdrnamelen += gvid.numcols;
  if(CFG->edithdrnodepos < 0)  CFG->edithdrnodepos += gvid.numcols;
  if(CFG->edithdrnodelen < 0)  CFG->edithdrnodelen += gvid.numcols;

  AreaInit();

  update_statuslines();
  ReadEcholists();            // Read the Confmail compatible echotag lists
  update_statuslines();
  XLAT->load_escsets();       // Read the escape tables
  update_statuslines();

  #if defined(__BORLANDC__) and defined(__MSDOS__)
  if(coreleft()/1024L <= 50L) {
    whelpcat(H_EMemoryWarning);
    call_help();
    whelpcat(H_General);
  }
  #endif
  
  // Initialize the messagebases
  update_statuslinef(CFG->language[LNG_LockShareCap], CFG->language[LNG_Checking]);
  gmb_wide_log = &LOG;
  gmb_wide_debug = cmdlinedebughg;
  gmb_wide_canlock = CFG->sharemode ? TestLockPath(CFG->goldpath) : false;
  gmb_wide_msgsize = CFG->editmsgsize;
  gmb_wide_usernames = CFG->usernames;
  gmb_wide_username = new const char*[gmb_wide_usernames];
  for(int w=0; w<gmb_wide_usernames; w++)
    gmb_wide_username[w] = CFG->username[w].name;
  gmb_wide_sharemode = CFG->sharemode;
  gmb_wide_dispsoftcr = CFG->dispsoftcr;
  gmb_wide_personalmail = CFG->personalmail;
  if(AL.msgbases & GMB_ADEPTXBBS) {
    update_statuslinef("%s AdeptXBBS", CFG->language[LNG_Checking]);
    AL.adeptxbbs_init(CFG->adeptxbbspath, CFG->adeptxbbsuserno);
  }
  if(AL.msgbases & GMB_EZYCOM) {
    update_statuslinef("%s Ezycom", CFG->language[LNG_Checking]);
    AL.ezycom_init(CFG->ezycommsgbasepath, CFG->ezycomuserbasepath, CFG->ezycomuserno);
  }
  if(AL.msgbases & GMB_FIDO) {
    update_statuslinef("%s Fido", CFG->language[LNG_Checking]);
    AL.fido_init(CFG->fidolastread, CFG->fidohwmarks, CFG->fidonullfix, CFG->fidouserno, CFG->squishuserpath);
  }
  if(AL.msgbases & GMB_GOLDBASE) {
    update_statuslinef("%s Goldbase", CFG->language[LNG_Checking]);
    AL.goldbase_init(CFG->goldbasepath, CFG->goldbasesyspath, CFG->goldbaseuserno);
  }
  if(AL.msgbases & GMB_HUDSON) {
    update_statuslinef("%s Hudson", CFG->language[LNG_Checking]);
    AL.hudson_init(CFG->hudsonpath, CFG->hudsonsyspath, CFG->hudsonuserno, CFG->hudsonsizewarn, CFG->ra2usersbbs);
  }
  if(AL.msgbases & GMB_JAM) {
    update_statuslinef("%s JAM", CFG->language[LNG_Checking]);
    AL.jam_init(CFG->jampath, CFG->jamharddelete);
  }
  if(AL.msgbases & GMB_PCBOARD) {
    update_statuslinef("%s PCBoard", CFG->language[LNG_Checking]);
    AL.pcboard_init(CFG->pcboardpath, CFG->pcboarduserno);
  }
  if(AL.msgbases & GMB_SQUISH) {
    update_statuslinef("%s Squish", CFG->language[LNG_Checking]);
    AL.squish_init(CFG->squishuserpath, CFG->squishuserno, CFG->squishdirect, CFG->squishscan);
  }
  if(AL.msgbases & GMB_WILDCAT) {
    update_statuslinef("%s WildCat!", CFG->language[LNG_Checking]);
    AL.wildcat_init(CFG->wildcatuserno);
  }
  update_statuslinef("%s ...", CFG->language[LNG_Checking]);

  // Delete the D'Bridge "mail waiting" semaphore files
  remove(AddPath(CFG->areapath, "DBRIDGE.NMW"));
  remove(AddPath(CFG->areapath, "DBRIDGE.EMW"));

  // Unlink windows
  wunlink(W_READ);
}


//  ------------------------------------------------------------------

