
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gekeys.h,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Keyboard.
//  ------------------------------------------------------------------


#include <gkbdsets.h>


//  ------------------------------------------------------------------
//
//  The keyboard system in GoldED
//
//  Keycodes in the range 0x0000-0xFDFF are normal keys.
//  Keycodes in the range 0xFE00-0xFEFF are macro numbers.
//  Keycodes in the range 0xFF00-0xFFFE are command numbers.
//
//  The global variable cmdkeys holds a list of all commands and
//  the keys or macro numbers, sorted by key/macro.
//
//  The global variable macros holds all the macros, sorted by key.
//  The global variable totmacros contains the total number of macros.
//
//  The macro list is searched first, then the command list, thus
//  allowing macros that override commands.
//
//  ------------------------------------------------------------------

const Key KK_Auto                 = 0xFD00;
const Key KK_Macro                = 0xFE00;
const Key KK_AreaMacro            = 0xFE01;
const Key KK_EditMacro            = 0xFE02;
const Key KK_FileMacro            = 0xFE03;
const Key KK_ListMacro            = 0xFE04;
const Key KK_NodeMacro            = 0xFE05;
const Key KK_ReadMacro            = 0xFE06;

const Key KK_Commands             = 0xFEE0;
const Key KK_ExternUtil01         = 0xFEE1;
const Key KK_ExternUtil02         = 0xFEE2;
const Key KK_ExternUtil03         = 0xFEE3;
const Key KK_ExternUtil04         = 0xFEE4;
const Key KK_ExternUtil05         = 0xFEE5;
const Key KK_ExternUtil06         = 0xFEE6;
const Key KK_ExternUtil07         = 0xFEE7;
const Key KK_ExternUtil08         = 0xFEE8;
const Key KK_ExternUtil09         = 0xFEE9;
const Key KK_ExternUtil10         = 0xFEEA;
const Key KK_ExternUtil11         = 0xFEEB;
const Key KK_ExternUtil12         = 0xFEEC;
const Key KK_ExternUtil13         = 0xFEED;
const Key KK_ExternUtil14         = 0xFEEE;
const Key KK_ExternUtil15         = 0xFEEF;
const Key KK_ExternUtil16         = 0xFEF0;
const Key KK_ExternUtil17         = 0xFEF1;
const Key KK_ExternUtil18         = 0xFEF2;
const Key KK_ExternUtil19         = 0xFEF3;
const Key KK_ExternUtil20         = 0xFEF4;
const Key KK_ExternUtil21         = 0xFEF5;
const Key KK_ExternUtil22         = 0xFEF6;
const Key KK_ExternUtil23         = 0xFEF7;
const Key KK_ExternUtil24         = 0xFEF8;
const Key KK_AreaAbort            = 0xFEF9;
const Key KK_AreaAskExit          = 0xFEFA;
const Key KK_AreaBoardnos         = 0xFEFB;
const Key KK_AreaDosShell         = 0xFEFC;
const Key KK_AreaGotoFirst        = 0xFEFD;
const Key KK_AreaGotoLast         = 0xFEFE;
const Key KK_AreaGotoNext         = 0xFEFF;
const Key KK_AreaGotoPrev         = 0xFF00;
const Key KK_AreaHeat             = 0xFF01;
const Key KK_AreaJump             = 0xFF02;
const Key KK_AreaJumpNextMatch    = 0xFF03;
const Key KK_AreaMainMenu         = 0xFF04;
const Key KK_AreaMenu             = 0xFF05;
const Key KK_AreaQuitNow          = 0xFF06;
const Key KK_AreaScan             = 0xFF07;
const Key KK_AreaScanPM           = 0xFF08;
const Key KK_AreaSelect           = 0xFF09;
const Key KK_AreaSoundkill        = 0xFF0A;
const Key KK_AreaToggle           = 0xFF0B;
const Key KK_AreaZap              = 0xFF0C;
const Key KK_EditAbort            = 0xFF0D;
const Key KK_EditAnchor           = 0xFF0E;
const Key KK_EditAskExit          = 0xFF0F;
const Key KK_EditClearDeleteBuf   = 0xFF10;
const Key KK_EditClearPasteBuf    = 0xFF11;
const Key KK_EditCopy             = 0xFF12;
const Key KK_EditCopyAbove        = 0xFF13;
const Key KK_EditCopyAboveChar    = 0xFF14;
const Key KK_EditCut              = 0xFF15;
const Key KK_EditDelChar          = 0xFF16;
const Key KK_EditDeleteEOL        = 0xFF17;
const Key KK_EditDelLeft          = 0xFF18;
const Key KK_EditDelLine          = 0xFF19;
const Key KK_EditDelLtWord        = 0xFF1A;
const Key KK_EditDelRtWord        = 0xFF1B;
const Key KK_EditDosShell         = 0xFF1C;
const Key KK_EditDupLine          = 0xFF1D;
const Key KK_EditExitMsg          = 0xFF1E;
const Key KK_EditExportText       = 0xFF1F;
const Key KK_EditGoBegLine        = 0xFF20;
const Key KK_EditGoBotLine        = 0xFF21;
const Key KK_EditGoBotMsg         = 0xFF22;
const Key KK_EditGoDown           = 0xFF23;
const Key KK_EditGoEOL            = 0xFF24;
const Key KK_EditGoLeft           = 0xFF25;
const Key KK_EditGoPgDn           = 0xFF26;
const Key KK_EditGoPgUp           = 0xFF27;
const Key KK_EditGoRight          = 0xFF28;
const Key KK_EditGoTopLine        = 0xFF29;
const Key KK_EditGoTopMsg         = 0xFF2A;
const Key KK_EditGoUp             = 0xFF2B;
const Key KK_EditGoWordLeft       = 0xFF2C;
const Key KK_EditGoWordRight      = 0xFF2D;
const Key KK_EditHeader           = 0xFF2E;
const Key KK_EditImportQuotebuf   = 0xFF2F;
const Key KK_EditImportText       = 0xFF30;
const Key KK_EditKillQuotes       = 0xFF31;
const Key KK_EditLoadFile         = 0xFF32;
const Key KK_EditLookupCursor     = 0xFF33;
const Key KK_EditLookupDest       = 0xFF34;
const Key KK_EditLookupOrig       = 0xFF35;
const Key KK_EditMainMenu         = 0xFF36;
const Key KK_EditMenu             = 0xFF37;
const Key KK_EditNewline          = 0xFF38;
const Key KK_EditPaste            = 0xFF39;
const Key KK_EditQuitNow          = 0xFF3A;
const Key KK_EditReflow           = 0xFF3B;
const Key KK_EditSaveFile         = 0xFF3C;
const Key KK_EditSaveMsg          = 0xFF3D;
const Key KK_EditSoundkill        = 0xFF3E;
const Key KK_EditSpellCheck       = 0xFF3F;
const Key KK_EditTab              = 0xFF40;
const Key KK_EditToggleCase       = 0xFF41;
const Key KK_EditToggleInsert     = 0xFF42;
const Key KK_EditToLower          = 0xFF43;
const Key KK_EditToUpper          = 0xFF44;
const Key KK_EditUnDelete         = 0xFF45;
const Key KK_FileAbort            = 0xFF46;
const Key KK_FileAskExit          = 0xFF47;
const Key KK_FileDosShell         = 0xFF48;
const Key KK_FileGotoFirst        = 0xFF49;
const Key KK_FileGotoLast         = 0xFF4A;
const Key KK_FileGotoNext         = 0xFF4B;
const Key KK_FileGotoPrev         = 0xFF4C;
const Key KK_FileMainMenu         = 0xFF4D;
const Key KK_FileMark             = 0xFF4E;
const Key KK_FileMarkAll          = 0xFF4F;
const Key KK_FileMenu             = 0xFF50;
const Key KK_FileQuitNow          = 0xFF51;
const Key KK_FileSelect           = 0xFF52;
const Key KK_FileToggleMark       = 0xFF53;
const Key KK_FileUnMark           = 0xFF54;
const Key KK_FileUnMarkAll        = 0xFF55;
const Key KK_ListAbort            = 0xFF56;
const Key KK_ListAskExit          = 0xFF57;
const Key KK_ListDosShell         = 0xFF58;
const Key KK_ListGotoBookMark     = 0xFF59;
const Key KK_ListGotoFirst        = 0xFF5A;
const Key KK_ListGotoLast         = 0xFF5B;
const Key KK_ListGotoNext         = 0xFF5C;
const Key KK_ListGotoPrev         = 0xFF5D;
const Key KK_ListMainMenu         = 0xFF5E;
const Key KK_ListMarkingOptions   = 0xFF5F;
const Key KK_ListMenu             = 0xFF60;
const Key KK_ListQuitNow          = 0xFF61;
const Key KK_ListSelect           = 0xFF62;
const Key KK_ListToggleBookMark   = 0xFF63;
const Key KK_ListToggleMark       = 0xFF64;
const Key KK_NodeAbort            = 0xFF65;
const Key KK_NodeAskExit          = 0xFF66;
const Key KK_NodeDosShell         = 0xFF67;
const Key KK_NodeGotoFirst        = 0xFF68;
const Key KK_NodeGotoLast         = 0xFF69;
const Key KK_NodeGotoNext         = 0xFF6A;
const Key KK_NodeGotoPrev         = 0xFF6B;
const Key KK_NodeMainMenu         = 0xFF6C;
const Key KK_NodeMenu             = 0xFF6D;
const Key KK_NodeQuitNow          = 0xFF6E;
const Key KK_NodeSelect           = 0xFF6F;
const Key KK_ReadAddToUserlist    = 0xFF70;
const Key KK_ReadAskExit          = 0xFF71;
const Key KK_ReadChangeAka        = 0xFF72;
const Key KK_ReadChangeAttrs      = 0xFF73;
const Key KK_ReadChangeCharsIn    = 0xFF74;
const Key KK_ReadChangeCharsOut   = 0xFF75;
const Key KK_ReadChangeMsg        = 0xFF76;
const Key KK_ReadChangeOrigin     = 0xFF77;
const Key KK_ReadChangeTemplate   = 0xFF78;
const Key KK_ReadChangeUsername   = 0xFF79;
const Key KK_ReadCommentMsg       = 0xFF7A;
const Key KK_ReadCopyMoveForward  = 0xFF7B;
const Key KK_ReadDecreaseMargin   = 0xFF7C;
const Key KK_ReadDeleteMsg        = 0xFF7D;
const Key KK_ReadDosShell         = 0xFF7E;
const Key KK_ReadFidoRenumber     = 0xFF7F;
const Key KK_ReadFileRequest      = 0xFF80;
const Key KK_ReadFindAll          = 0xFF81;
const Key KK_ReadFindHeader       = 0xFF82;
const Key KK_ReadGotoBookMark     = 0xFF83;
const Key KK_ReadGotoFirstMsg     = 0xFF84;
const Key KK_ReadGotoLastMsg      = 0xFF85;
const Key KK_ReadGotoMsgno        = 0xFF86;
const Key KK_ReadGotoNextArea     = 0xFF87;
const Key KK_ReadGotoNextMsg      = 0xFF88;
const Key KK_ReadGotoPrevArea     = 0xFF89;
const Key KK_ReadGotoPrevMsg      = 0xFF8A;
const Key KK_ReadGotoReplies      = 0xFF8B;
const Key KK_ReadGotoReply1st     = 0xFF8C;
const Key KK_ReadGotoReplyNext    = 0xFF8D;
const Key KK_ReadGotoReplyPrev    = 0xFF8E;
const Key KK_ReadIncreaseMargin   = 0xFF8F;
const Key KK_ReadLookupDest       = 0xFF90;
const Key KK_ReadLookupOrig       = 0xFF91;
const Key KK_ReadMainMenu         = 0xFF92;
const Key KK_ReadMakeUserlist     = 0xFF93;
const Key KK_ReadMarkingOptions   = 0xFF94;
const Key KK_ReadMenu             = 0xFF95;
const Key KK_ReadMessageList      = 0xFF96;
const Key KK_ReadMoveCommentMsg   = 0xFF97;
const Key KK_ReadMoveQuoteMsg     = 0xFF98;
const Key KK_ReadMsgContinue      = 0xFF99;
const Key KK_ReadMsgEnd           = 0xFF9A;
const Key KK_ReadMsgHome          = 0xFF9B;
const Key KK_ReadMsgLineDown      = 0xFF9C;
const Key KK_ReadMsgLineUp        = 0xFF9D;
const Key KK_ReadMsgPgDn          = 0xFF9E;
const Key KK_ReadMsgPgUp          = 0xFF9F;
const Key KK_ReadNewArea          = 0xFFA0;
const Key KK_ReadNewMsg           = 0xFFA1;
const Key KK_ReadQuitNow          = 0xFFA2;
const Key KK_ReadQuoteBuf         = 0xFFA3;
const Key KK_ReadQuoteMsg         = 0xFFA4;
const Key KK_ReadReplyMsg         = 0xFFA5;
const Key KK_ReadSoundkill        = 0xFFA6;
const Key KK_ReadToggleBookMark   = 0xFFA7;
const Key KK_ReadToggleHexdump    = 0xFFA8;
const Key KK_ReadToggleHidden     = 0xFFA9;
const Key KK_ReadToggleHiddKlud   = 0xFFAA;
const Key KK_ReadToggleKludge     = 0xFFAB;
const Key KK_ReadToggleMark       = 0xFFAC;
const Key KK_ReadToggleMarkRead   = 0xFFAD;
const Key KK_ReadTogglePageBar    = 0xFFAE;
const Key KK_ReadToggleQuote      = 0xFFAF;
const Key KK_ReadToggleRealMsgno  = 0xFFB0;
const Key KK_ReadToggleROT13      = 0xFFB1;
const Key KK_ReadToggleTwits      = 0xFFB2;
const Key KK_ReadTouchNetscan     = 0xFFB3;
const Key KK_ReadWriteMsg         = 0xFFB4;

const Key KK_ListToggleWideSubj   = 0xFFB5;
const Key KK_EditZapQuoteBelow    = 0xFFB6;
const Key KK_AreaTouchNetscan     = 0xFFB7;
const Key KK_ReadExternUtilMenu   = 0xFFB8;
const Key KK_ReadChangeTagline    = 0xFFB9;
const Key KK_ReadStealTagline     = 0xFFBA;
const Key KK_ReadUUDecode         = 0xFFBB;
const Key KK_AreaInfo             = 0xFFBC;
const Key KK_ReadChangeXlatImport = 0xFFBD;
const Key KK_AreaUndefine         = 0xFFBE;
const Key KK_EditUndefine         = 0xFFBF;
const Key KK_FileUndefine         = 0xFFC0;
const Key KK_ListUndefine         = 0xFFC1;
const Key KK_NodeUndefine         = 0xFFC2;
const Key KK_ReadUndefine         = 0xFFC3;
const Key KK_ListToggleDate       = 0xFFC4;
const Key KK_ReadTouchSemaphore   = 0xFFC5;
const Key KK_AreaWriteGoldlast    = 0xFFC6;
const Key KK_ReadChangeMood       = 0xFFC7;
const Key KK_ReadGotoNextUnread   = 0xFFC8;
const Key KK_ReadGotoPrevUnread   = 0xFFC9;

const Key KK_ZzzzLast             = 0xFFCA;


//  ------------------------------------------------------------------
//  Normal keycodes

const Key K_0         = '0';
const Key K_1         = '1';
const Key K_2         = '2';
const Key K_3         = '3';
const Key K_4         = '4';
const Key K_5         = '5';
const Key K_6         = '6';
const Key K_7         = '7';
const Key K_8         = '8';
const Key K_9         = '9';

const Key K_A         = 'A';
const Key K_B         = 'B';
const Key K_C         = 'C';
const Key K_D         = 'D';
const Key K_E         = 'E';
const Key K_F         = 'F';
const Key K_G         = 'G';
const Key K_H         = 'H';
const Key K_I         = 'I';
const Key K_J         = 'J';
const Key K_K         = 'K';
const Key K_L         = 'L';
const Key K_M         = 'M';
const Key K_N         = 'N';
const Key K_O         = 'O';
const Key K_P         = 'P';
const Key K_Q         = 'Q';
const Key K_R         = 'R';
const Key K_S         = 'S';
const Key K_T         = 'T';
const Key K_U         = 'U';
const Key K_V         = 'V';
const Key K_W         = 'W';
const Key K_X         = 'X';
const Key K_Y         = 'Y';
const Key K_Z         = 'Z';

const Key K_Lesser    = '<';
const Key K_Greater   = '>';
const Key K_Multi     = '*';
const Key K_Plus      = '+';
const Key K_Minus     = '-';


//  ------------------------------------------------------------------
//  Alt-Alphanum

const Key K_A0        = 0x8100;
const Key K_A1        = 0x7800;
const Key K_A2        = 0x7900;
const Key K_A3        = 0x7A00;
const Key K_A4        = 0x7B00;
const Key K_A5        = 0x7C00;
const Key K_A6        = 0x7D00;
const Key K_A7        = 0x7E00;
const Key K_A8        = 0x7F00;
const Key K_A9        = 0x8000;

const Key K_AA        = 0x1E00;
const Key K_AB        = 0x3000;
const Key K_AC        = 0x2E00;
const Key K_AD        = 0x2000;
const Key K_AE        = 0x1200;
const Key K_AF        = 0x2100;
const Key K_AG        = 0x2200;
const Key K_AH        = 0x2300;
const Key K_AI        = 0x1700;
const Key K_AJ        = 0x2400;
const Key K_AK        = 0x2500;
const Key K_AL        = 0x2600;
const Key K_AM        = 0x3200;
const Key K_AN        = 0x3100;
const Key K_AO        = 0x1800;
const Key K_AP        = 0x1900;
const Key K_AQ        = 0x1000;
const Key K_AR        = 0x1300;
const Key K_AS        = 0x1F00;
const Key K_AT        = 0x1400;
const Key K_AU        = 0x1600;
const Key K_AV        = 0x2F00;
const Key K_AW        = 0x1100;
const Key K_AX        = 0x2D00;
const Key K_AY        = 0x1500;
const Key K_AZ        = 0x2C00;


//  ------------------------------------------------------------------
//  Ctrl-Alphanum

const Key K_C2        = 0x0300;

const Key K_CA        = 0x1E01;
const Key K_CB        = 0x3002;
const Key K_CC        = 0x2E03;
const Key K_CD        = 0x2004;
const Key K_CE        = 0x1205;
const Key K_CF        = 0x2106;
const Key K_CG        = 0x2207;
const Key K_CH        = 0x2308;
const Key K_CI        = 0x1709;
const Key K_CJ        = 0x240A;
const Key K_CK        = 0x250B;
const Key K_CL        = 0x260C;
const Key K_CM        = 0x320D;
const Key K_CN        = 0x310E;
const Key K_CO        = 0x180F;
const Key K_CP        = 0x1910;
const Key K_CQ        = 0x1011;
const Key K_CR        = 0x1312;
const Key K_CS        = 0x1F13;
const Key K_CT        = 0x1414;
const Key K_CU        = 0x1615;
const Key K_CV        = 0x2F16;
const Key K_CW        = 0x1117;
const Key K_CX        = 0x2D18;
const Key K_CY        = 0x1519;
const Key K_CZ        = 0x2C1A;


//  ------------------------------------------------------------------
//  Function keys

const Key K_F1        = 0x3B00;
const Key K_F2        = 0x3C00;
const Key K_F3        = 0x3D00;
const Key K_F4        = 0x3E00;
const Key K_F5        = 0x3F00;
const Key K_F6        = 0x4000;
const Key K_F7        = 0x4100;
const Key K_F8        = 0x4200;
const Key K_F9        = 0x4300;
const Key K_F10       = 0x4400;
const Key K_F11       = 0x8500;
const Key K_F12       = 0x8600;

const Key K_AF1       = 0x6800;
const Key K_AF2       = 0x6900;
const Key K_AF3       = 0x6A00;
const Key K_AF4       = 0x6B00;
const Key K_AF5       = 0x6C00;
const Key K_AF6       = 0x6D00;
const Key K_AF7       = 0x6E00;
const Key K_AF8       = 0x6F00;
const Key K_AF9       = 0x7000;
const Key K_AF10      = 0x7100;
const Key K_AF11      = 0x8B00;
const Key K_AF12      = 0x8C00;

const Key K_CF1       = 0x5E00;
const Key K_CF2       = 0x5F00;
const Key K_CF3       = 0x6000;
const Key K_CF4       = 0x6100;
const Key K_CF5       = 0x6200;
const Key K_CF6       = 0x6300;
const Key K_CF7       = 0x6400;
const Key K_CF8       = 0x6500;
const Key K_CF9       = 0x6600;
const Key K_CF10      = 0x6700;
const Key K_CF11      = 0x8900;
const Key K_CF12      = 0x8A00;

const Key K_SF1       = 0x5400;
const Key K_SF2       = 0x5500;
const Key K_SF3       = 0x5600;
const Key K_SF4       = 0x5700;
const Key K_SF5       = 0x5800;
const Key K_SF6       = 0x5900;
const Key K_SF7       = 0x5A00;
const Key K_SF8       = 0x5B00;
const Key K_SF9       = 0x5C00;
const Key K_SF10      = 0x5D00;
const Key K_SF11      = 0x8700;
const Key K_SF12      = 0x8800;


//  ------------------------------------------------------------------
//  Special keys

const Key K_Home      = 0x4700;
const Key K_SHome     = 0x4737;
const Key K_CHome     = 0x7700;
const Key K_AHome     = 0x9700;

const Key K_End       = 0x4F00;
const Key K_SEnd      = 0x4F31;
const Key K_CEnd      = 0x7500;
const Key K_AEnd      = 0x9F00;

const Key K_Up        = 0x4800;
const Key K_SUp       = 0x4838;
const Key K_CUp       = 0x8D00;
const Key K_AUp       = 0x9800;

const Key K_Down      = 0x5000;
const Key K_SDown     = 0x5032;
const Key K_CDown     = 0x9100;
const Key K_ADown     = 0xA000;

const Key K_Left      = 0x4B00;
const Key K_SLeft     = 0x4B34;
const Key K_CLeft     = 0x7300;
const Key K_ALeft     = 0x9B00;

const Key K_Right     = 0x4D00;
const Key K_SRight    = 0x4D36;
const Key K_CRight    = 0x7400;
const Key K_ARight    = 0x9D00;

const Key K_PgUp      = 0x4900;
const Key K_SPgUp     = 0x4939;
const Key K_CPgUp     = 0x8400;
const Key K_APgUp     = 0x9900;

const Key K_PgDn      = 0x5100;
const Key K_SPgDn     = 0x5133;
const Key K_CPgDn     = 0x7600;
const Key K_APgDn     = 0xA100;

const Key K_BSpace    = 0x0E08;
const Key K_ABSpace   = 0x0E7F;
const Key K_CBSpace   = 0x007F;

const Key K_Mul       = 0x372A;
const Key K_Sub       = 0x4A2D;
const Key K_Add       = 0x4E2B;
const Key K_Div       = 0x352F;

const Key K_CGMul     = 0x7200;

const Key K_Esc       = 0x011B;

const Key K_AMinus    = 0x8200;

const Key K_ATab      = 0xA500;
const Key K_STab      = 0x0F00;
const Key K_Tab       = 0x0F09;

const Key K_AEnter    = 0x1C00;
const Key K_CEnter    = 0x1C0A;
const Key K_Enter     = 0x1C0D;

const Key K_Ins       = 0x5200;
const Key K_SIns      = 0x5230;
const Key K_CIns      = 0x9200;
const Key K_AIns      = 0xA200;

const Key K_Del       = 0x5300;
const Key K_SDel      = 0x532E;
const Key K_CDel      = 0x9300;
const Key K_ADel      = 0xA300;

const Key K_Key5      = 0x4C00;

const Key K_Space     = 0x3920;

const Key K_CtrlBreak = 0xFFFF;


//  ------------------------------------------------------------------
//  Key groups (type of key)

const int KT_A =  1;   // Arealist
const int KT_E =  2;   // Editor
const int KT_G =  4;   // General (not used)
const int KT_M =  8;   // Msg lister
const int KT_N = 16;   // Node browser
const int KT_R = 32;   // Reader                  (+ external utils)
const int KT_F = 64;   // File browser


//  ------------------------------------------------------------------

class golded_kbd {

protected:

  gkey  key;
  gkey  command;
  char* macrostring;
  word  command_crc;

  int switch_area();
  int switch_edit();
  int switch_file();
  int switch_list();
  int switch_node();
  int switch_read();

  int get_macro();

public:

  enum {
    success,
    error_unknown_keycode,
    error_unknown_command,
    error_unknown_in_macro
  };

  gkeyset area;
  gkeyset edit;
  gkeyset file;
  gkeyset glob;
  gkeyset list;
  gkeyset node;
  gkeyset read;

  int switch_keyboard(char cmd, word cmdcrc, word keycrc, char* extra);

};


//  ------------------------------------------------------------------

