
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geline.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Conversion of a raw message to a linked list of lines.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

const int BODYLINE = 0;
const int HEADERLINE = 1;
const int USERDEFINED = 2;
const int RFC_X = 3;

enum {
  MASK_FTS = 0x1000,
  FTS_AREA,
  FTS_INTL,
  FTS_FMPT,
  FTS_TOPT,
  FTS_MSGID,
  FTS_REPLY,
  FTS_SEENBY,
  FTS_PATH,
  FTS_ZZZZ
};

enum {
  MASK_FSC = 0x2000,
  FSC_CHARSET,
  FSC_CHRC,
  FSC_CHRS,
  FSC_DOMAIN,
  FSC_EID,
  FSC_ENC,
  FSC_FLAGS,
  FSC_FWDFROM,
  FSC_FWDORIG,
  FSC_FWDTO,
  FSC_FWDDEST,
  FSC_FWDSUBJ,
  FSC_FWDAREA,
  FSC_FWDMSGID,
  FSC_I51,
  FSC_MSGTO,
  FSC_PID,
  FSC_PTH,
  FSC_REPLYADDR,
  FSC_REPLYTO,
  FSC_SPLIT,
  FSC_SPTH,
  FSC_TID,
  FSC_ZZZZ
};

enum {
  MASK_XXX = 0x8000,
  XXX_ACUPDATE,
  XXX_CODEPAGE,
  XXX_DESTADDR,
  XXX_ENCRYPTION,
  XXX_EOT,
  XXX_GATECHK,
  XXX_GID,
  XXX_GIF,
  XXX_GMD,
  XXX_GROUP,
  XXX_MOOD,
  XXX_MSGSEQ,
  XXX_NOTE,
  XXX_ORIGID,
  XXX_ORIGINAL,
  XXX_ORIGREF,
  XXX_RECD,
  XXX_RFC,
  XXX_RFD,
  XXX_RID,
  XXX_SN,
  XXX_SOT,
  XXX_TCL1,
  XXX_TCL2,
  XXX_TZUTCINFO,
  XXX_TZUTC,
  XXX_TZ,
  XXX_VIA,
  XXX_XID,
  XXX_ZZZZ
};

enum {
  MASK_RFC = 0x4000,
  RFC_ALSO_CONTROL,
  RFC_APPARENTLY_TO,
  RFC_APPROVED,
  RFC_ARTICLE_NAMES,
  RFC_ARTICLE_UPDATES,
  RFC_BCC,
  RFC_CC,
  RFC_COMMENT,
  RFC_COMMENTS,
  RFC_CONTENT_DESCRIPTION,
  RFC_CONTENT_DISPOSITION,
  RFC_CONTENT_ID,
  RFC_CONTENT_LENGTH,
  RFC_CONTENT_TRANSFER_ENCODING,
  RFC_CONTENT_TYPE,
  RFC_CONTROL,
  RFC_DATE,
  RFC_DELIVERY_DATE,
  RFC_DISTRIBUTION,
  RFC_ENCRYPTED,
  RFC_ERRORS_TO,
  RFC_EXPIRES,
  RFC_FOLLOWUP_TO,
  RFC_FROM,
  RFC_FROMX,
  RFC_IN_REPLY_TO,
  RFC_KEYWORDS,
  RFC_LINES,
  RFC_MESSAGE_ID,
  RFC_MIME_VERSION,
  RFC_NEWSGROUPS,
  RFC_NEWS_SOFTWARE,
  RFC_NNTP_POSTING_HOST,
  RFC_NNTP_POSTING_USER,
  RFC_OLD_DATE,
  RFC_ORGANIZATION,
  RFC_ORIGINATOR,
  RFC_PATH,
  RFC_PRECEDENCE,
  RFC_PRIORITY,
  RFC_RECEIVED,
  RFC_REFERENCES,
  RFC_REPLY_PATH,
  RFC_REPLY_TO,
  RFC_RETURN_PATH,
  RFC_RETURN_RECEIPT_TO,
  RFC_SEE_ALSO,
  RFC_SENDER,
  RFC_STATUS,
  RFC_SUBJECT,
  RFC_SUMMARY,
  RFC_SUPERSEDES,
  RFC_TO,
  RFC_VERSION,
  RFC_XREF,
  RFC_X_CHARSET,
  RFC_X_CHAR_ESC,
  RFC_X_FTN_TO,
  RFC_X_MAILREADER,
  RFC_X_NEWSREADER,
  RFC_X_TO,
  RFC_RNEWS,
  RFC_ZZZZ
};

#define MASK_ALL (MASK_FTS|MASK_FSC|MASK_RFC|MASK_XXX)


//  ------------------------------------------------------------------

struct Kludges {
  char* key;
  uint  num;
  byte  req;
};


//  ------------------------------------------------------------------

const byte KCRQ_NONE  = 0x0000;
const byte KCRQ_COLON = 0x0001;
const byte KCRQ_CASE  = 0x0002;


//  ------------------------------------------------------------------

static Kludges __FAR fts_list[] = {

  { "AREA"                       , FTS_AREA                      , KCRQ_CASE  },
  { "INTL"                       , FTS_INTL                      , KCRQ_CASE  },
  { "FMPT"                       , FTS_FMPT                      , KCRQ_CASE  },
  { "TOPT"                       , FTS_TOPT                      , KCRQ_CASE  },
  { "MSGID"                      , FTS_MSGID                     , KCRQ_CASE  },
  { "REPLY"                      , FTS_REPLY                     , KCRQ_CASE  },
  { "SEEN-BY"                    , FTS_SEENBY                    , KCRQ_CASE  },
  { "PATH"                       , FTS_PATH                      , KCRQ_CASE  },
  { ""                           , FTS_ZZZZ                      , KCRQ_NONE  },
};


//  ------------------------------------------------------------------

static Kludges __FAR fsc_list[] = {

  { "CHARSET"                    , FSC_CHARSET                   , KCRQ_CASE  },
  { "CHRC"                       , FSC_CHRC                      , KCRQ_CASE  },
  { "CHRS"                       , FSC_CHRS                      , KCRQ_CASE  },
  { "DOMAIN"                     , FSC_DOMAIN                    , KCRQ_CASE  },
  { "EID"                        , FSC_EID                       , KCRQ_CASE  },
  { "ENC"                        , FSC_ENC                       , KCRQ_CASE  },
  { "FLAGS"                      , FSC_FLAGS                     , KCRQ_CASE  },
  { "FWDFROM"                    , FSC_FWDFROM                   , KCRQ_CASE  },
  { "FWDORIG"                    , FSC_FWDORIG                   , KCRQ_CASE  },
  { "FWDTO"                      , FSC_FWDTO                     , KCRQ_CASE  },
  { "FWDDEST"                    , FSC_FWDDEST                   , KCRQ_CASE  },
  { "FWDSUBJ"                    , FSC_FWDSUBJ                   , KCRQ_CASE  },
  { "FWDAREA"                    , FSC_FWDAREA                   , KCRQ_CASE  },
  { "FWDMSGID"                   , FSC_FWDMSGID                  , KCRQ_CASE  },
  { "I51"                        , FSC_I51                       , KCRQ_CASE  },
  { "MSGTO"                      , FSC_MSGTO                     , KCRQ_CASE  },
  { "PID"                        , FSC_PID                       , KCRQ_CASE  },
  { "PTH"                        , FSC_PTH                       , KCRQ_CASE  },
  { "REPLYADDR"                  , FSC_REPLYADDR                 , KCRQ_CASE  },
  { "REPLYTO"                    , FSC_REPLYTO                   , KCRQ_CASE  },
  { "SPLIT"                      , FSC_SPLIT                     , KCRQ_CASE  },
  { "SPTH"                       , FSC_SPTH                      , KCRQ_CASE  },
  { "TID"                        , FSC_TID                       , KCRQ_CASE  },
  { ""                           , FSC_ZZZZ                      , KCRQ_NONE  },
};


//  ------------------------------------------------------------------

static Kludges __FAR xxx_list[] = {

  { "ACUPDATE"                   , XXX_ACUPDATE                  , KCRQ_CASE  },
  { "CODEPAGE"                   , XXX_CODEPAGE                  , KCRQ_CASE  },
  { "DESTADDR"                   , XXX_DESTADDR                  , KCRQ_CASE  },
  { "ENCRYPTION"                 , XXX_ENCRYPTION                , KCRQ_CASE  },
  { "EOT"                        , XXX_EOT                       , KCRQ_CASE  },
  { "GATECHK"                    , XXX_GATECHK                   , KCRQ_CASE  },
  { "GID"                        , XXX_GID                       , KCRQ_CASE  },
  { "GIF"                        , XXX_GIF                       , KCRQ_CASE  },
  { "GMD"                        , XXX_GMD                       , KCRQ_CASE  },
  { "GROUP"                      , XXX_GROUP                     , KCRQ_CASE  },
  { "MOOD"                       , XXX_MOOD                      , KCRQ_CASE  },
  { "MSGSEQ"                     , XXX_MSGSEQ                    , KCRQ_CASE  },
  { "NOTE"                       , XXX_NOTE                      , KCRQ_CASE  },
  { "ORIGID"                     , XXX_ORIGID                    , KCRQ_CASE  },
  { "Original"                   , XXX_ORIGINAL                  , KCRQ_NONE  },
  { "ORIGREF"                    , XXX_ORIGREF                   , KCRQ_CASE  },
  { "Recd"                       , XXX_RECD                      , KCRQ_CASE  },
  { "RFC"                        , XXX_RFC                       , KCRQ_CASE  },
  { "RFD"                        , XXX_RFD                       , KCRQ_CASE  },
  { "RID"                        , XXX_RID                       , KCRQ_CASE  },
  { "SN"                         , XXX_SN                        , KCRQ_CASE  },
  { "SOT"                        , XXX_SOT                       , KCRQ_CASE  },
  { "TCL1"                       , XXX_TCL1                      , KCRQ_CASE  },
  { "TCL2"                       , XXX_TCL2                      , KCRQ_CASE  },
  { "TZUTCINFO"                  , XXX_TZUTCINFO                 , KCRQ_CASE  },
  { "TZUTC"                      , XXX_TZUTC                     , KCRQ_CASE  },
  { "TZ"                         , XXX_TZ                        , KCRQ_CASE  },
  { "Via"                        , XXX_VIA                       , KCRQ_NONE  },
  { "XID"                        , XXX_XID                       , KCRQ_CASE  },
  { ""                           , XXX_ZZZZ                      , KCRQ_NONE  }
};


//  ------------------------------------------------------------------

static Kludges __FAR rfc_list[] = {

  { "Also-Control"               , RFC_ALSO_CONTROL              , KCRQ_COLON },
  { "Apparently-To"              , RFC_APPARENTLY_TO             , KCRQ_COLON },
  { "Approved"                   , RFC_APPROVED                  , KCRQ_COLON },
  { "Article-Names"              , RFC_ARTICLE_NAMES             , KCRQ_COLON },
  { "Article-Updates"            , RFC_ARTICLE_UPDATES           , KCRQ_COLON },
  { "Bcc"                        , RFC_BCC                       , KCRQ_COLON },
  { "Cc"                         , RFC_CC                        , KCRQ_COLON },
  { "Comment"                    , RFC_COMMENT                   , KCRQ_COLON },
  { "Comments"                   , RFC_COMMENTS                  , KCRQ_COLON },
  { "Content-Description"        , RFC_CONTENT_DESCRIPTION       , KCRQ_COLON },
  { "Content-Disposition"        , RFC_CONTENT_DISPOSITION       , KCRQ_COLON },
  { "Content-ID"                 , RFC_CONTENT_ID                , KCRQ_COLON },
  { "Content-Length"             , RFC_CONTENT_LENGTH            , KCRQ_COLON },
  { "Content-Transfer-Encoding"  , RFC_CONTENT_TRANSFER_ENCODING , KCRQ_COLON },
  { "Content-Type"               , RFC_CONTENT_TYPE              , KCRQ_COLON },
  { "Control"                    , RFC_CONTROL                   , KCRQ_COLON },
  { "Date"                       , RFC_DATE                      , KCRQ_COLON },
  { "Delivery-Date"              , RFC_DELIVERY_DATE             , KCRQ_COLON },
  { "Distribution"               , RFC_DISTRIBUTION              , KCRQ_COLON },
  { "Encrypted"                  , RFC_ENCRYPTED                 , KCRQ_COLON },
  { "Errors-To"                  , RFC_ERRORS_TO                 , KCRQ_COLON },
  { "Expires"                    , RFC_EXPIRES                   , KCRQ_COLON },
  { "Followup-To"                , RFC_FOLLOWUP_TO               , KCRQ_COLON },
  { "From"                       , RFC_FROM                      , KCRQ_COLON },
  { "From"                       , RFC_FROMX                     , KCRQ_NONE  },
  { "In-Reply-To"                , RFC_IN_REPLY_TO               , KCRQ_COLON },
  { "Keywords"                   , RFC_KEYWORDS                  , KCRQ_COLON },
  { "Lines"                      , RFC_LINES                     , KCRQ_COLON },
  { "Message-ID"                 , RFC_MESSAGE_ID                , KCRQ_COLON },
  { "MIME-Version"               , RFC_MIME_VERSION              , KCRQ_COLON },
  { "Newsgroups"                 , RFC_NEWSGROUPS                , KCRQ_COLON },
  { "News-Software"              , RFC_NEWS_SOFTWARE             , KCRQ_COLON },
  { "NNTP-Posting-Host"          , RFC_NNTP_POSTING_HOST         , KCRQ_COLON },
  { "NNTP-Posting-User"          , RFC_NNTP_POSTING_USER         , KCRQ_COLON },
  { "Old-Date"                   , RFC_OLD_DATE                  , KCRQ_COLON },
  { "Organization"               , RFC_ORGANIZATION              , KCRQ_COLON },
  { "Originator"                 , RFC_ORIGINATOR                , KCRQ_COLON },
  { "Path"                       , RFC_PATH                      , KCRQ_COLON },
  { "Precedence"                 , RFC_PRECEDENCE                , KCRQ_COLON },
  { "Priority"                   , RFC_PRIORITY                  , KCRQ_COLON },
  { "Received"                   , RFC_RECEIVED                  , KCRQ_COLON },
  { "References"                 , RFC_REFERENCES                , KCRQ_COLON },
  { "Reply-Path"                 , RFC_REPLY_PATH                , KCRQ_COLON },
  { "Reply-To"                   , RFC_REPLY_TO                  , KCRQ_COLON },
  { "Return-Path"                , RFC_RETURN_PATH               , KCRQ_COLON },
  { "Return-Receipt-To"          , RFC_RETURN_RECEIPT_TO         , KCRQ_COLON },
  { "See-Also"                   , RFC_SEE_ALSO                  , KCRQ_COLON },
  { "Sender"                     , RFC_SENDER                    , KCRQ_COLON },
  { "Status"                     , RFC_STATUS                    , KCRQ_COLON },
  { "Subject"                    , RFC_SUBJECT                   , KCRQ_COLON },
  { "Summary"                    , RFC_SUMMARY                   , KCRQ_COLON },
  { "Supersedes"                 , RFC_SUPERSEDES                , KCRQ_COLON },
  { "To"                         , RFC_TO                        , KCRQ_COLON },
  { "Version"                    , RFC_VERSION                   , KCRQ_COLON },
  { "Xref"                       , RFC_XREF                      , KCRQ_COLON },
  { "X-Charset"                  , RFC_X_CHARSET                 , KCRQ_COLON },
  { "X-Char-Esc"                 , RFC_X_CHAR_ESC                , KCRQ_COLON },
  { "X-FTN-To"                   , RFC_X_FTN_TO                  , KCRQ_COLON },
  { "X-Mailreader"               , RFC_X_MAILREADER              , KCRQ_COLON },
  { "X-Newsreader"               , RFC_X_NEWSREADER              , KCRQ_COLON },
  { "X-To"                       , RFC_X_TO                      , KCRQ_COLON },
  { "#!"                         , RFC_RNEWS                     , KCRQ_NONE  },
  { ""                           , RFC_ZZZZ                      , KCRQ_NONE  },
};


//  ------------------------------------------------------------------

void GMsg::KludgeAREA(const char* ptr) {

  garealistdata* ap = AL.echoid_to_pointer(ptr);
  if(ap)
    kludge.area = ap->echoid();
}


//  ------------------------------------------------------------------

void GMsg::KludgeINTL(const char* ptr) {

  kludge.intl = ptr;

  word topt = destaddr.point;
  destaddr = ptr;
  destaddr.point = topt;

  ptr = strskip_txt(ptr);
  ptr = strskip_wht(ptr);
  word fmpt = origaddr.point;
  origaddr = ptr;
  origaddr.point = fmpt;
}


//  ------------------------------------------------------------------

void GMsg::KludgeFMPT(const char* ptr) {

  kludge.fmpt = ptr;
  origaddr.point = atow(ptr);
}


//  ------------------------------------------------------------------

void GMsg::KludgeTOPT(const char* ptr) {

  kludge.topt = ptr;
  destaddr.point = atow(ptr);
}


//  ------------------------------------------------------------------

void GMsg::KludgeMSGID(const char* ptr) {

  kludge.msgid = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeSPLIT(const char* ptr) {

  kludge.split = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeREPLY(const char* ptr) {

  kludge.reply = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeDOMAIN(const char* ptr) {

  char ddom[100], daddr[100], odom[100], oaddr[100];
  sscanf(ptr, "%s %s %s %s", ddom, daddr, odom, oaddr);
  origaddr        = oaddr;
  origaddr.domain = odom;
  destaddr        = daddr;
  destaddr.domain = ddom;
}


//  ------------------------------------------------------------------

void GMsg::KludgeFLAGS(const char* ptr) {

  kludge.flags = ptr;
  attr.get(ptr);
}


//  ------------------------------------------------------------------

void GMsg::KludgeSEENBY(const char* ptr) {

  seenby.add(ptr);
}


//  ------------------------------------------------------------------

void GMsg::KludgePATH(const char* ptr) {

  path.add(ptr);
}


//  ------------------------------------------------------------------

void GMsg::KludgeMSGTO(const char* ptr) {

  daddress = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgePID(const char* ptr) {

  kludge.pid = ptr;

  const char* goldpid[] = {
    "GED ",
    "GED2 ",
    "GED3 ",
    "GED386 ",
    "GED/2 ",
    "GEDW32 ",
    "GoldED",
    NULL
  };
  int n = 0;
  while(goldpid[n]) {
    if(strnieql(ptr, goldpid[n], strlen(goldpid[n]))) {
      goldmark = '';          // Recognize another GoldED msg
      break;
    }
    n++;
  }
}


//  ------------------------------------------------------------------

void GMsg::KludgeREPLYADDR(const char* ptr) {

  kludge.replyaddr = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeREPLYTO(const char* ptr) {

  kludge.replyto = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeGIF(const char* ptr) {

  kludge.gif = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeMOOD(const char* ptr) {

  kludge.mood = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeFWDFROM(const char* ptr) {

  kludge.fwdfrom = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeFWDORIG(const char* ptr) {

  kludge.fwdorig = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeFWDTO(const char* ptr) {

  kludge.fwdto = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeFWDDEST(const char* ptr) {

  kludge.fwddest = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeFWDSUBJ(const char* ptr) {

  kludge.fwdsubj = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeFWDAREA(const char* ptr) {

  kludge.fwdarea = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeFWDMSGID(const char* ptr) {

  kludge.fwdmsgid = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeFROM(const char* ptr) {

  if(header.from) {
    header.from += ", ";
    header.from += ptr;
  }
  else {
    header.from = ptr;
  }

  grfc822_mailbox mailbox = header.from;

  if(mailbox.name)
    oname = mailbox.name;
  if(mailbox.address)
    oaddress = mailbox.address;
}


//  ------------------------------------------------------------------

void GMsg::KludgeTO(const char* ptr) {

  if(header.to) {
    header.to += ", ";
    header.to += ptr;
  }
  else {
    header.to = ptr;
  }

  grfc822_mailbox mailbox = header.to;

  if(mailbox.name)
    dname = mailbox.name;
  if(mailbox.address)
    daddress = mailbox.address;
}


//  ------------------------------------------------------------------

void GMsg::KludgeREPLY_TO(const char* ptr) {

  header.reply_to = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeSUBJECT(const char* ptr) {

  header.subject = ptr;
  subject = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeDATE(const char* ptr) {

  header.date = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeCC(const char* ptr) {

  if(header.cc) {
    header.cc += ", ";
    header.cc += ptr;
  }
  else {
    header.cc = ptr;
  }
}


//  ------------------------------------------------------------------

void GMsg::KludgeBCC(const char* ptr) {

  if(header.bcc) {
    header.bcc += ", ";
    header.bcc += ptr;
  }
  else {
    header.bcc = ptr;
  }
}


//  ------------------------------------------------------------------

void GMsg::KludgeSENDER(const char* ptr) {

  header.sender = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeFOLLOWUPTO(const char* ptr) {

  header.followup_to = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeNEWSGROUPS(const char* ptr) {

  header.newsgroups = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeMESSAGE_ID(const char* ptr) {

  header.message_id = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeREFERENCES(const char* ptr) {

  header.references = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeIN_REPLY_TO(const char* ptr) {

  header.in_reply_to = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeORGANIZATION(const char* ptr) {

  header.organization = ptr;
}


//  ------------------------------------------------------------------

void GMsg::KludgeX_FTN_TO(const char* ptr) {

  header.x_ftn_to = ptr;
}


//  ------------------------------------------------------------------

char* GMsg::unwrap_line(int lineno, const char* ptr, int addspace) {

  if(linearray.data(lineno).iswrapped()) {
    uint len = strlen(ptr);
    char* uptr = throw_strdup(ptr);
    while((lineno+1 < linearray.size()) and linearray.data(lineno).iswrapped()) {
      lineno++;
      uint nextlen = linearray.length(lineno);
      uptr = (char*)throw_realloc(uptr, len+nextlen+2);
      if(addspace and (uptr[len-1] != ' ')) {
        strcat(uptr, " ");
        len++;
      }
      strcpy(uptr+len, linearray[lineno]);
      len += nextlen;
    }
    return uptr;
  }

  return NULL;
}


//  ------------------------------------------------------------------

bool GMsg::handle_kludges(int lineno, int kludgenum, const char* ptr) {

  glinedata& linedata = linearray.data(lineno);
  char* tmp;

  switch(kludgenum) {

    case FTS_AREA:
      KludgeAREA(ptr);
      return true;

    case FTS_INTL:
      linedata.kludge = GKLUD_INTL;
      KludgeINTL(ptr);
      return true;

    case FTS_FMPT:
      linedata.kludge = GKLUD_FMPT;
      KludgeFMPT(ptr);
      return true;

    case FTS_TOPT:
      linedata.kludge = GKLUD_TOPT;
      KludgeTOPT(ptr);
      return true;

    case FTS_MSGID:
      linedata.kludge = GKLUD_MSGID;
      tmp = unwrap_line(lineno, ptr, true);
      KludgeMSGID(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case FTS_REPLY:
      linedata.kludge = GKLUD_REPLY;
      tmp = unwrap_line(lineno, ptr, true);
      KludgeREPLY(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case FTS_SEENBY:
      linedata.kludge = GKLUD_SEENBY;
      KludgeSEENBY(ptr);
      return true;

    case FTS_PATH:
      linedata.kludge = GKLUD_PATH;
      KludgePATH(ptr);
      return true;

    ////////////////////////////////////////////////////////////

    case FSC_CHARSET:
      linedata.kludge = GKLUD_CHARSET;
      kludge.charset = ptr;
      return true;

    case FSC_CHRS:
      linedata.kludge = GKLUD_CHRS;
      kludge.chrs = ptr;
      return true;

    case FSC_DOMAIN:
      linedata.kludge = GKLUD_DOMAIN;
      KludgeDOMAIN(ptr);
      return true;

    case FSC_FLAGS:
      linedata.kludge = GKLUD_FLAGS;
      KludgeFLAGS(ptr);
      return true;

    case FSC_I51:
      linedata.kludge = GKLUD_I51;
      kludge.i51 = true;
      return true;

    case FSC_MSGTO:
      KludgeMSGTO(ptr);
      return true;

    case FSC_PID:
      linedata.kludge = GKLUD_PID;
      KludgePID(ptr);
      return true;

    case FSC_REPLYADDR:
      linedata.kludge = GKLUD_REPLYADDR;
      KludgeREPLYADDR(ptr);
      return true;

    case FSC_REPLYTO:
      linedata.kludge = GKLUD_REPLYTO;
      KludgeREPLYTO(ptr);
      return true;

    case FSC_FWDFROM:
      linedata.kludge = GKLUD_FWD;
      KludgeFWDFROM(ptr);
      return true;

    case FSC_FWDORIG:
      linedata.kludge = GKLUD_FWD;
      KludgeFWDORIG(ptr);
      return true;

    case FSC_FWDTO:
      linedata.kludge = GKLUD_FWD;
      KludgeFWDTO(ptr);
      return true;

    case FSC_FWDDEST:
      linedata.kludge = GKLUD_FWD;
      KludgeFWDDEST(ptr);
      return true;

    case FSC_FWDSUBJ:
      linedata.kludge = GKLUD_FWD;
      KludgeFWDSUBJ(ptr);
      return true;

    case FSC_FWDAREA:
      linedata.kludge = GKLUD_FWD;
      KludgeFWDAREA(ptr);
      return true;

    case FSC_FWDMSGID:
      linedata.kludge = GKLUD_FWD;
      KludgeFWDMSGID(ptr);
      return true;

    case FSC_SPLIT:
      linedata.kludge = GKLUD_SPLIT;
      kludge.split = ptr;
      return true;

    case FSC_ENC:
      linedata.kludge = GKLUD_ENC;
      kludge.enc = ptr;
      return true;

    case FSC_EID:
    case FSC_CHRC:
    case FSC_PTH:
    case FSC_SPTH:
    case FSC_TID:
      // Recognized but not processed
      return true;

    ////////////////////////////////////////////////////////////

    case XXX_DESTADDR:
      KludgeMSGTO(ptr);
      return true;

    case XXX_GIF:
      linedata.kludge = GKLUD_GIF;
      KludgeGIF(ptr);
      return true;

    case XXX_MOOD:
      linedata.kludge = GKLUD_MOOD;
      KludgeMOOD(ptr);
      return true;

    case XXX_ACUPDATE:
    case XXX_CODEPAGE:
    case XXX_ENCRYPTION:
    case XXX_EOT:
    case XXX_GATECHK:
    case XXX_GID:
    case XXX_GMD:
    case XXX_GROUP:
    case XXX_MSGSEQ:
    case XXX_NOTE:
    case XXX_ORIGID:
    case XXX_ORIGINAL:
    case XXX_ORIGREF:
    case XXX_RECD:
    case XXX_RFC:
    case XXX_RFD:
    case XXX_RID:
    case XXX_SN:
    case XXX_SOT:
    case XXX_TCL1:
    case XXX_TCL2:
    case XXX_TZUTCINFO:
    case XXX_TZUTC:
    case XXX_TZ:
    case XXX_VIA:
    case XXX_XID:
      // Recognized but not processed
      return true;
  }

  return false;
}


//  ------------------------------------------------------------------

bool GMsg::handle_rfcs(int lineno, int kludgenum, const char* ptr) {

  glinedata& linedata = linearray.data(lineno);
  char* tmp;

  switch(kludgenum) {

    case RFC_FROM:
      linedata.kludge = GKLUD_RFC;
      tmp = unwrap_line(lineno, ptr, true);
      KludgeFROM(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case RFC_NEWSGROUPS:
      linedata.kludge = GKLUD_RFC;
      tmp = unwrap_line(lineno, ptr, true);
      KludgeNEWSGROUPS(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case RFC_CC:
      linedata.kludge = GKLUD_RFC;
      tmp = unwrap_line(lineno, ptr, true);
      KludgeCC(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case RFC_BCC:
      linedata.kludge = GKLUD_RFC;
      tmp = unwrap_line(lineno, ptr, true);
      KludgeBCC(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case RFC_TO:
    case RFC_X_TO:
      linedata.kludge = GKLUD_RFC;
      tmp = unwrap_line(lineno, ptr, true);
      KludgeTO(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case RFC_DATE:
      linedata.kludge = GKLUD_RFC;
      KludgeDATE(ptr);
      return true;

    case RFC_SUBJECT:
      linedata.kludge = GKLUD_RFC;
      tmp = unwrap_line(lineno, ptr, true);
      KludgeSUBJECT(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case RFC_REPLY_TO:
      linedata.kludge = GKLUD_RFC;
      tmp = unwrap_line(lineno, ptr, true);
      KludgeREPLY_TO(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case RFC_MESSAGE_ID:
      linedata.kludge = GKLUD_RFC;
      tmp = unwrap_line(lineno, ptr, false);
      KludgeMESSAGE_ID(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case RFC_REFERENCES:
      linedata.kludge = GKLUD_RFC;
      tmp = unwrap_line(lineno, ptr, true);
      KludgeREFERENCES(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case RFC_IN_REPLY_TO:
      linedata.kludge = GKLUD_RFC;
      tmp = unwrap_line(lineno, ptr, true);
      KludgeIN_REPLY_TO(tmp ? tmp : ptr);
      throw_free(tmp);
      return true;

    case RFC_ORGANIZATION:
      linedata.kludge = GKLUD_RFC;
      KludgeORGANIZATION(ptr);
      return true;

    case RFC_SENDER:
      linedata.kludge = GKLUD_RFC;
      KludgeSENDER(ptr);
      return true;

    case RFC_FOLLOWUP_TO:
      linedata.kludge = GKLUD_RFC;
      KludgeFOLLOWUPTO(ptr);
      return true;

    case RFC_MIME_VERSION:
      linedata.kludge = GKLUD_RFC;
      header.mime_version = ptr;
      return true;

    case RFC_CONTENT_TYPE:
      linedata.kludge = GKLUD_RFC;
      header.content_type = ptr;
      return true;

    case RFC_CONTENT_TRANSFER_ENCODING:
      linedata.kludge = GKLUD_RFC;
      header.content_transfer_encoding = ptr;
      return true;

    case RFC_CONTENT_DESCRIPTION:
      linedata.kludge = GKLUD_RFC;
      header.content_description = ptr;
      return true;

    case RFC_CONTENT_DISPOSITION:
      linedata.kludge = GKLUD_RFC;
      header.content_disposition = ptr;
      return true;

    case RFC_CONTENT_ID:
      linedata.kludge = GKLUD_RFC;
      header.content_id = ptr;
      return true;

    case RFC_X_CHARSET:
      linedata.kludge = GKLUD_RFC;
      header.x_charset = ptr;
      return true;

    case RFC_X_CHAR_ESC:
      linedata.kludge = GKLUD_RFC;
      header.x_char_esc = ptr;
      return true;

    case RFC_X_MAILREADER:
      linedata.kludge = GKLUD_RFC;
      KludgePID(ptr);
      return true;

    case RFC_X_NEWSREADER:
      linedata.kludge = GKLUD_RFC;
      KludgePID(ptr);
      return true;

    case RFC_X_FTN_TO:
      linedata.kludge = GKLUD_RFC;
      header.x_ftn_to = ptr;
      return true;

    case RFC_X_FTN_FROM:
      linedata.kludge = GKLUD_RFC;
      header.x_ftn_from = ptr;
      return true;

    case RFC_APPROVED:
      linedata.kludge = GKLUD_RFC;
      header.approved = ptr;
      return true;

    case RFC_COMMENTS:
      linedata.kludge = GKLUD_RFC;
      header.comments = ptr;
      return true;

    case RFC_CONTROL:
      linedata.kludge = GKLUD_RFC;
      header.control = ptr;
      return true;

    case RFC_DISTRIBUTION:
      linedata.kludge = GKLUD_RFC;
      header.distribution = ptr;
      return true;

    case RFC_EXPIRES:
      linedata.kludge = GKLUD_RFC;
      header.expires = ptr;
      return true;

    case RFC_KEYWORDS:
      linedata.kludge = GKLUD_RFC;
      header.keywords = ptr;
      return true;

    case RFC_LINES:
      linedata.kludge = GKLUD_RFC;
      header.lines = ptr;
      return true;

    case RFC_ALSO_CONTROL:
    case RFC_APPARENTLY_TO:
    case RFC_ARTICLE_NAMES:
    case RFC_ARTICLE_UPDATES:
    case RFC_COMMENT:
    case RFC_CONTENT_LENGTH:
    case RFC_DELIVERY_DATE:
    case RFC_ENCRYPTED:
    case RFC_ERRORS_TO:
    case RFC_FROMX:
    case RFC_NEWS_SOFTWARE:
    case RFC_NNTP_POSTING_HOST:
    case RFC_NNTP_POSTING_USER:
    case RFC_OLD_DATE:
    case RFC_ORIGINATOR:
    case RFC_PATH:
    case RFC_PRECEDENCE:
    case RFC_PRIORITY:
    case RFC_RECEIVED:
    case RFC_REPLY_PATH:
    case RFC_RETURN_PATH:
    case RFC_RETURN_RECEIPT_TO:
    case RFC_SEE_ALSO:
    case RFC_STATUS:
    case RFC_SUMMARY:
    case RFC_SUPERSEDES:
    case RFC_VERSION:
    case RFC_XREF:
    case RFC_RNEWS:
      // Recognized but not processed
      return true;
  }

  return false;
}


//  ------------------------------------------------------------------

int ScanCtrlList(const Kludges* k, const gstring& kludge, char endchar) {

  while(*k->key) {
    if((k->req & KCRQ_CASE) ? kludge.eql(k->key) : kludge.ieql(k->key)) {
      if(k->req & KCRQ_COLON) {
        if(endchar == ':')
          return k->num;
      }
      else {
        return k->num;
      }
    }
    k++;
  }

  return 0;
}


//  ------------------------------------------------------------------

int GMsg::scan_line(int lineno, const char* ptr, int mask) {

  // Kludge number
  int kludgenum = 0;

  // Pointer to kludge id
  const char* kludgebeg = ptr;

  // Skip past "RFC" string, if any
  if(strnieql(kludgebeg, "RFC", 3) and ((kludgebeg[3] == '-') or (kludgebeg[3] == '.'))) {
    kludgebeg += 3;
    if(not isalpha(*kludgebeg))
      kludgebeg++;
  }

  // Keep copy of id terminating char
  while((*ptr != ' ') and (*ptr != ':') and *ptr)
    ptr++;
  char endchar = *ptr;
  gstring kludge;
  kludge.copyn(kludgebeg, (uint)(ptr-kludgebeg));

  // Search for it in the known kludges list
  if(kludge.length()) {
    Kludges* k;
    while(1) {
      if(mask & MASK_FTS) {
        k = fts_list;
        kludgenum = ScanCtrlList(k, kludge, endchar);
        if(kludgenum)
          break;
      }
      if(mask & MASK_FSC) {
        k = fsc_list;
        kludgenum = ScanCtrlList(k, kludge, endchar);
        if(kludgenum)
          break;
      }
      if(mask & MASK_RFC) {
        k = rfc_list;
        kludgenum = ScanCtrlList(k, kludge, endchar);
        if(kludgenum)
          break;
      }
      if(mask & MASK_XXX) {
        k = xxx_list;
        kludgenum = ScanCtrlList(k, kludge, endchar);
        if(kludgenum)
          break;
      }
      break;
    }
  }

  // Restore terminating char
  if(*ptr != ' ')
    ptr++;

  glinedata& linedata = linearray.data(lineno);

  if(kludgenum) {
    ptr = strskip_wht(ptr);
    linedata.type |= GLINE_KLUD;
    if(kludgenum & (MASK_FTS|MASK_FSC|MASK_XXX))
      handle_kludges(lineno, kludgenum, ptr);
    else if(kludgenum & MASK_RFC)
      handle_rfcs(lineno, kludgenum, ptr);
  }
  else {
    for(int k=0; k<CFG->kludge.size(); k++) {
      if(kludge.nieql(CFG->kludge[k], strlen(CFG->kludge[k]))) {
        linedata.type |= GLINE_KLUD;
        kludgenum = USERDEFINED;
        break;
      }
    }
    if(not linedata.iskludge()) {
      if(kludge.nieql("X-", 2) and (endchar == ':') and (mask & MASK_RFC)) {
        linedata.type |= GLINE_KLUD;
        kludgenum = RFC_X;
      }
    }
    if(not linedata.iskludge())
      kludgenum = (endchar == ':') ? HEADERLINE : BODYLINE;
  }

  return kludgenum;
}


//  ------------------------------------------------------------------

void GMsg::scan_kludges() {
  
  uint last_tagline = 0;
  uint last_tearline = 0;
  uint last_gateorigin = 0;
  uint last_origin = 0;

  // Scan for kludge-, tear- and originlines

  goldmark = ' ';   // Reset the "recognizer" code

  for(int n=0; n<linearray.size(); n++) {

    const char* ptr = linearray[n];
    glinedata& linedata = linearray.data(n);

    if(*ptr == CTRL_A) {

      int kludgenum = scan_line(n, ptr+1, MASK_ALL);
      if((kludgenum == BODYLINE) or (kludgenum == HEADERLINE))
        linedata.type |= GLINE_HIDD;
    }
    else {

      if(strneql(ptr, "AREA:", 5)) {
        linedata.type |= GLINE_KLUD;
        const char* areakludgeptr = ptr+5;   // Extract echoid from kludge
        areakludgeptr = strskip_wht(areakludgeptr);
        garealistdata* ap = AL.echoid_to_pointer(areakludgeptr);
        if(ap)
          kludge.area = ap->echoid();
      }

      if(not linedata.iskludge()) {

        // Check if it is a Tag- Tear- or Originline

        if(strneql("...", ptr, 3) or strneql("___", ptr, 3))
          last_tagline = n + 1;

        if(strneql("---", ptr, 3) and (ptr[3] == ' ' or ptr[3] == NUL))
          last_tearline = n + 1;

        if(strneql(" # Origin: ", ptr, 11))
          last_gateorigin = n + 1;

        if(strneql(" * Origin: ", ptr, 11))
          last_origin = n + 1;
      }
    }

    if(linedata.iswrapped()) {
      while((n < linearray.size()) and linearray.data(n).iswrapped()) {
        if(n+1 < linearray.size())
          linearray.data(n+1).type |= (linearray.data(n).type & (GLINE_HIDD|GLINE_KLUD));
        n++;
      }
    }
  }

  if(last_origin) {
    while(1) {
      if(last_tearline < last_origin) {
        if((last_tearline and (last_tagline <= last_tearline)) or (last_tagline < last_origin)) {
          linearray.data(last_origin-1).type |= GLINE_ORIG;
          origin = linearray[last_origin-1] + 11;
          if(last_origin < linearray.size()) {  // Get the next line too
            if(*linearray[last_origin] != CTRL_A and strncmp(linearray[last_origin], "SEEN-BY:", 8)) {
              origin += linearray[last_origin];
              linearray.data(last_origin).type |= GLINE_ORIG;     // Mark next line as Origin too
            }
          }
          for(uint n=last_origin; n<linearray.size(); n++) {
            if(strnieql(linearray[n], "SEEN-BY:", 8)) {
              linearray.data(n).kludge = GKLUD_SEENBY;
              linearray.data(n).type |= GLINE_KLUD;
            }
          }
          break;
        }
      }
      last_origin = 0;
      break;
    }
  }

  if(last_gateorigin) {
    if(last_origin == (last_gateorigin + 1))
      gate_origin = linearray[last_gateorigin-1] + 11;
    else
      last_gateorigin = 0;
  }

  if(last_tearline) {
    if(last_tagline < last_tearline) {
      linearray.data(last_tearline-1).type |= GLINE_TEAR;
      tearline = linearray[last_tearline-1]+3;
      tearline.btrim();
    }
    else {
      last_tearline = 0;
    }
  }

  if(last_tagline) {

    if(last_tearline or last_origin) {
      // Any line between the tagline and the tearline or origin must be empty
      uint final = (last_tearline ? last_tearline : last_origin) - 1;
      for(int n=last_tagline; n<final; n++) {
        if(linearray.length(n)) {
          last_tagline = 0;
          break;
        }
      }
    }
    if(last_tagline) {
      linearray.data(last_tagline-1).type |= GLINE_TAGL;
      if(area->taglinesupport()) {
        tagline = linearray[last_tagline-1]+3;
        tagline.btrim();
      }
    }
  }

  if(last_tagline == 0)
    tagline = "";
  if(last_tearline == 0)
    tearline = "";
  if(last_origin == 0) {
    origin = "";
    netname = "";
  }
  if(last_gateorigin == 0) {
    gate_origin = "";
    gate_netname = "";
  }

  if(tearline) {
    const char* tearid[] = {
      "GoldED",
      " GED ",
      " GED2 ",
      " GED3 ",
      " GED386 ",
      " GED/2 ",
      " GED/386 ",
      " GEDP16 ",
      " GEDP32 ",
      " GEDW32 ",
      " GED/W32 ",
      NULL
    };
    int n = 0;
    while(tearid[n]) {
      if(tearline.iinc(tearid[n])) {
        goldmark = '';          // Recognize another GoldED msg
        break;
      }
      n++;
    }
  }

  if(origin) {
    split_origin(origin, origin, netname);
  }

  if(gateorigin) {
    split_origin(gate_origin, gate_origin, gate_netname);
  }
}


//  ------------------------------------------------------------------

void GMsg::text_to_hexdump() {

  w_info(CFG->language[LNG_GenHexdump]);

  throw_release(txt);

  gstring dump;
  area->hexdumpmsg(*this, dump);

  char buf[256];
  const char* nl;
  const char* ptr = dump.c_str();

  do {
    nl = strchr(ptr, '\n');
    if(nl) {
      nl++;
      linearray.add(strxcpy(buf, ptr, nl-ptr));
      ptr = nl;
    }
  } while(nl);

  if(txt) {
    linearray.add("");
    linearray.add(CFG->language[LNG_Hexdumptext]);
    linearray.add("");
    ptr = txt;
    uint size = strlen(txt);
    for(uint idx=0; idx < size; ptr+=16,idx+=16) {
      sprintf(buf, "%04X   ", idx+(area->isfido()?190:0));
      HexDump16(buf+7, ptr, 16, HEX_DUMP2);
      linearray.add(buf);
    }
  }

  reindex_lines();

  w_info(NULL);
}


//  ------------------------------------------------------------------

bool GMsg::examine_kludge(char* ptr, bool& gotmime, bool& qpencoded) {

  char* kludge = ptr + 1;
  if(strnieql(kludge, "RFC", 3)) {
    kludge += 3;
    if(not isalpha(*kludge))
      kludge++;
  }
  while((*ptr != ' ') and (*ptr != ':') and *ptr)
    ptr++;
  char endchar = *ptr;
  *ptr = NUL;
  int kludgetype = -1;
  if(strieql(kludge, "I51"))
    kludgetype = FSC_I51;
  else if(strieql(kludge, "CHRS") or strieql(kludge, "CHARSET"))
    kludgetype = FSC_CHARSET;
  else if(strieql(kludge, "Content-Type"))
    kludgetype = RFC_CONTENT_TYPE;
  else if(strieql(kludge, "Content-Transfer-Encoding"))
    kludgetype = RFC_CONTENT_TRANSFER_ENCODING;
  else if(strieql(kludge, "X-Charset"))
    kludgetype = RFC_X_CHARSET;
  else if(strieql(kludge, "X-Char-Esc"))
    kludgetype = RFC_X_CHAR_ESC;
  *ptr = endchar;
  if(*ptr != ' ')
    ptr++;
  ptr = strskip_wht(ptr);
  char* keptr = strpbrk(ptr, "\r\n");
  if(keptr) {
    endchar = *keptr;
    *keptr = NUL;
  }
  gstring chsbuf;
  if(kludgetype == FSC_I51) {
    // Convert FSC-0051.003 to FSC-0054.003
    chsbuf = "LATIN-1";
    if(XLAT->load_charset(chsbuf, XLAT->Localset))
      charset = chsbuf;
  }
  else if(kludgetype == FSC_CHARSET) {
    int latin1qp = striinc("LATIN1QP", ptr) ? true : false;
    if(latin1qp)
      qpencoded = true;
    chsbuf = latin1qp ? "LATIN-1" : ptr;
    if(XLAT->load_charset(chsbuf, XLAT->localset))
      charset = chsbuf;
  }
  else if(kludgetype == RFC_CONTENT_TYPE) {
    if(striinc("iso-8859-1", ptr)) {
      chsbuf = "LATIN-1";
      if(XLAT->load_charset(chsbuf, XLAT->localset))
        charset = chsbuf;
      gotmime = true;
    }
  }
  else if(kludgetype == RFC_CONTENT_TRANSFER_ENCODING) {
    if(striinc("quoted-printable", ptr)) {
      qpencoded = true;
      if(charset.iinc("LATIN-1") {
        chsbuf = "LATIN1QP";
        if(XLAT->load_charset("LATIN-1", XLAT->localset))
          charset = chsbuf;
      }
    }
  }
  else if(kludgetype == RFC_X_CHARSET) {
    if(not gotmime) {
      if(striinc("8859-1", ptr) or striinc("Latin1", ptr))
        chsbuf = "LATIN-1";
      else
        chsbuf = XLAT->localset;
      if(XLAT->load_charset(chsbuf, XLAT->localset))
        charset = chsbuf;
    }
  }
  if(keptr)
    *keptr = endchar;

  return kludgetype != -1;
}


//  ------------------------------------------------------------------

void GMsg::scan_msg_body_top(bool& gotmime, bool& qpencoded) {
  
  int headerlines = 0;
  gstring chsbuf;
  for(int n=0; n<linearray.size(); n++) {
    const char* ptr = linearray[n];
    if(*ptr) {
      if(*ptr != CTRL_A) {
        int kludgetype = scan_line(n, ptr, MASK_RFC);
        if(kludgetype) {
          ptr = strchr(ptr, ' ');
          if(ptr) {
            ptr = strskip_wht(ptr);
            if(kludgetype == RFC_CONTENT_TYPE) {
              if(striinc("iso-8859-1", ptr)) {
                chsbuf = "LATIN-1";
                if(XLAT->load_charset(chsbuf, XLAT->localset))
                  charset = chsbuf;
                charset_active = XLAT->set.charset.active();
                gotmime = true;
              }
            }
            else if(kludgetype == RFC_CONTENT_TRANSFER_ENCODING) {
              if(striinc("quoted-printable", ptr)) {
                qpencoded = true;
                if(striinc("LATIN-1", charset.c_str())) {
                  chsbuf = "LATIN1QP";
                  if(XLAT->load_charset("LATIN-1", XLAT->localset))
                    charset = chsbuf;
                  charset_active = XLAT->set.charset.active();
                }
              }
            }
            else if(kludgetype == RFC_X_CHARSET) {
              if(not gotmime) {
                if(striinc("8859-1", ptr) or striinc("Latin1", ptr))
                  chsbuf = "LATIN-1";
                else
                  chsbuf = XLAT->localset;
                if(XLAT->load_charset(chsbuf, XLAT->localset))
                  charset = chsbuf;
                charset_active = XLAT->set.charset.active();
              }
            }
            else if(kludgetype == HEADERLINE) {
              linearray.data(n).type |= GLINE_HIDD;
            }
          }
          headerlines++;
          if(n+1 < linearray.size()) {
            char lwsp = *linearray[n+1];
            if((lwsp == ' ') or (lwsp == '\t')) {
              linearray.data(n+1).type |= linearray.data(n).type;
              n++;
            }
          }
        }
        else
          break;
      }
    }
    else {
      if(headerlines)
        linearray.data(n).type |= GLINE_KLUD;
      break;
    }
    if(linearray.data(n).iswrapped()) {
      while((n < linearray.size()) and linearray.data(n).iswrapped())
        n++;
    }
  }
}


//  ------------------------------------------------------------------

void GMsg::text_to_lines(int margin, bool set_position) {

  gmsg_quote quoting(area->quotestring().c_str(), CFG->dispsoftcr);

  line_width = margin;

  // Free all previously allocated lines
  linearray.reset();

  // Make a complete hexdump as a list of message lines
  if(area->attr().hex()) {
    text_to_hexdump();
    return;
  }

  char* ptr = txt;

  // Check if there is any text to start with
  if(CFG->dispsoftcr) {
    while((*ptr == LF) and *ptr)
      ptr++;
  }
  else {
    while((*ptr == LF or *ptr == SOFTCR) and *ptr)
      ptr++;
  }
  if(*ptr == NUL) {
    reindex_lines();
    return;
  }

  bool  firstemptyline = false;
  bool  gotmime = false;
  bool  qpencoded = area->xlatimport().ieql("LATIN1QP") ? true : false;
  bool  quoteflag = false;
  bool  quotewraphard = area->quotewraphard();
  bool  reflow = false;
  char  buf[256];
  char  ch;
  char  chln = 0;
  char  dochar;
  char  qbuf[500];
  char  qbuf2[500];
  char* btmp = NULL;
  char* lp = NULL;
  char* tmp = NULL;
  int   para = 0;
  int   wraps = 0;
  uint  n;
  int   qlen = 0;
  int   qlen2 = 0;
  uint  type = 0;
  bool  charset_active = false;

  *buf = *qbuf = *qbuf2 = NUL;

  if(margin < 0) {
    margin = -margin;
    quoteflag = true;
  }

  char* prev_ptr = "\xFF\xFF";

  // Set default conversion table for area
  if(not area->xlatimport().ieql(XLAT->localset)) {
    charset = area->xlatimport();
    XLAT->load_charset(charset, XLAT->localset);
    charset_active = XLAT->set.charset.active();
  }

  // Alloc space for one line
  char* linetmp = (char*)throw_calloc(1, margin+512);
  char* line_text = linetmp + 1;
  while(*ptr) {

    char* bptr = linetmp;
    char* bp = bptr;
    uint len = 0;

    // Reflow quotes
    if(reflow) {
      len = qlen;
      char* qptr = qbuf;
      reflow = false;
      for(n=0; n<qlen; n++)             // Insert previous quotestring
        *(++bp) = *qptr++;
      if(CFG->dispsoftcr) {
        while((*ptr == LF) and *ptr)
          ptr++;
      }
      else {
        while((*ptr == LF or *ptr == SOFTCR) and *ptr)
          ptr++;
      }
      type |= GLINE_QUOT | GLINE_HARD;
    }

    // Get type of line
    if(wraps == 0) {
      para = 0;
      if(*ptr == CTRL_A) {
        type |= GLINE_HARD;
        if(examine_kludge(ptr, gotmime, qpencoded))
          charset_active = XLAT->set.charset.active();
      }
      else if(quoting.is_quote(ptr)) {
        para = GLINE_QUOT;
        type |= GLINE_QUOT | GLINE_HARD;
        quoting.get_quotestring(ptr, qbuf, qlen);
      }
    }

    // Get one line
    ch = 0;
    tmp = NULL;
    while(*ptr and (len < (uint)margin)) {
      switch(*ptr) {
        case CR:
          do_cr:
          ptr++;
          ch = CR;
          if(CFG->dispsoftcr) {
            while((*ptr == LF) and *ptr)
              ptr++;
          }
          else {
            while((*ptr == LF or *ptr == SOFTCR) and *ptr)
              ptr++;
          }
          if(wraps) {
            if(para != GLINE_QUOT) {
              if(quoteflag) {
                if(*ptr == CR or quoting.is_quote(ptr) or *ptr == CTRL_A) {
                  wraps = 0;
                  break;
                }
                if(((ptr[0] == ptr[1]) and (ptr[0] == ptr[2])) or strneql(ptr, " * Origin: ", 11)) {
                  wraps = 0;
                  break;
                }
                if(strneql(ptr, "SEEN-BY:", 8)) {
                  wraps = 0;
                  break;
                }
                if((ptr[0] == prev_ptr[0]) and (ptr[1] == prev_ptr[1])) {
                  wraps = 0;
                  break;
                }
                lp = ptr;
                while(*lp == ' ')
                  lp++;
                if(*lp == CR) {
                  wraps = 0;
                  break;
                }
                else {
                  ptr = lp;
                }
                if(*ptr != ' ' and *ptr != HT and *bp != ' ') {
                  *(++bp) = ' ';
                  len++;
                }
                ch = 0;
                continue;
              }
              else {
                wraps=0;
                break;
              }
            }
            else {
              quoting.get_quotestring(ptr, qbuf2, qlen2);
              if(*ptr == CR) {
                wraps = 0;
                para = 0;
                break;
              }
              else if(quoting.equal(qbuf2, qbuf)) {
                lp = ptr + qlen2;
                if(quoting.is_quote(lp)) {
                  wraps = 0;
                  para = 0;
                  break;
                }
                else {
                  ptr = lp;
                  para = GLINE_QUOT;
                  if(*ptr != ' ' and *bp != ' ') {
                    *(++bp) = ' ';
                    len++;
                  }
                  ch = 0;
                  continue;
                }
              }
              else {
                wraps = 0;
                para = 0;
                break;
              }
            }
          }
          break;
        case 0x02:  // The I51 escape character
          if(XLAT->set.i51.active()) {
            for(n=0; n<XLAT->set.i51.size(); n++) {
              const char* ptr = XLAT->set.i51.chars(n);
              if(*(ptr+1) == ptr[0]) {
                if(*(ptr+2) == ptr[1]) {
                  const char* escp = ptr+2;
                  if(*escp) {
                    *(++bp) = *escp++;
                    if(*escp) {
                      *(++bp) = *escp++;
                      len++;
                      if(*escp) {
                        *(++bp) = *escp;
                        len++;
                      }
                    }
                  }
                  ptr += 2;
                  n = (uint)-1;
                  break;
                }
              }
            }
            if(n != (uint)-1)   // I51 char not found, use fallback method
              ptr++;
          }
          ptr++;
          break;
        case 29:  // The MNE escape character
          if(XLAT->set.mnemonic.active()) {
            for(n=0; n<XLAT->set.mnemonic.size(); n++) {
              const char* ptr = XLAT->set.mnemonic.chars(n);
              if(*(ptr+1) == ptr[0]) {
                if(*(ptr+2) == ptr[1]) {
                  const char* escp = ptr+2;
                  if(*escp) {
                    *(++bp) = *escp++;
                    if(*escp) {
                      *(++bp) = *escp++;
                      len++;
                      if(*escp) {
                        *(++bp) = *escp;
                        len++;
                      }
                    }
                  }
                  ptr += 2;
                  n = (uint)-1;
                  break;
                }
              }
            }
            if(n != (uint)-1)   // MNE char not found, use fallback method
              ptr++;
          }
          ptr++;
          break;
        case SOFTCR:
          if(CFG->dispsoftcr) {
            *(++bp) = *ptr++;
            ++len;
          }
          else {
            if(XLAT->set.composed.active()) {
              if(not (isspace(*(ptr-1)) or isspace(*(ptr+1)))) {
                for(n=0; n<XLAT->set.composed.size(); n++) {
                  const char* ptr = XLAT->set.composed.chars(n);
                  if(*(ptr-1) == ptr[0]) {
                    if(*(ptr+1) == ptr[1]) {
                      const char* escp = ptr+2;
                      if(*escp) {
                        *(bp) = *escp++;
                        if(*escp) {
                          *(++bp) = *escp++;
                          len++;
                          if(*escp) {
                            *(++bp) = *escp;
                            len++;
                          }
                        }
                      }
                      ptr++;
                      break;
                    }
                  }
                }
              }
              else if(para == GLINE_QUOT) {
                *ptr-- = CR;   // Fake a CR
              }
            }
            else if(para == GLINE_QUOT) {
              *ptr-- = CR;   // Fake a CR
            }
            ptr++;
          }
          break;
        case LF:
          ptr++;
          break;
        case CTRL_A:
          *(++bp) = *ptr++;
          ++len;
          break;
        case HT:
          do_ht:
          ptr++;
          if(CFG->disptabsize) {
            for(n=0; n<(CFG->disptabsize-(len%CFG->disptabsize)); n++) {
              *(++bp) = ' ';
              len++;
            }
          }
          else {
            *(++bp) = ' ';
            len++;
          }
          break;
        case '=':
          if(qpencoded) {
            if(isxdigit(ptr[1]) and isxdigit(ptr[2])) {
              // Decode the character
              dochar = (char)((ctox(ptr[1]) << 4) | ctox(ptr[2]));
              ptr += 3;
              //if((dochar == 0x20) and (*ptr == CR))   // Undocumented soft line break?
              //  ptr++;
              if(dochar == HT)
                goto do_ht;
              else if(dochar == CR)
                goto do_cr;
              goto chardo;
            }
            else if((ptr[1] == CR) or (ptr[1] == LF)) {
              // Skip soft line break
              ptr++;
              while((*ptr == CR) or (*ptr == LF))
                ptr++;
              break;
            }
          }
          goto defaultchardo;
        case ' ':
          if(len >= qlen) {
            tmp = ptr;
            btmp = bp;
          }
        default:
        defaultchardo:
          dochar = *ptr++;
        chardo:
          if(charset_active) {
            const char* ptr = XLAT->set.charset.chars(dochar);
            chln = *ptr++;
            while(chln--) {
              *(++bp) = *ptr++;
              ++len;
            }
          }
          else {
            *(++bp) = dochar;
            ++len;
          }
          break;
      }
      if(ch == CR)
        break;
    }

    *(bp+1) = NUL;

    if(len == (uint)margin) {
      if(*ptr == CR) {
        type |= GLINE_HARD;
        wraps = 0;
        ptr++;
      }
      else {
        wraps++;
        if(para == GLINE_QUOT)
          reflow = quotewraphard;
        type |= GLINE_WRAP;
        if(CFG->dispsoftcr) {
          while((*ptr == LF or *ptr == SOFTCR) and *ptr)
            ptr++;
          if(*bp == ' ' or *ptr == ' ' or *ptr == SOFTCR) {
            while((*ptr == ' ' or *ptr == SOFTCR) and *ptr)
              ptr++;
          }
          else {
            if(tmp) {
              bp = btmp;
              *(bp+1) = NUL;
              ptr = tmp+1;
            }
          }
        }
        else {
          while((*ptr == LF) and *ptr)
            ptr++;
          if(*bp == ' ' or *ptr == ' ') {
            while((*ptr == ' ') and *ptr)
              ptr++;
          }
          else {
            if(tmp) {
              bp = btmp;
              *(bp+1) = NUL;
              ptr = tmp+1;
            }
          }
        }
      }
    }
    else {
      type |= GLINE_HARD;
    }

    // Set line type
    if((type & GLINE_QUOT) and not quoting.is_quote(line_text))
      type ^= GLINE_QUOT;

    // Store line
    prev_ptr[0] = line_text[0];
    prev_ptr[1] = line_text[1];
    linearray.add(line_text, type);

    // Scan msg body top for RFC headerlines
    if((*line_text == NUL) and not firstemptyline) {
      firstemptyline = true;
      if(area->internetrfcbody())
        scan_msg_body_top(gotmime, qpencoded);
    }

    type = 0;

    if(CFG->dispsoftcr) {
      while((*ptr == LF) and *ptr)
        ptr++;
    }
    else {
      while((*ptr == LF or *ptr == SOFTCR) and *ptr)
        ptr++;
    }
  }

  throw_release(linetmp);

  // Scan for kludge-, tear- and originlines
  scan_kludges();

  if(area->isecho() and not oaddress) {
    const char* ptr = strrchr(origin.c_str(), '(' /*)*/ )
    if(ptr) {
      while(not isdigit(*ptr) and *ptr)
        ptr++;
      oaddress = ptr;
    }
  }

  if(not oaddress and msgid) {
    gftnaddrs msgid_address = msgid;
    if(msgid_address.valid())
      oaddress = msgid_address.str();
  }

  if(destaddr.zone == 0)
    destaddr.zone = origaddr.zone;

  if(not oname)
    oname = origname;
  if(not dname)
    dname = destname;
  if(not oaddress)
    oaddress = origaddr.str();
  if(not daddress)
    daddress = destaddr.str();

  // Charset translate header fields
  XLAT->set.translate(oname, qpencoded);
  XLAT->set.translate(dname, qpencoded);
  if(not (attr.frq() or attr.att() or attr.urq()))
    XLAT->set.translate(subject, qpencoded);

  // Make the index to the line index as allowed by config
  reindex_lines(set_position);
}


//  ------------------------------------------------------------------

void GMsg::reindex_lines(bool set_position, int viewhidden, int viewkludge, int viewquote) {

  gmsg_quote quoting(area->quotestring().c_str(), CFG->dispsoftcr);

  if(viewhidden == -1)
    viewhidden = area->viewhidden();
  if(viewkludge == -1)
    viewkludge = area->viewkludge();
  if(viewquote == -1)
    viewquote = area->viewquote();

  display.reset();

  for(int n=0; n<linearray.size(); n++) {
    if(n+1 < linearray.size()) {
      if(linearray.data(n).isquote()) {
        if(not linearray.data(n+1).isquote() and not strblank(linearray[n+1])) {
          if(CFG->quotespacing)
            linearray.insert(n+1, "", GLINE_HARD);
        }
      }
      else {
        if(striinc("@position", linearray[n])) {
          char* p = throw_xstrdup(linearray[n]);
          strischg(p, "@position", "");
          linearray.set(n, p);
          if(set_position)
            linearray.data(n).type |= GLINE_POSI;
          throw_xfree(p);
        }
        if(linearray.data(n+1).isquote() and not strblank(linearray[n])) {
          if(CFG->quotespacing)
            linearray.insert(n+1, "", GLINE_HARD);
        }
      }
    }
  }

  int  lineno = 0;
  char qbuf[50];
  char qbuf0[50];
  int  qlen = 0;
  int  qmatches = 0;
  int  textsize = 0;
  int  quotesize = 0;

  *qbuf0 = NUL;
  for(int x=0; x<linearray.size(); x++) {
    glinedata& linedata = linearray.data(x);
    if(linedata.iskludge()) {
      *qbuf0 = NUL;
      qmatches = 0;
      if(not viewkludge) {
        lineno++;
        continue;
      }
    }
    else if(linedata.ishidden()) {
      *qbuf0 = NUL;
      qmatches = 0;
      if(not viewhidden) {
        lineno++;
        continue;
      }
    }
    else if(linedata.isquote()) {
      if(not viewquote) {
        quoting.get_quotestr(linearray[x], qbuf, qlen);
        strtrim(qbuf);
        if(strieql(qbuf0, qbuf)) {
          if(strpbrk(linearray[x]+qlen, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"))
            qmatches++;
        }
        else {
          strcpy(qbuf0, qbuf);
          if(strpbrk(linearray[x]+qlen, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"))
            qmatches = 1;
          else
            qmatches = 0;
        }
        if(qmatches != 1) {
          lineno++;
          continue;
        }
      }
    }
    else {
      *qbuf0 = NUL;
      qmatches = 0;
    }
    int length = linearray.length(x);
    textsize += length;
    if(linedata.isquote())
      quotesize += length;
    display.add(lineno);
    lineno++;
  }

  quotepct = 100 - Pct(textsize, quotesize);
}


//  ------------------------------------------------------------------

int golded_xlat::load_charset(const char* import, const char* export) {

  return gedxlat::load_charset(set.charset, import, export, NOT CFG->dispsoftcr);
}


//  ------------------------------------------------------------------

