
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gemlst.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Message lister.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

static gactivearea* mlstarea;
static GMsg* MLstMsgPtr;
static GMsg* mlstmsgptr = NULL;
static uint msgmark2;
static MLst* mlst;
static int  mlst_bysiz;
static int  mlst_tosiz;
static int  mlst_resiz;
static int  fldadd1;
static int  fldadd2;


//  ------------------------------------------------------------------

const byte MLST_HIGH_FROM   =  1;
const byte MLST_HIGH_TO     =  2;
const byte MLST_HIGH_BOOK   =  4;
const byte MLST_HIGH_MARK   =  8;
const byte MLST_HIGH_UNREAD = 16;
const byte MLST_HIGH_UNSENT = 32;


//  ------------------------------------------------------------------

inline void mlst_with_date(int with_date) {

  if(with_date) {
    mlst_bysiz = 19;
    mlst_tosiz = 19;
    mlst_resiz = 20;
  }
  else {
    mlst_bysiz = 19+3;
    mlst_tosiz = 19+3;
    mlst_resiz = 20+4;
  }
}


//  ------------------------------------------------------------------

static void mlst_dodelayed(PInf* p) {
  
  // Update header and statusline

  if(mlstarea->msglistheader()) {
    mlstarea->loadmsg(*MLstMsgPtr, mlst[p->pos].msgno, CFG->dispmargin-CFG->disppagebar);
    mlst[p->pos].goldmark = goldmark;
    int mlstwh = whandle();
    HeaderView.Use(mlstarea, MLstMsgPtr);
    HeaderView.Paint();
    wactiv_(mlstwh);
  }

  if(CFG->msglistviewsubj)
    wtitle(mlst[p->pos].re, TCENTER|TBOTTOM, p->tattr);

  if(CFG->msglistpagebar)
    wscrollbar(W_VERT, p->maxidx+1, p->maxidx, p->idx);

  update_statuslinef(CFG->language[LNG_MsgLister], p->idx+1, p->maxidx+1, p->maxidx-p->idx);
}


//  ------------------------------------------------------------------

static void mlst_dispbuf(char* abuf, MLst* ml) {

  register int bysiz = mlst_bysiz + fldadd1;
  register int tosiz = mlst_tosiz + fldadd1;
  register int resiz = mlst_resiz + fldadd2;

  if(mlstarea->msglistwidesubj()) {
    resiz += tosiz + 1;
    tosiz = 0;
  }

  char nbuf[33], dbuf[20];
  strcpy(dbuf, CFG->language[LNG_n_a]);
  Time dt = 0;
  switch(mlstarea->msglistdate()) {
    case MSGLISTDATE_WRITTEN:   dt = ml->written;   break;
    case MSGLISTDATE_ARRIVED:   dt = ml->arrived;   break;
    case MSGLISTDATE_RECEIVED:  dt = ml->received;  break;
  }
  if(dt) {
    Tm* tmp = glocaltime(&dt);
    sprintf(dbuf, "%02d %s %02d", tmp->tm_mday, gsmonth[tmp->tm_mon], (tmp->tm_year >= 100) ? (tmp->tm_year-100) : tmp->tm_year);
  }
  if(mlstarea->msglistdate())
    strsetsz(dbuf, 10);
  else
    *dbuf = NUL;
  sprintf(nbuf, "%5lu", CFG->disprealmsgno ? ml->msgno : mlstarea->msgnset.absolute_to_relative(ml->msgno));
  sprintf(abuf, "%-5.5s%s%-*.*s %-*.*s%s%-*.*s %s",
    nbuf, ml->marks,
    bysiz, bysiz, ml->by,
    tosiz, tosiz, ml->to,
    (tosiz ? " " : ""),
    resiz, resiz, ml->re,
    dbuf
  );
}


//  ------------------------------------------------------------------

static void mlst_dispit(PInf*p, uint pos, int type) {

  register int bycol = 7;
  register int tocol = bycol + mlst_bysiz + 1 + fldadd1;
  register int bysiz = mlst_bysiz + fldadd1;
  register int tosiz = mlst_tosiz + fldadd1;

  register MLst* ml = &mlst[pos];

  int wattr, hattr;
  if(type == PICK_BAR) {
    wattr = p->sattr;
    hattr = p->sattr;
  }
  else if(ml->high & MLST_HIGH_UNSENT) {
    wattr = C_MENUW_UNSENT;
    hattr = C_MENUQ_UNSENTHIGH;
  }
  else if(ml->high & MLST_HIGH_UNREAD) {
    wattr = C_MENUW_UNREAD;
    hattr = C_MENUQ_UNREADHIGH;
  }
  else {
    wattr = p->wattr;
    hattr = p->hattr;
  }

  char buf[256];
  mlst_dispbuf(buf, ml);
  wprints(pos, 0, wattr, buf);

  if(ml->high & (MLST_HIGH_BOOK|MLST_HIGH_MARK))
    wprints(pos, 5, hattr, ml->marks);
  if(ml->high & MLST_HIGH_FROM)
    wprintns(pos, bycol, hattr, ml->by, bysiz);
  if((ml->high & MLST_HIGH_TO) and not mlstarea->msglistwidesubj())
    wprintns(pos, tocol, hattr, ml->to, tosiz);

  goldmark = ml->goldmark;
}


//  ------------------------------------------------------------------

static void mlst_disp(PInf* p) {

  mlst_dispit(p, p->pos, p->type);
}


//  ------------------------------------------------------------------

static void mlst_get_mlst(MLst* ml, PInf* p, int n) {

  GMsg& msg = *mlstmsgptr;

  ml->high = 0;

  strcpy(ml->marks, "  ");

  ml->msgno = mlstarea->msgnset.relative_to_absolute(p->idx + n + 1);

  if(mlstarea->bookmark() == mlstarea->msgnset.relative_to_absolute(p->idx + n + 1)) {
    ml->marks[0] = MMRK_BOOK;
    ml->high |= MLST_HIGH_BOOK;
  }

  if(mlstarea->markset.size()) {
    if(mlstarea->markset.find(ml->msgno)) {
      ml->marks[1] = MMRK_MARK;
      ml->high |= MLST_HIGH_MARK;
    }
  }

  if(mlstarea->msglistfast()) {
    mlstarea->loadhdr(msg, ml->msgno);
  }
  else {
    mlstarea->loadmsg(msg, ml->msgno, CFG->dispmargin-CFG->disppagebar);
  }
  ml->goldmark = goldmark;

  for(int x=0; x<CFG->usernames; x++) {
    if(msg.by_name().ieql(CFG->username[x].name)) {
      ml->high |= MLST_HIGH_FROM;
      msg.attr.fmu1();
    }
    if(msg.to.ieql(CFG->username[x].name)) {
      ml->high |= MLST_HIGH_TO;
      msg.attr.tou1();
    }
  }

  // Highlight FROM if local
  if(CFG->displocalhigh and msg.attr.loc())
    ml->high |= MLST_HIGH_FROM;

  // Highlight if unread
  if(msg.timesread == 0 and gmb_wide_timesread)
    ml->high |= MLST_HIGH_UNREAD;

  // Highlight if unsent
  if(msg.attr.uns())
    ml->high |= MLST_HIGH_UNSENT;

  ml->written = msg.date.written;
  ml->arrived = msg.date.arrived;
  ml->received = msg.date.received;
  strcpy(ml->by, msg.by_name().c_str());
  strcpy(ml->to, msg.to_name().c_str());
  strcpy(ml->re, msg.re.c_str());
}


//  ------------------------------------------------------------------

static void mlst_page(PInf* p) {

  char linebuf[200];
  register uint m, n;

  if(not mlstarea->msglistfast())
    w_info(CFG->language[LNG_Wait]);

  p->idx -= p->pos;
  m = p->maxidx - p->idx;

  for(n=0; n<=p->maxpos and n<=m; n++)
    mlst_get_mlst(&mlst[n], p, n);

  if(not mlstarea->msglistfast())
    w_info(NULL);

  for(n=0; n<=p->maxpos and n<=m; n++)
    mlst_dispit(p, n, (n==p->pos) ? PICK_BAR : PICK_DISP);

  memset(linebuf, _box_table[p->btype][1], gvid.numcols-2);
  linebuf[gvid.numcols-2] = NUL;

  if(n <= p->maxpos or (p->maxpos == p->maxidx and p->maxpos < (p->ylen-3)))
    wprints(n, 0, p->wattr, linebuf);

  memset(linebuf, ' ', gvid.numcols-2);
  linebuf[gvid.numcols-2] = NUL;

  for(++n; n<p->ylen-2; n++)
    wprints(n, 0, p->wattr, linebuf);
  p->idx += p->pos;
}


//  ------------------------------------------------------------------

static void mlst_center(PInf* p) {

  uint botroom = p->maxidx - p->idx;

  switch(CFG->displistcursor) {
    case LIST_TOP:
      if(botroom > p->maxpos)
        p->pos = 0;
      else
        p->pos = p->maxpos - botroom;
      break;
    case LIST_NEARTOP:
      {
        uint room;
        uint toproom = p->idx;
        if(toproom > (p->maxpos/4)) {
          if(botroom > (p->maxpos/4))
            room = p->maxpos/4;
          else if(botroom)
            room = p->maxpos - botroom;
          else
            room = p->maxpos;
        }
        else
          room = toproom;
        p->pos = room;
      }
      break;
    case LIST_MIDDLE:
      {
        uint room;
        uint toproom = p->idx;
        if(toproom > (p->maxpos/2)) {
          if(botroom > (p->maxpos/2))
            room = p->maxpos/2;
          else if(botroom)
            room = p->maxpos - botroom;
          else
            room = p->maxpos;
        }
        else
          room = toproom;
        p->pos = room;
      }
      break;
    case LIST_NEARBOTTOM:
      {
        uint room;
        uint toproom = p->idx;
        if(toproom > 3*(p->maxpos/4)) {
          if(botroom > 3*(p->maxpos/4))
            room = 3*(p->maxpos/4);
          else if(botroom)
            room = p->maxpos - botroom;
          else
            room = p->maxpos;
        }
        else
          room = toproom;
        p->pos = room;
      }
      break;
    case LIST_BOTTOM:
      p->pos = p->maxpos;
      break;
  }
  mlst_page(p);
}


//  ------------------------------------------------------------------

static void mlst_scroll(PInf* p, int direction) {

  if(direction == SUP) {
    memmove(&mlst[0], &mlst[1], p->maxpos*sizeof(MLst));
    mlst_get_mlst(&mlst[p->maxpos], p, 0);
  }
  else {
    memmove(&mlst[1], &mlst[0], p->maxpos*sizeof(MLst));
    mlst_get_mlst(&mlst[0], p, 0);
  }
  wscroll(1, direction);
}


//  ------------------------------------------------------------------

static void mlst_title(PInf* p) {

  register int bycol = 8;
  register int tocol = bycol + mlst_bysiz + 1 + fldadd1;
  register int recol = tocol + mlst_tosiz + 1 + fldadd1;
  register int dtcol = recol + mlst_resiz + 1 + fldadd2;
  if(mlstarea->msglistwidesubj())
    recol = tocol;

  wtitle(NULL, TCENTER, p->tattr);
  wmessage(CFG->disprealmsgno ? CFG->language[LNG_MsgReal] : CFG->language[LNG_Msg], TP_BORD, 3, p->tattr);
  wmessage(CFG->language[LNG_FromL], TP_BORD, bycol, p->tattr);
  if(not mlstarea->msglistwidesubj())
    wmessage(CFG->language[LNG_ToL], TP_BORD, tocol, p->tattr);
  wmessage(CFG->language[LNG_SubjL], TP_BORD, recol, p->tattr);
  switch(mlstarea->msglistdate()) {
    case MSGLISTDATE_WRITTEN:   wmessage(CFG->language[LNG_Written], TP_BORD, dtcol, p->tattr);   break;
    case MSGLISTDATE_ARRIVED:   wmessage(CFG->language[LNG_Arrived], TP_BORD, dtcol, p->tattr);   break;
    case MSGLISTDATE_RECEIVED:  wmessage(CFG->language[LNG_Received], TP_BORD, dtcol, p->tattr);  break;
  }
}


//  ------------------------------------------------------------------

static void mlst_open(PInf* p) {

  wopen_(p->ypos, p->xpos, p->ylen, p->xlen, p->btype, p->battr, p->wattr, p->sbattr);
  mlst_title(p);
  mlst_center(p);
}


//  ------------------------------------------------------------------

static void mlst_close(PInf* p) {

  NW(p);
  wclose();
}


//  ------------------------------------------------------------------

static int mlst_dokey(PInf* p, Key* keycode) {

  int tpos;
  Key key, kk;
  uint temp, tmpmsgno;

  key = *keycode;
  *keycode = 0;

  if(key < KK_Commands) {
    key = key_tolower(key);
    kk = SearchKey(key, ListKey, ListKeys);
    if(kk)
      key = kk;
  }

  // See if it's a listkey
  switch(key) {
    case KK_ListGotoPrev:
    case KK_ListGotoNext:
    case KK_ListGotoFirst:
    case KK_ListGotoLast:
    case KK_ListAskExit:
    case KK_ListQuitNow:
    case KK_ListAbort:
    case KK_ListSelect:
    case KK_ListToggleMark:
    case KK_ListToggleBookMark:
    case KK_ListGotoBookMark:
    case KK_ListMarkingOptions:
    case KK_ListDosShell:
    case Key_0:
    case Key_1:
    case Key_2:
    case Key_3:
    case Key_4:
    case Key_5:
    case Key_6:
    case Key_7:
    case Key_8:
    case Key_9:
      break;

    // If not a listkey, see if it matches a readkey
    default:
      kk = SearchKey(key, ReadKey, ReadKeys);
      if(kk)
        key = kk;
  }

  switch(key) {
    case KK_ListGotoPrev:
      *keycode = Key_Up;
      break;

    case KK_ListGotoNext:
      *keycode = Key_Dwn;
      break;

    case KK_ListGotoFirst:
      *keycode = Key_Home;
      break;

    case KK_ListGotoLast:
      *keycode = Key_End;
      break;

    case KK_ListAskExit:
      {
        GMenuQuit MenuQuit;
        p->aborted = gkbd.quitall = MenuQuit.Run();
        if(gkbd.quitall) {
          mlstarea->set_bookmark(mlstarea->msgnset.relative_to_absolute(msgmark2));
          return NO;
        }
      }
      break;

    case KK_ListQuitNow:
      p->aborted = gkbd.quitall = YES;
      mlstarea->set_bookmark(mlstarea->msgnset.relative_to_absolute(msgmark2));
      return NO;

    case KK_ListAbort:
      mlstarea->set_bookmark(mlstarea->msgnset.relative_to_absolute(msgmark2));
      p->aborted = YES;
      ///////////////// Drop Through

    case KK_ListSelect:
      return NO;

    case KK_ListToggleMark:
      temp = mlstarea->markset.find(mlst[p->pos].msgno);
      if(temp) {
        mlstarea->markset.delete_relative(temp);
        mlst[p->pos].marks[1] = ' ';
        mlst[p->pos].high &= ~MLST_HIGH_MARK;
      }
      else {
        mlstarea->markset.add(mlst[p->pos].msgno);
        mlst[p->pos].marks[1] = MMRK_MARK;
        mlst[p->pos].high |= MLST_HIGH_MARK;
      }
      if(p->idx < p->maxidx) {
        mlst_disp(p);
        *keycode = Key_Dwn;
      }
      break;

    case KK_ListToggleBookMark:
      if(mlstarea->bookmark() == mlst[p->pos].msgno) {
        mlst[p->pos].marks[0] = ' ';
        mlstarea->set_bookmark(0);
        mlst[p->pos].high &= ~MLST_HIGH_BOOK;
        mlst_disp(p);
      }
      else {
        temp = mlstarea->msgnset.absolute_to_relative(mlstarea->bookmark()-1);
        mlstarea->set_bookmark(mlst[p->pos].msgno);
        mlst[p->pos].marks[0] = MMRK_BOOK;
        mlst[p->pos].high |= MLST_HIGH_BOOK;
        mlst_disp(p);
        if(temp) {
          tpos = p->pos;
          p->pos += (temp - p->idx);
          if(p->pos <= p->maxpos) {
            mlst[p->pos].marks[0] = ' ';
            mlst[p->pos].high &= ~MLST_HIGH_BOOK;
            mlst_disp(p);
          }
          p->pos = tpos;
        }
      }
      break;

    case KK_ListGotoBookMark:
      if(mlstarea->bookmark()) {
        tpos = p->pos + ((mlstarea->msgnset.absolute_to_relative(mlstarea->bookmark()-1)) - p->idx);
        temp = p->idx + 1;
        p->idx = mlstarea->msgnset.absolute_to_relative(mlstarea->bookmark()-1);
        mlstarea->set_bookmark(mlstarea->msgnset.relative_to_absolute(temp));
        if(tpos >= 0 and (uint)tpos <= p->maxpos) {
          mlst[tpos].marks[0] = ' ';
          mlst[tpos].high &= ~MLST_HIGH_BOOK;
          mlst[p->pos].marks[0] = MMRK_BOOK;
          mlst[p->pos].high |= MLST_HIGH_BOOK;
          mlst_disp(p);
          p->pos = tpos;
          mlst_disp(p);
        }
        else
          mlst_center(p);
      }
      else
        SayBibi();
      break;

    case KK_ListMarkingOptions:
      {
        uint lrbak = mlstarea->lastread();
        mlstarea->set_lastread(p->idx + 1);
        MLstMsgPtr->msgno = mlstarea->msgnset.relative_to_absolute(mlstarea->lastread());
        MarkingOptions(*MLstMsgPtr, mlstarea);
        mlstarea->set_lastread(lrbak);
        mlst_page(p);
      }
      break;

    case KK_ListDosShell:
      ShellToDos(getenv("COMSPEC"), CFG->language[LNG_DOS_Shell], LGREY|_BLACK, YES, true, false, mlstarea);
      break;

    case KK_ListToggleWideSubj:
      mlstarea->toggle_msglistwidesubj();
      mlst_title(p);
      mlst_page(p);
      break;

    case KK_ListToggleDate:
      mlstarea->next_msglistdate();
      mlst_with_date(mlstarea->msglistdate());
      mlst_title(p);
      mlst_page(p);
      break;

    case Key_0:
    case Key_1:
    case Key_2:
    case Key_3:
    case Key_4:
    case Key_5:
    case Key_6:
    case Key_7:
    case Key_8:
    case Key_9:
    case KK_ReadGotoMsgno:
      tmpmsgno = p->idx;
      reader_keycode = key;
      GotoMsgno(*MLstMsgPtr, mlstarea);
      if(mlstarea->lastread()-1 != tmpmsgno) {
        p->idx = mlstarea->lastread()-1;
        mlst_center(p);
      }
      break;

    case KK_ReadMessageList:
      mlst_center(p);
      break;

    case Key_Tick:
      CheckTick(KK_ListQuitNow);
      break;

    case KK_ListUndefine:
      break;

    default:
      if(not PlayMacro(key, KT_M)) {
        if(gkbd.kbuf == NULL)
          kbput(key);
        return NO;
      }
  }
  return YES;
}


//  ------------------------------------------------------------------

uint MsgBrowser(GMsg& msg, gactivearea* area) {

  GFTRK("MsgBrowser");

  Pick pick;
  PInf p;

  memset(&p, 0, sizeof(PInf));
  memset(&pick, 0, sizeof(Pick));
  p.btype = W_BMENU;
  p.battr = C_MENUB;
  p.wattr = C_MENUW;
  p.sattr = C_MENUS;
  p.tattr = C_MENUT;
  p.hattr = C_MENUQ;
  p.sbattr = C_MENUPB;
  p.ypos = area->msglistheader() ? 6 : 1;
  p.ylen = gvid.numrows-p.ypos-1;
  p.xlen = gvid.numcols;
  p.helpcat = H_MessageBrowser;
  p.delay = 150;  // milliseconds
  p.listwrap = CFG->displistwrap;

  pick.open      = mlst_open;
  pick.close     = mlst_close;
  pick.disp      = mlst_disp;
  pick.page      = mlst_page;
  pick.scroll    = mlst_scroll;
  pick.dokey     = mlst_dokey;
  pick.dodelayed = mlst_dodelayed;
  
  p.maxidx = area->msgnset.size()-1;
  p.maxpos = minimum_of_two((uint)(p.ylen-3), p.maxidx);
  p.aborted = NO;

  p.idx = area->msgnset.absolute_to_relative(msg.msgno)-1;
  msgmark2 = area->msgnset.absolute_to_relative(area->bookmark());

  MLstMsgPtr = &msg;
  GMsg mlstmsg;
  mlstmsgptr = &mlstmsg;

  mlstarea = area;
  mlst = (MLst*)throw_calloc(p.maxpos+2, sizeof(MLst));

  if(area->msglistdate() != MSGLISTDATE_NONE) {
    if(area->msglistdate() != MSGLISTDATE_WRITTEN) {
      if(area->ishudson() or area->isgoldbase() or area->ispcboard())
        area->set_msglistdate(MSGLISTDATE_WRITTEN);
      else if((area->isezycom() or area->isfido()) and (area->msglistdate() == MSGLISTDATE_RECEIVED))
        area->set_msglistdate(MSGLISTDATE_WRITTEN);
    }
  }

  mlst_with_date(area->msglistdate());

  fldadd1 = (gvid.numcols-80)/3;
  fldadd2 = (gvid.numcols-80) - (fldadd1*2);

  _in_msglist = true;

  Picker(&p, &pick);

  _in_msglist = false;

  throw_release(mlst);

  GFTRK(NULL);

  if(not p.aborted)
    return p.idx+1;

  return 0;
}


//  ------------------------------------------------------------------

void MessageBrowse(GMsg& msg, gactivearea* area) {

  if(area->msgnset.size()) {
    uint temp = area->lastread();
    area->set_lastread(MsgBrowser(msg, area));
    if(area->lastread() == 0)
      area->set_lastread(temp);
    if(area->pmrkset.empty())
      area->set_readpersonal(false);
    if(area->markset.empty())
      area->set_readmarked(false);
    if(gkbd.quitall)
      QuitNow();
  }
}


//  ------------------------------------------------------------------

