
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gemnus.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Menus.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif


//  ------------------------------------------------------------------

void GMnuAsk::AskInit(int __row, int __col, char* __title, int __help) {

  Init();
  SetColor(C_ASKW, C_ASKQ, C_ASKN, C_ASKS, CFG->screenshadows ? C_SHADOW : -1);
  SetTitle(__title, C_ASKT);
  SetBorder(W_BASK, C_ASKB);
  SetPos(__row, __col);
  SetMask(M_CLOSE);
  SetHelp(__help);
}


//  ------------------------------------------------------------------

int GMnuYesNo::YesNo(int __row, int __col, char* __title, char* __yes, char* __no, int __help, int __yesno) {
  
  enum { TAG_ASKYES=100, TAG_ASKNO };

  HandleGEvent(EVTT_ASKYESNO);

  AskInit(__row, __col, __title, __help);

  Begin();
    Item(TAG_ASKYES, __yes);
    Item(TAG_ASKNO,  __no);
    SetTag(__yesno ? TAG_ASKYES : TAG_ASKNO);
  End();

  Start();

  return (finaltag == TAG_ASKYES) ? YES : NO;
}


//  ------------------------------------------------------------------

int GMenuQuit::Run() {

  int _yn = YesNo(1,2, CFG->language[LNG_QuitGoldED], CFG->language[LNG_QuitYes], CFG->language[LNG_QuitNo], H_AskExit);
  return _yn ? true : false;
}


//  ------------------------------------------------------------------

int GMenuReadonly::Run() {

  int _yn = NO;
  w_info(CFG->language[LNG_ReadOnlyWarn]);
  if(CFG->areareadonly == READONLY_SOFT)
    _yn = YesNo(6,0, CFG->language[LNG_IsReadOnly], CFG->language[LNG_ReadOnlyYes], CFG->language[LNG_ReadOnlyNo], H_AskReadOnly);
  else
    waitkeyt(10000);
  w_info(NULL);
  return _yn;
}


//  ------------------------------------------------------------------

int GMenuChange::Run(char* info) {

  w_info(info);
  int _yn = YesNo(6,0, CFG->language[LNG_Change], CFG->language[LNG_ChangeYes], CFG->language[LNG_ChangeNo], H_AskChange);
  w_info(NULL);
  return _yn;
}


//  ------------------------------------------------------------------

int GMenuLockLoop::Run() {

  w_info(CFG->language[LNG_WaitOrExit]);
  int _yn = YesNo(6,0, CFG->language[LNG_ReallyExit], CFG->language[LNG_QuitYes], CFG->language[LNG_QuitNo], H_AskExit);
  w_info(NULL);
  return _yn;
}


//  ------------------------------------------------------------------

int GMenuDelorig::Run() {

  return YesNo(6,0, CFG->language[LNG_DelOrig], CFG->language[LNG_DelOrigYes], CFG->language[LNG_DelOrigNo], H_AskDelOrig);
}


//  ------------------------------------------------------------------

int GMenuDropmsg::Run() {

  return YesNo(6,0, CFG->language[LNG_DropMsg], CFG->language[LNG_DropMsgYes], CFG->language[LNG_DropMsgNo], H_AskDropMsg, CFG->menudropmsg);
}


//  ------------------------------------------------------------------

int GMenuZonegate::Run() {

  return YesNo(6,0, CFG->language[LNG_ZoneGate], CFG->language[LNG_ZoneGateYes], CFG->language[LNG_ZoneGateNo], H_AskZoneGate);
}


//  ------------------------------------------------------------------

#define LNG__Forward    " Use FWD kludges? "
#define LNG__ForwardYes "Y       Yes Please.      "
#define LNG__ForwardNo  "N           No!          "

int GMenuForward::Run() {

  return YesNo(6,0, LNG__Forward, LNG__ForwardYes, LNG__ForwardNo, H_AskForward);
}


//  ------------------------------------------------------------------

int GMenuOverwrite::Run() {

  enum {TAG_APPEND=100, TAG_OVERWRITE, TAG_QUIT };

  HandleGEvent(EVTT_ASKYESNO);

  AskInit(6,0, CFG->language[LNG_FileExists], H_AskOverWrite);

  Begin();
    Item(TAG_APPEND,    CFG->language[LNG_Append]);
    Item(TAG_OVERWRITE, CFG->language[LNG_OverWrite]);
    Item(TAG_QUIT,      CFG->language[LNG_QuitExist]);
  End();

  Start();

  switch(finaltag) {
    case TAG_OVERWRITE:
      return true;
    case TAG_APPEND:
      return false;
  }

  return -1;
}


//  ------------------------------------------------------------------

int GMenuDelete::Run(int all, GMsg& __msg) {

  enum { TAG_YES=100, TAG_NO, TAG_NOASK };

  HandleGEvent(EVTT_ASKYESNO);

  char* _info = (__msg.attr.uns() and not __msg.attr.rcv()) ? CFG->language[LNG_WarnUnsent] : "";

  if(*_info)
    w_info(_info);

  AskInit(6,0, CFG->language[LNG_DeleteThis], H_AskDelete);

  Begin();
    Item(TAG_YES,   CFG->language[LNG_DeleteYes]);
    Item(TAG_NO,    CFG->language[LNG_DeleteNo]);
  if(all)
    Item(TAG_NOASK, CFG->language[LNG_DeleteNoAsk]);
  End();

  Start();

  if(*_info)
    w_info(NULL);

  switch(finaltag) {
    case TAG_YES:
      return true;
    case TAG_NOASK:
      return ALWAYS;
  }

  return false;
}


//  ------------------------------------------------------------------

static void PressY() { if(gkbd.kbuf == NULL) kbput(*CFG->language[LNG_GotoNextYes]); }
static void PressN() { if(gkbd.kbuf == NULL) kbput(*CFG->language[LNG_GotoNextNo]);  }

int GMenuNewarea::Run(gactivearea* area) {
                                    
  enum { TAG_YES=100, TAG_NO };

  HandleGEvent(EVTT_ENDOFMSGS);

  uint _temp = area->lastread();
  Key _yeskey = Key_Rgt;
  Key _nokey = Key_Lft;

  if(area->readmarked())
    _temp = area->markset.find(area->msgnset.relative_to_absolute(area->lastread()));

  if(_temp <= 1) {
    _yeskey = Key_Rgt;
    _nokey  = Key_Lft;
  }

  AskInit(6,0, CFG->language[LNG_GotoNext], H_AskNextArea);

  Begin();
    Item(TAG_YES, CFG->language[LNG_GotoNextYes], M_CLOSE, PressY, _yeskey);
    Item(TAG_NO,  CFG->language[LNG_GotoNextNo],  M_CLOSE, PressN, _nokey);
  End();

  Start();

  // Get the extra character from PressY/N
  if(finaltag != -1)
    getxch();

  return (finaltag == TAG_YES) ? true : false;
}


//  ------------------------------------------------------------------

int GMenuCMF::Run() {

  enum { TAG_COPY=100, TAG_MOVE, TAG_FORWARD, TAG_QUIT };

  static int __FAR _prevtag = TAG_FORWARD;

  update_statusline(CFG->language[LNG_CopyMoveForward]);

  AskInit(6,0, CFG->language[LNG_SelectAction], H_CopyMoveForward);

  Begin();
    Item(TAG_FORWARD, CFG->language[LNG_ForwardMessage]);
    Item(TAG_MOVE,    CFG->language[LNG_MoveMessage]);
    Item(TAG_COPY,    CFG->language[LNG_CopyMessage]);
    Item(TAG_QUIT,    CFG->language[LNG_QuitCMF]);
    SetTag(_prevtag);
  End();

  Start();

  _prevtag = finaltag;
  switch(finaltag) {
    case TAG_FORWARD:   return CMF_FORWARD;
    case TAG_MOVE:      return CMF_MOVE;
    case TAG_COPY:      return CMF_COPY;
  }
  _prevtag = TAG_FORWARD;
  return -1;
}


//  ------------------------------------------------------------------

int GMenuDomarks::Run(char* dowhat, gactivearea* area) {
                                    
  enum { TAG_MARKED=100, TAG_CURRENT, TAG_QUIT };

  static int _prevtag = MODE_CURRENT;
  if(CFG->menumarked == MODE_MARKED)
    _prevtag = TAG_MARKED;
  else if(CFG->menumarked == MODE_CURRENT)
    _prevtag = TAG_CURRENT;
  else if(CFG->menumarked == MODE_DONT)
    _prevtag = area->markset.size() ? TAG_MARKED : TAG_CURRENT;

  update_statuslinef("%i %s %s", area->markset.size(), area->markset.size() == 1 ? CFG->language[LNG_is] : CFG->language[LNG_are], CFG->language[LNG_marked]);

  AskInit(6,0, dowhat, H_AskDoMarks);

  Begin();
    Item(TAG_MARKED,  CFG->language[LNG_MarkedMsg]);
    Item(TAG_CURRENT, CFG->language[LNG_CurrentMsg]);
    Item(TAG_QUIT,    CFG->language[LNG_QuitMsgs]);
    SetTag(_prevtag);
  End();

  Start();

  _prevtag = finaltag;
  switch(finaltag) {
    case TAG_CURRENT:   return MODE_CURRENT;
    case TAG_MARKED:    return MODE_MARKED;
  }
  return MODE_DONT;
}


//  ------------------------------------------------------------------

int GMenuWriteMsg::Run() {
         
  enum {
    TAG_DISKFILE=100,
    TAG_PRINTER,
    TAG_DISKFILE_NOHDR,
    TAG_PRINTER_NOHDR,
    TAG_QUIT
  };

  static int __FAR _prevtag = TAG_DISKFILE;

  update_statusline(CFG->language[LNG_WriteMsgsTo]);

  AskInit(6,0, CFG->language[LNG_WriteTo], H_WriteMessage);

  Begin();
    Item(TAG_DISKFILE,       CFG->language[LNG_Diskfile]);
    Item(TAG_PRINTER,        CFG->language[LNG_Printer]);
    Item(TAG_DISKFILE_NOHDR, CFG->language[LNG_DiskfileNoHdr]);
    Item(TAG_PRINTER_NOHDR,  CFG->language[LNG_PrinterNoHdr]);
    Item(TAG_QUIT,           CFG->language[LNG_QuitWrite]);
    SetTag(_prevtag);
  End();

  Start();

  _prevtag = finaltag;
  switch(finaltag) {
    case TAG_DISKFILE:        return WRITE_FILE;
    case TAG_PRINTER:         return WRITE_PRINTER;
    case TAG_DISKFILE_NOHDR:  return WRITE_FILE_NOHDR;
    case TAG_PRINTER_NOHDR:   return WRITE_PRINTER_NOHDR;
  }
  _prevtag = TAG_DISKFILE;
  return -1;
}


//  ------------------------------------------------------------------

int GMenuMarkMsgs::Run() {
                                    
  static int __FAR _prevtag = TAG_MARKYOURMAIL;

  update_statusline(CFG->language[LNG_MarkingOptions]);

  AskInit(6,0, CFG->language[LNG_MarkWhat], H_MarkMessages);
  Begin();
    Item(TAG_MARKYOURMAIL, CFG->language[LNG_Yourmail]);
    Item(TAG_MARKHEADER,   CFG->language[LNG_FromToSubj]);
    Item(TAG_MARKTXTHDR,   CFG->language[LNG_TextHdr]);
    Item(TAG_MARKTHREAD,   CFG->language[LNG_Thread]);
    Item(TAG_MARKNEWMSGS,  CFG->language[LNG_NewMsgs]);
    Item(TAG_MARKOLDMSGS,  CFG->language[LNG_OldMsgs]);
    Item(TAG_MARKALLMSGS,  CFG->language[LNG_AllMsgs]);
    Item(TAG_MARKUNMARK,   CFG->language[LNG_Unmark]);
    Item(TAG_MARKRANGE,    CFG->language[LNG_Range]);
    Item(TAG_MARKTOGGLE,   CFG->language[LNG_Markstoggle]);
    Item(TAG_MARKQUIT,     CFG->language[LNG_QuitMarks]);
    SetTag(_prevtag);
  End();

  Start();

  if(finaltag == -1)
    return -1;
  else
    _prevtag = finaltag;

  return _prevtag;
}


//  ------------------------------------------------------------------

static void _ask_attributes() {

  AskAttributes(*MenuMsgPtr, MenuAreaPtr);
}


//  ------------------------------------------------------------------

int GMenuEditfile::Run(GMsg& msg, gactivearea* area) {

  if(not CFG->editsavemenu)
    return STATUS_SAVE;

  enum {
    TAG_YES = 100,
    TAG_NO,
    TAG_CONTINUE,
    TAG_ROT13,
    TAG_ATTRS,
    TAG_TAGLINE,
    TAG_ORIGIN,
    TAG_MOOD,
    TAG_VIEW,
    TAG_HEADER,
    TAG_UTILS = 1000
  };

  MenuMsgPtr = &msg;

  int _topline = 0;
  int _again;

  do {

    int _rot13 = YES;
    _again = true;

    update_statuslinef(CFG->language[LNG_Quotepct], msg.quotepct, msg.quotepct > 80 ? "!!!" : "");
    
    char _title[80];
    sprintf(_title, CFG->language[LNG_SaveMsg], msg.display.size());
    AskInit(6,0, _title, H_SaveMsg);

    #define LNG__MoodS "M Change Mood           "

    Begin();
      Item(TAG_YES,      CFG->language[LNG_YesGreat]);
      Item(TAG_NO,       CFG->language[LNG_Kickit]);
      Item(TAG_CONTINUE, CFG->language[LNG_Continue]);
      Item(TAG_ROT13,    CFG->language[LNG_Rot13],    _rot13 ? M_CLOSE : M_NOSEL);
      Item(TAG_ATTRS,    CFG->language[LNG_AttribS],  _ask_attributes, 0);
      if(CFG->tagline.size())
        Item(TAG_TAGLINE,CFG->language[LNG_TaglineS]);
      Item(TAG_ORIGIN,   CFG->language[LNG_OriginS]);
      if(CFG->mood.size())
        Item(TAG_MOOD,   LNG__MoodS);
      Item(TAG_VIEW,     CFG->language[LNG_View]);
      Item(TAG_HEADER,   CFG->language[LNG_HeaderEdit]);
      if(CFG->editsaveutil.First()) {
        int n = 0;
        do {
          Item(TAG_UTILS+(n++), (char*)CFG->editsaveutil.Text());  // WARNING!!!
        } while(CFG->editsaveutil.Next());
      }
    End();

    Start();

    switch(finaltag) {

      case TAG_YES:
        return STATUS_SAVE;

      case -1:
      case TAG_NO:
        {
          GMenuDropmsg MenuDropmsg;
          if(MenuDropmsg.Run())
            return STATUS_QUIT;
        }
        break;

      case TAG_CONTINUE:
        return STATUS_CONTINUE;

      case TAG_ROT13:
        Rot13(msg);
        BodyView.Use(area, &msg, _topline);
        BodyView.Paint();
        break;

      case TAG_TAGLINE:
        if(ChangeTagline(area)) {
          gstring buf;
          buf = area->tagline().c_str();
          if(buf[0] == '@')
            GetRandomLine(buf, buf.c_str()+1);
          msg.tagline = buf;
          for(int n=0; n<msg.linearray.size(); n++) {
            if(msg.linearray.data(n).istagline()) {
              buf = gstrf("%c%c%c %s ", area->taglinechar(), area->taglinechar(), area->taglinechar(), msg.tagline.c_str());
              buf.trim();
              msg.linearray.set(n, buf.c_str());
              break;
            }
          }
        }
        BodyView.Use(area, &msg, _topline);
        BodyView.Paint();
        break;

      case TAG_ORIGIN:
        if(ChangeOrigin(area)) {
          gstring buf;
          buf = area->origin();
          if(buf[0] == '@')
            GetRandomLine(buf, buf.c_str()+1);
          msg.origin = buf;
          msg.netname = area->netname();
          for(int n=0; n<msg.linearray.size(); n++) {
            if(msg.line.data(n).isorigin()) {
              msg.make_originline(buf);
              msg.linearray.set(n, buf.c_str());
              break;
            }
          }
        }
        BodyView.Use(area, &msg, _topline);
        BodyView.Paint();
        break;

      case TAG_MOOD:
        if(ChangeMood(area)) {
          gstring buf;
          buf = area->mood();
          if(buf[0] == '@')
            GetRandomLine(buf, buf.c_str()+1);
          msg.mood = buf;
          HeaderView.Use(area, &msg);
          HeaderView.Paint();
        }
        break;

      case TAG_VIEW:
        return STATUS_VIEW;

      case TAG_HEADER:
        EditHeaderinfo(MODE_CHANGE, HeaderView);
        break;

      default:
        if(finaltag >= TAG_UTILS) {
          ExternUtil(msg, CFG->editsaveutil.Number(finaltag-TAG_UTILS), area);
          BodyView.Use(area, &msg, _topline);
          BodyView.Paint();
        }
    }
  } while(_again);

  return STATUS_SAVE;
}


//  ------------------------------------------------------------------

enum {
  TAG_CCPROCESS = 100,
  TAG_CCIGNORE,
  TAG_CCATTRS,
  TAG_CCLIST,
    TAG_CCLISTKEEP,
    TAG_CCLISTNAMES,
    TAG_CCLISTVISIBLE,
    TAG_CCLISTHIDDEN,
    TAG_CCLISTREMOVE
};

static void SetCCListKeep()    { CFG->carboncopylist = CC_KEEP; }
static void SetCCListNames()   { CFG->carboncopylist = CC_NAMES; }
static void SetCCListVisible() { CFG->carboncopylist = CC_VISIBLE; }
static void SetCCListHidden()  { CFG->carboncopylist = CC_HIDDEN; }
static void SetCCListRemove()  { CFG->carboncopylist = CC_REMOVE; }

int GMenuCarbon::Run(GMsg& msg) {

  MenuMsgPtr = &msg;

  int _listtag = TAG_CCLISTKEEP;
  switch(CFG->carboncopylist) {
    case CC_KEEP:     _listtag = TAG_CCLISTKEEP;      break;
    case CC_NAMES:    _listtag = TAG_CCLISTNAMES;     break;
    case CC_VISIBLE:  _listtag = TAG_CCLISTVISIBLE;   break;
    case CC_HIDDEN:   _listtag = TAG_CCLISTHIDDEN;    break;
    case CC_REMOVE:   _listtag = TAG_CCLISTREMOVE;    break;
  }

  AskInit(6,0, CFG->language[LNG_CarbonCopy], H_CarbonCopy);

  Begin();
    Item(TAG_CCPROCESS, CFG->language[LNG_CCProcess]);
    Item(TAG_CCIGNORE,  CFG->language[LNG_CCIgnore]);
    Item(TAG_CCATTRS,   CFG->language[LNG_CCAttribs], _ask_attributes, 0);
    Item(TAG_CCLIST,    CFG->language[LNG_CCListFmt]);
    SetTitle(CFG->language[LNG_CCList]);
    Begin();
      Item(TAG_CCLISTKEEP,    CFG->language[LNG_CCListKeep],    SetCCListKeep);
      Item(TAG_CCLISTNAMES,   CFG->language[LNG_CCListNames],   SetCCListNames);
      Item(TAG_CCLISTVISIBLE, CFG->language[LNG_CCListVisible], SetCCListVisible);
      Item(TAG_CCLISTHIDDEN,  CFG->language[LNG_CCListHidden],  SetCCListHidden);
      Item(TAG_CCLISTREMOVE,  CFG->language[LNG_CCListRemove],  SetCCListRemove);
      SetTag(_listtag);
    End();
  End();

  SetEsc(false);

  Start();

  return finaltag == TAG_CCPROCESS;
}


//  ------------------------------------------------------------------

int GMnuAreaDo::With(char* __title, char* __all, char* __marked, char* __current, int __full, char* __no, int __help) {

  enum {
    TAG_ALL=100,
    TAG_MARKED,
    TAG_CURRENT,
    TAG_MATCHING,
    TAG_UNSCANNED,
    TAG_GROUP,
    TAG_QWK,
      TAG_IMPORTQWK,
      TAG_EXPORTQWK,
    TAG_SOUP,
      TAG_EXPORTSOUP,
      TAG_IMPORTSOUP,
    TAG_NO,
    TAG_LIST = 1000
  };

  AskInit(1,2, __title, __help);

  Begin();
    Item(TAG_ALL, __all);
    Item(TAG_MARKED, __marked);
    Item(TAG_CURRENT, __current);
  if(__full) {
    Item(TAG_MATCHING, CFG->language[LNG_Scan_Matching]);
    Item(TAG_UNSCANNED, CFG->language[LNG_Scan_Unscanned]);
    Item(TAG_GROUP, CFG->language[LNG_ScanGroup]);
    if(AL.listscan.First()) {
      int count = 0;
      do {
        Item(TAG_LIST+count, (char*)AL.listscan.menutext());  // WARNING!!!
        count++;
      } while(AL.listscan.Next());
    }
    if(*QWK->ImportPath()) {
    Item(TAG_QWK, CFG->language[LNG_ScanQWK]);
      SetTitle(CFG->language[LNG_QWKPacket]);
      Begin();
        Item(TAG_IMPORTQWK, CFG->language[LNG_ScanImport], M_CLALL);
        Item(TAG_EXPORTQWK, CFG->language[LNG_ScanExport], M_CLALL);
      End();
    }
    if(*CFG->soupimportpath) {
    Item(TAG_SOUP, CFG->language[LNG_ScanSoup]);
      SetTitle(CFG->language[LNG_SoupPacket]);
      Begin();
        Item(TAG_IMPORTSOUP, CFG->language[LNG_ScanImport], M_CLALL);
        Item(TAG_EXPORTSOUP, CFG->language[LNG_ScanExport], M_CLALL);
      End();
    }
  }
    Item(TAG_NO, __no);
    if(*area_maybe)
      SetTag(TAG_MATCHING);
  End();

  Start();

  if(finaltag >= TAG_LIST) {
    AL.listscan.CurrNo(finaltag-TAG_LIST);
    return SCAN_LIST;
  }

  switch(finaltag) {
    case TAG_ALL:         return SCAN_ALL;
    case TAG_GROUP:       return SCAN_GROUP;
    case TAG_MARKED:      return SCAN_MARKED;
    case TAG_CURRENT:     return SCAN_CURRENT;
    case TAG_MATCHING:    return SCAN_MATCHING;
    case TAG_UNSCANNED:   return SCAN_UNSCANNED;
    case TAG_EXPORTQWK:   return SCAN_EXPORTQWK;
    case TAG_IMPORTQWK:   return SCAN_IMPORTQWK;
    case TAG_EXPORTSOUP:  return SCAN_EXPORTSOUP;
    case TAG_IMPORTSOUP:  return SCAN_IMPORTSOUP;
  }

  return SCAN_QUIT;
}


//  ------------------------------------------------------------------

int GMenuAreaScan::Run(int pmscan) {

  return With((pmscan ? CFG->language[LNG_ScanPM] : CFG->language[LNG_Scan_Areas]), CFG->language[LNG_Scan_All], CFG->language[LNG_Scan_Marked], CFG->language[LNG_Scan_Current], true, CFG->language[LNG_No_Scan], H_ScanAreas);
}


//  ------------------------------------------------------------------

int GMenuAreaHeat::Run() {

  return With(CFG->language[LNG_Heat_Areas], CFG->language[LNG_Heat_All], CFG->language[LNG_Heat_Marked], CFG->language[LNG_Heat_Current], false, CFG->language[LNG_No_Heat], H_HeatAreas);
}


//  ------------------------------------------------------------------

int GMenuAreaZap::Run() {

  return With(CFG->language[LNG_Zap_Areas], CFG->language[LNG_Zap_All], CFG->language[LNG_Zap_Marked], CFG->language[LNG_Zap_Current], false, CFG->language[LNG_No_Zap], H_ZapAreas);
}


//  ------------------------------------------------------------------

int GMenuEditHeader::Run(int mode, GMsg& msg, gactivearea* area) {

  enum {
    TAG_INTERNAL = 100,
    TAG_EXTERNAL,
    TAG_SAVE,
    TAG_ATTRS,
    TAG_TEMPLATE,
    TAG_TAGLINE,
    TAG_ORIGIN,
    TAG_MOOD,
    TAG_QUIT
  };

  HeaderView.Use(area, &msg);
  HeaderView.Paint();

  int _tag;
  int _again;

  int doedithdr = CFG->editheaderfirst;
  if(doedithdr) {
    switch(mode) {
      case MODE_NEW:
        doedithdr &= EDITHEADERFIRST_NEW;
        break;
      case MODE_CHANGE:
        doedithdr &= EDITHEADERFIRST_CHANGES;
        break;
      case MODE_FORWARD:
        doedithdr &= EDITHEADERFIRST_FORWARDS;
        break;
      default:
        doedithdr &= EDITHEADERFIRST_REPLIES;
    }
  }

  do {

    update_statusline(CFG->language[LNG_EditHeader]);

    if(doedithdr) {
      _tag = EditHeaderinfo(mode, HeaderView);
      if((_tag == W_ESCPRESS) or gkbd.quitall)
        break;
    }

    do {

      _again = false;

      _tag = (CFG->editinternal ? TAG_INTERNAL : (*CFG->editexternal ? TAG_EXTERNAL : TAG_INTERNAL));

      if(CFG->editmenu) {

        Init();
        SetColor(C_MENUW, C_MENUQ, C_MENUN, C_MENUS, CFG->screenshadows ? C_SHADOW : -1);
        SetTitle(CFG->language[LNG_MsgOptions], C_MENUT);
        SetBorder(W_BMENU, C_MENUB);
        SetPos(6,0);
        SetMask(0);
        SetHelp(H_Message);

        #define LNG__Mood "M Moods           "

        Begin();
          Item(TAG_INTERNAL, CFG->language[LNG_InternalEd]);
        if(*CFG->editexternal)
          Item(TAG_EXTERNAL, CFG->language[LNG_ExternalEd]);
          Item(TAG_SAVE,     CFG->language[LNG_SaveNoEdit]);
          Item(TAG_ATTRS,    CFG->language[LNG_AttrO]);
          Item(TAG_TEMPLATE, CFG->language[LNG_Template]);
        if(CFG->tagline.size())
          Item(TAG_TAGLINE,  CFG->language[LNG_Tagline]);
          Item(TAG_ORIGIN,   CFG->language[LNG_Origin]);
        if(CFG->mood.size())
          Item(TAG_MOOD,     LNG__Mood);
          Item(TAG_QUIT,     CFG->language[LNG_QuitMessage]);
          SetTag(_tag);
        End();
        Start();
        _tag = finaltag;
      }
      if(_tag == -1)
        _tag = W_ESCPRESS;
      else {
        switch(_tag) {
          case TAG_ATTRS:
            AskAttributes(msg, area);
            CheckSubject(msg, msg.re, area);
            HeaderView.Use(area, &msg);
            HeaderView.Paint();
            _again = true;
            break;
          case TAG_ORIGIN:
            if(ChangeOrigin(area)) {
              if(area->origin()[0] == '@') {
                gstring o;
                GetRandomLine(o, area->origin().c_str()+1);
                area->set_origin(o);
              }
              msg.origin = area->origin();
            }
            _again = true;
            break;
          case TAG_TAGLINE:
            if(ChangeTagline(area)) {
              if(area->tagline()[0] == '@') {
                gstring t;
                GetRandomLine(t, area->tagline().c_str()+1);
                area->set_tagline(t);
              }
              msg.tagline = area->tagline();
            }
            _again = true;
            break;
          case TAG_TEMPLATE:
            if(ChangeTemplate(area) != -1)
              area->set_tpl(CFG->tpl[CFG->tplno].file);
            _again = true;
            break;
          case TAG_MOOD:
            if(ChangeMood(area)) {
              if(area->mood()[0] == '@') {
                gstring m;
                GetRandomLine(m, area->mood().c_str()+1);
                area->set_mood(m);
              }
              msg.mood = area->mood();
            }
            _again = true;
            break;
        }
      }
      update_statusline(CFG->language[LNG_EditHeader]);
    } while(_again);
  } while((_tag == W_ESCPRESS) and doedithdr);

  HeaderView.Use(area, &msg);
  HeaderView.Paint();
  
  if(not gkbd.quitall) {
    switch(_tag) {
      case W_ESCPRESS:
      case TAG_QUIT:
        return STATUS_DROP;
      case TAG_EXTERNAL:
        CFG->editinternal = false;
        return STATUS_EDIT;
      case TAG_INTERNAL:
        CFG->editinternal = true;
        return STATUS_EDIT;
      case TAG_SAVE:
        return STATUS_SAVE;
    }
  }
  return STATUS_DROP;
}


//  ------------------------------------------------------------------

int GMenuNavigate::Run() {

  enum {
    TAG_MAIN = 100,
      TAG_MAIN_SHELL   = (int)KK_ReadDosShell,
      TAG_MAIN_QUIT    = (int)KK_ReadQuitNow,
    TAG_EDIT = 101,
      TAG_EDIT_ENTER   = (int)KK_ReadNewMsg,
      TAG_EDIT_QUOTE   = (int)KK_ReadQuoteMsg,
      TAG_EDIT_COMMENT = (int)KK_ReadCommentMsg,
      TAG_EDIT_OTHER = 1010,
        TAG_EDIT_OTHER_QUOTE   = (int)KK_ReadMoveQuoteMsg,
        TAG_EDIT_OTHER_COMMENT = (int)KK_ReadMoveCommentMsg,
    TAG_SEL = 102,
      TAG_SEL_ORIGIN   = (int)KK_ReadChangeOrigin,
      TAG_SEL_USERNAME = (int)KK_ReadChangeUsername,
      TAG_SEL_AKA      = (int)KK_ReadChangeAka,
      TAG_SEL_ATTRS    = (int)KK_ReadChangeAttrs,
      TAG_SEL_TEMPLATE = (int)KK_ReadChangeTemplate,
    TAG_UTIL = 103,
      TAG_UTIL_CMF     = (int)KK_ReadCopyMoveForward,
      TAG_UTIL_LIST    = (int)KK_ReadMessageList,
      TAG_UTIL_WRITE   = (int)KK_ReadWriteMsg,
      TAG_UTIL_FREQ    = (int)KK_ReadFileRequest,
    TAG_HELP = 104,
      TAG_HELP_GENERAL,
      TAG_HELP_ABOUT,
    TAG_SEPARATOR,
    TAG_END
  };

  Init();
  SetColor(C_ASKW, C_ASKQ, C_ASKN, C_ASKS);
  SetTitle(NULL, C_ASKT);
  SetBorder(5, C_ASKB);
  SetPos(6,0, gvid.numcols, 1);
  SetMask(M_CLALL);
  SetHelp(9999);

  Begin(M_HORZ);
    SetBorder(W_BASK, C_ASKB);
    SetColor(C_ASKW, C_ASKQ, C_ASKN, C_ASKS, CFG->screenshadows ? C_SHADOW : -1);
    Item(TAG_MAIN, "M Main ");
    BeginPullDown();
      Item(TAG_MAIN_SHELL, "S Shell to OS ");
      Item(TAG_MAIN_QUIT,  "Q Quit GoldED ");
    End();
    Item(TAG_EDIT, "E Edit ");
    BeginPullDown();
      Item(TAG_EDIT_ENTER,   "E Enter new msg     ");
      Item(TAG_EDIT_QUOTE,   "Q Quote-reply       ");
      Item(TAG_EDIT_COMMENT, "C Comment-reply     ");
      Item(TAG_EDIT_OTHER,   "O Other area reply >");
      SetTitle(" Other Area ");
      Begin();
        Item(TAG_EDIT_OTHER_QUOTE,   "Q Quote-reply   ");
        Item(TAG_EDIT_OTHER_COMMENT, "C Comment-reply ");
      End();
    End();
    Item(TAG_SEL, "C Change ");
    BeginPullDown();
      Item(TAG_SEL_ORIGIN,    "O Origin    ");
      Item(TAG_SEL_USERNAME,  "U Username  ");
      Item(TAG_SEL_AKA,       "A Aka       ");
      Item(TAG_SEL_ATTRS,     "M Msg attrs ");
      Item(TAG_SEL_TEMPLATE,  "T Template  ");
    End();
    Item(TAG_UTIL, "U Util ");
    BeginPullDown();
      Item(TAG_UTIL_CMF,   "C Copy/Move/Forward     ");
      Item(TAG_UTIL_LIST,  "L List messages         ");
      Item(TAG_UTIL_WRITE, "W Write to disk/printer ");
      Item(TAG_UTIL_FREQ,  "F File request          ");
    End();
    Item(TAG_UTIL, "H Help ");
    BeginPullDown();
      Item(TAG_HELP_GENERAL, "G General ");
      Item(TAG_SEPARATOR,    "----------", M_NOSEL);
      Item(TAG_HELP_ABOUT,   "A About   ");
    End();
  End();
  Start();
  if((finaltag >= TAG_MAIN) and (finaltag < (TAG_END*10)))
    finaltag = -1;
  return finaltag;
}


//  ------------------------------------------------------------------

int GMenuConfirm::Run() {

  w_info(CFG->language[LNG_ConfirmInfo]);
  int _yn = YesNo(6,0, CFG->language[LNG_Confirm], CFG->language[LNG_ConfirmYes], CFG->language[LNG_ConfirmNo],[LNG_H_MenuConfirm], NULL);
  return _yn;
}


//  ------------------------------------------------------------------

