
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gemrks.cpp,v 1.1.1.1 1998/11/21 19:38:04 odinn Exp $
//  ------------------------------------------------------------------
//  Msg marks handling.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif


//  ------------------------------------------------------------------

int NextMarkedmsg(int direction, GMsg& msg, gactivearea* area) {

  gtag& tag = area->readpersonal() ? area->pmrkset : area->markset;

  if(tag.size()) {
    register uint n;
    ulong msgno = 0;
    if(direction == DIR_NEXT) {
      if(msg.msgno != tag[tag.size()-1]) {
        for(n=0; n<tag.size(); n++) {
          if(tag[n] > msg.msgno) {
            break;
          }
        }
        if(n >= tag.size())
          n = tag.size()-1;
        msgno = n+1;
      }
      else {
        n = tag.size()-1;
      }
      area->set_lastread(area->msgnset.absolute_to_relative(tag[n]));
    }
    else {
      if(msg.msgno != tag[0]) {
        n = tag.size();
        do {
          if(tag[--n] < msg.msgno)
            break;
        } while(n);
        msgno = n+1;
      }
      else {
        n = 0;
      }
      area->set_lastread(area->msgnset.absolute_to_relative(tag[n]));
    }
    return msgno ? 1 : 0;
  }
  return 0;
}


//  ------------------------------------------------------------------

void MarkMsgs_Unmark(gactivearea* area) {

  area->markset.reset();
  area->set_readmarked(false);
}


//  ------------------------------------------------------------------

void MarkMsgs_Toggle(gactivearea* area) {

  gtag tmp;

  // Transplant current marks
  tmp.resize(area->markset.size());
  tmp.tag = area->markset.tag;

  // Reset marks
  area->markset.resize(0);
  area->markset.tag = NULL;
  area->set_readmarked(false);

  if(tmp.size()) {
    area->markset.resize(area->msgnset.size()-tmp.size());
    uint m=0, n=0, x=0;
    while(n < area->msgnset.size()) {
      if(tmp.tag[x] != area->msgnset[n])
        area->markset[m++] = area->msgnset[n];
      else
        x++;
      n++;
    }
  }

  tmp.reset();
}


//  ------------------------------------------------------------------

void MarkMsgs_All(gactivearea* area) {

  area->markset.resize(area->msgnset.size());
  memcpy(area->markset.tag, area->msgnset.tag, area->msgnset.size()*sizeof(ulong));
}


//  ------------------------------------------------------------------

void MarkMsgs_New(gactivearea* area) {

  uint oldmarks = area->markset.size();
  area->markset.resize(area->markset.size()+(area->msgnset.size() - area->lastread()));
  memcpy(area->markset.tag+oldmarks, area->msgnset.tag+area->lastread(), (area->msgnset.size()-area->lastread())*sizeof(ulong));
  area->markset.sort();
  area->markset.eliminate_duplicates();
}


//  ------------------------------------------------------------------

void MarkMsgs_Old(gactivearea* area) {

  uint oldmarks = area->markset.size();
  area->markset.resize(area->markset.size() + (area->lastread() - 1));
  memcpy(area->markset.tag+oldmarks, area->msgnset.tag, (area->lastread()-1)*sizeof(ulong));
  area->markset.sort();
  area->markset.eliminate_duplicates();
}


//  ------------------------------------------------------------------

void MarkMsgs_Range(gactivearea* area) {

  uint markstart = area->msgnset.absolute_to_relative(area->bookmark());
  uint markstop = area->lastread();
  uint temp = minimum_of_two(markstart, markstop);
  uint mrks = (maximum_of_two(markstart, markstop) - minimum_of_two(markstart, markstop)) + 1;
  uint oldmarks = area->markset.size();
  area->markset.resize(area->markset.size() + mrks);
  for(uint n=0; n<mrks; n++)
    area->markset[oldmarks+n] = area->msgnset[temp+n-1];
  area->markset.sort();
  area->markset.eliminate_duplicates();
}


//  ------------------------------------------------------------------

static BmhArg* __FAR mark_pattern = NULL;
static int __FAR mark_pats = 0;

static int MarkPatterns(const char* str) {

  uint len = strlen(str);
  for(int n=0; n<mark_pats; n++)
    if(strbmhsearch(str, len, &mark_pattern[n]))
      return true;
  return false;
}


//  ------------------------------------------------------------------

void MarkMsgs_Txt(int item, gstring& markstring, gactivearea* area) {

  char buf[256], buf2[100];
  char patstr[10][BmhMAXPAT];

  mark_pattern = (BmhArg*)throw_calloc(10, sizeof(BmhArg));

  if(item == TAG_MARKHEADER or item == TAG_MARKTXTHDR) {
    if(not edit_string(markstring, sizeof(BmhPat), CFG->language[LNG_EnterMarkString], H_MarkString)) {
      throw_free(mark_pattern);
      return;
    }
  }

  GMsg msg;

  const char* mrkstr = markstring.c_str();

  int reverse = NO;
  int caseignore = true;
  int direction = DIR_NEXT;
  int what = (item == TAG_MARKTXTHDR) ? GFIND_HDRTXT : GFIND_HDR;
  int what2 = 0;

  while(strchr("!-+<>:=", *mrkstr)) {
    switch(*mrkstr) {
      case '!':
        reverse = true;
        break;
      case '-':
        direction = DIR_PREV;
        break;
      case '+':
        direction = DIR_NEXT;
        break;
      case '<':
        what2 |= GFIND_FROM;
        break;
      case '>':
        what2 |= GFIND_TO;
        break;
      case ':':
        what2 |= GFIND_SUBJECT;
        break;
      case '=':
        caseignore = false;
        break;
    }
    mrkstr++;
  }

  if(what2) {
    if(what & GFIND_HDR)
      what &= ~GFIND_HDR;
    what |= what2;
  }

  sprintf(buf, "%s \"%s\" ", CFG->language[LNG_SearchingFor], mrkstr);
  w_progress(PROGRESS_NEW, C_INFOW, 0, area->msgnset.size(), buf);
  mark_pats = 0;
  strcpy(buf2, mrkstr);
  char* ptr = strtok(buf2, "|;&");
  while(ptr and mark_pats<10) {
    if(*ptr) {
      memset(&mark_pattern[mark_pats], 0, sizeof(BmhArg));
      memset(patstr[mark_pats], 0, BmhMAXPAT);
      strcpy(patstr[mark_pats], ptr);
      strbmhinit(patstr[mark_pats], &mark_pattern[mark_pats], caseignore);
      mark_pats++;
    }
    ptr = strtok(NULL, "|;&");
  }

  uint n;
  int marked=0;
  for(n=area->lastread(); direction == DIR_NEXT ? (n<=area->msgnset.size()) : (n>=1); direction == DIR_NEXT ? n++ : n--) {
    if(kbxhit()) {
      if(kbxget() == Key_Esc) {
        HandleGEvent(EVTT_SEARCHFAILED);
        break;
      }
    }

    update_statuslinef(CFG->language[LNG_SearchingMsg], n, area->msgnset.size(), marked);
    w_progress(PROGRESS_UPDATE, C_INFOW, n, area->msgnset.size(), NULL);

    if(area->loadmsg(msg, area->msgnset[n-1], CFG->dispmargin-CFG->disppagebar)) {

      if(what & GFIND_FROM)
        if(MarkPatterns(msg.by_name().c_str()))
          msg.foundwhere |= GFIND_FROM;

      if(what & GFIND_TO)
        if(MarkPatterns(msg.to.c_str()))
          msg.foundwhere |= GFIND_TO;

      if(what & GFIND_SUBJECT)
        if(MarkPatterns(msg.re.c_str()))
          msg.foundwhere |= GFIND_SUBJECT;

      if(what & GFIND_BODY) {
        for(int l=0; l<msg.display.size(); l++) {
          int n = msg.display[l];
          if(MarkPatterns(msg.linearray[n])) {
            msg.foundwhere |= GFIND_BODY;
            msg.linearray.data(n).type |= GLINE_HIGH;
          }
        }
      }

      if(reverse ? not msg.foundwhere : msg.foundwhere) {
        area->markset.add(msg.msgno);
        update_statuslinef(CFG->language[LNG_SearchingMsg], n, area->msgnset.size(), ++marked);
      }
    }
  }

  w_progress(PROGRESS_QUIT, 0, 0, 0, NULL);

  throw_free(mark_pattern);
}


//  ------------------------------------------------------------------

void MarkMsgs_Thread(GMsg& msg, gactivearea* area) {

  int temp = 1;
  int endloop = NO;

  GMsg tempmsg;

  area->markset.add(msg.msgno);

  update_statuslinef(CFG->language[LNG_MarkingChain], temp);
  ulong msgno = msg.link.to();
  while(area->msgnset.absolute_to_relative(msgno)) {  // Search backwards
    if(kbxhit()) {
      if(kbxget() == Key_Esc) {
        HandleGEvent(EVTT_SEARCHFAILED);
        endloop = YES;
        break;
      }
    }
    temp++;
    area->markset.add(msgno);
    update_statuslinef(CFG->language[LNG_MarkingChain], temp);
    if(not area->loadhdr(tempmsg, msgno))
      tempmsg.link.to_set(0);
    msgno = tempmsg.link.to();
  }
  if(not endloop) {
    msgno = msg.link.first();
    while(area->msgnset.absolute_to_relative(msgno)) {  // Search forward
      if(kbxhit()) {
        if(kbxget() == Key_Esc) {
          HandleGEvent(EVTT_SEARCHFAILED);
          break;
        }
      }
      temp++;
      area->markset.add(msgno);
      update_statuslinef(CFG->language[LNG_MarkingChain], temp);
      if(not area->loadhdr(tempmsg, msgno))
        tempmsg.link.first_set(0);
      msgno = tempmsg.link.first();
    }
  }
}


//  ------------------------------------------------------------------

void MarkingOptions(GMsg& msg, gactivearea* area) {

  GFTRK("MarkingOptions");

  if(area->msgnset.size()) {

    static gstring markstring;

    GMenuMarkMsgs MenuMarkMsgs;
    int item = MenuMarkMsgs.Run();

    switch(item) {

      // ---------------------------------------------------------------
      case TAG_MARKUNMARK:
        MarkMsgs_Unmark(area);
        break;

      // ---------------------------------------------------------------
      case TAG_MARKTOGGLE:
        MarkMsgs_Toggle(area);
        break;

      // ---------------------------------------------------------------
      case TAG_MARKALLMSGS:
        MarkMsgs_All(area);
        break;

      // ---------------------------------------------------------------
      case TAG_MARKNEWMSGS:
        MarkMsgs_New(area);
        break;

      // ---------------------------------------------------------------
      case TAG_MARKOLDMSGS:
        MarkMsgs_Old(area);
        break;

      // ---------------------------------------------------------------
      case TAG_MARKRANGE:
        MarkMsgs_Range(area);
        break;

      // ---------------------------------------------------------------
      case TAG_MARKYOURMAIL:
        markstring = area->username().name;
        // Drop through!

      // ---------------------------------------------------------------
      case TAG_MARKHEADER:
        // Drop through!

      // ---------------------------------------------------------------
      case TAG_MARKTXTHDR:
        MarkMsgs_Txt(item, markstring, area);
        break;

      // ---------------------------------------------------------------
      case TAG_MARKTHREAD:
        MarkMsgs_Thread(msg, area);
        break;
    }
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

