
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gemsgs.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Misc. msg handling.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

static bool tokenxchg(char*& dst, char* tok, const gstring& src) {

  register uint toklen = strlen(tok);
  if(strnieql(dst, tok, toklen)) {
    memmove(dst+src.length(), dst+toklen, strlen(dst+toklen)+1);
    memcpy(dst, src.c_str(), src.length());
    dst += src.length();
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

void TokenXlat(int mode, gstring& input, GMsg& msg, GMsg& oldmsg, gactivearea* destarea, gactivearea* origarea) {

  char buf[1024];
  strcpy(buf, input.c_str());
  TokenXlat(mode, buf, msg, oldmsg, destarea, origarea);
  input = buf;
}


//  ------------------------------------------------------------------

void TokenXlat(int mode, char* input, GMsg& msg, GMsg& oldmsg, gactivearea* destarea, gactivearea* origarea) {

  if(origarea == NULL)
    origarea = destarea;

  char verbuf[5];
  sprintf(verbuf, "%4.4s", CFG->ver);

  char revbuf[6];
  if(isalpha(CFG->ver[4]))
    sprintf(revbuf, "%c%02d%02d", toupper(CFG->ver[4]), str2mon(CFG->vers.date), atoi(&CFG->vers.date[4]));
  else
    sprintf(revbuf, "%02d%02d", str2mon(CFG->vers.date), atoi(&CFG->vers.date[4]));

  #if defined(__OS2__)
  char* osslashbuf = "/2";
  #elif defined(__WIN32__)
  char* osslashbuf = "/W32";
  #else
  char* osslashbuf = "";
  #endif

  gstring origareaid;
  gstring modereptr;
  switch(mode) {
    case MODE_QUOTE:
    case MODE_COMMENT:
    case MODE_REPLY:
    case MODE_FORWARD:
      modereptr = oldmsg.re;
      origareaid = oldmsg.kludge.area.length() ? oldmsg.kludge.area : origarea->echoid();
      break;
    default:
      modereptr = msg.re;
      origareaid = origarea->echoid();
  }

  char buf[100];
  char* dst = input;

  while(*dst) {
    while((*dst != '@') and (*dst != LF) and *dst)
      dst++;
    if(*dst == LF)
      *dst++ = CR;
    else if(*dst == '@') {
      if(dst[1] == '@') {
        memmove(dst, dst+1, strlen(dst+1)+1);
        dst++;
      }
      else {
        gstring msg_to_name = msg.to_name();
        gstring oldmsg_by_name = oldmsg.by_name();
        gstring oldmsg_to_name = oldmsg.to_name();
        if(tokenxchg(dst, "@cecho", destarea->echoid()))
          continue;
        if(tokenxchg(dst, "@cdesc", destarea->desc()))
          continue;
        if(tokenxchg(dst, "@oecho", origareaid))
          continue;
        if(tokenxchg(dst, "@odesc", AL.echoid_to_pointer(origareaid)->desc()))
          continue;
        if(tokenxchg(dst, "@oaddr", oldmsg.orig.make_string(buf, oldmsg.odom.c_str())))
          continue;
        if(tokenxchg(dst, "@_oaddr", strsetsz(oldmsg.orig.make_string(buf, oldmsg.odom.c_str()), 19)))
          continue;
        if(tokenxchg(dst, "@oname", oldmsg_by_name.btrim()))
          continue;
        if(tokenxchg(dst, "@_oname", strsetsz(strcpy(buf, oldmsg.by_name().c_str()), 34)))
          continue;
        if(tokenxchg(dst, "@ofname", strlword(oldmsg.by_name().c_str())))
          continue;
        if(tokenxchg(dst, "@olname", strrword(oldmsg.by_name().c_str())))
          continue;
        if(tokenxchg(dst, "@odate", strftimeip(buf, 80, CFG->language[LNG_DateFmt], glocaltime(&oldmsg.date.written))))
          continue;
        if(tokenxchg(dst, "@otime", strftimeip(buf, 80, CFG->language[LNG_TimeFmt], glocaltime(&oldmsg.date.written))))
          continue;
        if(tokenxchg(dst, "@daddr", oldmsg.dest.make_string(buf, oldmsg.ddom.c_str())))
          continue;
        if(tokenxchg(dst, "@_daddr", strsetsz(oldmsg.dest.make_string(buf, oldmsg.ddom.c_str()), 19)))
          continue;
        if(tokenxchg(dst, "@dname", oldmsg_to_name.btrim()))
          continue;
        if(tokenxchg(dst, "@_dname", strsetsz(strcpy(buf, oldmsg.to_name().c_str()), 34)))
          continue;
        if(tokenxchg(dst, "@dfname", strlword(oldmsg.to_name().c_str())))
          continue;
        if(tokenxchg(dst, "@dlname", strrword(oldmsg.to_name().c_str())))
          continue;
        if(tokenxchg(dst, "@taddr", msg.dest.make_string(buf, msg.ddom.c_str())))
          continue;
        if(tokenxchg(dst, "@_taddr", strsetsz(msg.dest.make_string(buf, msg.ddom.c_str()), 19)))
          continue;
        if(tokenxchg(dst, "@tname", msg_to_name.btrim()))
          continue;
        if(tokenxchg(dst, "@_tname", strsetsz(strcpy(buf, msg.to_name().c_str()), 34)))
          continue;
        if(tokenxchg(dst, "@tfname", strlword(msg.to_name().c_str())))
          continue;
        if(tokenxchg(dst, "@tlname", strrword(msg.to_name().c_str())))
          continue;
        if(tokenxchg(dst, "@caddr", destarea->aka().make_string(buf)))
          continue;
        if(tokenxchg(dst, "@_caddr", strsetsz(destarea->aka().make_string(buf), 19)))
          continue;
        if(tokenxchg(dst, "@cname", destarea->username().name))
          continue;
        if(tokenxchg(dst, "@_cname", strsetsz(strcpy(buf, destarea->username().name.c_str()), 34)))
          continue;
        if(tokenxchg(dst, "@cfname", strlword(strcpy(buf, destarea->username().name.c_str()))))
          continue;
        if(tokenxchg(dst, "@clname", strrword(strcpy(buf, destarea->username().name.c_str()))))
          continue;
        if(tokenxchg(dst, "@cdate", strftimeip(buf, 80, CFG->language[LNG_DateFmt], glocaltime(&msg.date.written))))
          continue;
        if(tokenxchg(dst, "@ctime", strftimeip(buf, 80, CFG->language[LNG_TimeFmt], glocaltime(&msg.date.written))))
          continue;
        if(tokenxchg(dst, "@version", gversion))
          continue;
        if(tokenxchg(dst, "@ver", verbuf))
          continue;
        if(tokenxchg(dst, "@rev", revbuf))
          continue;
        if(tokenxchg(dst, "@pid", __shortgpid__))
          continue;
        if(tokenxchg(dst, "@longpid", __gpid__))
          continue;
        if(tokenxchg(dst, "@serialno", CFG->ser))
          continue;
        if(tokenxchg(dst, "@os2slash", osslashbuf))
          continue;
        if(tokenxchg(dst, "@osslash", osslashbuf))
          continue;
        if(tokenxchg(dst, "@subject", modereptr))
          continue;
        if(tokenxchg(dst, "@tagline", destarea->tagline()))
          continue;
        if(tokenxchg(dst, "@tearline", destarea->tearline()))
          continue;
        if(tokenxchg(dst, "@origin", destarea->origin()))
          continue;
        if(tokenxchg(dst, "@gif", msg.gif.length() ? AddPath(CFG->gifpath, msg.gif.c_str()) : ""))
          continue;
        if(tokenxchg(dst, "@ofrom", oldmsg.rfc.from))
          continue;
        if(tokenxchg(dst, "@oto", oldmsg.rfc.to))
          continue;
        if(tokenxchg(dst, "@omessageid", oldmsg.rfc.message_id))
          continue;
        dst++;
      }
    }
  }
}


//  ------------------------------------------------------------------
//  ROT13 De/crypting function

void Rot13(GMsg& msg) {

  char c;
  static char __FAR rot13_table[256];
  static bool __FAR done = false;
  #define ROT13_DECODE(c)   (((c) & 0x80) ? (c) : rot13_table[c])
         
  if(not done) {
    for(int n=0; n<256; n++) {
      c = (char)n;
      if     (c >= 'a' and c <= 'm') c += '\xD';
      else if(c >= 'n' and c <= 'z') c -= '\xD';
      else if(c >= 'A' and c <= 'M') c += '\xD';
      else if(c >= 'N' and c <= 'Z') c -= '\xD';
      rot13_table[n] = c;
    }
    done = true;
  }

  glineref line = msg.linearray;
  while(line.valid()) {
    if(not (line.type() & (GLINE_HIDD|GLINE_KLUD|GLINE_TEAR|GLINE_ORIG))) {
      char* ptr = line.set_text();
      while((c = *ptr) != NUL)
        *ptr++ = ROT13_DECODE(c);
    }
    line++;
  }
}


//  ------------------------------------------------------------------

void GMsg::load_textfile(const char* filename, int margin, bool raw) {

  gfile fp(filename, raw ? "rb" : "rt");
  if(fp) {

    uint len = (uint)fp.filelength();
    txt = (char*)throw_realloc(txt, len+512);

    if(raw) {
      fp.fread(txt, len);
    }
    else {

      char* txtptr = txt;
      *txtptr = NUL;

      char tabspaces[9];
      strcpy(tabspaces, "        ");
      tabspaces[CFG->disptabsize] = NUL;

      bool hardcr = false;

      char hardline[20];
      strcpy(hardline, CFG->edithardlines ? CFG->edithardline : "");
      strcat(hardline, "\n");
      int hardlen = strlen(hardline);

      gmsg_quote quoting(area->quotechars().c_str(), CFG->dispsoftcr);

      const int PBUFSIZE = 16384;
      char* buf = (char*)throw_malloc(PBUFSIZE);

      while(fp.fgets(buf, PBUFSIZE)) {

        strsrep(buf, "\t", tabspaces);

        if(strneql(buf, hardline, hardlen)) {
          hardcr = not hardcr;
          if(*txtptr == LF)
            *txtptr = CR;
          if(*txtptr)
            txtptr++;
          strcpy(buf, "\r");
        }
        else {
          char* ptr = buf;
          int end = strlen(buf)-1;
          while(*ptr == ' ')
            ptr++;
          if(*ptr != LF) {
            strsrep(buf, hardline, "\r");
            if(hardcr or *buf == CTRL_A) {
              buf[end] = CR;
              if(*txtptr == LF)
                *txtptr = CR;
            }
            else if(quoting.is_quote(buf)) {
              buf[end] = CR;
              if(*txtptr == LF)
                *txtptr = CR;
            }
            else if((buf[0] == buf[1]) and (buf[0] == buf[2])) {
              buf[end] = CR;
              if(*txtptr == LF)
                *txtptr = CR;
            }
            else if(!memcmp(buf, " * Origin: ", 11)) {
              buf[end] = CR;
              if(*txtptr == LF)
                *txtptr = CR;
            }
            else if(!strnicmp(ptr, "CC:", 3)) {
              buf[end] = CR;
              if(*txtptr == LF)
                *txtptr = CR;
            }
            else if(!strnicmp(ptr, "XC:", 3)) {
              buf[end] = CR;
              if(*txtptr == LF)
                *txtptr = CR;
            }
            else if(!strnicmp(buf, "SEEN-BY: ", 9)) {
              buf[end] = CR;
              if(*txtptr == LF)
                *txtptr = CR;
            }
            if(*txtptr == LF) {
              if(*(txtptr-1) != ' ' and *ptr != ' ')
                *txtptr++ = ' ';
            }
            else if(*txtptr != 0)
              txtptr++;
          }
          else {
            if(*txtptr == LF)
              *txtptr = CR;
            if(*txtptr)
              txtptr++;
            strcpy(buf, "\r");
          }
        }
        if(*(txtptr-1) == CR or *txtptr == NUL)
          ptr = buf;
        int size = strlen(ptr);
        memcpy(txtptr, ptr, size);
        txtptr += size-1;
      }

      if(*txtptr != CR)
        *(++txtptr) = CR;
      *(++txtptr) = NUL;
    }

    fp.fclose();

    throw_free(buf);
  }

  text_to_lines(margin, false);
}


//  ------------------------------------------------------------------

void GMsg::lines_to_text() {

  // Allow paragraphs up to 16k long in the transfer buffer
  int bufsize = 16384+1;
  char* buf = (char*)throw_malloc(bufsize);

  XLAT->translate(oname);
  XLAT->translate(dname);
  if(not (attr.frq() or attr.att() or attr.urq()))
    XLAT->translate(subject);

  int lfterm = CFG->editcrlfterm and not area->ispcboard();
  int hardterm = area->edithardterm() or area->ispcboard() or area->iswildcat();
  int isezycom = area->isezycom();

  uint alloc_size = 1024;

  // Approximate the size of the text
  for(int x=0; x<line.size(); x++)
    alloc_size += line.length(x);
  alloc_size += line.size() * (lfterm ? 2 : 1);

  // Allocate the text
  txt = (char*)throw_realloc(txt, alloc_size);

  // Setup variables for the loop
  uint size = 0;
  char* ptr = txt;

  // Build the text from the lines

  for(int n=0; n<line.size(); n++) {
    *buf = NUL;
    char* bptr;
    bool hterm = true;
    if(line.data(n).kludge) {
      bptr = stpcpy(buf, line[n]);
      while((n+1 < line.size()) and line.data(n).iswrapped()) {
        n++;
        bptr = stpcpy(bptr, line[n]);
        if((bptr-buf) > (bufsize-256)) {
          int bptrlen = (int)(bptr-buf);
          bufsize += 4096;
          buf = (char*)throw_realloc(buf, bufsize);
          bptr = buf + bptrlen;
        }
      }
    }
    else {
      bptr = XLAT->translate(buf, line[n], CFG->dispsoftcr);
      if(*buf and not line.data(n).ishard()) {
        if(n+1 < line.size()) {
          if((*(bptr-1) != ' ') and (*line[n+1] != ' '))
            if(isezycom or not hardterm)
              *bptr++ = ' ';
          if(not hardterm) {
            if(isezycom)
              *bptr++ = SOFTCR;
            hterm = false;
          }
        }
      }
    }
    if(hterm) {
      *bptr++ = CR;
      if(lfterm)
        *bptr++ = LF;
    }
    if(*buf) {
      uint buflen = (uint)(bptr-buf);
      size += buflen;
      if(size >= alloc_size) {
        alloc_size += 1024;
        txt = (char*)throw_realloc(txt, alloc_size);
      }
      ptr = txt + size;
      memcpy(ptr-buflen, buf, buflen);
    }
  }
  *ptr = NUL;

  throw_free(buf);
}


//  ------------------------------------------------------------------

GMsg& GMsg::copy(const GMsg& m) {

  reset();

  // copy base fields
  gmsg::copy(m);

  // copy own fields
  quotepct        = m.quotepct;
  foundwhere      = m.foundwhere;
  foundtopline    = m.foundtopline;
  line_width      = m.line_width;
  orig_timesread  = m.orig_timesread;
  kludge.area     = m.kludge.area;
  area            = m.area;

  // copy lines
  glineref fromline = (glinearray)m.line;
  while(fromline.valid()) {
    line.add(fromline.text());
    fromline++;
  }
  reindex_lines();

  return *this;
}


//  ------------------------------------------------------------------

void GMsg::wrap(const char* text, const char* prefix, gstring& first, gstring& second, int width) {

  // width = 40
  //                   1111111111222222222233333333334444444444
  //         01234567890123456789012345678901234567890123456789
  //         --------------------------------------------------
  //                                                v- end
  // Case 1: this is another test with a bit of text to wrap.
  // Case 2: this is a test with a line that need wrapping.
  // Case 3: thisxisxaxtestxwithxaxlinexthatxneedxwrapping.
  // Case 4: >thisxisxaxtestxwithxaxlinexthatxneedxwrapping.

  const char* end = text + width - 1;

  // Locate the word to be wrapped

  if(*end == ' ') {

    // Case 1: A space was found as the last char inside the margin
    //
    // Now we must locate the first word outside the margin.
    // NOTE: Leading spaces to this word will be stripped!

    // Begin at the first char outside the margin
    end++;
  }
  else {

    // Case 2: A non-space was found as the last char inside the margin
    //
    // Now we must locate the beginning of the word we found.

    // Keep copy of original pointer
    const char* at_end = end;

    // Search backwards until a space or the beginning of the line is found
    while((end > text) and (*(end-1) != ' '))
      end--;

    // Check if we hit leading spaces
    const char* space = end;
    if(space > text) {
      space--;
      while((space > text) and (*space == ' '))
        space--;
    }

    // Did we search all the way back to the beginning of the line?
    if((end == text) or (*space == ' ')) {

      // Case 3: There are no spaces within the margin or we hit leading spaces

      // We have to break it up at the margin
      end = at_end;
    }
  }

  // The end pointer now points to the location to be wrapped

  first.copy(text, end-1);

  second = prefix;
  second += end;
}


//  ------------------------------------------------------------------

void GMsg::wrap_lines(int textwidth, int quotewidth) {

  glineref line = linearray;
  glineref nextline = line;

  gstring textbuffer, first, second;

  while(line.valid()) {

    int length = line.length();
    int wrapwidth = line.isquote() ? quotewidth : textwidth;

    if(length > wrapwidth) {

      if(line.more())
        nextline = line.lineno() + 1;

      if(line.isquote()) {

        int quotelength;
        char quotebuffer[20];
        int nextquotelength = 0;
        char nextquotebuffer[20];

        quoting.get_quotestring(line.text(), quotebuffer, quotelength);

        if(line.more()) {
          if(nextline.isquote()) {
            quoting.get_quotestring(nextline.text(), nextquotebuffer, nextquotelength);
            if(not strieql(quotebuffer, nextquotebuffer))
              nextquotelength = 0;
            else if(line.text()[quotelength] == nextline.text()[quotelength])
              nextquotelength = 0;
          }
        }

        if(nextquotelength) {

          textbuffer = quotebuffer;
          while(nextquotelength) {
            nextquotelength = 0;
            textbuffer += nextline.text() + quotelength;
            textbuffer.trim();
            textbuffer += " ";
            nextline.erase();
            if(nextline.valid()) {
              if(nextline.isquote()) {
                quoting.get_quotestring(nextline.text(), nextquotebuffer, nextquotelength);
                if(not strieql(quotebuffer, nextquotebuffer))
                  nextquotelength = 0;
              }
            }
          }
          textbuffer.trim();

          do {
            wrap(textbuffer.c_str()+quotelength, quotebuffer, first, second, wrapwidth-quotelength);
            line.set_text(first.c_str());
            line.insert_below(second.c_str(), line.type());
            textbuffer = second;
          } while(textbuffer.length() > wrapwidth);
        }
        else {
          wrap(line.text()+quotelength, quotebuffer, first, second, wrapwidth-quotelength);
          line.set_text(first.c_str());
          line.insert_below(second.c_str(), line.type());
        }
        line++;
      }
      else {
        uint type = line.type();
        wrap(line.text(), "", first, second, wrapwidth);
        line.set_text(first.c_str());
        line.type() &= ~GLINE_HARD;
        line.type() |= GLINE_WRAP;
        line.insert_below(second.c_str(), type);
      }
    }
  }
}


//  ------------------------------------------------------------------

