
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: genode.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Nodelist lookup and browser.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include <gftnnlfd.h>
#include <gftnnlfu.h>
#include <gftnnlge.h>
#include <gftnnlv7.h>


//  ------------------------------------------------------------------

typedef char ListStr[160];


//  ------------------------------------------------------------------
//  Static data for the browser and associated functions

static ftn_nodelist_index_base* __FAR NL = NULL;
static ftn_nodelist_entry       __FAR entry;
static int                      __FAR lookupfd;
static int                      __FAR lookupfu;
static int                      __FAR lookupge;
static int                      __FAR lookupv7;


//  ------------------------------------------------------------------

class NodelistBrowser {

public:

  int   ypos;                         // Window Starting Row
  int   xpos;                         // Window Starting Column
  int   ylen;                         // Window Height
  int   xlen;                         // Window Width
  int   btype;                        // Window Border Type
  int   battr;                        // Window Border Color
  int   wattr;                        // Window Color
  int   tattr;                        // Window Title Color
  int   sattr;                        // Window Selection Bar Color
  int   hattr;                        // Window Highlight Color
  int   loattr;                       // Window LoAttr Color
  int   sbattr;                       // Window Scrollbar Color
  char* title;                        // Window Title
  int   helpcat;                      // Window Help Category

  int   pos;                          // Display Pos (1 to maxpos)
  int   maxpos;                       // Display Pos

  int   aborted;                      // True if aborted
  int   listwrap;                     // True if wrap-around is supported

  gactivearea* area;

  ftn_nodelist_entry* entries;
  ListStr* liststr;
  gwindow nodewin;
  gwindow listwin;
  char    user_maybe[45];
  char    titlet[80];
  int     user_fuzidx;
  int     newmaybe;
  int     firstkey;

  NodelistBrowser();
  ~NodelistBrowser();

  void Open();
  void Close();

  void BeforeCursor();
  void AfterCursor();

  void InitDisplay();

  void DisplayBar();
  void DisplayLine(int line=-1);
  void DisplayPage();

  void Home();
  void End();

  void Up();
  void Down();

  void PageUp();
  void PageDown();

  void Center();

  int Run();

  void BuildListString(int line);

  void ScrollUp();
  void ScrollDown();

  int DoKey(Key& keycode);
};


//  ------------------------------------------------------------------

NodelistBrowser::NodelistBrowser() {

}


//  ------------------------------------------------------------------

NodelistBrowser::~NodelistBrowser() {

  throw_release(liststr);
  throw_release(entries);
}


//  ------------------------------------------------------------------

void NodelistBrowser::Open() {

  maxpos = ylen - 4;
  
  vcurhide();
  nodewin.openxy(ypos, xpos, 2, xlen, 5, battr, wattr, sbattr);
  listwin.openxy(ypos+2, xpos, ylen-2, xlen, btype, battr, wattr, sbattr);

  entries = (ftn_nodelist_entry*)throw_calloc(maxpos, sizeof(ftn_nodelist_entry));
  liststr = (ListStr*)throw_calloc(maxpos, sizeof(ListStr));

  InitDisplay();
}


//  ------------------------------------------------------------------

void NodelistBrowser::Close() {

  listwin.close();
  nodewin.close();
}


//  ------------------------------------------------------------------

void NodelistBrowser::BeforeCursor() {

  // Nothing to do
}


//  ------------------------------------------------------------------

void NodelistBrowser::AfterCursor() {

  char buf[200], line1[200], line2[200];

  ftn_nodelist_entry* entryp = entries + (pos - 1);

  sprintf(line1, " %s%s%s%s%s%s ",
    entryp->name,
    (*entryp->system ? ", " : ""), entryp->system,
    (*entryp->status ? " <" : ""), entryp->status,
    (*entryp->status ? ">" : "")
  );
  sprintf(buf, "%s %s", CFG->language[LNG_Phone], *entryp->phone ? entryp->phone : "-Unpublished-");
  strrjust(strsetsz(buf, gvid.numcols-strlen(line1)-1));
  strcat(line1, buf);
  strcat(line1, " ");

  sprintf(line2, " %s%s%s",
    entryp->location,
    (*entryp->location ? ", " : ""), entryp->address
  );
  sprintf(buf, "%s%s%s%s",
    entryp->baud,
    (*entryp->baud ? " Bps" : ""),
    ((*entryp->baud and *entryp->flags) ? ", " : ""), entryp->flags
  );
  strrjust(strsetsz(buf, gvid.numcols-strlen(line2)-1));
  strcat(line2, buf);
  strcat(line2, " ");
  nodewin.prints(0,0, wattr, line1);
  nodewin.prints(1,0, wattr, line2);

  gstring nlname;
  if(NL->index_name() and NL->nodelist_name())
    nlname = gstrf(" %s / %s ", NL->index_name(), NL->nodelist_name());
  else if(NL->index_name())
    nlname = gstrf(" %s ", NL->index_name());
  if(nlname)
    listwin.title(nlname.c_str(), tattr, TCENTER|TBOTTOM);
}


//  ------------------------------------------------------------------

void NodelistBrowser::InitDisplay() {

  sprintf(titlet, CFG->language[LNG_Lookup], user_maybe);
  listwin.title(titlet, tattr, TCENTER);

  Center();
}


//  ------------------------------------------------------------------

void NodelistBrowser::DisplayBar() {

  listwin.prints(pos-1, 0, sattr, liststr[pos-1]);
}


//  ------------------------------------------------------------------

void NodelistBrowser::DisplayLine(int line) {

  if(line == -1)
    line = pos;
  listwin.prints(line-1, 0, wattr, liststr[line-1]);
}


//  ------------------------------------------------------------------

void NodelistBrowser::BuildListString(int line) {

  int x1 = (gvid.numcols-80)/3;
  int x2 = x1;
  int x3 = (gvid.numcols-80) - (x1+x2);

  ftn_nodelist_entry* entryp = entries + (line - 1);
  *entryp = NL->entry();

  if(NL->browsing_names())
    sprintf(liststr[line-1], " %-*.*s %-*.*s %-*.*s ", 24+x1, 24+x1, entryp->name, 21+x2, 21+x2, entryp->address, 29+x3, 29+x3, entryp->system);
  else
    sprintf(liststr[line-1], " %-*.*s %-*.*s %-*.*s ", 21+x2, 21+x2, entryp->address, 24+x1, 24+x1, entryp->name, 29+x3, 29+x3, entryp->system);
}


//  ------------------------------------------------------------------

void NodelistBrowser::DisplayPage() {

  w_info(CFG->language[LNG_Wait]);

  int line = 0;
  while(line < maxpos)
    entries[line++].addr.net = 0;

  NL->push_state();

  // Get entries above and including current entry
  line = pos;
  do {
   BuildListString(line--);
  } while(line and NL->previous());

  NL->pop_state();
  NL->push_state();

  // Get entries below current entry
  line = pos + 1;
  while((line <= maxpos) and NL->next())
    BuildListString(line++);

  NL->pop_state();

  w_info(NULL);

  line = 1;
  char linebuf[200];
  linebuf[gvid.numcols-2] = NUL;

  // Display blank lines if necessary
  memset(linebuf, ' ', gvid.numcols-2);
  while(line <= maxpos) {
    if(entries[line-1].addr.net)
      break;
    // Display separator line if necessary
    if(entries[line].addr.net)
      memset(linebuf, _box_table[btype][1], gvid.numcols-2);
    listwin.prints(line-1, 0, wattr, linebuf);
    line++;
  }

  // Display entry lines
  while((line <= maxpos) and entries[line-1].addr.net) {
    if(line == pos)
      DisplayBar();
    else
      DisplayLine(line);
    line++;
  }

  // Display separator line if necessary
  memset(linebuf, _box_table[btype][1], gvid.numcols-2);
  if(line <= maxpos) {
    listwin.prints(line-1, 0, wattr, linebuf);
    line++;
  }

  // Display blank lines if necessary
  memset(linebuf, ' ', gvid.numcols-2);
  while(line <= maxpos) {
    listwin.prints(line-1, 0, wattr, linebuf);
    line++;
  }
}


//  ------------------------------------------------------------------

void NodelistBrowser::Home() {

  pos = 1;
  NL->first();
  DisplayPage();
}


//  ------------------------------------------------------------------

void NodelistBrowser::End() {

  pos = maxpos;
  NL->last();
  DisplayPage();
}


//  ------------------------------------------------------------------

void NodelistBrowser::ScrollUp() {

  memmove(entries, entries+1, (maxpos-1)*sizeof(ftn_nodelist_entry));
  memmove(liststr, liststr+1, (maxpos-1)*sizeof(ListStr));
  BuildListString(maxpos);
  listwin.scroll_up();
}


//  ------------------------------------------------------------------

void NodelistBrowser::ScrollDown() {

  memmove(entries+1, entries, (maxpos-1)*sizeof(ftn_nodelist_entry));
  memmove(liststr+1, liststr, (maxpos-1)*sizeof(ListStr));
  BuildListString(1);
  listwin.scroll_down();
}


//  ------------------------------------------------------------------

void NodelistBrowser::Up() {

  if(NL->previous()) {
    if(pos > 1) {
      DisplayLine();
      pos--;
    }
    else {
      DisplayLine();
      ScrollDown();
    }
  }
  else if(listwrap) {
    End();
  }
}


//  ------------------------------------------------------------------

void NodelistBrowser::Down() {

  if(NL->next()) {
    if(pos < maxpos) {
      DisplayLine();
      pos++;
    }
    else {
      DisplayLine();
      ScrollUp();
    }
  }
  else if(listwrap) {
    Home();
  }
}


//  ------------------------------------------------------------------

void NodelistBrowser::PageUp() {

  if(pos > 1) {
    DisplayLine();
    do {
      if(not NL->previous())
        break;
    } while(--pos > 1);
  }
  else {
    int count = 1;
    while(count < maxpos) {
      if(not NL->previous())
        break;
      count++;
    }
    DisplayPage();
  }
}


//  ------------------------------------------------------------------

void NodelistBrowser::PageDown() {

  if(pos < maxpos) {
    DisplayLine();
    do {
      if(not NL->next())
        break;
    } while(++pos < maxpos);
  }
  else {
    int count = 1;
    while(count < maxpos) {
      if(not NL->next())
        break;
      count++;
    }
    DisplayPage();
  }
}


//  ------------------------------------------------------------------

void NodelistBrowser::Center() {

  switch(CFG->displistcursor) {
    case LIST_TOP:
      pos = 1;
      break;
    case LIST_NEARTOP:
      pos = (maxpos / 4) + 1;
      break;
    case LIST_MIDDLE:
      pos = (maxpos / 2) + 1;
      break;
    case LIST_NEARBOTTOM:
      pos = (3 * (maxpos / 4)) + 1;
      break;
    case LIST_BOTTOM:
      pos = maxpos;
      break;
  }
  DisplayPage();
}


//  ------------------------------------------------------------------

int NodelistBrowser::Run() {

  int keyok;
  Key newkey;
  Key keycode;

  gmou.HideCursor();

  Open();
  if(helpcat)
    whelppcat(helpcat);

  do {

    listwin.move_cursor(pos-1, 0);
    DisplayBar();
    AfterCursor();
    gmou.ShowCursor();
    keyok = true;

    do {
      newkey = keycode = getxchtick();
      if(newkey == Key_Tick)
        DoKey(keycode);
    } while(newkey == Key_Tick);

    do {
      newkey = 0;
      gmou.HideCursor();
      switch(keycode) {
        case Key_Up:
          BeforeCursor();
          Up();
          break;
        case Key_Dwn:
          BeforeCursor();
          Down();
          break;
        case Key_PgUp:
          BeforeCursor();
          PageUp();
          break;
        case Key_PgDn:
          BeforeCursor();
          PageDown();
          break;
        case Key_Home:
          BeforeCursor();
          Home();
          break;
        case Key_End:
          BeforeCursor();
          End();
          break;
        default:
          keyok = DoKey(keycode);
          newkey = keycode;
      }
      if(newkey)
        keycode = newkey;
    } while(newkey);
  } while(keyok);

  gmou.HideCursor();

  if(helpcat)
    whelpop();
  Close();

  gmou.ShowCursor();

  return aborted;
}


//  ------------------------------------------------------------------

int NodelistBrowser::DoKey(Key& keycode) {

  Key key = keycode;
  keycode = 0;
  if(key < KK_Commands) {
    Key tmpkey = key_tolower(key);
    Key kk = SearchKey(tmpkey, NodeKey, NodeKeys);
    if(kk)
      key = kk;
  }

  switch(key) {
    case KK_NodeAskExit:
      {
        GMenuQuit MenuQuit;
        if(MenuQuit.Run())
          keycode = KK_NodeQuitNow;
      }
      break;

    case KK_NodeQuitNow:
      gkbd.quitall = YES;
      if(gkbd.kbuf == NULL)
        kbput(Key_Esc);
      // Drop through

    case KK_NodeAbort:
      newmaybe = NO;
      aborted = YES;
      // Drop Through

    case KK_NodeSelect:
      if(newmaybe) {
        newmaybe = NO;
        firstkey = YES;
        ftn_addr matchaddr;
        matchaddr.set_all(0xFFFF);
        matchaddr.set(user_maybe);
        int gotzone = false;
        int gotnet = false;
        int gotnode = false;
        int gotpoint = false;
        if(matchaddr.zone != 0xFFFF)
          gotzone = true;
        else
          matchaddr.zone = 0;
        if(matchaddr.net != 0xFFFF)
          gotnet = true;
        else
          matchaddr.net = 0;
        if(matchaddr.node != 0xFFFF)
          gotnode = true;
        else
          matchaddr.node = 0;
        if(matchaddr.point != 0xFFFF)
          gotpoint = true;
        else
          matchaddr.point = 0;
        if(gotzone or gotnet or gotnode or gotpoint) {
          if(not gotzone)
            matchaddr.zone = area->aka().zone;
          if(not gotnet)
            matchaddr.net = area->aka().net;
          if(gotpoint and not gotnode)
            matchaddr.node = area->aka().node;
          if(matchaddr.point >= GFTN_FIRST)
            matchaddr.point = 0;
          if(matchaddr.node >= GFTN_FIRST)
            matchaddr.node = 0;
          if(matchaddr.net >= GFTN_FIRST)
            matchaddr.net = 0;
          if(matchaddr.zone >= GFTN_FIRST)
            matchaddr.zone = 0;
        }

        if(matchaddr.net)
          NL->find(matchaddr);
        else
          NL->find(user_maybe);
        InitDisplay();
      }
      else {
        BeforeCursor();
        return false;
      }
      break;

    case Key_Tab:
      if(NL->browsing_names() and NL->can_browse_address()) {
        strcpy(user_maybe, NL->address());
        NL->find(NL->addrs());
      }
      else if(NL->browsing_addresses() and NL->can_browse_name()) {
        strcpy(user_maybe, NL->name());
        NL->find(NL->name());
      }
      user_fuzidx = strlen(user_maybe);
      InitDisplay();
      break;

    case KK_NodeGotoPrev:
      keycode = Key_Up;
      firstkey = YES;
      break;

    case KK_NodeGotoNext:
      keycode = Key_Dwn;
      firstkey = YES;
      break;

    case KK_NodeGotoFirst:
      keycode = Key_Home;
      firstkey = YES;
      break;

    case KK_NodeGotoLast:
      keycode = Key_End;
      firstkey = YES;
      break;

    case KK_NodeDosShell:
      ShellToDos(getenv("COMSPEC"), CFG->language[LNG_DOS_Shell], LGREY|_BLACK, YES, true, false, area);
      break;

    case Key_Tick:
      CheckTick(KK_NodeQuitNow);
      break;

    case KK_NodeUndefine:
      break;

    default:
      if(not PlayMacro(key, KT_N)) {
        int n = key & 0xFF;
        if((user_fuzidx < 41) or (key == Key_BS)) {
          if(firstkey) {
            if((key != Key_BS) and not isspace(n)) {
              *user_maybe = 0;
              user_fuzidx = 0;
            }
          }
          firstkey = NO;
          if((n >= ' ') or (key == Key_BS)) {
            newmaybe = YES;
            if(key != Key_BS)
              user_maybe[user_fuzidx++] = (char)n;
            else if(user_fuzidx)
              user_maybe[--user_fuzidx] = 0;
            user_maybe[user_fuzidx] = 0;
            sprintf(titlet, CFG->language[LNG_Lookup], user_maybe);
            listwin.title(titlet, tattr, TCENTER);
          }
        }
      }
  }

  return true;
}


//  ------------------------------------------------------------------

static int browse_nodelist(GMsg& msg, const char* title, int topline, gactivearea* area) {

  w_info(NULL);

  NodelistBrowser* browser = new NodelistBrowser;
  throw_new(browser);

  if(topline == 0)
    ChgAttrs(NO, msg, area);   // Close the attributes menu

  browser->area = area;
  browser->btype = W_BMENU;
  browser->battr = C_MENUB;
  browser->wattr = C_MENUW;
  browser->sattr = C_MENUS;
  browser->tattr = C_MENUT;
  browser->hattr = C_MENUQ;
  browser->sbattr = C_MENUPB;
  browser->helpcat = H_NodelistBrowser;
  browser->ypos = topline;
  browser->xpos = 0;
  browser->ylen = gvid.numrows - browser->ypos - 1;
  browser->xlen = gvid.numcols;
  browser->listwrap = CFG->displistwrap;
  browser->firstkey = true;
  browser->newmaybe = false;
  browser->aborted = false;
  strcpy(browser->user_maybe, title);
  browser->user_fuzidx = strlen(browser->user_maybe);

  int aborted = browser->Run();
  if(not aborted)
    entry = browser->entries[browser->pos-1];

  delete browser;

  if(topline == 0)
    ChgAttrs(true, msg, area);  // Turn on the attributes menu again

  return not aborted;
}


//  ------------------------------------------------------------------

void Lookup(GMsg& msg, gftnaddrs& addr, gstring& name, int topline, char* status, gactivearea* area) {

  char* ptr;
  register int n;
  char buf[256], buf1[256];
  int found=false;
  int robotchk;
  int dolookup=true;
  int namelookup = true;
  ftn_addr matchaddr;
  int exactmatch = true;
  gstring tmpname;

  name.trim();
  if(name.empty()) {
    matchaddr = area->aka();
    namelookup = false;
    exactmatch = false;
  }
  tmpname = name;

  if(CFG->addressmacro.size()) {
    strcpy(buf, name.c_str());
    for(n=0; n<CFG->addressmacro.size(); n++) {
      address_macro amac = CFG->addressmacro[n];
      if(name.ieql(amac.macro)) {
        strcpy(buf1, amac.name);
        ptr = buf1;
        if(*buf1 == '@') {
          // UUCP/INTERNET addressing
          ptr = strchr(buf1, '/');
          if(ptr) {
            *ptr++ = NUL;
            msg.kludge.replyaddr = ptr;
            msg.destaddr = ptr;
            ptr = buf1+1;
          }
          else {
            msg.kludge.replyaddr = buf1 + 1;
            msg.destaddr = buf1 + 1;
            ptr = strcpy(buf1, "UUCP");
          }
        }
        name = ptr;
        addr = amac.address;
        name.trim();
        if(topline >= 0)
          return;
      }
    }
  }

  // Automatic internet gating
  if(strchr(name.c_str(), '@') and area->internetgate().addr.net) {
    msg.idest = name;
    if(area->internetgate().name.length()) {
      msg.replyaddr = name;
      msg.to = area->internetgate().name;
      name = msg.to;
    }
    addr = area->internetgate().addr;
    namelookup = false;
    if(topline >= 0)
      return;
  }

  // Check if name is really an address
  if(tmpname.length()) {
    matchaddr.set_all(0xFFFF);
    matchaddr.set(name);
    bool gotzone = false;
    bool gotnet = false;
    bool gotnode = false;
    bool gotpoint = false;
    if(matchaddr.zone != 65535u)
      gotzone = true;
    else
      matchaddr.zone = 0;
    if(matchaddr.net != 65535u)
      gotnet = true;
    else
      matchaddr.net = 0;
    if(matchaddr.node != 65535u)
      gotnode = true;
    else
      matchaddr.node = 0;
    if(matchaddr.point != 65535u)
      gotpoint = true;
    else
      matchaddr.point = 0;
    if(gotzone or gotnet or gotnode or gotpoint) {
      addr = matchaddr;
      if(not gotzone)
        addr.zone = area->aka().zone;
      if(not gotnet)
        addr.net = area->aka().net;
      if(gotpoint and not gotnode)
        addr.node = area->aka().node;
      matchaddr = addr;
      namelookup = false;
      if(matchaddr.point >= GFTN_FIRST) {
        matchaddr.point = 0;
        exactmatch = false;
      }
      if(matchaddr.node >= GFTN_FIRST) {
        matchaddr.node = 0;
        exactmatch = false;
      }
      if(matchaddr.net >= GFTN_FIRST) {
        matchaddr.net = 0;
        exactmatch = false;
      }
      if(matchaddr.zone >= GFTN_FIRST) {
        matchaddr.zone = 0;
        exactmatch = false;
      }
    }
  }

  if(namelookup) {
    const char* ptr = strrchr(name.c_str(), ' ');
    if(ptr) {
      addr = ptr + 1;
      if(addr.net) {
        // Address was given
        name.erase(ptr);
        name.trim();
        if(topline >= 0)
          return;
      }
    }
  }

  robotchk = NO;
  if(name.iinc("Sysop") or area->whoto().ieql(name))
    robotchk = true;
  else {
    for(n=0; n<CFG->robotname.size(); n++) {
      if(name.iinc(CFG->robotname[n])) {
        robotchk = true;
        break;
      }
    }
  }

  if(robotchk) {
    addr = 0;
    if(topline >= 0)
      return;
  }

  if(topline >= 0) {
    dolookup = false;
    if(area->isnet() and CFG->lookupnet)
      dolookup = true;
    else if(area->isecho() and CFG->lookupecho)
      dolookup = true;
    else if(CFG->lookuplocal)
      dolookup = true;
  }

  if(dolookup) {

    lookupv7 = lookupfd = lookupfu = false;

    if(*CFG->nodepathv7) {
      NL = new ftn_version7_nodelist_index;
      throw_new(NL);
      NL->set_path(CFG->nodepathv7);
      lookupv7 = true;
    }
    else if(*CFG->nodepathfd) {
      NL = new ftn_frontdoor_nodelist_index;
      throw_new(NL);
      NL->set_path(CFG->nodepathfd);
      lookupfd = true;
    }
    else if(*CFG->fidouserlist) {
      NL = new ftn_fidouser_nodelist_index;
      throw_new(NL);
      NL->set_path(CFG->fidouserlist);
      lookupfu = true;
    }
    else {
      CheckNodelists();
      NL = new ftn_golded_nodelist_index;
      throw_new(NL);
      NL->set_path(CFG->nodepath);
      lookupge = true;
    }

    if(NL) {

      if(NL->open()) {

        if(namelookup)
          NL->find(name.c_str());
        else
          NL->find(matchaddr);

        found = exactmatch ? NL->found() : false;
        if(found) {
          NL->push_state();
          if(NL->next())
            if(NL->found())
              found = false;
          NL->pop_state();
        }
        entry = NL->entry();

        if(not found or (topline < 0)) {
          strcpy(buf, information);
          update_statusline(status);
          found = browse_nodelist(msg, tmpname.c_str(), absolute(topline), area);
          update_statusline(buf);
        }
      }
      else {
        w_info(CFG->language[LNG_NoNodelist]);
        waitkeyt(10000);
        w_info(NULL);
      }

      NL->close();
      throw_delete(NL);
    }
  }

  if(found) {
    name = entry.name;
    addr = entry.addr;
  }
  else {
    addr = 0;
  }

  if(topline >= 0) {
    if(CFG->internetlookup) {
      if(*entry.system and strchr(entry.system, '@') and area->internetgate().addr.net) {
        msg.replyaddr = entry.system;
        msg.idest = entry.system;
        if(area->internetgate().name.length())
          msg.to = area->internetgate().name;
        else if(strlen(entry.system) >= sizeof(ftn_name))
          msg.to = "UUCP";
        else {
          msg.to = entry.system;
          msg.replyaddr = "";
          msg.idest = "";
        }
        name = msg.to;
        addr = area->internetgate().addr;
      }
    }
  }
}


//  ------------------------------------------------------------------
//  Lookup and display info about the node

void LookupNode(GMsg& msg, gstring& name, int what, gactivearea* area) {

  Subj statuslinebak;
  strcpy(statuslinebak, information);

  vcurhide();
  w_info(CFG->language[LNG_Wait]);

  ftn_addr addr;
  char buf[256];
  char* ptr = buf;

  switch(what) {
    case LOOK_DEST:
      if(area->isnet()) {
        addr = msg.dest;
        if(addr.zone == 0)
          addr.zone = area->aka().zone;
        addr.make_string(buf);
      }
      else {
        addr.reset();
        strcpy(buf, msg.to.c_str());
      }
      break;
    case LOOK_ORIG:
      addr = msg.orig;
      if(addr.zone == 0)
        addr.zone = area->aka().zone;
      addr.make_string(buf);
      break;
    case LOOK_NAME:
      addr = msg.orig;
      strcpy(buf, strskip_wht(name.c_str()));
      strtok(buf, " \t\n\r");   // Only lookup first word
      break;
  }
  gstring tmp = ptr;
  Lookup(msg, addr, tmp, -6, CFG->language[LNG_LookupInfo], area);

  update_statusline(statuslinebak);

  w_info(NULL);
}


//  ------------------------------------------------------------------

void CheckNodelists() {

  // Copy of previous timestamp
  static long __FAR oldft = 0;

  // Get timestamp of the .GXL file
  gstring file = AddPath(CFG->nodepath, "GOLDNODE.GXL");
  long ft = GetFiletime(file.c_str());

  // Check nodelists if timestamp changed
  if(ft != oldft) {

    // Keep copy of timestamp for later lookups
    oldft = ft;

    strcpy(NODE->addrindex, AddPath(CFG->nodepath, "GOLDNODE.GXA"));
    strcpy(NODE->nodeindex, AddPath(CFG->nodepath, "GOLDNODE.GXN"));

    Subj statuslinebak;
    strcpy(statuslinebak, information);

    update_statusline(CFG->language[LNG_CheckingNodelists]);

    FILE* fp = fsopen(file.c_str(), "rt", CFG->sharemode);
    if(fp) {

      // Read the list index
      char buf[256];
      NODE->nodelists = 0;
      while(fgets(buf, sizeof(buf), fp)) {
        NODE->nodelist = (Stamp*)throw_realloc(NODE->nodelist, (NODE->nodelists+1)*sizeof(Stamp));
        char* key;
        char* val=buf;
        getkeyval(&key, &val);
        strxcpy(NODE->nodelist[NODE->nodelists].fn, strbtrim(key), sizeof(Path));
        NODE->nodelist[NODE->nodelists].ft = atol(val);
        NODE->nodelists++;
      }
      fclose(fp);

      // Check the files
      if(CFG->nodelistwarn) {
        for(int n=0; n<NODE->nodelists; n++) {
          if(not fexist(NODE->nodelist[n].fn)) {
            w_infof(" %s %s ", CFG->language[LNG_NodelistMissing], NODE->nodelist[n].fn);
            HandleGEvent(EVTT_ATTENTION);
            waitkeyt(10000);
            w_info(NULL);
          }
          else if(GetFiletime(NODE->nodelist[n].fn) != NODE->nodelist[n].ft) {
            w_infof(" %s %s ", CFG->language[LNG_NodelistOutdated], NODE->nodelist[n].fn);
            HandleGEvent(EVTT_ATTENTION);
            waitkeyt(10000);
            w_info(NULL);
          }
        }
      }
    }

    update_statusline(statuslinebak);
  }

  throw_release(NODE->nodelist);
  NODE->nodelists = 0;
}


//  ------------------------------------------------------------------

