
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geplay.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Functions related to sound.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifdef __OS2__
#define INCL_BASE
#include <os2.h>
#endif

#ifdef __WIN32__
#include <windows.h>
#endif


//  ------------------------------------------------------------------

#define GOLD_SOUNDSUPPORT 1

#ifdef GOLD_SOUNDSUPPORT
#include <gsndall.h>
#endif


//  ------------------------------------------------------------------

#ifdef GOLD_SOUNDSUPPORT
gsound* snd = NULL;
#endif


//  ------------------------------------------------------------------

void _stdc ResetSound(void) {

  #ifdef GOLD_SOUNDSUPPORT
  if(snd->is_installed()) {
    while(snd->is_playing()) {
      if(kbxhit()) {
        if(kbxget() == Key_Esc) {
          snd->stop();
          break;
        }
      }
    }
  }
  delete snd;
  snd = NULL;
  #endif
}


//  ------------------------------------------------------------------

void InitSound() {

  // Find out if there are any sound files to be played
  #ifdef GOLD_SOUNDSUPPORT
  for(int n=0; n<CFG->events; n++) {
    if(CFG->event[n].play.type == PLAY_VOC) {
      snd = new gsound;
      if(snd->is_installed()) {
        if(not quiet)
          printf("\n* Soundcard support was successfully initialized.");
        atexit(ResetSound);
      }
      else {
        if(not quiet)
          printf("\n* Soundcard support could not be initialized!");
        ResetSound();
      }
      break;
    }
  }
  #endif
}


//  ------------------------------------------------------------------

void gplay::get_file(const gstring& filename) {

  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  _splitpath(filename, drive, dir, file, ext);

  if(*ext)
    type = PLAY_VOC;
  else {
    type = PLAY_DOS;
    if(strieql(filename, "TheEnd"))
      effectno = SND_THEEND;
    else if(strieql(filename, "GotIt"))
      effectno = SND_GOTIT;
    else if(strieql(filename, "TooBad"))
      effectno = SND_TOOBAD;
    else if(strieql(filename, "ToYou"))
      effectno = SND_TOYOU;
    else if(strieql(filename, "SayBiBi"))
      effectno = SND_SAYBIBI;
    else if(strieql(filename, "SOS"))
      effectno = SND_S_O_S;
    else
      effectno = 0xFFFF;
  }

  file = filename;
}


//  ------------------------------------------------------------------
//  Handle event

int HandleGEvent(uint event) {

  int retval = -1;     // Event could not be handled

  if(event == EVTT_BREAKLOOP) {
    #ifdef GOLD_SOUNDSUPPORT
    if(snd) {
      while(snd->is_playing()) {
        if(kbxhit()) {
          if(kbxget() == Key_Esc) {
            snd->stop();
            break;
          }
        }
      }
    }
    #endif
    return 0;
  }

  if(event == EVTT_STOPVOICE) {
    #ifdef GOLD_SOUNDSUPPORT
    if(snd)
      snd->stop();
    #endif
    return 0;
  }

  if(event == EVTT_REMOVEVOCBUF) {
    #ifdef GOLD_SOUNDSUPPORT
    if(snd) {
      if(not snd->is_playing())
        snd->unload();
    }
    #endif
    return 0;
  }

  if(CFG) {
    for(int n=0; n<CFG->events; n++) {
      if(event == CFG->event[n].type) {
        switch(CFG->event[n].command) {
          case EVTC_PLAY:
            CFG->event[n].play.play();
            retval = 0;   // Event was found and handled
            break;
        }
        break;
      }
    }
  }
  else {
    MakeNoise(SND_S_O_S);
  }

  return retval;
}


//  ------------------------------------------------------------------
//  Make infernal noises

int MakeNoise(int type) {

  #if defined(__MSDOS__)
  register int x;
  register int w = CFG->beepfactor;
  #endif
  register int n;

  if(CFG->beepnoises and not disablesound) {

    switch(type) {

      case SND_SAYBIBI:
        #if defined(__MSDOS__)
        beep();
        #elif defined(__OS2__)
        DosBeep(440, 111);
        #elif defined(__WIN32__)
        Beep(440, 111);
        #endif
        break;

      case SND_TOYOU:
        #if defined(__MSDOS__)
        for(n=100; n<750; n++) {
          sound(n);
          for(x=0;x<w;x++);
        }
        for(; n>100; n--) {
          sound(n);
          for(x=0;x<w;x++);
        }
        nosound();
        #elif defined(__OS2__)
        for(n=100; n<800; n+=200)
          DosBeep(n,1);
        for(; n>100; n-=200)
          DosBeep(n,1);
        #elif defined(__WIN32__)
        for(n=100; n<800; n+=200)
          Beep(n,1);
        for(; n>100; n-=200)
          Beep(n,1);
        #endif
        break;

      case SND_THEEND:
        #if defined(__MSDOS__)
        Pause2(55);
        sound(220);
        Pause2(110);
        sound(110);
        Pause2(165);
        nosound();
        #elif defined(__OS2__)
        DosBeep(220, 111);
        DosBeep(110, 167);
        #elif defined(__WIN32__)
        Beep(220, 111);
        Beep(110, 167);
        #endif
        break;

      case SND_GOTIT:
        #if defined(__MSDOS__)
        Pause2(55);
        sound(110);
        Pause2(55);
        sound(220);
        Pause2(55);
        sound(110);
        Pause2(55);
        sound(220);
        Pause2(110);
        nosound();
        #elif defined(__OS2__)
        DosBeep(110, 56);
        DosBeep(220, 56);
        DosBeep(110, 56);
        DosBeep(220, 111);
        #elif defined(__WIN32__)
        Beep(110, 56);
        Beep(220, 56);
        Beep(110, 56);
        Beep(220, 111);
        #endif
        break;

      case SND_TOOBAD:
        #if defined(__MSDOS__)
        Pause2(55);
        sound(440);
        Pause2(110);
        sound(220);
        Pause2(110);
        sound(110);
        Pause2(165);
        nosound();
        #elif defined(__OS2__)
        DosBeep(440, 111);
        DosBeep(220, 111);
        DosBeep(110, 167);
        #elif defined(__WIN32__)
        Beep(440, 111);
        Beep(220, 111);
        Beep(110, 167);
        #endif
        break;

      case SND_S_O_S:
        #if defined(__MSDOS__)
        Pause2(55);
        sound(1000);
        Pause2(55);
        nosound();
        Pause2(110);
        sound(1000);
        Pause2(55);
        nosound();
        Pause2(110);
        sound(1000);
        Pause2(55);
        nosound();
        Pause2(220);
        sound(1000);
        Pause2(165);
        nosound();
        Pause2(165);
        sound(1000);
        Pause2(165);
        nosound();
        Pause2(165);
        sound(1000);
        Pause2(165);
        nosound();
        Pause2(220);
        sound(1000);
        Pause2(55);
        nosound();
        Pause2(110);
        sound(1000);
        Pause2(55);
        nosound();
        Pause2(110);
        sound(1000);
        Pause2(55);
        nosound();
        Pause2(110);
        #elif defined(__OS2__)
        DosBeep(1000, 56);
        DosSleep(111);
        DosBeep(1000, 56);
        DosSleep(111);
        DosBeep(1000, 56);
        DosSleep(222);
        DosBeep(1000, 167);
        DosSleep(167);
        DosBeep(1000, 167);
        DosSleep(167);
        DosBeep(1000, 167);
        DosSleep(222);
        DosBeep(1000, 56);
        DosSleep(111);
        DosBeep(1000, 56);
        DosSleep(111);
        DosBeep(1000, 56);
        #elif defined(__WIN32__)
        Beep(1000, 56);
        Sleep(111);
        Beep(1000, 56);
        Sleep(111);
        Beep(1000, 56);
        Sleep(222);
        Beep(1000, 167);
        Sleep(167);
        Beep(1000, 167);
        Sleep(167);
        Beep(1000, 167);
        Sleep(222);
        Beep(1000, 56);
        Sleep(111);
        Beep(1000, 56);
        Sleep(111);
        Beep(1000, 56);
        #endif
        break;
    }
  }
  return(type);
}


//  ------------------------------------------------------------------
//  Play a file

int gplay::play() const {

  switch(type) {

    case PLAY_DOS:
      MakeNoise(effectno);
      break;

    case PLAY_VOC:
      if(not disablesound) {
        #ifdef GOLD_SOUNDSUPPORT
        if(snd) {
          snd->load(AddPath(CFG->soundpath, file));
          snd->play();
        }
        #endif
      }
      break;
  }

  return 0;
}


//  ------------------------------------------------------------------
//  Say a simple bibi

void SayBibi() {

  if(CFG->beepnoises)
    HandleGEvent(EVTT_SAYBIBI);
}


//  ------------------------------------------------------------------

