
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geplay.h,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------


#ifndef __GEPLAY_H
#define __GEPLAY_H


//  ------------------------------------------------------------------
//  Sound effect defines

const int SND_THEEND  = 0;
const int SND_GOTIT   = 1;
const int SND_TOOBAD  = 2;
const int SND_TOYOU   = 3;
const int SND_S_O_S   = 4;
const int SND_SAYBIBI = 5;


//  ------------------------------------------------------------------
//  Event command "Play". Types and structure

const byte PLAY_NO  =  0;   // No sound
const byte PLAY_DOS =  1;   // DOS beeping and effects
const byte PLAY_VOC =  2;   // Sound file

class gplay {

public:

  byte    type;
  int     effectno;   // For the DOS sound effects SND_xxx
  gstring file;
  int     play() const;

  gplay& operator=(const gplay& other) { type=other.type; effectno=other.effectno; file=other.file; return *this; }

  void get_file(const gstring& filename);

  friend gfile& operator<<(gfile& fp, const gplay& a);
  friend gfile& operator>>(gfile& fp, gplay& a);

};


//  ------------------------------------------------------------------
//  Event types (CRC16 values), commands and structure

const word EVTT_AREALIST      = 0x445D;
const word EVTT_ASKYESNO      = 0xB26C;
const word EVTT_ATTENTION     = 0x0F21;
const word EVTT_BLANKEROFF    = 0x16EF;
const word EVTT_BLANKERON     = 0x819F;
const word EVTT_DOSSHELL      = 0x2111;
const word EVTT_EDITCOMMENT   = 0x662C;
const word EVTT_ENDOFMSGS     = 0x95BC;
const word EVTT_ERRORFATAL    = 0xE9C1;
const word EVTT_EXIT          = 0x6090;
const word EVTT_JOBDONE       = 0xA81F;
const word EVTT_JOBFAILED     = 0xCC9A;
const word EVTT_MSGCOMMENT    = 0x0761;
const word EVTT_MSGCOPYING    = 0x087B;
const word EVTT_MSGDELETING   = 0x1AFF;
const word EVTT_MSGFROMYOU    = 0xBACB;
const word EVTT_MSGISLOCAL    = 0x2BD6;
const word EVTT_MSGISTWIT     = 0xC466;
const word EVTT_MSGLIST       = 0x84D0;
const word EVTT_MSGMOVING     = 0xAA1B;
const word EVTT_MSGNEW        = 0x2882;
const word EVTT_MSGPRINTING   = 0x29E0;
const word EVTT_MSGREPLY      = 0x3439;
const word EVTT_MSGTOYOU      = 0xFB77;
const word EVTT_MSGWRITING    = 0x0CA8;
const word EVTT_NODEBROWSE    = 0xB6B5;
const word EVTT_SAYBIBI       = 0xD39D;
const word EVTT_SEARCHFAILED  = 0xEB54;
const word EVTT_SEARCHSUCCESS = 0x2EB2;
const word EVTT_STARTUP       = 0x494B;
const word EVTT_TIMEOUT       = 0x4065;

const word EVTT_BREAKLOOP     = 0xFFFF;
const word EVTT_STOPVOICE     = 0xFFFE;
const word EVTT_REMOVEVOCBUF  = 0xFFFD;
const word EVTT_INITCARD      = 0xFFFC;

const int EVTC_DO_NOTHING    = 0;
const int EVTC_PLAY          = 1;


//  ------------------------------------------------------------------

class gevent {

public:

  gevent& operator=(const gevent& o) { type=o.type; command=o.command; play=o.play; return *this; }

  word  type;       // EVTT crc value
  int   command;    // EVTC value
  gplay play;
};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

