
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geprot.h,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Collective header for GE*.CPP source files.
//  ------------------------------------------------------------------


#ifndef __GEHDRS_H
#define __GEHDRS_H


//  ------------------------------------------------------------------
//  GCAREA

const char* AddStamp(const gstring& file, int filecheck=false);
void FixEchoid(char* echoid, int type);


//  ------------------------------------------------------------------
//  GCCFGG

void CfgAddress(char* value);
void CfgAddressmacro(char* value);
void CfgEvent(char* value);
void CfgFrqext(char* value);
void CfgRobotname(char* value);
void CfgUsername(char* value);

int  SwitchCfgAtoG(word crc, char* val);
int  SwitchCfgHtoZ(word crc, char* val);

int  ReadGoldCfg(int force);
int  ReadGoldedCfg(int& force);
void WriteGoldGed();

void InstallDetect(char* path);
int  InstallFinish();

bool CheckStamp(const gstamp& fntm);


//  ------------------------------------------------------------------
//  GCKEYS

int  ReadKeysCfg(int force);
void KeyCmdAdd(Key cmd, Key val, int type);
void RunMacro(gmacro* m);


//  ------------------------------------------------------------------
//  GCLANG

void LangInit();
int  ReadLangCfg(int force);
void LoadLanguage(const char* file);


//  ------------------------------------------------------------------
//  GCMISC

void HelpInit(gfile& fp, long offset);
int  ReadHelpCfg(int force);
int  ReadRandCfg(int force);
int  GetColor(char* ink);
int  MakeColor(char* colors);
void GetColors(char* value);
void ReadXlatTables();


//  ------------------------------------------------------------------
//  GEAREA

int  AreaPick(char* title, int wpos, int* idx);
void a_closeall();
void GetAreaconf();

void AreaInit();


//  ------------------------------------------------------------------
//  GECREG

void CheckReg();


//  ------------------------------------------------------------------
//  GECTRL

void replace_controllines(int mode, GMsg& msg, gactivearea* area);


//  ------------------------------------------------------------------
//  GEDOIT

void CmfMsgs(GMsg& msg, gactivearea* area);
void LoadText(GMsg& msg, const char* textfile);
void SaveLines(int mode, const char* savefile, GMsg& msg, gactivearea* area);
void WriteMsg(GMsg& msg, gactivearea* area);
void DelMsg(GMsg& msg, int direction, gactivearea* area);


//  ------------------------------------------------------------------
//  GEDOSS

void _stdc Cleanup(void);
int ShellToDos(char* command, char* message, int cls, int cursor, int swap=YES, int pause=NO, gactivearea* area=NULL, bool wait=true);


//  ------------------------------------------------------------------
//  GEEDIT

int  EditMsg(int mode, int& position, GMsg& msg, gactivearea* area);
void FreePastebuf();


//  ------------------------------------------------------------------
//  GEFILE

void CreateAutoMsg(GMsg& msg, gactivearea* area);
void CreateFileMsgs(int mode, GMsg& msg, gactivearea* area);
//void FileSelect(char* title, gfilespec* fspec, gactivearea* area);


//  ------------------------------------------------------------------
//  GEGLOB

void CfgInit();
void CfgInit2();
void StyleCodeHighlight(const char* text, int row, int col, VidPutStrCP fp);
int  PlayMacro(Key key, int type);


//  ------------------------------------------------------------------
//  GEHDRS

char* strconv(char* str, char* conv);
void build_header_display(GMsg& msg, header_display& display, int width, gactivearea* area);


//  ------------------------------------------------------------------
//  GEINIT

void Initialize(int argc, char* argv[]);


//  ------------------------------------------------------------------
//  GEMENU

void DispHeadAttrs(GMsg& msg);
void AskAttributes(GMsg& msg, gactivearea* area);
void ChgAttrs(int mode, GMsg& msg, gactivearea* area);
int  ChangeAka(gactivearea* area);
int  ChangeTagline(gactivearea* area);
int  ChangeOrigin(gactivearea* area);
int  ChangeTemplate(gactivearea* area);
int  ChangeUsername(gactivearea* area);
int  ChangeXlatImport(gactivearea* area);
int  ChangeMood(gactivearea* area);


//  ------------------------------------------------------------------
//  GEMLST

uint MsgBrowser(GMsg& msg);


//  ------------------------------------------------------------------
//  GEMRKS

int NextMarkedmsg(int direction, GMsg& msg, gactivearea* area);


//  ------------------------------------------------------------------
//  GEMSGS

void     TokenXlat(int mode, gstring& input, GMsg& msg, GMsg& oldmsg, gactivearea* destarea, gactivearea* origarea=NULL);
void     TokenXlat(int mode, char* input, GMsg& msg, GMsg& oldmsg, gactivearea* destarea, gactivearea* origarea=NULL);
void     Rot13(GMsg& msg);
int      DoCarboncopy(GMsg& msg, gactivearea* area);
int      FindString(GMsg& msg, const char* prompt, int what, gactivearea* area);
gstring& ParseInternetAddr(const gstring& str, gstring& name, gstring& addr);
gstring& ParseInternetAddr(const char* str, gstring& name, gstring& addr);


//  ------------------------------------------------------------------
//  GENODE

void Lookup(GMsg& msg, gftnaddrs& addr, gstring& name, int topline, char* status, gactivearea* area);
void LookupNode(GMsg& msg, gstring& name, int what, gactivearea* area);
void CheckNodelists();


//  ------------------------------------------------------------------
//  GEPLAY

void InitSound();
void FileToGPlay(gplay& playbuf, const gstring& filename);
int  HandleGEvent(uint event);
int  MakeNoise(int type);
void SayBibi();


//  ------------------------------------------------------------------
//  GEPOST

void CheckSubject(GMsg& msg, gstring& subj, gactivearea* area);
void MakeMsg(int mode, GMsg& oldmsg, gactivearea* destarea, gactivearea* origarea=NULL);
int  EditHeaderinfo(int mode, GMsgHeaderView &view);
void do_zonegating(GMsg& msg, gactivearea* area);
void do_fakenets(GMsg& msg, gactivearea* area);


//  ------------------------------------------------------------------
//  GEQWKS

int ImportQWK();
int ExportQWK();


//  ------------------------------------------------------------------
//  GERAND

gstring& GetRandomLine(gstring& str, const char* filename);


//  ------------------------------------------------------------------
//  GEREAD

void         TwitDeleteMsg(GMsg& msg, int direction, gactivearea* area);
gstring&     GetQuotebufName(gstring& quotebuf, gactivearea* area);
int          MsgIsTwit(GMsg& msg, bool& istwitto, bool& istwitsubj, gactivearea* area);
ulong        MsgHasReplies(GMsg& msg);
Key          ViewMessage(int istwit=NOT_TWIT);
int          LoadMessage(GMsg& msg, int margin, gactivearea* area);
void         PrintHeader(GMsg& msg, int fh, int width, gactivearea* area);
gactivearea* NextArea(GMsg& msg, int direction, gactivearea* area);
gactivearea* PrevArea(GMsg& msg, int direction, gactivearea* area);
void         Reader();
int          ExternUtil(GMsg& msg, int utilno, gactivearea* area);
void         ExternUtilMenu();
uint         MoveLastread(int direction, gactivearea* area);
void         ChangeAttributes(GMsg& msg, gactivearea* area);
void         ChangeMsg(GMsg& msg, gactivearea* area);
void         CommentMsg(GMsg& msg, gactivearea* area);
void         ConfirmMsg(GMsg& msg, gactivearea* area);
void         CopyMoveForward(GMsg& msg, gactivearea* area);
void         DecMargin();
void         DosShell(gactivearea* area);
void         ExitAsk();
void         FileRequest(GMsg& msg, gactivearea* area);
void         FindAll(GMsg& msg, int& topline, int& keyok, gactivearea* area);
void         FindHdr(GMsg& msg, int& topline, int& keyok, gactivearea* area);
void         GotoBookMark(GMsg& msg, gactivearea* area);
void         GotoFirstMsg(int& direction, gactivearea* area);
void         GotoLastMsg(int& direction, gactivearea* area);
void         GotoMsgno(GMsg& msg, gactivearea* area);
gactivearea* GotoNextMsg(GMsg& msg, int& direction, gactivearea* area);
gactivearea* GotoPrevMsg(GMsg& msg, int& direction, gactivearea* area);
void         GotoNextUnread(GMsg& msg, int& direction, gactivearea* area);
void         GotoPrevUnread(GMsg& msg, int& direction, gactivearea* area);
void         GotoReplies(GMsg& msg, int& direction, gactivearea* area);
void         GotoReply1st(GMsg& msg, int& direction, gactivearea* area);
void         GotoReplyNext(GMsg& msg, int& direction, gactivearea* area);
void         GotoReplyPrev(GMsg& msg, int& direction, gactivearea* area);
void         IncMargin();
void         MakeUserList(gactivearea* area);
void         MarkingOptions(GMsg& msg, gactivearea* area);
void         MessageBrowse(GMsg& msg, gactivearea* area);
gactivearea* NewArea(gactivearea* area, int areaid=-1);
void         NewMsg(GMsg& msg, gactivearea* area);
void         OtherAreaQuoteMsg(GMsg& msg, gactivearea* area, bool allow_pick=true);
void         OtherAreaCommentMsg(GMsg& msg, gactivearea* area, bool allow_pick=true);
void         QuitNow();
void         QuoteBuf(GMsg& msg, gactivearea* area);
void         QuoteMsg(GMsg& msg, gactivearea* area);
void         RenumberArea(gactivearea* area);
void         ReplyMsg(GMsg& msg, gactivearea* area);
void         ToggleBookMark(GMsg& msg, gactivearea* area);
void         ToggleHexdump(gactivearea* area);
void         ToggleHidden(GMsg& msg, gactivearea* area);
void         ToggleHiddKlud(GMsg& msg, gactivearea* area);
void         ToggleKludge(GMsg& msg, gactivearea* area);
void         ToggleMark(GMsg& msg, gactivearea* area);
void         ToggleMarkRead(GMsg& msg, int direction, gactivearea* area);
void         TogglePageBar();
void         ToggleQuote(GMsg& msg, gactivearea* area);
void         ToggleRealMsgno(GMsg& msg, gactivearea* area);
void         ToggleROT13(GMsg& msg, gactivearea* area);
void         ToggleTwits(gactivearea* area);
void         ToggleXlat();
void         TouchNetscan(bool popup=true);
void         UUDecode(GMsg& msg, gactivearea* area);
void         TouchSemaphore();


//  ------------------------------------------------------------------
//  GESCAN

void CheckSemaphores();


//  ------------------------------------------------------------------
//  GESOUP

char* CvtMSGIDtoMessageID(char* midbuf, char* msgid, ftn_addr* addr, char* domain, char* from, char* to, char* subject, Tm* datetime);
char* CvtMessageIDtoMSGID(const char* mptr, char* msgidbuf, const char* echoid, char* kludge);

int ImportSOUP();
int ExportSOUP();


//  ------------------------------------------------------------------
//  GETPLS

void CookieIndex(char* textfile, char* indexfile);
bool TemplateToLines(int mode, GMsg& msg, GMsg& oldmsg, gactivearea* destarea, gactivearea* origarea, int width, bool set_position=false);


//  ------------------------------------------------------------------
//  GEUTIL

void  ScreenBlankIdle();
int   AkaMatch(gstring& match, const gstring& addr);
void  call_help();
void  CheckTick(Key quitkey);
int   _stdc CmdKeyCmp(const gcmdkey* a, const gcmdkey* b);
char* CvtName(char* inp);
int   edit_string(gstring& buf, int buf_size, char* title, int helpcat);
int   GetAkaNo(const gftnaddrs& aka);
void  maketitle();
void  ProgMeter(int mode, int xx, int yy, long len, long barlen, int attr, long pos, long size);
int   ReadCfg(const gstring& cfg, int ignoreunknown=false);
void  ScanMsgTxtForAddr(GMsg& msg);
Key   SearchKey(Key key, gcmdkey* keys, int totkeys);
int   SearchTaglist(Echo* taglist, char* tag);
void  set_title(const char* t, int p, int a);
void  title_shadow();
void  update_statusline(const char* info);
void  _stdc update_statuslinef(const char* format, ...);
void  update_statuslines();
void  w_info(const char* info=NULL);
void  _stdc w_infof(const char* format, ...);
void  w_progress(int mode, int attr, long pos, long size, const char* title);
void  w_shadow();
void  ZonegateIt(ftn_addr& gate, const ftn_addr& orig, const ftn_addr& dest);
char* strtmp(const char* str);


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

