
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geqwks.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  QWK packet handling.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include "gs_qwk.h"


//  ------------------------------------------------------------------

static void ReadGldFile() {

  if(QWK->FoundBBS()) {

    gfile fp;
    Path gldfile;

    QWK->ResetConfNo();

    sprintf(gldfile, "%s%s.GLD", CFG->goldpath, QWK->BbsID());
    fp.fopen(gldfile, "rt");
    if(fp.isopen()) {
      char* key;
      char* val;
      char buf[256];
      while(fp.fgets(buf, sizeof(buf))) {
        val = strtrim(buf);
        getkeyval(&key, &val);
        strtrim(StripQuotes(val));
        if(QWK->FindEcho(val))
          QWK->ConfNo(atoi(key));
      }
      fp.fclose();
    }
  }
}


//  ------------------------------------------------------------------

int ImportQWK() {

  if(not *QWK->ImportPath())
    return 0;

  gfile fp;
  gfile fpb;   // For BBSID.GLD
  Path file;
  Path gldfile;
  char bbsid[9] = {""};

  // Parse the control file
  strcpy(file, AddPath(QWK->ImportPath(), "CONTROL.DAT"));
  fp.fopen(file, "rt");
  if(fp.isopen()) {
    char buf[256];
    int line = 0;
    int confno = 0;
    int confcnt = 0;
    int confnos = 0;
    while(fp.fgets(buf, sizeof(buf))) {

      line++;
      strtrim(buf);

      if((line >= 12) and (confcnt < confnos)) {
        if(line % 2) {
          if(fpb.isopen())
            fpb.printf("%u \"%s\"\n", confno, buf);
          confcnt++;
        }
        else
          confno = atoi(buf);
      }
      else if(line == 5) {
        char* ptr = strchr(buf, ',');
        if(ptr) {
          strxcpy(bbsid, strskip_wht(ptr+1), 9);
          sprintf(gldfile, "%s%s.GLD", CFG->goldpath, bbsid);
          fpb.fopen(gldfile, "wt");
        }
      }
      else if(line == 11)
        confnos = atoi(buf) + 1;
    }
    if(fpb.isopen())
      fpb.fclose();
    fp.fclose();
    remove(file);
  }

  if(strblank(bbsid))
    return 0;

  QWK->FindBBS(bbsid);
  ReadGldFile();

  int imported = 0;

  strcpy(file, AddPath(QWK->ImportPath(), "MESSAGES.DAT"));
  fp.fopen(file, "rb");
  if(fp.isopen()) {

    // Skip past product info header
    fp.fseekset(sizeof(QWKHdr));

    QWKHdr hdr;

    gtag tossed;

    gactivearea* area = NULL;

    GMsg msg;

    // Read each message, header first
    int tosstobadmsgs = false;
    int lastconfno = -1;
    int more;
    do {

      msg.reset();

      memset(&hdr, 0, sizeof(QWKHdr));
      more = 1 == fp.fread(&hdr, sizeof(QWKHdr));
      if(more) {

        char blocks[7];
        strxcpy(blocks, hdr.blocks, 7);
        uint msglen = atoi(blocks);
        if(msglen == 0) {
          //more = false;
          break;
        }
        msglen = (msglen-1)*128;

        if(hdr.confno != lastconfno) {
          lastconfno = hdr.confno;
          if(QWK->FindEcho(hdr.confno)) {
            int areaid = AL.echoid_to_id(QWK->EchoID());
            if(areaid != -1) {
              area->close();
              AL.deactivate(area);
              area = AL.activate(areaid);
              area->open();
              area->lock();
              tosstobadmsgs = false;
            }
          }
          else {
            int areaid = AL.echoid_to_id(QWK->BadMsgs());
            if(areaid != -1) {
              area->close();
              AL.deactivate(area);
              area = AL.activate(areaid);
              area->open();
              area->lock();
              tosstobadmsgs = true;
            }
            else {
              tosstobadmsgs = -1;
              fp.fseek(msglen, SEEK_CUR);
            }
          }
        }

        if(tosstobadmsgs != -1) {

          // Convert header
          msg.area = area;
          msg.orig = msg.oorig = area->aka();
          msg.dest = msg.odest = area->aka();
          msg.attr.pvt(hdr.status == '*' or hdr.status == '+');
          msg.attr.rcv(hdr.status == '+' or hdr.status == '-' or hdr.status == '`' or hdr.status == '^' or hdr.status == '#');
          msg.attr.del(hdr.activestatus == 226);
          msg.by.copyn(hdr.from, 26).trim();
          msg.to.copyn(hdr.to, 26).trim();
          msg.re.copyn(hdr.subject,  26).trim();
          int _year, _month, _day, _hour, _minute;
          sscanf(hdr.date, "%d%*c%d%*c%2d", &_month, &_day, &_year);
          sscanf(hdr.time, "%d%*c%2d", &_hour, &_minute);
          Tm _tm;
          _tm.tm_year  = (_year < 80) ? (_year+100) : _year;
          _tm.tm_mon   = _month - 1;
          _tm.tm_mday  = _day;
          _tm.tm_hour  = _hour;
          _tm.tm_min   = _minute;
          _tm.tm_sec   = 0;
          _tm.tm_isdst = 0;
          msg.date.written = gmktime(&_tm);
          msg.date.arrived = gtime(NULL);

          // Read message text
          char* txtptr = msg.txt = (char*)throw_calloc(1, msglen+128);
          if(tosstobadmsgs) {
            sprintf(msg.txt, "AREA:%s_%u\r", bbsid, hdr.confno);
            txtptr += strlen(msg.txt);
          }
          fp.fread(txtptr, msglen);
          strtrim(txtptr);
          strchg(txtptr, 0xE3, 0x0D);

          imported++;
          tossed += area->areaid();
          update_statuslinef("%s: %u", area->echoid().c_str(), imported);

          msg.text_to_lines(CFG->dispmargin-1);
          if(msg.rfc.message_id.length() or msg.rfc.references.length() or msg.rfc.in_reply_to.length() or msg.rfc.from.length()) {
            char kbuf[256];
            uint txtlen = strlen(msg.txt);
            if(msg.rfc.from.length()) {
              gstring _fromname;
              gstring _fromaddr;
              ParseInternetAddr(msg.rfc.from, _fromname, _fromaddr);
              if(CFG->internetgate.addr.net) {
                char abuf[40];
                sprintf(kbuf, "\x1""REPLYTO %s %s\r""\x1""REPLYADDR %s\r",
                  CFG->internetgate.addr.make_string(abuf),
                  *CFG->internetgate.name ? CFG->internetgate.name : "UUCP",
                  _fromaddr.c_str()
                );
                uint addlen = strlen(kbuf);
                msg.txt = (char*)throw_realloc(msg.txt, txtlen+addlen+1);
                memmove(msg.txt+addlen, msg.txt, txtlen+1);
                memcpy(msg.txt, kbuf, addlen);
                txtlen += addlen;
              }
            }
            if(msg.rfc.references.length()) {
              char* mptr = throw_xstrdup(msg.rfc.references.c_str());
              char* optr = mptr;
              char* sptr = mptr;
              while(*mptr) {
                if(not strnieql(mptr, "<NOMSGID_", 9)) {
                  sptr = strpbrk(mptr, " ,");
                  if(sptr == NULL)
                    sptr = mptr + strlen(mptr);
                  while(((*sptr == ' ') or (*sptr == ',')) and *sptr)
                    *sptr++ = NUL;
                  CvtMessageIDtoMSGID(mptr, kbuf, area->echoid().c_str(), "REPLY");
                  strcat(kbuf, "\r");
                  uint addlen = strlen(kbuf);
                  msg.txt = (char*)throw_realloc(msg.txt, txtlen+addlen+1);
                  memmove(msg.txt+addlen, msg.txt, txtlen+1);
                  memcpy(msg.txt, kbuf, addlen);
                  txtlen += addlen;
                }
                mptr = sptr;
              }
              throw_xfree(optr);
            }
            if(msg.rfc.in_reply_to.length()) {
              if(not msg.rfc.in_reply_to.nieql("<NOMSGID_", 9)) {
                CvtMessageIDtoMSGID(msg.rfc.in_reply_to.c_str(), kbuf, area->echoid().c_str(), "REPLY");
                strcat(kbuf, "\r");
                uint addlen = strlen(kbuf);
                msg.txt = (char*)throw_realloc(msg.txt, txtlen+addlen+1);
                memmove(msg.txt+addlen, msg.txt, txtlen+1);
                memcpy(msg.txt, kbuf, addlen);
                txtlen += addlen;
              }
            }
            if(msg.rfc.message_id.length()) {
              if(not msg.rfc.message_id.nieql("<NOMSGID_", 9)) {
                CvtMessageIDtoMSGID(msg.rfc.message_id.c_str(), kbuf, area->echoid().c_str(), "MSGID");
                strcat(kbuf, "\r");
                uint addlen = strlen(kbuf);
                msg.txt = (char*)throw_realloc(msg.txt, txtlen+addlen+1);
                memmove(msg.txt+addlen, msg.txt, txtlen+1);
                memcpy(msg.txt, kbuf, addlen);
                //txtlen += addlen;
              }
            }
          }

          if(msg.realto.length() and (msg.to.ieql("ALL") or msg.to.empty()))
            msg.to = msg.realto;

          area->savemsg(GMSG_NEW, msg);
        }
      }
    } while(more);

    area->close();
    AL.deactivate(area);

    fp.fclose();
    remove(file);

    if(*QWK->TossLog()) {
      fp.fopen(QWK->TossLog(), "at");
      if(fp.isopen()) {
        for(int n=0; n<tossed.size(); n++) {
          garealistdata* a = AL.id_to_pointer(tossed[n]);
          a->set_changed();
          fp.printf("%s\n", a->echoid().c_str());
        }
        fp.fclose();
      }
    }

    if(imported and *QWK->ReplyLinker()) {
      char buf[256];
      sprintf(buf, CFG->language[LNG_Replylinker], QWK->ReplyLinker());
      ShellToDos(QWK->ReplyLinker(), buf, LGREY|_BLACK, YES);
    }
  }

  return imported;
}


//  ------------------------------------------------------------------

int ExportQwkMsg(GMsg& msg, gfile& fp, int confno, int& pktmsgno, gactivearea* area) {

  // Prepare for Return Receipt Request
  char msg__re[26];
  if(QWK->ReceiptAllowed() and msg.attr.rrq())
    sprintf(msg__re, "RRR%-22.22s", msg.re.c_str());
  else
    strxcpy(msg__re, msg.re.c_str(), 26);

  // Build QWK header
  QWKHdr hdr;
  char buf[26];
  int tolen = msg.to.length();  tolen = minimum_of_two(25,tolen);
  int bylen = msg.by.length();  bylen = minimum_of_two(25,bylen);
  int relen = strlen(msg__re);  relen = minimum_of_two(25,relen);
  memset(&hdr, ' ', sizeof(QWKHdr));
  hdr.status = msg.attr.pvt() ? '*' : ' ';
  itoa(confno, buf, 10);
  memcpy(hdr.msgno, buf, strlen(buf));
  Tm* _tm = glocaltime(&msg.date.written);
  int _year = (_tm->tm_year >= 100) ? (_tm->tm_year-100) : _tm->tm_year;
  sprintf(buf, "%02d-%02d-%02d", _tm->tm_mon+1, _tm->tm_mday, _year);
  memcpy(hdr.date, buf, 8);
  sprintf(buf, "%02d:%02d", _tm->tm_hour, _tm->tm_min);
  memcpy(hdr.time, buf, 5);
  strxcpy(buf, msg.to.c_str(), tolen+1);
  if(not QWK->MixCaseAllowed())
    strupr(buf);
  memcpy(hdr.to, buf, tolen);
  strxcpy(buf, msg.by.c_str(), bylen+1);
  if(not QWK->MixCaseAllowed())
    strupr(buf);
  memcpy(hdr.from, buf, bylen);
  memcpy(hdr.subject, msg__re, relen);
  hdr.activestatus = '\xE1';
  hdr.confno = (word)confno;
  hdr.pktmsgno = (word)++pktmsgno;

  // Write preliminary header
  fp.fwrite(&hdr, sizeof(QWKHdr));

  // Write body
  #if 0
  strchg(msg.txt, 0x0D, 0xE3);
  fp.fwrite(msg.txt, msglen);
  if(endlen) {
    memset(&hdr, ' ', sizeof(QWKHdrT));
    fp.fwrite(&hdr, 128-endlen);
  }
  strchg(msg.txt, 0xE3, 0x0D);
  #endif

  char mbuf[512];
  uint msglen = 0;

  if(msg.header.x_char_esc.length() and msg.header.mime_version.empty()) {
    if(not striinc("MNEMONIC", XLAT->set.charset.export))
      XLAT->load_charset(XLAT->localset, "MNEMONIC");
  }
  else if(striinc("LATIN1QP", msg.charset.c_str())) {
    if(not striinc("LATIN1QP", XLAT->set.charset.export))
      XLAT->load_charset(XLAT->localset, "LATIN1QP");
  }
  else if(striinc("LATIN-1", msg.charset.c_str())) {
    if(not striinc("LATIN-1", XLAT->set.charset.export))
      XLAT->load_charset(XLAT->localset, "LATIN-1");
  }

  char qwkterm = '\xE3';
  //int lineno = 0;

  // Process kludges and write header lines
  for(int n=0; n<msg.linearray.size(); n++) {
    if(msg.linearray.data(n).iskludge()) {
      if(area->isinternet()) {
        if((msg.linearray.data(n).kludge == GKLUD_RFC) or (msg.linearray.data(n).kludge == 0)) {
          XLAT->translate(mbuf, msg.linearray[n]);
          msglen += fp.printf("%s%c", mbuf, qwkterm);
          //lineno++;
        }
        else if(msg.linearray.data(n).iswrapped()) {
          while((n+1 < msg.linearray.size()) and msg.linearray.data(n).iswrapped())
            n++;
        }
      }
      else {
        if(msg.linearray.data(n).iskludge() and QWK->KludgesAllowed()) {
          XLAT->translate(mbuf, msg.linearray[n]);
          msglen += fp.printf("%s%c", mbuf, qwkterm);
          //lineno++;
        }
      }
    }
  }

  // Write blank line after header lines
  if(area->internetrfcbody()) {
    msglen += fp.printf("%c", qwkterm);
    //lineno++;
  }

  // Write all message lines
  for(int i=0; i<msg.linearray.size(); i++) {
    if(not msg.linearray.data(i).iskludge()) {
      XLAT->translate(mbuf, msg.linearray[i]);
      msglen += fp.printf("%s%c", mbuf, qwkterm);
      //lineno++;
    }
  }

  // Calculate blocks
  uint endlen = msglen % 128;
  uint blocks = 1+(msglen/128)+(endlen?1:0);
  itoa(blocks, buf, 10);
  memcpy(hdr.blocks, buf, strlen(buf));

  // Write padding spaces at the end if necessary
  if(endlen) {
    char padding[128];
    memset(padding, ' ', 128);
    fp.fwrite(padding, 128-endlen);
  }

  // Re-write the header
  fp.fseek(-(blocks*128), SEEK_CUR);
  fp.fwrite(&hdr, sizeof(QWKHdr));
  fp.fseek((blocks-1)*128, SEEK_CUR);

  // Mark msg as sent
  msg.attr.snt1();
  msg.attr.scn1();
  msg.attr.uns0();
  area->savehdr(GMSG_UPDATE, msg);

  return 1;
}


//  ------------------------------------------------------------------

int ExportQwkArea(int areaid, gfile& fp, int confno, int& pktmsgno) {

  int exported = 0;

  gactivearea* area = AL.activate(areaid);

  area->open();
  area->lock();

  GMsg msg;

  for(uint n=0; n<area->markset.size(); n++) {
    if(area->loadmsg(msg, area->markset[n], 80)) {
      if(msg.attr.uns()) {
        exported += ExportQwkMsg(msg, fp, confno, pktmsgno, area);
        update_statuslinef("%s: %u", area->echoid().c_str(), exported);
      }
    }
  }

  area->unlock();
  area->close();
  AL.deactivate(area);

  return exported;
}


//  ------------------------------------------------------------------

int ExportQWK() {

  if(not *QWK->ExportPath())
    return 0;

  int exported = 0;

  gfile fp;
  Path scanfile;

  // Get the scan list
  strcpy(scanfile, AddPath(CFG->goldpath, "GOLDQWK.LST"));
  fp.fopen(scanfile, "rt");
  if(fp.isopen()) {
    char buf[256];
    while(fp.fgets(buf, sizeof(buf))) {
      char* ptr = strchr(buf, ' ');
      if(ptr) {
        *ptr++ = NUL;
        int a = AL.echoid_to_number(buf);
        if(a != -1 and AL[a]->isqwk())
          AL[a]->markset.add(atol(ptr));
      }
    }
    fp.fclose();
  }

  // Export from the QWK areas
  if(QWK->FirstBBS()) {
    do {
      ReadGldFile();
      Path replyfile;
      int pktmsgno = 0;
      if(QWK->FirstConf()) {
        sprintf(replyfile, "%s%s.MSG", QWK->ExportPath(), QWK->BbsID());
        fp.fopen(replyfile, "wb");
        if(fp.isopen()) {
          char firstrec[128];
          memset(firstrec, ' ', 128);
          memcpy(firstrec, QWK->BbsID(), strlen(QWK->BbsID()));
          fp.fwrite(firstrec, 128);
          pktmsgno = 0;
        }
        do {
          garealistdata* ap = AL.echoid_to_pointer(QWK->EchoID());
          if(ap->markset.size()) {
            if(QWK->ConfNo() != -1)
              exported += ExportQwkArea(ap->areaid(), fp, QWK->ConfNo(), pktmsgno);
          }
        } while(QWK->NextConf());
      }
      if(fp.isopen()) {
        fp.fclose();
        if(pktmsgno == 0)
          remove(replyfile);
      }
    } while(QWK->NextBBS());
  }

  // Delete the scanfile
  remove(scanfile);

  return exported;
}


//  ------------------------------------------------------------------

Qwk::Qwk() {

  bbs = bbsp = NULL;
  mapp = NULL;
  bbss = 0;
}


//  ------------------------------------------------------------------

Qwk::~Qwk() {

  Reset();
}


//  ------------------------------------------------------------------

void Qwk::Reset() {

  for(int n=0; n<bbss; n++)
    throw_release(bbs[n].map);
  throw_release(bbs);
  bbsp = NULL;
  mapp = NULL;
  bbss = 0;
}


//  ------------------------------------------------------------------

void Qwk::Save(gfile& fp) {

  fp.fwrite(&cfg, sizeof(QwkCfg));
  fp.fwrite(&bbss, sizeof(int));
  fp.fwrite(bbs, sizeof(QwkBbs), bbss);
  bbsp = bbs;
  for(int n=0; n<bbss; n++,bbsp++)
    fp.fwrite(bbsp->map, sizeof(QwkMap), bbsp->maps);
}


//  ------------------------------------------------------------------

void Qwk::Load(gfile& fp) {

  fp.fread(&cfg, sizeof(QwkCfg));
  fp.fread(&bbss, sizeof(int));
  if(bbss) {
    bbs = (QwkBbs*)throw_calloc(bbss, sizeof(QwkBbs));
    fp.fread(bbs, sizeof(QwkBbs), bbss);
    bbsp = bbs;
    int n;
    for(n=0; n<bbss; n++,bbsp++)
      bbsp->map = NULL;
    bbsp = bbs;
    for(n=0; n<bbss; n++,bbsp++) {
      bbsp->map = (QwkMap*)throw_calloc(bbsp->maps, sizeof(QwkMap));
      fp.fread(bbsp->map, sizeof(QwkMap), bbsp->maps);
    }
  }
}


//  ------------------------------------------------------------------

void Qwk::AddBBS(char* bbsid) {

  bbs = (QwkBbs*)throw_realloc(bbs, (bbss+1)*sizeof(QwkBbs));
  bbsp = bbs + bbss++;
  strxcpy(bbsp->bbsid, bbsid, sizeof(bbsp->bbsid));
  bbsp->kludges = false;
  bbsp->mixcase = false;
  bbsp->receipt = false;
  bbsp->maxlines = -1;
  bbsp->map = NULL;
  bbsp->maps = 0;
  mapp = NULL;
}


//  ------------------------------------------------------------------

void Qwk::AddMap(char* bbsid, char* echoid, char* confname, int confno) {

  FindAddBBS(bbsid);
  bbsp->map = (QwkMap*)throw_realloc(bbsp->map, (bbsp->maps+1)*sizeof(QwkMap));
  mapp = bbsp->map + bbsp->maps++;
  strxcpy(mapp->confname, confname, sizeof(mapp->confname));
  strxcpy(mapp->echoid, echoid, sizeof(mapp->echoid));
  mapp->confno = confno;
}


//  ------------------------------------------------------------------

int Qwk::FindBBS(char* bbsid) {

  bbsp = bbs;
  for(int n=0; n<bbss; n++,bbsp++) {
    if(strieql(bbsid, bbsp->bbsid)) {
      mapp = bbsp->map;
      return true;
    }
  }
  mapp = NULL;
  return false;
}


//  ------------------------------------------------------------------

void Qwk::FindAddBBS(char* bbsid) {

  if(not FindBBS(bbsid))
    AddBBS(bbsid);
}


//  ------------------------------------------------------------------

int Qwk::FirstBBS() {

  if(bbs) {
    bbsp = bbs;
    bbsn = 1;
    return true;
  }

  bbsp = NULL;
  bbsn = 0;
  return false;
}


//  ------------------------------------------------------------------

int Qwk::NextBBS() {

  if(bbsp and (bbsn < bbss)) {
    bbsp++;
    bbsn++;
    return true;
  }

  bbsp = NULL;
  bbsn = 0;
  return false;
}


//  ------------------------------------------------------------------

int Qwk::FindConf(char* echoid) {

  if(bbsp) {
    mapp = bbsp->map;
    for(int n=0; n<bbsp->maps; n++,mapp++)
      if(strieql(echoid, mapp->echoid))
        return true;
  }

  mapp = NULL;
  return false;
}


//  ------------------------------------------------------------------

int Qwk::FindEcho(char* confname) {
           
  if(bbsp) {
    mapp = bbsp->map;
    for(int n=0; n<bbsp->maps; n++,mapp++)
      if(strieql(confname, mapp->confname))
        return true;
  }

  mapp = NULL;
  return false;
}


//  ------------------------------------------------------------------

int Qwk::FindEcho(int confno) {

  if(bbsp) {
    mapp = bbsp->map;
    for(int n=0; n<bbsp->maps; n++,mapp++)
      if(confno == mapp->confno)
        return true;
  }

  mapp = NULL;
  return false;
}


//  ------------------------------------------------------------------

int Qwk::FirstConf() {

  if(bbsp and bbsp->map) {
    mapp = bbsp->map;
    mapn = 1;
    return true;
  }

  mapp = NULL;
  mapn = 0;
  return false;
}


//  ------------------------------------------------------------------

int Qwk::NextConf() {

  if(mapp and (mapn < bbsp->maps)) {
    mapp++;
    mapn++;
    return true;
  }

  mapp = NULL;
  mapn = 0;
  return false;
}


//  ------------------------------------------------------------------

char* Qwk::BbsID() {

  return bbsp ? bbsp->bbsid : "";
}


//  ------------------------------------------------------------------

char* Qwk::ConfName() {

  return mapp ? mapp->confname : "";
}


//  ------------------------------------------------------------------

int Qwk::ConfNo(int set) {

  if(mapp) {
    if(set != -2)
      mapp->confno = set;
    return mapp->confno;
  }
  return -1;
}


//  ------------------------------------------------------------------

char* Qwk::EchoID() {

  return mapp ? mapp->echoid : "";
}


//  ------------------------------------------------------------------

int Qwk::KludgesAllowed(int set) {

  if(bbsp) {
    if(set != -1)
      bbsp->kludges = set;
    return bbsp->kludges;
  }

  return false;
}


//  ------------------------------------------------------------------

int Qwk::MixCaseAllowed(int set) {

  if(bbsp) {
    if(set != -1)
      bbsp->mixcase = set;
    return bbsp->mixcase;
  }

  return false;
}


//  ------------------------------------------------------------------

int Qwk::ReceiptAllowed(int set) {

  if(bbsp) {
    if(set != -1)
      bbsp->receipt = set;
    return bbsp->receipt;
  }

  return false;
}


//  ------------------------------------------------------------------

int Qwk::MaxLines(int set) {

  if(bbsp) {
    if(set != -1)
      bbsp->maxlines = set;
    return bbsp->maxlines;
  }

  return 0;
}


//  ------------------------------------------------------------------

void Qwk::ResetConfNo(int set) {

  if(bbsp) {
    mapp = bbsp->map;
    for(int n=0; n<bbsp->maps; n++,mapp++)
      mapp->confno = set;
  }
}


//  ------------------------------------------------------------------

char* Qwk::BadMsgs(char* set) {

  if(set)
    strxcpy(cfg.badmsgs, set, sizeof(Echo));

  return cfg.badmsgs;
}


//  ------------------------------------------------------------------

char* Qwk::ExportPath(char* set) {

  if(set)
    PathCopy(cfg.exportpath, set);

  return cfg.exportpath;
}


//  ------------------------------------------------------------------

char* Qwk::ImportPath(char* set) {

  if(set)
    PathCopy(cfg.importpath, set);

  return cfg.importpath;
}


//  ------------------------------------------------------------------

char* Qwk::ReplyLinker(char* set) {

  if(set)
    strxcpy(cfg.replylinker, set, sizeof(Path));

  return cfg.replylinker;
}


//  ------------------------------------------------------------------

char* Qwk::TossLog(char* set) {

  if(set)
    strxcpy(cfg.tosslog, set, sizeof(Path));

  return cfg.tosslog;
}


//  ------------------------------------------------------------------

