
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geqwks.h,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  QWK Implementation Structures and Classes.
//  ------------------------------------------------------------------


#ifndef __GEQWKS_H
#define __GEQWKS_H


#include <gstrgstr.h>


//  ------------------------------------------------------------------

class gqwkcfg {
  
public:

  gstring badmsgs;
  gstring exportpath;
  gstring importpath;
  gstring replylinker;
  gstring tosslog;

};


//  ------------------------------------------------------------------

class gqwkmap {

public:

  char    confname[21];
  int     confno;
  gstring echoid;

};


//  ------------------------------------------------------------------

class gqwkbbs {

public:

  char bbsid[9];
  bool kludges;  // true if allowed to export kludges
  bool mixcase;  // true if mixed case is allowed
  bool receipt;  // true if return receipts are supported
  int  maxlines;

  gvector<gqwkmap> map;

};


//  ------------------------------------------------------------------

class gqwk {

public:

  gqwkcfg  cfg;    // Global QWK configuration
  gvector<gqwkbbs> bbs;    // Collection of BBS data
  gqwkbbs* bbsp;   // Pointer to current BBS
  int      bbsn;   // Number of current BBS
  gqwkmap* mapp;   // Pointer to current conference map
  int      mapn;   // Number of current conference map

public:

  gqwk();
  ~gqwk();

  void Reset();

  void Save(gfile& fp);
  void Load(gfile& fp);

  void AddBBS(char* bbsid);
  void AddMap(char* bbsid, char* echoid, char* confname, int confno=-1);

  int FindBBS(char* bbsid);
  void FindAddBBS(char* bbsid);
  int FoundBBS() { return !!bbsp; }
  int FirstBBS();
  int NextBBS();

  int FindConf(char* echoid);
  int FindEcho(char* confname);
  int FindEcho(int confno);
  int FoundConf() { return !!mapp; }
  int FirstConf();
  int NextConf();

  char* BbsID();
  char* ConfName();
  int ConfNo(int set=-2);
  char* EchoID();

  int KludgesAllowed(int set=-1);
  int MixCaseAllowed(int set=-1);
  int ReceiptAllowed(int set=-1);
  int MaxLines(int set=-1);

  void ResetConfNo(int set=-1);

  char* BadMsgs(char* set=NULL);
  char* ExportPath(char* set=NULL);
  char* ImportPath(char* set=NULL);
  char* ReplyLinker(char* set=NULL);
  char* TossLog(char* set=NULL);

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

