
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gerand.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Random system and control info handling.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

inline uint cfg_ctrlinfo(gactivearea* area) {

  if(area->isemail())
    return CFG->ctrlinfoemail;
  else if(area->isnewsgroup())
    return CFG->ctrlinfonews;
  else if(area->isnet())
    return CFG->ctrlinfonet;
  else if(area->isecho())
    return CFG->ctrlinfoecho;

  return CFG->ctrlinfolocal;
}


//  ------------------------------------------------------------------

void gactivearea::initialize() {

  set_address(data.address());
  set_akamatching((CFG->akamatchnet and isnet()) or (CFG->akamatchecho and isecho()) or (CFG->akamatchlocal and islocal()));
  set_areacopyaddid(CFG->areacopyaddid);
  set_areacopydirect(CFG->areacopydirect);
  set_areacopyto(CFG->areacopyto);
  set_areafreqdirect(CFG->areafreqdirect);
  set_areafreqto(CFG->areafreqto);
  set_areareplydirect(CFG->areareplydirect);
  set_areareplyto(CFG->areareplyto);
  set_attr(data.attr());
  set_ctrlinfo(cfg_ctrlinfo(this));
  set_edithardterm(CFG->edithardterm);
  set_editmixcase(CFG->editmixcase);
  set_editreplyre(CFG->editreplyre);
  set_followupto("");
  set_forcetemplate(CFG->forcetemplate);
  set_gif(CFG->gif.size() ? CFG->gif[CFG->gifno] : "");
  set_inputfile(CFG->inputfile);
  set_internetaddress(CFG->internetaddress);
  set_internetgate(CFG->internetgate);
  set_internetmsgid(CFG->internetmsgid);
  set_internetrfcbody(CFG->internetrfcbody);
  set_loadlanguage(CFG->loadlanguage);
  set_mood((CFG->mood.size() and CFG->moodno) ? CFG->mood[CFG->moodno-1] : "");
  set_msglistdate(CFG->msglistdate);
  set_msglistfast(CFG->msglistfast);
  set_msglistfirst(CFG->msglistfirst);
  set_msglistheader(CFG->msglistheader);
  set_msglistwidesubj(CFG->msglistwidesubj);
  set_netname(CFG->netname);
  set_newsgroups("");
  set_organization(CFG->organization);
  set_origin(CFG->origin[CFG->originno]);
  set_outputfile(CFG->outputfile);
  set_play(CFG->play);
  set_quotebuffile(CFG->quotebuffile);
  set_quotechars(CFG->quotechars);
  set_quotestring(CFG->quotestring);
  set_quotewraphard(CFG->quotewraphard);
  set_replyto(CFG->replyto);
  set_searchfor(CFG->searchfor);
  set_sender(CFG->sender);
  set_tagline(CFG->tagline.size() ? CFG->tagline[CFG->taglineno] : "");
  set_taglinechar(CFG->taglinechar);
  set_taglinefile(CFG->taglinefile);
  set_taglinesupport(CFG->taglinesupport);
  set_tearline(CFG->tearline);
  set_tpl(CFG->tpls ? CFG->tpl[CFG->tplno].file : "");
  set_templatematch(CFG->templatematch);
  set_twitmode(CFG->twitmode);
  set_usefwd(CFG->usefwd);
  set_username(CFG->username[CFG->usernameno]);
  set_viewhidden(CFG->viewhidden);
  set_viewkludge(CFG->viewkludge);
  set_viewquote(CFG->viewquote);
  set_whoto(CFG->whoto);
  set_xlatexport(XLAT->export);
  set_xlatimport(XLAT->import);

  LoadLanguage(loadlanguage().c_str());
}


//  ------------------------------------------------------------------
//  Get data from the Random System

void gactivearea::randomize(bool posting) {

  char grpid[10];
  if(CFG->areafilegroups and groupid()) {
    if(groupid() & 0x8000u)
      sprintf(grpid, "#%u", groupid() & 0x7FFF);
    else {
      grpid[0] = (char)(isalpha(groupid()) ? groupid() : NUL);
      grpid[1] = NUL;
    }
  }
  else {
    *grpid = NUL;
  }

  gstring eid = echoid();

  const char* found = GRP->SetGrp(eid.c_str());
  int curgrp = GRP->curgrp;
  if(*grpid) {
    found = GRP->SetGrp(grpid);
    if(found and (strieql(found, "*") or ((curgrp >= 0) and (curgrp < GRP->curgrp))))
      found = NULL;
    if(not found)
      found = GRP->SetGrp(eid.c_str());
  }

  if(found) {

    gfile fp;
    fp.fopen(CFG->randged, "rb", CFG->sharemode);
    if(fp.isopen()) {

      GRP->SetFp(fp);

      gstring rbuf;
      char buf[256];
      ftn_addr tmp_addr;

      GRP->GetItm(GRP_ADDRESS, group.address);

      GRP->GetItm(GRP_AKAMATCHING, group.akamatching);

      GRP->GetItm(GRP_ATTRIBUTES, group.attr);

      GRP->GetItm(GRP_AREACOPYADDID, group.areacopyaddid);

      GRP->GetItm(GRP_AREACOPYDIRECT, group.areacopydirect);

      GRP->GetItm(GRP_AREACOPYTO, group.areacopyto);

      GRP->GetItm(GRP_AREAFREQDIRECT, group.areafreqdirect);

      GRP->GetItm(GRP_AREAFREQTO, group.areafreqto);

      GRP->GetItm(GRP_AREAREPLYDIRECT, group.areareplydirect);

      GRP->GetItm(GRP_AREAREPLYTO, group.areareplyto);
    
      GRP->GetItm(GRP_CTRLINFO, group.ctrlinfo);

      GRP->GetItm(GRP_EDITHARDTERM, group.edithardterm);

      GRP->GetItm(GRP_EDITMIXCASE, group.editmixcase);

      GRP->GetItm(GRP_EDITREPLYRE, group.editreplyre);

      GRP->GetItm(GRP_FOLLOWUPTO, group.followupto);

      GRP->GetItm(GRP_FORCETEMPLATE, group.forcetemplate);

      if(GRP->GetItm(GRP_GIF, group.gif)) {
        if(group.gif[0] == '@')
          GetRandomLine(group.gif, group.gif.c_str()+1);
      }

      GRP->GetItm(GRP_INPUTFILE, group.inputfile);

      GRP->GetItm(GRP_INTERNETADDRESS, group.internetaddress);

      GRP->GetItm(GRP_INTERNETGATE, group.internetgate);

      GRP->GetItm(GRP_INTERNETMSGID, group.internetmsgid);

      GRP->GetItm(GRP_INTERNETRFCBODY, group.internetrfcbody);

      if(GRP->GetItm(GRP_LOADLANGUAGE, group.loadlanguage))
        LoadLanguage(group.loadlanguage.c_str());

      if(GRP->GetItm(GRP_MOOD, group.mood)) {
        if(group.mood[0] == '@')
          GetRandomLine(group.mood, group.mood.c_str()+1);
      }

      GRP->GetItm(GRP_MSGLISTDATE, group.msglistdate);

      GRP->GetItm(GRP_MSGLISTFAST, group.msglistfast);

      GRP->GetItm(GRP_MSGLISTFIRST, group.msglistfirst);

      GRP->GetItm(GRP_MSGLISTHEADER, group.msglistheader);

      GRP->GetItm(GRP_MSGLISTWIDESUBJ, group.msglistwidesubj);

      if(GRP->GetItm(GRP_NETNAME, group.netname)) {
        if(group.netname[0] == '@')
          GetRandomLine(group.netname, group-netname.c_str()+1);
      }

      GRP->GetItm(GRP_NEWSGROUPS, group.newsgroups);

      GRP->GetItm(GRP_ORGANIZATION, group.organization);

      GRP->GetItm(GRP_ORIGIN, group.origin);

      GRP->GetItm(GRP_OUTPUTFILE, group.outputfile);

      GRP->GetItm(GRP_QUOTEBUFFILE, group.quotebuffile);

      GRP->GetItm(GRP_QUOTECHARS, group.quotechars);

      GRP->GetItm(GRP_QUOTESTRING, group.quotestring);

      GRP->GetItm(GRP_QUOTEWRAPHARD, group.quotewraphard);

      if(GRP->GetItm(GRP_PLAY, group.play)) {
        if(group.play.file[0] == '@') {
          GetRandomLine(rbuf, group.play.file+1);
          group.play.get_file(rbuf.c_str());
        }
      }

      GRP->GetItm(GRP_REPLYTO, group.replyto);

      GRP->GetItm(GRP_SEARCHFOR, group.searchfor);

      GRP->GetItm(GRP_SENDER, group.sender);

      GRP->GetItm(GRP_TAGLINE, group.tagline);
      
      GRP->GetItm(GRP_TAGLINECHAR, group.taglinechar);

      GRP->GetItm(GRP_TAGLINEFILE, group.taglinefile);

      GRP->GetItm(GRP_TAGLINESUPPORT, group.taglinesupport);

      GRP->GetItm(GRP_TEARLINE, group.tearline);
      
      if(GRP->GetItm(GRP_TEMPLATE, group.tpl)) {
        if(group.tpl[0] == '@')
          GetRandomLine(group.tpl, group.tpl.c_str()+1);
      }

      GRP->GetItm(GRP_TEMPLATEMATCH, group.templatematch);

      GRP->GetItm(GRP_TWITMODE, group.twitmode);

      GRP->GetItm(GRP_USEFWD, group.usefwd);

      if(GRP->GetItm(GRP_USERNAME, rbuf)) {
        if(buf[0] == '@')
          GetRandomLine(rbuf, rbuf.c_str()+1);
        strcpy(buf, rbuf.c_str());
        tmp_addr.reset();
        char* aptr = NULL;
        if(posting) {
          aptr = strrchr(buf, ',');
          if(aptr) {
            *aptr++ = NUL;
            aptr = strskip_wht(aptr);
          }
          else {
            aptr = strrchr(buf, ' ');
            if(aptr) {
              aptr++;
              tmp_addr.set(aptr);
              if(tmp_addr.net == 0)
                aptr = NULL;
              else
                *aptr++ = NUL;
            }
          }
          if(aptr) {
            tmp_addr = address();
            tmp_addr.set(aptr);
            if(tmp_addr.net != 0)
              *aptr = NUL;
          }
          strbtrim(buf);
        }
        if(tmp_addr.net == 0)
          tmp_addr = address();
        group.username.name = buf;
        group.username.addr = tmp_addr;
        group.address = aptr;
      }

      if(not posting) {
        GRP->GetItm(GRP_VIEWHIDDEN, group.viewhidden);
        GRP->GetItm(GRP_VIEWKLUDGE, group.viewkludge);
        GRP->GetItm(GRP_VIEWQUOTE,  group.viewquote);
      }

      if(GRP->GetItm(GRP_WHOTO, group.whoto)) {
        if(group.whoto[0] == '@')
          GetRandomLine(group.whoto, group.whoto.c_str()+1);
      }

      GRP->GetItm(GRP_XLATEXPORT, group.xlatexport);

      GRP->GetItm(GRP_XLATIMPORT, group.xlatimport);

      fp.fclose();
    }
  }

  int n = 0;
  while(n < CFG->mailinglists) {
    if(echoid().ieql(CFG->mailinglist[n].echoid)) {
      group.whoto = *CFG->mailinglist[n].contribution ? CFG->mailinglist[n].contribution : CFG->mailinglist[n].sender;
      break;
    }
    n++;
  }
}


//  ------------------------------------------------------------------

gstring& GetRandomLine(gstring& str, const char* filename) {
  
  Path textfile, indexfile;
  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  strcpy(textfile, filename);
  _splitpath(textfile, drive, dir, file, ext);
  strcpy(ext, ".SDX");
  _makepath(indexfile, drive, dir, file, ext);

  FILE* fp = fsopen(AddPath(CFG->goldpath, textfile), "rb", CFG->sharemode);
  if(fp) {

    // Check if index exists or if it is older than the textfile
    const char* idxfile = AddPath(CFG->goldpath, indexfile);
    int idxexist = fexist(idxfile);
    if(idxexist)
      if(FiletimeCmp(textfile, idxfile) > 0)
        idxexist = false;

    FILE* fpi = fsopen(idxfile, idxexist ? "rb" : "wb+", CFG->sharemode);
    if(fpi) {

      // Create index if one was missing
      if(not idxexist) {
        setvbuf(fp, NULL, _IOFBF, 32000);
        setvbuf(fpi, NULL, _IOFBF, 16000);
        long fpos = 0;
        char buf[512];
        while(fgets(buf, sizeof(buf), fp)) {
          fwrite(&fpos, sizeof(long), 1, fpi);
          fpos += strlen(buf);
        }
      }

      // Get random line if there is at least one
      uint lines = (uint)(fsize(fpi)/sizeof(long));
      if(lines > 0) {

        // Select a random line
        long offset = 0;
        long ourline = rand() % lines;
        fseek(fpi, ourline*(long)sizeof(long), SEEK_SET);
        fread(&offset, sizeof(long), 1, fpi);

        // Get it
        char buf[512];
        fseek(fp, offset, SEEK_SET);
        fgets(buf, sizeof(buf), fp);
        str = buf;
        str.btrim().strip_quotes();
      }
      fclose(fpi);
    }
    fclose(fp);
  }

  return str;
}


//  ------------------------------------------------------------------

void GGrp::AddItm(int type, const gnode& data)     { Node n; strxcpy(n.name, data.name.c_str(), sizeof(Name)); n.addr = data.addr; Grp::AddItm(type, &n, sizeof(n)); }
void GGrp::AddItm(int type, const gplay& data)     { gstring tmp = data.file; Grp::AddItm(type, tmp); }
void GGrp::AddItm(int type, const ftn_addr& data)  { Grp::AddItm(type, &data, sizeof(data)); }
void GGrp::AddItm(int type, const ftn_attr& data)  { Grp::AddItm(type, &data, sizeof(data)); }

int GGrp::GetItm(int type, gnode& data)            { Node tmp; int ret = Grp::GetItm(type, &tmp, sizeof(tmp)); data = tmp; return ret; }
int GGrp::GetItm(int type, gplay& data)            { gstring tmp; int ret = Grp::GetItm(type, tmp); data.get_file(tmp.c_str(); return ret; }
int GGrp::GetItm(int type, ftn_addr& data)         { return Grp::GetItm(type, &data, sizeof(data); }
int GGrp::GetItm(int type, ftn_attr& data)         { return Grp::GetItm(type, &data, sizeof(data); }


//  ------------------------------------------------------------------

