
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geread.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Main reader and function key dispatcher.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------
//  Global data

int reader_gen_confirm = false;
int reader_finished;
int reader_done = false;
int reader_topline;
int reader_keyok;
int reader_rcv_noise = false;
Key reader_keycode;
Key reader_lastcode = 0;

GMsgHeaderView HeaderView;
GMsgBodyView   BodyView;


//  ------------------------------------------------------------------

Key ReaderGetKey() {

  Key keycode;
  do {
    keycode = getxchtick();
    if(keycode == Key_Tick)
      CheckTick(KK_ReadQuitNow);
  } while(keycode == Key_Tick);
  if(keycode < KK_Commands) {
    keycode = key_tolower(keycode);
    Key kk = SearchKey(keycode, ReadKey, ReadKeys);
    if(kk)
      keycode = kk;
  }
  return keycode;
}


//  ------------------------------------------------------------------

Key ViewMessage(int istwit) {

  Key keycode;
  int keyok;

  do {

    keycode = ReaderGetKey();
    keyok = true;

    switch(keycode) {
      case KK_ReadMsgLineUp:
        if(istwit <= TWIT_SHOW)
          BodyView.LineUp();
        break;

      case KK_ReadMsgLineDown:
        if(istwit <= TWIT_SHOW)
          BodyView.LineDown();
        break;

      case KK_ReadMsgPgUp:
        if(istwit <= TWIT_SHOW)
          BodyView.PageUp();
        break;

      case KK_ReadMsgPgDn:
        if(istwit <= TWIT_SHOW)
          BodyView.PageDown();
        break;

      case KK_ReadMsgContinue:
        if((istwit > TWIT_SHOW) or not BodyView.Continue())
          keyok = false;
        break;

      case KK_ReadMsgHome:
        BodyView.Top(istwit > TWIT_SHOW);
        istwit = NOT_TWIT;
        break;

      case KK_ReadMsgEnd:
        BodyView.Bottom();
        istwit = NOT_TWIT;
        break;

      default:
        keyok = false;
    }
  } while(keyok);

  return keycode;
}


//  ------------------------------------------------------------------

void Reader() {

  int istwit;
  bool istwitto, istwitsubj, jamskip;
  char buf[256], buf2[200];
  int direction;

  GMsg msg;

  reader_finished = false;

  // Sort all areas by the given sort order before scanning
  AL.sort();

  int personalmails = 0;
  int pmareas = 0;
  int pmscan = CFG->personalmail & PM_STARTUP;
  if(cmdlinedebughg)
    LOG.printf("- AreaScan during startup");
  AL.scan_areas(SCAN_STARTUP, 0, pmscan, personalmails, pmareas);
  startupscan_success = true;

  void CheckSemaphores();

  if(cmdlineimportsoup)
    ImportSOUP();

  AL.sort();
  THROW_CHECK();

  inforow = (gvid.numrows-1)/2;
  wclear();

  gactivearea* area = NULL;
  
  int startid = startecho.length() ? AL.echoid_to_id(startecho) : -1;
  if(startid == -1) {
    area = NewArea(area, AL.number_to_id(0));
  }
  else {
    area = AL.activate(startid);
    if(area->msglistfirst())
      kbput(KK_ReadMessageList);
  }

  THROW_CHECK();

  if(not reader_finished) {

    // Open the editor/reader window

    HeaderView.width           = gvid.numcols;
    HeaderView.height          = 6;
    HeaderView.border_color    = C_HEADB;
    HeaderView.window_color    = C_HEADW;
    HeaderView.title_color     = C_HEADT;
    HeaderView.highlight_color = C_HEADQ;
    HeaderView.from_color      = C_HEADBY;
    HeaderView.to_color        = C_HEADTO;
    HeaderView.subject_color   = C_HEADRE;
    HeaderView.Create();

    BodyView.at_row          = 6;
    BodyView.width           = gvid.numcols;
    BodyView.height          = gvid.numrows - 6 - 1;
    BodyView.border_color    = C_READB;
    BodyView.window_color    = C_READW;
    BodyView.scrollbar_color = C_READPB;
    BodyView.highlight_color = C_READH;
    BodyView.Create();

    do {

      area->set_readpersonal(false);
      area->set_readmarked(false);
      direction = DIR_NEXT;

      area->open();

      if(area->play().type)
        area->play().play();

      // Jump to next message. Go to personal mail if any.
      if(area->pmrkset.size()) {
        area->set_bookmark(area->lastread());
        area->set_readpersonal(false);
        ToggleMarkRead(msg, direction, area);
      }
      else {
        if(CFG->dispautonext or (area->lastread() == 0))
          MoveLastread(DIR_NEXT, area);
        area->set_bookmark(area->lastread());
      }

      // Main message reader loop
      do {

        // Set the default help for this window
        whelpcat(H_Reader);

        if(not area->attr().hex())
          reader_topline = 0;

        if(area->msgnset.size() and not area->lastread())
          area->set_lastread(1);

        if(LoadMessage(msg, CFG->dispmargin-CFG->disppagebar, area)) {

          do {

            HeaderView.Use(area, &msg);
            HeaderView.Paint();

            if(area->readmarked() or area->readpersonal()) {
              gtag& tag = area->readpersonal() ? area->pmrkset : area->markset;
              uint mtemp = tag.find(msg.msgno);
              sprintf(buf2, "%s [%s%s%s]",
                CFG->language[LNG_ReadMarked],
                area->viewhidden() ? "H" : "",
                area->viewkludge() ? "K" : "",
                area->twitmode() == TWIT_IGNORE ? "Ti" : area->twitmode() == TWIT_SKIP ? "Ts" : area->twitmode() == TWIT_BLANK ? "Tb" : area->twitmode() == TWIT_KILL ? "Tk" : ""
              );
              sprintf(buf, buf2,
                mtemp, tag.size(), tag.size()-mtemp
              );
            }
            else {
              sprintf(buf2, "%s [%s%s%s]",
                CFG->language[LNG_ReadAll],
                area->viewhidden() ? "H" : "",
                area->viewkludge() ? "K" : "",
                area->twitmode() == TWIT_IGNORE ? "Ti" : area->twitmode() == TWIT_SKIP ? "Ts" : area->twitmode() == TWIT_BLANK ? "Tb" : area->twitmode() == TWIT_KILL ? "Tk" : ""
              );
              sprintf(buf, buf2,
                area->lastread(), area->msgnset.size(), area->msgnset.size()-area->lastread()
              );
            }
            update_statusline(buf);

            reader_done = false;
            reader_keyok = false;

            // Do some updating first
            area->set_lastread(area->msgnset.absolute_to_relative(msg.msgno));
            area->update_data();

            // Is it a deleted JAM message?
            jamskip = area->isjam() and msg.attr.del() and CFG->jamskipdeleted;

            // Determine if the message is of the "twit" type
            istwit = MsgIsTwit(msg, istwitto, istwitsubj, area);

            if((istwit == TWIT_SKIP) or jamskip) {
              if(direction == DIR_NEXT) {
                if(area->lastread() < area->msgnset.size()) {
                  if(gkbd.kbuf == NULL) {
                    switch(reader_lastcode) {
                      case KK_ReadGotoReplies:
                      case KK_ReadGotoReply1st:
                      case KK_ReadGotoReplyNext:
                        if(MsgHasReplies(msg)) {
                          kbput(KK_ReadGotoReplies);
                        }
                        else {
                          SayBibi();
                          kbput(KK_ReadGotoReplyPrev);
                        }
                        break;
                      default:
                        kbput(KK_ReadGotoNextMsg);
                    }
                  }
                }
              }
              else {
                if(area->lastread() > 1) {
                  if(gkbd.kbuf == NULL) {
                    if(reader_lastcode == KK_ReadGotoReplyPrev) {
                      if(area->msgnset.absolute_to_relative(msg.link.to()))
                        kbput(KK_ReadGotoReplyPrev);
                      else {
                        SayBibi();
                        kbput(KK_ReadGotoReplies);
                      }
                    }
                    else
                      kbput(KK_ReadGotoPrevMsg);
                  }
                }
              }
            }

            if(not jamskip) {

              BodyView.Use(area, &msg, reader_topline);
              if(istwit > TWIT_SHOW) {
                BodyView.window.clear();
                if(istwit == TWIT_SKIP)
                  BodyView.window.prints(1, 0, C_READW, CFG->language[LNG_SkippingTwit]);
                else if(istwit == TWIT_KILL)
                  BodyView.window.prints(1, 0, C_READW, "Killing Twit Message...");
                else {
                  BodyView.window.prints(1, 0, C_READW,
                  istwitto    ? "Message to Twit - Press <Home> to read." :
                  (istwitsubj ? "Message with Twit subject - Press <Home> to read." :
                  CFG->language[LNG_FromTwit]));
                }
              }
              else {
                BodyView.Paint();
              }

              if(istwit)
                HandleGEvent(EVTT_MSGISTWIT);

              if(reader_rcv_noise) {
                if(CFG->beepyourmail == ALWAYS)
                  HandleGEvent(EVTT_MSGTOYOU);
                else if(reader_rcv_noise > 1 and CFG->beepyourmail == true)
                  HandleGEvent(EVTT_MSGTOYOU);
                reader_rcv_noise = false;
              }
              else {
                if(msg.attr.fmu())
                  HandleGEvent(EVTT_MSGFROMYOU);
              }

              if(msg.attr.loc() and CFG->beeplocalmsg)
                HandleGEvent(EVTT_MSGISLOCAL);

              if(gmb_wide_timesread and (msg.orig_timesread == 0))
                area->updatetimesread(msg);

              switch(istwit) {
                case TWIT_KILL:
                  TwitDeleteMsg(msg, direction, area);
                  continue;
                case TWIT_SKIP:
                  reader_keycode = ReaderGetKey();
                  break;
                default:
                  reader_keycode = ViewMessage(istwit);
              }

              reader_topline = BodyView.UpperLine();

              if(reader_gen_confirm)
                ConfirmMsg(msg, area);
            }
            else {
              reader_keycode = ReaderGetKey();
            }

            reader_gen_confirm = false;
            reader_lastcode = reader_keycode;

            switch(reader_keycode) {

              case KK_ReadMainMenu:
                {
                  GMenuNavigate MenuNavigate;
                  int finaltag = MenuNavigate.Run();
                  if((finaltag != W_ESCPRESS) and (finaltag != -1))
                    kbput((Key)finaltag);
                }
                break;

              case KK_ReadSoundkill:
                HandleGEvent(EVTT_STOPVOICE);
                #if defined(GFTRK_ENABLE)
                //__gftrk_log();
                #endif
                break;

              case KK_ReadChangeAttrs:
                ChangeAttributes(msg, area);
                break;

              case KK_ReadChangeAka:
                ChangeAka(area);
                break;

              case KK_ReadChangeTagline:
                ChangeTagline(area);
                break;

              case KK_ReadChangeMood:
                ChangeMood(area);
                break;

              case KK_ReadChangeOrigin:
                ChangeOrigin(area);
                break;

              case KK_ReadChangeUsername:
                ChangeUsername(area);
                break;

              case KK_ReadChangeTemplate:
                ChangeTemplate(area);
                break;

              case KK_ReadChangeXlatImport:
                ChangeXlatImport(area);
                break;

              case KK_ReadLookupDest:
                LookupNode(msg, gstring(""), LOOK_DEST, area);
                reader_keyok = true;
                break;

              case KK_ReadLookupOrig:
                LookupNode(msg, gstring(""), LOOK_ORIG, area);
                reader_keyok = true;
                break;

              case KK_ReadFidoRenumber:
                RenumberArea(area);
                break;

              case KK_ReadDecreaseMargin:
                DecMargin();
                break;

              case KK_ReadIncreaseMargin:
                IncMargin();
                break;

              case KK_ReadToggleHexdump:
                ToggleHexdump(area);
                break;

              case KK_ReadToggleTwits:
                ToggleTwits(area);
                break;

              case KK_ReadToggleMarkRead:
                ToggleMarkRead(msg, direction, area);
                break;

              case KK_ReadToggleMark:
                ToggleMark(msg, area);
                break;

              case KK_ReadToggleBookMark:
                ToggleBookMark(msg, area);
                break;

              case KK_ReadGotoBookMark:
                GotoBookMark(msg, area);
                break;

              case KK_ReadGotoPrevMsg:
                area = GotoPrevMsg(msg, direction, area);
                break;

              case KK_ReadGotoNextMsg:
              case KK_ReadMsgContinue:
                area = GotoNextMsg(msg, direction, area);
                break;

              case KK_ReadGotoNextUnread:
              case KK_ReadGotoPrevUnread:
                break;

              case KK_ReadGotoFirstMsg:
                GotoFirstMsg(direction, area);
                break;

              case KK_ReadGotoLastMsg:
                GotoLastMsg(direction, area);
                break;

              case KK_ReadGotoReplies:
                GotoReplies(msg, direction, area);
                break;

              case KK_ReadGotoReply1st:
                GotoReply1st(msg, direction, area);
                break;

              case KK_ReadGotoReplyNext:
                GotoReplyNext(msg, direction, area);
                break;

              case KK_ReadGotoReplyPrev:
                GotoReplyPrev(msg, direction, area);
                break;

              case KK_ReadToggleHiddKlud:
                ToggleHiddKlud(msg, area);
                break;

              case KK_ReadToggleKludge:
                ToggleKludge(msg, area);
                break;

              case KK_ReadToggleHidden:
                ToggleHidden(msg, area);
                break;

              case KK_ReadToggleQuote:
                ToggleQuote(msg, area);
                break;

              case KK_ReadToggleROT13:
                ToggleROT13(msg, area);
                break;

              case KK_ReadTogglePageBar:
                TogglePageBar();
                break;

              case KK_ReadToggleRealMsgno:
                ToggleRealMsgno(msg, area);
                break;

              case Key_0:
              case Key_1:
              case Key_2:
              case Key_3:
              case Key_4:
              case Key_5:
              case Key_6:
              case Key_7:
              case Key_8:
              case Key_9:
              case KK_ReadGotoMsgno:
                GotoMsgno(msg, area);
                break;

              case KK_ReadMessageList:
                MessageBrowse(msg, area);
                break;

              case KK_ReadDosShell:
                DosShell(area);
                break;

              case KK_ReadFindAll:
                FindAll(msg, reader_topline, reader_keyok, area);
                break;

              case KK_ReadFindHeader:
                FindHdr(msg, reader_topline, reader_keyok, area);
                break;

              case KK_ReadFileRequest:
                FileRequest(msg, area);
                break;

              case KK_ReadNewMsg:
                NewMsg(msg, area);
                break;

              case KK_ReadMoveQuoteMsg:
                OtherAreaQuoteMsg(msg, area);
                break;

              case KK_ReadMoveCommentMsg:
                OtherAreaCommentMsg(msg, area);
                break;

              case KK_ReadQuoteBuf:
                QuoteBuf(msg, area);
                break;

              case KK_ReadQuoteMsg:
                QuoteMsg(msg, area);
                break;

              case KK_ReadCommentMsg:
                CommentMsg(msg, area);
                break;

              case KK_ReadReplyMsg:
                ReplyMsg(msg, area);
                break;

              case KK_ReadChangeMsg:
                ChangeMsg(msg, area);
                break;

              case KK_ReadCopyMoveForward:
                CopyMoveForward(msg, area);
                break;

              case KK_ReadDeleteMsg:
                DelMsg(msg, direction, area);
                break;

              case KK_ReadMakeUserlist:
                MakeUserList(area);
                break;

              case KK_ReadWriteMsg:
                WriteMsg(msg, area);
                break;

              case KK_ReadMarkingOptions:
                MarkingOptions(msg, area);
                break;

              case KK_ReadGotoPrevArea:
                area = PrevArea(msg, direction, area);
                break;

              case KK_ReadGotoNextArea:
                area = NextArea(msg, direction, area);
                break;

              case KK_ReadNewArea:
                area = NewArea(area);
                break;

              case KK_ReadTouchNetscan:
                TouchNetscan();
                reader_keyok = true;
                break;

              case KK_ReadTouchSemaphore:
                TouchSemaphore();
                reader_keyok = true;
                break;

              case KK_ReadExternUtilMenu:
                ExternUtilMenu();
                break;

              case KK_ReadUUDecode:
                UUDecode(msg, area);
                break;

              case KK_ExternUtil01:   reader_keyok = ExternUtil(msg,  1, area);   break;
              case KK_ExternUtil02:   reader_keyok = ExternUtil(msg,  2, area);   break;
              case KK_ExternUtil03:   reader_keyok = ExternUtil(msg,  3, area);   break;
              case KK_ExternUtil04:   reader_keyok = ExternUtil(msg,  4, area);   break;
              case KK_ExternUtil05:   reader_keyok = ExternUtil(msg,  5, area);   break;
              case KK_ExternUtil06:   reader_keyok = ExternUtil(msg,  6, area);   break;
              case KK_ExternUtil07:   reader_keyok = ExternUtil(msg,  7, area);   break;
              case KK_ExternUtil08:   reader_keyok = ExternUtil(msg,  8, area);   break;
              case KK_ExternUtil09:   reader_keyok = ExternUtil(msg,  9, area);   break;
              case KK_ExternUtil10:   reader_keyok = ExternUtil(msg, 10, area);   break;
              case KK_ExternUtil11:   reader_keyok = ExternUtil(msg, 11, area);   break;
              case KK_ExternUtil12:   reader_keyok = ExternUtil(msg, 12, area);   break;
              case KK_ExternUtil13:   reader_keyok = ExternUtil(msg, 13, area);   break;
              case KK_ExternUtil14:   reader_keyok = ExternUtil(msg, 14, area);   break;
              case KK_ExternUtil15:   reader_keyok = ExternUtil(msg, 15, area);   break;
              case KK_ExternUtil16:   reader_keyok = ExternUtil(msg, 16, area);   break;
              case KK_ExternUtil17:   reader_keyok = ExternUtil(msg, 17, area);   break;
              case KK_ExternUtil18:   reader_keyok = ExternUtil(msg, 18, area);   break;
              case KK_ExternUtil19:   reader_keyok = ExternUtil(msg, 19, area);   break;
              case KK_ExternUtil20:   reader_keyok = ExternUtil(msg, 20, area);   break;
              case KK_ExternUtil21:   reader_keyok = ExternUtil(msg, 21, area);   break;
              case KK_ExternUtil22:   reader_keyok = ExternUtil(msg, 22, area);   break;
              case KK_ExternUtil23:   reader_keyok = ExternUtil(msg, 23, area);   break;
              case KK_ExternUtil24:   reader_keyok = ExternUtil(msg, 24, area);   break;

              case KK_ReadUndefine:
                reader_keyok = true;
                break;

              case KK_ReadQuitNow:
                QuitNow();
                break;

              case KK_ReadAskExit:
                ExitAsk();
                break;

              default:
                if(not PlayMacro(reader_keycode, KT_R)) {
                  SayBibi();
                  reader_keyok = true;
                }
            }
          } while(reader_keyok);
        }

        if(gkbd.quitall)
          break;

      } while(not reader_done);

      if(area and area->isopen())
        area->close();

    } while(not reader_finished and not gkbd.quitall);

    HeaderView.Destroy();
    BodyView.Destroy();
  }

  HandleGEvent(EVTT_EXIT);
}


//  ------------------------------------------------------------------
//  Determine if the message has replies

ulong MsgHasReplies(GMsg& msg) {

  if(msg.link.first())
    return msg.link.first();

  for(int n=0; n<msg.link.list_max(); n++)
    if(msg.link.list(n))
      return msg.link.list(n);

  if(msg.link.next())
    return msg.link.next();

  return 0;
}


//  ------------------------------------------------------------------
//  Determine if the message is of the "twit" type

int MsgIsTwit(GMsg& msg, bool& istwitto, bool& istwitsubj, gactivearea* area) {

  int n, istwit = false;
  istwitto = istwitsubj = false;

  // Check for twit names
  for(n=0; n<CFG->twitnames; n++) {
    if(msg.orig.match(CFG->twitname[n].addr)) {
      if(*CFG->twitname[n].name == NUL or strwild(msg.by_name().c_str(), CFG->twitname[n].name)) {
        istwit = true;
        break;
      }
    }
    if(CFG->twitto) {
      if(msg.dest.match(CFG->twitname[n].addr)) {
        if(*CFG->twitname[n].name == NUL or strwild(msg.to_name().c_str(), CFG->twitname[n].name)) {
          istwitto = true;
          istwit = true;
          break;
        }
      }
    }
  }

  // Check for twit subjects
  if(not istwit) {
    for(n=0; n<CFG->twitsubj.size(); n++) {
      if(msg.subject.iinc(CFG->twitsubj[n])) {
        istwitsubj = true;
        istwit = true;
        break;
      }
    }
  }

  if(not istwit)
    return NOT_TWIT;

  if(area->twitmode() == TWIT_BLANK)
    return TWIT_BLANK;

  if(area->twitmode() == TWIT_KILL)
    return TWIT_KILL;

  if(area->twitmode() == TWIT_IGNORE)
    return TWIT_SKIP;

  if((area->twitmode() == TWIT_SKIP) and not msg.attr.tou())
    return TWIT_SKIP;

  return TWIT_SHOW;
}


//  ------------------------------------------------------------------

int LoadMessage(GMsg& msg, int margin, gactivearea* area) {

  GFTRK("LoadMessage");

  if(area->loadmsg(msg, area->msgnset.relative_to_absolute(area->lastread()), margin)) {

    // Mark message as received, if it is for us
    if(msg.msgno and not area->attr().hex()) {
      for(uint n=0; n<CFG->usernames; n++) {
        // Check TO:
        if(msg.dest.match(CFG->username[n].addr)) {
          if(msg.to_name().ieql(CFG->username[n].name)) {
            if(CFG->username[n].addr.net != GFTN_ALL or msg.dest.net == 0 or not area->isnet())
              msg.attr.tou1();   // Set highlight mark
            else {
              for(uint x=0; x<CFG->akas; x++) {
                if(msg.dest.match(CFG->aka[x].addr)) {
                  msg.attr.tou1();   // Set highlight mark
                  break;
                }
              }
            }
            if(msg.attr.tou()) {
              reader_rcv_noise = 1;
              if(not msg.attr.rcv()) {              // Have we seen it?
                msg.date.received = gtime(NULL);    // Get current date
                msg.attr.rcv1();                    // Mark as received
                reader_rcv_noise++;
                break;
              }
            }
          }
        }

        // Check FROM:
        if(msg.orig.match(CFG->username[n].addr)) {
          if(msg.by.ieql(CFG->username[n].name)) {
            if(CFG->username[n].addr.net != GFTN_ALL or msg.orig.net == 0 or not area->isnet())
              msg.attr.fmu1();   // Set highlight mark
            else {
              for(uint x=0; x<CFG->akas; x++) {
                if(msg.orig.match(CFG->aka[x].addr)) {
                  msg.attr.fmu1();   // Set highlight mark
                  break;
                }
              }
            }
          }
        }
      }

      // Touch the msg if it's the first time received
      if(reader_rcv_noise > 1)
        msg.attr.upd1();   // Tell SaveHdr to "touch" the time stamp of the *.MSG file

      // Update the "Times Read" field
      msg.orig_timesread = msg.timesread++;

      if(reader_rcv_noise > 1)
        area->savehdr(GMSG_UPDATE, msg);

      msg.attr.upd0();

      if(reader_rcv_noise > 1 and area->isnet() and (msg.attr.cfm() or msg.attr.rrq()))
        reader_gen_confirm = true;
    }
  }
  else {
    msg.txt = throw_strdup("");
  }

  GFTRK(NULL);
  return true;
}


//  ------------------------------------------------------------------

uint MoveLastread(int direction, gactivearea* area) {

  uint relno = 0;

  if(direction == DIR_NEXT) {
    if(area->lastread() < area->msgnset.size()) {
      area->set_lastread(area->lastread()+1);
      relno = area->lastread();
    }
  }
  else {
    if(1 < area->lastread()) {
      area->set_lastread(area->lastread()-1);
      relno = area->lastread();
    }
  }
  return relno;
}


//  ------------------------------------------------------------------

void ToggleMarkRead(GMsg& msg, int direction, gactivearea* area) {

  if(area->pmrkset.size()) {
    area->toggle_readpersonal();
    if(area->readpersonal()) {
      bool is_marked = false;
      if(area->pmrkset.find(msg.msgno))
        is_marked = true;
      if(not is_marked)
        NextMarkedmsg(direction, msg, area);
    }
  }
  else if(area->markset.size()) {
    area->toggle_readmarked();
    if(area->readmarked()) {
      bool is_marked = false;
      if(area->markset.find(msg.msgno))
        is_marked = true;
      if(not is_marked)
        NextMarkedmsg(direction, msg, area);
    }
  }
  else if(area->readmarked())
    area->set_readmarked(false);
  else
    reader_keyok = true;
}


//  ------------------------------------------------------------------

void GotoBookMark(GMsg& msg, gactivearea* area) {

  if(area->bookmark()) {
    ulong temp = msg.msgno;
    area->set_lastread(area->msgnset.absolute_to_relative(area->bookmark()));
    area->set_bookmark(temp);
  }
  else {
    reader_keyok = true;
    SayBibi();
  }
}


//  ------------------------------------------------------------------

gactivearea* GoNextMsg(GMsg& msg, int direction, gactivearea* area) {

  GMenuNewarea MenuNewarea;

  if(area->readmarked() or area->readpersonal()) {
    if(not NextMarkedmsg(direction, msg, area)) {
      if(MenuNewarea.Run(area)) {
        return NewArea(area);
      }
      reader_keyok = true;
    }
  }
  else if(not MoveLastread(direction, area)) {
    if(MenuNewarea.Run(area)) {
      if(area->msgnset.absolute_to_relative(msg.msgno) == 1)
        if(direction == DIR_PREV)
          area->set_lastread(0);
      return NewArea(area);
    }
    reader_keyok = true;
  }
  return area;
}


//  ------------------------------------------------------------------

gactivearea* GotoNextMsg(GMsg& msg, int& direction, gactivearea* area) {

  direction = DIR_NEXT;
  return GoNextMsg(msg, direction, area);
}


//  ------------------------------------------------------------------

gactivearea* GotoPrevMsg(GMsg& msg, int& direction, gactivearea* area) {

  direction = DIR_PREV;
  return GoNextMsg(msg, direction, area);
}


//  ------------------------------------------------------------------

void GotoNextUnread(GMsg&, int& direction, gactivearea*) {

  direction = DIR_NEXT;
}


//  ------------------------------------------------------------------

void GotoPrevUnread(GMsg&, int& direction, gactivearea*) {

  direction = DIR_PREV;
}


//  ------------------------------------------------------------------

void GotoFirstMsg(int& direction, gactivearea* area) {

  direction = DIR_NEXT;
  if(area->readpersonal())
    area->set_lastread(area->pmrkset.absolute_to_relative(area->pmrkset[0]));
  else if(area->readmarked())
    area->set_lastread(area->msgnset.absolute_to_relative(area->markset[0]));
  else
    area->set_lastread(area->msgnset.size() ? 1 : 0);
}


//  ------------------------------------------------------------------

void GotoLastMsg(int& direction, gactivearea* area) {

  direction = DIR_PREV;
  if(area->readpersonal())
    area->set_lastread(area->msgnset.absolute_to_relative(area->pmrkset[area->pmrkset.size()-1]));
  else if(area->readmarked())
    area->set_lastread(area->msgnset.absolute_to_relative(area->markset[area->markset.size()-1]));
  else
    area->set_lastread(area->msgnset.size());
}


//  ------------------------------------------------------------------

struct ReplySel {
  uint    reln;
  char    msgno[12];
  gstring name;
  gstring addr;
  char    written[25];
};


//  ------------------------------------------------------------------

void GotoReplies(GMsg& msg, int& direction, gactivearea* area) {

  direction = DIR_NEXT;

  const int list_max = msg.link.list_max();

  char buf[200];
  uint replies = 0;
  uint maxname = 0;
  uint maxaddr = 0;
  uint gotolink = 0;
  uint maxmsgno = 0;
  uint maxwritten = 0;

  GMsg rmsg;
  ReplySel* rlist = (ReplySel*)throw_calloc(list_max+3, sizeof(ReplySel));

  for(int n=0; n<list_max+2; n++) {
    ulong msgn = n ? ((n!=list_max+1) ? msg.link.list(n-1) : msg.link.next()) : msg.link.first();
    uint reln = area->msgnset.absolute_to_relative(msgn);
    if(reln) {
      if(CFG->replylinklist == REPLYLINKLIST_FULL)
        area->loadmsg(rmsg, msgn, CFG->dispmargin-CFG->disppagebar);
      else
        area->loadhdr(rmsg, msgn);
      rlist[replies].msgno[0] = (n==list_max+1) ? '*' : '+';
      ulong msgno = CFG->disprealmsgno ? msgn : reln;
      ltoa(msgno, rlist[replies].msgno+1, 10);
      maxmsgno = maximum_of_two(maxmsgno, (uint)strlen(rlist[replies].msgno+1));
      rlist[replies].name = rmsg.by;
      maxname = maximum_of_two(maxname, rlist[replies].name.length());
      rmsg.orig.make_string(buf);
      gstring buf2 = buf;
      rlist[replies].addr = "(" + buf2 + ")";
      maxaddr = maximum_of_two(maxaddr, rlist[replies].addr.length());
      strftimei(rlist[replies].written, CFG->disphdrdateset.len, CFG->language[LNG_DateTimeFmt], glocaltime(&rmsg.written));
      maxwritten = maximum_of_two(maxwritten, (uint)strlen(rlist[replies].written));
      rlist[replies].reln = reln;
      replies++;
      if(gotolink == 0)
        gotolink = reln;
    }
  }

  int selected = 0;
  if(replies > 1) {
    uint maxname2 = gvid.numcols-15-maxmsgno-maxaddr-maxwritten;
    maxname = minimum_of_two(maxname, maxname2);
    char** listr = (char**)throw_calloc(list_max+3, sizeof(char*));
    int n;
    for(n=0; n<list_max+3; n++) {
      if(n<replies) {
        sprintf(buf, " %c %*s : %-*.*s  %-*s  %-*s ",
          rlist[n].msgno[0],
          maxmsgno, rlist[n].msgno+1,
          maxname, (int)maxname, rlist[n].name.c_str(),
          maxaddr, rlist[n].addr.c_str(),
          maxwritten, rlist[n].written
        );
        listr[n] = throw_strdup(buf);
      }
      else {
        listr[n] = NULL;
        break;
      }
    }
    set_title(CFG->language[LNG_Replies], TCENTER, C_ASKT);
    update_statusline(CFG->language[LNG_SelectReply]);
    whelppcat(9999);
    int pick_max = minimum_of_two(replies, (gvid.numrows-10));
    selected = wpickstr(6,0,6+pick_max+1,-1,W_BASK,C_ASKB,C_ASKW,C_ASKS,listr,0,title_shadow);
    whelpop();
    gotolink = (selected != -1) ? rlist[selected].reln : 0;
    for(n=0; n<replies; n++)
      throw_free(listr[n]);
    throw_free(listr);
  }

  throw_free(rlist);

  if(gotolink)
    area->set_lastread(gotolink);
  else {
    if(selected != -1)
      SayBibi();
    reader_keyok = true;
  }
}


//  ------------------------------------------------------------------
  
void GotoReply1st(GMsg& msg, int& direction, gactivearea* area) {

  direction = DIR_NEXT;
  uint gotolink = area->msgnset.absolute_to_relative(msg.link.first());
  if(gotolink)
    area->set_lastread(gotolink);
  else {
    SayBibi();
    reader_keyok = true;
  }
}


//  ------------------------------------------------------------------
  
void GotoReplyNext(GMsg& msg, int& direction, gactivearea* area) {

  direction = DIR_NEXT;
  uint gotolink = area->msgnset.absolute_to_relative(msg.link.next());
  if(gotolink)
    area->set_lastread(gotolink);
  else {
    SayBibi();
    reader_keyok = true;
  }
}


//  ------------------------------------------------------------------

void GotoReplyPrev(GMsg& msg, int& direction, gactivearea* area) {

  direction = DIR_PREV;
  uint gotolink = area->msgnset.absolute_to_relative(msg.link.to());
  if(gotolink)
    area->set_lastread(gotolink);
  else {
    SayBibi();
    reader_keyok = true;
  }
}


//  ------------------------------------------------------------------

