
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geread2.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Reader secondary functions.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include <gutluude.h>
#include <gwinput.h>


//  ------------------------------------------------------------------

static int __FAR prnfh;
static int __FAR prnacc;
static int __FAR prnmargin;
static int __FAR prnheader;


//  ------------------------------------------------------------------

void GMsg::save_textfile(const char* savefile, int mode) {

  gstring fnam;
  bool prn = false;
  
  if(mode == SAVELINES_WRITE and streql(savefile, "\001PRN")) {
    fnam = "PRN";
    prn = true;
  }
  else {
    fnam = savefile;
    if(mode == SAVELINES_WRITE)
      prnfh = sopen(fnam.c_str(), prnacc, CFG->sharemode, S_IWRITE);
    else
      prnfh = sopen(fnam.c_str(), O_CREAT|O_WRONLY|O_TRUNC|O_TEXT, CFG->sharemode, S_IWRITE);
  }

  int lines=0;

  if(prnfh != -1) {

    if(mode == SAVELINES_WRITE) {
      if(prnheader)
        PrintHeader(*this, prnfh, prnmargin, area);
      if(prn)
        lines = 6;
    }

    if(display.size()) {
      char buf[256];
      glineref line = linearray;
      for(int n=0; n<display.size(); n++) {
        line = display[n];
        uint lineisctrl = line.type() & (gline::tearline|gline::origin|gline::gateorigin|gline::hidden|gline::kludge);
        if(not ((mode == SAVELINES_SAVENOCTRL) and lineisctrl)) {
          char* ptr = strcpy(buf, line.text());
          while(*ptr) {
            if(mode == SAVELINES_WRITE) {
              // Replace control codes, except the ANSI escape code
              if(iscntrl(*ptr)) {
                // only allow ESC in file write
                if(prn or (*ptr != '\x1B')) {
                  *ptr = (*ptr == CTRL_A) ? '@' : '.';
                }
              }
            }
            ptr++;
          }
          ptr = buf;
          write(prnfh, ptr, strlen(ptr));
          if(mode == SAVELINES_NEW)
            if(CFG->edithardlines)
              if(line.ishard())
                if(not ((line.type() & (gline::tearline|gline::origin|gline::gateorigin|gline::kludge|gline::hidden|gline::quote)) or strblank(ptr)))
                  write(prnfh, CFG->edithardline.c_str(), CFG->edithardline.length());
          write(prnfh, "\n", 1);
          if(prn) {
            lines++;
            if(lines%CFG->printlength == 0 and CFG->printformfeed)
              write(prnfh, "\f", 1);
          }
        }
      }
    }

    // Add an empty line and formfeed at the bottom
    if(mode == SAVELINES_WRITE) {
      write(prnfh, "\n", 1);
    }
    if(prn) {
      if(CFG->printformfeed)           // Add formfeed
        write(prnfh, "\f", 1);
    }
    else {
      close(prnfh);
    }
  }
  else {
    char buf[256];
    sprintf(buf, CFG->language[LNG_CouldNotOpen], fnam.c_str());
    w_info(buf);
    waitkeyt(10000);
    w_info(NULL);
  }
}


//  ------------------------------------------------------------------

void WriteMsg(GMsg& msg, gactivearea* area) {

  GFTRK("WriteMsg");

  if(area->msgnset.size()) {

    char buf[256];

    int overwrite = false;

    prnfh = -1;
    prnacc = 0;

    GMenuDomarks MenuDomarks;
    int source = area->markset.size() ? MenuDomarks.Run(CFG->language[LNG_Write], area) : WRITE_CURRENT;

    if(source != WRITE_QUIT) {

      GMenuWriteMsg MenuWriteMsg;
      int target = MenuWriteMsg.Run();
      if(target != WRITE_QUIT) {

        prnheader = !(target & WRITE_NO_HEADER);

        if(target & WRITE_PRINTER)
          prnmargin = CFG->printmargin;
        else
          prnmargin = 79;

        if(source == WRITE_MARKED) {
          if(target & WRITE_FILE) {
            do {
              CFG->outputfile = area->outputfile();
              gstring outputfile = CFG->outputfile;
              if(not edit_string(outputfile, sizeof(Path), CFG->language[LNG_WriteMsgs], H_WriteMessage))
                goto Finish;
              CFG->outputfile = outputfile;
              if(*CFG->outputfile == '*')
                CFG->outputfile = CFG->outputfile.c_str()+1;
              else {
                for(int z=0; z<CFG->filealiase.size(); z++) {
                  if(CFG->outputfile.ieql(CFG->filealias[z].alias)) {
                    CFG->outputfile = CFG->filealias[z].file;
                    break;
                  }
                }
              }
              area->set_outputfile(CFG->outputfile);
              w_infof(" %s ", area->outputfile().c_str());
              if(area->outputfile().icmp("PRN") and strnicmp(area->outputfile().c_str(), "LPT", 3))
                if(fexist(area->outputfile().c_str())) {
                  GMenuOverwrite MenuOverwrite;
                  overwrite = MenuOverwrite.Run();
                }
            } while(overwrite == -1);
          }
          else if(target & WRITE_PRINTER) {
            prnfh = sopen("PRN", O_WRONLY|O_TEXT, CFG->sharemode, S_IWRITE);
            if(prnfh != -1)
              write(prnfh, &(CFG->printinit[1]), CFG->printinit[0]);
          }
          w_info(NULL);
          w_progress(PROGRESS_NEW, C_INFOW, 0, area->markset.size(), CFG->language[LNG_Writing]);
          for(uint n=0; n<area->markset.size(); n++) {
            if(overwrite and n)
              overwrite = NO;     // Overwrite only the first time
            if(kbxhit()) {
              if(kbxget() == Key_Esc) {
                HandleGEvent(EVTT_JOBFAILED);
                break;
              }
            }
            update_statuslinef(CFG->language[LNG_WritingMsg], n+1, area->markset.size());
            w_progress(PROGRESS_UPDATE, C_INFOW, n+1, area->markset.size(), CFG->language[LNG_Writing]);
            if(target & WRITE_PRINTER) {
              area->loadmsg(msg, area->markset[n], prnmargin);
              if(prnfh != -1)
                msg.save_textfile("\001PRN", SAVELINES_WRITE);
            }
            else {
              area->loadmsg(msg, area->markset[n], prnmargin);
              if(overwrite)
                prnacc = O_CREAT|O_WRONLY|O_TRUNC|O_TEXT;
              else
                prnacc = O_CREAT|O_WRONLY|O_APPEND|O_TEXT;
              msg.save_textfile(area->outputfile().c_str(), SAVELINES_WRITE);
            }
          }
          if(prnfh != -1)
            write(prnfh, &(CFG->printreset[1]), CFG->printreset[0]);
          w_progress(PROGRESS_QUIT, 0, 0, 0, NULL);
        }
        else if(source == WRITE_CURRENT) {
          if(target & WRITE_FILE) {
            do {
              overwrite = NO;
              CFG->outputfile = area->outputfile();
              gstring outputfile = CFG->outputfile;
              if(edit_string(outputfile, sizeof(Path), CFG->language[LNG_WriteMsgs], H_WriteMessage)) {
                CFG->outputfile = outputfile;
                if(*CFG->outputfile == '*')
                  CFG->outputfile = CFG->outputfile.c_str()+1;
                else {
                  for(int z=0; z<CFG->filealiase.size(); z++) {
                    if(CFG->outputfile.ieql(CFG->filealias[z].alias)) {
                      CFG->outputfile = CFG->filealias[z].file;
                      break;
                    }
                  }
                }
                area->set_outputfile(CFG->outputfile);
                w_infof(" %s ", area->outputfile().c_str());
                if(area->outputfile().icmp("PRN") and area->outputfile().nicmp("LPT", 3)) {
                  if(fexist(area->outputfile())) {
                    GMenuOverwrite MenuOverwrite;
                    overwrite = MenuOverwrite.Run();
                    if(overwrite == -1)   // Escape was hit
                      continue;
                  }
                }
                sprintf(buf, CFG->language[LNG_WritingFile], area->outputfile().c_str());
                w_info(buf);
                area->loadmsg(msg, msg.msgno, prnmargin);
                if(overwrite)
                  prnacc = O_CREAT|O_WRONLY|O_TRUNC|O_TEXT;
                else
                  prnacc = O_CREAT|O_WRONLY|O_APPEND|O_TEXT;
                msg.save_textfile(area->outputfile().c_str(), SAVELINES_WRITE);
                w_info(NULL);
              }
            } while(overwrite == -1);
          }
          else if(target & WRITE_PRINTER) {
            w_info(CFG->language[LNG_WritingPRN]);
            area->loadmsg(msg, msg.msgno, prnmargin);
            prnfh = sopen(CFG->printdevice.c_str(), O_WRONLY|O_TEXT, CFG->sharemode, S_IWRITE);
            if(prnfh != -1) {
              write(prnfh, &(CFG->printinit[1]), CFG->printinit[0]);
              msg.save_textfile("\001PRN", SAVELINES_WRITE);
              write(prnfh, &(CFG->printreset[1]), CFG->printreset[0]);
            }
            w_info(NULL);
          }
        }
      }
    }

  Finish:

    w_info(NULL);

    if(prnfh != -1) {
      close(prnfh);
      prnfh = -1;
    }
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void ChangeAttributes(GMsg& msg, gactivearea* area) {

  if(msg.msgno) {

    // Reload if hexdumped
    if(area->attr().hex()) {
      area->set_attr().hex0();
      area->loadmsg(msg, msg.msgno, CFG->dispmargin-CFG->disppagebar);
    }

    ftn_attr attr = msg.attr;
    AskAttributes(msg, area);
    if(attr != msg.attr) {
      replace_controllines(MODE_CHANGE, msg, area);
      msg.lines_to_text();
      area->savemsg(GMSG_UPDATE, msg);
    }
  }
}


//  ------------------------------------------------------------------

void DecMargin() {
  
  if(CFG->dispmargin > (gvid.numcols/3))
    CFG->dispmargin--;
  else {
    SayBibi();
    reader_keyok = YES;
  }
}


//  ------------------------------------------------------------------

void IncMargin() {

  if(CFG->dispmargin < gvid.numcols)
    CFG->dispmargin++;
  else {
    SayBibi();
    reader_keyok = YES;
  }
}


//  ------------------------------------------------------------------

void ToggleHexdump(gactivearea* area) {

  area->set_attr().hexX();
}


//  ------------------------------------------------------------------

void ToggleMark(GMsg& msg, gactivearea* area) {

  uint tmp = area->markset.find(msg.msgno);
  if(tmp)
    area->markset.delete_relative(tmp);
  else
    area->markset.add(msg.msgno);
  HeaderView.Use(area, &msg);
  HeaderView.Paint();
  reader_keyok = YES;
}


//  ------------------------------------------------------------------

void ToggleBookMark(GMsg& msg, gactivearea* area) {

  if(area->bookmark() == msg.msgno)
    area->set_bookmark(0);
  else
    area->set_bookmark(msg.msgno);
  HeaderView.Use(area, &msg);
  HeaderView.Paint();
  reader_keyok = YES;
}


//  ------------------------------------------------------------------

void ToggleHiddKlud(GMsg& msg, gactivearea* area) {

  area->toggle_viewhidden();
  area->toggle_viewkludge();
  msg.reindex_lines();
  reader_topline = 0;
  reader_keyok = YES;
}


//  ------------------------------------------------------------------

void ToggleKludge(GMsg& msg, gactivearea* area) {

  area->toggle_viewkludge();
  msg.reindex_lines();
  reader_topline = 0;
  reader_keyok = YES;
}


//  ------------------------------------------------------------------

void ToggleHidden(GMsg& msg, gactivearea* area) {

  area->toggle_viewhidden();
  msg.reindex_lines();
  reader_topline = 0;
  reader_keyok = YES;
}


//  ------------------------------------------------------------------

void ToggleQuote(GMsg& msg, gactivearea* area) {

  area->toggle_viewquote();
  msg.reindex_lines();
  reader_topline = 0;
  reader_keyok = YES;
}


//  ------------------------------------------------------------------

void ToggleROT13(GMsg& msg, gactivearea* area) {

  if(area->msgnset.size()) {
    msg.attr.rotX();
    Rot13(msg);
    reader_keyok = YES;
  }
}


//  ------------------------------------------------------------------

void TogglePageBar() {

  CFG->disppagebar ^= YES;
}


//  ------------------------------------------------------------------

void ToggleRealMsgno(GMsg& msg, gactivearea* area) {

  CFG->disprealmsgno ^= YES;
  HeaderView.Use(area, &msg);
  HeaderView.Paint();
  reader_keyok = YES;
}


//  ------------------------------------------------------------------

void ToggleTwits(gactivearea* area) {

  switch(area->twitmode()) {
    case TWIT_SHOW:
      area->set_twitmode(TWIT_BLANK);
      w_info(CFG->language[LNG_TwitBlanked]);
      waitkeyt(3000);
      w_info(NULL);
      break;
    case TWIT_BLANK:
      area->set_twitmode(TWIT_SKIP);
      w_info(CFG->language[LNG_TwitSkipped]);
      waitkeyt(3000);
      w_info(NULL);
      break;
    case TWIT_SKIP:
      area->set_twitmode(TWIT_IGNORE);
      w_info(CFG->language[LNG_TwitIgnoreSkip]);
      waitkeyt(3000);
      w_info(NULL);
      break;
    case TWIT_IGNORE:
      area->set_twitmode(TWIT_SHOW);
      w_info(CFG->language[LNG_TwitDisplayed]);
      waitkeyt(3000);
      w_info(NULL);
      break;
  }
  CFG->twitmode = area->twitmode();
}


//  ------------------------------------------------------------------

void GotoMsgno(GMsg& msg, gactivearea* area) {

  if(area->msgnset.size()) {

    // Set the active window
    int lastwh = whandle();
    int inhdr = area->msglistheader() or not _in_msglist;

    if(reader_keycode < KK_Macro)
      if(gkbd.kbuf == NULL)
        kbput(reader_keycode); // Put back the key

    char buf[150];
    char* ptr = buf;
    int list_max = msg.link.list_max();
    ulong* replies = (ulong*)throw_calloc(list_max+1, sizeof(ulong));
    ulong replyto, replynext;
    if(CFG->disprealmsgno) {
      ptr += sprintf(ptr, " %-5.5s:       %s %u", CFG->language[LNG_MsgReal], CFG->language[LNG_of], area->msgnset.size());
      replyto = msg.link.to();
      replies[0] = msg.link.first();
      replynext = msg.link.next();
      for(int replyn=1; replyn<list_max+1; replyn++)
        replies[replyn] = msg.link.list(replyn-1);
    }
    else {
      ptr += sprintf(ptr, " %-5.5s:       %s %u", CFG->language[LNG_Msg], CFG->language[LNG_of], area->msgnset.size());
      replyto = area->msgnset.absolute_to_relative(msg.link.to());
      replies[0] = area->msgnset.absolute_to_relative(msg.link.first());
      replynext = area->msgnset.absolute_to_relative(msg.link.next());
      for(int replyn=1; replyn<list_max+1; replyn++)
        replies[replyn] = area->msgnset.absolute_to_relative(msg.link.list(replyn-1));
    }
    if(replyto)
      ptr += sprintf(ptr, " -%lu", replyto);
    for(int replyn=0,plus=0; replyn<list_max+1; replyn++)
      if(replies[replyn])
        ptr += sprintf(ptr, " %s%lu", (plus++?"":"+"), replies[replyn]);
    if(replynext)
      sprintf(ptr, " *%lu", replynext);

    if(inhdr) {
      //wactiv_(W_HEAD);
      HeaderView.window.activate_quick();
    }
    else {
      wopen_(0,0,3,strlen(buf)+3,W_BHEAD,C_HEADB,C_HEADW);
      w_shadow();
      wtitle(CFG->language[LNG_EnterMsgno], TCENTER, C_HEADT);
    }

    int wrow = inhdr ? 1 : 0;
    wprints(wrow, 0, C_HEADW, buf);
    *buf = NUL;

    gwindow iwindow(whandle());

    gstring fbuf = buf;
    winput2 iform(iwindow);
    iform.setup(C_HEADW, C_HEADW, C_HEADI, '\xB0');
    iform.add_field(0, wrow, 8, 5, fbuf, 20, winput::cvt_none, winput::entry_new);
    iform.run(H_InputMsgno);
    if(iform.dropped)
      fbuf = "";

    long active = area->msgnset.size();
    long msgno = atol(fbuf.c_str());

    if(CFG->disprealmsgno)
      if(msgno > 0)
        msgno = area->msgnset.absolute_to_relative(msgno);

    if((msgno < 0) and ((active+msgno) > 0))
      msgno = active + msgno;

    if(in_range(msgno, 1L, active))
      area->set_lastread((uint)msgno);

    if(inhdr)
      wactiv_(lastwh);
    else
      wclose();

    throw_free(replies);
  }
}


//  ------------------------------------------------------------------

gactivearea* PrevArea(GMsg& msg, int direction, gactivearea* area) {

  int currarea = AL.id_to_number(area->areaid());
  if(currarea > 0) {
    currarea--;
    area->close();
    AL.deactivate(area);
    area = AL.activate(AL[currarea]->areaid());
    area->open();
    area->set_bookmark(area->lastread());
    if(area->pmrkset.size()) {
      area->set_readpersonal(false);
      ToggleMarkRead(msg, direction, area);
    }
  }
  else {
    HandleGEvent(EVTT_ENDOFMSGS);
  }

  return area;
}


//  ------------------------------------------------------------------

gactivearea* NextArea(GMsg& msg, int direction, gactivearea* area) {

  int currarea = AL.id_to_number(area->areaid());
  if(currarea < (AL.areas()-1)) {
    currarea++;
    area->close();
    AL.deactivate(area);
    area = AL.activate(AL[currarea]->areaid());
    area->open();
    area->set_bookmark(area->lastread());
    if(area->pmrkset.size()) {
      area->set_readpersonal(false);
      ToggleMarkRead(msg, direction, area);
    }
  }
  else {
    HandleGEvent(EVTT_ENDOFMSGS);
  }

  return area;
}


//  ------------------------------------------------------------------

gactivearea* NewArea(gactivearea* area, int areaid) {

  if(area) {
    if(areaid == -1)
      areaid = area->areaid();
    area->close();
    AL.deactivate(area);
    area = NULL;
  }

  int newarea = areaid == -1 ? 0 : areaid;
  newarea = AreaPick(CFG->language[LNG_NewArea], 0, &newarea);
  if(newarea != -1)
    areaid = newarea;

  AL.sort();
  reader_done = YES;

  if(gkbd.quitall)
    QuitNow();
  else {
    area = AL.activate(areaid);
    startecho = area->echoid();
    if(area->msglistfirst())
      kbput(KK_ReadMessageList);
  }

  return area;
}


//  ------------------------------------------------------------------

void DosShell(gactivearea* area) {

  ShellToDos(getenv("COMSPEC"), CFG->language[LNG_DOS_Shell], LGREY|_BLACK, YES, true, false, area);
  reader_keyok = YES;
}


//  ------------------------------------------------------------------

void TouchNetscan(bool popup) {

  FILE* fp = fsopen(AddPath(CFG->areapath, CFG->semaphore.netscan), "wb", CFG->sharemode);
  if(fp) {
    fclose(fp);
    if(popup) {
      w_infof(" %s ", CFG->semaphore.netscan);
      HandleGEvent(EVTT_JOBDONE);
      waitkeyt(10000);
      w_info(NULL);
    }
  }
}


//  ------------------------------------------------------------------

int ExternUtil(GMsg& msg, int utilno, gactivearea* area) {

  Path editorfile, tmpfile;
  strcpy(editorfile, AddPath(CFG->goldpath, CFG->editfile));
  mktemp(strcpy(tmpfile, AddPath(CFG->goldpath, "GEDTMPXXXXXX")));

  ExtUtil* extutil = CFG->externutil;

  for(int utlno=0; utlno<CFG->externutils; utlno++,extutil++) {

    if(extutil->utilno == utilno) {

      char cmdline[256];
      strcpy(cmdline, extutil->cmdline);

      int mode = (extutil->options & EXTUTIL_KEEPCTRL) ? SAVELINES_SAVE : SAVELINES_SAVENOCTRL;
      msg.save_textfile(editorfile, mode);
      if(striinc("@tmpfile", cmdline))
        msg.save_textfile(tmpfile, mode);

      strischg(cmdline, "@path", CFG->goldpath);
      strischg(cmdline, "@file", editorfile);
      strischg(cmdline, "@tmpfile", tmpfile);
      TokenXlat(MODE_NEW, cmdline, msg, msg, area);

      int pauseval = 0;
      if(extutil->options & EXTUTIL_PAUSEONERROR)
        pauseval = -1;
      if(extutil->options & EXTUTIL_PAUSE)
        pauseval = 1;

      ShellToDos(cmdline, "",
        extutil->options & EXTUTIL_CLS ? LGREY|_BLACK : 0,
        extutil->options & EXTUTIL_CURSOR,
        extutil->options & EXTUTIL_SWAP,
        pauseval, area
      );

      if(extutil->options & EXTUTIL_RELOAD) {

        if(not (extutil->options & EXTUTIL_KEEPCTRL)) {
          if(msg.tearline.length() or msg.origin.length()) {
            gfile fp;
            fp.fopen(editorfile, "at");
            if(fp.isopen()) {
              if(msg.tearline.length())
                fp.printf("--- %s\n", msg.tearline.c_str());
              if(msg.origin.length())
                fp.printf(" * Origin: %s\n", msg.origin.c_str());
              fp.fclose();
            }
          }
        }

        int hardlines = CFG->edithardlines;
        CFG->edithardlines = false;
        msg.load_textfile(editorfile, CFG->dispmargin-CFG->disppagebar);
        CFG->edithardlines = hardlines;

        if(extutil->options & EXTUTIL_WIPE)
          WipeFile(editorfile);
        WipeFile(tmpfile);

        glineref line = msg.linearray;
        if(streql(line.text(), "-----BEGIN PGP MESSAGE-----")) {
          line.insert("\x1ENC: PGP", GLINE_HARD|GLINE_KLUD, GKLUD_ENC);
          msg.kludge.enc = "PGP";
        }
      }

      return true;
    }
  }

  SayBibi();
  return true;
}


//  ------------------------------------------------------------------

void ExternUtilMenu() {

  // Not implemented yet
}


//  ------------------------------------------------------------------

void UUDecode(GMsg& msg, gactivearea* area) {

  #define LNG__UUdecodeinvalidpath " Invalid path for uudecoding - file not written "

  Path infile, outfile;

  if(is_dir(CFG->uudecodepath)) {

    w_info(CFG->language[LNG_WaitUUdecoding]);

    strcpy(infile, AddPath(CFG->goldpath, CFG->editfile));
    strcpy(outfile, CFG->uudecodepath);

    msg.save_textfile(infile, SAVELINES_SAVE);

    uudecode(infile, outfile);

    if(*outfile)
      w_infof(CFG->language[LNG_CompletedUUdecode], outfile);
    else
      w_infof(CFG->language[LNG_NotUUencoded]);
  }
  else {
    w_infof(LNG__UUdecodeinvalidpath);
  }
  waitkeyt(10000);
  w_info(NULL);
}


//  ------------------------------------------------------------------

void MakeUserList(gactivearea* area) {

  if(area->msgnset.size()) {

    gstring userlistfile = CFG->userlistfile;
    if(not edit_string(userlistfile, sizeof(Path), CFG->language[LNG_UserlistName], 0))
      return;

    strxcpy(CFG->userlistfile, userlistfile.c_str(), sizeof(Path));

    FILE* fp;
    word* crclist;
    word crc, crcs=0;
    register uint n,x;
    char userline[80], adrs[40];

    uint temp = area->lastread();

    GMsg msg;

    crclist = (word*)throw_calloc(area->msgnset.size()+1, sizeof(word));
    fp = fsopen(AddPath(CFG->goldpath, CFG->userlistfile), "ab", CFG->sharemode);
    if(fp) {
      w_progress(PROGRESS_NEW, C_INFOW, 0, area->msgnset.size(), CFG->language[LNG_GenUserlist]);
      for(n=0; n<area->msgnset.size(); n++) {
        update_statuslinef(CFG->language[LNG_ReadingMsg], n+1, area->msgnset.size());
        w_progress(PROGRESS_UPDATE, C_INFOW, n+1, area->msgnset.size(), NULL);
        area->loadmsg(msg, area->msgnset.relative_to_absolute(n + 1), CFG->dispmargin-CFG->disppagebar);
        crc = strCrc16(msg.by.c_str());
        for(x=0; x<crcs; x++)
          if(crclist[x] == crc)
            break;  // We have already used it
        if(x >= crcs) {
          crclist[crcs++] = crc;
          strrevname(userline, msg.by.c_str());
          msg.orig.make_string(adrs);
          fprintf(fp, "%-36.36s%24.24s\r\n", userline, adrs);
        }
      }
      throw_free(crclist);
      w_progress(PROGRESS_QUIT, 0, 0, 0, NULL);
      fclose(fp);
    }

    area->set_lastread(temp);
  }
}


//  ------------------------------------------------------------------

void TouchSemaphore() {

  gstring file;
  if(edit_string(file, sizeof(Path), CFG->language[LNG_TouchSemaphore], 0))
    TouchFile(file.c_str());
}


//  ------------------------------------------------------------------

void QuitNow() {

  reader_finished = YES;
  reader_done = YES;
}


//  ------------------------------------------------------------------

void ExitAsk() {

  GMenuQuit MenuQuit;
  reader_done = reader_finished = MenuQuit.Run();
}


//  ------------------------------------------------------------------
//  Get name of current quotebuffer filename

gstring& GetQuotebufName(gstring& quotebuf, gactivearea* area) {

  if(area->quotebuffile().size()) {
    quotebuf = area->quotebuffile();
    MakePathname(quotebuf, CFG->goldpath, quotebuf);
  }
  else {
    if(area->isezycom())
      quotebuf.strf("%sGLD%05u.QBF", CFG->ezycommsgbasepath.c_str(), area->board());
    else if(area->isfido())
      quotebuf.strf("%s%s", area->path().c_str(), "GOLDED.QBF");
    else if(area->isgoldbase())
      quotebuf.strf("%sGOLDG%03u.QBF", CFG->goldbasepath.c_str(), area->board());
    else if(area->ishudson())
      quotebuf.strf("%sGOLDH%03u.QBF", CFG->hudsonpath.c_str(), area->board());
    else
      quotebuf.strf("%s%s", area->path().c_str(), ".QBF");
  }

  return quotebuf;
}


//  ------------------------------------------------------------------

void QuoteBuf(GMsg& msg, gactivearea* area) {

  if(area->msgnset.size()) {

    gstring quotebuf;
    char openmode[4];

    TemplateToLines(MODE_QUOTEBUF, msg, msg, area, area, CFG->quotemargin);
    msg.lines_to_text();

    GetQuotebufName(quotebuf, area);
    w_infof(" %s ", quotebuf.c_str());

    switch(CFG->quotebufmode) {

      case QBUF_ASK:
        *openmode = NUL;
        if(fexist(quotebuf)) {
          GMenuOverwrite MenuOverwrite;
          switch(MenuOverwrite.Run()) {
            case true:
              strcpy(openmode, "wt");
              break;
            case false:
              strcpy(openmode, "at");
              break;
            default:
              *openmode = NUL;
          }
        }
        else {
          strcpy(openmode, "wt");
        }
        break;

      case QBUF_APPEND:
        strcpy(openmode, "at");
        break;

      case QBUF_OVERWRITE:
        strcpy(openmode, "wt");
        break;
    }

    if(*openmode) {

      FILE* fp = fsopen(quotebuf, openmode, CFG->sharemode);
      if(fp) {
        strchg(msg.txt, 0x0D, 0x0A);
        fputs(msg.txt, fp);
        fclose(fp);
      }

      HandleGEvent(EVTT_JOBDONE);
      waitkeyt(1000);
    }
    w_info(NULL);
  }
}


//  ------------------------------------------------------------------

