
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gescan.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Area scanning functions.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include <gmbprot.h>


//  ------------------------------------------------------------------

void arealist_page();


//  ------------------------------------------------------------------

void garealistdata::update_data() {

  if(scanned()) {

    uint unreadx = scancount() - lastread();
    if(scans() == 0)
      set_unread(unreadx);
    inc_scans();

    set_changed(unread() != unreadx);
    set_unread(unreadx);
    set_marked(unreadx);
  }
}


//  ------------------------------------------------------------------

void gactivearea::update_data() {

  if(scanned()) {
    set_scancount(area.scan_count);
    set_pmscancount(area.pmscan_count);
    set_lastread(area.lastread);
  }
  data.update_data();
}


//  ------------------------------------------------------------------

void gactivearea::scanarea() {

  if(cmdlinedebughg)
    LOG.printf("- scanarea: %s", echoid().c_str());

  area.scan_area();

  set_scanned();

  update_data();
}


//  ------------------------------------------------------------------

void gactivearea::scanareapm() {

  if(cmdlinedebughg)
    LOG.printf("- scanareapm: %s", echoid().c_str());

  area.scan_area_pm();

  set_scanned();
  set_pmscanned();

  update_data();
}


//  ------------------------------------------------------------------

int garealist::scan_areas(int mode, int currid, int pmscan, int& pmails, int& pmareas, const char* file) {

  gstrarray bag;
  int groupid = -1;

  // Load scan list into a string bag
  if(mode == SCAN_LIST) {
    gfile fp;
    Path tmp;
    strcpy(tmp, file ? file : listscan.file());
    char* listfile;
    char* option=tmp;
    getkeyval(&listfile, &option);
    fp.fopen(AddPath(CFG->goldpath, listfile), "rt");
    if(fp.isopen()) {
      char buf[512];
      while(fp.fgets(buf, sizeof(buf))) {
        strbtrim(buf);
        char* val = strtok(buf, ", \t");
        while(val) {
          bag.add(val);
          val = strtok(NULL, ", \t");
        }
      }
      fp.fclose();
      if(((*option == '-') or (*option == '/')) and strieql(option+1, "delete"))
        remove(listfile);
    }
  }
  else if(mode == SCAN_GROUP) {
    groupid = list[id_to_number(currid)]->groupid();
  }

  bool scanned = false;

  // For optimized overlay usage, sort arealist by msgbase type, path, board and echoid
  if(*CFG->areascansort) {
    sort(CFG->areascansort);
    scanned = true;
  }

  wide_open();

  for(int n=0; n<list.size(); n++) {

    // Check if ESC was pressed to skip the scan
    if(kbxhit()) {
      Key xch = kbxget();
      if(xch == Key_Esc)
        break;
      else
        kbput(xch);
    }

    if(not list[n]->isseparator()) {

      garealistdata* area = list[n];

      int scanit = false;

      int dopmscan     = pmscan and area->pmscan();
      int dopmscanexcl = pmscan and area->pmscanexcl();
      int dopmscanincl = pmscan and area->pmscanincl();

      int doscan     = area->scan()     or dopmscan;
      int doscanexcl = area->scanexcl() or dopmscanexcl;
      int doscanincl = area->scanincl() or dopmscanincl or (dopmscanexcl and doscan);

      switch(mode) {
        case SCAN_STARTUP:
          if(doscan and (not doscanexcl or doscanincl))
            scanit = true;
          break;
        case SCAN_ALL:
          if(not doscanexcl or doscanincl)
            scanit = true;
          break;
        case SCAN_CURRENT:
          scanit = area->areaid() == currid;
          break;
        case SCAN_MARKED:
          if(area->marked())
            scanit = true;
          break;
        case SCAN_MATCHING:
          if(striinc(area_maybe, area->echoid().c_str()))
            scanit = true;
          break;
        case SCAN_UNSCANNED:
          scanit = not (pmscan ? area->pmscanned() : area->scanned());
          break;
        case SCAN_GROUP:
          scanit = area->groupid() == groupid;
          break;
        case SCAN_NETMAIL:
          scanit = area->isnet();
          break;
        case SCAN_LIST:
          for(int b=0; b<bag.size(); b++) {
            if(strwild(area->echoid().c_str(), bag[b])) {
              scanit = true;
              break;
            }
          }
          break;
      }

      if(scanit) {
        gactivearea* a = activate_no(n);
        scanned = true;
        update_statuslinef("%s %s", 1+CFG->language[LNG_ScanningArea], a->echoid().c_str());
        if(dopmscan and (not dopmscanexcl or dopmscanincl)) {
          a->scanareapm();
          uint count = a->pmscancount();
          if(count) {
            pmails += count;
            pmareas++;
          }
          if(CFG->personalmail & PM_LISTONLY)
            a->pmrkset.reset();
        }
        else {
          a->scanarea();
        }
        deactivate(a);
      }
    }
  }

  wide_close();

  return scanned;
}


//  ------------------------------------------------------------------

void CheckSemaphores() {

  Path file;
  int scanned = 0;
  int pmareas = 0;
  int pmails = 0;

  if(fexist(CFG->semaphore.scanall)) {
    remove(CFG->semaphore.scanall);
    scanned += AL.scan_areas(SCAN_ALL, 0, false, pmails, pmareas);
  }

  if(fexist(CFG->semaphore.scanthis)) {
    sprintf(file, "%s -delete", CFG->semaphore.scanthis);
    scanned += AL.scan_areas(SCAN_LIST, 0, false, pmails, pmareas, file);
  }

  if(fexist(CFG->semaphore.scannetmail)) {
    remove(CFG->semaphore.scannetmail);
    scanned += AL.scan_areas(SCAN_NETMAIL, 0, false, pmails, pmareas);
  }

  if(fexist(CFG->semaphore.pmscanall)) {
    remove(CFG->semaphore.pmscanall);
    scanned += AL.scan_areas(SCAN_ALL, 0, true, pmails, pmareas);
  }

  if(fexist(CFG->semaphore.pmscanthis)) {
    sprintf(file, "%s -delete", CFG->semaphore.pmscanthis);
    scanned += AL.scan_areas(SCAN_LIST, 0, true, pmails, pmareas, file);
  }

  if(fexist(CFG->semaphore.pmscannetmail)) {
    remove(CFG->semaphore.pmscannetmail);
    scanned += AL.scan_areas(SCAN_NETMAIL, 0, true, pmails, pmareas);
  }

  if(fexist(CFG->semaphore.qwkimport)) {
    remove(CFG->semaphore.qwkimport);
    ImportQWK();
    scanned++;
  }

  if(fexist(CFG->semaphore.qwkexport)) {
    remove(CFG->semaphore.qwkexport);
    ExportQWK();
  }

  if(fexist(CFG->semaphore.soupimport)) {
    remove(CFG->semaphore.soupimport);
    ImportSOUP();
    scanned++;
  }

  if(fexist(CFG->semaphore.soupexport)) {
    remove(CFG->semaphore.soupexport);
    ExportSOUP();
  }

  if(scanned) {
    AL.sort();
    arealist_page();
  }
}


//  ------------------------------------------------------------------

