
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gesoup.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Internet SOUP packet handling.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

char* CvtMSGIDtoMessageID(char* midbuf, char* msgid, ftn_addr* addr, char* domain, char* from, char* to, char* subject, Tm* datetime) {

  if(msgid) {
    char* mptr = msgid;
    char* eptr = strrchr(msgid, ' ');
    int msgidrfc = false;
    int quoted = (*msgid == '\"');

    *midbuf = NUL;

    // Find end of quoted string
    if(quoted) {
      mptr++;
      eptr--;
      if(*eptr != '\"')
        return midbuf;
    }

    if(*mptr == '<') {
      char* ptr = strchr(mptr, '@');
      if(ptr and (ptr < eptr)) {
        ptr = strchr(ptr, '>');
        if(ptr and (ptr < eptr)) {
          msgidrfc = true;
          eptr = ptr;
        }
        else
          return midbuf;
      }
    }

    char* bptr = midbuf;
    if(msgidrfc) {
      while(mptr <= eptr) {
        if(quoted and (*mptr == '\"'))
          if(mptr[1] == '\"')
            mptr++;
        *bptr++ = *mptr++;
      }
      *bptr = NUL;
    }
    else {
      if(quoted)
        mptr--;
      bptr = stpcpy(bptr, "<MSGID_");
      while(*mptr) {
        if(*mptr == ' ') {
          *bptr++ = '_';
          mptr++;
        }
        else if((*mptr < ' ') or strchr("()<>@,;:\\\"[]/=_", *mptr) or (*mptr >= 0x80)) {
          sprintf(bptr, "=%02X", *mptr++);
          bptr += 3;
        }
        else {
          *bptr++ = *mptr++;
        }
      }
      *bptr++ = '@';
      bptr = stpcpy(bptr, domain);
      *bptr++ = '>';
      *bptr = NUL;
    }
  }
  else {
    dword crc32 = CRC32_MASK_CCITT;
    crc32 = strCrc32(from,    NO, crc32);
    crc32 = strCrc32(to,      NO, crc32);
    crc32 = strCrc32(subject, NO, crc32);
    crc32 ^= CRC32_MASK_CCITT;
    sprintf(midbuf,
      "<NOMSGID_%u=3A%u=2F%u.%u_%02u%02u%02u_%02u%02u%02u_%08lx@%s>",
      addr->zone, addr->net, addr->node, addr->point,
      datetime->tm_year, datetime->tm_mon+1, datetime->tm_mday,
      datetime->tm_hour, datetime->tm_min,   datetime->tm_sec,
      crc32, domain
    );
  }

  return midbuf;
}


//  ------------------------------------------------------------------
                      
char* CvtMessageIDtoMSGID(const char* mptr, char* msgidbuf, const char* echoid, char* kludge) {

  sprintf(msgidbuf, "\x1""%s: ", kludge);
  char* bptr = msgidbuf + strlen(msgidbuf);

  if(strnieql(mptr, "<MSGID_", 7)) {
    mptr += 7;
    while((*mptr != '@') and *mptr) {
      if(*mptr == '=') {
        *bptr++ = (char)((ctox(mptr[1]) << 4) | ctox(mptr[2]));
        mptr += 3;
      }
      else if(*mptr == '_') {
        *bptr++ = ' ';
        mptr++;
      }
      else {
        *bptr++ = *mptr++;
      }
    }
    *bptr = NUL;
  }
  else {
    int spaces = strchr(mptr, ' ') ? true : false;
    dword crc32 = CRC32_MASK_CCITT;
    crc32 = strCrc32(mptr, NO, crc32);
    crc32 = strCrc32(echoid, YES, crc32);
    crc32 ^= CRC32_MASK_CCITT;
    if(spaces)
      *bptr++ = '\"';
    while(*mptr) {
      if(spaces and (*mptr == '\"'))
        *bptr++ = '\"';
      *bptr++ = *mptr++;
    }
    if(spaces)
      *bptr++ = '\"';
    sprintf(bptr, " %08lx", crc32);
  }

  return msgidbuf;
}


//  ------------------------------------------------------------------

char* UnfoldLine(char* mptr) {

  register char* eptr = mptr;
  do {
    eptr = strskip_to(eptr, '\n');
    if(*eptr) {
      register char lwsp = eptr[1];
      if((lwsp == ' ') or (lwsp == '\t'))
        *eptr = lwsp;
      else {
        *eptr++ = NUL;
        break;
      }
    }
  } while(*eptr);

  return eptr;
}


//  ------------------------------------------------------------------
//  Display a "twirly"

#if 0

#define TWIRLY_FACTOR 10
#define ISTWIRLY(n) (((n)%TWIRLY_FACTOR)==0)

static char twirly() {

  static int n=0;

  switch((n++)%4) {
    case 0: return '|';
    case 1: return '/';
    case 2: return '-';
    case 3: return '\\';
  }
  return ' ';
}

#endif


//  ------------------------------------------------------------------

void ProcessSoupMsg(char* lbuf, GMsg& msg, int& msgs, char* areaname, int tosstobadmsgs, gtag& tossed, gactivearea* area) {

  gactivearea* entryarea = area;

  if(msg.txt) {

    msgs++;

    msg.orig = msg.oorig = CFG->internetgate.addr.net ? CFG->internetgate.addr : area->aka();
    msg.dest = msg.odest = area->aka();
    msg.date.arrived = gtime(NULL);

    int lineno = 0;
    int inhdr = true;
    char* mptr = msg.txt;
    Orig origin = { "" };
    char ftnmsgid[201] = { "" };
    char ftnreply[201] = { "" };

    if(tosstobadmsgs) {
      char buf[256];
      sprintf(buf, "AREA:%s", areaname);
      msg.linearray.add(buf, GLINE_HARD);
    }

    while(*mptr) {

      register char* eptr = mptr;
      do {
        eptr = strskip_to(eptr, '\n');
        if(*eptr) {
          register char lwsp = eptr[1];
          if(inhdr and (mptr != eptr) and ((lwsp == ' ') or (lwsp == '\t'))) {
            *eptr = lwsp;
          }
          else {
            *eptr++ = NUL;
            break;
          }
        }
      } while(*eptr);

      strchg(mptr, '\t', ' ');

      if(inhdr) {
        if(*mptr == NUL) {
          inhdr = false;
          if(*ftnmsgid)
            msg.linearray.add(ftnmsgid, GLINE_HARD|GLINE_KLUD);
          if(*ftnreply)
            msg.linearray.add(ftnreply, GLINE_HARD|GLINE_KLUD);
        }
        else {

          bool addkludge = true;
          sprintf(lbuf, "\001RFC-%s", mptr);

          if(strnieql(mptr, "From: ", 6)) {
            gstring fromname;
            gstring fromaddr;
            mptr = strskip_wht(mptr+6);
            ParseInternetAddr(mptr, fromname, fromaddr);
            msg.by = fromname.length() ? fromname : fromaddr;
            if(CFG->internetgate.addr.net) {
              char abuf[40];
              char kbuf[2048];
              sprintf(kbuf, "\x1""REPLYTO %s %s",
                CFG->internetgate.addr.make_string(abuf),
                *CFG->internetgate.name ? CFG->internetgate.name : "UUCP"
              );
              msg.linearray.add(kbuf, GLINE_HARD|GLINE_KLUD);
              sprintf(kbuf, "\x1""REPLYADDR %s", fromaddr.c_str());
              msg.linearray.add(kbuf, GLINE_HARD|GLINE_KLUD);
            }
          }
          else if(strnieql(mptr, "To: ", 4)) {
            gstring toname;
            gstring toaddr;
            mptr = strskip_wht(mptr+4);
            ParseInternetAddr(mptr, toname, toaddr);
            msg.to = toname.length() ? toname : toaddr;
          }
          else if(strnieql(mptr, "X-To: ", 6)) {
            gstring toname;
            gstring toaddr;
            mptr = strskip_wht(mptr+6);
            ParseInternetAddr(mptr, toname, toaddr);
            msg.to = toname.length() ? toname : toaddr;
          }
          else if(strnieql(mptr, "Subject: ", 9)) {
            mptr = strskip_wht(mptr+9);
            msg.re = mptr;
          }
          else if(strnieql(mptr, "Date: ", 6)) {
            char weekbuf[4], monthbuf[4], delim;
            int year, month, day, hour, minute, second;
            char* ptr = strskip_wht(mptr+6);
            if(isdigit(*ptr)) {
              sscanf(ptr, " %d %3s %d %d%c%d%c%d",
                &day, monthbuf, &year,
                &hour, &delim, &minute, &delim, &second
              );
            }
            else {
              sscanf(ptr, " %3s, %d %3s %d %d%c%d%c%d",
                weekbuf, &day, monthbuf, &year,
                &hour, &delim, &minute, &delim, &second
              );
            }
            month = str2mon(monthbuf);
            Tm tm;
            tm.tm_year  = (year < 80) ? (year+100) : year;
            tm.tm_mon   = month - 1;
            tm.tm_mday  = day;
            tm.tm_hour  = hour;
            tm.tm_min   = minute;
            tm.tm_sec   = second;
            tm.tm_isdst = 0;
            msg.date.written = gmktime(&tm);
          }
          else if(strnieql(mptr, "Organization: ", 14)) {
            strxcpy(origin, strskip_wht(mptr+14), sizeof(Orig));
          }
          else if(strnieql(mptr, "Message-ID: ", 12)) {
            mptr = strskip_wht(mptr+12);
            char msgidbuf[200];
            CvtMessageIDtoMSGID(mptr, msgidbuf, area->echoid().c_str(), "MSGID");
            if(not strnieql(mptr, "<NOMSGID_", 9))
              strxcpy(ftnmsgid, msgidbuf, 201);
          }
          else if(strnieql(mptr, "References: ", 12)) {
            mptr = strskip_wht(mptr+12);
            char msgidbuf[200];
            while(*mptr) {
              char* sptr = strpbrk(mptr, " ,>");
              if(sptr == NULL)
                sptr = mptr + strlen(mptr);
              while(((*sptr == ' ') or (*sptr == ',') or (*sptr == '>')) and *sptr)
                *sptr++ = NUL;
              CvtMessageIDtoMSGID(mptr, msgidbuf, area->echoid().c_str(), "REPLY");
              if(not strnieql(mptr, "<NOMSGID_", 9))
                strxcpy(ftnreply, msgidbuf, 201);
              mptr = sptr;
            }
          }
          else if(strnieql(mptr, "In-Reply-To: ", 13)) {
            mptr = strskip_wht(mptr+12);
            char msgidbuf[200];
            CvtMessageIDtoMSGID(mptr, msgidbuf, area->echoid().c_str(), "REPLY");
            if(not strnieql(mptr, "<NOMSGID_", 9))
              strxcpy(ftnreply, msgidbuf, 201);
          }
          else if(strnieql(mptr, "Sender: ", 8)) {
            mptr = strskip_wht(mptr+7);
            gstring sendername;
            gstring senderaddr;
            ParseInternetAddr(mptr, sendername, senderaddr);
            int n = 0;
            while(n < CFG->mailinglists) {
              if(senderaddr.ieql(CFG->mailinglist[n].sender)) {
                int areaid = AL.echoid_to_id(CFG->mailinglist[n].echoid);
                if(areaid != -1) {
                  area = AL.activate(areaid);
                  area->open();
                  area->lock();
                }
                break;
              }
              n++;
            }
          }

          if(addkludge)
            msg.linearray.add(lbuf, GLINE_HARD|GLINE_KLUD);
        }
      }
      else {
        msg.linearray.add(mptr, GLINE_HARD);
        #if 0
        if(ISTWIRLY(lineno))
          update_statuslinef("%s: %u (%c)", area->echoid(), msgs, twirly());
        #endif
        lineno++;
      }

      mptr = eptr;
    }

    if(msg.to.empty())
      msg.to = "All";

    tossed += area->areaid();

    update_statuslinef("%s: %u", area->echoid().c_str(), msgs);

    msg.area = area;
    msg.lines_to_text();
    area->savemsg(GMSG_NEW, msg);
    msg.reset();

    if(entryarea != area) {
      area->inc_scans();
      area->unlock();
      area->close();
      AL.deactivate(area);
    }
  }
}


//  ------------------------------------------------------------------

dword swapendian(dword n) {

  byte* c = (byte*)&n;
  n = ((c[0]*256 + c[1])*256 + c[2])*256 + c[3];
  return n;
}


//  ------------------------------------------------------------------

int ImportSOUP() {

  int imported = 0;

  if(*CFG->soupimportpath) {

    const int MBUF_SIZE = 16384;
    const int LBUF_SIZE = 16384;

    gfile fpa;   // For AREAS file
    gfile fpm;   // For *.MSG files

    int importedmsgs = 0;

    Path areasfile;
    strcpy(areasfile, AddPath(CFG->soupimportpath, "AREAS"));
    fpa.fopen(areasfile, "rt");
    if(fpa.isopen()) {

      gtag tossed;

      char buf[2048];

      XLAT->load_charset("N/A", "N/A");

      char* mbuf = (char*)throw_malloc(MBUF_SIZE);
      char* lbuf = (char*)throw_malloc(LBUF_SIZE);

      GMsg msg;

      while(fpa.fgets(buf, sizeof(buf))) {

        char* delim = "\t\n";
        char* prefix   = strtok(buf,  delim);
        char* areaname = strtok(NULL, delim);
        char* encoding = strtok(NULL, delim);

        char msgfmt = *encoding++;
        char idxfmt = (char)(*encoding ? *encoding++ : 0);
        char kind   = (char)(*encoding ? *encoding   : 0);

        int isemail = false;
        int isnews = false;
        switch(msgfmt) {
          case 'M':
            // not supported yet
            break;
          case 'm':
          case 'b':
            isemail = true;
            break;
          case 'u':
            if(kind == 'm') {
              isemail = true;
              break;
            }
          case 'B':
            isnews = true;
            break;
        }

        Path msgfile, idxfile;
        strcpy(msgfile, prefix);
        strcat(msgfile, ".MSG");
        strcpy(msgfile, AddPath(CFG->soupimportpath, msgfile));
        if(idxfmt) {
          strcpy(idxfile, prefix);
          strcat(idxfile, ".IDX");
          strcpy(idxfile, AddPath(CFG->soupimportpath, idxfile));
        }

        strupr(areaname);
        int tosstobadmsgs = false;
        if(isemail)
          areaname = CFG->soupemail;
        int areaid = AL.echoid_to_id(areaname);
        if(areaid == -1) {
          areaid = AL.echoid_to_id(CFG->soupbadmsgs);
          tosstobadmsgs = true;
        }

        if((areaid != -1) and (isemail or isnews)) {

          gactivearea* area = AL.activate(areaid);

          fpm.fopen(msgfile, "rb");
          if(fpm.isopen()) {

            imported++;

            int msgs = 0;

            area->open();
            area->lock();

            char* txtptr = NULL;
            uint txtlen = 0;

            if((msgfmt == 'b') or (msgfmt == 'B')) {

              // Get binary formats

              dword msglen = 0;
              while(fpm.fread(&msglen, 4) == 1) {

                msglen = swapendian(msglen);
                uint msglensz = (uint)msglen;
                if(msglen != msglensz)
                  msglensz--;
                msg.txt = (char*)throw_calloc(1, msglensz+1);
                fpm.fread(msg.txt, msglensz);
                if(msglen != msglensz)
                  fpm.fseek(msglen-msglensz, SEEK_CUR);
                ProcessSoupMsg(lbuf, msg, msgs, areaname, tosstobadmsgs, tossed, area);
              }
            }
            else {

              // Get non-binary formats

              while(fpm.fgets(mbuf, MBUF_SIZE)) {

                if(msgfmt == 'u') {
                  if(strneql(mbuf, "#! rnews ", 9)) {
                    dword msglen = atol(mbuf+9);
                    uint msglensz = (uint)msglen;
                    if(msglen != msglensz)
                      msglensz--;
                    msg.txt = (char*)throw_calloc(1, msglensz+1);
                    fpm.fread(msg.txt, msglensz);
                    if(msglen != msglensz)
                      fpm.fseek(msglen-msglensz, SEEK_CUR);
                    ProcessSoupMsg(lbuf, msg, msgs, areaname, tosstobadmsgs, tossed, area);
                  }
                  else {
                    w_infof(CFG->language[LNG_ErrorInSoup], msgfile);
                    waitkeyt(10000);
                    w_info(NULL);
                    break;
                  }
                }
                else if(msgfmt == 'm') {
                  if(strneql(mbuf, "From ", 5)) {
                    msg.txt = txtptr;
                    txtptr = NULL;
                    txtlen = 0;
                    ProcessSoupMsg(lbuf, msg, msgs, areaname, tosstobadmsgs, tossed, area);
                  }
                  uint len = strlen(mbuf);
                  txtptr = (char*)throw_realloc(txtptr, txtlen+len+1);
                  strcpy(txtptr+txtlen, mbuf);
                  txtlen += len;
                }
                else if(msgfmt == 'M') {
                  // Not supported yet
                }
              }

              if(msgfmt == 'm') {
                if(txtptr) {
                  msg.txt = txtptr;
                  txtptr = NULL;
                  ProcessSoupMsg(lbuf, msg, msgs, areaname, tosstobadmsgs, tossed, area);
                }
              }
            }

            throw_free(txtptr);

            area->inc_scans();
            area->unlock();
            area->close();
            AL.deactivate(area);

            if(msgs)
              importedmsgs += msgs;

            fpm.fclose();
          }
        }

        remove(msgfile);
        if(idxfmt)
          remove(idxfile);
      }

      throw_free(lbuf);
      throw_free(mbuf);

      fpa.fclose();
      remove(areasfile);

      if(*CFG->souptosslog) {
        fpa.fopen(CFG->souptosslog, "at");
        if(fpa.isopen()) {
          for(int n=0; n<tossed.size(); n++) {
            garealistdata* a = AL.id_to_pointer(tossed[n]);
            a->set_changed();
            fpa.printf("%s\n", a->echoid().c_str());
          }
          fpa.fclose();
        }
      }

      if(importedmsgs and *CFG->soupreplylinker) {
        sprintf(buf, CFG->language[LNG_Replylinker], CFG->soupreplylinker);
        ShellToDos(CFG->soupreplylinker, buf, LGREY|_BLACK, YES);
      }
    }
  }

  if(imported) {
    write_goldlast();
  }

  return imported;
}


//  ------------------------------------------------------------------

int ExportSoupMsg(GMsg& msg, char* msgfile, gfile& fp, int ismail, gactivearea* area) {

  NW(ismail);

  if(not fp.isopen()) {
    fp.open(AddPath(CFG->soupexportpath, msgfile), O_RDWR|O_CREAT|O_BINARY, "rb+");
    if(fp.isopen())
      fp.fseek(0, SEEK_END);
  }

  if(fp.isopen()) {

    char mbuf[512];

    // Write placeholder for message length
    dword msglen = 0xFFFFFFFFL;
    fp.fwrite(&msglen, 4);
    msglen = 0;

    bool qp = false;
    if(msg.header.x_char_esc.length() and msg.header.mime_version.empty()) {
      if(not striinc("MNEMONIC", XLAT->set.charset.export))
        XLAT->load_charset(XLAT->localset, "MNEMONIC");
    }
    else if(striinc("LATIN1QP", msg.charset.c_str())) {
      qp = true;
      if(not striinc("LATIN1QP", XLAT->set.charset.export))
        XLAT->load_charset(XLAT->localset, "LATIN1QP");
    }
    else if(striinc("LATIN-1", msg.charset.c_str())) {
      if(not striinc("LATIN-1", XLAT->set.charset.export))
        XLAT->load_charset(XLAT->localset, "LATIN-1");
    }

    // Process kludges and write header lines
    for(int n=0; n<msg.linearray.size(); n++) {
      if(msg.linearray.data(n).iskludge()) {
        if((msg.linearray.data(n).kludge == GKLUD_RFC) or (msg.linearray.data(n).kludge == 0)) {
          XLAT->translate(mbuf, (*msg.linearray[n] == CTRL_A) ? msg.linearray[n]+1 : msg.linearray[n]);
          msglen += fp.printf("%s%s\n", (msg.linearray.data(n).kludge == 0) ? " " : "", mbuf);
        }
        else if(msg.linearray.data(n).iswrapped()) {
          while((n+1 < msg.linearray.size()) and msg.linearray.data(n).iswrapped())
            n++;
        }
      }
    }

    // Write blank line after header lines
    msglen += fp.printf("\n");

    // Write all message lines
    for(int i=0; i<msg.linearray.size(); i++) {
      if(not (msg.linearray.data(i).type & (GLINE_HIDD|GLINE_KLUD|GLINE_TEAR|GLINE_ORIG))) {
        XLAT->translate(mbuf, msg.linearray[i]);
        char* mptr = mbuf;
        if(qp and strlen(mptr) > 76) {
          // 12345v7890
          //     =FF    - back one
          //    =FF     - back two
          //   =FF      - okay
          do {
            char* mbeg = mptr;
            mptr += 75;
            if(*(mptr-2) == '=')
              mptr -= 2;
            else if(*(mptr-1) == '=')
              mptr--;
            int mlen = (int)(mptr - mbeg);
            msglen += fp.printf("%*.*s=\n", mlen, mlen, mbeg);
          } while(strlen(mptr) > 76);
        }
        msglen += fp.printf("%s\n", mptr);
      }
    }

    // Re-write the correct message length
    fp.fseek(-(msglen+4), SEEK_CUR);
    dword be_msglen = swapendian(msglen);
    fp.fwrite(&be_msglen, 4);
    fp.fseek(msglen, SEEK_CUR);

    msg.attr.snt1();
    msg.attr.scn1();
    msg.attr.uns0();
    area->savehdr(GMSG_UPDATE, msg);

    return 1;
  }

  return 0;
}


//  ------------------------------------------------------------------

int ExportSoupArea(int areaid, char* msgfile, gfile& fp, int ismail) {

  int exported = 0;

  gactivearea* area = AL.activate(areaid);

  area->open();
  area->lock();

  GMsg msg;

  for(int n=0; n<area->markset.size(); n++) {
    if(area->loadmsg(msg, area->markset[n], 76)) {
      if(msg.attr.uns() and not msg.attr.del()) {
        exported += ExportSoupMsg(msg, msgfile, fp, ismail, area);
        update_statuslinef("%s: %u", area->echoid().c_str(), exported);
      }
    }
  }

  area->inc_scans();
  area->unlock();
  area->close();
  AL.deactivate(area);

  return exported;
}


//  ------------------------------------------------------------------

int ExportSOUP() {

  int mailexported = 0;
  int newsexported = 0;

  if(*CFG->soupexportpath) {

    Path scanfile;
    gfile fp, mfp, nfp;

    // Get the scan list
    strcpy(scanfile, AddPath(CFG->goldpath, "GOLDSOUP.LST"));
    fp.fopen(scanfile, "rt");
    if(fp.isopen()) {
      char buf[256];
      while(fp.fgets(buf, sizeof(buf))) {
        char* ptr = strchr(buf, ' ');
        if(ptr) {
          *ptr++ = NUL;
          int a = AL.echoid_to_number(buf);
          if(a != -1)
            AL[a]->markset.add(atol(ptr));
        }
      }
      fp.fclose();
    }

    // Export from the e-mail and newsgroup areas
    for(int na=0; na<AL.areas(); na++) {
      garealistdata* ap = AL[na];
      if(ap->markset.size()) {
        if(ap->isemail())
          mailexported += ExportSoupArea(ap->areaid(), "GOLDMAIL.MSG", mfp, true);
        else if(ap->isnewsgroup())
          newsexported += ExportSoupArea(ap->areaid(), "GOLDNEWS.MSG", nfp, false);
      }
    }

    // Close any open SOUP files
    if(mfp.isopen())
      mfp.fclose();
    if(nfp.isopen())
      nfp.fclose();

    // Update the REPLIES file
    if(mailexported or newsexported) {
      fp.open(AddPath(CFG->soupexportpath, "REPLIES"), O_RDWR|O_CREAT|O_BINARY, "rb+");
      if(fp.isopen()) {
        char buf[512];
        int hasmail = false;
        int hasnews = false;
        while(fp.fgets(buf, sizeof(buf))) {
          strtok(buf, "\t\n");
          if(strieql(buf, "GOLDMAIL"))
            hasmail = true;
          else if(strieql(buf, "GOLDNEWS"))
            hasnews = true;
        }
        if(mailexported and not hasmail)
          fp.printf("GOLDMAIL\tmail\tbn\n");
        if(newsexported and not hasnews)
          fp.printf("GOLDNEWS\tnews\tBn\n");
        fp.fclose();
      }
    }

    // Delete the scanfile
    remove(scanfile);
  }

  return mailexported + newsexported;
}


//  ------------------------------------------------------------------

