
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: getpls.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Template handling.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

enum TPL_TOKEN_IDS {

  TPLTOKEN_FORWARD,
  TPLTOKEN_CHANGED,
  TPLTOKEN_NET,
  TPLTOKEN_ECHO,
  TPLTOKEN_LOCAL,
  TPLTOKEN_MOVED,
  TPLTOKEN_NEW,
  TPLTOKEN_REPLY,
  TPLTOKEN_QUOTED,
  TPLTOKEN_COMMENT,
  TPLTOKEN_QUOTEBUF,
  TPLTOKEN_ATTRIB,
  TPLTOKEN_SPELLCHECKER,
  TPLTOKEN_SETSUBJ,
  TPLTOKEN_SETFROM,
  TPLTOKEN_SETTO,
  TPLTOKEN_FORCESUBJ,
  TPLTOKEN_FORCEFROM,
  TPLTOKEN_FORCETO,
  TPLTOKEN_XLATEXPORT,
  TPLTOKEN_LOADLANGUAGE,
  TPLTOKEN_RANDOM,
  TPLTOKEN_QUOTE,
  TPLTOKEN_INCLUDE,
  TPLTOKEN_MESSAGE,
  TPLTOKEN_CONFIRM
};


//  ------------------------------------------------------------------

struct tpl_token {

  char* token;
  int   length;
};


//  ------------------------------------------------------------------

#define CSTR_COMMA_SIZEOF_CSTR(s) s, (sizeof(s)-1)

tpl_token token_list[] = {

  { CSTR_COMMA_SIZEOF_CSTR("forward")      },
  { CSTR_COMMA_SIZEOF_CSTR("changed")      },
  { CSTR_COMMA_SIZEOF_CSTR("net")          },
  { CSTR_COMMA_SIZEOF_CSTR("echo")         },
  { CSTR_COMMA_SIZEOF_CSTR("local")        },
  { CSTR_COMMA_SIZEOF_CSTR("moved")        },
  { CSTR_COMMA_SIZEOF_CSTR("new")          },
  { CSTR_COMMA_SIZEOF_CSTR("reply")        },
  { CSTR_COMMA_SIZEOF_CSTR("quoted")       },
  { CSTR_COMMA_SIZEOF_CSTR("comment")      },
  { CSTR_COMMA_SIZEOF_CSTR("quotebuf")     },
  { CSTR_COMMA_SIZEOF_CSTR("attrib")       },
  { CSTR_COMMA_SIZEOF_CSTR("spellchecker") },
  { CSTR_COMMA_SIZEOF_CSTR("setsubj")      },
  { CSTR_COMMA_SIZEOF_CSTR("setfrom")      },
  { CSTR_COMMA_SIZEOF_CSTR("setto")        },
  { CSTR_COMMA_SIZEOF_CSTR("forcesubj")    },
  { CSTR_COMMA_SIZEOF_CSTR("forcefrom")    },
  { CSTR_COMMA_SIZEOF_CSTR("forceto")      },
  { CSTR_COMMA_SIZEOF_CSTR("xlatexport")   },
  { CSTR_COMMA_SIZEOF_CSTR("loadlanguage") },
  { CSTR_COMMA_SIZEOF_CSTR("random")       },
  { CSTR_COMMA_SIZEOF_CSTR("quote")        },
  { CSTR_COMMA_SIZEOF_CSTR("include")      },
  { CSTR_COMMA_SIZEOF_CSTR("message")      },
  { CSTR_COMMA_SIZEOF_CSTR("confirm")      }
};


//  ------------------------------------------------------------------

int end_token = sizeof(token_list) / sizeof(tpl_token);


//  ------------------------------------------------------------------

char* template_token(int& token, char* ptr) {

  for(token=0; token<end_token; token++) {
    if(strnieql(token_list[token].token, ptr, token_list[token].length))
      return ptr + token_list[token].length;
  }
  return NULL;
}


//  ------------------------------------------------------------------

inline int is_initial(char c) {

  return isalpha(c) or (c > 127);
}
 

//  ------------------------------------------------------------------

int make_quotestring(char* buf, const char* name, const char* quotestring) {

  int initials_len = 0;
  char middle[10] = { "" };
  char initials[10];

  const char* ptr = strskip_wht(name);
  while(*ptr) {
    while(not is_initial(*ptr) and (*ptr != '@') and *ptr)
      ptr++;
    if(*ptr == '@')
      break;
    if(*ptr) {
      initials[initials_len++] = *ptr;
      if(initials_len == 9)
        break;
      ptr++;
    }
    while(is_initial(*ptr) and *ptr)
      ptr++;
  }
  initials[initials_len] = NUL;

  if(initials_len > 2) {
    for(int n=1; n<initials_len-1; n++)
      middle[n-1] = initials[n];
    middle[n-1] = NUL;
  }

  char* bptr = buf;
  ptr = quotestring;
  int quotestring_len = strlen(quotestring);

  for(int n=0; n<quotestring_len; n++,ptr++) {
    switch(*ptr) {
      case 'F':
      case 'f':
        if(*initials) {
          *bptr++ = *initials;
          *bptr = NUL;
        }
        break;
      case 'M':
      case 'm':
        strcat(bptr, middle);
        break;
      case 'L':
      case 'l':
        if(initials_len > 1) {
          *bptr++ = initials[initials_len-1];
          *bptr = NUL;
        }
        break;
      default:
        *bptr++ = *ptr;
        *bptr = NUL;
    }
  }

  ptr = strskip_wht(buf);

  return (int)(ptr-buf);
}


//  ------------------------------------------------------------------

void template_quote(GMsg& msg, GMsg& omsg, gactivearea* destarea) {

  gmsg_quote quoting(destarea->quotestring().c_str(), CFG->dispsoftcr);

  char quotestr[40];
  int qstrlen = make_quotestring(quotestr, omsg.oname.c_str(), destarea->quotestring().c_str());

  glineref line = omsg.linearray;

  for(int n=0; n<omsg.display.size(); n++) {

    line = omsg.display[n];

    char* quote = throw_xstrdup(line.text());

    if(line.istearline()) {
      line.type() &= ~GLINE_TEAR;
      if(*CFG->invalidate.tearline[0] == NUL)
        continue;
    }
    else if(line.isorigin()) {
      line.type() &= ~GLINE_ORIG;
      if(*CFG->invalidate.origin[0] == NUL)
        continue;
    }

    if(*CFG->invalidate.tearline[0])
      strischg(quote, CFG->invalidate.tearline[0], CFG->invalidate.tearline[1]);
    if(*CFG->invalidate.origin[0])
      strischg(quote, CFG->invalidate.origin[0], CFG->invalidate.origin[1]);
    if(*CFG->invalidate.seenby[0])
      strischg(quote, CFG->invalidate.seenby[0], CFG->invalidate.seenby[1]);
    strchg(quote, CTRL_A, '@');

    char buf[256];

    if(quoting.is_quote(line.text())) {
      int qbuflen;
      char qbuf[40];
      quote += quoting.get_quotestring(quote, qbuf, qbuflen);
      while(isspace(*quote))
        quote++;
      strbtrim(qbuf);
      char* ptr = qbuf;
      if(not quoting.is_quotechar(ptr)) {
        ptr = qbuf+strlen(qbuf);
        while(ptr >= qbuf) {
          if(quoting.is_quotechar(ptr))
            break;
          ptr--;
        }
      }
      qbuflen = (int)(ptr-qbuf);
      sprintf(buf, "%*.*s%*.*s>%s %s",
        qstrlen, qstrlen, quotestr, qbuflen, qbuflen, qbuf, qbuf+qbuflen, quote
      );
    }
    else if((not strblank(quote)) or CFG->quoteblank)
      sprintf(buf, "%s%s", quotestr, quote);
    else
      *buf = NUL;
    strtrimspace(buf);
    msg.linearray.add(buf, GLINE_HARD);
  }
}


//  ------------------------------------------------------------------

void template_message(int mode, GMsg& msg, GMsg& omsg) {

  char buf[256];
  glineref line = omsg.linearray;

  for(int n=0; n<omsg.display.size(); n++) {

    line = omsg.display[n];

    strcpy(buf, line.text());

    if(mode == MODE_FORWARD) {
      if(line.istearline()) {
        // Invalidate tearline
        line.type() &= ~GLINE_TEAR;
        if(*CFG->invalidate.tearline[0] == NUL)
          continue;
        strischg(buf, CFG->invalidate.tearline[0], CFG->invalidate.tearline[1]);
      }
      else if(line.isorigin()) {
        // Invalidate originline
        line.type() &= ~GLINE_ORIG;
        if(*CFG->invalidate.origin[0] == NUL)
          continue;
        strischg(buf, CFG->invalidate.origin[0], CFG->invalidate.origin[1]);
      }
      // Invalidate SEEN-BY's
      if(*CFG->invalidate.seenby[0])
        strischg(buf, CFG->invalidate.seenby[0], CFG->invalidate.seenby[1]);
      // Invalidate kludge chars
      strchg(buf, CTRL_A, '@');
    }

    msg.linearray.add(buf, line.type());
  }
}


//  ------------------------------------------------------------------

void template_random(int mode, char* ptr, GMsg& msg, GMsg& omsg, gactivearea* destarea, gactivearea* origarea) {

  Path textfile, indexfile;

  strtrim(ptr);
  ptr = strskip_txt(ptr);
  ptr = strskip_wht(ptr);
  if(*ptr) {
    char* tptr = ptr;
    ptr = strskip_txt(ptr);
    if(*ptr) {
      *ptr++ = NUL;
      strcpy(textfile, tptr);
      ptr = strskip_wht(ptr);
      if(*ptr) {
        tptr = ptr;
        ptr = strskip_txt(ptr);
        *ptr = NUL;
        strcpy(indexfile, tptr);
      }
    }
    else {
      strcpy(textfile, tptr);
    }
  }
  else {
    strcpy(textfile, "RANDOM.TXT");
  }

  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  _splitpath(textfile, drive, dir, file, ext);

  strcpy(ext, ".MDX");
  _makepath(indexfile, drive, dir, file, ext);

  MakePathname(textfile, CFG->cookiepath, textfile);
  MakePathname(indexfile, CFG->cookiepath, indexfile);

  // Check if index exists or if it is older than the textfile
  int idxexist = fexist(indexfile);
  if(idxexist)
    if(FiletimeCmp(textfile, indexfile) > 0)
      idxexist = false;

  // If index file is missing, make one
  if(not idxexist)
    CookieIndex(textfile, indexfile);

  // Get a random cookie
  gfile tfp(textfile, "rt");
  if(tfp) {
    gfile ifp(indexfile, "rb");
    if(ifp) {
      ifp.fseek(0L, SEEK_END);
      int idxs = (int)(ifp.ftell()/4L);
      if(idxs) {

        long fpos;
        ifp.fseekset((long)(rand()%idxs)*4L);
        ifp.fread(&fpos, sizeof(long));
        tfp.fseekset(fpos);

        char buf[256];
        while(tfp.fgets(buf, sizeof(buf))) {
          if(*buf != '\n') {
            if(*buf == '+')
              *buf = ' ';
            TokenXlat(mode, buf, msg, omsg, destarea, origarea);
            strtrim(buf);
            msg.linearray.add(buf, GLINE_HARD);
          }
          else
            break;
        }
      }
      ifp.fclose();
    }
    tfp.fclose();
  }
}


//  ------------------------------------------------------------------

void template_include(int mode, char* ptr, GMsg& msg, GMsg& omsg, gactivearea* destarea, gactivearea* origarea) {

  Path textfile;

  strbtrim(ptr);
  strcpy(textfile, ptr);
  MakePathname(textfile, CFG->templatepath, textfile);

  gfile tfp(textfile, "rt");
  if(tfp) {
    char buf[256];
    while(tfp.fgets(buf, sizeof(buf))) {
      TokenXlat(mode, buf, msg, omsg, destarea, origarea);
      strtrim(buf);
      msg.linearray.add(buf, GLINE_HARD);
    }
    tfp.fclose();
  }
}


//  ------------------------------------------------------------------

bool template_filename(char* tplfile, GMsg& msg, gactivearea* destarea) {

  if(destarea->templatematch() and not (tplselected or destarea->isnewsgroup() or destarea->isemail())) {

    int tn = 0;
    Tpl* tp = CFG->tpl;
    while(tn++ < CFG->tpls) {
      if(tp->match.net and msg.destaddr.match(tp->match)) {
        strcpy(tplfile, tp->file);
        break;
      }
      tp++;
    }
  }

  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  _splitpath(tplfile, drive, dir, file, ext);

  if(*drive == 0 and *dir == 0)
    strcpy(tplfile, AddPath(CFG->templatepath, tplfile));

  if((not fexist(tplfile)) and CFG->tpls)
    strcpy(tplfile, AddPath(CFG->templatepath, CFG->tpl[CFG->tplno].file));

  bool tmptpl = false;

  if((not fexist(tplfile)) or (not CFG->tpls)) {
    tmptpl = true;
    mktemp(strcpy(tplfile, AddPath(CFG->templatepath, "GEDTPLXXXXXX")));
    gfile fp(tplfile, "wt");
    if(fp) {
      fp.fputs("@moved* Replying to a msg in @oecho (@odesc)\n@moved\n");
      fp.fputs("@changed* Changed by @cname (@caddr), @cdate @ctime.\n@changed\n");
      fp.fputs("@forward* Forwarded from @oecho by @cname (@caddr).\n");
      fp.fputs("@forward* Originally by: @oname (@oaddr), @odate @otime.\n");
      fp.fputs("@forward* Originally to: @dname.\n");
      fp.fputs("@forward\n");
      fp.fputs("@message\n");
      fp.fputs("@forward\n");
      fp.fputs("Hello @tfname.\n");
      fp.fputs("@new\n");
      fp.fputs("@position\n");
      fp.fputs("@replyReplying to a msg dated @odate @otime, from @oname to @dname.\n");
      fp.fputs("@reply@position\n");
      fp.fputs("@quoted@odate @otime, @oname wrote to @dname:\n");
      fp.fputs("@quoted@position\n");
      fp.fputs("@comment@odate @otime, @oname wrote to @dname:\n");
      fp.fputs("@comment@position\n");
      fp.fputs("@quotebuf\n");
      fp.fputs("@quotebuf@odate @otime, @oname wrote to @dname:\n");
      fp.fputs("@quotebuf\n");
      fp.fputs("@quote\n\n");
      fp.fputs("@cfname\n\n");
      fp.fclose();
    }
  }

  return tmptpl;
}


//  ------------------------------------------------------------------

bool TemplateToLines(int mode, GMsg& msg, GMsg& omsg, gactivearea* destarea, gactivearea* origarea, int width, bool set_position) {

  if(destarea->isnet() and (mode == MODE_NEW)) {
    for(int n=0; n<CFG->robotname.size(); n++) {
      if(msg.dname.iinc(CFG->robotname[n])) {
        CFG->edithardlines = false;
        gstring robottext = CFG->language[LNG_RobotMsg];
        TokenXlat(mode, robottext, msg, omsg, destarea, origarea);
        gstrarray robotmsg;
        tokenize(robotmsg, robottext, "\r");
        for(int n=0; n<robotmsg.size(); n++)
          msg.linearray.add(robotmsg[n], GLINE_HARD);
        return false;
      }
    }
  }

  Path tplfile;
  strcpy(tplfile, destarea->tpl().c_str());
  bool tmptpl = template_filename(tplfile, msg, destarea);

  gfile fp(tplfile, "rt");
  if(not fp) {
    LOG.ErrOpen();
    LOG.printf("! A template file could not be opened.");
    LOG.printf(": %s", tplfile);
    LOG.ErrOSInfo();
    OpenErrorExit();
  }

  bool disphdr = false;

  char buf[256];

  while(fp.fgets(buf, sizeof(buf))) {

    char* ptr = strskip_wht(buf);

    if(*ptr != ';') {

      ptr = buf;
      bool chg = false;
      bool quotebufline = false;
      int token = end_token;

      do {

        do {
          // Find next '@' or NUL
          while(*ptr and (*ptr != '@'))
            ptr++;
          // Skip past double '@'s
          if(*ptr and (ptr[1] == '@'))
            ptr += 2;
        } while(*ptr and (*ptr != '@'));

        if(*ptr == NUL)
          break;

        char* tokenptr = template_token(token, ++ptr);

        if(token < end_token) {

          // Remove the token
          memmove(--ptr, tokenptr, strlen(tokenptr)+1);

          switch(token) {

            case TPLTOKEN_FORWARD:
              if(mode != MODE_FORWARD)
                goto loop_next;
              token = end_token;
              break;

            case TPLTOKEN_CHANGED:
              if(mode != MODE_CHANGE)
                goto loop_next;
              chg = true;
              token = end_token;
              break;

            case TPLTOKEN_NET:
              if(not destarea->isnet())
                goto loop_next;
              token = end_token;
              break;

            case TPLTOKEN_ECHO:
              if(not destarea->isecho())
                goto loop_next;
              token = end_token;
              break;

            case TPLTOKEN_LOCAL:
              if(not destarea->islocal())
                goto loop_next;
              token = end_token;
              break;

            case TPLTOKEN_MOVED:
              if(destarea == origarea)
                goto loop_next;
              if(mode == MODE_FORWARD)
                goto loop_next;
              if(origarea->areareplydirect() and omsg.kludge.area.ieql(destarea->echoid()))
                goto loop_next;
              if(omsg.kludge.fwdarea.ieql(destarea->echoid()))
                goto loop_next;
              token = end_token;
              break;

            case TPLTOKEN_NEW:
              if(mode != MODE_NEW)
                goto loop_next;
              token = end_token;
              break;

            case TPLTOKEN_REPLY:
              if(mode != MODE_REPLY)
                goto loop_next;
              token = end_token;
              break;

            case TPLTOKEN_QUOTED:
              if(mode != MODE_QUOTE)
                goto loop_next;
              token = end_token;
              break;

            case TPLTOKEN_CONFIRM:
              if(mode != MODE_CONFIRM)
                goto loop_next;
              token = end_token;
              break;

            case TPLTOKEN_COMMENT:
              if(mode != MODE_COMMENT)
                goto loop_next;
              token = end_token;
              break;

            case TPLTOKEN_QUOTEBUF:
              if(mode != MODE_QUOTEBUF)
                goto loop_next;
              quotebufline = true;
              token = end_token;
              break;
          }
        }

      } while(*ptr and (token>=end_token));

      if(token < end_token) {

        switch(token) {

          case TPLTOKEN_ATTRIB:
            if(mode != MODE_QUOTEBUF) {
              msg.attr.get(ptr);
              disphdr = true;
            }
            continue;

          case TPLTOKEN_SPELLCHECKER:
            if(mode != MODE_QUOTEBUF)
              CFG->editspellchecker = strskip_wht(ptr);
            continue;

          case TPLTOKEN_SETSUBJ:
          case TPLTOKEN_FORCESUBJ:
            if(mode != MODE_QUOTEBUF) {
              if(msg.subject.blank() or (token == TPLTOKEN_FORCESUBJ)) {
                strbtrim(ptr);
                StripQuotes(ptr);
                msg.subject = ptr;
                disphdr = true;
              }
            }
            continue;

          case TPLTOKEN_SETFROM:
          case TPLTOKEN_FORCEFROM:
            if(mode != MODE_QUOTEBUF) {
              if(msg.oname.blank() or (token == TPLTOKEN_FORCEFROM)) {
                strbtrim(ptr);
                StripQuotes(ptr);
                msg.oname = ptr;
                disphdr = true;
              }
            }
            continue;

          case TPLTOKEN_SETTO:
          case TPLTOKEN_FORCETO:
            if(mode != MODE_QUOTEBUF) {
              if(msg.dname.blank() or (token == TPLTOKEN_FORCETO)) {
                strbtrim(ptr);
                StripQuotes(ptr);
                msg.dname = ptr;
                disphdr = true;
              }
            }
            continue;

          case TPLTOKEN_XLATEXPORT:
            if(mode != MODE_QUOTEBUF)
              destarea->set_xlatexport(strbtrim(ptr));
            continue;

          case TPLTOKEN_LOADLANGUAGE:
            strbtrim(ptr);
            LoadLanguage(ptr);
            disphdr = true;
            continue;

          case TPLTOKEN_RANDOM:
            if(mode != MODE_QUOTEBUF)
              template_random(mode, ptr, msg, omsg, destarea, origarea);
            continue;

          case TPLTOKEN_INCLUDE:
            if(mode != MODE_QUOTEBUF)
              template_include(mode, ptr, msg, omsg, destarea, origarea);
            continue;

          case TPLTOKEN_QUOTE:
            if(mode & MODE_QUOTING)
              template_quote(msg, omsg, destarea);
            continue;

          case TPLTOKEN_MESSAGE:
            if(mode & (MODE_FORWARD|MODE_CHANGE))
              template_message(mode, msg, omsg);
            continue;
        }
      }

      if(mode == MODE_CHANGE)
        if(not chg)
          continue;
      if((mode == MODE_QUOTEBUF) and not quotebufline)
        continue;

      TokenXlat(mode, buf, msg, omsg, destarea, origarea);
      msg.linearray.add(buf, GLINE_HARD);
    }
    loop_next:
    ;
  }

  fp.fclose();

  if((mode != MODE_CHANGE) and (mode != MODE_QUOTEBUF)) {
    gstring bufline;
    uint ctrlinfo = destarea->ctrlinfo();
    if(msg.tagline) {
      msg.make_tagline(bufline);
      msg.linearray.add(bufline.c_str(), GLINE_TAGL|GLINE_HARD);
    }
    if(ctrlinfo & CI_TEAR) {
      msg.make_tearline(bufline);
      msg.linearray.add(bufline.c_str(), GLINE_TEAR|GLINE_HARD);
    }
    if(ctrlinfo & CI_ORIG) {
      msg.make_originline(bufline);
      msg.linearray.add(bufline.c_str(), GLINE_ORIG|GLINE_HARD);
    }
  }

  if(tmptpl)
    remove(tplfile);

  msg.wrap_lines(width, width);
  msg.reindex_lines(set_position);

  return disphdr;
}


//  ------------------------------------------------------------------

void ChangeMsg(GMsg& msg, gactivearea* area) {

  register int n, x, changemsg=NO;

  if(area->msgnset.size()) {

    reader_keyok = YES;

    if(area->attr().hex()) {
      area->set_attr().hex0();
      area->loadmsg(msg, msg.msgno, CFG->dispmargin-CFG->disppagebar);
    }

    for(n=0; n<CFG->usernames; n++) {
      // Check FROM:
      if(msg.origaddr.match(CFG->username[n].addr)) {
        if(msg.oname.ieql(CFG->username[n].name)) {
          if(CFG->username[n].addr.net != ftn::wildcard_all or msg.origaddr.invalid() or not area->isnet())
            reader_keyok = NO;
          else {
            for(x=0; x<CFG->akas; x++) {
              if(msg.origaddr.match(CFG->aka[x].addr)) {
                reader_keyok = NO;
                break;
              }
            }
          }
        }
      }
    }

    if(reader_keyok == YES) {
      GMenuChange MenuChange;
      reader_keyok = !MenuChange.Run(CFG->language[LNG_ChangeWarn]);
      if(reader_keyok == NO)
        changemsg = true;
    }
    else
      changemsg = true;

    if(changemsg) {
      if(msg.attr.loc() and not area->islocal()) {
        if(msg.attr.snt() or not msg.attr.uns()) {
          GMenuChange MenuChange;
          reader_keyok = !MenuChange.Run(CFG->language[LNG_WarnAlreadySent]);
        }
      }
      if(not reader_keyok)
        MakeMsg(MODE_CHANGE, msg, area);
    }
  }
}


//  ------------------------------------------------------------------

void NewMsg(GMsg& msg, gactivearea* area) {

  if(area->attr().r_o()) {
    GMenuReadonly MenuReadonly;
    reader_keyok = !MenuReadonly.Run();
  }

  if(not reader_keyok) {
    area->set_attr().hex0();
    reader_topline = 0;
    MakeMsg(MODE_NEW, msg, area);
  }
}


//  ------------------------------------------------------------------

void ConfirmMsg(GMsg& msg, gactivearea* area) {

  int doit = CFG->confirmresponse;
  if(CFG->confirmresponse == ASK) {
    GMenuConfirm MenuConfirm;
    doit = MenuConfirm.Run();
  }

  reader_gen_confirm = NO;

  if(doit and area->isnet() and (msg.attr.cfm() or msg.attr.rrq())) {
    update_statusline(CFG->language[LNG_GenCfmReceipt]);
    area->set_attr().hex0();
    GMsg cmsg;
    cmsg.copy(msg);
    MakeMsg(MODE_CONFIRM, cmsg, area);
  }
}


//  ------------------------------------------------------------------

void ReplyMsg(GMsg& msg, gactivearea* area) {

  if(area->msgnset.size()) {

    if(area->areareplydirect() and msg.kludge.area.length()) {
      if(AL.echoid_to_number(msg.kludge.area) != -1) {
        OtherAreaQuoteMsg(msg, area, false);
        return;
      }
    }

    if(area->attr().r_o()) {
      GMenuReadonly MenuReadonly;
      reader_keyok = !MenuReadonly.Run();
    }

    if(not reader_keyok) {
      area->set_attr().hex0();
      reader_topline = 0;
      MakeMsg(MODE_REPLY, msg, area);
    }
  }
}


//  ------------------------------------------------------------------

void OtherAreaMsg(int mode, GMsg& msg, int destareaid, gactivearea* origarea) {

  gactivearea* destarea = AL.activate(destareaid);

  if(destarea->attr().r_o()) {
    GMenuReadonly MenuReadonly;
    reader_keyok = !MenuReadonly.Run();
  }

  if(not reader_keyok) {
    origarea->set_attr().hex0();
    reader_topline = 0;
    int viewhidden = origarea->viewhidden();
    int viewkludge = origarea->viewkludge();
    int viewquote  = origarea->viewquote();
    destarea->open();
    destarea->set_viewhidden(viewhidden);
    destarea->set_viewkludge(viewkludge);
    destarea->set_viewquote(viewquote);
    MakeMsg(mode, msg, destarea, origarea);
    destarea->close();
  }

  AL.deactivate(destarea);
}


//  ------------------------------------------------------------------

void OtherAreaQuoteMsg(GMsg& msg, gactivearea* area, bool allow_pick) {

  if(area->msgnset.size()) {

    int destareaid = area->areaid();
    if(area->areareplyto().length())
      destareaid = AL.echoid_to_id(area->areareplyto());

    const gstring& destinationecho = msg.kludge.fwdarea ? msg.kludge.fwdarea : msg.kludge.area;
    if(destinationecho.length()) {
      for(int n=0; n<AL.areas(); n++) {
        if(AL[n]->echoid().ieql(destinationecho)) {
          destareaid = AL[n]->areaid();
          break;
        }
      }
    }

    if(allow_pick or not area->areareplydirect())
      destareaid = AreaPick(CFG->language[LNG_ReplyArea], 6, &destareaid);

    if(destareaid != -1)
      OtherAreaMsg(MODE_QUOTE, msg, destareaid, area);
  }
}


//  ------------------------------------------------------------------

void QuoteMsg(GMsg& msg, gactivearea* area) {

  if(area->msgnset.size()) {

    if(area->areareplydirect() and msg.kludge.area.length()) {
      if(AL.echoid_to_number(msg.kludge.area) != -1) {
        OtherAreaQuoteMsg(msg, area, false);
        return;
      }
    }

    if(area->attr().r_o()) {
      GMenuReadonly MenuReadonly;
      reader_keyok = !MenuReadonly.Run();
    }

    if(not reader_keyok) {
      area->set_attr().hex0();
      reader_topline = 0;
      MakeMsg(MODE_QUOTE, msg, area);
    }
  }
}


//  ------------------------------------------------------------------

void OtherAreaCommentMsg(GMsg& msg, gactivearea* area, bool allow_pick) {

  if(area->msgnset.size()) {

    int destareaid = area->areaid();
    if(area->areareplyto().length())
      destareaid = AL.echoid_to_id(area->areareplyto());

    const gstring& destinationecho = msg.kludge.fwdarea ? msg.kludge.fwdarea : msg.kludge.area;
    if(destinationecho.length()) {
      for(int n=0; n<AL.areas(); n++) {
        if(AL[n]->echoid().ieql(destinationecho)) {
          destareaid = AL[n]->areaid();
          break;
        }
      }
    }

    if(allow_pick or not area->areareplydirect())
      destareaid = AreaPick(CFG->language[LNG_ReplyArea], 6, &destareaid);

    if(destareaid != -1)
      OtherAreaMsg(MODE_COMMENT, msg, destareaid, area);
  }
}


//  ------------------------------------------------------------------

void CommentMsg(GMsg& msg, gactivearea* area) {

  if(area->msgnset.size()) {
   
    if(area->areareplydirect() and msg.kludge.area.length()) {
      if(AL.echoid_to_number(msg.kludge.area) != -1) {
        OtherAreaCommentMsg(msg, area, false);
        return;
      }
    }

    if(area->attr().r_o()) {
      GMenuReadonly MenuReadonly;
      reader_keyok = !MenuReadonly.Run();
    }

    if(not reader_keyok) {
      area->set_attr().hex0();
      reader_topline = 0;
      MakeMsg(MODE_COMMENT, msg, area);
    }
  }
}


//  ------------------------------------------------------------------

