
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geutil.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Global utility functions (static).
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif


//  ------------------------------------------------------------------

void update_statuslines() {

  char buf[200];
  static int __FAR called = NO;

  HandleGEvent(EVTT_REMOVEVOCBUF);

  if((CFG->dispstatusline or not called) /* and not gkbd.inhelp */) {

    called = YES;

    char sep = _box_table[W_BSTAT][3];
    char help[80], meminfo[80], clkinfo[80];
    *clkinfo = NUL;
    *meminfo = NUL;
    *help = NUL;

    #if defined(__MSDOS__) and defined(__BORLANDC__)
    long mem = coreleft()/1024L;
    sprintf(meminfo, " %c %luk", sep, mem);
    #elif defined(__WATCOMC__) and defined(__386__)
    #endif

    if(CFG->statuslineclock) {
      Time t = gtime();
      strftimei(help, 40, CFG->language[LNG_StatusLineTimeFmt], glocaltime(&t));
      sprintf(clkinfo, " %c %s", sep, help);
    }

    if(CFG->statuslinehelp == -1)
      *help = NUL;
    else if(CFG->statuslinehelp)
      sprintf(help, "%s %c ", CFG->language[LNG_StatusLineHelp], sep);
    else
      sprintf(help, "%s%c%s%c ", __gpid__, goldmark, CFG->ver, sep);
    
    int len = gvid.numcols-strlen(help)-strlen(meminfo)-strlen(clkinfo)-2;
    sprintf(buf, " %s%-*.*s%s%s ", help, len, len, information, meminfo, clkinfo);

    gmou.GetStatus();
    if(gmou.Row() == gvid.numrows-1)
      gmou.HideCursor();
    wwprintstr(W_STAT, 0,0, C_STATW, buf);
    if(gmou.Row() == gvid.numrows-1)
      gmou.ShowCursor();
  }
}


//  ------------------------------------------------------------------

void update_statusline(const char* info) {

  strxcpy(information, info, sizeof(information));
  update_statuslines();
}


//  ------------------------------------------------------------------

void _stdc update_statuslinef(const char* format, ...) {

  va_list argptr;
  va_start(argptr, format);
  vsprintf(information, format, argptr);
  va_end(argptr);
  update_statuslines();
}


//  ------------------------------------------------------------------

void w_shadow() {

  if(CFG->screenshadows)
    wshadow(C_SHADOW);
}


//  ------------------------------------------------------------------

void w_info(const char* info) {

  static int __FAR wh=-1;
  static int __FAR srow;
  static int __FAR scol;
  static int __FAR erow;
  static int __FAR ecol;
  static int __FAR len;
  static char __FAR buf[150] = { "" };

  int prev_wh = whandle();
  if(wh != -1)
    wactiv_(wh);

  if(info) {
    uint tmp = strlen(info);
    if(wh == -1) {
      len = tmp;
      srow = inforow;
      erow = srow+3-1;
      scol = ((gvid.numcols-len)/2)-1;
      ecol = scol+len+1;
      wh = wopen(srow, scol, erow, ecol, W_BINFO, C_INFOB, C_INFOW);
      w_shadow();
    }
    else {
      if(len != tmp) {
        len = tmp;
        scol = ((gvid.numcols-len)/2)-1;
        ecol = scol+len+1;
        wclose();
        wh = wopen(srow, scol, erow, ecol, W_BINFO, C_INFOB, C_INFOW);
        w_shadow();
      }
    }
    if(not streql(buf, info)) {
      strcpy(buf, info);
      wprints(0, 0, C_INFOW, buf);
    }
  }
  else {
    if(wh != -1) {
      *buf = NUL;
      wclose();
      wh = -1;
    }
  }

  wactiv_(prev_wh);
}


//  ------------------------------------------------------------------


void _stdc w_infof(const char* format, ...) {

  char winfobuf[150];
  va_list argptr;
  va_start(argptr, format);
  vsprintf(winfobuf, format, argptr);
  va_end(argptr);
  w_info(winfobuf);
}


//  ------------------------------------------------------------------

void w_progress(int mode, int attr, long pos, long size, const char* title) {

  static int __FAR wh = -1;

  int prev_wh = whandle();
  if(wh != -1)
    wactiv_(wh);

  switch(mode) {
    case PROGRESS_NEW:
      oops:
      if(wh == -1) {
        wh = wopen_(inforow,((gvid.numcols-63)/2)-1,3,63,W_BINFO,C_INFOB,C_INFOW);
        set_title(title,TCENTER,C_INFOT);
        title_shadow();
      }
    case PROGRESS_UPDATE:
      if(wh == -1)
        goto oops;  // Oops, someone forgot to open the window..
      wpropbar(PROP_BARGRAPH,1,0,-59,1,attr,pos,size);
      break;
    case PROGRESS_QUIT:
      if(wh != -1) {
        wclose();
        wunlink(wh);
        wh = -1;
      }
      break;
  }

  wactiv_(prev_wh);
}


//  ------------------------------------------------------------------

void maketitle() {

  wtitle(m_title, m_titlepos, m_titleattr);
}


//  ------------------------------------------------------------------

void set_title(const char* t, int p, int a) {

  strcpy(m_title, t);
  m_titlepos  = p;
  m_titleattr = a;
}


//  ------------------------------------------------------------------

void title_shadow() {

  maketitle();
  w_shadow();
}


//  ------------------------------------------------------------------

int _stdc CmdKeyCmp(const CmdKey* a, const CmdKey* b) {

  int cmp = compare_two(a->type, b->type);
  if(cmp)
    return cmp;
  return compare_two(a->key, b->key);
}


//  ------------------------------------------------------------------

static int KeyCmp(const Key* a, const Key* b) {

  return compare_two(*a, *b);
}


//  ------------------------------------------------------------------

Key SearchKey(Key key, CmdKey* keys, int totkeys) {

  char* kmin;
  char* probe;
  int again = 0;
  register int i, j, tkeys=totkeys;

  do {
    kmin = (char*)keys;
    while(tkeys > 0) {
      i = tkeys >> 1;
      probe = kmin + i * sizeof(CmdKey);
      j = KeyCmp(&key, (Key*)probe);
      if(j == 0)
        return(((CmdKey*)probe)->cmd);
      else if(j < 0)
        tkeys = i;
      else {
        kmin = probe + sizeof(CmdKey);
        tkeys = tkeys - i - 1;
      }
    }

    // Key not found. Try again, this time without the scancode.
    key &= 0x00FF;
    tkeys = totkeys;
  } while(not again++ and key);

  return 0;
}


//  ------------------------------------------------------------------

static void call_func_geutil(VfvCP func) {

  struct _menu_t* menu;
  int sline, eline, row, col;

  vcurget(&sline, &eline);
  vposget(&row, &col);
  menu = gwin.cmenu;
  (*func)();
  gwin.cmenu = menu;
  vposset(row, col);
  vcurset(sline, eline);
}

  
//  ------------------------------------------------------------------

void call_help() {

  // search through onkey linked list for a
  // matching defined onkey.  if one is found,
  // then save the current environment, call the
  // onkey's function, and restore the environment.

  KBnd* onkey = gkbd.onkey;
  while(onkey != NULL) {
    if(onkey->keycode == Key_F1) {
      call_func_geutil(onkey->func);
      break;
    }
    onkey = onkey->prev;
  }
}


//  ------------------------------------------------------------------

void CheckTick(Key quitkey) {

  if(CFG->timeout) {
    if(gkbd.tickvalue >= (gkbd.tickpress+(CFG->timeout*1000L))) {
      kbput(quitkey);
      return;
    }
  }

  extern bool _in_arealist;
  if(_in_arealist) {
    if(CFG->semaphore.idletime and not blanked) {
      if(gkbd.tickvalue >= (gkbd.tickpress+(CFG->semaphore.idletime*1000L))) {
        CheckSemaphores();
        gkbdtickpressreset();
        return;
      }
    }
  }

  if(CFG->screenblanker) {
    if(gkbd.tickvalue >= (gkbd.tickpress+(CFG->screenblanker*1000L))) {

      blanked = true;
      ScreenBlankIdle();
      kbdsettickfunc(ScreenBlankIdle);

      getxch();

      blanked = false;
      ScreenBlankIdle();
      kbdsettickfunc(update_statuslines);
    }
  }
}


//  ------------------------------------------------------------------

