
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geutil2.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Global utility functions (on overlay).
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

#include <gwinput.h>


//  ------------------------------------------------------------------

int edit_string(gstring& buf, int buf_size, char* title, int helpcat) {

  gwindow window;

  window.openxy(inforow,((gvid.numcols-63)/2)-1,3,63,W_BASK,C_ASKB,C_ASKW);
  window.title(title, C_ASKT);
  if(CFG->screenshadows)
    window.shadow(C_SHADOW);

  winput2 iform(window);

  iform.setup(C_ASKW, C_ASKW, C_ASKQ, '\xB0');
  iform.add_field(0, 0, 1, 59, buf, buf_size, winput::cvt_none, fieldupd);
  iform.run(helpcat);

  window.close();

  if(iform.dropped)
    return false;

  buf.trim();
  return buf.length() ? true : false;
}


//  ------------------------------------------------------------------

int PopupLocked(long tries, int isopen, const char* file) {

  // Close popup window if requested
  if(tries == 0) {
    w_info(NULL);
    return true;
  }

  // Tell the world
  char buf[200];
  if(strlen(file) > (gvid.numcols-25))
    sprintf(buf, "[..]%*.*s", gvid.numcols-30, gvid.numcols-30, file);
  else
    strcpy(buf, file);
  w_infof(CFG->language[LNG_WaitLocked], buf);
  update_statuslinef(CFG->language[LNG_RetryOrESC], isopen ? CFG->language[LNG_RetryLock] : CFG->language[LNG_RetryOpen] , tries);

  // Check for keypress
  if(kbxhit()) {

    // Return fail value if Escape was pressed
    Key keyval = kbxget();
    if(keyval == Key_Esc) {
      GMenuLockLoop MenuLockLoop;
      if(MenuLockLoop.Run()) {
        w_info(NULL);
        return false;
      }
    }
  }

  // Pause approx 1 second before trying again
  Pause(1000);

  // Try again
  return true;
}


//  ------------------------------------------------------------------
//  Build zonegate address from orig/dest address

void ZonegateIt(ftn_addr& gate, const ftn_addr& orig, const ftn_addr& dest) {

  gate.zone  = orig.zone;  // Send to the local zoneGate
  gate.net   = orig.zone;
  gate.node  = dest.zone;
  gate.point = 0;
}


//  ------------------------------------------------------------------

int golded_cfg::address_find(const gftnaddrs& find) {

  // Try a 5D match first
  int total = address.size();
  for(int n=0; n<total; n++) {
    if(address[n].equals(find))
      return n+1;
  }

  // That failed, so try a 4D match
  for(int m=0; m<total; m++) {
    if(address[m].equals4d(find))
      return m+1;
  }

  // Not found
  return 0;
}


//  ------------------------------------------------------------------

int AkaMatchOne(const ftn_addr& mask, const ftn_addr& addr) {

  int match = 0;

  // Count matching parts
  if((mask.zone == 0) or (addr.zone == 0) or (mask.zone == GFTN_ALL) or (mask.zone == addr.zone)) {
    match++;
    if((mask.net == GFTN_ALL) or (mask.net == addr.net)) {
      match++;
      if((mask.node == GFTN_ALL) or (mask.node == addr.node)) {
        match++;
        if((mask.point == GFTN_ALL) or (mask.point == addr.point)) {
          match++;
        }
      }
    }
  }

  return match;
}


//  ------------------------------------------------------------------

int AkaMatch(ftn_addr& match, const ftn_addr& addr) {

  int aka;

  for(aka=0; aka<CFG->akamatches; aka++) {
    if(addr.match(CFG->akamatch[aka].mask)) {
      int akano = CFG->address_find(CFG->akamatch[aka].aka);
      if(akano) {
        match = CFG->aka[akano-1].addr;
        return akano-1;
      }
    }
  }

  int bestaka = -1;
  int bestmatch = 0;
  int matchaka = 0;

  for(aka=0; aka<CFG->akas; aka++) {

    if(match == CFG->aka[aka].addr)
      matchaka = aka;

    // Ignore fakenet addresses
    if(CFG->aka[aka].addr.net != CFG->aka[aka].pointnet) {

      int match = AkaMatchOne(CFG->aka[aka].addr, addr);

      // Note if we found a better match
      if(match > bestmatch) {
        bestmatch = match;
        bestaka = aka;
      }
    }
  }

  // If there was no matching aka, use the aka
  if(bestaka == -1)
    bestaka = matchaka;
  else {
    // If the current address matches at least as good as the best match, don't change it
    if(bestmatch <= AkaMatchOne(match, addr)) {
      int akano = CFG->address_find(match);
      if(akano)
        return akano-1;
    }
  }

  match = CFG->aka[bestaka].addr;
  return bestaka;
}


//  ------------------------------------------------------------------

void ScreenBlankIdle() {

  static int __FAR whb = -1;
  static int __FAR whh = -1;
  static int __FAR wascurhid = false;
  static Clock __FAR lastmoved = 0;
  int windowheight = 4;

  char blankmsg1[80];
  char blankmsg2[80];
  Time t = gtime();
  strftimei(blankmsg2, 40, CFG->language[LNG_StatusLineTimeFmt], glocaltime(&t));
  sprintf(blankmsg1, " %s %s %s ", __gpid__, gversion, blankmsg2);
  sprintf(blankmsg2, " %s ", CFG->language[LNG_BlankMsg]);
  if(strblank(blankmsg2)) {
    *blankmsg2 = NUL;
    windowheight--;
  }
  register int b1 = strlen(blankmsg1);
  register int b2 = strlen(blankmsg2);
  int blankmsglen = maximum_of_two(b1,b2);
  int ry = rand() % (gvid.numrows-windowheight);
  int rx = rand() % ((gvid.numcols-blankmsglen)-2);

  if(blanked and (whb == -1)) {
    wascurhid = vcurhidden();
    vcurhide();
    if(C_BACKB != 0)
      gvid.setoverscan(0);
    whb = wopen(0,0,gvid.numrows-1,gvid.numcols-1, 5, 7, 7);
    if(CFG->screenblankertype == BLANK_SLIDEWIN)
      whh = wopen_(ry, rx, windowheight, blankmsglen+2, W_BINFO, C_INFOB, C_INFOW);
    lastmoved = gkbd.tickvalue;
  }

  if(not blanked and (whb != -1)) {
    if(CFG->screenblankertype == BLANK_SLIDEWIN)  {
      wactiv_(whh);
      wclose();
      whh = -1;
    }
    wactiv_(whb);
    wclose();
    whb = -1;
    if(C_BACKB != 0)
      gvid.setoverscan(C_BACKB);
    if(wascurhid)
      vcurhide();
    else
      vcurshow();
    return;
  }

  if(gkbd.tickvalue > (lastmoved+5000L)) {
    lastmoved = gkbd.tickvalue;
    if(CFG->screenblankertype == BLANK_SLIDEWIN)
      wslide(ry, rx);
  }

  if(CFG->screenblankertype == BLANK_SLIDEWIN) {
    wcenters(0, C_INFOW, blankmsg1);
    if(*blankmsg2)
      wcenters(1, C_INFOW, blankmsg2);
  }

  // If timed out, exit screenblanker
  if(CFG->timeout)
    if(gkbd.tickvalue >= (gkbd.tickpress+(CFG->timeout*1000L)))
      kbput(Key_Tick);
}


//  ------------------------------------------------------------------

int SearchTaglist(Echo* taglist, char* tag) {

  register int n = 0;

  while(*taglist[n]) {
    if(strieql(taglist[n], tag))
      break;
    n++;
  }
  return(n);
}


//  ------------------------------------------------------------------

void HudsSizewarn() {

  whelpcat(H_EWarnMsgtxt);
  call_help();
  whelpcat(H_General);
}


//  ------------------------------------------------------------------

void HGWarnRebuild() {

  whelpcat(H_EQbaseRebuild);
  call_help();
}


//  ------------------------------------------------------------------

void FidoRenumberProgress(const char* s) {

  update_statusline(s);
}


//  ------------------------------------------------------------------

void address_macro::reset() {

  macro = "";
  name = "";
  address = "";
  subject = "";
  attribs = "";
}


//  ------------------------------------------------------------------

address_macro& address_macro::operator=(const char* s) {

  bool subjwasquoted = false;
  const char* ptr = s;
  const char* end = strchr(ptr, ',');           //  macro^,name,address,subj,attribs
  if(end) {
    // Got macroname
    amac.macro.copy(ptr, end++);
    ptr = end;
    end = strchr(strskip_wht(ptr), ',');        //  macro,name^,address,subj,attribs
    if(end) {
      // Got real name
      amac.name.copy(ptr, end++);
      ptr = end;
      end = strchr(strskip_wht(ptr), ',');      //  macro,name,address^,subj,attribs
      // Got address
      if(end)
        amac.address.copy(ptr, end++);
      else
        amac.address = ptr;
      if(end) {
        // Got subject
        ptr = strskip_wht(end);
        if((*ptr == '\"') or (*ptr == '\'')) {
          end = strchr(ptr+1, *ptr);
          if(end) {
            subjwasquoted = true;
            *end++ = NUL;
            ptr++;
          }
        }
        if(end)
          amac.subject.copy(ptr, end++);
        else
          amac.subject = ptr;
        // Got quoted subj, so look for attribs
        if(subjwasquoted) {
          end = strchr(end, ',');               //  macro,name,address,subj^,attribs
          if(end) {
            amac.attribs = end + 1;
          }
        }
      }
    }
  }

  return *this;
}


//  ------------------------------------------------------------------

void do_zonegating(GMsg& msg, gactivearea* area) {

  ftn_addr orig = msg.oaddress;
  ftn_addr dest = msg.daddress;

  if(area->isnet() and CFG->zonegating and (dest.zone != orig.zone)) {
    if(not msg.attr.dir()) {
      GMenuZonegate MenuZonegate;
      if(CFG->zonegating == YES or MenuZonegate.Run())
        ZonegateIt(msg.destaddr, orig, dest);
    }
  }
}


//  ------------------------------------------------------------------

void do_fakenets(GMsg& msg, gactivearea* area) {

  msg.origaddr = msg.oaddress;
  if(area->isnet() and msg.origaddr.point) {
    for(uint n=0; n<CFG->akas; n++) {
      if(CFG->aka[n].addr == msg.origaddr) {
        // Use fakenet to everybody
        if(CFG->aka[n].pointnet) {
          // Create fake address
          msg.origaddr.net   = CFG->aka[n].pointnet;
          msg.origaddr.node  = msg.origaddr.point;
          msg.origaddr.point = 0;
        }
        break;
      }
    }
  }
}


//  ------------------------------------------------------------------

