
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geview.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Message viewer class implementation.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
//#pragma option -Od
#endif

//  ------------------------------------------------------------------

GMsgHeaderView::GMsgHeaderView() {

  msg = NULL;
  alive = false;
  at_row = at_column = width = height = 0;
  border_type = gwindow::bordertype_none;
  window_color = from_color = to_color = subject_color = BLUE|_LGREY;
  title_color = highlight_color = RED|_LGREY;
  border_color = YELLOW|_LGREY;
}


//  ------------------------------------------------------------------

GMsgHeaderView::~GMsgHeaderView() {

  Destroy();
}


//  ------------------------------------------------------------------

void GMsgHeaderView::Create() {

  alive = true;

  window.openxy(
    at_row,
    at_column,
    height,
    width,
    border_type,
    border_color,
    window_color
  );
}


//  ------------------------------------------------------------------

void GMsgHeaderView::Destroy() {

  if(alive)
    window.close();
  else
    alive = false;
}


//  ------------------------------------------------------------------

void GMsgHeaderView::Use(gactivearea* areaptr, GMsg* msgptr) {

  area = areaptr;
  msg = msgptr;
}

 
//  ------------------------------------------------------------------

void GMsgHeaderView::Paint() {

  header_display display;

  build_header_display(*msg, display, width, area);

  char headerline[200];
  memset(headerline, window.boxchar_upper_horizontal_line(), width);
  headerline[width] = NUL;

  // Paint the total header in the window
  char borderchar[2] = " ";
  *borderchar = window.boxchar_upper_horizontal_line();
  window.prints(0, 0, border_color, borderchar);
  window.prints(0, 1, title_color, display.description);
  window.prints(0, display.description.length()+1, border_color, headerline+display.description.length()+display.echoid.length()-2);
  window.prints(0, width-display.echoid.length()-1, title_color, display.echoid);
  window.prints(0, width-1, border_color, borderchar);
  window.prints(5, 0, border_color, headerline);
  window.prints(1, 0, window_color, display.msgnos);
  if(bookmark)
    window.prints(1, 5, highlight_color, "\x11");
  if(markmark)
    window.prints(1, 7, highlight_color, "\x10");
  window.prints(2, 0, window_color, CFG->language[LNG_From]);
  window.prints(2, CFG->disphdrnameset.pos, ((msg->foundwhere&GFIND_FROM) or msg->attr.fmu() or (msg->attr.loc() and CFG->displocalhigh)) ? highlight_color : from_color, display.from_name);
  window.prints(3, 0, window_color, CFG->language[LNG_To]);
  window.prints(3, CFG->disphdrnameset.pos, ((msg->foundwhere&GFIND_TO) or msg->attr.tou()) ? highlight_color : to_color, display.to_name);
  if(not area->isinternet()) {
    window.prints(2, CFG->disphdrnodeset.pos, from_color, display.from_address);
    window.prints(3, CFG->disphdrnodeset.pos, to_color, display.to_address);
  }
  window.prints(2, CFG->disphdrdateset.pos, from_color, display.date_written);
  window.prints(3, CFG->disphdrdateset.pos, to_color, display.date_arrived);
  window.prints(4, 0, window_color, display.lngsubj);
  window.prints(4, display.lngsubj.length(), (msg->foundwhere&GFIND_SUBJECT) ? highlight_color : subject_color, display.subject);

  // Calculate attach file sizes
  if(msg->attr.att()) {
    if(CFG->dispattachsize) {
      gstring subj;
      char buf2[GMAXPATH];
      int begpos = strlen(CFG->language[LNG_File]);
      strcpy(buf, msg->subject.c_str());
      char* ptr2;
      char* ptr = strtok(buf, " ");
      while(ptr) {
        if(*ptr == '^')
          ptr++;
        if((isalpha(*ptr) and (ptr[1] == ':')) or ((ptr[0] == '\\') and (ptr[1] == '\\')))
          ptr2 = ptr;
        else {
          sprintf(buf2, "%s%s", CFG->inboundpath, ptr);
          ptr2 = buf2;
        }
        long sz = GetFilesize(ptr2);
        if(sz == -1)
          sprintf(subj, " %s ", CFG->language[LNG_n_a]);
        else {
          switch(CFG->dispattachsize) {
            case ATTACH_BYTES:
              subj = gstrf(" %li ", sz);
              break;
            case ATTACH_KBYTES:
              subj = gstrf(" %lik ", (sz+512L)/1024L);
              break;
          }
        }
        window.prints(5, begpos+int(ptr-buf)-1, title_color, subj);
        ptr = strtok(NULL, " ");
      }
    }
  }

  if(CFG->dispmsgsize and msg->txt and not _in_editor) {
    gstring subj;
    uint len = strlen(msg->txt);
    switch(CFG->dispmsgsize) {
      case DISPMSGSIZE_BYTES:
        subj = gstrf("%u", len);
        break;
      case DISPMSGSIZE_KBYTES:
        subj = gstrf("%uk", (len+512)/1024);
        break;
      case DISPMSGSIZE_LINES:
        subj = gstrf("%u", msg->display.size());
        break;
    }
    if(subj)
      window.prints(5, 1, title_color, subj);

    if(msg->mood or msg->gif) {
      gstring mood;
      gstring gif;
      if(msg->mood.length()) {
        mood = "[MOOD:" + msg->mood + "]";
      if(msg->gif.length())
        gif = "[GIF:" + msg->gif + "]";
      subj = mood;
      subj += gif;
      window.prints(5, width-strlen(subj)-1, title_color, subj);
    }
  }
}


//  ------------------------------------------------------------------

GMsgBodyView::GMsgBodyView() {

  msg = NULL;
  alive = false;
  scrollbar_visible = false;
  at_row = at_column = width = height = visible_width = 0;
  border_type = gwindow::bordertype_none;
  window_color = BLACK|_LGREY;
  highlight_color = WHITE|_RED;
  scrollbar_color = DGREY|_LGREY;
  border_color = YELLOW|_LGREY;
}


//  ------------------------------------------------------------------

GMsgBodyView::~GMsgBodyView() {

  Destroy();
}


//  ------------------------------------------------------------------

void GMsgBodyView::Create() {

  alive = true;

  window.openxy(
    at_row,
    at_column,
    height,
    width,
    border_type,
    border_color,
    window_color,
    scrollbar_color
  );

  visible_width = width - (CFG->disppagebar ? 1 : 0);
}


//  ------------------------------------------------------------------

void GMsgBodyView::Destroy() {

  if(alive)
    window.close();
  else
    alive = false;
}


//  ------------------------------------------------------------------

void GMsgBodyView::Use(gactivearea* areaptr, GMsg* msgptr, int startline) {

  area = areaptr;
  msg = msgptr;

  // Find the first highlighted line and go to it
  if(msg->foundwhere & GFIND_BODY) {
    if(msg->display.size() > height)
      if(msg->foundtopline > 5)
        startline = msg->foundtopline - 5;
  }

  if(startline >= msg->display.size())
    startline = msg->display.size() - 1;

  if(startline < 0)
    startline = 0;

  upperline = startline;
  lowerline = upperline + height - 1;
  can_pagedown = lowerline < (msg->display.size()-1);

  scrollbar_visible = CFG->disppagebar ? (msg->display.size() > height) : false;
  visible_width = scrollbar_visible ? width-1 : width;
  window.set_scrollbar_color(scrollbar_visible ? scrollbar_color : -1);
}

 
//  ------------------------------------------------------------------

void GMsgBodyView::PaintLine(int row, int lineno) {

  // Calculate effective coordinates for vputs
  int vrow = gwin.active->srow + row;

  // Get the line
  int len = 0;
  glinedata line;
  const char* text = "";
  if(lineno != -1) {
    int index = msg->display[lineno];
    text = msg->line[index];
    line = msg->line.data(index);
    len = msg->line.length(index);
  }
  else {
    line.type = 0;
  }

  // Determine line color
  int color = C_READW;
  if(line.ishiddklud())
    color = C_READK;
  else if(line.isquote())
    color = C_READQ;
  else if(line.istagline())
    color = C_READG;
  else if(line.istearline())
    color = C_READT;
  else if(line.isorigin())
    color = C_READO;

  // Print it
  vputns(vrow, 0, color, text, visible_width);

  // Stylecode highlightning
  if(CFG->stylecodes)
    StyleCodeHighlight(text, vrow, 0, (VidPutStrCP)vputs);

  // Handle highlighting
  if(line.ishighlighted()) {
    for(int n=0; n<find_pats; n++) {
      char* patstr = find_patstr[n];
      uint patlen = strlen(patstr);
      if(len > patlen) {
        uint endpos = len - patlen;
        const char* ptr = text;
        for(int x=0; x<=endpos; x++,ptr++) {
          if(strnieql(patstr, ptr, patlen)) {
            BmhPat highstr;
            strxcpy(highstr, ptr, patlen+1);
            vputs(vrow, (uint)(ptr-text), highlight_color, highstr);
          }
        }
      }
    }
  }
}


//  ------------------------------------------------------------------

void GMsgBodyView::Clear() {

  window.activate_quick();
  window.clear();
}

  
//  ------------------------------------------------------------------

void GMsgBodyView::Paint() {

  window.activate_quick();
  
  int endline = msg->display.size();
  for(int row=0, line=upperline; row<height; row++)
    PaintLine(row, line < endline ? line++ : -1);

  UpdateScrollbar();
}


//  ------------------------------------------------------------------

int GMsgBodyView::Top(int redraw) {

  if(redraw or (msg->display.size() > height)) {
    can_pagedown = true;
    upperline = 0;
    lowerline = height - 1;
    Paint();
    return true;
  }
  
  return false;
}


//  ------------------------------------------------------------------

int GMsgBodyView::Bottom() {
  
  if(msg->display.size() > height) {
    if(msg->display.size() <= height) {
      can_pagedown = true;
      upperline = 0;
    }
    else {
      can_pagedown = false;
      upperline = msg->display.size() - height;
    }
    lowerline = upperline + height - 1;
    Paint();
    return true;
  }
  
  return false;
}


//  ------------------------------------------------------------------

int GMsgBodyView::PageUp() {

  if(msg->display.size() > height) {
    can_pagedown = true;
    if(upperline) {
      upperline -= height - 1;
      if(upperline < 0)
        upperline = 0;
      lowerline = upperline + height - 1;
      Paint();
      return true;
    }
  }
  
  return false;
}


//  ------------------------------------------------------------------

int GMsgBodyView::PageDown() {

  if(msg->display.size() > height) {
    if(can_pagedown) {
      if(lowerline < msg->display.size()-1) {
        lowerline += height - 1;
        if(lowerline >= msg->display.size()-1)
          can_pagedown = false;
        upperline = lowerline - height + 1;
        Paint();
        return true;
      }
    }
  }
  
  return false;
}


//  ------------------------------------------------------------------

int GMsgBodyView::LineUp() {
  
  if(msg->display.size() > height) {
    if(upperline) {
      window.scroll_down();
      upperline--;
      lowerline--;
      gmou.HideCursor();
      PaintLine(0, upperline);
      gmou.ShowCursor();
      can_pagedown = lowerline < (msg->display.size()-1);
      UpdateScrollbar();
      return true;
    }
  }
  return false;
}


//  ------------------------------------------------------------------

int GMsgBodyView::LineDown() {

  if(msg->display.size() > height) {
    if(lowerline < (msg->display.size()-1)) {
      window.scroll_up();
      upperline++;
      lowerline++;
      gmou.HideCursor();
      PaintLine(height-1, lowerline);
      gmou.ShowCursor();
      can_pagedown = lowerline < (msg->display.size()-1);
      UpdateScrollbar();
      return true;
    }
  }

  return false;
}


//  ------------------------------------------------------------------

int GMsgBodyView::Continue() {
  
  if(can_pagedown)
    return PageDown();
  
  return false;
}


//  ------------------------------------------------------------------

int GMsgBodyView::ThumbTrack(int pos) {

  return ThumbPosition(pos);
}


//  ------------------------------------------------------------------

int GMsgBodyView::ThumbPosition(int pos) {
  
  upperline = pos;
  lowerline = upperline + height - 1;
  can_pagedown = lowerline < (msg->display.size()-1);
  Paint();
  return true;
}


//  ------------------------------------------------------------------

void GMsgBodyView::UpdateScrollbar() {

  if(CFG->disppagebar) {
    if(scrollbar_visible or (CFG->disppagebar == ALWAYS)) {
      window.vscrollbar(msg->display.size(), msg->display.size()-height, upperline);
    }
  }
}


//  ------------------------------------------------------------------

