
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmarea.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Arealist functions.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <gmbezyc.h>
#include <gmbfido.h>
#include <gmbgold.h>
#include <gmbhuds.h>
#include <gmbjamm.h>
#include <gmbpcbd.h>
#include <gmbsqsh.h>
#include <gmbwcat.h>
#include <gmbxbbs.h>


//  ------------------------------------------------------------------

garealist::garealist() {

  msgbases = 0;

  adeptxbbs_manager = new gmb_adeptxbbs_manager;
  ezycom_manager    = new gmb_ezycom_manager;
  fido_manager      = new gmb_fido_manager;
  goldbase_manager  = new gmb_goldbase_manager;
  hudson_manager    = new gmb_hudson_manager;
  jam_manager       = new gmb_jam_manager;
  pcboard_manager   = new gmb_pcboard_manager;
  squish_manager    = new gmb_squish_manager;
  wildcat_manager   = new gmb_wildcat_manager;
}


//  ------------------------------------------------------------------

garealist::~garealist() {

  reset();
  
  delete adeptxbbs_manager;
  delete ezycom_manager;
  delete fido_manager;
  delete goldbase_manager;
  delete hudson_manager;
  delete jam_manager;
  delete pcboard_manager;
  delete squish_manager;
  delete wildcat_manager;
}


//  ------------------------------------------------------------------

void garealist::reset() {

  listscan.Reset();
  for(int n=0; n<list.size(); n++)
    delete list[n];
  list.reset();
  
  adeptxbbs_manager->reset();
  ezycom_manager->reset();
  fido_manager->reset();
  goldbase_manager->reset();
  hudson_manager->reset();
  jam_manager ->reset();
  pcboard_manager->reset();
  squish_manager->reset();
  wildcat_manager->reset();
}


//  ------------------------------------------------------------------

void garealist::adeptxbbs_init(const gstring& path, int userno) {

  adeptxbbs_manager->set_path(path);
  adeptxbbs_manager->set_userno(userno);
  adeptxbbs_manager->initialize();
}


//  ------------------------------------------------------------------

void garealist::ezycom_init(const gstring& msgbasepath, const gstring& userbasepath, int userno) {

  ezycom_manager->set_msgbasepath(msgbasepath);
  ezycom_manager->set_userbasepath(userbasepath);
  ezycom_manager->set_userno(userno);
  ezycom_manager->initialize();
}


//  ------------------------------------------------------------------

void garealist::fido_init(const gstring& lastread, int hwmarks, int nullfix, int userno, const gstring& userpath) {

  fido_manager->set_lastread(lastread);
  fido_manager->set_hwmarks(hwmarks);
  fido_manager->set_nullfix(nullfix);
  fido_manager->set_userno(userno);
  fido_manager->set_userpath(userpath);
  fido_manager->initialize();
}


//  ------------------------------------------------------------------

void garealist::goldbase_init(const gstring& path, const gstring& syspath, int userno) {

  goldbase_manager->set_path(path);
  goldbase_manager->set_syspath(syspath);
  goldbase_manager->set_userno(userno);
  goldbase_manager->initialize();
}


//  ------------------------------------------------------------------

void garealist::hudson_init(const gstring& path, const gstring& syspath, int userno, long sizewarn, int ra2usersbbs) {

  hudson_manager->set_path(path);
  hudson_manager->set_syspath(syspath);
  hudson_manager->set_userno(userno);
  hudson_manager->set_sizewarn(sizewarn);
  hudson_manager->set_ra2usersbbs(ra2usersbbs);
  hudson_manager->initialize();
}


//  ------------------------------------------------------------------

void garealist::jam_init(const gstring& path, int harddelete) {

  jam_manager->set_path(path);
  jam_manager->set_harddelete(harddelete);
  jam_manager->initialize();
}


//  ------------------------------------------------------------------

void garealist::pcboard_init(const gstring& path, int userno) {

  pcboard_manager->set_path(path);
  pcboard_manager->set_userno(userno);
  pcboard_manager->initialize();
}


//  ------------------------------------------------------------------

void garealist::squish_init(const gstring& userpath, int userno, int direct, int squishscan) {

  squish_manager->set_userpath(userpath);
  squish_manager->set_userno(userno);
  squish_manager->set_direct(direct);
  squish_manager->set_squishscan(squishscan);
  squish_manager->initialize();
}


//  ------------------------------------------------------------------

void garealist::wildcat_init(int userno) {

  wildcat_manager->set_userno(userno);
  wildcat_manager->initialize();
}


//  ------------------------------------------------------------------

gactivearea::gactivearea(gmb_area& a, garealistdata& d, garealist& l)
  : area(a), data(d), list(l),
    msgnset(a.msgnset),
    pmrkset(a.pmrkset),
    markset(d.markset)
{
  mode_readmarked = mode_readpersonal = false;
  info_bookmark = 0;
  initialize();
}


//  ------------------------------------------------------------------

gactivearea::~gactivearea() {

}


//  ------------------------------------------------------------------

gactivearea* garealist::get_active(int n) {

  gmb_area* a = NULL;
  garealistdata* data = list[n];
  switch(data->msgbase()) {
    case GMB_FIDO:
    case GMB_FTS1:
    case GMB_OPUS:
      a = fido_manager->activate(data);
      break;
    case GMB_HUDSON:
      a = hudson_manager->activate(data);
      break;
    case GMB_GOLDBASE:
      a = goldbase_manager->activate(data);
      break;
    case GMB_JAM:
      a = jam_manager->activate(data);
      break;
    case GMB_SQUISH:
      a = squish_manager->activate(data);
      break;
    case GMB_EZYCOM:
      a = ezycom_manager->activate(data);
      break;
    case GMB_PCBOARD:
      a = pcboard_manager->activate(data);
      break;
    case GMB_WILDCAT:
      a = wildcat_manager->activate(data);
      break;
    case GMB_ADEPTXBBS:
      a = adeptxbbs_manager->activate(data);
      break;
  }

  gactivearea* area = new gactivearea(*a, *data, *this);

  return area;
}


//  ------------------------------------------------------------------

gactivearea* garealist::activate(int areaid) {

  for(int n=0; n<list.size(); n++) {
    if(list[n]->areaid() == areaid)
      return get_active(n);
  }

  return NULL;
}


//  ------------------------------------------------------------------

void garealist::deactivate(gactivearea* area) {

  if(area) {
    switch(area->msgbase()) {
      case GMB_FIDO:
      case GMB_FTS1:
      case GMB_OPUS:
        fido_manager->deactivate(area->area_pointer());
        break;
      case GMB_HUDSON:
        hudson_manager->deactivate(area->area_pointer());
        break;
      case GMB_GOLDBASE:
        goldbase_manager->deactivate(area->area_pointer());
        break;
      case GMB_JAM:
        jam_manager->deactivate(area->area_pointer());
        break;
      case GMB_SQUISH:
        squish_manager->deactivate(area->area_pointer());
        break;
      case GMB_EZYCOM:
        ezycom_manager->deactivate(area->area_pointer());
        break;
      case GMB_PCBOARD:
        pcboard_manager->deactivate(area->area_pointer());
        break;
      case GMB_WILDCAT:
        wildcat_manager->deactivate(area->area_pointer());
        break;
      case GMB_ADEPTXBBS:
        adeptxbbs_manager->deactivate(area->area_pointer());
        break;
    }
    delete area;
  }
}


//  ------------------------------------------------------------------

void garealist::wide_open() {

  if(msgbases & GMB_ADEPTXBBS)  adeptxbbs_manager->wide_open();
  if(msgbases & GMB_EZYCOM)     ezycom_manager->wide_open();
  if(msgbases & GMB_FIDO)       fido_manager->wide_open();
  if(msgbases & GMB_GOLDBASE)   goldbase_manager->wide_open();
  if(msgbases & GMB_HUDSON)     hudson_manager->wide_open();
  if(msgbases & GMB_JAM)        jam_manager->wide_open();
  if(msgbases & GMB_PCBOARD)    pcboard_manager->wide_open();
  if(msgbases & GMB_WILDCAT)    wildcat_manager->wide_open();
}


//  ------------------------------------------------------------------

void garealist::wide_close() {

  if(msgbases & GMB_WILDCAT)    wildcat_manager->wide_close();
  if(msgbases & GMB_PCBOARD)    pcboard_manager->wide_close();
  if(msgbases & GMB_JAM)        jam_manager->wide_close();
  if(msgbases & GMB_HUDSON)     hudson_manager->wide_close();
  if(msgbases & GMB_GOLDBASE)   goldbase_manager->wide_close();
  if(msgbases & GMB_FIDO)       fido_manager->wide_close();
  if(msgbases & GMB_EZYCOM)     ezycom_manager->wide_close();
  if(msgbases & GMB_ADEPTXBBS)  adeptxbbs_manager->wide_close();
}


//  ------------------------------------------------------------------

