
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmarea.h,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------


#include <gmbarea.h>


//  ------------------------------------------------------------------

class garealist;
class gactivearea;


//  ------------------------------------------------------------------
//  Search locations

const uint GFIND_FROM    = 0x0001;
const uint GFIND_TO      = 0x0002;
const uint GFIND_SUBJECT = 0x0004;
const uint GFIND_BODY    = 0x0008;
const uint GFIND_HDR     = GFIND_FROM | GFIND_TO | GFIND_SUBJECT;
const uint GFIND_HDRTXT  = GFIND_HDR | GFIND_BODY;


//  ------------------------------------------------------------------
//  Charset encoding formats

const int GCHENC_CMP = 0x0001;    // Composed characters
const int GCHENC_I51 = 0x0002;    // FSC-51
const int GCHENC_MNE = 0x0004;    // K.Simonsen Mnemonic
const int GCHENC_QP  = 0x0008;    // Quoted-Printable


//  ------------------------------------------------------------------
//  GoldED message header

class GMsg : public gmsg {

protected:

  static ulong msgcount;

  void  KludgeAREA(const char* ptr);
  void  KludgeINTL(const char* ptr);
  void  KludgeFMPT(const char* ptr);
  void  KludgeTOPT(const char* ptr);
  void  KludgeMSGID(const char* ptr);
  void  KludgeREPLY(const char* ptr);
  void  KludgeDOMAIN(const char* ptr);
  void  KludgeFLAGS(const char* ptr);
  void  KludgeSEENBY(const char* ptr);
  void  KludgePATH(const char* ptr);
  void  KludgeMSGTO(const char* ptr);
  void  KludgePID(const char* ptr);
  void  KludgeREPLYADDR(const char* ptr);
  void  KludgeREPLYTO(const char* ptr);
  void  KludgeGIF(const char* ptr);
  void  KludgeMOOD(const char* ptr);
  void  KludgeFWDFROM(const char* ptr);
  void  KludgeFWDORIG(const char* ptr);
  void  KludgeFWDTO(const char* ptr);
  void  KludgeFWDDEST(const char* ptr);
  void  KludgeFWDSUBJ(const char* ptr);
  void  KludgeFWDAREA(const char* ptr);
  void  KludgeFWDMSGID(const char* ptr);

  void  KludgeREPLY_TO(const char* ptr);
  void  KludgeSENDER(const char* ptr);
  void  KludgeFROM(const char* ptr);
  void  KludgeTO(const char* ptr);
  void  KludgeCC(const char* ptr);
  void  KludgeBCC(const char* ptr);
  void  KludgeNEWSGROUPS(const char* ptr);
  void  KludgeFOLLOWUPTO(const char* ptr);
  void  KludgeSUBJECT(const char* ptr);
  void  KludgeDATE(const char*);
  void  KludgeMESSAGE_ID(const char* ptr);
  void  KludgeREFERENCES(const char* ptr);
  void  KludgeIN_REPLY_TO(const char* ptr);
  void  KludgeORGANIZATION(const char* ptr);

  char* unwrap_line(int lineno, const char* ptr, int addspace);

  bool  handle_kludges(int lineno, int kludgenum, const char* ptr);
  bool  handle_rfcs(int lineno, int kludgenum, const char* ptr);

  void  scan_kludges();
  int   scan_line(int lineno, const char* ptr, int mask);

  void  text_to_hexdump();

  bool  examine_kludge(char* ptr, bool& gotmime, bool& qpencoded);
  void  scan_msg_body_top(bool& gotmime, bool& qpencoded);

public:

  GMsg();
  ~GMsg();

  void reset(bool normal=true);

  int          quotepct;         // Quote/text percentage

  uint         foundwhere;       // Where searched string was found (GFIND_*)
  int          foundtopline;     // First line where a searched string was found

  uint         orig_timesread;   // Original timesread

  gactivearea* area;             // Pointer to active area

  int          line_width;       // Line width used

  gvector<int> display;          // Index of displayable lines

  void wrap_lines(int textwidth, int quotewidth);

  void lines_to_text();
  void text_to_lines(int linewidth, bool set_position=false);

  void reindex_lines(bool set_position=false, int viewhidden=-1, int viewkludge=-1, int viewquote=-1);

  void load_textfile(const char* filename, int margin, bool raw=false);
  void save_textfile(const char* filename, int mode=0);

  void make_kludge_flags(gstring& buf);
  void make_tagline(gstring& buf);
  void make_tearline(gstring& buf);
  void make_originline(gstring& buf);

  GMsg& copy(const GMsg& m);

};


//  ------------------------------------------------------------------
//  Random System

enum {
  GRP_ADDRESS,
  GRP_AKAMATCHING,
  GRP_AREACOPYADDID,
  GRP_AREACOPYDIRECT,
  GRP_AREACOPYTO,
  GRP_AREAFREQDIRECT,
  GRP_AREAFREQTO,
  GRP_AREAREPLYDIRECT,
  GRP_AREAREPLYTO,
  GRP_ATTRIBUTES,
  GRP_CTRLINFO,
  GRP_EDITHARDTERM,
  GRP_EDITMIXCASE,
  GRP_EDITREPLYRE,
  GRP_FOLLOWUPTO,
  GRP_FORCETEMPLATE,
  GRP_GIF,
  GRP_INPUTFILE,
  GRP_INTERNETADDRESS,
  GRP_INTERNETGATE,
  GRP_INTERNETMSGID,
  GRP_INTERNETRFCBODY,
  GRP_LOADLANGUAGE,
  GRP_MOOD,
  GRP_MSGLISTDATE,
  GRP_MSGLISTFAST,
  GRP_MSGLISTFIRST,
  GRP_MSGLISTHEADER,
  GRP_MSGLISTWIDESUBJ,
  GRP_NETNAME,
  GRP_NEWSGROUPS,
  GRP_ORGANIZATION,
  GRP_ORIGIN,
  GRP_OUTPUTFILE,
  GRP_PLAY,
  GRP_QUOTEBUFFILE,
  GRP_QUOTECHARS,
  GRP_QUOTESTRING,
  GRP_QUOTEWRAPHARD,
  GRP_REPLYTO,
  GRP_SEARCHFOR,
  GRP_SENDER,
  GRP_SIGNATURE,
  GRP_TAGLINE,
  GRP_TAGLINECHAR,
  GRP_TAGLINEFILE,
  GRP_TAGLINESUPPORT,
  GRP_TEARLINE,
  GRP_TEMPLATE,
  GRP_TEMPLATEMATCH,
  GRP_TWITMODE,
  GRP_USEFWD,
  GRP_USERNAME,
  GRP_VIEWHIDDEN,
  GRP_VIEWKLUDGE,
  GRP_VIEWQUOTE,
  GRP_WHOTO,
  GRP_XLATEXPORT,
  GRP_XLATIMPORT,

  GRP_MAX
};


//  ------------------------------------------------------------------

extern garealist AL;


//  ------------------------------------------------------------------

