
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: golded4.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  Main entry point and overlay size setting function.
//  ------------------------------------------------------------------


#include "golded.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif


//  ------------------------------------------------------------------

#if defined(__BORLANDC__) and defined(__MSDOS__)
bool overlaytype = OVL_EMS;
#endif


//  ------------------------------------------------------------------
//  Main function

int _stdc main(int argc, char *argv[]) {

  #if defined(__BORLANDC__) and defined(__MSDOS__)
  set_new_handler(0);
  #endif

  // Allow more than 20 files to be opened
  #if defined(__WATCOMC__)
  _grow_handles(30);
  #endif

  // Overlay swapping
  #if defined(__BORLANDC__) and defined(__MSDOS__)
  switch(overlaytype) {
    case OVL_EMS:
      _OvrInitEms(0,0,0);
      break;
    case OVL_EXT:
      _OvrInitExt(0,0);
      break;
  }
  #endif

  Initialize(argc, argv);
  Reader();
  return errorlevel;
}


//  ------------------------------------------------------------------
//  Set overlay buffer size based on commandline/environment variable
//  Called from the init code

#if defined(__BORLANDC__) and defined(__MSDOS__)

//unsigned _ovrbuffer = 100*64;    // Set default overlay buffer to 100k

void set_ovr_stuff(char* p) {

  if(isdigit(*p)
    _ovrbuffer = atoi(p)*64;
  else {
    if(strnieql(p, "EMS"))
      overlaytype = OVL_EMS;
    else if(strnieql(p, "EXT"))
      overlaytype = OVL_EXT;
    else if(strnieql(p, "DISK"))
      overlaytype = OVL_DISK;
  }
}

void _stdc setovrbuf(void) {

  // First try commandline parameters
  char buf[256];
  bool gotit = false;
  char* ptr = (char*)MK_FP(_psp, 0x80);
  strncpy(buf, ptr+1, *ptr);
  buf[(byte)*ptr] = 0;
  ptr = buf;
  while(*ptr) {
    ptr = strskip_wht(ptr);
    if(strchr("-/", *ptr++)) {
      if(toupper(*ptr++) == 'O') {
        set_ovr_stuff((*ptr == '=') ? ptr+1 : ptr);
        gotit = true;
        break;
      }
    }
    ptr = strskip_txt(ptr);
  }
  if(not gotit) {
    // Then check environment
    word* wptr = (word*)MK_FP(_psp, 0x2C);
    ptr = (char*)MK_FP(*wptr, 0);
    do {
      if(strnieql(ptr, "GEDCMD=", 7)) {
        char* ptr2 = ptr+7;
        while(*ptr2) {
          ptr2 = strskip_wht(ptr2);
          if(strchr("-/", *ptr2++)) {
            if(toupper(*ptr2++) == 'O') {
              set_ovr_stuff((*ptr2 == '=') ? ptr2+1 : ptr2);
              gotit = true;
              break;
            }
          }
          ptr2 = strskip_txt(ptr2);
        }
        break;
      }
      ptr += strlen(ptr) + 1;
    } while(*ptr);
  }
}

//  ------------------------------------------------------------------
//  The following pragma makes the startup (C0) code call setovrbuf()
//  as the highest priority. Priority 0 comes just before the overlay
//  manager is called (which has priority 1).
//
//  NOTE: This has only been tested with BC++ 3.0 and 3.1, but should
//        work with later versions too.
//  ------------------------------------------------------------------

#pragma startup setovrbuf 0

#endif


//  ------------------------------------------------------------------
//  Remove exception handling in Borland C++ 4.xx

#if defined(__MSDOS__) and (__BCPLUSPLUS__ > 0x310)

extern "C" void _ExceptInit(void) {}

new_handler _new_handler = NULL;

new_handler set_new_handler(new_handler p) {

  new_handler t = _new_handler;
  _new_handler = p;
  return t;
}

void *operator new( size_t size ) {

  void * p = 0;

  size = size ? size : 1;
  while((p = malloc(size)) == NULL && _new_handler != NULL)
    _new_handler();

  return p;
}

#endif


//  ------------------------------------------------------------------
//  Program version

#if defined(__DJGPP__)
# define __SHORTGPID__ "GED3DJ"
# define __GPID__      "GoldED/3DJ"
#elif defined(__MSDOS__) and defined(__386__)
# define __SHORTGPID__ "GED3"
# define __GPID__      "GoldED/386"
#elif defined(__MSDOS__)
# define __SHORTGPID__ "GED"
# define __GPID__      "GoldED"
#elif defined(__OS2__)
# define __SHORTGPID__ "GED2"
# define __GPID__      "GoldED/2"
#elif defined(__WIN32__)
# define __SHORTGPID__ "GEDW32"
# define __GPID__      "GoldED/W32"
#elif defined(__linux__)
# define __SHORTGPID__ "GEDLNX"
# define __GPID__      "GoldED/LNX"
#endif

#define __GVER__ "4.00a1"
#define __GREV__ "Alpha1"
//#define __GREV__ ""

#define __GOLDED_MAJOR__ 4
#define __GOLDED_MINOR__ 0
#define __GOLDED_MIPID__ 1
#define __GOLDED_REV__   1


//  ------------------------------------------------------------------
//  Global compile constants

char* __shortgpid__ = __SHORTGPID__;
char* __gpid__ = __GPID__;
char* __gver__ = __GVER__;
char* __grev__ = __GREV__;
char* __date__ = __DATE__;
char* __time__ = __TIME__;
int __golded_major__ = __GOLDED_MAJOR__;
int __golded_minor__ = __GOLDED_MINOR__;
int __golded_mipid__ = __GOLDED_MIPID__;
int __golded_rev__   = __GOLDED_REV__;

// Full Product ID (for PID kludge)
char __pid__[40] = { __SHORTGPID__ };


//  ------------------------------------------------------------------

