
//  ------------------------------------------------------------------
//  GoldED. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gwinfsel.cpp,v 1.1.1.1 1998/11/21 19:38:05 odinn Exp $
//  ------------------------------------------------------------------
//  File selection window.
//  ------------------------------------------------------------------


#include <gwinpick.h>
#include <gdirposx.h>


//  ------------------------------------------------------------------

class gfileselector : public gwinpick {

protected:

  void fill_bar(int pos, int idx);

public:

  gwindow window;

  gvector<gstring> bar;
  gvector<gffblk>  fblk;
  int              selfiles;
  int              totfiles;
  long             selbytes;
  long             totbytes;
  
  void open();
  void close();
  void precursor();
  void do_delayed();
  void display_line(bool is_bar=false);
  void display_page();
  void scroll(int where);
  bool handle_key(Key& code);

};


//  ------------------------------------------------------------------

void gfileselector::fill_bar(int pos, int idx) {

  gffblk& fb = fblk[idx];
  struct tm& ft = *glocaltime(&fb.mtime);    
  bar[pos].strf(
    "%c%-*.*s %8lu %2d-%02d-%02d %2d:%02d ",
    fb.selected ? MMRK_MARK : ' ',
    width-62, width-62,
    fb.name,
    fb.size,
    ft.tm_mday,
    ft.tm_mon+1,
    ft.tm_year,
    ft.tm_hour,
    ft.tm_min
  );
}


//  ------------------------------------------------------------------

void gfileselector::display_line(bool is_bar) {

  window.prints(position, 0, is_bar ? color_bar : color_window, bar[position].c_str());
  window.printc(position, 0, is_bar ? color_bar : (*bar[position] == ' ' ? color_window : color_marked), *bar[position]);
}


//  ------------------------------------------------------------------

void gfileselector::display_page() {

  index -= position;
  uint n, m = maximum_index - index;

  for(n=0; n<=maximum_position and n<=m; n++)
    fill_bar(n, index+n);
  if(n <= maximum_position) {
    bar[n].resize(0);
    bar[n].resize(width-36, _box_table[btype][1]);
  }
  for(++n; n<=maximum_position; n++) {
    bar[n].resize(0);
    bar[n].resize(width-36, ' ');
  }
  for(n=0; n<=maximum_position; n++) {
    window.prints(n, 0, n==position ? color_bar : (*bar[n] != ' ' ? color_border : color_window), bar[n].c_str());
    window.printc(n, 0, n==position ? color_bar : (*bar[n]==MMRK_MARK ? color_marked : (*bar[n] != ' ' ? color_border : color_window)), *bar[n]);
  }
  index += position;
}


//  ------------------------------------------------------------------

void gfileselector::scroll(int where) {

  if(where == SUP) {
    for(int n=1; n<=maximum_position; n++)
      bar[n-1] = bar[n];
  }
  else {
    for(int n=maximum_position; n; n--)
      bar[n] = bar[n-1];
  }

  fill_bar(where==SDOWN ? 0 : maximum_position, index);

  window.scrollbox(0, 0, maximum_position, width-37, 1, where);
}


//  ------------------------------------------------------------------

void gfileselector::do_delayed() {

  if(CFG->filelistpagebar)
    window.vscrollbar(maximum_index+1, maximum_index, index);
}


//  ------------------------------------------------------------------

void gfileselect::update_count_display() {

  window.printf(1, width-34, color_window, "%s %10s", CFG->language[LNG_SelectedFiles], longdotstr(selfiles));
  window.printf(2, width-34, color_window, "%s %10s", CFG->language[LNG_SelectedBytes], longdotstr(selbytes));
}


//  ------------------------------------------------------------------

void gfileselect::unmark_all() {

  for(int n=0; n<=maximum_index; n++)
    fblk[n].selected = false;
  selfiles = 0;
  selbytes = 0;
  display_page();
  update_count_display();
}


//  ------------------------------------------------------------------

void gfileselect::mark_all() {

  for(int n=0; n<=maximum_index; n++)
    fblk[n].selected = true;
  selfiles = totfiles;
  selbytes = totbytes;
  display_page();
  update_count_display();
}


//  ------------------------------------------------------------------

void gfileselect::select() {

  if(not selfiles) {  // If no files selected, select cursor file
    fblk[index].selected = true;
    selbytes += fblk[index].size;
    selfiles++;
    bar[position].set_char(0, MMRK_MARK);
    update_count_display();
  }
}


//  ------------------------------------------------------------------

void gfileselect::toggle_mark() {

  if(fblk[p->idx].selected)
    unmark();
  else
    mark();
}


//  ------------------------------------------------------------------

void gfileselect::toggle_mark_down() {

  toggle_mark();
  cursor_down();
}


//  ------------------------------------------------------------------

void gfileselect::mark() {

  if(not fblk[index].selected) {
    fblk[index].selected = true;
    selbytes += fblk[index].size;
    selfiles++;
    bar[position].set_char(0, MMRK_MARK);
    update_count_display();
  }
}


//  ------------------------------------------------------------------

void gfileselect::unmark() {

  if(fblk[index].selected) {
    fblk[index].selected = false;
    selbytes -= fblk[index].size;
    selfiles--;
    bar[position].set_char(0, ' ');
    update_count_display();
  }
}


//  ------------------------------------------------------------------

void gfileselect::mark_down() {

  mark();
  cursor_down();
}


//  ------------------------------------------------------------------

void gfileselect::unmark_down() {

  unmark();
  cursor_down();
}


//  ------------------------------------------------------------------

bool gfileselect::handle_key() {

  if(not gwinpick::handle_key()) {

    switch(key) {
      case Key_StrG:
      case Key_S_8:
        mark_all();
        break;

      case Key_Sls:
        unmark_all();
        break;

      case Key_Ins:
      case Key_PlsG:
      case Key_S_Equ:
        mark_down();
        break;

      case Key_Del:
      case Key_Min:
      case Key_MinG:
        unmark_down();
        break;

      case Key_Space:
        toggle_mark_down();
        break;

      case Key_Ent:
        select();
        return false;

      case Key_Esc:
        unmark_all();
        return false;

      default:
        return false;
    }
  }

  return true;
}


//  ------------------------------------------------------------------

static int _stdc cmp_ffb(const gffblk* a, const gffblk* b) {

  return stricmp(a->name, b->name);
}


//  ------------------------------------------------------------------

static int _stdc cmp_ffbs(const gffblk* a, const gffblk* b) {

  int cmp = compare_two(b->selected, a->selected);
  if(cmp)
    return cmp;

  return stricmp(a->name, b->name);
}


//  ------------------------------------------------------------------

void gfileselector::run(const gstring& title, gfilespec& fspec) {

  PInf p;
  Pick pick;
  char buf[256], tmp[80];
//  char drive[GMAXDRIVE], dir[GMAXDIR], name[GMAXFILE], ext[GMAXEXT];
  uint MIN_POS=0, MAX_POS=MIN_POS+gvid.numrows-10;
  int done=false, winop=NO;

  const char* wild_all = GOLD_WILDCARD_ALL_FILES;

  p.aborted = NO;
  selfiles = totfiles = 0;
  selbytes = totbytes = 0;

  if(fspec.path.last_char() == GOLD_SLASH_CHR)
    fspec.path += wild_all;
  gstring fbuf = fspec.path;

  _splitpath(fbuf, drive, dir, name, ext);
  if(fbuf.empty()) {
    fspec.path = wild_all;
    fbuf = wild_all;
  }
  else if(not fbuf.find_chars("*?") and is_dir(fbuf)) {
    AddBackslash(fspec.path) += wild_all;
    AddBackslash(fbuf) += wild_all;
  }
  _splitpath(fbuf, drive, dir, name, ext);

  f.open(fbuf);
  done = not f.first();
  f.close();

  if((strpbrk(fbuf, "*?")) or done) {
    winop = YES;
    vcurhide();
    wopen_(6,0, MAX_POS+3, width, color_bordertype, color_border, color_window, color_scrollbar);

    wvline(0, width-36, MAX_POS+1, color_bordertype, color_border);
    whline(MAX_POS-5, width-36, 36, color_bordertype, color_border);

    sprintf(buf, " %s ", fbuf.trim());
    wmessage(buf, TP_BORD, 1, color_title);
    wmessage(title.c_str(), TP_BORD, width-34, color_title);

    update_statusline(CFG->language[LNG_SelectFiles]);

    sprintf(buf, "%s %10s", CFG->language[LNG_SelectedFiles], longdotstr(tmp, selfiles));
    wprints(1,         width-34, color_window, buf);
    sprintf(buf, "%s %10s", CFG->language[LNG_SelectedBytes], longdotstr(tmp, selbytes));
    wprints(2,         width-34, color_window, buf);

    sprintf(buf, "%s %10s", CFG->language[LNG_TotalFiles], longdotstr(tmp, totfiles));
    wprints(4,         width-34, color_window, buf);
    sprintf(buf, "%s %10s", CFG->language[LNG_TotalBytes], longdotstr(tmp, totbytes));
    wprints(5,         width-34, color_window, buf);

    wprints(MAX_POS-4, width-34, color_window, CFG->language[LNG_MoveFileBar]);
    wprints(MAX_POS-3, width-34, color_window, CFG->language[LNG_ToggleFile]);
    wprints(MAX_POS-2, width-34, color_window, CFG->language[LNG_AcceptFiles]);
    wprints(MAX_POS-1, width-34, color_window, CFG->language[LNG_SelectAllFiles]);
    wprints(MAX_POS-0, width-34, color_window, CFG->language[LNG_AbortFiles]);

    wprints(MAX_POS/2, 0, color_window, CFG->language[LNG_ScanningDirectory]);
  }

  bar = (Barlist*)throw_calloc(gvid.numrows-7, sizeof(Barlist));
  FFblk* fb;

  // if drive was specified, change to it
  Path spec, namext;
  char* q = strcpy(spec, fbuf);

  // split up input filespec into its current
  // working directory and filename/extension
  char* r = strrchr(q, GOLD_SLASH_CHR);
  #if defined(GOLD_HAS_DRIVES)
  if(r == NULL)
    if(q[1] == ':')
      r = q + 1;
  #endif
  if(r == NULL) {
    *dir = NUL;
    strcpy(namext, q);
  }
  else {
    strins(" ", ++r, 0);
    *r++ = NUL;
    strcpy(dir, q);
    strcpy(namext, r);
  }

  gposixdir d;
  if(d.open(dir)) {	
    while(d.read()) {
      if(not d.is_directory() and gwildmati(d.name(), namext)) {
	fblk = (FFblk*)throw_realloc(fblk, (totfiles+1)*sizeof(FFblk));
	fb = fblk + totfiles;
	fb->mtime = d.modified_time();
	fb->size = d.size();
	strcpy(fb->name, d.name());
	fb->selected = false;
	totbytes += d.size();
	totfiles++;
      }
    }
  }
  f.close();

  if(totfiles) {
    if(totfiles > 1) {
      qsort(fblk, totfiles, sizeof(FFblk), (StdCmpCP)cmp_ffb);

      sprintf(buf, "%s %10s", CFG->language[LNG_TotalFiles], longdotstr(tmp, totfiles));
      wprints(4,         width-34, color_window, buf);
      sprintf(buf, "%s %10s", CFG->language[LNG_TotalBytes], longdotstr(tmp, totbytes));
      wprints(5,         width-34, color_window, buf);

      memset(&p, 0, sizeof(PInf));
      memset(&pick, 0, sizeof(Pick));

      p.delay  = 0;
      p.maxidx = totfiles-1;
      p.maxpos = minimum_of_two((uint)MAX_POS, p.maxidx);
      p.helpcat = H_FileSelect;
      p.listwrap = CFG->displistwrap;

      pick.disp   = fs_disp;
      pick.page   = fs_page;
      pick.scroll = fs_scroll;
      pick.dokey  = fs_dokey;
      pick.dodelayed = fs_dodelayed;
                                 // 12345678901234567890123456789012345678901234
      wprints(MAX_POS/2,0,color_window, "                                            ");

      fs_page(&p);

      if(p.maxpos == p.maxidx and p.maxpos < MAX_POS) {
        memset(buf, p.btype == 7 ? '-' : '', width-36);
        *(buf+width-36) = NUL;
        wprints(p.maxpos+1, 0, color_window, buf);
      }

      Picker(&p, &pick);
    }
    else {
      selfiles = 1;
      fblk[0].selected = YES;
    }

    throw_release(bar);

    if(p.aborted or selfiles == 0) {
      fspec.fblk = NULL;
      fspec.files = 0;
      throw_release(fblk);
    }
    else {
      // Realloc to include only the selected files
      qsort(fblk, totfiles, sizeof(FFblk), (StdCmpCP)cmp_ffbs);
      fspec.fblk = fblk = (FFblk*)throw_realloc(fblk, (selfiles+1)*sizeof(FFblk));
      _splitpath(fspec.path, drive, dir, name, ext);
      #ifdef GOLD_HAS_DRIVES
      if(not *drive) {
        Path cwd;
        gedcwd(cwd);
        sprintf(drive, "%c:", *cwd);
      }
      if(not *dir) {
        *dir = '\\';
        getcurdir((toupper(*drive)-'A')+1, dir+1);
        AddBackslash(dir);
      }
      if(*dir != '\\') {
        *buf = '\\';
        getcurdir((toupper(*drive)-'A')+1, buf+1);
        AddBackslash(buf);
        strcat(buf, dir);
        strcpy(dir, buf);
      }
      sprintf(fspec.path, "%s%s", drive, dir);
      #else
      AddBackslash(strcpy(fspec.path, dir));
      #endif
      fspec.files = selfiles;
      fblk = NULL;
    }
  }
  else {
    window.prints(height/2, 0, color_window, CFG->language[LNG_NoFilesFound]);
    update_statusline(CFG->language[LNG_FilesPressKey]);
    throw_release(fblk);
    throw_release(bar);
    waitkeyt(10000);
  }

  if(winop) {
    wclose();
    vcurshow();
  }
}


//  ------------------------------------------------------------------

bool golded_fileselect::handle_key(Key& code) {

  register uint n;
  char buf[256], tmp[80];
  Key kk, key = *code;

  *keycode = 0;
  if(key < KK_Commands) {
    key = key_tolower(key);
    kk = SearchKey(key, FileKey, FileKeys);
    if(kk)
      key = kk;
  }

  switch(key) {
    case KK_FileUnMarkAll:
      unmark_all();
      break;

    case KK_FileMarkAll:
      mark_all();
      break;

    case KK_FileAskExit:
      {
        GMenuQuit MenuQuit;
        if(MenuQuit.Run())
          *keycode = KK_FileQuitNow;
      }
      break;

    case KK_FileQuitNow:
      gkbd.quitall = true;
      ////////////// Drop through

    case KK_FileAbort:
      unmark_all();
      return false;

    case KK_FileSelect:
      select();
      return false;

    case KK_FileToggleMark:
      toggle_mark();
      break;

    case KK_FileMark:
      mark();
      break;

    case KK_FileUnMark:
      unmark();
      break;

    case KK_FileGotoPrev:
      *keycode = Key_Up;
      break;

    case KK_FileGotoNext:
      *keycode = Key_Dwn;
      break;

    case KK_FileGotoFirst:
      *keycode = Key_Home;
      break;

    case KK_FileGotoLast:
      *keycode = Key_End;
      break;

    case KK_FileDosShell:
      ShellToDos(getenv("COMSPEC"), CFG->language[LNG_DOS_Shell], LGREY|_BLACK, YES, true, false, fs_area);
      break;

    case Key_Tick:
      CheckTick(KK_FileQuitNow);
      break;

    case KK_FileUndefine:
      break;

    default:
      if(not PlayMacro(key, KT_F))
        SayBibi();
  }
  return(YES);
}


//  ------------------------------------------------------------------

