
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gcrcall.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  CRC-16, CRC-32 and Hashing.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GCRCALL_H
#define __GCRCALL_H


//  ------------------------------------------------------------------
//  Required headers

#include <gstrall.h>


//  ------------------------------------------------------------------
//  Declare the CRC tables

extern word __FAR __crc16_table[];
extern dword __FAR __crc32_table[];


//  ------------------------------------------------------------------
//  Generate/update a CRC-16 or CRC-32 value
//
//  These inline functions were derived from various #define macro
//  implementations. Some of those implementations masked the right
//  shifts to ensure correct zero fill. But this is C++, where zero
//  fill is guaranteed for unsigned operands, and besides we use
//  prototyped unsigned parameters anyway, so we have no problem here.

inline word updCrc16(byte ch, word crc) { return (word)(__crc16_table[byte(crc >> 8)] ^ (crc << 8) ^ (ch)); }
inline dword updCrc32(byte ch, dword crc) { return (dword)(__crc32_table[byte(crc) ^ byte(ch)] ^ (crc >> 8)); }


//  ------------------------------------------------------------------
//  Define CRC masks in the "normal" and "CCITT" variants

const word CRC16_MASK_NORMAL = 0;
const word CRC16_MASK_CCITT  = 0xFFFFU;
const dword CRC32_MASK_NORMAL = 0;
const dword CRC32_MASK_CCITT  = 0xFFFFFFFFUL;


//  ------------------------------------------------------------------
//  Prototypes

word strCrc16(const char* s,         int nocase=YES, word mask=CRC16_MASK_NORMAL);
word memCrc16(const void* m, word l, int nocase=YES, word mask=CRC16_MASK_NORMAL);
word memCrc16(const void* m, long l, int nocase=YES, word mask=CRC16_MASK_NORMAL);

inline word gcrc16u(const char* s) { return strCrc16(s); }

dword strCrc32(const char* s,         int nocase=YES, dword mask=CRC32_MASK_NORMAL);
dword memCrc32(const void* m, word l, int nocase=YES, dword mask=CRC32_MASK_NORMAL);
dword memCrc32(const void* m, long l, int nocase=YES, dword mask=CRC32_MASK_NORMAL);

dword strHash32(const char* s,         int nocase=YES);
dword memHash32(const void* m, word l, int nocase=YES);
dword memHash32(const void* m, long l, int nocase=YES);

dword strWord32(const char* __str, int __case=CASE_UPPER);
dword memWord32(const char* __str, int __len, int __case=CASE_UPPER);


//  ------------------------------------------------------------------
//  Macro to build a CHR4

#define MK_CHR4(a,b,c,d) ((dword)(a)|((dword)(b)<<8)|((dword)(c)<<16)|((dword)(d)<<24))


//  ------------------------------------------------------------------
//  Get keyword/value pairs and crc

void getkeyval(char** key, char** val);
inline void get_keyval(char*& key, char*& val) { getkeyval(&key, &val); }
void getkeyvaleql(char** key, char** val, int eql);
word getkeyvalcrc(char** key, char** val);
word get_keyvalcrc(char*& key, char*& val);
inline word get_keyvalcrc(char*& key, char*& val) { return getkeyvalcrc(&key, &val); }
inline word getvalcrc(char*& val) { char* key; return getkeyvalcrc(&key, &val); }
inline void getkeyvalcrc(char** key, char** val, word* crc) { *crc = getkeyvalcrc(key, val); }
word GetKeywordValue(char* str, char** keyword, char** value);


//  ------------------------------------------------------------------
//  Generate 32-bit word value from memory

inline dword memWord32(const char* __str, int __len, int __case) {

  dword _val = 0;

  if(__len != 0)
    _val |= ((dword)CaseTo(__case, *(__str+0)));
  if(__len > 1)
    _val |= ((dword)CaseTo(__case, *(__str+1))) << 8;
  if(__len > 2)
    _val |= ((dword)CaseTo(__case, *(__str+2))) << 16;
  if(__len > 3)
    _val |= ((dword)CaseTo(__case, *(__str+3))) << 24;

  return _val;
}


//  ------------------------------------------------------------------
//  Generate 32-bit word value from string

inline dword strWord32(const char* __str, int __case) {

  return memWord32(__str, strlen(__str), __case);
}


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

