
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gcrckeyv.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  CRC-16, CRC-32 and Hashing
//  ------------------------------------------------------------------


#include <ctype.h>
#include <gcrcall.h>


//  ------------------------------------------------------------------
//  Split current value (val) into keyword (key) and new value
//  If "eql" is true, the '=' char is a keyword separator char.

void getkeyvaleql(char** key, char** val, int eql) {
  
  register char* p = *val;

  // Skip leading whitespace
  _strskip_wht(p);

  // Get the keyword
  switch(*p) {

    // Strip "double" quotes
    case '\"':
      *key = ++p;
      do {
        _strskip_to(p, '\"');
        if(*(p-1) != '\\')
          break;
        else
          p++;
      } while(*p);
      break;

    // Strip 'single' quotes
    case '\'':
      *key = ++p;
      do {
        _strskip_to(p, '\'');
        if(*(p-1) != '\\')
          break;
        else
          p++;
      } while(*p);
      break;

    // Strip round brackets
    case '(': /*)*/
      *key = ++p;
      _strskip_to(p, /*(*/ ')');
      break;

    // Get straight keyword
    default:
      *key = p;
      while(*p > ' ') {
        if(eql and (*p == '='))
          break;
        p++;
      }
  }

  // Nul-terminate keyword and remove trailing comma if any
  if(*p) {
    if((*p != '\"') and (*p != '\'') and (*p != /*(*/ ')')) {
      register char* q = p;
      while(*(--q) == ',')
        *q = NUL;
    }
    *p++ = NUL;
  }

  // Skip whitespace and '=' if any
  _strskip_wht(p);
  if(eql and (*p == '=')) {
    p++;
    _strskip_wht(p);
  }

  // Get the value
  *val = p;

  // Handle comments, allowing ;'s in quoted strings
  while(*p and *p != ';') {
    if((*p == '\"') and (*(p-1) != '\\')) {
      p++;
      while(*p and not ((*p == '\"') and (*(p-1) != '\\')))
        p++;
    }
    if(*p)
      p++;
  }
  *p = NUL;

  // Trim whitespace from end of value
  p = *val;
  if(*p) {
    _strskip_to(p, NUL);
    while((*(--p) <= ' ') and *p)
      *p = NUL;
  }
  strschg(*key, "\\\"", "\"");
}


//  ------------------------------------------------------------------
//  Split current value (val) into keyword (key) and new value

void getkeyval(char** key, char** val) {

  getkeyvaleql(key, val, false);
}


//  ------------------------------------------------------------------
//  Get keyword and value and calculate/return CRC16 of keyword

word getkeyvalcrc(char** key, char** val) {
  
  // First get keyword and value
  getkeyvaleql(key, val, false);

  // Init variables
  register char* p = *key;
  register word c = 0;

  // Calculate CRC16 of keyword
  while(*p)
    c = updCrc16((byte)toupper(*p++), c);

  // Return keyword CRC
  return c;
}


//  ------------------------------------------------------------------

