
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gdbgerr.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Debugging and logging.
//  ------------------------------------------------------------------


#include <gdos.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gdbgerr.h>


//  ------------------------------------------------------------------

int errorlevel = 0;
int error_exit = 0;

char* etext[] = {
  "No",
  "Open",
  "Read",
  "Seek",
  "Memory",
  "Index",
  "Write",
  "Tell",
  "Close",
  "Config",
  "Break",
  "NoKeys",
  "Test",
  "Environment",
  "Lock",
  "Pointer",
  ""
};


//  ------------------------------------------------------------------

#if not defined(__linux__)
const int GERRLIST_MAX = 89;
char* __FAR gerrlist[] = {
  /* 000 */  "No error",
  /* 001 */  "Function number invalid",
  /* 002 */  "File not found",
  /* 003 */  "Path not found",
  /* 004 */  "Too many open files (no handles available)",
  /* 005 */  "Access denied",
  /* 006 */  "Invalid handle",
  /* 007 */  "Memory control block destroyed",
  /* 008 */  "Insufficient memory",
  /* 009 */  "Memory block address invalid",
  /* 010 */  "Environment invalid",
  /* 011 */  "Format invalid",
  /* 012 */  "Access code invalid",
  /* 013 */  "Data invalid",
  /* 014 */  "Reserved",
  /* 015 */  "Invalid drive",
  /* 016 */  "Attempted to remove current directory",
  /* 017 */  "Not same device",
  /* 018 */  "No more files",
  /* 019 */  "Disk write-protected",
  /* 020 */  "Unknown unit",
  /* 021 */  "Not ready",
  /* 022 */  "Unknown command",
  /* 023 */  "Data error (CRC)",
  /* 024 */  "Bad request structure length",
  /* 025 */  "Seek error",
  /* 026 */  "Unknown media type (non-DOS disk)",
  /* 027 */  "Sector not found",
  /* 028 */  "Printer out of paper",
  /* 029 */  "Write fault",
  /* 030 */  "Read fault",
  /* 031 */  "General failure",
  /* 032 */  "Sharing violation",
  /* 033 */  "Lock violation",
  /* 034 */  "Disk change invalid",
  /* 035 */  "FCB unavailable",
  /* 036 */  "Sharing buffer overflow",
  /* 037 */  "Code page mismatch",
  /* 038 */  "Cannot complete file operation (out of input)",
  /* 039 */  "Insufficient disk space",
  /* 040 */  "Reserved",
  /* 041 */  "Reserved",
  /* 042 */  "Reserved",
  /* 043 */  "Reserved",
  /* 044 */  "Reserved",
  /* 045 */  "Reserved",
  /* 046 */  "Reserved",
  /* 047 */  "Reserved",
  /* 048 */  "Reserved",
  /* 049 */  "Reserved",
  /* 050 */  "Network request not supported",
  /* 051 */  "Remote computer not listening",
  /* 052 */  "Duplicate name on network",
  /* 053 */  "Network path not found",
  /* 054 */  "Network busy",
  /* 055 */  "Network device no longer exists",
  /* 056 */  "Network BIOS command limit exceeded",
  /* 057 */  "Network adapter hardware error",
  /* 058 */  "Incorrect response from network",
  /* 059 */  "Unexpected network error",
  /* 060 */  "Incompatible remote adapter",
  /* 061 */  "Print queue full",
  /* 062 */  "No spool space",
  /* 063 */  "Print cancelled",
  /* 064 */  "Network name was deleted",
  /* 065 */  "Network access denied",
  /* 066 */  "Network device type incorrect",
  /* 067 */  "Network name not found",
  /* 068 */  "Network name limit exceeded",
  /* 069 */  "Network BIOS session limit exceeded",
  /* 070 */  "Temporarily paused",
  /* 071 */  "Network request not accepted",
  /* 072 */  "Network print/disk redirection paused",
  /* 073 */  "Network software not installed / invalid network version",
  /* 074 */  "Unexpected adapter close / account expired",
  /* 075 */  "Password expired",
  /* 076 */  "Login attempt invalid at this time",
  /* 077 */  "Disk limit exceeded on network node",
  /* 078 */  "Not logged in to network node",
  /* 079 */  "Reserved",
  /* 080 */  "File exists",
  /* 081 */  "Reserved",
  /* 082 */  "Cannot make directory",
  /* 083 */  "Fail on INT 24h",
  /* 084 */  "Too many redirections",
  /* 085 */  "Duplicate redirection",
  /* 086 */  "Invalid password",
  /* 087 */  "Invalid parameter",
  /* 088 */  "Network write fault",
  /* 089 */  "Function not supported on network",
  /* 090 */  "Unknown error"
};
#endif


// -------------------------------------------------------------------

static char __FAR gerrbuf[100];
int __FAR gerrno = 0;

#if defined(__DJGPP__)
extern unsigned short _osmajor, _osminor;
#endif

#if defined(__linux__)
#include <sys/utsname.h>
int _osmajor, _osminor;
#endif

// -------------------------------------------------------------------

void gerrsetosinfo() {

#if defined(__DJGPP__)
  DOSERROR de;

  dosexterr(&de);

  gerrno = de.exterror;
#elif defined(__linux__)
  gerrno = errno;
#else
  gerrno = _doserrno;
#endif

}


// -------------------------------------------------------------------

char* gerrinfo(int __type, char* __file, int __line) {

  switch(__type) {
    case ERR_MEMO:
    case ERR_INDX:
    case ERR_CONF:
    case ERR_BREAK:
    case ERR_NOKEY:
    case ERR_TEST:
    case ERR_ENVR:
    case ERR_MPTR:
      // No report about the OS error in these cases
      break;
    default:
      if(gerrno == 0)
        gerrsetosinfo();
  }
  char* _tmp = strrchr(__file, '\\');
  _tmp = _tmp ? (_tmp+1) : __file;
  sprintf(gerrbuf, "%s error exit at [%s,%u].", etext[__type], _tmp, __line);
  return gerrbuf;
}


// -------------------------------------------------------------------

char* gerrosinfo() {

  #if defined(__MSDOS__)
  const char* osname = "DOS";
  #elif defined(__OS2__)
  const char* osname = "OS/2";
  #elif defined(__WIN32__)
  const char* osname = "Windows";
  #elif defined(__DJGPP__)
  const char* osname = _os_flavor;
  #elif defined(__UNIX__) and not defined(__linux__)
  const char* osname = "Unix";
  #endif

  #if defined(__linux__)
  struct utsname info;

  if(uname(&info)!=-1)
    sprintf(gerrbuf,"%s %s reports error %u: %s.", info.sysname,info.release, errno, strerror(errno));
  else
    strcpy(gerrbuf,"fatal error -- gerrosinfo()");

  #else

  int _tmp = (gerrno > GERRLIST_MAX) ? (GERRLIST_MAX+1) : gerrno;
  sprintf(gerrbuf, "%s %d.%02d reports error %u: %s.", osname, _osmajor, _osminor, gerrno, gerrlist[_tmp]);
  #endif

  return gerrbuf;
}


// -------------------------------------------------------------------

#if 0
void _halt_(E_PROTO, char* estr) {

  error_exit = _error_;

  if(estr == NULL)
    estr = (_error_ == ERR_MEMO) ? "Out of memory" : "";

  char* _ptr = strrchr(strupr(_file_), '\\');
  _ptr = _ptr ? _ptr+1 : _file_;

  printf("* %s error at line %u in src file %s" NL, etext[_error_], _line_, _ptr);
  printf("* Cause: %s" NL, estr);

  if(errorlevel == 0)
    errorlevel = -1;

  exit(errorlevel);
}
#endif


//  ------------------------------------------------------------------

