
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gdbgexpt.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Simple exception handler.
//  ------------------------------------------------------------------
//  Based on code from the book "Practical Data Structures in C++" by
//  Bryan Flamig, 1993.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GDBGEXPT_H
#define __GDBGEXPT_H


//  ------------------------------------------------------------------
//  Required headers

#include <gdefs.h>


//  ------------------------------------------------------------------
//  Exception codes

enum GExceptionCode {
  NONE,
  NOTREADY,
  NOTWRITEABLE,
  READONLYERR,
  NOTOPENERR,
  CREATERR,
  OPENERR,
  CLOSERR,
  SEEKERR,
  RWERR,
  EOFERR,
  FTYPERR,
  CORRUPTERR,
  PATHERR,
  DANGLINGPTR,
  NULLPTR,
  CACHEFULL,
  STKFULL,
  STKEMPTY,
  ASSERTERR
};


class GExceptionHandler {

protected:

  //  ----------------------------------------------------------------
  //  Virtual functions

  virtual void Report(char* __file, int __line, char* __msg, char* __dev);
  virtual void TakeAction();


public:

  //  ----------------------------------------------------------------
  //  Variables

  GExceptionCode exception;


  //  ----------------------------------------------------------------
  //  Constructor

  GExceptionHandler();


  //  ----------------------------------------------------------------
  //  User functions

  void Clear();

  void PrtException();
  void PrtException(GExceptionCode __code);

  virtual void Throw(GExceptionCode __code, char* __file=NULL, int __line=0, char* __msg=NULL, char* __dev=NULL);


  //  ----------------------------------------------------------------
};


//  ------------------------------------------------------------------

inline GExceptionHandler::GExceptionHandler() {

  exception = NONE;
}


//  ------------------------------------------------------------------

inline void GExceptionHandler::Clear() {

  exception = NONE;
}

          
//  ------------------------------------------------------------------
//  Globals

extern GExceptionHandler DefGExceptionHandler;
extern GExceptionHandler *excp;


//  ------------------------------------------------------------------
//  Convenient macro

#define THROW(__code) Throw(__code, __FILE__, __LINE__)


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

