
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gdefs.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Basic definitions and types.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GOLDALL_H
#define __GOLDALL_H


//  ------------------------------------------------------------------
//  Required headers

#include <stddef.h>
#include <gcmpall.h>


//  ------------------------------------------------------------------
//  Define portability and shorthand notation

#ifndef and
#define not      !
#define and      &&
#define or       ||
#endif

#ifndef true
#define true  1
#define false 0
#endif

#define NO     0
#define YES    1
#define ALWAYS 2
#define ASK    2
#define AUTO   3
#define MAYBE  4

#define NUL '\x00'    // Common ASCII control codes
#define BEL '\x07'
#define BS  '\x08'
#define HT  '\x09'
#define LF  '\x0A'
#define FF  '\x0C'
#define CR  '\x0D'
#define ESC '\x1B'

#ifdef __UNIX__
#define NL "\r\n"
#else
#define NL "\n"
#endif

#define NULL_CHARP ((char*)NULL)
#define NULL_CONST_CHARP ((const char*)NULL)


//  ------------------------------------------------------------------
//  Special character constants

#define CTRL_A '\x01'   // FidoNet kludge line char
#define SOFTCR '\x8D'   // "Soft" carriage-return


//  ------------------------------------------------------------------
//  Supplements for the built-in types

#ifndef __GOLDWARE_HAS_BOOL
typedef          int     bool;
#endif
typedef   signed char   schar;
typedef unsigned char   uchar;
typedef unsigned short ushort;
typedef unsigned int     uint;
typedef unsigned long   ulong;
typedef unsigned char    byte;
typedef unsigned short   word;
typedef unsigned long   dword;



//  ------------------------------------------------------------------
//  Common function-pointer types

typedef void (*VfvCP)();
typedef int (*IfvCP)();
typedef int (*IfcpCP)(char*);


//  ------------------------------------------------------------------
//  Function pointer for stdlib qsort(), bsearch() compare functions

typedef int (* _stdc StdCmpCP)(const void*, const void*);


//  ------------------------------------------------------------------
//  Utility templates

template <class T> inline bool in_range(T a, T b, T c)   { return (a >= b) and (a <= c); }
template <class T> inline    T absolute(T a)             { return a < 0 ? -a : a; }
template <class T> inline  int compare_two(T a, T b)     { return a < b ? -1 : a > b ? 1 : 0; }
template <class T> inline    T minimum_of_two(T a, T b)  { return a < b ? a : b; }
template <class T> inline    T maximum_of_two(T a, T b)  { return a > b ? a : b; }
template <class T> inline  int zero_or_one(T e)          { return e ? 1 : 0; }
template <class T> inline bool make_bool(T a)            { return a ? true : false; }


//  ------------------------------------------------------------------
//  Macros for integer division with correct .5 rounding
//
//  Sorry about the obfuscated code ;-)
//
//  Examples:   int   a = 100/6;              // a == 16
//              int   b = DivInt(100,6);      // b == 17
//              float c = 100.0/6.0;          // c == 16.66667

#if 0
#define DivLng(a,b)  ((((long)(a)*10L)/((long)(b))+5L)/10L)
#define DivInt(a,b)  ((int)(DivLng((a),(b))))
#define DivShr(a,b)  ((short)(DivLng((a),(b))))
#endif


//  ------------------------------------------------------------------
//  Handy macro for safe casting.           Public domain by Bob Stout
//  ------------------------------------------------------------------
//
//  Example of CAST macro at work
//
//  union {
//    char  ch[4];
//    int   i[2];
//  } my_union;
//
//  long longvar;
//
//  longvar = (long)my_union;         // Illegal cast
//  longvar = CAST(long, my_union);   // Legal cast
//
//  ------------------------------------------------------------------

#define CAST(new_type,old_object) (*((new_type *)&(old_object)))


//  ------------------------------------------------------------------
//  Get size of structure member

#define sizeofmember(__struct, __member)  sizeof(((__struct*)0)->__member)


//  ------------------------------------------------------------------
//  A "short" which is short only with some 32-bit compilers

#if defined(__WATCOMC__)
#define __short32 short
#else
#define __short32 int
#endif


//  ------------------------------------------------------------------
//  Legacy defines

#ifndef AND
#define NOT      !
#define AND      &&
#define OR       ||
#endif

#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif

#define RngV in_range
#define AbsV absolute
#define CmpV compare_two
#define MinV minimum_of_two
#define MaxV maximum_of_two


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

