
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gdirff.cpp,v 1.5 1999/01/16 03:07:44 damware Exp $
//  ------------------------------------------------------------------
//  Portable directory access class.
//  Based on Public Domain source from Bob Stouts C SNIPPETS archive.
//  ------------------------------------------------------------------


#include <gfilutil.h>
#include <gmemdbg.h>
#include <gdirff.h>


//  ------------------------------------------------------------------

#if defined(__OS2__)

#define INCL_DOS
#include <os2.h>

struct gfiledata {
  const char*  mask;
  ULONG        mask_include_attributes;
  HDIR         handle;
  FILEFINDBUF3 file;
};

#elif defined(__WIN32__)

#include <dos.h>
#include <windows.h>

struct gfiledata {
  const char*     mask;
  long            mask_include_attributes;
  HANDLE          handle;
  WIN32_FIND_DATA file;
  SYSTEMTIME      filetime;
};

#elif defined(__MSDOS__)

#include <gio.h>
#include <dos.h>

#if defined(__WATCOMC__)
struct  ftime   {
    unsigned    ft_tsec  : 5;   /* Two second interval */
    unsigned    ft_min   : 6;   /* Minutes */
    unsigned    ft_hour  : 5;   /* Hours */
    unsigned    ft_day   : 5;   /* Days */
    unsigned    ft_month : 4;   /* Months */
    unsigned    ft_year  : 7;   /* Year */
};
#endif

struct gfiledata {
  const char* mask;
  unsigned    mask_include_attributes;
  find_t      file;
  ftime&      filetime() { return *((ftime*)&file.wr_time); }
};

#elif defined(__UNIX__)

#include <gtimall.h>
#include <gwildmat.h>

#define FILE_RDONLY     0x00000001
#define FILE_HIDDEN     0x00000002
#define FILE_DIR        0x00000010
#define FILE_NORMAL     0x00000080

struct gfiledata {
  const char* mask;
  unsigned    mask_include_attributes;
  struct tm*  ftime;
};       

#else

struct gfiledata {
  const char* mask;
  unsigned    mask_include_attributes;
  //find_t      file;
  //ftime&      filetime() { return *((ftime*)&file.wr_time); }
};

#endif

#define FILEDATA (*(gfiledata*)filedata)


//  ------------------------------------------------------------------

const char* gdir::mask() { return FILEDATA.mask; }

#if defined(__OS2__)

long gdir::size() { return FILEDATA.file.cbFile; }
const char* gdir::name() { return FILEDATA.file.achName; }

int gdir::normal()    { return !!(FILEDATA.file.attrFile & FILE_NORMAL); }
int gdir::readonly()  { return !!(FILEDATA.file.attrFile & FILE_READONLY); }
int gdir::hidden()    { return !!(FILEDATA.file.attrFile & FILE_HIDDEN); }
int gdir::system()    { return !!(FILEDATA.file.attrFile & FILE_SYSTEM); }
int gdir::directory() { return !!(FILEDATA.file.attrFile & FILE_DIRECTORY); }
int gdir::archived()  { return !!(FILEDATA.file.attrFile & FILE_ARCHIVED); }

void gdir::find_normal()    { FILEDATA.mask_include_attributes |= FILE_NORMAL; }
void gdir::find_readonly()  { FILEDATA.mask_include_attributes |= FILE_READONLY; }
void gdir::find_hidden()    { FILEDATA.mask_include_attributes |= FILE_HIDDEN; }
void gdir::find_system()    { FILEDATA.mask_include_attributes |= FILE_SYSTEM; }
void gdir::find_directory() { FILEDATA.mask_include_attributes |= FILE_DIRECTORY; }
void gdir::find_archived()  { FILEDATA.mask_include_attributes |= FILE_ARCHIVED; }

int gdir::year()   { return FILEDATA.file.fdateLastWrite.year + 1980; }
int gdir::month()  { return FILEDATA.file.fdateLastWrite.month; }
int gdir::day()    { return FILEDATA.file.fdateLastWrite.day; }
int gdir::hour()   { return FILEDATA.file.ftimeLastWrite.hours; }
int gdir::minute() { return FILEDATA.file.ftimeLastWrite.minutes; }
int gdir::second() { return FILEDATA.file.ftimeLastWrite.twosecs/2; }


//  ------------------------------------------------------------------

#elif defined(__WIN32__)

#define FILE_ATTRIBUTE_HID_SYS_DIR (FILE_ATTRIBUTE_HIDDEN|FILE_ATTRIBUTE_SYSTEM|FILE_ATTRIBUTE_DIRECTORY)

long gdir::size() { return FILEDATA.file.nFileSizeLow; }
const char* gdir::name() { return FILEDATA.file.cFileName; }

int gdir::normal()    { return !!(FILEDATA.file.dwFileAttributes & FILE_ATTRIBUTE_NORMAL); }
int gdir::readonly()  { return !!(FILEDATA.file.dwFileAttributes & FILE_ATTRIBUTE_READONLY); }
int gdir::hidden()    { return !!(FILEDATA.file.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN); }
int gdir::system()    { return !!(FILEDATA.file.dwFileAttributes & FILE_ATTRIBUTE_SYSTEM); }
int gdir::directory() { return !!(FILEDATA.file.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY); }
int gdir::archived()  { return !!(FILEDATA.file.dwFileAttributes & FILE_ATTRIBUTE_ARCHIVE); }

void gdir::find_normal()    { FILEDATA.mask_include_attributes |= FILE_ATTRIBUTE_NORMAL|FILE_ATTRIBUTE_READONLY|FILE_ATTRIBUTE_ARCHIVE; }
void gdir::find_readonly()  { FILEDATA.mask_include_attributes |= FILE_ATTRIBUTE_READONLY; }
void gdir::find_hidden()    { FILEDATA.mask_include_attributes |= FILE_ATTRIBUTE_HIDDEN; }
void gdir::find_system()    { FILEDATA.mask_include_attributes |= FILE_ATTRIBUTE_SYSTEM; }
void gdir::find_directory() { FILEDATA.mask_include_attributes |= FILE_ATTRIBUTE_DIRECTORY; }
void gdir::find_archived()  { FILEDATA.mask_include_attributes |= FILE_ATTRIBUTE_ARCHIVE; }

int gdir::year()   { return FILEDATA.filetime.wYear; }
int gdir::month()  { return FILEDATA.filetime.wMonth; }
int gdir::day()    { return FILEDATA.filetime.wDay; }
int gdir::hour()   { return FILEDATA.filetime.wHour; }
int gdir::minute() { return FILEDATA.filetime.wMinute; }
int gdir::second() { return FILEDATA.filetime.wSecond; }


//  ------------------------------------------------------------------

#elif defined(__MSDOS__)

long gdir::size() { return FILEDATA.file.size; }
const char* gdir::name() { return FILEDATA.file.name; }

int gdir::normal()    { return !!(FILEDATA.file.attrib & _A_NORMAL); }
int gdir::readonly()  { return !!(FILEDATA.file.attrib & _A_RDONLY); }
int gdir::hidden()    { return !!(FILEDATA.file.attrib & _A_HIDDEN); }
int gdir::system()    { return !!(FILEDATA.file.attrib & _A_SYSTEM); }
int gdir::directory() { return !!(FILEDATA.file.attrib & _A_SUBDIR); }
int gdir::archived()  { return !!(FILEDATA.file.attrib & _A_ARCH); }

void gdir::find_normal()    { FILEDATA.mask_include_attributes |= _A_NORMAL; }
void gdir::find_readonly()  { FILEDATA.mask_include_attributes |= _A_RDONLY; }
void gdir::find_hidden()    { FILEDATA.mask_include_attributes |= _A_HIDDEN; }
void gdir::find_system()    { FILEDATA.mask_include_attributes |= _A_SYSTEM; }
void gdir::find_directory() { FILEDATA.mask_include_attributes |= _A_SUBDIR; }
void gdir::find_archived()  { FILEDATA.mask_include_attributes |= _A_ARCH; }

int gdir::year()   { return FILEDATA.filetime().ft_year + 1980; }
int gdir::month()  { return FILEDATA.filetime().ft_month; }
int gdir::day()    { return FILEDATA.filetime().ft_day; }
int gdir::hour()   { return FILEDATA.filetime().ft_hour; }
int gdir::minute() { return FILEDATA.filetime().ft_min; }
int gdir::second() { return FILEDATA.filetime().ft_tsec * 2; }


//  ------------------------------------------------------------------
 
#elif defined(__UNIX__)
 
long gdir::size() { return ud.size(); }
const char* gdir::name() { return ud.name(); }
 
int gdir::normal()    { return ud.is_file(); }
int gdir::readonly()  { return ud.is_readonly(); }
int gdir::hidden()    { return ud.is_hidden(); }
int gdir::system()    { return false; }
int gdir::directory() { return ud.is_directory(); }
int gdir::archived()  { return true; }
 
void gdir::find_normal()    { FILEDATA.mask_include_attributes |= FILE_NORMAL; }
void gdir::find_readonly()  { FILEDATA.mask_include_attributes |= FILE_RDONLY; }
void gdir::find_hidden()    { FILEDATA.mask_include_attributes |= FILE_HIDDEN; }
void gdir::find_system()    { }
void gdir::find_directory() { FILEDATA.mask_include_attributes |= FILE_DIR; }
void gdir::find_archived()  { }
 
int gdir::year()   { return FILEDATA.ftime->tm_year + 1900; }
int gdir::month()  { return FILEDATA.ftime->tm_mon + 1; }
int gdir::day()    { return FILEDATA.ftime->tm_mday; }
int gdir::hour()   { return FILEDATA.ftime->tm_hour; }
int gdir::minute() { return FILEDATA.ftime->tm_min; }
int gdir::second() { return FILEDATA.ftime->tm_sec * 2; }

                
//  ------------------------------------------------------------------

#else

long gdir::size() { return 0; }
const char* gdir::name() { return ""; }

int gdir::normal()    { return false; }
int gdir::readonly()  { return false; }
int gdir::hidden()    { return false; }
int gdir::system()    { return false; }
int gdir::directory() { return false; }
int gdir::archived()  { return false; }

void gdir::find_normal()    { }
void gdir::find_readonly()  { }
void gdir::find_hidden()    { }
void gdir::find_system()    { }
void gdir::find_directory() { }
void gdir::find_archived()  { }

int gdir::year()   { return 0; }
int gdir::month()  { return 0; }
int gdir::day()    { return 0; }
int gdir::hour()   { return 0; }
int gdir::minute() { return 0; }
int gdir::second() { return 0; }


#endif

//  ------------------------------------------------------------------

gdir::gdir() {

  filedata = NULL;
}


//  ------------------------------------------------------------------

gdir::~gdir() {

  close();
}


//  ------------------------------------------------------------------

int gdir::open(const char* mask) {

  if(!filedata)
    filedata = (void*)throw_calloc(1, sizeof(gfiledata));

  FILEDATA.mask = mask;
  find_normal();

  #if defined(__UNIX__)

  // split up input filespec into its current
  // working directory and filename/extension

  Path dir;
  char* r = strrchr(mask, GOLD_SLASH_CHR);
  if(r == NULL) {
    *dir = NUL;
    FILEDATA.mask=mask;
  }
  else {
    strxcpy(dir, mask, sizeof(dir));
    dir[++r - mask] = NUL;
    FILEDATA.mask=r;
  }
  if(not ud.open(dir))
    return false;

  #endif           

  return true;
}


//  ------------------------------------------------------------------

int gdir::close() {

  if(filedata) {
    #if defined(__OS2__)

    DosFindClose(FILEDATA.handle);

    #elif defined(__WIN32__)

    FindClose(FILEDATA.handle);

    #elif defined(__UNIX__)
    
    ud.close();

    #endif
    throw_release(filedata);
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int gdir::first() {

  #if defined(__OS2__)

  ULONG cnt = 1;
  FILEDATA.handle = (HDIR)-1;
  return DosFindFirst((PSZ)FILEDATA.mask, &FILEDATA.handle, FILEDATA.mask_include_attributes, &FILEDATA.file, sizeof(FILEFINDBUF3), &cnt, FIL_STANDARD) == 0;

  #elif defined(__WIN32__)

  FILETIME local;
  FILEDATA.handle = FindFirstFile(FILEDATA.mask, &FILEDATA.file);
  if(FILEDATA.handle == INVALID_HANDLE_VALUE)
    return false;
  while(not (((FILEDATA.file.dwFileAttributes & FILE_ATTRIBUTE_HID_SYS_DIR) == 0) or ((FILEDATA.mask_include_attributes & FILEDATA.file.dwFileAttributes & FILE_ATTRIBUTE_HID_SYS_DIR) != 0)))
    if(FindNextFile(FILEDATA.handle, &FILEDATA.file) == 0)
      return false;
  FileTimeToLocalFileTime(&FILEDATA.file.ftLastWriteTime, &local);
  FileTimeToSystemTime(&local, &FILEDATA.filetime);
  return true;

  #elif defined(__MSDOS__)

  return _dos_findfirst((char*)FILEDATA.mask, FILEDATA.mask_include_attributes, &FILEDATA.file) == 0;

  #elif defined(__UNIX__)

  while(ud.read()) {
    if(u_checkentry()) {
      time_t mtime = ud.modified_time();
      FILEDATA.ftime = localtime(&mtime);
      return true;
    }
  }
  return false;
                      
  #else

  return false;

  #endif
}


//  ------------------------------------------------------------------

int gdir::next() {

  #if defined(__OS2__)

  ULONG cnt = 1;
  return DosFindNext(FILEDATA.handle, &FILEDATA.file, sizeof(FILEFINDBUF3), &cnt) == 0;

  #elif defined(__WIN32__)

  FILETIME local;
  do {
    if(FindNextFile(FILEDATA.handle, &FILEDATA.file) == 0)
      return false;
  } while(not (((FILEDATA.file.dwFileAttributes & FILE_ATTRIBUTE_HID_SYS_DIR) == 0) or ((FILEDATA.mask_include_attributes & FILEDATA.file.dwFileAttributes & FILE_ATTRIBUTE_HID_SYS_DIR) != 0)));
  FileTimeToLocalFileTime(&FILEDATA.file.ftLastWriteTime, &local);
  FileTimeToSystemTime(&local, &FILEDATA.filetime);
  return true;

  #elif defined(__MSDOS__)

  return _dos_findnext(&FILEDATA.file) == 0;

  #elif defined(__UNIX__)

  while(ud.read()) {
    if(u_checkentry()) {
      time_t mtime = ud.modified_time();
      FILEDATA.ftime = localtime(&mtime);
      return true;
    }
  }
  return false;
              
  #else

  return false;

  #endif
}

 
#if defined(__UNIX__)

bool gdir::u_checkentry() {

  if(not gwildmat(ud.name(),FILEDATA.mask))
    return false;

  if((FILEDATA.mask_include_attributes & FILE_HIDDEN) == 0) 
    if(ud.is_hidden()) return false;
  if((FILEDATA.mask_include_attributes & FILE_DIR) == 0)
    if(ud.is_directory()) return false;
  if(FILEDATA.mask_include_attributes & FILE_RDONLY)
    if(not ud.is_readonly()) return false;
  if(FILEDATA.mask_include_attributes & FILE_NORMAL)
    if(not ud.is_file()) return false;

  return true;
}
 
#endif                 

//  ------------------------------------------------------------------

