
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gdirposx.cpp,v 1.6 1999/02/06 13:29:05 odinn Exp $
//  ------------------------------------------------------------------
//  Posix-based directory class.
//  ------------------------------------------------------------------


#include <string.h>
#include <gfilutil.h>
#include <gdirposx.h>
#include <gmemdbg.h>

//  ------------------------------------------------------------------

#if defined(__DJGPP__)
int __opendir_flags = __OPENDIR_PRESERVE_CASE;
#endif


//  ------------------------------------------------------------------

gposixdir::gposixdir() {
  reset();
}


//  ------------------------------------------------------------------

gposixdir::gposixdir(const char* dirpath) {

  reset();
  open(dirpath);
}

 
//  ------------------------------------------------------------------

void gposixdir::reset() {
  handle = NULL;

  basepath = new Path; throw_new(basepath);
  longname = new Path; throw_new(longname);
}            


//  ------------------------------------------------------------------

gposixdir::~gposixdir() {

  if(handle)
    close();

  throw_delete(basepath);
  throw_delete(longname);
}


//  ------------------------------------------------------------------

bool gposixdir::open(const char* dirpath) {

  strcpy(basepath, dirpath);

  AddBackslash(basepath);
  
  handle = opendir(basepath);
  if(handle)
    return true;

  return false;
}


//  ------------------------------------------------------------------

bool gposixdir::read() {

  info.st_size = -1;
  if(handle) {
    entry = readdir(handle);
    if(entry)
      return true;
  }

  return false;
}


//  ------------------------------------------------------------------

void gposixdir::rewind() {

  #if defined(__WATCOMC__) and (__WATCOMC__ < 1100)
  closedir(handle);
  handle = opendir(basepath);
  #else
  rewinddir(handle);
  #endif
}


//  ------------------------------------------------------------------

void gposixdir::close() {

  closedir(handle);
  handle = NULL;
}


//  ------------------------------------------------------------------

void gposixdir::statthis() {

  strcpy(longname, basepath);
  strcat(longname, name());
  stat(longname, &info);
}


//  ------------------------------------------------------------------

inline void gposixdir::getinfo() {

  if(info.st_size == -1)
    statthis();
}


//  ------------------------------------------------------------------

const char* gposixdir::path() {

  return basepath;
}


//  ------------------------------------------------------------------

const char* gposixdir::name() {

  return entry->d_name;
}


//  ------------------------------------------------------------------

const char* gposixdir::fullname() {

  getinfo();
  return longname;
}


//  ------------------------------------------------------------------

long gposixdir::size() {

  getinfo();
  return info.st_size;
}


//  ------------------------------------------------------------------

time_t gposixdir::accessed_time() {

  getinfo();
  return info.st_atime;
}


//  ------------------------------------------------------------------

time_t gposixdir::modified_time() {

  getinfo();
  return info.st_mtime;
}


//  ------------------------------------------------------------------

time_t gposixdir::creation_time() {

  getinfo();
  return info.st_ctime;
}


//  ------------------------------------------------------------------

bool gposixdir::is_file() {

  getinfo();
  return S_ISREG(info.st_mode) ? true : false;
}


//  ------------------------------------------------------------------

bool gposixdir::is_directory() {

  getinfo();
  return S_ISDIR(info.st_mode) ? true : false;
}


//  ------------------------------------------------------------------

bool gposixdir::is_readonly() {

  getinfo();
  return (info.st_mode & S_IWUSR) ? false : true;
}


//  ------------------------------------------------------------------

bool gposixdir::is_hidden() {

  if(*entry->d_name == '.') {
    if(strcmp(entry->d_name, ".") and strcmp(entry->d_name, ".."))
      return true;
  }
  return false;
}


//  ------------------------------------------------------------------

