
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geval.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Expression evaluator base class.
//  ------------------------------------------------------------------


#include <geval.h>


//  ------------------------------------------------------------------

geval::geval() {

  // nothing to do
}


//  ------------------------------------------------------------------

geval::~geval() {

  // nothing
}


//  ------------------------------------------------------------------

geval::ops geval::pop_operator() {

  ops o = ostk.back();
  ostk.pop_back();
  return o;
}


//  ------------------------------------------------------------------

int geval::pop_value() {

  int v = vstk.back();
  vstk.pop_back();
  return v;
}


//  ------------------------------------------------------------------

void geval::push_value(int v) {

  vstk.push_back(v);
}


//  ------------------------------------------------------------------

void geval::push_operator(geval::ops o) {

  ostk.push_back(o);
}


//  ------------------------------------------------------------------

int geval::evaluate_op(geval::ops o, int y, int x) {

  switch(o) {
    case addition:          return y + x;
    case subtraction:       return y - x;
    case multiplication:    return y * x;
    case division:          return y / x;
    case modulus:           return y % x;
    case negation:          return -x;
    case logic_not:         return (not x) ? 1 : 0;
    case logic_and:         return (y and x) ? 1 : 0;
    case logic_xor:         return (y ^ x) ? 1 : 0;
    case equal:             return (y == x) ? 1 : 0;
    case not_equal:         return (y != x) ? 1 : 0;
    case greater:           return (y > x) ? 1 : 0;
    case greater_or_equal:  return (y >= x) ? 1 : 0;
    case lesser:            return (y < x) ? 1 : 0;
    case lesser_or_equal:   return (y <= x) ? 1 : 0;
    default:                ;
  }

  return 0;
}


//  ------------------------------------------------------------------

