
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: geval.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Expression evaluator classes.
//  ------------------------------------------------------------------


#ifndef __GEVAL_H
#define __GEVAL_H

#include <gvector.h>


//  ------------------------------------------------------------------
//  Base class

class geval {

public:

  enum ops {
    lowest,
    logic_or,
    logic_and,
    logic_xor,
    not_equal,
    equal,
    greater_or_equal,
    greater,
    lesser_or_equal,
    lesser,
    subtraction,
    addition,
    modulus,
    division,
    multiplication,
    negation,
    logic_not,
    parenthesis_right,
    parenthesis_left,
    highest
  };

protected:

  gvector<ops> ostk;
  gvector<int> vstk;

  ops pop_operator();
  int pop_value();

public:

  geval();
  ~geval();

  void push_value(int v);
  void push_operator(ops o);

  int evaluate_op(ops o, int y, int x);
  int evaluate_ops(ops* o, int* y, int* x);

};


//  ------------------------------------------------------------------

inline int geval::evaluate_ops(ops* o, int* y, int* x) {

  return evaluate_op(*o, *y, *x);
}


//  ------------------------------------------------------------------
//  Reverse Polish Notation expression evaluator.

class gevalrpn : public geval {

public:

  gevalrpn();
  ~gevalrpn();

  int evaluate();

};


//  ------------------------------------------------------------------
//  Human (left-to-right) expression evaluator.

class gevalhum : public geval {

public:

  gevalhum();
  ~gevalhum();

  int evaluate();

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

