
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gfilport.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Ports of OS-specific file/path functions.
//  ------------------------------------------------------------------


#include <time.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <gio.h>
#include <gdos.h>
#include <gdirect.h>

#include <gtimall.h>
#include <gstrall.h>
#include <gfilutil.h>

#if defined(__OS2__)
#define INCL_BASE
#define INCL_ERROR_H
#include <os2.h>
#endif

#if defined(__WIN32__)
#include <windows.h>
#endif


//  ------------------------------------------------------------------

#if not defined(__GNUC__)

int mkdir(const char* path, int) {

  return mkdir(path);
}

#endif


//  ------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//  ------------------------------------------------------------------

#if defined(__WATCOMC__) or defined(__GNUC__)

int getcurdir(int drive, char* path) {

  #if defined(__OS2__)

  ULONG length = GMAXDIR;
  return DosQueryCurrentDir(drive, (PBYTE)path, &length) ? -1 : 0;

  #elif defined(__WIN32__)

  int size;
  char bufI[GMAXDIR + 3];
  char envname[4];
  unsigned drivemask;

  // If the default drive is specified, simply use GetCurrentDirectory.
  if(drive == 0) {
    size = (int)GetCurrentDirectory(sizeof(bufI), bufI);
    if(size == 0 || size > GMAXDIR) {
      errno = ENOMEM;
      return -1;
    }
  }
  else {
    drivemask = GetLogicalDrives();
    if(drivemask & (1 << (drive-1))) {

      // Construct environment variable name for the specified drive.
      envname[0] = '=';
      envname[1] = (char)(drive + 'A' - 1);
      envname[2] = ':';
      envname[3] = '\0';

      // The environment variable =D:, where D is the drive, contains
      // the current directory for that drive.
      if (GetEnvironmentVariable(envname, bufI, sizeof(bufI)) == 0) {
        // The environment variable was not found, assume the current directory is the root.
        bufI[0] = envname[1];
        bufI[1] = ':';
        bufI[2] = '\\';
        bufI[3] = '\0';
      }
    }
    else
      return -1;
  }

  // If the buffer length is too small to contain the directory name, return an error.
  if(strlen(bufI) >= GMAXDIR) {
    errno = ERANGE;
    return -1;
  }

  strcpy(path, bufI+3);
  return 0;

  #elif defined(__MSDOS__)

  #if defined(__DJGPP__)

  union REGS r;
  memset(&r, 0, sizeof(r));
  r.h.ah = 0x47;
  r.h.dl = (unsigned char)drive;
  r.x.si = (unsigned long)path;
  intdos(&r, &r);

  #else

  union REGPACK r;
  memset(&r, 0, sizeof(r));
  r.h.ah = 0x47;
  r.h.dl = (unsigned char)drive;
  r.w.ds = FP_SEG(path);
  #if defined(__386__)
  r.x.esi = FP_OFF(path);
  #else
  r.w.si = FP_OFF(path);
  #endif
  intr(0x21, &r);

  #endif

  return (r.w.flags & 1) ? -1 : 0;

  #else

  NW(drive);
  getcwd(path, GMAXPATH);
  return 0;

  #endif
}

#endif


//  ------------------------------------------------------------------

#if defined(__WATCOMC__)

char* mktemp(char* temp) {

  int len;
  register char *cp;
  char i, j, k, l, m;

  len = strlen(temp);
  if(len < 6)
    return 0;
  cp = temp + len - 6;
  if(strcmp(cp, "XXXXXX") != 0)
    return 0;
  cp[2] = '.';
  for(i = 'A'; i <= 'Z'; i++) {
    cp[0] = i;
    for(j = 'A'; j <= 'Z'; j++) {
      cp[1] = j;
      for(k = 'A'; k <= 'Z'; k++) {
        cp[3] = k;
        for(l = 'A'; l <= 'Z'; l++) {
          cp[4] = l;
          for(m = 'A'; m <= 'Z'; m++) {
            cp[5] = m;
            if(access(temp, 0) == -1)
              return temp;
          }
        }
      }
    }
  }
  return 0;
}

#endif


//  ------------------------------------------------------------------

#if (defined(__BORLANDC__) and defined(__OS2__)) or defined(__GNUC__)

long filelength(int handle) {

  long pos = lseek(handle, 0L, SEEK_CUR);
  if(pos == -1)
    return pos;
  long size = lseek(handle, 0L, SEEK_END);
  lseek(handle, pos, SEEK_SET);
  return size;
}

#endif


//  ------------------------------------------------------------------

#if (defined(__WATCOMC__) or defined(__EMX__)) and defined(__OS2__)

#if defined(__WATCOMC__)
int lock(int handle, unsigned long ofs, unsigned long length)
#else
int lock(int handle, long ofs, long length)
#endif

{

  int rc;
  APIRET apiret;
  FILELOCK urange, lrange;

  lrange.lOffset = ofs;
  lrange.lRange = length;
  urange.lRange = urange.lOffset = 0;

  if((apiret = DosSetFileLocks((HFILE)handle, &urange, &lrange, 0, 0)) != 0) {
    if(apiret == ERROR_LOCK_VIOLATION) {
      errno = EACCES;
      rc = -1;
    }
    else
      rc = (int)apiret;
  }
  else
    rc = 0;

  return rc;
}

#endif


//  ------------------------------------------------------------------

#if (defined(__WATCOMC__) or defined(__EMX__)) and defined(__OS2__)

#if defined(__WATCOMC__)
int unlock(int handle, unsigned long ofs, unsigned long length)
#else
int unlock(int handle, long ofs, long length)
#endif

{

  int rc;
  FILELOCK urange, lrange;
  APIRET apiret;

  urange.lOffset = ofs;
  urange.lRange = length;
  lrange.lRange = lrange.lOffset = 0;

  if((apiret = DosSetFileLocks((HFILE)handle, &urange, &lrange, 0, 0)) != 0) {
    if(apiret == ERROR_LOCK_VIOLATION) {
      errno = EACCES;
      rc = -1;
    }
    else
      rc = apiret;
  }
  else
    rc = 0;

  return rc;
}

#endif


//  ------------------------------------------------------------------

#if defined(__UNIX__)

int lock(int handle, long offset, long length) {

  struct flock fl;
  fl.l_type = F_WRLCK;
  fl.l_whence = SEEK_SET;
  fl.l_start = offset;
  fl.l_len = length;

  return fcntl(handle, F_SETLK, &fl);
}

#endif


//  ------------------------------------------------------------------

#if defined(__UNIX__)

int unlock(int handle, long offset, long length) {

  struct flock fl;
  fl.l_type = F_UNLCK;
  fl.l_whence = SEEK_SET;
  fl.l_start = offset;
  fl.l_len = length;

  return fcntl(handle, F_SETLK, &fl);
}

#endif


//  ------------------------------------------------------------------

#if defined(__GNUC__)

int sopen(const char* path, int access, int shflag, int mode) {

  return open(path, access|shflag, mode);
}

#endif


//  -----------------------------------------------------------------

#if defined(__GNUC__)

void _makepath(char* path, const char* drive, const char* dir, const char* name, const char* ext) {

  if(drive and *drive) {
    *path++ = *drive;
    *path++ = ':';
  }
  if(dir and *dir) {
    path = stpcpy(path,dir);
    if(*(path-1) != '\\' and *(path-1) != '/')
      *path++ = GOLD_SLASH_CHR;
  }
  if(name)
    path = stpcpy(path,name);
  if(ext and *ext) {
    if(*ext != '.')
      *path++ = '.';
    path=stpcpy(path,ext);
  }
  *path = NUL;
}

#endif


//  -----------------------------------------------------------------

#if defined(__GNUC__)

void _splitpath(const char* path, char* drive, char* dir, char* name, char* ext) {

  int c, n = 0;

  if((path[0]) and ((path[1] == ':') or (path[2] == ':'))) {
    do {
      c=path[n];
      drive[n] = (char)c;
      ++n;
    } while (c != ':');
  }

  drive[n] = NUL;

  int dir_start = n;
  int name_start = n;
  int ext_start = 0;

  while((c=path[n]) != NUL) {
    switch(c) {
      case '.' :
        if(!((path[n+1] == '.') or (path[n+1] == '\\') or (path[n+1] == '/')))
          ext_start=n;
        ++n;
        break;
      case '/' :
      case '\\':
        name_start=++n;
        ext_start=0;
        break;
      default:
        ++n;
        break;
    }
  } 

  int path_end=n;
  if(ext_start == 0)
    ext_start=n;

  n=dir_start;
  int p = 0;
  if(n < name_start) {
    while((n < name_start) and (p < GMAXDIR-1)) {
      dir[p++]=path[n++];
    }
  }
  dir[p]=NUL;

  n=name_start;
  p=0;
  if(n < ext_start) {
    while((n < ext_start) and (p < GMAXFILE-1)) {
      name[p++]=path[n++];
    }
  }
  name[p]=NUL;

  p=0;
  if((ext_start >= name_start) and (path[ext_start])) {
    n=ext_start;
    while((n < path_end) and (p < GMAXEXT-1)) {
      ext[p++]=path[n++];
    }
  }
  ext[p]=NUL;
}

#endif


//  ------------------------------------------------------------------

#if defined(__GNUC__) and not defined(__EMX__)

int chsize(int handle, long size) {

  return ftruncate(handle, size);
}

#endif


//  ------------------------------------------------------------------

#if defined(__BORLANDC__) and defined(__OS2__)

// The opendir function is broken in Borland C++ 1.5.
// This is a fixed version of the RTL source.

int __IOerror(int __doserror);

#define DIRMAGIC 0xddaa

#define SEARCH_ATTR (FILE_HIDDEN|FILE_SYSTEM|FILE_DIRECTORY)

DIR * _RTLENTRY _EXPFUNC opendir(const char *dirname)
{
    char *name;
    int len;
    DIR *dir;
    ULONG nfiles;
    APIRET apiret;

    /* Allocate space for a copy of the directory name, plus
     * room for the "*.*" we will concatenate to the end.
     */
    len = strlen(dirname);
    if ((name = (char*)malloc(len+5)) == NULL)
    {
        errno = ENOMEM;
        return (NULL);
    }
    strcpy(name,dirname);
    if (len-- && name[len] != ':' && name[len] != '\\' && name[len] != '/')
        strcat(name,"\\*");
    else
        strcat(name,"*");

    /* Allocate space for a DIR structure.
     */
    if ((dir = (DIR*)malloc(sizeof(DIR))) == NULL)
    {
        errno = ENOMEM;
        free(name);
        return (NULL);
    }

    /* Search for the first few files to see if the directory exists,
     * and to obtain directory handle for future DosFindNext() calls.
     */
    dir->_d_hdir = HDIR_CREATE;
    nfiles = 2;
    if ((apiret = DosFindFirst((PSZ)name, &dir->_d_hdir, SEARCH_ATTR,
            (PVOID)&dir->_d_buf[0], sizeof(dir->_d_buf),
            &nfiles,FIL_STANDARD)) != 0)
    {
        free(name);
        free(dir);
        __IOerror((int)apiret);     /* set errno */
        return (NULL);
    }

    /* Everything is OK.  Save information in the DIR structure, return it.
     */
    dir->_d_dirname = name;
    dir->_d_magic = DIRMAGIC;
    dir->_d_nfiles = (unsigned)nfiles;
    dir->_d_bufp = &dir->_d_buf[0];
    return dir;
}


void rewinddir(DIR *dir)
{
    ULONG nfiles;

    /* Verify the handle.
     */
    if (dir->_d_magic != DIRMAGIC)
            return;

    /* Search for the first few files.
     */
    nfiles = 2;
    if (DosFindFirst((PSZ)dir->_d_dirname, &dir->_d_hdir, SEARCH_ATTR,
            (PVOID)&dir->_d_buf[0], sizeof(dir->_d_buf),
            &nfiles, FIL_STANDARD) == 0)
    {
        dir->_d_nfiles = (unsigned)nfiles;
        dir->_d_bufp = &dir->_d_buf[0];
    }
}


struct dirent * readdir(DIR *dir)
{
    ULONG nfiles;
    FILEFINDBUF3 *ff;

    /* Verify the handle.
     */
    if (dir->_d_magic != DIRMAGIC)
    {
        errno = EBADF;
        return (NULL);
    }

    /* If all files in the buffer have been returned, find some more files.
     */
    if (dir->_d_nfiles == 0)
    {
        nfiles = 2;
        if (DosFindNext(dir->_d_hdir,(PFILEFINDBUF)&dir->_d_buf[0],
                sizeof(dir->_d_buf), &nfiles) != 0)
            return (NULL);
        dir->_d_nfiles = (unsigned)nfiles;
        dir->_d_bufp = &dir->_d_buf[0];
    }

    /* Return the filename of the current file in the buffer,
     * advance the buffer pointer.
     */
    ff = (FILEFINDBUF3 *)(dir->_d_bufp);
    dir->_d_bufp += (int)ff->oNextEntryOffset;
    dir->_d_nfiles--;

    return ((struct dirent *)&ff->achName[0]);
}


int closedir(DIR *dir)
{
    /* Verify the handle.
     */
    if (dir == NULL || dir->_d_magic != DIRMAGIC)
    {
        errno = EBADF;
        return (-1);
    }

    dir->_d_magic = 0;          /* prevent accidental use after closing */
    DosFindClose(dir->_d_hdir); /* close directory handle */
    free(dir->_d_dirname);      /* free directory name */
    free(dir);                  /* free directory structure */
    return 0;
}

#endif


//  ------------------------------------------------------------------
//  The stat function is broken in Borland C++ 5.0.
//  This is a fixed version of the RTL source.

#if defined(__BORLANDC__) and defined(__WIN32__)

int __NTerror(void);

extern void _statcvt(
    struct stat *bufP,      /* stat structure to be filled in */
    DWORD attr,             /* NT file attributes */
    FILETIME *ctime,        /* NT time of file creation */
    FILETIME *atime,        /* NT time of last access */
    FILETIME *wtime,        /* NT time of last write */
    DWORD fsize);           /* NT low word of file size */

int _RTLENTRY _EXPFUNC stat (const char *pathP, struct stat *bufP)
{
    WIN32_FIND_DATA ff;
    HANDLE      hfile;              /* file handle */
    char        curdir[MAX_PATH];   /* current directory */
    char        DriveChar;
    char       *full = NULL;

    /* Assume it is a disk file and try to get the FindFirst info.
     */
    memset(bufP, 0, sizeof(struct stat));   /* Zero the structure   */
    bufP->st_nlink = 1;

    if ((hfile = FindFirstFile((char *)pathP, &ff)) == (HANDLE)-1)
    {
        /* Check for special case of ROOT directory
         */
        if ((strpbrk(pathP, "\\/.") != NULL) &&
           ((full = _fullpath(NULL, pathP, 0)) != NULL) &&
           (strlen(full) == 3))
        {
            if (GetDriveType(full) < 2)
                return (__NTerror());

            bufP->st_mode = S_IFDIR;
            bufP->st_dev = bufP->st_rdev = (dev_t)(toupper(full[0]) - 'A' + 1);

            free(full);
            return 0;
        }
        else
            if (full)
                free (full);

        /* It may not be a disk file.  Try to open the file for reading
         * so we can find out the type of the file.
         */
        if ((hfile = CreateFile((PSZ)pathP, GENERIC_READ,
                      FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
                      OPEN_EXISTING, 0, NULL)) == (HANDLE)-1)
            return (__NTerror());

        /* Get the handle type to see if it is a character device or pipe.
         */
        bufP->st_mode = (mode_t)(GetFileType(hfile) == FILE_TYPE_PIPE ? S_IFIFO : S_IFCHR);
        CloseHandle(hfile);
        return (0);
    }

    /* It it a disk file, convert the NT file info to a stat structure.
     */
    FindClose(hfile);
    _statcvt(bufP, ff.dwFileAttributes, &ff.ftCreationTime,
             &ff.ftLastAccessTime, &ff.ftLastWriteTime, ff.nFileSizeLow);

    /* Determine the disk device by parsing the drive name.
     * If no drive name, assume current drive.
     */
    if (pathP[1] == ':')
        DriveChar = pathP[0];
    else
    {
        if (GetCurrentDirectory(sizeof(curdir), curdir) == 0)
            DriveChar = 'A';        /* reasonable default if failure */
        else
            DriveChar = curdir[0];
    }
    if (DriveChar >= 'a' && DriveChar <= 'z')
        bufP->st_dev = (dev_t)(DriveChar - 'a');
    else if (DriveChar >= 'A' && DriveChar <= 'Z')
        bufP->st_dev = (dev_t)(DriveChar - 'A');
    bufP->st_rdev = bufP->st_dev;
    return (0);
}

#endif

//  ------------------------------------------------------------------

#ifdef __cplusplus
}
#endif


//  ------------------------------------------------------------------

