
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gfilutil.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  File/disk handling functions.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#ifndef __GFILUTIL_H
#define __GFILUTIL_H


//  ------------------------------------------------------------------

#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <gio.h>
#include <gshare.h>
#include <gdirect.h>

#include <gstrgstr.h>


//  ------------------------------------------------------------------M

// Really??? #if defined(__GNUC__) and not defined(__EMX__) and not defined(__DJGPP__)
#if defined(__UNIX__)
#define O_TEXT 0
#define O_BINARY 0
#endif

#ifndef S_IWUSR
#define S_IWUSR S_IWRITE
#define S_IRUSR S_IREAD
#endif

#if defined(__UNIX__)
#define S_STDRW (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)
#define S_STDRD (S_IRUSR|S_IRGRP|S_IROTH)
#else
#define S_STDRW (S_IRUSR|S_IWUSR)
#define S_STDRD S_IRUSR
#endif


//  ------------------------------------------------------------------
//  Standardized split/merge defines

#if defined(__MSDOS__) and not defined(__386__)
#define GMAXPATH    80
#define GMAXDRIVE   66               // Allows "\\server\volume" syntax
#define GMAXDIR     66
#define GMAXFILE     9
#define GMAXEXT      5
#define GMAXFILEEXT 13
#else
#define GMAXPATH    260
#define GMAXDRIVE   265
#define GMAXDIR     256
#define GMAXFILE    256
#define GMAXEXT     256
#define GMAXFILEEXT 256
#endif


//  ------------------------------------------------------------------
//  Misc. defines

#if defined(__MSDOS__) or defined(__OS2__) or defined(__WIN32__)
  #define GOLD_SLASH_CHR                    '\\'              // Backslash
  #define GOLD_SLASH_STR                    "\\"
  #define GOLD_WRONG_SLASH_CHR              '/'               // Fwrdslash
  #define GOLD_WRONG_SLASH_STR              "/"
  #define GOLD_SHELL_ENV                    "COMSPEC"
#else
  #define GOLD_SLASH_CHR                    '/'               // Fwrdslash
  #define GOLD_SLASH_STR                    "/"
  #define GOLD_WRONG_SLASH_CHR              '\\'              // Backslash
  #define GOLD_WRONG_SLASH_STR              "\\"
  #define GOLD_SHELL_ENV                    "SHELL"
#endif


//  ------------------------------------------------------------------

#if defined(__MSDOS__)
  #define GOLD_WILDCARD_ALL_FILES "*.*"
#else
  #define GOLD_WILDCARD_ALL_FILES "*"
#endif


//  ------------------------------------------------------------------
//  Path typedefs

typedef char Path[GMAXPATH];


//  ------------------------------------------------------------------

class gfile;

#define GPATHLEN 256

class gpath {

protected:

  char path[GPATHLEN];
  bool isdir;

public:

  gpath() { *path = NUL; isdir = false; }

  int size() { return sizeof(path); }

  const char* prepare(const char* p);
  const char* concat(const char* p);
  const char* copy(const char* p);
  const char* copypath(const char* p);

  char* getdrive(char* buf);
  char* getdir(char* buf);
  char* getfile(char* buf);
  char* getname(char* buf);
  char* getnameext(char* buf);
  char* getext(char buf);

  void setdrive(const char* p);
  void setdir(const char* p);
  void setfile(const char* p);
  void setname(const char* p);
  void setnameext(const char* p);
  void setext(const char* p);

  void normalize();
  void make();

  inline bool is_directory() { return isdir; }
  inline bool is_file()      { return not isdir; }

  inline const char* operator+= (const char* p) { return concat(p); }

  inline operator void*() { return path; }
  inline operator const void*() { return path; }
  inline operator const char*() { return path; }

  void save(gfile& f);
  void load(gfile& f);
};


//  ------------------------------------------------------------------
//  Structure for filetime stamp checking

struct Stamp {
  long ft;   // Timestamp
  Path fn;   // Filename
  byte fc;   // Filecheck
};


//  ------------------------------------------------------------------
//  Structure for filetime stamp checking

class gstamp {

public:

  long    ft;   // Timestamp
  gstring fn;   // Filename
  byte    fc;   // Filecheck

  gstamp& operator=(const gstamp& x) { ft=x.ft; fn=x.fn; fc=x.fc; return *this; }
  
  void save(gfile& f);
  void load(gfile& f);
};


//  ------------------------------------------------------------------
//  Prototypes

FILE* fsopen(const char* path, const char* type, int shflag);
inline FILE* fsopen(const gstring& path, const char* type, int shflag) { return fsopen(path.c_str(), type, shflag); }

int fexist(const char* filename);
inline int fexist(const gstring& filename) { return fexist(filename.c_str()); }
int is_dir(const char* path);
inline int is_dir(const gstring& path) { return is_dir(path.c_str()); }

long diskfree(int drive);
long FiletimeCmp(const char* file1, const char* file2);
inline long FiletimeCmp(const gstring& file1, const gstring& file2) { return FiletimeCmp(file1.c_str(), file2.c_str()); }
long fsize(FILE* fp);
long GetFilesize(const char* file);
long GetFiletime(const char* file);
inline long GetFiletime(const gstring& file) { return GetFiletime(file.c_str()); }

const char* AddPath(const char* path, const char* file);
const char* AddPath(const gstring& path, const char* file);
inline const char* AddPath(const gstring& path, const gstring& file) { return AddPath(path, file.c_str()); }
char* MakePath(char* pathfile, const char* path, const char* file);
void MakePathname(char* pathname, const char* path, const char* name);
void MakePathname(gstring& pathname, const gstring& path, const gstring& name);

char* AddBackslash(char* p);
gstring& AddBackslash(gstring& p);
char* StripBackslash(char* p);

char* PathCopy(char* dst, const char* src);
void PathCopy(gstring& dst, const char* src);
inline void gpathcopy(gstring& dst, const char* src) { PathCopy(dst, src); }

int ftrunc(FILE* fp, long size);

void TouchFile(const char* __filename);

int PathInvalid(const char* __path);

int TestLockPath(const char* __path);

void flushfile(int fh);

const char* CleanFilename(const char* __file);

int ForceDeleteFile(const char* file);

void WipeFile(const char* file, int options=0);

int strschg_environ(char* s);
int strschg_environ(gstring& s);

char* MapPath(char* map, bool reverse = false);
inline char* ReMapPath(char* map) { return MapPath(map, true); };


//  ------------------------------------------------------------------

inline long lseekset(int fh, long offset) { return lseek(fh, offset, SEEK_SET); }
inline long lseekset(int fh, long record, long recordsize) { return lseek(fh, record*recordsize, SEEK_SET); }

int gchdir(const char* dir);

int compare_filetimes(const gstring& file1, const gstring& file2);


#if defined(__GNUC__)
extern "C" int eof(int h);
#if not defined(__EMX__)
#include <stdio.h>
FILE* _fsopen(const char *path, const char *type, int shflag);
#endif
#endif


//  ------------------------------------------------------------------

#if not defined(__GNUC__)

int mkdir(const char* path, int);

#endif


//  ------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//  ------------------------------------------------------------------

#if defined(__WATCOMC__) or defined(__GNUC__)

int getcurdir(int drive, char *path);

#endif


//  ------------------------------------------------------------------

#if defined(__WATCOMC__)

char* mktemp(char* temp);

#endif


//  ------------------------------------------------------------------

#if (defined(__BORLANDC__) and defined(__OS2__)) or defined(__GNUC__)

long filelength(int fh);

#endif


//  ------------------------------------------------------------------

#if defined(__GNUC__)

void _makepath(char* path, const char* drive, const char* dir, const char* name, const char* ext);
void _splitpath(const char* path, char* drive, char* dir, char* name, char* ext);

#endif


//  ------------------------------------------------------------------

#if defined(__GNUC__)
#undef sopen
int sopen(const char* path, int access, int shflag, int mode=S_STDRD);

#if not defined(__DJGPP__)
int chsize(int handle, long size);
int lock(int handle, long offset, long length);
int unlock(int handle, long offset, long length);
inline long tell(int fh) { return lseek(fh, 0, SEEK_CUR); }
#endif

#endif


//  ------------------------------------------------------------------

#ifdef __cplusplus
}
#endif

//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

