
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gfilutl2.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  File utility functions
//  ------------------------------------------------------------------


#include <stdlib.h>
#include <gstrgstr.h>
#include <gfilutil.h>


//  ------------------------------------------------------------------
//  Compare filetimes

int compare_filetimes(const gstring& file1, const gstring& file2) {

  struct stat st;
  ulong mtime1=0, mtime2=0;
  if(stat(file1.c_str(), &st) == 0)
    mtime1 = st.st_mtime;
  if(stat(file2.c_str(), &st) == 0)
    mtime2 = st.st_mtime;
  return compare_two(mtime1, mtime2);
}


//  ------------------------------------------------------------------

gstring& AddBackslash(gstring& p) {

  p.chg(GOLD_WRONG_SLASH_CHR, GOLD_SLASH_CHR);
  if(p[p.length()-1] != GOLD_SLASH_CHR)
    p += GOLD_SLASH_STR;

  return p;
}


//  ------------------------------------------------------------------
//  Add path to filename if no path is present

const char* AddPath(const gstring& path, const char* file) {

  if(strpbrk(file, "/\\")) {
    // Don't add path if the filename already contains one
    return file;
  }
  else if(path and ((*file == '.') or strchr("/\\", path[path.length()-1]))) {
    // Build path+filename if path ends with a slash or backslash
    return stradd(path.c_str(), file);
  }
  else {
    // Use filename in path or file
    return path ? path.c_str() : file;
  }
}


//  ------------------------------------------------------------------
//  Add path to filename, if no path is set

void MakePathname(gstring& pathname, const gstring& path, const gstring& name) {

  Path pn;
  strcpy(pn, pathname.c_str());
  MakePathname(pn, path.c_str(), name.c_str());
  pathname = pn;
}


//  ------------------------------------------------------------------

void PathCopy(gstring& dst, const char* src) {

  dst = src;
  AddBackslash(dst);
}


//  ------------------------------------------------------------------

const char* gpath::copy(const char* p) {

  return strxcpy(path, p, size());
}


//  ------------------------------------------------------------------

const char* gpath::copypath(const char* p) {

  copy(p);
  AddBackslash(path);
  return path;
}


//  ------------------------------------------------------------------

const char* gpath::prepare(const char* p) {

  copy(p);
  strchg(path, GOLD_WRONG_SLASH_CHR, GOLD_SLASH_CHR);
  MapPath(path);
  strschg_environ(path);

  #if 0
  normalize();
  if(is_dir(path)) {
    AddBackslash(path);
    isdir = true;
  }
  else
    isdir = false;
  #endif

  return path;
}


//  ------------------------------------------------------------------

const char* gpath::concat(const char* p)  {

  char buf[GPATHLEN];
  //char buf2[GPATHLEN];
  gpath p2;

  p2.prepare(p);
  p2.getdrive(buf);

  strcat(path, p);
  normalize();

  return path;
}


//  ------------------------------------------------------------------

char* gpath::getdrive(char* buf) {

  if(isalpha(*path) and (path[1] == ':' or path[2] == ':'))
    strxcpy(buf, path, path[1] == ':' ? 3 : 4);
  else if (path[0] == GOLD_SLASH_CHR and path[1] == GOLD_SLASH_CHR) {
    char* q = strchr(path+2, GOLD_SLASH_CHR);
    if(not q)
      strcpy(buf, path);
    else
      strxcpy(buf, path, (int) (q-path+1));
  }
  else
    *buf = NUL;

  return buf;
}


//  ------------------------------------------------------------------

char* gpath::getdir(char* buf) {

  char buf2[GPATHLEN];

  getdrive(buf2);

  char* p = &path[strlen(buf2)];

  char* q = strrchr(p, GOLD_SLASH_CHR);
  if(q) {
    strxcpy(buf, p, (int)(q-p+2));
  }
  else
    *buf = NUL;

  return buf;
}


//  ------------------------------------------------------------------

void gpath::normalize() {

  char buf[GPATHLEN];
  char buf2[GPATHLEN];

  getdrive(buf2);
  getdir(buf);

  if(*buf == GOLD_SLASH_CHR)
    strcat(buf2, GOLD_SLASH_STR);

  for(char* part = strtok(buf, GOLD_SLASH_STR); part; part = strtok(NULL, GOLD_SLASH_STR)) {

    if(streql(part, ".")) {
      // do nothing, it's something like "test/./test2/"

    }
    else if(streql(part, "..")) {
      // go back one directory level
      int len = strlen(buf2);

      if(len and buf2[len-1] == GOLD_SLASH_CHR) {
        buf2[len-1] = NUL;

        char* p = strrchr(buf2, GOLD_SLASH_CHR);
        if(p)
          *(p+1)= NUL;
      }
    }
    else {
      strcat(buf2, part);
      strcat(buf2, GOLD_SLASH_STR);
    }
  }

  getfile(buf);
  strcat(buf2, buf);
  strcpy(path, buf2);
}


//  ------------------------------------------------------------------

char* gpath::getfile(char* buf) {

  char* p = strrchr(path, GOLD_SLASH_CHR);

  return strcpy(buf, p ? p+1 : path);
}


//  ------------------------------------------------------------------

int strschg_environ(gstring& s) {

  const char* b;
  const char* e;
  char* replace;
  char find[41];
  char findrep[41];
  int replaced = 0;

  do {
    b = s.chr('%');
    if(b) {
      e = strchr(b+1, '%');
      if(e) {
        strxcpy(find, b+1, (size_t)(e-b));
        strxcpy(findrep, b, (size_t)(e-b)+2);
        replace = getenv(find);
        replace = replace ? replace : "";
        s.ichange(findrep, replace);
        replaced++;
      }
    }
  } while(b and e);

  replace = getenv("HOME");
  if(replace) {
    s.ichange("~", replace);
    replaced++;
  }


  return replaced;
}


//  ------------------------------------------------------------------

