
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gftnadrs.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  FidoNet Technology Network - 5D address class.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#ifndef __GFTNADRS_H
#define __GFTNADRS_H


//  ------------------------------------------------------------------

#include <gftnall.h>
#include <gstrgstr.h>


//  ------------------------------------------------------------------

class gftnaddrs {

public:

  word    zone;
  word    net;
  word    node;
  word    point;
  gstring domain;

protected:

  gstring         address;

  static bool     getaddrpart(word& part, const char* s);

  const gstring&  build_address(bool normal=true);

public:

  gftnaddrs();
  gftnaddrs(const char* s);
  gftnaddrs(const gstring& s);
  gftnaddrs(const ftn_addr& a);
  gftnaddrs(const gftnaddrs& a);
  gftnaddrs(const ftn_aka& a);

  void           reset()                                  { operator=(0); }

  bool           valid()                                  { return net != 0; }
  bool           invalid()                                { return not valid(); }

  gftnaddrs&     operator=(int a);
  gftnaddrs&     operator=(const char* s);
  gftnaddrs&     operator=(const gstring& s);
  gftnaddrs&     operator=(const ftn_addr& a);
  gftnaddrs&     operator=(const gftnaddrs& a);
  gftnaddrs&     operator=(const ftn_aka& a);

  gftnaddrs&     set(const char* s);
  gftnaddrs&     set(const gstring& s);

  const gstring& str(bool normal=true)                    { return build_address(normal); }
  const char*    c_str(bool normal=true)                  { return build_address(normal).c_str(); }

  bool           match(const gftnaddrs& mask) const;

  bool           equals(const ftn_addr& other) const      { return compare(other) == 0; }
  bool           equals(const gftnaddrs& other) const     { return compare(other) == 0; }
  bool           equal4d(const gftnaddrs& other) const    { return compare4d(other) == 0; }

  int            compare(const ftn_addr& other) const;
  int            compare(const gftnaddrs& other, bool fourdee=false) const;
  int            compare4d(const gftnaddrs& other) const  { return compare(other, true); }

  bool           operator==(const ftn_addr& b) const      { return equals(b); }
  bool           operator==(const gftnaddrs& b) const     { return equals(b); }
  bool           operator!=(const ftn_addr& b) const      { return !equals(b); }
  bool           operator!=(const gftnaddrs& b) const     { return !equals(b); }

  friend gfile&  operator<<(gfile& fp, const gftnaddrs& a);
  friend gfile&  operator>>(gfile& fp, gftnaddrs& a);

};


//  ------------------------------------------------------------------

class gftnnode {

public:

  gftnnode() { }
  gftnnode(const gftnnode& n) { operator=(n); }
  gftnnode(const gstring& n, const gftnaddrs& a) { name = n; addr = a; }
  ~gftnnode() { }

  gstring   name;
  gftnaddrs addr;

  gftnnode& operator=(const gftnnode& n) { name = n.name; addr = n.addr; return *this; }

  friend gfile& operator<<(gfile& fp, const gftnnode& a);
  friend gfile& operator>>(gfile& fp, gftnnode& a);

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

