
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gftnnlge.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  GoldED nodelist indexing.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GFTNNLGE_H
#define __GFTNNLGE_H


//  ------------------------------------------------------------------
//  Required

#include <gftnnl.h>
#include <gfilutil.h>


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif

struct _GEIdx {
  long     pos;        // File Number OR'ed with pos in nodelist file
  ftn_addr addr;       // Node address
  char     name[36];   // Name in reversed form
};

#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------

class ftn_golded_nodelist_index : public ftn_nodelist_index_base {

protected:

  struct fstamp {
    Path filename;
    long stamp;
  };

  int      fha;
  int      fhn;
  int      fhx;

  fstamp*  nodelist;
  int      nodelists;

  int      lastfileno;

  _GEIdx   current;

  long     node;
  long     maxnode;

  long     statenode;

  char     searchname[80];
  ftn_addr searchaddr;

  void     fetchdata();
  void     getnode();
  int      namecmp() const;
  int      addrcmp() const;
  void     compare()           { exactmatch = not (namebrowse ? namecmp() : addrcmp()); }
  bool     searchfirst();
  bool     search();

public:

  ftn_golded_nodelist_index();
  virtual ~ftn_golded_nodelist_index();

  bool can_browse_name() const     { return true; }
  bool can_browse_address() const  { return true; }

  bool open();
  void close();

  bool find(const char* name);
  bool find(const ftn_addr& addr);

  bool previous();
  bool next();

  void first();
  void last();

  void push_state();
  void pop_state();

  const char* index_name() const;
  const char* nodelist_name() const;

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

