
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gftnpkts.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  FidoNet Technology Packet structures etc.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GPKTALL_H
#define __GPKTALL_H


//  ------------------------------------------------------------------
//  Required headers

#include <gdefs.h>
              

//  ------------------------------------------------------------------
//  FTS-1 Rev.15 ("Type 2.0" or "Stoneage")

struct PktHdrFTS1 {

  word  orig_node;      // (* of packet, not of messages in packet *)
  word  dest_node;      // (* of packet, not of messages in packet *)
  word  year;           // (* of packet creation, e.g. 1986 *)
  word  month;          // (* of packet creation, 0-11 for Jan-Dec *)
  word  day;            // (* of packet creation, 1-31 *)
  word  hour;           // (* of packet creation, 0-23 *)
  word  minute;         // (* of packet creation, 0-59 *)
  word  second;         // (* of packet creation, 0-59 *)
  word  rate;           // (* max baud rate of orig and dest, 0=SEA *
  word  ver;            // (* old type-1 packets now obsolete *)
  word  orig_net;       // (* of packet, not of messages in packet *)
  word  dest_net;       // (* of packet, not of messages in packet *)
  char  product;        // (* 0 for Fido, write to FTSC for others *)
  char  rev_lev;        // (* binary serial number (otherwise null)*)
  char  password[8];    // (* session password  (otherwise null)   *)
  word  orig_zone,      // (* zone of pkt sender (otherwise null)  *)
  word  dest_zone;      // (* zone of pkt receiver (otherwise null)*)
  char  fill[20];       // (* fill *)
};


//  ------------------------------------------------------------------
//  FSC-39 Rev.3 ("Type 2+")

struct PktHdrFSC39 {
                        // Field    Ofs Siz Type  Description                Expected value(s)
                        // -------  --- --- ----  -------------------------- -----------------
  word  orig_node;      // OrgNode  0x0   2 Word  Origination node address   0-65535
  word  dest_node;      // DstNode    2   2 Word  Destination node address   1-65535
  word  year;           // Year       4   2  Int  Year packet generated      19??-2???
  word  month;          // Month      6   2  Int  Month  "        "          0-11 (0=Jan)
  word  day;            // Day        8   2  Int  Day    "        "          1-31
  word  hour;           // Hour       A   2  Int  Hour   "        "          0-23
  word  minute;         // Min        C   2  Int  Minute "        "          0-59
  word  second;         // Sec        E   2  Int  Second "        "          0-59
  word  rate;           // Baud      10   2  Int  Baud Rate (not in use)     ????
  word  ver;            // PktVer    12   2  Int  Packet Version             Always 2
  word  orig_net;       // OrgNet    14   2 Word  Origination net address    1-65535
  word  dest_net;       // DstNet    16   2 Word  Destination net address    1-65535
  char  product;        // PrdCodL   18   1 Byte  FTSC Product Code     (lo) 1-255
  char  rev_lev;        // PVMajor   19   1 Byte  FTSC Product Rev   (major) 1-255
  char  password[8];    // Password  1A   8 Char  Packet password            A-Z,0-9
  word  qm_orig_zone;   // QOrgZone  22   2  Int  Orig Zone (ZMailQ,QMail)   1-65535
  word  qm_dest_zone;   // QDstZone  24   2  Int  Dest Zone (ZMailQ,QMail)   1-65535
  char  filler[2];      // Filler    26   2 Byte  Spare Change               ?
  word  capword2;       // CapValid  28   2 Word  Capability Word Valid Copy BitField
  char  product2,       // PrdCodH   2A   1 Byte  FTSC Product Code     (hi) 1-255
  char  rev_lev2;       // PVMinor   2B   1 Byte  FTSC Product Rev   (minor) 1-255
  word  capword;        // CapWord   2C   2 Word  Capability Word            BitField
  word  orig_zone;      // OrigZone  2E   2  Int  Origination Zone           1-65535
  word  dest_zone;      // DestZone  30   2  Int  Destination Zone           1-65535
  word  orig_point;     // OrigPoint 32   2  Int  Origination Point          1-65535
  word  dest_point;     // DestPoint 34   2  Int  Destination Point          1-65535
  long  pr_data;        // ProdData  36   4 Long  Product-specific data      Whatever
};


//  ------------------------------------------------------------------
//  FSC-45 Rev.1 ("Type 2.2")

struct PktHdrFSC45 {

  word  onode;          // Originating node number
  word  dnode;          // Destination node number
  word  opoint;         // Originating point number
  word  dpoint;         // Destination point number
  char  zeros[8];       // Reserved, must be zero
  word  subver;         // Packet sub-version (2)
  word  version;        // Packet version (2)
  word  onet;           // Originating network
  word  dnet;           // Destination network
  char  product;        // Product code
  char  rev_lev;        // Product revision level
  char  password[8];    // Password
  word  ozone;          // Originating zone
  word  dzone;          // Destination zone
  char  odomain[8];     // Originating domain
  char  ddomain[8];     // Destination domain
  long  specific;       // Product specific data
};


//  ------------------------------------------------------------------
//  FSC-48 Rev.1 ("Type 2.1"?)

struct PktHdrFSC48 {

  word  orig_node;      // origNode   (* of packet, not of messages in packet *)
  word  dest_node;      // destNode   (* of packet, not of messages in packet *)
  word  year;           // year       (* of packet creation, e.g. 1986 *)
  word  month;          // month      (* of packet creation, 0-11 for Jan-Dec *)
  word  day;            // day        (* of packet creation, 1-31 *)
  word  hour;           // hour       (* of packet creation, 0-23 *)
  word  minute;         // minute     (* of packet creation, 0-59 *)
  word  second;         // second     (* of packet creation, 0-59 *)
  word  rate;           // baud       (* max baud rate of orig and dest *)
  word  ver;            // PacketType (* old type-1 packets now obsolete *)
  word  orig_net;       // origNet    (* of packet, not of messages in packet. set to -1 if orig=point    *)
  word  dest_net;       // destNet    (* of packet, not of messages in packet *)
  char  product;        // productCode Lo (* 0 for Fido, write to FTSC for others *)
  char  rev_lev;        // serialNo Maj (* binary serial number (otherwise null)*)
  char  password[8];    // password   (* session pasword  (otherwise null)    *)
  word  qm_orig_zone;   // origZone   (* zone of pkt sender (otherwise null)  *)
  word  qm_dest_zone;   // destZone   (* zone of pkt receiver (otherwise null)*)
  word  aux_net;        // auxNet     (* contains Orignet if Origin is a point *)
  word  capword2;       // CW validation copy (* Must be equal to CW to be valid *)
  char  product2;       // ProductCode Hi
  char  rev_lev2;       // revision Minor
  word  capword;        //
  word  orig_zone;      // origZone   (* zone of pkt sender (otherwise null)  *)
  word  dest_zone;      // destZone   (* zone of pkt receiver (otherwise null)*)
  word  orig_point;     // origPoint  (* point of pkt sender (otherwise null)  *)
  word  dest_point;     // destPoint  (* point of pkt receiver (otherwise null)*)
  long  pr_data;        // ProdData   (* Product specific filler *)
};


//  ------------------------------------------------------------------
//  Fido Packet Message attributes (attr) (informational)

//      Name        Value      FTS-1 Description
#define PKTPRIVATE  0x0001  // Private
#define PKTCRASH    0x0002  // Crash
#define PKTREAD     0x0004  // Recd
#define PKTSENT     0x0008  // Sent
#define PKTFILE     0x0010  // FileAttached
#define PKTFWD      0x0020  // InTransit
#define PKTORPHAN   0x0040  // Orphan
#define PKTKILL     0x0080  // KillSent
#define PKTLOCAL    0x0100  // Local
#define PKTHOLD     0x0200  // HoldForPickup
#define PKTRSV      0x0400  // unused
#define PKTFRQ      0x0800  // FileRequest
#define PKTRRQ      0x1000  // ReturnReceiptRequest
#define PKTCPT      0x2000  // IsReturnReceipt
#define PKTARQ      0x4000  // AuditRequest
#define PKTURQ      0x8000  // FileUpdateReq


//  ------------------------------------------------------------------
//  Message Headers

struct MsgHdr {

  word  onode;
  word  dnode;
  word  onet;
  word  dnet;
  word  attr;
  short  cost;
};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

