
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: ggrpmgr.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Group manager class.
//  ------------------------------------------------------------------


#ifndef __GGRPMGR_H
#define __GGRPMGR_H


//  ------------------------------------------------------------------

#include <gvector.h>
#include <gstrgstr.h>


//  ------------------------------------------------------------------

class gfile;


//  ------------------------------------------------------------------

class group_manager {

protected:

  class grp {
    friend          class group_manager;
  protected:
    class itm {
      friend        class group_manager;
    protected:
      gvector<long> position;
    public:
      void          add(long p)              { position.push_back(p); }
      int           size() const             { return position.size(); }
      long          operator[](int n) const  { return position[n]; }
    };
    gvector<int>    members;
    gvector<itm>    items;
  public:
    gstring         name;
    grp&            operator=(const grp&)    { return *this; }
    const itm&      operator[](int n) const  { return items[n]; }
  };

  gvector<gstring>  members;
  gvector<grp>      groups;
  int               current;
  int               types;

public:

  int               size() const             { return groups.size(); }

  void              set_types(int n)         { types = n; }

  void              add_group(const gstring& name);
  void              add_member(const gstring& name);
  void              add_item(int type, long position);

  void              inherit_from(int group);
  void              inherit_from(const gstring& name) { inherit_from(find_named_group(name)); }

  int               find_named_group(const gstring& name);
  int               find_matching_group(const gstring& name);

  int               count(int group, int type);
  long              position(int group, int type, int at);

  const grp&        operator[](int n) const  { return groups[n]; }

};


//  ------------------------------------------------------------------

class group_mgr_file : public group_manager {

protected:

  bool get_item(int type, int index);

  long baseoffset;

public:

  gfile* fp;

  group_mgr_file() { current = 0; baseoffset = 0; fp = NULL; }
  ~group_mgr_file() { }

  void set_fp(gfile& f)   { fp = &f; }
  void set_offset(long o) { baseoffset = o; }
  void set_current(int c) { current = c; }

};

//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

