
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: ggrpmgr1.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Group items class.
//  ------------------------------------------------------------------


#include <gwildmat.h>
#include <ggrpmgr.h>


//  ------------------------------------------------------------------

void group_manager::add_group(const gstring& name) {

  current = groups.size();
  groups.resize(current+1);
  groups[current].name = name;
  groups[current].items.resize(types);
}


//  ------------------------------------------------------------------

void group_manager::add_member(const gstring& name) {

  int m;

  // See if we have it already
  int members_size = members.size();
  for(m=0; m<members_size; m++)
    if(name.ieql(members[m]))
      break;

  // Add it if we don't
  if(m == members_size) {
    members.resize(m+1);
    members[m] = name;
  }

  // Add it to the group
  groups[current].members.push_back(m);
}


//  ------------------------------------------------------------------

void group_manager::add_item(int type, long position) {

  groups[current].items[type].add(position);
}


//  ------------------------------------------------------------------

void group_manager::inherit_from(int group) {

  grp& sgroup = groups[group];
  grp& dgroup = groups[current];

  int sgroup_items_size = sgroup.items.size();
  for(int i=0; i<sgroup_items_size; i++) {
    grp::itm& sitems = sgroup.items[i];
    int sitems_position_size = sitems.position.size();
    if(sitems_position_size) {
      grp::itm& ditems = dgroup.items[i];
      for(int p=0; p<sitems_position_size; p++)
        ditems.add(sitems[p]);
    }
  }
}


//  ------------------------------------------------------------------

int group_manager::find_named_group(const gstring& name) {

  int groups_size = groups.size();
  for(int n=0; n<groups_size; n++)
    if(name.ieql(groups[n].name))
      return n;

  return -1;
}


//  ------------------------------------------------------------------

int group_manager::find_matching_group(const gstring& name) {

  // Find member
  int members_size = members.size();
  for(int m=0; m<members_size; m++) {
    if(strwild(name.c_str(), members[m].c_str())) {

      // Found member, find group containing member
      int groups_size = groups.size();
      for(int g=0; g<groups_size; g++) {
        gvector<int>& gmembers = groups[g].members;
        int gmembers_size = gmembers.size();
        for(int n=0; n<gmembers_size; n++) {
          if(gmembers[n] == m)
            return g;
        }
      }
    }
  }

  return -1;
}


//  ------------------------------------------------------------------

int group_manager::count(int group, int type) {

  return groups[group][type].size();
}


//  ------------------------------------------------------------------

long group_manager::position(int group, int type, int at) {

  return groups[group][type][at];
}


//  ------------------------------------------------------------------

