
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: ghdrpars.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  RFC822 mailbox parser.
//  ------------------------------------------------------------------


#include <gmemdbg.h>
#include <ghdrpars.h>


//  ------------------------------------------------------------------

grfc822_mailbox::~grfc822_mailbox() {

  delete[] text;
}


//  ------------------------------------------------------------------

grfc822_mailbox::grfc822_mailbox() {

  text = NULL;
}


//  ------------------------------------------------------------------

grfc822_mailbox::grfc822_mailbox(const gstring& s) {

  operator=(s);
}


//  ------------------------------------------------------------------

void grfc822_mailbox::operator=(const gstring& s) {

  text = new char[s.length()+1];    throw_new(text);
  start = strcpy(text, s.c_str());
  process();
}


//  ------------------------------------------------------------------

void grfc822_mailbox::process() {

  commaptr = NULL;
  if(strchr(start, ',')) {
    bool inquotes = false;
    commaptr = start;
    while(commaptr) {
      if(*commaptr == '\"')
        inquotes ^= true;
      else if((*commaptr == ',') and not inquotes) {
        *commaptr = NUL;
        break;
      }
      commaptr++;
    }
  }

  mbox = start;
  char* endchar = start + strlen(start) - 1;
  if(*endchar == /*(*/ ')') {
    char* begchar = endchar;
    int pcnt = 0;
    while(begchar > start) {
      if(*begchar == /*(*/ ')')
        pcnt++;
      else if(*begchar == '(' /*)*/)
        pcnt--;
      if(pcnt == 0)
        break;
      begchar--;
    }
    if(*begchar == '(' /*)*/)
      begchar++;
    phrase.copyn(begchar, (uint)(endchar-begchar)).btrim();
    addr_spec.copyn(start, (uint)(begchar-start)-1).btrim();
  }
  else if(*endchar == '>') {
    char* endaddr = endchar;
    while(*endchar != '<' and endchar > start)
      endchar--;
    char* begaddr = endchar;
    if(*endchar == '<') {
      begaddr++;
      if(endchar > start)
        endchar--;
    }
    start = strskip_wht(start);
    phrase.copyn(start, (uint)(endchar-start)).btrim();
    addr_spec.copyn(begaddr, (uint)(endaddr-begaddr)).btrim();
  }
  else {
    if(strchr(start, '@'))
      addr_spec = start;
    else
      phrase = start;
  }

  if(addr_spec[0] == '@') {
    const char* ptr = strchr(addr_spec.c_str(), ':');
    if(ptr)
      addr_spec = ptr+1;
  }

  phrase.strip_quotes();

  if(commaptr) {
    *commaptr = ',';
    start = strskip_wht(commaptr+1);
  }
  else {
    start = start + strlen(start);
  }
}


//  ------------------------------------------------------------------

bool grfc822_mailbox::first() {

  start = text;
  return true;
}


//  ------------------------------------------------------------------

bool grfc822_mailbox::next() {

  if(*start) {
    process();
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

gstring& ParseInternetAddr(const gstring& s, gstring& name, gstring& addr) {

  grfc822_mailbox mailbox = s;

  name = mailbox.name();
  addr = mailbox.address();

  return name;
}


//  ------------------------------------------------------------------

