
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gkbdbase.cpp,v 1.5 1999/02/05 22:59:36 damware Exp $
//  ------------------------------------------------------------------
//  Keyboard functions.
//  ------------------------------------------------------------------


#include <ctype.h>
#include <gkbdcode.h>
#include <gkbdbase.h>
#include <gmemdbg.h>
#include <gmemall.h>
#include <gtimall.h>

//#define GKBD_DEBUG 1

#if defined(GKBD_DEBUG)
#include <stdio.h>
#define debug_printf(s) printf(s)
#define debug_cond_printf(c,s) { if(c) printf(s); }
#define debug_printf1(s,a) printf(s,a)
#define debug_printf2(s,a,b) printf(s,a,b)
#else
#define debug_printf(s)
#define debug_cond_printf(c,s)
#define debug_printf1(s,a)
#define debug_printf2(s,a,b)
#endif

#if defined(__OS2__)
#define INCL_BASE
#include <os2.h>
#endif

#ifdef __WIN32__
#include <windows.h>
#endif

#if defined(__UNIX__)
#include <gkbdunix.h>
#endif

#if defined(__DJGPP__)
#include <sys/farptr.h>
#endif


//  ------------------------------------------------------------------

#if defined(__WIN32__)
#define KBD_TEXTMODE (ENABLE_LINE_INPUT | ENABLE_ECHO_INPUT | ENABLE_PROCESSED_INPUT | ENABLE_WINDOW_INPUT | ENABLE_MOUSE_INPUT)
#endif


//  ------------------------------------------------------------------
//  Global keyboard data

#if defined(__WIN32__)
HANDLE gkbd_hin;
DWORD  gkbd_kbdmode;
int    gkbd_nt;
#endif

GKbd __FAR gkbd;

int __FAR blanked = false;


//  ------------------------------------------------------------------
//  Pointer to original interrupt

//#define GOLD_TICK 1

#ifdef GOLD_TICK
#if defined(__BORLANDC__) and defined(__MSDOS__)
void interrupt (*__oldtick_handler)(...);
#endif
#endif


//  ------------------------------------------------------------------
//  Tick handler

#ifdef GOLD_TICK
#if defined(__MSDOS__)
void interrupt __tick_handler(...) {

  // Stuff a fake Tick keystroke into
  // keyboard buffer once per second
  _disable();
  static dword _lasttime = 0;
  dword _thistime = *((dword*)0x46C);
  if((_thistime-_lasttime) > 18) {
    _lasttime = _thistime;
    kbput_(Key_Tick);
  }
  _enable();

  // Call the original interrupt
  __oldtick_handler();

  i86 cpu;
  cpu.r.w.ax = 0x1000;
  cpu.genint(0x15);
}
#endif
#endif

//  ------------------------------------------------------------------
//  Keyboard Class Initializer

void GKbd::Init() {

  #if defined(__OS2__)
  KBDINFO kbstInfo;
  kbstInfo.cb = sizeof(kbstInfo);
  KbdGetStatus(&kbstInfo, 0);
  kbstInfo.fsMask = (USHORT)((kbstInfo.fsMask & 0xFFF7) | 0x0004);
  KbdSetStatus(&kbstInfo, 0);
  #endif

  #if defined(__WIN32__)
  OSVERSIONINFO osversion;
  osversion.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  GetVersionEx(&osversion);
  gkbd_nt = (osversion.dwPlatformId & VER_PLATFORM_WIN32_NT) ? true : false;

  debug_printf ("Windows data:\n");
  debug_printf1("MajorVersion %lu\n", osversion.dwMajorVersion);
  debug_printf1("MinorVersion %lu\n", osversion.dwMinorVersion);
  debug_printf1("BuildNumber  %lu\n", osversion.dwBuildNumber);
  debug_printf1("PlatformId   %lu\n", osversion.dwPlatformId);
  debug_printf1("CSDVersion   %s\n",  osversion.szCSDVersion);

  gkbd_hin = CreateFile("CONIN$", GENERIC_READ | GENERIC_WRITE,
                         FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
                         OPEN_EXISTING, 0, NULL);
  GetConsoleMode(gkbd_hin, &gkbd_kbdmode);
  if(gkbd_kbdmode & KBD_TEXTMODE)
    SetConsoleMode(gkbd_hin, gkbd_kbdmode & ~KBD_TEXTMODE);
  #endif
  
  #if defined(__UNIX__)
  gkbd_tty_init();
  #endif
}


//  ------------------------------------------------------------------
//  Keyboard Class constructor

GKbd::GKbd() {

  kbuf = NULL;
  onkey  = NULL;
  curronkey = NULL;
  inmenu = 0;
  source = 0;
  polling = 0;
  tickinterval = 0;
  tickvalue = 0;
  tickfunc = NULL;
  tickpress = 0;
  inidle = 0;
  quitall = NO;

  // Detect enhanced keyboard by checking bit 4 at 0x00000496
  #if defined(__DJGPP__)
  extkbd = _farpeekb (_dos_ds, 0x0496) & (1 << 4);
  #elif defined(__MSDOS__)
  extkbd = *((byte*)0x0496) & (1 << 4);
  #elif defined(__OS2__) or defined(__WIN32__)
  extkbd = true;
  #endif

  Init();

  #if defined(__UNIX__)

  gkbd_keymap_init();
  
  char escseq[2];
  escseq[1] = NUL;
  for(int n=0; n<256; n++) {
    escseq[0] = (char)n;
    if(n == 0x7F)
      gkbd_define_keysym(escseq, Key_BS);
    else if(n == 0x09)
      gkbd_define_keysym(escseq, Key_Tab);
    else
      gkbd_define_keysym(escseq, (n < 128) ? (scancode_table[n]|n) : n);
  }

  gkbd_define_keysym("^@", 0);

  gkbd_define_keysym("\033[A", Key_Up);
  gkbd_define_keysym("\033[B", Key_Dwn);
  gkbd_define_keysym("\033[C", Key_Rgt);
  gkbd_define_keysym("\033[D", Key_Lft);

  gkbd_define_keysym("\033[[W", Key_C_Up);
  gkbd_define_keysym("\033[[Z", Key_C_Dwn);
  gkbd_define_keysym("\033[[Y", Key_C_Rgt);
  gkbd_define_keysym("\033[[X", Key_C_Lft);

  gkbd_define_keysym("\033[1~", Key_Home);
  gkbd_define_keysym("\033[7~", Key_Home);
  gkbd_define_keysym("\033[2~", Key_Ins);
  gkbd_define_keysym("\033[3~", Key_Del);
  gkbd_define_keysym("\033[4~", Key_End);
  gkbd_define_keysym("\033[8~", Key_End);
  gkbd_define_keysym("\033[5~", Key_PgUp);
  gkbd_define_keysym("\033[6~", Key_PgDn);

  gkbd_define_keysym("\033[[A",  Key_F1);
  gkbd_define_keysym("\033[[B",  Key_F2);
  gkbd_define_keysym("\033[[C",  Key_F3);
  gkbd_define_keysym("\033[[D",  Key_F4);
  gkbd_define_keysym("\033[[E",  Key_F5);
  gkbd_define_keysym("\033[17~", Key_F6);
  gkbd_define_keysym("\033[18~", Key_F7);
  gkbd_define_keysym("\033[19~", Key_F8);
  gkbd_define_keysym("\033[20~", Key_F9);
  gkbd_define_keysym("\033[21~", Key_F10);

  gkbd_define_keysym("\033[23~", Key_S_F1);
  gkbd_define_keysym("\033[24~", Key_S_F2);
  gkbd_define_keysym("\033[25~", Key_S_F3);
  gkbd_define_keysym("\033[26~", Key_S_F4);
  gkbd_define_keysym("\033[28~", Key_S_F5);
  gkbd_define_keysym("\033[29~", Key_S_F6);
  gkbd_define_keysym("\033[31~", Key_S_F7);
  gkbd_define_keysym("\033[32~", Key_S_F8);
  gkbd_define_keysym("\033[33~", Key_S_F9);
  gkbd_define_keysym("\033[34~", Key_S_F10);

  gkbd_define_keysym("\033""0", Key_A_0);
  gkbd_define_keysym("\033""1", Key_A_1);
  gkbd_define_keysym("\033""2", Key_A_2);
  gkbd_define_keysym("\033""3", Key_A_3);
  gkbd_define_keysym("\033""4", Key_A_4);
  gkbd_define_keysym("\033""5", Key_A_5);
  gkbd_define_keysym("\033""6", Key_A_6);
  gkbd_define_keysym("\033""7", Key_A_7);
  gkbd_define_keysym("\033""8", Key_A_8);
  gkbd_define_keysym("\033""9", Key_A_9);

  gkbd_define_keysym("\033a", Key_A_A);
  gkbd_define_keysym("\033b", Key_A_B);
  gkbd_define_keysym("\033c", Key_A_C);
  gkbd_define_keysym("\033d", Key_A_D);
  gkbd_define_keysym("\033e", Key_A_E);
  gkbd_define_keysym("\033f", Key_A_F);
  gkbd_define_keysym("\033g", Key_A_G);
  gkbd_define_keysym("\033h", Key_A_H);
  gkbd_define_keysym("\033i", Key_A_I);
  gkbd_define_keysym("\033j", Key_A_J);
  gkbd_define_keysym("\033k", Key_A_K);
  gkbd_define_keysym("\033l", Key_A_L);
  gkbd_define_keysym("\033m", Key_A_M);
  gkbd_define_keysym("\033n", Key_A_N);
  gkbd_define_keysym("\033o", Key_A_O);
  gkbd_define_keysym("\033p", Key_A_P);
  gkbd_define_keysym("\033q", Key_A_Q);
  gkbd_define_keysym("\033r", Key_A_R);
  gkbd_define_keysym("\033s", Key_A_S);
  gkbd_define_keysym("\033t", Key_A_T);
  gkbd_define_keysym("\033u", Key_A_U);
  gkbd_define_keysym("\033v", Key_A_V);
  gkbd_define_keysym("\033w", Key_A_W);
  gkbd_define_keysym("\033x", Key_A_X);
  gkbd_define_keysym("\033y", Key_A_Y);
  gkbd_define_keysym("\033z", Key_A_Z);

  gkbd_define_keysym("\x0D", Key_Ent);
  gkbd_define_keysym("\033\x7F", Key_A_BS);
  gkbd_define_keysym("\033\x0D", Key_A_Ent);
  gkbd_define_keysym("\033\x09", Key_A_Tab);

  #endif

  #ifdef GOLD_TICK
  #if defined(__MSDOS__)
  _disable();
  __oldtick_handler = _dos_getvect(0x1C);
  _dos_setvect(0x1C, __tick_handler);
  _enable();
  #endif
  #endif
}


//  ------------------------------------------------------------------
//  Keyboard Class destructor

GKbd::~GKbd() {

  #ifdef GOLD_TICK
  #if defined(__MSDOS__)
  _disable();
  _dos_setvect(0x1C, __oldtick_handler);
  _enable();
  #endif
  #endif

  #if defined(__WIN32__)
  if(gkbd_kbdmode & KBD_TEXTMODE)
    SetConsoleMode(gkbd_hin, gkbd_kbdmode);
  #endif

  #if defined(__UNIX__)
  gkbd_keymap_reset();
  gkbd_tty_reset();
  #endif
}


//  ------------------------------------------------------------------
//  Local table for scancode()

Key __FAR scancode_table[] = {

  Key_C_2   & 0xFF00u,     //  0x0300          C <2 @>     [NUL]
  Key_C_A   & 0xFF00u,     //  0x1E01          C <A>       [SOH]
  Key_C_B   & 0xFF00u,     //  0x3002          C <B>       [STX]
  Key_C_C   & 0xFF00u,     //  0x2E03          C <C>       [ETX]
  Key_C_D   & 0xFF00u,     //  0x2004          C <D>       [EOT]
  Key_C_E   & 0xFF00u,     //  0x1205          C <E>       [ENQ]
  Key_C_F   & 0xFF00u,     //  0x2106          C <F>       [ACK]
  Key_C_G   & 0xFF00u,     //  0x2207          C <G>       [BEL]
  Key_C_H   & 0xFF00u,     //  0x2308          C <H>       [BS]
  Key_C_I   & 0xFF00u,     //  0x1709          C <I>       [HT]
  Key_C_J   & 0xFF00u,     //  0x240A          C <J>       [LF]
  Key_C_K   & 0xFF00u,     //  0x250B          C <K>       [VT]
  Key_C_L   & 0xFF00u,     //  0x260C          C <L>       [FF]
  Key_C_M   & 0xFF00u,     //  0x320D          C <M>       [CR]
  Key_C_N   & 0xFF00u,     //  0x310E          C <N>       [SO]
  Key_C_O   & 0xFF00u,     //  0x180F          C <O>       [SI]
  Key_C_P   & 0xFF00u,     //  0x1910          C <P>       [DLE]
  Key_C_Q   & 0xFF00u,     //  0x1011          C <Q>       [DC1]
  Key_C_R   & 0xFF00u,     //  0x1312          C <R>       [DC2]
  Key_C_S   & 0xFF00u,     //  0x1F13          C <S>       [DC3]
  Key_C_T   & 0xFF00u,     //  0x1414          C <T>       [DC4]
  Key_C_U   & 0xFF00u,     //  0x1615          C <U>       [NAK]
  Key_C_V   & 0xFF00u,     //  0x2F16          C <V>       [SYN]
  Key_C_W   & 0xFF00u,     //  0x1117          C <W>       [ETB]
  Key_C_X   & 0xFF00u,     //  0x2D18          C <X>       [CAN]
  Key_C_Y   & 0xFF00u,     //  0x1519          C <Y>       [EM]
  Key_C_Z   & 0xFF00u,     //  0x2C1A          C <Z>       [SUB]
  Key_Esc   & 0xFF00u,     //  0x011B          C <[ {>     [ESC] (was: 0x1A1B)
  Key_C_Bsl & 0xFF00u,     //  0x2B1C          C <\ |>     [FS]
  Key_C_Rbr & 0xFF00u,     //  0x1B1D          C <] }>     [GS]
  Key_C_6   & 0xFF00u,     //  0x071E          C <7 &>     [RS]
  Key_C_Min & 0xFF00u,     //  0x0C1F          C <- _>
  Key_Space & 0xFF00u,     //  0x3920            <Space>
  Key_S_1   & 0xFF00u,     //  0x0221          <1 !>
  Key_S_Quo & 0xFF00u,     //  0x2822          <' ">
  Key_S_3   & 0xFF00u,     //  0x0423          <3 #>
  Key_S_4   & 0xFF00u,     //  0x0524          <4 $>
  Key_S_5   & 0xFF00u,     //  0x0625          <5 %>
  Key_S_7   & 0xFF00u,     //  0x0826          <7 &>
  Key_Quo   & 0xFF00u,     //  0x2827          <'>
  Key_S_9   & 0xFF00u,     //  0x0A28          <9 (>
  Key_S_0   & 0xFF00u,     //  0x0B29          <0 )>
  Key_S_8   & 0xFF00u,     //  0x092A          <8 *>
  Key_S_Equ & 0xFF00u,     //  0x0D2B          <= +>
  Key_Com   & 0xFF00u,     //  0x332C            <,>
  Key_Min   & 0xFF00u,     //  0x0C2D          <->
  Key_Dot   & 0xFF00u,     //  0x342E            <.>
  Key_Sls   & 0xFF00u,     //  0x352F          </>
  Key_0     & 0xFF00u,     //  0x0B30          <0>
  Key_1     & 0xFF00u,     //  0x0231          <1>
  Key_2     & 0xFF00u,     //  0x0332          <2>
  Key_3     & 0xFF00u,     //  0x0433          <3>
  Key_4     & 0xFF00u,     //  0x0534          <4>
  Key_5     & 0xFF00u,     //  0x0635          <5>
  Key_6     & 0xFF00u,     //  0x0736          <6>
  Key_7     & 0xFF00u,     //  0x0837          <7>
  Key_8     & 0xFF00u,     //  0x0938          <8>
  Key_9     & 0xFF00u,     //  0x0A39          <9>
  Key_S_Smi & 0xFF00u,     //  0x273A          <; :>
  Key_Smi   & 0xFF00u,     //  0x273B          <;>
  Key_S_Com & 0xFF00u,     //  0x333C          <, >>
  Key_Equ   & 0xFF00u,     //  0x0D3D          <=>
  Key_S_Dot & 0xFF00u,     //  0x343E          <. <>
  Key_S_Sls & 0xFF00u,     //  0x353F          </ ?>
  Key_S_2   & 0xFF00u,     //  0x0340          <2 @>
  Key_S_A   & 0xFF00u,     //  0x1E41          <A>
  Key_S_B   & 0xFF00u,     //  0x3042          <B>
  Key_S_C   & 0xFF00u,     //  0x2E43          <C>
  Key_S_D   & 0xFF00u,     //  0x2044          <D>
  Key_S_E   & 0xFF00u,     //  0x1245          <E>
  Key_S_F   & 0xFF00u,     //  0x2146          <F>
  Key_S_G   & 0xFF00u,     //  0x2247          <G>
  Key_S_H   & 0xFF00u,     //  0x2348          <H>
  Key_S_I   & 0xFF00u,     //  0x1749          <I>
  Key_S_J   & 0xFF00u,     //  0x244A          <J>
  Key_S_K   & 0xFF00u,     //  0x254B          <K>
  Key_S_L   & 0xFF00u,     //  0x264C          <L>
  Key_S_M   & 0xFF00u,     //  0x324D          <M>
  Key_S_N   & 0xFF00u,     //  0x314E          <N>
  Key_S_O   & 0xFF00u,     //  0x184F          <O>
  Key_S_P   & 0xFF00u,     //  0x1950          <P>
  Key_S_Q   & 0xFF00u,     //  0x1051          <Q>
  Key_S_R   & 0xFF00u,     //  0x1352          <R>
  Key_S_S   & 0xFF00u,     //  0x1F53          <S>
  Key_S_T   & 0xFF00u,     //  0x1454          <T>
  Key_S_U   & 0xFF00u,     //  0x1655          <U>
  Key_S_V   & 0xFF00u,     //  0x2F56          <V>
  Key_S_W   & 0xFF00u,     //  0x1157          <W>
  Key_S_X   & 0xFF00u,     //  0x2D58          <X>
  Key_S_Y   & 0xFF00u,     //  0x1559          <Y>
  Key_S_Z   & 0xFF00u,     //  0x2C5A          <Z>
  Key_Lbr   & 0xFF00u,     //  0x1A5B          <[>
  Key_Bsl   & 0xFF00u,     //  0x2B5C          <\>
  Key_Rbr   & 0xFF00u,     //  0x1B5D          <]>
  Key_S_6   & 0xFF00u,     //  0x075E          <6 ^>
  Key_S_Min & 0xFF00u,     //  0x0C5F          <- _>
  Key_Grv   & 0xFF00u,     //  0x2960          <`>
  Key_A     & 0xFF00u,     //  0x1E61          <a>
  Key_B     & 0xFF00u,     //  0x3062          <b>
  Key_C     & 0xFF00u,     //  0x2E63          <c>
  Key_D     & 0xFF00u,     //  0x2064          <d>
  Key_E     & 0xFF00u,     //  0x1265          <e>
  Key_F     & 0xFF00u,     //  0x2166          <f>
  Key_G     & 0xFF00u,     //  0x2267          <g>
  Key_H     & 0xFF00u,     //  0x2368          <h>
  Key_I     & 0xFF00u,     //  0x1769          <i>
  Key_J     & 0xFF00u,     //  0x246A          <j>
  Key_K     & 0xFF00u,     //  0x256B          <k>
  Key_L     & 0xFF00u,     //  0x266C          <l>
  Key_M     & 0xFF00u,     //  0x326D          <m>
  Key_N     & 0xFF00u,     //  0x316E          <n>
  Key_O     & 0xFF00u,     //  0x186F          <o>
  Key_P     & 0xFF00u,     //  0x1970          <p>
  Key_Q     & 0xFF00u,     //  0x1071          <q>
  Key_R     & 0xFF00u,     //  0x1372          <r>
  Key_S     & 0xFF00u,     //  0x1F73          <s>
  Key_T     & 0xFF00u,     //  0x1474          <t>
  Key_U     & 0xFF00u,     //  0x1675          <u>
  Key_V     & 0xFF00u,     //  0x2F76          <v>
  Key_W     & 0xFF00u,     //  0x1177          <w>
  Key_X     & 0xFF00u,     //  0x2D78          <x>
  Key_Y     & 0xFF00u,     //  0x1579          <y>
  Key_Z     & 0xFF00u,     //  0x2C7A          <z>
  Key_S_Lbr & 0xFF00u,     //  0x1A7B          <[ {>
  Key_S_Bsl & 0xFF00u,     //  0x2B7C          <\ |>
  Key_S_Rbr & 0xFF00u,     //  0x1B7D          <] }>
  Key_S_Grv & 0xFF00u,     //  0x297E          <` ~>
  Key_C_BS  & 0xFF00u      //  0x0E7F          C <BS>      [RUB]
};


//  ------------------------------------------------------------------
//  Returns the scan code of an ASCII character

byte scancode(Key ch) {

  if(KCodAsc(ch) <= 127)
    return (byte)(scancode_table[KCodAsc(ch)] >> 8);
  return 0;
}


//  ------------------------------------------------------------------
//  Translate scancode for ASCII keys

Key keyscanxlat(Key k) {

  // Only translate ASCII keys
  if(KCodAsc(k) and (KCodAsc(k) <= 0x7F)) {

    // if scancode zero and ascii-code non-zero, it's a "explicit" character,
    // entered by ALT + <Numpad 0-9>, so don't change it
    //if(KCodScn(k) == 0x00)
    //  return k;

    // Check for certain ctrl-keys
    switch(KCodAsc(k)) {

      case 0x08:  // CtrlH or BackSpace                     23/0E
        if(KCodScn(k) == 0x0E)
          return k;
        else
          break;

      case 0x09:  // CtrlI or Tab                           17/0F
        if(KCodScn(k) == 0x0F)
          return k;
        else
          break;

      case 0x0A:  // CtrlJ or CtrlEnter or GreyCtrlEnter    24/1C/E0
      case 0x0D:  // CtrlM or Enter or GreyEnter            32/1C/E0
        if(KCodScn(k) == 0x1C)
          return k;
        else if(KCodScn(k) == 0xE0) {
          KCodScn(k) = 0x1C;  // Translate Numpad-Enter to main Enter
          return k;
        }
        else
          break;

      case 0x1B:  // Ctrl[ or Esc                           1A/01
        if(KCodScn(k) == 0x01)
          return k;
        else
          break;

      case 0x15: // CtrlU or Shift3 (on german keyboards)   16/04
        if(KCodScn(k) == 0x04)
          return k;
        break;
    }

    // Translate scancode of ASCII key to a known value
    return (Key)(scancode_table[KCodAsc(k)] | KCodAsc(k));
  }
  else {

    // Check if its the center key and translate it to '5'
    if(k == 0x4c00)
      k = (Key)(scancode_table['5'] | '5');

    // Check for extended key and fix it if necessary
    if(KCodAsc(k) == 0xE0)
      if(KCodScn(k))
        KCodAsc(k) = 0x00;
  }

  return k;
}


//  ------------------------------------------------------------------
//  The following table maps NT virtual keycodes to PC BIOS keyboard
//  values.  For each virtual keycode there are four possible BIOS
//  values: normal, shift, Ctrl, and ALT.  Key combinations that have
//  no BIOS equivalent have a value of -1, and are ignored.  Extended
//  (non-ASCII) key values have bit 8 set to 1 using the EXT macro.

#if defined(__WIN32__)

#define EXT(key)    ((key)|0x10000)
#define ISEXT(val)  ((val)&0x10000)
#define EXTVAL(val) ((val)&0xFF)

struct kbd {
  int keycode;            // virtual keycode
  int normal;             // BIOS keycode - normal
  int shift;              // BIOS keycode - Shift-
  int ctrl;               // BIOS keycode - Ctrl-
  int alt;                // BIOS keycode - Alt-
} __gkbd_nt2b_table [] =
{

//  ------------------------------------------------------------------
//  Virtual key   Normal      Shift       Control     Alt

  { VK_BACK,      0x0E08,     0x0E08,     0x0E7F,     EXT(14)  },
  { VK_TAB,       0x0F09,     EXT(15),    EXT(148),   EXT(165) },
  { VK_RETURN,    0x1C0D,     0x1C0D,     0x1C0A,     EXT(166) },
  { VK_ESCAPE,    0x011B,     0x011B,     0x011B,     EXT(1)   },
  { VK_SPACE,     0x20,       0x20,       0x20,       0x20,    },

  { '0',          '0',        ')',        -1,         EXT(129) },
  { '1',          '1',        '!',        -1,         EXT(120) },
  { '2',          '2',        '@',        EXT(3),     EXT(121) },
  { '3',          '3',        '#',        -1,         EXT(122) },
  { '4',          '4',        '$',        -1,         EXT(123) },
  { '5',          '5',        '%',        -1,         EXT(124) },
  { '6',          '6',        '^',        0x1E,       EXT(125) },
  { '7',          '7',        '&',        -1,         EXT(126) },
  { '8',          '8',        '*',        -1,         EXT(127) },
  { '9',          '9',        '(',        -1,         EXT(128) },

  { 'A',          'a',        'A',        0x01,       EXT(30)  },
  { 'B',          'b',        'B',        0x02,       EXT(48)  },
  { 'C',          'c',        'C',        0x03,       EXT(46)  },
  { 'D',          'd',        'D',        0x04,       EXT(32)  },
  { 'E',          'e',        'E',        0x05,       EXT(18)  },
  { 'F',          'f',        'F',        0x06,       EXT(33)  },
  { 'G',          'g',        'G',        0x07,       EXT(34)  },
  { 'H',          'h',        'H',        0x08,       EXT(35)  },
  { 'I',          'i',        'I',        0x09,       EXT(23)  },
  { 'J',          'j',        'J',        0x0A,       EXT(36)  },
  { 'K',          'k',        'K',        0x0B,       EXT(37)  },
  { 'L',          'l',        'L',        0x0C,       EXT(38)  },
  { 'M',          'm',        'M',        0x0D,       EXT(50)  },
  { 'N',          'n',        'N',        0x0E,       EXT(49)  },
  { 'O',          'o',        'O',        0x0F,       EXT(24)  },
  { 'P',          'p',        'P',        0x10,       EXT(25)  },
  { 'Q',          'q',        'Q',        0x11,       EXT(16)  },
  { 'R',          'r',        'R',        0x12,       EXT(19)  },
  { 'S',          's',        'S',        0x13,       EXT(31)  },
  { 'T',          't',        'T',        0x14,       EXT(20)  },
  { 'U',          'u',        'U',        0x15,       EXT(22)  },
  { 'V',          'v',        'V',        0x16,       EXT(47)  },
  { 'W',          'w',        'W',        0x17,       EXT(17)  },
  { 'X',          'x',        'X',        0x18,       EXT(45)  },
  { 'Y',          'y',        'Y',        0x19,       EXT(21)  },
  { 'Z',          'z',        'Z',        0x1A,       EXT(44)  },

  { VK_PRIOR,     EXT(73),    EXT(73),    EXT(132),   EXT(153) },
  { VK_NEXT,      EXT(81),    EXT(81),    EXT(118),   EXT(161) },
  { VK_END,       EXT(79),    EXT(79),    EXT(117),   EXT(159) },
  { VK_HOME,      EXT(71),    EXT(71),    EXT(119),   EXT(151) },
  { VK_LEFT,      EXT(75),    EXT(75),    EXT(115),   EXT(155) },
  { VK_UP,        EXT(72),    EXT(72),    EXT(141),   EXT(152) },
  { VK_RIGHT,     EXT(77),    EXT(77),    EXT(116),   EXT(157) },
  { VK_DOWN,      EXT(80),    EXT(80),    EXT(145),   EXT(160) },
  { VK_INSERT,    EXT(82),    EXT(82),    EXT(146),   EXT(162) },
  { VK_DELETE,    EXT(83),    EXT(83),    EXT(147),   EXT(163) },
  { VK_NUMPAD0,   '0',        EXT(82),    EXT(146),   -1       },
  { VK_NUMPAD1,   '1',        EXT(79),    EXT(117),   -1       },
  { VK_NUMPAD2,   '2',        EXT(80),    EXT(145),   -1       },
  { VK_NUMPAD3,   '3',        EXT(81),    EXT(118),   -1       },
  { VK_NUMPAD4,   '4',        EXT(75),    EXT(115),   -1       },
  { VK_NUMPAD5,   '5',        EXT(76),    EXT(143),   -1       },
  { VK_NUMPAD6,   '6',        EXT(77),    EXT(116),   -1       },
  { VK_NUMPAD7,   '7',        EXT(71),    EXT(119),   -1       },
  { VK_NUMPAD8,   '8',        EXT(72),    EXT(141),   -1       },
  { VK_NUMPAD9,   '9',        EXT(73),    EXT(132),   -1       },
  { VK_MULTIPLY,  '*',        '*',        EXT(150),   EXT(55)  },
  { VK_ADD,       '+',        '+',        EXT(144),   EXT(78)  },
  { VK_SUBTRACT,  '-',        '-',        EXT(142),   EXT(74)  },
  { VK_DECIMAL,   '.',        '.',        EXT(83),    EXT(147) },
  { VK_DIVIDE,    '/',        '/',        EXT(149),   EXT(164) },
  { VK_F1,        EXT(59),    EXT(84),    EXT(94),    EXT(104) },
  { VK_F2,        EXT(60),    EXT(85),    EXT(95),    EXT(105) },
  { VK_F3,        EXT(61),    EXT(86),    EXT(96),    EXT(106) },
  { VK_F4,        EXT(62),    EXT(87),    EXT(97),    EXT(107) },
  { VK_F5,        EXT(63),    EXT(88),    EXT(98),    EXT(108) },
  { VK_F6,        EXT(64),    EXT(89),    EXT(99),    EXT(109) },
  { VK_F7,        EXT(65),    EXT(90),    EXT(100),   EXT(110) },
  { VK_F8,        EXT(66),    EXT(91),    EXT(101),   EXT(111) },
  { VK_F9,        EXT(67),    EXT(92),    EXT(102),   EXT(112) },
  { VK_F10,       EXT(68),    EXT(93),    EXT(103),   EXT(113) },
  { VK_F11,       EXT(133),   EXT(135),   EXT(137),   EXT(139) },
  { VK_F12,       EXT(134),   EXT(136),   EXT(138),   EXT(140) },

  { -1,           -1,         -1,         -1,         -1       }  // THE END
};


//  ------------------------------------------------------------------

int gkbd_nt2bios(INPUT_RECORD& inp) {

  register int keycode = inp.Event.KeyEvent.wVirtualKeyCode;
  register int state   = inp.Event.KeyEvent.dwControlKeyState;
  register int ascii   = inp.Event.KeyEvent.uChar.AsciiChar;

  // Look up the virtual keycode in the table. Ignore unrecognized keys.

  kbd* k = &__gkbd_nt2b_table[0];
  while((keycode != k->keycode) and (k->keycode != -1))
    k++;
  if(k->keycode == -1) {  // value not in table
    int c = ascii;
    return c ? c : -1;
  }

  // Check the state of the shift keys. ALT has highest
  // priority, followed by Control, followed by Shift.
  // Select the appropriate table entry based on shift state.

  int c;
  if(state & (RIGHT_ALT_PRESSED | LEFT_ALT_PRESSED))
    c = k->alt;
  else if(state & (RIGHT_CTRL_PRESSED | LEFT_CTRL_PRESSED))
    c = k->ctrl;
  else if(state & SHIFT_PRESSED)
    c = k->shift;
  else {
    // If it is a letter key, use the ASCII value supplied
    // by NT to take into account the CapsLock state.
    if(isupper(keycode))
      c = ascii;
    else
      c = k->normal;
  }

  if(c != -1)
    if(ascii and not (state & LEFT_ALT_PRESSED))
      if(isalnum(keycode))
        return ascii;
  if(ISEXT(c))
    return EXTVAL(c) << 8;

  return c;
}

//  ------------------------------------------------------------------

bool is_numpad_key(const INPUT_RECORD& inp) {

  if(not (inp.Event.KeyEvent.dwControlKeyState & ENHANCED_KEY)) {
    switch(inp.Event.KeyEvent.wVirtualKeyCode) {
      case 0x0C:
      case VK_PRIOR:
      case VK_NEXT:
      case VK_END:
      case VK_HOME:
      case VK_LEFT:
      case VK_UP:
      case VK_RIGHT:
      case VK_DOWN:
      case VK_INSERT:
      case VK_DELETE:
        debug_printf("Numpad key\n");
        return true;
    }
  }
  return false;
}
#endif


//  ------------------------------------------------------------------
//  Get key stroke

Key kbxget_raw(int mode) {
//  mode - =0 - wait for key is pressed (returns code)
//         =1 - test if keystroke is available (returns code if YES,
//              otherwise returns 0)
//         =2 - return Shifts key status
  Key k;

  #if defined(__MSDOS__)

  if(gkbd.extkbd)
    mode |= 0x10;
  i86 cpu;
  cpu.r.h.ah = (byte)mode;
  cpu.genint(0x16);
  if(mode & 0x01)
    if(cpu.r.w.flags & 0x40)   // if ZF is set, no key is available
      return 0;
  k = (Key)cpu.r.w.ax;

  #elif defined(__OS2__)

  KBDKEYINFO kb;
  mode &= 0xF;
  if(mode == 0)
    KbdCharIn(&kb, IO_WAIT, 0);
  else if(mode == 2) {
    KbdPeek(&kb, 0);
    if(kb.fbStatus)
      return (Key)(kb.fsState & (RSHIFT|LSHIFT|CTRL|ALT));
    else
      return 0;
  }
  else {
    KbdPeek(&kb, 0);
    if(!(kb.fbStatus & 0x40))
      return 0;
  }
  KCodScn(k) = kb.chScan;
  KCodAsc(k) = kb.chChar;
  if(0x000 == KCodKey(k))
    return KEY_BRK;
  if(0xE0 == KCodScn(k))
    KCodScn(k) = 0x1C;
  else {
    if(0xE0 == KCodAsc(k))
      if(KCodScn(k))
        KCodAsc(k) = 0x00;
  }

  #elif defined(__WIN32__)

  INPUT_RECORD inp;
  DWORD nread;

  if(mode == 2) {
    return 0;
  }
  else if(mode & 0x01) {

    // Peek at next key
    k = 0;
    PeekConsoleInput(gkbd_hin, &inp, 1, &nread);
    if(nread) {
      if((inp.EventType & KEY_EVENT) and inp.Event.KeyEvent.bKeyDown) {
        int kc = gkbd_nt2bios(inp);
        if((kc != -1) or (inp.Event.KeyEvent.wVirtualKeyCode == 0xBA)) {
          k = (Key)kc;
          return k;
        }
      }
      // Discard other events
      ReadConsoleInput(gkbd_hin, &inp, 1, &nread);
    }
  }
  else {

    // Get next key
    inp.Event.KeyEvent.bKeyDown = false;
    while(1) {

      PeekConsoleInput(gkbd_hin, &inp, 1, &nread);
      if(not nread) {
        WaitForSingleObject(gkbd_hin, 1000);
        continue;
      }

      if(inp.EventType == KEY_EVENT and inp.Event.KeyEvent.bKeyDown) {
        bool alt_pressed = (inp.Event.KeyEvent.dwControlKeyState & (LEFT_ALT_PRESSED|RIGHT_ALT_PRESSED)) ? true : false;
        bool right_alt_pressed = (inp.Event.KeyEvent.dwControlKeyState & RIGHT_ALT_PRESSED) ? true : false;
        bool enhanced_key = (inp.Event.KeyEvent.dwControlKeyState & ENHANCED_KEY) ? true : false;
        bool numpad_key = is_numpad_key(inp);
        int vk = inp.Event.KeyEvent.wVirtualKeyCode;
        char raw_ch = inp.Event.KeyEvent.uChar.AsciiChar;
        int kc;
        int test;
        char ch;

        debug_printf2("vk=0x%02X, raw=0x%02X", vk, (int)raw_ch);
        if(enhanced_key and (raw_ch == 0xE0))
          inp.Event.KeyEvent.uChar.AsciiChar = raw_ch = 0;
        debug_printf1(", kc=0x%04X\n", gkbd_nt2bios(inp));
        debug_cond_printf(enhanced_key, "Enhanced key\n");
        debug_cond_printf(inp.Event.KeyEvent.uChar.AsciiChar, "AsciiChar\n");
        debug_cond_printf(alt_pressed, "alt_pressed\n");
        debug_cond_printf(right_alt_pressed, "right_alt_pressed\n");
        if(gkbd_nt)
          test = (inp.Event.KeyEvent.uChar.AsciiChar and not alt_pressed and vk != -1) or (alt_pressed and vk == -1) or (alt_pressed and numpad_key);
        else
          test = (inp.Event.KeyEvent.uChar.AsciiChar and not alt_pressed and vk != -1) or (alt_pressed and vk == -1) or (alt_pressed and numpad_key) or (vk == 0xBA);
        if(test) {
          // Ascii char
          if(gkbd_nt and not (alt_pressed and numpad_key)) {
            ch = raw_ch;
            debug_printf("Before ReadConsoleInput\n");
            ReadConsoleInput(gkbd_hin, &inp, 1, &nread);
            debug_printf("After  ReadConsoleInput\n");
          }
          else {
            debug_printf("Before ReadConsole\n");
            ReadConsole(gkbd_hin, &ch, 1, &nread, NULL);
            debug_printf("After  ReadConsole\n");
          }
          debug_printf2("VK=0x%02X, ch=0x%02X\n", vk, (int)ch);
          if(alt_pressed) {
            k = (Key)ch;
            debug_printf("---break 2\n");
            break;
          }
          if(gkbd_nt) {
            if(ch == '\x5E' or ch == '\x7E' or ch == '\x60' or ch == '\xF9' or ch == '\xEF')
              inp.Event.KeyEvent.wVirtualKeyCode = (word)(ch << 8);
          }
          else {
            if(ch == '\x5E' or ch == '\x7E' or ch == '\x60' or ch == '\x27')
              inp.Event.KeyEvent.wVirtualKeyCode = (word)(ch << 8);
          }
          inp.Event.KeyEvent.uChar.AsciiChar = ch;
        }
        else {
          // Control keycode
          debug_printf("Before ReadConsoleInput1\n");
          ReadConsoleInput(gkbd_hin, &inp, 1, &nread);
          debug_printf("After  ReadConsoleInput1\n");
        }
        kc = gkbd_nt2bios(inp);
        if(kc != -1) {
          k = (Key)kc;
          debug_printf("---break 1\n");
          break;
        }
      }
      else {
        // Discard other events
        debug_printf("Before ReadConsoleInput2\n");
        ReadConsoleInput(gkbd_hin, &inp, 1, &nread);
        debug_printf("After  ReadConsoleInput2\n");
      }
    }
  }

  #elif defined(__UNIX__)

  if(mode == 2) {
    return 0;
  }
  else if(mode & 0x01) {

    // Peek at next key
    return gkbd_input_pending() ? 0xFFFF : 0;
  }
  else {

    k = gkbd_getmappedkey();
  }

  #endif

  return k;
}


//  ------------------------------------------------------------------
//  Get key stroke

Key kbxget(int mode) {

  return keyscanxlat(kbxget_raw(mode));
}


//  ------------------------------------------------------------------
//  Returns keycode of waiting key or zero if none

Key kbxhit() {

  return kbxget(0x01);
}


//  ------------------------------------------------------------------
//  Clears internal keyboard buffer

void kbclear() {

  while(gkbd.kbuf != NULL) {

    KBuf *kbuf = gkbd.kbuf->next;
    throw_free(gkbd.kbuf);
    gkbd.kbuf = kbuf;
  }
}


//  ------------------------------------------------------------------
//  Clear keyboard buffer

void clearkeys() {

  while(kbxhit())
    kbxget(0x00);
}


//  ------------------------------------------------------------------
//  Puts a keystroke into the CXL keyboard "buffer"

int kbput(Key xch) {

  KBuf* kbuf;
  KBuf* temp;

  // allocate space for another keypress record
  kbuf=(KBuf*)throw_malloc(sizeof(KBuf));

  // find last record in linked list
  if((temp=gkbd.kbuf)!=NULL)
    for(;temp->next!=NULL;temp=temp->next);

  // add new record to end of linked list
  kbuf->next=NULL;
  kbuf->prev=temp;
  if(temp != NULL)
    temp->next=kbuf;

  // add keypress info to new record
  kbuf->xch=xch;

  // if kbuf pointer was NULL, point it to new record
  if(gkbd.kbuf == NULL)
    gkbd.kbuf=kbuf;

  // return normally
  return 0;
}


//  ------------------------------------------------------------------
//  Put keys into the real keyboard buffer

Key kbput_(Key xch) {

  #if defined(__MSDOS__)

  #define BufStart (word)peek(0x40,0x80)
  #define BufEnd   (word)peek(0x40,0x82)
  #define BufHead  (word)peek(0x40,0x1A)
  #define BufTail  (word)peek(0x40,0x1C)
  #define BufTail_(a) poke(0x40,0x1C,(word)(a))

  word OldBufTail;

  OldBufTail = BufTail;
  if(BufTail == BufEnd-2)
    BufTail_(BufStart);
  else
    BufTail_(BufTail+2);

  if(BufTail == BufHead)
    BufTail_(OldBufTail);
  else {
    poke(0x40, OldBufTail, xch);
  }

  #endif

  return xch;
}


//  ------------------------------------------------------------------
//  Put keys into the real keyboard buffer

void kbputs_(char* str) {

  char* p;

  for(p=str; *p ;p++)
    kbput_(Key((scancode(*p)<<8)|*p));
}


//  ------------------------------------------------------------------
//  Change defined "on-key" list pointer

KBnd* chgonkey(KBnd* list) {

  KBnd* temp;

  temp = gkbd.onkey;
  gkbd.onkey = list;

  return temp;
}


//  ------------------------------------------------------------------
//  Frees all active onkey definitions from memory

void freonkey() {

  KBnd* temp;

  // free all onkey records in linked list
  while(gkbd.onkey!=NULL) {
    temp = gkbd.onkey->prev;
    throw_free(gkbd.onkey);
    gkbd.onkey = temp;
  }
}


//  ------------------------------------------------------------------
//  Attaches/detaches a key to a function

int setonkey(Key keycode, VfvCP func, Key pass) {

  // search for a keycode that is already defined
  KBnd* onkey = gkbd.onkey;
  while(onkey) {
    if(onkey->keycode == keycode)
      break;
    onkey = onkey->prev;
  }

  // check to see if a key detachment is being requested
  if(func == NULL) {

    // if no defined onkey was found, then error
    if(onkey == NULL)
      return 2;

    // delete record from linked list
    KBnd* prev = onkey->prev;
    KBnd* next = onkey->next;
    if(prev)
      prev->next = next;
    if(next)
      next->prev = prev;
    if(onkey == gkbd.onkey)
      gkbd.onkey = prev;

    // free memory allocated for deleted record
    throw_free(onkey);
  }
  else {

    // if key was found, change func pointer
    // otherwise create a new onkey record
    if(onkey)
      onkey->func = func;
    else {

      // allocate memory for new record
      onkey = (KBnd*)throw_malloc(sizeof(KBnd));
      if(onkey == NULL)
        return 1;

      // add new record to linked list
      if(gkbd.onkey)
        gkbd.onkey->next = onkey;
      onkey->prev = gkbd.onkey;
      onkey->next = NULL;
      gkbd.onkey = onkey;

      // save info in onkey record
      gkbd.onkey->keycode = keycode;
      gkbd.onkey->func = func;
      gkbd.onkey->pass = pass;
    }
  }

  // return normally
  return 0;
}


//  ------------------------------------------------------------------

Key key_tolower(Key __keycode) {

  if(isupper(KCodAsc(__keycode)))
    return (Key)(__keycode + 'a' - 'A');
  return __keycode;
}


//  ------------------------------------------------------------------

#if 0

// Tick counter
long __kbxget_ticks = 0;

//  ------------------------------------------------------------------
//  Get key with timer tick support

Key __kbxget(int __mode, long __ticks, VfvCP __idlefunc) {

  // Convert milliseconds to timer ticks (18.2/sec)
  __ticks = (__ticks*1000L)/54945L;

  // The system timer tick interrupt number
  const int G_CLOCK_TICK = 0x1C;

  // Install a tick handler if a tick is requested
  if(__ticks) {
    __kbxget_ticks = __ticks;
    __kbxget_oldint = getvect(G_CLOCK_TICK);
    setvect(G_CLOCK_TICK, __kbxget_handler);
  }

  Key _keyval = 0;
  int _idle = 0;

  // Idle loop
  do {

    // Call the regular key-get function
    _keyval = kbxget(__mode);

    // Is the key a tick and there is an idle function?
    _idle = (_keyval == Key_Tick) and __idlefunc;

    // Call the idle function if we are idle
    if(_idle) {
      (*__idlefunc)();

      // Reset ticks
      __kbxget_ticks = __ticks;
    }

  } while(_idle);

  // Uninstall the tick handler
  if(__ticks)
    setvect(G_CLOCK_TICK, __kbxget_oldint);

  // Return key value
  return _keyval;
}

#endif


//  ------------------------------------------------------------------
//  Thread that waits for a key and sets a semaphore when key is read

#if 0

void __kbxget_thread(void* threadno) {

  NW(threadno);

  while(1) {

    KBDKEYINFO kb;
    KbdCharIn(&kb, IO_WAIT, 0);

    Key k;
    KCodScn(k) = kb.chScan;
    KCodAsc(k) = kb.chChar;
    if(k == 0x0000)
      k = Key_C_Brk;
    else {
      if(0xE0 == KCodScn(k))
        KCodScn(k) = 0x1C;
      else {
        if(0xE0 == KCodAsc(k))
          if(KCodScn(k))
            KCodAsc(k) = 0x00;
      }
    }
  }
}

#endif


//  ------------------------------------------------------------------

