
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gkbdbase.h,v 1.2 1999/01/29 22:46:08 leonid Exp $
//  ------------------------------------------------------------------
//  Keyboard functions.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GKBDBASE_H
#define __GKBDBASE_H


//  ------------------------------------------------------------------
//  Required headers

#include <gtimall.h>

//  ------------------------------------------------------------------
//  Simple types

typedef word Key;
typedef Key gkey;


//  ------------------------------------------------------------------
//  Keycode object

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif

union KCod {
  gkey    ckey;               // KCodKey() : Word overlay
  struct {
    byte asc;                // KCodAsc() : ASCII char | 0=ext
    byte scn;                // KCodScn() : Scancode
  } c;
};

#define KCodKey(x) (CAST(KCod,x).ckey)
#define KCodAsc(x) (CAST(KCod,x).c.asc)
#define KCodScn(x) (CAST(KCod,x).c.scn)

#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------
//  Definition of kbuf record

struct KBuf {
  KBuf* prev;         // previous record
  KBuf* next;         // next record
  gkey   xch;          // keypress
};


//  ------------------------------------------------------------------
//  Definition of onkey record

struct KBnd {
  KBnd* prev;         // pointer to previous record
  KBnd* next;         // pointer to next record
  gkey   keycode;      // Scan/ASCII code of trap key
  VfvCP func;         // address of onkey function
  gkey   pass;         // key to pass back, 0=don't pass
};


//  ------------------------------------------------------------------
//  Definition of keyboard info record

class GKbd {

public:

  KBuf*   kbuf;           // Pointer to head record in key buffer
  KBnd*   onkey;          // Pointer to head record in onkey list
  KBnd*   curronkey;      // Pointer to current onkey record
  int    inmenu;         // In-menu flag used by menuing functions
  int    source;         // Source of keypress 0=kb, 1=kbuf, 2=mouse
  int    extkbd;         // Extended keyboard 0=none, 1=yes
  int    polling;        // Keyboard polling enabled
  Clock  tickinterval;   // Minimum interval between ticks
  Clock  tickvalue;      // Value from last tick
  VfvCP   tickfunc;       // Function to call when a tick is generated
  Clock  tickpress;      // Tick value at last keypress
  bool   inidle;         // In-idle flag used by tickfunc
  bool   quitall;        // Quit-all flag for menus etc.

  void Init();
  GKbd();
  ~GKbd();
};

extern GKbd __FAR gkbd;


//  ------------------------------------------------------------------
//  Keyboard status codes returned from kbstat()

#define RSHIFT      1       // right shift pressed
#define LSHIFT      2       // left shift pressed
#define CTRL        4       // [Ctrl] pressed
#define ALT         8       // [Alt] pressed
#define SCRLOCK     16      // [Scroll Lock] toggled
#define NUMLOCK     32      // [Num Lock] toggled
#define CAPSLOCK    64      // [Caps Lock] toggled
#define INS         128     // [Ins] toggled


//  ------------------------------------------------------------------

extern gkey __FAR scancode_table[];


//  ------------------------------------------------------------------
//  Function prototypes

KBnd* chgonkey  (KBnd* kblist);
void  clearkeys ();
void  freonkey  ();
int   setonkey  (gkey keycode, VfvCP func, gkey pass);
gkey  getxch    (int __tick=false);
void  kbclear   ();
gkey  kbmhit    ();
gkey  kbxget    (int mode=0);
gkey  kbxhit    ();
int   kbput     (gkey xch);
word  kbput_    (gkey xch);
void  kbputs_   (char* str);
byte  scancode  (gkey ch);
gkey  waitkey   ();
gkey  waitkeyt  (int duration);

gkey  key_tolower(gkey __keycode);

gkey  keyscanxlat(gkey k);

gkey  __kbxget(int __mode=0, long __ticks=0, VfvCP __idlefunc=NULL);

void gkbdtickpressreset();
void gkbdtickvaluereset();


//  ------------------------------------------------------------------
//  Inline functions

inline gkey getxchtick()  { return getxch(true); }
inline void kbdsettickfunc(VfvCP func)  { gkbd.tickfunc = func; }


//  ------------------------------------------------------------------
//  Shorthand definitions of keyboard scancodes

#define  KEY_BRK  0xFFFF     // ^Break return from _KeyHit()/_KeyGet()


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

