
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gkbdsets.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Key command and macro sets.
//  ------------------------------------------------------------------


#include <gkbdbase.h>
#include <gvector.h>


//  ------------------------------------------------------------------

#ifndef __GKBDSETS_H
#define __GKBDSETS_H


//  ------------------------------------------------------------------
//  Keyboard commands

class gkeycommand {

public:

  gkey keycode;
  gkey command;

  gkeycommand() { }
  gkeycommand(gkey k, gkey c) { keycode = k; command = c; }

  void strip_scancode() { keycode &= 0x00FF; }

  gkeycommand& operator=(const gkeycommand& a) { keycode = a.keycode; command = a.command; return *this; }

  void load(gfile& f);
  void save(gfile& f) const;

};


//  ------------------------------------------------------------------
//  Keyboard macros

class gkeymacro {

public:

  enum {
    max_commands = 128
  };

  gkey keycode;
  gkey commands[max_commands];

  gkeymacro() { }
  gkeymacro(gkey k) { keycode = k; }

  void strip_scancode() { keycode &= 0x00FF; }

  gkeymacro& operator=(const gkeymacro& a);

  void load(gfile& f);
  void save(gfile& f) const;

};


//  ------------------------------------------------------------------
//  Sets of key commands and macros

class gkeyset {

protected:

  gvector<gkeycommand> command;
  gvector<gkeymacro>   macro;

public:

  gkeycommand* bsearch_command(gkey find);
  gkeymacro*   bsearch_macro(gkey find);

  void add_command(const gkeycommand& c);
  void add_macro(const gkeymacro& m);

  void load(gfile& f) { gvector_load(f, command); gvector_load(f, macro); }
  void save(gfile& f) const { gvector_save(f, command); gvector_save(f, macro); }

};


//  ------------------------------------------------------------------

int compare_gkeycommand(const gkeycommand* a, const gkeycommand* b);
int compare_gkeymacro(const gkeymacro* a, const gkeymacro* b);

gkey gkeyname_to_keycode(word crc);


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

