
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: glang2.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Language handling.
//  ------------------------------------------------------------------


#include <gcrcall.h>
#include <gmemdbg.h>

#include <glang.h>


//  ------------------------------------------------------------------

glang::glang() {

  lang = NULL;
  langmax = 0;
}


//  ------------------------------------------------------------------

glang::~glang() {

  reset();
}


//  ------------------------------------------------------------------

void glang::reset() {

  glangcrc* l = lang;
  for(int n=0; n<langmax; n++,l++) {
    if(l->allocated())
      throw_xrelease(l->str);
  }
}


//  ------------------------------------------------------------------

int glang::crc2no(word crc) {

  glangcrc* l = lang;
  for(int n=0; n<langmax; n++,l++) {
    if(l->crc == crc)
      return n;
  }
  return -1;
}


//  ------------------------------------------------------------------

void glang::change(int n, const char* s) {

  glangcrc* l = lang + n;
  if(l->allocated())
    throw_xfree(l->str);
  l->str = throw_xstrdup(s);
}


//  ------------------------------------------------------------------

void glang::save(gfile& f) {

  int n;
  glangcrc* l = lang;
  for(n=0; n<langmax; n++,l++) {
    if(l->allocated()) {
      char* s = l->str;
      int len = strlen(s)+1;
      f << n;
      f << len;
      f.fwrite(s, len);
    }
  }
  n = -1;
  f << n;
}


//  ------------------------------------------------------------------

void glang::load(gfile& f) {
  
  int n, len;
  do {
    f >> n;
    if(n != -1) {
      glangcrc* l = lang + n;
      f >> len;
      char* s = (char*)throw_xmalloc(len);
      f.fread(s, len);
      if(l->allocated())
        throw_xfree(l->str);
      l->str = s;
    }
  } while(n != -1);
}


//  ------------------------------------------------------------------

bool glang::switch_language(word crc, char* str) {

  strschg(str, "\\n", "\n");
  strschg(str, "\\r", "\r");
  strschg(str, "\\\"", "\"");

  int item = crc2no(crc);
  if(item != -1) {
    change(item, str);
    return true;
  }

  return false;
}


//  ------------------------------------------------------------------

bool glang::load_language(const char* file) {

  gfile fp(file, "rt");
  if(fp) {

    fp.setvbuf();

    char buf[1024];
    while(fp.fgets(buf, sizeof(buf))) {
      char* val = strskip_wht(buf);
      if(isalpha(*val)) {
        word crc = getvalcrc(val);
        switch_language(crc, val);
      }
    }

    fp.fclose();

    return true;
  }

  return false;
}


//  ------------------------------------------------------------------

