
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: glog.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Log file class.
//  ------------------------------------------------------------------


#include <stdio.h>
#include <stdarg.h>
#include <gdbgerr.h>
#include <gfilutil.h>
#include <gstrall.h>
#include <glog.h>


//  ------------------------------------------------------------------

struct tm* glog::time_now;
int        glog::count = 0;
time_t     glog::secs_now;
char       glog::timebuf[20];


//  ------------------------------------------------------------------

glog::glog() {

  bufsize = 0;
  logtype = GLOG_FD;
  progname = NULL;
  status = GOLD_NO_ERROR;
  storelines = -1;
}


//  ------------------------------------------------------------------

glog::~glog() {

  close();
}


//  ------------------------------------------------------------------

int glog::open(const char* filename, const char* name, const char* shortname, int type, uint bufsz, int shflag) {

  fp.fopen(filename, "at", shflag);
  if(fp.status) {
    status = fp.status;
    return status;
  }

  count++;
  bufsize = bufsz;
  fp.setvbuf(NULL, bufsize ? _IOFBF : _IONBF, bufsize);

  init(name, shortname, type);

  return status;
}


//  ------------------------------------------------------------------

void glog::close() {

  fp.fclose();
  count--;
}


//  ------------------------------------------------------------------

void glog::init(const char* name, const char* shortname, int type) {

  lineswritten = 0;
  logtype = (type!=GLOG_NONE) ? type : logtype;
  progname = name ? name : progname;
  shortprogname = shortname ? shortname : (shortprogname ? shortprogname : progname);
}


//  ------------------------------------------------------------------

void _stdc glog::printf(const char* format, ...) {

  va_list argptr;
  char buf[256];
  char logbuf[256];

  gtime((ulong*)&secs_now);
  time_now = (Tm*)glocaltime((ulong*)&secs_now);

  lineswritten++;

  if(lineswritten == 1) {

    switch(logtype) {

      case GLOG_FD:
        strftimei(timebuf, 20, "%a %d %b %y", time_now);
        sprintf(logbuf, "\n----------  %s, %s\n", timebuf, progname);
        break;

      case GLOG_MAX:
        strftimei(timebuf, 20, "%d %b %H:%M:%S", time_now);
        sprintf(logbuf, "\n+ %s %4.4s Begin, %s\n", timebuf, shortprogname, progname);
        break;

      case GLOG_BINK:
        strftimei(timebuf, 20, "%d-%b %H:%M:%S", time_now);
        sprintf(logbuf, "\n> %s %4.4s %s\n", timebuf, shortprogname, progname);
        break;

      case GLOG_QBBS:
        strftimei(timebuf, 20, "%d-%b-%y %H:%M", time_now);
        sprintf(logbuf, "\n%s  **************************************************\n%s  %s\n", timebuf, timebuf, progname);
        break;

      case GLOG_DB:
        strftimei(timebuf, 20, "%m/%d/%y %H:%M", time_now);
        sprintf(logbuf, "\n%s  %s\n", timebuf, progname);
        break;
    }

    if(fp.isopen())
      fp.printf("%s", logbuf);
  }

  *buf = NUL;
  va_start(argptr, format);
  vsprintf(buf, format, argptr);
  va_end(argptr);

  if(*buf == '!')
    store();

  switch(logtype) {

    case GLOG_FD:
      strftimei(timebuf, 10, "%H:%M:%S", time_now);
      sprintf(logbuf, "%c %s  %s", *buf, timebuf, buf+2);
      break;

    case GLOG_MAX:
      strftimei(timebuf, 20, "%d %b %H:%M:%S", time_now);
      sprintf(logbuf, "%c %s %4.4s %s", *buf, timebuf, shortprogname, buf+2);
      break;

    case GLOG_BINK:
      strftimei(timebuf, 20, "%d-%b %H:%M:%S", time_now);
      sprintf(logbuf, "%c %s %4.4s %s", *buf, timebuf, shortprogname, buf+2);
      break;

    case GLOG_QBBS:
      strftimei(timebuf, 20, "%d-%b-%y %H:%M", time_now);
      sprintf(logbuf, "%s  %s", timebuf, buf+2);
      break;

    case GLOG_DB:
      strftimei(timebuf, 20, "%m/%d/%y %H:%M", time_now);
      sprintf(logbuf, "%s  %s", timebuf, buf+2);
      break;
  }
  if(fp.isopen())
    fp.printf("%s\n", logbuf);
  if(storelines != -1) {
    if(storelines < GLOG_STORELINES)
      strxcpy(storeline[storelines], logbuf, 79);
    storelines++;
  }
}


//  ------------------------------------------------------------------

void glog::store() {

  if(storelines == -1)
    storelines = 0;
}


//  ------------------------------------------------------------------

