
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmd5.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//
//  Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
//  rights reserved.
//
//  License to copy and use this software is granted provided that it
//  is identified as the "RSA Data Security, Inc. MD5 Message-Digest
//  Algorithm" in all material mentioning or referencing this software
//  or this function.
//
//  License is also granted to make and use derivative works provided
//  that such works are identified as "derived from the RSA Data
//  Security, Inc. MD5 Message-Digest Algorithm" in all material
//  mentioning or referencing the derived work.
//
//  RSA Data Security, Inc. makes no representations concerning either
//  the merchantability of this software or the suitability of this
//  software for any particular purpose. It is provided "as is"
//  without express or implied warranty of any kind.
//
//  These notices must be retained in any copies of any part of this
//  documentation and/or software.
//
//  ------------------------------------------------------------------
//  This C++ port is Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------


#ifndef __GMD5_H
#define __GMD5_H

#include <gdefs.h>


//  ------------------------------------------------------------------

class gmd5 {

protected:

  static char padding[64];

  dword state[4];        // state (ABCD)
  dword count[2];        // number of bits, modulo 2^64 (lsb first)
  char buffer[64];       // input buffer

  dword calc_f(dword x, dword y, dword z);
  dword calc_g(dword x, dword y, dword z);
  dword calc_h(dword x, dword y, dword z);
  dword calc_i(dword x, dword y, dword z);

  dword rotate_left(dword x, int n);

  void calc_ff(dword& a, dword b, dword c, dword d, dword x, int s, dword ac);
  void calc_gg(dword& a, dword b, dword c, dword d, dword x, int s, dword ac);
  void calc_hh(dword& a, dword b, dword c, dword d, dword x, int s, dword ac);
  void calc_ii(dword& a, dword b, dword c, dword d, dword x, int s, dword ac);

  void transform(const char* block);

  void encode(char* output, const dword* input, uint len);
  void decode(dword* output, const char* input, uint len);

public:

  char digest[16];

  gmd5();

  void begin();
  void update(ulong input);
  void update(const char* input);
  void update(const char* input, uint inputlen);
  void end();

  int operator==(gmd5& m);
  int operator!=(gmd5& m) { return !operator==(m); }

  char operator[](uint i) const { return digest[i]; }
  const char* operator()() { return digest; }
};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

