
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmemall.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Memory handling.
//  ------------------------------------------------------------------


#ifndef __GMEMALL_H
#define __GMEMALL_H


//  ------------------------------------------------------------------

//#include <dos.h>
#include <malloc.h>
#include <string.h>
#include <gdbgerr.h>
#if defined(__MSDOS__)
#include <gmemi86.h>
#endif


//  ------------------------------------------------------------------

inline void* mallox(size_t __items, size_t __size, size_t __cache=100) {

  return malloc((__items+__cache)*__size);
}

//  ------------------------------------------------------------------

inline void* callox(size_t __items, size_t __size, size_t __cache=100) {

  return calloc(__items+__cache, __size);
}


//  ------------------------------------------------------------------

inline void* reallox(void* __ptr, size_t __items, size_t __size, size_t __cache=100) {

  return ((__items % __cache) and __ptr) ? __ptr : realloc(__ptr, (__items+__cache)*__size);
}


//  ------------------------------------------------------------------

#if defined(__MSDOS__)
#if defined(__WATCOMC__)

inline short peek(unsigned segment, unsigned offset) { return *((short __far *)MK_FP(segment,offset)); }
inline char  peekb(unsigned segment, unsigned offset) { return *((char __far *)MK_FP(segment,offset)); }
inline void  poke(unsigned segment, unsigned offset, short value) { *((short __far *)MK_FP(segment,offset)) = value; }
inline void  pokeb(unsigned segment, unsigned offset, char value) { *((char __far *)MK_FP(segment,offset)) = value; }

#elif defined(__DJGPP__)

#include <go32.h>
#include <sys/farptr.h>

inline short peek (unsigned segment, unsigned offset) { return _farpeekw(_dos_ds, segment*16 + offset); }
inline char  peekb(unsigned segment, unsigned offset) { return _farpeekb(_dos_ds, segment*16 + offset); }
inline void  poke (unsigned segment, unsigned offset, short value) { _farpokew(_dos_ds, segment*16 + offset, value); }
inline void  pokeb(unsigned segment, unsigned offset, char value)  { _farpokeb(_dos_ds, segment*16 + offset, value); }

#endif
#endif


//  ------------------------------------------------------------------

#define HEX_DUMP1 1
#define HEX_DUMP2 2

char* HexDump16(char* strbuf, const char* memptr, int limit, const char* fmt, int fmtno=0);
inline char* HexDump16(char* strbuf, const char* memptr, int limit, int fmtno) { return HexDump16(strbuf, memptr, limit, NULL, fmtno); }

class gstring;

gstring& addhexdump(gstring& dump, const void* data, size_t datalen, const char* lineterm);


//  ------------------------------------------------------------------

void* memdup(void* src, uint size);

#if defined(__OS2_) or defined(__WATCOMC__)
#define farcalloc(num,size)       calloc(num,size)
#define farmalloc(num)            malloc(num)
#define farrealloc(buffer,size)   realloc(buffer,size)
#define farfree(buff)             free(buff)
#endif


//  ------------------------------------------------------------------
//  EMS functions

#if defined(__MSDOS__) and not defined(__386__)

int   emsalloc(int numpages);
int   emsdealloc(int emmhandle);
int   emsexist();
word  emsframe();
word  emsfree();
int   emsmap(int emmhandle, int logpage, int phypage);
int   emsread(byte* dest, word emsofs, word numbytes);
word  emstotal();
char* emsver();
int   emswrite(byte* src, word emsofs, word numbytes);


//  ------------------------------------------------------------------
//  XMS functions

extern "C" {

  // A 3rd party lib is used for the actual funcs
  #include <gmemxms.h>
}

//  ------------------------------------------------------------------
//  XMS handling class

class GOLDXMS {

  protected:  //  ----------------------------------------------------

    int                  installed;
    word                  handle;
    word                  freemem;
    word                  totmem;
    struct EMMMoveStruct  movebuf;

  public:   //  ------------------------------------------------------

    GOLDXMS();

    int errorcode;

    int FreeMem();

    int Alloc(int kb);
    int Free(word hdl=0);

    int CopyTo(char* src, word idx, word size, word hdl=0);
    int CopyFrom(char* dest, word idx, word size, word hdl=0);

};


//  ------------------------------------------------------------------

inline GOLDXMS::GOLDXMS() {

  handle = 0;
  errorcode = 0;

  installed = XMS_Setup();
}


//  ------------------------------------------------------------------

inline int GOLDXMS::FreeMem() {

  errorcode = XMS_FreeMem(&freemem, &totmem);

  return freemem;
}


//  ------------------------------------------------------------------

inline int GOLDXMS::Alloc(int kb) {

  errorcode = XMS_AllocEMB(kb, &handle);

  return errorcode ? 0 : handle;
}


//  ------------------------------------------------------------------

inline int GOLDXMS::Free(word hdl) {

  errorcode = XMS_FreeEMB(hdl ? hdl : handle);

  return errorcode;
}


//  ------------------------------------------------------------------

inline int GOLDXMS::CopyTo(char* src, word idx, word size, word hdl) {

  movebuf.TransferLength = size + ((size%2) ? 1 : 0);
  movebuf.SourceHandle   = 0;
  movebuf.SourceOffset   = (long)src;
  movebuf.DestHandle     = hdl ? hdl : handle;
  movebuf.DestOffset     = idx;

  errorcode = XMS_MoveEMB(&movebuf);

  return errorcode;
}


//  ------------------------------------------------------------------

inline int GOLDXMS::CopyFrom(char* dest, word idx, word size, word hdl) {

  movebuf.TransferLength = size + ((size%2) ? 1 : 0);
  movebuf.SourceHandle   = hdl ? hdl : handle;
  movebuf.SourceOffset   = idx;
  movebuf.DestHandle     = 0;
  movebuf.DestOffset     = (long)dest;

  errorcode = XMS_MoveEMB(&movebuf);

  return errorcode;
}

#endif


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

