
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmemblks.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Memory blocks.
//  ------------------------------------------------------------------


#ifndef __GMEMBLKS_H
#define __GMEMBLKS_H


//  ------------------------------------------------------------------

#include <gstrall.h>


//  ------------------------------------------------------------------

class gfile;


//  ------------------------------------------------------------------

class gmemblockview {

public:

  uint first;
  uint last;

  gmemblockview()                  { first = last = 0; }
  gmemblockview(uint f, uint l)    { first=f; last=l; }
  gmemblockview(gmemblockview* x)  { first=x->first; last=x->last; }

  bool empty()                      { return first == last; }

};


//  ------------------------------------------------------------------

class gmemblock {

protected:

  struct lineindex {
    uint offset;
    uint length;
  };

  char*       block;
  char*       blockidx;
  uint        blocklen;
  int         blocklines;
  uint        blocksize;
  uint        blockreserved;
  uint        alloclen;
  int         alloclines;
  int         lineextra;
  int         linesize;
  int         linelen;

  gmemblockview** view;
  int             views;

  void        adjust_block(uint len);
  lineindex*  adjust_index(int lines);
  void        adjust_offsets(int index, int difference);

  char*       blockidxptr(int n) const               { return blockidx + (linelen * n); }
  lineindex&  blockindex(int n) const                { return *((lineindex*)blockidxptr(n)); }

  char*       lineextraptr(int n) const              { return blockidxptr(n) + sizeof(lineindex); }

public:

  gmemblock(int block_size=4096, int line_size=2048, int line_extra=0);
  ~gmemblock();

  void        reset(bool init=false);

  void        reserve(uint n, int l=0)                { adjust_block(n); if(l) adjust_index(l); blockreserved=n; }

  int         size() const                            { return blocklines; }
  bool        empty() const                           { return size() == 0; }
  uint        length(int n) const                     { return blockindex(n).length; }

  int         begin() const                           { return 0; }
  int         end() const                             { return size(); }

  int         front() const                           { return begin(); }
  int         back() const                            { return size()-1; }

  int         push_front(const char* s, uint l)       { return insert(begin(), s, l); }
  int         push_back(const char* s, uint l)        { return add(s, l); }

  void        pop_front()                             { erase(begin()); }
  void        pop_back()                              { erase(end()-1); }

  int         add(const char* s, uint l);

  int         insert(int n, const char* s, uint l);

  void        erase(int position)                     { erase(position, position+1); }
  void        erase(int first, int last);

  int         set(int n, const char* s, uint l);

  char*       get(int n) const                        { return block + blockindex(n).offset; }
  char*       get(int n, char* s) const;
  char*       get(int n, char* s, uint l) const;

  const char* at(int n) const                         { return get(n); }
  const char* operator[](int n) const                 { return get(n); }

  gmemblock& operator=(const gmemblock& o);

  gmemblockview* add_view(uint first, uint last);
  void           remove_view(gmemblockview* v);

  void save(gfile& fp);
  void load(gfile& fp);

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

