
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmemems.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  EMS functions.
//  ------------------------------------------------------------------


#include <dos.h>
#include <mem.h>
#include <string.h>
#include <gmemall.h>


//  ------------------------------------------------------------------

#if defined(__MSDOS__) and not defined(__386__)


//  ------------------------------------------------------------------
//  Allocates pages of EMS memory

int emsalloc(int numpages) {

  union REGS regs;

  regs.x.bx=numpages;
  regs.h.ah=0x43;                           // allocate EMS pages
  int86(0x67,&regs,&regs);
  if(regs.h.ah)
    return(-1);                             // check for EMM error
  return(regs.x.dx);                        // return EMM handle
}


//  ------------------------------------------------------------------
//  Deallocates pages of EMS memory

int emsdealloc(int handle) {

  union REGS regs;

  regs.x.dx=handle;
  regs.h.ah=0x45;                           // deallocate EMS pages
  int86(0x67,&regs,&regs);
  return(regs.h.ah);                        // return EMM error code
}


//  ------------------------------------------------------------------
//  Determines if the EMM device driver is loaded

int emsexist() {

  static char* driver = "EMMXXXX0";
  register int i, match=YES;
  char* p;
  union REGS regs;
  struct SREGS sregs;

  regs.x.ax=0x3567;                   // get EMS interrupt vector
  int86x(0x21,&regs,&regs,&sregs);
  p=(char*)MK_FP(sregs.es,10);        // point to EMM driver ID
  for(i=0;i<8;i++) {                  // test for valid driver name
    if(p[i]!=driver[i]) {
      match=NO;
      break;
    }
  }
  return(match);
}


//  ------------------------------------------------------------------
//  Returns the current EMM frame segment

word emsframe() {

  union REGS regs;

  regs.h.ah=0x41;                     // EMM get page frame address
  int86(0x67,&regs,&regs);
  if(regs.h.ah)
    return(0);                        // check for EMM error
  return(regs.x.bx);                  // return EMS page frame address
}


//  ------------------------------------------------------------------
//  Returns the number of unallocated EMS pages

word emsfree() {

  union REGS regs;

  regs.h.ah=0x42;                           // get unallocated pages
  int86(0x67,&regs,&regs);
  if(regs.h.ah)
    return(0);                              // check for EMM error
  return(regs.x.bx);
}


//  ------------------------------------------------------------------
//  Maps logical EMS pages to physical DOS pages

int emsmap(int handle, int logpage, int phypage) {

  union REGS regs;

  regs.x.bx=logpage;
  regs.h.al=(byte)phypage;
  regs.x.dx=handle;
  regs.h.ah=0x44;           //  map handle pages
  int86(0x67,&regs,&regs);
  return(regs.h.ah);
}


//  ------------------------------------------------------------------
//  Copies characters from EMS memory

int emsread(byte* dest, word emsofs, word numbytes) {

  word emsseg;
  byte* src;

  emsseg=emsframe();
  if(!emsseg)
    return(1);
  src=(byte*)MK_FP(emsseg,emsofs);
  _fmemcpy(dest, src, numbytes);
  return(0);
}


//  ------------------------------------------------------------------
//  Returns the total number of EMS pages

word emstotal() {

  union REGS regs;

  regs.h.ah=0x42;                     // get unallocated/total pages
  int86(0x67,&regs,&regs);
  if(regs.h.ah)
    return(0);                        // check for EMM error
  return(regs.x.dx);
}


//  ------------------------------------------------------------------
//  Returns the EMS version in a text string

char* emsver() {

  static char* version = "\0.\0\0";
  union REGS regs;
  byte ver;

  regs.h.ah=0x46;                     // get EMS version
  int86(0x67,&regs,&regs);
  if(regs.h.ah)
    return(NULL);                     // check for EMM error
  ver=regs.h.al;
  version[0]=(char)((ver>>4)+0x30);           // shift bits & convert to ASCII
  version[2]=(char)((ver&0x0f)+0x30);
  return(version);
}


//  ------------------------------------------------------------------
//  Copies characters to EMS memory

int emswrite(byte* src, word emsofs, word numbytes) {

  word emsseg;
  byte* dest;

  emsseg=emsframe();                  // get EMS page frame address
  if(!emsseg)
    return(1);                        // test for EMM error
  dest=(byte*)MK_FP(emsseg,emsofs);
  _fmemcpy(dest, src, numbytes);
  return(0);                          // return normally
}


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

