
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmemi86.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Intel 80x86 interrupt portable wrapper class.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GMEMI86_H
#define __GMEMI86_H

#if defined(__MSDOS__)


//  ------------------------------------------------------------------
//  Required headers

#include <dos.h>
#include <gdefs.h>


//  ------------------------------------------------------------------

#if defined(__WATCOMC__)

#define GREGPACK REGPACK

#elif defined(__DJGPP__)

#include <dpmi.h>
typedef union {
  struct {
    unsigned long edi;
    unsigned long esi;
    unsigned long ebp;
    unsigned long res;
    unsigned long ebx;
    unsigned long edx;
    unsigned long ecx;
    unsigned long eax;
  } x;
  struct {
    unsigned short di, di_hi;
    unsigned short si, si_hi;
    unsigned short bp, bp_hi;
    unsigned short res, res_hi;
    unsigned short bx, bx_hi;
    unsigned short dx, dx_hi;
    unsigned short cx, cx_hi;
    unsigned short ax, ax_hi;
    unsigned short flags;
    unsigned short es;
    unsigned short ds;
    unsigned short fs;
    unsigned short gs;
    unsigned short ip;
    unsigned short cs;
    unsigned short sp;
    unsigned short ss;
  } w;
  struct {
    unsigned char edi[4];
    unsigned char esi[4];
    unsigned char ebp[4];
    unsigned char res[4];
    unsigned char bl, bh, ebx_b2, ebx_b3;
    unsigned char dl, dh, edx_b2, edx_b3;
    unsigned char cl, ch, ecx_b2, ecx_b3;
    unsigned char al, ah, eax_b2, eax_b3;
  } h;
} __gdpmi_regs;
  
#define GREGPACK __gdpmi_regs

#elif defined(__BORLANDC__) and defined(__DPMI32__)

struct REGPACKB {
  byte al, ah;  word : 16;
  byte bl, bh;  word : 16;
  byte cl, ch;  word : 16;
  byte dl, dh;  word : 16;
};

struct REGPACKW {
  word ax;  word : 16;
  word bx;  word : 16;
  word cx;  word : 16;
  word dx;  word : 16;
  word si;  word : 16;
  word di;  word : 16;
  word cflag;
  word flags;
  word es;
  word cs;
  word ss;
  word ds;
  word fs;
  word gs;
};

struct REGPACKX {
  dword eax;
  dword ebx;
  dword ecx;
  dword edx;
  dword esi;
  dword edi;
  dword cflag;
  dword flags;
  word es;
  word cs;
  word ss;
  word ds;
  word fs;
  word gs;
};

union GREGPACK {
  struct REGPACKB h;
  struct REGPACKW w;
  struct REGPACKX x;
};

#else

struct REGPACKB {
  byte al, ah;
  byte bl, bh;
  byte cl, ch;
  byte dl, dh;
};

struct REGPACKW {
  word ax;
  word bx;
  word cx;
  word dx;
  word bp;
  word si;
  word di;
  word ds;
  word es;
  word flags;
};

struct REGPACKX {
  word eax;
  word ebx;
  word ecx;
  word edx;
  word ebp;
  word esi;
  word edi;
  word ds;
  word es;
  word flags;
  word fs;
  word gs;
};

union GREGPACK {
  struct REGPACKB h;
  struct REGPACKW w;
  struct REGPACKX x;
};

#endif


//  ------------------------------------------------------------------

class i86 {

public:

  GREGPACK r;

  uint al()    { return r.h.al; }
  uint ah()    { return r.h.ah; }
  uint ax()    { return r.w.ax; }
  uint bl()    { return r.h.bl; }
  uint bh()    { return r.h.bh; }
  uint bx()    { return r.w.bx; }
  uint cl()    { return r.h.cl; }
  uint ch()    { return r.h.ch; }
  uint cx()    { return r.w.cx; }
  uint dl()    { return r.h.dl; }
  uint dh()    { return r.h.dh; }
  uint dx()    { return r.w.dx; }
  uint bp()    { return r.w.bp; }
  uint si()    { return r.w.si; }
  uint di()    { return r.w.di; }
  uint ds()    { return r.w.ds; }
  uint es()    { return r.w.es; }
  uint flags() { return r.w.flags; }

  void al(ulong b)    { r.h.al = (byte)b; }
  void ah(ulong b)    { r.h.ah = (byte)b; }
  void ax(ulong w)    { r.w.ax = (word)w; }
  void bl(ulong b)    { r.h.bl = (byte)b; }
  void bh(ulong b)    { r.h.bh = (byte)b; }
  void bx(ulong w)    { r.w.bx = (word)w; }
  void cl(ulong b)    { r.h.cl = (byte)b; }
  void ch(ulong b)    { r.h.ch = (byte)b; }
  void cx(ulong w)    { r.w.cx = (word)w; }
  void dl(ulong b)    { r.h.dl = (byte)b; }
  void dh(ulong b)    { r.h.dh = (byte)b; }
  void dx(ulong w)    { r.w.dx = (word)w; }
  void bp(ulong w)    { r.w.bp = (word)w; }
  void si(ulong w)    { r.w.si = (word)w; }
  void di(ulong w)    { r.w.di = (word)w; }
  void ds(ulong w)    { r.w.ds = (word)w; }
  void es(ulong w)    { r.w.es = (word)w; }
  void flags(ulong w) { r.w.flags = (word)w; }

  void zeroseg();          // Zero segment registers
  void genint(int intno);      // Generate int with zeroed segs
  void genintseg(int intno);   // Generate int with segments
};


//  ------------------------------------------------------------------
//  Inline implementations
 
inline void i86::zeroseg() {

  #if defined(__DJGPP__)

  r.w.ds = r.w.es = 0;
  r.w.fs = r.w.gs = 0;
  r.w.cs = r.w.ss = 0;

  #else

  r.x.ds = r.x.es = 0;
  #if defined(__386__) and not defined(__WINDOWS__)
  r.x.fs = r.x.gs = 0;
  #endif
  #if defined(__BORLANDC__) and defined(__DPMI32__)
  r.x.cs = r.x.ss = 0;
  #endif

  #endif
}

inline void i86::genintseg(int intno) {

  #if defined(__BORLANDC__) and defined(__DPMI32__)
  int386x(intno, (REGS*)&r, (REGS*)&r, (SREGS*)&r.x.es);
  #elif defined(__DJGPP__)
  __dpmi_int(intno, (__dpmi_regs*)&r);
  #else
  intr(intno, (REGPACK*)&r);
  #endif
}

inline void i86::genint(int intno) {

  zeroseg();
  genintseg(intno);
}


//  ------------------------------------------------------------------
//  A more portable version of MK_FP()

#if defined(__WATCOMC__) and defined(__MSDOS__) and defined(__386__)

#define gmkfp(s,o) ((s << 4) + o)

#elif defined(__DJGPP__)

#include <sys/nearptr.h>
#include <crt0.h>

inline void * gmkfp(unsigned short seg, unsigned short ofs) {
  if(!(_crt0_startup_flags & _CRT0_FLAG_NEARPTR))
    if(!__djgpp_nearptr_enable())
      return (void *)0;
  return (void *)(seg*16 + ofs + __djgpp_conventional_base);
}

#else

#define gmkfp(s,o) MK_FP(s,o)

#endif


//  ------------------------------------------------------------------

#endif
#endif

//  ------------------------------------------------------------------

