
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmemutil.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Miscellaneous memory functions
//  ------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include <gmemall.h>
#include <gmemdbg.h>
#include <gstrgstr.h>


//  ------------------------------------------------------------------
//  Dump 16 bytes as hex and characters

char* HexDump16(char* strbuf, const char* memptr, int limit, const char* fmt, int fmtno) {

  register int n;
  word mem[16];
  char str[17];
  register word* mptr=mem;
  register char* sptr=str;
  const char* fmts[] = {
    fmt,
    "%02X%02X %02X%02X %02X%02X %02X%02X  %02X%02X %02X%02X %02X%02X %02X%02X  %s",
    "%02X %02X %02X %02X  %02X %02X %02X %02X  %02X %02X %02X %02X  %02X %02X %02X %02X  %s"
  };

  limit = (limit > 16) ? 16 : limit;

  for(n=0; n<limit; n++, memptr++) {
    *mptr++ = *memptr;
    *sptr++ = *memptr >= ' ' ? *memptr : '.';
  }
  for(; n<16; n++, memptr++) {
    *mptr++ = 0;
    *sptr++ = '.';
  }
  *sptr = NUL;

  sprintf(strbuf, fmts[fmtno],
    mem[0], mem[1], mem[2],  mem[3],  mem[4],  mem[5],  mem[6],  mem[7],
    mem[8], mem[9], mem[10], mem[11], mem[12], mem[13], mem[14], mem[15],
    str
  );

  return strbuf;
}


//  ------------------------------------------------------------------
//  Allocate memory, copy data into it and return a pointer to it

void* memdup(void* src, uint size) {

  void* ptr = throw_malloc(size);
  if(ptr)
    memcpy(ptr, src, size);

  return ptr;
}


//  ------------------------------------------------------------------

gstring& addhexdump(gstring& dump, const void* data, size_t datalen, const char* lineterm) {

  char buf[256];
  register uint pos = 0;
  register const char* ptr = (const char*)data;

  while(pos < datalen) {

    register uint dataleft = datalen - pos;
    sprintf(buf, "%04X   ", pos);
    HexDump16(buf+7, ptr, (dataleft < 16) ? dataleft : 16, HEX_DUMP2);
    dump += strcat(buf, lineterm);
    ptr += 16;
    pos += 16;
  }

  return dump;
}


//  ------------------------------------------------------------------

