
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmemxms.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  XMS library.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GMEMXMS_H
#define __GMEMXMS_H


//  ------------------------------------------------------------------

/*  Copyright 1990,1991 by Michael Graff.        USE AT YOUR OWN RISK       */
/*              See end of this file for more information.                  */
/*                                                                          */
/*  Version 1.2, 08-Mar-91                                                  */
/*  Version 1.3, 07-May-91                                                  */
/*  Version 1.4, 27-May-91                                                  */
/*  Version 1.5, 12-Jun-91    Fixed a few little errors                     */

#include <dos.h>

    /*  The HMA can be used for data storage and code.                      */
    /* This needs to be defined in ONE place in your code as:               */
    /*    void far *HMAAddr = MK_FP(0xffff, 0x0010);                        */
    /* and others as:                                                       */
    /*  extern void far *HMAAddr;                                           */

struct EMMMoveStruct {
    unsigned long int   TransferLength;
    unsigned short        SourceHandle;
    unsigned long int   SourceOffset;
    unsigned short        DestHandle;
    unsigned long int   DestOffset;
};  /*  Needed for the 640k to extended memory move routines                */

extern unsigned short far cdecl XMS_Setup(void);    /* MUST BE CALLED PRIOR TO      */
                                            /* OTHER XMS_ ROUTINES!         */

extern unsigned char far cdecl XMS_FreeMem(   /* returns free and total extended  */
         unsigned short far *freemem,     /* free extended memory in K        */
         unsigned short far *totmem);     /* total extended memory in K       */

extern unsigned char far cdecl XMS_Version(   /* Returns version information      */
         unsigned short far *version,     /* BCD protocol version (2.0)       */
         unsigned short far *internal,    /* BCD internal version number      */
         unsigned short far *HMA);        /* is the HMA free?                 */

extern unsigned char far cdecl XMS_RequestHMA(    /* Try to allocate HMA          */
         unsigned short mysize);  /* amount you'll be using in K.             */
                                /*  Applications should use 0xffff to       */
                                /*  allocate all 64k                        */

extern unsigned char far cdecl XMS_ReleaseHMA(void);  /* give up the HMA          */

extern unsigned char far cdecl XMS_GlobalEnableA20(void);  /* when the A20 line   */
extern unsigned char far cdecl XMS_GlobalDisableA20(void); /* is enabled, the HMA */
extern unsigned char far cdecl XMS_LocalEnableA20(void);   /* can be accessed.    */
extern unsigned char far cdecl XMS_LocalDisableA20(void);

extern unsigned char far cdecl XMS_QueryA20(  /* Find out if A20 is enabled       */
         unsigned short far *state);

extern unsigned char far cdecl XMS_AllocEMB(  /* Allocate extended momory         */
         unsigned short mysize,       /* size in K needed                     */
         unsigned short far *handle); /* handle for this EMB                  */

extern unsigned char far cdecl XMS_FreeEMB(   /* Release an EMB                   */
         unsigned short handle);      /* handle to release                    */

extern unsigned char far cdecl XMS_MoveEMB(       /* Move between real memory and */
         struct EMMMoveStruct far *MoveRec);/* extended memory.             */

extern unsigned char far cdecl XMS_LockEMB(   /* make sure a memory block doesn't */
         unsigned short handle,           /* move.                            */
         void far *address);    /* LINEAR memory address for this block     */

extern unsigned char far cdecl XMS_UnlockEMB( /* undo a lock                      */
         unsigned short handle);

extern unsigned char far cdecl XMS_GetEMBHandleInfo(   /* info on a handle        */
         unsigned short handle,               /* handle info wanted on        */
         unsigned char far *LockCount,      /* number of locks enabled      */
         unsigned char far *EMBHandlesFree, /* total handles free in system */
         unsigned short far *EMBlength);      /* size of this handle in K     */

extern unsigned char far cdecl XMS_ReallocEMB(    /* Try to change the size of    */
         unsigned short handle,               /* this block.                  */
         unsigned short newsize);             /* new size in K                */

extern unsigned char far cdecl XMS_RequestUMB(    /* request UPPER MEMORY BLOCK   */
         unsigned short SizeWanded,           /* size in PARAGRAPHS needed    */
         unsigned short far *segaddr,         /* segment address of block     */
         unsigned short far *SizeUgot);       /* size of block you got        */

extern unsigned char far cdecl XMS_ReleaseUMB(    /* Release UMB                  */
         unsigned short segaddr);

/*
------------------------------------------------------------------------------

If you find any errors, want to suggest improvements, or wish to use these
routines in a commercial application, please write me at the following
address:
            Michael Graff
            418 8th Avenue
            Grinnell, IA  50112

            E-MAIL:  explorer@iastate.edu, ga.mlg@isumvs.bitnet


You can use these routines for personal use without restrictions, other than
I would like to get a copy of YOUR source.

If you use these routines in a program and make a profit, please share some
small part of it with me!  (Donations of all kinds accepted, from very old
hardware to old compact disks to a copy of part or all of your source to
a thank you.)


--Michael


------------------------------------------------------------------------------


DEFINITIONS: ( stolen from the XMS 2.0 spec doc )
-------------------------------------------------

    Extended
    Memory      -   Memory in 80286 and 80386 based machines which is located
                    above the 1MB address boundary.

    High Memory
    Area (HMA)  -   The first 64K of extended memory.  The High Memory
            Area is unique because code can be executed in it while
            in real mode.  The HMA officially starts at FFFF:10h
            and ends at FFFF:FFFFh making it 64K-16 bytes in length.

    Upper Memory
    Blocks (UMBs)-  Blocks of memory available on some 80x86 based machines
                    which are located between DOS's 640K limit and the
                    1MB address boundary.  The number, size, and location
                    of these blocks vary widely depending upon the types
                    of hardware adapter cards installed in the machine.

    Extended Memory
    Blocks (EMBs)-  Blocks of extended memory located above the HMA which
                    can only be used for data storage.

    A20 Line    -   The 21st address line of 80x86 CPUs.  Enabling the A20
                    line allows access to the HMA.

HIGH MEMORY AREA RESTRICTIONS:
------------------------------

-   Far pointers to data located in the HMA cannot be passed to DOS.  DOS
    normalizes any pointer which is passed into it.  This will cause data
    addresses in the HMA to be invalidated.

-   Disk I/O directly into the HMA (via DOS, INT 13h, or otherwise) is not
    recommended.

-   Programs, especially drivers and TSRs, which use the HMA *MUST* use
    as much of it as possible.  If a driver or TSR is unable to use at
    least 90% of the available HMA (typically ~58K), they must provide
    a command line switch for overriding HMA usage.  This will allow
    the user to configure his machine for optimum use of the HMA.

-   Device drivers and TSRs cannot leave the A20 line permanently turned
    on.  Several applications rely on 1MB memory wrap and will overwrite the
    HMA if the A20 line is left enabled potentially causing a system crash.

-   Interrupt vectors must not point into the HMA.  This is a result of
    the previous restriction.  Note that interrupt vectors can point into
    any allocated upper memory blocks however.

ERROR CODE INDEX:   ( functions of type UNSIGNED CHAR return these codes )
--------------------------------------------------------------------------

    80h if the function is not implemented
    81h if a VDISK device is detected
    82h if an A20 error occurs
    8Eh if a general driver error occurs
    8Fh if an unrecoverable driver error occurs
    90h if the HMA does not exist
    91h if the HMA is already in use
    92h if DX is less than the /HMAMIN= parameter
    93h if the HMA is not allocated
    94h if the A20 line is still enabled
    A0h if all extended memory is allocated
    A1h if all available extended memory handles are in use
    A2h if the handle is invalid
    A3h if the SourceHandle is invalid
    A4h if the SourceOffset is invalid
    A5h if the DestHandle is invalid
    A6h if the DestOffset is invalid
    A7h if the Length is invalid
    A8h if the move has an invalid overlap
    A9h if a parity error occurs
    AAh if the block is not locked
    ABh if the block is locked
    ACh if the block's lock count overflows
    ADh if the lock fails
    B0h if a smaller UMB is available
    B1h if no UMBs are available
    B2h if the UMB segment number is invalid


--------------------------------------------------------------------------

Note:  $Id: gmemxms.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $

--------------------------------------------------------------------------------

XMSLIB - access extended memory using the XMS driver on a 286 or better machine.

--------------------------------------------------------------------------------


Version 1.0:
        Never released.
Version 1.1:
        Never released.
Version 1.2:
        First ``public'' release to the Usenet community.
Version 1.3:
        Fixed one major (and stupid) bug:  I had a JMP instruction in the
        XMS_Setup routine that caused the function to always think there was an
        XMS driver loaded.  Thanks to all those who pointed that one out...
        Also, thanks to DTF, one FAR directive was missing from the xlslib.h
        header file.
Version 1.4:
        Fixed a few more typos and a bug or two.  All memory models are now
        supported, even tiny and huge.  An object file, produced by Turbo
        Assembler 2.5 using the large model, is included in this release
        as well for those who may not have tasm.
Version 1.5:
        Fixed even more bugs in the code, cleaned up a lot of stuff, and made
        this work with *all* memory models.  (They all work for me, that is...)
        I also changed a few variable names (although this should not change
        things much, since they were in the asm source) and removed the
        definition of the HMA from xmslib.h.  (I got linker errors, so until
        I work around them, I had to do this.  Maybe I should put it in the
        .ASM file and make it extern in the header file?)

        I should note that I used HIMEM.SYS to test this, since this is the
        only XMS I have access to.


-------------------------------------------------------------------------------
-- explorer@iastate.edu,  Michael Graff, 418 8th Ave, Grinnell, Iowa,  50112.--
-------------------------------------------------------------------------------

*/


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

