
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsg.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Message structures and classes.
//  ------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>

#include <gmemdbg.h>
#include <gstrmail.h>
#include <gmsgmime.h>

#include <gmsg.h>


//  ------------------------------------------------------------------

void split_origin(const gstring& s, gstring& origin, gstring& netname, gstring& address) {

  char buf[256];
  strcpy(buf, s.c_str());
  char* ptr = strrchr(buf, '(' /*)*/ );
  if(ptr) {
    if(ptr != buf)
      *((ptr++)-1) = NUL;
    else
      *ptr++ = NUL;
    origin = buf;
    char* nnptr = ptr;
    while(not isdigit(*ptr) and *ptr != /*(*/ ')' and *ptr)
      ptr++;
    char* addr = ptr;
    if(*ptr and isdigit(*ptr)) {
      while(*ptr !=  /*(*/ ')' and *ptr)
        ptr++;
      *ptr = NUL;
      address = addr;
    }
    *addr = NUL;
    netname = nnptr;
  }
}


//  ------------------------------------------------------------------

void gmsg_links::reset() {

  reply_to = reply_first = reply_next = 0;

  for(int n=0; n<list_max(); n++)
    reply_list[n] = 0;
}


//  ------------------------------------------------------------------

gmsg::gmsg() {

  reset(false);
}


//  ------------------------------------------------------------------

gmsg::~gmsg() {

  reset();
}


//  ------------------------------------------------------------------

void gmsg::reset(bool normal) {

  if(normal) {
    link.reset();
  }

  msgno = 0;
  date.written.reset();
  date.arrived.reset();
  date.received.reset();
  attr.reset();
  cost = 0;
  timesread = 0;

  memset(&adeptxbbs, 0, sizeof(adeptxbbs));
  memset(&ezycom, 0, sizeof(ezycom));
  memset(&goldbase, 0, sizeof(goldbase));
  memset(&hudson, 0, sizeof(hudson));
  memset(&jam, 0, sizeof(jam));
  memset(&pcboard, 0, sizeof(pcboard));
  memset(&wildcat, 0, sizeof(wildcat));
}


//  ------------------------------------------------------------------

struct chrp_int {
  const char* str;
  int         id;
};


//  ------------------------------------------------------------------

class chrp_int_find {

protected:

  chrp_int* index;

public:

  chrp_int_find(chrp_int* i) { index = i; }

  int find(const char* str, int len);

};


//  ------------------------------------------------------------------

int chrp_int_find::find(const char* str, int len) {

  chrp_int* iptr = index;
  while(iptr->str) {
    if(strnieql(str, iptr->str, len))
      return iptr->id;
    iptr++;
  }

  return 0;
}


//  ------------------------------------------------------------------

int gmsg::identify_kludge(const char* id, int idlen) {

  chrp_int ids[] = {
    { "INTL",      gkludge::intl },
    { "FMPT",      gkludge::fmpt },
    { "TOPT",      gkludge::topt },
    { "AREA:",     gkludge::area },
    { "MSGID:",    gkludge::msgid },
    { "REPLY:",    gkludge::reply },
    { "REPLYADDR", gkludge::replyaddr },
    { "REPLYTO",   gkludge::replyto },
    { "PID:",      gkludge::pid },
    { "SPLIT:",    gkludge::split },
    { "I51",       gkludge::i51 },
    { "FLAGS",     gkludge::flags },
    { "CHRS:",     gkludge::chrs },
    { "CHARSET:",  gkludge::charset },
    { "ENC:",      gkludge::enc },
    { "FWDFROM",   gkludge::fwdfrom },
    { "FWDORIG",   gkludge::fwdorig },
    { "FWDTO",     gkludge::fwdto },
    { "FWDDEST",   gkludge::fwddest },
    { "FWDSUBJ",   gkludge::fwdsubj },
    { "FWDAREA",   gkludge::fwdarea },
    { "FWDMSGID",  gkludge::fwdmsgid },
    { "MOOD:",     gkludge::mood },
    { "GIF:",      gkludge::gif },
    { "VIA:",      gkludge::via },
    { "VIA",       gkludge::via },
    { "SEEN-BY:",  gkludge::seen_by },
    { "PATH:",     gkludge::path },
    { NULL,        0 }
  };

  chrp_int_find x(ids);
  return x.find(id, idlen);
}


//  ------------------------------------------------------------------

int gmsg::identify_header(const char* id, int idlen) {

  chrp_int ids[] = {
    { "Also-Control:"              , gheader::also_control              },
    { "Apparently-To:"             , gheader::apparently_to             },
    { "Approved-By:"               , gheader::approved_by               },
    { "Approved:"                  , gheader::approved                  },
    { "Archive-name:"              , gheader::archive_name              },
    { "Article-Names:"             , gheader::article_names             },
    { "Article-Updates:"           , gheader::article_updates           },
    { "Bcc:"                       , gheader::bcc                       },
    { "Cache-Post-Path:"           , gheader::cache_post_path           },
    { "Cc:"                        , gheader::cc                        },
    { "Comment:"                   , gheader::comment                   },
    { "Comments:"                  , gheader::comments                  },
    { "Content-Description:"       , gheader::content_description       },
    { "Content-Disposition:"       , gheader::content_disposition       },
    { "Content-ID:"                , gheader::content_id                },
    { "Content-Length:"            , gheader::content_length            },
    { "Content-Transfer-Encoding:" , gheader::content_transfer_encoding },
    { "Content-Type:"              , gheader::content_type              },
    { "Control:"                   , gheader::control                   },
    { "Date:"                      , gheader::date                      },
    { "Delivery-Date:"             , gheader::delivery_date             },
    { "Distribution:"              , gheader::distribution              },
    { "Encrypted:"                 , gheader::encrypted                 },
    { "Errors-To:"                 , gheader::errors_to                 },
    { "Expires:"                   , gheader::expires                   },
    { "Followup-To:"               , gheader::followup_to               },
    { "From"                       , gheader::fromx                     },
    { "From:"                      , gheader::from                      },
    { "In-Reply-To:"               , gheader::in_reply_to               },
    { "Keywords:"                  , gheader::keywords                  },
    { "Last-modified:"             , gheader::last_modified             },
    { "Lines:"                     , gheader::lines                     },
    { "Mail-Copies-To:"            , gheader::mail_copies_to            },
    { "Message-ID:"                , gheader::message_id                },
    { "MIME-Version:"              , gheader::mime_version              },
    { "News-Software:"             , gheader::news_software             },
    { "Newsgroups:"                , gheader::newsgroups                },
    { "NNTP-Posting-Host:"         , gheader::nntp_posting_host         },
    { "NNTP-Posting-User:"         , gheader::nntp_posting_user         },
    { "Old-Date:"                  , gheader::old_date                  },
    { "Organization:"              , gheader::organization              },
    { "Originator:"                , gheader::originator                },
    { "Path:"                      , gheader::path                      },
    { "Posting-frequency:"         , gheader::posting_frequency         },
    { "Precedence:"                , gheader::precedence                },
    { "Priority:"                  , gheader::priority                  },
    { "Received:"                  , gheader::received                  },
    { "References:"                , gheader::references                },
    { "Reply-Path:"                , gheader::reply_path                },
    { "Reply-To:"                  , gheader::reply_to                  },
    { "Return-Path:"               , gheader::return_path               },
    { "Return-Receipt-To:"         , gheader::return_receipt_to         },
    { "See-Also:"                  , gheader::see_also                  },
    { "Sender:"                    , gheader::sender                    },
    { "Status:"                    , gheader::status                    },
    { "Subject:"                   , gheader::subject                   },
    { "Summary:"                   , gheader::summary                   },
    { "Supersedes:"                , gheader::supersedes                },
    { "TechSubList:"               , gheader::techsublist               },
    { "To:"                        , gheader::to                        },
    { "URL:"                       , gheader::url                       },
    { "Version:"                   , gheader::version                   },
    { "X-Char-Esc:"                , gheader::x_char_esc                },
    { "X-Charset:"                 , gheader::x_charset                 },
    { "X-FTN-From:"                , gheader::x_ftn_from                },
    { "X-FTN-To:"                  , gheader::x_ftn_to                  },
    { "X-Mailreader:"              , gheader::x_mailreader              },
    { "X-Newsreader:"              , gheader::x_newsreader              },
    { "X-To:"                      , gheader::x_to                      },
    { "Xref:"                      , gheader::xref                      },
    { NULL                         , 0                                  }
  };

  chrp_int_find x(ids);
  return x.find(id, idlen);
}


//  ------------------------------------------------------------------

void gmsg::add_kludge(int id, const char* cptr, int fieldoffset) {

  const char* field = cptr + fieldoffset;

  switch(id) {
    case gkludge::intl:       kludge.intl = field;       break;
    case gkludge::fmpt:       kludge.fmpt = field;       break;
    case gkludge::topt:       kludge.topt = field;       break;
    case gkludge::area:       kludge.area = field;       break;
    case gkludge::msgid:      kludge.msgid = field;      break;
    case gkludge::reply:      kludge.reply = field;      break;
    case gkludge::replyaddr:  kludge.replyaddr = field;  break;
    case gkludge::replyto:    kludge.replyto = field;    break;
    case gkludge::pid:        kludge.pid = field;        break;
    case gkludge::split:      kludge.split = field;      break;
    case gkludge::i51:        kludge.i51 = true;         break;
    case gkludge::flags:      kludge.flags = field;      break;
    case gkludge::chrs:       kludge.chrs = field;       break;
    case gkludge::charset:    kludge.charset = field;    break;
    case gkludge::enc:        kludge.enc = field;        break;
    case gkludge::fwdfrom:    kludge.fwdfrom = field;    break;
    case gkludge::fwdorig:    kludge.fwdorig = field;    break;
    case gkludge::fwdto:      kludge.fwdto = field;      break;
    case gkludge::fwddest:    kludge.fwddest = field;    break;
    case gkludge::fwdsubj:    kludge.fwdsubj = field;    break;
    case gkludge::fwdarea:    kludge.fwdarea = field;    break;
    case gkludge::fwdmsgid:   kludge.fwdmsgid = field;   break;
    case gkludge::mood:       kludge.mood = field;       break;
    case gkludge::gif:        kludge.gif = field;        break;
    case gkludge::via:        kludge.via += field;       break;
    case gkludge::seen_by:    kludge.seen_by += field;   break;
    case gkludge::path:       kludge.path += field;      break;
    default:                  kludge.other += cptr;
  }
}


//  ------------------------------------------------------------------

void gmsg::add_header(int id, const char* cptr, int fieldoffset) {

  const char* field = cptr + fieldoffset;

  switch(id) {
    case gheader::reply_to:                   header.reply_to = field;                   break;
    case gheader::sender:                     header.sender = field;                     break;
    case gheader::from:                       header.from += field;                      break;
    case gheader::to:                         header.to += field;                        break;
    case gheader::cc:                         header.cc += field;                        break;
    case gheader::bcc:                        header.bcc += field;                       break;
    case gheader::subject:                    header.subject = field;                    break;
    case gheader::date:                       header.date = field;                       break;
    case gheader::keywords:                   header.keywords = field;                   break;
    case gheader::comments:                   header.comments += field;                  break;
    case gheader::message_id:                 header.message_id = field;                 break;
    case gheader::in_reply_to:                header.in_reply_to = field;                break;
    case gheader::references:                 header.references = field;                 break;
    case gheader::newsgroups:                 header.newsgroups = field;                 break;
    case gheader::organization:               header.organization = field;               break;
    case gheader::followup_to:                header.followup_to = field;                break;
    case gheader::expires:                    header.expires = field;                    break;
    case gheader::lines:                      header.lines = field;                      break;
    case gheader::distribution:               header.distribution = field;               break;
    case gheader::control:                    header.control = field;                    break;
    case gheader::approved:                   header.approved = field;                   break;
    case gheader::mime_version:               header.mime_version = field;               break;
    case gheader::content_type:               header.content_type = field;               break;
    case gheader::content_transfer_encoding:  header.content_transfer_encoding = field;  break;
    case gheader::content_id:                 header.content_id = field;                 break;
    case gheader::content_description:        header.content_description = field;        break;
    case gheader::content_disposition:        header.content_disposition = field;        break;
    case gheader::x_charset:                  header.x_charset = field;                  break;
    case gheader::x_char_esc:                 header.x_char_esc = field;                 break;
    case gheader::x_to:                       header.x_to = field;                       break;
    case gheader::x_ftn_from:                 header.x_ftn_from = field;                 break;
    case gheader::x_ftn_to:                   header.x_ftn_to = field;                   break;
    default:                                  header.other += cptr;
  }
}


//  ------------------------------------------------------------------

bool gmsg::split_controlline(const char* cptr, int& id, int& idlen, int& field) {

  bool header_found = false;
  const char* idptr = cptr;

  if(*idptr == '\x1')
    idptr++;

  if(strnieql(idptr, "RFC-", 4) or strnieql(idptr, "RFC.", 4)) {
    header_found = true;
    idptr += 4;
  }

  id = (int)(idptr-cptr);
  const char* fptr = strskip_to(idptr, ": ");
  idlen = (int)(fptr-idptr) + (*fptr == ':' ? 1 : 0);
  fptr = strskip_past(fptr, ": ");
  field = (int)(fptr-cptr);
  return header_found;
}


//  ------------------------------------------------------------------

bool gmsg::valid_kludge(const char* cptr, int idlen) {

  for(int n=0; n<idlen; cptr++,n++) {
    if(islower(*cptr) or (not isalnum(*cptr) and *cptr != ':'))
      return false;
  }
  return true;
}


//  ------------------------------------------------------------------

bool gmsg::add_control(const char* cptr) {

  int id, idlen, field, what;
  bool got_header = split_controlline(cptr, id, idlen, field);
  if(got_header)
    what = identify_header(cptr+id, idlen);
  else {
    what = identify_kludge(cptr+id, idlen);
    if(not what) {
      what = identify_header(cptr+id, idlen);
      if(what)
        got_header = true;
      else if(internet)
        got_header = not valid_kludge(cptr+id, idlen);
    }
  }
  if(got_header)
    add_header(what, cptr, field);
  else
    add_kludge(what, cptr, field);

  return got_header;
}


//  ------------------------------------------------------------------

bool gmsg::get_boundary(const char* field, gstring& boundary) {

  gmime_content_type content_type;
  content_type.crack(field);
  if(content_type.type.ieql("multipart")) {
    for(int p=0; p<content_type.parameters.size(); p++) {
      if(content_type.parameters[p].attribute.ieql("boundary")) {
        boundary = content_type.parameters[p].value;
        return true;
      }
    }
  }
  return false;
}


//  ------------------------------------------------------------------

void gmsg::markup_lines() {

  gstring buffer;
  gstrarray boundaries;
  bool find_area = true;
  bool text_found = false;
  bool header_found = false;
  bool in_body = false;
  int last_tagline = 0;
  int last_tearline = 0;
  int last_origin = 0;
  int last_gateorigin = 0;
  int last_signature = 0;

  if(internet)
    header_found = true;

  for(int n=0; n<text.lines(); n++) {
    if(fidonet and (*text[n] == '\x1')) {
      buffer = text[n];
      int id, idlen, field, what;
      const char* cptr = buffer.c_str();
      bool got_header = split_controlline(cptr, id, idlen, field);
      if(not text_found) {
        if(got_header)
          what = identify_header(cptr+id, idlen);
        else {
          what = identify_kludge(cptr+id, idlen);
          if(not what) {
            what = identify_header(cptr+id, idlen);
            if(what)
              got_header = true;
            else if(internet)
              got_header = not valid_kludge(cptr+id, idlen);
          }
        }
        if(got_header)
          internet = header_found = true;
        uint type = got_header ? gline::header : gline::kludge;
        text.type(n) |= type;
        text.id(n) = what;
        buffer = text[n];
        while((n+1 < text.lines()) and (*text[n+1] == '\x1') and is_mail_lwsp(text[n+1][1])) {
          buffer += text[++n]+1;
          text.type(n) |= gline::folded | type;
          text.id(n) = what;
        }
        // cptr = buffer.c_str();
        // if(got_header)
        //   add_header(what, cptr, field);
        // else
        //   add_kludge(what, cptr, field);
      }
      else {

        what = identify_kludge(cptr+id, idlen);
        if(what) {
          text.type(n) |= gline::kludge;
          text.id(n) = what;
        }
        else {
          text.type(n) |= gline::hidden | gline::body;
          text.id(n) = 0;
        }
      }
      if(find_area and strnieql(buffer.c_str()+1, "AREA:", 5))
        find_area = false;
    }
    else {
      if(not text_found) {
        if(find_area) {
          find_area = false;
          if(strnieql(text[n], "AREA:", 5)) {
            text.type(n) |= gline::kludge;
            text.id(n) = gkludge::area;
            // kludge.area = strskip_wht(text[n]+5);
            continue;
          }
        }
        text_found = true;
        in_body = true;
      }
      if(header_found) {
        in_body = false;
        if(*text[n] == NUL) {
          text.type(n) |= gline::header_end;
          header_found = false;
          in_body = true;
          continue;
        }
        else {
          int id, idlen, field, what;
          const char* cptr = text[n];
          split_controlline(text[n], id, idlen, field);
          what = identify_header(cptr+id, idlen);
          if(what == 0 and cptr[id+idlen-1] != ':') {
            header_found = false;
            in_body = true;
          }
          else {
            buffer = text[n];
            text.type(n) |= gline::header;
            text.id(n) = what;
            while((n+1 < text.lines()) and is_mail_lwsp(*text[n+1])) {
              buffer += text[++n];
              text.type(n) |= gline::folded | gline::header;
              text.id(n) = what;
            }
            cptr = buffer.c_str();
            split_controlline(cptr, id, idlen, field);
            what = identify_header(cptr+id, idlen);
            if(what == gheader::content_type) {
              gstring boundary;
              if(get_boundary(cptr+field, boundary))
                boundaries.add(boundary);
            }
            // add_header(what, cptr, field);
            continue;
          }
        }
      }
      if(in_body) {
        const char* cptr = text[n];
        if(internet and boundaries.size() and strneql(cptr, "--", 2)) {
          const char* bptr = cptr + 2;
          int b = boundaries.back();
          int l = boundaries.length(b);
          if(strneql(bptr, boundaries[b], l)) {
            if(is_mail_lwsp(bptr+l)) {
              //text.type(n-1) &= ~gline::hard;
              text.type(n) |= gline::boundary;
              header_found = true;
              continue;
            }
            else if(strneql(bptr+l, "--", 2) and is_mail_lwsp(bptr+l+2)) {
              //text.type(n-1) &= ~gline::hard;
              text.type(n) |= gline::boundary_end;
              header_found = true;
              boundaries.erase(b);
              continue;
            }
          }
        }
        text.type(n) |= gline::body;
        if(gmsg_quote::is_quote(cptr))
          text.type(n) |= gline::quote;
        else if(fidonet and (strneql(cptr, "... ", 4) or strneql(cptr, "___ ", 4)))
          last_tagline = n + 1;
        else if(fidonet and strneql(cptr, "---", 3) and (cptr[3] == ' ' or cptr[3] == NUL))
          last_tearline = n + 1;
        else if(fidonet and strneql(cptr, " # Origin: ", 11))
          last_gateorigin = n + 1;
        else if(fidonet and strneql(cptr, " * Origin: ", 11))
          last_origin = n + 1;
        else if(internet and streql(cptr, "--"))
          last_signature = n + 1;
      }
    }
  }

  if(last_origin) {
    while(1) {
      if(last_tearline < last_origin) {
        if((last_tearline and (last_tagline <= last_tearline)) or (last_tagline < last_origin)) {
          text.type(last_origin-1) |= gline::origin;
          // split_origin(text[last_origin-1]+11, origin_text, origin_netname, origin_address);
          for(int n=last_origin; n<text.lines(); n++) {
            if(strnieql(text[n], "SEEN-BY:", 8)) {
              // kludge.seen_by += strskip_wht(text[n]+8);
              text.type(n) &= ~gline::body;
              text.type(n) |= gline::kludge;
              text.id(n) = gkludge::seen_by;
            }
            if(strnieql(text[n], "\x1PATH:", 6)) {
              // kludge.path += strskip_wht(text[n]+6);
              text.type(n) &= ~(gline::hidden|gline::body);
              text.type(n) |= gline::kludge;
              text.id(n) = gkludge::path;
            }
          }
          break;
        }
      }
      last_origin = 0;
      break;
    }
  }

  if(last_tearline) {
    if(last_tagline < last_tearline) {
      text.type(last_tearline-1) |= gline::tearline;
      // tearline = text[last_tearline-1]+3;
      // tearline.btrim();
    }
    else {
      last_tearline = 0;
    }
  }

  if(last_gateorigin) {
    if(last_origin == (last_gateorigin + 1)) {
      text.type(last_gateorigin-1) |= gline::gateorigin;
      // split_origin(text[last_gateorigin-1]+11, gateorigin_text, gateorigin_netname, gateorigin_address);
    }
    else {
      last_gateorigin = 0;
    }
  }

  if(last_tagline) {
    if(last_tearline or last_origin) {
      // Any line between the tagline and the tearline or origin must be empty
      int final = (last_tearline ? last_tearline : last_origin)-1;
      for(int n=last_tagline; n<final; n++) {
        if(text.length(n)) {
          last_tagline = 0;
          break;
        }
      }
    }
    if(last_tagline) {
      text.type(last_tagline-1) |= gline::tagline;
      // tagline = text[last_tagline-1]+3;
      // tagline.btrim();
    }
  }

  if(last_signature) {
    int final = last_origin ? (last_tearline ? last_tearline : last_origin)-1 : text.lines();
    if(last_signature < final) {
      text.type(last_signature-1) |= gline::signature;
      for(int n=last_signature; n<final; n++) {
        text.type(n) &= ~gline::quote;
        text.type(n) |= gline::signature;
        // signature += text[n];
      }
    }
  }
}


//  ------------------------------------------------------------------

void gmsg::process_lines() {

  glineref line = text;

  while(line.valid()) {

    if(line.isbody()) {

      // translate it
    }
    else {

      // process control
    }

    line++;
  }
}


//  ------------------------------------------------------------------

