
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsg.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Message structures and classes.
//  ------------------------------------------------------------------


#ifndef __GMSG_H
#define __GMSG_H


//  ------------------------------------------------------------------

#include <gftnadrs.h>
#include <gstrgstr.h>
#include <gstrblks.h>
#include <gstrxlat.h>
#include <gmsgattr.h>
#include <gmsgline.h>
#include <gmsgtime.h>


//  ------------------------------------------------------------------

class gkludge {

public:

  enum {
    aaa = 0x1000,
    intl,
    fmpt,
    topt,
    area,
    msgid,
    reply,
    replyaddr,
    replyto,
    pid,
    split,
    i51,
    flags,
    chrs,
    charset,
    codepage,
    enc,
    fwdfrom,
    fwdorig,
    fwdto,
    fwddest,
    fwdsubj,
    fwdarea,
    fwdmsgid,
    mood,
    gif,
    via,
    seen_by,
    path,
    zzz
  };

  static bool iskludge(int id) { return in_range(id, aaa+1, zzz-1); }
};


//  ------------------------------------------------------------------

class gheader {

public:

  enum {
    aaa = 0x2000,
    also_control              ,
    apparently_to             ,
    approved_by               ,
    approved                  ,
    archive_name              ,
    article_names             ,
    article_updates           ,
    bcc                       ,
    cache_post_path           ,
    cc                        ,
    comment                   ,
    comments                  ,
    content_description       ,
    content_disposition       ,
    content_id                ,
    content_length            ,
    content_transfer_encoding ,
    content_type              ,
    control                   ,
    date                      ,
    delivery_date             ,
    distribution              ,
    encrypted                 ,
    errors_to                 ,
    expires                   ,
    followup_to               ,
    fromx                     ,
    from                      ,
    in_reply_to               ,
    keywords                  ,
    last_modified             ,
    lines                     ,
    mail_copies_to            ,
    message_id                ,
    mime_version              ,
    news_software             ,
    newsgroups                ,
    nntp_posting_host         ,
    nntp_posting_user         ,
    old_date                  ,
    organization              ,
    originator                ,
    path                      ,
    posting_frequency         ,
    precedence                ,
    priority                  ,
    received                  ,
    references                ,
    reply_path                ,
    reply_to                  ,
    return_path               ,
    return_receipt_to         ,
    see_also                  ,
    sender                    ,
    status                    ,
    subject                   ,
    summary                   ,
    supersedes                ,
    techsublist               ,
    to                        ,
    url                       ,
    version                   ,
    x_char_esc                ,
    x_charset                 ,
    x_ftn_from                ,
    x_ftn_to                  ,
    x_mailreader              ,
    x_newsreader              ,
    x_to                      ,
    xref                      ,
    zzz
  };

  static bool isheader(int id) { return in_range(id, aaa+1, zzz-1); }
};


//  ------------------------------------------------------------------

class gmsg_mime {

public:



};


//  ------------------------------------------------------------------
//  Message modes

const uint GMSG_HDR         = 0x0001;
const uint GMSG_TXT         = 0x0002;
const uint GMSG_HDRTXT      = 0x0003;  // GMSG_HDR|GMSG_TXT
const uint GMSG_NEW         = 0x0010;
const uint GMSG_UPDATE      = 0x0020;
const uint GMSG_DELETE      = 0x0040;
const uint GMSG_COPY        = 0x0080;
const uint GMSG_MOVE        = 0x0100;
const uint GMSG_UNS_NOT_RCV = 0x0200;
const uint GMSG_NOLSTUPD    = 0x8000;


//  ------------------------------------------------------------------

class gmsg_quote {

protected:

  static char quotechars[20];

public:

  gmsg_quote(const char* qstr);

  void set_quotechars(const char* s);

  bool is_quotechar(const char* s);
  static bool is_quote(const char* s);

  int  get_quotestring(const char* s, char* qbuf, int& qlen);

  bool equal(const char* q1, const char* q2);

};


//  ------------------------------------------------------------------

class gmsg_links {

private:

  enum { list_limit = 29 };

  ulong reply_to;
  ulong reply_first;
  ulong reply_list[list_limit];
  ulong reply_next;

public:

  gmsg_links() { reset(); }

  void reset();

  int list_max() const          { return list_limit; }
  
  void to_set(ulong m)          { reply_to = m; }
  void first_set(ulong m)       { reply_first = m; }
  void list_set(int n, ulong m) { reply_list[n] = m; }
  void next_set(ulong m)        { reply_next = m; }
  
  ulong to() const              { return reply_to; }
  ulong first() const           { return reply_first; }
  ulong list(int n) const       { return reply_list[n]; }
  ulong next() const            { return reply_next; }

};


//  ------------------------------------------------------------------

struct gmsg_adeptxbbs_fields {

  ulong iflags;       // Internet related flags
  ulong oflags;       // Other network related flags
  long  start;
  ulong length;
};


//  ------------------------------------------------------------------

struct gmsg_ezycom_fields {

  byte extattr;
  long startposition;
  long messagelength;
};


//  ------------------------------------------------------------------

struct gmsg_goldbase_fields {

  long startrec;
  word numrecs;
};


//  ------------------------------------------------------------------

struct gmsg_hudson_fields {

  word startrec;
  word numrecs;
};


//  ------------------------------------------------------------------

struct gmsg_jam_fields {

  ulong offset;        // Text offset
  ulong txtlen;
  ulong subfieldlen;   // Size of subfields
};


//  ------------------------------------------------------------------

struct gmsg_pcboard_fields {

  char      status;         // Msg header status byte
  byte      exthdrflags;    // Msg extended header flags
  char      password[13];   // Password needed to read the message
  gmsg_time reply_written;  // Timestamp of the original
  gstrarray exthdrs;        // List of extended headers
  long      txtstart;
  long      txtlength;
  uint      txtblocks;
};


//  ------------------------------------------------------------------

struct gmsg_wildcat_fields {

  char  from_title[11];
  long  from_userid;
  char  to_title[11];
  long  to_userid;
  char  network[9];
  char  internal_attach[13];
  char  external_attach[13];
  ulong next_unread;
  ulong prev_unread;
  char  reserved[20];
  long  txtlength;
};


//  ------------------------------------------------------------------

class gmsg_headers {

public:

  // RFC822:
  gstrarray from;
  gstrarray to;
  gstrarray cc;
  gstrarray bcc;
  gstrarray comments;
  gstring reply_to;
  gstring sender;
  gstring subject;
  gstring date;
  gstring keywords;
  gstring message_id;
  gstring in_reply_to;
  gstring references;

  // RFC1036:
  gstring newsgroups;
  gstring organization;
  gstring followup_to;
  gstring expires;
  gstring lines;
  gstring distribution;
  gstring control;
  gstring approved;

  // MIME:
  gstring mime_version;
  gstring content_type;
  gstring content_transfer_encoding;
  gstring content_id;
  gstring content_description;
  gstring content_disposition;

  // Special:
  gstring x_charset;
  gstring x_char_esc;
  gstring x_to;
  gstring x_ftn_from;
  gstring x_ftn_to;
  gstring x_ftn_via;
  gstring x_ftn_domain;

  // All other:
  gstrarray other;

  gmsg_headers& copy(const gmsg_headers& m);

  void reset();

};


//  ------------------------------------------------------------------

class gmsg_kludges {

public:

  // FTS-1 INTL/FMTP/TOPT
  gstring intl;
  gstring fmpt;
  gstring topt;

  // FTS-4 AREA/SEEN-BY/PATH
  gstring area;
  gstrarray seen_by;
  gstrarray path;

  // FTS-9 MSGID/REPLY
  gstring msgid;
  gstring reply;

  // FSC-35 REPLYADDR/REPLYTO
  gstring replyaddr;
  gstring replyto;

  // FSC-46 PID
  gstring pid;

  // FSC-47 SPLIT
  gstring split;

  // FSC-50 CHARSET
  gstring charset;

  // FSC-51 I51
  bool i51;

  // FSC-53 FLAGS
  gstring flags;

  // FSC-54 CHRS
  gstring chrs;

  // FSC-73 ENC
  gstring enc;
  
  // FSC-92 FWDxxxxx
  gstring fwdfrom;
  gstring fwdorig;
  gstring fwdto;
  gstring fwddest;
  gstring fwdsubj;
  gstring fwdarea;
  gstring fwdmsgid;

  // Special MOOD/GIF
  gstring mood;
  gstring gif;

  // Special CC
  gstrarray cc;

  // Special VIA
  gstrarray via;

  // All other:
  gstrarray other;

  gmsg_headers& copy(const gmsg_headers& m);

  void reset();

};


//  ------------------------------------------------------------------

class gmsg_dates {

public:

  gmsg_time written;
  gmsg_time arrived;
  gmsg_time received;
  gmsg_time replied;

};


//  ------------------------------------------------------------------
//  Base class

class gmsg {

protected:

  int  identify_kludge(const char* id, int idlen);
  int  identify_header(const char* id, int idlen);
  bool valid_kludge(const char* id, int idlen);
  void add_kludge(int id, const char* cptr, int fieldoffset);
  void add_header(int id, const char* cptr, int fieldoffset);
  bool split_controlline(const char* cptr, int& id, int& idlen, int& field);
  void wrap(const char* text, const char* prefix, gstring& first, gstring& second, int width);
  bool get_boundary(const char* field, gstring& boundary);

public:

  gmsg();
  ~gmsg();

  void reset(bool normal=true);

  // Indicator for Internet e-mail or news article
  bool                  internet;
  
  // Indicator for FTN netmail or echomail
  bool                  fidonet;

  // Message number and replylinks
  ulong                 msgno;
  gmsg_links            link;

  // FTN raw name/address
  gstring               origname;
  gftnaddrs             origaddr;
  gstring               destname;
  gftnaddrs             destaddr;

  // Processed name/address
  gstring               oname;
  gstring               oaddress;
  gstring               dname;
  gstring               daddress;

  // Subject
  gstring               subject;

  // Dates
  gmsg_dates            date;

  // FTN miscellaneous fields
  ftn_attr              attr;
  uint                  cost;
  uint                  timesread;

  // FTN file attach and request
  gstrarray             fileattach;
  gstrarray             fileattach_with_alias;
  gstrarray             fileattach_wildcard;
  gstrarray             fileattach_indirect;
  gstrarray             filerequest;

  // Msgbase-specific fields
  gmsg_adeptxbbs_fields adeptxbbs;
  gmsg_ezycom_fields    ezycom;
  gmsg_goldbase_fields  goldbase;
  gmsg_hudson_fields    hudson;
  gmsg_jam_fields       jam;
  gmsg_pcboard_fields   pcboard;
  gmsg_wildcat_fields   wildcat;

  // FTN kludges and RFC headers
  gmsg_kludges          kludge;
  gmsg_headers          header;

  // FTN Tagline, tearline and origins
  gstring               tagline;
  gstring               tearline;
  gstring               origin_text;
  gstring               origin_netname;
  gstring               origin_address;
  gstring               gateorigin_text;
  gstring               gateorigin_netname;
  gstring               gateorigin_address;

  // Internet signature
  gstrarray             signature;

  // Raw text lines
  glinearray            text;

  // Processed text lines
  glinearray            view;

  // Line processing
  void                  markup_lines();
  void                  process_lines();
  void                  format_lines();
  void                  translate(gcharxlat& charset);

  bool                  add_control(const char* cptr);

  void                  wrap_lines(int textwidth, int quotewidth);

  //void                  extract_controlinfo();
  //void                  mime_preprocess();
  //void                  translate_for_viewing();
  //void                  format_for_viewing();

  // Message content copy function
  gmsg&                 copy(const gmsg& m);

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

