
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsgattr.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Message attributes.
//  ------------------------------------------------------------------


#include <gstrall.h>
#include <gfile.h>
#include <gmsgattr.h>


//  ------------------------------------------------------------------
//  Add attributes

void ftn_attr::add(const ftn_attr& b) {

  if(b.pvt())  pvt1();
  if(b.cra())  cra1();
  if(b.rcv())  rcv1();
  if(b.snt())  snt1();
  if(b.att())  att1();
  if(b.trs())  trs1();
  if(b.orp())  orp1();
  if(b.k_s())  k_s1();
  if(b.loc())  loc1();
  if(b.hld())  hld1();
  if(b.rsv())  rsv1();
  if(b.frq())  frq1();
  if(b.rrq())  rrq1();
  if(b.rrc())  rrc1();
  if(b.arq())  arq1();
  if(b.urq())  urq1();

  if(b.grp())  grp1_();
  if(b.imm())  imm1();
  if(b.dir())  dir1();
  if(b.tfs())  tfs1();
  if(b.kfs())  kfs1();
  if(b.lok())  lok1();
  if(b.a_s())  a_s1();
  if(b.zon())  zon1();
  if(b.hub())  hub1();
  if(b.xma())  xma1();
  if(b.cfm())  cfm1();
  if(b.hir())  hir1();
  if(b.cov())  cov1();
  if(b.sig())  sig1();
  if(b.let())  let1();
  if(b.uns())  uns1();

  if(b.del())  del1();
  if(b.nwm())  nwm1();
  if(b.rot())  rot1();
  if(b.hex())  hex1();
  if(b.upd())  upd1();
  if(b.tou())  tou1();
  if(b.fmu())  fmu1();
  if(b.scn())  scn1();
  if(b.pos())  pos1();
  if(b.r_o())  r_o1();
  if(b.nok())  nok1();
  if(b.fax())  fax1();
  if(b.prn())  prn1();
}
 

//  ------------------------------------------------------------------
//  Create string representation of an attribute set

gstring& ftn_attr::make_string(gstring& s) const {

  s = "";

  if(del())  s += "DEL ";
  if(rcv())  s += "Rcv ";
  if(snt())  s += "Snt ";
  if(uns())  s += "Uns ";
  if(imm())  s += "Imm ";
  if(dir())  s += "Dir ";
  if(lok())  s += "Lok ";
  if(tfs())  s += "Tfs ";
  if(kfs())  s += "Kfs ";
  if(pvt())  s += "Pvt ";
  if(cra())  s += "Cra ";
  if(loc())  s += "Loc ";
  if(hld())  s += "Hld ";
  if(k_s())  s += "K/s ";
  if(frq())  s += "Frq ";
  if(urq())  s += "URq ";
  if(att())  s += "Att ";
  if(arq())  s += "ARq ";
  if(rrq())  s += "RRq ";
  if(rrc())  s += "RRc ";
  if(trs())  s += "Trs ";
  if(orp())  s += "Orp ";
  if(a_s())  s += "A/s ";
  if(zon())  s += "Zon ";
  if(hub())  s += "Hub ";
  if(xma())  s += "Xma ";
  if(cfm())  s += "Cfm ";
  if(hir())  s += "Hir ";
  if(cov())  s += "Cov ";
  if(sig())  s += "Sig ";
  if(let())  s += "Let ";
  if(fax())  s += "Fax ";
  if(rsv())  s += "Rsv ";
  if(grp())  s += "Grp ";
  if(scn())  s += "Scn ";
  if(prn())  s += "Prn ";

  return s.trim();
}


//  ------------------------------------------------------------------

void ftn_attr::get(const gstring& s) {

  if(s.iinc("PVT"))  pvt1();
  if(s.iinc("CRA"))  cra1();
  if(s.iinc("RCV"))  rcv1();
  if(s.iinc("SNT"))  snt1();
  if(s.iinc("ATT"))  att1();
  if(s.iinc("TRS"))  trs1();
  if(s.iinc("ORP"))  orp1();
  if(s.iinc("K/S"))  k_s1();
  if(s.iinc("LOC"))  loc1();
  if(s.iinc("HLD"))  hld1();
  if(s.iinc("RSV"))  rsv1();
  if(s.iinc("FRQ"))  frq1();
  if(s.iinc("RRQ"))  rrq1();
  if(s.iinc("RRC"))  rrc1();
  if(s.iinc("ARQ"))  arq1();
  if(s.iinc("URQ"))  urq1();

  if(s.iinc("GRP"))  grp1_();
  if(s.iinc("IMM"))  imm1();
  if(s.iinc("DIR"))  dir1();
  if(s.iinc("TFS"))  tfs1();
  if(s.iinc("KFS"))  kfs1();
  if(s.iinc("LOK"))  lok1();
  if(s.iinc("A/S"))  a_s1();
  if(s.iinc("ZON"))  zon1();
  if(s.iinc("HUB"))  hub1();
  if(s.iinc("XMA"))  xma1();
  if(s.iinc("CFM"))  cfm1();
  if(s.iinc("HIR"))  hir1();
  if(s.iinc("COV"))  cov1();
  if(s.iinc("SIG"))  sig1();
  if(s.iinc("LET"))  let1();
  if(s.iinc("UNS"))  uns1();

  if(s.iinc("DEL"))  del1();
  if(s.iinc("NWM"))  nwm1();
  if(s.iinc("ROT"))  rot1();
  if(s.iinc("HEX"))  hex1();
  if(s.iinc("UPD"))  upd1();
  if(s.iinc("TOU"))  tou1();
  if(s.iinc("FMU"))  fmu1();
  if(s.iinc("SCN"))  scn1();
  if(s.iinc("POS"))  pos1();
  if(s.iinc("R/O"))  r_o1();
  if(s.iinc("NOK"))  nok1();
  if(s.iinc("FAX"))  fax1();
  if(s.iinc("PRN"))  prn1();
}


//  ------------------------------------------------------------------

bool ftn_attr::equals(const ftn_attr& b) const {

  return (attr1 == b.attr1) and (attr2 == b.attr2);
}


//  ------------------------------------------------------------------

gfile& operator<<(gfile& fp, const ftn_attr& a) {

  fp << a.attr1 << a.attr2;
  return fp;
}


//  ------------------------------------------------------------------

gfile& operator>>(gfile& fp, ftn_attr& a) {

  fp >> a.attr1 >> a.attr2;
  return fp;
}


//  ------------------------------------------------------------------

