
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsglexa.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  RFC822 mail lexical analyzer.
//  ------------------------------------------------------------------


#include <gstrmail.h>
#include <gmsglexa.h>


//  ------------------------------------------------------------------

gmail_lexanalyzer::gmail_lexanalyzer() {

}


//  ------------------------------------------------------------------

gmail_lexanalyzer::~gmail_lexanalyzer() {

}


//  ------------------------------------------------------------------

void gmail_lexanalyzer::add_token(int id, const char* text, int length) {

  gmail_token t;
  t.id = id;
  t.text.copyn(text, length);
  token.push_back(t);
}


//  ------------------------------------------------------------------

void gmail_lexanalyzer::disassemble(const char* t) {

  const char* p = t;
  while(*p) {
    if(is_mail_lwsp(*p))
      add_token(gmail_token::space, " ", 1);
    p = strskip_lwsp(p);
    if(*p) {
      const char* o = p;
      if(is_mail_special(*p)) {
        if(*p == '\"') {
          // quoted-string = <"> *(qtext / quoted-pair) <">
          p++;
          while(*p) {
            if(*p == '\\') {
              // skip past quoted-pair
              p++;
            }
            else if(*p == '\"') {
              // end of quoted-string
              p++;
              break;
            }
            p++;
          }
          add_token(gmail_token::quoted_string, o, (int)(p-o));
        }
        else if(*p == '\x5B') {
          // domain-literal = "[" *(dtext / quoted-pair) "]"
          p++;
          while(*p) {
            if(*p == '\\') {
              // skip past quoted-pair
              p++;
            }
            else if(*p == '\x5D') {
              // end of domain-literal
              p++;
              break;
            }
            p++;
          }
          add_token(gmail_token::domain_literal, o, (int)(p-o));
        }
        else if(*p == '\x28') {
          // comment = "(" *(ctext / quoted-pair / comment) ")"
          p++;
          int nested = 1;
          while(*p) {
            if(*p == '\\') {
              // skip past quoted-pair
              p++;
            }
            else if(*p == '\x28') {
              // nested comment
              nested++;
            }
            else if(*p == '\x29') {
              // end of comment
              nested--;
              p++;
              if(!nested)
                break;
            }
            p++;
          }
          add_token(gmail_token::comment, o, (int)(p-o));
        }
        else {
          // individual special
          add_token(gmail_token::special, o, 1);
          p++;
        }
      }
      else {
        // atom
        p = strskip_to_delimiters(p);
        add_token(gmail_token::atom, o, (int)(p-o));
      }
    }
  }
}


//  ------------------------------------------------------------------

char* gmail_lexanalyzer::assemble(char* t, int l) {

  *t = NUL;
  char* p = t;
  for(int n=0; n<token.size(); n++) {
    if(l or (not token[n].is_comment()) and not token[n].is_space())
      p = stpcpy(p, token[n].text.c_str());
  }
  return t;
}


//  ------------------------------------------------------------------

