
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsglexa.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  RFC822 mail lexical analyzer.
//  ------------------------------------------------------------------

#ifndef __GMSGLEXA_H
#define __GMSGLEXA_H

#include <gstrgstr.h>
#include <gvector.h>


//  ------------------------------------------------------------------

class gmail_token {

public:

  enum {
    special,
    quoted_string,
    domain_literal,
    comment,
    atom,
    space
  };

  int id;
  gstring text;

  bool is_special() const        { return id == special; }
  bool is_quoted_string() const  { return id == quoted_string; }
  bool is_domain_literal() const { return id == domain_literal; }
  bool is_comment() const        { return id == comment; }
  bool is_atom() const           { return id == atom; }
  bool is_space() const          { return id == space; }
  bool is_word() const           { return is_atom() or is_quoted_string(); }

  gmail_token& operator=(const gmail_token& x) { id = x.id; text = x.text; return *this; }

};


//  ------------------------------------------------------------------

class gmail_lexanalyzer {

public:

  gvector<gmail_token> token;

  gmail_lexanalyzer();
  ~gmail_lexanalyzer();

  void add_token(int id, const char* text, int length);

  void disassemble(const char* t);

  char* assemble(char* t, int l);
  char* assemble_lexical(char* t)   { return assemble(t, true); }
  char* assemble_canonical(char* t) { return assemble(t, false); }

  int size() const { return token.size(); }

  gmail_token& operator[](int n) { return token[n]; }

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

