
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsgline.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Message lines.
//  ------------------------------------------------------------------


#include <gmsgline.h>


//  ------------------------------------------------------------------

int glinearray::setdata(int n, uint type, uint id) {

  gline& d = data(n);
  d.type = type;
  d.id = id;

  return n;
}


//  ------------------------------------------------------------------

int glinearray::add(const char* s, uint type, uint id) {

  return setdata(gstrarray::add(s), type, id);
}


//  ------------------------------------------------------------------

int glinearray::insert(int n, const char* s, uint type, uint id) {

  return setdata(gstrarray::insert(n, s), type, id);
}


//  ------------------------------------------------------------------

int glinearray::set(int n, const char* s) {

  return gstrarray::set(n, s);
}


//  ------------------------------------------------------------------

int glinearray::set(int n, const char* s, uint type, uint id) {

  return setdata(gstrarray::set(n, s), type, id);
}


//  ------------------------------------------------------------------

void glinearray::add(glineref r, int lines) {

  while(lines) {
    add(r.text(), r.type(), r.id());
    lines--;
    r++;
  }
}


//  ------------------------------------------------------------------

void glinearray::insert(int n, glineref r, int lines) {

  while(lines) {
    insert(n++, r.text(), r.type(), r.id());
    lines--;
    r++;
  }
}


//  ------------------------------------------------------------------

inline void glineref::set_peek()                    { peekaboo = line->text(current); }

glineref::glineref()                                { line = NULL; current = 0; peekaboo = NULL; }
glineref::glineref(glineref& r)                     { line = r.line; current = r.current; set_peek(); }
glineref::glineref(glinearray& l, int n)            { line = &l; current = n; set_peek(); }
glineref::glineref(glinearray* l, int n)            { line = l; current = n; set_peek(); }

int glineref::length() const                       { return line->length(current); }

char* glineref::set_text()                          { return line->text(current); }
void glineref::set_text(const char* s)              { line->set(current, s); }

const char* glineref::text() const                  { return line->text(current); }
uint& glineref::type() const                        { return line->type(current); }
uint& glineref::id() const                          { return line->id(current); }

bool glineref::iskludge() const                     { return line->iskludge(current); }
bool glineref::isheader() const                     { return line->isheader(current); }
bool glineref::isheaderend() const                  { return line->isheaderend(current); }
bool glineref::isboundary() const                   { return line->isboundary(current); }
bool glineref::ishidden() const                     { return line->ishidden(current); }
bool glineref::isbody() const                       { return line->isbody(current); }
bool glineref::isquote() const                      { return line->isquote(current); }
bool glineref::issignature() const                  { return line->issignature(current); }
bool glineref::istagline() const                    { return line->istagline(current); }
bool glineref::istearline() const                   { return line->istearline(current); }
bool glineref::isgateorigin() const                 { return line->isgateorigin(current); }
bool glineref::isorigin() const                     { return line->isorigin(current); }
bool glineref::ishard() const                       { return line->ishard(current); }
bool glineref::iswrapped() const                    { return line->iswrapped(current); }
bool glineref::isblock() const                      { return line->isblock(current); }
bool glineref::isposition() const                   { return line->isposition(current); }
bool glineref::ishighlighted() const                { return line->ishighlighted(current); }

void glineref::operator++(int)                      { if(current < line->lines()) current++; set_peek(); }
void glineref::operator--(int)                      { if(current) current--; set_peek(); }

glineref glineref::operator+(int n)                 { return glineref(line, current+n); }
glineref glineref::operator-(int n)                 { return glineref(line, current-n); }

void glineref::operator+=(int n)                    { current += n; set_peek(); }
void glineref::operator-=(int n)                    { current -= n; set_peek(); }

void glineref::operator=(int n)                     { current = n; set_peek(); }
void glineref::operator=(const glineref& b)         { line = b.line; current = b.current; set_peek(); }
void glineref::operator=(glinearray& b)             { line = &b; current = 0; set_peek(); }

void glineref::go_first()                           { current = 0; set_peek(); }
void glineref::go_last()                            { current = line->lines() ? line->lines()-1 : 0; set_peek(); }

bool glineref::at_last() const                      { return current+1 == line->lines(); }
bool glineref::more() const                         { return current+1 < line->lines(); }
bool glineref::valid() const                        { return current < line->lines(); }
bool glineref::beyond_last() const                  { return current >= line->lines(); }

glineref glineref::next()                           { return glineref(line, current+1); }
glineref glineref::previous()                       { return glineref(line, current-1); }

void glineref::erase(int lines)                     { line->erase(current, current+lines); set_peek(); }

void glineref::insert_below(const char* s)                  { current++; line->insert(current, s); set_peek(); }
void glineref::insert_below(const char* s, uint t, uint i)  { current++; line->insert(current, s, t, i); set_peek(); }
void glineref::insert_below(glineref r, int l)              { current++; line->insert(current, r, l); set_peek(); }

void glineref::insert(const char* s)                        { line->insert(current, s); set_peek(); }
void glineref::insert(const char* s, uint t, uint i)        { line->insert(current, s, t, i); set_peek(); }
void glineref::insert(glineref r, int l)                    { line->insert(current, r, l); set_peek(); }


//  ------------------------------------------------------------------

glinearray::glinearray() : gstrarray(sizeof(gline)) {

  crlf_is_newline = false;
  lf_is_newline = false;
  cr_is_newline = true;
  softcr_is_newline = false;
  pcboard_cr_is_newline = false;

  strip_lf = true;
  //strip_lf = false;
  strip_nul = false;
  strip_softcr = false;
  nul_terminates = true;
  convert_nul_to_space = false;
}


//  ------------------------------------------------------------------

glinearray::~glinearray() {

}
  

//  ------------------------------------------------------------------

void glinearray::raw_to_lines(uint len) {

  bool end_of_text = false;

  register char* iptr = block;
  register uint  ilen = len;

  while(not end_of_text) {

    bool hard = false;
    char* optr = iptr;
    char* bptr = iptr;
    register uint olen = 0;
    register char ch = NUL;

    // Main loop
    while(ilen) {
      ch = *iptr;
      if(ch == '\r') {
        // 0x0D Carriage Return
        if(cr_is_newline or crlf_is_newline) {
          newline:
          hard = true;
          *optr++ = NUL;
          iptr++;
          ilen--;
          goto loop_complete;
        }
        goto any_char;
      }
      else if(ch == '\n') {
        // 0x0A Line Feed
        if(lf_is_newline)
          goto newline;
        if(strip_lf)
          goto strip_char;
        goto any_char;
      }
      else if(ch == '\x8D') {
        // 0x8D FTN "Soft" Carriage Return
        if(strip_softcr)
          goto strip_char;
        if(softcr_is_newline)
          goto newline;
        goto any_char;
      }
      else if(ch == NUL) {
        // 0x00 NUL termination character
        if(nul_terminates) {
          end_of_text = true;
          ilen = 0;
          if(olen == 0)
            goto loop_end;
          else
            goto loop_complete;
        }
        if(strip_nul)
          goto strip_char;
        if(convert_nul_to_space)
          ch = ' ';
        goto any_char;
      }
      else if(ch == '\xE3') {
        // 0xE3 PCBoard original line termination character
        if(pcboard_cr_is_newline)
          goto newline;
        goto any_char;
      }
      else {
        // Any other character
        any_char:
        *optr++ = ch;
        olen++;
      }

      strip_char:
      iptr++;
      ilen--;
      continue;
    }

    loop_complete:

    if((ch == NUL) or (ilen == 0)) {
      end_of_text = true;
      *optr = NUL;
    }

    if(ilen or olen) {
      olen++;
      glineinfo* line = (glineinfo*)adjust_index(blocklines+1) + blocklines;
      line->offset = (uint)(bptr-block);
      line->length = olen;
      line->type = hard ? gline::hard : 0;
      line->id = 0;
      blocklen += olen;
      blocklines++;
    }

    loop_end:
    ;
  }
}


//  ------------------------------------------------------------------

char* glinearray::lines_to_raw() {

  return NULL;
}


//  ------------------------------------------------------------------

