
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsgline.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Message lines.
//  ------------------------------------------------------------------


#ifndef __GMSGLINE_H
#define __GMSGLINE_H


//  ------------------------------------------------------------------

#include <gstrblks.h>


//  ------------------------------------------------------------------

class gline {

public:

  enum {

    hard         = 0x0001,
    block        = 0x0002,
    highlighted  = 0x0002,
    position     = 0x0004,
    folded       = 0x0008,
    wrapped      = 0x0008,

    kludge       = 0x0010,
    hidden       = 0x0020,
    quote        = 0x0040,
    signature    = 0x0080,

    tagline      = 0x0100,
    tearline     = 0x0200,
    origin       = 0x0400,
    gateorigin   = 0x0800,

    body         = 0x1000,
    header       = 0x2000,
    header_end   = 0x4000,
    boundary     = 0x8000,

    boundary_end = boundary|header_end,

    control     = kludge|hidden|tagline|tearline|origin|gateorigin|signature|boundary
  };

  uint type;
  uint id;

  bool iskludge() const       { return !!(type & gline::kludge); }
  bool isheader() const       { return !!(type & gline::header); }
  bool isheaderend() const    { return !!(type & gline::header_end); }
  bool isboundary() const     { return !!(type & gline::boundary); }
  bool ishidden() const       { return !!(type & gline::hidden); }
  bool isbody() const         { return !!(type & gline::body); }
  bool isquote() const        { return !!(type & gline::quote); }
  bool issignature() const    { return !!(type & gline::signature); }
  bool istagline() const      { return !!(type & gline::tagline); }
  bool istearline() const     { return !!(type & gline::tearline); }
  bool isgateorigin() const   { return !!(type & gline::gateorigin); }
  bool isorigin() const       { return !!(type & gline::origin); }
  bool ishard() const         { return !!(type & gline::hard); }
  bool isblock() const        { return !!(type & gline::block); }
  bool isposition() const     { return !!(type & gline::position); }
  bool ishighlighted() const  { return !!(type & gline::highlighted); }
  bool iswrapped() const      { return not ishard(); }

};


//  ------------------------------------------------------------------

struct glineinfo {
  uint offset;
  uint length;
  uint type;
  uint id;
};


//  ------------------------------------------------------------------

class glineref;

class glinearray : public gstrarray {

public:

  bool   crlf_is_newline;
  bool   lf_is_newline;
  bool   cr_is_newline;
  bool   nul_is_newline;
  bool   softcr_is_newline;
  bool   pcboard_cr_is_newline;

  bool   strip_lf;
  bool   strip_nul;
  bool   strip_softcr;
  bool   nul_terminates;
  bool   convert_nul_to_space;

  glinearray();
  ~glinearray();

  char*  pointer()                    { return block; }
  void   set_length(uint len)         { blocklen = len; }

  void   raw_to_lines(uint len);
  char*  lines_to_raw();

  int    lines() const                { return size(); }

  char*  text(int n) const            { return get(n); }
  uint&  type(int n) const            { return data(n).type; }
  uint&  id(int n) const              { return data(n).id; }

  bool   iskludge(int n) const        { return data(n).iskludge(); }
  bool   isheader(int n) const        { return data(n).isheader(); }
  bool   isheaderend(int n) const     { return data(n).isheaderend(); }
  bool   isboundary(int n) const      { return data(n).isboundary(); }
  bool   ishidden(int n) const        { return data(n).ishidden(); }
  bool   isbody(int n) const          { return data(n).isbody(); }
  bool   isquote(int n) const         { return data(n).isquote(); }
  bool   issignature(int n) const     { return data(n).issignature(); }
  bool   istagline(int n) const       { return data(n).istagline(); }
  bool   istearline(int n) const      { return data(n).istearline(); }
  bool   isgateorigin(int n) const    { return data(n).isgateorigin(); }
  bool   isorigin(int n) const        { return data(n).isorigin(); }
  bool   ishard(int n) const          { return data(n).ishard(); }
  bool   iswrapped(int n) const       { return data(n).iswrapped(); }
  bool   isblock(int n) const         { return data(n).isblock(); }
  bool   isposition(int n) const      { return data(n).isposition(); }
  bool   ishighlighted(int n) const   { return data(n).ishighlighted(); }

  gline& data(int n) const            { return *((gline*)lineextraptr(n)); }

  void   add(glineref r, int lines=1);
  void   insert(int n, glineref r, int lines=1);

  int    add(const char* s, uint type=0, uint id=0);
  int    set(int n, const char* s);
  int    set(int n, const char* s, uint type, uint id);
  int    insert(int n, const char* s, uint type=0, uint id=0);
  int    setdata(int n, uint type=0, uint id=0);

};


//  ------------------------------------------------------------------

class glineref {

protected:

  glinearray* line;
  int         current;
  char*       peekaboo;

  void        set_peek();

public:

  glineref();
  glineref(glineref& r);
  glineref(glinearray& l, int n=0);
  glineref(glinearray* l, int n);

  int         length() const;

  char*       set_text();
  void        set_text(const char* s);
  void        set_text(const gstring& s)           { set_text(s.c_str()); }

  const char* text() const;
  uint&       type() const;
  uint&       id() const;

  bool        iskludge() const;
  bool        isheader() const;
  bool        isheaderend() const;
  bool        isboundary() const;
  bool        ishidden() const;
  bool        isbody() const;
  bool        isquote() const;
  bool        issignature() const;
  bool        istagline() const;
  bool        istearline() const;
  bool        isgateorigin() const;
  bool        isorigin() const;
  bool        ishard() const;
  bool        iswrapped() const;
  bool        isblock() const;
  bool        isposition() const;
  bool        ishighlighted() const;

  void        operator++(int);
  void        operator--(int);

  glineref    operator+(int n);
  glineref    operator-(int n);

  void        operator+=(int n);
  void        operator-=(int n);

  void        operator=(int n);
  void        operator=(const glineref& b);
  void        operator=(glinearray& b);

  bool        operator==(const glineref& b) const  { return current == b.current; }
  bool        operator!=(const glineref& b) const  { return current != b.current; }

  operator    int() const                          { return current; }
  int         lineno() const                       { return current; }

  void        go_first();
  void        go_last();

  bool        at_last() const;
  bool        more() const;
  bool        valid() const;
  bool        beyond_last() const;

  glineref    next();
  glineref    previous();

  void        erase(int lines=1);

  void        insert_below(const char* s);
  void        insert_below(const char* s, uint type, uint id);
  void        insert_below(glineref r, int lines=1);

  void        insert(const char* s);
  void        insert(const char* s, uint type, uint id);
  void        insert(glineref r, int lines=1);

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

