
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsgquot.cpp,v 1.2 1999/01/16 03:07:44 damware Exp $
//  ------------------------------------------------------------------
//  Message structures and classes.
//  ------------------------------------------------------------------


#include <gmsg.h>


//  ------------------------------------------------------------------

char gmsg_quote::quotechars[] = { ">" };


//  ------------------------------------------------------------------

gmsg_quote::gmsg_quote(const char* qstr) {

  strxcpy(quotechars, qstr, sizeof(quotechars));
}


//  ------------------------------------------------------------------

bool gmsg_quote::is_quotechar(const char* s) {

  if(*s) {
    if(*s == '>')
      return true;
    if(*quotechars)
      if(strchr(quotechars, *s))
        return true;
  }

  return false;
}


//  ------------------------------------------------------------------

bool gmsg_quote::is_quote(const char* s) {

  const char* ptr = s;
  const char* endptr = ptr + 7;

  // Skip leading whitespace
  while((*ptr) and (*ptr == ' '))
    ptr++;

  // Check for empty string
  if(*ptr == NUL)
    return false;

  // Check for userdefined quotechars after first whitespace
  if(*quotechars)
    if(strchr(quotechars, *ptr))
      return true;

  while((ptr < endptr) and *ptr) {

    if(*ptr == '>') {
      // Found true quote char
      return true;
    }
    else if((((byte)*ptr) < ' ') or strchr("<\"\'", *ptr)) {
      // Found a char that cannot occur in a quotestring
      return false;
    }
    ptr++;
  }

  return false;
}


//  ------------------------------------------------------------------

int gmsg_quote::get_quotestring(const char* s, char* qbuf, int& qlen) {

  if(is_quote(s)) {

    const char* lp = s;

    // Naughty goto-point for quotestring skipping
    MoreQuotes:

    // Skip leading spaces
    while(*lp == ' ')
      lp++;

    // Find the end of the quotestring
    if(is_quotechar(lp)) {
      // Type 1 : ">xxxx>"
      lp++;
      while(*lp == ' ')
        lp++;
      if(is_quote(lp))
        goto MoreQuotes;
      if(not is_quotechar(lp-1)) {
        while(not is_quotechar(lp))
          lp--;
        lp++;
      }
    }
    else {
      // Type 2: "xxxx>"
      while(not (is_quotechar(lp) and (not is_quotechar(lp+1))) and *lp)
        ++lp;
      if(is_quote(lp))
        goto MoreQuotes;
      if(*lp)
        lp++;
    }

    // lp now points to the character after the quotestring

    // Include a trailing space
    qlen = (int)(lp - s);
    if(*lp == ' ')
      qlen++;

    // Transfer the quotestring
    memcpy(qbuf, s, qlen);
  }
  else {
    qlen = 0;
  }

  qbuf[qlen] = NUL;

  return qlen;
}


//  ------------------------------------------------------------------

bool gmsg_quote::equal(const char* q1, const char* q2) {

  q1--;
  q2--;

  do {
    do {
      q1++;
    } while(isspace(*q1));
    do {
      q2++;
    } while(isspace(*q2));
    if(*q2 == 0)
      return true;
    if(*q1 != *q2)
      return false;
  } while(*q1 and *q2);

  return *q1 == *q2;
}


//  ------------------------------------------------------------------

