
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsgtime.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Message date and time class.
//  ------------------------------------------------------------------


#include <stdio.h>
#include <string.h>
#include <gstrall.h>
#include <gutlmisc.h>
#include <gmsgtime.h>


//  ------------------------------------------------------------------

char gmsg_time::datebuf[80];
uint gmsg_time::daylist[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
uint gmsg_time::ydaylist[] = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };


//  ------------------------------------------------------------------

gmsg_time::gmsg_time() {

  reset();
}
 

//  ------------------------------------------------------------------

void gmsg_time::reset() {

  year       =
  month      =
  day        =
  hour       =
  minute     =
  second     =
  utc_offset = 0;
}
 

//  ------------------------------------------------------------------

void gmsg_time::copy(const gmsg_time& t) {

  year       = t.year;
  month      = t.month;
  day        = t.day;
  hour       = t.hour;
  minute     = t.minute;
  second     = t.second;
  utc_offset = t.utc_offset;
}
 

//  ------------------------------------------------------------------

const char* gmsg_time::weekday() const {

  return __gsweekday[weekday_number()];
}


//  ------------------------------------------------------------------

int gmsg_time::weekday_number() const {

  return day_of_week(year, month, day);
}


//  ------------------------------------------------------------------

int gmsg_time::year_twodigits() const {

  return year % 100;
}


//  ------------------------------------------------------------------

int gmsg_time::year_day() const {

  gmsg_time t = *this;
  t.month = 1;
  t.day = 1;

  return (int)((unixtime() - t.unixtime()) / 86400L);
}


//  ------------------------------------------------------------------

int gmsg_time::utc_offset_minutes() const {

  return (utc_offset/100)*60 + (utc_offset%100);
}


//  ------------------------------------------------------------------

bool gmsg_time::valid() const {

  if(year == 0 or year > 2108)
    return false;
  if((month < 1) or (month > 12))
    return false;
  if((day < 1) or (day > 31))
    return false;
  if(hour > 24)
    return false;
  if(minute > 62)
    return false;
  if(second > 62)
    return false;
  return true;
}


//  ------------------------------------------------------------------

void gmsg_time::validate() {

  if(not valid())
    reset();
}


//  ------------------------------------------------------------------

ulong gmsg_time::unixtime_utc() const {

  return unixtime() - utc_offset_minutes()*60;
}
 

//  ------------------------------------------------------------------

ulong gmsg_time::unixtime() const {

  uint uyear   = year;
  uint umonth  = month - 1;
  uint uday    = day;
  uint uhour   = hour;
  uint uminute = minute;
  uint usecond = second;

  uminute += usecond / 60;
  usecond %= 60;              // Seconds are normalized
  uhour += uminute / 60;
  uminute %= 60;              // Minutes are normalized
  uday += uhour / 24;
  uhour %= 24;                // Hours are normalized

  uyear += umonth / 12;        // Normalize month (not necessarily final)
  umonth %= 12;

  while(uday >= daylist[umonth]) {
    if(!(uyear & 3) and (umonth == 1)) {
      if(uday > 28) {
        uday -= 29;
        umonth++;
      }
      else
        break;
    }
    else {
      uday -= daylist[umonth];
      umonth++;
    }
    uyear += umonth / 12;      // Normalize month
    umonth %= 12;
  }

  uyear -= 1970;
  int leaps = (uyear + 2) / 4;
  if(not ((uyear+1970) & 3) and (umonth < 2))
    leaps--;

  ulong days = uyear*365L + leaps + ydaylist[umonth] + uday;

  return days*86400L + uhour*3600L + uminute*60L + usecond;
}
 

//  ------------------------------------------------------------------

gfiletime gmsg_time::filetime_utc() const {

  gmsg_time t = unixtime_utc();
  return t.filetime();
}


//  ------------------------------------------------------------------

gfiletime gmsg_time::filetime() const {

  gfiletime t;

  t.ft_year  = year - 1980;
  t.ft_month = (unsigned)month;
  t.ft_day   = (unsigned)day;
  t.ft_hour  = (unsigned)hour;
  t.ft_min   = (unsigned)minute;
  t.ft_tsec  = second / 2;

  return t;
}


//  ------------------------------------------------------------------

gopustime gmsg_time::opustime_utc() const {

  gmsg_time t = unixtime_utc();
  return t.opustime();
}


//  ------------------------------------------------------------------

gopustime gmsg_time::opustime() const {

  gopustime t;

  t.ft_year  = year - 1980;
  t.ft_month = (unsigned)month;
  t.ft_day   = (unsigned)day;
  t.ft_hour  = (unsigned)hour;
  t.ft_min   = (unsigned)minute;
  t.ft_tsec  = second / 2;

  return t;
}


//  ------------------------------------------------------------------

const char* gmsg_time::fts1_utc() {

  gmsg_time t = unixtime_utc();
  return t.fts1();
}

  
//  ------------------------------------------------------------------

const char* gmsg_time::fts1() {

  // 1234567890123456789
  // dd mmm yy  HH:MM:SS

  return format("%d %b %y  %H:%M:%S");
}


//  ------------------------------------------------------------------

const char* gmsg_time::rfc822() {

  // 1234567890123456789012345678901
  // www, dd mmm yyyy HH:MM:SS shhmm

  return format("%a, %d %b %Y %H:%M:%S %z");
}


//  ------------------------------------------------------------------

const char* gmsg_time::format(const char* fmt, bool alt_lang) {

  int i;
  char buf[25];
  char* p = buf;
  char* s = datebuf;

  *s = NUL;

  if(valid()) {

    while(*fmt) {
      if(*fmt != '%')
        *s++ = *fmt;
      else {
        strcpy(buf, "00");
        fmt++;
        switch(*fmt) {
          case 'a':
            p = alt_lang ? gsweekday[weekday_number()] : __gsweekday[weekday_number()];
            break;
          case 'A':
            p = alt_lang ? glweekday[weekday_number()] : __glweekday[weekday_number()];
            break;
          case 'b':
            p = alt_lang ? gsmonth[month-1] : __gsmonth[month-1];
            break;
          case 'B':
            p = alt_lang ? glmonth[month-1] : __glmonth[month-1];
            break;
          case 'd':
            itoa(day, buf + zero_or_one(day < 10), 10);
            break;
          case 'H':
            itoa(hour, buf + zero_or_one(hour < 10), 10);
            break;
          case 'I':
            i = (hour == 12) ? 12 : hour % 12;
            itoa(i, buf + zero_or_one(i < 10), 10);
            break;
          case 'j':
            i = year_day() + 1;
            itoa(i, buf + zero_or_one(i < 10) + zero_or_one(i < 100), 10);
            break;
          case 'm':
            i = month;
            itoa(i, buf + zero_or_one(i < 10), 10);
            break;
          case 'M':
            itoa(minute, buf + zero_or_one(minute < 10), 10);
            break;
          case 'p':
            p = gampm[hour / 12];
            break;
          case 'S':
            itoa(second, buf + zero_or_one(second < 10), 10);
            break;
          case 'U':
            i = weekday_number() - (year_day() % 7);
            if(i < 0)
              i += 7;
            i = (year_day() + i) / 7;
            itoa(i, buf + zero_or_one(i < 10), 10);
            break;
          case 'w':
            itoa(weekday_number(), buf, 10);
            break;
          case 'W':
            i = weekday_number() - (year_day() % 7);
            if(i < 0)
              i += 7;
            i = (year_day() + i - 1) / 7;
            i++;  // NON-STANDARD: make weekno in range 01..53
            itoa(i, buf + zero_or_one(i < 10), 10);
            break;
          case 'y':
            i = year_twodigits();
            itoa(i, buf + zero_or_one(i < 10), 10);
            break;
          case 'Y':
            itoa(year, buf, 10);
            break;
          case 'z': // NON-STANDARD: offset from UTC in +/-HHMM format
            i = absolute(utc_offset);
            *buf = (utc_offset < 0) ? '-' : '+';
            itoa(i, buf + 1 + zero_or_one(i < 1000), 10);
            break;
          case '%':
            p = "%";
        }
        strcpy(s, p);
        s += strlen(p);
      }
      p = buf;
      fmt++;
    }
  }

  return datebuf;
}


//  ------------------------------------------------------------------

void gmsg_time::convert_filetime(gfiletime f) {

  reset();

  if((f.ft_day >= 1) and (f.ft_day <= 31)) {
    if((f.ft_month >= 1) and (f.ft_month <= 12)) {
      year   = f.ft_year + 1980;
      month  = f.ft_month;
      day    = f.ft_day;
      hour   = f.ft_hour;
      minute = f.ft_min;
      second = f.ft_tsec * 2;
    }
  }
}


//  ------------------------------------------------------------------

void gmsg_time::convert_opustime(gopustime f) {

  reset();

  if((f.ft_day >= 1) and (f.ft_day <= 31)) {
    if((f.ft_month >= 1) and (f.ft_month <= 12)) {
      year   = f.ft_year + 1980;
      month  = f.ft_month;
      day    = f.ft_day;
      hour   = f.ft_hour;
      minute = f.ft_min;
      second = f.ft_tsec * 2;
    }
  }
}


//  ------------------------------------------------------------------

void gmsg_time::convert_unixtime(ulong u) {

  reset();

  if(u) {

    second = (uint)(u % 60);
    u /= 60;                              // Time in minutes
    minute = (uint)(u % 60);
    u /= 60;                              // Time in hours
    uint i = (uint)(u / (1461L * 24L));   // Number of 4 year blocks
    year = (i << 2);
    year += 1970;
    u %= 1461L * 24L;                     // Hours since end of last 4 year block

    for(;;) {
      uint hpery = 365 * 24;
      if((year & 3) == 0)
        hpery += 24;
      if(u < hpery)
        break;
      year++;
      u -= hpery;
    }                                     // At end, time is number of hours into current year

    hour = (uint)(u % 24);
    u /= 24;                              // Time in days
    u++;

    if((year & 3) == 0) {
      if(u > 60)
        u--;
      else if(u == 60) {
        month = 1;
        day = 29;
        return;
      }
    }

    for(month=0; daylist[month] < u; month++)
      u -= daylist[month];

    day = (uint)u;
    month++;
  }
}


//  ------------------------------------------------------------------

void gmsg_time::convert_fts1(const char* s) {

  reset();

  bool date_ok = false;

  s = strskip_wht(s);
  if(not isdigit(*s)) {
    // Skip past weekday string (SEA format)
    s = strskip_wht(strskip_txt(s));
  }
  if(*s) {
    if(isdigit(*s)) {
      day = atoi(s);
      s = strskip_wht(strskip_txt(s));
      if(isalpha(*s)) {
        month = str2mon(s);
        if(month) {
          s = strskip_wht(strskip_txt(s));
          if(isdigit(*s)) {
            year = atoi(s);
            if(year < 80)
              year += 2000;
            else if(year < 100)
              year += 1900;
            s = strskip_wht(strskip_txt(s));
            if(isdigit(*s)) {
              hour = atoi(s);
              s = strskip_digits(s);
              if(*s and isdigit(s[1])) {
                minute = atoi(++s);
                date_ok = true;
                // The seconds part is only in the FTS-1 format
                s = strskip_digits(s);
                if(*s and isdigit(s[1])) {
                  second = atoi(++s);
                }
              }
            }
          }
        }
      }
    }
  }

  if(not date_ok)
    reset();
}


//  ------------------------------------------------------------------

void gmsg_time::convert_rfc822(const char* s) {

  // RFC822 Date: BNF
  //
  // date-time   =  [ day "," ] date time        ; dd mm yy
  //                                             ;  hh:mm:ss zzz
  //
  // day         =  "Mon"  / "Tue" /  "Wed"  / "Thu"
  //             /  "Fri"  / "Sat" /  "Sun"
  //
  // date        =  1*2DIGIT month 2DIGIT        ; day month year
  //                                             ;  e.g. 20 Jun 82
  //
  // month       =  "Jan"  /  "Feb" /  "Mar"  /  "Apr"
  //             /  "May"  /  "Jun" /  "Jul"  /  "Aug"
  //             /  "Sep"  /  "Oct" /  "Nov"  /  "Dec"
  //
  // time        =  hour zone                    ; ANSI and Military
  //
  // hour        =  2DIGIT ":" 2DIGIT [":" 2DIGIT]
  //                                             ; 00:00:00 - 23:59:59
  //
  // zone        =  "UT"  / "GMT"                ; Universal Time
  //                                             ; North American : UT
  //             /  "EST" / "EDT"                ;  Eastern:  - 5/ - 4
  //             /  "CST" / "CDT"                ;  Central:  - 6/ - 5
  //             /  "MST" / "MDT"                ;  Mountain: - 7/ - 6
  //             /  "PST" / "PDT"                ;  Pacific:  - 8/ - 7
  //             /  1ALPHA                       ; Military: Z = UT;
  //                                             ;  A:-1; (J not used)
  //                                             ;  M:-12; N:+1; Y:+12
  //             / ( ("+" / "-") 4DIGIT )        ; Local differential
  //                                             ;  hours+min. (HHMM)

  reset();

  bool date_ok = false;

  s = strskip_wht(s);
  if(not isdigit(*s)) {
    // Skip past weekday string
    s = strskip_wht(strskip_txt(s));
  }
  if(*s) {
    if(isdigit(*s)) {
      day = atoi(s);
      s = strskip_wht(strskip_txt(s));
      if(isalpha(*s)) {
        month = str2mon(s);
        if(month) {
          s = strskip_wht(strskip_txt(s));
          if(isdigit(*s)) {
            year = atoi(s);
            if(year < 80)
              year += 2000;
            else if(year < 100)
              year += 1900;
            s = strskip_wht(strskip_txt(s));
            if(isdigit(*s)) {
              hour = atoi(s);
              s = strskip_digits(s);
              if(*s and isdigit(s[1])) {
                minute = atoi(++s);
                date_ok = true;
                // The seconds part is optional
                s = strskip_digits(s);
                if(*s and isdigit(s[1])) {
                  second = atoi(++s);
                  s = strskip_txt(s);
                }
                s = strskip_wht(s);
                // Get timezone
                if(isalpha(*s)) {
                  const char* zone = s;
                  char buf[100];
                  s = strskip_txt(s);
                  strxcpy(buf, zone, 1+(s-zone));
                  if(strlen(buf) == 1) {
                    // Military format
                    switch(toupper(*buf)) {
                      case 'A': utc_offset =  -100; break;
                      case 'B': utc_offset =  -200; break;
                      case 'C': utc_offset =  -300; break;
                      case 'D': utc_offset =  -400; break;
                      case 'E': utc_offset =  -500; break;
                      case 'F': utc_offset =  -600; break;
                      case 'G': utc_offset =  -700; break;
                      case 'H': utc_offset =  -800; break;
                      case 'I': utc_offset =  -900; break;
                      case 'K': utc_offset = -1000; break;
                      case 'L': utc_offset = -1100; break;
                      case 'M': utc_offset = -1200; break;
                      case 'N': utc_offset =   100; break;
                      case 'O': utc_offset =   200; break;
                      case 'P': utc_offset =   300; break;
                      case 'Q': utc_offset =   400; break;
                      case 'R': utc_offset =   500; break;
                      case 'S': utc_offset =   600; break;
                      case 'T': utc_offset =   700; break;
                      case 'U': utc_offset =   800; break;
                      case 'V': utc_offset =   900; break;
                      case 'W': utc_offset =  1000; break;
                      case 'X': utc_offset =  1100; break;
                      case 'Y': utc_offset =  1200; break;
                    }
                  }
                  else {
                    // Try North American zones
                    if(strieql(buf, "EST"))
                      utc_offset = -500;
                    else if(strieql(buf, "EDT"))
                      utc_offset = -400;
                    else if(strieql(buf, "CST"))
                      utc_offset = -600;
                    else if(strieql(buf, "CDT"))
                      utc_offset = -500;
                    else if(strieql(buf, "MST"))
                      utc_offset = -700;
                    else if(strieql(buf, "MDT"))
                      utc_offset = -600;
                    else if(strieql(buf, "PST"))
                      utc_offset = -800;
                    else if(strieql(buf, "PDT"))
                      utc_offset = -700;
                  }
                }
                else {
                  // Local differential [+/-]hhmm
                  if(*s == '+')
                    s++;
                  utc_offset = atoi(s);
                }
              }
            }
          }
        }
      }
    }
  }

  if(not date_ok)
    reset();
}


//  ------------------------------------------------------------------

