
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsgtime.h,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Message date and time class.
//  ------------------------------------------------------------------


#ifndef __GMSGTIME_H
#define __GMSGTIME_H


//  ------------------------------------------------------------------

#include <gtimall.h>


//  ------------------------------------------------------------------

class gmsg_time {

protected:

  static uint daylist[12];
  static uint ydaylist[12];
  static char datebuf[80];
  
public:

  uint year;
  uint month;
  uint day;
  uint hour;
  uint minute;
  uint second;
  int  utc_offset;

  gmsg_time();
  gmsg_time(ulong t)                  { convert_unixtime(t); }
  gmsg_time(gfiletime t)              { convert_filetime(t); }
  gmsg_time(gopustime t)              { convert_opustime(t); }
  gmsg_time(const gmsg_time& t)       { copy(t); }

  gmsg_time&  operator=(ulong t)      { convert_unixtime(t); return *this; }
  gmsg_time&  operator=(gfiletime t)  { convert_filetime(t); return *this; }
  gmsg_time&  operator=(gopustime t)  { convert_opustime(t); return *this; }
  gmsg_time&  operator=(const gmsg_time& t)  { copy(t); return *this; }

  void        copy(const gmsg_time& t);

  const char* weekday() const;
  int         weekday_number() const;

  int         year_twodigits() const;
  int         year_day() const;

  int         utc_offset_minutes() const;

  void        reset();

  bool        valid() const;
  void        validate();

  gfiletime   filetime() const;
  gfiletime   filetime_utc() const;

  gopustime   opustime() const;
  gopustime   opustime_utc() const;

  ulong       unixtime() const;
  ulong       unixtime_utc() const;

  const char* fts1();
  const char* fts1_utc();

  const char* rfc822();

  const char* format(const char* fmt, bool alt_lang=false);
  const char* format_alt_lang(const char* fmt) { return format(fmt, true); }

  void        convert_filetime(gfiletime t);
  void        convert_opustime(gopustime t);
  void        convert_unixtime(ulong t);
  void        convert_fts1(const char* s);
  void        convert_rfc822(const char* s);

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

