
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsgwrap.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Message structures and classes.
//  ------------------------------------------------------------------


#include <gmsg.h>


//  ------------------------------------------------------------------

void gmsg::wrap(const char* text, const char* prefix, gstring& first, gstring& second, int width) {

  // width = 40
  //                   1111111111222222222233333333334444444444
  //         01234567890123456789012345678901234567890123456789
  //         --------------------------------------------------
  //                                                v- end
  // Case 1: this is another test with a bit of text to wrap.
  // Case 2: this is a test with a line that need wrapping.
  // Case 3: thisxisxaxtestxwithxaxlinexthatxneedxwrapping.
  // Case 4: >thisxisxaxtestxwithxaxlinexthatxneedxwrapping.

  const char* end = text + width - 1;

  // Locate the word to be wrapped

  if(*end == ' ') {

    // Case 1: A space was found as the last char inside the margin
    //
    // Now we must locate the first word outside the margin.
    // NOTE: Leading spaces to this word will be stripped!

    // Begin at the first char outside the margin
    end++;
  }
  else {

    // Case 2: A non-space was found as the last char inside the margin
    //
    // Now we must locate the beginning of the word we found.

    // Keep copy of original pointer
    const char* at_end = end;

    // Search backwards until a space or the beginning of the line is found
    while((end > text) and (*(end-1) != ' '))
      end--;

    // Check if we hit leading spaces
    const char* space = end;
    if(space > text) {
      space--;
      while((space > text) and (*space == ' '))
        space--;
    }

    // Did we search all the way back to the beginning of the line?
    if((end == text) or (*space == ' ')) {

      // Case 3: There are no spaces within the margin or we hit leading spaces

      // We have to break it up at the margin
      end = at_end + 1;
    }
  }

  // The end pointer now points to the location to be wrapped

  first.copy(text, end);

  second = prefix;
  second += end;
}


//  ------------------------------------------------------------------

void gmsg::wrap_lines(int textwidth, int quotewidth) {

  glineref line = view;
  glineref nextline = line;
  gmsg_quote quoting(">");

  gstring textbuffer, first, second;

  while(line.valid()) {

    int length = line.length();
    int wrapwidth = line.isquote() ? quotewidth : textwidth;

    if(length > wrapwidth) {

      if(line.more())
        nextline = line.lineno() + 1;

      if(line.isquote()) {

        int  quotelength;
        char quotebuffer[20];
        int  nextquotelength = 0;
        char nextquotebuffer[20];

        quoting.get_quotestring(line.text(), quotebuffer, quotelength);

        if(line.more()) {
          if(nextline.isquote()) {
            quoting.get_quotestring(nextline.text(), nextquotebuffer, nextquotelength);
            if(not strieql(quotebuffer, nextquotebuffer))
              nextquotelength = 0;
            else if(line.text()[quotelength] == nextline.text()[quotelength])
              nextquotelength = 0;
          }
        }

        if(nextquotelength) {

          textbuffer = quotebuffer;
          while(nextquotelength) {
            nextquotelength = 0;
            textbuffer += nextline.text() + quotelength;
            textbuffer.trim();
            textbuffer += " ";
            nextline.erase();
            if(nextline.valid()) {
              if(nextline.isquote()) {
                quoting.get_quotestring(nextline.text(), nextquotebuffer, nextquotelength);
                if(not strieql(quotebuffer, nextquotebuffer))
                  nextquotelength = 0;
              }
            }
          }
          textbuffer.trim();

          do {
            wrap(textbuffer.c_str()+quotelength, quotebuffer, first, second, wrapwidth-quotelength);
            line.set_text(first.c_str());
            line.insert_below(second.c_str(), line.type(), line.id());
            textbuffer = second;
          } while(textbuffer.length() > wrapwidth);
        }
        else {
          wrap(line.text()+quotelength, quotebuffer, first, second, wrapwidth-quotelength);
          line.set_text(first.c_str());
          line.insert_below(second.c_str(), line.type(), line.id());
        }
        line++;
      }
      else {
        uint type = line.type();
        wrap(line.text(), "", first, second, wrapwidth);
        line.set_text(first.c_str());
        line.type() &= ~gline::hard;
        line.type() |= gline::wrapped;
        line.insert_below(second.c_str(), type, line.id());
      }
    }
    else {
      line++;
    }
  }
}


//  ------------------------------------------------------------------

