
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmsgxlat.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Message character decoding and translation.
//  ------------------------------------------------------------------


#include <gmsg.h>
#include <gmsgmime.h>
#include <gstrmail.h>
#include <gutlcode.h>
#include <gvector.h>


//  ------------------------------------------------------------------

void gmsg::translate(gcharxlat& charset) {

  enum { not_coded, base64, quoted_printable };

  int decode = not_coded;
  gstring newtext;
  gmime_content_type content_type;
  const char* cptr;
  const char* bptr;
  int id, idlen, field;
  bool text_plain = true;
  char buffer1[256];
  char buffer2[256];
  int lineno = 0;
  bool soft_line_break = false;
  bool previous_soft_line_break = false;
  base64_engine b64;

  glineref rawline(text);

  while(rawline.valid()) {

    cptr = rawline.text();

    if(rawline.isboundary()) {
      charset.reset_from_charset();
      text_plain = true;
    }

    switch(rawline.id()) {
      case gkludge::i51:
        charset.set_from_codepage(0);
        charset.set_from_charset("iso-8859-1");
        break;

      case gkludge::chrs:
      case gkludge::charset:
        split_controlline(cptr, id, idlen, field);
        charset.set_from_charset(cptr+field);
        break;

      case gkludge::codepage:
        split_controlline(cptr, id, idlen, field);
        charset.set_from_codepage(atoi(cptr+field));
        break;

      case gheader::content_type:
        text_plain = false;
        split_controlline(cptr, id, idlen, field);
        content_type.crack(cptr+field);
        if(content_type.type.ieql("text")) {
          if(content_type.subtype.ieql("plain"))
            text_plain = true;
          for(int p=0; p<content_type.parameters.size(); p++) {
            if(content_type.parameters[p].attribute.ieql("charset")) {
              charset.set_from_charset(content_type.parameters[p].value.c_str());
              break;
            }
          }
        }
        break;

      case gheader::content_transfer_encoding:
        split_controlline(cptr, id, idlen, field);
        if(strieql(cptr+field, "base64"))
          decode = base64;
        else if(strieql(cptr+field, "quoted-printable"))
          decode = quoted_printable;
        break;
    }

    if(rawline.isheaderend()) {
      if(decode and not text_plain)
        decode = not_coded;
    }

    if(rawline.isbody() and *cptr) {
      if(decode) {
        if(decode == base64)
          b64.decode(buffer1, cptr);
        else if(decode == quoted_printable) {
          const char* eptr = cptr + strlen(cptr) - 1;
          while(is_mail_lwsp(*eptr) and (eptr > cptr))
            eptr--;
          if(*eptr == '=') {
            soft_line_break = true;
            eptr--;
          }
          const char* iptr = cptr;
          char* optr = buffer1;
          while(iptr <= eptr) {
            char c = *iptr;
            if(c == '=') {
              char c1 = iptr[1];
              char c2 = iptr[2];
              if(isxdigit(c1) and isxdigit(c2)) {
                c = (char)((xtoi(c1) << 4) | xtoi(c2));
                iptr += 2;
              }
            }
            *optr++ = c;
            iptr++;
          }
          *optr = NUL;
        }
        bptr = charset.translate(buffer1, buffer2);
        if(previous_soft_line_break) {
          newtext = view[lineno-1];
          newtext += bptr;
          view.set(lineno-1, newtext.c_str());
        }
        else {
          view.add(bptr, rawline.type(), rawline.id());
          lineno++;
        }
        previous_soft_line_break = soft_line_break;
        soft_line_break = false;
      }
      else {
        bptr = charset.translate(cptr, buffer2);
        view.add(bptr, rawline.type(), rawline.id());
        lineno++;
      }
    }
    else {
      view.add(rawline);
      lineno++;
    }

    rawline++;
  }
}


//  ------------------------------------------------------------------

