
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gnntp.cpp,v 1.2 1998/12/27 16:08:12 odinn Exp $
//  ------------------------------------------------------------------
//  NNTP class implementation.
//  ------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gsock.h>


//  ------------------------------------------------------------------

void gnntp::crack_list() {


}


//  ------------------------------------------------------------------

void gnntp::crack_list_active_times() {


}


//  ------------------------------------------------------------------

void gnntp::crack_list_distributions() {


}


//  ------------------------------------------------------------------

void gnntp::crack_list_distrib_pats() {


}


//  ------------------------------------------------------------------

void gnntp::crack_list_newsgroups() {


}


//  ------------------------------------------------------------------

void gnntp::crack_list_overview_fmt() {


}


//  ------------------------------------------------------------------

void gnntp::crack_xover() {


}


//  ------------------------------------------------------------------

gnntp::gnntp(gsocket& s) : socket(s) {

  mode = mode_none;
  rspcode = 0;
  *rspbuffer = NUL;
  *cmdbuffer = NUL;
  *linebuffer = NUL;
  global_posting_allowed = false;
  current_article_number = 0;
  *current_article_messageid = NUL;

  cracked_group = NULL;
  cracked_last_article = 0;
  cracked_first_article = 0;
  cracked_posting_allowed = false;

  cracked_timecreated = 0;
  cracked_creator_email = NULL;

  cracked_distribution_value = NULL;
  cracked_distribution_explanation = NULL;

  cracked_weight = 0;
  cracked_group_pattern = NULL;

  cracked_group_description = NULL;

  cracked_header = NULL;
  cracked_header_full = false;

  cracked_article_number = 0;
  cracked_subject = NULL;
  cracked_author = NULL;
  cracked_date = NULL;
  cracked_message_id = NULL;
  cracked_references = NULL;
  cracked_byte_count = 0;
  cracked_line_count = 0;
  cracked_xover_other = NULL;
}


//  ------------------------------------------------------------------

gnntp::~gnntp() {

  if(socket.is_open())
    close();
}


//  ------------------------------------------------------------------

int gnntp::open(const char* server) {

  if(not socket.open(server, "nntp"))
    return false;
  if(get_response()) {
    if((response_code() != 200) and (response_code() != 201))
      return false;
    global_posting_allowed = response_code() == 200;
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

void gnntp::close() {

  if(socket.is_open()) {
    send_command("QUIT");
    socket.close();
  }
}


//  ------------------------------------------------------------------

int gnntp::get_response() {

  *rspbuffer = NUL;
  if(socket.strget(rspbuffer, sizeof(rspbuffer)) == 0) {
    rspcode = atoi(rspbuffer);
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

const char* gnntp::getline() {

  lineptr = linebuffer;
  if(socket.strget(linebuffer, sizeof(linebuffer)) != 0)
    lineptr = NULL;
  else if(*linebuffer == '.') {
    if(linebuffer[1])
      lineptr = linebuffer+1;
    else
      lineptr = NULL;
  }
  if(mode and lineptr) {
    switch(mode) {
      case mode_list:               crack_list();               break;
      case mode_list_active_times:  crack_list_active_times();  break;
      case mode_list_distributions: crack_list_distributions(); break;
      case mode_list_distrib_pats:  crack_list_distrib_pats();  break;
      case mode_list_newsgroups:    crack_list_newsgroups();    break;
      case mode_list_overview_fmt:  crack_list_overview_fmt();  break;
      case mode_xover:              crack_xover();              break;
    }
  }
  else if(!lineptr) {
    mode = mode_none;
  }
  return lineptr;
}


//  ------------------------------------------------------------------

int gnntp::putline(const char* line) {

  if(line == NULL) {
    strcpy(linebuffer, ".");
  }
  else if(*line == '.') {
    *linebuffer = '.';
    strcpy(linebuffer+1, line);
  }
  else {
    strcpy(linebuffer, line);
  }
  int ok = socket.strput(linebuffer) == 0;
  if(ok) {
    if((line == NULL) and (mode == mode_post)) {
      mode = mode_none;
      if(get_response())
        if(response_code() == 240)
          return true;
      return false;
    }
  }
  return ok;
}


//  ------------------------------------------------------------------

int gnntp::send_command(const char* command) {

  if(socket.strput(command ? command : cmdbuffer) == 0)
    if(get_response())
      return true;
  return false;
}


//  ------------------------------------------------------------------

int gnntp::send_command(int ok_code) {

  return send_command(cmdbuffer, ok_code);
}


//  ------------------------------------------------------------------

int gnntp::send_command(const char* command, int ok_code) {

  if(send_command(command))
    if(response_code() == ok_code)
      return true;
  mode = mode_none;
  return false;
}


//  ------------------------------------------------------------------

int gnntp::send_command(const char* command, int ok_code1, int ok_code2) {

  if(send_command(command))
    if((response_code() >= ok_code1) and (response_code() <= ok_code2))
      return true;
  mode = mode_none;
  return false;
}


//  ------------------------------------------------------------------

int gnntp::specify_article(const char* command) {

  if(send_command(command, 220, 223)) {
    get_rsp_num_msgid();
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int gnntp::specify_article(const char* command, const char* messageid) {

  sprintf(cmdbuffer, "%s%s%s", command, messageid ? " " : "", messageid ? messageid : "");
  return specify_article(cmdbuffer);
}


//  ------------------------------------------------------------------

int gnntp::specify_article(const char* command, ulong number) {

  sprintf(cmdbuffer, "%s %lu", command, number);
  return specify_article(cmdbuffer);
}


//  ------------------------------------------------------------------

int gnntp::article(const char* messageid) {

  return specify_article("ARTICLE", messageid);
}


//  ------------------------------------------------------------------

int gnntp::article(ulong number) {

  return specify_article("ARTICLE", number);
}


//  ------------------------------------------------------------------

int gnntp::head(const char* messageid) {

  return specify_article("HEAD", messageid);
}


//  ------------------------------------------------------------------

int gnntp::head(ulong number) {

  return specify_article("HEAD", number);
}


//  ------------------------------------------------------------------

int gnntp::body(const char* messageid) {

  return specify_article("BODY", messageid);
}


//  ------------------------------------------------------------------

int gnntp::body(ulong number) {

  return specify_article("BODY", number);
}


//  ------------------------------------------------------------------

int gnntp::stat(const char* messageid) {

  return specify_article("STAT", messageid);
}


//  ------------------------------------------------------------------

int gnntp::stat(ulong number) {

  return specify_article("STAT", number);
}


//  ------------------------------------------------------------------

int gnntp::group(const char* newsgroup_name) {

  build_command("GROUP", newsgroup_name);
  if(send_command(211)) {
    sscanf(response_text(), "%lu %lu %lu",
      &number_of_articles,
      &first_article_number,
      &last_article_number
    );
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int gnntp::help() {

  return send_command("HELP", 100);
}


//  ------------------------------------------------------------------

void gnntp::get_rsp_num_msgid() {

  sscanf(response_text(), "%lu %s",
    &current_article_number,
    current_article_messageid
  );
}


//  ------------------------------------------------------------------

int gnntp::last_or_next(const char* command) {

  if(send_command(command, 223)) {
    get_rsp_num_msgid();
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int gnntp::last() {

  return last_or_next("LAST");
}


//  ------------------------------------------------------------------

int gnntp::next() {

  return last_or_next("NEXT");
}


//  ------------------------------------------------------------------

int gnntp::mode_reader() {

  return send_command("MODE READER", 200, 201);
}


//  ------------------------------------------------------------------

int gnntp::list(const char* pattern) {

  mode = mode_list;
  strcpy(cmdbuffer, "LIST");
  if(pattern) {
    append_cmd_argument("ACTIVE");
    append_cmd_argument(pattern);
  }
  return send_command(215);
}


//  ------------------------------------------------------------------

int gnntp::newgroups(const char* date, const char* time, const char* gmt, const char* distributions) {

  mode = mode_list;
  sprintf(cmdbuffer, "NEWGROUPS %s %s %s", date, time, gmt);
  append_cmd_argument(distributions);
  return send_command(231);
}


//  ------------------------------------------------------------------

int gnntp::newnews(const char* newsgroups, const char* date, const char* time, const char* gmt, const char* distribution) {

  sprintf(cmdbuffer, "NEWNEWS %s %s %s %s", newsgroups, date, time, gmt);
  append_cmd_argument(distribution);
  return send_command(230);
}


//  ------------------------------------------------------------------

int gnntp::post() {

  if(send_command("POST", 340)) {
    mode = mode_post;
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

void gnntp::append_cmd_argument(const char* argument) {

  if(argument) {
    strcat(cmdbuffer, " ");
    strcat(cmdbuffer, argument);
  }
}


//  ------------------------------------------------------------------

void gnntp::build_command(const char* command, const char* argument) {

  strcpy(cmdbuffer, command);
  append_cmd_argument(argument);
}


//  ------------------------------------------------------------------

int gnntp::listgroup(const char* newsgroup) {
  
  build_command("LISTGROUP", newsgroup);
  return send_command(215);
}


//  ------------------------------------------------------------------

int gnntp::list_newsgroups(const char* pattern) {

  mode = mode_list_newsgroups;
  build_command("LIST NEWSGROUPS", pattern);
  return send_command(215);
}


//  ------------------------------------------------------------------

int gnntp::list_active_times() {

  mode = mode_list_active_times;
  return send_command("LIST ACTIVE.TIMES", 215);
}


//  ------------------------------------------------------------------

int gnntp::list_distributions() {

  mode = mode_list_distributions;
  return send_command("LIST DISTRIBUTIONS", 215);
}


//  ------------------------------------------------------------------

int gnntp::list_distrib_pats() {

  mode = mode_list_distrib_pats;
  return send_command("LIST DISTRIB.PATS", 215);
}


//  ------------------------------------------------------------------

int gnntp::list_subscriptions() {

  return send_command("LIST SUBSCRIPTIONS", 215);
}


//  ------------------------------------------------------------------

int gnntp::xgtitle(const char* pattern) {

  mode = mode_list_newsgroups;
  build_command("XGTITLE", pattern);
  return send_command(282);
}


//  ------------------------------------------------------------------

int gnntp::xhdr(const char* header, const char* range) {

  sprintf(cmdbuffer, "XHDR %s", header);
  append_cmd_argument(range);
  return send_command(221);
}


//  ------------------------------------------------------------------

int gnntp::list_overview_fmt() {

  mode = mode_list_overview_fmt;
  return send_command("LIST OVERVIEW.FMT", 215);
}


//  ------------------------------------------------------------------

int gnntp::xover(const char* range) {

  mode = mode_xover;
  build_command("XOVER", range);
  return send_command(224);
}


//  ------------------------------------------------------------------

