
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gpop3.cpp,v 1.2 1998/12/27 16:08:12 odinn Exp $
//  ------------------------------------------------------------------
//  POP3 class implementation.
//  ------------------------------------------------------------------


#include <stdio.h>
#include <gstrall.h>
#include <gsock.h>


//  ------------------------------------------------------------------

int gpop3::get_response() {

  *rspbuffer = NUL;
  if(socket.strget(rspbuffer, sizeof(rspbuffer)) == 0) {
    rspstatus = strnieql(rspbuffer, "+OK", 3);
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

void gpop3::crack_list() {

  sscanf(line_pointer(), "%lu %lu",
    &msg_number,
    &msg_size
  );
}


//  ------------------------------------------------------------------

const char* gpop3::getline() {

  lineptr = linebuffer;
  if(socket.strget(linebuffer, sizeof(linebuffer)) != 0)
    lineptr = NULL;
  else if(*linebuffer == '.') {
    if(linebuffer[1])
      lineptr = linebuffer+1;
    else
      lineptr = NULL;
  }
  if(list_mode and lineptr)
    crack_list();
  else if(!lineptr)
    list_mode = false;
  return lineptr;
}


//  ------------------------------------------------------------------

int gpop3::send_command(const char* command) {

  if(socket.strput(command ? command : cmdbuffer) == 0)
    if(get_response())
      return positive_status();
  return false;
}


//  ------------------------------------------------------------------

gpop3::gpop3(gsocket& s) : socket(s) {

  list_mode = false;
  rspstatus = false;
  *rspbuffer = NUL;
  *cmdbuffer = NUL;
  *linebuffer = NUL;
  number_of_messages = 0;
  maildrop_size = 0;
  msg_number = 0;
  msg_size = 0;
}


//  ------------------------------------------------------------------

gpop3::~gpop3() {

  if(socket.is_open())
    close();
}


//  ------------------------------------------------------------------

int gpop3::open(const char* server, const char* userid, const char* password) {

  if(not socket.open(server, "pop3"))
    return false;
  if(get_response()) {
    if(positive_status()) {
      if(user(userid)) {
        if(pass(password)) {
          return true;
        }
      }
    }
  }
  return false;
}


//  ------------------------------------------------------------------

void gpop3::close() {

  send_command("QUIT");
  socket.close();
}


//  ------------------------------------------------------------------

int gpop3::user(const char* userid) {

  sprintf(cmdbuffer, "USER %s", userid);
  return send_command();
}


//  ------------------------------------------------------------------

int gpop3::pass(const char* password) {

  sprintf(cmdbuffer, "PASS %s", password);
  return send_command();
}


//  ------------------------------------------------------------------

int gpop3::stat() {

  if(send_command("STAT")) {
    sscanf(response_text(), "%lu %lu",
      &number_of_messages,
      &maildrop_size
    );
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int gpop3::list(ulong number) {

  strcpy(cmdbuffer, "LIST");
  if(number)
    sprintf(cmdbuffer+4, " %lu", number);
  if(send_command()) {
    if(number) {
      sscanf(response_text(), "%lu %lu",
        &msg_number,
        &msg_size
      );
    }
    else {
      list_mode = true;
    }
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int gpop3::retr(ulong number) {

  sprintf(cmdbuffer, "RETR %lu", number);
  return send_command();
}


//  ------------------------------------------------------------------

int gpop3::dele(ulong number) {

  sprintf(cmdbuffer, "DELE %lu", number);
  return send_command();
}


//  ------------------------------------------------------------------

int gpop3::rset() {

  return send_command("RSET");
}


//  ------------------------------------------------------------------

int gpop3::noop() {

  return send_command("NOOP");
}


//  ------------------------------------------------------------------

int gpop3::top(ulong number, ulong lines) {

  sprintf(cmdbuffer, "TOP %lu %lu", number, lines);
  return send_command();
}


//  ------------------------------------------------------------------

