
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gsearch.h,v 1.1.1.1 1998/11/21 19:38:09 odinn Exp $
//  ------------------------------------------------------------------
//  Search.
//  ------------------------------------------------------------------


#ifndef __GSEARCH_H
#define __GSEARCH_H

#include <gstrgstr.h>


//  ------------------------------------------------------------------

class gregex;
class gfuzzy;
class gbmh;


//  ------------------------------------------------------------------

class gsearch {

public:

  enum patterntype {
    regex,                // Extended regular expressions
    wildcard,             // Shell-style with wildcards
    fuzzy,                // Fuzzy matching
    plain                 // Plain BMH match
  };

protected:

  class gregex* regexp;
  class gfuzzy* fuzzyp;
  class gbmh*   plainp;

public:

  // Configuration
  gstring        id;
  gstring        pattern;
  patterntype    type;
  bool           case_sensitive;
  bool           reverse;
  int            fuzzydegree;
  int            score_success;
  int            score_failure;

  gsearch();
  virtual ~gsearch();

  void set_id(const char* a)          { id = a; }
  void set_type(patterntype a)        { type = a; }
  void set_pattern(const char* a);
  void set_case_sensitive(bool a)     { case_sensitive = a; }
  void set_reverse(bool a)            { reverse = a; }
  void set_fuzzydegree(int a)         { fuzzydegree = a; }
  void set_score_success(int a)       { score_success = a; }
  void set_score_failure(int a)       { score_failure = a; }

  // Result of last search
  bool found;
  int score;

  // Search a string for the pattern.
  // Return true for success, false for failure.
  bool search(const char* string);
  bool search(const gstring& string);
  bool search(const char* string, int& result);
  bool search(const gstring& string, int& result);

  gsearch& operator=(const gsearch& a);

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

