
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gsmtp.cpp,v 1.2 1998/12/27 16:08:12 odinn Exp $
//  ------------------------------------------------------------------
//  SMTP class implementation.
//  ------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include <gstrall.h>
#include <gsock.h>


//  ------------------------------------------------------------------

int gsmtp::get_response() {

  strcpy(rspbuffer, "000 ");
  do {
    if(socket.strget(rspbuffer, sizeof(rspbuffer)) != 0) {
      rspcode = 0;
      return false;
    }
  } while(rspbuffer[3] == '-');
  rspcode = atoi(rspbuffer);
  return true;
}


//  ------------------------------------------------------------------

int gsmtp::send_command(const char* command) {

  if(socket.strput(command ? command : cmdbuffer) == 0)
    if(get_response())
      return true;
  return false;
}


//  ------------------------------------------------------------------

int gsmtp::send_command(int ok_code) {

  return send_command(cmdbuffer, ok_code);
}


//  ------------------------------------------------------------------

int gsmtp::send_command(const char* command, int ok_code) {

  if(send_command(command))
    if(response_code() == ok_code)
      return true;
  return false;
}


//  ------------------------------------------------------------------

int gsmtp::send_command(int ok_code1, int ok_code2) {

  if(send_command())
    if((response_code() >= ok_code1) and (response_code() <= ok_code2))
      return true;
  return false;
}


//  ------------------------------------------------------------------

int gsmtp::putline(const char* line) {

  if(line == NULL) {
    strcpy(linebuffer, ".");
  }
  else if(*line == '.') {
    *linebuffer = '.';
    strcpy(linebuffer+1, line);
  }
  else {
    strcpy(linebuffer, line);
  }
  int ok = socket.strput(linebuffer) == 0;
  if(ok) {
    if(line == NULL) {
      if(get_response())
        if(response_code() == 250)
          return true;
      return false;
    }
  }
  return ok;
}


//  ------------------------------------------------------------------

void gsmtp::append_cmd_argument(const char* argument) {

  if(argument) {
    strcat(cmdbuffer, " ");
    strcat(cmdbuffer, argument);
  }
}


//  ------------------------------------------------------------------

gsmtp::gsmtp(gsocket& s) : socket(s) {

  rspcode = 0;
  *rspbuffer = NUL;
  *cmdbuffer = NUL;
  *linebuffer = NUL;
}


//  ------------------------------------------------------------------

gsmtp::~gsmtp() {

  if(socket.is_open())
    close();
}


//  ------------------------------------------------------------------

int gsmtp::open(const char* server, const char* domain) {

  if(not socket.open(server, "smtp"))
    return false;
  if(get_response()) {
    if(response_code() == 220) {
      sprintf(cmdbuffer, "EHLO %s", domain);
      if(send_command(250)) {
        // Get service extensions
        return true;
      }
      else {
        sprintf(cmdbuffer, "HELO %s", domain);
        if(send_command(250))
          return true;
      }
    }
  }

  return false;
}


//  ------------------------------------------------------------------

void gsmtp::close() {

  send_command("QUIT");
  socket.close();
}


//  ------------------------------------------------------------------

int gsmtp::mail(const char* address) {

  sprintf(cmdbuffer, "MAIL FROM:<%s>", address);
  return send_command(250);
}


//  ------------------------------------------------------------------

int gsmtp::rcpt(const char* address) {

  sprintf(cmdbuffer, "RCPT TO:<%s>", address);
  return send_command(250, 251);
}


//  ------------------------------------------------------------------

int gsmtp::data() {

  return send_command("DATA", 354);
}


//  ------------------------------------------------------------------

int gsmtp::help(const char* command) {

  strcpy(cmdbuffer, "HELP");
  append_cmd_argument(command);
  return send_command(211, 214);
}


//  ------------------------------------------------------------------

int gsmtp::rset() {

  return send_command("RSET", 250);
}


//  ------------------------------------------------------------------

