
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gsnd.cpp,v 1.1.1.1 1998/11/21 19:38:06 odinn Exp $
//  ------------------------------------------------------------------
//  Sound class.
//  ------------------------------------------------------------------


#include <gdefs.h>

#if defined(__DOS__)
  #include <gmemi86.h>
  #include <gmemdbg.h>
#elif defined(__OS2__)
  #include <stdio.h>
  #include <stdlib.h>
  #define INCL_DOSMODULEMGR
  #include <os2.h>
  #ifndef __EMX__
  extern "C" {
  #include <os2me.h>
  }
  #endif
#elif defined(__WIN32__) and not defined(__EMX__)
  #include <stdio.h>
  #include <windows.h>
  #include <mmsystem.h>
#endif

#include <gstrall.h>
#include <gasmamis.h>
#include <gsndall.h>

#if defined(__OS2__) and not defined(__EMX__)
#define __OS2_MM__ 1
#endif

#if defined(__WIN32__) and not defined(__EMX__)
#define __WIN32_MM__ 1
#endif


//  ------------------------------------------------------------------

#if defined(__OS2_MM__) or defined(__WIN32_MM__)

HMODULE mdmHandle;
const int BUFFERSIZE = 128;

#endif


//  ------------------------------------------------------------------

#if defined(__OS2_MM__)

extern "C" {

DWORD (APIENTRY *mciSendStringPtr)(LPSTR a, LPSTR b, WORD c, HWND d, WORD e);
DWORD (APIENTRY *mciGetErrorStringPtr)(DWORD a, LPSTR b, WORD c);

inline DWORD APIENTRY mciSendString(LPSTR a, LPSTR b, WORD c, HWND d, WORD e)  { return (*mciSendStringPtr)(a, b, c, d, e); }
inline DWORD APIENTRY mciGetErrorString(DWORD a, LPSTR b, WORD c)  { return (*mciGetErrorStringPtr)(a, b, c); }

}


//  ------------------------------------------------------------------

int SendmciString(char* string, char* his_buffer) {

  char our_buffer[BUFFERSIZE], *return_buffer;

  return_buffer = his_buffer ? his_buffer : our_buffer;
  memset(return_buffer, 0, BUFFERSIZE);
  
  DWORD rc = mciSendString((LPSTR)string, (LPSTR)return_buffer, BUFFERSIZE, 0, 0);

  if(rc == MCIERR_SUCCESS)
    return 1;
  else {
    mciGetErrorString(rc, (LPSTR)return_buffer, BUFFERSIZE);
    return 0;
  }
}


//  ------------------------------------------------------------------

#elif defined(__WIN32_MM__)

int SendmciString(char* string, char* his_buffer) {

  char our_buffer[BUFFERSIZE], *return_buffer;

  return_buffer = his_buffer ? his_buffer : our_buffer;
  memset(return_buffer, 0, BUFFERSIZE);
  
  MCIERROR rc = mciSendString(string, return_buffer, BUFFERSIZE, NULL);

  if(rc == 0)
    return 1;
  else {
    mciGetErrorString(rc, return_buffer, BUFFERSIZE);
    return 0;
  }
}


#endif


//  ------------------------------------------------------------------

#ifdef DEBUG
int debug = false;
#endif


//  ------------------------------------------------------------------

gsnd::gsnd() {

  #if defined(__DOS__)
  mpx = -1;
  #if (defined(__WATCOMC__) and defined(__MSDOS__) and defined(__386__)) or defined(__DJGPP__)
  buffer = -1;
  #else
  buffer = NULL;
  #endif
  data = NULL;
  #endif

  api_open = false;
  file_open = false;
}


//  ------------------------------------------------------------------

gsnd::~gsnd() {

  if(api_open)
    close_api();

  free_buffer();
}


//  ------------------------------------------------------------------

#if defined(__DOS__)
int gsnd::call_api(uint al, uint bx) {

  if(api_open) {
    i86 cpu;
    cpu.al(al);
    cpu.ah(mpx);
    cpu.bx(bx);
    cpu.genint(0x2D);
    return (int)cpu.ax();
  }
  return 0;
}
#endif


//  ------------------------------------------------------------------

void gsnd::free_buffer() {

  #if defined(__DOS__)
  #if (defined(__WATCOMC__) and defined(__MSDOS__) and defined(__386__)) or defined(__DJGPP__)
  if(buffer != -1) {
    i86 cpu;
    cpu.ax(0x101);
    cpu.dx(buffer);
    cpu.genint(0x31);
    buffer = -1;
  }
  #else
  if(buffer) {
    farfree(buffer);
    buffer = NULL;
  }
  #endif
  #endif
}


//  ------------------------------------------------------------------

int gsnd::open_api() {

  #if defined(__DOS__)

  i86 cpu;

  // Installation check
  for(mpx=0; mpx<256; mpx++) {
    cpu.ah(mpx);
    cpu.al(GSAPI_INSTALL_CHECK);
    cpu.genint(0x2D);
    if(cpu.al() == 0xFF) {
      #ifdef DEBUG
      if(debug) printf("gsnd: examining signature at %04X:%04X for mpx %u\n", cpu.dx(), cpu.di(), mpx);
      #endif
      amis_signature* signature = (amis_signature*)gmkfp(cpu.dx(), cpu.di());
      #ifdef DEBUG
      if(debug) printf("gsnd: version is %04X\n", cpu.cx());
      if(debug) printf("gsnd: manufacturer is \"%8.8s\"\n", signature->manufacturer);
      if(debug) printf("gsnd: product name is \"%8.8s\"\n", signature->product_name);
      if(debug) printf("gsnd: description  is \"%s\"\n", signature->product_description);
      #endif
      if((cpu.cx() >= 0x0100) and (cpu.cx() < 0x0200))
        if(strnieql(signature->product_name, "GoldSAPI", 8))
          break;
    }
  }

  // Return if not installed
  if(mpx >= 256) {
    mpx = -1;
    return -1;
  }

  // Call the open api function
  cpu.al(GSAPI_OPEN_API);
  cpu.ah(mpx);
  cpu.genint(0x2D);
  if(cpu.al()) {
    #ifdef DEBUG
    if(debug) printf("gsnd: open api returned %02Xh\n", cpu.al());
    #endif
    return cpu.al();
  }
  #ifdef DEBUG
  if(debug) printf("gsnd: using data at %04X:%04X, length %u\n", cpu.dx(), cpu.di(), cpu.cx());
  #endif
  data = (gsapidata*)gmkfp(cpu.dx(), cpu.di());
  key_value = cpu.bx();

  api_open = true;
  return 0;

  #elif defined(__OS2_MM__)

  CHAR tmp[100];
  APIRET rc;

  rc = DosLoadModule((PSZ)tmp, sizeof(tmp), (PSZ)"MDM.DLL", &mdmHandle);
  if(rc != 0)
    return -1;

  #ifdef __WATCOMC__
  rc = DosQueryProcAddr(mdmHandle, 0, (PSZ)"mciSendString", (PFN)&mciSendStringPtr);
  #else
  rc = DosQueryProcAddr(mdmHandle, 0, (PSZ)"mciSendString", &(PFN)mciSendStringPtr);
  #endif
  if(rc != 0)
    return -1;

  #ifdef __WATCOMC__
  rc = DosQueryProcAddr(mdmHandle, 0, (PSZ)"mciGetErrorString", (PFN)&mciGetErrorStringPtr);
  #else
  rc = DosQueryProcAddr(mdmHandle, 0, (PSZ)"mciGetErrorString", &(PFN)mciGetErrorStringPtr);
  #endif
  if(rc != 0)
    return -1;

  api_open = true;
  return 0;

  #elif defined(__WIN32_MM__)

  api_open = true;
  return 0;

  #else

  api_open = false;
  return -1;

  #endif
}


//  ------------------------------------------------------------------

int gsnd::close_api() {

  #if defined(__DOS__)

  return call_api(GSAPI_CLOSE_API, key_value);

  #elif defined(__OS2_MM__)

  if(api_open) {

    if(file_open)
      close();

    DosFreeModule(mdmHandle);

    api_open = false;
  }
  return 0;

  #elif defined(__WIN32_MM__)

  if(api_open) {

    if(file_open)
      close();

    api_open = false;
  }

  return 0;

  #else

  return -1;

  #endif
}


//  ------------------------------------------------------------------

int gsnd::open(const char* file) {

  #if defined(__DOS__)

  strcpy(data->parameters, file);

  try_again:
  int retval = call_api(GSAPI_OPEN_AND_LOAD_FILE);
  switch(retval) {
    case 0x01:
      close();
      goto try_again;
    case 0x02:
      free_buffer();
      #if (defined(__WATCOMC__) and defined(__MSDOS__) and defined(__386__)) or defined(__DJGPP__)
      {
        i86 cpu;
        long memory_needed = (data->buffer_length + 16L) / 16L;
        // Determine how much is available
        cpu.ax(0x100);
        cpu.bx(65535u);
        cpu.genint(0x31);
        if(memory_needed > (long)cpu.bx()*16L)
          return -1;
        cpu.ax(0x100);
        cpu.bx(memory_needed);
        cpu.genint(0x31);
        data->buffer_segment = (word)cpu.ax();
        data->buffer_offset = 0;
        buffer = cpu.dx();
      }
      #else
      buffer = (char*)farmalloc(data->buffer_length);
      if(buffer == NULL)
        return false;
      data->buffer_segment = FP_SEG(buffer);
      data->buffer_offset = FP_OFF(buffer);
      #endif
      retval = call_api(GSAPI_OPEN_AND_LOAD_FILE);
  }

  if(retval == 0)
    file_open = true;

  #ifdef DEBUG
  if(debug) printf("gsnd::open: retval = %04X\n", retval);
  #endif

  return retval;

  #elif defined(__OS2_MM__) or defined(__WIN32_MM__)

  if(file_open)
    close();

  char buf[_MAX_PATH];
  sprintf(buf, "open %s alias noise wait", file);
  if(SendmciString(buf, NULL)) {
    file_open = true;
    return 0;
  }
  return -1;

  #else

  NW(file);
  return -1;

  #endif
}


//  ------------------------------------------------------------------

int gsnd::close() {

  #if defined(__DOS__)

  if(file_open) {
    file_open = false;
    return call_api(GSAPI_CLOSE_FILE);
  }
  return 1;

  #elif defined(__OS2_MM__) or defined(__WIN32_MM__)

  if(file_open) {
    SendmciString("close noise wait", NULL);
    file_open = false;
    return 0;
  }
  return 0xFFFF;

  #else

  return 0xFFFF;

  #endif
}


//  ------------------------------------------------------------------

int gsnd::play(uint sample_rate) {

  #if defined(__DOS__)

  return file_open ? call_api(GSAPI_PLAY, sample_rate) : 1;

  #elif defined(__OS2_MM__) or defined(__WIN32_MM__)

  NW(sample_rate);

  if(file_open) {
    SendmciString("seek noise to start", NULL);
    SendmciString("play noise", NULL);
    return 0;
  }
  return 1;

  #else

  NW(sample_rate);
  return 1;

  #endif
}


//  ------------------------------------------------------------------

int gsnd::stop() {

  #if defined(__DOS__)

  return file_open ? call_api(GSAPI_STOP) : 1;

  #elif defined(__OS2_MM__) or defined(__WIN32_MM__)

  if(file_open) {
    SendmciString("stop noise wait", NULL);
    return 0;
  }
  return 1;

  #else

  return 1;

  #endif
}


//  ------------------------------------------------------------------

int gsnd::pause() {

  #if defined(__DOS__)

  return file_open ? call_api(GSAPI_PAUSE) : 1;

  #elif defined(__OS2_MM__) or defined(__WIN32_MM__)

  if(file_open) {
    SendmciString("pause noise", NULL);
    return 0;
  }
  return 1;

  #else

  return 1;

  #endif
}


//  ------------------------------------------------------------------

int gsnd::resume() {

  #if defined(__DOS__)

  return file_open ? call_api(GSAPI_RESUME) : 1;

  #elif defined(__OS2_MM__) or defined(__WIN32_MM__)

  if(file_open) {
    SendmciString("resume noise", NULL);
    return 0;
  }
  return 1;

  #else

  return 1;

  #endif
}


//  ------------------------------------------------------------------

int gsnd::break_loop(int method) {

  #if defined(__DOS__)

  return file_open ? call_api(GSAPI_BREAK_LOOP, method) : 1;

  #elif defined(__OS2_MM__) or defined(__WIN32_MM__)

  NW(method);
  return 0xFFFF;

  #else

  NW(method);
  return 0xFFFF;

  #endif
}



//  ------------------------------------------------------------------

void gsnd::speaker(int onoff) {

  #if defined(__DOS__)

  call_api(GSAPI_SPEAKER_ON_OFF, onoff ? 1 : 0);

  #elif defined(__OS2_MM__) or defined(__WIN32_MM__)

  // Not implemented
  NW(onoff);

  #else

  NW(onoff);

  #endif
}


//  ------------------------------------------------------------------

uint gsnd::get_sample_rate() {

  #if defined(__DOS__)

  return file_open ? data->sample_rate : 0;

  #elif defined(__OS2_MM__) or defined(__WIN32_MM__)

  return 0;

  #else

  return 0;

  #endif
}


//  ------------------------------------------------------------------

int gsnd::is_playing() {

  #if defined(__DOS__)

  if(file_open)
    return (int)data->status;
  return false;

  #elif defined(__OS2_MM__) or defined(__WIN32_MM__)

  char return_buffer[BUFFERSIZE];
  
  if(SendmciString("status noise mode wait", return_buffer))
    return strieql(return_buffer, "playing") or strieql(return_buffer, "seeking");
  return 0;

  #else

  return 0;

  #endif
}


//  ------------------------------------------------------------------

