
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gsocket.cpp,v 1.2 1998/12/27 16:08:12 odinn Exp $
//  ------------------------------------------------------------------
//  Sockets class.
//  ------------------------------------------------------------------


#include <ctype.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <gkbdbase.h>
#include <gkbdcode.h>
#include <gmemdbg.h>
#include <gutlmtsk.h>
#include <gsock.h>

#ifdef __OS2__
#define INCL_BASE
#include <os2.h>
#endif

#ifdef __WIN32__
#include <winsock.h>
static WSADATA WSAData;
#endif


//  ------------------------------------------------------------------

#ifdef __OS2__
static HMODULE __hmod_so32dll;
static HMODULE __hmod_tcp32dll;
#endif

int gsocket::initialized = false;


//  ------------------------------------------------------------------

const int gsocket_maxerror = 67;
const char* gsocket_error[] = {
  /* 000 */  "?",
  /* 001 */  "?",
  /* 002 */  "?",
  /* 003 */  "?",
  /* 004 */  "?",
  /* 005 */  "?",
  /* 006 */  "?",
  /* 007 */  "?",
  /* 008 */  "?",
  /* 009 */  "?",
  /* 010 */  "?",
  /* 011 */  "?",
  /* 012 */  "?",
  /* 013 */  "?",
  /* 014 */  "?",
  /* 015 */  "?",
  /* 016 */  "?",
  /* 017 */  "?",
  /* 018 */  "?",
  /* 019 */  "?",
  /* 020 */  "?",
  /* 021 */  "?",
  /* 022 */  "?",
  /* 023 */  "?",
  /* 024 */  "?",
  /* 025 */  "?",
  /* 026 */  "?",
  /* 027 */  "?",
  /* 028 */  "?",
  /* 029 */  "?",
  /* 030 */  "?",
  /* 031 */  "?",
  /* 032 */  "?",
  /* 033 */  "?",
  /* 034 */  "?",
  /* 035 */  "Operation would block",
  /* 036 */  "Operation now in progress",
  /* 037 */  "Operation already in progress",
  /* 038 */  "Socket operation on non-socket",
  /* 039 */  "Destination address required",
  /* 040 */  "Message too long",
  /* 041 */  "Protocol wrong type for socket",
  /* 042 */  "Protocol not available",
  /* 043 */  "Protocol not supported",
  /* 044 */  "Socket type not supported",
  /* 045 */  "Operation not supported on socket",
  /* 046 */  "Protocol family not supported",
  /* 047 */  "Address family not supported by protocol family",
  /* 048 */  "Address already in use",
  /* 049 */  "Can't assign requested address",
  /* 050 */  "Network is down",
  /* 051 */  "Network is unreachable",
  /* 052 */  "Network dropped connection on reset",
  /* 053 */  "Software caused connection abort",
  /* 054 */  "Connection reset by peer",
  /* 055 */  "No buffer space available",
  /* 056 */  "Socket is already connected",
  /* 057 */  "Socket is not connected",
  /* 058 */  "Can't send after socket shutdown",
  /* 059 */  "Too many references: can't splice",
  /* 060 */  "Connection timed out",
  /* 061 */  "Connection refused",
  /* 062 */  "Too many levels of symbolic links",
  /* 063 */  "File name too long",
  /* 064 */  "Host is down",
  /* 065 */  "No route to host",
  /* 066 */  "Directory not empty",
  /* 067 */  "",
  /* 1001 */ "Authoritative Answer: Host not found",
  /* 1002 */ "Non-Authoritative: Host not found, or SERVERFAIL",
  /* 1003 */ "Non recoverable errors, FORMERR, REFUSED, NOTIMP",
  /* 1004 */ "Valid name, no data record of requested type"
};


//  ------------------------------------------------------------------

#define ANSI_BLACK   '0'
#define ANSI_RED     '1'
#define ANSI_GREEN   '2'
#define ANSI_YELLOW  '3'
#define ANSI_BLUE    '4'
#define ANSI_MAGENTA '5'
#define ANSI_CYAN    '6'
#define ANSI_WHITE   '7'


//  ------------------------------------------------------------------
//  OS/2 socket functions

#ifdef __OS2__
extern "C" {
static int (APIENTRY *sock_init)(void);
static int (APIENTRY *sock_errno)(void);
}
#endif


//  ------------------------------------------------------------------
//  OS/2 low-level socket functions

#ifdef __OS2__
extern "C" {
static int (APIENTRY *accept)(int handle, sockaddr* addr, int* addrlen);
static int (APIENTRY *bind)(int handle, const sockaddr* addr, int namelen);
static int (APIENTRY *closesocket)(int handle);
static int (APIENTRY *connect)(int handle, const sockaddr* name, int namelen);
static int (APIENTRY *getpeername)(int handle, sockaddr* name, int* namelen);
static int (APIENTRY *getsockname)(int handle, sockaddr* name, int* namelen);
static int (APIENTRY *getsockopt)(int handle, int level, int optname, char* optval, int* optlen);
static int (APIENTRY *ioctlsocket)(int handle, int request, void* argp, int datalen=sizeof(ulong));
static int (APIENTRY *listen)(int handle, int backlog);
static int (APIENTRY *recv)(int handle, char* buf, int len, int flags);
static int (APIENTRY *recvfrom)(int handle, char* buf, int len, int flags, sockaddr* from, int* fromlen);
static int (APIENTRY *os2select)(int* sockets, int socket_nread, int socket_nwrite, int socket_nexcept, long mode);
static int (APIENTRY *send)(int handle, const char* buf, int len, int flags);
static int (APIENTRY *sendto)(int handle, const char* buf, int len, int flags, const sockaddr* to, int tolen);
static int (APIENTRY *setsockopt)(int handle, int level, int optname, const char* optval, int optlen);
static int (APIENTRY *shutdown)(int handle, int how);
static int (APIENTRY *socket)(int af, int type, int protocol);
static char* (APIENTRY *inet_ntoa)(in_addr in);
static ulong (APIENTRY *inet_addr)(const char* cp);
}
#endif


//  ------------------------------------------------------------------
//  OS/2 low-level database functions

#ifdef __OS2__
extern "C" {
static int (APIENTRY *gethostname)(char* name, int namelen);
static hostent* (APIENTRY *gethostbyaddr)(const char* addr, int len, int type);
static hostent* (APIENTRY *gethostbyname)(const char* name);
static servent* (APIENTRY *getservbyport)(int port, const char* proto);
static servent* (APIENTRY *getservbyname)(const char* name, const char* proto);
static protoent* (APIENTRY *getprotobynumber)(int number);
static protoent* (APIENTRY *getprotobyname)(const char* name);
static int APIENTRY *h_errno;
}
#endif


//  ------------------------------------------------------------------
//  OS/2 socket functions

#ifdef __OS2__
int gsocket::sock_init() { return ::sock_init(); }
int gsocket::sock_errno() { return ::sock_errno(); }
#else
int gsocket::sock_errno() { return ::h_errno; }
#endif


//  ------------------------------------------------------------------
//  Low-level socket functions

int gsocket::accept(sockaddr* addr, int* addrlen) { return ::accept(handle, addr, addrlen); }
int gsocket::bind(const sockaddr* addr, int namelen) { return ::bind(handle, addr, namelen); }
int gsocket::closesocket() { return ::closesocket(handle); }
int gsocket::connect(const sockaddr* name, int namelen) { return ::connect(handle, name, namelen); }
int gsocket::gethostname(char* name, int namelen) { return ::gethostname(name, namelen); }
int gsocket::getpeername(sockaddr* name, int* namelen) { return ::getpeername(handle, name, namelen); }
int gsocket::getsockname(sockaddr* name, int* namelen) { return ::getsockname(handle, name, namelen); }
int gsocket::getsockopt(int level, int optname, char* optval, int* optlen) { return ::getsockopt(handle, level, optname, optval, optlen); }
int gsocket::ioctlsocket(long cmd, ulong* argp) { return ::ioctlsocket(handle, cmd, argp); }
int gsocket::listen(int backlog) { return ::listen(handle, backlog); }
int gsocket::recv(char* buf, int len, int flags) { return ::recv(handle, buf, len, flags); }
int gsocket::recvfrom(char* buf, int len, int flags, sockaddr* from, int* fromlen) { return ::recvfrom(handle, buf, len, flags, from, fromlen); }
int gsocket::send(const char* buf, int len, int flags) { return ::send(handle, buf, len, flags); }
int gsocket::sendto(const char* buf, int len, int flags, const sockaddr* to, int tolen) { return ::sendto(handle, buf, len, flags, to, tolen); }
int gsocket::setsockopt(int level, int optname, const char* optval, int optlen) { return ::setsockopt(handle, level, optname, optval, optlen); }
int gsocket::shutdown(int how) { return ::shutdown(handle, how); }
int gsocket::socket(int af, int type, int protocol) { return ::socket(af, type, protocol); }
//int gsocket::h_errno() { return ::h_errno; }
ulong gsocket::inet_addr(const char* cp) { return ::inet_addr(cp); }
const char* gsocket::inet_ntoa(in_addr in) { return ::inet_ntoa(in); }


//  ------------------------------------------------------------------

int gsocket::startup() {

  int init_failed = not initialized;

  if(not initialized) {

    #ifdef __OS2__

    char obj[9];
    ULONG count = 0;
    int so32dll_loaded = false;
    int tcp32dll_loaded = false;

    DosEnterMustComplete(&count);
    DosEnterCritSec();

    if(DosLoadModule(obj, sizeof(obj), "SO32DLL", &__hmod_so32dll) == 0) {
      so32dll_loaded = true;
      if(DosLoadModule(obj, sizeof(obj), "TCP32DLL", &__hmod_tcp32dll) == 0) {
        tcp32dll_loaded = true;
        DosQueryProcAddr(__hmod_so32dll, 0, "SOCK_INIT", (PFN*)&::sock_init);
        DosQueryProcAddr(__hmod_so32dll, 0, "SOCK_ERRNO", (PFN*)&::sock_errno);
        DosQueryProcAddr(__hmod_so32dll, 0, "ACCEPT", (PFN*)&::accept);
        DosQueryProcAddr(__hmod_so32dll, 0, "BIND", (PFN*)&::bind);
        DosQueryProcAddr(__hmod_so32dll, 0, "SOCLOSE", (PFN*)&::closesocket);
        DosQueryProcAddr(__hmod_so32dll, 0, "CONNECT", (PFN*)&::connect);
        DosQueryProcAddr(__hmod_so32dll, 0, "GETPEERNAME", (PFN*)&::getpeername);
        DosQueryProcAddr(__hmod_so32dll, 0, "GETSOCKNAME", (PFN*)&::getsockname);
        DosQueryProcAddr(__hmod_so32dll, 0, "GETSOCKOPT", (PFN*)&::getsockopt);
        DosQueryProcAddr(__hmod_so32dll, 0, "IOCTL", (PFN*)&::ioctlsocket);
        DosQueryProcAddr(__hmod_so32dll, 0, "LISTEN", (PFN*)&::listen);
        DosQueryProcAddr(__hmod_so32dll, 0, "RECV", (PFN*)&::recv);
        DosQueryProcAddr(__hmod_so32dll, 0, "RECVFROM", (PFN*)&::recvfrom);
        DosQueryProcAddr(__hmod_so32dll, 0, "SELECT", (PFN*)&::os2select);
        DosQueryProcAddr(__hmod_so32dll, 0, "SEND", (PFN*)&::send);
        DosQueryProcAddr(__hmod_so32dll, 0, "SENDTO", (PFN*)&::sendto);
        DosQueryProcAddr(__hmod_so32dll, 0, "SETSOCKOPT", (PFN*)&::setsockopt);
        DosQueryProcAddr(__hmod_so32dll, 0, "SHUTDOWN", (PFN*)&::shutdown);
        DosQueryProcAddr(__hmod_so32dll, 0, "SOCKET", (PFN*)&::socket);
        DosQueryProcAddr(__hmod_tcp32dll, 0, "GETHOSTBYADDR", (PFN*)&::gethostbyaddr);
        DosQueryProcAddr(__hmod_tcp32dll, 0, "GETHOSTBYNAME", (PFN*)&::gethostbyname);
        DosQueryProcAddr(__hmod_tcp32dll, 0, "GETHOSTNAME", (PFN*)&::gethostname);
        DosQueryProcAddr(__hmod_tcp32dll, 0, "GETSERVBYPORT", (PFN*)&::getservbyport);
        DosQueryProcAddr(__hmod_tcp32dll, 0, "GETSERVBYNAME", (PFN*)&::getservbyname);
        DosQueryProcAddr(__hmod_tcp32dll, 0, "GETPROTOBYNUMBER", (PFN*)&::getprotobynumber);
        DosQueryProcAddr(__hmod_tcp32dll, 0, "GETPROTOBYNAME", (PFN*)&::getprotobyname);
        DosQueryProcAddr(__hmod_tcp32dll, 0, "INET_ADDR", (PFN*)&::inet_addr);
        DosQueryProcAddr(__hmod_tcp32dll, 0, "INET_NTOA", (PFN*)&::inet_ntoa);
        DosQueryProcAddr(__hmod_tcp32dll, 0, "H_ERRNO", (PFN*)&::h_errno);
        init_failed = sock_init();
      }
    }

    if(init_failed) {
      if(so32dll_loaded)
        DosFreeModule(__hmod_so32dll);
      if(tcp32dll_loaded)
        DosFreeModule(__hmod_tcp32dll);
    }
    else {
      initialized = true;
    }

    DosExitCritSec();
    DosExitMustComplete(&count);

    #endif

    #ifdef __WIN32__

    WORD wVerReq = MAKEWORD(1,1);
    init_failed = WSAStartup(wVerReq, &WSAData);
    if(not init_failed)
      initialized = true;

    #endif
  }

  return not init_failed;
}


//  ------------------------------------------------------------------

void _stdc gsocket::debug_printf(const char* format, ...) {

  va_list argptr;
  char buf[256];
  *buf = NUL;
  va_start(argptr, format);
  vsprintf(buf, format, argptr);
  va_end(argptr);

  char color = '0';
  switch(*buf) {
    case 'C': color = ANSI_BLUE;    break;
    case 'E': color = ANSI_RED;     break;
    case 'M': color = ANSI_GREEN;   break;
    case 'S': color = ANSI_MAGENTA; break;
  }

  ::printf("\x1B[0;3%c;47m%c: %s\x1B[0;30;47m\n", color, *buf, buf+1);
}


//  ------------------------------------------------------------------

void gsocket::debug_error(const char* text, int error) {

  int e = error - 10000;
  debug_printf("E%s: Error %i (%s)", text, error, e < gsocket_maxerror ? (e < 0 ? gsocket_error[error] : gsocket_error[e]) : (e > 1000 ? gsocket_error[e-1000+gsocket_maxerror] : "?"));
}


//  ------------------------------------------------------------------

void gsocket::debug_client(const char* text) {

  debug_printf("C%s", text);
}


//  ------------------------------------------------------------------

void gsocket::debug_server(const char* text) {

  debug_printf("S%s", text);
}


//  ------------------------------------------------------------------

void gsocket::display_hostent(const hostent* hp) {

  char** alias = hp->h_aliases;
  char** address = hp->h_addr_list;
  debug_printf("SHost: %s", hp->h_name);
  if(alias and *alias)
    while(*alias)
      debug_printf("SAlias: %s", *alias++);
  if(address and *address)
    while(*address)
      debug_printf("SAddress: %s", inet_ntoa(*(in_addr*)(*address++)));
}


//  ------------------------------------------------------------------

void gsocket::display_servent(const servent* sp) {

  char** alias = sp->s_aliases;
  debug_printf("SService: %s", sp->s_name);
  if(alias and *alias)
    while(*alias)
      debug_printf("SAlias: %s", *alias++);
  debug_printf("SPort: %i", ntohs((ushort)sp->s_port));
  debug_printf("SProtocol: %s", sp->s_proto);
}


//  ------------------------------------------------------------------

void gsocket::display_protoent(const protoent* pp) {

  char** alias = pp->p_aliases;
  debug_printf("SProtocol: %s (%i)", pp->p_name, pp->p_proto);
  if(alias and *alias)
    while(*alias)
      debug_printf("SAlias: %s", *alias++);
}


//  ------------------------------------------------------------------

const hostent* gsocket::gethostbyaddr(const char* addr, int len, int type) {

  hostent* h = ::gethostbyaddr(addr, len, type);
  if(h) {
    if(debug) display_hostent(h);
  }
  else {
    if(debug) debug_error("gethostbyaddr", sock_errno());
  }
  return h;
}


//  ------------------------------------------------------------------

const hostent* gsocket::gethostbyname(const char* name) {

  hostent* h = ::gethostbyname(name);
  if(h) {
    if(debug) display_hostent(h);
  }
  else {
    if(debug) debug_error("gethostbyname", sock_errno());
  }
  return h;
}


//  ------------------------------------------------------------------

const servent* gsocket::getservbyport(int port, const char* proto) {

  servent* s = ::getservbyport(port, proto);
  if(s) {
    if(debug) display_servent(s);
  }
  else {
    if(debug) debug_error("getservbyport", sock_errno());
  }
  return s;
}


//  ------------------------------------------------------------------

const servent* gsocket::getservbyname(const char* name, const char* proto) {

  servent* s = ::getservbyname(name, proto);
  if(s) {
    if(debug) display_servent(s);
  }
  else {
    if(debug) debug_error("getservbyname", sock_errno());
  }
  return s;
}


//  ------------------------------------------------------------------

const protoent* gsocket::getprotobynumber(int proto) {

  protoent* p = ::getprotobynumber(proto);
  if(p) {
    if(debug) display_protoent(p);
  }
  else {
    if(debug) debug_error("getprotobynumber", sock_errno());
  }
  return p;
}


//  ------------------------------------------------------------------

const protoent* gsocket::getprotobyname(const char* name) {

  protoent* p = ::getprotobyname(name);
  if(p) {
    if(debug) display_protoent(p);
  }
  else {
    if(debug) debug_error("getprotobyname", sock_errno());
  }
  return p;
}


//  ------------------------------------------------------------------

int gsocket::idle() {

  if(kbxhit() == Key_Esc) {
    kbxget();
    return false;
  }
  if(gmtsk.detected)
    gmtsk.timeslice();
  return true;
}


//  ------------------------------------------------------------------

int gsocket::select_poll(int mask) {

  select_result = 0;

  #ifdef __OS2__
  int select_handles[3];
  select_handles[0] = handle;
  select_handles[1] = handle;
  select_handles[2] = handle;
  int result = ::os2select(select_handles, 1, 1, 1, 0);
  #else
  fd_set readable_set;
  fd_set writable_set;
  fd_set excepted_set;
  FD_ZERO(&readable_set);
  FD_ZERO(&writable_set);
  FD_ZERO(&excepted_set);
  FD_SET(handle, &readable_set);
  FD_SET(handle, &writable_set);
  FD_SET(handle, &excepted_set);
  timeval timeout = { 0, 0 };
  int result = ::select(1, &readable_set, &writable_set, &excepted_set, &timeout);
  #endif

  if(result == SOCKET_ERROR) {
    if(debug) debug_error("select", sock_errno());
    return 0;
  }
  else if(result == 0) {
    return 0;
  }

  #ifdef __OS2__
  if(select_handles[0] != -1)
    select_result |= select_readable;
  if(select_handles[1] != -1)
    select_result |= select_writable;
  if(select_handles[2] != -1)
    select_result |= select_excepted;
  #else
  if(FD_ISSET(handle, &readable_set))
    select_result |= select_readable;
  if(FD_ISSET(handle, &writable_set))
    select_result |= select_writable;
  if(FD_ISSET(handle, &excepted_set))
    select_result |= select_excepted;
  #endif

  return select_result & mask;
}


//  ------------------------------------------------------------------

int gsocket::wait_for_readable() {

  while(not select_poll(select_readable|select_excepted)) {
    if(!idle()) {
      if(debug) debug_printf("EUser exit in wait_for_readable");
      return false;
    }
  }
  if(select_result & select_excepted)
    return false;
  return true;
}


//  ------------------------------------------------------------------

int gsocket::wait_for_writable() {

  while(not select_poll(select_writable|select_excepted)) {
    if(!idle()) {
      if(debug) debug_printf("EUser exit in wait_for_writable");
      return false;
    }
  }
  if(select_result & select_excepted)
    return false;
  return true;
}


//  ------------------------------------------------------------------

int gsocket::wait_for_excepted() {

  while(not select_poll(select_excepted)) {
    if(!idle()) {
      if(debug) debug_printf("EUser exit in wait_for_excepted");
      return false;
    }
  }
  return true;
}


//  ------------------------------------------------------------------

gsocket::gsocket() {

  recv_buffer_size = 4096;
  recv_bufsize = 0;
  recv_buffer = NULL;
  recv_buf = NULL;
  line_buffer_complete = false;
  line_buffer_size = 520;
  line_buffer = NULL;
  handle = INVALID_SOCKET;
  errorcode = 0;
  waiting = false;
  select_result = 0;
  blocking = true;
  debug = false;
}


//  ------------------------------------------------------------------

gsocket::~gsocket() {

  if(handle != INVALID_SOCKET)
    close();
  throw_xfree(recv_buffer);
  throw_xfree(line_buffer);

  #ifdef __WIN32__
  WSACleanup();
  #endif
}


//  ------------------------------------------------------------------

int gsocket::open(const char* host, const char* service) {

  if(not initialized and not startup())
    return false;

  if(debug) debug_printf("CCalling getservbyname(%s)", service);
  const servent* sp = getservbyname(service, "tcp");
  if(!sp)
    return false;
  return open(host, ntohs((ushort)sp->s_port));
}


//  ------------------------------------------------------------------

int gsocket::open(const char* host, int port) {

  if(not initialized and not startup())
    return false;

  const hostent *hp;
  ulong inaddr;
  sockaddr_in ad;

  memset(&ad, 0, sizeof(ad));
  ad.sin_family = AF_INET;

  inaddr = inet_addr(host);
  if(inaddr != INADDR_NONE)
    memcpy(&ad.sin_addr, &inaddr, sizeof(inaddr));
  else {
    if(debug) debug_printf("CCalling gethostbyname(%s)", host);
    hp = gethostbyname(host);
    if(!hp)
      return false;
    memcpy(&ad.sin_addr, hp->h_addr_list[0], hp->h_length);
  }
  ad.sin_port = htons((ushort)port);

  if(debug) debug_printf("CCalling socket()");
  handle = ::socket(AF_INET, SOCK_STREAM, 0);
  if(handle < 0) {
    if(debug) debug_error("socket", sock_errno());
    return false;
  }

  if(not blocking) {
    ulong nonblocking = true;
    if(debug) debug_printf("CCalling ioctlsocket() to switch to non-blocked mode");
    if(ioctlsocket(FIONBIO, &nonblocking) != 0) {
      if(debug) debug_error("ioctlsocket", sock_errno());
    }
    else {
      blocking = false;
    }
  }

  if(debug) debug_printf("CCalling connect()");
  if(::connect(handle, (sockaddr*)&ad, sizeof(ad)) < 0) {
    if(not blocking and ((sock_errno() == gsocket_error_inprogress) or (sock_errno() == gsocket_error_wouldblock))) {
      if(debug) debug_printf("MWaiting for connect (non-blocking)");
      if(!wait_for_writable()) {
        if(debug) debug_error("Connect failed!", sock_errno());
        close();
        return false;
      }
    }
    else {
      if(debug) debug_error("connect", sock_errno());
      close();
      return false;
    }
  }
  if(debug) debug_printf("MConnection made.");

  sockaddr peername; int peernamelen = sizeof(peername);
  getpeername(&peername, &peernamelen);
  debug_printf("SPeer address: %s", inet_ntoa(((sockaddr_in*)&peername)->sin_addr));
  sockaddr localname; int localnamelen = sizeof(localname);
  getsockname(&localname, &localnamelen);
  debug_printf("SLocal address: %s", inet_ntoa(((sockaddr_in*)&localname)->sin_addr));

  if(debug) debug_printf("MWaiting for server to respond ...");

  if(!recv_buffer) {
    //recv_buffer_size = opt;
    recv_buffer = (char*)throw_xmalloc(recv_buffer_size);
    line_buffer = (char*)throw_xmalloc(line_buffer_size);
  }
  recv_buf = recv_buffer;
  recv_bufsize = 0;

  return true;
}


//  ------------------------------------------------------------------

void gsocket::close() {

  if(handle != INVALID_SOCKET) {
    ::shutdown(handle, 2);
    ::closesocket(handle);
    handle = INVALID_SOCKET;
  }
}


//  ------------------------------------------------------------------

int gsocket::read(char* buf, int len) {

  while(len) {
    if(not blocking and !wait_for_readable()) {
      if(debug) debug_error("read/recv/non-blocking", sock_errno());
      return -1;
    }
    int n = ::recv(handle, buf, len, 0);
    if(n < 0) {
      if(debug) debug_error("read/recv", sock_errno());
      return -1;
    }
    len -= n;
    buf += n;
  }
  return 0;
}


//  ------------------------------------------------------------------

int gsocket::write(const char* buf, int len) {

  while(len) {
    if(not blocking and !wait_for_writable()) {
      if(debug) debug_error("write/send/non-blocking", sock_errno());
      return -1;
    }
    int n = ::send(handle, buf, len, 0);
    if(n < 0) {
      if(debug) debug_error("write/send", sock_errno());
      return -1;
    }
    len -= n;
    buf += n;
  }
  return 0;
}


//  ------------------------------------------------------------------

int gsocket::strget(char* buf, int len) {

  int copysize;
  int stringpos = 0;
  int fl_return_string = 0;

  while(1) {

    if(recv_bufsize) {

      // get position of LF in the buffer
      char* lfptr = (char*)memchr(recv_buf, '\n', recv_bufsize);

      // if there is a LF in the buffer, then ...
      if(lfptr) {
        copysize = lfptr - recv_buf + 1;
        fl_return_string = 1;
      }
      else
        copysize = recv_bufsize;

      // make sure we won't write more than len-1 characters
      if(copysize > ((len - 1) - stringpos)) {
        copysize = (len - 1) - stringpos;
        fl_return_string = 1;
      }

      // copy copysize characters from buffer into the string
      memcpy(buf + stringpos, recv_buf, copysize);
      stringpos += copysize;

      // if we got the whole string, then set buffer to the unused data and return the string
      if(fl_return_string) {
        if(buf[stringpos-2] == '\r')
          buf[stringpos-2] = '\0';
        else if(buf[stringpos-1] == '\n')
          buf[stringpos-1] = '\0';
        else
          buf[stringpos] = '\0';
        recv_bufsize -= copysize;
        recv_buf += copysize;
        if(debug) debug_server(buf);
        return 0;
      }
    }

    // reset buffer and receive more data into buffer
    if(not blocking and !wait_for_readable()) {
      if(debug) debug_error("strget/recv/non-blocking", sock_errno());
      return -1;
    }
    recv_buf = recv_buffer;
    recv_bufsize = ::recv(handle, recv_buffer, recv_buffer_size, 0);
    if(recv_bufsize <= 0) {
      recv_bufsize = 0;
      if(debug) debug_error("strget/recv", sock_errno());
      return -1;
    }
  }
}


//  ------------------------------------------------------------------

int gsocket::strput(const char* buf) {

  if(debug) debug_client(buf);
  int rc = write(buf, strlen(buf));
  if(rc)
    return rc;
  return write("\r\n", 2);
}


//  ------------------------------------------------------------------

int gsocket::printf(const char* format, ...) {

  char buf[2048];
  va_list ap;
  va_start(ap, format);
  vsprintf(buf, format, ap);
  va_end(ap);
  return strput(buf);
}


//  ------------------------------------------------------------------

int gsocket::getline() {

  errorcode = 0;
  int stringpos = 0;
  int fl_return_string = false;

  while(1) {

    if(recv_bufsize) {

      // get position of LF in the buffer
      char* lfptr = (char*)memchr(recv_buf, '\n', recv_bufsize);

      // if there is a LF in the buffer, then ...
      int copysize = lfptr ? (lfptr - recv_buf + 1) : recv_bufsize;
      if(lfptr)
        fl_return_string = true;

      // make sure we won't write more than line_buffer_size-1 characters
      if(copysize > ((line_buffer_size - 1) - stringpos)) {
        copysize = (line_buffer_size - 1) - stringpos;
        fl_return_string = true;
      }

      // copy copysize characters from buffer into the string
      memcpy(line_buffer + stringpos, recv_buf, copysize);
      stringpos += copysize;

      // if we got the whole string, then set buffer to the unused
      // data and return the string
      if(fl_return_string) {
        if(line_buffer[stringpos-2] == '\r')
          line_buffer[stringpos-2] = NUL;
        else if(line_buffer[stringpos-1] == '\n')
          line_buffer[stringpos-1] = NUL;
        else
          line_buffer[stringpos] = NUL;
        recv_bufsize -= copysize;
        recv_buf += copysize;
        if(debug) debug_server(line_buffer);
        return true;
      }
    }

    // reset buffer and receive more data into buffer
    if(not blocking and !wait_for_readable()) {
      errorcode = sock_errno();
      if(debug) debug_error("getline/recv/non-blocking", errorcode);
      return false;
    }
    recv_buf = recv_buffer;
    recv_bufsize = ::recv(handle, recv_buffer, recv_buffer_size, 0);
    if(recv_bufsize == SOCKET_ERROR or recv_bufsize == 0) {
      recv_bufsize = 0;
      errorcode = sock_errno();
      if(debug) debug_error("getline/recv", errorcode);
      return false;
    }
  }
}


//  ------------------------------------------------------------------

int gsocket::putline() {

  if(debug) debug_client(line_buffer);
  if(write(line_buffer, strlen(line_buffer)))
    return false;
  return write("\r\n", 2) ? false : true;
}


//  ------------------------------------------------------------------

