
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gsocket.h,v 1.1.1.1 1998/11/21 19:38:09 odinn Exp $
//  ------------------------------------------------------------------
//  Sockets class.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#ifndef __G_SOCKET_H
#define __G_SOCKET_H

#include <gdefs.h>

#ifdef __WIN32__
#include <winsock.h>
#endif

#ifdef __OS2__

//  ------------------------------------------------------------------
//  Structures returned by network data base library, taken from the
//  BSD file netdb.h.  All addresses are supplied in host order, and
//  returned in network order (suitable for use in system calls).
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  The new type to be used in all instances which refer to sockets.

typedef uint SOCKET;


//  ------------------------------------------------------------------

struct hostent {

  char*  h_name;        // official name of host
  char** h_aliases;     // alias list
  int    h_addrtype;    // host address type
  int    h_length;      // length of address
  char** h_addr_list;   // list of addresses from name server
};


//  ------------------------------------------------------------------

struct servent {
  char*  s_name;      // official service name
  char** s_aliases;   // alias list
  int    s_port;      // port #
  char*  s_proto;     // protocol to use
};


//  ------------------------------------------------------------------

struct protoent {
  char*  p_name;      // official protocol name
  char** p_aliases;   // alias list
  short  p_proto;     // protocol #
};


//  ------------------------------------------------------------------
//  Constants and structures defined by the internet system, Per RFC
//  790, September 1981, taken from the BSD file netinet/in.h.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Protocols

#define IPPROTO_IP              0               // dummy for IP
#define IPPROTO_ICMP            1               // control message protocol
#define IPPROTO_GGP             2               // gateway^2 (deprecated)
#define IPPROTO_TCP             6               // tcp
#define IPPROTO_PUP             12              // pup
#define IPPROTO_UDP             17              // user datagram protocol
#define IPPROTO_IDP             22              // xns idp
#define IPPROTO_ND              77              // UNOFFICIAL net disk proto

#define IPPROTO_RAW             255             // raw IP packet
#define IPPROTO_MAX             256


//  ------------------------------------------------------------------
//  Port/socket numbers: network standard functions

#define IPPORT_ECHO             7
#define IPPORT_DISCARD          9
#define IPPORT_SYSTAT           11
#define IPPORT_DAYTIME          13
#define IPPORT_NETSTAT          15
#define IPPORT_FTP              21
#define IPPORT_TELNET           23
#define IPPORT_SMTP             25
#define IPPORT_TIMESERVER       37
#define IPPORT_NAMESERVER       42
#define IPPORT_WHOIS            43
#define IPPORT_MTP              57


//  ------------------------------------------------------------------
//  Port/socket numbers: host specific functions

#define IPPORT_TFTP             69
#define IPPORT_RJE              77
#define IPPORT_FINGER           79
#define IPPORT_TTYLINK          87
#define IPPORT_SUPDUP           95


//  ------------------------------------------------------------------
//  Internet address

struct in_addr {
  ulong s_addr;
};


//  ------------------------------------------------------------------

#define INADDR_ANY              0x00000000
#define INADDR_LOOPBACK         0x7F000001
#define INADDR_BROADCAST        0xFFFFFFFF
#define INADDR_NONE             0xFFFFFFFF


//  ------------------------------------------------------------------
//  Socket address, internet style.

struct sockaddr_in {
  short   sin_family;
  ushort  sin_port;
  in_addr sin_addr;
  char    sin_zero[8];
};


//  ------------------------------------------------------------------
//  This is used instead of -1, since the SOCKET type is unsigned.

#define INVALID_SOCKET  (SOCKET)(~0)
#define SOCKET_ERROR            (-1)


//  ------------------------------------------------------------------
//  Types

#define SOCK_STREAM     1               // stream socket
#define SOCK_DGRAM      2               // datagram socket
#define SOCK_RAW        3               // raw-protocol interface
#define SOCK_RDM        4               // reliably-delivered message
#define SOCK_SEQPACKET  5               // sequenced packet stream


//  ------------------------------------------------------------------
//  Option flags per-socket.

#define SO_DEBUG        0x0001          // turn on debugging info recording
#define SO_ACCEPTCONN   0x0002          // socket has had listen()
#define SO_REUSEADDR    0x0004          // allow local address reuse
#define SO_KEEPALIVE    0x0008          // keep connections alive
#define SO_DONTROUTE    0x0010          // just use interface addresses
#define SO_BROADCAST    0x0020          // permit sending of broadcast msgs
#define SO_USELOOPBACK  0x0040          // bypass hardware when possible
#define SO_LINGER       0x0080          // linger on close if data present
#define SO_OOBINLINE    0x0100          // leave received OOB data in line

#define SO_DONTLINGER   ~SO_LINGER


//  ------------------------------------------------------------------
//  Additional options.

#define SO_SNDBUF       0x1001          // send buffer size
#define SO_RCVBUF       0x1002          // receive buffer size
#define SO_SNDLOWAT     0x1003          // send low-water mark
#define SO_RCVLOWAT     0x1004          // receive low-water mark
#define SO_SNDTIMEO     0x1005          // send timeout
#define SO_RCVTIMEO     0x1006          // receive timeout
#define SO_ERROR        0x1007          // get error status and clear
#define SO_TYPE         0x1008          // get socket type


//  ------------------------------------------------------------------
//  TCP options.

#define TCP_NODELAY     0x0001


//  ------------------------------------------------------------------
//  Address families.

#define AF_UNSPEC       0               // unspecified
#define AF_UNIX         1               // local to host (pipes, portals)
#define AF_INET         2               // internetwork: UDP, TCP, etc.
#define AF_IMPLINK      3               // arpanet imp addresses
#define AF_PUP          4               // pup protocols: e.g. BSP
#define AF_CHAOS        5               // mit CHAOS protocols
#define AF_IPX          6               // IPX and SPX
#define AF_NS           6               // XEROX NS protocols
#define AF_ISO          7               // ISO protocols
#define AF_OSI          AF_ISO          // OSI is ISO
#define AF_ECMA         8               // european computer manufacturers
#define AF_DATAKIT      9               // datakit protocols
#define AF_CCITT        10              // CCITT protocols, X.25 etc
#define AF_SNA          11              // IBM SNA
#define AF_DECnet       12              // DECnet
#define AF_DLI          13              // Direct data link interface
#define AF_LAT          14              // LAT
#define AF_HYLINK       15              // NSC Hyperchannel
#define AF_APPLETALK    16              // AppleTalk
#define AF_NETBIOS      17              // NetBios-style addresses

#define AF_MAX          18


//  ------------------------------------------------------------------
//  Structure used by kernel to store most addresses.

struct sockaddr {

  ushort sa_family;     // address family
  char   sa_data[14];   // up to 14 bytes of direct address
};


//  ------------------------------------------------------------------
//  Protocol families, same as address families for now.

#define PF_UNSPEC       AF_UNSPEC
#define PF_UNIX         AF_UNIX
#define PF_INET         AF_INET
#define PF_IMPLINK      AF_IMPLINK
#define PF_PUP          AF_PUP
#define PF_CHAOS        AF_CHAOS
#define PF_NS           AF_NS
#define PF_IPX          AF_IPX
#define PF_ISO          AF_ISO
#define PF_OSI          AF_OSI
#define PF_ECMA         AF_ECMA
#define PF_DATAKIT      AF_DATAKIT
#define PF_CCITT        AF_CCITT
#define PF_SNA          AF_SNA
#define PF_DECnet       AF_DECnet
#define PF_DLI          AF_DLI
#define PF_LAT          AF_LAT
#define PF_HYLINK       AF_HYLINK
#define PF_APPLETALK    AF_APPLETALK

#define PF_MAX          AF_MAX


//  ------------------------------------------------------------------
//  Structure used for manipulating linger option.

struct linger {
  ushort l_onoff;                // option on/off
  ushort l_linger;               // linger time
};


//  ------------------------------------------------------------------
//  Level number for (get/set)sockopt() to apply to socket itself.

#define SOL_SOCKET      0xffff          // options for socket level


//  ------------------------------------------------------------------

#define MSG_OOB         0x1             // process out-of-band data
#define MSG_PEEK        0x2             // peek at incoming message
#define MSG_DONTROUTE   0x4             // send without using routing tables
#define MSG_MAXIOVLEN   16
#define MSG_PARTIAL     0x8000          // partial send or recv for message xport


//  ------------------------------------------------------------------
//  Define constant based on rfc883, used by gethostbyxxxx() calls.

#define MAXGETHOSTSTRUCT        1024


//  ------------------------------------------------------------------

#define _IOC(a,b) ((a<<8)|b)
#define _IOW(a,b,c) _IOC(a,b)
#define _IOR(a,b,c) _IOC(a,b)
#define _IOWR(a,b,c) _IOC(a,b)

#define FIONREAD        _IOC('f', 127)
#define FIONBIO         _IOC('f', 126)
#define FIOASYNC        _IOC('f', 125)
#define FIOTCPCKSUM     _IOC('f', 128)
#define FIONSTATUS      _IOC('f', 120)
#define FIONURG         _IOC('f', 121)

/* socket i/o controls */
#define SIOCSHIWAT      _IOW('s',  0, int)	/* set high watermark */
#define SIOCGHIWAT      _IOR('s',  1, int)	/* get high watermark */
#define SIOCSLOWAT      _IOW('s',  2, int)	/* set low watermark */
#define SIOCGLOWAT      _IOR('s',  3, int)	/* get low watermark */
#define SIOCATMARK      _IOR('s',  7, int)	/* at oob mark? */
#define SIOCSPGRP       _IOW('s',  8, int)	/* set process group */
#define SIOCGPGRP       _IOR('s',  9, int)	/* get process group */

#define SIOCADDRT       _IOW('r', 10, struct ortentry)	/* add route */
#define SIOCDELRT       _IOW('r', 11, struct ortentry)	/* delete route */

#define SIOCSIFADDR     _IOW('i', 12, struct ifreq)	/* set ifnet address */
#define SIOCGIFADDR     _IOWR('i',13, struct ifreg)	/* get ifnet addres */
#define OSIOCGIFADDR	SIOCGIFADDR
#define SIOCSIFDSTADDR  _IOW('i', 14, struct ifreq)	/* set p-p address */
#define SIOCGIFDSTADDR  _IOWR('i',15, struct ifreq)	/* get p-p address */
#define OSIOCGIFDSTADDR	SIOCGIFDSTADDR
#define SIOCSIFFLAGS    _IOW('i', 16, struct ifreq)	/* set ifnet flags */
#define SIOCGIFFLAGS    _IOWR('i',17, struct ifreq)	/* get ifnet flags */
#define SIOCGIFBRDADDR  _IOWR('i',18, struct ifreq)	/* get broadcast addr */
#define OSIOCGIFBRDADDR	SIOCGIFBRDADDR
#define SIOCSIFBRDADDR  _IOW('i', 19, struct ifreq)	/* set broadcast addr */
#define SIOCGIFCONF     _IOWR('i',20, struct ifreq)	/* get ifnet list */
#define OSIOCGIFCONF	SIOCGIFCONF
#define SIOCGIFNETMASK  _IOWR('i',21, struct ifreq)	/* get net addr mask */
#define OSIOCGIFNETMASK	SIOCGIFNETMASK
#define SIOCSIFNETMASK  _IOW('i', 22, struct ifreq)	/* set net addr mask */
#define SIOCGIFMETRIC   _IOWR('i',23, struct ifreq)	/* get if metric */
#define SIOCSIFMETRIC   _IOW('i', 24, struct ifreq)	/* set if metric */

#define SIOCSARP        _IOW('i', 30, struct arpreq)	/* set arp entry */
#define SIOCGARP        _IOWR('i',31, struct arpreq)	/* get arp entry */
#define OSIOCGARP	SIOCGARP
#define SIOCDARP        _IOW('i', 32, struct arpreq)	/* delete arp entry */

#endif


//  ------------------------------------------------------------------
//  Socket errors returned by sock_error()

const int gsocket_error_addrinuse      = 10048;
const int gsocket_error_addrnotavail   = 10049;
const int gsocket_error_afnosupport    = 10047;
const int gsocket_error_already        = 10037;
const int gsocket_error_connaborted    = 10053;
const int gsocket_error_connrefused    = 10061;
const int gsocket_error_connreset      = 10054;
const int gsocket_error_destaddrreq    = 10039;
const int gsocket_error_dquot          = 10069;
const int gsocket_error_hostdown       = 10064;
const int gsocket_error_hostunreach    = 10065;
const int gsocket_error_inprogress     = 10036;
const int gsocket_error_isconn         = 10056;
const int gsocket_error_loop           = 10062;
const int gsocket_error_msgsize        = 10040;
const int gsocket_error_nametoolong    = 10063;
const int gsocket_error_netdown        = 10050;
const int gsocket_error_netreset       = 10052;
const int gsocket_error_netunreach     = 10051;
const int gsocket_error_nobufs         = 10055;
const int gsocket_error_noprotoopt     = 10042;
const int gsocket_error_notconn        = 10057;
const int gsocket_error_notempty       = 10066;
const int gsocket_error_notsock        = 10038;
const int gsocket_error_opnotsupp      = 10045;
const int gsocket_error_pfnosupport    = 10046;
const int gsocket_error_proclim        = 10067;
const int gsocket_error_protonosupport = 10043;
const int gsocket_error_prototype      = 10041;
const int gsocket_error_remote         = 10071;
const int gsocket_error_shutdown       = 10058;
const int gsocket_error_socktnosupport = 10044;
const int gsocket_error_stale          = 10070;
const int gsocket_error_timedout       = 10060;
const int gsocket_error_toomanyrefs    = 10059;
const int gsocket_error_users          = 10068;
const int gsocket_error_wouldblock     = 10035;


//  ------------------------------------------------------------------

const int gsocket_select_readable = 1;
const int gsocket_select_writable = 2;
const int gsocket_select_excepted = 4;


//  ------------------------------------------------------------------
//  Socket class

class gsocket {

protected:

  static int initialized;

  int recv_buffer_size;
  int recv_bufsize;
  char* recv_buffer;
  char* recv_buf;

  int line_buffer_complete;
  int line_buffer_size;
  char* line_buffer;

  int handle;
  int errorcode;
  int waiting;

  int select_result;
  int select_poll(int mask=0);

public:

  enum {
    select_readable = 1,
    select_writable = 2,
    select_excepted = 4
  };

  int blocking;

  int debug;

  gsocket();
  virtual ~gsocket();

  int error() { return errorcode; }
  char* linebuffer() { return line_buffer; }

  #if !defined(__WATCOMC__)
  virtual void _stdc debug_printf(const char* format, ...);
  #endif
  virtual int idle();

  void debug_error(const char* text, int error);
  void debug_client(const char* text);
  void debug_server(const char* text);

  void display_hostent(const hostent* hp);
  void display_servent(const servent* sp);
  void display_protoent(const protoent* pp);

  int startup();

  // OS/2 socket functions
  int sock_init();
  int sock_errno();

  // Low-level socket functions
  int accept(sockaddr* addr, int* addrlen);
  int bind(const sockaddr* addr, int namelen);
  int closesocket();
  int connect(const sockaddr* name, int namelen);
  int getpeername(sockaddr* name, int* namelen);
  int getsockname(sockaddr* name, int* namelen);
  int getsockopt(int level, int optname, char* optval, int* optlen);
  ulong htonl(ulong hostlong);
  ushort htons(ushort hostshort);
  ulong inet_addr(const char* cp);
  const char* inet_ntoa(in_addr in);
  int ioctlsocket(long cmd, ulong* argp);
  int listen(int backlog);
  ulong ntohl(ulong netlong);
  ushort ntohs(ushort netshort);
  int recv(char* buf, int len, int flags);
  int recvfrom(char* buf, int len, int flags, sockaddr* from, int* fromlen);
  int send(const char* buf, int len, int flags);
  int sendto(const char* buf, int len, int flags, const sockaddr* to, int tolen);
  int setsockopt(int level, int optname, const char* optval, int optlen);
  int shutdown(int how);
  int socket(int af, int type, int protocol);

  // Low-level database functions
  const hostent* gethostbyaddr(const char* addr, int len, int type);
  const hostent* gethostbyname(const char* name);
  int gethostname(char* name, int namelen);
  const servent* getservbyport(int port, const char* proto);
  const servent* getservbyname(const char* name, const char* proto);
  const protoent* getprotobynumber(int proto);
  const protoent* getprotobyname(const char* name);
  //int h_errno();

  // Higher-level socket functions
  int open(const char* host, const char* service);
  int open(const char* host, int port);
  void close();
  int connected() { return handle != SOCKET_ERROR; }
  int is_open() { return handle != INVALID_SOCKET; }
  int is_readable() { return !!(select_result & select_readable); }
  int is_writable() { return !!(select_result & select_writable); }
  int is_excepted() { return !!(select_result & select_excepted); }
  int wait_for_readable();
  int wait_for_writable();
  int wait_for_excepted();
  int read(char* buf, int len);
  int write(const char* buf, int len);
  int strget(char* buf, int len);
  int strput(const char* buf);
  int printf(const char* format, ...);

  int getline();
  int putline();
};


//  ------------------------------------------------------------------

inline ushort gsocket::htons(ushort hostshort) { return (ushort)((hostshort << 8) | (hostshort >> 8)); }
inline ushort gsocket::ntohs(ushort netshort) { return htons(netshort); }
inline ulong gsocket::htonl(ulong hostlong) { return htons((ushort)(hostlong >> 16)) | (htons((ushort)hostlong) << 16); }
inline ulong gsocket::ntohl(ulong netlong) { return htonl(netlong); }


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

