
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gstrall.h,v 1.2 1998/11/22 15:33:13 odinn Exp $
//  ------------------------------------------------------------------
//  String manipulation functions.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GSTRALL_H
#define __GSTRALL_H


//  ------------------------------------------------------------------
//  Required headers

#include <ctype.h>
#include <string.h>
#include <gdefs.h>


//  ------------------------------------------------------------------

#if defined(__GNUC__) and not defined(__EMX__)
#define stricmp strcasecmp
#define strnicmp strncasecmp
char* strupr(char* s);
char* strlwr(char* s);
#endif


//  ------------------------------------------------------------------
//  Function prototypes

int cvaltype(char ch, int ctype);
int strblank(const char* str);
char* strbmatch(char* str, char* strarr[]);
int strchg(char* str, char oldch, char newch);
char* strcode(char* str, const char* key);
char* strdel(const char* substr, char* str);
char* strdela(const char* substr, char* str);
int strichg(char* str, char oldch, char newch);
char* stridel(const char* substr, char* str);
char* stridela(const char* substr, char* str);
int strnicmpw(const char* str1, const char* str2, int len);
const char* striincw(const char* str1, const char* str2);
const char* striinc(const char* str1, const char* str2);
const char* strinc(const char* str1, const char* str2);
char* strins(const char* instr, char* str, int st_pos);
int striocc(char* str, char ch);
char* strischg(char* str, const char* find, const char* replace);
int strisocc(char* str1, char* str2);
char* strisrep(char* str, const char* search, const char* replace);
char* strljust(char* str);
char* strltrim(char* str);
int strmatch(char* str1, char* str2);
char* strmid(char* str, int st_pos, int num_chars);
int strocc(char* str, char ch);
char* strrjust(char* str);
char* strrol(char* str, int count);
char* strror(char* str, int count);
char* strschg(char* str, const char* find, const char* replace);
char* strsetsz(char* str, int newsize);
char* strshl(char* str, int count);
char* strshr(char* str, int count);
int strsocc(char* str1, char* str2);
char* strsrep(char* str, const char* search, const char* replace);
char* strtrim(char* str);
char* struplow(char* str);
char touplow(char* str, char* pos, char ch);

char* longdotstr(long num);                    // Convert a long to a dotted string: xxx.xxx.xxx.xxx
char* longdotstr(char* str, long num);                    // Convert a long to a dotted string: xxx.xxx.xxx.xxx
char* longtostr(char* str, long num);                     // Convert a long to a string (sign is ignored)
const char* stradd(const char* str1, const char* str2);                     // Add two strings, return pointer to added string
char* strcap(char* str);                                  // Convert string to Caps
char* strfind(char* findin, char* find, long length);     // Find data (case sensitive)
char* strfindtxt(char* findin, char* find, long length);  // Find text (case insensitive)
char* StripQuotes(char* str);
inline char* strstripquotes(char* str) { return StripQuotes(str); }
char* strxcpy(char* d, const char* s, int n);                        // Copy max n chars and nul-term dest
char* strtrunc(char* dest, const char* src, int width);

char* strc2p(char* str);
char* strc2pc(char* dest, const char* str);
char* strnp2c(char* str, int n);
char* strnp2cc(char* dest, const char* str, int n);
char* strp2c(char* str);

const char* strlword(const char* str);
const char* strrword(const char* str);

char* _stdc strcat_(char* dest, ...);                                // strcat for one or more strings
char* _stdc strcpy_(char* dest, ...);                                // strcpy for one or more strings
char* _stdc strlink(char* chain, ...);                               // Links strings together to a chain
char* _stdc strsjust(char* chain, int len, int spacing, ...);        // Justifies N strings

int strioccs(char* str, char* chs);
int stroccs(char* str, char* chs);
int strtrimc(char* buf);                                   // Trim whitespace off end and return last char

char* strtrimspace(char* str);

int is_upper(char ch);
char to_lower(char ch);
char to_upper(char ch);

char* strrevname(char* reversedname, const char* name);
char* strunrevname(char* unreversedname, const char* name);


//  ------------------------------------------------------------------
//  Inliners

#define STRNP2C(str)        strnp2c(str, sizeof(str)-1)
#define STRNP2CC(dest,str)  strnp2cc(dest, str, sizeof(str)-1)

inline int iswhite(byte c)   { return c < '!' and c; }
inline int isumlaut(byte ch) { return (ch>=128 and ch<=155) or (ch==157) or (ch>=159 and ch<=165) or (ch==225); }
inline int isxalnum(byte ch) { return isalnum(ch) or isumlaut(ch); }

inline char* strbtrim  (char* st)          { return strtrim(strltrim(st)); }
inline char* strleft   (char* st, int nu)  { return strmid(st,0,nu); }
inline char* strright  (char* st, int nu)  { return strmid(st,strlen(st)-nu,nu); }
inline char* strrtrim  (char* st)          { return strtrim(st); }

inline char* inttostr  (char* str, long num)            { return longtostr(str, num); }
inline int streql    (const char* str1, const char* str2)          { return !strcmp(str1,str2); }
inline int strieql   (const char* str1, const char* str2)          { return !stricmp(str1,str2); }
inline int strneql   (const char* str1, const char* str2, int n)  { return !strncmp(str1,str2,n); }
inline int strnieql  (const char* str1, const char* str2, int n)  { return !strnicmp(str1,str2,n); }

inline int memeql    (const char* str1, const char* str2, int n)  { return !memcmp(str1,str2,n); }

#define fastSkipTok(p)    { while('!' <= *(p)) (p)++; while(('!' > *(p)) and *(p)) (p)++; }
#define fastSkipWht(p)    { while(('!' > *(p)) and *(p)) (p)++; }
#define fastSkipTxt(p)    { while('!' <= *(p)) (p)++; }
#define fastSkipTo(p,c)   { register char cCc=(c); while((*(p)!=cCc) and *(p)) (p)++; }
#define fastNulTermNxt(p) { if(*(p)) *(p)++ = NUL; }

inline const char* strskip_to(const char* p, char* s) { const char* t=strpbrk(p, s); if(t) return t; return p+strlen(p); }
const char* strskip_txt(const char* p);
const char* strskip_wht(const char* p);
const char* strskip_past(const char* p, char c);
const char* strskip_past(const char* p, const char* s);
const char* strskip_digits(const char* ptr);

char* strskip_to(char* p, char c);
inline char* strskip_to(char* p, char* s) { char* t=strpbrk(p, s); if(t) return t; return p+strlen(p); }
char* strskip_txt(char* p);
char* strskip_wht(char* p);
char* strskip_past(char* p, char c);
char* strskip_past(char* p, char* s);
inline char* strskip_digits(char* ptr) { return (char*)strskip_digits((const char*)ptr); }

#define _strtrim_wht(p)     { int i; for(i=strlen(p)-1; ('!' > p[i]) and (i >= 0); i--); p[i+1] = NUL; }
#define _strskip_wht(p)     { while(('!' > *(p)) and *(p)) (p)++; }
#define _strskip_txt(p)     { while('!' <= *(p)) (p)++; }
#define _strskip_to(p,c)    { register char cCc=(c); while((*(p)!=cCc) and *(p)) (p)++; }
#define _strskip_past(p,c)  { register char cCc=(c); while((*(p)==cCc) and *(p)) (p)++; }

char* NulTermNxt(char* p);
inline char* SkipTo(char* p, char c) { return strskip_to(p, c); }
char* SkipToNul(char* p);
char* SkipToXlat(char* p, char c, char a, char b);
inline char* SkipTxt(char* p) { return strskip_txt(p); }
inline char* SkipWht(char* p) { return strskip_wht(p); }

#if defined(__GNUC__) or defined(__WATCOMC__)
inline char * stpcpy(char* dest, const char* src) {
  strcpy(dest, src);
  return dest + strlen(src);
}
#endif

#ifndef isoctal
#define isoctal(c) (((c) >= '0') and ((c) <= '7'))
#endif

char* strcvtc(char* s);

int strlookup(const char** table, const char* lookup);

char* _stdc gstrf(const char* format, ...);

const char* ulong_to_cstr(ulong n);


//  ------------------------------------------------------------------
//  Character case conversion

const int CASE_NEUTRAL = 0;
const int CASE_UPPER   = 1;
const int CASE_LOWER   = 2;

inline char CaseTo(int __case, char __chr) {

  return (char)((__case==CASE_UPPER) ? toupper(__chr) : ((__case==CASE_LOWER) ? tolower(__chr) : __chr));
}


//  ------------------------------------------------------------------
//  String tokenizer class

class GTok {

protected:

  char* separator;
  char* token;

public:

  GTok(char* sep=NULL);

  char* First(char* buf)  { token = strtok(buf,  separator); return token; }
  char* Next()           { token = strtok(NULL, separator); return token; }
  char* Token()          { return token; }
};


//  ------------------------------------------------------------------
//  String tokenizer class

class gtokenizer {

protected:

  char* separator;
  char* token;
  char* input;

public:

  gtokenizer(char* buf)  { separator = ", \t"; input = buf; }
  gtokenizer(char* sep, char* buf)  { separator = sep; input = buf; }

  const char* get()    { token = strtok(input, separator); input = NULL; return token; }
  char*       str()    { return token; }
  const char* c_str()  { return token; }

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

