
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gstrbags.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  String bag class.
//  ------------------------------------------------------------------


#ifdef __BORLANDC__
//#pragma option -Od
#endif

#include <gmemdbg.h>
#include <gfile.h>
#include <gstrall.h>
#include <gstrbags.h>


//  ------------------------------------------------------------------

#if defined(__MSDOS__) and not defined(__386__)
const int BLOCKSIZE = 256;
#else
const int BLOCKSIZE = 4096;
#endif


//  ------------------------------------------------------------------

GStrBag::GStrBag() {

  items = 0;
  bagsize = 0;
  bag = NULL;
  blocksize = BLOCKSIZE;
  currno = 0;
}


//  ------------------------------------------------------------------

GStrBag::~GStrBag() {

  Reset();
}


//  ------------------------------------------------------------------

void GStrBag::Reset() {

  throw_release(bag);
}


//  ------------------------------------------------------------------

int GStrBag::Add(const char* string) {

  return Add(string, strlen(string)+1);
}


//  ------------------------------------------------------------------

int GStrBag::Add(const void* data, int length) {

  if(items == 0)
    bag = (char*)throw_malloc(blocksize);
  register int currsize = bagsize + (items*sizeof(int));
  register int currsizeblks = (currsize/blocksize) + 1;
  register int newsizeblks = ((currsize+length+sizeof(int))/blocksize) + 1;
  if(newsizeblks != currsizeblks)
    bag = (char*)throw_realloc(bag, newsizeblks*blocksize);
  memmove(bag+bagsize+length, bag+bagsize, items*sizeof(int));
  memcpy(bag+bagsize, data, length);
  ((int*)(bag+bagsize+length))[items] = bagsize;
  bagsize += length;
  return items++;
}


//  ------------------------------------------------------------------

void GStrBag::Change(int index, const char* string) {

  Change(index, string, strlen(string)+1);
}


//  ------------------------------------------------------------------

void GStrBag::Change(int index, const void* data, int length) {

  int oldpos = Pos(index);
  int oldlen = strlen(bag+oldpos)+1;   //////////////////////////////
  int lendiff = length - oldlen;
  int oldsize = bagsize+(items*sizeof(int));
  int movesize = oldsize - oldpos - oldlen;
  int currsizeblks = (oldsize/blocksize) + 1;
  int newsizeblks = ((oldsize+lendiff)/blocksize) + 1;
  if(lendiff > 0) {
    if(newsizeblks != currsizeblks)
      bag = (char*)throw_realloc(bag, oldsize+lendiff);
    memmove(bag+oldpos+length, bag+oldpos+oldlen, movesize);
  }
  else if(lendiff < 0) {
    memmove(bag+oldpos+length, bag+oldpos+oldlen, movesize);
    if(newsizeblks != currsizeblks)
      bag = (char*)throw_realloc(bag, oldsize+lendiff);
  }
  memcpy(bag+oldpos, data, length);
  bagsize += lendiff;
  for(int n=index+1; n<items; n++)
    Pos(n) += lendiff;
}


//  ------------------------------------------------------------------

const char* GStrBag::operator[](int index) {

  return Index(index);
}


//  ------------------------------------------------------------------

void GStrBag::Load(gfile& fp) {

  fp.fread(&bagsize, sizeof(bagsize));
  fp.fread(&items, sizeof(items));
  int size = bagsize + (items*sizeof(int));
  bag = (char*)throw_realloc(bag, ((size/blocksize)+1)*blocksize);
  fp.fread(bag, size);
}


//  ------------------------------------------------------------------

void GStrBag::Save(gfile& fp) const {

  fp.fwrite(&bagsize, sizeof(bagsize));
  fp.fwrite(&items, sizeof(items));
  fp.fwrite(bag, bagsize+(items*sizeof(int)));
}


//  ------------------------------------------------------------------

void GStrSet3::Put(const char* s1, const char* s2, const char* s3) {

  int len1 = strlen(s1 ? s1 : "")+1;
  int len2 = strlen(s2 ? s2 : "")+1;
  int len3 = strlen(s3 ? s3 : "")+1;
  cfg.size = len1 + len2 + len3;
  set = new char[cfg.size];
  throw_new(set);
  cfg.pos2 = len1;
  cfg.pos3 = len1 + len2;
  memcpy(set, s1 ? s1 : "", len1);
  memcpy(set+cfg.pos2, s2 ? s2 : "", len2);
  memcpy(set+cfg.pos3, s3 ? s3 : "", len3);
}


//  ------------------------------------------------------------------

void GStrSet3::Change(const char* s1, const char* s2, const char* s3) {
  
  char* oldset = set;
  Put(s1, s2, s3);
  delete[] oldset;
}


//  ------------------------------------------------------------------

void GStrSet3::Load(gfile& fp) {

  fp.fread(&cfg, sizeof(cfg));
  set = new char[cfg.size];
  throw_new(set);
  fp.fread(set, cfg.size);
}


//  ------------------------------------------------------------------

void GStrSet3::Save(gfile& fp) const {

  fp.fwrite(&cfg, sizeof(cfg));
  fp.fwrite(set, cfg.size);
}


//  ------------------------------------------------------------------

