
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gstrblks.h,v 1.1.1.1 1998/11/21 19:38:09 odinn Exp $
//  ------------------------------------------------------------------
//  String blocks.
//  ------------------------------------------------------------------


#ifndef __GSTRBLKS_H
#define __GSTRBLKS_H


//  ------------------------------------------------------------------

#include <gstrgstr.h>
#include <gmemblks.h>


//  ------------------------------------------------------------------

class gstrarray : public gmemblock {

public:

  gstrarray(int extra=0) : gmemblock(8192, 4096, extra)  { }
  ~gstrarray()  { }

  uint        length(int n) const               { return gmemblock::length(n)-1; }

  int         add(const char* s);
  int         add(const gstring& s);
  int         set(int n, const char* s);
  int         set(int n, const gstring& s);
  int         insert(int n, const char* s);
  int         insert(int n, const gstring& s);

  char*       get(int n) const                  { return gmemblock::get(n); }
  const char* c_str(int n) const                { return gmemblock::get(n); }
  const char* operator[](int n) const           { return get(n); }

  void        operator+=(const char* s)         { add(s); }
  void        operator+=(const gstring& s)      { add(s); }

};


//  ------------------------------------------------------------------

class gstrmultiarray : public gstrarray {

protected:

  int multi;

  void operator[](int);

public:

  gstrmultiarray(int m) { multi = m; }
  ~gstrmultiarray() { }

  int         size()                             { return gstrarray::size() / multi; }

  uint        length(int n, int m) const         { return gstrarray::length((n*multi)+m); }

  int         set(int n, int m, const char* s)   { return gstrarray::set((n*multi)+m, s); }

  const char* get(int n, int m) const            { return gstrarray::get((n*multi)+m); }

};


//  ------------------------------------------------------------------

void tokenize(gstrarray& array, const char* str, const char* delim=NULL);
inline void tokenize(gstrarray& array, const gstring& str, const char* delim=NULL)  { tokenize(array, str.c_str(), delim); }


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

