
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gstrctyp.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  String upper/lower and NLS functions.
//  ------------------------------------------------------------------


#include <ctype.h>
#include <string.h>
#include <gstrall.h>
#include <gutlmisc.h>


//  ------------------------------------------------------------------
//  Takes a string and makes it lowercase with first letters caps

#if 0
char* strcap(char* str) {

  char alf = true;

  for(char* p=str; *p; p++) {
    if(isalpha(*p)) {
      if(alf) {
        *p = (char)toupper(*p);
        alf = false;
      }
      else {
        *p = (char)tolower(*p);
      }
    }
    else {
      alf = true;
    }
  }

  return str;
}
#endif


//  ------------------------------------------------------------------
//  Converts a string to mixed upper & lower case

char* struplow(char* str) {

  register int i;

  for(i=0; *(str+i); i++)
    *(str+i) = touplow(str, str+i, *(str+i));

  return str;
}


//  ------------------------------------------------------------------
//  Converts a character to upper or lower case depending on the
//  previous character in the string

char touplow(char* str, char* pos, char ch) {

  register char i;

  switch(*(pos-1)) {                  // check previous character
    case ' ':                         // see if it is a separator
    case '-':
    case '_':
    case ',':
    case '.':
    case '/':
      i = (char)toupper(ch);                // if it is, convert to upper
      break;
    default:
      if(pos==str)                    // see if at beginning of string
        i = (char)toupper(ch);              // if so, convert to upper
      else
        i = (char)tolower(ch);              // otherwise, convert to lower
  }

  return i;                           // return converted character
}


//  ------------------------------------------------------------------
//  Alternative toupper function. Works only with CP 865 DK

char to_upper(char ch) {

  if(islower(ch))
    return (char)toupper(ch);
  else if((byte)ch > 127) {
    switch(ch) {
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
    }
  }
  return(ch);
}


//  ------------------------------------------------------------------
//  Alternative tolower function. Works only with CP 865 DK

char to_lower(char ch) {

  if(isupper(ch))
    return (char)tolower(ch);
  else if((byte)ch > 127) {
    switch(ch) {
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
      case '':
        return('');
    }
  }
  return(ch);
}


//  ------------------------------------------------------------------
//  Alternative isupper function. Works only with CP 865 DK

int is_upper(char ch) {

  if(isupper(ch))
    return(YES);
  else if((byte)ch > 127) {
    switch(ch) {
      case '':
        return(YES);
      case '':
        return(YES);
      case '':
        return(YES);
      case '':
        return(YES);
      case '':
        return(YES);
      case '':
        return(YES);
      case '':
        return(YES);
      case '':
        return(YES);
      case '':
        return(YES);
    }
  }
  return(NO);
}


//  ------------------------------------------------------------------

char* strcvtc(char* str) {

  char c;
  register char* s = str;
  register char* d = s;
  while(*s) {
    if(*s == '\\') {
      s++;
      switch(*s) {
        case 'n':  *d++ = '\n'; goto skip_constant;
        case 't':  *d++ = '\t'; goto skip_constant;
        case 'v':  *d++ = '\v'; goto skip_constant;
        case 'b':  *d++ = '\b'; goto skip_constant;
        case 'r':  *d++ = '\r'; goto skip_constant;
        case 'f':  *d++ = '\f'; goto skip_constant;
        case 'a':  *d++ = '\a'; goto skip_constant;
        case '\\': *d++ = '\\'; goto skip_constant;
        case '\'': *d++ = '\''; goto skip_constant;
        case '\"': *d++ = '\"'; goto skip_constant;
        case '\?': *d++ = '\?'; goto skip_constant;
        case 'x':
          if(isxdigit(s[1])) {
            s++;
            c = 0;
            while(isxdigit(*s)) {
              c <<= 4;
              c |= (char)xtoi(*s);
              s++;
            }
            *d++ = c;
            continue;
          }
          break;
        default:
          if(isoctal(*s)) {
            c = 0;
            while(isoctal(*s)) {
              c <<= 3;
              c |= (char)(*s - '0');
              s++;
            }
            *d++ = c;
          }
          else {
            s++;
            continue;
          }
      }
      *d++ = *s++;
      continue;

      skip_constant:
      s++;
      continue;
    }
    *d++ = *s++;
  }
  *d = NUL;
  return str;
}


//  ------------------------------------------------------------------

