
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gstrgstr.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  Simple string class.
//  ------------------------------------------------------------------


#include <stdlib.h>
#include <gmemdbg.h>
#include <gfile.h>
#include <gstrall.h>
#include <gstrgstr.h>


//  ------------------------------------------------------------------

char* gstring::null_txt = "";
const int gstring::npos = (int)-1;


//  ------------------------------------------------------------------

gstring::gstring()                             { reset(); }
gstring::gstring(char c)                       { construct(1); *txt = c; txt[1] = NUL; len = 0; }
gstring::gstring(const char* s)                { reset(); copy(s); }
gstring::gstring(const gstring& s)             { reset(); allocate(s.cap); len=s.len; strcpy(txt, s.txt); }
gstring::gstring(const char* s, int n)         { reset(); copy_n(s, n); }
gstring::gstring(int n, char c)                { reset(); allocate(n); memset(txt, c, n); txt[n] = NUL; len = n; }
gstring::~gstring()                            { free(); }

gstring& gstring::operator=(char c)            { allocate(1); *txt = c; txt[1] = NUL; len = c ? 1 : 0; return *this; }
gstring& gstring::operator=(const char* s)     { copy(s); return *this; }
gstring& gstring::operator=(const gstring& s)  { allocate(s.cap); len=s.len; strcpy(txt, s.txt); return *this; }

gstring& gstring::append(char c)               { resize(len+1, c); return *this; }
gstring& gstring::append(const char* s)        { int n=strlen(s); resize(len+n); strcat(txt, s); return *this; }
gstring& gstring::append(const gstring& s)     { int n=s.len; resize(len+n); strcat(txt, s.txt); return *this; }


//  ------------------------------------------------------------------

void gstring::construct(int n) {

  txt = null_txt;
  cap = 0;
  allocate(n);
}


//  ------------------------------------------------------------------

void gstring::allocate(int n) {

  if((n > cap) or (n == 0)) {
    cap = n;
    if(txt != null_txt)
      ::free(txt);
    txt = (char*)malloc(cap+1);
    throw_new(txt);
  }
}


//  ------------------------------------------------------------------

void gstring::resize(int n, char c) {

  if((n > cap) or (n == 0)) {
    cap = n;
    if(txt == null_txt)
      txt = NULL;
    txt = (char*)realloc(txt, cap+1);
    throw_new(txt);
  }
  if(n > len)
    memset(txt+len, c, n-len);
  txt[n] = NUL;
  len = n;
}


//  ------------------------------------------------------------------

void gstring::free() {

  if(txt != null_txt)
    ::free(txt);
}


//  ------------------------------------------------------------------

void gstring::copy(const char* s) {

  if(s) {
    len = strlen(s);
    allocate(len);
    memmove(txt, s, len+1);
  }
}


//  ------------------------------------------------------------------

void gstring::copy_n(const char* s, int n) {

  if(s) {
    allocate(n);
    memmove(txt, s, n);
    txt[n] = NUL;
    len = strlen(txt);
  }
}


//  ------------------------------------------------------------------

int gstring::find(char c, int pos) const {

  const char* f = strchr(txt+pos, c);
  return f ? int(f-txt) : npos;
}


//  ------------------------------------------------------------------

gstring& gstring::trim() {

  strtrim(txt);
  len = strlen(txt);
  return *this;
}


//  ------------------------------------------------------------------

gstring& gstring::btrim() {

  strbtrim(txt);
  len = strlen(txt);
  return *this;
}


//  ------------------------------------------------------------------

gstring& gstring::upper() {

  strupr(txt);
  return *this;
}


//  ------------------------------------------------------------------

gstring& gstring::lower() {

  strlwr(txt);
  return *this;
}


//  ------------------------------------------------------------------

gstring& gstring::strip_quotes() {

  StripQuotes(txt);
  len = strlen(txt);
  return *this;
}


//  ------------------------------------------------------------------

bool gstring::blank() const {

  return strblank(txt);
}


//  ------------------------------------------------------------------

void gstring::erase(const char* from) {

  len = (int)(from-txt);
  txt[len] = NUL;
}


//  ------------------------------------------------------------------

void gstring::save(gfile& fp) const {

  fp << len;
  ::save(fp, txt, len);
}


//  ------------------------------------------------------------------

gfile& operator<<(gfile& fp, const gstring& s) {

  s.save(fp);
  return fp;
}


//  ------------------------------------------------------------------

void gstring::load(gfile& fp) {

  fp >> len;
  allocate(len);
  ::load(fp, txt, len);
}


//  ------------------------------------------------------------------

gfile& operator>>(gfile& fp, gstring& s) {

  s.load(fp);
  return fp;
}


//  ------------------------------------------------------------------

bool gstring::ichange(const char* from, const char* to) {

  char* tmp = (char*)throw_malloc(len*2);
  strcpy(tmp, txt);
  bool changed = strischg(tmp, from, to) != NULL;
  if(changed)
    copy(tmp);
  throw_free(tmp);
  return changed;
}


//  ------------------------------------------------------------------

int _stdc gstring::strf(const char* format, ...) {

  char buf[2048];
  va_list argptr;
  va_start(argptr, format);
  vsprintf(buf, format, argptr);
  va_end(argptr);
  copy(buf);
  return length();
}


//  ------------------------------------------------------------------

int _stdc compare_gstring(const gstring* a, const gstring* b) {

  return a->cmp(*b);
}


//  ------------------------------------------------------------------

int _stdc comparei_gstring(const gstring* a, const gstring* b) {

  return a->icmp(*b);
}


//  ------------------------------------------------------------------

