
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gstrmail.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  RFC822 related character/string classes and functions.
//  ------------------------------------------------------------------


#include <gstrall.h>
#include <gstrmail.h>


//  ------------------------------------------------------------------

mail_ctype __FAR mail_ctype_global;


//  ------------------------------------------------------------------

mail_ctype::mail_ctype() {

  memset(table,     mail_char|mail_ctl,   32*sizeof(uint));
  memset(table+32,  mail_char,            96*sizeof(uint));
  memset(table+'A', mail_char|mail_alpha, ('Z'-'A'+1)*sizeof(uint));
  memset(table+'a', mail_char|mail_alpha, ('z'-'a'+1)*sizeof(uint));
  memset(table+128, 0,                    128*sizeof(uint));
  table[127]  |= mail_ctl;
  table[0]    |= mail_delimiters;
  table[' ']  |= mail_delimiters | mail_lwsp;
  table[HT]   |= mail_delimiters | mail_lwsp;
  table['(']  |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table[')']  |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table['<']  |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table['>']  |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table['@']  |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table[',']  |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table[';']  |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table[':']  |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table['\\'] |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table['\"'] |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table['[']  |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table[']']  |= mail_delimiters | mail_special | mime_tspecial | mime_especial;
  table['.']  |= mail_delimiters | mail_special                 | mime_especial;
  table['/']  |=                                  mime_tspecial | mime_especial;
  table['?']  |=                                  mime_tspecial | mime_especial;
  table['=']  |=                                  mime_tspecial | mime_especial;
}


//  ------------------------------------------------------------------

const char* strskip_lwsp(const char* p) {

  while(is_mail_lwsp(*p))
    p++;
  return p;
}


//  ------------------------------------------------------------------

const char* strskip_mail_quote(const char* p) {

  if(*p == '\"')
    p++;
  while(*p and (*p != '\"')) {
    if(*p == '\\')
      p++;
    p++;
  }
  if(*p == '\"')
    p++;

  return p;
}


//  ------------------------------------------------------------------

const char* strskip_delimiters(const char* p) {

  while(is_mail_delimiters(*p))
    p++;
  return p;
}


//  ------------------------------------------------------------------

const char* strskip_to_delimiters(const char* p) {

  while(!is_mail_delimiters(*p))
    p++;
  return p;
}


//  ------------------------------------------------------------------

char* strtrim_lwsp(char* p) {

  char* p0 = p;
  p += strlen(p) - 1;
  while(is_mail_lwsp(*p)) {
    if(p == p0)
      break;
    p--;
  }
  return p;
}


//  ------------------------------------------------------------------

char* strmail_quote(char* ot, const char* it) {

  char* o = ot;
  const char* i = it;

  if(*o == '\"')
    o++;
  while(*i) {
    if(is_mail_special(*i))
      *o++ = '\\';
    *o++ = *i++;
  }
  if(*ot == '\"')
    *o++ = '\"';
  *o = NUL;

  return ot;
}


//  ------------------------------------------------------------------

char* strmail_unquote(char* ot, const char* it) {

  register char* o = ot;
  register const char* i = it;
  bool was_quoted = false;

  if(*i == '\"') {
    was_quoted = true;
    i++;
  }
  while(*i) {
    if(*i == '\\')
      i++;
    *o++ = *i++;
  }
  if(was_quoted)
    o--;
  *o = NUL;

  return ot;
}


//  ------------------------------------------------------------------

#if 0
char* strmail_unquote_qtext(char* ot, const char* it) {

  char* o = ot;
  const char* i = it;

  if(*i == '\"')
    i++;
  while(*i)
    *o++ = *i++;
  if(*it == '\"')
    o--;
  *o = NUL;

  return ot;
}
#endif


//  ------------------------------------------------------------------

int is_mail_lwsp(const char* s) {

  while(is_mail_lwsp(*s))
    s++;
  return *s == NUL;
}


//  ------------------------------------------------------------------

