
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gstrxlat.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  Character translation.
//  ------------------------------------------------------------------


#include <gstrxlat.h>


//  ------------------------------------------------------------------

gcharxlat::gcharxlat() {

  charset = NULL;
  active = false;
}


//  ------------------------------------------------------------------

gcharxlat::~gcharxlat() {

  delete[] charset;
}


//  ------------------------------------------------------------------

void gcharxlat::initialize() {

  charset = new gcharalias[256];
  throw_new(charset);
}


//  ------------------------------------------------------------------

void gcharxlat::reset_table() {

  active = false;
  for(int n=0; n<256; n++) {
    charset[n][0] = (char)n;
    charset[n][1] = NUL;
  }
}


//  ------------------------------------------------------------------

void gcharxlat::reset_from_charset() {

  from_charset = "";
  reset_table();
}


//  ------------------------------------------------------------------

const char* gcharxlat::get_charset_name(const char* name) {

  // TODO
  return name;
}


//  ------------------------------------------------------------------

bool gcharxlat::load_charsets() {

  // TODO
  return false;
}


//  ------------------------------------------------------------------

void gcharxlat::build_table() {

  reset_table();

  if(to_charset and from_charset) {

    if(to_charset != from_charset) {
      if(load_charsets()) {
        for(int f=0; f<xlat_from.size(); f++) {
          bool found = false;
          char ch = xlat_from[f].ch;
          word iso = xlat_from[f].iso;
          for(int t=0; t<xlat_to.size(); t++) {
            if(iso == xlat_to[t].iso) {
              charset[ch][0] = xlat_to[t].ch;
              charset[ch][1] = NUL;
              found = true;
              break;
            }
          }
          if(not found) {
            for(int a=0; a<xlat_alias.size(); a++) {
              if(iso == xlat_alias[a].iso) {
                strcpy(charset[ch], xlat_alias[a].alias);
                break;
              }
            }
          }
        }
        active = true;
      }
    }
  }
}


//  ------------------------------------------------------------------

void gcharxlat::set_to_charset(const gstring& to) {

  to_charset = get_charset_name(to.c_str());
  build_table();
}


//  ------------------------------------------------------------------

void gcharxlat::set_from_charset(const char* from) {

  from_charset = get_charset_name(from);
  build_table();
}


//  ------------------------------------------------------------------

void gcharxlat::set_from_codepage(int from) {

  from_charset.strf("CP%i", from);
  from_charset = get_charset_name(from_charset.c_str());
  build_table();
}


//  ------------------------------------------------------------------

const char* gcharxlat::translate(const char* from, char* to) {

  if(active) {
    while(*from) {
      const char* tptr = charset[*from++];
      while(*tptr)
        *to++ = *tptr++;
    }
    *to = NUL;
    return to;
  }
  return from;
}


//  ------------------------------------------------------------------

