
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gtag.h,v 1.1.1.1 1998/11/21 19:38:09 odinn Exp $
//  ------------------------------------------------------------------
//  Tag number types and set class.
//  ------------------------------------------------------------------


#ifndef __GTAG_H
#define __GTAG_H


//  ------------------------------------------------------------------

#include <gdefs.h>


//  ------------------------------------------------------------------
//  Constants

const ulong GTAGN_INVALID = 0;
const uint  GRELN_INVALID = 0;

const ulong GTAGN_MAX = ((ulong)-1);
const uint  GRELN_MAX = ((uint)-1);

const bool  GTAGN_CLOSEST = true;
const bool  GTAGN_EXACT   = false;


//  ------------------------------------------------------------------
//  Tag class

class gtag {

protected:

  uint delete_absolute_resize(ulong tagn);

public:

  ulong* tag;                // tag array
  uint   tags;               // tags in array
  uint   allocated;          // actual allocated tags
  uint   granularity;        // memory allocation optimization

  gtag();
  ~gtag();

  uint   size()   { return tags; }
  bool   empty()  { return size() == 0; }

  void   reset();
  ulong* resize(uint newtags);

  uint   find(ulong absolute)  { return absolute_to_relative(absolute); }

  uint   absolute_to_relative(ulong absolute);
  uint   absolute_to_relative(ulong absolute, bool closest);

  ulong  relative_to_absolute(uint relative);

  ulong* append(ulong tagn);
  ulong* add(ulong tagn);

  uint   delete_absolute(ulong tagn);
  uint   delete_relative(uint reln);

  void   sort();
  void   eliminate_duplicates();

  ulong& operator[](uint n)      { return tag[n]; }
  ulong& at(uint n)              { return tag[n]; }

  void   operator+=(ulong tagn)  { add(tagn); }

};


//  ------------------------------------------------------------------
//  Prototypes

int _stdc gtag_compare(const ulong* a, const ulong* b);


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

