
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gtimall.h,v 1.1.1.1 1998/11/21 19:38:09 odinn Exp $
//  ------------------------------------------------------------------
//  Time utility functions.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GTIMALL_H
#define __GTIMALL_H


//  ------------------------------------------------------------------
//  Required headers

#include <time.h>
#include <stddef.h>
#include <gdefs.h>


//  ------------------------------------------------------------------
//  Re-type the standard time types

typedef ulong Time;
typedef struct tm Tm;
typedef long Clock;


//  ------------------------------------------------------------------
//  DOS "findfirst" timestamp

struct gfiletime {
  unsigned ft_tsec  : 5;   // Second / 2
  unsigned ft_min   : 6;   // Minutes
  unsigned ft_hour  : 5;   // Hours
  unsigned ft_day   : 5;   // Days
  unsigned ft_month : 4;   // Months
  unsigned ft_year  : 7;   // Year - 80

  const char* c_str(char* buf);
  dword number() { return *(dword*)this; }
};

typedef gfiletime FFTime;


//  ------------------------------------------------------------------
//  Opus DOS-style file timestamp

struct gopustime {
  unsigned ft_day   : 5;   // Days
  unsigned ft_month : 4;   // Months
  unsigned ft_year  : 7;   // Year - 80
  unsigned ft_tsec  : 5;   // Second / 2
  unsigned ft_min   : 6;   // Minutes
  unsigned ft_hour  : 5;   // Hours

  const char* c_str(char* buf);
  dword number() { return *(dword*)this; }
};

typedef gopustime FTime;


//  ------------------------------------------------------------------

struct GTime {
  int year;
  int month;
  int day;
  int hour;
  int minute;
  int second;
  int msecond;
};


//  ------------------------------------------------------------------
//  Externs for strftimei()

extern char* __gsweekday[];
extern char* __glweekday[];
extern char* __gsmonth[];
extern char* __glmonth[];
extern char* __gampm[];

extern char* gsweekday[];
extern char* glweekday[];
extern char* gsmonth[];
extern char* glmonth[];
extern char* gampm[];

extern const char* gmonths[];


//  ------------------------------------------------------------------
//  Prototypes

void   Pause      (long duration);
void   Pause2     (long duration);
int    str2mon    (const char* ptr);
char*  sysdate    (int dtype);
char*  systime    (int ttype);

GTime* gtimems    (GTime* t=NULL);
ulong  gtime      (ulong* tloc=NULL);
Tm*    glocaltime (const ulong* clock);
ulong  gmktime    (Tm* t);
uint   strftimei  (char* s, uint maxsize, const char* fmt, const Tm* t);
Clock  gclock     ();

FTime  TimeToFTime(ulong __time);
ulong FTimeToTime(FTime* __ftime, Tm* __tm=NULL);

inline char* strftimeip(char* s, uint maxsize, const char* fmt, const Tm* t) { strftimei(s, maxsize, fmt, t); return s; }

ulong FidoTimeToUnix(char* __fidotime);
void   ftime_to_tm(FTime* FT, struct tm *TM);
char*  FTimeToStr(char* buf, FTime &t);
char*  TimeToStr(char* buf, ulong t);
void   GetFTime(int fh, FTime& t);

int    day_of_week(int year, int month, int day);


//  ------------------------------------------------------------------

long YMD2JDN(unsigned yr, unsigned mo, unsigned day);
void JDN2YMD(long scalar, unsigned *yr, unsigned *mo, unsigned *day);

inline int      isleap(unsigned yr)             { return yr % 400 == 0 || (yr % 4 == 0 && yr % 100 != 0); }
inline unsigned months_to_days(unsigned month)  { return (month * 3057 - 3007) / 100; }
inline long     years_to_days(unsigned yr)      { return yr * 365L + yr / 4 - yr / 100 + yr / 400; }


//  ------------------------------------------------------------------

class GTim {

public:

  ulong starttime;

  GTim() { starttime=gtime(); }
};


//  ------------------------------------------------------------------

extern GTim gtim;


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

