
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gtimutil.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  Time utility functions.
//  ------------------------------------------------------------------


#include <time.h>
#include <stdio.h>
#include <stdlib.h>

#include <gio.h>

#include <gmemall.h>
#include <gstrall.h>
#include <gtimall.h>
#include <gutlmisc.h>
#include <gutlmtsk.h>

#ifdef __OS2__
#define INCL_BASE
#include <os2.h>
#endif

#ifdef __WIN32__
#include <windows.h>
#endif


//  ------------------------------------------------------------------

GTim gtim;


//  ------------------------------------------------------------------

const char* gmonths[] = {
  "ERR",
  "Jan", "Feb", "Mar",
  "Apr", "May", "Jun",
  "Jul", "Aug", "Sep",
  "Oct", "Nov", "Dec",
  "ERR"
};


//  ------------------------------------------------------------------

static uint __FAR Days[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
static uint __FAR YDays[12] = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };
static Tm  __FAR tmX;


//  ------------------------------------------------------------------

GTime* gtimems(GTime* t) {

  static GTime __FAR tt;
  GTime* tp = t ? t : &tt;

  #if defined(__MSDOS__)

  i86 cpu;

  // Get system date
  cpu.r.h.ah = 0x2A;
  cpu.genint(0x21);
  tp->year  = cpu.r.w.cx;
  tp->month = cpu.r.h.dh;
  tp->day   = cpu.r.h.dl;

  // Get system time
  cpu.r.h.ah = 0x2C;
  cpu.genint(0x21);
  tp->hour    = cpu.r.h.ch;
  tp->minute  = cpu.r.h.cl;
  tp->second  = cpu.r.h.dh;
  tp->msecond = (int)cpu.r.h.dl*10;

  // Get system date again if midnight was just passed
  if((tp->hour == 0) and (tp->minute == 0)) {
    cpu.r.h.ah = 0x2A;
    cpu.genint(0x21);
    tp->year  = cpu.r.w.cx;
    tp->month = cpu.r.h.dh;
    tp->day   = cpu.r.h.dl;
  }

  #elif defined(__OS2__)

  DATETIME dt;
  DosGetDateTime(&dt);
  tp->year    = dt.year;
  tp->month   = dt.month;
  tp->day     = dt.day;
  tp->hour    = dt.hours;
  tp->minute  = dt.minutes;
  tp->second  = dt.seconds;
  tp->msecond = (int)dt.hundredths*10;

  #elif defined(__WIN32__)

  SYSTEMTIME dt;
  GetLocalTime(&dt);
  tp->year    = dt.wYear;
  tp->month   = dt.wMonth;
  tp->day     = dt.wDay;
  tp->hour    = dt.wHour;
  tp->minute  = dt.wMinute;
  tp->second  = dt.wSecond;
  tp->msecond = dt.wMilliseconds;

  #elif defined(__GNUC__)
  
  time_t ttt = time(NULL);
  tm& dt = *localtime(&ttt);

  tp->year    = dt.tm_year + 1900;
  tp->month   = dt.tm_mon + 1;
  tp->day     = dt.tm_mday;
  tp->hour    = dt.tm_hour;
  tp->minute  = dt.tm_min;
  tp->second  = dt.tm_sec;
  tp->msecond = 0;

  #endif

  return tp;
}


//  ------------------------------------------------------------------

ulong gtime2sec(GTime* dt) {

  ulong x = 24L * 60L * 60L * 3652L;  // Convert from 1980 to 1970 base date GMT
  register int i = dt->year - 1980;
  x += (i >> 2) * (1461L * 24L * 60L * 60L);
  x += (i & 3) * (24L * 60L * 60L * 365L);
  if(i & 3)
    x += 24L * 3600L;
  register int days = 0;
  i = dt->month - 1;                   // Add in months
  while(i > 0) {
    i--;
    days += Days[i];
  }
  days += dt->day - 1;
  if(dt->month > 2 and (dt->year & 3) == 0)
    days++;                           // Currently in leap year
  int hours = days * 24 + dt->hour;   // Find hours
  x += hours * 3600L;
  x += 60L * dt->minute + dt->second;

  return x;
}


//  ------------------------------------------------------------------

ulong gtime(ulong* tloc) {

  GTime dt;
  gtimems(&dt);

  ulong x = gtime2sec(&dt);

  if(tloc)
    *tloc = x;

  return x;
}


//  ------------------------------------------------------------------

Tm* glocaltime(const ulong* clock) {

  ulong time = *clock;

  tmX.tm_sec = (int)(time % 60);
  time /= 60;                             /* Time in minutes */
  tmX.tm_min = (int)(time % 60);
  time /= 60;                             /* Time in hours */
  uint i = (uint)(time / (1461L * 24L));   /* Number of 4 year blocks */
  tmX.tm_year = (i << 2);
  tmX.tm_year += 70;
  uint cumdays = 1461 * i;
  time %= 1461L * 24L;        /* Hours since end of last 4 year block */

  for(;;) {
    uint hpery = 365 * 24;
    if((tmX.tm_year & 3) == 0)
      hpery += 24;
    if(time < hpery)
      break;
    cumdays += hpery / 24;
    tmX.tm_year++;
    time -= hpery;
  }   /* at end, time is number of hours into current year */

  tmX.tm_isdst = 0;

  tmX.tm_hour = (int)(time % 24);
  time /= 24;             /* Time in days */
  tmX.tm_yday = (int)time;
  cumdays += (int)time + 4;
  tmX.tm_wday = cumdays % 7;
  time++;

  if((tmX.tm_year & 3) == 0) {
    if(time > 60)
      time--;
    else if(time == 60) {
      tmX.tm_mon = 1;
      tmX.tm_mday = 29;
      return &tmX;
    }
  }

  for(tmX.tm_mon = 0; Days[tmX.tm_mon] < time; tmX.tm_mon++)
    time -= Days[tmX.tm_mon];

  tmX.tm_mday = (int)time;

  return &tmX;
}

//  ------------------------------------------------------------------
//  Converts a date expressed in local time to the number of seconds
//  elapsed since year 70 (1970) in calendar time.
//  All arguments are zero based.
//  ------------------------------------------------------------------

static ulong gtotalsec(uint year, uint month, uint day, uint hour, uint min, uint sec) {

  int leaps;
  ulong days, secs;

  if(year >= 1970)
    year -= 1900;
  if(year < 70 or year > 138)
    return (ulong)-1;

  min += sec / 60;
  sec %= 60;              // Seconds are normalized
  hour += min / 60;
  min %= 60;              // Minutes are normalized
  day += hour / 24;
  hour %= 24;             // Hours are normalized

  year += month / 12;     // Normalize month (not necessarily final)
  month %= 12;

  while(day >= Days[month]) {
    if(!(year & 3) and (month == 1)) {
      if(day > 28) {
        day -= 29;
        month++;
      }
      else
        break;
    }
    else {
      day -= Days[month];
      month++;
    }
    year += month / 12;   // Normalize month
    month %= 12;
  }

  year -= 70;
  leaps = (year + 2) / 4;

  if(!((year+70) & 3) and (month < 2))
    leaps--;

  days = year*365L + leaps + YDays[month] + day;

  secs = days*86400L + hour*3600L + min*60L + sec;

  return secs > 0 ? secs : (ulong)-1;
}


//  ------------------------------------------------------------------

ulong gmktime(Tm* t) {

  ulong secs = gtotalsec(t->tm_year, t->tm_mon, t->tm_mday-1, t->tm_hour, t->tm_min, t->tm_sec);

  if(secs != (ulong)-1){
    glocaltime(&secs);
    *t = tmX;
  }

  return secs;
}


//  ------------------------------------------------------------------
//  Returns the value of the BIOS timer

#if defined(__MSDOS__)

dword gtimer() {

  dword* p = (dword*)0x046C;

  _disable();
  dword value = *p;
  _enable();

  return value;
}

#endif


//  ------------------------------------------------------------------
//  Suspends execution for specified duration, multitasker friendly

void Pause(long duration) {

  #if defined(__OS2__)

  DosSleep(duration);

  #elif defined(__WIN32__)

  Sleep(duration);

  #else

  Clock stop = gclock() + duration;
  while(gclock() < stop)
    gmtsk.timeslice();

  #endif
}


//  ------------------------------------------------------------------
//  Suspends execution for specified duration

void Pause2(long duration) {

  #if defined(__OS2__)

  DosSleep(duration);

  #elif defined(__WIN32__)

  Sleep(duration);

  #elif defined(__EMX__)

  _sleep2(duration);

  #elif defined(__GNUC__)

  usleep(duration*1000);

  #else

  dword stop = gtimer() + ((duration*182L) / 10000L);
  while(gtimer() < stop);

  #endif
}


//  ------------------------------------------------------------------
//  Returns an ASCII string containing the system date

#if 0
char* sysdate(int dtype) {

  char* textmonth[12] = {
    "January",
    "February",
    "March",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December"
  };

  static char __FAR buf[20];
  char textyear[5];
  unsigned int year;
  unsigned char month,day;

  #if defined(__MSDOS__)

  i86 cpu;
  cpu.r.h.ah = 0x2A;                           // DOS Get System Date
  cpu.genint(0x21);
  year=cpu.r.w.cx;
  month=cpu.r.h.dh;
  day=cpu.r.h.dl;

  #elif defined(__OS2__)

  DATETIME date;
  DosGetDateTime(&date);
  year = date.year;
  month = date.month;
  day = date.day;

  #elif defined(__WIN32__)

  SYSTEMTIME date;
  GetLocalTime(&date);
  year = date.wYear;
  month = date.wMonth;
  day = date.wDay;

  #endif

  itoa(year,textyear,10);

  switch(dtype) {                     // test date type requested
      case 0:
          sprintf(buf,"%s %u, %s",textmonth[month-1],day,textyear);
          break;
      case 1:
          sprintf(buf,"%u %.3s %s",day,textmonth[month-1],textyear+2);
          break;
      case 2:
          sprintf(buf,"%u-%u-%s",month,day,textyear+2);
          break;
      case 3:
          sprintf(buf,"%u/%u/%s",month,day,textyear+2);
          break;
      case 4:
          sprintf(buf,"%u/%u/%s",day,month,textyear+2);
          break;
      case 5:
          sprintf(buf,"%02u.%02u.%s",day,month,textyear+2);
          break;
      default:
          sprintf(buf,"%02u/%02u/%s",month,day,textyear+2);
  }

  return(buf);                        // return address of buffer
}
#endif


//  ------------------------------------------------------------------
//  Returns a string containing the system time

#if 0
char* systime(int ttype) {

  unsigned char hours,mins,secs,hunds;
  int pm=0;
  static char __FAR buf[15];

  #if defined(__MSDOS__)
  i86 cpu;
  cpu.r.h.ah=0x2c;                           // DOS Get System Time
  cpu.genint(0x21);
  hours=cpu.r.h.ch;
  mins=cpu.r.h.cl;
  secs=cpu.r.h.dh;
  hunds=cpu.r.h.dl;
  #elif defined(__OS2__)
  DATETIME date;
  DosGetDateTime(&date);
  hours = date.hours;
  mins = date.minutes;
  secs = date.seconds;
  hunds = date.hundredths;
  #elif defined(__WIN32__)
  SYSTEMTIME date;
  GetLocalTime(&date);
  hours = date.wHour;
  mins = date.wMinute;
  secs = date.wSecond;
  hunds = date.wMilliseconds/10;
  #endif

  if(hours>11) pm=1;

  switch(ttype) {                     // test requested time type
      case 0:
          sprintf(buf,"%02u:%02u:%02u.%02u",hours,mins,secs,hunds);
          break;
      case 1:
          sprintf(buf,"%02u:%02u:%02u",hours,mins,secs);
          break;
      case 2:
          if(!hours) hours=12;
          sprintf(buf,"%u:%02u %cM",(hours>12)?(hours-12):hours,mins,
              pm?'P':'A');
          break;
      case 3:
          if(!hours) hours=12;
          sprintf(buf,"%u:%02u%c",(hours>12)?(hours-12):hours,mins,
              pm?'p':'a');
          break;
      case 4:
          if(!hours) hours=12;
          sprintf(buf,"%u:%02u",(hours>12)?(hours-12):hours,mins);
          break;
      default:
          sprintf(buf,"%02u:%02u",hours,mins);
  }

  return(buf);
}
#endif


//  ------------------------------------------------------------------
//  strftimei()
//
//  A version of strftime(), which takes it's weekday and month names
//  from external arrays.
//
//  The code is modified from the code in the TC++ 1.0 runtime lib.
//
//  Odinn Sorensen, 910706.
//  ------------------------------------------------------------------


char* __gsweekday[7] = {
  "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
};

char* __glweekday[7] = {
  "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"
};

char* __gsmonth[12] = {
  "Jan", "Feb", "Mar", "Apr", "May", "Jun",
  "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

char* __glmonth[12] = {
  "January", "February", "March",     "April",   "May",      "June",
  "July",    "August",   "September", "October", "November", "December"
};

char* gampm[2] = {
  "AM", "PM"
};

char* gsweekday[7] = {
  NULL, NULL, NULL, NULL, NULL, NULL, NULL
};

char* glweekday[7] = {
  NULL, NULL, NULL, NULL, NULL, NULL, NULL
};

char* gsmonth[12] = {
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
};

char* glmonth[12] = {
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
};


//  ------------------------------------------------------------------

uint strftimei(char* s, uint maxsize, const char* fmt, const Tm* t) {

  char buf[25];

  char* p = buf;
  int i;
  uint len;

  for(len = 1; (len < maxsize) and *fmt; ++fmt, p = buf) {
    if(*fmt != '%') {
      len++;
      *s++ = *fmt;
    }
    else {
      strcpy(buf, "00");

      switch(*++fmt) {
        case 'a':
          p = gsweekday[t->tm_wday];
          if(p == NULL)
            p = __gsweekday[t->tm_wday];
          break;
        case 'A':
          p = glweekday[t->tm_wday];
          if(p == NULL)
            p = __glweekday[t->tm_wday];
          break;
        case 'b':
          p = gsmonth[t->tm_mon];
          if(p == NULL)
            p = __gsmonth[t->tm_mon];
          break;
        case 'B':
          p = glmonth[t->tm_mon];
          if(p == NULL)
            p = __glmonth[t->tm_mon];
          break;
        case 'd':
          itoa(t->tm_mday, buf + (t->tm_mday < 10), 10);
          break;
        case 'H':
          itoa(t->tm_hour, buf + (t->tm_hour < 10), 10);
          break;
        case 'I':
          if(t->tm_hour == 12)
            i = 12;
          else
            i = t->tm_hour % 12;
          itoa(i, buf + (i < 10), 10);
          break;
        case 'j':
          i = t->tm_yday + 1;
          itoa(i, buf + (i < 10) + (i < 100), 10);
          break;
        case 'm':
          i = t->tm_mon + 1;
          itoa(i, buf + (i < 10), 10);
          break;
        case 'M':
          itoa(t->tm_min, buf + (t->tm_min < 10), 10);
          break;
        case 'p':
          p = gampm[t->tm_hour / 12];
          break;
        case 'S':
          itoa(t->tm_sec, buf + (t->tm_sec < 10), 10);
          break;
        case 'U':
          i = t->tm_wday - (t->tm_yday % 7);
          if(i < 0)
            i += 7;
          i = (t->tm_yday + i) / 7;
          itoa(i, buf + (i < 10), 10);
          break;
        case 'w':
          itoa(t->tm_wday, buf, 10);
          break;
        case 'W':
          i = t->tm_wday - (t->tm_yday % 7);
          if(i < 0)
            i += 7;
          i = (t->tm_yday + i - 1) / 7;
          i++;                                    // OHS change: Make weekno in range 01..53
          itoa(i, buf + (i < 10), 10);
          break;
        case 'y':
          i = t->tm_year % 100;
          itoa(i, buf + (i < 10), 10);
          break;
        case 'Y':
          itoa(1900 + t->tm_year, buf, 10);
          break;
        case '%':
          p = "%";
      }

      i = minimum_of_two(strlen(p), (size_t)maxsize - len);
      strncpy(s, p, i);
      len += i;
      s += i;
    }
  }

  *s = '\0';

  if(*fmt)
    return 0;
  else
    return len - 1;
}


//  ------------------------------------------------------------------
//  Convert string-month to integer

int str2mon(const char* ptr) {

  int mon;

  switch(toupper(*ptr)) {
    case 'A':
      if(toupper(ptr[1]) == 'P')
        mon = 4;
      else
        mon = 8;
      break;
    case 'D':
      mon = 12;
      break;
    case 'F':
      mon = 2;
      break;
    case 'J':
      if(toupper(ptr[1]) == 'A')
        mon = 1;
      else if(toupper(ptr[2]) == 'L')
        mon = 7;
      else
        mon = 6;
      break;
    case 'M':
      if(toupper(ptr[2]) == 'R')
        mon = 3;
      else
        mon = 5;
      break;
    case 'N':
      mon = 11;
      break;
    case 'O':
      mon = 10;
      break;
    case 'S':
      mon = 9;
      break;
    default:
      mon = 0;
  }

  return mon;
}


//  ------------------------------------------------------------------

ulong FTimeToTime(FTime* __ftime, Tm* __tm) {

  Tm _tm;
  ulong _time = 0;

  if(__tm == NULL)
    __tm = &_tm;

  // Only try to convert a valid date
  if(__ftime->ft_year >= 5) {   // FidoNet standards didn't exist before 1985
    if((__ftime->ft_day >= 1) and (__ftime->ft_day <= 31)) {
      if((__ftime->ft_month >= 1) and (__ftime->ft_month <= 12)) {

        __tm->tm_year  = __ftime->ft_year + 80;
        __tm->tm_mon   = __ftime->ft_month - 1;
        __tm->tm_mday  = __ftime->ft_day;
        __tm->tm_hour  = __ftime->ft_hour;
        __tm->tm_min   = __ftime->ft_min;
        __tm->tm_sec   = __ftime->ft_tsec * 2;
        __tm->tm_isdst = 0;

        _time = gmktime(__tm);

        if(_time == (ulong)0xFFFFFFFFL)
          _time = 0;
      }
    }
  }

  return _time;
}


//  ------------------------------------------------------------------

FTime TimeToFTime(ulong __time) {

  FTime _ft;
  memset(&_ft, 0, sizeof(FTime));

  if(__time) {

    Tm* _tmp = glocaltime(&__time);
    _ft.ft_year  = (word)(_tmp->tm_year - 80);
    _ft.ft_month = (word)(_tmp->tm_mon + 1);
    _ft.ft_day   = (word)(_tmp->tm_mday);
    _ft.ft_hour  = (word)(_tmp->tm_hour);
    _ft.ft_min   = (word)(_tmp->tm_min);
    _ft.ft_tsec  = (word)(_tmp->tm_sec / 2);
  }

  return _ft;
}


//  ------------------------------------------------------------------
//  Clock ticks in milliseconds since process start

Clock gclock() {

  GTime t;
  gtimems(&t);
  ulong thistime = gtime2sec(&t);
  return ((thistime-gtim.starttime)*1000L) + t.msecond;
}


//  ------------------------------------------------------------------

ulong FidoTimeToUnix(char* ptr) {

  bool date_ok = false;
  int year=0, month=0, day=0;
  int hour=0, minute=0, second=0;

  ptr = strskip_wht(ptr);
  if(not isdigit(*ptr)) {
    // Skip past weekday string (SEA format)
    ptr = strskip_wht(strskip_txt(ptr));
  }
  if(*ptr) {
    if(isdigit(*ptr)) {
      day = atoi(ptr);
      ptr = strskip_wht(strskip_txt(ptr));
      if(isalpha(*ptr)) {
        month = str2mon(ptr);
        if(month) {
          ptr = strskip_wht(strskip_txt(ptr));
          if(isdigit(*ptr)) {
            year = atoi(ptr);
            ptr = strskip_wht(strskip_txt(ptr));
            if(isdigit(*ptr)) {
              hour = atoi(ptr);
              ptr = strskip_digits(ptr);
              if(*ptr and isdigit(ptr[1])) {
                minute = atoi(++ptr);
                date_ok = true;
                // The seconds part is only in the FTS-1 format
                ptr = strskip_digits(ptr);
                if(*ptr and isdigit(ptr[1])) {
                  second = atoi(++ptr);
                }
              }
            }
          }
        }
      }
    }
  }

  // Convert datetime to UNIX timestamp
  if(date_ok) {
    Tm t;
    t.tm_year  = (year < 80) ? (year+100) : year;
    t.tm_mon   = month - 1;
    t.tm_mday  = day;
    t.tm_hour  = hour;
    t.tm_min   = minute;
    t.tm_sec   = second;
    t.tm_isdst = 0;
    return gmktime(&t);
  }
  return (ulong)-1;
}


//  ------------------------------------------------------------------

char* TimeToStr(char* buf, ulong t) {

  strftimei(buf, 20, "%Y-%m-%d %H:%M:%S", glocaltime(&t));
  return buf;
}


//  ------------------------------------------------------------------

char* FTimeToStr(char* buf, FTime& t) {

  sprintf(buf, "%04u-%02u-%02u %02u:%02u:%02u",
    t.ft_year+1980, t.ft_month, t.ft_day,
    t.ft_hour, t.ft_min, t.ft_tsec*2
  );
  return buf;
}


//  ------------------------------------------------------------------

const char* gfiletime::c_str(char* buf) {

  sprintf(buf, "%04u-%02u-%02u %02u:%02u:%02u",
    ft_year+1980, ft_month, ft_day,
    ft_hour, ft_min, ft_tsec*2
  );
  return buf;
}


//  ------------------------------------------------------------------

const char* gopustime::c_str(char* buf) {

  sprintf(buf, "%04u-%02u-%02u %02u:%02u:%02u",
    ft_year+1980, ft_month, ft_day,
    ft_hour, ft_min, ft_tsec*2
  );
  return buf;
}


//  ------------------------------------------------------------------

int day_of_week(int year, int month, int day) {

  #define ADJ 6

  if(month < 3) {
    month += 13;
    year--;
  }
  else {
    month++;
  }

  return (day + 26 * month / 10 + year + year / 4 - year / 100 + year / 400 + 6) % 7;
}


//  ------------------------------------------------------------------

