
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gtxtline.h,v 1.1.1.1 1998/11/21 19:38:09 odinn Exp $
//  ------------------------------------------------------------------
//  Text line class.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#ifndef __GTXTLINE_H
#define __GTXTLINE_H

#include <gthing.h>


//  ------------------------------------------------------------------

class text_line_data {

public:

  uint id;        // identifier
  uint status;    // status
  byte offset;    // offset into text (for field body etc.)
  char text[1];   // text
};


//  ------------------------------------------------------------------

class text_line_mgr : public thing_base {

public:

  text_line_mgr(uint b, uint i);
  ~text_line_mgr();

  text_line_data& operator[](uint h) const { return *(text_line_data*)thing_base::operator[](h); }

  uint length(int h) const { return thing_base::length(h)-sizeof(text_line_data); }
};


//  ------------------------------------------------------------------

class text_line {

protected:

  text_line_mgr& mgr;
  int line_number;

  void insert_at(int l, uint i, uint s, const char* t, uint o=0);
  void insert_at_with_len(int l, uint i, uint s, const char* t, uint z, uint o=0);

public:

  text_line(text_line_mgr& m);
  ~text_line();

  int lineno() const            { return line_number; }
  int lines() const             { return mgr.size(); }

  void goto_line(int l)         { line_number = l; }
  void goto_next_line()         { line_number++; }
  void goto_prev_line()         { line_number--; }

  int first();
  int next();
  int previous();
  int last();

  int is_last() const           { return line_number == lines(); }

  void remove();

  void add(uint i, uint s, const char* t, uint o=0);
  void add_with_len(uint i, uint s, const char* t, uint z, uint o=0);

  void insert(uint i, uint s, const char* t, uint o=0);
  void insert_below(uint i, uint s, const char* t, uint o=0);

  uint id() const               { return mgr[lineno()-1].id; }
  uint status() const           { return mgr[lineno()-1].status; }
  uint length() const           { return mgr.length(lineno()-1); }
  uint offset() const           { return mgr[lineno()-1].offset; }
  const char* text() const      { return mgr[lineno()-1].text; }
  const char* text_at_offset() const { return text()+offset(); }

  void set_id(uint i)           { mgr[lineno()-1].id = i; }
  void set_status(uint s)       { mgr[lineno()-1].status = s; }
  void set_offset(uint o)       { mgr[lineno()-1].offset = (byte)o; }
  const char* set_text(const char* t);
};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

