
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gtxtpara.h,v 1.1.1.1 1998/11/21 19:38:09 odinn Exp $
//  ------------------------------------------------------------------
//  Text -> Paragraph conversion.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GTXTPARA_H
#define __GTXTPARA_H


//  ------------------------------------------------------------------
//  Required

#include <gcrcall.h>
#include <gmemall.h>

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


//  ------------------------------------------------------------------
//  CHR4 values for standard non-kludge control lines

const dword CHR4_AREA          = MK_CHR4('A','R','E','A');
const dword CHR4_TEARLINESPACE = MK_CHR4('-','-','-',' ');
const dword CHR4_TEARLINENUL   = MK_CHR4('-','-','-','\0');
const dword CHR4_ORIGIN        = MK_CHR4(' ','*',' ','O');
const dword CHR4_SEENBY        = MK_CHR4('S','E','E','N');


//  ------------------------------------------------------------------
//  CHR4 values for standard known kludges

const dword CHR4_INTL    = MK_CHR4('\001','I','N','T');
const dword CHR4_FMPT    = MK_CHR4('\001','F','M','P');
const dword CHR4_TOPT    = MK_CHR4('\001','T','O','P');
const dword CHR4_MSGID   = MK_CHR4('\001','M','S','G');
const dword CHR4_REPLY   = MK_CHR4('\001','R','E','P');
const dword CHR4_PID     = MK_CHR4('\001','P','I','D');
const dword CHR4_SEENBY1 = MK_CHR4('\001','S','E','E');
const dword CHR4_PATH    = MK_CHR4('\001','P','A','T');
const dword CHR4_CHARSET = MK_CHR4('\001','C','H','A');
const dword CHR4_CHRS    = MK_CHR4('\001','C','H','R');
const dword CHR4_VIA     = MK_CHR4('\001','V','I','A');
const dword CHR4_FLAGS   = MK_CHR4('\001','F','L','A');


//  ------------------------------------------------------------------
//  Control line enumeration

enum {

  // Non-ctrl line
  NOT_CTRL = 0,

  // Standard non-kludge control lines
  CTRL_AREA,
  CTRL_TEARLINE,
  CTRL_ORIGIN,

  // Standard known kludges
  CTRL_KLUDGE,
  CTRL_INTL,
  CTRL_FMPT,
  CTRL_TOPT,
  CTRL_MSGID,
  CTRL_REPLY,
  CTRL_PID,
  CTRL_SEENBY,
  CTRL_SEENBY1,
  CTRL_PATH,
  CTRL_CHARSET,
  CTRL_CHRS,
  CTRL_VIA,
  CTRL_FLAGS,

  // Unknown kludges
  CTRL_UNKNOWNKLUDGE
};


//  ------------------------------------------------------------------

struct GParaData {

  char*  text;
  uint  length;
  uint control;
};


//  ------------------------------------------------------------------

class GParagraph {

public:

  int lines;
  char softcr;
  GParaData* paraidx;

  GParagraph();
  ~GParagraph();

  void AddParagraph(char* __text, uint __length);
  int ConvertText(char* __text, uint __length);
  uint CheckCtrlLines();
};


//  ------------------------------------------------------------------

inline GParagraph::GParagraph() {

  lines = 0;
  softcr = LF;
  paraidx = NULL;
}


//  ------------------------------------------------------------------

inline GParagraph::~GParagraph() {

  free(paraidx);
}


//  ------------------------------------------------------------------

inline void GParagraph::AddParagraph(char* __text, uint __length) {

  paraidx = (GParaData*)reallox(paraidx, lines+1, sizeof(GParaData));
  GParaData* _para = paraidx + lines++;
  _para->text = __text;
  _para->length = __length;
  _para->control = 0;
}


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack()
#endif

#endif

//  ------------------------------------------------------------------

