
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gutlclip.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  Misc. clipboards support.
//  ------------------------------------------------------------------


#include <gstrall.h>
#include <gutlmisc.h>
#include <gutlclip.h>


//  ------------------------------------------------------------------

gclipbrd::gclipbrd() {

  len = -1;
  clipdata = NULL;
  #ifdef __WIN32__
  hglb = NULL;
  #endif
  #if 0
  //#ifdef __OS2__
  whnd = NULL;
  #endif
}


//  ------------------------------------------------------------------

bool gclipbrd::openread() {

  #if defined(__WIN32__)
  if(IsClipboardFormatAvailable(CF_TEXT) and OpenClipboard(NULL)) {

    hglb = GetClipboardData(CF_OEMTEXT);

    if(hglb) {
      clipdata = (char*) GlobalLock(hglb);
      len = strlen(clipdata);
      return true;
    }
  }
  #endif
  #if 0
  //#if defined(__OS2__)
    SWCNTRL swctl;

    WinQuerySwitchEntry(WinQuerySwitchHandle(0, getpid()), &swctl);
    whnd = swctl.hwnd;

    whnd = WinInitialize(0);

    if(whnd and WinOpenClipbrd(whnd)) {

      if((clipdata = PCHAR(WinQueryClipbrdData(whnd, CF_TEXT)))!=NULL) {
        printf("clipdata=%s\n", clipdata);

        //printf("len=%d\n", len = strlen(clipdata));
        //len = 10;
        //return true;
      }
      else
        WinCloseClipbrd(whnd);
    }
    WinTerminate(whnd);
  #endif

  return false;
}


//  ------------------------------------------------------------------

bool gclipbrd::writeclipbrd(char* buf) {

  #if defined(__WIN32__)
  LPTSTR  lptstr;
  HGLOBAL hglobal;
  bool result = false;
  int len;

  if(OpenClipboard(NULL)) {
    if((len = strlen(buf))>0) {
      if((hglobal = GlobalAlloc(GMEM_DDESHARE, (len+1)))!=NULL) {

        lptstr = (char*) GlobalLock(hglobal);
        memcpy(lptstr, buf, len);

        GlobalUnlock(hglobal);
        EmptyClipboard();
        SetClipboardData(CF_OEMTEXT, hglobal);
        result = true;
      }
    }
    CloseClipboard();
  }
  return result;
  #elif defined(__OS2__)
  SWCNTRL swctl;
  bool result = false;

  WinQuerySwitchEntry(WinQuerySwitchHandle(0, getpid()), &swctl);
  whnd = swctl.hwnd;

  if(WinOpenClipbrd(whnd)) {
    ulong len;
    void* pShareBuf;

    WinEmptyClipbrd(whnd);

    if((len=strlen(buf))>0) {
      if((DosAllocSharedMem(&pShareBuf, NULL, len+1, PAG_COMMIT|OBJ_GIVEABLE|PAG_READ|PAG_WRITE))==NO_ERROR) {
        memcpy(pShareBuf, buf, len+1);

        if(WinSetClipbrdData(whnd, (ULONG) pShareBuf, CF_TEXT, CFI_POINTER))
          result = true;
      }
    }

    WinCloseClipbrd(whnd);
  }
  return result;
  #else
    NW(buf);
    return false;
  #endif
}


//  ------------------------------------------------------------------

char* gclipbrd::read(char* buffer, int maxlen) {

  if(len>0) {
    int i = MinV(len, maxlen);
    char* p = strpbrk(clipdata, "\r\n");
    if(p and (p-clipdata < i)) {
      i = p - clipdata;
      if(len > i and strchr("\r\n", *(p+1)) and (*p != *(p+1)))
        ++i;
    }
    strtrim(strxcpy(buffer, clipdata, ++i));
    strcat(buffer, "\n");
    len -= i;
    clipdata += i;
    return buffer;
  }

  return NULL;
}


//  ------------------------------------------------------------------

void gclipbrd::close() {

  #if defined(__WIN32__)
  if(len>=0) {
    if(hglb)
      GlobalUnlock(hglb);
    CloseClipboard();
    len = -1;
    clipdata = NULL;
  }
  #endif
  #if 0
  //#if defined(__OS2__)
  if(len>=0) {
    if(whnd) {
      WinCloseClipbrd(whnd);
      len = -1;
      clipdata = NULL;
    }
  }
  #endif
}


//  ------------------------------------------------------------------

