
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gutlgrp.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  Group/Random System.
//  ------------------------------------------------------------------


#include <stdlib.h>
//#include <gcfgbase.h>
#include <gmemdbg.h>
#include <gfile.h>
#include <gstrall.h>
#include <gstrgstr.h>
#include <gwildmat.h>
#include <gutlgrp.h>
#include <glog.h>


//  ------------------------------------------------------------------

#define TLOG if(throw_log) throw_log


//  ------------------------------------------------------------------
//  Compare group items by group, type and offset

int _stdc GrpItmCmp(const GrpItm* a, const GrpItm* b) {

  int cmp;

  cmp = compare_two(a->grpno,  b->grpno);    if(cmp) return cmp;
  cmp = compare_two(a->type,   b->type);     if(cmp) return cmp;
  cmp = compare_two(a->offset, b->offset);

  return cmp;
}


//  ------------------------------------------------------------------
//  Constructor

Grp::Grp(int __types, int __idsize) {

  types = __types;
  idsz = __idsize;

  id  = NULL;  ids  = 0;
  mbr = NULL;  mbrs = 0;
  grp = NULL;  grps = 0;
  itx = NULL;  grps = 0;
  itm = NULL;  itms = 0;
  ifp = NULL;  ifps = 0;
}


//  ------------------------------------------------------------------
//  Destructor

Grp::~Grp() {

  throw_free(id);
  throw_free(mbr);
  throw_free(grp);
  throw_free(itx);
  throw_free(itm);
}


//  ------------------------------------------------------------------
//  Set item data file

void Grp::SetFp(gfile& __fp) {

  ifp = &__fp;
}


//  ------------------------------------------------------------------
//  Add an ID string. Return string number

int Grp::addid(const char* __id) {

  // Allocate memory for the new ID
  id = (char*)throw_realloc(id, (ids+1)*idsz);

  // Zero out string field. Just cosmetics.
  memset(id+(ids*idsz), 0, idsz);

  // Copy the ID string
  strxcpy(id+(ids*idsz), __id, idsz);

  // Count it and return the string number
  return ids++;
}


//  ------------------------------------------------------------------
//  Add a new group member

void Grp::addmbr(int __id) {
  
  // Allocate memory for the new member
  mbr = (GrpMbr*)throw_realloc(mbr, (mbrs+1)*sizeof(GrpMbr));

  // Set member ID and group it belongs to
  mbr[mbrs].id = __id;
  mbr[mbrs].grpno = grps-1;

  // Increment member count
  mbrs++;
}


//  ------------------------------------------------------------------
//  Add a new group member

void Grp::AddMbr(const char* __id) {
  
  // Get member ID string number and add the member
  addmbr(addid(__id));
}


//  ------------------------------------------------------------------
//  Add a new group

void Grp::AddGrp(const char* __id) {

  // Allocate memory for the new group
  grp = (GrpLst*)throw_realloc(grp, (grps+1)*sizeof(GrpLst));

  // Allocate memory for the associated item type index
  itx = (GrpItx*)throw_realloc(itx, (grps+1)*types*sizeof(GrpItx));

  // Set group ID
  int _id = addid(__id);
  grp[grps].id = _id;

  // Reset Item Type Index
  itxp = itx + (grps*types);
  for(curtyp=0; curtyp<types; curtyp++) {
    itxp->cnt = 0;
    itxp->idx = -1;
    itxp++;
  }

  // Increment group count
  grps++;

  // Add the group itself as a member
  addmbr(_id);
}


//  ------------------------------------------------------------------

void Grp::AddItm(int __type, const gstring& __data) {

  int __size = __data.storage_size();

  // Allocate memory for the new item
  itm = (GrpItm*)throw_realloc(itm, (itms+1)*sizeof(GrpItm));
  itmp = itm + itms;

  // Set item information
  itmp->grpno  = grps-1;
  itmp->type   = __type;
  itmp->offset = ifps;
  itmp->size   = __size;

  // Write item to the file
  *ifp << __data;
  ifps += __size;

  // Update count in group item type index
  itx[((grps-1)*types)+__type].cnt++;

  // Increment item count
  itms++;
}


//  ------------------------------------------------------------------

void Grp::AddItm(int __type, void* __data, int __size) {

  // Allocate memory for the new item
  itm = (GrpItm*)throw_realloc(itm, (itms+1)*sizeof(GrpItm));
  itmp = itm + itms;

  // Set item information
  itmp->grpno  = grps-1;
  itmp->type   = __type;
  itmp->offset = ifps;
  itmp->size   = __size;

  // Write item to the file
  ifp->fwrite(__data, __size);
  ifps += __size;

  // Update count in group item type index
  itx[((grps-1)*types)+__type].cnt++;

  // Increment item count
  itms++;
}


//  ------------------------------------------------------------------

void Grp::Sort() {

  // Sort items by group number, type and offset in file
  qsort(itm, itms, sizeof(GrpItm), (StdCmpCP)GrpItmCmp);

  // Setup loop variables
  itmp = itm;
  itxp = itx;
  curitm = 0;
  curgrp = -1;
  curtyp = -1;

  // Find individual groups for processing
  while(curitm < itms) {
    
    // When next group is found
    if(itmp->grpno != curgrp) {

      // Set current group
      curgrp = itmp->grpno;

      // Set Item Type Index pointer
      itxp = itx + (curgrp*types);

      // Reset current type
      curtyp = -1;
    }

    // When next type is found
    if(itmp->type != curtyp) {

      // Set current type
      curtyp = itmp->type;

      // Record index of first item of this type
      itxp[curtyp].idx = curitm;
    }

    // Increment current item and item pointer
    curitm++;
    itmp++;
  }
}


//  ------------------------------------------------------------------
//  Build group of groups. Return number of matching groups.

#if 0
int Grp::BldGrp(const char* __id) {

  // Reset counts
  igrps = 0;
  memset(igrp, 0, types*sizeof(int));

  // Group dupe check
  byte* chk = (byte*)throw_calloc(1, grps);

  // Search for matching members
  mbrp = mbr;
  for(curmbr=0; curmbr < mbrs; curmbr++) {

    // When a matching member is found
    if(strwild(__id, id+(mbrp->id*idsz)) {

      // If we have not yet used the group
      curgrp = mbrp->grpno;
      if(not chk[curgrp]) {

        // We have now
        chk[curgrp]++;
        igrps++;

        // Add item type counts to super-group
        int* igrpp = igrp;
        itxp = itx + (curgrp*types);
        for(curtyp=0; curtyp<types; curtyp++)
          *igrpp++ += (itxp++)->cnt;
      }
    }
    mbrp++;
  }
}
#endif


//  ------------------------------------------------------------------

const char* Grp::SetGrp(const char* __id) {

  // Search for the member that match the ID
  mbrp = mbr;
  for(curmbr=0; curmbr < mbrs; curmbr++) {
    register char* idp = id+(mbrp->id*idsz);
    if(strwild(__id, idp)) {
      curgrp = mbrp->grpno;
      return idp;
    }
    mbrp++;
  }

  // Not found
  curgrp = -1;
  return NULL;
}


//  ------------------------------------------------------------------

int Grp::GetItm(int __type, gstring& __data, int __no) {

  // Return error if a current group is not set
  if(curgrp == -1)
    return -1;

  // Pointer to item type index for this group and type
  itxp = itx + (curgrp*types) + __type;
  if(itxp->cnt) {

    // Get random item if requested
    curitm = (__no == -1) ? (rand() % itxp->cnt) : __no;

    // Pointer to item info
    itmp = itm + itxp->idx + curitm;

    // Get string
    ifp->fseekset(itmp->offset);
    *ifp >> __data;
  }

  // Return number of items of this type in the group
  return itxp->cnt;
}


//  ------------------------------------------------------------------

int Grp::GetItm(int __type, void* __data, int __size, int __no) {

  // Return error if a current group is not set
  if(curgrp == -1)
    return -1;

  // Pointer to item type index for this group and type
  itxp = itx + (curgrp*types) + __type;
  if(itxp->cnt) {

    // Get random item if requested
    curitm = (__no == -1) ? (rand() % itxp->cnt) : __no;

    // Pointer to item info
    itmp = itm + itxp->idx + curitm;

    // Check size
    if(__size != -1) {
      if(itmp->size > __size) {
        #if defined(THROW_LOG)
        TLOG->printf("! Warning: %u bytes overflow in group %s, item type %u", itmp->size-__size, id+(mbrp->id*idsz), __type);
	#endif
      }
      __size = -1;
    }

    // Adjust size
    __size = (__size == -1) ? itmp->size : minimum_of_two(itmp->size, __size);

    // Read item from the file
    if(__data and __size) {
      ifp->fseekset(itmp->offset);
      ifp->fread(__data, __size);
    }
  }

  // Return number of items of this type in the group
  return itxp->cnt;
}


//  ------------------------------------------------------------------

void Grp::Save(gfile& __fp) {

  // Write counts
  __fp.fwrite(&ids,  sizeof(int));
  __fp.fwrite(&mbrs, sizeof(int));
  __fp.fwrite(&grps, sizeof(int));
  __fp.fwrite(&itms, sizeof(int));

  // Write data
  int itxs = grps*types;
  __fp.fwrite(id,  ids*idsz);
  __fp.fwrite(mbr, mbrs*sizeof(GrpMbr));
  __fp.fwrite(grp, grps*sizeof(GrpLst));
  __fp.fwrite(itx, itxs*sizeof(GrpItx));
  __fp.fwrite(itm, itms*sizeof(GrpItm));
}


//  ------------------------------------------------------------------

void Grp::Load(gfile& __fp) {

  // Read counts
  __fp.fread(&ids,  sizeof(int));
  __fp.fread(&mbrs, sizeof(int));
  __fp.fread(&grps, sizeof(int));
  __fp.fread(&itms, sizeof(int));

  // Allocate memory
  int itxs = grps*types;
  if(ids)  id  = (char*)   throw_calloc(ids, idsz);
  if(mbrs) mbr = (GrpMbr*) throw_calloc(mbrs, sizeof(GrpMbr));
  if(grps) grp = (GrpLst*) throw_calloc(grps, sizeof(GrpLst));
  if(grps) itx = (GrpItx*) throw_calloc(itxs, sizeof(GrpItx));
  if(itms) itm = (GrpItm*) throw_calloc(itms, sizeof(GrpItm));

  // Read data
  if(ids)  __fp.fread(id,  ids*idsz);
  if(mbrs) __fp.fread(mbr, mbrs*sizeof(GrpMbr));
  if(grps) __fp.fread(grp, grps*sizeof(GrpLst));
  if(grps) __fp.fread(itx, itxs*sizeof(GrpItx));
  if(itms) __fp.fread(itm, itms*sizeof(GrpItm));
}


//  ------------------------------------------------------------------

