
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gutlgrp.h,v 1.1.1.1 1998/11/21 19:38:09 odinn Exp $
//  ------------------------------------------------------------------
//  Group/Random System.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GUTLGRP2_H
#define __GUTLGRP2_H


//  ------------------------------------------------------------------
//  Required

class gfile;


//  ------------------------------------------------------------------
//  Group Member                    64k limit: 65535/4 = 16383 members

struct GrpMbr {

  int  id;         // Member ID string number
  int  grpno;      // Group number
};


//  ------------------------------------------------------------------
//  Group List                                   64k limit: see GrpItx

struct GrpLst {

  int  id;         // Group ID string number
};


//  ------------------------------------------------------------------
//  Group Item Type Index            64k limit: 65535/4 = 16383 groups

struct GrpItx {

  int cnt;         // Number of items of this type
  int idx;         // Index to first item of this type
};


//  ------------------------------------------------------------------
//  Group Item                        64k limit: 65535/10 = 6553 items

struct GrpItm {

  int  grpno;      // Group number
  int  type;       // Item type
  long  offset;     // Offset in data file
  int  size;       // Size of item
};


//  ------------------------------------------------------------------
//  Group class

class Grp {

public:

  //  ----------------------------------------------------------------
  //  Data

  int    types;    // Number of different item types
  int    idsz;     // ID size

  char*    id;       // List of ID strings
  int    ids;      // Number of ID strings

  GrpMbr* mbr;      // List of group members
  GrpMbr* mbrp;     // Pointer to group member
  int    mbrs;     // Number of group members
  int    curmbr;   // Current member number

  GrpLst* grp;      // List of groups
  int    grps;     // Number of groups
  int    curgrp;   // Current group number
  
  GrpItx* itx;      // List of Item Type Indexes (parallell with grp)
  GrpItx* itxp;     // Pointer to Item Type Index

  GrpItm* itm;      // List of group items
  GrpItm* itmp;     // Pointer to group item
  int    itms;     // Number of group items
  int    curitm;   // Current item number
  int    curtyp;   // Current item type

  gfile*  ifp;      // Item data file
  long    ifps;     // Item data file offset

  #if 0
  int*    igrp;     // Super-group item type counts
  int*    igrps;    // Groups in super-group
  #endif


  //  ----------------------------------------------------------------
  //  Constructor and destructor

  Grp(int __types, int __idsize);
  ~Grp();


  //  ----------------------------------------------------------------
  //  Internal functions

  int addid(const char* __id);
  void addmbr(int __id);


  //  ----------------------------------------------------------------
  //  User functions

  void SetFp(gfile& __fp);

  void AddGrp(const char* __id);
  void AddMbr(const char* __id);

  void AddItm(int __type, void* __data, int __size);
  void AddItm(int __type, int __data)   { AddItm(__type, &__data, sizeof(__data)); }
//  void AddItm(int __type, bool __data)  { AddItm(__type, &__data, sizeof(__data)); }
  void AddItm(int __type, char __data)  { AddItm(__type, &__data, sizeof(__data)); }
  void AddItm(int __type, const gstring& __data);

  void Sort();

  const char* SetGrp(const char* __id);
  int GetItm(int __type, void* __data, int __size=-1, int __no=-1);
  int GetItm(int __type, int __data, int __no=-1)       { return GetItm(__type, &__data, sizeof(__data), __no); }
//  int GetItm(int __type, bool __data, int __no=-1)      { return GetItm(__type, &__data, sizeof(__data), __no); }
  int GetItm(int __type, char __data, int __no=-1)      { return GetItm(__type, &__data, sizeof(__data), __no); }
  int GetItm(int __type, gstring& __data, int __no=-1);

  void Save(gfile& __fp);
  void Load(gfile& __fp);

  #if 0
  int BldGrp(char* __id);
  #endif

};


//  ------------------------------------------------------------------
//  Prototypes

int _stdc GrpItmCmp(const GrpItm* a, const GrpItm* b);


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

