
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gutlmisc.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  Utility functions.
//  ------------------------------------------------------------------


#include <stdlib.h>
#include <gstrall.h>
#include <gutlmisc.h>
#include <gmemall.h>

#if defined(__DJGPP__)
#include <unistd.h>
#endif

#if defined(__OS2__)
#define INCL_BASE
#include <os2.h>
#endif


//  ------------------------------------------------------------------
//  Get yes/no value

int GetYesno(const char* value) {

  if(isdigit(*value))
    return atoi(value);

  switch(toupper(*value)) {

    case NUL:   // Blank
    case 'T':   // True
    case 'Y':   // Yes
      return YES;

    case 'F':   // False
    case 'N':   // No
      return NO;

    case 'O':   // On or Off
      if(toupper(value[1]) == 'N')
        return YES;
      else
        return NO;

    case 'A':   // Always, Ask or Auto
      if(toupper(value[1]) == 'L')
        return ALWAYS;
      else if(toupper(value[1]) == 'S')
        return ASK;
      else
        return AUTO;

    case 'M':   // Maybe
      return MAYBE;
  }

  return NO;
}


//  ------------------------------------------------------------------
//  Calculates a percentage

int Pct(ulong x, ulong y) {

  if(x) {

    ulong p = (((x-y)*100)/x);
    ulong r = (((x-y)*100)%x);

    if(((r*10)/x)>4)
      p++;

    return (int)p;
  }

  return 100;
}


//  ------------------------------------------------------------------
//  Calculates next tab stop from given column

int tabstop(int column, int tabwidth) {

  int sum = column + tabwidth;
  return sum - (sum % tabwidth);
}


//  ------------------------------------------------------------------
//  Convert hex string to integer

ulong atoulx(const char* s) {

  ulong retval = 0;

  s = strskip_wht(s);

  while(isxdigit(*s)) {
    retval <<= 4;
    retval |= xtoi(*s);
    s++;
  }

  return retval;
}


//  ------------------------------------------------------------------
//  Convert hex string to integer

int atox(const char* s) {

  return (int)atoulx(s);
}


//  ------------------------------------------------------------------

char* ltob(char* dst, ulong value, int fill) {

  register char* p = dst;

  for(int b=1,g=0; b<33; b++) {
    if(value & 0x80000000L) {
      g = 1;
      *p++ = '1';
    }
    else {
      if(g or (b >= fill))
        *p++ = '0';
    }
    value <<= 1;
  }
  *p = NUL;

  return dst;
}


// ------------------------------------------------------------------

dword B2L(dword b) {

  register byte* bp = (byte*)&b;
  return ((*bp + ((dword)bp[1] << 8) + ((dword)bp[2] << 16)) | 0x800000L) >> (24 - (bp[3] - 0x80));
}


// ------------------------------------------------------------------

dword L2B(dword l) {

  // Special case for zero
  if(l == 0)
    return 0;

  // Look for the largest power of 2
  register int s = 31;
  register dword p = 0x80000000UL;
  while(s > 0) {
    if(p & l)
      break;
    p >>= 1;
    s--;
  }

  // Fillup the mantisse with useful information
  dword b = 0;
  int n = 0;
  int e = s--;
  l -= 1UL << e;
  while(l > 0) {
    register dword t = (1UL << s);
    if(l >= t) {
      b += 1UL << (22 - n);
      if(l < t)
        break;
      l -= t;
    }
    n++;
    s--;
  }

  return b | ((0x81UL+e) << 24);
}


//  ------------------------------------------------------------------

#if defined(__OS2__)

void beep() {

  DosBeep(440, 111);
}

#else

void beep() {

  _stdoutch(BEL);
}

#endif


//  ------------------------------------------------------------------

long gatol(const char* a) {

  return atol(a);
}

//  ------------------------------------------------------------------

const char* ulong_to_cstr(ulong n) {

  static char number[33];
  ultoa(n, number, 10);
  return number;
}


//  ------------------------------------------------------------------

#if defined(__GNUC__)

static char* longtoa(long value, char* strP, int radix, bool maybeSigned, char hexStyle) {

  char buf [34];
  char c, *p, *bufp;

  p = strP;

  /* If the requested radix is invalid, generate an empty result.
   */
  if (radix >= 2 && radix <= 36)
  {

      /* If the value is signed and less than zero, generate a minus sign.
       */
      if (value < 0 && maybeSigned != 0)
      {
          *p++ = '-';
          value = -value;
      }

      /* Now loop, taking each digit as modulo radix, and reducing the value
       * by dividing by radix, until the value is zeroed.  Note that
       * at least one loop occurs even if the value begins as 0,
       * since we want "0" to be generated rather than "".
       */
      bufp = buf;
      for (;;)
      {
          *bufp++ = (char)((unsigned long)value % radix);
          if ((value = (unsigned long)value / radix) == 0)
              break;
      }

      /* The digits in the buffer must now be copied in reverse order into
       * the target string, translating to ASCII as they are moved.
       */
      while (bufp != buf)
      {
          if ((c = *--bufp) < 10)
              *p++ = (char)(c + '0');
          else
              *p++ = (char)((c - 10) + hexStyle);
      }
  }

  /* terminate the output string with a zero.
   */
  *p = '\0';
  return (strP);          /* return a pointer to the string */
}


//  ------------------------------------------------------------------

#if 0
char* _itoa(int value, char* strP, int radix) {

  return longtoa(value, strP, radix, true, 'A');
}
#endif


//  ------------------------------------------------------------------

char* itoa(int value, char* strP, int radix) {

  return longtoa(value, strP, radix, true, 'A');
}


//  ------------------------------------------------------------------

char* ltoa(long value, char* strP, int radix) {

  return longtoa(value, strP, radix, true, 'A');
}


//  ------------------------------------------------------------------

char* ultoa(unsigned long value, char* strP, int radix) {

  return longtoa(value, strP, radix, false, 'A');
}

#endif


//  ------------------------------------------------------------------

bool gtable2value(const char* str, int& val, const gstrvaltable* table, int tablesize) {

  int total = tablesize / sizeof(gstrvaltable);
  for(int n=0; n<total; n++,table++) {
    if(strieql(str, table->str)) {
      val = table->val;
      return true;
    }
  }

  return false;
}


//  ------------------------------------------------------------------

