
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gutluude.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  UUDecoder. Based on source from the Berkerly original.
//  ------------------------------------------------------------------


#include <stdio.h>
#include <gcrcall.h>
#include <gfilutil.h>


//  ------------------------------------------------------------------

#define UUMAXCHAR 256
#define LINELEN 256
#define NORMLEN 60    // Allows for 80 encoded chars per line


//  ------------------------------------------------------------------

int uudecode(char* infile, char* outfile) {

  char buf[LINELEN], outl[LINELEN];
  int cdlen[NORMLEN + 3];
  int chtbl[UUMAXCHAR];
  int i, j, n, c;
  Path outpath;
  int complete = false;

  strcpy(outpath, outfile);
  *outfile = NUL;

  FILE* fpi = fopen(infile, "rt");

  if(fpi) {

    // Set up the default translation table.
    for(i = 0; i < ' '; i++)
      chtbl[i] = -1;
    for(i = ' ', j = 0; i < ' ' + 64; i++, j++)
      chtbl[i] = j;
    for(i = ' ' + 64; i < UUMAXCHAR; i++)
      chtbl[i] = -1;
    chtbl['`'] = chtbl[' '];  /* common mutation */
    chtbl['~'] = chtbl['^'];  /* an other common mutation */
    char blank = ' ';

    // Set up the line length table, to avoid computing lotsa * and / ...
    cdlen[0] = 1;
    for(i = 1, j = 5; i <= NORMLEN; i += 3, j += 4)
      cdlen[i] = (cdlen[i + 1] = (cdlen[i + 2] = j));

    // search for header or translation table line.
    char* key;
    char* val;
    while(fgets((val=buf), sizeof(buf), fpi)) {

      getkeyval(&key, &val);  // key = "begin"

      if(strieql(key, "table")) {

        for(c = 0; c < UUMAXCHAR; c++)
          chtbl[c] = -1;

        n = 0;
        int again = 1;
        while(again) {

          if(not fgets(buf, sizeof(buf), fpi))
            break;

          char* cpt = buf + strlen(buf) - 1;
          *cpt = ' ';
          while(*(cpt) == ' ') {
            *cpt = 0;
            cpt--;
          }
          cpt = buf;
          while((c = *cpt) != 0) {
            if(n == 0)
              blank = (char)c;
            chtbl[c] = n++;
            if(n >= 64) {
              again = 0;
              break;
            }
            cpt++;
          }
        }
        continue;
      }

      if(streql(key, "begin"))
        break;
    }

    getkeyval(&key, &val);  // key = mode
    getkeyval(&key, &val);  // key = filename

    if(*key) {

      strcpy(outfile, AddPath(outpath, key));
      FILE* fpo = fopen(outfile, "wb");
      if(fpo) {


        do {

          if(not fgets(buf, sizeof(buf), fpi))
            break;

          uint len = strlen(buf);
          if(len)
            buf[--len] = '\0';

          // Is it an unprotected empty line before the end line ?
          if(len == 0)
            continue;

          // Get the binary line length.
          n = chtbl[*buf];
          if(n >= 0) {

            // Is it the empty line before the end line ?
            if(n == 0)
              continue;

            // Pad with blanks.
            c = len;
            int rlen = cdlen[n];
            char* bp = buf + c;
            while(c < rlen) {
              *bp++ = blank;
              c++;
            }

            // Output a group of 3 bytes (4 input characters).
            char* ut = outl;
            len = n;
            bp = buf + 1;
            while(n > 0) {
              *(ut++) = (char)((chtbl[*bp] << 2) | (chtbl[bp[1]] >> 4));
              n--;
              if(n) {
                *(ut++) = (char)((chtbl[bp[1]] << 4) | (chtbl[bp[2]] >> 2));
                n--;
              }
              if(n) {
                *(ut++) = (char)((chtbl[bp[2]] << 6) | chtbl[bp[3]]);
                n--;
              }
              bp += 4;
            }
            fwrite(outl, 1, len, fpo);

            continue;
          }

          complete = strnieql(buf, "end", 3);
        } while(not complete);

        fclose(fpo);
      }
    }

    fclose(fpi);
  }

  if(not complete)
    *outfile = NUL;

  return 0;
}


//  ------------------------------------------------------------------

