
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gvectorp.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  Vector template class for lists of pointers.
//  ------------------------------------------------------------------


#include <gmemdbg.h>
#include <gvectorp.h>


//  ------------------------------------------------------------------

gvectp_base::gvectp_base() {

  ptr = NULL;
  ptr_size = 0;
  last_ptr = 0;
}


//  ------------------------------------------------------------------

gvectp_base::~gvectp_base() {

  reset();
}


//  ------------------------------------------------------------------

void gvectp_base::reset() {

  if(ptr) {
    throw_xfree(ptr);
    ptr = NULL;
    ptr_size = 0;
  }
}


//  ------------------------------------------------------------------

void** gvectp_base::locate(void* l) {

  void** p = ptr;
  for(int n=0; n<ptr_size; n++,p++)
    if(*p == l)
      return p;
  return NULL;
}


//  ------------------------------------------------------------------

void gvectp_base::reserve(int n) {

  if(n > ptr_size) {
    ptr = (void**)throw_xrealloc(ptr, n*sizeof(void*));
    for(int i=ptr_size; i<n; i++)
      ptr[i] = NULL;
    ptr_size = n;
  }
}


//  ------------------------------------------------------------------

int gvectp_base::add(void* a) {

  void** p;

  //if((last_ptr < ptr_size) and (ptr[last_ptr] == NULL))
  //  p = ptr[last_ptr++];
  //else
    p = locate(NULL);

  if(p)
    *p = a;
  else {
    ptr = (void**)throw_xrealloc(ptr, (ptr_size+1)*sizeof(void*));
    ptr[ptr_size] = a;
    ptr_size++;
  }

  return ptr_size-1;
}


//  ------------------------------------------------------------------

void gvectp_base::remove(void* r) {

  void** p = locate(r);
  if(p)
    *p = NULL;
}


//  ------------------------------------------------------------------

