
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gvectorp.h,v 1.1.1.1 1998/11/21 19:38:09 odinn Exp $
//  ------------------------------------------------------------------
//  Vector template class for lists of pointers.
//  ------------------------------------------------------------------


#ifndef __GVECTORP_H
#define __GVECTORP_H


//  ------------------------------------------------------------------

#include <gdefs.h>


//  ------------------------------------------------------------------

class gvectp_base {

protected:

  void** ptr;
  int ptr_size;
  int last_ptr;

  void** locate(void* p);

public:

  gvectp_base();
  ~gvectp_base();

  void reset();

  int size() const { return ptr_size; }

  void resize(int n) { reserve(n); }
  void reserve(int n);

  int add(void* a);
  void remove(void* r);

  void** begin() const { return ptr; }
  void** end() const { return begin() + size(); }

  void* operator[](int n) const  { return *(begin() + n); }
};


//  ------------------------------------------------------------------

template <class T>
class gvectp : public gvectp_base {

public:

  int add(T* a)               { return gvectp_base::add(a); }
  void remove(T* r)            { gvectp_base::remove(r); }

  T** begin() const            { return (T**)ptr; }
  T** end() const              { return (T**)(begin() + size()); }

  T*& pointer_at(int n) const        { return *(begin() + n); }
  T*& operator[](int n) const        { return *(begin() + n); }
  T& reference_at(int n) const        { return **(begin() + n); }

};


//  ------------------------------------------------------------------

template <class T>
class gvectref : public gvectp_base {

public:

  int add(T* a)               { return gvectp_base::add(a); }
  void remove(T* r)            { gvectp_base::remove(r); }

  T** begin() const            { return (T**)ptr; }
  T** end() const              { return (T**)(begin() + size()); }

  T*& pointer_at(int n) const        { return *(begin() + n); }
  T& operator[](int n) const        { return **(begin() + n); }
  T& reference_at(int n) const        { return **(begin() + n); }

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

