
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gvidbase.cpp,v 1.4 1999/02/07 11:36:16 odinn Exp $
//  ------------------------------------------------------------------
//  Device-independent video functions.
//  ------------------------------------------------------------------


#if defined(__WATCOMC__)
#include <conio.h>
#endif
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>

#include <gmemall.h>
#include <gmemdbg.h>
#include <gstrall.h>
#include <gvidall.h>

#if defined(__OS2__)
#define INCL_BASE
#include <os2.h>
#ifndef __EMX__
#define PCCH CHAR*
#endif
#endif

#ifdef __WIN32__
#include <windows.h>
#endif

#ifdef __GNUC__
#include <unistd.h>
#endif


//  ------------------------------------------------------------------
//  Check if Borland C++ for OS/2 1.0 header has been fixed

#if defined(__OS2__) and defined(__BORLANDC__)
  #if __BORLANDC__ <= 0x400
    #ifndef BCOS2_BSESUB_FIXED
    #error There is a bug in the BSESUB.H header. Please fix it.
    //
    // Add/change the following in BSESUB.H:
    //
    // #define BCOS2_BSESUB_FIXED
    // APIRET16  APIENTRY16    VioGetState (PVOID16 pState, HVIO hvio);
    // APIRET16  APIENTRY16    VioSetState (PVOID16 pState, HVIO hvio);
    //
    // Borland forgot this (was only PVOID)      ^^
    //
    #endif
  #endif
#endif


//  ------------------------------------------------------------------

static bool __vcurhidden = false;

//  ------------------------------------------------------------------

#ifdef __WIN32__
extern HANDLE gvid_hout;
#endif

//  ------------------------------------------------------------------

#if defined(__UNIX__)


//  ------------------------------------------------------------------

extern int gvid_stdout;
extern bool gvid_xterm;
extern const char* gvid_acs_enable;
extern const char* gvid_acs_disable;
int gvid_last_attr = 0;


//  ------------------------------------------------------------------

void _stdc gvid_printf(const char* fmt, ...) {

  char buf[1024];
  va_list argptr;
  va_start(argptr, fmt);
  int n = vsprintf(buf, fmt, argptr);
  va_end(argptr);

  write(gvid_stdout, buf, n);
}


//  ------------------------------------------------------------------
//  Control chars      01234567890123456789012345678901

const char* gvid_x0 = "x@xxxxxxxxxxxxxx><xxxxxx^vxxxx^v";


//  ------------------------------------------------------------------
//  Straight through - no conversion

#if 0
const char* gvid_x1_straight = 
  "\x80\x81\x82\x83\x84\x85\x86\x87"
  "\x88\x89\x8A\x8B\x8C\x8D\x8E\x8F"
  "\x90\x91\x92\x93\x94\x95\x96\x97"
  "\x98\x99\x9A\x9B\x9C\x9D\x9E\x9F";
#endif


//  ------------------------------------------------------------------
//  Convert CP865 to ISO-8859-1

const char* gvid_x1_cp865_to_latin1 = 
  "\xC7\xFC\xE9\xE2\xE4\xE0\xE5\xE7"
  "\xEA\xEB\xE8\xEF\xEE\xEC\xC4\xC5"
  "\xC9\xE6\xC6\xF4\xF6\xF2\xFB\xF9"
  "\xFF\xD6\xDC\xF8\xA3\xD8\x78\x78";


//  ------------------------------------------------------------------

inline void gvid_cvtchr(char& ch) {
  
  register const char* x0 = gvid_x0;
  register const char* x1 = gvid_x1_cp865_to_latin1;
    
  if(ch < ' ')
    ch = x0[ch];
  else if(in_range(ch, '\x80', '\x9F'))
    ch = x1[ch-128];
}


//  ------------------------------------------------------------------											
  
void gvid_cvtstr(char* s, int len) {

  register char* p = s;

  for(int n=0; n<len; n++,p++)
    gvid_cvtchr(*p);
}


//  ------------------------------------------------------------------

void gvid_cvtstr(word* ws, int len) {
  
  register word* wp = ws;

  for(int n=0; n<len; n++,wp++) 
    gvid_cvtchr(*(char*)wp);
}


//  ------------------------------------------------------------------

static int _atr_to_ans[8] = {
  0,  // BLACK    0
  4,  // BLUE     1
  2,  // GREEN    2
  6,  // CYAN     3
  1,  // RED      4
  5,  // MAGENTA  5
  3,  // BROWN    6
  7   // WHITE    7
};


//  ------------------------------------------------------------------

inline int vatr2ansin(int x) {

  return (x & 8) ? 1 : 0;
}


//  ------------------------------------------------------------------

inline int vatr2ansfg(int x) {

  return _atr_to_ans[x & 7];
}


//  ------------------------------------------------------------------

inline int vatr2ansbg(int x) {

  return _atr_to_ans[(x>>4) & 7];
}


//  ------------------------------------------------------------------

void vputansi(int row, int col, word* buf, int len) {

  char ch;
  int in, fg, bg, acs;
  int atr = gvid_last_attr;
  int in0 = vatr2ansin(atr);
  int fg0 = vatr2ansfg(atr);
  int bg0 = vatr2ansbg(atr);
  int acs0 = atr & ACSET;

  // Get pointer to ANSI line buffer
  char* ptr = gvid.bufansi;

  // Get pointer to video memory image
  byte* p = (byte*)buf;

  for(int n=0; n<len; n++,p+=2) {    // For each screen element

    if(p[1] != atr) {                 // If attribute is different

      atr = p[1];                     // Store new attribute
      gvid_last_attr = atr;

      in = vatr2ansin(atr);           // Get intensity
      fg = vatr2ansfg(atr);           // Get foreground color
      bg = vatr2ansbg(atr);           // Get background color
      acs = atr & ACSET;              // Get Alt Color Set

      if(acs != acs0) {
        ptr = stpcpy(ptr, acs ? gvid_acs_enable : gvid_acs_disable);
	acs0 = acs;
      }

      *ptr++ = 0x1B;                  // Start ANSI color sequence
      *ptr++ = '[';

      if(in != in0) {                 // Set intensity if different
        if(in)
          *ptr++ = '1';               // Intense
        else {
          *ptr++ = '0';               // Reset
          fg0 = bg0 = -1;
        }
        in0 = in;
        if((fg != fg0) or (bg != bg0))
          *ptr++ = ';';
      }

      if(fg != fg0) {                 // Set foreground if different
        *ptr++ = '3';
        *ptr++ = (char)('0' + fg);
        fg0 = fg;
        if(bg != bg0)
          *ptr++ = ';';
      }

      if(bg != bg0) {                 // Set background if different
        *ptr++ = '4';
        *ptr++ = (char)('0' + bg);
        bg0 = bg;
      }

      *ptr++ = 'm';                   // End ANSI color sequence
    }

    ch = p[0];
    gvid_cvtchr(ch);
    *ptr++ = ch;                    // Output the character
  }

  *ptr = NUL;                         // Terminate string

  // Print complete ANSI string at the specified position
  gvid_printf("\x1B[%u;%uH%s", row+1, col+1, gvid.bufansi);
}


//  ------------------------------------------------------------------

#endif


//  ------------------------------------------------------------------

extern int __gdvdetected;


//  ------------------------------------------------------------------
//  Output a char to stdout

void _stdoutch(char ch) {

  fputc(ch, stdout);
}


//  ------------------------------------------------------------------
//  Converts an attribute to monochrome equivalent

int mapattr(int attr) {

  switch(attr&112) {      // test for a light background

    case _LGREY:
    case _GREEN:
    case _CYAN:
    case _BROWN:
      attr &= 240;        // foreground = black
      attr |= 112;        // background = light grey
      break;

    default:
      if((attr&15)==8)    // if foreground = dark grey
        attr &= 247;      // clear intensity bit
      attr |= 7;          // foreground = light grey
      attr &= 143;        // background = black
  }

  return attr;              // return converted attribute
}

                            
//  ------------------------------------------------------------------
//  Reverses the attribute given

int revsattr(int attr) {

  return (int)(((attr>>4)&0x07)|((attr<<4)&0x70)|(attr&0x80)|(attr&0x08));
}


//  ------------------------------------------------------------------

#if defined(__UNIX__)
char* gvid_newattr(int& attr) {

  // 12345678901234567890
  // E[1;33;44mE[11m
  static char newattr[20];
  *newattr = NUL;
  if(attr != gvid_last_attr) {
    if((attr & ~ACSET) != (gvid_last_attr & ~ACSET)) {
      sprintf(newattr, "\033[%c;3%u;4%um",
        vatr2ansin(attr) ? '1' : '0',
        vatr2ansfg(attr),
        vatr2ansbg(attr)
      );
    }
    if((attr & ACSET) != (gvid_last_attr & ACSET))
      strcat(newattr, (attr & ACSET) ? gvid_acs_enable : gvid_acs_disable);
    gvid_last_attr = attr;
  }

  return newattr;
}
#endif


//  ------------------------------------------------------------------
//  Print character and attribute at specfied location

void vputw(int row, int col, word chat) {

  #if defined(__MSDOS__)

  if(gvid.isdma()) {
    *(gvid.dmaptr+((row*gvid.numcols)+col)) = chat;
  }
  else if(gvid.isbios() or gvid.iscga()) {
    i86 cpu;
    cpu.r.h.ah = 2;
    cpu.r.h.bh = 0;
    cpu.r.h.dh = (byte)row;
    cpu.r.h.dl = (byte)col;
    cpu.genint(0x10);
    cpu.r.h.ah = 9;
    cpu.r.h.al = (byte)(chat&0xFF);
    cpu.r.h.bh = 0;
    cpu.r.h.bl = (byte)(chat>>8);
    cpu.r.w.cx = 1;
    cpu.genint(0x10);
  }

  #elif defined(__OS2__)

  BYTE tmp[2];
  tmp[0] = (BYTE)(chat & 0xFF);
  tmp[1] = (BYTE)(chat >> 8);
  VioWrtNCell(tmp, 1, (USHORT)row, (USHORT)col, 0);

  #elif defined(__WIN32__)

  COORD coord;
  DWORD x;
  word atr = (word) (chat >> 8);

  coord.X = (SHORT) col;
  coord.Y = (SHORT) row;
  WriteConsoleOutputAttribute(gvid_hout,           &atr, 1, coord, &x);
  WriteConsoleOutputCharacter(gvid_hout, (char* ) &chat, 1, coord, &x);

  #elif defined(__UNIX__)

  char chr = (char)(chat & 0xFF);
  int atr = chat >> 8;
  char* color = gvid_newattr(atr);
  chat = (word)(chr | (atr << 8));
      
  gvid_cvtstr(&chat, 1);
  *(gvid.dmaptr+((row*gvid.numcols)+col)) = chat;

  gvid_printf("\033[%u;%uH%s%c", row+1, col+1, color, chr);

  #endif
}


//  ------------------------------------------------------------------
//  Print attrib/char buffer at specfied location

void vputws(int row, int col, word* buf, uint len) {

  #if defined(__MSDOS__)

  if(gvid.isdma()) {
    memcpy((gvid.dmaptr+((row*gvid.numcols)+col)), buf, len*2);
  }
  else if(gvid.isbios() or gvid.iscga()) {
    i86 cpu;
    byte* p = (byte*)buf;
    for(uint n=0; n<len; n++) {
      cpu.r.h.ah = 2;
      cpu.r.h.bh = 0;
      cpu.r.h.dh = (byte)row;
      cpu.r.h.dl = (byte)col++;
      cpu.genint(0x10);
      cpu.r.h.ah = 9;
      cpu.r.h.al = *p++;
      cpu.r.h.bh = 0;
      cpu.r.h.bl = *p++;
      cpu.r.w.cx = 1;
      cpu.genint(0x10);
    }
  }

  #elif defined(__OS2__)

  VioWrtCellStr((PCCH)buf, (USHORT)(len*2), (USHORT)row, (USHORT)col, 0);

  #elif defined(__WIN32__)

  COORD coord;
  DWORD x;

  coord.X = (SHORT) col;
  coord.Y = (SHORT) row;

  char* p = (char *) buf;
  word* q = gvid.bufwrd;
  char* c = gvid.bufchr;

  for(int i = 0; i < len; i++) {
    *c++ =  *p++;
    *q++ =  *p++;
  }

  WriteConsoleOutputAttribute(gvid_hout, gvid.bufwrd, len, coord, &x);
  WriteConsoleOutputCharacter(gvid_hout, gvid.bufchr, len, coord, &x);

  #elif defined(__UNIX__)

  gvid_cvtstr(buf, len);
  memcpy((gvid.dmaptr+((row*gvid.numcols)+col)), buf, len*2);
  vputansi(row, col, buf, len);

  #endif
}


//  ------------------------------------------------------------------
//  Print character and attribute at specfied location

void vputc(int row, int col, int atr, char chr) {

  #if defined(__MSDOS__)

  if(gvid.isdma()) {
    *(gvid.dmaptr+((row*gvid.numcols)+col)) = (word)((atr<<8)|chr);
  }
  else if(gvid.isbios() or gvid.iscga()) {
    i86 cpu;
    cpu.r.h.ah = 2;
    cpu.r.h.bh = 0;
    cpu.r.h.dh = (byte)row;
    cpu.r.h.dl = (byte)col;
    cpu.genint(0x10);
    cpu.r.h.ah = 9;
    cpu.r.h.al = chr;
    cpu.r.h.bh = 0;
    cpu.r.h.bl = (byte)atr;
    cpu.r.w.cx = 1;
    cpu.genint(0x10);
  }

  #elif defined(__OS2__)

  BYTE tmp[2];
  tmp[0] = chr;
  tmp[1] = (BYTE)atr;
  VioWrtNCell(tmp, 1, (USHORT)row, (USHORT)col, 0);

  #elif defined(__WIN32__)

  COORD coord;
  DWORD x;

  coord.X = (SHORT) col;
  coord.Y = (SHORT) row;
  WriteConsoleOutputAttribute(gvid_hout, (word *) &atr, 1, coord, &x);
  WriteConsoleOutputCharacter(gvid_hout, (char *) &chr, 1, coord, &x);

  #elif defined(__UNIX__)

  char* color = gvid_newattr(atr);
  gvid_cvtstr(&chr, 1);
  *(gvid.dmaptr+((row*gvid.numcols)+col)) = (word)((atr<<8)|chr);

  gvid_printf("\033[%u;%uH%s%c", row+1, col+1, color, chr);

  #endif
}


//  ------------------------------------------------------------------
//  Print string with attribute at specfied location

void vputs(int row, int col, int atr, const char* str) {

  #if defined(__MSDOS__)

  if(gvid.isdma()) {
    byte* p = (byte*)(gvid.dmaptr+((row*gvid.numcols)+col));
    while(*str) {
      *p++ = *str++;
      *p++ = (byte)atr;
    }
  }
  else if(gvid.isbios() or gvid.iscga()) {
    i86 cpu;
    for(const char* q=str; *q; q++) {
      // Write as fast as possible on XT bios...
      cpu.r.h.ah = 2;
      cpu.r.h.bh = 0;
      cpu.r.h.dh = (byte)row;
      cpu.r.h.dl = (byte)(col++);
      cpu.genint(0x10);
      cpu.r.h.ah = 9;
      cpu.r.h.al = *q;
      cpu.r.h.bh = 0;
      cpu.r.h.bl = (byte)atr;
      cpu.r.w.cx = 1;
      cpu.genint(0x10);
    }
  }

  #elif defined(__OS2__)

  VioWrtCharStrAtt((PCCH)str, (USHORT)strlen(str), (USHORT)row, (USHORT)col, (PBYTE)&atr, 0);

  #elif defined(__WIN32__)

  COORD coord;
  DWORD x;
  int len = strlen(str);

  word* p = gvid.bufwrd;
  for(register int i = 0; i < len; i++)
    *p++ = (word) atr;

  coord.X = (SHORT) col;
  coord.Y = (SHORT) row;
  WriteConsoleOutputAttribute(gvid_hout, gvid.bufwrd, len, coord, &x);
  WriteConsoleOutputCharacter(gvid_hout, str, len, coord, &x);

  #elif defined(__UNIX__)

  char buf[1024];
  strcpy(buf, str);
  char* color = gvid_newattr(atr);  
  gvid_cvtstr(buf, strlen(buf));
  byte* p = (byte*)(gvid.dmaptr+((row*gvid.numcols)+col));
  while(*str) {
    *p++ = *str++;
    *p++ = (byte)atr;
  }

  gvid_printf("\033[%u;%uH%s%s", row+1, col+1, color, buf);

  #endif
}


//  ------------------------------------------------------------------
//  Print string with attribute at specfied location

void _vputns(int row, int col, int atr, const char* str, uint width) {

  char fillchar = ' ';

  byte* p = (byte*)(gvid.dmaptr+((row*gvid.numcols)+col));
  while(width--) {
    *p++ = *str ? *str++ : fillchar;
    *p++ = (byte)atr;
  }
}


//  ------------------------------------------------------------------
//  Print string with attribute at specfied location

void vputns(int row, int col, int atr, const char* str, uint width) {

  char fillchar = ' ';

  #if defined(__MSDOS__)

  if(gvid.isdma()) {
    _vputns(row, col, atr, str, width);
  }
  else if(gvid.isbios() or gvid.iscga()) {
    i86 cpu;
    while(width--) {
      // Write as fast as possible on XT bios...
      cpu.r.h.ah = 2;
      cpu.r.h.bh = 0;
      cpu.r.h.dh = (byte)row;
      cpu.r.h.dl = (byte)(col++);
      cpu.genint(0x10);
      cpu.r.h.ah = 9;
      cpu.r.h.al = *str ? *str++ : fillchar;
      cpu.r.h.bh = 0;
      cpu.r.h.bl = (byte)atr;
      cpu.r.w.cx = 1;
      cpu.genint(0x10);
    }
  }

  #elif defined(__OS2__)

  uint len = strlen(str);
  
  VioWrtCharStrAtt((PCCH)str, (USHORT)minimum_of_two(len,width), (USHORT)row, (USHORT)col, (PBYTE)&atr, 0);

  if(width > len) {
    BYTE tmp[2];
    tmp[0] = fillchar;
    tmp[1] = (BYTE)atr;
    VioWrtNCell(tmp, (USHORT)(width-len), (USHORT)row, (USHORT)(col+len), 0);
  }

  #elif defined(__WIN32__)

  COORD coord;
  DWORD x;
  int len = minimum_of_two(strlen(str), width);

  word* p = gvid.bufwrd;
  for(register int i = 0; i < len; i++)
    *p++ = (word) atr;

  coord.X = (SHORT) col;
  coord.Y = (SHORT) row;
  WriteConsoleOutputAttribute(gvid_hout, gvid.bufwrd, len, coord, &x);
  WriteConsoleOutputCharacter(gvid_hout, str, len, coord, &x);

  if(width > len) {
    coord.X += (SHORT) len;
    len = width - len;

    FillConsoleOutputAttribute(gvid_hout, (word) atr, len, coord, &x);
    FillConsoleOutputCharacter(gvid_hout, fillchar, len, coord, &x);
  }

  #elif defined(__UNIX__)

  char* color = gvid_newattr(atr);
  
  uint len = strlen(str);
  uint min_len = minimum_of_two(len, width);
  char buf[1024];
  strcpy(buf, str);
  gvid_cvtstr(buf, len);

  _vputns(row, col, atr, buf, width);

  char fillbuf[256];
  if(width > len) {
    memset(fillbuf, fillchar, width-len);
    fillbuf[width-len] = NUL;
  }
  else {
    *fillbuf = NUL;
  }
  
  gvid_printf("\033[%u;%uH%s%*.*s%s", row+1, col+1, color,
    min_len, min_len, buf, fillbuf
  );

  #endif
}


//  ------------------------------------------------------------------
//  Print horizontal line of character and attribute

void _vputx(int row, int col, int atr, char chr, uint len) {

  word* w = gvid.dmaptr+((row*gvid.numcols)+col);
  word tmp = (word)((atr<<8) | chr);
  for(uint n=0; n<len; n++)
    *w++ = tmp;
}


//  ------------------------------------------------------------------
//  Print horizontal line of character and attribute

void vputx(int row, int col, int atr, char chr, uint len) {

  #if defined(__MSDOS__)

  if(gvid.isdma()) {
    _vputx(row, col, atr, chr, len);
  }
  else if(gvid.isbios() or gvid.iscga()) {
    i86 cpu;
    cpu.r.h.ah = 2;
    cpu.r.h.bh = 0;
    cpu.r.h.dh = (byte)row;
    cpu.r.h.dl = (byte)col;
    cpu.genint(0x10);
    cpu.r.h.ah = 9;
    cpu.r.h.al = chr;
    cpu.r.h.bh = 0;
    cpu.r.h.bl = (byte)atr;
    cpu.r.w.cx = (word)len;
    cpu.genint(0x10);
  }

  #elif defined(__OS2__)

  BYTE tmp[2];
  tmp[0] = chr;
  tmp[1] = (BYTE)atr;
  VioWrtNCell(tmp, (USHORT)len, (USHORT)row, (USHORT)col, 0);

  #elif defined(__WIN32__)

  COORD c;
  c.X = (SHORT)col;
  c.Y = (SHORT)row;
  DWORD wr;
  FillConsoleOutputCharacter(gvid_hout,       chr, len, c, &wr);
  FillConsoleOutputAttribute(gvid_hout, (WORD)atr, len, c, &wr);

  #elif defined(__UNIX__)

  char buf[256];
  char* color = gvid_newattr(atr);
  gvid_cvtchr(chr);
  _vputx(row, col, atr, chr, len);
  memset(buf, chr, len);
  buf[len] = NUL;
  gvid_printf("\033[%u;%uH%s%s", row+1, col+1, color, buf);

  #endif
}


//  ------------------------------------------------------------------
//  Print vertical line of character and attribute

void _vputy(int row, int col, int atr, char chr, uint len) {

  word* w = gvid.dmaptr+((row*gvid.numcols)+col);
  word tmp = (word)((atr<<8) | chr);
  for(uint n=0; n<len; n++) {
    *w = tmp;
    w += gvid.numcols;
  }
}


//  ------------------------------------------------------------------
//  Print vertical line of character and attribute

void vputy(int row, int col, int atr, char chr, uint len) {

  #if defined(__MSDOS__)

  if(gvid.isdma()) {
    _vputy(row, col, atr, chr, len);
  }
  else if(gvid.isbios() or gvid.iscga()) {
    for(uint n=0; n<len; n++) {
      i86 cpu;
      cpu.r.h.ah = 2;
      cpu.r.h.bh = 0;
      cpu.r.h.dh = (byte)row++;
      cpu.r.h.dl = (byte)col;
      cpu.genint(0x10);
      cpu.r.h.ah = 9;
      cpu.r.h.al = chr;
      cpu.r.h.bh = 0;
      cpu.r.h.bl = (byte)atr;
      cpu.r.w.cx = 1;
      cpu.genint(0x10);
    }
  }

  #elif defined(__OS2__)

  BYTE tmp[2];
  tmp[0] = chr;
  tmp[1] = (BYTE)atr;
  for(int n=0; n<len; n++)
    VioWrtNCell(tmp, 1, (USHORT)row++, (USHORT)col, 0);

  #elif defined(__WIN32__)

  COORD coord;
  DWORD x;

  coord.X = (SHORT) col;

  for(int i=0; i < len; i++) {
    coord.Y = (SHORT) row++;
    WriteConsoleOutputAttribute(gvid_hout, (word *) &atr, 1, coord, &x);
    WriteConsoleOutputCharacter(gvid_hout, (char *) &chr, 1, coord, &x);
  }

  #elif defined(__UNIX__)

  char* color = gvid_newattr(atr);
  gvid_cvtchr(chr);
  _vputy(row, col, atr, chr, len);

  char buf[2048];
  sprintf(buf, "\033[%u;%uH%s", row+1, col+1, color);

  char* p = buf + strlen(buf);
  for(uint n=0; n<(len-1); n++) {
    *p++ = chr;
    if(col == gvid.numcols-1) {
      sprintf(p, "\033[%u;%uH", row+n+2, col+1);
      p += strlen(p);
    }
    else {
      strcpy(p, "\033[D\033[B");
      p += 6;
    }
  }
  *p++ = chr;
  *p = NUL;
  gvid_printf("%s", buf);
  
  #endif
}


//  ------------------------------------------------------------------
//  Get character and attribute at cursor position

word _vgetw(int row, int col) {

  return *(gvid.dmaptr+((row*gvid.numcols)+col));
}


//  ------------------------------------------------------------------
//  Get character and attribute at cursor position

word vgetw(int row, int col) {

  #if defined(__MSDOS__)

  if(gvid.isdma()) {
    return _vgetw(row, col);
  }
  else if(gvid.isbios() or gvid.iscga()) {
    i86 cpu;
    cpu.r.h.ah = 2;
    cpu.r.h.bh = 0;
    cpu.r.h.dh = (byte)row;
    cpu.r.h.dl = (byte)col;
    cpu.genint(0x10);
    cpu.r.h.ah = 8;
    cpu.r.h.bh = 0;
    cpu.genint(0x10);
    return cpu.r.w.ax;
  }
  return 0;

  #elif defined(__OS2__)

  word chat;
  USHORT _row=(USHORT)row, _col=(USHORT)col, len=sizeof(chat);

  #if defined(__EMX__)
  VioReadCellStr((PCH)&chat, &len, _row, _col, 0);
  #else
  VioReadCellStr((CHAR*)&chat, &len, _row, _col, 0);
  #endif

  return chat;

  #elif defined(__WIN32__)

  COORD coord;
  DWORD x;
  word atr;
  char chr;

  coord.X = (SHORT) col;
  coord.Y = (SHORT) row;

  ReadConsoleOutputAttribute(gvid_hout, &atr, 1, coord, &x);
  ReadConsoleOutputCharacter(gvid_hout, &chr, 1, coord, &x);

  return (word)((atr << 8) | chr);

  #elif defined(__UNIX__)

  return _vgetw(row, col);

  #endif
}


//  ------------------------------------------------------------------
//  Get character and attribute at cursor position

void _vgetc(int row, int col, int* atr, char* chr) {

  word w = *(word*)(gvid.dmaptr+((row*gvid.numcols)+col));
  *chr = (char)(w & 0xFF);
  *atr = (int)(w >> 8);
}


//  ------------------------------------------------------------------
//  Get character and attribute at cursor position

void vgetc(int row, int col, int* atr, char* chr) {

  #if defined(__MSDOS__)

  if(gvid.isdma()) {
    _vgetc(row, col, atr, chr);
  }
  else if(gvid.isbios() or gvid.iscga()) {
    i86 cpu;
    cpu.r.h.ah = 2;
    cpu.r.h.bh = 0;
    cpu.r.h.dh = (byte)row;
    cpu.r.h.dl = (byte)col;
    cpu.genint(0x10);
    cpu.r.h.ah = 8;
    cpu.r.h.bh = 0;
    cpu.genint(0x10);
    *chr = cpu.r.h.al;
    *atr = cpu.r.h.ah;
  }

  #elif defined(__OS2__)

  CHAR tmp[2];
  USHORT _row=(USHORT)row, _col=(USHORT)col, len=sizeof(tmp);

  #if defined(__EMX__)
  VioReadCellStr((PCH)tmp, &len, _row, _col, 0);
  #else
  VioReadCellStr(tmp, &len, _row, _col, 0);
  #endif

  *chr = tmp[0];
  *atr = tmp[1];

  #elif defined(__WIN32__)

  COORD coord;
  DWORD x;

  coord.X = (SHORT) col;
  coord.Y = (SHORT) row;

  ReadConsoleOutputAttribute(gvid_hout, (word* ) atr, 1, coord, &x);
  ReadConsoleOutputCharacter(gvid_hout, (char* ) chr, 1, coord, &x);

  #elif defined(__UNIX__)

  _vgetc(row, col, atr, chr);

  #endif
}


//  ------------------------------------------------------------------
//  Scroll screen area

void _vscroll(int srow, int scol, int erow, int ecol, int atr, int lines) {

  if(lines > 0) {
    while(lines--) {
      int nrow = srow;
      int l = ((ecol - scol) + 1);
      word* scrptr = gvid.dmaptr+((srow*gvid.numcols)+scol);
      while(nrow++ < erow) {
        memcpy(scrptr, scrptr + gvid.numcols, l*sizeof(word));
        scrptr += gvid.numcols;
      }
      while(l)
        *(scrptr + --l) = (word)((atr << 8) | ' ');
    }
  }
  else {
    while(lines++) {
      int nrow = erow;
      int l = ((ecol - scol) + 1);
      word* scrptr = gvid.dmaptr+((erow*gvid.numcols)+scol);
      while(nrow-- >= (srow + 1)) {
        memcpy(scrptr, scrptr - gvid.numcols, l*sizeof(word));
        scrptr -= gvid.numcols;
      }
      while(l)
        *(scrptr + --l) = (word)((atr << 8) | ' ');
    }
  }
}


//  ------------------------------------------------------------------
//  Scroll screen area

void vscroll(int srow, int scol, int erow, int ecol, int atr, int lines) {

  #if defined(__MSDOS__)

  if(gvid.isdma()) {
    _vscroll(srow, scol, erow, ecol, atr, lines);
  }
  else if(gvid.isbios() or gvid.iscga()) {
    i86 cpu;
    cpu.r.h.ah = (byte)(lines > 0 ? 6 : 7);
    cpu.r.h.al = (byte)absolute(lines);
    cpu.r.h.bh = (byte)atr;
    cpu.r.h.ch = (byte)srow;
    cpu.r.h.cl = (byte)scol;
    cpu.r.h.dh = (byte)erow;
    cpu.r.h.dl = (byte)ecol;
    cpu.genint(0x10);
  }

  #elif defined(__OS2__)

  BYTE tmp[2];
  tmp[0] = ' ';
  tmp[1] = (BYTE)atr;
  if(lines > 0)
    VioScrollUp((USHORT)srow, (USHORT)scol, (USHORT)erow, (USHORT)ecol, (USHORT)lines, tmp, 0);
  else
    VioScrollDn((USHORT)srow, (USHORT)scol, (USHORT)erow, (USHORT)ecol, (USHORT)-lines, tmp, 0);

  #elif defined(__WIN32__)

  CHAR_INFO fill;
  fill.Char.UnicodeChar = 0;
  fill.Char.AsciiChar = ' ';
  fill.Attributes = (WORD)atr;

  SMALL_RECT r;
  r.Left   = (SHORT)scol;
  r.Top    = (SHORT)srow;
  r.Right  = (SHORT)ecol;
  r.Bottom = (SHORT)erow;

  COORD c;
  c.X = (SHORT)scol;
  c.Y = (SHORT)(srow - lines);

  ScrollConsoleScreenBuffer(gvid_hout, &r, &r, c, &fill);

  #elif defined(__UNIX__)

  _vscroll(srow, scol, erow, ecol, atr, lines);

  word* ptr = gvid.dmaptr + ((srow*gvid.numcols) + scol);
  int len = ecol-scol+1;
  for(int nrow=srow; nrow<=erow; nrow++) {
    vputansi(nrow, scol, ptr, len);
    ptr += gvid.numcols;
  }

  #endif
}


//  ------------------------------------------------------------------
//  Returns true if cursor invisible

bool vcurhidden() {

  return __vcurhidden;
}

//  ------------------------------------------------------------------
//  Get cursor position

void vposget(int* row, int* col) {

  #if defined(__MSDOS__)

  i86 cpu;
  cpu.r.h.ah = 3;
  cpu.r.h.bh = 0;
  cpu.genint(0x10);
  gvid.currow = cpu.r.h.dh;
  gvid.curcol = cpu.r.h.dl;

  #elif defined(__OS2__)

  USHORT _getrow, _getcol;
  VioGetCurPos(&_getrow, &_getcol, 0);
  gvid.currow = _getrow;
  gvid.curcol = _getcol;

  #elif defined(__WIN32__)

  CONSOLE_SCREEN_BUFFER_INFO i;
  GetConsoleScreenBufferInfo(gvid_hout, &i);
  gvid.currow = i.dwCursorPosition.Y;
  gvid.curcol = i.dwCursorPosition.X;

  #elif defined(__UNIX__)

  // Not available

  #endif

  *row = gvid.currow;
  *col = gvid.curcol;
}


//  ------------------------------------------------------------------
//  Set cursor position

void vposset(int row, int col) {

  gvid.currow = row;
  gvid.curcol = col;

  #if defined(__MSDOS__)

  i86 cpu;
  cpu.r.h.ah = 2;
  cpu.r.h.bh = 0;
  cpu.r.h.dh = (byte)row;
  cpu.r.h.dl = (byte)col;
  cpu.genint(0x10);

  #elif defined(__OS2__)

  VioSetCurPos((USHORT)row, (USHORT)col, 0);

  #elif defined(__WIN32__)

    // No need to set the cursor position if its not visible
    // Strangely, this is a major speedup to screen-output

    if(__vcurhidden)
      return;

  COORD c;
  c.X = (SHORT)col;
  c.Y = (SHORT)row;
  SetConsoleCursorPosition(gvid_hout, c);

  #elif defined(__UNIX__)

  gvid_printf("\x1B[%u;%uH", row+1, col+1);

  #endif
}


//  ------------------------------------------------------------------
//  Clears the screen and homes the cursor

void vclrscr() {

  vclrscr((byte)(vgetw(gvid.currow, gvid.curcol) >> 8));
}


//  ------------------------------------------------------------------
//  Clears the screen using given attribute and homes the cursor

void _vclrscr(int atr) {

  word* w = gvid.dmaptr;
  int len = gvid.numrows * gvid.numcols;
  word chat = (word)((atr<<8) | ' ');
  for(int n=0; n<len; n++)
    *w++ = chat;
}


//  ------------------------------------------------------------------
//  Clears the screen using given attribute and homes the cursor

void vclrscr(int atr) {

  #if defined(__MSDOS__)

  if(gvid.isdma()) {
    _vclrscr(atr);
  }
  else if(gvid.isbios() or gvid.iscga()) {
    i86 cpu;
    cpu.r.w.ax = 0x0600;           // clear screen by scrolling it
    cpu.r.h.bh = (byte)atr;
    cpu.r.w.cx = 0;
    cpu.r.h.dh = (byte)(gvid.numrows - 1);
    cpu.r.h.dl = (byte)(gvid.numcols - 1);
    cpu.genint(0x10);
  }

  #elif defined(__OS2__)

  BYTE tmp[2];
  tmp[0] = ' ';
  tmp[1] = (BYTE)atr;
  VioScrollUp(0, 0, 0xFFFF, 0xFFFF, 0xFFFF, tmp, 0);

  #elif defined(__WIN32__)

  COORD c;
  c.X = c.Y = 0;
  DWORD wr, len = gvid.numrows * gvid.numcols;
  FillConsoleOutputCharacter(gvid_hout,       ' ', len, c, &wr);
  FillConsoleOutputAttribute(gvid_hout, (WORD)atr, len, c, &wr);

  #elif defined(__UNIX__)

  _vclrscr(atr);

  gvid_printf("%s\x1B[2J", gvid_newattr(atr));

  #endif

  vposset(0,0);
}


//  ------------------------------------------------------------------
//  Saves the current screen and returns pointer to buffer

void _vsave(word* buf, int len1, int srow, int scol, int erow) {

  const int len2 = len1*sizeof(word);
  word* ptr = gvid.dmaptr + ((srow*gvid.numcols) + scol);
  for(int nrow=srow; nrow<=erow; nrow++) {
    memcpy(buf, ptr, len2);
    ptr += gvid.numcols;
    buf += len1;
  }
}


//  ------------------------------------------------------------------
//  Saves the current screen and returns pointer to buffer

word* vsave(int __srow, int __scol, int __erow, int __ecol) {

  if(__srow == -1)  __srow = 0;
  if(__scol == -1)  __scol = 0;
  if(__erow == -1)  __erow = gvid.numrows-1;
  if(__ecol == -1)  __ecol = gvid.numcols-1;

  word* sbuf = (word*)throw_xcalloc((((__erow-__srow+1)*(__ecol-__scol+1))+4), sizeof(word));
  if(sbuf) {

    word* buf = sbuf;

    buf[0] = (word)__srow;
    buf[1] = (word)__scol;
    buf[2] = (word)__erow;
    buf[3] = (word)__ecol;

    #if defined(__MSDOS__)

    int srow = *buf++;
    int scol = *buf++;
    int erow = *buf++;
    int ecol = *buf++;

    int len1 = ecol-scol+1;

    if(gvid.isdma()) {
      _vsave(buf, len1, srow, scol, erow);
    }
    else if(gvid.isbios() or gvid.iscga()) {
      i86 cpu;
      byte* p = (byte*)buf;
      for(byte row=(byte)srow; row<=erow; row++) {
        for(byte col=(byte)scol; col<=ecol; col++) {
          cpu.r.h.ah = 2;
          cpu.r.h.bh = 0;
          cpu.r.h.dh = row;
          cpu.r.h.dl = col;
          cpu.genint(0x10);
          cpu.r.h.ah = 8;
          cpu.r.h.bh = 0;
          cpu.genint(0x10);
          *p++ = cpu.r.h.al;
          *p++ = cpu.r.h.ah;
        }
      }
    }

    #elif defined(__OS2__)

    USHORT srow = *buf++;
    USHORT scol = *buf++;
    USHORT erow = *buf++;
    USHORT ecol = *buf++;

    USHORT len1 = (USHORT)(ecol-scol+1);

    #if defined(__BORLANDC__)
    PCHAR16 ptr = (PCHAR16)buf;
    #else
    PCH ptr = (PCH)buf;
    #endif

    USHORT len2 = (USHORT)(len1*sizeof(word));
    for(USHORT nrow=srow; nrow<=erow; nrow++) {
      VioReadCellStr(ptr, &len2, nrow, scol, 0);
      ptr += len2;
    }

    #elif defined(__WIN32__)

    COORD coord;
    DWORD x;
    SHORT srow = *buf++;
    SHORT scol = *buf++;
    SHORT erow = *buf++;
    SHORT ecol = *buf++;

    coord.X = scol;

    SHORT len1 = (SHORT)(ecol-scol+1);
    char* p = (char *) buf;

    for(SHORT nrow=srow; nrow<=erow; nrow++) {
      coord.Y = nrow;
      ReadConsoleOutputAttribute(gvid_hout, gvid.bufwrd, len1, coord, &x);
      ReadConsoleOutputCharacter(gvid_hout, gvid.bufchr, len1, coord, &x);

      word* q = gvid.bufwrd;
      char* c = gvid.bufchr;

      for(int i = 0; i < len1; i++) {
        *p++ =  *c++;
        *p++ =  (byte) *q++;
      }
    }

    #elif defined(__UNIX__)

    int srow = *buf++;
    int scol = *buf++;
    int erow = *buf++;
    int ecol = *buf++;

    int len1 = ecol-scol+1;

    _vsave(buf, len1, srow, scol, erow);

    #endif
  }

  return sbuf;
}


//  ------------------------------------------------------------------
//  Redraws a previously saved screen

void _vredraw(word* buf, int len1, int srow, int scol, int erow) {

  const int len2 = len1*sizeof(word);
  word* ptr = gvid.dmaptr + ((srow*gvid.numcols) + scol);
  for(int nrow=srow; nrow<=erow; nrow++) {
    memcpy(ptr, buf, len2);
    ptr += gvid.numcols;
    buf += len1;
  }
}


//  ------------------------------------------------------------------
//  Redraws a previously saved screen

void vredraw(word* buf, int __srow, int __scol, int __erow, int __ecol) {

  if(__srow != -1)  buf[0] = (word)__srow;
  if(__scol != -1)  buf[1] = (word)__scol;
  if(__erow != -1)  buf[2] = (word)__erow;
  if(__ecol != -1)  buf[3] = (word)__ecol;

  #if defined(__MSDOS__)

  int srow = *buf++;
  int scol = *buf++;
  int erow = *buf++;
  int ecol = *buf++;

  int len1 = ecol-scol+1;

  if(gvid.isdma()) {
    _vredraw(buf, len1, srow, scol, erow);
  }
  else if(gvid.isbios() or gvid.iscga()) {
    i86 cpu;
    byte* p = (byte*)buf;
    for(byte row=(byte)srow; row<=erow; row++) {
      for(byte col=(byte)scol; col<=ecol; col++) {
        cpu.r.h.ah = 2;
        cpu.r.h.bh = 0;
        cpu.r.h.dh = row;
        cpu.r.h.dl = col;
        cpu.genint(0x10);
        cpu.r.h.ah = 9;
        cpu.r.h.al = *p++;
        cpu.r.h.bh = 0;
        cpu.r.h.bl = *p++;
        cpu.r.w.cx = 1;
        cpu.genint(0x10);
      }
    }
  }

  #elif defined(__OS2__)

  USHORT srow = *buf++;
  USHORT scol = *buf++;
  USHORT erow = *buf++;
  USHORT ecol = *buf++;

  USHORT len1 = (USHORT)(ecol-scol+1);
  USHORT len2 = (USHORT)(len1*sizeof(word));

  #if defined(__BORLANDC__)
  PCHAR16 ptr = (PCHAR16)buf;
  #else
  PCH ptr = (PCH)buf;
  #endif

  for(USHORT nrow=srow; nrow<=erow; nrow++) {
    VioWrtCellStr(ptr, len2, nrow, scol, 0);
    ptr += len2;
  }

  #elif defined(__WIN32__)

  SHORT srow = *buf++;
  SHORT scol = *buf++;
  SHORT erow = *buf++;
  SHORT ecol = *buf++;

  SHORT len1 = (SHORT)(ecol-scol+1);

  for(SHORT nrow=srow; nrow<=erow; nrow++) {
    vputws(nrow, scol, buf, len1);
    buf += len1;
  }

  #elif defined(__UNIX__)

  int srow = *buf++;
  int scol = *buf++;
  int erow = *buf++;
  int ecol = *buf++;

  int len1 = ecol-scol+1;

  _vredraw(buf, len1, srow, scol, erow);
  
  int atr = *buf >> 8;
  char* color = gvid_newattr(atr);
  gvid_printf("%s", color);

  for(int nrow=srow; nrow<=erow; nrow++) {
    vputansi(nrow, scol, buf, len1);
    buf += len1;
  }

  #endif
}


//  ------------------------------------------------------------------
//  Restores a previously saved screen and frees buffer

void vrestore(word* buf, int srow, int scol, int erow, int ecol) {

  vredraw(buf, srow, scol, erow, ecol);
  throw_xfree(buf);
}


//  ------------------------------------------------------------------
//  Sets the cursor shape/size

void vcurset(int sline, int eline) {

  if(eline) {
    gvid.curr.cursor.start = sline;
    gvid.curr.cursor.end = eline;
    __vcurhidden = false;
  }

  #if defined(__MSDOS__)

  if(eline == 0) {
    int _dvhide = __gdvdetected ? 0x01 : 0x30;
    sline = ((gvid.adapter>=V_HGC) and (gvid.adapter<=V_INCOLOR)) ? 0x3F : _dvhide;
  }

  i86 cpu;
  cpu.r.h.ah = 1;
  cpu.r.h.ch = (byte)sline;
  cpu.r.h.cl = (byte)eline;
  cpu.genint(0x10);

  #elif defined(__OS2__)

  VIOCURSORINFO vioci;
  VioGetCurType(&vioci, 0);
  vioci.yStart = (USHORT)sline;
  vioci.cEnd   = (USHORT)eline;
  vioci.attr   = (USHORT)((eline == 0) ? 0xFFFF : gvid.curr.color.textattr);
  VioSetCurType(&vioci, 0);

  #elif defined(__WIN32__)

  CONSOLE_CURSOR_INFO cci;

  if(eline)
    vposset(gvid.currow, gvid.curcol);
  else if(not __vcurhidden) {

    // We're just about to hide the cursor. At least under Win9x,
    // M$ can't really hide the cursor in window mode. So move it
    // it to a place where it's less nerve-racking.

    // Odinn: This is not the fully correct solution.
    //        When in the GoldED IE, the cursor moves when the status
    //        line is updated (clock). That's a whole lot more
    //        nerve-racking.
    //        Besides, the cursor is hidden just fine in my Win95?
    #if 0
    COORD c;
    c.X = 0;
    c.Y = (SHORT)(gvid.numrows-1);
    SetConsoleCursorPosition(gvid_hout, c);
    #endif
  }

  cci.dwSize = (eline and sline) ? sline : 1;
  cci.bVisible = eline ? true : false;

  SetConsoleCursorInfo(gvid_hout, &cci);

  #elif defined(__UNIX__)

  gvid_printf("\033[?25%c", eline ? 'h' : 'l');

  #endif
}


//  ------------------------------------------------------------------
//  Hides the cursor

void vcurhide() {

  if(not __vcurhidden) {
    vcurset(0,0);
    __vcurhidden = true;

  }
}


//  ------------------------------------------------------------------
//  Reveals the cursor

void vcurshow() {

  if(__vcurhidden) {
    vcurset(gvid.curr.cursor.start, gvid.curr.cursor.end);
    __vcurhidden = false;
  }
}


//  ------------------------------------------------------------------
//  Sets a large cursor

void vcurlarge() {

  vcurshow();

  #if defined(__MSDOS__)

  switch(gvid.adapter) {
    case V_CGA:
      vcurset(1,7);
      break;
    case V_EGA:
      if(gvid.numrows == 25) {
        vcurset(1,7);
      }
      else {
        word* p = (word*)0x0463;  // video BIOS data area
        outpw(*p,0x000A);         // update cursor start register
        outpw(*p,0x0A0B);         // update cursor end register
      }
      break;
    case V_VGA:
      vcurset(1,7);
      break;
    default:    // one of the monochrome cards
      vcurset(1,12);
  }

  #elif defined(__OS2__)

  if(gvid.numrows == 25)
    vcurset(1,12);
  else
    vcurset(1,7);

  #elif defined(__WIN32__)

  vcurset(90, true);

  #endif
}


//  ------------------------------------------------------------------
//  Sets a small cursor

void vcursmall() {

  vcurshow();

  #if defined(__MSDOS__)

  switch(gvid.adapter) {
    case V_CGA:
      vcurset(6,7);
      break;
    case V_EGA:
      if(gvid.numrows == 25) {
        vcurset(6,7);
      }
      else {
        word* p = (word*)0x0463;    // video BIOS data area
        outpw(*p,0x060A);           // update cursor start register
        outpw(*p,0x000B);           // update cursor end register
      }
      break;
    case V_VGA:
      vcurset(6,7);
      break;
    default:    // one of the monochrome cards
      vcurset(11,12);
  }

  #elif defined(__OS2__)

  switch(gvid.numrows) {
    case 25: vcurset(11, 12); break;
    case 28: vcurset(9, 11); break;
    default: vcurset(6, 7);
  }

  #elif defined(__WIN32__)

  vcurset(13, true);

  #endif
}


//  ------------------------------------------------------------------
//  Table of characters used to display boxes
//
//  Access box table characters via:
//      _box_table[boxtype][x]
//
//  where:
//      boxtype is the number of the box type you want to use (0 - 5)
//
//      x will be one of the following:
//           0 - upper left corner
//           1 - upper horizontal line
//           2 - upper right corner
//           3 - left vertical line
//           4 - right vertical line
//           5 - lower left corner
//           6 - lower horizontal line
//           7 - lower right corner
//           8 - middle junction
//           9 - left vertical junction
//          10 - right vertical junction
//          11 - upper horizontal junction
//          12 - lower horizontal junction
//          13 - checkerboard
//  ------------------------------------------------------------------

char* __FAR _box_table[] = {

  #if 0 // defined(__linux__)

  ".-.||`-'+||--",    // box type 0     Single border
  ".-.||`-'+||--",    // box type 1     Double border
  ".-.||`-'+||--",    // box type 2     Single top
  ".-.||`-'+||--",    // box type 3     Double top
  "             ",    // box type 4     With empty border
  ".-.||`-'+||--",    // box type 5     No border at all
  ".-.||`-'+||--",    // box type 6     Blocky border
  ".-.||`-'+||--",    // box type 7     ASCII border

  #if 0
  "",    // box type 0     Single border
  "",    // box type 1     Double border
  "",    // box type 2     Single top
  "",    // box type 3     Double top
  "             ",    // box type 4     With empty border
  ".-.||`-'+||--",    // box type 5     No border at all
  ".-.||`-'+||--",    // box type 6     Blocky border
  ".-.||`-'+||--",    // box type 7     ASCII border
  "lqkxxmqjntuwv",    // box type 8     xterm single border
  #endif

  #else

  "Ŀô",    // box type 0     Single border
  "ͻͼ̹ʰ",    // box type 1     Double border
  "ķĽǶа",    // box type 2     Single top
  "͸;Ƶϰ",    // box type 3     Double top
  "             ",    // box type 4     With empty border
  "Ŀô",    // box type 5     No border at all
  "ݰ",    // box type 6     Blocky border
  ".-.||`-'+||--##",    // box type 7     ASCII border
  "lqkxxmqjntuwvaa",    // box type 8     xterm single border
  
  #endif
};


//  ------------------------------------------------------------------

#if defined(__UNIX__)
void gvid_boxcvt(char* s) {
  
  while(*s) {
    switch(*s) {
      case '': *s = _box_table[8][ 0]; break;
      case '': *s = _box_table[8][ 1]; break;
      case '': *s = _box_table[8][ 2]; break;
      case '': *s = _box_table[8][ 4]; break;
      case '': *s = _box_table[8][ 5]; break;
      case '': *s = _box_table[8][ 7]; break;
      case '': *s = _box_table[8][ 8]; break;
      case '': *s = _box_table[8][ 9]; break;
      case '': *s = _box_table[8][10]; break;
      case '': *s = _box_table[8][11]; break;
      case '': *s = _box_table[8][12]; break;
    }
    s++;
  }
}
#endif


//  ------------------------------------------------------------------
//  Draws a text box on the screen

void vbox(int srow, int scol, int erow, int ecol, int box, int hiattr, int loattr) {

  if(loattr == -1)
    loattr = hiattr;
  else if(loattr == -2)
    loattr = (int)((hiattr & 0x08) ? (hiattr & 0xF7) : (hiattr | 0x08));

  #if defined(__UNIX__)
  hiattr |= ACSET;
  loattr |= ACSET;
  #endif
  
  vputc(srow,   scol,   hiattr, _box_table[box][0]);               // Top left corner
  vputx(srow,   scol+1, hiattr, _box_table[box][1], ecol-scol-1);  // Top border
  vputc(srow,   ecol,   loattr, _box_table[box][2]);               // Top right corner
  vputy(srow+1, scol,   hiattr, _box_table[box][3], erow-srow-1);  // Left border
  vputy(srow+1, ecol,   loattr, _box_table[box][4], erow-srow-1);  // Right border
  vputc(erow,   scol,   hiattr, _box_table[box][5]);               // Bottom left corner
  vputx(erow,   scol+1, loattr, _box_table[box][6], ecol-scol-1);  // Bottom border
  vputc(erow,   ecol,   loattr, _box_table[box][7]);               // Bottom right corner
}


//  ------------------------------------------------------------------
//  Fills an area of screen with a character & attribute

void vfill(int srow, int scol, int erow, int ecol, char chr, int atr) {

  int width = ecol-scol+1;
  for(int crow=srow; crow<=erow; crow++)
    vputx(crow, scol, atr, chr, width);
}


//  ------------------------------------------------------------------
