
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gvidinit.cpp,v 1.3 1999/02/02 02:04:00 damware Exp $
//  ------------------------------------------------------------------
//  Device-independent video functions
//  ------------------------------------------------------------------


//#include <dos.h>
#if defined(__WATCOMC__)
#include <conio.h>
#endif
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>

#include <gmemall.h>
#include <gmemdbg.h>
#include <gstrall.h>
#include <gvidall.h>

#if defined(__OS2__)
#define INCL_BASE
#include <os2.h>
#endif

#if defined(__WIN32__)
#include <windows.h>
#endif

#if defined(__UNIX__)
#include <sys/ioctl.h>
#include <termios.h>
#include <unistd.h>
#include <errno.h>
#endif

#if defined(__DJGPP__)
#include <sys/farptr.h>
#endif


//  ------------------------------------------------------------------
//  Check if Borland C++ for OS/2 1.0 header has been fixed

#if defined(__OS2__) and defined(__BORLANDC__)
  #if __BORLANDC__ <= 0x400
    #ifndef BCOS2_BSESUB_FIXED
    #error There is a bug in the BSESUB.H header. Please fix it.
    //
    // Add/change the following in BSESUB.H:
    //
    // #define BCOS2_BSESUB_FIXED
    // APIRET16  APIENTRY16    VioGetState (PVOID16 pState, HVIO hvio);
    // APIRET16  APIENTRY16    VioSetState (PVOID16 pState, HVIO hvio);
    //
    // Borland forgot this (was only PVOID)      ^^
    //
    #endif
  #endif
#endif


//  ------------------------------------------------------------------

#ifdef __WIN32__
//#define GOLD_VIDEO_DEBUG
#endif


//  ------------------------------------------------------------------
//  Global video data

#ifdef __WIN32__
HANDLE gvid_hout;
#endif

GVid __FAR gvid;

int __gdvdetected = false;

#ifdef GOLD_VIDEO_DEBUG
int gvideo_debug = true;
#endif

#if defined(__UNIX__)

int gvid_stdout = -1;
bool gvid_xterm = false;
const char* gvid_acs_enable;
const char* gvid_acs_disable;

void _vputx(int row, int col, int atr, char chr, uint len);
void _stdc gvid_printf(const char* fmt, ...);

#endif


//  ------------------------------------------------------------------
//  Video Class constructor

GVid::GVid() {

  dmaptr = dmadir = NULL;
  bufchr = NULL;
  bufwrd = NULL;
  bufansi = NULL;

  init();
}


//  ------------------------------------------------------------------
//  Video Class destructor

GVid::~GVid() {
  
  #if defined(__UNIX__)
  //gvid_printf("\033[?25h\033[0;37;40m");
  //gvid_printf("\033c\033[?25h\033[0;37;40m");
  
  // "\033<"        Enter ANSI mode
  // "\033[?5l"     Normal screen
  // "\033[0m"      Normal character attributes

  gvid_printf("\033<\033[?5l\033[0m");
  
  #endif
  if(dmaptr != dmadir)  throw_xfree(dmaptr);
  if(bufwrd)            throw_xfree(bufwrd);
  if(bufchr)            throw_xfree(bufchr);
  if(bufansi)           throw_xfree(bufansi);
}


//  ------------------------------------------------------------------

void GVid::init() {

  #ifdef GOLD_VIDEO_DEBUG
  const char* _eptr = getenv("GDEBUG");
  if(_eptr)
    gvideo_debug = !!striinc("VIDEO", _eptr);
  #endif

  // Detect video adapter
  detectadapter();

  // Detect original video information
  detectinfo(&orig);
  memcpy(&curr, &orig, sizeof(GVidInfo));

  #if defined(__MSDOS__)
  device = GVID_DMA;
  #elif defined(__OS2__)
  device = GVID_OS2;
  #elif defined(__WIN32__)
  device = GVID_W32;
  #endif

  #if defined(__WATCOMC__) and defined(__386__)
  dmaptr = dmadir = (word*)(gvid.videoseg << 4);
  #elif defined(__DJGPP__)
  dmaptr = dmadir = (word*)gmkfp(gvid.videoseg, 0);
  #elif defined(__OS2__) or defined(__WIN32__) or defined(__UNIX__)
  dmaptr = (word*)throw_xcalloc((orig.screen.rows+1)*orig.screen.columns, sizeof(word));
  dmadir = NULL;
  #else
  dmaptr = dmadir = (word*)MK_FP(gvid.videoseg, 0);
  #endif

  bufchr = NULL;
  bufwrd = NULL;
  bufansi = NULL;

  resetcurr();

  vcursmall();
}


//  ------------------------------------------------------------------
//  Structure for real-mode interrupt calls

#if defined(__WATCOMC__) and defined(__MSDOS__) and defined(__386__)
static struct rminfo {
  long EDI;
  long ESI;
  long EBP;
  long reserved_by_system;
  long EBX;
  long EDX;
  long ECX;
  long EAX;
  short flags;
  short ES,DS,FS,GS,IP,CS,SP,SS;
} RMI;

#endif


//  ------------------------------------------------------------------
//  Video adapter detect

int GVid::detectadapter() {

  #ifdef GOLD_VIDEO_DEBUG
  if(gvideo_debug)
    printf("\nNow in detectadapter()\n");
  #endif

  // Initialize to a valid value
  adapter = GV_NONE;

  #if defined(__MSDOS__)

  i86 cpu;

  // Get video mode
  cpu.r.h.ah = V_GET_MODE;
  cpu.genint(0x10);
  int _got_mode = cpu.r.h.al;

  // Check for PS/2 compatible video BIOS by calling the get
  // video configuration function. If it exists, the video
  // configuration code will be returned in BL.

  cpu.r.w.ax = 0x1A00;
  cpu.genint(0x10);

  if(cpu.r.h.al == 0x1A) {

    #ifdef GOLD_VIDEO_DEBUG
    if(gvideo_debug)
      printf("Int 10h ax=1A00h returned bl=%02Xh\n", cpu.r.h.bl);
    #endif

    switch(cpu.r.h.bl) {
      case 0x00:
        adapter = GV_NONE;
        break;
      case 0x01:
        adapter = V_MDA;
        break;
      case 0x02:
        adapter = V_CGA;
        break;
      case 0x04:
        adapter = V_EGA;
        break;
      case 0x05:
        adapter = V_EGAMONO;
        break;
      case 0x07:
        adapter = (_got_mode == 7) ? V_VGAMONO : V_VGA;
        break;
      case 0x08:
        adapter = (_got_mode == 7) ? V_VGAMONO : V_VGA;
        break;
      case 0x0A:
      case 0x0C:
        adapter = V_MCGA;
        break;
      case 0x0B:
        adapter = V_MCGAMONO;
        break;
      default:
        adapter = V_VGA;    // We hope it is VGA compatible!
    }
  }
  else {

    // OK, we know that it's not a PS/2 BIOS, so check for an EGA.
    // If an EGA is not present, BH will be unchanged on return.

    cpu.r.h.ah = 0x12;
    cpu.r.h.bl = 0x10;
    cpu.genint(0x10);

    #ifdef GOLD_VIDEO_DEBUG
    if(gvideo_debug)
      printf("Int 10h ah=12h bl=10h returned bl=%02Xh\n", cpu.r.h.bl);
    #endif

    if(cpu.r.h.bl - 0x10) {
      adapter = cpu.r.h.bh ? V_EGAMONO : V_EGA;
    }
    else {

      // Now we know it's not an EGA. Get the BIOS equipment
      // flag and test for CGA, MDA, or no video adapter.

      cpu.genint(0x11);

      #ifdef GOLD_VIDEO_DEBUG
      if(gvideo_debug)
        printf("Int 11h returned al=%02Xh\n", cpu.r.h.al);
      #endif

      switch(cpu.r.h.al & 0x30) {
        case 0x00:
          adapter = (_got_mode == 7) ? V_VGAMONO : V_VGA;   // EGA, VGA, or PGA
          break;
        case 0x10:
          adapter = GV_NONE;   // 40x25 color
          break;
        case 0x20:
          adapter = V_CGA;    // 80x25 color
          break;
        case 0x30:
          adapter = V_MDA;    // 80x25 monochrome
          break;
      }
    }
  }

  #ifdef GOLD_VIDEO_DEBUG
  if(gvideo_debug) {
    const char* _adaptername = "";
    switch(adapter) {
      case GV_NONE:      _adaptername = "None";              break;
      case V_MDA:       _adaptername = "MDA";               break;
      case V_HGC:       _adaptername = "HGC";               break;
      case V_HGCPLUS:   _adaptername = "HGC Plus";          break;
      case V_INCOLOR:   _adaptername = "Hercules Incolor";  break;
      case V_CGA:       _adaptername = "CGA";               break;
      case V_EGA:       _adaptername = "EGA";               break;
      case V_EGAMONO:   _adaptername = "EGA mono";          break;
      case V_MCGA:      _adaptername = "MCGA";              break;
      case V_MCGAMONO:  _adaptername = "MCGA mono";         break;
      case V_VGA:       _adaptername = "VGA";               break;
      case V_VGAMONO:   _adaptername = "VGA mono";          break;
    }
    printf("Detected card is %s (%s)\n", _adaptername, (adapter & V_MONO) ? "Monochrome" : "Color");
  }
  #endif

  // Set video segment
  #if defined(__BORLANDC__) and defined(__DPMI32__)
  videoseg = (word)((adapter & V_MONO) ? __SegB000 : __SegB800);
  #else
  videoseg = (word)((adapter & V_MONO) ? 0xB000 : 0xB800);
  #endif

  // check for presence of DESQview by using the DOS Set
  // System Date function and trying to set an invalid date

  cpu.r.w.ax = 0x2B01;  // DOS Set System Date
  cpu.r.h.ch = 'D';
  cpu.r.h.cl = 'E';
  cpu.r.h.dh = 'S';
  cpu.r.h.dl = 'Q';
  cpu.genint(0x21);

  // if error, then DESQview not present
  if(cpu.r.h.al != 0xFF) {

    __gdvdetected = true;

    #if defined(__WATCOMC__) and defined(__MSDOS__) and defined(__386__)
      union REGS regs;
      struct SREGS sregs;
      memset(&RMI, 0, sizeof(RMI));
      memset(&regs, 0, sizeof(REGS));
      memset(&sregs, 0, sizeof(SREGS));
      RMI.EAX = 0x0000FE00;
      RMI.ES = videoseg;
      regs.w.ax = 0x0300;
      regs.h.bl = 0x10;
      regs.h.bh = 0;
      regs.w.cx = 0;
      sregs.es = FP_SEG(&RMI);
      regs.x.edi = FP_OFF(&RMI);
      int386x(0x31, &regs, &regs, &sregs);
      videoseg = RMI.ES;
    #else
      cpu.zeroseg();
      cpu.r.h.ah = 0xFE;        // DV get alternate video buffer
      cpu.r.w.es = videoseg;
      cpu.r.w.di = 0x0000;
      cpu.genintseg(0x10);
      videoseg = cpu.r.w.es;
    #endif

    #ifdef GOLD_VIDEO_DEBUG
    if(gvideo_debug)
      printf("DESQview detected, video segment %04Xh\n", videoseg);
    #endif
  }

  #elif defined(__OS2__)

  {
    #if 0
    VIOMODEINFO viomodeinfo;
    memset(&viomodeinfo, 0, sizeof(VIOMODEINFO));
    viomodeinfo.cb = sizeof(VIOMODEINFO);
    VioGetMode(&viomodeinfo, 0);
    adapter = V_VGA | (viomodeinfo.fbType ? 0 : V_MONO);
    #endif

    VIOCONFIGINFO vioconfiginfo;
    memset(&vioconfiginfo, 0, sizeof(VIOCONFIGINFO));
    vioconfiginfo.cb = sizeof(VIOCONFIGINFO);
    VioGetConfig(0, &vioconfiginfo, 0);
    switch(vioconfiginfo.adapter) {
      case DISPLAY_MONOCHROME:
        adapter = V_MDA;
        break;
      case DISPLAY_CGA:
        adapter = V_CGA;
        break;
      case DISPLAY_EGA:
        adapter = V_EGA;
        break;
      case DISPLAY_VGA:
        adapter = V_VGA;
        break;
      default:
        adapter = V_VGA;    // We hope it is VGA compatible!
    }
    if(vioconfiginfo.display == DISPLAY_MONOCHROME)
      adapter |= V_MONO;
  }

  #elif defined(__WIN32__)

  gvid_hout = CreateFile("CONOUT$", GENERIC_READ | GENERIC_WRITE,
                     FILE_SHARE_WRITE | FILE_SHARE_READ, NULL,
                     OPEN_EXISTING,
                     FILE_FLAG_NO_BUFFERING|FILE_FLAG_WRITE_THROUGH, NULL);

  SetFileApisToOEM();

  adapter = V_VGA;

  #if defined(GOLD_VIDEO_DEBUG)
  if(gvideo_debug)
    printf("\nCreateFile(\"CONOUT$\"): hout = %i\n", gvid_hout);
  #endif

  #elif defined(__UNIX__)

  const char* term = getenv("TERM");
  if(term and strneql(term, "xterm", 5)) {
    gvid_xterm = true;
    for(int n=0; n<8; n++)
      _box_table[n] = _box_table[8];
  }
    
  gvid_acs_enable  = gvid_xterm ? "\033)0\033(B\016" : "\033[11m";
  gvid_acs_disable = gvid_xterm ? "\033(B\033)B\017" : "\033[10m";

  gvid_stdout = fileno(stdout);

  adapter = V_VGA;

  #endif

  return adapter;
}


//  ------------------------------------------------------------------
//  Video info detect

void GVid::detectinfo(GVidInfo* _info) {

  // Reset all original values
  memset(_info, 0, sizeof(GVidInfo));

  #if defined(__MSDOS__)

  i86 cpu;

  // Get video mode and number of columns
  cpu.r.h.ah = V_GET_MODE;
  cpu.genint(0x10);
  _info->screen.mode = cpu.r.h.al;
  _info->screen.columns = cpu.r.h.ah;

  #ifdef GOLD_VIDEO_DEBUG
  if(gvideo_debug)
    printf("Int 10h ah=0Fh returned al=%02Xh (mode), ah=%u (columns)\n", cpu.r.h.al, cpu.r.h.ah);
  #endif

  // Get the number of screen rows
  if(adapter >= V_EGA) {
    cpu.r.w.ax = V_GET_FONT_INFO;
    cpu.r.w.dx = 0;
    cpu.genint(0x10);
    _info->screen.rows = cpu.r.h.dl + ((adapter & V_EGA) ? 0 : 1);
    if(_info->screen.rows == 24)  // Normally nonsense
      _info->screen.rows++;
    //_info->screen.cheight = cpu.r.w.cx;
    _info->screen.cheight = 8;
    _info->screen.cwidth = 8;

    #ifdef GOLD_VIDEO_DEBUG
    if(gvideo_debug)
      printf("Int 10h ax=1130h dx=0 returned dl=%u (rows)\n", cpu.r.h.dl);
    #endif
  }
  else {
    _info->screen.rows = 25;
    _info->screen.cheight = 8;
    _info->screen.cwidth = 8;
  }

  // Get character attribute under the cursor
  cpu.r.h.ah = V_RD_CHAR_ATTR;
  cpu.r.h.bh = 0;
  cpu.genint(0x10);
  _info->color.textattr = cpu.r.h.ah;

  // Get cursor position and form
  cpu.r.h.ah = V_GET_CURSOR_POS;
  cpu.r.h.bh = 0;
  cpu.genint(0x10);
  _info->cursor.row = cpu.r.h.dh;
  _info->cursor.column = cpu.r.h.dl;
  _info->cursor.start = cpu.r.h.ch;
  _info->cursor.end = cpu.r.h.cl;
  _info->cursor.attr = (word)_info->color.textattr;
  #ifdef GOLD_VIDEO_DEBUG
  if(gvideo_debug)
    printf("Cursor row %u, column %u, startscan %u, endscan %u\n",
      _info->cursor.row,
      _info->cursor.column,
      _info->cursor.start,
      _info->cursor.end
    );
  #endif

  // Get overscan color
  if(adapter & V_VGA) {
    cpu.r.w.ax = 0x1008;
    cpu.r.h.bh = 0xFF;
    cpu.genint(0x10);
    _info->color.overscan = cpu.r.h.bh;
    #ifdef GOLD_VIDEO_DEBUG
    if(gvideo_debug)
      printf("Int 10h ax=1008h bh=FFh returned bh=%u (overscan)\n", cpu.r.h.bh);
    #endif
  }

  // Get intensity state
  {
    // Check bit 5 at 0000:0465
    #if defined(__DJGPP__)
    _info->color.intensity = (_farpeekb (_dos_ds, 0x465) & 0x20) ? 0 : 1;
    #else
    byte* _bptr = (byte*)0x0465;
    _info->color.intensity = (*_bptr & 0x20) ? 0 : 1;
    #endif

    #ifdef GOLD_VIDEO_DEBUG
    if(gvideo_debug)
      printf("Intense detected: %u\n", _info->color.intensity);
    #endif
  }

  #elif defined(__OS2__)

  // Get video mode and number of rows and columns
  {
    VIOMODEINFO viomodeinfo;
    memset(&viomodeinfo, 0, sizeof(VIOMODEINFO));
    viomodeinfo.cb = sizeof(VIOMODEINFO);
    VioGetMode(&viomodeinfo, 0);
    _info->screen.mode = viomodeinfo.fbType;
    _info->screen.rows = viomodeinfo.row;
    _info->screen.columns = viomodeinfo.col;
  }

  // Get cursor position and character attribute under the cursor
  {
    USHORT usRow, usColumn;
    VioGetCurPos(&usRow, &usColumn, 0);
    _info->cursor.row = usRow;
    _info->cursor.column = usColumn;
    BYTE chat[2];
    USHORT len = 2;
    #if defined(__EMX__)
    VioReadCellStr((PCH)chat, &len, usRow, usColumn, 0);
    #else
    VioReadCellStr((CHAR*)chat, &len, usRow, usColumn, 0);
    #endif
    _info->color.textattr = chat[1];
  }

  // Get cursor form
  {
    VIOCURSORINFO viocursorinfo;
    memset(&viocursorinfo, 0, sizeof(VIOCURSORINFO));
    VioGetCurType(&viocursorinfo, 0);
    _info->cursor.start = viocursorinfo.yStart;
    _info->cursor.end = viocursorinfo.cEnd;
    _info->cursor.attr = viocursorinfo.attr;
  }

  // Get intensity state
  {
    VIOINTENSITY viointensity;
    memset(&viointensity, 0, sizeof(VIOINTENSITY));
    viointensity.cb = sizeof(VIOINTENSITY);
    viointensity.type = 0x0002;
    VioGetState(&viointensity, 0);
    _info->color.intensity = viointensity.fs ? 1 : 0;
  }

  // Get overscan color
  {
    VIOOVERSCAN viooverscan;
    memset(&viooverscan, 0, sizeof(VIOOVERSCAN));
    viooverscan.cb = sizeof(VIOOVERSCAN);
    viooverscan.type = 0x0001;
    VioGetState(&viooverscan, 0);
    _info->color.overscan = (int)viooverscan.color;
  }

  #elif defined(__WIN32__)

  #ifdef GOLD_VIDEO_DEBUG
  if(gvideo_debug)
    printf("GetConsoleScreenBufferInfo:");
  #endif

  // Get video mode and number of rows and columns
  CONSOLE_SCREEN_BUFFER_INFO csbi;
  GetConsoleScreenBufferInfo(gvid_hout, &csbi);

  #ifdef GOLD_VIDEO_DEBUG
  if(gvideo_debug) {
    printf(
      "  dwSize              = %i,%i\n"
      "  dwCursorPosition    = %i,%i\n"
      "  wAttributes         = %u\n"
      "  srWindow            = %i,%i,%i,%i\n"
      "  dwMaximumWindowSize = %i,%i\n",
      csbi.dwSize.X, csbi.dwSize.Y,
      csbi.dwCursorPosition.X, csbi.dwCursorPosition.Y,
      csbi.wAttributes,
      csbi.srWindow.Left, csbi.srWindow.Top, csbi.srWindow.Right, csbi.srWindow.Bottom,
      csbi.dwMaximumWindowSize.X, csbi.dwMaximumWindowSize.Y
    );
  }
  #endif

  _info->screen.mode = 0;
  _info->screen.rows = csbi.srWindow.Bottom - csbi.srWindow.Top + 1;
  _info->screen.columns = csbi.srWindow.Right - csbi.srWindow.Left + 1;

  // Get cursor position and character attribute under the cursor
  _info->cursor.row = csbi.dwCursorPosition.Y - csbi.srWindow.Top;
  _info->cursor.column = csbi.dwCursorPosition.X - csbi.srWindow.Left;
  _info->color.textattr = csbi.wAttributes;

  #ifdef GOLD_VIDEO_DEBUG
  if(gvideo_debug)
    printf("GetConsoleCursorInfo:");
  #endif

  // Get cursor form
  CONSOLE_CURSOR_INFO cci;
  GetConsoleCursorInfo(gvid_hout, &cci);
   
  #ifdef GOLD_VIDEO_DEBUG
  if(gvideo_debug) {
    printf(
      "  dwSize   = %i\n"
      "  bVisible = %u\n",
      cci.dwSize, cci.bVisible
    );
  }
  #endif

  _info->cursor.start = (int) 1;
  _info->cursor.end = cci.bVisible ? (int) 15 : 0;
  _info->cursor.attr = (word)(cci.bVisible ? 7 : 0);

  // Get intensity state
  _info->color.intensity = 1;

  // Get overscan color
  _info->color.overscan = 0;

  #elif defined(__UNIX__)

  int r = 0, c = 0;

  if(r <= 0) {
    char *s = getenv("LINES");
    if(s) {
      //printf("getenv(\"LINES\") = %s\n", s);
      r = atoi(s);
    }
  }

  if(c <= 0) {
    char *s = getenv("COLUMNS");
    if(s) {
      //printf("getenv(\"COLUMNS\") = %s\n", s);
      c = atoi(s);
    }
  }

  //printf("init1: c=%i, r=%i\n", c, r);

  #if defined(TIOCGSIZE)
  if(r <= 0 or c <= 0) {
    struct ttysize sz;

    do {
      if((ioctl(1,TIOCGSIZE,&sz) == 0) or (ioctl(0, TIOCGSIZE, &sz) == 0) or (ioctl(2, TIOCGSIZE, &sz) == 0)) {
        c = (int)sz.ts_cols;
        r = (int)sz.ts_lines;
        break;
      }
    } while(errno == EINTR);
  }
  //printf("init2: c=%i, r=%i\n", c, r);
  #elif defined(TIOCGWINSZ)
  if(r <= 0 or c <= 0) {
    struct winsize wind_struct;

    do {
      if((ioctl(1,TIOCGWINSZ,&wind_struct) == 0) or (ioctl(0, TIOCGWINSZ, &wind_struct) == 0) or (ioctl(2, TIOCGWINSZ, &wind_struct) == 0)) {
        c = (int)wind_struct.ws_col;
        r = (int)wind_struct.ws_row;
        break;
      }
    } while(errno == EINTR);
  }
  //printf("init3: c=%i, r=%i\n", c, r);
  #endif


  if((r <= 0) or (r > 200))
    r = 24;
  if((c <= 0) or (c > 250))
    c = 80;

  //printf("init4: c=%i, r=%i\n", c, r);

  //*dmadir = 0;

  _info->screen.mode = 0;
  _info->screen.rows = r;
  _info->screen.columns = c;
  _info->cursor.row = 0;
  _info->cursor.column = 0;
  _info->color.textattr = 7;
  _info->cursor.start = 11;
  _info->cursor.end = 12;
  _info->cursor.attr = 7;
  _info->color.intensity = 1;
  _info->color.overscan = 0;
  
  #endif

  getpalette(_info->color.palette);
}


//  ------------------------------------------------------------------
//  Reset current video info

void GVid::resetcurr() {

  currow = curr.cursor.row;
  curcol = curr.cursor.column;

  numrows = curr.screen.rows;
  numcols = curr.screen.columns;

  throw_xfree(bufchr);
  throw_xfree(bufwrd);
  throw_xfree(bufansi);

  bufchr = (char*)throw_xcalloc(1, numcols+1);
  bufwrd = (word*)throw_xcalloc(2, numcols+1);
  bufansi = (char*)throw_xcalloc(1, (11*numcols)+1);

  #if defined(__UNIX__)

  // Clear internal buffer
  for(int n=0; n<numrows; n++)
    _vputx(n, 0, LGREY, ' ', numcols);

  #endif

  setdevice(device);
}


//  ------------------------------------------------------------------
//  Sets video output device

void GVid::setdevice(int _device) {

  device = _device;
}


//  ------------------------------------------------------------------
//  Sets video mode

void GVid::setmode(int _mode) {

  if(_mode) {
    #if defined(__MSDOS__)

    i86 cpu;
    cpu.r.h.ah = 0x00;
    cpu.r.h.al = (byte)_mode;
    cpu.genint(0x10);

    #endif
  }

  detectinfo(&curr);
  resetcurr();
}


//  ------------------------------------------------------------------
//  Sets screen rows

void GVid::setrows(int _rows) {


  int origrows = curr.screen.rows;

  #if defined(__MSDOS__)
  i86 cpu;

  // Set video mode 3 (80xNN)
  if(curr.screen.mode != 3) {
    cpu.r.w.ax = 0x0003;
    cpu.genint(0x10);
  }

  if(adapter >= V_EGA) {
    if(_rows == 28 and adapter >= V_VGA) {  // vga-only
      cpu.r.w.ax = 0x1202;
      cpu.r.h.bl = 0x30;
      cpu.genint(0x10);
      cpu.r.w.ax = 0x1111;
      cpu.r.h.bl = 0;
      cpu.genint(0x10);
    }
    else if(_rows >= 43) {
      cpu.r.w.ax = 0x1112;    // Load 8x8 character set
      cpu.r.h.bl = 0x00;
      cpu.genint(0x10);
      cpu.r.w.ax = 0x1200;    // Select alternate print-screen routine
      cpu.r.h.bl = 0x20;
      cpu.genint(0x10);
      if(adapter & V_EGA) {
        // Disable cursor size emulation
        byte* _bptr = (byte*)0x0487;
        *_bptr |= (byte)0x01;
        // Set cursor size
        cpu.r.h.ah = 0x01;
        cpu.r.h.al = (byte)orig.screen.mode;
        cpu.r.w.cx = 0x0600;
        cpu.genint(0x10);
      }
    }
    else {
      if(adapter & V_EGA) {
        // Enable cursor size emulation
        byte* _bptr = (byte*)0x0487;
        *_bptr &= (byte)0xFE;
      }
      // Set cursor size
      cpu.r.h.ah = 0x01;
      cpu.r.h.al = (byte)orig.screen.mode;
      cpu.r.w.cx = 0x0607;
      cpu.genint(0x10);
    }
  }


  #elif defined(__OS2__)

  VIOMODEINFO viomodeinfo;
  memset(&viomodeinfo, 0, sizeof(VIOMODEINFO));
  viomodeinfo.cb = sizeof(VIOMODEINFO);
  VioGetMode(&viomodeinfo, 0);
  viomodeinfo.row = (USHORT)_rows;
  VioSetMode(&viomodeinfo, 0);

  #elif defined(__WIN32__) or defined(__UNIX__)

  NW(_rows);

  #endif

  if(origrows < _rows)
    vfill(origrows, 0, _rows, 80, ' ', 7);

  detectinfo(&curr);
  resetcurr();
}


//  ------------------------------------------------------------------
//  Set the screen border (overscan) color

void GVid::setoverscan(int _overscan) {

  #if defined(__MSDOS__)

  i86 cpu;
  cpu.r.h.ah = 0x0B;
  cpu.r.h.bh = 0x00;
  cpu.r.h.bl = (byte)_overscan;
  cpu.genint(0x10);

  #elif defined(__OS2__)

  VIOOVERSCAN viooverscan;
  memset(&viooverscan, 0, sizeof(VIOOVERSCAN));
  viooverscan.cb = sizeof(VIOOVERSCAN);
  viooverscan.type = 0x0001;
  VioGetState(&viooverscan, 0);
  viooverscan.color = (BYTE)_overscan;
  VioSetState(&viooverscan, 0);

  #elif defined(__WIN32__) or defined(__UNIX__)

  NW(_overscan);

  #endif
}


//  ------------------------------------------------------------------
//  Set intensity/blinking state

void GVid::setintensity(int _intensity) {

  #if defined(__MSDOS__)

  if(adapter & V_CGA) {
    word* _wptr = (word*)0x0463;
    byte* _bptr = (byte*)0x0465;
    uint port = *_wptr + 4;
    uint reg = *_bptr;
    if(_intensity)
      reg &= 0xDF;
    else
      reg |= 0x20;
    outp(port, reg);
  }
  else {
    i86 cpu;
    cpu.r.w.ax = 0x1003;
    cpu.r.h.bh = 0x00;
    cpu.r.h.bl = (byte)(_intensity ? 0 : 1);
    cpu.genint(0x10);
  }

  #elif defined(__OS2__)

  VIOINTENSITY viointensity;
  memset(&viointensity, 0, sizeof(VIOINTENSITY));
  viointensity.cb = sizeof(VIOINTENSITY);
  viointensity.fs = (USHORT)(_intensity ? 1 : 0);
  viointensity.type = 0x0002;
  VioSetState(&viointensity, 0);

  #elif defined(__WIN32__) or defined(__UNIX__)

  NW(_intensity);

  #endif
}


//  ------------------------------------------------------------------

void GVid::getpalette(int* _palette) {

  #if defined(__MSDOS__)

  // Get palette state
  if(adapter & V_VGA) {
    i86 cpu;
    for(byte n=0; n<16; n++) {
      cpu.r.w.ax = 0x1007;    // GET INDIVIDUAL PALETTE REGISTER
      cpu.r.h.bh = 0xFF;
      cpu.r.h.bl = n;
      cpu.genint(0x10);
      _palette[n] = cpu.r.h.bh;
    }
  }
  else {
    // Set to standard palette colors
    for(byte n=0; n<16; n++)
      _palette[n] = n + ((n > 7) ? 48 : 0);
  }

  #elif defined(__OS2__)

  // Get palette state
  BYTE viopalstate[38];
  PVIOPALSTATE pviopalstate;
  memset(viopalstate, 0, sizeof(viopalstate));
  pviopalstate = (PVIOPALSTATE)viopalstate;
  pviopalstate->cb = sizeof(viopalstate);
  pviopalstate->type = 0;
  pviopalstate->iFirst = 0;
  VioGetState(pviopalstate, 0);
  for(byte n=0; n<16; n++)
    _palette[n] = pviopalstate->acolor[n];

  #elif defined(__WIN32__) or defined(__UNIX__)

  NW(_palette);

  #endif
}


//  ------------------------------------------------------------------

void GVid::setpalette(int* _palette) {

  #if defined(__MSDOS__)

  if(adapter & (V_EGA|V_MCGA|V_VGA)) {
    i86 cpu;
    for(byte n=0; n<16; n++) {
      if(_palette[n] != -1) {
        cpu.r.w.ax = 0x1000;
        cpu.r.h.bl = n;
        cpu.r.h.bh = (byte)_palette[n];
        cpu.genint(0x10);
      }
    }
  }

  #elif defined(__OS2__)

  BYTE viopalstate[38];
  PVIOPALSTATE pviopalstate;
  memset(viopalstate, 0, sizeof(viopalstate));
  pviopalstate = (PVIOPALSTATE)viopalstate;
  pviopalstate->cb = sizeof(viopalstate);
  pviopalstate->type = 0;
  pviopalstate->iFirst = 0;
  VioGetState(pviopalstate, 0);
  for(byte n=0; n<16; n++)
    if(_palette[n] != -1)
      pviopalstate->acolor[n] = (USHORT)_palette[n];
  VioSetState(pviopalstate, 0);

  #elif defined(__WIN32__) or defined(__UNIX__)

  NW(_palette);

  #endif
}


//  ------------------------------------------------------------------

void GVid::restore_cursor() {

  vcurset(orig.cursor.start, orig.cursor.end);
  vcurshow();
//  vposset(orig.cursor.row, 0);
//  vputx(orig.cursor.row, 0, orig.color.textattr, ' ', orig.screen.columns);
  vposset(orig.cursor.row-1, 0);
}


//  ------------------------------------------------------------------

void GVid::resize_screen(int columns, int rows) {

  gvid.numcols = curr.screen.columns = columns;
  gvid.numrows = curr.screen.rows    = rows;

  bufchr = (char*)throw_xrealloc(bufchr, numcols);
  bufwrd = (word*)throw_xrealloc(bufwrd, numcols*2);
  bufansi = (char*)throw_xrealloc(bufansi, 1+(11*numcols));

  #if defined(__OS2__) or defined(__WIN32__) or defined(__UNIX__)
  dmaptr = (word*)throw_xrealloc(dmaptr, (rows+1)*columns*sizeof(word));
  #endif
}


//  ------------------------------------------------------------------

