
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gwinfile.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  View file in current window.
//  ------------------------------------------------------------------


#include <gfile.h>
#include <gstrall.h>
#include <gmemdbg.h>
#include <gkbdcode.h>
#include <gwindow.h>
#include <gwinfile.h>


//  ------------------------------------------------------------------

void gwin_viewfile(gwindow& window, char* file, bool pagebar) {

  struct strlst {
    int  len;
    long pos;
  };

  int xofs = 0;
  int xlen = window.width() - (window.has_border() ? 2 : 0);
  int ylen = window.height() - (window.has_border() ? 2 : 0);

  gfile fp(file, "rb");
  if(fp) {

    vcurhide();

    int n, i;
    int xmax = 0;
    int strs = 0;
    char buf[2560];
    long fpos = 0L;
    strlst* str = NULL;

    while(fp.fgets(buf, sizeof(buf))) {
      if((strs%100) == 0)
        str = (strlst*)throw_xrealloc(str, (strs+102)*sizeof(strlst));
      int len = strlen(buf);
      int trimmed_len = strlen(strtrim(buf));
      str[strs].pos = fpos;
      str[strs].len = trimmed_len;
      xmax = maximum_of_two(xmax, trimmed_len);
      fpos += len;
      strs++;
    }

    char** scr = (char**)throw_xcalloc(ylen+1, sizeof(char*));
    for(n=0; n<ylen; n++)
      scr[n] = (char*)throw_xcalloc(1, xmax+1);

    bool pbar = pagebar and (strs > ylen);
    if(pbar)
      xlen--;

    int curr = 0;
    fp.rewind();
    for(n=0,i=0; n<ylen and i<strs; n++,i++) {
      fp.fgets(buf, sizeof(buf));
      strcpy(scr[n], strtrim(buf));
      window.printf(n, 0, "%-*.*s", xlen, xlen, str[i].len-xofs < 0 ? "" : scr[n]+xofs);
    }

    Key xch;

    do {

      if(pbar)
        window.vscrollbar(strs, strs-ylen, curr);

      xch = getxch();

      sprintf(buf, " xch = 0x%04X ", xch);
      window.title(buf);

      switch(xch) {

        case Key_Dwn:
          if(strs > ylen and (curr+ylen-1) < strs-1) {
            curr++;
            fp.fseekset(str[curr+ylen-1].pos);
            fp.fgets(buf, sizeof(buf));
            strcpy(scr[ylen-1], strtrim(buf));
            window.scroll_up();
            window.printf(ylen-1, 0, "%-*.*s", xlen, xlen, str[curr+ylen-1].len-xofs < 0 ? "" : scr[ylen-1]+xofs);
          }
          break;

        case Key_Up:
          if(strs > ylen and curr) {
            curr--;
            fp.fseekset(str[curr].pos);
            fp.fgets(buf, sizeof(buf));
            strcpy(scr[0], strtrim(buf));
            window.scroll_down();
            window.printf(0, 0, "%-*.*s", xlen, xlen, str[curr].len-xofs < 0 ? "" : scr[0]+xofs);
          }
          break;

        case Key_PgUp:
          if(strs > ylen)
            for(n=0; n<ylen; n++)
              kbput(Key_Up);
          break;

        case Key_PgDn:
          if(strs > ylen)
            for(n=0; n<ylen; n++)
              kbput(Key_Dwn);
          break;

        case Key_C_Lft:
          xofs -= 7;

        case Key_Lft:
          xofs--;
          if(xofs < 0)
            xofs = 0;
          fp.fseekset(str[curr].pos);
          for(n=0,i=curr; n<ylen and i<strs; n++,i++) {
            fp.fgets(buf, sizeof(buf));
            strcpy(scr[n], strtrim(buf));
            window.printf(n, 0, "%-*.*s", xlen, xlen, xofs > str[i].len ? "" : scr[n]+xofs);
          }
          break;

        case Key_C_Rgt:
          xofs += 7;

        case Key_Rgt:
          xofs++;
          if(xofs > (xmax-xlen))
            xofs = xmax-xlen;
          fp.fseekset(str[curr].pos);
          for(n=0,i=curr; n<ylen and i<strs; n++,i++) {
            fp.fgets(buf, sizeof(buf));
            strcpy(scr[n], strtrim(buf));
            window.printf(n, 0, "%-*.*s", xlen, xlen, xofs > str[i].len ? "" : scr[n]+xofs);
          }
          break;

        case Key_C_Home:
          xofs = 0;
          fp.fseekset(str[curr].pos);
          for(n=0,i=curr; n<ylen and i<strs; n++,i++) {
            fp.fgets(buf, sizeof(buf));
            strcpy(scr[n], strtrim(buf));
            window.printf(n, 0, "%-*.*s", xlen, xlen, str[i].len-xofs <= 0 ? "" : scr[n]+xofs);
          }
          break;

        case Key_C_End:
          xofs = xmax-xlen;
          fp.fseekset(str[curr].pos);
          for(n=0,i=curr; n<ylen and i<strs; n++,i++) {
            fp.fgets(buf, sizeof(buf));
            strcpy(scr[n], strtrim(buf));
            window.printf(n, 0, "%-*.*s", xlen, xlen, str[i].len-xofs <= 0 ? "" : scr[n]+xofs);
          }
          break;

        case Key_Home:
          if(strs > ylen) {
            curr = 0;
            fp.fseekset(str[curr].pos);
            for(n=0,i=0; n<ylen and i<strs; n++,i++) {
              fp.fgets(buf, sizeof(buf));
              strcpy(scr[n], strtrim(buf));
              window.printf(n, 0, "%-*.*s", xlen, xlen, str[i].len-xofs <= 0 ? "" : scr[n]+xofs);
            }
          }
          break;

        case Key_End:
          if(strs > ylen) {
            curr = strs-ylen;
            fp.fseekset(str[curr].pos);
            for(n=0,i=curr; n<ylen and i<strs; n++,i++) {
              fp.fgets(buf, sizeof(buf));
              strcpy(scr[n], strtrim(buf));
              window.printf(n, 0, "%-*.*s", xlen, xlen, str[i].len-xofs <= 0 ? "" : scr[n]+xofs);
            }
          }
          break;

      }
    } while(xch != Key_Esc);

    for(n=0; n<ylen; n++)
      throw_xfree(scr[n]);
    throw_xfree(scr);
    throw_xfree(str);
    
    vcurshow();

    fp.fclose();
  }
}


//  ------------------------------------------------------------------

