
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gwininit.cpp,v 1.1.1.1 1998/11/21 19:38:07 odinn Exp $
//  ------------------------------------------------------------------
//  Based on CXL Mike Smedley.
//  Windowing kernel.
//  ------------------------------------------------------------------


#include <gdos.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include <gmemdbg.h>
#include <gutlmisc.h>
#include <gwinall.h>
#include <gkbdcode.h>


//  ------------------------------------------------------------------

#define GOLD_INLINE inline
#define GOLD_WCHK


//  ------------------------------------------------------------------
//  Global window data

GWin __FAR gwin;


//  ------------------------------------------------------------------
//  Window Class constructor

GWin::GWin() {

  active     = NULL;          // pointer to active window
  hidden     = NULL;          // pointer to last hidden window
  menu       = NULL;          // pointer to head menu record
  cmenu      = NULL;          // pointer to current menu record
  helptr     = NULL;          // pointer to help info record
  handle     = 0;             // last handle given to a window
  help       = 0;             // pointer to current help category
  werrno     = W_NOERROR;     // error num from last window func
  total      = 0;             // total number of open windows
  mlevel     = 0;             // system variable used in menus
  ilevel     = 0;             // system variable used in menus
  esc        = true;          // check for Esc in input funcions?
  tabwidth   = 8;             // window TTY output tab width
  fillch     = ' ';           // character to fill windows with
  style      = STYLE_NORMAL;  // window opening style
}


//  ------------------------------------------------------------------
//  Window Class destructor

GWin::~GWin() {

  // No action defined yet
}


//  ------------------------------------------------------------------
//  Displays a string in centered in active window

int wcenters(int wrow, int attr, const char* str) {

  register int window_width, string_length;
  int start_column, border;

  // check for active window

  if(!gwin.total)
    return gwin.werrno=W_NOACTIVE;

  // check for valid row

  if(wchkcoord(wrow,0))
    return gwin.werrno=W_INVCOORD;

  // check for window border

  border=gwin.active->border;

  // calculate start column & window width

  start_column = gwin.active->scol+border;
  window_width = (gwin.active->ecol-border)-start_column+1;

  // check length of input string

  string_length=strlen(str);
  if(string_length>window_width)
    return gwin.werrno=W_STRLONG;

  // display the string

  vputs(
    gwin.active->srow+wrow+border,
    ((window_width/2)+start_column)-(string_length/2),
    attr,
    str
  );

  // return normally

  return gwin.werrno=W_NOERROR;
}


//  ------------------------------------------------------------------
//  Moves active window to a new location

#if 0
int wmove(int nsrow, int nscol) {

  word* p, q;
  int nerow,necol,shattr=-1;

  // check for active windows
  if(!gwin.total)
    return(gwin.werrno=W_NOACTIVE);

  // check for valid coordinates
  if(nsrow<0 or nscol<0)
    return(gwin.werrno=W_INVCOORD);

  // if window has shadow, close it before the move
  if(gwin.active->wsbuf!=NULL) {
    shattr=gwin.active->wsattr;
    wshadoff();
  }

  // save current window
  if((q=vsave(gwin.active->srow,gwin.active->scol,gwin.active->erow,gwin.active->ecol))==NULL)
    return(gwin.werrno=W_ALLOCERR);

  // restore what current window's buffer holds
  vrestore(gwin.active->wbuf);

  // calculate new end row and new end column
  nerow = gwin.active->erow-gwin.active->srow+nsrow;
  necol = gwin.active->ecol-gwin.active->scol+nscol;

  // save area of screen where window will be moved to
  if((p=vsave(nsrow,nscol,nerow,necol))==NULL)
    return(gwin.werrno=W_ALLOCERR);

  // change coordinates in saved window's buffer and restore window to new coordinates
  q[0] = (word)nsrow;
  q[1] = (word)nscol;
  q[2] = (word)nerow;
  q[3] = (word)necol;
  vrestore(q);

  // update window record
  gwin.active->wbuf   = p;
  gwin.active->row    = gwin.active->row-gwin.active->srow+nsrow;
  gwin.active->column = gwin.active->column-gwin.active->scol+nscol;
  gwin.active->srow   = nsrow;
  gwin.active->scol   = nscol;
  gwin.active->erow   = nerow;
  gwin.active->ecol   = necol;

  // if window has shadow, redraw it
  if(shattr!=-1)
    wshadow((int)shattr);

  // reset cursor position
  vposset(gwin.active->row,gwin.active->column);

  // return normally
  return gwin.werrno=W_NOERROR;
}
#endif


//  ------------------------------------------------------------------
//  Smoothly drag a window 1 position in given direction

int wdrag(int direction) {

  int srow, scol, erow, ecol, fill_row, fill_col, i;
  int nsrow, nscol, nerow, necol, shad_attr=-1;
  int chars_per_line, lines_per_win;
  int vert_movement, horz_movement;
  word* win_image;
  word* wp;
  word* p;
  register word* src;
  register word* dest;

  // check for active window
  if(!gwin.total)
    return(gwin.werrno=W_NOACTIVE);

  // get window coordinates
  srow = gwin.active->srow;
  scol = gwin.active->scol;
  erow = gwin.active->erow;
  ecol = gwin.active->ecol;

  // calculate lines-per-window and characters-per-line
  lines_per_win  = erow - srow + 1;
  chars_per_line = ecol - scol + 1;

  // determine facts about direction of move
  vert_movement=horz_movement=0;
  switch(direction) {
    case D_DOWN:
      vert_movement=1;
      lines_per_win--;
      break;
    case D_UP:
      vert_movement=-1;
      lines_per_win--;
      break;
    case D_LEFT:
      horz_movement=-1;
      chars_per_line--;
      break;
    case D_RIGHT:
    default:
      horz_movement=1;
      chars_per_line--;
  }

  // calculate new window coordinates
  nsrow = srow + vert_movement;
  nscol = scol + horz_movement;
  nerow = erow + vert_movement;
  necol = ecol + horz_movement;

  // if window has shadow, close it before the move
  if(gwin.active->wsbuf!=NULL) {
    shad_attr = gwin.active->wsattr;
    wshadoff();
  }

  // save the current window image
  win_image = vsave(srow,scol,erow,ecol);
  if(win_image==NULL)
    return(gwin.werrno=W_ALLOCERR);

  // save the area where the window will relocate to
  wp = vsave(nsrow,nscol,nerow,necol);
  if(wp==NULL) {
    throw_xfree(win_image);
    return(gwin.werrno=W_ALLOCERR);
  }

  // change coordinates in saved window's buffer
  // and restore window to new coordinates
  win_image[0] = (word)nsrow;
  win_image[1] = (word)nscol;
  win_image[2] = (word)nerow;
  win_image[3] = (word)necol;
  vrestore(win_image);

  // start buffer positions past coordinates
  src  = gwin.active->wbuf + 4;
  dest = wp + 4;

  if(direction==D_DOWN)
    src  += chars_per_line;
  if(direction==D_UP)
    dest += chars_per_line;

  // do the transfer of buffer contents
  for(i=0; i < lines_per_win; i++) {

    if(direction==D_LEFT)
      dest++;
    if(direction==D_RIGHT)
      src++;

    // move 1 line
    memmove(dest, src, chars_per_line*sizeof(word));
    src  += chars_per_line;
    dest += chars_per_line;

    if(direction==D_LEFT)
      src++;
    if(direction==D_RIGHT)
      dest++;
  }

  // erase the trail that was left-over
  p = gwin.active->wbuf + 4;
  if(vert_movement) {
    if(direction==D_DOWN)
      fill_row=srow;
    else {
      p += (lines_per_win * chars_per_line);
      fill_row=erow;
    }
    vputx(fill_row, scol, (int)((*p)>>8), (char)*p, ecol-scol+1);
  }
  else {
    if(direction==D_RIGHT)
      fill_col=scol;
    else {
      p += chars_per_line;
      fill_col=ecol;
    }
    for(i=srow;i<=erow;i++,p+=chars_per_line+1)
      vputc(i, fill_col, (int)((*p)>>8), (char)*p);
  }

  // free old window buffer
  throw_xfree(gwin.active->wbuf);

  // update window record
  gwin.active->wbuf   = wp;
  gwin.active->row    = gwin.active->row    - gwin.active->srow + nsrow;
  gwin.active->column = gwin.active->column - gwin.active->scol + nscol;
  gwin.active->srow   = nsrow;
  gwin.active->scol   = nscol;
  gwin.active->erow   = nerow;
  gwin.active->ecol   = necol;

  // if window has shadow, redraw it
  if(shad_attr!=-1)
    wshadow((int)shad_attr);

  // reset cursor position
  vposset(gwin.active->row,gwin.active->column);

  // return normally
  return(gwin.werrno=W_NOERROR);
}


//  ------------------------------------------------------------------
//  Slides active window to a new location

int wslide(int nsrow, int nscol) {

  int shattr=-1, err=0;

  // check for active windows
  if(!gwin.total)
    return gwin.werrno=W_NOACTIVE;

  // check for valid coordinates
  if(nsrow<0 or nscol<0)
    return gwin.werrno=W_INVCOORD;

  // if window has shadow, close it before the move
  if(gwin.active->wsbuf!=NULL) {
    shattr = gwin.active->wsattr;
    wshadoff();
  }

  // slide it on over
  while(gwin.active->scol>nscol and !err) err = wdrag(D_LEFT);
  while(gwin.active->scol<nscol and !err) err = wdrag(D_RIGHT);
  while(gwin.active->srow>nsrow and !err) err = wdrag(D_UP);
  while(gwin.active->srow<nsrow and !err) err = wdrag(D_DOWN);

  // test for error
  if(gwin.werrno)
    return gwin.werrno;

  // if window has shadow, redraw it
  if(shattr!=-1)
    wshadow((int)shattr);

  // return normally
  return gwin.werrno=W_NOERROR;
}


//  ------------------------------------------------------------------
//  Resizes the active window

#if 0
int wsize(int nerow, int necol) {

  int crow, ccol, mrow, mcol, shattr=-1;

  // check for active window
  if(!gwin.total)
    return gwin.werrno=W_NOACTIVE;

  // check for valid coordinates
  if((nerow<=gwin.active->srow+gwin.active->border) or (necol<=gwin.active->scol+gwin.active->border))
    return gwin.werrno=W_INVCOORD;

  // allocate memory block for new size
  word* newbuf = (word*)throw_xmalloc(((nerow-gwin.active->srow+1)*(necol-gwin.active->scol+1)+4)*sizeof(word));
  if(newbuf==NULL)
    return gwin.werrno=W_ALLOCERR;

  // if window has shadow, close it before the sizing
  if(gwin.active->wsbuf!=NULL) {
    shattr = gwin.active->wsattr;
    wshadoff();
  }

  // calculate maximum row and maximum column of scan
  if(nerow > gwin.active->erow)
    mrow = nerow;
  else
    mrow = gwin.active->erow;
  if(necol > gwin.active->ecol)
    mcol = necol;
  else
    mcol = gwin.active->ecol;

  // transfer needed contents of old buffer to new buffer
  word* src  = gwin.active->wbuf + 4;
  word* dest = newbuf + 4;
  for(crow=gwin.active->srow;crow<=mrow;crow++) {
    for(ccol=gwin.active->scol;ccol<=mcol;ccol++) {
      if((crow > nerow and ccol > gwin.active->ecol) or (crow > gwin.active->erow and ccol > necol))
        continue;
      if(crow > gwin.active->erow or ccol > gwin.active->ecol) {
        *dest++ = vgetw(crow,ccol);
        vputc(crow, ccol, gwin.active->wattr, gwin.fillch);
      }
      else {
        if(crow > nerow or ccol > necol)
          vputw(crow, ccol, *src++);
        else {
          if((crow==gwin.active->erow or ccol==gwin.active->ecol) and gwin.active->border)
            vputc(crow, ccol, gwin.active->wattr, gwin.fillch);
          *dest++ = *src++;
        }
      }
    }
  }

  // free old window's buffer
  throw_xrelease(gwin.active->wbuf);

  // update cursor coordinates if neccessary
  if((gwin.active->row>(nerow-gwin.active->border)) or (gwin.active->column>(necol-gwin.active->border)))
    wgotoxy(0,0);

  // update window's record
  gwin.active->wbuf = newbuf;
  gwin.active->erow = nerow;
  gwin.active->ecol = necol;

  // update window buffer's header
  newbuf[0] = (word)gwin.active->srow;
  newbuf[1] = (word)gwin.active->scol;
  newbuf[2] = (word)nerow;
  newbuf[3] = (word)necol;

  // redraw border if one exists
  if(gwin.active->border)
    wborder(gwin.active->btype);

  // if window has shadow, then re-open it
  if(shattr!=-1)
    wshadow((int)shattr);

  // return normally
  return gwin.werrno=W_NOERROR;
}
#endif


//  ------------------------------------------------------------------

