
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gwinput.h,v 1.1.1.1 1998/11/21 19:38:09 odinn Exp $
//  ------------------------------------------------------------------
//  Input form and field editing.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GWINPUT_H
#define __GWINPUT_H

#include <gstrgstr.h>
#include <gwindow.h>


//  ------------------------------------------------------------------

class winput {

public:

  class field {

  public:

    winput* form;

    int pos;
    int max_pos;
    int attr;
    bool fill_acs;
    char fill;
    int entry;

    char* buf;
    int buf_left_pos;
    int buf_end_pos;
    int buf_pos;
    int buf_len;

    char* destination;

    int id;
    int row;
    int column;
    int max_column;
    int conversion;
    int entry_mode;

    field* prev;
    field* next;

    field(winput* iform, int idnum, int wrow, int wcol, int field_width, char* dest, int dest_size, int cvt, int mode);
    ~field();

    int visible();

    void move_left();
    void move_right();

    int left();
    int right();
    int left_word();
    int right_word();
    int delete_left();
    int delete_char();
    int insert_char(char ch);
    int overwrite_char(char ch);
    int home();
    int end();

    void update();
    void activate();
    void deactivate();

    void restore();
    void commit();

    void convert();
    int adjust_mode();
    void conditional();

    void move_cursor();
    void draw(int from_pos=0);
  };

  field* first_field;
  field* current;

  enum {
    entry_new,
    entry_update,
    entry_conditional
  };

  enum {
    cvt_none,
    cvt_lowercase,
    cvt_uppercase,
    cvt_mixedcase
  };

  int idle_attr;
  int active_attr;
  int edit_attr;
  bool fill_acs;

  char idle_fill;
  char active_fill;
  char edit_fill;

  int insert_mode;

  bool done;
  bool dropped;

  int start_id;

  int cursor_was_hidden;

  gwindow &window;

  winput(gwindow &w);
  virtual ~winput();

  void setup(int i_attr, int a_attr, int e_attr, char fill, bool f_acs);

  void add_field(int idnum, int wrow, int wcol, int field_width, char* dest, int dest_size, int cvt=winput::cvt_none, int mode=winput::entry_conditional);

  int first(int id=0);
  int next();
  int previous();
  int last();

  int first_visible();
  int next_visible();
  int previous_visible();
  int last_visible();

  int move_to(int wrow, int wcol);

  field* field_at(int wrow, int wcol);
  field* get_field(int id);

  void draw_all();
  void show_cursor();

  void drop_form();
  void form_complete();
  void field_complete();
  void go_next_field();
  void go_previous_field();
  void go_up();
  void go_down();
  void go_left();
  void go_right();
  void delete_left();
  void delete_char();
  void go_field_begin();
  void go_field_end();
  void go_form_begin();
  void go_form_end();
  void toggle_insert();
  void restore_field();
  void delete_left_word();
  void delete_right_word();
  void clear_to_end_field();
  void clear_to_end_form();
  void go_left_word();
  void go_right_word();
  void enter_char(char ch);

  void prepare_form();
  void finish_form();

  int handle_key(uint key);

  // These are supposed to be overridden by the inheriting class
  virtual int handle_other_keys(uint& key);
  virtual int validate();
  virtual void before();
  virtual void after();
};


//  ------------------------------------------------------------------

class winput2 : public winput {

public:

  winput2(gwindow &w) : winput(w) { }

  int run(int helpcat);
};


//  ------------------------------------------------------------------

class gwinput {

public:

  class field {

  public:

    gwinput* form;

    int pos;
    int max_pos;
    int attr;
    bool fill_acs;
    char fill;
    int entry;

    char* buf;
    int buf_left_pos;
    int buf_end_pos;
    int buf_pos;
    int buf_len;

    gstring& destination;

    int id;
    int row;
    int column;
    int max_column;
    int conversion;
    int entry_mode;

    field* prev;
    field* next;

    field(gwinput* iform, int idnum, int wrow, int wcol, int field_width, gstring& dest, int dest_size, int cvt, int mode);
    ~field();

    bool visible();

    void move_left();
    void move_right();

    bool left();
    bool right();
    bool left_word();
    bool right_word();
    bool delete_left();
    bool delete_char();
    bool delete_word(bool left);
    bool insert_char(char ch);
    bool overwrite_char(char ch);
    bool home();
    bool end();

    void update();
    void activate();
    void deactivate();

    void restore();
    void commit();

    void convert();
    bool adjust_mode();
    void conditional();

    void move_cursor();
    void draw(int from_pos=0);
  };

  field* first_field;
  field* current;

  enum {
    entry_new,
    entry_update,
    entry_conditional,
    entry_noedit
  };

  enum {
    cvt_none,
    cvt_lowercase,
    cvt_uppercase,
    cvt_mixedcase
  };

  int idle_attr;
  int active_attr;
  int edit_attr;
  bool fill_acs;

  char idle_fill;
  char active_fill;
  char edit_fill;

  int insert_mode;

  int done;
  int dropped;

  int start_id;

  bool cursor_was_hidden;

  gwindow &window;

  gwinput(gwindow &w);
  virtual ~gwinput();

  void setup(int i_attr, int a_attr, int e_attr, char fill, bool fill_acs);

  void add_field(int idnum, int wrow, int wcol, int field_width, gstring& dest, int dest_size, int cvt=gwinput::cvt_none, int mode=gwinput::entry_conditional);

  bool first(int id=0);
  bool next();
  bool previous();
  bool last();

  bool first_visible();
  bool next_visible();
  bool previous_visible();
  bool last_visible();

  bool move_to(int wrow, int wcol);

  field* field_at(int wrow, int wcol);
  field* get_field(int id);

  void draw_all();
  void reload_all();
  void show_cursor();

  void drop_form();
  void form_complete();
  void field_complete();
  void go_next_field();
  void go_previous_field();
  void go_up();
  void go_down();
  void go_left();
  void go_right();
  void delete_left();
  void delete_char();
  void go_field_begin();
  void go_field_end();
  void go_form_begin();
  void go_form_end();
  void toggle_insert();
  void restore_field();
  void delete_left_word();
  void delete_right_word();
  void clear_to_end_field();
  void clear_to_end_form();
  void go_left_word();
  void go_right_word();
  void enter_char(char ch);

  void prepare_form();
  void finish_form();

  bool handle_key(uint key);

  // These are supposed to be overridden by the inheriting class
  virtual bool handle_other_keys(uint& key);
  virtual bool validate();
  virtual void before();
  virtual void after();
};


//  ------------------------------------------------------------------

class gwinput2 : public gwinput {

public:

  gwinput2(gwindow &w) : gwinput(w) { }

  bool run(int helpcat);
};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

