
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gwinput1.cpp,v 1.1.1.1 1998/11/21 19:38:08 odinn Exp $
//  ------------------------------------------------------------------
//  Input form and field editing.
//  ------------------------------------------------------------------


#include <string.h>
#include <gkbdcode.h>
#include <gmemdbg.h>
#include <gstrall.h>
#include <gwinall.h>
#include <gwinhelp.h>
#include <gwinput.h>


//  ------------------------------------------------------------------

winput::winput(gwindow &w) : window(w) {

  first_field = current = NULL;
  fill_acs = false;
  idle_attr = active_attr = edit_attr = 7;
  idle_fill = active_fill = edit_fill = ' ';
  insert_mode = true;
  done = dropped = false;
  start_id = 0;
}


//  ------------------------------------------------------------------

winput::~winput() {

  field* current = first_field;
  if(current) {
    do {
      field* junk = current;
      current = current->next;
      delete junk;
    } while(current);
  }
}


//  ------------------------------------------------------------------

void winput::setup(int i_attr, int a_attr, int e_attr, char fill, bool f_acs) {

  idle_attr = i_attr;
  active_attr = a_attr;
  edit_attr = e_attr;
  active_fill = edit_fill = fill;
  fill_acs = f_acs;
}


//  ------------------------------------------------------------------

int winput::validate() {

  return true;
}


//  ------------------------------------------------------------------

void winput::before() {

  current->activate();
}


//  ------------------------------------------------------------------

void winput::after() {

  current->deactivate();
}


//  ------------------------------------------------------------------

void winput::add_field(int idnum, int wrow, int wcol, int field_width, char* dest, int dest_size, int cvt, int mode) {

  field* fld = new field(this, idnum, wrow, wcol, field_width, dest, dest_size, cvt, mode);
  throw_new(fld);
  if(current) {
    current->next = fld;
    fld->prev = current;
    current = fld;
  }
  else {
    first_field = current = fld;
  }
}


//  ------------------------------------------------------------------

int winput::move_to(int wrow, int wcol) {

  field* f = field_at(wrow, wcol);
  if(f) {
    after();
    current = f;
    before();
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

winput::field* winput::field_at(int wrow, int wcol) {

  field* here = first_field;
  do {
    if(here->row == wrow)
      if((here->column <= wcol) and (wcol <= here->max_column))
        return here;
    here = here->next;
  } while(here);
  return NULL;
}


//  ------------------------------------------------------------------

winput::field* winput::get_field(int id) {

  field* here = first_field;
  do {
    if(here->id == id)
      return here;
    here = here->next;
  } while(here);
  return NULL;
}


//  ------------------------------------------------------------------

void winput::draw_all() {

  first();
  do {
    current->draw();
  } while(next());
}


//  ------------------------------------------------------------------

int winput::first_visible() {

  if(first()) {
    if(not current->visible())
      if(next_visible())
        return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::next_visible() {

  field* here = current;
  while(here->next) {
    here = here->next;
    if(here->visible()) {
      current = here;
      return true;
    }
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::previous_visible() {

  field* here = current;
  while(here->prev) {
    here = here->prev;
    if(here->visible()) {
      current = here;
      return true;
    }
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::last_visible() {

  if(last()) {
    if(not current->visible())
      if(previous_visible())
        return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::first(int id) {

  if(first_field) {
    current = first_field;
    if(id)
      while(current->id != id)
        next();
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::next() {

  if(current->next) {
    current = current->next;
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::previous() {

  if(current->prev) {
    current = current->prev;
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::last() {

  if(first_field) {
    current = first_field;
    while(next())
      ;
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

void winput::show_cursor() {

  if(insert_mode)
    vcursmall();
  else
    vcurlarge();
}


//  ------------------------------------------------------------------

void winput::drop_form() {

  after();
  done = true;
  dropped = true;
}


//  ------------------------------------------------------------------

void winput::form_complete() {

  after();
  done = true;
}


//  ------------------------------------------------------------------

void winput::field_complete() {

  if(validate()) {
    after();
    if(!next_visible()) {
      done = true;
      return;
    }
    before();
  }
}


//  ------------------------------------------------------------------

void winput::go_next_field() {

  after();
  if(!next_visible())
    first_visible();
  before();
}


//  ------------------------------------------------------------------

void winput::go_previous_field() {

  after();
  if(!previous_visible())
    last_visible();
  before();
}


//  ------------------------------------------------------------------

void winput::go_up() {

  move_to(current->row-1, current->column+current->pos);
}


//  ------------------------------------------------------------------

void winput::go_down() {

  move_to(current->row+1, current->column+current->pos);
}


//  ------------------------------------------------------------------

void winput::go_left() {

  current->left();
}


//  ------------------------------------------------------------------

void winput::go_right() {

  current->right();
}


//  ------------------------------------------------------------------

void winput::delete_left() {

  current->delete_left();
}


//  ------------------------------------------------------------------

void winput::delete_char() {

  current->delete_char();
}


//  ------------------------------------------------------------------

void winput::go_field_begin() {

  current->home();
}


//  ------------------------------------------------------------------

void winput::go_field_end() {

  current->end();
}


//  ------------------------------------------------------------------

void winput::go_form_begin() {

  after();
  first_visible();
  before();
  current->home();
}


//  ------------------------------------------------------------------

void winput::go_form_end() {

  after();
  last_visible();
  before();
  current->end();
}


//  ------------------------------------------------------------------

void winput::toggle_insert() {

  insert_mode ^= true;
  show_cursor();
}


//  ------------------------------------------------------------------

void winput::restore_field() {

  current->restore();
}


//  ------------------------------------------------------------------

void winput::delete_left_word() {

}


//  ------------------------------------------------------------------

void winput::delete_right_word() {

}


//  ------------------------------------------------------------------

void winput::clear_to_end_field() {

}


//  ------------------------------------------------------------------

void winput::clear_to_end_form() {

}


//  ------------------------------------------------------------------

void winput::go_left_word() {

  current->left_word();
}


//  ------------------------------------------------------------------

void winput::go_right_word() {

  current->right_word();
}


//  ------------------------------------------------------------------

void winput::enter_char(char ch) {

  if(ch) {
    if(insert_mode)
      current->insert_char(ch);
    else
      current->overwrite_char(ch);
  }
}


//  ------------------------------------------------------------------

void winput::prepare_form() {

  cursor_was_hidden = vcurhidden();
  draw_all();
  first(start_id);
  before();
  show_cursor();
}


//  ------------------------------------------------------------------

void winput::finish_form() {

  if(!dropped) {
    first();
    do {
      current->commit();
    } while(next());
  }

  if(cursor_was_hidden)
    vcurhide();
}


//  ------------------------------------------------------------------

int winput::handle_other_keys(uint&) {

  return false;
}


//  ------------------------------------------------------------------

int winput::handle_key(uint key) {

  switch(key) {
    case Key_Esc:           drop_form();                 break;
    case Key_C_Ent:         form_complete();             break;
    case Key_Ent:           field_complete();            break;
    case Key_Tab:           go_next_field();             break;
    case Key_S_Tab:         go_previous_field();         break;
    case Key_Up:            go_up();                     break;
    case Key_Dwn:           go_down();                   break;
    case Key_Lft:           go_left();                   break;
    case Key_Rgt:           go_right();                  break;
    case Key_BS:            delete_left();               break;
    case Key_Del:           delete_char();               break;
    case Key_Home:          go_field_begin();            break;
    case Key_End:           go_field_end();              break;
    case Key_C_Home:        go_form_begin();             break;
    case Key_C_End:         go_form_end();               break;
    case Key_Ins:           toggle_insert();             break;
    case Key_C_R:           restore_field();             break;
    case Key_C_BS:          delete_left_word();          break;
    case Key_C_T:           delete_right_word();         break;
    case Key_C_U:           clear_to_end_field();        break;
    case Key_C_Y:           clear_to_end_form();         break;
    case Key_C_Lft:         go_left_word();              break;
    case Key_C_Rgt:         go_right_word();             break;
    default:
      if(!handle_other_keys(key))
        enter_char(KCodAsc(key));
  }

  return !done;
}


//  ------------------------------------------------------------------

winput::field::field(winput* iform, int idnum, int wrow, int wcol, int field_width, char* dest, int dest_size, int cvt, int mode) {

  prev = next = NULL;
  pos = buf_pos = buf_left_pos = 0;

  form = iform;
  id = idnum;
  row = wrow;
  column = wcol;
  max_pos = field_width - 1;
  max_column = wcol + max_pos;
  buf_len = dest_size - 1;
  buf = new char[dest_size];
  throw_new(buf);
  conversion = cvt;
  destination = dest;
  entry = entry_mode = mode;
  attr = form->idle_attr;
  fill = form->idle_fill;
  fill_acs = form->fill_acs;
  if(entry == winput::entry_new)
    *buf = NUL;
  else
    strcpy(buf, dest);
  convert();
  buf_end_pos = strlen(buf);
}


//  ------------------------------------------------------------------

winput::field::~field() {

  delete[] buf;
}


//  ------------------------------------------------------------------

int winput::field::visible() {

  return max_pos != -1;
}


//  ------------------------------------------------------------------

void winput::field::convert() {

  switch(conversion) {
    case winput::cvt_lowercase:
      strlwr(buf);
      break;
    case winput::cvt_uppercase:
      strupr(buf);
      break;
    case winput::cvt_mixedcase:
      struplow(buf);
      break;
  }
}


//  ------------------------------------------------------------------

void winput::field::update() {

  buf_end_pos = strlen(buf);
  end();
}


//  ------------------------------------------------------------------

void winput::field::activate() {

  buf_end_pos = strlen(buf);
  entry = entry_mode;
  if(entry == winput::entry_conditional) {
    attr = form->active_attr;
    fill = form->active_fill;
    entry = winput::entry_update;   // cheat adjust_mode() in end()
    end();
    entry = winput::entry_conditional;
  }
  else {
    home();
  }
}


//  ------------------------------------------------------------------

void winput::field::deactivate() {

  fill = form->idle_fill;
  attr = form->idle_attr;
  draw();
}


//  ------------------------------------------------------------------

void winput::field::restore() {

  strcpy(buf, destination);
  convert();
  buf_end_pos = strlen(buf);
  activate();
}


//  ------------------------------------------------------------------

void winput::field::commit() {

  strcpy(destination, buf);
}


//  ------------------------------------------------------------------

void winput::field::move_cursor() {

  form->window.move_cursor(row, column+pos);
}


//  ------------------------------------------------------------------

void winput::field::draw(int from_pos) {

  if(visible())
    form->window.printns(row, column+from_pos, attr, buf+buf_left_pos+from_pos, 1+max_pos-from_pos, fill, attr | (fill_acs ? ACSET : 0));
}


//  ------------------------------------------------------------------

int winput::field::adjust_mode() {

  if(entry != winput::entry_update) {
    entry = winput::entry_update;
    attr = form->edit_attr;
    fill = form->edit_fill;
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

void winput::field::conditional() {

  if(entry == winput::entry_conditional) {
    pos = buf_pos = buf_left_pos = buf_end_pos = 0;
    *buf = NUL;
    adjust_mode();
    draw();
  }
}


//  ------------------------------------------------------------------

void winput::field::move_left() {

  buf_pos--;
  if(pos > 0) {
    pos--;
    move_cursor();
  }
  else {
    buf_left_pos--;
    draw();
  }
}


//  ------------------------------------------------------------------

void winput::field::move_right() {

  buf_pos++;
  if(pos < max_pos) {
    pos++;
    move_cursor();
  }
  else {
    buf_left_pos++;
    draw();
  }
}


//  ------------------------------------------------------------------

int winput::field::left() {

  if(adjust_mode())
    draw();

  if(buf_pos > 0) {
    move_left();
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::field::right() {

  if(adjust_mode())
    draw();

  if(buf_pos < buf_end_pos) {
    move_right();
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::field::left_word() {

  if(adjust_mode())
    draw();

  if(buf_pos > 0) {
    move_left();
    if(!isxalnum(buf[buf_pos])) {
      while(!isxalnum(buf[buf_pos]) and (buf_pos > 0))
        move_left();
      while(isxalnum(buf[buf_pos]) and (buf_pos > 0))
        move_left();
    }
    else {
      while(isxalnum(buf[buf_pos]) and (buf_pos > 0))
        move_left();
    }

    if(buf_pos != 0)
      move_right();
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::field::right_word() {

  if(adjust_mode())
    draw();

  if(buf_pos < buf_end_pos) {
    move_right();
    if(!isxalnum(buf[buf_pos])) {
      while(!isxalnum(buf[buf_pos]) and ((buf_pos+1) <= buf_end_pos))
        move_right();
    }
    else {
      while(isxalnum(buf[buf_pos]) and ((buf_pos+1) <= buf_end_pos))
        move_right();
      while(!isxalnum(buf[buf_pos]) and ((buf_pos+1) <= buf_end_pos))
        move_right();
    }
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::field::delete_left() {

  if(adjust_mode())
    draw();

  if(buf_pos > 0) {
    left();
    return delete_char();
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::field::delete_char() {

  if(adjust_mode())
    draw();

  if(buf_pos < buf_end_pos) {
    buf_end_pos--;
    memmove(buf+buf_pos, buf+buf_pos+1, buf_len-buf_pos);
    draw(pos);
    return true;
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::field::insert_char(char ch) {

  conditional();

  if(buf_end_pos < buf_len) {
    int len = buf_end_pos - buf_pos;
    memmove(buf+buf_pos+1, buf+buf_pos, len+1);
    buf_end_pos++;
    return overwrite_char(ch);
  }
  return false;
}


//  ------------------------------------------------------------------

int winput::field::overwrite_char(char ch) {

  conditional();

  switch(conversion) {
    case winput::cvt_lowercase:
      ch = (char)tolower(ch);
      break;
    case winput::cvt_uppercase:
      ch = (char)toupper(ch);
      break;
  }

  buf[buf_pos] = ch;
  if(buf_pos == buf_end_pos) {
    buf_end_pos++;
    buf[buf_end_pos] = NUL;
  }

  if(conversion == winput::cvt_mixedcase) {
    struplow(buf);
    draw();
  }
  else {
    draw(pos);
  }

  right();

  return true;
}


//  ------------------------------------------------------------------

int winput::field::home() {

  adjust_mode();

  pos = buf_pos = buf_left_pos = 0;
  draw();
  move_cursor();
  return true;
}


//  ------------------------------------------------------------------

int winput::field::end() {

  adjust_mode();

  buf_pos = buf_end_pos;
  if(buf_pos == buf_len)
    buf_pos--;
  pos = minimum_of_two(max_pos, buf_pos);
  buf_left_pos = buf_pos - pos;
  draw();
  move_cursor();
  return true;
}


//  ------------------------------------------------------------------

int winput2::run(int helpcat) {

  prepare_form();
  whelppcat(helpcat);

  while(handle_key(getxch()));

  whelpop();
  finish_form();

  return !dropped;
}


//  ------------------------------------------------------------------

#if defined(TESTMAIN)

void main() {

  enum {
    FIELD_A,
    FIELD_B,
    FIELD_C,
    FIELD_D,
    FIELD_E
  };

  gwindow w;

  w.open(0, 0, gvid.numrows-1, gvid.numcols-1, 5, YELLOW|_LGREY, BLUE|_LGREY);

  w.horizontal_line(0, 0, w.width(), 0, YELLOW|_LGREY);
  w.horizontal_line(5, 0, w.width(), 0, YELLOW|_LGREY);

  w.prints(1, 0, BLUE|_LGREY, " Msg  : 117 of 123");
  w.prints(2, 0, BLUE|_LGREY, " From : ");
  w.prints(3, 0, BLUE|_LGREY, " To   : ");
  w.prints(4, 0, BLUE|_LGREY, " Subj : ");

  winput* i = new winput(w);

  char str_a[128] = { "Odinn Sorensen" };
  char str_b[128] = { "2:236/77" };
  char str_c[128] = { "Who Knows" };
  char str_d[128] = { "2:236/77.123" };
  char str_e[128] = { "Testing my new input class" };

  i->idle_attr   =  BLUE|_LGREY;
  i->active_attr = BLACK|_LGREY;
  i->edit_attr   =   RED|_LGREY;

  i->active_fill = '\xB0';
  i->edit_fill   = '\xB0';
  i->fill_acs    = true;

  i->add_field(FIELD_A, 2,  8, 35, str_a, 127);
  i->add_field(FIELD_B, 2, 44, 16, str_b, 127);
  i->add_field(FIELD_C, 3,  8, 35, str_c, 127);
  i->add_field(FIELD_D, 3, 44, 16, str_d, 127);
  i->add_field(FIELD_E, 4,  8, 71, str_e, 127);

  i->start_id = FIELD_E;

  i->prepare_form();

  KeyT key = 0;
  do {

    {
      // DEBUG
      char buf[256];
      sprintf(buf, "bp:%i blp:%i", i->current->buf_pos, i->current->buf_left_pos);
      wprintfs(gvid.numrows-1, 0, WHITE|_BLUE, " %-*s ", gvid.numcols-2, buf);
    }

    key = getxch();
  } while(i->handle_key(key));

  i->finish_form();

  delete i;

  window.close();

  gvid.restore_cursor();
}

#endif


//  ------------------------------------------------------------------

