
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Library General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Library General Public License for more details.
//
//  You should have received a copy of the GNU Library General Public
//  License along with this library; if not, write to the Free
//  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gs_fech2.h,v 1.1.1.1 1998/11/21 19:38:10 odinn Exp $
//  ------------------------------------------------------------------


#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


/********************************************************/
/* Structures of FastEcho 1.10, File: FASTECHO.CFG     */
/* (c)1992 by Tobias Burchhardt                         */
/********************************************************/

/********************************************************/
/* FASTECHO.CFG = <CONFIG>                              */
/*                + <CONFIG.offset>                     */
/*                + <CONFIG.NodeCnt * Node>             */
/*                + <CONFIG.AreaCnt * Area>             */
/********************************************************/

#define REVISION        4       /* current revision     */

/********************************************************/
/* CONFIG.flags                                         */
/********************************************************/
#define RETEAR          0x0001
#define AUTOCREATE	0x0002
#define KILLEMPTY	0x0004
#define KILLDUPES	0x0008
#define FRODO	        0x0010
#define DBRIDGE	 	0x0020
#define BINKLEY		0x0040
#define SWAP_XMS	0x0100
#define SWAP_EMS	0x0200
#define SWAP_DISK	0x0400

/********************************************************/
/* CONFIG.CC.what                                       */
/********************************************************/
#define FROM            1
#define TO		2
#define SUBJECT		3


/********************************************************/
/* Area.board (1-200 = QBBS)                            */
/********************************************************/
#define NO_BOARD        0x4000  /* everything but QBBS  */
#define DELETED         0x8000  /* never written        */

/********************************************************/
/* Area.flags.type                                      */
/********************************************************/
#define QBBS            0
#define FIDO		1
#define SQUISH		2
#define PT_BOARD	7

/********************************************************/
/* Area.type                                            */
/********************************************************/
#define ECHOMAIL        0
#define NETMAIL         1
#define LOCAL           2
#define BADMAILBOARD	3
#define DUPEBOARD	4


/********************************************************/
/* Node.flags                                           */
/********************************************************/
#define DDDD            0x0001
#define TOSSCAN		0x0002
#define DIRECT		0x0004
#define CRASH		0x0008
#define HOLD		0x0080
#define DBUG		0x0100
#define ARCMAIL060	0x0200
#define PASSIVE		0x0400
#define PACKER		0x7000

#define PACKS(x) (short)(((x)&PACKER)>>12)        /* Index of used Packer */


/********************************************************/
/* Types                                                */
/********************************************************/
typedef unsigned char byte,uchar;
typedef unsigned short word,uint;
typedef unsigned long dword,ulong;


/********************************************************/
/* Structures                                           */
/********************************************************/

typedef struct
{
 unsigned short zone,net,node,point;
} Address;  // Replaced with ftn_addr in this headerfile

#define _MAXPATH 56

typedef struct CONFIGURATION2
{
 short revision;
 ulong flags;
 short  NodeCnt,AreaCnt,resv1;
 char NetMPath[_MAXPATH],
      MsgBase[_MAXPATH],
      InBound[_MAXPATH],
      OutBound[_MAXPATH],
      Unpacker[_MAXPATH],
      LogFile[_MAXPATH],
      OriginLine[8][_MAXPATH],
      StatFile[_MAXPATH],
      SwapPath[_MAXPATH];
 char RetearFrom[5][40],
      RetearTo[5][40];
 struct
 {
  char tag[4];
  char name[_MAXPATH];
 } Packer[8];
 struct
 {
  uchar what;
  char object[31];
  unsigned char board,len;
 } CC[10];
 short security,def_days,def_messages;
 struct
 {
  ftn_addr main;
  char domain[28];
  short pointnet;
  ulong flags;
 } akas[11];
 short autorenum;
 long resv2;
 unsigned short duperecords;
 unsigned short msglen;
 short resv3;
 char TempPath[_MAXPATH];
 short graphics;
 char AreaFixHelp[_MAXPATH];
 char resv4[1918];
 unsigned long offset;
} CONFIG2;

typedef struct
{
 char name[41],type;
 short board,messages;
 struct
 {
  unsigned char  origin : 3;
  unsigned char  group  : 5;
  unsigned char  type   : 3;
  unsigned char  umlaut : 1;
  unsigned char  aka    : 4;
 } flags;
 short days;
 short conference;                /* 0 - 991, see Node.areas */
 short read_sec,write_sec;
 short resv;
 char path[_MAXPATH];
 char desc[42];
 char resv2[4];
} FeArea2;

typedef struct
{
 ftn_addr addr;
 short aka;
 unsigned short flags;
 unsigned short sec_level;
 char arcmailpw[10];
 ftn_addr routes[15];
 uchar areas[124];              /* Bit-field with 992 bits, Byte 0/Bit 7 */
                                /* is conference 0, etc.                 */

 ulong groups;                  /* Bit-field, Bit 31 = 'A', 30 = 'B' etc.*/
 char areafixpw[10];
} FeNode2;


#if 0

/********************************************************/
/* Routines to access Node.areas, Node.groups           */
/********************************************************/

unsigned short AddBam(uchar *bam,unsigned short nr)
{
uchar c=(1<<(7-(nr&7))),d;

 d=bam[nr/8]&c;
 bam[nr/8]|=c;
 return(d);
}

void FreeBam(uchar *bam,unsigned short nr)
{
 bam[nr/8]&=~(1<<(7-(nr&7)));
}

short GetBam(uchar *bam,unsigned short nr)
{
 if(bam[nr/8]&(1<<(7-(nr&7)))) return(true);
 return(false);
}

#define IsActive(nr,area)      GetBam(Node[nr].areas,area)
#define SetActive(nr,area)     AddBam(Node[nr].areas,area)
#define SetDeActive(nr,area)   FreeBam(Node[nr].areas,area)

#endif

