
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Library General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Library General Public License for more details.
//
//  You should have received a copy of the GNU Library General Public
//  License along with this library; if not, write to the Free
//  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gs_fm098.h,v 1.1.1.1 1998/11/21 19:38:11 odinn Exp $
//  ------------------------------------------------------------------

/*
   FMSTRUCT.H

   File structures for FMail 0.98
   Copyright (C) 1994 Folkert J. Wijnstra. All rights reserved.

   All information in this document is subject to change at any time
   without prior notice!

   Strings are NUL terminated arrays of char type.
   Path names are \ terminated.
*/


typedef unsigned short ushort;
typedef unsigned char bit;


/* ********** General structures ********** */

typedef struct
{
   char     programName[46];
   ushort   memRequired;      } archiverInfo;

typedef char pathType[48];

typedef struct
{
   ushort   zone;
   ushort   net;
   ushort   node;
   ushort   point; } nodeNumType;

typedef struct
{
   nodeNumType nodeNum;
   ushort      fakeNet; } nodeFakeType;


/* ********** File header structure ********** */

#define DATATYPE_CF    0x0102
#define DATATYPE_NO    0x0202
#define DATATYPE_AD    0x0401
#define DATATYPE_AE    0x0402

typedef struct
{
   char            versionString[32]; /* Always starts with 'FMail' */
   ushort          revNumber;         /* Is now 0x0100 */
   ushort          dataType;          /* See #defines above */
   ushort          headerSize;
   long            creationDate;
   long            lastModified;
   ushort          totalRecords;
   ushort          recordSize;
} headerType;


/* ********** FMAIL.CFG ********** */

#define MAX_AKAS   11
#define MAX_USERS  16
#define MAX_UPLREQ 16
#define MAX_MATCH  16

#define LOG_INBOUND   0x0001
#define LOG_OUTBOUND  0x0002
#define LOG_PKTINFO   0x0004
#define LOG_XPKTINFO  0x0008
#define LOG_UNEXPPWD  0x0010
#define LOG_SENTRCVD  0x0020
#define LOG_STATS     0x0040
#define LOG_MSGBASE   0x8000
#define LOG_ECHOEXP   0x8000
#define LOG_NETIMP    0x8000
#define LOG_NETEXP    0x8000
#define LOG_WARNINGS  0x8000
#define LOG_ALWAYS    0x8000
#define LOG_DEBUG     0x8000

typedef nodeFakeType akaListType[MAX_AKAS];

typedef struct
{
   bit      useEMS       :  1; /* BIT 0 */
   bit      checkBreak   :  1; /* BIT 1 */
   bit      swap         :  1; /* BIT 2 */
   bit      swapEMS      :  1; /* BIT 3 */
   bit      swapXMS      :  1; /* BIT 4 */
   bit                   :  1;
   bit      monochrome   :  1; /* BIT 6 */
   bit      commentFFD   :  1; /* BIT 7 */
   bit      PTAreasBBS   :  1; /* BIT 8 */
   bit      commentFRA   :  1; /* BIT 9 */
   bit                   :  1; /* BIT 10 */
   bit      incBDRRA     :  1; /* BIT 11 */
   bit                   :  1; /* BIT 12 */
   bit                   :  1;
   bit                   :  1;
   bit      RA2          :  1; /* BIT 15 */  } genOptionsType;

typedef struct
{
   bit      removeNetKludges : 1; /* Bit 0 */
   bit                       : 1;
   bit      checkPktDest     : 1; /* Bit 2 */
   bit                       : 1;
   bit      createSema       : 1; /* Bit 4 */
   bit                       : 1;
   bit                       : 1;
   bit                       : 1;
   bit      dupDetection     : 1; /* Bit 8 */
   bit      ignoreMSGID      : 1; /* Bit 9 */
   bit      ARCmail060       : 1; /* Bit 10 */
   bit      extNames         : 1; /* Bit 11 */
   bit      persNetmail      : 1; /* Bit 12 */
   bit      privateImport    : 1; /* Bit 13 */
   bit      keepExpNetmail   : 1; /* Bit 14 */
   bit      killEmptyNetmail : 1; /* Bit 15 */  } mailOptionsType;

typedef struct
{
   bit      sortNew      : 1; /* bit  0   */
   bit      sortSubject  : 1; /* bit  1   */
   bit      updateChains : 1; /* bit  2   */
   bit      reTear       : 1; /* bit  3   */
   bit                   : 1; /* bit  4   */
   bit                   : 1; /* bit  5   */
   bit      removeRe     : 1; /* bit  6   */
   bit      removeLfSr   : 1; /* bit  7   */
   bit      scanAlways   : 1; /* bit  8   */
   bit      scanUpdate   : 1; /* bit  9   */
   bit      multiLine    : 1; /* bit 10   */
   bit                   : 1; /* bit 11   */
   bit      quickToss    : 1; /* bit 12   */
   bit                   : 1; /* bit 13-14 */
   bit                   : 1; /* bit 13-14 */
   bit      sysopImport  : 1; /* bit 15   */ } mbOptionsType;

typedef struct
{
   bit      keepRequest  : 1; /* Bit  0 */
   bit      keepReceipt  : 1; /* Bit  1 */
   bit                   : 1; /* Bit 2-3 */
   bit                   : 1; /* Bit 2-3 */
   bit      autoDiscArea : 1; /* Bit  4 */
   bit                   : 1; /* Bit 5-8 */
   bit                   : 1; /* Bit 5-8 */
   bit                   : 1; /* Bit 5-8 */
   bit                   : 1; /* Bit 5-8 */
   bit      allowAddAll  : 1; /* Bit  9 */
   bit      allowActive  : 1; /* Bit 10 */
   bit                   : 1; /* Bit 11 */
   bit      allowPassword: 1; /* Bit 12 */
   bit      allowPktPwd  : 1; /* Bit 13 */
   bit      allowNotify  : 1; /* Bit 14 */
   bit      allowCompr   : 1; /* Bit 15 */  } mgrOptionsType;
#if 0
typedef struct
{
   bit      active      : 1; /* Bit  0 */
   bit      tinySeenBy  : 1; /* Bit  1 */
   bit      security    : 1; /* Bit  2 */
   bit                  : 1; /* Bit  3 */
   bit      private     : 1; /* Bit  4 */
   bit      impSeenBy   : 1; /* Bit  5 */
   bit      checkSeenBy : 1; /* Bit  6 */
   bit                  : 1; /* Bit  7 */
   bit      local       : 1; /* Bit  8 */
   bit                  : 1; /* Bit  9 */
   bit      passThrough : 1; /* Bit 10 */
   bit                  : 1; /* Bit 11-13 */
   bit                  : 1; /* Bit 11-13 */
   bit                  : 1; /* Bit 11-13 */
   bit      arrivalDate : 1; /* Bit 14 */
   bit      sysopRead   : 1; /* Bit 15 */     } defaultOptionsType;
#endif
typedef struct
{
   bit      addPlusPrefix :  1; /* BIT 0 */
   bit                    :  1;
   bit                    :  1;
   bit                    :  1;
   bit      unconditional :  1; /* BIT 4 */
   bit                    : 11;    } uplOptType;

typedef struct
{
   char  userName[36];
   char  reserved[28];
} userType;

typedef struct
{
   nodeNumType node;
   char        program[9];
   char        password[17];
   char        fileName[13];
   char        fileType;
   long        groups;
   char        originAka;
   uplOptType  options;
   char        reserved[9];  } uplinkReqType;

typedef struct
{
   ushort   valid;
   ushort   zone;
   ushort   net;
   ushort   node;    } akaMatchNodeType;

typedef struct
{
   akaMatchNodeType amNode;
   ushort           aka;    } akaMatchType;

typedef struct
{
   char            versionMajor;
   char            versionMinor;
   long            creationDate;
   unsigned long   key;
   unsigned long   reservedKey;
   unsigned long   relKey1;
   unsigned long   relKey2;
   char            reserved1[22];
   mgrOptionsType  mgrOptions;
   akaListType     akaList;
   nodeFakeType    reservedAka[16-MAX_AKAS];
   ushort          netmailBoard[MAX_AKAS];
   ushort          reservedNet[16-MAX_AKAS];
   genOptionsType  genOptions;
   mbOptionsType   mbOptions;
   mailOptionsType mailOptions;
   ushort          maxPktSize;
   ushort          reserved2;
   ushort          mailer;
   ushort          bbsProgram;
   ushort          maxBundleSize;
   ushort          extraHandles; /* 0-235 */
   ushort          autoRenumber;
   ushort          bufSize;
   ushort          ftBufSize;
   ushort          allowedNumNetmail;
   ushort          logInfo;
   ushort          logStyle;
   char            reserved3[68];
   ushort          colorSet;
   char            sysopName[36];
   ushort          defaultArc;
   char            reserved4[24];
   char            tearType;
   char            tearLine[25];
   pathType        summaryLogName;
   ushort          recBoard;
   ushort          badBoard;
   ushort          dupBoard;
   char            topic1[16];
   char            topic2[16];
   pathType        bbsPath;
   pathType        netPath;
   pathType        sentPath;
   pathType        rcvdPath;
   pathType        inPath;
   pathType        outPath;
   pathType        securePath;
   pathType        logName;
   pathType        swapPath;
   pathType        semaphorePath;
   pathType        pmailPath;
   pathType        areaMgrLogName;
   pathType        autoRAPath;
   pathType        autoFolderFdPath;
   pathType        autoAreasBBSPath;
   pathType        autoGoldEdAreasPath;
   archiverInfo    unArc;
   archiverInfo    unZip;
   archiverInfo    unLzh;
   archiverInfo    unPak;
   archiverInfo    unZoo;
   archiverInfo    unArj;
   archiverInfo    unSqz;
   archiverInfo    GUS;
   archiverInfo    arc;
   archiverInfo    zip;
   archiverInfo    lzh;
   archiverInfo    pak;
   archiverInfo    zoo;
   archiverInfo    arj;
   archiverInfo    sqz;
   archiverInfo    customArc;
   char            reserved5[105];
   char            groupsQBBS[11];
   ushort          templateSecQBBS[11];
   char            templateFlagsQBBS[11][4];
   char            attr2RA[11];
   char            aliasesQBBS[11];
   ushort          groupRA[11];
   ushort          altGroupRA[11][3];
   char            qwkNameSBBS[11][13];
   ushort          minAgeSBBS[11];
   ushort          daysRcvdAKA[11];
   char            replyStatSBBS[11];
   ushort          attrSBBS[11];
   char            groupDescr[26][27];
   char            reserved6[9];
   char            msgKindsRA[11];
   char            attrRA[11];
   ushort          readSecRA[11];
   char            readFlagsRA[11][4];
   ushort          writeSecRA[11];
   char            writeFlagsRA[11][4];
   ushort          sysopSecRA[11];
   char            sysopFlagsRA[11][4];
   ushort          daysAKA[11];
   ushort          msgsAKA[11];
   char            descrAKA[11][51];
   userType        users[MAX_USERS];
   akaMatchType    akaMatch[MAX_MATCH];
   char            reserved7[2048-10*MAX_MATCH];
   uplinkReqType   uplinkReq[MAX_UPLREQ];   } configType;



/* ********** FMAIL.AR ********** */

#define MAX_AREAS    512
#define MAX_FORWARD   64

#define MB_PATH_LEN_OLD   19
#define MB_PATH_LEN       61
#define ECHONAME_LEN_090  25
#define ECHONAME_LEN      51
#define COMMENT_LEN       51
#define ORGLINE_LEN       59

typedef char areaNameType[ECHONAME_LEN];

typedef struct
{
   bit      active      : 1; /* Bit  0 */
   bit      tinySeenBy  : 1; /* Bit  1 */
   bit      security    : 1; /* Bit  2 */
   bit                  : 1; /* Bit  3 */
   bit      Private     : 1; /* Bit  4 */
   bit      impSeenBy   : 1; /* Bit  5 */
   bit      checkSeenBy : 1; /* Bit  6 */
   bit                  : 1; /* Bit  7 */
   bit      local       : 1; /* Bit  8 */
   bit      disconnected: 1; /* Bit  9 */
   bit      reserved    : 1; /* Bit 10 */
   bit                  : 1; /* Bit 11-13 */
   bit                  : 1; /* Bit 11-13 */
   bit                  : 1; /* Bit 11-13 */
   bit      arrivalDate : 1; /* Bit 14 */
   bit      sysopRead   : 1; /* Bit 15 */   } areaOptionsType;

typedef struct      /* OLD!!! */
{
   char            zero; /* Should always be zero */
   char            msgBasePath[MB_PATH_LEN_OLD];
   char            groupsQBBS;
   char            flagsTemplateQBBS[4];
   char            comment[COMMENT_LEN];
   long            group;
   ushort          board;
   ushort          address;
   ushort          alsoSeenBy;
   ushort          groupRA;
   ushort          altGroupRA[3];
   areaOptionsType options;
   ushort          outStatus;
   ushort          days;
   ushort          msgs;
   ushort          daysRcvd;
   ushort          templateSecQBBS;
   ushort          readSecRA;
   char            flagsRdRA[4];
   ushort          writeSecRA;
   char            flagsWrRA[4];
   ushort          sysopSecRA;
   char            flagsSysRA[4];
   char            attrRA;
   char            msgKindsRA;
   ushort          attrSBBS;
   char            replyStatSBBS;
   areaNameType    areaName;
   char            qwkName[13];
   ushort          minAgeSBBS;
   char            attr2RA;
   char            aliasesQBBS;
   char            originLine[ORGLINE_LEN];
   nodeNumType     export[MAX_FORWARD];  } rawEchoTypeOld;

typedef struct
{
   ushort          signature;
   ushort          reserved;
   areaNameType    areaName;
   char            comment[COMMENT_LEN];
   areaOptionsType options;
   ushort          reserved1;
   char            msgBaseType;
   char            msgBasePath[MB_PATH_LEN];
   ushort          board;
   char            originLine[ORGLINE_LEN];
   ushort          address;
   long            group;
   ushort          alsoSeenBy;
   ushort          msgs;
   ushort          days;
   ushort          daysRcvd;

   nodeNumType     export[MAX_FORWARD];

   ushort          readSecRA;
   char            flagsRdNotRA[4];
   char            flagsRdRA[4];
   ushort          writeSecRA;
   char            flagsWrNotRA[4];
   char            flagsWrRA[4];
   ushort          sysopSecRA;
   char            flagsSysRA[4];
   char            flagsSysNotRA[4];
   ushort          templateSecQBBS;
   char            flagsTemplateQBBS[4];
   char            flagsReserved[4];

   char            attrRA;
   char            attr2RA;
   ushort          groupRA;
   ushort          altGroupRA[3];
   char            msgKindsRA;
   char            qwkName[13];
   ushort          minAgeSBBS;
   ushort          attrSBBS;
   char            replyStatSBBS;
   char            groupsQBBS;
   char            aliasesQBBS;      } rawEchoType;



/* ********** FMAIL.NOD ********** */

#define MAX_NODEMGR      256
#define PKT_TYPE_2PLUS   1
#define CAPABILITY       PKT_TYPE_2PLUS

typedef struct
{
   bit      fixDate     : 1; /* Bit 0 */
   bit      tinySeenBy  : 1; /* Bit 1 */
   bit                  : 1; /* Bit 2 */
   bit      ignorePwd   : 1; /* Bit 3 */
   bit      active      : 1; /* Bit 4 */
   bit                  : 1;
   bit                  : 1;
   bit      packNetmail : 1; /* Bit 7 */
   bit                  : 1; /* Bit 8 */
   bit                  : 1;
   bit                  : 1;
   bit                  : 1;
   bit      forwardReq  : 1; /* Bit 12 */
   bit      remMaint    : 1; /* Bit 13 */
   bit      allowRescan : 1; /* Bit 14 */
   bit      notify      : 1;   } nodeOptionsType;

typedef struct /* OLD !!! */
{
   nodeNumType     node;
   char            reserved1[2];
   ushort          capability;
   ushort          archiver;
   nodeOptionsType options;
   unsigned long   groups;
   ushort          outStatus;
   char            reserved2[32];
   char            password[18];
   char            packetPwd[10];
   char            reserved[2];
   nodeNumType     viaNode;
   char            sysopName[36];   } nodeInfoTypeOld;

/* See Area File for header structure */

typedef struct
{
   ushort          signature; /* contains "ND" */
   ushort          reserved;
   nodeNumType     node;
   nodeNumType     viaNode;
   ushort          capability;
   nodeOptionsType options;
   ushort          archiver;
   unsigned long   groups;
   ushort          outStatus;
   char            sysopName[36];
   char            password[18];
   char            packetPwd[10]; } nodeInfoType;



/* ********** FMAIL.PCK ********** */

#define PACK_STR_SIZE 64
#define MAX_PACK      64

typedef char packEntryType[PACK_STR_SIZE];
typedef packEntryType packType[MAX_PACK];



/* ********** FMAIL.BDE ********** */

#define MAX_BAD_ECHOS 50

typedef struct
{
   areaNameType badEchoName;
   nodeNumType  srcNode;
   int          destAka;      } badEchoType;

