
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Library General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Library General Public License for more details.
//
//  You should have received a copy of the GNU Library General Public
//  License along with this library; if not, write to the Free
//  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gs_fm10g.h,v 1.1.1.1 1998/11/21 19:38:11 odinn Exp $
//  ------------------------------------------------------------------


#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif

/*
   FMSTRUCT.H

   File structures for FMail 1.0g
   Copyright (C) 1994 Folkert J. Wijnstra. All rights reserved.

   All information in this document is subject to change at any time
   without prior notice!

   Strings are NUL terminated arrays of char type.
   Path names always end on a \ character (followed by NUL).
*/


/**** Modify the type definitions below if necessary for your compiler ****/


typedef unsigned short u16;
typedef signed short   s16;
typedef unsigned long  u32;
typedef signed long    s32;
typedef unsigned char  bit;


/* ********** General structures ********** */

typedef struct
{  char  programName[46];
   u16   memRequired;      } archiverInfo;

typedef char pathType[48];

typedef struct
{  u16   zone;
   u16   net;
   u16   node;
   u16   point; } nodeNumType;

typedef struct
{  nodeNumType nodeNum;
   u16         fakeNet; } nodeFakeType;


/* ********** File header structure ********** */

#define DATATYPE_CF    0x0102
#define DATATYPE_NO    0x0202
#define DATATYPE_AD    0x0401
#define DATATYPE_AE    0x0402

typedef struct
{  char  versionString[32]; /* Always starts with 'FMail' */
   u16   revNumber;         /* Is now 0x0100 */
   u16   dataType;          /* See #defines above */
   u16   headerSize;
   s32   creationDate;
   s32   lastModified;
   u16   totalRecords;
   u16   recordSize;
} headerType;


/* The structure below is used by the Areas File and (only partly)
   by the Config File */

typedef struct
{  bit      active      : 1; /* Bit  0 */
   bit      tinySeenBy  : 1; /* Bit  1 */
   bit      security    : 1; /* Bit  2 */
   bit                  : 1; /* Bit  3 */
   bit      allowPrivate: 1; /* Bit  4 */
   bit      impSeenBy   : 1; /* Bit  5 */
   bit      checkSeenBy : 1; /* Bit  6 */
   bit                  : 1; /* Bit  7 */
   bit      local       : 1; /* Bit  8 */
   bit      disconnected: 1; /* Bit  9 */
   bit      reserved    : 1; /* Bit 10 */
   bit      allowAreafix: 1; /* Bit 11 */
   bit                  : 1; /* Bit 12-13 */
   bit                  : 1; /* Bit 12-13 */
   bit      arrivalDate : 1; /* Bit 14 */
   bit      sysopRead   : 1; /* Bit 15 */   } areaOptionsType;


/* ********** FMAIL.CFG ********** */

#define MAX_AKAS      16
#define MAX_NETAKAS   11
#define MAX_USERS     16
#define MAX_UPLREQ    16
#define MAX_MATCH     16

#define LOG_INBOUND   0x0001
#define LOG_OUTBOUND  0x0002
#define LOG_PKTINFO   0x0004
#define LOG_XPKTINFO  0x0008
#define LOG_UNEXPPWD  0x0010
#define LOG_SENTRCVD  0x0020
#define LOG_STATS     0x0040
#define LOG_PACK      0x0080
#define LOG_MSGBASE   0x0100
#define LOG_ECHOEXP   0x0200
#define LOG_NETIMP    0x0400
#define LOG_NETEXP    0x0800
#define LOG_ALWAYS    0x8000
#define LOG_DEBUG     0x8000

typedef nodeFakeType akaListType[MAX_AKAS];

typedef struct
{
   bit      useEMS       : 1; /* BIT 0 */
   bit      checkBreak   : 1; /* BIT 1 */
   bit      swap         : 1; /* BIT 2 */
   bit      swapEMS      : 1; /* BIT 3 */
   bit      swapXMS      : 1; /* BIT 4 */
   bit                   : 1;
   bit      monochrome   : 1; /* BIT 6 */
   bit      commentFFD   : 1; /* BIT 7 */
   bit      PTAreasBBS   : 1; /* BIT 8 */
   bit      commentFRA   : 1; /* BIT 9 */
   bit                   : 1; /* BIT 10 */
   bit      incBDRRA     : 1; /* BIT 11 */
   bit                   : 1; /* BIT 12 */
   bit                   : 1;
   bit                   : 1;
   bit      _RA2         : 1; /* BIT 15 */  } genOptionsType;

typedef struct
{
   bit      removeNetKludges : 1; /* Bit 0 */
   bit                       : 1;
   bit      checkPktDest     : 1; /* Bit 2 */
   bit                       : 1;
   bit      createSema       : 1; /* Bit 4 */
   bit                       : 1;
   bit      warnNewMail      : 1; /* bit 6 */
   bit      killBadFAtt      : 1; /* Bit 7 */
   bit      dupDetection     : 1; /* Bit 8 */
   bit      ignoreMSGID      : 1; /* Bit 9 */
   bit      ARCmail060       : 1; /* Bit 10 */
   bit      extNames         : 1; /* Bit 11 */
   bit      persNetmail      : 1; /* Bit 12 */
   bit      privateImport    : 1; /* Bit 13 */
   bit      keepExpNetmail   : 1; /* Bit 14 */
   bit      killEmptyNetmail : 1; /* Bit 15 */  } mailOptionsType;

typedef struct
{
   bit      sortNew      : 1; /* bit  0   */
   bit      sortSubject  : 1; /* bit  1   */
   bit      updateChains : 1; /* bit  2   */
   bit      reTear       : 1; /* bit  3   */
   bit                   : 1; /* bit  4   */
   bit                   : 1; /* bit  5   */
   bit      removeRe     : 1; /* bit  6   */
   bit      removeLfSr   : 1; /* bit  7   */
   bit      scanAlways   : 1; /* bit  8   */
   bit      scanUpdate   : 1; /* bit  9   */
   bit      multiLine    : 1; /* bit 10   */
   bit                   : 1; /* bit 11   */
   bit      quickToss    : 1; /* bit 12   */
   bit                   : 1; /* bit 13   */
   bit                   : 1; /* bit 14   */
   bit      sysopImport  : 1; /* bit 15   */ } mbOptionsType;

typedef struct
{
   bit      keepRequest  : 1; /* Bit  0 */
   bit      keepReceipt  : 1; /* Bit  1 */
   bit                   : 1; /* Bit  2 */
   bit                   : 1; /* Bit  3 */
   bit      autoDiscArea : 1; /* Bit  4 */
   bit      autoDiscDel  : 1; /* Bit  5 has temp. no effect, rec is always deleted */
   bit                   : 1; /* Bit  6 */
   bit                   : 1; /* Bit  7 */
   bit                   : 1; /* Bit  8 */
   bit      allowAddAll  : 1; /* Bit  9 */
   bit      allowActive  : 1; /* Bit 10 */
   bit                   : 1; /* Bit 11 */
   bit      allowPassword: 1; /* Bit 12 */
   bit      allowPktPwd  : 1; /* Bit 13 */
   bit      allowNotify  : 1; /* Bit 14 */
   bit      allowCompr   : 1; /* Bit 15 */ } mgrOptionsType;
#if 0
typedef struct
{
   bit      active      : 1; /* Bit  0 */
   bit      tinySeenBy  : 1; /* Bit  1 */
   bit      security    : 1; /* Bit  2 */
   bit                  : 1; /* Bit  3 */
   bit      allowPrivate: 1; /* Bit  4 */
   bit      impSeenBy   : 1; /* Bit  5 */
   bit      checkSeenBy : 1; /* Bit  6 */
   bit                  : 1; /* Bit  7 */
   bit      local       : 1; /* Bit  8 */
   bit                  : 1; /* Bit  9 */
   bit      passThrough : 1; /* Bit 10 */
   bit                  : 1; /* Bit 11-13 */
   bit                  : 1; /* Bit 11-13 */
   bit                  : 1; /* Bit 11-13 */
   bit      arrivalDate : 1; /* Bit 14 */
   bit      sysopRead   : 1; /* Bit 15 */     } defaultOptionsType;
#endif
typedef struct
{
   bit      addPlusPrefix :  1; /* BIT 0 */
   bit                    :  1;
   bit                    :  1;
   bit                    :  1;
   bit      unconditional :  1; /* BIT 4 */
   u16                    : 11;    } uplOptType;

typedef struct
{
   char  userName[36];
   char  reserved[28];
} userType;

typedef struct
{
   nodeNumType node;
   char        program[9];
   char        password[17];
   char        fileName[13];
   char        fileType;
   u32         groups;
   char        originAka;
   uplOptType  options;
   char        reserved[9];  } uplinkReqType;

typedef struct
{  u16   valid;
   u16   zone;
   u16   net;
   u16   node;    } akaMatchNodeType;

typedef struct
{
   akaMatchNodeType amNode;
   u16              aka;    } akaMatchType;

/* ATTENTION: FMAIL.CFG does not use the new config file type yet !!! */

typedef struct
{
   char            versionMajor;
   char            versionMinor;
   s32             creationDate;
   u32             key;
   u32             reservedKey;
   u32             relKey1;
   u32             relKey2;
   char            reserved1[22];
   mgrOptionsType  mgrOptions;
   akaListType     akaList;
/* nodeFakeType    reservedAka[16-MAX_AKAS]; */
   u16             netmailBoard[MAX_NETAKAS];
   u16             reservedNet[16-MAX_NETAKAS];
   genOptionsType  genOptions;
   mbOptionsType   mbOptions;
   mailOptionsType mailOptions;
   u16             maxPktSize;
   u16             reserved2;
   u16             mailer;
   u16             bbsProgram;
   u16             maxBundleSize;
   u16             extraHandles; /* 0-235 */
   u16             autoRenumber;
   u16             bufSize;
   u16             ftBufSize;
   u16             allowedNumNetmail;
   u16             logInfo;
   u16             logStyle;
   char            reserved3[68];
   u16             colorSet;
   char            sysopName[36];
   u16             defaultArc;
   char            reserved4[24];
   char            tearType;
   char            tearLine[25];
   pathType        summaryLogName;
   u16             recBoard;
   u16             badBoard;
   u16             dupBoard;
   char            topic1[16];
   char            topic2[16];
   pathType        bbsPath;
   pathType        netPath;
   pathType        sentPath;
   pathType        rcvdPath;
   pathType        inPath;
   pathType        outPath;
   pathType        securePath;
   pathType        logName;
   pathType        swapPath;
   pathType        semaphorePath;
   pathType        pmailPath;
   pathType        areaMgrLogName;
   pathType        autoRAPath;
   pathType        autoFolderFdPath;
   pathType        autoAreasBBSPath;
   pathType        autoGoldEdAreasPath;
   archiverInfo    unArc;
   archiverInfo    unZip;
   archiverInfo    unLzh;
   archiverInfo    unPak;
   archiverInfo    unZoo;
   archiverInfo    unArj;
   archiverInfo    unSqz;
   archiverInfo    GUS;
   archiverInfo    arc;
   archiverInfo    zip;
   archiverInfo    lzh;
   archiverInfo    pak;
   archiverInfo    zoo;
   archiverInfo    arj;
   archiverInfo    sqz;
   archiverInfo    customArc;
   char            reserved5[83];
   areaOptionsType optionsAKA[MAX_NETAKAS]; /* 'areaOptionsType' below */
   char            groupsQBBS[MAX_NETAKAS];
   u16             templateSecQBBS[MAX_NETAKAS];
   char            templateFlagsQBBS[MAX_NETAKAS][4];
   char            attr2RA[MAX_NETAKAS];
   char            aliasesQBBS[MAX_NETAKAS];
   u16             groupRA[MAX_NETAKAS];
   u16             altGroupRA[MAX_NETAKAS][3];
   char            qwkNameSBBS[MAX_NETAKAS][13];
   u16             minAgeSBBS[MAX_NETAKAS];
   u16             daysRcvdAKA[MAX_NETAKAS];
   char            replyStatSBBS[MAX_NETAKAS];
   u16             attrSBBS[MAX_NETAKAS];
   char            groupDescr[26][27];
   char            reserved6[9];
   char            msgKindsRA[MAX_NETAKAS];
   char            attrRA[MAX_NETAKAS];
   u16             readSecRA[MAX_NETAKAS];
   char            readFlagsRA[MAX_NETAKAS][4];
   u16             writeSecRA[MAX_NETAKAS];
   char            writeFlagsRA[MAX_NETAKAS][4];
   u16             sysopSecRA[MAX_NETAKAS];
   char            sysopFlagsRA[MAX_NETAKAS][4];
   u16             daysAKA[MAX_NETAKAS];
   u16             msgsAKA[MAX_NETAKAS];
   char            descrAKA[MAX_NETAKAS][51];
   userType        users[MAX_USERS];
   akaMatchType    akaMatch[MAX_MATCH];
   char            reserved7[2048-10*MAX_MATCH];
   uplinkReqType   uplinkReq[MAX_UPLREQ];   } configType;



/* ********** FMAIL.AR ********** */

#ifdef __FMAILX__
#define MAX_AREAS   4096
#else
#define MAX_AREAS    512
#endif
#define MAX_FORWARD   64

#define MB_PATH_LEN_OLD   19
#define MB_PATH_LEN       61
#define ECHONAME_LEN_090  25
#define ECHONAME_LEN      51
#define COMMENT_LEN       51
#define ORGLINE_LEN       59

typedef char areaNameType[ECHONAME_LEN];

typedef struct      /* OLD!!! */
{
   char            zero; /* Should always be zero */
   char            msgBasePath[MB_PATH_LEN_OLD];
   char            groupsQBBS;
   char            flagsTemplateQBBS[4];
   char            comment[COMMENT_LEN];
   u32             group;
   u16             board;
   u16             address;
   u16             alsoSeenBy;
   u16             groupRA;
   u16             altGroupRA[3];
   areaOptionsType options;
   u16             outStatus;
   u16             days;
   u16             msgs;
   u16             daysRcvd;
   u16             templateSecQBBS;
   u16             readSecRA;
   char            flagsRdRA[4];
   u16             writeSecRA;
   char            flagsWrRA[4];
   u16             sysopSecRA;
   char            flagsSysRA[4];
   char            attrRA;
   char            msgKindsRA;
   u16             attrSBBS;
   char            replyStatSBBS;
   areaNameType    areaName;
   char            qwkName[13];
   u16             minAgeSBBS;
   char            attr2RA;
   char            aliasesQBBS;
   char            originLine[ORGLINE_LEN];
   nodeNumType     export[MAX_FORWARD];  } rawEchoTypeOld;

typedef struct
{
   u16             signature;
   u16             writeLevel;
   areaNameType    areaName;
   char            comment[COMMENT_LEN];
   areaOptionsType options;
   u16             boardNumRA;
   char            msgBaseType;
   char            msgBasePath[MB_PATH_LEN];
   u16             board;
   char            originLine[ORGLINE_LEN];
   u16             address;
   u32             group;
   u16             alsoSeenBy;
   u16             msgs;
   u16             days;
   u16             daysRcvd;

   nodeNumType     export[MAX_FORWARD];

   u16             readSecRA;
   char            flagsRdNotRA[4];
   char            flagsRdRA[4];
   u16             writeSecRA;
   char            flagsWrNotRA[4];
   char            flagsWrRA[4];
   u16             sysopSecRA;
   char            flagsSysRA[4];
   char            flagsSysNotRA[4];
   u16             templateSecQBBS;
   char            flagsTemplateQBBS[4];
   char            reserved2;
   u16             netReplyBoardRA;
   char            boardTypeRA;
   char            attrRA;
   char            attr2RA;
   u16             groupRA;
   u16             altGroupRA[3];
   char            msgKindsRA;
   char            qwkName[13];
   u16             minAgeSBBS;
   u16             attrSBBS;
   char            replyStatSBBS;
   char            groupsQBBS;
   char            aliasesQBBS;  } rawEchoType;



/* ********** FMAIL.NOD ********** */

#ifdef __FMAILX__
#define MAX_NODEMGR      512
#else
#define MAX_NODEMGR      256
#endif

#define PKT_TYPE_2PLUS   1
#define CAPABILITY       PKT_TYPE_2PLUS

typedef struct
{
   bit      fixDate     : 1; /* Bit 0 */
   bit      tinySeenBy  : 1; /* Bit 1 */
   bit                  : 1; /* Bit 2 */
   bit      ignorePwd   : 1; /* Bit 3 */
   bit      active      : 1; /* Bit 4 */
   bit                  : 1;
   bit      routeToPoint: 1; /* Bit 6 */
   bit      packNetmail : 1; /* Bit 7 */
   bit                  : 1; /* Bit 8 */
   bit                  : 1;
   bit                  : 1;
   bit                  : 1;
   bit      forwardReq  : 1; /* Bit 12 */
   bit      remMaint    : 1; /* Bit 13 */
   bit      allowRescan : 1; /* Bit 14 */
   bit      notify      : 1;   } nodeOptionsType;

typedef struct /* OLD !!! */
{
   nodeNumType     node;
   char            reserved1[2];
   u16             capability;
   u16             archiver;
   nodeOptionsType options;
   u32             groups;
   u16             outStatus;
   char            reserved2[32];
   char            password[18];
   char            packetPwd[10];
   char            reserved[2];
   nodeNumType     viaNode;
   char            sysopName[36];   } nodeInfoTypeOld;

/* See Area File for header structure */

typedef struct
{
   u16             signature; /* contains "ND" */
   u16             writeLevel;
   nodeNumType     node;
   nodeNumType     viaNode;
   u16             capability;
   nodeOptionsType options;
   u16             archiver;
   u32             groups;
   u16             outStatus;
   char            sysopName[36];
   char            password[18];
   char            packetPwd[9];
   char            useAka;        } nodeInfoType;



/* ********** FMAIL.PCK ********** */

#define PACK_STR_SIZE 64
#define MAX_PACK      128

typedef char packEntryType[PACK_STR_SIZE];
typedef packEntryType packType[MAX_PACK];



/* ********** FMAIL.BDE ********** */

#define MAX_BAD_ECHOS 50

typedef struct
{
   areaNameType badEchoName;
   nodeNumType  srcNode;
   s16          destAka;      } badEchoType;

#if defined(GOLD_CANPACK)
#pragma pack()
#endif

