
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Library General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Library General Public License for more details.
//
//  You should have received a copy of the GNU Library General Public
//  License along with this library; if not, write to the Free
//  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gs_ge100.h,v 1.1.1.1 1998/11/21 19:38:11 odinn Exp $
//  ------------------------------------------------------------------

/*
**  gestruct.h
**
**  System data files definitions for GEcho 1.00
**
**  Copyright (C) 1992 Gerard J. van der Land. All rights reserved.
**
**  All information in this document is subject to change at any time
**  without prior notice.
**
**  Last revision: 12-Oct-92
**
**  Strings are NUL padded and NUL terminated arrays of char type.
**  Path names are back slash ('\') terminated.
*/

#define GE_THISREV 0x0002  /* System file revision level */
#define AKAS           11  /* Main + AKAs */
#define UPLINKS        10  /* AreaMgr uplinks */
#define USERS          10  /* User names */
#define MAXAREAS     1024  /* Area records */
#define MAXSYSTEMS    350  /* Systems per area */
#define MAXNODES      512  /* Node records */

/*
**  Datatypes:
**
**  typedef unsigned char byte;   ( 8 bits)
**  typedef unsigned short word;  (16 bits)
**  typedef unsigned long dword;  (32 bits)
*/

/* --- Address type */

typedef struct
   {
   word zone;
   word net;
   word node;
   word point;
   }
   ADDRESS;

/* --- Log levels */

#define LOG_INBOUND     0x0001
#define LOG_OUTBOUND    0x0002
#define LOG_PACKETS     0x0004
#define LOG_UNEXPECT    0x0008
#define LOG_AREAMGR     0x0010
#define LOG_EXTPKTINFO  0x0040
#define LOG_NETEXPORT   0x0100
#define LOG_NETIMPORT   0x0200
#define LOG_NETPACK     0x0400
#define LOG_NETMOVED    0x0800
#define LOG_STATISTICS  0x2000
#define LOG_MBUTIL      0x4000
#define LOG_DEBUG       0x8000

/* --- Log styles */

#define LOG_FD       0
#define LOG_BINK     1
#define LOG_QUICK    2
#define LOG_DBRIDGE  3

/* --- Setup option bits */

#define NOKILLNULL 0x0001  /* Don't kill null netmail messages while tossing */
#define RESCANOK   0x0002  /* Allow rescan */
#define KEEPREQS   0x0004  /* Keep AreaMgr requests */
#define NONODEADD  0x0008  /* Don't automatically add NodeMgr records */
#define NOUSERSBBS 0x0010  /* Don't let Import check USERS.BBS */
#define USEBUFFERS 0x0020  /* Use outbound buffers */
#define KEEPNET    0x0040  /* Don't use Kill/Sent on exported netmail */
#define KEEPMGR    0x0080  /* Don't use Kill/Sent on MGR receipts */
#define KILLDUPES  0x0200  /* Kill duplicate messages */
#define ARCMAIL    0x0400  /* Use ARCmail 0.60 naming for out-of-zone mail */
#define NOCRSTRIP  0x0800  /* Don't strip Soft-CRs */
#define NOPIDSTRIP 0x1000  /* Don't strip PIDs */
#define NOAUTODISC 0x2000  /* Don't automatically disconnect empty PT areas */
#define NOCHECKEND 0x4000  /* Don't check for valid end of archives */
#define SETPVT     0x8000  /* Set Pvt on imported netmail messages */

/* --- Extra option bits */

#define NOCHKDEST  0x0001  /* Don't check packet destination */
#define AREASBBS   0x0002  /* Use AREAS.BBS for MBUTIL */
#define PASSIVEOK  0x0004  /* Allow %PASSIVE */

/* --- Compression types */

#define PR_ARC  0  /* For compressed mail files created by ARC or PKPAK */
#define PR_ARJ  1  /* For compressed mail files created by ARJ */
#define PR_LZH  2  /* For compressed mail files created by LHA */
#define PR_PAK  3  /* For compressed mail files created by PAK */
#define PR_ZIP  4  /* For compressed mail files created by PKZIP */
#define PR_ZOO  5  /* For compressed mail files created by ZOO */
#define PR_SQZ  6  /* For compressed mail files created by SQZ */

/* --- Locking method */

#define LOCK_OFF    0  /* Deny Write (Exclusive) */
#define LOCK_RA101  1  /* RemoteAccess 1.01 (SHARE) */
#define LOCK_RA110  2  /* RemoteAccess 1.10/1.11 (SHARE) */

/* --- Semaphore mode */

#define SEMPAHORE_OFF  0  /* Don't use semaphores */
#define SEMAPHORE_FD   1  /* FrontDoor 2.xx */
#define SEMAPHORE_IM   2  /* InterMail 2.xx */
#define SEMAPHORE_DB   3  /* D'Bridge 1.3x */
#define SEMAPHORE_BT   4  /* BinkleyTerm 2.5x */

/* --- Mailer type */

#define MAILER_FD  0  /* FrontDoor */
#define MAILER_DB  1  /* D'Bridge */
#define MAILER_BT  2  /* BinkleyTerm */

/* --- Uplink option bits */

#define UPLINK_DIRECT  0x01
#define UPLINK_ALWAYS  0x02
#define UPLINK_ADDPLUS 0x04

typedef struct
   {
   ADDRESS address;    /* Uplink address */
   char areafix[9];    /* AreaFix program */
   char password[17];  /* AreaFix password */
   char filename[13];  /* "Forward List" filename */
   byte unused[6];
   byte options;       /* See --- Uplink options bits */
   byte filetype;      /* 0 = Random, 1 = "<areaname> <description>" */
   dword groups;       /* Nodes must have one of these groups */
   byte origin;        /* Origin AKA */
   }
   UPLINK;

typedef struct
   {
   word zone;
   word net;
   byte aka;
   }
   AKAMATCH;

/* --- SETUP.GE structure */

typedef struct
   {
   word sysrev;              /* Must contain GE_THISREV */
   word options;             /* Options bits, see --- Setup option bits */
   word autorenum;           /* Auto renumber value */
   word maxpktsize;          /* Maximum packet size, 0 = unlimited */
   byte logstyle;            /* See --- Log styles */
   byte oldnetmailboard;     /* Netmail board, must be zero now */
   byte badboard;            /* Where bad echomail is stored (0 = path) */
   byte dupboard;            /* Where duplicates are stored (0 = path) */
   byte recoveryboard;       /* Recovery board (1-200, 0 = delete) */
   byte filebuffer;          /* Size (in KB) of MBU file I/O buffer */
   byte days;                /* Days to keep old mail around */
   byte swapping;            /* Swapping method */
   byte compr_default;       /* Default compresion type */
   byte color[15];           /* See --- Personal Mail colors */
   ADDRESS aka[AKAS];        /* Main address and AKAs */
   word pointnet[AKAS];      /* Pointnets for all addresses */
   dword gekey;              /* GEcho registration key */
   dword mbukey;             /* MBUTIL registration key */
   char geregto[51];         /* Text used to generate the GEcho key */
   char mburegto[51];        /* Text used to generate the MBUTIL key */
   char username[USERS][36]; /* User names */
   char mbpath[53];          /* Hudson message base path */
   char mailpath[53];        /* Netmail path */
   char inbound_path[53];    /* Where incoming compressed mail is stored */
   char outbound_path[53];   /* Where outgoing compressed mail is stored */
   char editor[65];          /* External full screen editor used by PM */
   char nodepath[53];        /* Nodelist directory used by PMNC and PM */
   char areasfile[65];       /* AREAS.BBS style file */
   char logfile[65];         /* GEcho/MBUTIL log file */
   char mgrlogfile[65];      /* AreaMgr log file */
   char swap_path[53];       /* Swap path */
   char tear_line[31];       /* Tearline to be placed by MBUTIL Export */
   char originline[20][61];  /* Origin lines */
   char compr_prog[10][13];       /* Compression program filenames */
   char compr_switches[10][20];   /* Compression program switches */
   char decompr_prog[10][13];     /* Decompression program filenames */
   char decompr_switches[10][20]; /* Decompression program switches */
   char groups[26][21];      /* Descriptions of area groups */
   byte lockmode;            /* See --- Locking method */
   char secure_path[53];     /* From which secure PKTs are tossed */
   char rcvdmailpath[53];    /* Directory to which Rcvd netmail is moved */
   char sentmailpath[53];    /* Directory to which Sent netmail is moved */
   char semaphorepath[53];   /* Where FD rescan files are stored */
   byte version_major;
   byte version_minor;
   byte semaphore_mode;      /* See --- Semaphore modes */
   char badecho_path[53];    /* Where sec. violating and unknown mail is stored */
   byte mailer_type;         /* See --- Mailer type */
   word loglevel;            /* See --- Log level */
   AKAMATCH akamatch[20];    /* AKA matching table */
   char mbulogfile[65];      /* MBUTIL log file */
   word maxqqqs;             /* Max. number of QQQ info stored in memory */
   byte maxqqqopen;          /* Not used */
   byte maxhandles;          /* Max. number of files used by GEcho */
   word maxarcsize;          /* Max. archive size, 0 = unlimited */
   byte unused[2];           /* Unused space */
   word extraoptions;        /* See --- Extra option bits */
   byte firstboard;          /* First available new board (0 = *.MSG) */
   word reserved1;           /* Reserved */
   word copy_persmail;       /* Copy personal mail for user (bits 0-9) */
   byte persmailboard[USERS];/* Personal mail board (0 = path) */
   dword public_groups;      /* Public groups (bits 0-25) */
   word dupentries;          /* Number of duplicate entries in ECHODUPE.GE */
   byte rcvdboard;           /* Where Rcvd netmail is moved to (0 = path) */
   byte sentboard;           /* Where Sent netmail is moved to (0 = path) */
   byte akaboard[AKAS];      /* Netmail boards for AKAs */
   byte userboard[USERS];    /* Netmail boards for system users, 255 = use AKA board */
   byte reserved2;           /* Reserved */
   UPLINK uplink[UPLINKS];   /* Uplink manager information */
   char persmail_path[53];   /* Path to store copies of personal echomail */
   char outpkts_path[53];    /* Where outbound packets are temp. stored */
   word compr_mem[10];       /* Memory needed for compression programs */
   word decompr_mem[10];     /* Memory needed for decompression programs */
   dword pwdcrc;             /* CRC-32 of access password, -1L = no password */
   word default_msgs;        /* Maximum number of messages       (Purge) */
   word default_days;        /* Maximum age of non-Rcvd messages (Purge) */
   char gus_prog[13];        /* General Unpack Shell program filename */
   char gus_switches[20];    /* GUS switches */
   word gus_mem;             /* Memory needed for GUS */
   word default_rcvd;        /* Maximum age of Rcvd messages     (Purge) */
   }
   SETUP_GE;

/***************************************************************************/

/* --- Area option bits */

#define IMPORTSB 0x0001  /* Import SEEN-BY lines to message base */
#define SECURITY 0x0002  /* Only accept mail from systems in the exportlist */
#define PASSTHRU 0x0004  /* Mail is not imported, only forwarded */
#define VISIBLE  0x0008  /* Area is visible for anyone in AreaMgr's %LIST */
#define REMOVED  0x0010  /* Area should be removed by GSETUP Pack */
#define NOUNLINK 0x0020  /* Do not allow users to unlink this area */
#define TINYSB   0x0040  /* Tiny SEEN-BYs without systems not in exportlist */
#define PVT      0x0080  /* Private bits are preserved and are not stripped */
#define CHECKSB  0x0100  /* Use SEEN-BYs for duplicate prevention */
#define NOSLEEP  0x0200  /* Do not allow users to make this area passive */
#define SDM      0x0400  /* Area is *.MSG style */
#define NOTIFIED 0x4000  /* Sysop notified that area was disconnected */
#define UPLDISC  0x8000  /* Disconnected from uplink (only for PT areas) */

/* --- Area type */

#define ECHOMAIL  0
#define NETMAIL   1
#define LOCAL     2
#define BADECHO   3
#define PERSONAL  4

/* --- AREAFILE.GE header */

typedef struct
   {
   word hdrsize;  /* sizeof(AREAFILE_HDR) */
   word recsize;  /* sizeof(AREAFILE_GE) */
   word systems;  /* Number of entries in the export list (1-MAXSYSTEMS) */
   }
   AREAFILE_HDR;

/* --- AREAFILE.GE record */

typedef struct
   {
   char name[51];       /* AREA tag, must be uppercase, no spaces */
   char comment[61];    /* Description of the topics discussed in area */
   char path[51];       /* Path where the *.MSG files are stored */
   char originline[61]; /* Custom origin line, used if origlinenr = 0 */
   word areanumber;     /* Area number (1-MAXAREAS, 1-200 = Hudson) */
   char group;          /* Group (A-Z) */
   word options;        /* See --- Area options bits */
   byte originlinenr;   /* Origin line (1-20, 0 = custom) */
   byte pkt_origin;     /* Address for the packet/Origin line (0-10) */
   word seenbys;        /* Addresses (bits 0-10) to add to the SEEN-BY */
   word reserved;
   word msgs;           /* Maximum number of messages       (MBUTIL Purge) */
   word days;           /* Maximum age of non-Rcvd messages (MBUTIL Purge) */
   word rcvddays;       /* Maximum age of Rcvd messages     (MBUTIL Purge) */
   byte areatype;       /* See --- Area type */
   }
   AREAFILE_GE;

/* --- Export entry status bits */

#define EXPT_NOREAD   0x01  /* Accept from but don't forward to this system */
#define EXPT_NOWRITE  0x02  /* Don't accept from but do foward to this system */
#define EXPT_HOLD     0x04  /* Temporary don't send this area to this system */
#define EXPT_NOUNLINK 0x08  /* Don't allow system to disconnect */

/* --- Export list entry */

typedef struct
   {
   ADDRESS address;
   byte status;
   }
   EXPORTENTRY;

/* --- AREAFILE.GEX index entry */

typedef struct
   {
   long crc32;       /* CRC-32 on areaname */
   word areanumber;  /* Area number (1-MAXAREAS, 1-200 = Hudson) */
   long offset;      /* File offset of record in AREAFILE.GE */
   }
   AREAFILE_GEX;

/***************************************************************************/

/* --- Status:
   0x0000 = None
   0x0002 = Crash
   0x0200 = Hold
   0xFFFF = Removed entry
*/

/* --- Node option bits */

#define REMOTEMAINT  0x0001
#define ALLOWRESCAN  0x0002
#define FORWARDREQ   0x0004
#define DIRECT       0x0008
#define NONOTIFY     0x0010

/* --- NODEFILE.GE header */

typedef struct
   {
   word hdrsize;  /* sizeof(NODEFILE_HDR) */
   word recsize;  /* sizeof(NODEFILE_GE) */
   }
   NODEFILE_HDR;

/* --- NODEFILE.GE record */

typedef struct
   {
   ADDRESS address;   /* Address of the system */
   char sysop[36];    /* Name of the sysop or point */
   char pktpwd[9];    /* Packet (session) password */
   char mgrpwd[17];   /* AreaMgr password */
   dword groups;      /* Authorized groups (bits 0-25) */
   word options;      /* See --- Node option bits */
   byte comprtype;    /* Compression type (0 - 9, 10 = PKT) */
   word status;       /* File attach status. See above */
   ADDRESS route_to;  /* Address to route mail files to */
   }
   NODEFILE_GE;

/* --- NODEFILE.GEX index entry */

typedef struct
   {
   ADDRESS address;  /* Address of the system */
   long offset;      /* File offset of record in NODEFILE.GE */
   }
   NODEFILE_GEX;

/***************************************************************************/

/* --- Routed node entry */

typedef struct
   {
   ADDRESS node;
   byte status;
   byte via;
   }
   ROUTE;

/* --- PACKFILE.GE structure */

typedef struct
   {
   ADDRESS Via[60];
   ROUTE   Route[640];
   }
   PACKFILE_GE;

/***************************************************************************/

/* --- ECHODUPE.GE structure */

typedef struct
   {
   word pointer;  /* Next offset */
   word entries;  /* Number of entries in the database */
/*
   word crc32_high[entries];     32 bit CRCs on msg headers, high portions
   word crc32_low[entries];      32 bit CRCs on msg headers, low portions
*/
   }
   ECHODUPE_GE;

/***************************************************************************/

/* --- FTSCPROD.GE record */

typedef struct
   {
   byte cap;       /* Capability, 0 = Type 2.0, 1 = Type 2.1, 2 = Type 2+ */
   char name[30];  /* Name of product */
   }
   FTSCPROD_GE;

/***************************************************************************/

/* --- Personal Mail colors (same as used by FrontDoor editor FM) */

#define top_line               sys.color[ 0]
#define status_line            sys.color[ 1]
#define error                  sys.color[ 2]
#define text                   sys.color[ 3]
#define text_quotes            sys.color[ 4]
#define text_reverse           sys.color[ 5]
#define text_hardCRs           sys.color[ 6]
#define header                 sys.color[ 7]
#define header_data            sys.color[ 8]
#define header_data_highlight  sys.color[ 9]
#define data_entry             sys.color[10]
#define window_frame           sys.color[11]
#define window_text            sys.color[12]
#define window_select          sys.color[13]
#define window_highlight       sys.color[14]

/* end of file "gestruct.h" */
