
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Library General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Library General Public License for more details.
//
//  You should have received a copy of the GNU Library General Public
//  License along with this library; if not, write to the Free
//  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gs_im120.h,v 1.1.1.1 1998/11/21 19:38:11 odinn Exp $
//  ------------------------------------------------------------------

/* ============================================================== */
/*                                                                */
/*               IMAIL DEFINITIONS and STRUCTURES                 */
/*                                                                */
/* ============================================================== */
/*                                                                */
/* IMAIL structures released by permission of the author, Fabiano */
/* Fabris. For commercial applications of this information,       */
/* please contact the author directly.                            */
/*                                                                */
/* Copyright (C) 1990-1992 by Fabiano Fabris. All rights reserved */
/*                                                                */
/* ============================================================== */


#include <direct.h>
#include <io.h>

/* -------------------------------------------------------- */
/*  Data type definitions                                   */
/* -------------------------------------------------------- */

#define word                 unsigned short
#define byte                 unsigned char
#define boolean              short
#define integer              short
typedef unsigned char        bit;

/* -------------------------------------------------------- */
/*  Internal limits                                         */
/* -------------------------------------------------------- */

#define MAXAKAS 16                   /* Max # of addresses  */
#define MAXPACKERS 10                /* Max # of packer def */
#define MAXEXPORT 60                 /* Max export defs     */
#define MAXVIA 40                    /* max nodes packed via*/
#define MAXPACK 32                   /* max default pack cmd*/
#define MAXFWDLINK 10                /* max fwd link structs*/
#define MAXDOMAIN 30                 /* max # domians       */

#define _MAXPATH 80
#define _MAXFILE 9
#define _MAXEXT  5

struct naddress {
  word         zone;             /*  Zone Number             */
  word         net;              /*  Net Number              */
  word         node;             /*  Node Number             */
  word         point;            /*  Point Number            */
};

struct paddress {
  word         zone;             /* Zone Number              */
  word         net;              /* Net Number               */
  word         node;             /* Node Number              */
  word         point;            /* Point Number             */
  bit          allnets:1;        /* Pack all nets in zone    */
  bit          allnodes:1;       /* Pack all nodes in net    */
  bit          allpts:1;         /* Pack all points under nd */
  bit          junk:5;
};

struct pack_struct {
  struct naddress dst;           /* pack via this node       */
  struct paddress nodes[MAXVIA]; /* nodes to pack            */
};

struct fwd_link {
  char         areasfile[_MAXFILE+_MAXEXT];   /* name of areas file          */
  char         toprogram[10];               /* name of area manager        */
  char         password[21];                /* area manager password       */
  struct naddress uplink;                   /* address of uplink           */
};

struct domain_type {
  char         dname[21];                   /* domain name                 */
  word         zone;                        /* zone number                 */
};

/* -------------------------------------------------------- */
/*  IMAIL.CF Structure                                      */
/* -------------------------------------------------------- */

typedef struct im_config_type {
  byte         im_ver_maj;                  /* Major Version               */
  byte         im_ver_min;                  /* Minor Version               */
  char         sysop[37];                   /* name of sysop               */
  struct naddress aka[MAXAKAS];             /* the AKAs                    */
  struct domain_type domains[MAXDOMAIN];    /* domain names & zones        */
  byte         netmailbrd[MAXAKAS];         /* net board for each aka      */
  byte         empty[MAXAKAS];              /* reserved                    */
  char         netmail[_MAXPATH];            /* net mail subdirectory       */
  char         netfile[_MAXPATH];            /* inbound files directory     */
  char         outbound[_MAXPATH];           /* outbound directory          */
  char         quickbbs[_MAXPATH];           /* QuickBBS system directory   */
  char         reserved_1[_MAXPATH];         /* Dupes directory for IFILE   */
  char         logfilename[_MAXPATH];        /* Log file name               */
  char         reserved_2[_MAXPATH];         /* log file for IFILE          */
  long         last_run;                    /* time of last echo stat run  */
  word         last_scan_msg;               /* hi msg # from last scan     */
  word         max_pkt_size;                /* max size of pkt to create   */
  word         max_dupes;                   /* max dupes kept in dbase     */
  word         msgid_ctr;                   /* MSGID counter               */
  byte         badmsgs;                     /* bad message board           */
  byte         dupebrd;                     /* dupe message board          */
  byte         log_level;                   /* logging level               */
  bit          rtnrecpt:1;                  /* True if to send rtn recpt   */
  bit          checkbad:1;                  /* True if to check badmsgs    */
  bit          ARCmail06:1;                 /* ARCmail 0.6 compatibility   */
  bit          environment:1;               /* FroDo or Binkley            */
  bit          unused:1;
  bit          auto_add:1;                  /* Auto-add unknown areas      */
  bit          forward_for:1;               /* forward PKTs not for us     */
  bit          bits1:1;
  bit          singleextract:1;             /* extract 1 bundle at a time  */
  bit          trunc_sent:1;                /* 1 = Trunc 0 = Delete        */
  bit          keep_recpt:1;                /* keep areafix message        */
  bit          rescan:1;                    /* flag allow AreaLink rescan  */
  bit          reply_kill:1;                /* flag auto-rply kill/sent    */
  bit          swap_ems:1;                  /* swap to EMS                 */
  bit          swap_ext:1;                  /* swap to extended memory     */
  bit          bits2:1;
  word         days;                        /* days before auto-reply      */
  char         replytext[_MAXFILE+_MAXEXT];   /* path to reply text          */
  char         dflt_origin[64];             /* default origin line         */
  char         alnk_help[_MAXFILE+_MAXEXT];   /* AreaLink help text          */
  char         new_areas[_MAXFILE+_MAXEXT];   /* name of file for new areas  */
  char         packers[MAXPACKERS][40];     /* Packer defintions           */
  char         arcunpak[40];                /* cmd to de-arc .ARC files    */
  char         arjunpak[40];                /* cmd to de-arc .ARJ files    */
  char         pkpakunpak[40];              /* cmd to de-arc .PKA files    */
  char         pkzipunpak[40];              /* cmd to de-arc .ZIP files    */
  char         lharcunpak[40];              /* cmd to de-arc .LZH files    */
  char         zoounpak[40];                /* cmd to de-arc .ZOO files    */
  char         pakunpak[40];                /* cmd to de-arc .PAK files    */
  char         unkunpak[40];                /* cmd to de-arc unknown files */
  struct pack_struct pack[MAXPACK];         /* default PACK routing        */
  struct fwd_link fwd[MAXFWDLINK];          /* forward link requests       */
  char         filler[20];                  /* reserved                    */
} im_config_type;


/* -------------------------------------------------------- */
/*  IMAIL.AR Structure                                      */
/* -------------------------------------------------------- */

typedef struct areas_record_type {
  char         aname[41];            /* area name                   */
  char         comment[61];          /* area comment                */
  byte         brd;                  /* board number                */
  char         group;                /* area group                  */
  char         origin[64];           /* origin line to use          */
  char         o_addr;               /* address for origin          */
  char         use_akas[MAXAKAS];    /* addresses for seen-bys      */
  bit          active:1;             /* flag area active            */
  bit          passth:1;             /* passthrough area flag       */
  bit          tiny_seen:1;          /* tiny seen-by flag           */
  bit          secure:1;             /* secure flag                 */
  bit          keep_seen:1;          /* keep seen-by flag           */
  bit          rsvd:1;               /* Reserved field              */
  bit          deleted:1;            /* flag deleted area           */
  bit          auto_added:1;         /* flag auto-added record      */
  word         total;                /* for echo stat summary       */
  word         dupe_count;           /* duplicate counter           */
  byte         reserved[8];          /* Reserved field              */
  byte         days;                 /* days to keep messages       */
  word         msgs;                 /* num messages to keep        */
  char         filler[5];
  struct naddress export[MAXEXPORT]; /* export list                 */
  struct areas_record_type *next_area;
  struct areas_record_type *prev_area;
} areas_record_type;

/*
   Notes:
   1) The last two fields are not saved to IMAIL.AR
   2) The entries in 'o_addr' and 'use_akas' are indexes into
      the list of AKAs in IMAIL.CF, minus 1. eg:
         im_config.aka[o_addr-1]
      A value of 0 means 'no address'
*/


