
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Library General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Library General Public License for more details.
//
//  You should have received a copy of the GNU Library General Public
//  License along with this library; if not, write to the Free
//  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gxmax2.cpp,v 1.1.1.1 1998/11/21 19:38:10 odinn Exp $
//  ------------------------------------------------------------------
//  Read areas from Maximus (1.xx - 2.xx)
//  ------------------------------------------------------------------


#include <stdlib.h>
#include <gmemdbg.h>
#include <gstrall.h>
#include <gedacfg.h>
#include <gs_max.h>


//  ------------------------------------------------------------------
//  Read areas from Maximus 2.xx

void gareafile::ReadMaximus2(char* mxpath, char* areafile, char* options) {

  FILE* fp;
  AreaCfg aa;
  Path prmfile;

  MakePathname(prmfile, mxpath, "max.prm");

  m_pointers* prmp = (m_pointers*)throw_calloc(1, sizeof(m_pointers));
  m_pointers& prm = *prmp;

  fp = fsopen(prmfile, "rb", sharemode);
  if(fp) {

    if(not quiet)
      printf("\n* Reading %s", areafile);
    ChkAddStamp(areafile, options);

    fread(prmp, sizeof(m_pointers), 1, fp);
    long heapsz = fsize(fp) - prm.heap_offset;
    char* offsets = (char*)throw_calloc(1, (uint)heapsz);
    fread(offsets, (uint)heapsz, 1, fp);
    //CfgUsername(PRM(sysop));
    if(*squishuserpath == NUL)
      PathCopy(squishuserpath, PRM(user_file));
    throw_free(offsets);
    
    fclose(fp);
  }

  if(*squishuserpath == NUL)
    PathCopy(squishuserpath, mxpath);

  fp = fsopen(areafile, "rb", sharemode);
  if(fp) {
    setvbuf(fp, NULL, _IOFBF, 32000);

    if(not quiet)
      printf("\n* Reading %s", areafile);
    ChkAddStamp(areafile, options);

    _area* area = (_area *)throw_malloc(sizeof(_area));
    fread(area, sizeof(_area), 1, fp);
    word slen = area->struct_len;
    area = (_area *)throw_realloc(area, slen);

    do {
      if(not strblank((char*)area->msginfo)) {
        aa.reset();
        switch(area->type & 0x0F) {
          case MSGTYPE_SQUISH:
            aa.msgbase = GMB_SQUISH;
            aa.setpath((char*)area->msgpath);
            break;
          case 0x00:
          case MSGTYPE_SDM:
            aa.msgbase = fidomsgtype;
            aa.setpath((char*)area->msgpath);
            break;
        }
        if(aa.msgbase) {
          aa.aka = CAST(ftn_addr, prm.address[area->origin_aka]);
          aa.setdesc((char*)area->msginfo);
          aa.setechoid((char*)area->msgname);
          aa.setorigin((char*)area->origin);
          if(area->attrib[0] & (ECHO|CONF)) {
            aa.type = GMB_ECHO;
            aa.attr = attribsecho;
          }
          else if(area->attrib[0] & SYSMAIL) {
            aa.type = GMB_NET;
            aa.attr = attribsnet;
          }
          else {
            aa.type = GMB_LOCAL;
            aa.attr = attribslocal;
          }
          AddNewArea(aa);
        }
      }
    } while(fread(area, slen, 1, fp) == 1);

    throw_free(area);

    fclose(fp);
  }

  throw_free(prmp);
}


//  ------------------------------------------------------------------

