
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmoezyc.h,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Ezycom msgbase.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GMEZYC_H
#define __GMEZYC_H


//  ------------------------------------------------------------------

#include <gmoarea.h>
#include <gusrezyc.h>

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


//  ------------------------------------------------------------------
//  Ezycom constants

#define EZYC_MAXNAME      36
#define EZYC_MAXSUBJ      73
#define EZYC_MAXMESS102 1024
#define EZYC_MAXMESS110 1536
#define EZYC_USERSBUF    100

#define EZYC_USERDELETED 0x0001


//  ------------------------------------------------------------------

const byte EZYC_MSGATTR_DELETED  = 0x01;
const byte EZYC_MSGATTR_NETPEND  = 0x02;
const byte EZYC_MSGATTR_RESERVED = 0x04;
const byte EZYC_MSGATTR_PRIVATE  = 0x08;
const byte EZYC_MSGATTR_RECEIVED = 0x10;
const byte EZYC_MSGATTR_ECHOPEND = 0x20;
const byte EZYC_MSGATTR_LOCAL    = 0x40;
const byte EZYC_MSGATTR_NOKILL   = 0x80;

const byte EZYC_NETATTR_KILLSENT = 0x01;
const byte EZYC_NETATTR_SENT     = 0x02;
const byte EZYC_NETATTR_ATTACH   = 0x04;
const byte EZYC_NETATTR_CRASH    = 0x08;
const byte EZYC_NETATTR_FREQ     = 0x10;
const byte EZYC_NETATTR_RREQ     = 0x20;
const byte EZYC_NETATTR_AREQ     = 0x40;
const byte EZYC_NETATTR_RREC     = 0x80;

const byte EZYC_EXTATTR_RSVD1    = 0x01;
const byte EZYC_EXTATTR_RSVD2    = 0x02;
const byte EZYC_EXTATTR_RSVD3    = 0x04;
const byte EZYC_EXTATTR_RSVD4    = 0x08;
const byte EZYC_EXTATTR_RSVD5    = 0x10;
const byte EZYC_EXTATTR_RSVD6    = 0x20;
const byte EZYC_EXTATTR_RSVD7    = 0x40;
const byte EZYC_EXTATTR_SEEN     = 0x80;


//  ------------------------------------------------------------------
//  Ezycom message header

struct EzycHdr {

  word    replyto;
  word    reply1st;

  ulong   startposition;
  ulong   messagelength;

  Addr    destnet;
  Addr    orignet;

  word    cost;           // redefine to replynext?

  byte    msgattr;
  byte    netattr;
  byte    extattr;

  FTime   posttimedate;
  FTime   recvtimedate;

  char    whoto[EZYC_MAXNAME];
  char    whofrom[EZYC_MAXNAME];
  char    subject[EZYC_MAXSUBJ];
};


//  ------------------------------------------------------------------
//  Ezycom LASTCOMB.BBS record structure

struct EzycLast {
  word combinedinfo;
  word lastreadinfo[16];
};


//  ------------------------------------------------------------------
//  Ezycom MSGFAST.BBS record structure

struct EzycFast {
  dword whoto;       // 32 bit CRC on WhoTo in MSGHxxx.BBS
  word msgboard;
  word msgnumber;
};


//  ------------------------------------------------------------------
//  Ezycom MSGDLTD.BBS record

struct EzycDeleted {
  word msgboard;
  word msgnumber;
};


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------

struct EzycData {
  int fhhdr;
  int fhtxt;
  int fhnow;
  int omode;
  int smode;
  int timesposted;
  int islocked;
  Path ezyfile;
};


//  ------------------------------------------------------------------

struct EzycWide {
  int         ver;
  EzycomUser* user;
  int         userno;
  int         maxmess;
  const char* msgbasepath;
  const char* userbasepath;
};


//  ------------------------------------------------------------------

class EzycomArea : public gmo_area {

protected:

  EzycWide* wide;
  EzycData* data;

  void data_open();
  void data_close();

  char* ret_mess_xxx(char* __path, byte __type);
  char* ret_mess_area(char* __path);

  int raw_open();

  void test_raw_open(int __fileline);
  void raw_close();
  int test_open(const char* __file, int __mode, int __share);

  void save_lastread();

  void raw_scan(int __keep_index);

  int load_message(int __mode, gmsg* __msg, EzycHdr& __hdr);
  void save_message(int __mode, gmsg* __msg, EzycHdr& __hdr);


public:

  EzycomArea() { wide = NULL; data = NULL; }
  virtual ~EzycomArea() {}

  //  ----------------------------------------------------------------
  //  Messagebase member functions
  
  void open();
  void close();

  void suspend();
  void resume();

  void lock();
  void unlock();

  void scan();
  void scan_area();
  void scan_area_pm();

  int load_hdr(gmsg* msg);
  int load_msg(gmsg* msg);

  void save_hdr(int mode, gmsg* msg);
  void save_msg(int mode, gmsg* msg);

  void del_msg(gmsg* msg);

  void new_msgno(gmsg* msg);
  char* user_lookup(char* lookfor);
  int renumber();

  void update_timesread(gmsg* msg);

  Line* make_dump_msg(Line*& lin, gmsg* msg);
};


//  ------------------------------------------------------------------

extern EzycWide* __FAR ezycomwide;
extern EzycData* __FAR ezycomdata;
extern int      __FAR ezycomdatano;


//  ------------------------------------------------------------------

#if defined(__BORLANDC__) and defined(__MSDOS__)
inline void EzycomArea::open() { }
inline void EzycomArea::save_lastread() { }
inline void EzycomArea::close() { }
inline void EzycomArea::suspend() { }
inline void EzycomArea::resume() { }
inline void EzycomArea::scan() { }
inline void EzycomArea::scan_area() { }
inline void EzycomArea::scan_area_pm() { }
inline int EzycomArea::load_message(int, gmsg*, EzycHdr&) { return FALSE; }
inline int EzycomArea::load_hdr(gmsg*) { return FALSE; }
inline int EzycomArea::load_msg(gmsg*) { return FALSE; }
inline void EzycomArea::lock() { }
inline void EzycomArea::unlock() { }
inline void EzycomArea::save_message(int, gmsg*, EzycHdr&) { }
inline void EzycomArea::save_hdr(int, gmsg*) { }
inline void EzycomArea::save_msg(int, gmsg*) { }
inline void EzycomArea::del_msg(gmsg*) { }
inline void EzycomArea::new_msgno(gmsg*) { }
inline char* EzycomArea::user_lookup(char*) { return NULL; }
inline int EzycomArea::renumber() { return FALSE; }
inline void EzycomArea::update_timesread(gmsg*) { }
inline Line* EzycomArea::make_dump_msg(Line*&, gmsg*) { return NULL; }
#endif


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

