
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmogold.h,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Goldbase.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GMGOLD_H
#define __GMGOLD_H


//  ------------------------------------------------------------------
//  Required

#include <gmoarea.h>
#include <gusrgold.h>


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


//  ------------------------------------------------------------------
//  Defines

#define GOLD_DELETEDMSGNO  0xFFFFFFFFL
#define GOLD_MAXBOARD      500
#define GOLD_EXT           ".dat"
#define GOLD_NAME          "Goldbase"


//  ------------------------------------------------------------------
//  Goldbase Message numbers

typedef dword GoldMsgn;


//  ------------------------------------------------------------------
//  Goldbase Board numbers

typedef word GoldBoard;


//  ------------------------------------------------------------------
//  Goldbase Message Attributes

#define GOLD_DELETED    0x01
#define GOLD_NETTRANS   0x02
#define GOLD_NETMSG     0x04
#define GOLD_PVT        0x08
#define GOLD_RECEIVED   0x10
#define GOLD_ECHOTRANS  0x20
#define GOLD_LOCAL      0x40
#define GOLD_GROUPMSG   0x80

#define GOLD_KILLSENT   0x01
#define GOLD_SENT       0x02
#define GOLD_ATTACH     0x04
#define GOLD_CRASH      0x08
#define GOLD_RETRECREQ  0x10
#define GOLD_AUDITREQ   0x20
#define GOLD_RETREC     0x40
#define GOLD_FREQ       0x80


//  ------------------------------------------------------------------
//  Goldbase Message Header (MSGHDR.DAT)

struct GoldHdr {
  long    msgno;
  long    replyto;
  long    reply1st;
  word    timesread;
  long    startrec;
  word    numrecs;
  word    destnet;
  word    destnode;
  word    orignet;
  word    orignode;
  byte    destzone;
  byte    origzone;
  word    cost;
  word    msgattr;
  word    netattr;
  word    board;
  char    time[6];
  char    date[9];
  char    to[36];
  char    by[36];
  char    re[73];
};


//  ------------------------------------------------------------------
//  Goldbase Messagebase Information (MSGINFO.DAT)

struct GoldInfo {
  long  low;
  long  high;
  long  total;
  word  active[GOLD_MAXBOARD];
};


//  ------------------------------------------------------------------
//  Goldbase Message Header Index (MSGIDX.DAT)

struct GoldIdx {
  long  msgno;
  word  board;
};


//  ------------------------------------------------------------------
//  Goldbase Message Name Index (MSGTOIDX.DAT)

struct GoldToIdx {
  char  name[36];
};


//  ------------------------------------------------------------------
//  Goldbase Lastread (LASTREAD.DAT)

typedef long GoldLast[GOLD_MAXBOARD];


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------

struct GoldScan {
  uint count;
  uint active;
  ulong lastread;
  uint lastreadreln;
  ulong lastreadfound;
  ulong firstmsgno;
  ulong lastmsgno;
  uint pmcount;
};


//  ------------------------------------------------------------------

struct GoldWide {
  int      fhtxt;
  int      fhhdr;
  int      fhidx;
  int      fhinf;
  int      fhlrd;
  int      fhtoi;
  int      fhusr;
  int      isopen;
  int      islocked;
  int      timesposted;
  long      msgidxsize;
  GoldIdx*  msgidxptr;
  GoldInfo msginfo;
  GoldLast lastrec;
  GUser*    user;
  int      userno;
  GoldScan* scn;
  GoldIdx*  pmscan;
  uint      pmscantotal;
  int      iswideopen;
  int      ispmscanned;
  int      iswidescanned;
  const char* path;
  const char* syspath;

  void init();
  void exit();

  void open();
  void close();

  void lock();
  void unlock();

  void scan();

  void raw_open_scan();
  void scan_pm();

  void realloc_pm_scan();

  void update_netecho(char* __name, GoldMsgn __hdridx, int __delete);

  int test_open(char* __file, int __oaccess);
  void raw_open(int __oaccess=0, int __all=TRUE);
  void raw_close();
  void refresh();
  void save_lastread(GoldBoard board, GoldMsgn msgno);
};


//  ------------------------------------------------------------------

class GoldArea : public gmo_area {

protected:

  GoldWide* wide;

  void data_open();
  void data_close();

  void raw_scan(int __keep_index);

  GoldMsgn get_hdr_idx(gmsg* __msg, char* __file__, int __line__);

  int load_message(int __mode, gmsg* __msg, GoldHdr& __hdr);
  void save_message(int __mode, gmsg* __msg, GoldHdr& __hdr);

public:

  GoldArea() { wide = NULL; }
  virtual ~GoldArea() {}

  //  ----------------------------------------------------------------
  //  Messagebase member functions
  
  void open();
  void close();

  void suspend();
  void resume();

  void lock();
  void unlock();

  void scan();
  void scan_area();
  void scan_area_pm();

  int load_hdr(gmsg* msg);
  int load_msg(gmsg* msg);

  void save_hdr(int mode, gmsg* msg);
  void save_msg(int mode, gmsg* msg);

  void del_msg(gmsg* msg);

  void new_msgno(gmsg* msg);
  char* user_lookup(char* lookfor);
  int renumber();

  void update_timesread(gmsg* msg);

  Line* make_dump_msg(Line*& lin, gmsg* msg);
};


//  ------------------------------------------------------------------

extern GoldWide* __FAR goldbasewide;


//  ------------------------------------------------------------------

#if defined(__BORLANDC__) and defined(__MSDOS__)
inline void GoldArea::open() { }
inline void GoldArea::close() { }
inline void GoldArea::suspend() { }
inline void GoldArea::resume() { }
inline void GoldArea::scan() { }
inline void GoldArea::scan_area() { }
inline void GoldArea::scan_area_pm() { }
inline int GoldArea::load_message(int, gmsg*, GoldHdr&) { return FALSE; }
inline int GoldArea::load_hdr(gmsg*) { return FALSE; }
inline int GoldArea::load_msg(gmsg*) { return FALSE; }
inline void GoldArea::lock() { }
inline void GoldArea::unlock() { }
inline void GoldArea::save_message(int, gmsg*, GoldHdr&) { }
inline void GoldArea::save_hdr(int, gmsg*) { }
inline void GoldArea::save_msg(int, gmsg*) { }
inline void GoldArea::del_msg(gmsg*) { }
inline void GoldArea::new_msgno(gmsg*) { }
inline char* GoldArea::user_lookup(char*) { return NULL; }
inline int GoldArea::renumber() { return FALSE; }
inline void GoldArea::update_timesread(gmsg*) { }
inline Line* GoldArea::make_dump_msg(Line*&, gmsg*) { return NULL; }
#endif


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

