
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmohuds.h,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Hudson.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GMHUDS_H
#define __GMHUDS_H


//  ------------------------------------------------------------------
//  Required

#include <gmoarea.h>
#include <gusrhuds.h>
#include <gusrra2.h>


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


//  ------------------------------------------------------------------
//  Defines

#define HUDS_DELETEDMSGNO   0xFFFFu
#define HUDS_MAXBOARD       200
#define HUDS_EXT            ".bbs"
#define HUDS_NAME           "Hudson"


//  ------------------------------------------------------------------
//  Hudson Message numbers

typedef word HudsMsgn;


//  ------------------------------------------------------------------
//  Hudson Board numbers

typedef byte HudsBoard;


//  ------------------------------------------------------------------
//  Hudson Message Attributes

#define HUDS_DELETED    0x01
#define HUDS_NETTRANS   0x02
#define HUDS_NETMSG     0x04
#define HUDS_PVT        0x08
#define HUDS_RECEIVED   0x10
#define HUDS_ECHOTRANS  0x20
#define HUDS_LOCAL      0x40
#define HUDS_GROUPMSG   0x80

#define HUDS_KILLSENT   0x01
#define HUDS_SENT       0x02
#define HUDS_ATTACH     0x04
#define HUDS_CRASH      0x08
#define HUDS_RETRECREQ  0x10
#define HUDS_AUDITREQ   0x20
#define HUDS_RETREC     0x40
#define HUDS_FREQ       0x80


//  ------------------------------------------------------------------
//  Hudson Message Header (MSGHDR.BBS)

struct HudsHdr {
  word    msgno;
  word    replyto;
  word    reply1st;
  word    timesread;    // redefine to replynext?
  word    startrec;
  word    numrecs;
  word    destnet;
  word    destnode;
  word    orignet;
  word    orignode;
  byte    destzone;
  byte    origzone;
  word    cost;
  byte    msgattr;
  byte    netattr;
  byte    board;
  char    time[6];
  char    date[9];
  char    to[36];
  char    by[36];
  char    re[73];
};


//  ------------------------------------------------------------------
//  Hudson Messagebase Information (MSGINFO.BBS)

struct HudsInfo {
  word  low;
  word  high;
  word  total;
  word  active[HUDS_MAXBOARD];
};


//  ------------------------------------------------------------------
//  Hudson Message Header Index (MSGIDX.BBS)

struct HudsIdx {
  word  msgno;
  byte  board;
};


//  ------------------------------------------------------------------
//  Hudson Message Name Index (MSGTOIDX.BBS)

struct HudsToIdx {
  char  name[36];
};


//  ------------------------------------------------------------------
//  Hudson Lastread (LASTREAD.BBS)

typedef word HudsLast[HUDS_MAXBOARD];


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------

struct HudsScan {  
  uint count;
  uint active;
  ulong lastread;
  uint lastreadreln;
  ulong lastreadfound;
  ulong firstmsgno;
  ulong lastmsgno;
  uint pmcount;
};


//  ------------------------------------------------------------------

struct HudsWide {
  int      fhtxt;
  int      fhhdr;
  int      fhidx;
  int      fhinf;
  int      fhlrd;
  int      fhtoi;
  int      fhusr;
  int      fhuix;
  int      fhuxi;
  int      isopen;
  int      islocked;
  int      timesposted;
  long      msgidxsize;
  HudsIdx*  msgidxptr;
  HudsInfo msginfo;
  HudsLast lastrec;
  GUser*    user;
  int      userno;
  HudsScan* scn;
  HudsIdx*  pmscan;
  uint     pmscantotal;
  int      iswideopen;
  int      ispmscanned;
  int      iswidescanned;
  const char* path;
  const char* syspath;
  long sizewarn;
  int  ra2usersbbs;

  void init();
  void exit();

  void open();
  void close();

  void lock();
  void unlock();

  void scan();

  void raw_open_scan();
  void scan_pm();

  void realloc_pm_scan();

  void update_netecho(char* __name, HudsMsgn __hdridx, int __delete);

  int test_open(char* __file, int __oaccess);
  void raw_open(int __oaccess=0, int __all=TRUE);
  void raw_close();
  void refresh();
  void save_lastread(HudsBoard board, HudsMsgn msgno);
};


//  ------------------------------------------------------------------

class HudsArea : public gmo_area {

protected:

  HudsWide* wide;

  void data_open();
  void data_close();

  void raw_scan(int __keep_index);

  HudsMsgn get_hdr_idx(gmsg* __msg, char* __file__, int __line__);

  int load_message(int __mode, gmsg* __msg, HudsHdr& __hdr);
  void save_message(int __mode, gmsg* __msg, HudsHdr& __hdr);
  
public:

  HudsArea() { wide = NULL; }
  virtual ~HudsArea() {}

  //  ----------------------------------------------------------------
  //  Messagebase member functions
  
  void open();
  void close();

  void suspend();
  void resume();

  void lock();
  void unlock();

  void scan();
  void scan_area();
  void scan_area_pm();

  int load_hdr(gmsg* msg);
  int load_msg(gmsg* msg);

  void save_hdr(int mode, gmsg* msg);
  void save_msg(int mode, gmsg* msg);

  void del_msg(gmsg* msg);

  void new_msgno(gmsg* msg);
  char* user_lookup(char* lookfor);
  int renumber();

  void update_timesread(gmsg* msg);

  Line* make_dump_msg(Line*& lin, gmsg* msg);
};


//  ------------------------------------------------------------------

extern HudsWide* __FAR hudsonwide;


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

