
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbezyc.h,v 1.1.1.1 1998/11/21 19:38:14 odinn Exp $
//  ------------------------------------------------------------------
//  Ezycom msgbase.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GMEZYC_H
#define __GMEZYC_H


//  ------------------------------------------------------------------

#include <gmbarea.h>
#include <gusrezyc.h>

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


//  ------------------------------------------------------------------
//  Ezycom constants

#define EZYC_MAXNAME      36
#define EZYC_MAXSUBJ      73
#define EZYC_MAXMESS102 1024
#define EZYC_MAXMESS110 1536
#define EZYC_USERSBUF    100

#define EZYC_USERDELETED 0x0001


//  ------------------------------------------------------------------

const byte EZYC_MSGATTR_DELETED  = 0x01;
const byte EZYC_MSGATTR_NETPEND  = 0x02;
const byte EZYC_MSGATTR_RESERVED = 0x04;
const byte EZYC_MSGATTR_PRIVATE  = 0x08;
const byte EZYC_MSGATTR_RECEIVED = 0x10;
const byte EZYC_MSGATTR_ECHOPEND = 0x20;
const byte EZYC_MSGATTR_LOCAL    = 0x40;
const byte EZYC_MSGATTR_NOKILL   = 0x80;

const byte EZYC_NETATTR_KILLSENT = 0x01;
const byte EZYC_NETATTR_SENT     = 0x02;
const byte EZYC_NETATTR_ATTACH   = 0x04;
const byte EZYC_NETATTR_CRASH    = 0x08;
const byte EZYC_NETATTR_FREQ     = 0x10;
const byte EZYC_NETATTR_RREQ     = 0x20;
const byte EZYC_NETATTR_AREQ     = 0x40;
const byte EZYC_NETATTR_RREC     = 0x80;

const byte EZYC_EXTATTR_RSVD1    = 0x01;
const byte EZYC_EXTATTR_RSVD2    = 0x02;
const byte EZYC_EXTATTR_RSVD3    = 0x04;
const byte EZYC_EXTATTR_RSVD4    = 0x08;
const byte EZYC_EXTATTR_RSVD5    = 0x10;
const byte EZYC_EXTATTR_RSVD6    = 0x20;
const byte EZYC_EXTATTR_RSVD7    = 0x40;
const byte EZYC_EXTATTR_SEEN     = 0x80;


//  ------------------------------------------------------------------
//  Ezycom message header

struct EzycHdr {

  word    replyto;
  word    reply1st;

  ulong   startposition;
  ulong   messagelength;

  ftn_addr destnet;
  ftn_addr orignet;

  word    cost;           // redefine to replynext?

  byte    msgattr;
  byte    netattr;
  byte    extattr;

  gfiletime posttimedate;
  gfiletime recvtimedate;

  char    whoto[EZYC_MAXNAME];
  char    whofrom[EZYC_MAXNAME];
  char    subject[EZYC_MAXSUBJ];
};


//  ------------------------------------------------------------------
//  Ezycom LASTCOMB.BBS record structure

struct EzycLast {
  word combinedinfo;
  word lastreadinfo[16];
};


//  ------------------------------------------------------------------
//  Ezycom MSGFAST.BBS record structure

struct EzycFast {
  dword whoto;       // 32 bit CRC on WhoTo in MSGHxxx.BBS
  word msgboard;
  word msgnumber;
};


//  ------------------------------------------------------------------
//  Ezycom MSGDLTD.BBS record

struct EzycDeleted {
  word msgboard;
  word msgnumber;
};


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------

class gmb_ezycom_area : public gmb_area {

protected:

  gmb_ezycom_manager* wide;

  int fhhdr;
  int fhtxt;
  int fhnow;
  int omode;
  int smode;
  int timesposted;
  int islocked;
  gstring ezyfile;

  gstring& ret_mess_xxx(gstring& path, byte type);
  char* ret_mess_area(char* path);
  int   raw_open();
  void  test_raw_open(int fileline);
  void  raw_close();
  int   test_open(const char* file, int mode, int share);
  void  raw_scan(int keep_index);
  bool  load_message(int mode, gmsg& msg, EzycHdr& hdr);
  void  save_message(int mode, gmsg& msg, EzycHdr& hdr);

public:

  gmb_ezycom_area(gareacfg* c, gmb_ezycom_manager* m) : gmb_area(c) { wide = m; }
  virtual ~gmb_ezycom_area() { }

  void init(gareacfg* c, gmb_ezycom_manager* m)  { gmb_area::init(c); wide = m; }

  void activate();
  void deactivate();

  void open();
  void close();

  void save_lastread();

  void suspend();
  void resume();

  void lock();
  void unlock();

  void scan();
  void scan_area();
  void scan_area_pm();

  bool load_hdr(gmsg& msg);
  bool load_msg(gmsg& msg);

  void save_hdr(int mode, gmsg& msg);
  void save_msg(int mode, gmsg& msg);

  void del_msg(gmsg& msg);

  void new_msgno(gmsg& msg);
  bool user_lookup(gstring& lookfor);
  bool renumber();

  void update_timesread(gmsg& msg);

  void hexdump_msg(gmsg& msg, gstring& dump);

};


//  ------------------------------------------------------------------

class gmb_ezycom_manager : public gmb_area_manager {

protected:

  int         ver;
  EzycomUser  user;
  int         maxmess;

  gstring     msgbasepath;
  gstring     userbasepath;
  int         userno;

  friend class gmb_ezycom_area;

public:

  gmb_ezycom_manager();
  ~gmb_ezycom_manager();

  void initialize();
  void reset();

  void wide_open()   { }
  void wide_close()  { }

  gmb_area* new_area(gareacfg* c);
  void init_area(gmb_area* a, gareacfg* c);

  void set_msgbasepath(const gstring& m)   { msgbasepath = m; }
  void set_userbasepath(const gstring& u)  { userbasepath = u; }
  void set_userno(int u)                   { userno = u; }

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

