
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbezyc1.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Ezycom msgbase handling
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <stdlib.h>

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>
#include <glog.h>

#include <gmbezyc.h>


//  ------------------------------------------------------------------

gmb_ezycom_manager::gmb_ezycom_manager() {

  ver = 0;
  userno = 0;
  maxmess = 0;
}


//  ------------------------------------------------------------------

gmb_ezycom_manager::~gmb_ezycom_manager() {

  reset();
}


//  ------------------------------------------------------------------

void gmb_ezycom_manager::initialize() {

  Path _path;
  *_path = NUL;
  char* _ptr = getenv("EZY");
  if(_ptr and *_ptr) {
    _ptr = strcpy(_path, _ptr);
    char* _ptr2 = strchr(_ptr, ' ');
    if(_ptr2)
      *_ptr2 = NUL;
    AddBackslash(_path);
  }
  const char* _file = "";
  _ptr = getenv("TASK");
  if(_ptr and *_ptr) {
    char _tmp[20];
    sprintf(_tmp, "CONFIG.%u", atoi(_ptr));
    _file = AddPath(_path, _tmp);
  }
  if(not fexist(_file))
    _file = AddPath(_path, "CONFIG.EZY");

  ver = 102;
  maxmess = EZYC_MAXMESS102;
  int _fh = ::sopen(_file, O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
  if(_fh != -1) {
    char _verstr[9];
    read(_fh, _verstr, 9);
    close(_fh);
    strp2c(_verstr);
    if(strnicmp(_verstr, "1.10", 4) >= 0) {
      ver = 110;
      maxmess = EZYC_MAXMESS110;
    }
  }

  const char* _username = gmb_wide_username[0];
  user.ver = ver;
  if(userno == -1) {
    user.fh = ::sopen(AddPath(userbasepath.c_str(), "USERS.BBS"), O_RDWR|O_CREAT|O_BINARY, gmb_wide_sharemode, S_STDRW);
    if(user.fh != -1) {
      user.extfh = ::sopen(AddPath(userbasepath.c_str(), "USERSEXT.BBS"), O_RDWR|O_CREAT|O_BINARY, gmb_wide_sharemode, S_STDRW);
      if(user.extfh != -1) {
        user.find(_username);
        if(not user.found) {
          gmb_wide_log->printf("* User \"%s\" not found in %sUSERS.BBS.", _username, userbasepath.c_str());
          user.add(_username);
          gmb_wide_log->printf("* Now added with user number %u.", user.index);
        }
        close(user.extfh);
      }
      close(user.fh);
    }
    userno = user.index;
  }
}


//  ------------------------------------------------------------------

void gmb_ezycom_manager::reset() {

  for(int n=0; n<active.size(); n++)
    delete active[n];
  active.reset();
}


//  ------------------------------------------------------------------

gmb_area* gmb_ezycom_manager::new_area(gareacfg* c) {

  gmb_ezycom_area* area = new gmb_ezycom_area(c, this);
  throw_new(area);
  return area;
}


//  ------------------------------------------------------------------

void gmb_ezycom_manager::init_area(gmb_area* a, gareacfg* c) {

  gmb_ezycom_area* area = (gmb_ezycom_area*)a;
  area->init(c, this);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::activate() {

  refs++;
  if(refs == 1) {
    fhhdr = fhtxt = fhnow = -1;
    omode = O_RDONLY;
    smode = SH_DENYNO;
    timesposted = 0;
  }
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::deactivate() {

  if(refs) {
    if(refs == 1) {
      if(open_refs) {
        open_refs = 1;
        close();
      }
    }
    refs--;
  }
}


//  ------------------------------------------------------------------

gstring& gmb_ezycom_area::ret_mess_xxx(gstring& path, byte type) {

  Path cpath;
  if(wide->ver >= 110) {
    sprintf(cpath, "%sAREA%u\\M%c%05u.BBS",
      wide->msgbasepath.c_str(),
      ((board()-1) / 100) + 1,
      (type == 1) ? 'H' : 'T',
      board()
    );
  }
  else {
    sprintf(cpath, "%sAREA%u\\MSG%c%03u.BBS",
      wide->msgbasepath.c_str(),
      ((board()-1) / 100) + 1,
      (type == 1) ? 'H' : 'T',
      (board() > 999) ? board()/10 : board()
    );
  }
  path = cpath;
  return path;
}


//  ------------------------------------------------------------------

char* gmb_ezycom_area::ret_mess_area(char* __path) {

  sprintf(__path, "%sAREA%u",
    wide->msgbasepath.c_str(),
    ((board()-1) / 100) + 1
  );
  return __path;
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::raw_close() {

  GFTRK("EzycomRawClose");

  if(fhhdr != -1)  ::close(fhhdr);  fhhdr = -1;
  if(fhtxt != -1)  ::close(fhtxt);  fhtxt = -1;
  if(fhnow != -1)  ::close(fhnow);  fhnow = -1;
  if(omode == O_WRONLY)
    remove(AddPath(wide->msgbasepath.c_str(), "EZYMSG.NOW"));

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

int gmb_ezycom_area::test_open(const char* __file, int __mode, int __share) {

  GFTRK("EzycomTestOpen");

  int _fh;
  long _tries = 0;
  Path _path;
  strcpy(_path, __file);

  do {

    _fh = ::sopen(_path, __mode, __share, S_STDRW);
    if(_fh == -1) {

      // Tell the world
      if(PopupLocked(++_tries, false, _path) == false) {
        gmb_wide_log->ErrOpen();
        raw_close();
        gmb_wide_log->printf("! A Ezycom msgbase file could not be opened.");
        gmb_wide_log->printf(": %s.", _path);
        gmb_wide_log->ErrOSInfo();
        OpenErrorExit();
      }
    }
  } while(_fh == -1);

  // Remove the popup window
  if(_tries)
    PopupLocked(0, 0, NULL);

  GFTRK(NULL);

  return _fh;
}


//  ------------------------------------------------------------------

int gmb_ezycom_area::raw_open() {

  GFTRK("EzycomRawOpen");

  int _tryagain = 0;
  int _x_errno;
  int _x_doserrno;

  do {

    int _sopen_access = omode | O_BINARY;
    int _sopen_permit = 0;

    if(not fexist(ret_mess_xxx(ezyfile,1).c_str())) {
      _sopen_access |= O_CREAT;
      _sopen_permit = S_STDRW;
    }

    ret_mess_xxx(ezyfile, 1);
    fhhdr = ::sopen(ezyfile.c_str(), _sopen_access, smode, _sopen_permit);
    if(fhhdr != -1) {
      ret_mess_xxx(ezyfile, 2);
      fhtxt = ::sopen(ezyfile.c_str(), _sopen_access, smode, _sopen_permit);
      if(fhtxt != -1) {
        if(omode == O_WRONLY) {

          // Create semaphore file
          byte _sema = 0;
          fhnow = ::sopen(AddPath(wide->msgbasepath.c_str(), "EZYMSG.NOW"), O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, gmb_wide_sharemode, S_STDRW);
          write(fhnow, &_sema, 1);  // Write some dummy data
        }

        GFTRK(NULL);
        return true;
      }
    }
    _x_errno = errno;
    _x_doserrno = _doserrno;
    raw_close();
    errno = _x_errno;
    _doserrno = _x_doserrno;
    if(fexist(AddPath(wide->msgbasepath.c_str(), "EZYMSG.NOW"))) {
      _x_errno = _x_doserrno = EACCES;
      break;
    }
    Path _path;
    if(not is_dir(ret_mess_area(_path))) {
      mkdir(ret_mess_area(_path), S_IWUSR);
      _tryagain++;
    }
  } while(_tryagain == 1);

  errno = _x_errno;
  _doserrno = _x_doserrno;

  GFTRK(NULL);

  return false;
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::test_raw_open(int __fileline) {

  GFTRK("EzycomTestRawOpen");

  int _isopen;
  long _tries = 0;

  do {

    _isopen = raw_open();

    if(not _isopen) {

      // Tell the world
      if((errno != EACCES) or PopupLocked(++_tries, false, ezyfile.c_str()) == false) {

        // User requested to exit
        gmb_wide_log->erropen(__FILE__, __fileline);
        gmb_wide_log->printf("! A Ezycom msgbase file could not be opened.");
        gmb_wide_log->printf(": %s.", ezyfile.c_str());
        gmb_wide_log->ErrOSInfo();
        OpenErrorExit();
      }
    }
  } while(not _isopen);

  // Remove the popup window
  if(_tries)
    PopupLocked(0, 0, NULL);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::open() {

  GFTRK("EzycomOpen");

  open_refs++;
  if(open_refs == 1) {
    test_raw_open(__LINE__);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::close() {

  GFTRK("EzycomClose");

  if(open_refs) {
    if(open_refs == 1) {
      raw_close();
      msgnset.reset();
      pmrkset.reset();
    }
    open_refs--;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::save_lastread() {

  GFTRK("EzycomSaveLastread");

  int _fh = test_open(AddPath(wide->userbasepath.c_str(), "LASTCOMB.BBS"), O_RDWR|O_CREAT|O_BINARY, SH_DENYNO);
  if(_fh != -1) {
    word _lastread = (word)(msgnset.relative_to_absolute(lastread)+1);
    lseekset(_fh, (long)wide->userno * ((long)wide->maxmess / 16) * (long)sizeof(EzycLast) +
      (((board() - 1) / 16) * sizeof(EzycLast) + sizeof(word)) +
      (board()-1) % 16 * sizeof(word)
    );
    write(_fh, &_lastread, sizeof(word));
    ::close(_fh);
  }

  if(timesposted) {
    wide->user.extfh = ::sopen(AddPath(wide->userbasepath.c_str(), "USERSEXT.BBS"), O_RDWR|O_BINARY, SH_DENYNO, S_STDRW);
    if(wide->user.extfh != -1) {
      wide->user.moveto(wide->userno);
      wide->user.inctimesposted(timesposted);
      timesposted = 0;
      ::close(wide->user.extfh);
    }
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::suspend() {

  GFTRK("EzycomSuspend");

  save_lastread();
  raw_close();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::resume() {

  GFTRK("EzycomResume");

  raw_open();
  if(lock_refs)
    lock();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

