
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbezyc3.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Ezycom msgbase handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>
#include <gutlmisc.h>

#include <gmbezyc.h>


//  ------------------------------------------------------------------

bool gmb_ezycom_area::load_message(int mode, gmsg& msg, EzycHdr& hdr) {

  // Load the header
  memset(&hdr, 0, sizeof(EzycHdr));
  lseekset(fhhdr, msg.msgno-1, sizeof(EzycHdr));
  read(fhhdr, &hdr, sizeof(EzycHdr));

  // Convert header data

  msg.link.to_set(hdr.replyto);
  msg.link.first_set(hdr.reply1st);

  msg.cost = hdr.cost;

  msg.ezycom.startposition = hdr.startposition;
  msg.ezycom.messagelength = hdr.messagelength;

  msg.origname = strnp2c(hdr.whofrom, EZYC_MAXNAME-1);
  msg.destname = strnp2c(hdr.whoto,   EZYC_MAXNAME-1);
  msg.subject  = strnp2c(hdr.subject, EZYC_MAXSUBJ-1);

  msg.origaddr = hdr.orignet;
  if(isnet())
    msg.destaddr = hdr.destnet;

  // Convert date and time
  msg.date.written = hdr.posttimedate;
  msg.date.arrived = hdr.recvtimedate;

  // Convert attributes
  msg.attr.del(hdr.msgattr & EZYC_MSGATTR_DELETED);
  msg.attr.pvt(hdr.msgattr & EZYC_MSGATTR_PRIVATE);
  msg.attr.rcv(hdr.msgattr & EZYC_MSGATTR_RECEIVED);
  msg.attr.loc(hdr.msgattr & EZYC_MSGATTR_LOCAL);
  msg.attr.lok(hdr.msgattr & EZYC_MSGATTR_NOKILL);
  msg.attr.k_s(hdr.netattr & EZYC_NETATTR_KILLSENT);
  msg.attr.snt(hdr.netattr & EZYC_NETATTR_SENT);
  msg.attr.att(hdr.netattr & EZYC_NETATTR_ATTACH);
  msg.attr.cra(hdr.netattr & EZYC_NETATTR_CRASH);
  msg.attr.frq(hdr.netattr & EZYC_NETATTR_FREQ);
  msg.attr.rrq(hdr.netattr & EZYC_NETATTR_RREQ);
  msg.attr.arq(hdr.netattr & EZYC_NETATTR_AREQ);
  msg.attr.rrc(hdr.netattr & EZYC_NETATTR_RREC);
  msg.attr.uns(((hdr.msgattr & EZYC_MSGATTR_NETPEND) or (hdr.msgattr & EZYC_MSGATTR_ECHOPEND)) ? 1 : 0);

  msg.ezycom.extattr = hdr.extattr;
  msg.timesread = (hdr.extattr & EZYC_EXTATTR_SEEN) ? 1 : 0;

  if(mode & GMSG_TXT) {

    ulong textlen = hdr.messagelength;
    if(textlen > gmb_wide_msgsize)
      textlen = gmb_wide_msgsize;

    msg.text.reserve((uint)textlen+1);

    lseekset(fhtxt, hdr.startposition);
    read(fhtxt, msg.text.pointer(), (uint)textlen);

    msg.text.raw_to_lines((uint)textlen);
  }

  GFTRK(NULL);

  return true;
}


//  ------------------------------------------------------------------

bool gmb_ezycom_area::load_hdr(gmsg& msg) {

  GFTRK("EzycomLoadHdr");

  EzycHdr _hdr;
  return load_message(GMSG_HDR, msg, _hdr);
}


//  ------------------------------------------------------------------

bool gmb_ezycom_area::load_msg(gmsg& msg) {

  GFTRK("EzycomLoadMsg");

  EzycHdr _hdr;
  return load_message(GMSG_HDRTXT, msg, _hdr);
}


//  ------------------------------------------------------------------

