
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbezyc4.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Ezycom msgbase handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gcrcall.h>
#include <gdbgerr.h>
#include <gdbgtrk.h>
#include <gstrall.h>
#include <gutlmisc.h>

#include <gmbezyc.h>


//  ------------------------------------------------------------------

void gmb_ezycom_area::lock() {
                        
  GFTRK("EzycomLock");

  lock_refs++;
  if(lock_refs == 1) {
    raw_close();
    omode = O_RDWR;
    smode = SH_DENYWR;
    test_raw_open(__LINE__);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::unlock() {
                        
  GFTRK("EzycomUnlock");

  if(lock_refs) {
    if(lock_refs == 1) {
      raw_close();
      omode = O_RDONLY;
      smode = SH_DENYNO;
      test_raw_open(__LINE__);
    }
    lock_refs--;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::save_message(int mode, gmsg& msg, EzycHdr& hdr) {

  lock();

  if(mode & GMSG_NEW)
    msg.msgno = (filelength(fhhdr) / sizeof(EzycHdr)) + 1;

  // Reset header
  memset(&hdr, 0, sizeof(EzycHdr));

  // Convert attributes
  hdr.msgattr |= (byte)(msg.attr.del() ? EZYC_MSGATTR_DELETED  : 0);
  hdr.msgattr |= (byte)(msg.attr.pvt() ? EZYC_MSGATTR_PRIVATE  : 0);
  hdr.msgattr |= (byte)(msg.attr.rcv() ? EZYC_MSGATTR_RECEIVED : 0);
  hdr.msgattr |= (byte)(msg.attr.loc() ? EZYC_MSGATTR_LOCAL    : 0);
  hdr.msgattr |= (byte)(msg.attr.lok() ? EZYC_MSGATTR_NOKILL   : 0);
  hdr.netattr |= (byte)(msg.attr.k_s() ? EZYC_NETATTR_KILLSENT : 0);
  hdr.netattr |= (byte)(msg.attr.snt() ? EZYC_NETATTR_SENT     : 0);
  hdr.netattr |= (byte)(msg.attr.att() ? EZYC_NETATTR_ATTACH   : 0);
  hdr.netattr |= (byte)(msg.attr.cra() ? EZYC_NETATTR_CRASH    : 0);
  hdr.netattr |= (byte)(msg.attr.frq() ? EZYC_NETATTR_FREQ     : 0);
  hdr.netattr |= (byte)(msg.attr.rrq() ? EZYC_NETATTR_RREQ     : 0);
  hdr.netattr |= (byte)(msg.attr.arq() ? EZYC_NETATTR_AREQ     : 0);
  hdr.netattr |= (byte)(msg.attr.rrc() ? EZYC_NETATTR_RREC     : 0);
  hdr.extattr = msg.ezycom.extattr;
  hdr.extattr |= (byte)(msg.timesread  ? EZYC_EXTATTR_SEEN     : 0);

  if(msg.attr.uns()) {
    if(isnet())
      hdr.msgattr |= EZYC_MSGATTR_NETPEND;
    else if(isecho())
      hdr.msgattr |= EZYC_MSGATTR_ECHOPEND;
  }

  // Delete if requested
  if(mode & GMSG_DELETE)
    hdr.msgattr |= EZYC_MSGATTR_DELETED;

  // Convert header data

  hdr.posttimedate = msg.date.written.filetime();
  hdr.recvtimedate = msg.date.arrived.filetime();

  strc2p(strxcpy(hdr.whoto,   msg.destname.c_str(), sizeof(hdr.whoto)));
  strc2p(strxcpy(hdr.whofrom, msg.origname.c_str(), sizeof(hdr.whofrom)));
  strc2p(strxcpy(hdr.subject, msg.subject.c_str(), sizeof(hdr.subject)));

  hdr.orignet.zone  = msg.origaddr.zone;
  hdr.orignet.net   = msg.origaddr.net;
  hdr.orignet.node  = msg.origaddr.node;
  hdr.orignet.point = msg.origaddr.point;

  hdr.destnet.zone  = msg.destaddr.zone;
  hdr.destnet.net   = msg.destaddr.net;
  hdr.destnet.node  = msg.destaddr.node;
  hdr.destnet.point = msg.destaddr.point;

  hdr.replyto = (word)msg.link.to();
  hdr.reply1st = (word)msg.link.first();

  hdr.cost = (word)msg.cost;

  hdr.startposition = msg.ezycom.startposition;
  hdr.messagelength = msg.ezycom.messagelength;

  if(mode & GMSG_TXT) {

    // Write the message text
    char* txt = msg.text.lines_to_raw();
    uint size = strlen(txt) + 1;
    if((mode & GMSG_NEW) or (size > hdr.messagelength))
      hdr.startposition = filelength(fhtxt);
    lseekset(fhtxt, hdr.startposition);
    write(fhtxt, txt, size);
    hdr.messagelength = size;
  }

  // Write header record
  lseekset(fhhdr, msg.msgno-1, sizeof(EzycHdr));
  write(fhhdr, &hdr, sizeof(EzycHdr));

  int fh;

  // Update MSGEXPRT.BBS, which tells ezymail/ezynet which areas to scan for mail
  fh = test_open(AddPath(wide->msgbasepath.c_str(), "MSGEXPRT.BBS"), O_WRONLY|O_CREAT|O_BINARY, SH_DENYNO);
  if(fh == -1) {
    gmb_wide_log->ErrOpen();
    gmb_wide_log->printf("! A Ezycom msgbase file could not be opened.");
    gmb_wide_log->printf(": %s.", AddPath(wide->msgbasepath.c_str(), "MSGEXPRT.BBS"));
    gmb_wide_log->ErrOSInfo();
    OpenErrorExit();
  }
  byte tmp = true;
  lseekset(fh, board()-1);
  write(fh, &tmp, sizeof(byte));
  ::close(fh);

  // Update MSGFAST.BBS
  EzycFast msgfast;
  msgfast.msgboard = (word)board();
  msgfast.msgnumber = (word)msg.msgno;
  msgfast.whoto = CRC32_MASK_CCITT;
  const char* ptr = msg.destname.c_str();
  while(*ptr) {
    msgfast.whoto = updCrc32((char)toupper(*ptr), msgfast.whoto);
    ptr++;
  }
  fh = test_open(AddPath(wide->msgbasepath.c_str(), "MSGFAST.BBS"), O_WRONLY|O_CREAT|O_BINARY, SH_DENYWR);
  if(fh == -1) {
    gmb_wide_log->ErrOpen();
    gmb_wide_log->printf("! A Ezycom msgbase file could not be opened.");
    gmb_wide_log->printf(": %s.", AddPath(wide->msgbasepath.c_str(), "MSGFAST.BBS"));
    gmb_wide_log->ErrOSInfo();
    OpenErrorExit();
  }
  lseek(fh, 0, SEEK_END);
  write(fh, &msgfast, sizeof(EzycFast));
  ::close(fh);

  // Update MSGCOUNT.BBS
  fh = test_open(AddPath(wide->msgbasepath.c_str(), "MSGCOUNT.BBS"), O_WRONLY|O_CREAT|O_BINARY, SH_DENYNO);
  if(fh == -1) {
    gmb_wide_log->ErrOpen();
    gmb_wide_log->printf("! A Ezycom msgbase file could not be opened.");
    gmb_wide_log->printf(": %s.", AddPath(wide->msgbasepath.c_str(), "MSGCOUNT.BBS"));
    gmb_wide_log->ErrOSInfo();
    OpenErrorExit();
  }

  if(gmb_wide_canlock) {

    // Try to get the lock
    long tries = 0;
    while(::lock(fh, (board()-1)*2, 2) == -1) {

      // Tell the world
      if(PopupLocked(++tries, true, AddPath(wide->msgbasepath.c_str(), "MSGCOUNT.BBS")) == false) {

        // User requested to exit
        gmb_wide_log->ErrLock();
        gmb_wide_log->printf("! A Ezycom msgbase file could not be locked.");
        gmb_wide_log->printf(": %s.", AddPath(wide->msgbasepath.c_str(), "MSGCOUNT.BBS"));
        gmb_wide_log->ErrOSInfo();
        LockErrorExit();
      }
    }

    // Remove the popup window
    if(tries)
      PopupLocked(0, 0, NULL);
  }

  word tmpword = (word)(filelength(fhhdr) / sizeof(EzycHdr));
  lseekset(fh, (board()-1)*2);
  write(fh, &tmpword, sizeof(word));
  ::unlock(fh, (board()-1)*2, 2);
  ::close(fh);

  // Update internals if new
  if(mode & GMSG_NEW) {

    // Count our msgs
    timesposted++;

    // Update internal array
    msgnset.append(msg.msgno);
    scan_count++;
  }

  unlock();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::save_hdr(int mode, gmsg& msg) {

  GFTRK("EzycomSaveHdr");

  EzycHdr hdr;
  save_message(mode|GMSG_HDR, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::save_msg(int mode, gmsg& msg) {

  GFTRK("EzycomSaveMsg");

  EzycHdr hdr;
  save_message(mode|GMSG_HDRTXT, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::del_msg(gmsg& msg) {

  GFTRK("EzycomDelMsg");

  EzycHdr hdr;
  save_message(GMSG_HDR | GMSG_DELETE, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::new_msgno(gmsg& msg) {

  GFTRK("EzycomNewMsgno");

  msg.msgno = (filelength(fhhdr) / sizeof(EzycHdr)) + 1;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_ezycom_area::update_timesread(gmsg&) {

}


//  ------------------------------------------------------------------

