
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbfido.h,v 1.1.1.1 1998/11/21 19:38:14 odinn Exp $
//  ------------------------------------------------------------------
//  Fido/Opus *.MSG
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GMFIDO_H
#define __GMFIDO_H


//  ------------------------------------------------------------------

#include <gmbarea.h>
#include <gusrmax.h>


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


//  ------------------------------------------------------------------

const uint FIDO_SCANBUFSIZE = 100;


//  ------------------------------------------------------------------
//  Fido message attributes

#define FIDO_PVT        0x0001
#define FIDO_CRASH      0x0002
#define FIDO_RECEIVED   0x0004
#define FIDO_SENT       0x0008
#define FIDO_ATTACH     0x0010
#define FIDO_TRANSIT    0x0020
#define FIDO_ORPHAN     0x0040
#define FIDO_KILLSENT   0x0080
#define FIDO_LOCAL      0x0100
#define FIDO_HOLD       0x0200
#define FIDO_RESERVED   0x0400
#define FIDO_FREQ       0x0800
#define FIDO_RETRECREQ  0x1000
#define FIDO_RETREC     0x2000
#define FIDO_AUDITREQ   0x4000
#define FIDO_UPDREQ     0x8000


//  ------------------------------------------------------------------
//  Fido style message header (FTS-0001.015 compatible)

struct FidoHdr {
  char  by[36];
  char  to[36];
  char  re[72];
  char  datetime[20];
  word  timesread;
  word  destnode;
  word  orignode;
  word  cost;
  word  orignet;
  word  destnet;
  union {
    struct {
      word  destzone;
      word  origzone;
      word  destpoint;
      word  origpoint;
    } ftsc;
    struct {
      gopustime  written;
      gopustime  arrived;
    } opus;
  };
  word  replyto;
  word  attr;
  word  reply1st;
};


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------

class gmb_fido_area : public gmb_area {

protected:

  gmb_fido_manager* wide;

  ulong highwatermark;

  char* build_msgname(char* buf, ulong msgno);
  int   test_open(const char* file, int openmode, int sharemode, int fail=NO);
  bool  load_message(int mode, gmsg& msg, FidoHdr& hdr);
  void  save_message(int mode, gmsg& msg, FidoHdr& hdr);

public:

  gmb_fido_area(gareacfg* c, gmb_fido_manager* m) : gmb_area(c) { wide = m; }
  virtual ~gmb_fido_area() { }

  void init(gareacfg* c, gmb_fido_manager* m)  { gmb_area::init(c); wide = m; }

  void activate();
  void deactivate();

  void open();
  void close();

  void save_lastread();

  void suspend();
  void resume();

  void lock();
  void unlock();

  void scan();
  void scan_area();
  void scan_area_pm();

  bool load_hdr(gmsg& msg);
  bool load_msg(gmsg& msg);

  void save_hdr(int mode, gmsg& msg);
  void save_msg(int mode, gmsg& msg);

  void del_msg(gmsg& msg);

  void new_msgno(gmsg& msg);
  bool user_lookup(gstring& lookfor);
  bool renumber();

  void update_timesread(gmsg& msg);

  void hexdump_msg(gmsg& msg, gstring& dump);

  void set_highwater_mark();
  void reset_highwater_mark();

};


//  ------------------------------------------------------------------

class gmb_fido_manager : public gmb_area_manager {

protected:

  MaximusUser user;

  int         userno;
  int         fidohwmarks;
  int         fidonullfix;
  gstring     fidolastread;
  gstring     squishuserpath;

  friend class gmb_fido_area;

public:

  gmb_fido_manager();
  ~gmb_fido_manager();

  void initialize();
  void reset();

  void wide_open()   { }
  void wide_close()  { }

  gmb_area* new_area(gareacfg* c);
  void init_area(gmb_area* a, gareacfg* c);

  void set_lastread(const gstring& l)  { fidolastread = l; }
  void set_hwmarks(int h)              { fidohwmarks = h; }
  void set_nullfix(int n)              { fidonullfix = n; }
  void set_userno(int u)               { userno = u; }
  void set_userpath(const gstring& u)  { squishuserpath = u; }

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

