
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbfido2.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Fido/Opus/FTSC (*.MSG) type handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <stdlib.h>

#include <gdbgtrk.h>
#include <gwildmat.h>
#include <gdirposx.h>

#ifdef __BORLANDC__
//#pragma option -Od
#endif

#include <gmbfido.h>


//  ------------------------------------------------------------------

void gmb_fido_area::scan() {

  GFTRK("gmb_fido_area::scan");

  register uint _active = 0;
  register ulong* _msgnoptr = NULL;

  gposixdir d;
  if(d.open(path().c_str())) {
    while(d.read()) {
      if(gwildmati(d.name(), "*.msg")) {
        register ulong _msgno = (ulong)atol(d.name());
        if(_msgno) {
          if((_active % FIDO_SCANBUFSIZE) == 0) {
            ulong* _msgndx = msgnset.resize(_active+FIDO_SCANBUFSIZE);
            _msgnoptr = _msgndx + _active;
          }
          *_msgnoptr++ = _msgno;
          _active++;
        }
      }
    }
    d.close();
  }

  // Sort the index
  msgnset.resize(_active);
  msgnset.sort();

  // Get the lastread msgno
  word _lastread = 0;
  int _fh = ::sopen(AddPath(path().c_str(), wide->fidolastread.c_str()), O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
  if(_fh != -1) {
    lseekset(_fh, wide->userno, sizeof(word));
    read(_fh, &_lastread, sizeof(word));
    ::close(_fh);
  }

  register uint  _lastread_reln = 0;
  register ulong _firstmsgno = 0;
  register ulong _lastmsgno = 0;
  register ulong _lastreadfound = 0;

  if(_active) {

    register uint _count = 1;

    _msgnoptr   = msgnset.tag;
    _firstmsgno = msgnset.at(0);
    _lastmsgno  = msgnset.at(_active-1);

    while(1) {

      // Set lastread pointer
      if((*_msgnoptr >= _lastread) and (_lastread_reln == 0)) {
        _lastreadfound = *_msgnoptr;
        _lastread_reln = _count - (*_msgnoptr != _lastread ? 1 : 0);
        break;
      }
      if((++_count) > _active)
        break;
      _msgnoptr++;
    }
  }

  // If the exact lastread was not found
  if(_active and (_lastreadfound != _lastread)) {

    // Higher than highest or lower than lowest?
    if(_lastread > _lastmsgno)
      _lastread_reln = _active;
    else if(_lastread < _firstmsgno)
      _lastread_reln = 0;
  }

  // Read highwater mark
  highwatermark = 0;
  if(isecho() and wide->fidohwmarks) {
    FidoHdr _hdr;
    _fh = test_open(AddPath(path().c_str(), "1.msg"), O_RDONLY|O_BINARY, gmb_wide_sharemode);
    if(_fh != -1) {
      read(_fh, &_hdr, sizeof(FidoHdr));
      highwatermark = _hdr.replyto;
      ::close(_fh);
    }
  }

  // Update area data
  pmscan_count = 0;
  scan_count = _active;
  msgnset.resize(_active);
  lastread = _lastread_reln;
  lastreadentry = _lastreadfound;

  if(gmb_wide_debug) {
    gmb_wide_log->printf("- %s: t:%u, l:%u, fm:%lu, hm:%lu, lr:%lu, u:%u",
      echoid().c_str(),
      msgnset.size(),
      lastread,
      _firstmsgno,
      _lastmsgno,
      _lastread,
      wide->userno
    );
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_fido_area::scan_area() {

  GFTRK("gmb_fido_area::scan_area");

  scan();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_fido_area::scan_area_pm() {

  GFTRK("gmb_fido_area::scan_area_pm");

  scan_area();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_fido_area::set_highwater_mark() {

  if(isecho()) {
    open();
    gmsg hmsg;
    hmsg.msgno = 1;
    if(load_hdr(hmsg)) {
      hmsg.link.to_set(msgnset.relative_to_absolute(msgnset.size()));
      save_hdr(GMSG_UPDATE, hmsg);
    }
    close();
  }
}


//  ------------------------------------------------------------------

void gmb_fido_area::reset_highwater_mark() {

  if(isecho()) {
    open();
    gmsg hmsg;
    hmsg.msgno = 1;
    if(load_hdr(hmsg)) {
      hmsg.link.to_set(msgnset.relative_to_absolute(1));
      save_hdr(GMSG_UPDATE, hmsg);
    }
    close();
  }
}


//  ------------------------------------------------------------------

