
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbfido5.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Fido/Opus/FTSC (*.MSG) type handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgtrk.h>
#include <gmemall.h>
#include <gstrall.h>

#include <gmbprot.h>
#include <gmbfido.h>


//  ------------------------------------------------------------------

bool gmb_fido_area::user_lookup(gstring& lookfor) {

  Path userfile;
  PathCopy(userfile, AddPath(wide->squishuserpath.c_str(), "USER.BBS"));
  wide->user.fh = ::sopen(userfile, O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
  char buf[256];
  if(wide->user.findwild(lookfor.c_str(), buf))
    lookfor = buf;
  ::close(wide->user.fh);

  return wide->user.found;
}


//  ------------------------------------------------------------------

bool gmb_fido_area::renumber() {

  GFTRK("FidoRenumber");

  // Save the current lastread
  save_lastread();

  // Rescan the area
  scan();

  // In echo or local, start with 2.MSG so we don't conflict
  // with highwater marks, unless there is already a 1.MSG
  ulong _msgno1st = 1;
  if((not isnet()) and (msgnset.at(0) != 1))
    _msgno1st++;

  // Renumber *.MSG files
  register uint _count = 0;
  register ulong _msgno = _msgno1st;
  while(_count < msgnset.size()) {

    // Only renumber a msg if different
    if(_msgno != msgnset.at(_count)) {

      Path _oldname, _newname;
      sprintf(_oldname, "%s%lu.msg", path().c_str(), msgnset.at(_count));
      sprintf(_newname, "%s%lu.msg", path().c_str(), _msgno);

      // Get the file attribute of the message
      struct stat st;
      stat(_oldname, &st);
      bool readonly = not (st.st_mode & S_IWUSR);
      if(readonly)
        chmod(_oldname, S_STDRW);

      // Show progress
      char buf[100];
      sprintf(buf,
        "%s: %lu.msg -> %lu.msg%s",
        "Renumbering",
        msgnset.at(_count),
        _msgno,
        readonly ? " (locked)" : ""
      );
      FidoRenumberProgress(buf);

      // Update the msg
      int _fh = test_open(_oldname, O_RDWR|O_BINARY, SH_DENYWR);
      if(_fh != -1) {

        // Read the header
        FidoHdr _hdr;
        read(_fh, &_hdr, sizeof(FidoHdr));

        // Update the replylinks
        if(_hdr.replyto or _hdr.reply1st) {
          _hdr.replyto  = (word)msgnset.absolute_to_relative(_hdr.replyto);
          _hdr.reply1st = (word)msgnset.absolute_to_relative(_hdr.reply1st);
          lseekset(_fh, 0);
          write(_fh, &_hdr, sizeof(FidoHdr));
        }
        ::close(_fh);

        // Rename the message file
        rename(_oldname, _newname);
        if(readonly)
          chmod(_oldname, S_STDRD);
      }
    }
    _msgno++;
    _count++;
  }

  // Renumber the personal mail marks
  for(_count=0; _count<pmrkset.size(); _count++)
    pmrkset.at(_count) = msgnset.absolute_to_relative(pmrkset.at(_count)) + _msgno1st - 1;

  // Renumber the index
  for(_count=0; _count<msgnset.size(); _count++)
    msgnset.at(_count) = _count + _msgno1st;

  GFTRK(NULL);

  return true;
}


//  ------------------------------------------------------------------
//  Hexdump the current message header

void gmb_fido_area::hexdump_msg(gmsg& msg, gstring& dump) {

  GFTRK("FidoMakeDump");

  FidoHdr _hdr;
  load_message(GMSG_HDRTXT, msg, _hdr);

  char buf[100];

  dump =  "Hexdump of Fido/Opus-style message header and text\n";
  dump += "------------------------------------------------------------------------------\n";
  dump += "\n";
  dump += gstrf("File     : %s%lu.msg\n", path().c_str(), msg.msgno);
  dump += gstrf("From     : %-35.35s\n", _hdr.by);
  dump += gstrf("To       : %-35.35s\n", _hdr.to);
  dump += gstrf("Subject  : %-72.72s\n", _hdr.re);
  dump += gstrf("DateTime : %-20.20s\n", _hdr.datetime);
  if(isopus())
    dump += gstrf("OrigAddr : %u/%u\n", _hdr.orignet, _hdr.orignode);
  else
    dump += gstrf("OrigAddr : %u:%u/%u.%u\n", _hdr.ftsc.origzone, _hdr.orignet, _hdr.orignode, _hdr.ftsc.origpoint);
  if(isopus())
    dump += gstrf("DestAddr : %u/%u\n", _hdr.destnet, _hdr.destnode);
  else
    dump += gstrf("DestAddr : %u:%u/%u.%u\n\n", _hdr.ftsc.destzone, _hdr.destnet, _hdr.destnode, _hdr.ftsc.destpoint);
  dump += gstrf("Reply    : %u  See : %u\n", _hdr.replyto, _hdr.reply1st);
  dump += gstrf("TimesRead: %u  Cost: %u\n", _hdr.timesread, _hdr.cost);
  dump += gstrf("Attr     : %u (%04X)\n", *(word*)&_hdr.attr, *(word*)&_hdr.attr);
  if(isopus()) {
    dump += gstrf("Written  : %s (%08lXh)\n", _hdr.opus.written.c_str(buf), _hdr.opus.written.number());
    dump += gstrf("Arrived  : %s (%08lXh)\n", _hdr.opus.arrived.c_str(buf), _hdr.opus.arrived.number());
  }
  dump += gstrf("UserRecno: %u (%s)\n", wide->userno, gmb_wide_username[0]);

  dump += "\n";
  dump += "Hexdump of message header:";
  dump += "\n";

  addhexdump(dump, &_hdr, sizeof(FidoHdr), "\n");

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

