
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbgold.h,v 1.1.1.1 1998/11/21 19:38:14 odinn Exp $
//  ------------------------------------------------------------------
//  Goldbase.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GMGOLD_H
#define __GMGOLD_H


//  ------------------------------------------------------------------
//  Required

#include <gmbarea.h>
#include <gusrgold.h>


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


//  ------------------------------------------------------------------
//  Defines

#define GOLD_DELETEDMSGNO  0xFFFFFFFFL
#define GOLD_MAXBOARD      500
#define GOLD_EXT           ".DAT"
#define GOLD_NAME          "Goldbase"


//  ------------------------------------------------------------------
//  Goldbase Message numbers

typedef dword GoldMsgn;


//  ------------------------------------------------------------------
//  Goldbase Board numbers

typedef word GoldBoard;


//  ------------------------------------------------------------------
//  Goldbase Message Attributes

#define GOLD_DELETED    0x01
#define GOLD_NETTRANS   0x02
#define GOLD_NETMSG     0x04
#define GOLD_PVT        0x08
#define GOLD_RECEIVED   0x10
#define GOLD_ECHOTRANS  0x20
#define GOLD_LOCAL      0x40
#define GOLD_GROUPMSG   0x80

#define GOLD_KILLSENT   0x01
#define GOLD_SENT       0x02
#define GOLD_ATTACH     0x04
#define GOLD_CRASH      0x08
#define GOLD_RETRECREQ  0x10
#define GOLD_AUDITREQ   0x20
#define GOLD_RETREC     0x40
#define GOLD_FREQ       0x80


//  ------------------------------------------------------------------
//  Goldbase Message Header (MSGHDR.DAT)

struct GoldHdr {
  long    msgno;
  long    replyto;
  long    reply1st;
  word    timesread;
  long    startrec;
  word    numrecs;
  word    destnet;
  word    destnode;
  word    orignet;
  word    orignode;
  byte    destzone;
  byte    origzone;
  word    cost;
  word    msgattr;
  word    netattr;
  word    board;
  char    time[6];
  char    date[9];
  char    to[36];
  char    by[36];
  char    re[73];
};


//  ------------------------------------------------------------------
//  Goldbase Messagebase Information (MSGINFO.DAT)

struct GoldInfo {
  long  low;
  long  high;
  long  total;
  word  active[GOLD_MAXBOARD];
};


//  ------------------------------------------------------------------
//  Goldbase Message Header Index (MSGIDX.DAT)

struct GoldIdx {
  long  msgno;
  word  board;
};


//  ------------------------------------------------------------------
//  Goldbase Message Name Index (MSGTOIDX.DAT)

struct GoldToIdx {
  char  name[36];
};


//  ------------------------------------------------------------------
//  Goldbase Lastread (LASTREAD.DAT)

typedef long GoldLast[GOLD_MAXBOARD];


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------

struct GoldScan {
  uint  count;
  uint  active;
  ulong lastread;
  uint  lastreadreln;
  ulong lastreadfound;
  ulong firstmsgno;
  ulong lastmsgno;
  uint  pmcount;
};


//  ------------------------------------------------------------------

class gmb_goldbase_widearea {

public:

  gstring      path;
  gstring      syspath;
  int          userno;

  int          fhtxt;
  int          fhhdr;
  int          fhidx;
  int          fhinf;
  int          fhlrd;
  int          fhtoi;
  int          fhusr;
  int          timesposted;
  long         msgidxsize;
  GoldIdx*     msgidxptr;
  GoldInfo     msginfo;
  GoldLast     lastrec;
  GoldbaseUser goldbaseuser;
  GUser*       user;
  GoldScan*    scn;
  GoldIdx*     pmscan;
  uint         pmscantotal;
  int          ispmscanned;
  int          iswidescanned;
  int          refs;
  int          open_refs;
  int          lock_refs;
  bool         just_scanning;

  void init();
  void reset();

  void activate();
  void deactivate();

  void open();
  void close();

  void lock();
  void unlock();

  void scan();

  void raw_open_scan();
  void scan_pm();

  void realloc_pm_scan();

  void update_netecho(char* name, GoldMsgn hdridx, int deleteyn);

  int  test_open(char* file, int oaccess);
  void raw_open(int oaccess=0);
  void raw_close();
  void raw_lock();
  void refresh();
  void save_lastread(GoldBoard board, GoldMsgn msgno);

};


//  ------------------------------------------------------------------

class gmb_goldbase_manager : public gmb_area_manager {

protected:

  gmb_goldbase_widearea wide;

  friend class gmb_goldbase_area;

public:

  gmb_goldbase_manager();
  ~gmb_goldbase_manager();

  void initialize();
  void reset();

  void wide_open()   { wide.just_scanning = true; wide.open(); wide.lock(); }
  void wide_close()  { wide.unlock(); wide.close(); wide.just_scanning = false; }

  gmb_area* new_area(gareacfg* c);
  void init_area(gmb_area* a, gareacfg* c);

  void set_path(const gstring& p)     { wide.path = p; }
  void set_syspath(const gstring& s)  { wide.syspath = s; }
  void set_userno(int u)              { wide.userno = u; }

};


//  ------------------------------------------------------------------

class gmb_goldbase_area : public gmb_area {

protected:

  gmb_goldbase_widearea* wide;

  void raw_scan(int keep_index);

  GoldMsgn get_hdr_idx(gmsg& msg, char* file, int line);

  bool load_message(int mode, gmsg& msg, GoldHdr& hdr);
  void save_message(int mode, gmsg& msg, GoldHdr& hdr);

public:

  gmb_goldbase_area(gareacfg* c, gmb_goldbase_manager* m) : gmb_area(c) { wide = &m->wide; }
  virtual ~gmb_goldbase_area() { }

  void init(gareacfg* c, gmb_goldbase_manager* m)  { gmb_area::init(c); wide = &m->wide; }

  void activate();
  void deactivate();

  void open();
  void close();

  void save_lastread();

  void suspend();
  void resume();

  void lock();
  void unlock();

  void scan();
  void scan_area();
  void scan_area_pm();

  bool load_hdr(gmsg& msg);
  bool load_msg(gmsg& msg);

  void save_hdr(int mode, gmsg& msg);
  void save_msg(int mode, gmsg& msg);

  void del_msg(gmsg& msg);

  void new_msgno(gmsg& msg);
  bool user_lookup(gstring& lookfor);
  bool renumber();

  void update_timesread(gmsg& msg);

  void hexdump_msg(gmsg& msg, gstring& dump);

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

