
//  ------------------------------------------------------------------
//  The Goldware Library.
//  Copyright (C) Odinn Sorensen. All Rights Reserved.
//  ------------------------------------------------------------------
//  $Id: gmbgold1.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Hudson / Goldbase msgbase handling
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <stdlib.h>

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>

#include <gmbprot.h>
#include <gmbgold.h>


//  ------------------------------------------------------------------

#define __GOLDBASE


//  ------------------------------------------------------------------

gmb_goldbase_manager::gmb_goldbase_manager() {

}


//  ------------------------------------------------------------------

gmb_goldbase_manager::~gmb_goldbase_manager() {

  reset();
}


//  ------------------------------------------------------------------

gmb_area* gmb_goldbase_manager::new_area(gareacfg* c) {

  gmb_goldbase_area* area = new gmb_goldbase_area(c, this);
  throw_new(area);
  return area;
}


//  ------------------------------------------------------------------

void gmb_goldbase_manager::init_area(gmb_area* a, gareacfg* c) {

  gmb_goldbase_area* area = (gmb_goldbase_area*)a;
  area->init(c, this);
}


//  ------------------------------------------------------------------

void gmb_goldbase_manager::initialize() {

  wide.init();
}


//  ------------------------------------------------------------------

void gmb_goldbase_manager::reset() {

  wide.reset();
}


//  ------------------------------------------------------------------

void gmb_goldbase_widearea::init() {

  GFTRK("GoldInit");

  if(gmb_wide_debug)
    gmb_wide_log->printf("- Begin init for %s.", path.c_str());

  fhusr = -1;
  fhtxt = fhhdr = fhidx = -1;
  fhinf = fhlrd = fhtoi = -1;
  timesposted = 0;
  msgidxsize = 0;
  msgidxptr = NULL;
  pmscantotal = 0;
  pmscan = NULL;
  scn = NULL;
  just_scanning = false;

  // Open complete msgbase, create if none exists
  if(not fexist(AddPath(path.c_str(), "MSGHDR" GOLD_EXT))) {
    gmb_wide_log->printf("* Creating new msgbase at %s", path.c_str());
    raw_open(O_CREAT);
    if(filelength(fhinf) == 0) {
      memset(&msginfo, 0, sizeof(GoldInfo));
      write(fhinf, &msginfo, sizeof(GoldInfo));
    }
    if(filelength(fhlrd) == 0) {
      memset(lastrec, 0, sizeof(GoldLast));
      write(fhlrd, lastrec, sizeof(GoldLast));
    }
  }
  else {
    raw_open();
  }

  // Lock msgbase to ensure data integrity
  lock();

  // Check if MSGTXT.BBS is approaching dangerous size
  #ifdef __HUDSON
  if(filelength(fhtxt) > sizewarn)
    GoldSizewarn();
  #endif

  // Check for mismatch between the header and the index files
  long _hdrsize = filelength(fhhdr)/(long)sizeof(GoldHdr);
  long _idxsize = filelength(fhidx)/(long)sizeof(GoldIdx);
  long _toisize = filelength(fhtoi)/(long)sizeof(GoldToIdx);
  if((_hdrsize != _idxsize) or (_hdrsize != _toisize)) {
    raw_close();
    HGWarnRebuild();
    gmb_wide_log->ErrIndex();
    gmb_wide_log->printf("! The " GOLD_NAME " msgbase files do not have the same number of records.");
    gmb_wide_log->printf(": %sMSGHDR" GOLD_EXT "   (%lu records).", path.c_str(), _hdrsize);
    gmb_wide_log->printf(": %sMSGIDX" GOLD_EXT "   (%lu records).", path.c_str(), _idxsize);
    gmb_wide_log->printf(": %sMSGTOIDX" GOLD_EXT " (%lu records).", path.c_str(), _toisize);
    gmb_wide_log->printf("+ Advice: You need to run a msgbase index rebuild utility.");
    IndexErrorExit();
  }

  // Detect USERS.BBS format unless user has configured it
  #ifdef __HUDSON
  if(ra2usersbbs == 0) {

    // Get size of USERS.BBS
    long len = filelength(fhusr);

    // Does size match Hudson format?
    int goldmatch = (len % sizeof(GoldUsers)) == 0;
    if(goldmatch)
      ra2usersbbs = 1;

    // Does size match RA2 format?
    int ra2match = (len % sizeof(RA2Users)) == 0;
    if(ra2match)
      ra2usersbbs = 2;

    // If it matches both of them
    if(goldmatch and ra2match) {

      // Check version in CONFIG.RA to make sure
      Path rapath, file;
      char* ptr = getenv("RA");
      if(ptr)
        AddBackslash(strcpy(rapath, ptr));
      MakePathname(file, rapath, "MESSAGES.RA");
      int fh = ::sopen(file, O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
      if(fh != -1) {
        word VersionID = 0;
        read(fh, &VersionID, sizeof(word));
        if(VersionID >= 0x200)
          ra2usersbbs = 2;
        else
          ra2usersbbs = 1;
        ::close(fh);
      }
    }

    // If it does not match either of them
    if(not goldmatch and not ra2match) {
      gmb_wide_log->ErrIndex();
      gmb_wide_log->printf("! The USERS.BBS file has an incorrect size.");
      gmb_wide_log->printf(": %sUSERS.BBS, %lu bytes.", path.c_str(), len);
      gmb_wide_log->printf(": Should be %lu bytes if it's in RA2 format.", (len/(long)sizeof(RA2Users))*(long)sizeof(RA2Users));
      gmb_wide_log->printf(": Should be %lu bytes if it's in Hudson format.", (len/(long)sizeof(HudsUsers))*(long)sizeof(HudsUsers));
      if(ra2usersbbs)
        gmb_wide_log->printf(": It appears to be in %s format.", (ra2usersbbs == 2) ? "RA2" : "Hudson");
      gmb_wide_log->printf("+ Advice: Run a userbase packing utility.");
      IndexErrorExit();
    }
  }
  #endif

  #ifdef __HUDSON
  fhuix = fhuxi = -1;
  if(ra2usersbbs == 2) {
    if(gmb_wide_debug)
      gmb_wide_log->printf("- Using a RA2 format userbase.");
    user = &ra2user;
  }
  else {
    if(gmb_wide_debug)
      gmb_wide_log->printf("- Using a Hudson format userbase.");
    user = &goldbaseuser;
  }
  #else
  user = &goldbaseuser;
  #endif

  // Open RA2 files
  #ifdef __HUDSON
  if(ra2usersbbs == 2) {
    RA2User* _user2 = (RA2User*)user;
    _user2->idxfh = fhuix = test_open("USERSIDX.BBS", O_CREAT);
    _user2->xifh = fhuxi = test_open("USERSXI.BBS", O_CREAT);
  }
  #endif

  // Find user
  const char* _username = gmb_wide_username[0];
  if(userno == -1) {
    user->fh = fhusr;
    user->find(_username);
    if(not user->found) {
      gmb_wide_log->printf("* User \"%s\" not found in %sUSERS" GOLD_EXT ".", _username, path.c_str());
      user->add(_username);
      gmb_wide_log->printf("* Now added with user number %u.", user->index);
    }
    userno = user->index;
  }

  if(gmb_wide_debug)
    gmb_wide_log->printf("- Using user record number %u.", userno);

  // Close RA2 files
  #ifdef __HUDSON
  if(ra2usersbbs == 2) {
    ::close(fhuix);
    ::close(fhuxi);
  }
  #endif

  // Unlock and close
  unlock();
  raw_close();

  if(gmb_wide_debug)
    gmb_wide_log->printf("- End init for %s.", path.c_str());

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_goldbase_widearea::reset() {

  throw_release(msgidxptr);
}


//  ------------------------------------------------------------------

void gmb_goldbase_widearea::save_lastread(GoldBoard board, GoldMsgn msgno) {

  GFTRK("GoldSaveLast");

  // Update lastread record
  GoldMsgn _lastread = lastrec[board-1] = msgno;
  lseekset(fhlrd, (long)userno*(long)sizeof(GoldLast));
  write(fhlrd, lastrec, sizeof(GoldLast));

  // Update user record
  user->fh = fhusr;
  user->moveto(userno);
  if(user->lastread() < _lastread)
    user->lastread(_lastread);
  if(timesposted) {
    user->inctimesposted(timesposted);
    timesposted = 0;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_goldbase_widearea::activate() {

  refs++;
  if(refs == 1) {

  }
}


//  ------------------------------------------------------------------

void gmb_goldbase_widearea::deactivate() {

  if(refs) {
    if(refs == 1) {
      if(open_refs) {
        open_refs = 1;
        close();
      }
    }
    refs--;
  }
}


//  ------------------------------------------------------------------

void gmb_goldbase_area::activate() {

  refs++;
  if(refs == 1) {
    wide->activate();
  }
}


//  ------------------------------------------------------------------

void gmb_goldbase_area::deactivate() {

  if(refs) {
    if(refs == 1) {
      if(open_refs) {
        open_refs = 1;
        close();
        wide->deactivate();
      }
    }
    refs--;
  }
}


//  ------------------------------------------------------------------

void gmb_goldbase_area::open() {

  GFTRK("gmb_goldbase_area::open");

  open_refs++;
  if(open_refs)
    wide->open();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_goldbase_area::close() {

  GFTRK("gmb_goldbase_area::close");

  if(open_refs) {
    if(open_refs == 1) {
      if(lock_refs) {
        lock_refs = 1;
        unlock();
      }
      wide->close();
      msgnset.reset();
      pmrkset.reset();
    }
    open_refs--;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_goldbase_area::save_lastread() {

  wide->save_lastread((GoldBoard)board(), (GoldMsgn)msgnset.relative_to_absolute(lastread));
}


//  ------------------------------------------------------------------

void gmb_goldbase_area::suspend() {

  GFTRK("GoldSuspend");

  save_lastread();
  wide->raw_close();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_goldbase_area::resume() {

  GFTRK("GoldResume");

  wide->raw_open();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

