
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbhuds2.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Hudson / Goldbase msgbase handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <limits.h>
#include <stdlib.h>

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>

#include <gmbhuds.h>


//  ------------------------------------------------------------------

void gmb_hudson_widearea::raw_close() {

  GFTRK("gmb_hudson_widearea::raw_close");

  if(fhtxt != -1)  ::close(fhtxt);  fhtxt = -1;
  if(fhhdr != -1)  ::close(fhhdr);  fhhdr = -1;
  if(fhidx != -1)  ::close(fhidx);  fhidx = -1;
  if(fhinf != -1)  ::close(fhinf);  fhinf = -1;
  if(fhlrd != -1)  ::close(fhlrd);  fhlrd = -1;
  if(fhtoi != -1)  ::close(fhtoi);  fhtoi = -1;
  if(fhusr != -1)  ::close(fhusr);  fhusr = -1;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

int gmb_hudson_widearea::test_open(char* __file, int __oaccess) {
  
  GFTRK("gmb_hudson_widearea::test_open");

  int _fh;
  long _tries = 0;

  __oaccess |= O_RDWR|O_BINARY;

  do {

    Path _testfn;
    strcpy(_testfn, AddPath(path.c_str(), __file));
    int _omode = (__oaccess & O_CREAT) ? S_STDRW : S_STDRD;
    _fh = ::sopen(_testfn, __oaccess, gmb_wide_sharemode, _omode);
    if(_fh == -1) {

      if(errno == ENOENT) {
        if(_tries == 0) {
          __oaccess |= O_CREAT;
          _tries++;
          continue;
        }
      }

      // Request the other program to unlock
      if(errno != ENOENT)
        TouchFile(AddPath(path.c_str(), "MBUNLOCK.NOW"));

      // Tell the world
      if(PopupLocked(++_tries, false, _testfn) == false) {

        // User requested to exit
        gmb_wide_log->ErrOpen();
        raw_close();
        gmb_wide_log->printf("! A " HUDS_NAME " msgbase file could not be opened.");
        gmb_wide_log->printf(": %s.", _testfn);
        gmb_wide_log->ErrOSInfo();
        OpenErrorExit();
      }
    }
  } while(_fh == -1);

  // Remove the popup window
  if(_tries)
    PopupLocked(0, 0, NULL);

  GFTRK(NULL);

  return _fh;
}


//  ------------------------------------------------------------------

void gmb_hudson_widearea::raw_open(int __oaccess) {

  GFTRK("gmb_hudson_widearea::raw_open");

  fhidx = test_open("MSGIDX"   HUDS_EXT, __oaccess);
  fhinf = test_open("MSGINFO"  HUDS_EXT, __oaccess);
  fhlrd = test_open("LASTREAD" HUDS_EXT, __oaccess);
  fhtoi = test_open("MSGTOIDX" HUDS_EXT, __oaccess);
  if(not just_scanning) {
    fhhdr = test_open("MSGHDR"   HUDS_EXT, __oaccess);
    fhtxt = test_open("MSGTXT"   HUDS_EXT, __oaccess);
    fhusr = test_open("USERS"    HUDS_EXT, __oaccess);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_widearea::refresh() {

  GFTRK("gmb_hudson_widearea::refresh");

  // (Re)Allocate memory to hold the complete MSGIDX.BBS/DAT
  msgidxsize = filelength(fhidx);
  #if defined(__MSDOS__) and not defined(__386__)
  if((msgidxsize+sizeof(HudsIdx)) >= UINT_MAX) {
    gmb_wide_log->ErrIndex();
    gmb_wide_log->printf("! A " HUDS_NAME " msgbase index was too large to handle.");
    gmb_wide_log->printf(": %s with %lu msgs.", path.c_str(), (long)msgidxsize/sizeof(HudsIdx));
    gmb_wide_log->printf("+ Info: GoldED-DOS only supports up to %lu msgs in a " HUDS_NAME " msgbase.", (long)(UINT_MAX-1)/sizeof(HudsIdx));
    gmb_wide_log->printf("+ Advice: Purge and pack the msgbase or use GoldED/386, GoldED/2 or GoldED/W32.");
    IndexErrorExit();
  }
  #endif
  msgidxptr = (HudsIdx*)throw_realloc(msgidxptr, (uint)(msgidxsize+sizeof(HudsIdx)));

  // Load MSGIDX.BBS/DAT
  lseek(fhidx, 0, SEEK_SET);
  read(fhidx, msgidxptr, (uint)msgidxsize);

  // Load MSGINFO.BBS/DAT
  lseek(fhinf, 0, SEEK_SET);
  read(fhinf, &msginfo, sizeof(HudsInfo));

  // Load LASTREAD.BBS/DAT
  lseek(fhlrd, (long)userno*(long)sizeof(HudsLast), SEEK_SET);
  read(fhlrd, lastrec, sizeof(HudsLast));

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_widearea::open() {

  GFTRK("gmb_hudson_widearea::open");

  open_refs++;
  if(open_refs == 1) {
    raw_open();
    refresh();
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_widearea::close() {

  GFTRK("gmb_hudson_widearea::close");

  if(open_refs) {
    if(open_refs == 1) {
      if(lock_refs) {
        lock_refs = 1;
        unlock();
      }
      msgidxsize = 0;
      timesposted = 0;
      pmscantotal = 0;
      ispmscanned = false;
      iswidescanned = false;
      throw_release(msgidxptr);
      throw_release(pmscan);
      throw_release(scn);
      raw_close();
    }
    open_refs--;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_widearea::raw_lock() {

  if(gmb_wide_canlock) {

    long _tries = 0;

    // Try to get the lock
    while(::lock(fhinf, sizeof(HudsInfo)+1, 1) == -1) {

      // Tell the world
      if(PopupLocked(++_tries, true, AddPath(path.c_str(), "MSGINFO" HUDS_EXT)) == false) {

        // User requested to exit
        gmb_wide_log->ErrLock();
        raw_close();
        gmb_wide_log->printf("! A " HUDS_NAME " msgbase file could not be locked.");
        gmb_wide_log->printf(": %sMSGINFO" HUDS_EXT ".", path.c_str());
        gmb_wide_log->ErrOSInfo();
        LockErrorExit();
      }

      // Request the other program to unlock
      TouchFile(AddPath(path.c_str(), "MBUNLOCK.NOW"));
    }

    // Remove the popup window
    if(_tries)
      PopupLocked(0, 0, NULL);
  }
}


//  ------------------------------------------------------------------

void gmb_hudson_widearea::lock() {

  GFTRK("gmb_hudson_widearea::lock");

  lock_refs++;
  if(lock_refs == 1) {
    raw_lock();
    refresh();
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_widearea::unlock() {

  GFTRK("gmb_hudson_widearea::unlock");

  if(lock_refs) {
    if(lock_refs == 1) {
      if(gmb_wide_canlock)
        ::unlock(fhinf, sizeof(HudsInfo)+1, 1);
    }
    lock_refs--;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_area::lock() {

  lock_refs++;
  if(lock_refs == 1)
    wide->lock();
}


//  ------------------------------------------------------------------

void gmb_hudson_area::unlock() {

  if(lock_refs) {
    if(lock_refs == 1)
      wide->unlock();
    lock_refs--;
  }
}


//  ------------------------------------------------------------------

void gmb_hudson_widearea::scan() {
    
  GFTRK("gmb_hudson_widearea::scan");

  iswidescanned = true;

  // Alloc scan array
  throw_free(scn);
  scn = (HudsScan*)throw_calloc(HUDS_MAXBOARD, sizeof(HudsScan));

  // Board and scan array pointer
  register int _board;
  register HudsScan* _scan = scn;

  // Init wide scan array
  _board = 0;
  while(_board < HUDS_MAXBOARD) {
    _scan->active = msginfo.active[_board];
    _scan->lastread = lastrec[_board];
    _board++;
    _scan++;
  }

  // Setup MSGIDX pointer and counters
  register HudsIdx* _msgidx_ptr = msgidxptr;
  register uint    _msgidx_count = 0;
  register uint    _msgidx_total = (uint)(msgidxsize/sizeof(HudsIdx));
  int         _invalidboards = 0;

  // Scan the index
  while(_msgidx_count < _msgidx_total) {

    // Is the msg not deleted?
    if(_msgidx_ptr->msgno != HUDS_DELETEDMSGNO) {

      register int _idxboard = _msgidx_ptr->board;
      if(_idxboard and (_idxboard <= HUDS_MAXBOARD)) {

        _scan = scn + (_idxboard - 1);

        // Get message number
        _scan->lastmsgno = _msgidx_ptr->msgno;
        _scan->count++;

        // Set first message number
        if(not _scan->firstmsgno)
          _scan->firstmsgno = _scan->lastmsgno;

        // Set lastread pointer
        if((_scan->lastmsgno >= _scan->lastread) and (_scan->lastreadreln == 0)) {
          _scan->lastreadfound = _scan->lastmsgno;
          _scan->lastreadreln = _scan->count - (_scan->lastmsgno != _scan->lastread ? 1 : 0);
        }
      }
      else {
        _invalidboards++;
      }
    }

    // Go to next record
    _msgidx_count++;
    _msgidx_ptr++;
  }

  if(_invalidboards) {
    gmb_wide_log->printf("! Found %u msgs with an invalid board number (0 or >%u).", _invalidboards, HUDS_MAXBOARD);
    gmb_wide_log->printf("! In the " HUDS_NAME " msgbase at %s.", path.c_str());
    gmb_wide_log->printf(": Info: Your msgbase may be partially corrupted.");
    gmb_wide_log->printf("+ Advice: Run a msgbase index rebuild/recover utility.");
  }

  // Check/fix boards
  _board = 0;
  _scan = scn;
  while(_board < HUDS_MAXBOARD) {

    // Check/fix lastreads
    if(_scan->count and (_scan->lastreadfound != _scan->lastread)) {
      if(_scan->lastread > _scan->lastmsgno)
        _scan->lastreadreln = _scan->count;
      else if(_scan->lastread < _scan->firstmsgno)
        _scan->lastreadreln = 0;
    }

    if(gmb_wide_debug) {
      gmb_wide_log->printf("- b:%u: t:%u, l:%u, fm:%lu, hm:%lu, lr:%lu, u:%u",
        _board,
        _scan->count,
        _scan->lastreadreln,
        _scan->firstmsgno,
        _scan->lastmsgno,
        _scan->lastread,
        userno
      );
    }

    // Check active message count and log it if different
    if(_scan->active != _scan->count) {
      gmb_wide_log->printf("! Counted %u active msgs in " HUDS_NAME " board %u.", _scan->count, _board+1);
      gmb_wide_log->printf("! According to MSGINFO" HUDS_EXT " there should be %u active msgs.", _scan->active);
      gmb_wide_log->printf(": Info: A program did not update MSGINFO" HUDS_EXT " correctly.");
      gmb_wide_log->printf("+ Advice: Run a msgbase index rebuild utility.");
    }

    _board++;
    _scan++;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_area::raw_scan(int __keep_index) {
    
  GFTRK("gmb_hudson_area::raw_scan");

  if(wide->open_refs and not wide->iswidescanned)
    wide->scan();

  // Get wide scan data if any
  if(wide->just_scanning) {

    // Update area data
    register HudsScan* _scan = wide->scn + (board() - 1);
    pmscan_count = 0;
    scan_count = _scan->count;
    lastread = _scan->lastreadreln;
    lastreadentry = _scan->lastreadfound;
  }
  else {

    // Open the msgbase if it wasn't already
    wide->open();

    // Get the number of active msgs in the area
    register HudsBoard _board = (HudsBoard)board();
    register uint _active = wide->msginfo.active[_board-1];
    register HudsMsgn _lastread = wide->lastrec[_board-1];

    // Setup pointers and counts
    register uint    _msg_count = 0;
    register HudsIdx* _msgidx_ptr = wide->msgidxptr;
    register uint    _msgidx_count = 0;
    register uint    _msgidx_total = (uint)(wide->msgidxsize/sizeof(HudsIdx));
    register uint    _lastread_reln = 0;

    // (Re)Allocate index
    if(__keep_index)
      msgnset.resize(_active);

    // Index pointers
    register ulong* _msgno_ptr = msgnset.tag;

    // Fill index
    register ulong _firstmsgno = 0;
    register ulong _lastmsgno = 0;
    register ulong _lastreadfound = 0;
    while(_msgidx_count < _msgidx_total) {

      // Is it our board and is the msg not deleted?
      if((_msgidx_ptr->board == _board) and (_msgidx_ptr->msgno != HUDS_DELETEDMSGNO)) {

        // Get message number
        _lastmsgno = _msgidx_ptr->msgno;
        _msg_count++;

        // Set first message number
        if(not _firstmsgno)
          _firstmsgno = _lastmsgno;

        // Transfer data to the index
        if(__keep_index)
          *_msgno_ptr++ = _lastmsgno;

        // Set lastread pointer
        if((_lastmsgno >= _lastread) and (_lastread_reln == 0)) {
          _lastreadfound = _lastmsgno;
          _lastread_reln = _msg_count - (_lastmsgno != _lastread ? 1 : 0);
        }

        // Break loop as soon as we have all we need
        if(_msg_count == _active)
          break;
      }

      // Go to next record
      _msgidx_count++;
      _msgidx_ptr++;
    }

    // If the exact lastread was not found
    if(_msg_count and (_lastreadfound != _lastread)) {

      // Higher than highest or lower than lowest?
      if(_lastread > _lastmsgno)
        _lastread_reln = _msg_count;
      else if(_lastread < _firstmsgno)
        _lastread_reln = 0;
    }

    // Check active message count and log it if different
    if(_active != _msg_count) {
      gmb_wide_log->printf("! Counted %u active msgs in " HUDS_NAME " board %u (%s).", _msg_count, board(), echoid().c_str());
      gmb_wide_log->printf("! According to MSGINFO" HUDS_EXT " there should be %u active msgs.", _active);
      gmb_wide_log->printf(": Info: A program did not update MSGINFO" HUDS_EXT " correctly.");
      gmb_wide_log->printf("+ Advice: Run a msgbase index rebuild utility.");
    }

    // Update area data
    pmscan_count = 0;
    scan_count = _msg_count;
    msgnset.resize(_msg_count);
    lastread = _lastread_reln;
    lastreadentry = _lastreadfound;

    if(gmb_wide_debug) {
      gmb_wide_log->printf("- b:%u: t:%u, l:%u, fm:%lu, hm:%lu, lr:%lu, u:%u",
        board(),
        msgnset.size(),
        lastread,
        _firstmsgno,
        _lastmsgno,
        _lastread,
        wide->userno
      );
    }

    // Close the msgbase again if we opened it in here
    wide->close();
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_area::scan() {

  GFTRK("gmb_hudson_area::scan");

  raw_scan(true);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_area::scan_area() {

  GFTRK("gmb_hudson_area::scan_area");

  raw_scan(false);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

int _stdc gmb_hudson_idxcmp(const HudsIdx* a, const HudsIdx* b) {

  register int cmp;
  if((cmp = a->board - b->board) != 0)
    return cmp;
  if((cmp = (int)(a->msgno - b->msgno)) != 0)
    return cmp;
  return cmp;
}


//  ------------------------------------------------------------------

#define TOIDXBUFSZ 100u
#define PMSCANBUFSZ 50u

void gmb_hudson_widearea::realloc_pm_scan() {

  pmscan = (HudsIdx*)throw_realloc(pmscan, (pmscantotal+PMSCANBUFSZ)*sizeof(HudsIdx));
}


//  ------------------------------------------------------------------

void gmb_hudson_widearea::scan_pm() {
    
  GFTRK("gmb_hudson_widearea::scan_pm");

  if(not iswidescanned)
    scan();

  ispmscanned = true;

  HudsToIdx* toidxbuf = (HudsToIdx*)throw_calloc(TOIDXBUFSZ, sizeof(HudsToIdx));
  uint totrecs = (uint)(filelength(fhtoi) / sizeof(HudsToIdx));
  HudsIdx* idxptr = msgidxptr;
  throw_release(pmscan);
  pmscantotal = 0;
  int invalidboards = 0;
  int gotpm, idxboard, umax, u;
  uint getrecs, rec;
  HudsToIdx* toidx;

  do {

    rec = 0;
    gotpm = false;
    getrecs = minimum_of_two(TOIDXBUFSZ, totrecs);
    read(fhtoi, toidxbuf, getrecs*sizeof(HudsToIdx));

    for(toidx=toidxbuf; rec<getrecs; idxptr++,toidx++,rec++) {

      // Skip deleted msgs
      if(idxptr->msgno == HUDS_DELETEDMSGNO)
        continue;

      // Skip msgs in invalid boards
      idxboard = idxptr->board;
      if(not (idxboard and (idxboard <= HUDS_MAXBOARD))) {
        invalidboards++;
        continue;
      }

      // If after lastread
      if(idxptr->msgno <= scn[idxboard-1].lastreadfound)
        continue;

      // Convert name string
      strnp2c(toidx->name, sizeof(HudsToIdx)-1);

      // Is it a PM?
      umax = (gmb_wide_personalmail & PM_ALLNAMES) ? gmb_wide_usernames : 1;
      for(u=0; u<umax; u++) {
        if(strieql(toidx->name, gmb_wide_username[u])) {
          gotpm = true;
          break;
        }
      }

      if(gotpm) {
        if((pmscantotal % PMSCANBUFSZ) == 0)
          realloc_pm_scan();
        pmscan[pmscantotal++] = *idxptr;
        scn[idxboard-1].pmcount++;
        gotpm = false;
      }
    }
    totrecs -= getrecs;
  } while(totrecs);

  throw_free(toidxbuf);
  pmscan = (HudsIdx*)throw_realloc(pmscan, (pmscantotal+1)*sizeof(HudsIdx));

  // Sort all pm records in board/msgno order
  qsort(pmscan, pmscantotal, sizeof(HudsIdx), (StdCmpCP)gmb_hudson_idxcmp);

  if(invalidboards) {
    gmb_wide_log->printf("! Found %u msgs with an invalid board number (0 or >%u).", invalidboards, HUDS_MAXBOARD);
    gmb_wide_log->printf("! In the " HUDS_NAME " msgbase at %s.", path.c_str());
    gmb_wide_log->printf(": Info: Your msgbase may be partially corrupted.");
    gmb_wide_log->printf("+ Advice: Run a msgbase index rebuild/recover utility.");
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_area::scan_area_pm() {

  GFTRK("gmb_hudson_area::scan_area_pm");

  pmrkset.reset();

  if(wide->open_refs and not wide->ispmscanned)
    wide->scan_pm();

  // Get wide scan data if any
  if(wide->scn) {

    register HudsBoard _board = (HudsBoard)board();
    register HudsScan* _scan = wide->scn + (_board - 1);

    if(wide->pmscan) {
      if(_scan->pmcount) {

        register HudsIdx* pmscanptr = wide->pmscan;
        while(pmscanptr->board != _board)
          pmscanptr++;

        register uint n = 0;
        register uint cnt = _scan->pmcount;
        while(n < cnt) {
          pmrkset.append(pmscanptr->msgno);
          pmscanptr++;
          n++;
        }
      }
    }

    // Update area data
    scan_count = _scan->count;
    pmscan_count = pmrkset.size();
    msgnset.resize(_scan->count);
    lastread = _scan->lastreadreln;
    lastreadentry = _scan->lastreadfound;

    if(gmb_wide_debug) {
      gmb_wide_log->printf("- b:%u: t:%u, l:%u, fm:%lu, hm:%lu, lr:%lu, u:%u, pm:%i",
        _board,
        _scan->count,
        _scan->lastreadreln,
        _scan->firstmsgno,
        _scan->lastmsgno,
        _scan->lastread,
        wide->userno,
        pmrkset.size()
      );
    }

  }
  else {
    if(gmb_wide_debug)
      gmb_wide_log->printf("- Oops!  Fell into empty bracket.");
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

