
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbhuds3.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Hudson / Goldbase msgbase handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>

#include <gmbhuds.h>


//  ------------------------------------------------------------------

HudsMsgn gmb_hudson_area::get_hdr_idx(gmsg& msg, char* file, int line) {

  GFTRK("HudsGetHdrIdx");

  HudsMsgn _count = 0;
  HudsMsgn _total = (HudsMsgn)(wide->msgidxsize/sizeof(HudsIdx));
  HudsIdx* _msgidx_ptr = wide->msgidxptr;

  register ulong _msgno = msg.msgno;
  if(_msgidx_ptr) {
    while(_count <= _total) {
      if(_msgno == _msgidx_ptr->msgno) {
        GFTRK(NULL);
        return _count;
      }
      _msgidx_ptr++;
      _count++;
    }
  }

  // Report error
  ulong _lmsgno = lastread ? msgnset.at(lastread-1) : msgnset.at(0);
  ulong _lread = wide->lastrec[board()-1];
  ulong _active = wide->msginfo.active[board()-1];
  gmb_wide_log->errindex(file, line);
  gmb_wide_log->printf("! Failed to locate a msgno in an internal " HUDS_NAME " index.");
  gmb_wide_log->printf(": Msgno %lu (%lXh) in board %u (%s,%u,%u).", _msgno, _msgno, board(), echoid().c_str(), msgnset.size(), lastread);
  gmb_wide_log->printf(": First Msgno       : %lu (%lXh).", msgnset.at(0), msgnset.at(0));
  gmb_wide_log->printf(": Msgno at lastread : %lu (%lXh).", _lmsgno, _lmsgno);
  gmb_wide_log->printf(": Real lastread msg : %lu (%lXh).", _lread, _lread);
  gmb_wide_log->printf(": Real active msgs  : %lu (%lXh).", _active, _active);
  if(msgnset.size() > 2) {
    _lmsgno = msgnset.at(msgnset.size()-3);
    gmb_wide_log->printf(": Highest-2 msgno   : %lu (%lXh).", _lmsgno, _lmsgno);
  }
  if(msgnset.size() > 1) {
    _lmsgno = msgnset.at(msgnset.size()-2);
    gmb_wide_log->printf(": Highest-1 msgno   : %lu (%lXh).", _lmsgno, _lmsgno);
  }
  if(msgnset.size()) {
    _lmsgno = msgnset.at(msgnset.size()-1);
    gmb_wide_log->printf(": Highest msgno     : %lu (%lXh).", _lmsgno, _lmsgno);
  }
  if(_msgno == 0xEEEEEEEEL) {
    gmb_wide_log->printf("+ Info: This indicates a serious bug.");
    gmb_wide_log->printf("+ Advice: Report to the Author immediately.");
  }
  gmb_wide_log->printf("+ Advice: Restart or run a msgbase index rebuild utility.");
  IndexErrorExit();

  GFTRK(NULL);

  return HUDS_DELETEDMSGNO;
}


//  ------------------------------------------------------------------

bool gmb_hudson_area::load_message(int mode, gmsg& msg, HudsHdr& hdr) {

  if(msg.msgno == 0) {
    GFTRK(NULL);
    return false;
  }

  // Read header
  HudsMsgn _hdridx = get_hdr_idx(msg, __FILE__, __LINE__);
  lseek(wide->fhhdr, (long)_hdridx*(long)sizeof(HudsHdr), SEEK_SET);
  read(wide->fhhdr, &hdr, sizeof(HudsHdr));

  msg.msgno = hdr.msgno;
  msg.link.to_set(hdr.replyto);
  msg.link.first_set(hdr.reply1st);

  msg.cost = hdr.cost;
  msg.timesread = hdr.timesread;

  msg.origname = strnp2c(hdr.by, sizeof(hdr.by)-1);
  msg.destname = strnp2c(hdr.to, sizeof(hdr.to)-1);
  msg.subject  = strnp2c(hdr.re, sizeof(hdr.re)-1);

  msg.origaddr.zone  = hdr.origzone;
  msg.origaddr.net   = hdr.orignet;
  msg.origaddr.node  = hdr.orignode;
  msg.origaddr.point = 0;

  if(isnet()) {
    msg.destaddr.zone  = hdr.destzone;
    msg.destaddr.net   = hdr.destnet;
    msg.destaddr.node  = hdr.destnode;
    msg.destaddr.point = 0;
  }

  // Convert date and time
  int year, month, day, hour, minute;
  sscanf(hdr.date+1, "%d%*c%d%*c%2d", &month, &day, &year);
  sscanf(hdr.time+1, "%d%*c%2d", &hour, &minute);
  msg.date.written.year   = (year < 80) ? year+2000 : year+1900;
  msg.date.written.month  = month;
  msg.date.written.day    = day;
  msg.date.written.hour   = hour;
  msg.date.written.minute = minute;

  // Convert attributes
  msg.attr.del(hdr.msgattr & HUDS_DELETED);
  msg.attr.pvt(hdr.msgattr & HUDS_PVT);
  msg.attr.rcv(hdr.msgattr & HUDS_RECEIVED);
  msg.attr.loc(hdr.msgattr & HUDS_LOCAL);
  msg.attr.grp(hdr.msgattr & HUDS_GROUPMSG);
  msg.attr.k_s(hdr.netattr & HUDS_KILLSENT);
  msg.attr.att(hdr.netattr & HUDS_ATTACH);
  msg.attr.cra(hdr.netattr & HUDS_CRASH);
  msg.attr.rrq(hdr.netattr & HUDS_RETRECREQ);
  msg.attr.arq(hdr.netattr & HUDS_AUDITREQ);
  msg.attr.rrc(hdr.netattr & HUDS_RETREC);
  msg.attr.frq(hdr.netattr & HUDS_FREQ);
  msg.attr.uns((hdr.netattr & HUDS_SENT) ? 0 : ((hdr.msgattr & (HUDS_NETTRANS|HUDS_ECHOTRANS)) ? 1 : 0));
  msg.attr.snt(msg.attr.loc() and not msg.attr.uns());

  msg.hudson.startrec = hdr.startrec;
  msg.hudson.numrecs = hdr.numrecs;

  // If message text is used
  if(mode & GMSG_TXT) {

    // Get length of message text and adjust if necessary
    uint numrecs = hdr.numrecs;
    long txtlen = (long)numrecs*256L;
    if(txtlen > gmb_wide_msgsize) {
      txtlen = gmb_wide_msgsize;
      numrecs = (uint)(txtlen/256L);
    }

    // Allocate memory for message text
    msg.text.reserve(numrecs*256);
    uint textlen = 0;

    // Read message text
    if(numrecs) {

      // Seek to, and read the raw text
      lseek(wide->fhtxt, (long)hdr.startrec*256L, SEEK_SET);
      read(wide->fhtxt, msg.text.pointer(), numrecs*256);

      // Set up loop variables
      char* src = msg.text.pointer();
      char* dst = src;
      uint count = 0;

      // Conversion loop
      while(count++ < numrecs) {

        // Get block length
        byte len = *src++;
        textlen += len;

        // Move text over the length byte
        memmove(dst, src, len);

        // Move pointers
        dst += len;
        src += len;
      }
    }

    msg.text.set_length(textlen);
    msg.text.raw_to_lines(textlen);
  }

  GFTRK(NULL);

  return true;
}


//  ------------------------------------------------------------------

bool gmb_hudson_area::load_hdr(gmsg& msg) {

  GFTRK("HudsLoadHdr");

  HudsHdr _hdr;
  return load_message(GMSG_HDR, msg, _hdr);
}


//  ------------------------------------------------------------------

bool gmb_hudson_area::load_msg(gmsg& msg) {

  GFTRK("HudsLoadMsg");

  HudsHdr _hdr;
  return load_message(GMSG_HDRTXT, msg, _hdr);
}


//  ------------------------------------------------------------------

