
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbhuds4.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Hudson / Goldbase msgbase handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <stdlib.h>

#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>

#include <gmbhuds.h>


//  ------------------------------------------------------------------

int _stdc gmb_hudson_msgncmp(const HudsMsgn* a, const HudsMsgn* b) {

  return compare_two(*a, *b);
}


//  ------------------------------------------------------------------

void gmb_hudson_widearea::update_netecho(char* filename, HudsMsgn hdridx, int deleteit) {

  GFTRK("HudsUpdateNetEcho");

  Path name;
  strcpy(name, AddPath(syspath.c_str(), filename));
  int fh = test_open(name, O_CREAT);

  // Get size of the file
  uint total = (uint)(filelength(fh)/sizeof(HudsMsgn));

  // Allocate memory for the scanning index
  HudsMsgn* scanidx = (HudsMsgn*)throw_calloc(total+5, sizeof(HudsMsgn));

  // Read the scanning index
  read(fh, scanidx, total*sizeof(HudsMsgn));

  // Close the file
  ::close(fh);

  // Search for the specified header index
  uint pos = 0;
  uint closest = 0;
  while(pos < total) {
    if(scanidx[pos] <= hdridx)
      closest = pos;
    if(scanidx[pos] == hdridx)
      break;
    pos++;
  }

  // Delete or add the header index
  if(deleteit) {
    if(pos < total) {
      memmove(scanidx+pos, scanidx+pos+1, (total-pos-1)*sizeof(HudsMsgn));
      total--;
    }
  }
  else {
    if(scanidx[closest] != hdridx) {
      scanidx[total++] = hdridx;
      qsort(scanidx, total, sizeof(HudsMsgn), (StdCmpCP)gmb_hudson_msgncmp);
    }
  }

  // Write the changed scanning file from scratch
  if(total) {
    fh = test_open(name, O_CREAT|O_TRUNC);
    write(fh, scanidx, total*sizeof(HudsMsgn));
    ::close(fh);
  }
  else {
    // If the last header index was removed, delete the file
    remove(name);
  }

  // Free the memory
  throw_free(scanidx);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_area::save_message(int mode, gmsg& msg, HudsHdr& hdr) {

  // Lock msgbase before doing anything
  lock();

  // Find header index
  HudsMsgn hdridx;
  if(mode & GMSG_NEW) {
    msg.msgno = wide->msginfo.high + 1;
    hdridx = (HudsMsgn)(filelength(wide->fhhdr)/(long)sizeof(HudsHdr));
  }
  else {
    hdridx = get_hdr_idx(msg, __FILE__, __LINE__);
  }

  hdr.board = (HudsBoard)board();
  hdr.msgno = (HudsMsgn)msg.msgno;
  hdr.replyto = (HudsMsgn)msg.link.to();
  hdr.reply1st = (HudsMsgn)msg.link.first();

  sprintf(hdr.date, "%02d-%02d-%02d", msg.date.written.month, msg.date.written.day, msg.date.written.year_twodigits());
  sprintf(hdr.time, "%02d:%02d", msg.date.written.hour, msg.date.written.minute);
  strc2p(hdr.date);
  strc2p(hdr.time);

  strc2p(strxcpy(hdr.to, msg.destname.c_str(), sizeof(hdr.to)));
  strc2p(strxcpy(hdr.by, msg.origname.c_str(), sizeof(hdr.by)));
  strc2p(strxcpy(hdr.re, msg.subject.c_str(), sizeof(hdr.re)));

  hdr.origzone = (byte)msg.origaddr.zone;
  hdr.orignet  =       msg.origaddr.net;
  hdr.orignode =       msg.origaddr.node;

  hdr.destzone = (byte)msg.destaddr.zone;
  hdr.destnet  =       msg.destaddr.net;
  hdr.destnode =       msg.destaddr.node;

  hdr.cost      = (word)msg.cost;
  hdr.timesread = (word)msg.timesread;

  // Transfer attributes
  hdr.msgattr = hdr.netattr = 0;
  hdr.msgattr |= (byte)(msg.attr.del() ? HUDS_DELETED   : 0);
  hdr.msgattr |= (byte)(msg.attr.pvt() ? HUDS_PVT       : 0);
  hdr.msgattr |= (byte)(msg.attr.rcv() ? HUDS_RECEIVED  : 0);
  hdr.msgattr |= (byte)(msg.attr.loc() ? HUDS_LOCAL     : 0);
  hdr.msgattr |= (byte)(msg.attr.grp() ? HUDS_GROUPMSG  : 0);
  hdr.netattr |= (byte)(msg.attr.k_s() ? HUDS_KILLSENT  : 0);
  hdr.netattr |= (byte)(msg.attr.snt() ? HUDS_SENT      : 0);
  hdr.netattr |= (byte)(msg.attr.att() ? HUDS_ATTACH    : 0);
  hdr.netattr |= (byte)(msg.attr.cra() ? HUDS_CRASH     : 0);
  hdr.netattr |= (byte)(msg.attr.rrq() ? HUDS_RETRECREQ : 0);
  hdr.netattr |= (byte)(msg.attr.arq() ? HUDS_AUDITREQ  : 0);
  hdr.netattr |= (byte)(msg.attr.rrc() ? HUDS_RETREC    : 0);
  hdr.netattr |= (byte)(msg.attr.frq() ? HUDS_FREQ      : 0);
  if(isnet())
    hdr.msgattr |= HUDS_NETMSG;
  if(msg.attr.uns()) {
    if(isnet())
      hdr.msgattr |= HUDS_NETTRANS;
    else if(isecho())
      hdr.msgattr |= HUDS_ECHOTRANS;
  }

  if(mode & GMSG_DELETE)
    hdr.msgattr |= HUDS_DELETED;

  hdr.startrec = msg.hudson.startrec;
  hdr.numrecs = msg.hudson.numrecs;

  // Write to MSGTXT.BBS/DAT
  if(mode & GMSG_TXT) {

    register char* txt = msg.text.lines_to_raw();

    // If the msg is new or the text is too large to fit
    uint txtlen = strlen(txt)+1;
    if((mode & GMSG_NEW) or (txtlen > ((long)msg.hudson.numrecs*255L)))
      hdr.startrec = (HudsMsgn)(filelength(wide->fhtxt)/256L);

    // Calculate the number of text records to write
    register uint fullrecs = txtlen / 255;
    register byte extrarec = (byte)(txtlen % 255);
    hdr.numrecs = (word)(fullrecs + (extrarec ? 1 : 0));

    // Seek to the text write position
    lseek(wide->fhtxt, (long)hdr.startrec*256L, SEEK_SET);

    // Write the message text
    register uint count = 0;
    char txtbuf[256];
    char* txtptr = txtbuf + 1;
    *txtbuf = 255;
    while(count < fullrecs) {
      memcpy(txtptr, txt, 255);
      write(wide->fhtxt, txtbuf, 256);
      txt += 255;
      count++;
    }
    if(extrarec) {
      *txtbuf = extrarec;
      memset(txtptr, 0, 255);
      memcpy(txtptr, txt, extrarec);
      write(wide->fhtxt, txtbuf, 256);
    }
  }

  // Write to MSGHDR.BBS/DAT
  lseek(wide->fhhdr, (long)hdridx*(long)sizeof(HudsHdr), SEEK_SET);
  write(wide->fhhdr, &hdr, sizeof(HudsHdr));

  // Write to MSGIDX.BBS/DAT
  if(mode & GMSG_NEW) {
    wide->msgidxsize += sizeof(HudsIdx);
    wide->msgidxptr = (HudsIdx*)throw_realloc(wide->msgidxptr, (uint)(wide->msgidxsize+sizeof(HudsIdx)));
  }
  HudsIdx* idxp = wide->msgidxptr + (uint)hdridx;
  idxp->board = hdr.board;
  idxp->msgno = (HudsMsgn)((mode & GMSG_DELETE) ? HUDS_DELETEDMSGNO : hdr.msgno);
  lseek(wide->fhidx, (long)hdridx*(long)sizeof(HudsIdx), SEEK_SET);
  write(wide->fhidx, idxp, sizeof(HudsIdx));

  // Write to MSGTOIDX.BBS/DAT
  lseek(wide->fhtoi, (long)hdridx*(long)sizeof(HudsToIdx), SEEK_SET);
  if(mode & GMSG_DELETE)
    write(wide->fhtoi, "\xB* Deleted *\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", sizeof(HudsToIdx));
  else if(hdr.msgattr & HUDS_RECEIVED)
    write(wide->fhtoi, "\xC* Received *\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", sizeof(HudsToIdx));
  else
    write(wide->fhtoi, hdr.to, sizeof(HudsToIdx));

  // Write to MSGINFO.BBS/DAT
  if(mode & GMSG_DELETE) {
    wide->msginfo.total--;
    wide->msginfo.active[hdr.board-1]--;
    if(hdr.msgno == wide->msginfo.low) {
      register uint count = 0;
      register HudsMsgn total = (HudsMsgn)(wide->msgidxsize/sizeof(HudsIdx));
      register HudsIdx* msgidx_ptr = wide->msgidxptr;
      while((count++) < total) {
        if(msgidx_ptr->msgno != HUDS_DELETEDMSGNO) {
          wide->msginfo.low = msgidx_ptr->msgno;
          break;
        }
        msgidx_ptr++;
      }
    }
    if(hdr.msgno == wide->msginfo.high) {
      register uint count = (uint)(wide->msgidxsize/sizeof(HudsIdx));
      register HudsIdx* msgidx_ptr = wide->msgidxptr + (count-1);
      while(count--) {
        if(msgidx_ptr->msgno != HUDS_DELETEDMSGNO) {
          wide->msginfo.high = msgidx_ptr->msgno;
          break;
        }
        msgidx_ptr--;
      }
    }
  }
  else if(mode & GMSG_NEW) {
    wide->msginfo.total++;
    wide->msginfo.active[hdr.board-1]++;
    if(hdr.msgno < wide->msginfo.low)
      wide->msginfo.low = hdr.msgno;
    if(hdr.msgno > wide->msginfo.high)
      wide->msginfo.high = hdr.msgno;
  }
  lseek(wide->fhinf, 0, SEEK_SET);
  write(wide->fhinf, &wide->msginfo, sizeof(HudsInfo));

  // Update scanning files
  if((hdr.msgattr & HUDS_NETTRANS) or (mode & GMSG_DELETE))
    wide->update_netecho("NETMAIL" HUDS_EXT, hdridx, hdr.msgattr & HUDS_DELETED);
  if((hdr.msgattr & HUDS_ECHOTRANS) or (mode & GMSG_DELETE))
    wide->update_netecho("ECHOMAIL" HUDS_EXT, hdridx, hdr.msgattr & HUDS_DELETED);

  if(mode & GMSG_NEW) {

    // Count our msgs
    wide->timesposted++;

    // Add new msgno to index
    msgnset.append(msg.msgno);
    scan_count++;
  }

  // Unlock msgbase after use
  unlock();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_area::save_hdr(int mode, gmsg& msg) {

  GFTRK("HudsSaveHdr");

  HudsHdr hdr;
  save_message(mode|GMSG_HDR, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_hudson_area::save_msg(int mode, gmsg& msg) {

  GFTRK("HudsSaveMsg");

  HudsHdr hdr;
  save_message(mode|GMSG_HDRTXT, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_hudson_area::del_msg(gmsg& msg) {

  GFTRK("HudsDelMsg");

  HudsHdr hdr;
  save_message(GMSG_HDR | GMSG_DELETE, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_hudson_area::new_msgno(gmsg& msg) {

  GFTRK("HudsNewMsgno");

  msg.msgno = wide->msginfo.high + 1;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_hudson_area::update_timesread(gmsg& msg) {

  GFTRK("gmb_hudson_area::update_timesread");

  bool do_lock = gmb_wide_timesread & TIMESREAD_LOCK ? true : false;
  if(do_lock)
    lock();

  HudsMsgn hdridx = get_hdr_idx(msg, __FILE__, __LINE__);
  ::lseekset(wide->fhhdr, (long)hdridx*(long)sizeof(HudsHdr));

  HudsHdr hdr;
  ::read(wide->fhhdr, &hdr, sizeof(HudsHdr));

  hdr.timesread = (word)msg.timesread;

  ::lseekset(wide->fhhdr, (long)hdridx*(long)sizeof(HudsHdr));
  ::write(wide->fhhdr, &hdr, sizeof(HudsHdr));

  if(do_lock)
    unlock();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

