
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbhuds5.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Hudson / Goldbase msgbase handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgtrk.h>
#include <gmemall.h>
#include <gstrall.h>
#include <gutlmisc.h>

#include <gmbhuds.h>


//  ------------------------------------------------------------------

bool gmb_hudson_area::user_lookup(gstring& lookfor) {

  wide->user->fh = wide->fhusr;
  char buf[256];
  if(wide->user->findwild(lookfor.c_str(), buf))
    lookfor = buf;

  return wide->user->found;
}


//  ------------------------------------------------------------------

bool gmb_hudson_area::renumber() {

  return false;
}


//  ------------------------------------------------------------------

void gmb_hudson_area::hexdump_msg(gmsg& msg, gstring& dump) {

  GFTRK("HudsMakeDump");

  HudsHdr _hdr;
  load_message(GMSG_HDRTXT, msg, _hdr);

  strnp2c(_hdr.date,9);
  strnp2c(_hdr.time,5);

  char buf[100];

  dump =  "Hexdump of " HUDS_NAME " message header and text\n";
  dump += "------------------------------------------------------------------------------\n";
  dump += "\n";
  dump += gstrf("Msgbase   : %s\n", wide->path.c_str());
  dump += gstrf("BoardNo   : %u\n", board());
  dump += gstrf("From      : %-35.35s\n", msg.origname.c_str());
  dump += gstrf("To        : %-35.35s\n", msg.destname.c_str());
  dump += gstrf("Subject   : %-72.72s\n", msg.subject.c_str());
  dump += gstrf("DateTime  : %8.8s %5.5s\n", _hdr.date, _hdr.time);
  dump += gstrf("OrigAddr  : %u:%u/%u\n", _hdr.origzone, _hdr.orignet, _hdr.orignode);
  dump += gstrf("DestAddr  : %u:%u/%u\n", _hdr.destzone, _hdr.destnet, _hdr.destnode);
  dump += gstrf("Reply     : %u\n", _hdr.replyto);
  dump += gstrf("See       : %u\n", _hdr.reply1st);
  dump += gstrf("TimesRead : %u\n", _hdr.timesread);
  dump += gstrf("Cost      : %u\n", _hdr.cost);
  dump += gstrf("MsgAttr   : %02Xh (%sb)\n", _hdr.msgattr, ltob(buf, _hdr.msgattr, 8));
  dump += gstrf("NetAttr   : %02Xh (%sb)\n", _hdr.netattr, ltob(buf, _hdr.netattr, 8));
  dump += gstrf("Msgno     : %lu\n", (long)_hdr.msgno);
  dump += gstrf("Board     : %u\n", _hdr.board);
  dump += gstrf("StartRec  : %lu\n", (long)_hdr.startrec);
  dump += gstrf("NumRecs   : %lu\n", (long)_hdr.numrecs);
  dump += gstrf("UserRecno : %u (%s)\n", wide->userno, gmb_wide_username[0]);
  dump += "\n";
  dump += "Dump of MSGINFO" HUDS_EXT ":\n";
  dump += "\n";
  dump += gstrf("Total Active  : %lu (%lXh)\n", (ulong)wide->msginfo.total, (ulong)wide->msginfo.total);
  dump += gstrf("Low/High Msgno: %lu (%lXh)  %lu (%lXh)\n",
    (ulong)wide->msginfo.low,   (ulong)wide->msginfo.low,
    (ulong)wide->msginfo.high,  (ulong)wide->msginfo.high
  );
  for(int brd=0; brd<HUDS_MAXBOARD; brd+=10) {
    dump += gstrf("Board %03u-%03u : %5u %5u %5u %5u %5u %5u %5u %5u %5u %5u\n",
      brd+1,  brd+10,  wide->msginfo.active[brd+0],
      wide->msginfo.active[brd+1],  wide->msginfo.active[brd+2],  wide->msginfo.active[brd+3],
      wide->msginfo.active[brd+4],  wide->msginfo.active[brd+5],  wide->msginfo.active[brd+6],
      wide->msginfo.active[brd+7],  wide->msginfo.active[brd+8],  wide->msginfo.active[brd+9]
    );
  }
  dump += "\n";
  dump += "Dump of LASTREAD" HUDS_EXT ":\n";
  dump += "\n";
  for(int lrd=0; lrd<HUDS_MAXBOARD; lrd+=5) {
    dump += gstrf("Board %03u-%03u : %10lu %10lu %10lu %10lu %10lu\n",
      lrd+1,  lrd+5,  (ulong)wide->lastrec[lrd+0],
      (ulong)wide->lastrec[lrd+1],  (ulong)wide->lastrec[lrd+2],
      (ulong)wide->lastrec[lrd+3],  (ulong)wide->lastrec[lrd+4]
    );
  }
  dump += "\n";
  dump += "Hexdump of message header:\n";
  dump += "\n";

  addhexdump(dump, &_hdr, sizeof(HudsHdr), "\n");

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

