
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbjamm1.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  JAM msgbase implementation, initialization.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>
#include <gcrcall.h>

#include <gmbjamm.h>


//  ------------------------------------------------------------------

gmb_jam_manager::gmb_jam_manager() {

  harddelete = true;
  usercrc = 0;
  userid = 0;
  lookreplies = true;
}


//  ------------------------------------------------------------------

gmb_jam_manager::~gmb_jam_manager() {

}


//  ------------------------------------------------------------------

void gmb_jam_manager::initialize() {

  // Calculate CRC32 of our username for the lastreads
  char* name = throw_strdup(gmb_wide_username[0]);
  userid = usercrc = strCrc32(strlwr(name), NO, CRC32_MASK_CCITT);
  throw_free(name);
}


//  ------------------------------------------------------------------

void gmb_jam_manager::reset() {

  for(int n=0; n<active.size(); n++)
    delete active[n];
  active.reset();
}


//  ------------------------------------------------------------------

gmb_area* gmb_jam_manager::new_area(gareacfg* c) {

  gmb_jam_area* area = new gmb_jam_area(c, this);
  throw_new(area);
  return area;
}


//  ------------------------------------------------------------------

void gmb_jam_manager::init_area(gmb_area* a, gareacfg* c) {

  gmb_jam_area* area = (gmb_jam_area*)a;
  area->init(c, this);
}


//  ------------------------------------------------------------------

void gmb_jam_area::activate() {

  refs++;
  if(refs == 1) {
    fhjhr = fhjdt = fhjdx = fhjlr = -1;
    open_refs = lock_refs = 0;
    just_scanning = false;
    timesposted = 0;
    lastpos = 0;
  }
}


//  ------------------------------------------------------------------

void gmb_jam_area::deactivate() {

  if(refs) {
    if(refs == 1) {
      if(open_refs) {
        open_refs = 1;
        close();
      }
    }
    refs--;
  }
}


//  ------------------------------------------------------------------

void gmb_jam_area::open() {

  GFTRK("gmb_jam_area::open");

  open_refs++;
  if(open_refs == 1)
    open_1st();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_jam_area::close() {

  GFTRK("gmb_jam_area::close");

  if(open_refs) {
    if(open_refs == 1) {
      if(lock_refs) {
        lock_refs = 1;
        unlock();
      }
      raw_close();
      msgnset.reset();
      pmrkset.reset();
    }
    open_refs--;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_jam_area::suspend() {

  GFTRK("gmb_jam_area::suspend");

  save_lastread();
  raw_close();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_jam_area::resume() {

  GFTRK("gmb_jam_area::resume");

  raw_open();
  if(lock_refs)
    raw_lock();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

