
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbjamm2.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  JAM msgbase implementation, scanning.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>
#include <gcrcall.h>

#include <gmbjamm.h>


//  ------------------------------------------------------------------

int gmb_jam_area::test_open(const char* file) {

  GFTRK("gmb_jam_area::test_open");

  int fh;
  long tries = 0;

  do {

    fh = ::sopen(file, O_RDWR|O_CREAT|O_BINARY, gmb_wide_sharemode, S_STDRW);
    if(fh == -1) {

      // Tell the world
      if((errno != EACCES) or (PopupLocked(++tries, false, file) == false)) {
        gmb_wide_log->ErrOpen();
        raw_close();
        gmb_wide_log->printf("! A JAM msgbase file could not be opened.");
        gmb_wide_log->printf(": %s.", file);
        gmb_wide_log->ErrOSInfo();
        OpenErrorExit();
      }
    }
  } while(fh == -1);

  // Remove the popup window
  if(tries)
    PopupLocked(0, 0, NULL);

  GFTRK(NULL);

  return fh;
}


//  ------------------------------------------------------------------

void gmb_jam_area::raw_open() {

  GFTRK("gmb_jam_area::raw_open");

  Path file;
  sprintf(file, "%s.JHR", path().c_str());  fhjhr = test_open(file);
  sprintf(file, "%s.JDX", path().c_str());  fhjdx = test_open(file);
  sprintf(file, "%s.JLR", path().c_str());  fhjlr = test_open(file);
  if(not just_scanning) {
    sprintf(file, "%s.JDT", path().c_str());  fhjdt = test_open(file);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_jam_area::raw_close() {

  GFTRK("gmb_jam_area::raw_close");

  if(fhjlr != -1)  ::close(fhjlr);  fhjlr = -1;
  if(fhjdx != -1)  ::close(fhjdx);  fhjdx = -1;
  if(fhjdt != -1)  ::close(fhjdt);  fhjdt = -1;
  if(fhjhr != -1)  ::close(fhjhr);  fhjhr = -1;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_jam_area::open_1st() {

  // Open the msgbase files
  raw_open();

  // Read the header info
  memset(&hdrinfo, 0, sizeof(JamHdrInfo));
  read(fhjhr, &hdrinfo, sizeof(JamHdrInfo));

  // Is the signature invalid?
  if(memcmp(hdrinfo.signature, JAM_SIGNATURE, 4)) {

    // Initialize header info
    memcpy(hdrinfo.signature, JAM_SIGNATURE, 4);
    hdrinfo.datecreated = gtime(NULL);
    hdrinfo.passwordcrc = 0xFFFFFFFFL;
    hdrinfo.basemsgnum  = 1;

    // Write header info
    lseekset(fhjhr, 0);
    write(fhjhr, &hdrinfo, sizeof(JamHdrInfo));
  }

  // Adjust base msg number if necessary
  if(hdrinfo.basemsgnum == 0)
    hdrinfo.basemsgnum = 1;

  // Seek to beginning of the .JLR
  lseekset(fhjlr, 0);

  // Search for the userid (not the usercrc?!)
  lastpos = 0;
  int founduser = false;
  while(read(fhjlr, &lastrec, sizeof(JamLast)) == sizeof(JamLast)) {
    if(lastrec.usercrc == wide->usercrc) {
      founduser = true;
      break;
    }
    lastpos++;
  }

  // If user was not found, init the lastread with our values
  if(not founduser) {
    lastrec.usercrc = wide->usercrc;
    lastrec.userid  = wide->userid;
    lastrec.lastread = 0;
    lastrec.highread = 0;
  }

  timesposted = 0;
}


//  ------------------------------------------------------------------

void gmb_jam_area::raw_scan(bool keep_index, bool scanpm) {

  GFTRK("JamRawScan");

  // Open the msgbase if it wasn't already

  bool was_open = open_refs > 0;
  if(not was_open) {
    if(not keep_index or scanpm)
      just_scanning = true;
    open_1st();
    just_scanning = false;
  }

  // Get some sizes
  long jdxlen = filelength(fhjdx);
  #if defined(__MSDOS__) and not defined(__386__)
  if((jdxlen/sizeof(JamIndex)) > (65535L/sizeof(JamIndex))) {
    gmb_wide_log->ErrIndex();
    gmb_wide_log->printf("! A JAM msgbase index was too large to handle.");
    gmb_wide_log->printf(": %s (%s) with %lu msgs.", path().c_str(), echoid().c_str(), jdxlen/sizeof(JamIndex));
    gmb_wide_log->printf(": Info: GoldED DOS 16-bit does not support more than %lu msgs in a JAM area.", 65535L/sizeof(JamIndex));
    gmb_wide_log->printf("+ Advice: Purge and pack the area or use a 32-bit version of GoldED.");
    IndexErrorExit();
  }
  #endif
  uint jdxsize  = (uint)jdxlen;
  uint jdxtotal = jdxsize / sizeof(JamIndex);

  // (Re)Allocate message index
  if(keep_index)
    msgnset.resize(jdxtotal);

  // Allocate buffer to hold .JDX data
  JamIndex* jdxbuf = (JamIndex*)throw_malloc(jdxsize+1);

  // Read the entire .JDX file into memory
  lseekset(fhjdx, 0);
  read(fhjdx, jdxbuf, jdxsize);

  // Variables for the loop
  register uint active = 0;
  register ulong firstmsgno = 0;
  register ulong lastmsgno = 0;
  register ulong lastreadfound = 0;
  register ulong msgno = hdrinfo.basemsgnum;
  register ulong total = hdrinfo.basemsgnum + jdxtotal;
  register ulong lastread_msgn = lastrec.lastread;
  register uint  lastread_reln = 0;
  register ulong* msgndxptr = msgnset.tag;
  register JamIndex* jdxptr = jdxbuf;

  // Fill message index
  while(msgno < total) {
    if(jdxptr->hdroffset != 0xFFFFFFFFL) {
      active++;
      if(not firstmsgno)
        firstmsgno = msgno;
      if(keep_index)
        *msgndxptr++ = msgno;
      if((msgno >= lastread_msgn) and (lastread_reln == 0)) {
        lastreadfound = msgno;
        lastread_reln = (uint)(active - (msgno != lastread_msgn ? 1 : 0));
      }
      lastmsgno = msgno;
    }
    jdxptr++;
    msgno++;
  }

  // If the exact lastread was not found
  if(active and (lastreadfound != lastread_msgn)) {

    // Higher than highest or lower than lowest?
    if(lastread_msgn > lastmsgno)
      lastread_reln = active;
    else if(lastread_msgn < firstmsgno)
      lastread_reln = 0;
  }

  // Update area data
  scan_count = active;
  msgnset.resize(active);
  lastread = lastread_reln;
  lastreadentry = lastreadfound;

  // Scan for personal mail
  pmrkset.reset();
  if(scanpm) {
    gstring uname;
    int umax = (gmb_wide_personalmail & PM_ALLNAMES) ? gmb_wide_usernames : 1;
    dword* ucrc = (dword*)throw_calloc(umax, sizeof(dword));
    for(int uc=0; uc<umax; uc++) {
      uname = gmb_wide_username[uc];
      uname.lower();
      ucrc[uc] = strCrc32(uname.c_str(), NO, CRC32_MASK_CCITT);
    }
    register uint n = lastread + 1;
    register uint cnt = msgnset.size();
    register int gotpm = false;
    while(n <= cnt) {
      JamIndex* idx = jdxbuf + (uint)(msgnset.at(n-1) - hdrinfo.basemsgnum);
      for(int u=0; u<umax; u++) {
        if(idx->usercrc == ucrc[u]) {
          gotpm = true;
          break;
        }
      }
      if(gotpm) {
        JamHdr hdr;
        lseekset(fhjhr, idx->hdroffset);
        read(fhjhr, &hdr, sizeof(JamHdr));
        if(not (hdr.attribute & JAMATTR_READ)) {
          if(not (hdr.attribute & JAMATTR_DELETED)) {
            pmrkset.append(hdr.messagenumber);
          }
        }
        gotpm = false;
      }
      n++;
    }
    throw_free(ucrc);
  }
  pmscan_count = pmrkset.size();

  if(gmb_wide_debug) {
    gmb_wide_log->printf("- %s: t:%u, l:%u, fm:%lu, hm:%lu, lr:%lu, u:%u, pm:%i",
      echoid().c_str(),
      msgnset.size(),
      lastread,
      firstmsgno,
      lastmsgno,
      lastread_msgn,
      lastpos,
      scanpm ? pmrkset.size() : -1
    );
  }


  // Free the .JDX buffer
  throw_free(jdxbuf);

  // Close the msgbase again if we opened it in here
  if(not was_open)
    raw_close();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_jam_area::scan() {

  GFTRK("gmb_jam_area::scan");

  raw_scan(true);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_jam_area::scan_area() {

  GFTRK("gmb_jam_area::scan_area");

  raw_scan(false);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_jam_area::scan_area_pm() {

  GFTRK("gmb_jam_area::scan_area_pm");

  raw_scan(true, true);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

