
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbjamm3.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  JAM msgbase implementation, load.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <stdlib.h>

#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>

#include <gmbjamm.h>


//  ------------------------------------------------------------------

bool gmb_jam_area::load_message(int mode, gmsg& msg, JamHdr& hdr) {

  // Read index record for msg
  JamIndex _idx;
  memset(&_idx, 0, sizeof(JamIndex));
  lseekset(fhjdx, msg.msgno-hdrinfo.basemsgnum, sizeof(JamIndex));
  read(fhjdx, &_idx, sizeof(JamIndex));

  if(_idx.hdroffset == 0xFFFFFFFFL) {
    GFTRK(NULL);
    return false;
  }

  // Read message header
  memset(&hdr, 0, sizeof(JamHdr));
  lseekset(fhjhr, _idx.hdroffset);
  read(fhjhr, &hdr, sizeof(JamHdr));

  msg.link.to_set(hdr.replyto);
  msg.link.first_set(hdr.reply1st);
  msg.link.next_set(hdr.replynext);

  msg.cost = (uint)hdr.cost;
  msg.timesread = (uint)hdr.timesread;

  msg.date.written = hdr.datewritten;
  msg.date.arrived = hdr.dateprocessed;
  msg.date.received = hdr.datereceived;

  msg.attr.loc(hdr.attribute & JAMATTR_LOCAL);
  msg.attr.trs(hdr.attribute & JAMATTR_INTRANSIT);
  msg.attr.pvt(hdr.attribute & JAMATTR_PRIVATE);
  msg.attr.rcv(hdr.attribute & JAMATTR_READ);
  msg.attr.snt(hdr.attribute & JAMATTR_SENT);
  msg.attr.k_s(hdr.attribute & JAMATTR_KILLSENT);
  msg.attr.a_s(hdr.attribute & JAMATTR_ARCHIVESENT);
  msg.attr.hld(hdr.attribute & JAMATTR_HOLD);
  msg.attr.cra(hdr.attribute & JAMATTR_CRASH);
  msg.attr.imm(hdr.attribute & JAMATTR_IMMEDIATE);
  msg.attr.dir(hdr.attribute & JAMATTR_DIRECT);
  msg.attr.zon(hdr.attribute & JAMATTR_GATE);
  msg.attr.frq(hdr.attribute & JAMATTR_FILEREQUEST);
  msg.attr.att(hdr.attribute & JAMATTR_FILEATTACH);
  msg.attr.tfs(hdr.attribute & JAMATTR_TRUNCFILE);
  msg.attr.kfs(hdr.attribute & JAMATTR_KILLFILE);
  msg.attr.rrq(hdr.attribute & JAMATTR_RECEIPTREQ);
  msg.attr.cfm(hdr.attribute & JAMATTR_CONFIRMREQ);
  msg.attr.orp(hdr.attribute & JAMATTR_ORPHAN);
  msg.attr.lok(hdr.attribute & JAMATTR_LOCKED);
  msg.attr.del(hdr.attribute & JAMATTR_DELETED);

  if(isnet() or isecho())
    msg.attr.uns(msg.attr.loc() and not msg.attr.snt());
  else
    msg.attr.uns0();

  // Get reply numbers in chain
  if(wide->lookreplies and msg.link.first()) {
    int r = 0;
    ulong m = msg.link.first();
    while(m and (r<msg.link.list_max())) {
      JamHdr _rhdr;
      memset(&_rhdr, 0, sizeof(JamHdr));
      lseekset(fhjdx, m-hdrinfo.basemsgnum, sizeof(JamIndex));
      read(fhjdx, &_idx, sizeof(JamIndex));
      lseekset(fhjhr, _idx.hdroffset);
      read(fhjhr, &_rhdr, sizeof(JamHdr));
      m = _rhdr.replynext;
      if(m)
        msg.link.list_set(r++, m);
    }
  }

  // Allocate space for the subfields
  msg.jam.subfieldlen = hdr.subfieldlen;
  byte* subfield = (byte*)throw_xmalloc((uint)(hdr.subfieldlen+1));

  // Read the subfields
  read(fhjhr, subfield, (uint)hdr.subfieldlen);

  // Pointer to the subfields
  JamSubField* subfieldptr = (JamSubField*)subfield;

  // Process the subfields
  int daddress_count = 0;
  uint subfieldpos = 0;

  while(subfieldpos < hdr.subfieldlen) {

    subfieldpos += sizeof(JamSubFieldHdr);
    uint datlen = (uint)subfieldptr->datlen;

    if(subfieldpos > hdr.subfieldlen)
      break;

    if((subfieldpos + datlen) > hdr.subfieldlen)
      datlen = (uint)(hdr.subfieldlen - subfieldpos);

    char* buf = subfieldptr->buffer;
    char bufendchar = buf[datlen];
    buf[datlen] = NUL;
    int buflen;

    char* ptr;

    switch(subfieldptr->loid) {

      case JAMSUB_OADDRESS:
        msg.origaddr = buf;
        msg.oaddress = buf;
        break;

      case JAMSUB_DADDRESS:
        if(daddress_count == 0) {
          msg.destaddr = buf;
          msg.daddress = buf;
        }
        else {
          msg.kludge.cc.add(buf);
        }
        break;

      case JAMSUB_SENDERNAME:
        msg.origname = buf;
        break;

      case JAMSUB_RECEIVERNAME:
        msg.destname = buf;
        break;

      case JAMSUB_MSGID:
        msg.kludge.msgid = buf;
        break;

      case JAMSUB_REPLYID:
        msg.kludge.reply = buf;
        break;

      case JAMSUB_SUBJECT:
        msg.subject = buf;
        break;

      case JAMSUB_PID:
        msg.kludge.pid = buf;
        break;

      case JAMSUB_TRACE:
        msg.kludge.via.add(buf);
        break;

      case JAMSUB_ENCLOSEDFILE:
        msg.fileattach.add(buf);
        break;

      case JAMSUB_ENCLOSEDFILEWALIAS:
        buflen = strlen(buf);
        if(buflen != datlen)
          buf[buflen] = ' ';
        msg.fileattach_with_alias.add(buf);
        break;

      case JAMSUB_ENCLOSEDFREQ:
        buflen = strlen(buf);
        if(buflen != datlen)
          buf[buflen] = ' ';
        msg.filerequest.add(buf);
        break;

      case JAMSUB_ENCLOSEDFILEWCARD:
        msg.fileattach_wildcard.add(buf);
        break;

      case JAMSUB_ENCLOSEDINDIRECFILE:
        msg.fileattach_indirect.add(buf);
        break;

      case JAMSUB_FTSKLUDGE:
        msg.add_control(buf);
        break;

      case JAMSUB_SEENBY2D:
        ptr = buf;
        while(strlen(ptr) > 70) {
          char* tmp = ptr;
          ptr += 70;
          while(*ptr != ' ')
            ptr--;
          *ptr++ = NUL;
          msg.kludge.seen_by.add(tmp);
        }
        break;

      case JAMSUB_PATH2D:
        ptr = buf;
        while(strlen(ptr) > 72) {
          char* tmp = ptr;
          ptr += 72;
          while(*ptr != ' ')
            ptr--;
          *ptr++ = NUL;
          msg.kludge.path.add(tmp);
        }
        break;

      case JAMSUB_FLAGS:
        msg.kludge.flags = buf;
        break;

      case JAMSUB_TZUTCINFO:
        msg.date.written.utc_offset = atoi(buf);
        msg.date.arrived.utc_offset = atoi(buf);
        msg.date.received.utc_offset = atoi(buf);
        break;

      default:
        ptr = gstrf("JAM%u: %s", buf);
        msg.kludge.other.add(ptr);
    }

    buf[datlen] = bufendchar;

    subfieldpos += datlen;
    subfieldptr = (JamSubField*)(subfield + subfieldpos);
  }

  // Free subfield buffer
  throw_xfree(subfield);

  msg.jam.offset = hdr.offset;
  msg.jam.txtlen = hdr.txtlen;

  if(mode & GMSG_TXT) {

    // Make sure the msg size is within user/system limits
    ulong textlen = hdr.txtlen;
    if(textlen > gmb_wide_msgsize)
      textlen = gmb_wide_msgsize;

    // Allocate memory for the message text
    msg.text.reserve((uint)textlen+1);

    // Read the message text
    lseekset(fhjdt, hdr.offset);
    read(fhjdt, msg.text.pointer(), (uint)textlen);

    msg.text.raw_to_lines((uint)textlen);
  }

  GFTRK(NULL);

  return true;
}


//  ------------------------------------------------------------------

bool gmb_jam_area::load_hdr(gmsg& msg) {

  GFTRK("gmb_jam_area::load_hdr");

  JamHdr _hdr;
  return load_message(GMSG_HDR, msg, _hdr);
}


//  ------------------------------------------------------------------

bool gmb_jam_area::load_msg(gmsg& msg) {

  GFTRK("gmb_jam_area::load_msg");

  JamHdr _hdr;
  return load_message(GMSG_HDRTXT, msg, _hdr);
}


//  ------------------------------------------------------------------

