
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbjamm5.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  JAM msgbase implementation, utilities.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gmemall.h>
#include <gstrall.h>
#include <gutlmisc.h>

#include <gmbjamm.h>


//  ------------------------------------------------------------------

bool gmb_jam_area::user_lookup(gstring&) {

  return false;
}


//  ------------------------------------------------------------------

bool gmb_jam_area::renumber() {

  return false;
}


//  ------------------------------------------------------------------

void gmb_jam_area::hexdump_msg(gmsg& __msg, gstring& dump) {

  GFTRK("gmb_jam_area::hexdump_msg");

  // Read base header
  JamHdrInfo* _base = (JamHdrInfo*)throw_calloc(1, sizeof(JamHdrInfo));
  lseekset(fhjhr, 0);
  read(fhjhr, _base, sizeof(JamHdrInfo));

  // Read index record for msg
  JamIndex _idx;
  lseekset(fhjdx, __msg.msgno-hdrinfo.basemsgnum, sizeof(JamIndex));
  read(fhjdx, &_idx, sizeof(JamIndex));

  // Allocate space for the header
  JamHdr* _hdr = (JamHdr*)throw_calloc(1, (uint)(sizeof(JamHdr)+17));

  // Read message header
  lseekset(fhjhr, _idx.hdroffset);
  read(fhjhr, _hdr, sizeof(JamHdr));

  // Allocate space for the subfields
  byte* _subfield = (byte*)throw_calloc(1, (uint)(_hdr->subfieldlen+17));

  // Read the subfields
  read(fhjhr, _subfield, (uint)_hdr->subfieldlen);

  // Allocate memory for the raw message text
  char* txt = (char*)throw_calloc(1, (uint)_hdr->txtlen+16);

  // Read the message text
  lseekset(fhjdt, _hdr->offset);
  read(fhjdt, txt, (uint)_hdr->txtlen);

  char buf[100];

  dump =  "Hexdump of JAM message header, subfields and text\n";
  dump += "------------------------------------------------------------------------------\n";
  dump += "\n";
  dump += gstrf("Msgbase       : %s\n",     path().c_str());
  dump += gstrf("Signature     : %s\n",     _hdr->signature);
  dump += gstrf("Revision      : %u\n",     _hdr->revision);
  dump += gstrf("ReservedWord  : %u\n",     _hdr->reservedword);
  dump += gstrf("SubfieldLen   : %lu\n",    _hdr->subfieldlen);
  dump += gstrf("TimesRead     : %lu\n",    _hdr->timesread);
  dump += gstrf("MSGIDcrc      : %08lXh\n", _hdr->msgidcrc);
  dump += gstrf("REPLYcrc      : %08lXh\n", _hdr->replycrc);
  dump += gstrf("ReplyTo       : %lu\n",    _hdr->replyto);
  dump += gstrf("Reply1st      : %lu\n",    _hdr->reply1st);
  dump += gstrf("ReplyNext     : %lu\n",    _hdr->replynext);
  dump += gstrf("DateWritten   : %s (%08lXh)\n", TimeToStr(buf, _hdr->datewritten), _hdr->datewritten);
  dump += gstrf("DateReceived  : %s (%08lXh)\n", TimeToStr(buf, _hdr->datereceived), _hdr->datereceived);
  dump += gstrf("DateProcessed : %s (%08lXh)\n", TimeToStr(buf, _hdr->dateprocessed), _hdr->dateprocessed);
  dump += gstrf("MessageNumber : %lu\n",    _hdr->messagenumber);
  dump += gstrf("Attribute     : %08lXh (%sb)\n", _hdr->attribute,  ltob(buf, _hdr->attribute, 0));
  dump += gstrf("Attribute2    : %08lXh (%sb)\n", _hdr->attribute2, ltob(buf, _hdr->attribute2, 0));
  dump += gstrf("Offset        : %lu\n",    _hdr->offset);
  dump += gstrf("TxtLen        : %lu\n",    _hdr->txtlen);
  dump += gstrf("PasswordCRC   : %08lXh\n", _hdr->passwordcrc);
  dump += gstrf("Cost          : %lu\n",    _hdr->cost);
  dump += "\n";
  dump += "Index Record:\n";
  dump += "\n";
  dump += gstrf("UserCrc       : %08lXh\n", _idx.usercrc);
  dump += gstrf("HeaderOffset  : %08lXh (%lu)\n", _idx.hdroffset, _idx.hdroffset);
  dump += "\n";
  dump += "Lastread Record:\n";
  dump += "\n";
  dump += gstrf("Index         : %lu\n",    lastpos);
  dump += gstrf("UserCrc       : %08lXh\n", lastrec.usercrc);
  dump += gstrf("UserId        : %08lXh\n", lastrec.userid);
  dump += gstrf("Lastread      : %lu\n",    lastrec.lastread);
  dump += gstrf("Highread      : %lu\n",    lastrec.highread);
  dump += "\n";
  dump += "Base Header:\n";
  dump += "\n";
  dump += gstrf("DateCreated   : %s (%08lXh)\n", TimeToStr(buf, _base->datecreated), _base->datecreated);
  dump += gstrf("ModCounter    : %lu\n",    _base->modcounter);
  dump += gstrf("ActiveMsgs    : %lu\n",    _base->activemsgs);
  dump += gstrf("PasswordCRC   : %08lXh\n", _base->passwordcrc);
  dump += gstrf("BaseMsgNum    : %lu\n",    _base->basemsgnum);
  dump += gstrf("HighWaterMark : %lu\n",    _base->highwatermark);
  dump += "\n";
  dump += "Subfields:\n";
  dump += "\n";

  addhexdump(dump, _subfield, (uint)_hdr->subfieldlen, "\n");
  dump += "\n";

  // Process the subfields
  JamSubField* _subfieldptr = (JamSubField*)_subfield;
  uint _subfieldpos = 0;
  while(_subfieldpos < _hdr->subfieldlen) {
    _subfieldpos += sizeof(JamSubFieldHdr);
    uint _datlen = (uint)_subfieldptr->datlen;
    if(_subfieldpos > _hdr->subfieldlen) {
      dump += gstrf("Error: SubfieldHdr at pos %u exceeds SubfieldLen (%u)!\n", _subfieldpos-sizeof(JamSubFieldHdr), _hdr->subfieldlen);
      break;
    }
    if((_subfieldpos + _datlen) > _hdr->subfieldlen) {
      _datlen = (uint)(_hdr->subfieldlen - _subfieldpos);
      dump += gstrf("Error: SubfieldData at pos %u is too long (%u)!  Cut to %u.\n", _subfieldpos, (uint)_subfieldptr->datlen, _datlen);
    }
    dump += gstrf("%05u [%3u]: \"%-*.*s\"\n",
      _subfieldptr->loid, _datlen,
      minimum_of_two(60, (int)_datlen),
      minimum_of_two(60, (int)_datlen),
      _subfieldptr->buffer
    );
    _subfieldpos += _datlen;
    _subfieldptr = (JamSubField*)(_subfield + _subfieldpos);
  }

  // Free subfield buffer
  throw_free(_subfield);

  dump += "\n";
  dump += "Hexdump of message header:\n";
  dump += "\n";

  addhexdump(dump, _hdr, sizeof(JamHdr), "\n");

  // Free header and base
  throw_free(_hdr);
  throw_free(_base);
  throw_free(txt);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

