
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbpcbd.h,v 1.1.1.1 1998/11/21 19:38:14 odinn Exp $
//  ------------------------------------------------------------------
//  PCBoard Structures.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GMPCBD_H
#define __GMPCBD_H


//  ------------------------------------------------------------------

#include <gmbarea.h>
#include <gs_pcb.h>
#include <gusrpcb.h>


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


//  ------------------------------------------------------------------

typedef char PcbName[25+1];


//  ------------------------------------------------------------------
//  PCBoard Message Base Header (<msgbase>. file)

struct PcbBase {

  long  highmsgno;                // High Message Number       (0 to 16,700,000)
  long  lowmsgno;                 // Low Message Number        (0 to 16,700,000)
  long  active;                   // Number of Active Messages (0 to 32,767)
  long  callers;                  // Number of System Callers (Main Message Base Only)
  char  locked[6];                // The "LOCKED" field for pre-14.2 systems (see note 1)
  byte  reserved[106];            // Reserved for future use
};


//  ------------------------------------------------------------------
//  PCBoard Individual Message Header (in <msgbase>. file)

struct PcbHdr {

  char  status;                   // Message Status Flag (see note 2)
  long  msgno;                    // Message Number   (0 to 16,700,000)
  long  refno;                    // Reference Number (0 to 16,700,000)
  byte  blocks;                   // Number of 128 Byte Blocks in Message (see note 3)
  char  date[8];                  // Date of Message Entry (in "mm-dd-yy" format)
  char  time[5];                  // Time of Message Entry (in "hh:mm" format)
  char  destname[25];             // Name of the User to whom the Message is Addressed
  long  replydate;                // Date of the Reply Message (in yymmdd format)
  char  replytime[5];             // Time of the Reply Message (in "hh:mm" format)
  char  hasreply;                 // The Letter "R" if the Message has a Reply
  char  origname[25];             // Name of the User who wrote the Message
  char  subject[25];              // Subject of the Message
  char  password[12];             // Password Need to Read the Message (if any)
  byte  activestatus;             // Active Status (225 = active, 226 = inactive)
  char  echoed;                   // The Letter "E" if the Message is to be Echoed
  byte  reserved[4];              // Reserved for future use
  byte  exthdrflags;              // Extended Header Flags (version 15.0)
  byte  reserved2;                // Reserved for future use
};


//  ------------------------------------------------------------------
//  PCBoard Extended Header (version 15.0) (in <msgbase>. file)

struct PcbExtHdr {

  short  id;                       // Extended Header ID = must be equal to 40FFh
  char  function[7];              // Extended Header Function
  char  colon;                    // A colon (:) character
  char  desc[60];                 // Extended Header Description (subj, to, from, etc)
  char  status;                   // Status (N or R)
  char  separator;                // Line Separator (E3h, or 0Dh for foreign systems)
};


//  ------------------------------------------------------------------
//  PCBoard Version 15.0 Style Index (<msgbase>.IDX file)

struct PcbIdx {

  long     offset;                // Offset (0 if none, >0 if active, <0 if killed)
  long     num;                   // Message Number
  char     to[25];                // TO Name
  char     from[25];              // FROM Name
  char     status;                // Status Character (from Message Header)
  word     date;                  // Date of Message (in julian date format)
  char     reserved[3];           // Reserved for future use
};


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------

class gmb_pcboard_area : public gmb_area {

protected:

  gmb_pcboard_manager* wide;

  int      fhmsg;
  int      fhidx;
  int      islocked;
  PcbBase  base;
  int      just_scanning;

  void data_open();
  void data_close();

  int test_open(const char* file);

  void raw_open();
  void raw_close();

  void raw_scan(int keep_index, int scanpm=false);
  bool load_message(int mode, gmsg& msg, PcbHdr& hdr);
  void save_message(int mode, gmsg& msg, PcbHdr& hdr);

public:

  gmb_pcboard_area(gareacfg* c, gmb_pcboard_manager* m) : gmb_area(c) { wide = m; }
  virtual ~gmb_pcboard_area() { }

  void init(gareacfg* c, gmb_pcboard_manager* m)  { gmb_area::init(c); wide = m; }

  void activate();
  void deactivate();

  void open();
  void close();

  void save_lastread();

  void suspend();
  void resume();

  void lock();
  void unlock();

  void scan();
  void scan_area();
  void scan_area_pm();

  bool load_hdr(gmsg& msg);
  bool load_msg(gmsg& msg);

  void save_hdr(int mode, gmsg& msg);
  void save_msg(int mode, gmsg& msg);

  void del_msg(gmsg& msg);

  void new_msgno(gmsg& msg);
  bool user_lookup(gstring& lookfor);
  bool renumber();

  void update_timesread(gmsg& msg);

  void hexdump_msg(gmsg& msg, gstring& dump);

};


//  ------------------------------------------------------------------

class gmb_pcboard_manager : public gmb_area_manager {

protected:

  int            open_refs;
  int            fhusr;
  int            fhinf;
  int            userno;
  PcbUser        user;
  Path           usersidxpath;
  Path           usersinf;
  Path           users;
  int            numareas;
  PcbUsers       usersrec;
  PcbUsersInfHdr usershdr;
  long           usershdrsize;
  long*          lastread;
  long           confbytelen;
  long           extconflen;
  int            foreign;
  gstring        path;

  void open();
  void close();

  int test_open(const char* file);

  friend class gmb_pcboard_area;

public:

  gmb_pcboard_manager();
  ~gmb_pcboard_manager();

  void initialize();
  void reset();

  void wide_open()   { open(); }
  void wide_close()  { close(); }

  gmb_area* new_area(gareacfg* c);
  void init_area(gmb_area* a, gareacfg* c);

  void set_path(const gstring& p)  { path = p; }
  void set_userno(int u)           { userno = u; }

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

