
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbpcbd1.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  PCBoard msgbase handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>
#include <gutlmisc.h>

#include <gmbprot.h>
#include <gmbpcbd.h>

              
//  ------------------------------------------------------------------

gmb_pcboard_manager::gmb_pcboard_manager() {

  userno = 0;
}


//  ------------------------------------------------------------------

gmb_pcboard_manager::~gmb_pcboard_manager() {

  reset();
}


//  ------------------------------------------------------------------

gmb_area* gmb_pcboard_manager::new_area(gareacfg* c) {

  gmb_pcboard_area* area = new gmb_pcboard_area(c, this);
  throw_new(area);
  return area;
}


//  ------------------------------------------------------------------

void gmb_pcboard_manager::init_area(gmb_area* a, gareacfg* c) {

  gmb_pcboard_area* area = (gmb_pcboard_area*)a;
  area->init(c, this);
}


// ------------------------------------------------------------------

void gmb_pcboard_manager::initialize() {

  fhusr = fhinf = -1;

  // Get PCBoard path
  Path _path;
  char* _ptr = getenv("PCBOARD");
  if(_ptr)
    AddBackslash(strcpy(_path, _ptr));
  else
    strcpy(_path, path.c_str());

  Path _cnamespath;
  *_cnamespath = NUL;

  // Open PCBOARD.DAT
  gfile fp;
  const char* _file = AddPath(_path, "PCBOARD.DAT");
  fp.fopen(_file, "rt", gmb_wide_sharemode);
  if(fp.isopen()) {

    // Get some paths/filenames
    int _line = 0;
    char _buf[256];
    fp.setvbuf(NULL, _IOFBF, 8192);
    while(fp.fgets(_buf, sizeof(_buf))) {
      _line++;
      if(_line == 28)
        strxcpy(usersidxpath, strbtrim(_buf), sizeof(Path));
      else if(_line == 29)
        strxcpy(users, strbtrim(_buf), sizeof(Path));
      else if(_line == 31)
        strxcpy(_cnamespath, strbtrim(_buf), sizeof(Path));
      else if(_line == 180)
        strxcpy(usersinf, strbtrim(_buf), sizeof(Path));
      else if(_line == 208)
        foreign = atoi(_buf);
    }
    fp.fclose();

    // Open CNAMES.@@@
    _file = AddPath(_cnamespath, ".@@@");
    fp.fopen(_file, "rb", gmb_wide_sharemode);

    // Get board numbers for lastread indexing in the userfiles
    word _recsize = 0;
    fp.setvbuf(NULL, _IOFBF, 8192);
    fp.fread(&_recsize, 2);
    if(_recsize) {
      PcbConf* _cnames = (PcbConf*)throw_calloc(1, _recsize);
      int _rec = 0;
      numareas = (int)((fp.filelength()-2)/(long)_recsize);
      confbytelen = (numareas/8) + ((numareas%8) != 0 ? 1 : 0);
      if(confbytelen < 5)
        confbytelen = 5;
      extconflen = confbytelen - 5;
      lastread = (long*)throw_calloc(numareas, sizeof(long));
      while(fp.fread(_cnames, _recsize) == 1) {
        PcbAdjustArea(_rec, _cnames->msgfile);
        _rec++;
      }
      throw_free(_cnames);
    }
    fp.fclose();

    const char* _username = gmb_wide_username[0];
    user.fh = ::sopen(AddPath(_path, users), O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
    if(user.fh != -1) {
      user.fhinf = ::sopen(AddPath(_path, usersinf), O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
      if(user.fhinf != -1) {
        ::read(user.fhinf, &usershdr, sizeof(PcbUsersInfHdr));
        usershdrsize = sizeof(PcbUsersInfHdr) + (usershdr.numofapps*sizeof(PcbUsersInfApp));
        user.usershdr = &usershdr;
        user.usershdrsize = usershdrsize;
        if(userno == -1) {
          user.find(_username);
          if(not user.found) {
            userno = 0;
            //gmb_wide_log->printf("* User \"%s\" not found in %sUSERS.", _username, _path);
            //user.add(_username);
            //gmb_wide_log->printf("* Now added with user number %u.", user.index);
          }
        }
        ::close(user.fhinf);
      }
      ::close(user.fh);
    }
    userno = user.index;
  }
  else {

    gmb_wide_log->ErrOpen();
    gmb_wide_log->printf("! Configured for PCBoard, but can't find it.");
    gmb_wide_log->printf(": %s.", _file);
    gmb_wide_log->printf("+ Advice: Add or correct the PCBOARDPATH keyword.");
    gmb_wide_log->ErrOSInfo();
    OpenErrorExit();
  }
}


//  ------------------------------------------------------------------

void gmb_pcboard_manager::reset() {

  for(int n=0; n<active.size(); n++)
    delete active[n];
  active.reset();
  throw_release(lastread);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::activate() {

  refs++;
  if(refs == 1) {
    fhmsg = fhidx = -1;
    memset(&base, 0x20, sizeof(PcbBase));
    base.highmsgno = 0;
    base.lowmsgno  = 0;
    base.active    = 0;
  }
}


// ------------------------------------------------------------------

void gmb_pcboard_area::deactivate() {

  if(refs) {
    if(refs == 1) {
      if(open_refs) {
        open_refs = 1;
        close();
      }
    }
    refs--;
  }
}


// ------------------------------------------------------------------

void gmb_pcboard_area::open() {

  GFTRK("gmb_pcboard_area::Open");

  open_refs++;
  if(open_refs == 1) {
    wide->open();
    raw_open();
  }

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::close() {

  GFTRK("gmb_pcboard_area::Close");

  if(open_refs) {
    if(open_refs == 1) {
      if(lock_refs) {
        lock_refs = 1;
        unlock();
      }
      raw_close();
      msgnset.reset();
      pmrkset.reset();
      wide->close();
    }
    open_refs--;
  }

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::save_lastread() {

  GFTRK("gmb_pcboard_area::SaveLastread");

  dword tmplr = msgnset.relative_to_absolute(lastread);
  if(board() < 40) {
    // Write lastreads to USERS file
    lseekset(wide->fhusr, (long)wide->userno*(long)sizeof(PcbUsers));
    read(wide->fhusr, &wide->usersrec, sizeof(PcbUsers));
    wide->usersrec.lastmsgread[board()] = L2B(tmplr);
    lseekset(wide->fhusr, (long)wide->userno*(long)sizeof(PcbUsers));
    write(wide->fhusr, &wide->usersrec, sizeof(PcbUsers));
  }
  else {
    // Write lastreads to USERS.INF file
    if(wide->extconflen) {
      long _offset = (wide->usersrec.usersinfrec-1)*wide->usershdr.totalrecsize;
      _offset +=     wide->usershdrsize;
      _offset +=     wide->usershdr.sizeofrec;
      _offset += 2 * wide->confbytelen;
      _offset += 3 * wide->extconflen;
      _offset += sizeof(dword) * (board() - 40);
      lseekset(wide->fhinf, _offset);
      write(wide->fhinf, &tmplr, sizeof(dword));
    }
  }

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::suspend() {

  GFTRK("gmb_pcboard_area::Suspend");

  save_lastread();
  raw_close();
  wide->close();

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::resume() {

  GFTRK("gmb_pcboard_area::Resume");

  wide->open();
  raw_open();

  GFTRK(NULL);
}


// ------------------------------------------------------------------

