
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbpcbd2.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  PCBoard msgbase handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>
#include <gutlmisc.h>

#include <gmbprot.h>
#include <gmbpcbd.h>

              
//  ------------------------------------------------------------------

int gmb_pcboard_manager::test_open(const char* __file) {

  GFTRK("gmb_pcboard_manager::test_open");

  int _fh;
  long _tries = 0;

  do {

    _fh = ::sopen(__file, O_RDWR|O_BINARY|O_CREAT, gmb_wide_sharemode, S_STDRW);
    if(_fh == -1) {

      // Tell the world
      if(PopupLocked(++_tries, false, __file) == false) {
        gmb_wide_log->ErrOpen();
        close();
        gmb_wide_log->printf("! A PCBoard msgbase file could not be opened.");
        gmb_wide_log->printf(": %s.", __file);
        gmb_wide_log->ErrOSInfo();
        OpenErrorExit();
      }
    }
  } while(_fh == -1);

  // Remove the popup window
  if(_tries)
    PopupLocked(0, 0, NULL);

  GFTRK(NULL);

  return _fh;
}


// ------------------------------------------------------------------

void gmb_pcboard_manager::open() {

  GFTRK("gmb_pcboard_manager::open");

  open_refs++;
  if(open_refs == 1) {

    fhusr = test_open(users);
    fhinf = test_open(usersinf);
    user.fh = fhusr;
    user.fhinf = fhinf;

    // Read lastreads from USERS file
    lseekset(fhusr, (long)userno*(long)sizeof(PcbUsers));
    read(fhusr, &usersrec, sizeof(PcbUsers));
    int _maxlr = minimum_of_two(numareas, 40);
    for(int n=0; n<_maxlr; n++)
      lastread[n] = B2L(usersrec.lastmsgread[n]);

    // Read lastreads from USERS.INF file
    if(extconflen) {
      long _offset = (usersrec.usersinfrec-1)*usershdr.totalrecsize;
      _offset +=     usershdrsize;
      _offset +=     usershdr.sizeofrec;
      _offset += 2 * confbytelen;
      _offset += 3 * extconflen;
      lseekset(fhinf, _offset);
      read(fhinf, lastread+40, (numareas-40)*sizeof(long));
    }
  }

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_pcboard_manager::close() {

  GFTRK("gmb_pcboard_manager::close");

  if(open_refs) {
    if(open_refs == 1) {
      if(fhinf != -1)  ::close(fhinf);  fhinf = -1;
      if(fhusr != -1)  ::close(fhusr);  fhusr = -1;
    }
    open_refs--;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_pcboard_area::raw_close() {

  GFTRK("gmb_pcboard_area::raw_close");

  if(fhidx != -1)  ::close(fhidx);  fhidx = -1;
  if(fhmsg != -1)  ::close(fhmsg);  fhmsg = -1;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

int gmb_pcboard_area::test_open(const char* __file) {

  GFTRK("gmb_pcboard_area::test_open");

  int _fh;
  long _tries = 0;

  do {

    _fh = ::sopen(__file, O_RDWR|O_BINARY|O_CREAT, gmb_wide_sharemode, S_STDRW);
    if(_fh == -1) {

      // Tell the world
      if(PopupLocked(++_tries, false, __file) == false) {
        gmb_wide_log->ErrOpen();
        raw_close();
        wide->close();
        gmb_wide_log->printf("! A PCBoard msgbase file could not be opened.");
        gmb_wide_log->printf(": %s.", __file);
        gmb_wide_log->ErrOSInfo();
        OpenErrorExit();
      }
    }
  } while(_fh == -1);

  // Remove the popup window
  if(_tries)
    PopupLocked(0, 0, NULL);

  GFTRK(NULL);

  return _fh;
}


// ------------------------------------------------------------------

void gmb_pcboard_area::raw_open() {

  GFTRK("gmb_pcboard_area::raw_open");

  if(not just_scanning)
    fhmsg = test_open(path().c_str());
  fhidx = test_open(AddPath(path().c_str(), ".IDX"));

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::raw_scan(int keep_index, int scanpm) {

  GFTRK("gmb_pcboard_area::raw_scan");

  // Open the msgbase if it wasn't already
  bool was_open = open_refs > 0;
  if(not was_open) {
    if(not keep_index)
      just_scanning = true;
    raw_open();
    just_scanning = false;
  }

  bool was_wideopen = wide->open_refs > 0;
  if(not was_wideopen)
    wide->open();

  // Load the base header
  if(keep_index) {
    lseekset(fhmsg, 0);
    read(fhmsg, &base, sizeof(PcbBase));
    base.highmsgno = B2L(base.highmsgno);
    base.lowmsgno  = B2L(base.lowmsgno);
    base.active    = B2L(base.active);
  }
  
  // Get some sizes
  long _idxlen = filelength(fhidx);
  #if defined(__MSDOS__) and not defined(__386__)
  if((_idxlen/sizeof(PcbIdx)) > (65535L/sizeof(PcbIdx))) {
    gmb_wide_log->ErrIndex();
    gmb_wide_log->printf("! A PCBoard msgbase index was too large to handle.");
    gmb_wide_log->printf(": %s (%s) with %lu msgs.", path().c_str(), echoid().c_str(), _idxlen/sizeof(PcbIdx));
    gmb_wide_log->printf("+ Info: GoldED-DOS does not (yet) support more than %lu msgs in a PCBoard area.", 65535L/sizeof(PcbIdx));
    gmb_wide_log->printf("+ Advice: Purge and pack the area or use GoldED/386 or GoldED/2.");
    IndexErrorExit();
  }
  #endif
  uint _idxsize  = (uint)_idxlen;
  uint _idxtotal = _idxsize / sizeof(PcbIdx);

  // (Re)Allocate message index
  if(keep_index)
    msgnset.resize(_idxtotal);

  // Allocate buffer to hold .IDX data
  PcbIdx* _idxbuf = (PcbIdx*)throw_malloc(_idxsize+1);

  // Read the entire .IDX file into memory
  lseekset(fhidx, 0);
  read(fhidx, _idxbuf, _idxsize);

  // Variables for the loop
  register uint _active = 0;
  register ulong _firstmsgno = 0;
  register ulong _lastmsgno = 0;
  register ulong _lastreadfound = 0;
  register ulong _totalmsgs = _idxtotal;
  register ulong _lastread = wide->lastread[board()];
  register uint _lastread_reln = 0;
  register ulong* _msgndxptr = msgnset.tag;
  register PcbIdx* _idxptr = _idxbuf;

  // Fill message index
  register uint n = 0;
  while((n++) < _totalmsgs) {
    if(_idxptr->offset > 0) {
      _active++;
      _lastmsgno = _idxptr->num;
      if(not _firstmsgno)
        _firstmsgno = _lastmsgno;
      if(keep_index)
        *_msgndxptr++ = _lastmsgno;
      if((_lastmsgno >= _lastread) and (_lastread_reln == 0)) {
        _lastreadfound = _lastmsgno;
        _lastread_reln = (uint)(_active - (_lastmsgno != _lastread ? 1 : 0));
      }
    }
    _idxptr++;
  }

  // If the exact lastread was not found
  if(_active and (_lastreadfound != _lastread)) {

    // Higher than highest or lower than lowest?
    if(_lastread > _lastmsgno)
      _lastread_reln = _active;
    else if(_lastread < _firstmsgno)
      _lastread_reln = 0;
  }

  // Update area data
  scan_count = _active;
  msgnset.resize(_active);
  lastread = _lastread_reln;
  lastreadentry = _lastreadfound;

  // Scan for personal mail
  pmrkset.reset();
  if(scanpm) {
    int umax = (gmb_wide_personalmail & PM_ALLNAMES) ? gmb_wide_usernames : 1;
    PcbName* uname = (PcbName*)throw_calloc(umax, sizeof(PcbName));
    for(int un=0; un<umax; un++)
      strsetsz(strxcpy(uname[un], gmb_wide_username[un], sizeof(PcbName)), 25);
    register uint n = lastread + 1;
    register uint cnt = msgnset.size();
    register int gotpm = false;
    while(n <= cnt) {
      PcbIdx* idx = _idxbuf + (uint)(msgnset.at(n-1) - base.lowmsgno);
      for(int u=0; u<umax; u++) {
        if(strnieql(idx->to, uname[u], 25)) {
          gotpm = true;
          break;
        }
      }
      if(gotpm) {
        switch(idx->status) {
          case '+':
          case '-':
          case '`':
          case '^':
          case '#':
            break;
          default:
            pmrkset.append(idx->num);
        }
        gotpm = false;
      }
      n++;
    }
    throw_free(uname);
  }
  pmscan_count = pmrkset.size();

  // Free the .IDX buffer
  throw_free(_idxbuf);

  if(not was_wideopen)
    wide->close();

  // Close the msgbase again if we opened it in here
  if(not was_open)
    raw_close();

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::scan() {

  GFTRK("gmb_pcboard_area::scan");

  raw_scan(true);

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::scan_area() {

  GFTRK("gmb_pcboard_area::scan_area");

  raw_scan(false);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_pcboard_area::scan_area_pm() {

  GFTRK("gmb_pcboard_area::scan_area_pm");

  raw_scan(true, true);

  GFTRK(NULL);
}


// ------------------------------------------------------------------

