
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbpcbd3.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  PCBoard msgbase handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>
#include <gutlmisc.h>

#include <gmbpcbd.h>

              
//  ------------------------------------------------------------------

bool gmb_pcboard_area::load_message(int, gmsg& msg, PcbHdr& hdr) {

  // Read index record for msg
  PcbIdx _idx;
  lseekset(fhidx, (msg.msgno-base.lowmsgno)*(long)sizeof(PcbIdx));
  read(fhidx, &_idx, sizeof(PcbIdx));
  msg.pcboard.txtstart = _idx.offset;

  // Read message header
  lseekset(fhmsg, absolute(_idx.offset));
  read(fhmsg, &hdr, sizeof(PcbHdr));
  msg.pcboard.txtlength = (hdr.blocks-1)*128;
  msg.pcboard.txtblocks = hdr.blocks;

  msg.timesread = 1;

  // Convert attributes
  msg.pcboard.status = hdr.status;
  if(not islocal())
    msg.attr.uns(hdr.date[5] != '\xC4');
  msg.attr.pvt(hdr.status == '*' or hdr.status == '+');
  msg.attr.rcv(hdr.status == '+' or hdr.status == '-' or hdr.status == '`' or hdr.status == '^' or hdr.status == '#');
  msg.attr.del(hdr.activestatus == 226);
  msg.pcboard.exthdrflags = hdr.exthdrflags;

  // Convert msgno and replylinks
  hdr.msgno = B2L(hdr.msgno);
  hdr.refno = B2L(hdr.refno);
  msg.link.to_set(hdr.refno);

  // Convert date and time
  int year, month, day, hour, minute;
  sscanf(hdr.date, "%d%*c%d%*c%2d", &month, &day, &year);
  sscanf(hdr.time, "%d%*c%2d", &hour, &minute);
  msg.date.written.year   = (year < 80) ? year+2000 : year+1900;
  msg.date.written.month  = month;
  msg.date.written.day    = day;
  msg.date.written.hour   = hour;
  msg.date.written.minute = minute;

  // Convert names and subject and password
  msg.origname.copyn(hdr.origname, 25).trim();
  msg.destname.copyn(hdr.destname, 25).trim();
  msg.subject.copyn(hdr.subject,  25).trim();
  strtrim(strncpy(msg.pcboard.password, hdr.password, 12));
  
  // Make sure the msg size is within user/system limits
  long textlen = msg.pcboard.txtlength;
  if(textlen > gmb_wide_msgsize)
    textlen = gmb_wide_msgsize;

  // Allocate memory for the message text
  msg.text.reserve((uint)textlen+1);

  // Read the message text
  read(fhmsg, msg.text.pointer(), (uint)textlen);
  if(not wide->foreign)
    msg.text.pcboard_cr_is_newline = true;
  msg.text.raw_to_lines((uint)textlen);

  // Get extended headers
  char eby[122] = {""};
  char eto[122] = {""};
  for(int n=0; n<msg.text.lines(); n++) {
    PcbExtHdr* ehdr = (PcbExtHdr*)msg.text[n];
    if(ehdr->id == 0x40FF) {
      if     (strnieql(ehdr->function, "TO     ", 7))
        strncpy(eto, ehdr->desc, 60);
      else if(strnieql(ehdr->function, "TO2    ", 7))
        strncpy(eto+60, ehdr->desc, 60);
      else if(strnieql(ehdr->function, "FROM   ", 7))
        strncpy(eby, ehdr->desc, 60);
      else if(strnieql(ehdr->function, "FROM2  ", 7))
        strncpy(eby+60, ehdr->desc, 60);
      else if(strnieql(ehdr->function, "SUBJECT", 7))
        msg.subject.copyn(ehdr->desc, 60).trim();
      else
        msg.pcboard.exthdrs.add(msg.text[n]);
      msg.text.erase(n--);
    }
  }

  strtrim(eby);
  if(*eby)
    msg.origname = eby;

  strtrim(eto);
  if(*eto)
    msg.destname = eto;

  if(isnet() and not isemail()) {
    const char* ptr = strchr(msg.origname.c_str(), '@');
    if(ptr) {
      msg.origaddr = ptr + 1;
      msg.origname.erase(ptr);
    }
    ptr = strchr(msg.destname.c_str(), '@');
    if(ptr) {
      char buf[256];
      strcpy(buf, ptr+1);
      char* ptr2 = strskip_txt(buf);
      if(*ptr2) {
        *ptr2++ = NUL;
        while(*ptr2) {
          if(*ptr2 == '+') {
            // get flags
            switch(toupper(*(++ptr2))) {
              case 'C': msg.attr.cra1(); break;
              case 'D': msg.attr.dir1(); break;
            }
          }
          else if(*ptr2 == '[') {
            // get from-aka address
            ptr2++;
            char* ptr3 = ptr2;
            ptr2 = strskip_to(ptr2, ']');
            if(*ptr2) {
              *ptr2++ = NUL;
              msg.origaddr = ptr3;
            }
          }
          ptr2++;
        }
      }
      msg.destaddr = ptr + 1;
      msg.destname.erase(ptr);
    }
  }

  GFTRK(NULL);

  return true;
}


// ------------------------------------------------------------------

bool gmb_pcboard_area::load_hdr(gmsg& msg) {

  GFTRK("PcbLoadHdr");

  PcbHdr hdr;
  return load_message(GMSG_HDR, msg, hdr);
}


// ------------------------------------------------------------------

bool gmb_pcboard_area::load_msg(gmsg& msg) {

  GFTRK("PcbLoadMsg");

  PcbHdr hdr;
  return load_message(GMSG_HDRTXT, msg, hdr);
}


// ------------------------------------------------------------------

