
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbpcbd4.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  PCBoard msgbase handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgerr.h>
#include <gdbgtrk.h>
#include <gstrall.h>
#include <gutlmisc.h>

#include <gmbpcbd.h>

              
// ------------------------------------------------------------------

void gmb_pcboard_area::lock() {

  GFTRK("gmb_pcboard_area::lock");

  lock_refs++;
  if(lock_refs == 1) {
    if(gmb_wide_canlock) {
      long tries = 0;
      while(::lock(fhmsg, 16, 6) == -1) {
        if(PopupLocked(++tries, true, path().c_str()) == false) {
          gmb_wide_log->ErrLock();
          raw_close();
          wide->close();
          gmb_wide_log->printf("! A PCBoard msgbase file could not be locked.");
          gmb_wide_log->printf(": %s.", path().c_str());
          gmb_wide_log->ErrOSInfo();
          LockErrorExit();
        }
      }
      if(tries)
        PopupLocked(0, 0, NULL);
    }
    lseekset(fhmsg, 0);
    read(fhmsg, &base, sizeof(PcbBase));
    lseekset(fhmsg, 0);
    memcpy(base.locked, "LOCKED", 6);
    write(fhmsg, &base, sizeof(PcbBase));
    base.highmsgno = B2L(base.highmsgno);
    base.lowmsgno  = B2L(base.lowmsgno);
    base.active    = B2L(base.active);
  }

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::unlock() {

  GFTRK("gmb_pcboard_area::unlock");

  if(lock_refs) {
    if(lock_refs == 1) {
      if(gmb_wide_canlock)
        ::unlock(fhmsg, 16, 6);
      lseekset(fhmsg, 0);
      memset(base.locked, ' ', 6);
      base.highmsgno = L2B(base.highmsgno);
      base.lowmsgno  = L2B(base.lowmsgno);
      base.active    = L2B(base.active);
      write(fhmsg, &base, sizeof(PcbBase));
      base.highmsgno = B2L(base.highmsgno);
      base.lowmsgno  = B2L(base.lowmsgno);
      base.active    = B2L(base.active);
    }
    lock_refs--;
  }

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::save_message(int mode, gmsg& msg, PcbHdr& hdr) {

  // Lock and refresh base
  lock();

  // Reset header
  memset(&hdr, 0, sizeof(PcbHdr));

  // Find msgno and index offset
  PcbIdx idx;
  ulong oldmsgno = msg.msgno;
  long oldtxtstart = msg.pcboard.txtstart;
  long oldtxtlength = msg.pcboard.txtlength;
  if(mode & GMSG_TXT) {
    if(base.lowmsgno == 0)
      base.lowmsgno = 1;
    base.highmsgno++;
    base.active++;
    msg.msgno = base.highmsgno;
    msgnset.append(msg.msgno);
    scan_count++;
  }
  long idxoffset = (msg.msgno-base.lowmsgno)*(long)sizeof(PcbIdx);
  hdr.blocks = (byte)msg.pcboard.txtblocks;
  idx.num = msg.msgno;
  idx.reserved[0] = idx.reserved[1] = idx.reserved[2] = 0;

  // Convert msgno and replylinks
  hdr.msgno = L2B(msg.msgno);
  hdr.refno = L2B(msg.link.to());

  // Convert attributes
  char status = msg.pcboard.status;
  int pvt = msg.attr.pvt();
  int rcv = msg.attr.rcv();
  if(rcv) {
    switch(status) {
      case '$':   hdr.status = '#';   break;
      case '!':   hdr.status = '#';   break;
      case '%':   hdr.status = '^';   break;
      case '~':   hdr.status = '`';   break;
      case '*':   hdr.status = '+';   break;
      case ' ':   hdr.status = '-';   break;
      default:    hdr.status = pvt ? '+' : '-';
    }
  }
  else {
    if(status and not pvt)
      hdr.status = status;
    else if(pvt)
      hdr.status = '*';
    else
      hdr.status = ' ';
  }
  idx.status = hdr.status;
  hdr.activestatus = (mode & GMSG_DELETE) ? '\xE2' : '\xE1';
  hdr.echoed = ((isnet() or isecho()) and not isinternet()) ? 'E' : ' ';

  // Convert dates and times
  char dtbuf[9];
  char undoc = msg.attr.uns() ? '-' : '\xC4';
  sprintf(dtbuf, "%02d-%02d%c%02d",
    msg.date.written.month, msg.date.written.day, undoc, msg.date.written.year_twodigits()
  );
  memcpy(hdr.date, dtbuf, 8);
  sprintf(dtbuf, "%02d:%02d", msg.date.written.hour, msg.date.written.minute);
  memcpy(hdr.time, dtbuf, 5);
  idx.date = (word)YMD2JDN(msg.date.written.year, msg.date.written.month, msg.date.written.day);
  if(msg.link.first()) {
    hdr.hasreply = 'R';
    hdr.replydate = L2B((10000L*msg.pcboard.reply_written.year_twodigits()) +
                    (100L*(msg.pcboard.reply_written.month)) + msg.pcboard.reply_written.day);
    sprintf(dtbuf, "%02d:%02d", msg.pcboard.reply_written.hour, msg.pcboard.reply_written.minute);
    memcpy(hdr.replytime, dtbuf, 5);
  }
  else {
    hdr.hasreply = ' ';
    hdr.replydate = L2B(0);
    memcpy(hdr.replytime, "     ", 5);
  }

  // Convert names, subject and password
  char tobuf[150];
  char bybuf[150];
  strxcpy(tobuf, msg.destname.c_str(), sizeof(tobuf));
  strxcpy(bybuf, msg.origname.c_str(), sizeof(bybuf));
  int tolen = strlen(tobuf);
  int bylen = strlen(bybuf);
  int relen = msg.subject.length();
  int pwlen = strlen(msg.pcboard.password);
  memset(hdr.destname, ' ', 25);
  memset(hdr.origname, ' ', 25);
  memset(hdr.subject,  ' ', 25);
  memset(hdr.password, ' ', 12);
  strncpy(hdr.destname, tobuf, minimum_of_two(tolen, 25));
  strncpy(hdr.origname, bybuf, minimum_of_two(bylen, 25));
  strncpy(hdr.subject,  msg.subject.c_str(), minimum_of_two(relen, 25));
  strncpy(hdr.password, msg.pcboard.password, minimum_of_two(pwlen, 12));
  memcpy(idx.to, hdr.destname, 25);
  memcpy(idx.from, hdr.origname, 25);
  if(isnet()) {
    gftnaddrs addr = msg.destaddr;
    addr.domain = "";
    char toaddr[40];
    char byaddr[40];
    strcpy(toaddr, addr.c_str());
    strcpy(byaddr, " [");
    addr = msg.origaddr;
    addr.domain = "";
    strcpy(byaddr+2, addr.c_str());
    strcat(byaddr, "]");
    sprintf(tobuf+strlen(tobuf), "@%s%s%s%s",
      toaddr,
      msg.attr.cra() ? " +C" : "",
      msg.attr.dir() ? " +D" : "",
      byaddr
    );
    tolen = strlen(tobuf);
    bylen = strlen(bybuf);
  }

  // Translate msg text to PCB linefeeds if running non-foreign system
  if(not wide->foreign)
    msg.text.pcboard_cr_is_newline = true;
  char* txt = msg.text.lines_to_raw();

  // Determine size of the msg text including extended headers
  uint txtlen = 0;
  if(mode & GMSG_TXT) {
    msg.pcboard.txtlength = txtlen = strlen(txt);
    if((tolen > 25) or isnet()) {
      msg.pcboard.txtlength += sizeof(PcbExtHdr);
      hdr.exthdrflags |= 0x01;
    }
    if((bylen > 25) or isnet()) {
      msg.pcboard.txtlength += sizeof(PcbExtHdr);
      hdr.exthdrflags |= 0x02;
    }
    if(relen > 25) {
      msg.pcboard.txtlength += sizeof(PcbExtHdr);
      hdr.exthdrflags |= 0x04;
    }

    // Calculate new number of blocks
    hdr.blocks = (byte)(1 + (msg.pcboard.txtlength/128) + ((msg.pcboard.txtlength%128)?1:0));

    // Determine where to write the message text
    msg.pcboard.txtstart = filelength(fhmsg);
    msg.pcboard.txtblocks = hdr.blocks;
  }

  // Write index
  idx.offset = (mode & GMSG_DELETE) ? -msg.pcboard.txtstart : msg.pcboard.txtstart;
  lseekset(fhidx, idxoffset);
  write(fhidx, &idx, sizeof(PcbIdx));

  // Write header
  lseekset(fhmsg, absolute(idx.offset));
  write(fhmsg, &hdr, sizeof(PcbHdr));

  // Writing msg text?
  if(mode & GMSG_TXT) {

    uint txtlenwritten = 0;

    // Write extended headers
    if((tolen > 25) or isnet()) {
      PcbExtHdr ehdr;
      ehdr.id = 0x40FF;
      ehdr.colon = ':';
      ehdr.status = 'N';
      ehdr.separator = wide->foreign ? '\x0D' : '\xE3';
      memcpy(ehdr.function, "TO     ", 7);
      memset(ehdr.desc, ' ', 60);
      memcpy(ehdr.desc, tobuf, minimum_of_two(60,tolen));
      write(fhmsg, &ehdr, sizeof(PcbExtHdr));
      txtlenwritten += sizeof(PcbExtHdr);
      if(tolen > 60) {
        memcpy(ehdr.function, "TO2    ", 7);
        memset(ehdr.desc, ' ', 60);
        memcpy(ehdr.desc, tobuf+60, minimum_of_two(60,(tolen-60)));
        write(fhmsg, &ehdr, sizeof(PcbExtHdr));
        txtlenwritten += sizeof(PcbExtHdr);
      }
    }
    if((bylen > 25) or isnet()) {
      PcbExtHdr ehdr;
      ehdr.id = 0x40FF;
      ehdr.colon = ':';
      ehdr.status = 'N';
      ehdr.separator = wide->foreign ? '\x0D' : '\xE3';
      memcpy(ehdr.function, "FROM   ", 7);
      memset(ehdr.desc, ' ', 60);
      memcpy(ehdr.desc, bybuf, minimum_of_two(60,bylen));
      write(fhmsg, &ehdr, sizeof(PcbExtHdr));
      txtlenwritten += sizeof(PcbExtHdr);
      if(bylen > 60) {
        memcpy(ehdr.function, "FROM2  ", 7);
        memset(ehdr.desc, ' ', 60);
        memcpy(ehdr.desc, bybuf+60, minimum_of_two(60,(bylen-60)));
        write(fhmsg, &ehdr, sizeof(PcbExtHdr));
        txtlenwritten += sizeof(PcbExtHdr);
      }
    }
    if(relen > 25) {
      PcbExtHdr ehdr;
      ehdr.id = 0x40FF;
      memcpy(ehdr.function, "SUBJECT", 7);
      ehdr.colon = ':';
      memset(ehdr.desc, ' ', 60);
      memcpy(ehdr.desc, msg.subject.c_str(), minimum_of_two(60,relen));
      ehdr.status = 'N';
      ehdr.separator = wide->foreign ? '\x0D' : '\xE3';
      write(fhmsg, &ehdr, sizeof(PcbExtHdr));
      txtlenwritten += sizeof(PcbExtHdr);
    }

    // Write message text
    write(fhmsg, txt, txtlen);
    txtlenwritten += txtlen;

    // Write remainder of the last message block (if any)
    uint remainlen = ((hdr.blocks-1)*128) - txtlenwritten;
    if(remainlen) {
      byte remainrecord[128];
      memset(remainrecord, ' ', 128);
      write(fhmsg, remainrecord, remainlen);
    }
  }

  if(not (mode & GMSG_DELETE)) {
    // Set the mail waiting flag
    int status = true;
    // Reset it if the msg is being received
    if((mode & GMSG_UPDATE) and msg.attr.rcv())
      status = false;
    // Don't touch the flag if the msg was already received
    if(not (status and msg.attr.rcv()))
      wide->user.update_mail_waiting(msg.destname.c_str(), board(), status);
  }

  if((mode & GMSG_TXT) and (mode & GMSG_UPDATE)) {
    msg.msgno = oldmsgno;
    msg.pcboard.txtstart = oldtxtstart;
    msg.pcboard.txtlength = oldtxtlength;
    del_msg(msg);
  }

  // Unlock and update base
  unlock();

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::save_hdr(int mode, gmsg& msg) {

  GFTRK("gmb_pcboard_area::save_hdr");

  PcbHdr hdr;
  save_message(mode|GMSG_HDR, msg, hdr);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::save_msg(int mode, gmsg& msg) {

  GFTRK("gmb_pcboard_area::save_msg");

  PcbHdr hdr;
  save_message(mode|GMSG_HDRTXT, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_pcboard_area::del_msg(gmsg& msg) {

  GFTRK("gmb_pcboard_area::del_msg");

  PcbHdr hdr;
  save_message(GMSG_HDR | GMSG_DELETE, msg, hdr);
}


// ------------------------------------------------------------------

void gmb_pcboard_area::new_msgno(gmsg& msg) {

  GFTRK("gmb_pcboard_area::NewMsgno");

  msg.msgno = base.highmsgno + 1;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_pcboard_area::update_timesread(gmsg&) {

}


// ------------------------------------------------------------------

