
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbpcbd5.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  PCBoard msgbase handling.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gmemall.h>
#include <gstrall.h>
#include <gutlmisc.h>

#include <gmbpcbd.h>

              
// ------------------------------------------------------------------

bool gmb_pcboard_area::user_lookup(gstring&) {

  return false;
}


// ------------------------------------------------------------------

bool gmb_pcboard_area::renumber() {

  return false;
}


// ------------------------------------------------------------------

void gmb_pcboard_area::hexdump_msg(gmsg& __msg, gstring& dump) {

  GFTRK("gmb_pcboard_area::hexdump_msg");

  // Read lastread records
  byte _mailwaiting = 0;
  dword* dumplr = (dword*)throw_calloc(wide->numareas, sizeof(dword));
  lseekset(wide->fhusr, (long)wide->userno*(long)sizeof(PcbUsers));
  read(wide->fhusr, &wide->usersrec, sizeof(PcbUsers));
  int _maxlr = minimum_of_two(wide->numareas, 40);
  int n;
  for(n=0; n<_maxlr; n++)
    dumplr[n] = wide->usersrec.lastmsgread[n];
  long _offset = (wide->usersrec.usersinfrec-1)*wide->usershdr.totalrecsize;
  _offset +=     wide->usershdrsize;
  _offset +=     wide->usershdr.sizeofrec;
  long _offset2 = _offset;
  _offset2 += board() / 8;
  lseekset(wide->fhinf, _offset2);
  read(wide->fhinf, &_mailwaiting, 1);
  byte bitmask = (byte)(1 << (board() % 8));
  _mailwaiting &= bitmask;
  _offset += 2 * wide->confbytelen;
  _offset += 3 * wide->extconflen;
  if(wide->extconflen) {
    lseekset(wide->fhinf, _offset);
    read(wide->fhinf, dumplr+40, (wide->numareas-40)*sizeof(long));
  }

  // Read index record for msg
  PcbIdx _idx;
  lseekset(fhidx, (__msg.msgno-base.lowmsgno)*(long)sizeof(PcbIdx));
  read(fhidx, &_idx, sizeof(PcbIdx));

  // Read message header
  PcbHdr _hdr;
  lseekset(fhmsg, absolute(_idx.offset));
  read(fhmsg, &_hdr, sizeof(PcbHdr));
  
  // Read the message text
  uint _msgsize = (_hdr.blocks-1)*128;
  char* txt = (char*)throw_calloc(1, _msgsize+256);
  read(fhmsg, txt, _msgsize);

  // Convert Julian date in the index
  unsigned _year, _month, _day;
  JDN2YMD(_idx.date, &_year, &_month, &_day);

  dump =  "Hexdump of PCBoard message header and text\n";
  dump += "------------------------------------------------------------------------------\n";
  dump += "\n";
  dump += gstrf("Msgbase   : %s\n", path().c_str());
  dump += gstrf("BoardNo   : %u\n", board());
  dump += gstrf("MsgOffset : %li\n", _idx.offset);
  dump += gstrf("Status    : \'%c\'\n", _hdr.status);
  dump += gstrf("Msgno     : %li  (%08lXh)\n", B2L(_hdr.msgno), _hdr.msgno);
  dump += gstrf("Refno     : %li  (%08lXh)\n", B2L(_hdr.refno), _hdr.refno);
  dump += gstrf("Blocks    : %i\n", _hdr.blocks);
  dump += gstrf("Date/Time : %8.8s %5.5s\n", _hdr.date, _hdr.time);
  dump += gstrf("IdxDate   : %02u-%02u-%04u (%u)\n", _month, _day, _year, _idx.date);
  dump += gstrf("DestName  : %-25.25s\n", _hdr.destname);
  dump += gstrf("ReplyDate : %li\n", B2L(_hdr.replydate));
  dump += gstrf("ReplyTime : %5.5s\n", _hdr.replytime);
  dump += gstrf("HasReply  : \'%c\'\n", _hdr.hasreply);
  dump += gstrf("OrigName  : %-25.25s\n", _hdr.origname);
  dump += gstrf("Subject   : %-25.25s\n", _hdr.subject);
  dump += gstrf("Password  : %-12.12s\n", _hdr.password);
  dump += gstrf("Active    : %u%s\n", _hdr.activestatus, (_hdr.activestatus == 226) ? " (Deleted)" : "");
  dump += gstrf("Echoed    : \'%c\'\n", _hdr.echoed);
  dump += gstrf("Reserved  : %02Xh %02Xh %02Xh %02Xh \n", _hdr.reserved[0], _hdr.reserved[1], _hdr.reserved[2], _hdr.reserved[3]);
  dump += gstrf("ExtHdrFlg : %02Xh\n", _hdr.exthdrflags);
  dump += gstrf("Reserved2 : %02Xh\n", _hdr.reserved2);
  dump += "\n";
  dump += "Message Base Header:\n";
  dump += "\n";
  dump += gstrf("HighMsgno : %li\n", base.highmsgno);
  dump += gstrf("LowMsgno  : %li\n", base.lowmsgno);
  dump += gstrf("Active    : %li\n", base.active);
  dump += gstrf("Locked    : %6.6s\n", base.locked);

  dump += "\n";
  dump += gstrf("Mail Waiting: %s\n", _mailwaiting ? "Yes" : "No");
  dump += "\n";
  dump += "Lastreads from the USERS file:\n";
  dump += "\n";
  for(n=0; n<_maxlr; n++) {
    dword tmplr = B2L(dumplr[n]);
    dump += gstrf("Board %4u : %10lu / %08lXh / %08lXh\n", n, tmplr, tmplr, dumplr[n]);
  }

  dump += "\n";
  dump += "Lastreads from the USERS.INF file:\n";
  dump += "\n";
  for(; n<wide->numareas; n++)
    dump += gstrf("Board %4u : %10lu / %08lXh\n", n, dumplr[n], dumplr[n]);

  dump += "\n";
  dump += "Hexdump of message header:\n";
  dump += "\n";

  addhexdump(dump, &_hdr, sizeof(PcbHdr), "\n");

  throw_free(dumplr);
  throw_free(txt);

  GFTRK(NULL);
}


// ------------------------------------------------------------------

