
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbsqsh.h,v 1.1.1.1 1998/11/21 19:38:14 odinn Exp $
//  ------------------------------------------------------------------
//  Squish msgbase.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GMSQSH_H
#define __GMSQSH_H


//  ------------------------------------------------------------------

#include <gmbarea.h>
#include <limits.h>
#include <gusrmax.h>


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


//  ------------------------------------------------------------------

#if defined(__MSDOS__) and not defined(__386__)
const long MAX_IDXBUFSIZE = 65300L;
#else
const long MAX_IDXBUFSIZE = LONG_MAX;
#endif


//  ------------------------------------------------------------------
//  Bitmasks for 'attr'

#define MSGPRIVATE 0x00000001L
#define MSGCRASH   0x00000002L
#define MSGREAD    0x00000004L
#define MSGSENT    0x00000008L
#define MSGFILE    0x00000010L
#define MSGFWD     0x00000020L
#define MSGORPHAN  0x00000040L
#define MSGKILL    0x00000080L
#define MSGLOCAL   0x00000100L
#define MSGHOLD    0x00000200L
#define MSGXX2     0x00000400L
#define MSGFRQ     0x00000800L
#define MSGRRQ     0x00001000L
#define MSGCPT     0x00002000L
#define MSGARQ     0x00004000L
#define MSGURQ     0x00008000L
#define MSGSCANNED 0x00010000L
#define MSGUID     0x00020000L
#define MSGPRINTED 0x00040000L
#define MSGSEEN    0x00080000L
#define MSGLOK     0x40000000L


//  ------------------------------------------------------------------
//  Max number of stored replies to one msg

#define MAX_REPLY 9


//  ------------------------------------------------------------------
//  Structure of message headers in .SQD

struct SqshHdr {
  dword    attr;
  char     from[36];
  char     to[36];
  char     subj[72];
  ftn_addr orig;
  ftn_addr dest;
  gopustime date_written;   // When user wrote the msg (UTC)
  gopustime date_arrived;   // When msg arrived on-line (UTC)
  short    utc_offset;     // Offset from UTC of message writer, in minutes.
  dword    replyto;
  dword    replies[MAX_REPLY];
  dword    umsgid;
  char     ftsc_date[20];
};


//  ------------------------------------------------------------------
//  Squish frames

const dword SQFRAMEID      = 0xAFAE4453L;
const dword SQFRAME_NULL   = 0;
const word SQFRAME_NORMAL = 0;
const word SQFRAME_FREE   = 1;


//  ------------------------------------------------------------------
//  Structure of message frames in .SQD

struct SqshFrm {
  dword id;               // Must always equal SQFRAMEID
  long  next;             // Offset of next frame
  long  prev;             // Offset of previous frame
  dword length;           // Length of this frame
  dword totsize;          // Length of data in frame (hdr+ctl+txt)
  dword ctlsize;          // Length of control info
  word  type;             // Frm type (normal or free)
  word  reserved;         // Reserved
};


//  ------------------------------------------------------------------
//  Structure of index file .SQI

struct SqshIdx {
  long  offset;           // Offset of frame in .SQD
  dword msgno;            // Message number
  dword hash;             // Hash value of TO: name
};


//  ------------------------------------------------------------------
//  Structure of base record in .SQD

struct SqshBase {
  word  size;             // sizeof(SqshBase)
  word  reserved1;        // reserved
  dword totalmsgs;        // Number of messages in area
  dword highestmsg;       // Highest msg in area. Same as totalmsgs
  dword protmsgs;         // Skip killing first x msgs in area
  dword highwatermark;    // Relno (not Tagno) of HWM
  dword nextmsgno;        // Next message number to use
  char  name[80];         // Base name of SquishFile
  long  firstframe;       // Offset of first frame in file
  long  lastframe;        // Offset to last frame in file
  long  firstfreeframe;   // Offset of first FREE frame in file
  long  lastfreeframe;    // Offset of last free frame in file
  long  endframe;         // Pointer to end of file
  dword maxmsgs;          // Max # of msgs to keep in area
  word  daystokeep;       // Max age of msgs in area (for packing util)
  word  framesize;        // sizeof(SqshFrm)
  byte  reserved2[124];   // Reserved by Squish for future use
};


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------

class gmb_squish_area : public gmb_area {

protected:

  gmb_squish_manager* wide;

  int      fhsqd;
  int      fhsqi;
  SqshBase base;
  SqshIdx* idx;
  bool     softlock;

  int  test_open(const char* file);
  void raw_scan(int keep_index, int scanpm=false);
  void raw_open();
  void raw_close();
  void raw_lock();
  void refresh();
  void read_frm(dword offset, SqshFrm* frm);
  void write_frm(dword offset, SqshFrm* frm);
  void upd_frm_next(dword offset, SqshFrm* frm, dword next);
  void upd_frm_prev(dword offset, SqshFrm* frm, dword prev);
  void add_to_free_chain(dword delframe, SqshFrm* delfrm);
  void delete_msg(uint reln);
  void init_frm(SqshFrm* frm);
  void excess_frm(dword lastframe, dword newframe, SqshFrm* newfrm, dword totsize);
  uint find_msgn(ulong tagn);
  bool load_message(int mode, gmsg& msg, SqshHdr& hdr);
  void save_message(int mode, gmsg& msg, SqshHdr& hdr);

public:

  gmb_squish_area(gareacfg* c, gmb_squish_manager* m) : gmb_area(c) { wide = m; }
  virtual ~gmb_squish_area() { }

  void init(gareacfg* c, gmb_squish_manager* m)  { gmb_area::init(c); wide = m; }

  void activate();
  void deactivate();

  void open();
  void close();

  void save_lastread();

  void suspend();
  void resume();

  void lock();
  void unlock();

  void scan();
  void scan_area();
  void scan_area_pm();

  bool load_hdr(gmsg& msg);
  bool load_msg(gmsg& msg);

  void save_hdr(int mode, gmsg& msg);
  void save_msg(int mode, gmsg& msg);

  void del_msg(gmsg& msg);

  void new_msgno(gmsg& msg);
  bool user_lookup(gstring& lookfor);
  bool renumber();

  void update_timesread(gmsg& msg);

  void hexdump_msg(gmsg& msg, gstring& dump);

};


//  ------------------------------------------------------------------
//  Squish scanning types

const int SQS_API   = 1;
const int SQS_QUICK = 2;


//  ------------------------------------------------------------------
//  Squish recycling types

const int SQUISHRECYCLE_NO      = false;
const int SQUISHRECYCLE_YES     = true;
const int SQUISHRECYCLE_MSGAPI1 = SQUISHRECYCLE_YES + 1;
const int SQUISHRECYCLE_MSGAPI2 = SQUISHRECYCLE_YES + 2;


//  ------------------------------------------------------------------

class gmb_squish_manager : public gmb_area_manager {

protected:

  MaximusUser user;

  gstring     userpath;
  int         userno;
  int         direct;
  int         recycle;
  int         squishscan;

  friend class gmb_squish_area;

public:

  gmb_squish_manager();
  ~gmb_squish_manager();

  void initialize();
  void reset();

  void wide_open()   { }
  void wide_close()  { }

  gmb_area* new_area(gareacfg* c);
  void init_area(gmb_area* a, gareacfg* c);

  void set_userpath(const gstring& u)  { userpath = u; }
  void set_userno(int u)               { userno = u; }
  void set_direct(int d)               { direct = d; }
  void set_squishscan(int s)           { squishscan = s; }

};


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

