
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbsqsh2.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Squish msgbase handling and Maximus user functions.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gmemdbg.h>
#include <gdbgerr.h>
#include <gdbgtrk.h>
#include <gstrall.h>
#include <gcrcall.h>

#include <gmbsqsh.h>

              
//  ------------------------------------------------------------------

void gmb_squish_area::refresh() {

  GFTRK("gmb_squish_area::refresh");

  // Create new base record or read the existing one
  if(fhsqd != -1) {
    if(::filelength(fhsqd) < sizeof(SqshBase)) {
      memset(&base, 0, sizeof(SqshBase));
      base.size = sizeof(SqshBase);
      strxcpy(base.name, path().c_str(), sizeof(base.name));
      base.endframe = base.size;
      base.framesize = sizeof(SqshFrm);
      base.nextmsgno = 2;
      ::lseekset(fhsqd, 0);
      ::write(fhsqd, &base, sizeof(SqshBase));
    }
    else {
      ::lseekset(fhsqd, 0);
      ::read(fhsqd, &base, sizeof(SqshBase));
    }
  }

  // Are there any msgs?
  if(base.totalmsgs) {

    // Check for physical limit in DOS real mode
    #if defined(__MSDOS__) and not defined(__386__)
    if(base.totalmsgs > (65535L/sizeof(SqshIdx))) {
      gmb_wide_log->ErrIndex();
      gmb_wide_log->printf("! A Squish msgbase index was too large to handle.");
      gmb_wide_log->printf(": %s (%s) with %lu msgs.", path(), echoid(), base.totalmsgs);
      gmb_wide_log->printf("+ Info: GoldED-DOS only supports up to %lu msgs in a Squish area.", 65535L/sizeof(SqshIdx));
      gmb_wide_log->printf("+ Advice: Purge and pack the area or use GoldED/386 or GoldED/2.");
      IndexErrorExit();
    }
    #endif

    // Read the index file
    idx = (SqshIdx*)throw_realloc(idx, (uint)(base.totalmsgs*sizeof(SqshIdx)));
    ::lseekset(fhsqi, 0);
    ::read(fhsqi, idx, (uint)(base.totalmsgs*sizeof(SqshIdx)));
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_squish_area::raw_scan(int __keep_index, int __scanpm) {

  GFTRK("gmb_squish_area::raw_scan");

  // Load the lastread
  dword _lastread = 0;
  int _fh = ::sopen(AddPath(path().c_str(), ".SQL"), O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
  if(_fh != -1) {
    ::lseekset(_fh, wide->userno, sizeof(dword));
    ::read(_fh, &_lastread, sizeof(dword));
    ::close(_fh);
  }

  // Open Squish files for scanning unless they are already open
  if(open_refs == 0) {

    idx = NULL;
    base.totalmsgs = 0;

    // Open index file
    fhsqi = ::sopen(AddPath(path().c_str(), ".SQI"), O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
    if(fhsqi != -1) {

      // Get the number of index records
      base.totalmsgs = ::filelength(fhsqi) / sizeof(SqshIdx);

      // If there appear to be only one record, experience shows that
      // there may in fact be zero!  So check the *.SQD base record for
      // the exact number of msgs in this case or if requested.
      if((base.totalmsgs == 1) or (wide->squishscan == SQS_API)) {

        // Open, read and close data file
        fhsqd = ::sopen(AddPath(path().c_str(), ".SQD"), O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
        if(fhsqd != -1) {
          ::read(fhsqd, &base, sizeof(SqshBase));
          ::close(fhsqd);
          fhsqd = -1;
        }
      }
      else {
        fhsqd = -1;
      }

      // Allocate index buffer and read from file
      refresh();

      // Close index file
      ::close(fhsqi);
      fhsqi = -1;
    } 
  }

  register ulong _msgno;
  register SqshIdx* _sqiptr = idx;
  register dword  _totalmsgs = base.totalmsgs;
  register ulong _firstmsgno = _totalmsgs ? _sqiptr->msgno : 0;
  register ulong _lastmsgno = 0;
  register uint _active = 0;
  register uint _lastread_reln = 0;
  register ulong _lastreadfound = 0;

  if(base.totalmsgs) {

    // (Re)allocate message index
    if(__keep_index)
      msgnset.resize((uint)base.totalmsgs);

    register ulong* _msgndxptr = msgnset.tag;

    // Fill message index
    while(_active < _totalmsgs) {

      _active++;
      _msgno = (_sqiptr++)->msgno;
      if(__keep_index)
        *_msgndxptr++ = _msgno;

      // Check for premature end of index (free frames)
      if((_msgno <= _lastmsgno) or (_msgno == 0xFFFFFFFFL)) {
        _active--;
        if((_msgno == _lastmsgno) and _active == 1) {
          _lastread_reln = 0;
          _active = 0;
        }
        break;
      }

      // Get the lastread
      if((_msgno >= _lastread) and (_lastread_reln == 0)) {
        _lastreadfound = _msgno;
        _lastread_reln = _active - (_msgno != _lastread ? 1 : 0);
      }

      // Store last message number
      _lastmsgno = _msgno;
    }

    // If the exact lastread was not found
    if(_active and (_lastreadfound != _lastread)) {

      // Higher than highest or lower than lowest?
      if(_lastread > _lastmsgno)
        _lastread_reln = _active;
      else if(_lastread < _firstmsgno)
        _lastread_reln = 0;
    }
  }

  // Update area data
  scan_count = _active;
  msgnset.resize(_active);
  lastread = _lastread_reln;
  lastreadentry = _lastreadfound;

  // Scan for personal mail
  pmrkset.reset();
  if(__scanpm) {
    int umax = (gmb_wide_personalmail & PM_ALLNAMES) ? gmb_wide_usernames : 1;
    dword* uhash = (dword*)throw_calloc(umax, sizeof(dword));
    for(int uh=0; uh<umax; uh++)
      uhash[uh] = strHash32(gmb_wide_username[uh]);
    register uint n = lastread + 1;
    register uint cnt = msgnset.size();
    register int gotpm = false;
    while(n <= cnt) {
      SqshIdx* idxp = idx + (n-1);
      for(int u=0; u<umax; u++) {
        if((idxp->hash & 0x80000000LU) == 0) {
          if(idxp->hash == uhash[u]) {
            gotpm = true;
            break;
          }
        }
      }
      if(gotpm) {
        pmrkset.append(msgnset.at(n-1));
        gotpm = false;
      }
      n++;
    }
    throw_free(uhash);
  }
  pmscan_count = pmrkset.size();

  if(gmb_wide_debug) {
    gmb_wide_log->printf("- %s: t:%u, l:%u, fm:%lu, hm:%lu, lr:%lu, u:%u, pm:%i",
      echoid().c_str(),
      msgnset.size(),
      lastread,
      _firstmsgno,
      _lastmsgno,
      _lastread,
      wide->userno,
      __scanpm ? pmrkset.size() : -1
    );
  }

  // Free index buffer if just counting
  if(not __keep_index or __scanpm)
    throw_release(idx);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_squish_area::scan() {

  GFTRK("gmb_squish_area::scan");

  raw_scan(true);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_squish_area::scan_area() {

  GFTRK("gmb_squish_area::scan_area");

  raw_scan(false);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_squish_area::scan_area_pm() {

  GFTRK("gmb_squish_area::scan_area_pm");

  raw_scan(true, true);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

