
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbsqsh3.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Squish msgbase handling and Maximus user functions.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gtxtpara.h>
#include <gstrall.h>

#include <gmbsqsh.h>

              
//  ------------------------------------------------------------------

bool gmb_squish_area::load_message(int mode, gmsg& msg, SqshHdr& hdr) {

  uint reln = msgnset.absolute_to_relative(msg.msgno);

  if(reln == 0) {
    GFTRK(NULL);
    return false;
  }

  // Load the message frame
  SqshFrm frm;
  lseekset(fhsqd, idx[reln-1].offset);
  read(fhsqd, &frm, sizeof(SqshFrm));

  // Load the message header
  memset(&hdr, 0, sizeof(SqshHdr));
  read(fhsqd, &hdr, sizeof(SqshHdr));

  // Read control info and message text
  if(mode & GMSG_TXT) {

    if(frm.length) {

      // Read and convert Squish control info block
      if(frm.ctlsize) {
        char* ctlbuf = (char*)throw_xmalloc((uint)frm.ctlsize+1);
        read(fhsqd, ctlbuf, (uint)frm.ctlsize);
        gtokenizer token("\x1", ctlbuf);
        while(token.get()) {
          if(*token.c_str())
            msg.add_control(token.c_str());
        }
        throw_xfree(ctlbuf);
      }

      // Read message text
      ulong textlen = frm.totsize - frm.ctlsize - sizeof(SqshHdr);
      if(textlen) {
        if(textlen > 16777216L) {
          gmb_wide_log->printf("- Memory allocation failure imminent. Info follows:");
          gmb_wide_log->printf(": Msgbase: %s.SQ*", path().c_str());
          gmb_wide_log->printf(": Msgno=%lu, relno=%u", msg.msgno, reln);
          gmb_wide_log->printf(": Index: offset=%lu, msgno=%lu", idx[reln].offset, idx[reln].msgno);
          gmb_wide_log->printf(": Frame: id=%08lX, len=%lu, tot=%lu, ctl=%lu", frm.id, frm.length, frm.totsize, frm.ctlsize);
          gmb_wide_log->printf(": Frame: prev=%08lX (%lu), next=%08lX (%lu)", frm.prev, frm.prev, frm.next, frm.next);
          gmb_wide_log->printf("+ Advice: This problem is usually due to a partially corrupted msgbase.");
          gmb_wide_log->printf("+         You should run SQREIDX first and see it if fixes it.");
          gmb_wide_log->printf("+         If that doesn't help, run SQFIX.");
          gmb_wide_log->printf("+         Please send this information to the Author.");
        }
        if(textlen > gmb_wide_msgsize)
          textlen = gmb_wide_msgsize;
        msg.text.reserve((uint)textlen+1);
        read(fhsqd, msg.text.pointer(), (uint)textlen);
        msg.text.raw_to_lines((uint)textlen);
      }
    }
  }

  // Convert header

  msg.link.to_set(hdr.replyto);
  msg.link.first_set(hdr.replies[0]);

  // Convert link.list
  int q = 0;
  int r = hdr.replies[0] == hdr.replies[1] ? 2 : 1;
  while(r<=8) {
    if(hdr.replies[r] and hdr.replies[r-1] != hdr.replies[r])
      msg.link.list_set(q++, hdr.replies[r]);
    r++;
  }

  msg.origname.copyn(hdr.from, sizeof(hdr.from));
  msg.destname.copyn(hdr.to,   sizeof(hdr.to));
  msg.subject.copyn(hdr.subj, sizeof(hdr.subj));

  msg.origaddr = hdr.orig;
  if(isnet())
    msg.destaddr = hdr.dest;

  msg.timesread = (hdr.attr & MSGSEEN) ? 1 : 0;
  
  // Convert date and time
  msg.date.written = hdr.date_written;
  if(not msg.date.written.valid())
    msg.date.written.convert_fts1(hdr.ftsc_date);
  msg.date.written.utc_offset = (hdr.utc_offset/60)*100 + (hdr.utc_offset%60);
  msg.date.arrived = hdr.date_arrived;

  // Convert attributes
  msg.attr.pvt(hdr.attr & MSGPRIVATE);
  msg.attr.cra(hdr.attr & MSGCRASH);
  msg.attr.rcv(hdr.attr & MSGREAD);
  msg.attr.snt(hdr.attr & MSGSENT);
  msg.attr.att(hdr.attr & MSGFILE);
  msg.attr.trs(hdr.attr & MSGFWD);
  msg.attr.orp(hdr.attr & MSGORPHAN);
  msg.attr.k_s(hdr.attr & MSGKILL);
  msg.attr.loc(hdr.attr & MSGLOCAL);
  msg.attr.hld(hdr.attr & MSGHOLD);
  msg.attr.rsv(hdr.attr & MSGXX2);
  msg.attr.frq(hdr.attr & MSGFRQ);
  msg.attr.rrq(hdr.attr & MSGRRQ);
  msg.attr.rrc(hdr.attr & MSGCPT);
  msg.attr.arq(hdr.attr & MSGARQ);
  msg.attr.urq(hdr.attr & MSGURQ);
  msg.attr.scn(hdr.attr & MSGSCANNED);
  msg.attr.prn(hdr.attr & MSGPRINTED);
  msg.attr.lok(hdr.attr & MSGLOK);

  if(isnet())
    msg.attr.uns(msg.attr.loc() and not msg.attr.snt());
  else if(isecho())
    msg.attr.uns(msg.attr.loc() and ((hdr.attr & MSGSCANNED) ? 0 : 1));
  else
    msg.attr.uns0();

  GFTRK(NULL);

  return true;
}


//  ------------------------------------------------------------------

bool gmb_squish_area::load_hdr(gmsg& msg) {

  GFTRK("SquishLoadHdr");

  SqshHdr hdr;
  return load_message(GMSG_HDR, msg, hdr);
}


//  ------------------------------------------------------------------

bool gmb_squish_area::load_msg(gmsg& msg) {

  GFTRK("SquishLoadMsg");

  SqshHdr hdr;
  return load_message(GMSG_HDRTXT, msg, hdr);
}


//  ------------------------------------------------------------------

