
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbsqsh5.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  Squish msgbase handling and Maximus user functions.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgtrk.h>
#include <gmemdbg.h>
#include <gmemall.h>
#include <gstrall.h>
#include <gutlmisc.h>

#include <gmbsqsh.h>

              
//  ------------------------------------------------------------------

bool gmb_squish_area::user_lookup(gstring& lookfor) {

  Path userfile;
  PathCopy(userfile, AddPath(wide->userpath.c_str(), "USER.BBS"));
  wide->user.fh = ::sopen(userfile, O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
  char buf[256];
  if(wide->user.findwild(lookfor.c_str(), buf))
    lookfor = buf;
  ::close(wide->user.fh);

  return wide->user.found;
}


//  ------------------------------------------------------------------

bool gmb_squish_area::renumber() {

  return false;
}


//  ------------------------------------------------------------------

void gmb_squish_area::hexdump_msg(gmsg& msg, gstring& dump) {

  GFTRK("SquishMakeDump");

  // Load the base record
  SqshBase _base;
  lseekset(fhsqd, 0);
  read(fhsqd, &_base, sizeof(SqshBase));

  // Load the index record
  SqshIdx _idx;
  uint _reln = msgnset.absolute_to_relative(msg.msgno);
  lseekset(fhsqi, _reln-1, sizeof(SqshIdx));
  read(fhsqi, &_idx, sizeof(SqshIdx));

  // Load the message frame
  SqshFrm _frm;
  dword _frame = _idx.offset;
  lseekset(fhsqd, _frame);
  read(fhsqd, &_frm, sizeof(SqshFrm));

  // Load the header
  SqshHdr _hdr;
  read(fhsqd, &_hdr, sizeof(SqshHdr));

  // Load the control info
  char* _ctl = (char*)throw_xcalloc(1, _frm.ctlsize+1);
  read(fhsqd, _ctl, _frm.ctlsize);

  // Load the message text
  uint _txtlen = (uint)(_frm.totsize - _frm.ctlsize - sizeof(SqshHdr));
  char* txt = (char*)throw_calloc(1, 1+_txtlen);
  read(fhsqd, txt, _txtlen);
  txt[_txtlen] = NUL;

  char buf[100];

  dump =  "Hexdump of Squish-style message header and text\n";
  dump += "------------------------------------------------------------------------------\n";
  dump += "\n";
  dump += gstrf("Msgbase        : %s\n", path().c_str());
  dump += gstrf("From           : %-36.36s\n", _hdr.from);
  dump += gstrf("To             : %-36.36s\n", _hdr.to);
  dump += gstrf("Subject        : %-72.72s\n", _hdr.subj);
  dump += gstrf("DateTime       : %-20.20s\n", _hdr.ftsc_date);
  dump += gstrf("OrigAddr       : %u:%u/%u.%u\n", _hdr.orig.zone, _hdr.orig.net, _hdr.orig.node, _hdr.orig.point);
  dump += gstrf("DestAddr       : %u:%u/%u.%u\n", _hdr.dest.zone, _hdr.dest.net, _hdr.dest.node, _hdr.dest.point);
  dump += gstrf("Umsgid         : %lu\n", _hdr.umsgid);
  dump += gstrf("Reply          : %lu\n", _hdr.replyto);
  dump += gstrf("See            : %lu, %lu, %lu, %lu, %lu, %lu, %lu, %lu, %lu\n",
    _hdr.replies[0], _hdr.replies[1], _hdr.replies[2],
    _hdr.replies[3], _hdr.replies[4], _hdr.replies[5],
    _hdr.replies[6], _hdr.replies[7], _hdr.replies[8]
  );
  dump += gstrf("Attr           : %08lXh (%sb)\n", _hdr.attr, ltob(buf,_hdr.attr,0));
  dump += gstrf("DateWritten    : %s (%08lXh)\n", _hdr.date_written.c_str(buf), _hdr.date_written.number());
  dump += gstrf("DateArrived    : %s (%08lXh)\n", _hdr.date_arrived.c_str(buf), _hdr.date_arrived.number());
  dump += gstrf("UTC-Offset     : %u\n", _hdr.utc_offset);
  dump += gstrf("UserRecno      : %u (%s)\n", wide->userno, gmb_wide_username[0]);
  dump += "\n";
  dump += "Message Base Record:\n";
  dump += "\n";
  dump += gstrf("TotalMsgs      : %lu\n", _base.totalmsgs);
  dump += gstrf("HighestMsg     : %lu\n", _base.highestmsg);
  dump += gstrf("NextMsgno      : %lu\n", _base.nextmsgno);
  dump += gstrf("HighWaterMark  : %lu\n", _base.highwatermark);
  dump += gstrf("FirstFrame     : %08lXh (%lu)\n", _base.firstframe, _base.firstframe);
  dump += gstrf("LastFrame      : %08lXh (%lu)\n", _base.lastframe, _base.lastframe);
  dump += gstrf("FirstFreeFrame : %08lXh (%lu)\n", _base.firstfreeframe, _base.firstfreeframe);
  dump += gstrf("LastFreeFrame  : %08lXh (%lu)\n", _base.lastfreeframe, _base.lastfreeframe);
  dump += gstrf("EndFrame       : %08lXh (%lu)\n", _base.endframe, _base.endframe);
  dump += gstrf("Max/Skip/Days  : %lu  %lu  %u\n", _base.maxmsgs, _base.protmsgs, _base.daystokeep);
  dump += "\n";
  dump += "Message Index Record:\n";
  dump += "\n";
  dump += gstrf("FrameOffset    : %08lXh (%lu)\n", _idx.offset, _idx.offset);
  dump += gstrf("MessageNumber  : %08lXh (%lu)\n", _idx.msgno, _idx.msgno);
  dump += gstrf("HashValue      : %08lXh (%lu)\n", _idx.hash, _idx.hash);
  dump += "\n";
  dump += "Message Frame Record:\n";
  dump += "\n";
  dump += gstrf("Frame-ID       : %08lXh%s\n", _frm.id, (_frm.id != SQFRAMEID) ? " (error, should be AFAE4453h!)" : "");
  dump += gstrf("ThisFrame      : %08lXh (%lu)\n", _frame, _frame);
  dump += gstrf("PrevFrame      : %08lXh (%lu)\n", _frm.prev, _frm.prev);
  dump += gstrf("NextFrame      : %08lXh (%lu)\n", _frm.next, _frm.next);
  dump += gstrf("FrameLength    : %lu\n", _frm.length);
  dump += gstrf("TotalLength    : %lu\n", _frm.totsize);
  dump += gstrf("CtrlLength     : %lu\n", _frm.ctlsize);

  dump += "\n";
  dump += "Hexdump of message header:\n";
  dump += "\n";

  addhexdump(dump, &_hdr, sizeof(SqshHdr), "\n");

  dump += "\n";
  dump += "Hexdump of control info:\n";
  dump += "\n";

  addhexdump(dump, _ctl, _frm.ctlsize, "\n");

  throw_xfree(_ctl);
  throw_xfree(txt);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

