
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbwcat1.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  WildCat! 4.x messagebase engine.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>

#include <gmbwcat.h>


//  ------------------------------------------------------------------

gmb_wildcat_manager::gmb_wildcat_manager() {

  userno = 0;
}


//  ------------------------------------------------------------------

gmb_wildcat_manager::~gmb_wildcat_manager() {

  reset();
}


//  ------------------------------------------------------------------

gmb_area* gmb_wildcat_manager::new_area(gareacfg* c) {

  gmb_wildcat_area* area = new gmb_wildcat_area(c, this);
  throw_new(area);
  return area;
}


//  ------------------------------------------------------------------

void gmb_wildcat_manager::init_area(gmb_area* a, gareacfg* c) {

  gmb_wildcat_area* area = (gmb_wildcat_area*)a;
  area->init(c, this);
}


//  ------------------------------------------------------------------

void gmb_wildcat_manager::initialize() {

}


//  ------------------------------------------------------------------

void gmb_wildcat_manager::reset() {

  for(int n=0; n<active.size(); n++)
    delete active[n];
  active.reset();
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::raw_close() {

  GFTRK("gmb_wildcat_area::RawClose");

  if(fhix != -1)   ::close(fhix);   fhix = -1;
  if(fhdat != -1)  ::close(fhdat);  fhdat = -1;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

int gmb_wildcat_area::test_open(const char* __file) {

  GFTRK("gmb_wildcat_area::TestOpen");

  int _fh;
  long _tries = 0;

  do {

    _fh = ::sopen(__file, O_RDWR|O_BINARY|O_CREAT, gmb_wide_sharemode, S_STDRW);
    if(_fh == -1) {

      // Tell the world
      if(PopupLocked(++_tries, false, __file) == false) {

        // User requested to exit
        gmb_wide_log->ErrOpen();
        raw_close();
        gmb_wide_log->printf("! A WildCat! msgbase file could not be opened.");
        gmb_wide_log->printf(": %s.", __file);
        gmb_wide_log->ErrOSInfo();
        OpenErrorExit();
      }
    }
  } while(_fh == -1);

  // Remove the popup window
  if(_tries)
    PopupLocked(0, 0, NULL);

  GFTRK(NULL);

  return _fh;
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::raw_open() {

  GFTRK("gmb_wildcat_area::RawOpen");

  fhix  = test_open(AddPath(path().c_str(), ".IX"));
  fhdat = test_open(AddPath(path().c_str(), ".DAT"));

  GFTRK(NULL);
}


// ------------------------------------------------------------------

void gmb_wildcat_area::activate() {

  refs++;
  if(refs == 1) {
    memset(&base, 0, sizeof(WCatBase));
    fhix = fhdat = -1;
    idx = NULL;
  }
}


// ------------------------------------------------------------------

void gmb_wildcat_area::deactivate() {

  if(refs) {
    if(refs == 1) {
      if(open_refs) {
        open_refs = 1;
        close();
      }
    }
    refs--;
  }
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::open() {

  GFTRK("gmb_wildcat_area::Open");

  open_refs++;
  if(open_refs == 1) {
    raw_open();
    refresh();
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::close() {

  GFTRK("gmb_wildcat_area::Close");

  if(open_refs) {
    if(open_refs == 1) {
      if(lock_refs) {
        lock_refs = 1;
        unlock();
      }
      raw_close();
      msgnset.reset();
      pmrkset.reset();
      throw_release(idx);
      memset(&base, 0, sizeof(WCatBase));
    }
    open_refs--;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::save_lastread() {

  GFTRK("gmb_wildcat_area::SaveLastread");

  int fh = ::sopen(AddPath(path().c_str(), ".LRD"), O_RDWR|O_CREAT|O_BINARY, gmb_wide_sharemode, S_STDRW);
  if(fh != -1) {
    word _lastread = (word)msgnset.relative_to_absolute(lastread);
    lseekset(fh, wide->userno, sizeof(word));
    write(fh, &_lastread, sizeof(word));
    ::close(fh);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::suspend() {

  GFTRK("gmb_wildcat_area::Suspend");

  save_lastread();
  raw_close();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::resume() {

  GFTRK("gmb_wildcat_area::Resume");

  raw_open();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

