
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbwcat2.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  WildCat! 4.x messagebase engine.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>

#include <gmbwcat.h>


//  ------------------------------------------------------------------

void gmb_wildcat_area::refresh() {

  GFTRK("gmb_wildcat_area::refresh");

  // Create new base record or read the existing one
  if(fhix != -1) {
    if(filelength(fhix) < sizeof(WCatBase)) {
      base.recsize = sizeof(WCatIdx);
      base.active = 0;
      base.nextmsgno = 1;
      lseekset(fhix, 0);
      write(fhix, &base, sizeof(WCatBase));
    }
    else {
      lseekset(fhix, 0);
      read(fhix, &base, sizeof(WCatBase));
    }
  }

  long ixnum = (filelength(fhix)-(long)sizeof(WCatBase)) / (long)sizeof(WCatIdx);

  // Are there any msgs?
  if(ixnum) {

    // Check for physical limit in DOS real mode
    #if defined(__MSDOS__) and not defined(__386__)
    if(ixnum > (65535L/sizeof(WCatIdx))) {
      gmb_wide_log->ErrIndex();
      gmb_wide_log->printf("! A WildCat! msgbase index was too large to handle.");
      gmb_wide_log->printf(": %s (%s) with %lu index records.", path().c_str(), echoid().c_str(), ixnum);
      gmb_wide_log->printf("+ Info: GoldED-DOS only supports up to %lu msgs in a WildCat! area.", 65535L/sizeof(WCatIdx));
      gmb_wide_log->printf("+ Advice: Purge and pack the area or use GoldED/386 or GoldED/2.");
      IndexErrorExit();
    }
    #endif

    // Read the index file
    idx = (WCatIdx*)throw_realloc(idx, (uint)(ixnum*sizeof(WCatIdx)));
    lseekset(fhix, sizeof(WCatBase));
    read(fhix, idx, (uint)(ixnum*sizeof(WCatIdx)));
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::raw_scan(int __keep_index, int __scanpm) {

  GFTRK("gmb_wildcat_area::raw_scan");

  // Load the lastread
  dword _lastread = 0;
  int _fh = ::sopen(AddPath(path().c_str(), ".LRD"), O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
  if(_fh != -1) {
    lseekset(_fh, wide->userno, sizeof(dword));
    read(_fh, &_lastread, sizeof(dword));
    ::close(_fh);
  }

  // Open WildCat! files for scanning unless they are already open
  if(open_refs == 0) {

    // Open index file
    fhix = ::sopen(AddPath(path().c_str(), ".IX"), O_RDONLY|O_BINARY, gmb_wide_sharemode, S_STDRD);
    if(fhix != -1) {

      // Allocate index buffer and read from file
      refresh();

      // Close index file
      ::close(fhix);
      fhix = -1;
    } 
  }

  register uint _active = 0;
  register uint _lastread_reln = 0;
  register ulong _lastreadfound = 0;

  if(base.active) {

    // (Re)allocate message index
    if(__keep_index)
      msgnset.resize((uint)base.active);

    // Variables for the loop
    register ulong _msgno;
    register ulong* _msgndxptr = msgnset.tag;
    register WCatIdx* _ixptr = idx;
    register dword  _totalmsgs = base.active;
    register ulong _firstmsgno = _totalmsgs ? _ixptr->msgno : 0;
    register ulong _lastmsgno = 0;

    // Fill message index
    while(_active < _totalmsgs) {

      _active++;
      _msgno = (_ixptr++)->msgno;
      if(not _firstmsgno)
        _firstmsgno = _msgno;
      if(__keep_index)
        *_msgndxptr++ = _msgno;

      // Get the lastread
      if((_msgno >= _lastread) and (_lastread_reln == 0)) {
        _lastreadfound = _msgno;
        _lastread_reln = (uint)(_active - (_msgno != _lastread ? 1 : 0));
      }

      // Store last message number
      _lastmsgno = _msgno;
    }

    // If the exact lastread was not found
    if(_active and (_lastreadfound != _lastread)) {

      // Higher than highest or lower than lowest?
      if(_lastread > _lastmsgno)
        _lastread_reln = _active;
      else if(_lastread < _firstmsgno)
        _lastread_reln = 0;
    }
  }

  // Update area data
  scan_count = _active;
  msgnset.resize(_active);
  lastread = _lastread_reln;
  lastreadentry = _lastreadfound;

  // Scan for personal mail
  pmrkset.reset();
  if(__scanpm) {
    // Not implemented yet
  }
  pmscan_count = pmrkset.size();

  // Free index buffer if just counting
  if(not __keep_index or __scanpm)
    throw_release(idx);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::scan() {

  GFTRK("gmb_wildcat_area::scan");

  raw_scan(true);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::scan_area() {

  GFTRK("gmb_wildcat_area::scan_area");

  raw_scan(false);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::scan_area_pm() {

  GFTRK("gmb_wildcat_area::scan_area_pm");

  raw_scan(true, true);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

