
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbwcat3.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  WildCat! 4.x messagebase engine.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>

#include <gmbwcat.h>


//  ------------------------------------------------------------------

bool gmb_wildcat_area::load_message(int mode, gmsg& msg, WCatHdr& hdr) {

  // Setup some local variables for speed
  int _fhdat = fhdat;
  WCatIdx* _idx = idx;
  uint _reln = msgnset.absolute_to_relative(msg.msgno);

  // Load the message header
  memset(&hdr, 0, sizeof(WCatHdr));
  lseekset(_fhdat, _idx[_reln-1].offset);
  read(_fhdat, &hdr, sizeof(WCatHdr));

  // Convert header

  msg.link.to_set(hdr.reference);

  msg.origname = strnp2c(hdr.from, 70);
  msg.destname = strnp2c(hdr.to, 70);
  msg.subject  = strnp2c(hdr.subject, 70);

  strnp2cc(msg.wildcat.from_title,      hdr.fromtitle, 10);
  strnp2cc(msg.wildcat.to_title,        hdr.totitle, 10);
  strnp2cc(msg.wildcat.network,         hdr.network, 8);
  strnp2cc(msg.wildcat.internal_attach, hdr.internalattach, 12);
  strnp2cc(msg.wildcat.external_attach, hdr.externalattach, 12);

  memcpy(msg.wildcat.reserved, hdr.reserved, 20);

  msg.wildcat.from_userid = hdr.fromuserid;
  msg.wildcat.to_userid = hdr.touserid;
  msg.wildcat.next_unread = hdr.nextunread;
  msg.wildcat.prev_unread = hdr.prevunread;

  msg.cost = (uint)hdr.cost;
  msg.timesread = 1;

  msg.origaddr.zone  = hdr.origaddr.zone;
  msg.origaddr.net   = hdr.origaddr.net;
  msg.origaddr.node  = hdr.origaddr.node;
  msg.origaddr.point = hdr.origaddr.point;

  if(isnet()) {
    msg.destaddr.zone  = hdr.destaddr.zone;
    msg.destaddr.net   = hdr.destaddr.net;
    msg.destaddr.node  = hdr.destaddr.node;
    msg.destaddr.point = hdr.destaddr.point;
  }
  
  // Convert date and time
  if(hdr.msgdate and hdr.msgtime) {
    uint year, month, day;
    JDN2YMD(hdr.msgdate+1, &year, &month, &day);
    long hms = hdr.msgtime-1;
    msg.date.written.year   = year + 1900;
    msg.date.written.month  = month;
    msg.date.written.day    = day;
    msg.date.written.hour   = (int)(hms / 3600L);
    msg.date.written.minute = (int)((hms % 3600L) / 60L);
    msg.date.written.second = (int)(hms - (((long)msg.date.written.hour*3600L)+(long)msg.date.written.minute*60L));
  }
  if(hdr.readdate and hdr.readtime) {
    uint year, month, day;
    JDN2YMD(hdr.readdate+1, &year, &month, &day);
    long hms = hdr.readtime-1;
    msg.date.received.year   = year + 1900;
    msg.date.received.month  = month;
    msg.date.received.day    = day;
    msg.date.received.hour   = (int)(hms / 3600L);
    msg.date.received.minute = (int)((hms % 3600L) / 60L);
    msg.date.received.second = (int)(hms - (((long)msg.date.received.hour*3600L)+(long)msg.date.received.minute*60L));
  }

  // Convert fidoflag attributes
  msg.attr.pvt(hdr.fidoflags & FIDO_PVT);
  msg.attr.cra(hdr.fidoflags & FIDO_CRASH);
  msg.attr.rcv(hdr.fidoflags & FIDO_RECEIVED);
  msg.attr.snt(hdr.fidoflags & FIDO_SENT);
  msg.attr.att(hdr.fidoflags & FIDO_ATTACH);
  msg.attr.trs(hdr.fidoflags & FIDO_TRANSIT);
  msg.attr.orp(hdr.fidoflags & FIDO_ORPHAN);
  msg.attr.k_s(hdr.fidoflags & FIDO_KILLSENT);
  msg.attr.loc(hdr.fidoflags & FIDO_LOCAL);
  msg.attr.hld(hdr.fidoflags & FIDO_HOLD);
  msg.attr.rsv(hdr.fidoflags & FIDO_RESERVED);
  msg.attr.frq(hdr.fidoflags & FIDO_FREQ);
  msg.attr.rrq(hdr.fidoflags & FIDO_RETRECREQ);
  msg.attr.rrc(hdr.fidoflags & FIDO_RETREC);
  msg.attr.arq(hdr.fidoflags & FIDO_AUDITREQ);
  msg.attr.urq(hdr.fidoflags & FIDO_UPDREQ);

  // Convert mflag attributes
  msg.attr.pvt(hdr.mflags & mfPrivate);
  msg.attr.rab(hdr.mflags & mfReceiveable);
  msg.attr.rcv(hdr.mflags & mfReceived);
  msg.attr.rrc(hdr.mflags & mfReceipt);
  msg.attr.car(hdr.mflags & mfCarboned);
  msg.attr.fwd(hdr.mflags & mfForwarded);
  msg.attr.efl(hdr.mflags & mfEchoFlag);
  msg.attr.hrp(hdr.mflags & mfHasReplies);
  msg.attr.del(hdr.mflags & mfDeleted);
  msg.attr.tag(hdr.mflags & mfTagged);
  msg.attr.snt(hdr.mflags & mfSent);

  // Set the unsent attribute
  if(isnet())
    msg.attr.uns(msg.attr.loc() and not msg.attr.snt());
  else
    msg.attr.uns0();

  msg.wildcat.txtlength = hdr.msgbytes;

  // If message text is requested
  if(mode & GMSG_TXT) {

    uint textlen = hdr.msgbytes;
    if(textlen > gmb_wide_msgsize)
      textlen = (uint)gmb_wide_msgsize;

    // Allocate space for the message text
    msg.text.reserve(textlen+1);

    // Read the message text
    read(_fhdat, msg.text.pointer(), textlen);

    // Convert kludge char from NUL to CTRL-A
    char* p = msg.text.pointer();
    for(uint n=0; n<textlen; n++,p++) {
      if(*p == NUL)
        *p = CTRL_A;
    }

    msg.text.raw_to_lines(textlen);
  }

  GFTRK(NULL);

  // Success
  return true;
}


//  ------------------------------------------------------------------

bool gmb_wildcat_area::load_hdr(gmsg& msg) {

  GFTRK("gmb_wildcat_area::LoadHdr");

  WCatHdr _hdr;
  return load_message(GMSG_HDR, msg, _hdr);
}


//  ------------------------------------------------------------------

bool gmb_wildcat_area::load_msg(gmsg& msg) {

  GFTRK("gmb_wildcat_area::LoadMsg");

  WCatHdr _hdr;
  return load_message(GMSG_HDRTXT, msg, _hdr);
}


//  ------------------------------------------------------------------

