
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbwcat4.cpp,v 1.1.1.1 1998/11/21 19:38:13 odinn Exp $
//  ------------------------------------------------------------------
//  WildCat! 4.x messagebase engine.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>

#include <gmbwcat.h>


//  ------------------------------------------------------------------

void gmb_wildcat_area::lock() {

  GFTRK("gmb_wildcat_area::Lock");

  lock_refs++;
  if(lock_refs == 1) {
    if(gmb_wide_canlock) {
      long tries = 0;
      while(::lock(fhix, 0, 1) == -1) {
        if(PopupLocked(++tries, true, path().c_str()) == false) {
          gmb_wide_log->ErrLock();
          raw_close();
          gmb_wide_log->printf("! A WildCat! msgbase file could not be locked.");
          gmb_wide_log->printf(": %s.IX.", path().c_str());
          gmb_wide_log->ErrOSInfo();
          LockErrorExit();
        }
      }
      if(tries)
        PopupLocked(0, 0, NULL);
    }
    refresh();
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::unlock() {

  GFTRK("gmb_wildcat_area::Unlock");

  if(lock_refs) {
    if(lock_refs == 1) {
      if(gmb_wide_canlock)
        ::unlock(fhix, 0, 1);
    }
    lock_refs--;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::save_message(int mode, gmsg& msg, WCatHdr& hdr) {

  lock();

  memset(&hdr, 0, sizeof(WCatHdr));

  if(mode & GMSG_NEW) {
    base.active++;
    msg.msgno = base.nextmsgno++;
    msgnset.append(msg.msgno);
    idx = (WCatIdx*)throw_realloc(idx, base.active*sizeof(WCatIdx));
    hdr.mflags |= mfReceiveable;
    lseekset(fhix, 0);
    write(fhix, &base, sizeof(WCatBase));
    scan_count++;
  }
  else if(mode & GMSG_DELETE) {
    hdr.mflags |= mfDeleted;
  }

  hdr.magicnumber = MagicHeaderActive;
  hdr.msgno = (word)msg.msgno;

  strc2p(strupr(strxcpy(hdr.from, msg.origname.c_str(), sizeof(hdr.from))));
  strc2p(strcpy(hdr.fromtitle, msg.wildcat.from_title));
  hdr.fromuserid = msg.wildcat.from_userid;

  strc2p(strupr(strxcpy(hdr.to, msg.destname.c_str(), sizeof(hdr.to))));
  strc2p(strcpy(hdr.totitle, msg.wildcat.to_title));
  hdr.touserid = msg.wildcat.to_userid;

  strc2p(strxcpy(hdr.subject, msg.subject.c_str(), sizeof(hdr.subject)));
  
  if(!*msg.wildcat.network and (isnet() or isecho()))
    strcpy(msg.wildcat.network, "FTSC");
  strc2p(strcpy(hdr.network, msg.wildcat.network));

  if(msg.date.written.valid()) {
    hdr.msgdate = (word)(YMD2JDN(msg.date.written.year, msg.date.written.month, msg.date.written.day)-1);
    hdr.msgtime = (((long)msg.date.written.hour*3600L)+((long)msg.date.written.minute*60L)+msg.date.written.second)+1;
  }
  if(msg.date.received.valid()) {
    hdr.readdate = (word)YMD2JDN(msg.date.received.year, msg.date.received.month, msg.date.received.day);
    hdr.readtime = (((long)msg.date.received.hour*3600L)+((long)msg.date.received.minute*60L)+msg.date.received.second)+1;
  }

  hdr.mflags |= (word)(msg.attr.pvt() ? mfPrivate     : 0);
  hdr.mflags |= (word)(msg.attr.rab() ? mfReceiveable : 0);
  hdr.mflags |= (word)(msg.attr.rcv() ? mfReceived    : 0);
  hdr.mflags |= (word)(msg.attr.rrc() ? mfReceipt     : 0);
  hdr.mflags |= (word)(msg.attr.car() ? mfCarboned    : 0);
  hdr.mflags |= (word)(msg.attr.fwd() ? mfForwarded   : 0);
  hdr.mflags |= (word)(msg.attr.efl() ? mfEchoFlag    : 0);
  hdr.mflags |= (word)(msg.attr.hrp() ? mfHasReplies  : 0);
  hdr.mflags |= (word)(msg.attr.del() ? mfDeleted     : 0);
  hdr.mflags |= (word)(msg.attr.tag() ? mfTagged      : 0);
  hdr.mflags |= (word)(msg.attr.snt() ? mfSent        : 0);
  if(msg.link.first())
    hdr.mflags |= mfHasReplies;

  hdr.reference = (word)msg.link.to();

  hdr.origaddr = msg.origaddr;
  hdr.destaddr = msg.destaddr;

  hdr.msgbytes = (word)msg.wildcat.txtlength;

  strc2p(strcpy(hdr.internalattach, msg.wildcat.internal_attach));
  strc2p(strcpy(hdr.externalattach, msg.wildcat.external_attach));

  hdr.prevunread = (word)msg.wildcat.prev_unread;
  hdr.nextunread = (word)msg.wildcat.next_unread;

  hdr.fidoflags |= (word)(msg.attr.pvt() ? FIDO_PVT       : 0);
  hdr.fidoflags |= (word)(msg.attr.cra() ? FIDO_CRASH     : 0);
  hdr.fidoflags |= (word)(msg.attr.rcv() ? FIDO_RECEIVED  : 0);
  hdr.fidoflags |= (word)(msg.attr.snt() ? FIDO_SENT      : 0);
  hdr.fidoflags |= (word)(msg.attr.att() ? FIDO_ATTACH    : 0);
  hdr.fidoflags |= (word)(msg.attr.trs() ? FIDO_TRANSIT   : 0);
  hdr.fidoflags |= (word)(msg.attr.orp() ? FIDO_ORPHAN    : 0);
  hdr.fidoflags |= (word)(msg.attr.k_s() ? FIDO_KILLSENT  : 0);
  hdr.fidoflags |= (word)(msg.attr.loc() ? FIDO_LOCAL     : 0);
  hdr.fidoflags |= (word)(msg.attr.hld() ? FIDO_HOLD      : 0);
  hdr.fidoflags |= (word)(msg.attr.rsv() ? FIDO_RESERVED  : 0);
  hdr.fidoflags |= (word)(msg.attr.frq() ? FIDO_FREQ      : 0);
  hdr.fidoflags |= (word)(msg.attr.rrq() ? FIDO_RETRECREQ : 0);
  hdr.fidoflags |= (word)(msg.attr.rrc() ? FIDO_RETREC    : 0);
  hdr.fidoflags |= (word)(msg.attr.arq() ? FIDO_AUDITREQ  : 0);
  hdr.fidoflags |= (word)(msg.attr.urq() ? FIDO_UPDREQ    : 0);

  hdr.cost = msg.cost;

  memcpy(hdr.reserved, msg.wildcat.reserved, 20);

  char* text = NULL;
  uint reln = msgnset.absolute_to_relative(msg.msgno)-1;
  long datstart = idx[reln].offset;

  if(mode & GMSG_TXT) {

    char* txt = msg.text.lines_to_raw();
    text = throw_strdup(txt);
    word size = (word)strlen(text);
    hdr.msgbytes = size;
    word n = 0;
    char* ptr = text;
    while(n < size) {
      if(*ptr == CTRL_A)
        *ptr = NUL;
      ptr++;
      n++;
    }

    if((mode & GMSG_NEW) or (size > hdr.msgbytes)) {
      if(size > hdr.msgbytes) {
        ulong magic = MagicHeaderInactive;
        lseekset(fhdat, idx[reln].offset);
        write(fhdat, &magic, 4);
      }
      datstart = filelength(fhdat);
      idx[reln].msgno = hdr.msgno;
      idx[reln].offset = datstart;
      lseekset(fhix, sizeof(WCatBase)+((long)reln*sizeof(WCatIdx)));
      write(fhix, &idx[reln], sizeof(WCatIdx));
    }
    hdr.msgbytes = size;
  }

  lseekset(fhdat, datstart);
  write(fhdat, &hdr, sizeof(WCatHdr));
  if(text) {
    write(fhdat, text, hdr.msgbytes);
    throw_free(text);
  }
  
  unlock();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::save_hdr(int mode, gmsg& msg) {

  GFTRK("gmb_wildcat_area::SaveHdr");

  WCatHdr hdr;
  save_message(mode|GMSG_HDR, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::save_msg(int mode, gmsg& msg) {

  GFTRK("gmb_wildcat_area::SaveMsg");

  WCatHdr hdr;
  save_message(mode|GMSG_HDRTXT, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::del_msg(gmsg& msg) {

  GFTRK("gmb_wildcat_area::DelMsg");

  WCatHdr hdr;
  save_message(GMSG_HDR | GMSG_DELETE, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::new_msgno(gmsg& msg) {

  GFTRK("gmb_wildcat_area::NewMsgno");

  msg.msgno = base.nextmsgno;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_wildcat_area::update_timesread(gmsg&) {

}


//  ------------------------------------------------------------------

