
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbxbbs.h,v 1.1.1.1 1998/11/21 19:38:14 odinn Exp $
//  ------------------------------------------------------------------
//  AdeptXBBS structures.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Only include once!

#ifndef __GMXBBS_H
#define __GMXBBS_H


//  ------------------------------------------------------------------
//  Required

#include <gmbarea.h>
#include <gusrxbbs.h>


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


//  ------------------------------------------------------------------
//  Bitmaps for the header.fflags

#define FFLAGS_MSGPRIVATE  0x0001  // private message,
#define FFLAGS_MSGCRASH    0x0002  // accept for forwarding
#define FFLAGS_MSGREAD     0x0004  // read by addressee
#define FFLAGS_MSGSENT     0x0008  // sent OK (remote)
#define FFLAGS_MSGFILE     0x0010  // file attached to msg
#define FFLAGS_MSGFWD      0x0020  // being forwarded
#define FFLAGS_MSGORPHAN   0x0040  // unknown dest node
#define FFLAGS_MSGKILL     0x0080  // kill after mailing
#define FFLAGS_MSGLOCAL    0x0100  // FidoNet vs. local
#define FFLAGS_MSGXX1      0x0200  //
#define FFLAGS_MSGXX2      0x0400  // STRIPPED by FidoNet<tm>
#define FFLAGS_MSGFRQ      0x0800  // file request
#define FFLAGS_MSGRRQ      0x1000  // receipt requested
#define FFLAGS_MSGCPT      0x2000  // is a return receipt
#define FFLAGS_MSGARQ      0x4000  // audit trail requested
#define FFLAGS_MSGURQ      0x8000  // update request


//  ------------------------------------------------------------------
//  Bitmaps for the header.xflags

#define XFLAGS_MSGDELETED  0x0001  // deleted message,
#define XFLAGS_MSGANON     0x0002  // anonymous message
#define XFLAGS_MSGECHO     0x0004  // unmoved echo message
#define XFLAGS_MSGNET      0x0008  // unmoved net message

#define XFLAGS_MSGHOLD     0x0010  // file attached to msg
#define XFLAGS_MSGHOST     0x0020  // being forwarded
#define XFLAGS_MSGSCANNED  0x0040  // Fidonet scanned
#define XFLAGS_MSGKEEP     0x0080  // don't delete

#define XFLAGS_MSGTREATED  0x0100  // soft cr's & lf's removed
#define XFLAGS_MSGPACKED   0x0200  // message LZSS compressed
#define XFLAGS_MSGGSCAN    0x0400  // GroupMail scanned
#define XFLAGS_MSGRSCAN    0x0800  // rfc0822 scanned

#define XFLAGS_MSGARCHIVED 0x4000  // Message stored
#define XFLAGS_MSGTAGGED   0x8000  // used by offline readers


//  ------------------------------------------------------------------
//  AdeptXBBS Message Header (*.Data)

struct XbbsHdr {
  char     majorversion; // Minor revision number of this message style
  char     minorversion; // Minor revision number of this message style
  ushort   structlen;    // The length of this data structure
  char     from[60];     // Who the message is from
  char     to[60];       // Who the message is to
  char     subj[70];     // The subject of the message
  char     date[35];     // Date the message was written
  char     indate[4];    // Import date
  ulong    msgnum;       // Current message number
  ulong    timesread;    // Number of times the message has been read
  Time     timerecv;     // Time user received this message
  ulong    length;       // Length of message stored in .Text Data file
  long     start;        // Pointer to starting byte in .Text Data file
  ulong    extra1;       // Extra space  // Was going to be for reply
  ulong    extra2;       // Extra space  // linking instead came up with
  ulong    extra3;       // Extra space  // a better method
  ftn_addr origaddr;     // Messages origin
  ftn_addr destaddr;     // Messages destination
  ushort   cost;         // Cost to send this message
  ushort   fflags;       // Fidonet related flags
  ushort   xflags;       // XBBS related flags
  ulong    iflags;       // Internet related flags
  ulong    oflags;       // Other network related flags
};


//  ------------------------------------------------------------------
//  AdeptXBBS Message Index (*.Index)

struct XbbsIdx {
  ushort   to;             // Checksum of the to field
  ushort   from;           // Checksum of the from field
  ushort   subj;           // Checksum of the subject field
  ulong   msgidcrc;       // MSGID 32-bit CRC of address field (starting value 0xFFFFFFFF)
  ulong   msgidserialno;  // MSGID Serial number field
  ulong   replycrc;       // REPLY 32-bit CRC of address field (starting value 0xFFFFFFFF)
  ulong   replyserialno;  // REPLY Serial number field
};


//  ------------------------------------------------------------------
//  AdeptXBBS Personal Mail Index

struct XbbsPmi {
  long  areanumber;
  long  msgnumber;
  char  from[60];
  char  subject[70];
  char  date[20];
};


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------

class gmb_adeptxbbs_area : public gmb_area {

protected:

  gmb_adeptxbbs_manager* wide;

  int      fhdata;
  int      fhindex;
  int      fhlmr;
  int      fhtext;
  XbbsIdx* idx;
  uint     idx_size;

  void raw_open();
  void raw_close();
  void refresh();

  void lock_file(int handle, long position, long length);
  void unlock_file(int handle, long position, long length);

  int  test_open(const char* __file, int sharemode=-1);
  void raw_scan(int __keep_index, int __scanpm=false);
  void save_message(int __mode, gmsg& __msg, XbbsHdr& __hdr);
  bool load_message(int __mode, gmsg& __msg, XbbsHdr& __hdr);

  void update_personal_mail(gmsg& __msg, XbbsHdr& __hdr, int __status);

public:

  gmb_adeptxbbs_area(gareacfg* c, gmb_adeptxbbs_manager* m) : gmb_area(c) { wide = m; }
  virtual ~gmb_adeptxbbs_area() { }

  void init(gareacfg* c, gmb_adeptxbbs_manager* m)  { gmb_area::init(c); wide = m; }

  void activate();
  void deactivate();

  void open();
  void close();

  void save_lastread();

  void suspend();
  void resume();

  void lock();
  void unlock();

  void scan();
  void scan_area();
  void scan_area_pm();

  bool load_hdr(gmsg& msg);
  bool load_msg(gmsg& msg);

  void save_hdr(int mode, gmsg& msg);
  void save_msg(int mode, gmsg& msg);

  void del_msg(gmsg& msg);

  void new_msgno(gmsg& msg);
  bool user_lookup(gstring& lookfor);
  bool renumber();

  void update_timesread(gmsg& msg);

  void hexdump_msg(gmsg& msg, gstring& dump);

};


//  ------------------------------------------------------------------

class gmb_adeptxbbs_manager : public gmb_area_manager {

protected:

  int      fhpmi;
  int      userno;
  XbbsUser user;
  XbbsPmi* pmi;
  gstring  path;
  int      open_refs;

  void open();
  void close();

  friend class gmb_adeptxbbs_area;

public:

  gmb_adeptxbbs_manager();
  ~gmb_adeptxbbs_manager();

  void initialize();
  void reset();

  void wide_open()   { open(); }
  void wide_close()  { close(); }

  gmb_area* new_area(gareacfg* c);
  void init_area(gmb_area* a, gareacfg* c);

  void set_path(const gstring& p)  { path = p; }
  void set_userno(int n)           { userno = n; }

};


//  ------------------------------------------------------------------

#if not defined(__OS2__)
inline void gmb_adeptxbbs_area::open() { }
inline void gmb_adeptxbbs_area::save_lastread() { }
inline void gmb_adeptxbbs_area::close() { }
inline void gmb_adeptxbbs_area::suspend() { }
inline void gmb_adeptxbbs_area::resume() { }
inline void gmb_adeptxbbs_area::scan() { }
inline void gmb_adeptxbbs_area::scan_area() { }
inline void gmb_adeptxbbs_area::scan_area_pm() { }
inline bool gmb_adeptxbbs_area::load_hdr(gmsg&) { return false; }
inline bool gmb_adeptxbbs_area::load_msg(gmsg&) { return false; }
inline void gmb_adeptxbbs_area::lock() { }
inline void gmb_adeptxbbs_area::unlock() { }
inline void gmb_adeptxbbs_area::save_hdr(int, gmsg&) { }
inline void gmb_adeptxbbs_area::save_msg(int, gmsg&) { }
inline void gmb_adeptxbbs_area::del_msg(gmsg&) { }
inline void gmb_adeptxbbs_area::new_msgno(gmsg&) { }
inline bool gmb_adeptxbbs_area::user_lookup(gstring&) { return false; }
inline bool gmb_adeptxbbs_area::renumber() { return false; }
inline void gmb_adeptxbbs_area::update_timesread(gmsg&) { }
inline void gmb_adeptxbbs_area::hexdump_msg(gmsg&, gstring&) { }
inline gmb_adeptxbbs_manager::gmb_adeptxbbs_manager() { }
inline gmb_adeptxbbs_manager::~gmb_adeptxbbs_manager() { }
inline void gmb_adeptxbbs_manager::open() { }
inline void gmb_adeptxbbs_manager::close() { }
inline void gmb_adeptxbbs_manager::initialize() { }
inline void gmb_adeptxbbs_manager::reset() { }
inline gmb_area* gmb_adeptxbbs_manager::new_area(gareacfg*) { return NULL; }
inline void gmb_adeptxbbs_manager::init_area(gmb_area*, gareacfg*) { }
#endif


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

