
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbxbbs1.cpp,v 1.1.1.1 1998/11/21 19:38:14 odinn Exp $
//  ------------------------------------------------------------------
//  AdeptXBBS messagebase engine.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>

#include <gmbxbbs.h>


//  ------------------------------------------------------------------

#if defined(__OS2__)


//  ------------------------------------------------------------------

gmb_adeptxbbs_manager::gmb_adeptxbbs_manager() {

  userno = 0;
  user.fh = -1;
  fhpmi = -1;
  pmi = NULL;
  open_refs = 0;
}


//  ------------------------------------------------------------------

gmb_adeptxbbs_manager::~gmb_adeptxbbs_manager() {

  reset();
}


//  ------------------------------------------------------------------

gmb_area* gmb_adeptxbbs_manager::new_area(gareacfg* c) {

  gmb_adeptxbbs_area* area = new gmb_adeptxbbs_area(c, this);
  throw_new(area);
  return area;
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_manager::init_area(gmb_area* a, gareacfg* c) {

  gmb_adeptxbbs_area* area = (gmb_adeptxbbs_area*)a;
  area->init(c, this);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_manager::initialize() {

  const char* _username = gmb_wide_username[0];
  if(userno == -1) {
    user.fh = ::sopen(AddPath(path.c_str(), "Users"), O_RDONLY|O_BINARY, gmb_wide_sharemode);
    if(user.fh != -1) {
      user.find(_username);
      if(not user.found) {
        userno = 0;
        //gmb_wide_log->printf("* User \"%s\" not found in %sUsers.", _username, path);
        //user.add(_username);
        //gmb_wide_log->printf("* Now added with user number %u.", user.index);
      }
      ::close(user.fh);
      user.fh = -1;
    }
    userno = user.index;
  }
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_manager::reset() {

  for(int n=0; n<active.size(); n++)
    delete active[n];
  active.reset();
}


//  ------------------------------------------------------------------

int gmb_adeptxbbs_area::test_open(const char* __file, int sharemode) {

  GFTRK("gmb_adeptxbbs_area::TestOpen");

  int _fh;
  long _tries = 0;
  if(sharemode == -1)
    sharemode = gmb_wide_sharemode;

  do {

    _fh = ::sopen(__file, O_RDWR|O_BINARY|O_CREAT, sharemode, S_STDRW);
    if(_fh == -1) {

      // Tell the world
      if(PopupLocked(++_tries, false, __file) == false) {

        // User requested to exit
        gmb_wide_log->ErrOpen();
        raw_close();
        wide->close();
        gmb_wide_log->printf("! An AdeptXBBS msgbase file could not be opened.");
        gmb_wide_log->printf(": %s.", __file);
        gmb_wide_log->ErrOSInfo();
        OpenErrorExit();
      }
    }
  } while(_fh == -1);

  // Remove the popup window
  if(_tries)
    PopupLocked(0, 0, NULL);

  GFTRK(NULL);

  return _fh;
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::raw_open() {

  GFTRK("gmb_adeptxbbs_area::RawOpen");

  fhdata  = test_open(AddPath(path().c_str(), ".Data"));
  fhindex = test_open(AddPath(path().c_str(), ".Index"));
  fhtext  = test_open(AddPath(path().c_str(), ".Text"));

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::raw_close() {

  GFTRK("gmb_adeptxbbs_area::RawClose");

  if(fhdata != -1)   ::close(fhdata);   fhdata = -1;
  if(fhindex != -1)  ::close(fhindex);  fhindex = -1;
  if(fhtext != -1)   ::close(fhtext);   fhtext = -1;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::activate() {

  refs++;
  if(refs == 1) {
    fhdata = fhindex = fhlmr = fhtext = -1;
    idx_size = 0;
    idx = NULL;
  }
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::deactivate() {

  if(refs) {
    if(refs == 1) {
      if(open_refs) {
        open_refs = 1;
        close();
      }
    }
    refs--;
  }
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::open() {

  GFTRK("gmb_adeptxbbs_area::Open");

  open_refs++;
  if(open_refs == 1) {
    wide->open();
    raw_open();
    refresh();
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::close() {

  GFTRK("gmb_adeptxbbs_area::Close");

  if(open_refs) {
    if(open_refs == 1) {
      if(lock_refs) {
        lock_refs = 1;
        unlock();
      }
      raw_close();
      wide->close();
      msgnset.reset();
      pmrkset.reset();
      throw_release(idx);
      idx_size = 0;
    }
    open_refs--;
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::save_lastread() {

  GFTRK("gmb_adeptxbbs_area::SaveLastread");

  int fh = ::sopen(AddPath(path().c_str(), ".LMR"), O_RDWR|O_CREAT|O_BINARY, gmb_wide_sharemode, S_STDRW);
  if(fh != -1) {
    ulong _lastread = msgnset.relative_to_absolute(lastread);
    lseekset(fh, wide->userno+1, sizeof(ulong));
    write(fh, &_lastread, sizeof(ulong));
    ::close(fh);
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::suspend() {

  GFTRK("gmb_adeptxbbs_area::Suspend");

  save_lastread();
  wide->close();
  raw_close();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::resume() {

  GFTRK("gmb_adeptxbbs_area::Resume");

  raw_open();
  wide->open();

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

