
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbxbbs2.cpp,v 1.1.1.1 1998/11/21 19:38:14 odinn Exp $
//  ------------------------------------------------------------------
//  AdeptXBBS messagebase engine.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gmemdbg.h>
#include <gdbgtrk.h>

#include <gmbxbbs.h>


//  ------------------------------------------------------------------

#if defined(__OS2__)


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::refresh() {

  GFTRK("gmb_adeptxbbs_area::refresh");

  long indexnum = filelength(fhindex) / (long)sizeof(XbbsIdx);

  // Are there any msgs?
  if(indexnum) {

    // Check for physical limit in DOS real mode
    #if defined(__MSDOS__) and not defined(__386__)
    if(indexnum > (65535L/sizeof(XbbsIdx))) {
      gmb_wide_log->ErrIndex();
      gmb_wide_log->printf("! An AdeptXBBS msgbase index was too large to handle.");
      gmb_wide_log->printf(": %s (%s) with %lu index records.", path(), echoid(), indexnum);
      gmb_wide_log->printf("+ Info: GoldED-DOS only supports up to %lu msgs in an AdeptXBBS area.", 65535L/sizeof(XbbsIdx));
      gmb_wide_log->printf("+ Advice: Purge and pack the area or use GoldED/386 or GoldED/2.");
      IndexErrorExit();
    }
    #endif

    // Read the index file
    idx = (XbbsIdx*)throw_realloc(idx, (uint)(indexnum*sizeof(XbbsIdx)));
    lseekset(fhindex, 0);
    read(fhindex, idx, (uint)(indexnum*sizeof(XbbsIdx)));
  }

  idx_size = (uint)indexnum;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_manager::open() {

  GFTRK("gmb_adeptxbbs_manager::open");

  open_refs++;
  if(open_refs == 1) {
    user.fh = ::sopen(AddPath(path.c_str(), "Users"), O_RDONLY|O_BINARY, gmb_wide_sharemode);
  }

  // TODO: Open the personal mail index

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_manager::close() {

  GFTRK("gmb_adeptxbbs_manager::close");

  if(open_refs) {
    if(open_refs == 1) {
      ::close(user.fh);
      user.fh = -1;
    }
    open_refs--;
  }

  // TODO: Close the personal mail index

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::raw_scan(int __keep_index, int __scanpm) {

  GFTRK("gmb_adeptxbbs_area::raw_scan");

  // Load the lastread
  ulong _lastread = 0;
  int _fh = ::sopen(AddPath(path().c_str(), ".LMR"), O_RDONLY|O_BINARY, gmb_wide_sharemode);
  if(_fh != -1) {
    lseekset(_fh, wide->userno+1, sizeof(ulong));
    read(_fh, &_lastread, sizeof(ulong));
    ::close(_fh);
  }

  // Open AdeptXBBS files for scanning unless they are already open
  if(open_refs == 0) {

    // Open index file
    fhindex = ::sopen(AddPath(path().c_str(), ".Index"), O_RDONLY|O_BINARY, gmb_wide_sharemode);
    if(fhindex != -1) {

      // Allocate index buffer and read from file
      refresh();

      // Close index file
      ::close(fhindex);
      fhindex = -1;
    } 
  }

  register uint _active = 0;
  register uint _lastread_reln = 0;
  register ulong _lastreadfound = 0;

  if(idx_size) {

    // (Re)allocate message index
    if(__keep_index)
      msgnset.resize(idx_size);

    // Variables for the loop
    register ulong _msgno;
    register ulong* _msgndxptr = msgnset.tag;
    register dword  _totalmsgs = idx_size;
    register ulong _firstmsgno = _totalmsgs ? 1 : 0;
    register ulong _lastmsgno = 0;

    // Fill message index
    while(_active < _totalmsgs) {

      _active++;
      _msgno = _active;
      if(not _firstmsgno)
        _firstmsgno = _msgno;
      if(__keep_index)
        *_msgndxptr++ = _msgno;

      // Get the lastread
      if((_msgno >= _lastread) and (_lastread_reln == 0)) {
        _lastreadfound = _msgno;
        _lastread_reln = (uint)(_active - (_msgno != _lastread ? 1 : 0));
      }

      // Store last message number
      _lastmsgno = _msgno;
    }

    // If the exact lastread was not found
    if(_active and (_lastreadfound != _lastread)) {

      // Higher than highest or lower than lowest?
      if(_lastread > _lastmsgno)
        _lastread_reln = _active;
      else if(_lastread < _firstmsgno)
        _lastread_reln = 0;
    }
  }

  // Update area data
  scan_count = _active;
  msgnset.resize(_active);
  lastread = _lastread_reln;
  lastreadentry = _lastreadfound;

  // Scan for personal mail
  pmrkset.reset();
  if(__scanpm) {
    // Not implemented yet
  }
  pmscan_count = pmrkset.size();

  // Free index buffer if just counting
  if(not __keep_index or __scanpm)
    throw_release(idx);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::scan() {

  GFTRK("gmb_adeptxbbs_area::scan");

  raw_scan(true);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::scan_area() {

  GFTRK("gmb_adeptxbbs_area::scan_area");

  raw_scan(false);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::scan_area_pm() {

  GFTRK("gmb_adeptxbbs_area::scan_area_pm");

  raw_scan(true, true);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

