
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbxbbs3.cpp,v 1.1.1.1 1998/11/21 19:38:14 odinn Exp $
//  ------------------------------------------------------------------
//  AdeptXBBS messagebase engine.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>

#include <gmbxbbs.h>


//  ------------------------------------------------------------------

#if defined(__OS2__)


//  ------------------------------------------------------------------

bool gmb_adeptxbbs_area::load_message(int mode, gmsg& msg, XbbsHdr& hdr) {

  // Setup some local variables for speed
  int _fhdata = fhdata;
  uint _reln = msgnset.absolute_to_relative(msg.msgno);

  // Load the message header
  memset(&hdr, 0, sizeof(XbbsHdr));
  lseekset(_fhdata, (long)(_reln-1)*(long)sizeof(XbbsHdr));
  read(_fhdata, &hdr, sizeof(XbbsHdr));

  // Convert header

  msg.origname = hdr.from;
  msg.destname = hdr.to;
  msg.subject  = hdr.subj;

  msg.origaddr.zone  = hdr.origaddr.zone;
  msg.origaddr.net   = hdr.origaddr.net;
  msg.origaddr.node  = hdr.origaddr.node;
  msg.origaddr.point = hdr.origaddr.point;

  if(isnet()) {
    msg.destaddr.zone  = hdr.destaddr.zone;
    msg.destaddr.net   = hdr.destaddr.net;
    msg.destaddr.node  = hdr.destaddr.node;
    msg.destaddr.point = hdr.destaddr.point;
  }
  
  msg.date.written.convert_fts1(hdr.date);
  msg.date.received = hdr.timerecv;
  if(hdr.indate[2]) {
    msg.date.arrived.day   = hdr.indate[2];
    msg.date.arrived.month = hdr.indate[1];
    msg.date.arrived.year  = hdr.indate[0] + 1989;
  }

  msg.cost      = hdr.cost;
  msg.timesread = hdr.timesread;

  msg.adeptxbbs.start  = hdr.start;
  msg.adeptxbbs.length = hdr.length;

  // Convert fflag attributes
  msg.attr.pvt(hdr.fflags & FFLAGS_MSGPRIVATE);
  msg.attr.cra(hdr.fflags & FFLAGS_MSGCRASH);
  msg.attr.rcv(hdr.fflags & FFLAGS_MSGREAD);
  msg.attr.snt(hdr.fflags & FFLAGS_MSGSENT);
  msg.attr.att(hdr.fflags & FFLAGS_MSGFILE);
  msg.attr.trs(hdr.fflags & FFLAGS_MSGFWD);
  msg.attr.orp(hdr.fflags & FFLAGS_MSGORPHAN);
  msg.attr.k_s(hdr.fflags & FFLAGS_MSGKILL);
  msg.attr.loc(hdr.fflags & FFLAGS_MSGLOCAL);
  msg.attr.frq(hdr.fflags & FFLAGS_MSGFRQ);
  msg.attr.rrq(hdr.fflags & FFLAGS_MSGRRQ);
  msg.attr.rrc(hdr.fflags & FFLAGS_MSGCPT);
  msg.attr.arq(hdr.fflags & FFLAGS_MSGARQ);
  msg.attr.urq(hdr.fflags & FFLAGS_MSGURQ);

  // Convert xflag attributes
  msg.attr.del(hdr.xflags & XFLAGS_MSGDELETED);
  msg.attr.ano(hdr.xflags & XFLAGS_MSGANON);
  msg.attr.ume(hdr.xflags & XFLAGS_MSGECHO);
  msg.attr.umn(hdr.xflags & XFLAGS_MSGNET);
  msg.attr.fsc(hdr.xflags & XFLAGS_MSGSCANNED);
  msg.attr.lok(hdr.xflags & XFLAGS_MSGKEEP);
  msg.attr.trt(hdr.xflags & XFLAGS_MSGTREATED);
  msg.attr.lzs(hdr.xflags & XFLAGS_MSGPACKED);
  msg.attr.gsc(hdr.xflags & XFLAGS_MSGGSCAN);
  msg.attr.rsc(hdr.xflags & XFLAGS_MSGRSCAN);
  msg.attr.arc(hdr.xflags & XFLAGS_MSGARCHIVED);
  msg.attr.tag(hdr.xflags & XFLAGS_MSGTAGGED);

  // Set the unsent attribute
  #if 0
  if(isnet())
    msg.attr.uns((msg.attr.loc() and not msg.attr.snt()) or (hdr.xflags & XFLAGS_MSGNET));
  else
    msg.attr.uns(hdr.xflags & XFLAGS_MSGECHO);
  #endif

  if(isnet() or isecho())
    msg.attr.uns(not (hdr.xflags & XFLAGS_MSGSCANNED));

  msg.adeptxbbs.iflags = hdr.iflags;
  msg.adeptxbbs.oflags = hdr.oflags;

  // If message text is requested
  if(mode & GMSG_TXT) {

    ulong textlen = hdr.length;
    if(textlen > gmb_wide_msgsize)
      textlen = gmb_wide_msgsize;

    // Allocate space for the message text
    msg.text.reserve((uint)textlen+1);

    // Read the message text
    lseekset(fhtext, hdr.start);
    read(fhtext, msg.text.pointer(), (uint)textlen);

    msg.text.raw_to_lines((uint)textlen);
  }

  GFTRK(NULL);

  // Success
  return true;
}


//  ------------------------------------------------------------------

bool gmb_adeptxbbs_area::load_hdr(gmsg& msg) {

  GFTRK("gmb_adeptxbbs_area::LoadHdr");

  XbbsHdr _hdr;
  return load_message(GMSG_HDR, msg, _hdr);
}


//  ------------------------------------------------------------------

bool gmb_adeptxbbs_area::load_msg(gmsg& msg) {

  GFTRK("gmb_adeptxbbs_area::LoadMsg");

  XbbsHdr _hdr;
  return load_message(GMSG_HDRTXT, msg, _hdr);
}


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

