
//  ------------------------------------------------------------------
//  The Goldware Library. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: gmbxbbs4.cpp,v 1.1.1.1 1998/11/21 19:38:14 odinn Exp $
//  ------------------------------------------------------------------
//  AdeptXBBS messagebase engine.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------

#include <gdbgerr.h>
#include <gmemdbg.h>
#include <gdbgtrk.h>
#include <gstrall.h>
#include <gcrcall.h>
#include <gutlmisc.h>

#include <gmbxbbs.h>


//  ------------------------------------------------------------------

#if defined(__OS2__)


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::lock() {

  lock_refs++;
  if(lock_refs == 1) {
    // Not applicable in AdeptXBBS
  }
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::unlock() {

  if(lock_refs) {
    if(lock_refs == 1) {
      // Not applicable in AdeptXBBS
    }
    lock_refs--;
  }
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::lock_file(int handle, long position, long length) {

  GFTRK("gmb_adeptxbbs_area::LockFile");

  long tries = 0;
  while(::lock(handle, position, length) == -1) {
    if(PopupLocked(++tries, true, path().c_str()) == false) {
      gmb_wide_log->ErrLock();
      raw_close();
      Path file;
      strcpy(file, path().c_str());
      if(handle == fhdata)
        strcat(file, ".Data");
      else if(handle == fhtext)
        strcat(file, ".Text");
      else if(handle == fhindex)
        strcat(file, ".Index");
      else if(handle == wide->fhpmi)
        sprintf(file, "%sPersonal_Mail\\%s", wide->path.c_str(), gmb_wide_username[0]);
      gmb_wide_log->printf("! An AdeptXBBS msgbase file could not be locked.");
      gmb_wide_log->printf(": %s.", file);
      gmb_wide_log->ErrOSInfo();
      LockErrorExit();
    }
  }
  if(tries)
    PopupLocked(0, 0, NULL);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::unlock_file(int handle, long position, long length) {

  GFTRK("gmb_adeptxbbs_area::UnlockFile");

  ::unlock(handle, position, length);

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

ushort gmb_adeptxbbs_checksum(char* str) {

  ushort checksum = 0;

  while(*str) {
    checksum ^= (ushort)toupper(*str++);
    if(checksum & 1)
      checksum = (ushort)((checksum >> 1) ^ 0xA000);
    else
      checksum >>= 1;
  }

  return checksum;
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::save_message(int mode, gmsg& msg, XbbsHdr& hdr) {

  // Reset header
  memset(&hdr, 0, sizeof(XbbsHdr));

  if(mode & GMSG_NEW) {
    idx_size++;
    msg.msgno = idx_size;
    msgnset.append(msg.msgno);
    scan_count++;
    idx = (XbbsIdx*)throw_realloc(idx, idx_size*sizeof(XbbsIdx));
  }

  strxcpy(hdr.from, msg.origname.c_str(), sizeof(hdr.from));
  strxcpy(hdr.to,   msg.destname.c_str(), sizeof(hdr.to));
  strxcpy(hdr.subj, msg.subject.c_str(), sizeof(hdr.subj));

  strcpy(hdr.date, msg.date.written.fts1());
  if(msg.date.arrived.unixtime()) {
    hdr.indate[0] = (byte)(msg.date.arrived.year - 1989);
    hdr.indate[1] = (byte)(msg.date.arrived.month);
    hdr.indate[2] = (byte)(msg.date.arrived.day);
    hdr.indate[3] = 0;
  }
  hdr.timerecv = msg.date.received.unixtime();

  hdr.msgnum = msg.msgno;
  hdr.timesread = msg.timesread;
  hdr.origaddr = msg.origaddr;
  hdr.destaddr = msg.destaddr;
  hdr.cost = (ushort)msg.cost;

  // Transfer attributes
  hdr.fflags |= (ushort)(msg.attr.pvt() ? FFLAGS_MSGPRIVATE  : 0);
  hdr.fflags |= (ushort)(msg.attr.cra() ? FFLAGS_MSGCRASH    : 0);
  hdr.fflags |= (ushort)(msg.attr.rcv() ? FFLAGS_MSGREAD     : 0);
  hdr.fflags |= (ushort)(msg.attr.snt() ? FFLAGS_MSGSENT     : 0);
  hdr.fflags |= (ushort)(msg.attr.att() ? FFLAGS_MSGFILE     : 0);
  hdr.fflags |= (ushort)(msg.attr.trs() ? FFLAGS_MSGFWD      : 0);
  hdr.fflags |= (ushort)(msg.attr.orp() ? FFLAGS_MSGORPHAN   : 0);
  hdr.fflags |= (ushort)(msg.attr.k_s() ? FFLAGS_MSGKILL     : 0);
  hdr.fflags |= (ushort)(msg.attr.loc() ? FFLAGS_MSGLOCAL    : 0);
  hdr.fflags |= (ushort)(msg.attr.rsv() ? FFLAGS_MSGXX2      : 0);
  hdr.fflags |= (ushort)(msg.attr.frq() ? FFLAGS_MSGFRQ      : 0);
  hdr.fflags |= (ushort)(msg.attr.rrq() ? FFLAGS_MSGRRQ      : 0);
  hdr.fflags |= (ushort)(msg.attr.rrc() ? FFLAGS_MSGCPT      : 0);
  hdr.fflags |= (ushort)(msg.attr.arq() ? FFLAGS_MSGARQ      : 0);
  hdr.fflags |= (ushort)(msg.attr.urq() ? FFLAGS_MSGURQ      : 0);

  hdr.xflags |= (ushort)(msg.attr.del() ? XFLAGS_MSGDELETED  : 0);
  hdr.xflags |= (ushort)(msg.attr.ano() ? XFLAGS_MSGANON     : 0);
  hdr.xflags |= (ushort)(msg.attr.fsc() ? XFLAGS_MSGSCANNED  : 0);
  hdr.xflags |= (ushort)(msg.attr.lok() ? XFLAGS_MSGKEEP     : 0);
  hdr.xflags |= (ushort)(msg.attr.trt() ? XFLAGS_MSGTREATED  : 0);
  hdr.xflags |= (ushort)(msg.attr.lzs() ? XFLAGS_MSGPACKED   : 0);
  hdr.xflags |= (ushort)(msg.attr.gsc() ? XFLAGS_MSGGSCAN    : 0);
  hdr.xflags |= (ushort)(msg.attr.rsc() ? XFLAGS_MSGRSCAN    : 0);
  hdr.xflags |= (ushort)(msg.attr.arc() ? XFLAGS_MSGARCHIVED : 0);
  hdr.xflags |= (ushort)(msg.attr.tag() ? XFLAGS_MSGTAGGED   : 0);

  if(not (mode & GMSG_UPDATE)) {
    if(msg.attr.uns()) {
      hdr.xflags &= ~(XFLAGS_MSGSCANNED|XFLAGS_MSGGSCAN|XFLAGS_MSGRSCAN);
      if(isnet())
        hdr.xflags |= XFLAGS_MSGNET;
      else if(isecho())
        hdr.xflags |= XFLAGS_MSGECHO;
    }
    else {
      if(isnet()) {
        hdr.fflags |= FFLAGS_MSGSENT;
        hdr.xflags |= XFLAGS_MSGNET | XFLAGS_MSGSCANNED;
      }
      else if(isecho()) {
        hdr.fflags |= FFLAGS_MSGSENT;
        hdr.xflags |= XFLAGS_MSGECHO | XFLAGS_MSGSCANNED;
      }
    }
  }

  hdr.iflags = msg.adeptxbbs.iflags;
  hdr.oflags = msg.adeptxbbs.oflags;

  hdr.start  = msg.adeptxbbs.start;
  hdr.length = msg.adeptxbbs.length;

  if(mode & GMSG_TXT) {

    char* txt = msg.text.lines_to_raw();

    // Write the message text
    uint size = strlen(txt) + 1;
    if((mode & GMSG_NEW) or (size > hdr.length))
      hdr.start = filelength(fhtext);
    lseekset(fhtext, hdr.start);
    lock_file(fhtext, hdr.start, 640L*1024L);
    write(fhtext, txt, size);
    unlock_file(fhtext, hdr.start, 640L*1024L);
    hdr.length = size;
  }

  // Write header record
  long position = (msg.msgno-1L)*(long)sizeof(XbbsHdr);
  lseekset(fhdata, position);
  lock_file(fhdata, position, sizeof(XbbsHdr));
  write(fhdata, &hdr, sizeof(XbbsHdr));
  unlock_file(fhdata, position, sizeof(XbbsHdr));

  // Build and write index record unless we are just updating the header
  if(mode & GMSG_TXT) {
    XbbsIdx idxr;
    char buf[1024];
    idxr.to   = gmb_adeptxbbs_checksum(hdr.to);
    idxr.from = gmb_adeptxbbs_checksum(hdr.from);
    idxr.subj = gmb_adeptxbbs_checksum(hdr.subj);
    idxr.msgidcrc = idxr.msgidserialno = 0;
    idxr.replycrc = idxr.replyserialno = 0;
    if(msg.kludge.msgid.length()) {
      strcpy(buf, msg.kludge.msgid.c_str());
      char* ptr = strrchr(buf, ' ');
      if(ptr) {
        *ptr++ = NUL;
        idxr.msgidcrc = strCrc32(buf, true, CRC32_MASK_CCITT);
        idxr.msgidserialno = atoulx(ptr);
      }
    }
    if(msg.kludge.reply.length()) {
      strcpy(buf, msg.kludge.reply.c_str());
      char* ptr = strrchr(buf, ' ');
      if(ptr) {
        *ptr++ = NUL;
        idxr.replycrc = strCrc32(buf, true, CRC32_MASK_CCITT);
        idxr.replyserialno = atoulx(ptr);
      }
    }
    memcpy(idx+msg.msgno-1, &idxr, sizeof(XbbsIdx));
    position = (msg.msgno-1L)*(long)sizeof(XbbsIdx);
    lseekset(fhindex, position);
    lock_file(fhindex, position, sizeof(XbbsIdx));
    write(fhindex, &idxr, sizeof(XbbsIdx));
    unlock_file(fhindex, position, sizeof(XbbsIdx));
  }

  if(not (mode & GMSG_DELETE)) {
    // Add/update personal mail record
    bool addpm = true;
    // Remove it if the msg is being received
    if((mode & GMSG_UPDATE) and msg.attr.rcv())
      addpm = false;
    // Don't touch it if the msg was already received
    if(not (addpm and msg.attr.rcv())) {
      // Only do it if the person is on this BBS
      if(wide->user.find(msg.destname.c_str()))
        update_personal_mail(msg, hdr, addpm);
    }
  }

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::save_hdr(int mode, gmsg& msg) {

  GFTRK("gmb_adeptxbbs_area::SaveHdr");

  XbbsHdr hdr;
  save_message(mode|GMSG_HDR, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::save_msg(int mode, gmsg& msg) {

  GFTRK("gmb_adeptxbbs_area::SaveMsg");

  XbbsHdr hdr;
  save_message(mode|GMSG_HDRTXT, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::del_msg(gmsg& msg) {

  GFTRK("gmb_adeptxbbs_area::DelMsg");

  XbbsHdr hdr;
  save_message(GMSG_HDR | GMSG_DELETE, msg, hdr);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::new_msgno(gmsg& msg) {

  GFTRK("gmb_adeptxbbs_area::NewMsgno");

  msg.msgno = idx_size + 1;

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::update_personal_mail(gmsg& msg, XbbsHdr& hdr, int addpm) {

  XbbsPmi pmi;
  pmi.areanumber = board();
  pmi.msgnumber = msg.msgno;
  strxcpy(pmi.from, msg.origname.c_str(), sizeof(pmi.from));
  strxcpy(pmi.subject, msg.subject.c_str(), sizeof(pmi.subject));
  strxcpy(pmi.date, hdr.date, sizeof(pmi.date));

  int pmino = 0;
  XbbsPmi* pmiptr = NULL;

  gfile fp;
  Path pmifile;
  sprintf(pmifile, "%sPersonal_Mail\\%s", wide->path.c_str(), hdr.to);
  int fh = test_open(pmifile, SH_DENYWR);

  int pmirecs = (int)(filelength(fh) / sizeof(XbbsPmi));
  XbbsPmi* pmilist = (XbbsPmi*)throw_calloc((pmirecs+1), sizeof(XbbsPmi));
  ::read(fh, pmilist, sizeof(XbbsPmi)*pmirecs);

  if(pmilist) {
    pmiptr = pmilist;
    for(pmino=0; pmino<pmirecs; pmino++, pmiptr++) {
      if(pmiptr->areanumber == pmi.areanumber)
        if(pmiptr->msgnumber == pmi.msgnumber)
          break;
    }
    if(pmino == pmirecs)
      pmiptr = NULL;
  }

  if(addpm) {
    if(not pmiptr) {
      pmirecs++;
      pmilist = (XbbsPmi*)throw_realloc(pmilist, pmirecs*sizeof(XbbsPmi));
      pmiptr = pmilist + pmirecs - 1;
    }
    memcpy(pmiptr, &pmi, sizeof(XbbsPmi));
  }
  else {
    if(pmiptr) {
      memmove(pmiptr, pmiptr+1, (pmirecs-pmino-1)*sizeof(XbbsPmi));
      pmirecs--;
    }
  }

  if(pmirecs) {
    lseekset(fh, 0);
    ::write(fh, pmilist, sizeof(XbbsPmi)*pmirecs);
    if(pmiptr and not addpm)
      chsize(fh, sizeof(XbbsPmi)*pmirecs);
    ::close(fh);
  }
  else {
    ::close(fh);
    remove(pmifile);
  }

  throw_free(pmilist);
}


//  ------------------------------------------------------------------

void gmb_adeptxbbs_area::update_timesread(gmsg& msg) {

  GFTRK("gmb_adeptxbbs_area::update_timesread");

  XbbsHdr hdr;

  long position = (msgnset.absolute_to_relative(msg.msgno)-1)*(long)sizeof(XbbsHdr);

  ::lseekset(fhdata, position);
  ::read(fhdata, &hdr, sizeof(XbbsHdr));

  hdr.timesread = (word)msg.timesread;

  ::lseekset(fhdata, position);
  int do_lock = (gmb_wide_timesread & TIMESREAD_LOCK);
  if(do_lock)
    lock_file(fhdata, position, sizeof(XbbsHdr));
  ::write(fhdata, &hdr, sizeof(XbbsHdr));
  if(do_lock)
    unlock_file(fhdata, position, sizeof(XbbsHdr));

  GFTRK(NULL);
}


//  ------------------------------------------------------------------

#endif

//  ------------------------------------------------------------------

