
//  ------------------------------------------------------------------
//  The Goldware Utilities. Copyright (C) Odinn Sorensen.
//  ------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//  ------------------------------------------------------------------
//  $Id: goldnode.cpp,v 1.9 1999/02/07 11:42:29 odinn Exp $
//  ------------------------------------------------------------------
//  GoldNODE - A nodelist compiler for GoldED.
//  ------------------------------------------------------------------


//  ------------------------------------------------------------------
//  Standard library headers

#include <new.h>
#if defined(__BORLANDC__) || defined(__WATCOMC__)
#include <conio.h>
#endif
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>


//  ------------------------------------------------------------------
//  Goldware Library headers

#include <gdefs.h>
#include <gcrcall.h>
#include <gmemall.h>
#include <gfilutil.h>
#include <gftnall.h>
#include <gftnnlge.h>
#include <gstrall.h>
#include <gshare.h>
#include <gtimall.h>
#include <gutlmisc.h>
#include <gdirff.h>
#include <gccfgg.h>

#if 0
#define GOLDNODE_STATS 1
#endif

#ifdef GOLDNODE_STATS
#include <math.h>
#endif


//  ------------------------------------------------------------------
//  Config versions

#define __GVER__ "3.0.1"         // Visible version
#define __GREV__ ""              // Visible version


//  ------------------------------------------------------------------

typedef word Nix;


//  ------------------------------------------------------------------

#if defined(GOLD_CANPACK)
#pragma pack(1)
#endif


//  Block types

const byte iDOS  = 1;
const byte iXMS  = 2;
const byte iEMS  = 4;
const byte iDISK = 8;

//  Blocks structure

struct NBlks {
  byte  type;       // iDOS, iXMS, iEMS or iDISK
  long  pos;        // File position, if iDISK; Handle, if iXMS or iEMS
  byte __HUGE *na;  // GNidx's with variable length names
  int  nai[1024];   // Index to .na
  int  nais;        // Nodes in index
  int  len;         // Size of na
};


//  ------------------------------------------------------------------
//  Structure for GOLDNODE.LIX

#define NUM_NODELISTS 45
#define MAX_NODE      50

struct NodeLix {

  // Nodelists
  uint     nodelists;                // Number of nodelists
  Stamp    nodelist[NUM_NODELISTS];  // nodelist files,stamps,update marker
  Addr     nodezone[NUM_NODELISTS];  // nodelist zones

  // Excludenodes
  uint     excludenodes;
  Addr     excludenode[MAX_NODE];

  // Includenodes
  uint     includenodes;
  Addr     includenode[MAX_NODE];

  // Index files
  Path     addrindex;
  Path     nodeindex;
  Path     listindex;            // nodelist address index file
};


#if defined(GOLD_CANPACK)
#pragma pack()
#endif


//  ------------------------------------------------------------------

#define NUM_USERLISTS 10
#define MAXNODENAME   35

const word _POINT = 0x0001;
const word _NODE  = 0x0002;
const word _NET   = 0x0003;
const word _ZONE  = 0x0004;
const word _TEST  = 777;

char*  VR = __GVER__;            // GoldNODE version
char*  RV = __GREV__;            // GoldNODE revision

Path   configfile;               // Path to GOLDED.CFG
Path   nodepath;                 // Path to the nodelist files
Path   temppath;                 // Path where the temp file is placed
Stamp  userlist[NUM_USERLISTS];  // Userlist files,stamps,update marker
Addr   userzone[NUM_USERLISTS];  // Userlist zones
uint   userlists;                // Number of userlists
uint   maxname = MAXNODENAME;    // Limits of size of node names

NodeLix* Nodelist = NULL;

long   runtime = 0;
int    sh_mod = SH_DENYWR;
int    fidouser = NO;
Path   fidouserlst;
int    ignoredups = NO;
Nix    dups = 0;
int    quiet = NO;

NBlks  blk;                      // Temporary block
NBlks* blks[66];                 // List of blocks
uint   blocks;                   // Total number of blocks
uint   blkno = 0;                // Current block

Nix    maxnodes = 65500u;        // Max number of nodes to read
Nix    nodes = 0;                // Total number of nodes in the index
Nix*   aidx = NULL;              // Main index to all the nodes
Nix*   iidx = NULL;              // Main index to all the nodes
Nix*   nidx = NULL;              // Main index to all the nodes

#ifdef GOLDNODE_STATS
bool make_stats = false;
Path statfilename = { "goldnode.stt" };

struct nl_stat {
  int nodename[100];
  int location[100];
  int sysopname[100];
};

nl_stat statistic;
#endif

//  ------------------------------------------------------------------

#if defined(__MSDOS__) and not defined(__386__)
dword   memdos = 0;
GOLDXMS XMS;
int     useems = YES;
int     usexms = YES;
int     emspages = 0;
int     xmskb = 0;
FILE*   diskblk;                  // Disk block
Path    diskfile;                 // Temporary disk block
int     usedisk = NO;

char*   XMS_msg[] = {
  "Function not implemented",                               // 80h
  "Vdisk was detected",                                     // 81h
  "An A20 error occurred",                                  // 82h
  "",                                                       // 83h
  "",                                                       // 84h
  "",                                                       // 85h
  "",                                                       // 86h
  "",                                                       // 87h
  "",                                                       // 88h
  "",                                                       // 89h
  "",                                                       // 8Ah
  "",                                                       // 8Bh
  "",                                                       // 8Ch
  "",                                                       // 8Dh
  "A general driver error",                                 // 8Eh
  "Unrecoverable driver error",                             // 8Fh
  "HMA does not exist or is not managed by XMS provider",   // 90h
  "HMA is already in use",                                  // 91h
  "DX is less than the /HMAMIN= parameter",                 // 92h
  "HMA is not allocated",                                   // 93h
  "A20 line still enabled",                                 // 94h
  "",                                                       // 95h
  "",                                                       // 96h
  "",                                                       // 97h
  "",                                                       // 98h
  "",                                                       // 99h
  "",                                                       // 9Ah
  "",                                                       // 9Bh
  "",                                                       // 9Ch
  "",                                                       // 9Dh
  "",                                                       // 9Eh
  "",                                                       // 9Fh
  "All extended memory is allocated",                       // A0h
  "All available extended memory handles are allocated",    // A1h
  "Invalid handle",                                         // A2h
  "Source handle is invalid",                               // A3h
  "Source offset is invalid",                               // A4h
  "Destination handle is invalid",                          // A5h
  "Destination offset is invalid",                          // A6h
  "Length is invalid",                                      // A7h
  "Move has an invalid overlap",                            // A8h
  "Parity error occurred",                                  // A9h
  "Block is not locked",                                    // AAh
  "Block is locked",                                        // ABh
  "Block lock count overflowed",                            // ACh
  "Lock failed",                                            // ADh
  "",                                                       // AEh
  "",                                                       // AFh
  "Only a smaller UMB is available",                        // B0h
  "No UMB's are available",                                 // B1h
  "UMB segment number is invalid",                          // B2h
};
#endif


//  ------------------------------------------------------------------

#if defined(__UNIX__) or defined(__DJGPP__)
#define fast_printf    if(not quiet) printf
#define fast_putchar   if(not quiet) putchar
#define fast_crlf      "\n"
#else
#define fast_printf    if(not quiet) cprintf
#define fast_putchar   if(not quiet) putch
#define fast_crlf      "\r\n"
#endif


//  ------------------------------------------------------------------

#if defined(__GNUC__) or (defined(__BORLANDC__) and defined(__OS2__))
#define farmalloc(a)   malloc(a)
#define farcalloc(a,b) calloc(a,b)
#define farfree(a)     free(a)
#endif

#define farrelease(p) farfree(p); (p)=NULL
#define release(p) free(p); (p)=NULL


//  ------------------------------------------------------------------
//  Make a ulong

inline ulong MKUL(ulong ui) { return 1+ui-1; }


//  ------------------------------------------------------------------

inline char gtoupper(char c) { return (char)(((c >= 'a') and (c <= 'z')) ? (c-('a'-'A')) : c); }

static int gstricmp(char* a, char* b) {

  int c=0;
  while(!c) {
    register char aa = *a++;
    register char bb = *b++;
    c = CmpV(gtoupper(aa), gtoupper(bb));
    if(c or aa==NUL or bb==NUL)
      return c;
  }
  return 0;
}


//  ------------------------------------------------------------------
//  Display a "twirly"

#define TWIRLY_FACTOR 511
#define ISTWIRLY(n) (((n)&TWIRLY_FACTOR)==0)

static void twirly() {

  static int n=0;

  switch((n++)%4) {
    case 0:   fast_putchar('|');   break;
    case 1:   fast_putchar('/');   break;
    case 2:   fast_putchar('-');   break;
    case 3:   fast_putchar('\\');  break;
  }

  fast_putchar('\b');
}


//  ------------------------------------------------------------------
//  Initialize block

static void init_blk() {

  blk.len = 0;
  blk.type = 0;
  blk.nais = 0;
}


//  ------------------------------------------------------------------
//  Store the block in DOS, XMS, EMS memory or on DISK

static int store_blk() {

  memcpy(blks[blkno], &blk, sizeof(NBlks));
  blks[blkno]->na = NULL;

  // Try to get main memory
  blks[blkno]->na = (byte __HUGE *)farmalloc(blk.len+100);
  if(blks[blkno]->na) {
    blks[blkno]->type = iDOS;
    memcpy(blks[blkno++]->na, blk.na, blk.len);
    init_blk();
    return(1);
  }

  #if defined(__MSDOS__) and not defined(__386__)

  int hdl;

  // Try to get XMS memory
  if(usexms) {
    hdl = XMS.Alloc((blk.len/1024)+2);
    if(hdl) {
      xmskb += (blk.len/1024)+2;
      blks[blkno]->type = iXMS;
      blks[blkno++]->pos = hdl;
      if(XMS.CopyTo((char*)blk.na, 0, blk.len)) {
        fast_printf("Error copying to XMS block." fast_crlf "");
        fast_printf("XMS error %02X: \"%s\"." fast_crlf "",
          XMS.errorcode,
          XMS_msg[XMS.errorcode-0x80]
        );
      }
      init_blk();
      return(1);
    }
    else {
      fast_printf("" fast_crlf "* XMS error %02X: \"%s\"." fast_crlf "",
        XMS.errorcode,
        XMS_msg[XMS.errorcode-0x80]
      );
      if(XMS.errorcode == 0xA1)
        fast_printf("* You need to increase the number of XMS handles in your memory manager." fast_crlf "");
      usexms = NO;
    }
  }

  // Try to get EMS memory
  if(useems and emsexist()) {
    hdl = emsalloc((blk.len/16384)+1);
    if(hdl != -1) {
      emspages += (blk.len/16384)+1;

      union REGS regs;
      struct SREGS sregs;
      static char* emsprogname = "GOLDNODE";

      regs.h.ah = 0x53;     // EMS Get/set handle name
      regs.h.al = 0x01;     // Set handle name
      sregs.ds = FP_SEG(emsprogname);
      regs.x.si = FP_OFF(emsprogname);
      regs.x.dx = hdl;
      int86x(0x67,&regs,&regs,&sregs);

      blks[blkno]->type = iEMS;
      blks[blkno++]->pos = hdl;
      for(int n=0; n<4; n++)
        emsmap(hdl, n, n);
      emswrite((byte*)blk.na, 0, blk.len);
      init_blk();
      return(1);
    }
    else {
      useems = NO;
    }
  }

  // Use DISK file
  if(usedisk) {
    blks[blkno]->type = iDISK;
    blks[blkno++]->pos = ftell(diskblk);
    fwrite(blk.na, blk.len, 1, diskblk);
    init_blk();
    return(1);
  }

  #endif

  return 0;
}


//  ------------------------------------------------------------------

static int match_addr_mask(Addr* mask, Addr* addr) {

  if(mask->zone == GFTN_ALL or mask->zone == addr->zone)
    if(mask->net == GFTN_ALL or mask->net == addr->net)
      if(mask->node == GFTN_ALL or mask->node == addr->node)
        if(mask->point == GFTN_ALL or mask->point == addr->point)
          return(YES);

  return(NO);
}


//  ------------------------------------------------------------------

static int macro_addr(char* str, int ap, Addr* addr) {

  word part = 0xFFFD;

  if(*str == NUL)
    part = 0;
  else if(isdigit(*str))
    part = atow(str);
  else if(*str == '?' or *str == '*')
    part = GFTN_ALL;

  if(ap == _TEST) {
    if(part == 0xFFFD)
      return(NO);
    else
      return(YES);
  }

  switch(ap) {
    case _ZONE:
      addr->zone = part;
      break;
    case _NET:
      addr->net = part;
      break;
    case _NODE:
      addr->node = part;
      break;
    case _POINT:
      addr->point = part;
      break;
    default:
      return NO;
  }
  return YES;
}


//  ------------------------------------------------------------------

static char* fast_parse_addr(char* str, Addr* addr) {

  char* net;
  char* node;
  char* point;
  char* domain;
  char* space;

  space = strchr(str, ' ');
  if(space)
    *space = NUL;

  net = strchr(str, ':');
  node = strchr(str, '/');
  point = strchr(str, '.');
  domain = strchr(str, '@');
  if(domain and point)
    if((dword)point > (dword)domain)
      point = NULL;

  if(space)
    *space = ' ';

  if(net) {
    addr->zone = atow(str);
    addr->net = atow(net+1);
    if(node)
      addr->node = atow(node+1);
  }
  else {
    if(node) {
      if(*str != '/')
        addr->net = atow(str);
      addr->node = atow(node+1);
    }
    else {
      if(point != str)
        addr->node = atow(str);
    }
  }
  if(point)
    addr->point = atow(point+1);

  return domain;
}


//  ------------------------------------------------------------------

static char* parse_address(char* str, Addr* addr, Addr* main) {

  char* domain = NULL;

  str = SkipWht(str);

  if(isdigit(*str) or *str == '.' or macro_addr(str, _TEST, addr)) {

    char* net = strchr(str, ':');
    char* node = strchr(str, '/');
    char* point = strchr(str, '.');
    domain = strchr(str, '@');
    if(domain and point)
      if((dword)point > (dword)domain)
        point = NULL;

    if(net) {
      macro_addr(str, _ZONE, addr);
      macro_addr(net+1, _NET, addr);
      if(node)
        macro_addr(node+1, _NODE, addr);  // zone:net/node
      else
        addr->node = main->node;                // zone:net
    }
    else {
      addr->zone = main->zone;
      if(node) {
        if(*str != '/')
          macro_addr(str, _NET, addr);      // net/node
        else
          addr->net = main->net;                  // /node
        macro_addr(node+1, _NODE, addr);
      }
      else {
        if(point == str)
          addr->node = main->node;                // .point
        else
          macro_addr(str, _NODE, addr);     // node.point
        addr->net = main->net;
      }
    }
    if(point)
      macro_addr(point+1, _POINT, addr);
  }
  if(domain == NULL)
    domain = str+strlen(str);   // point at NUL char

  return(domain);
}


//  ------------------------------------------------------------------

static char* make_addr_str(char* str, Addr* addr, char* domain) {

  register char* ptr = str;
  static char buf[20];

  *ptr = NUL;

  if(addr->zone) {
    if(addr->zone == GFTN_ALL)
      ptr = stpcpy(ptr, "*");
    else
      ptr = stpcpy(ptr, itoa(addr->zone, buf, 10));
    *ptr++ = ':';
  }

  if(addr->net == GFTN_ALL)
    ptr = stpcpy(ptr, "*");
  else
    ptr = stpcpy(ptr, itoa(addr->net, buf, 10));
  *ptr++ = '/';

  if(addr->node == GFTN_ALL)
    ptr = stpcpy(ptr, "*");
  else
    ptr = stpcpy(ptr, itoa(addr->node, buf, 10));

  if(addr->point) {
    *ptr++ = '.';
    if(addr->point == GFTN_ALL)
      ptr = stpcpy(ptr, "*");
    else
      ptr = stpcpy(ptr, itoa(addr->point, buf, 10));
  }

  if(domain and *domain) {
    *ptr++ = '@';
    ptr = stpcpy(ptr, domain);
  }

  *ptr = NUL;

  return(str);
}


//  ------------------------------------------------------------------
//  Get a node

static _GEIdx* get_nlst(_GEIdx* aa, const long a) {

  register Nix A=(Nix)(a/1024);
  #if defined(__MSDOS__) and not defined(__386__)
  static int hdl, n;
  static _GEIdx AA, BB;
  static int lasthandle = 0, store = 0;
  static long lasta = -1, lastb = -1;
  #endif

  // Get a node

  switch(blks[A]->type) {

    case iDOS:
      return (_GEIdx*)(&blks[A]->na[blks[A]->nai[(uint)(a%1024)]]);

    #if defined(__MSDOS__) and not defined(__386__)
    case iEMS:
      hdl = (int)blks[A]->pos;
      if(hdl != lasthandle) {
        for(n=0; n<4; n++)
          emsmap(hdl, n, n);
        lasthandle = hdl;
      }
      emsread((byte*)aa, blks[A]->nai[(uint)(a%1024)], sizeof(_GEIdx));
      break;

    case iXMS:
      if(XMS.CopyFrom((char*)aa, blks[A]->nai[(uint)(a%1024)], sizeof(_GEIdx), (int)blks[A]->pos)) {
        fast_printf("Error copying from XMS block." fast_crlf "");
        fast_printf("XMS error %02X: \"%s\"." fast_crlf "",
          XMS.errorcode,
          XMS_msg[XMS.errorcode-0x80]
        );
      }
      break;

    case iDISK:
      if(a == lasta)
        memcpy(aa, &AA, sizeof(_GEIdx));   // Get it from the buffer
      else if(a == lastb)
        memcpy(aa, &BB, sizeof(_GEIdx));   // Get it from the buffer
      else {
        fseek(diskblk, blks[A]->pos+(long)(blks[A]->nai[(uint)(a%1024)]), SEEK_SET);
        fread(aa, sizeof(_GEIdx), 1, diskblk);
        store = !store;
        if(store) {
          memcpy(&AA, aa, sizeof(_GEIdx));         // Store in buffer also
          lasta = a;
        }
        else {
          memcpy(&BB, aa, sizeof(_GEIdx));         // Store in buffer also
          lastb = a;
        }
      }
      break;
    #endif
  }
  return aa;
}


//  ------------------------------------------------------------------
//  Compare two nodes by name/address/file/pos

static int cmp_nnlsts(const long a, const long b) {

  register int cmp;
  static _GEIdx* A;
  static _GEIdx* B;
  static _GEIdx AA, BB;

  A = get_nlst(&AA, a);
  B = get_nlst(&BB, b);

  if((cmp = gstricmp(A->name, B->name)) != 0)
    return(cmp);
  if((cmp = CmpV(A->addr.zone, B->addr.zone)) != 0)
    return(cmp);
  if((cmp = CmpV(A->addr.net, B->addr.net)) != 0)
    return(cmp);
  if((cmp = CmpV(A->addr.node, B->addr.node)) != 0)
    return(cmp);
  if((cmp = CmpV(A->addr.point, B->addr.point)) != 0)
    return(cmp);
  if((cmp = CmpV(A->pos, B->pos)) != 0)
    return(cmp);
  return(0);
}


//  ------------------------------------------------------------------
//  Compare two nodes by address/name/file/pos

static int cmp_anlsts(const long a, const long b) {

  register int cmp;
  static _GEIdx* A;
  static _GEIdx* B;
  static _GEIdx AA, BB;

  A = get_nlst(&AA, nidx[(uint)a]);
  B = get_nlst(&BB, nidx[(uint)b]);

  if((cmp = CmpV(A->addr.zone,B->addr.zone)) != 0)
    return(cmp);
  if((cmp = CmpV(A->addr.net, B->addr.net)) != 0)
    return(cmp);
  if((cmp = CmpV(A->addr.node, B->addr.node)) != 0)
    return(cmp);
  if((cmp = CmpV(A->addr.point, B->addr.point)) != 0)
    return(cmp);
  if((cmp = gstricmp(A->name, B->name)) != 0)
    return(cmp);
  if((cmp = CmpV(A->pos, B->pos)) != 0)
    return(cmp);
  return(0);
}


//  ------------------------------------------------------------------

static void nrqsort(Nix* base, long nelem, int(*fcmp)(const long,const long)) {

  static struct {
    long L, R;
  } stack[25];

  register long i, j, L, R, w, x;
  register int s, iteration = 0;

  s = 0;
  stack[s].L = 0;
  stack[s].R = nelem-1;

  do {
    L = stack[s].L;
    R = stack[s].R;
    s--;

    do {
      i = L;
      j = R;
      x = base[(uint)((L+R) >> 1)];

      do {
        while(fcmp(base[(uint)i], x) < 0)
          i++;
        while(fcmp(x, base[(uint)j]) < 0)
          j--;
        if(i <= j) {
          w = base[(uint)i];
          base[(uint)i] = base[(uint)j];
          base[(uint)j] = (Nix)w;
          i++;
          j--;
        }
      } while(i <= j);

      if(j-L < R-i) {
        if(i < R) {
          s++;
          stack[s].L = i;
          stack[s].R = R;
        }
        R = j;
      }
      else {
        if(L < j) {
          s++;
          stack[s].L = L;
          stack[s].R = j;
        }
        L = i;
      }

      if(ISTWIRLY(iteration))
        twirly();
      iteration++;

    } while(L < R);
  } while(s >= 0);
}


//  ------------------------------------------------------------------

static void sort_nodes() {

  char* name;
  _GEIdx* nlstp;
  char buf[256];
  register long nn, xx;
  FILE *fp, *fido;
  _GEIdx nlstz, nlst;

  #if defined(__MSDOS__) and not defined(__386__)
  // Report disk/memory usage
  //      12345678901234567890123456789012345678901234567890123456789012345678901234567890
  dword memdosused = memdos - coreleft();
  fast_printf("* Memory used: DOS %lu kb, XMS %u kb, EMS %u kb, DISK %lu bytes." fast_crlf "", memdosused/1024L, xmskb, emspages*16, fsize(diskblk));
  #endif

  // Prepare index
  nidx = (Nix*)blk.na;
  for(nn=0; nn<nodes; nn++)
    nidx[(uint)nn] = (Nix)nn;

  // Sort by name
  fast_printf("" fast_crlf "* Sorting by name ");
  nrqsort(nidx, nodes, cmp_nnlsts);

  // Write the name-sorted .GXN
  fp = fsopen(Nodelist->nodeindex, "wb", sh_mod);
  if(fp) {
    name = strrchr(Nodelist->nodeindex, GOLD_SLASH_CHR);
    if(name == NULL)
      name = Nodelist->nodeindex;
    else
      name++;   // Skip past the '\'
    fido = NULL;
    if(fidouser)
      fido = fsopen(fidouserlst, "wt", sh_mod);
    if(fido == NULL) {
      fast_printf("\b, writing %s ", name);
      fidouser = NO;
    }
    else {
      fast_printf("\b, writing %s and %s ", name, fidouserlst);
    }

    if(ignoredups)
      memset(&nlstz, 0, sizeof(_GEIdx));

    for(nn=xx=0; nn<nodes; nn++) {

      if(ISTWIRLY(nn))
        twirly();

      nlstp = get_nlst(&nlst, nidx[(uint)nn]);
      if(ignoredups) {
        if(match_addr_mask(&nlstp->addr, &nlstz.addr)) {
          if(strieql(nlstp->name, nlstz.name)) {
            // Ignore duplicate nodes
            aidx[(uint)nn] = (Nix)nn;    // Prepare for address sort
            iidx[(uint)nn] = 0xFFFF;      // Indicate ignored node
            dups++;
            continue;
          }
        }
        aidx[(uint)nn] = (Nix)nn;        // Prepare for address sort
        iidx[(uint)nn] = (Nix)(xx++);    // Prepare for address sort
        fwrite(nlstp, sizeof(_GEIdx), 1, fp);
        memcpy(&nlstz, nlstp, sizeof(_GEIdx));
      }
      else {
        aidx[(uint)nn] = (Nix)nn;        // Prepare for address sort
        iidx[(uint)nn] = (Nix)(xx++);    // Prepare for address sort
        fwrite(nlstp, sizeof(_GEIdx), 1, fp);
      }
      if(fidouser) {
        make_addr_str(buf, &nlstp->addr, "");
        fprintf(fido, "%-36.36s%24.24s\n", nlstp->name, buf);
      }
    }
    if(fido)
      fclose(fido);
    fclose(fp);
  }

  // Sort by address
  fast_printf(" " fast_crlf "* Sorting by node ");
  nrqsort(aidx, nodes, cmp_anlsts);

  // Write the address-sorted .GXA
  fp = fsopen(Nodelist->addrindex, "wb", sh_mod);
  if(fp) {
    name = strrchr(Nodelist->addrindex, GOLD_SLASH_CHR);
    if(name == NULL)
      name = Nodelist->addrindex;
    else
      name++;   // Skip past the '\'
    fast_printf("\b, writing %s ", name);
    for(nn=0; nn<nodes; nn++) {
      if(ISTWIRLY(nn))
        twirly();
      if(iidx[(uint)MKUL(aidx[(uint)nn])] != 0xFFFF)
        fwrite(&iidx[(uint)MKUL(aidx[(uint)nn])], sizeof(Nix), 1, fp);
    }
    fclose(fp);
  }

  // Write the list index in .GXL
  fp = fsopen(Nodelist->listindex, "wt", sh_mod);
  if(fp) {
    name = strrchr(Nodelist->listindex, GOLD_SLASH_CHR);
    if(name == NULL)
      name = Nodelist->listindex;
    else
      name++;   // Skip past the '\'
    fast_printf(" " fast_crlf "* Writing %s" fast_crlf "", name);
    for(int n=0; n<Nodelist->nodelists; n++) {
      if(*Nodelist->nodelist[n].fn)
        fprintf(fp, "%s %lu\n", Nodelist->nodelist[n].fn, Nodelist->nodelist[n].ft);
    }
    fclose(fp);
  }

  // Note compile time
  runtime = time(NULL) - runtime;

  if(dups) {
    fast_printf("" fast_crlf "* Total duplicate nodes: %u." fast_crlf "", dups);
  }
  fast_printf("" fast_crlf "* Nodelist compile completed. Compile time: %lu min, %lu sec." fast_crlf "", runtime/60, runtime%60);
}


//  ------------------------------------------------------------------

static char* CvtName(char* inp) {

  char buf[300];
  register char* p;
  register char* q;

  // Convert underlines to spaces

  p = inp;
  while(*p)
    if(*(p++) == '_')
      *(p-1) = ' ';

  // Strip leading spaces

  p = inp;
  while(isspace(*p))
    p++;
  q = &inp[strlen(p)-1];

  // Strip trailing spaces

  while(isspace(*q))
    *q-- = NUL;

  // Search for last space or point

  while(*q != ' ' and *q != '.' and q > p)
    q--;

  // If last char is a point, find last space instead

  if(*(q+1) == 0)
    while(*q != ' ' and q > p)
      q--;

  // Exchange last name and first name(s)

  if(p != q) {
    strcpy(buf, q+1);
    strcat(buf, ", ");
    *(q+(*q == '.' ? 1 : 0)) = 0;
    strcat(buf, p);
    strcpy(inp, buf);
  }
  struplow(inp);
  return inp;
}


//  ------------------------------------------------------------------

#ifdef GOLDNODE_STATS
void calc_statistic(FILE* ofp, int* observation, float N) {

  int i;
  float mean = 0.0;
  float sumfrekvens = 0.0;
  float varians = 0.0;

  //              12   12345   12345   123456   123456789012
  fprintf(ofp, ".---------------------------------------------.\n");
  fprintf(ofp, "|   x |  h(x) |  f(x) | x*f(x) | (x-m)^2*f(x) |\n");
  fprintf(ofp, "|-----+-------+-------+--------+--------------|\n");

  for(i=0; i<100; i++) {
    float x = i;
    if(observation[i]) {
      float hyppighed = observation[i];
      float frekvens = hyppighed / N;
      mean += x * frekvens;
      sumfrekvens += frekvens;
    }
  }

  for(i=0; i<100; i++) {
    float x = i;
    if(observation[i]) {
      float hyppighed = observation[i];
      float frekvens = hyppighed / N;
      float vartmp =  (x-mean)*(x-mean)*frekvens;
      varians += vartmp;
      fprintf(ofp, "| %3i | %5i | %5.3f | %6.3f | %12.3f | \n", i, observation[i], frekvens, x*frekvens, vartmp);
    }
  }

  fprintf(ofp, "|-----+-------+-------+--------+--------------|\n");
  fprintf(ofp, "| sum | %5.0f | %5.3f | %5.3f | %12.3f |\n", N, sumfrekvens, mean, varians);
  fprintf(ofp, "`---------------------------------------------'\n");
  fprintf(ofp, "\n");
  fprintf(ofp, "Mean: %.1f\n", mean);
  fprintf(ofp, "Variance = %.1f\n", varians);
  fprintf(ofp, "Standard deviation = %.1f\n", sqrt(varians));
  fprintf(ofp, "\n");
}
#endif


//  ------------------------------------------------------------------
//  some useful string operations

inline void index_line(char* p, char* ptrs[5]) {

  for(int i=0; i<5; i++) {
    char* q = p;
    while(*q != ',' and *q) {
      if(*q == '_')
        *q = ' ';
      q++;
    }
    if(*q) {
      ptrs[i] = p;
      *q++ = NUL;
      p = q;
    }
  }
}


//  ------------------------------------------------------------------
//  Read the nodelists and userlists

static void read_nodelists() {

  long pos;
  FILE* lfp;
  register char* ptr;
  _GEIdx nlst, nlstz;
  char buf[512], buf2[100];
  int point;
  uint fno;
  register uint n, line, len, no;
  char* name;
  char* lp[5];

  init_blk();

  // Open output file
  Path tmpfile;
  strcpy(tmpfile, AddPath(temppath, "goldnode.tix"));
  FILE* ofp = fsopen(tmpfile, "wb+", sh_mod);
  if(ofp == NULL) {
    fast_printf("Error Opening Indexfile %s!" fast_crlf "", tmpfile);
    return;
  }
  setvbuf(ofp, NULL, _IOFBF, 8192);

  fast_printf("" fast_crlf "* Compiling nodelists:" fast_crlf "");

  // Delete the current indexfiles so they don't take up space
  remove(Nodelist->addrindex);
  remove(Nodelist->nodeindex);

  // Compile nodelists
  for(fno=0; fno<Nodelist->nodelists; fno++) {

    lfp = fsopen(Nodelist->nodelist[fno].fn, "rb", sh_mod);
    if(lfp) {

      setvbuf(lfp, NULL, _IOFBF, 32000);
      Nodelist->nodelist[fno].ft = GetFiletime(Nodelist->nodelist[fno].fn);

      // Initialize for each nodelist file
      no = 0;
      pos = 0;
      line = 0;
      point = YES;
      memset(&nlst, 0, sizeof(_GEIdx));
      memset(&nlstz, 0, sizeof(_GEIdx));
      nlst.addr = Nodelist->nodezone[fno];
      nlstz.addr = Nodelist->nodezone[fno];
      name = strrchr(Nodelist->nodelist[fno].fn, GOLD_SLASH_CHR);
      if(name == NULL)
        name = Nodelist->nodelist[fno].fn;
      else
        name++;   // Skip past the '\'

      if(nodes < maxnodes) {

        // Read all nodes
        while(fgets(buf, sizeof(buf), lfp)) {
          line++;

          // Break out if eof-marker is found
          if(*buf == '\x1A')
            break;

          // Note file position
          nlst.pos = pos;

          // Get line length and fix possible errors
          uint llen = strlen(buf);
          ptr = buf+llen-1;
          while(llen and not (*ptr == '\r' or *ptr == '\n' or *ptr == '\x1A')) {
            buf[llen] = ' ';
            fast_printf("\r* |-%-12s     Warning line %u - Invalid NUL char encountered." fast_crlf "", name, line);
            llen = strlen(buf);
            ptr = buf+llen-1;
          }
          pos += llen;

          // Skip whitespace
          ptr = buf;
          while(isspace(*ptr))
            ptr++;

          if(*ptr != ';' and *ptr) {

            // First test for FD pvt extension
            if(toupper(*ptr) == 'B') {   // Boss
              nlst.addr.reset();
              parse_address(ptr+5, &nlst.addr, &nlstz.addr);
              point = YES;
              continue;
            }

            // Test for Goldware extension
            if(isdigit(*ptr)) {
              nlst.addr.reset();
              parse_address(ptr+5, &nlst.addr, &nlstz.addr);
              point = YES;
            }

            // Hold,32,TriCom,Hornbaek,Lars_Joergensen,45-12345678,2400,XX

            // Form the full node address
            index_line(ptr, lp);

            // NOTE: I use the fact that the third letter in lp[0] is unique
            //       for all valid attrs to speed up processing

            switch(*lp[0] ? toupper(lp[0][2]) : 0) {
              case 'N':   // zone
                nlst.addr.zone = nlst.addr.net = atow(lp[1]);
                nlst.addr.node = nlst.addr.point = 0;
                point = NO;
                break;

              case 'G':   // Region
                nlst.addr.net = atow(lp[1]);
                nlst.addr.node = nlst.addr.point = 0;
                point = NO;
                if(nlst.addr.net >= 10000)
                  continue;
                break;

              case 'S':   // Host
                {
                  nlst.addr.net = atow(lp[1]);
                  nlst.addr.node = nlst.addr.point = 0;
                  point = NO;
                  Addr a;
                  fast_parse_addr(lp[2],&a);
                  if(a.net) {                   // Is POINTS24 format ?
                    nlst.addr.net = a.net;
                    nlst.addr.node = a.node;
                    nlst.addr.point = 0;
                    point = YES;
                  }
                }
                break;

              case 'B':   // Hub
                nlst.addr.node = atow(lp[1]);
                nlst.addr.point = 0;
                point = NO;
                break;

              case 'I':   // point
                nlst.addr.point = atow(lp[1]);
                break;

              case 'T':   // Pvt
              case 'W':   // Down
              case 'L':   // Hold
              default:
                if(point)
                  nlst.addr.point = atow(lp[1]);
                else {
                  nlst.addr.node = atow(lp[1]);
                  nlst.addr.point = 0;
                }
                break;
            }

            if(ISTWIRLY(no)) {
              fast_printf("\r* \\--%-12s     Zone %-5u  Net %-5u  Nodes %u", name, nlst.addr.zone, nlst.addr.net, no);
            }

            // Check address against the exclude masks
            for(n=1; n<=Nodelist->excludenodes; n++) {
              if(match_addr_mask(&Nodelist->excludenode[n-1], &nlst.addr)) {
                n = 0;
                break;
              }
            }

            // Check address against the include masks
            if(not n) {
              for(n=1; n<=Nodelist->includenodes; n++) {
                if(match_addr_mask(&Nodelist->includenode[n-1], &nlst.addr)) {
                  n = 0;
                  break;
                }
              }
              n = !n;
            }

            if(n) {   // Address was okay

              // Convert name to Goldware standard
              strxcpy(nlst.name, CvtName(lp[4]), sizeof(nlst.name));

              // Prepare the rest
              nlst.pos |= ((((dword)fno) << 24) & 0xFF000000L);

              // Write record to output file
              fwrite(&nlst, sizeof(_GEIdx), 1, ofp);

              // Count the node
              nodes++;
              no++;

              // Stop if limit is reached
              if(nodes >= maxnodes)
                break;
            }
          }
        }
      }

      fast_printf("\r* %c--%-12s   Nodes read: %5u     Total read: %5u", fno+1==Nodelist->nodelists?'\\':'|', name, no, nodes);

      if(nodes >= maxnodes) {
        fast_printf("  (Limit reached)" fast_crlf "");
      }
      else {
        fast_printf("         " fast_crlf "");
      }

      fclose(lfp);
    }
    else {
      fast_printf("Error Opening nodelist %s!" fast_crlf "", Nodelist->nodelist[fno].fn);
      strcpy(Nodelist->nodelist[fno].fn,"\0");
    }
  }

  // Compile userlists
  if(userlists) {
    fast_printf("" fast_crlf "* Compiling userlists:" fast_crlf "");
  }

  for(fno=0; fno<userlists and nodes < maxnodes; fno++) {

    no = 0;

    lfp = fsopen(userlist[fno].fn, "rt", sh_mod);
    if(lfp) {

      setvbuf(lfp, NULL, _IOFBF, 32000);

      name = strrchr(userlist[fno].fn, GOLD_SLASH_CHR);
      if(name == NULL)
        name = userlist[fno].fn;
      else
        name++;   // Skip past the '\'

      if(nodes < maxnodes) {

        while(fgets(buf, sizeof(buf), lfp)) {

          // Get node data
          strbtrim(buf);
          ptr = buf + strlen(buf) - 1;
          while(*ptr != ' ')
            ptr--;
          memset(&nlst, 0, sizeof(_GEIdx));
          nlst.addr = userzone[fno];
          fast_parse_addr(ptr+1, &nlst.addr);
          *ptr = NUL;
          strbtrim(buf);

          // Convert "lastname, firstname" to "firstname lastname"
          ptr = strchr(buf, ',');
          if(ptr) {
            *ptr++ = NUL;
            strcpy(buf2, SkipWht(ptr));
            strcat(buf2, " ");
            strcat(buf2, buf);
            ptr = buf2;
          }
          else {
            ptr = buf;
          }

          // Convert name to Goldware standard
          strxcpy(nlst.name, CvtName(ptr), sizeof(nlst.name));

          // Check address against the exclude masks
          for(n=1; n<=Nodelist->excludenodes; n++) {
            if(match_addr_mask(&Nodelist->excludenode[n-1], &nlst.addr)) {
              n = 0;
              break;
            }
          }

          // Check address against the include masks
          if(not n) {
            for(n=1; n<=Nodelist->includenodes; n++) {
              if(match_addr_mask(&Nodelist->includenode[n-1], &nlst.addr)) {
                n = 0;
                break;
              }
            }
            n = !n;
          }

          if(n) {   // Address was okay

            if(ISTWIRLY(nodes))
              fast_printf("\r* \\--%s: %u", name, nodes);

            // Indicate userlist
            nlst.pos = (long)0xFFFFFFFFL;

            // Write record to output file
            fwrite(&nlst, sizeof(_GEIdx), 1, ofp);

            // Count the node
            nodes++;
            no++;

            // Stop if limit is reached
            if(nodes >= maxnodes)
              break;
          }
        }
      }

      fast_printf("\r* %c--%-12s     Nodes read: %5u     Total read: %5u", fno+1==userlists?'\\':'|', name, no, nodes);

      if(nodes >= maxnodes) {
        fast_printf("  (Limit reached)" fast_crlf "");
      }
      else {
        fast_printf("         " fast_crlf "");
      }

      fclose(lfp);
    }
    else {
      fast_printf("Error Opening Userlist %s!" fast_crlf "", userlist[fno].fn);
    }
  }

  #ifdef GOLDNODE_STATS
  if(not make_stats) {
  #endif

    // Pre-allocate memory for sorting indexes
    aidx = (Nix*)farcalloc(nodes+1, sizeof(Nix));
    iidx = (Nix*)farcalloc(nodes+1, sizeof(Nix));

    if(aidx == NULL or iidx == NULL) {
      farrelease(aidx);
      farrelease(iidx);
      fast_printf("" fast_crlf "" fast_crlf "Out of memory for index! [1]" fast_crlf "");
    }

    #if defined(__MSDOS__) and not defined(__386__)
    // Open disk block
    if(usedisk) {
      diskblk = fsopen(diskfile, "wb+", sh_mod);
      if(diskblk == NULL) {
        fast_printf("" fast_crlf "" fast_crlf "Can't open temporary file!" fast_crlf "");
        return;
      }
    }
    #endif

    // Allocate temporary block space (used later for index)
    if(((long)(1024+1)*(long)sizeof(_GEIdx)) > ((long)(nodes+1)*(long)sizeof(Nix)))
      blk.na = (byte __HUGE *)farcalloc(1024+1, sizeof(_GEIdx));
    else
      blk.na = (byte __HUGE *)farcalloc(nodes+1, sizeof(Nix));
    if(blk.na == NULL) {
      #if defined(__MSDOS__) and not defined(__386__)
      if(usedisk) {
        fclose(diskblk);
        remove(diskfile);
      }
      #endif
      fast_printf("" fast_crlf "" fast_crlf "Out of memory for index! [2]" fast_crlf "");
      return;
    }

    // Allocate blocks
    blocks = (int)((nodes / 1024) + ((nodes % 1024) ? 1 : 0));
    for(n=0; n<blocks; n++) {
      blks[n] = (NBlks*)calloc(1, sizeof(NBlks));
      if(blks[n] == NULL) {

        #if defined(__MSDOS__) and not defined(__386__)
        if(usedisk) {
          fclose(diskblk);
          remove(diskfile);
        }
        #endif

        farrelease(blk.na);
        for(; n; n--)
          release(blks[n-1]);
        fast_printf("" fast_crlf "" fast_crlf "Out of memory for index! [3]" fast_crlf "");
        return;
      }
    }

    rewind(ofp);

    if(nodes==0) {
      fast_printf(fast_crlf "Nothing to do!" fast_crlf);
      return;
    }

    fast_printf("" fast_crlf "* Reading unsorted nodelist index.");

    long totalnamelen = 0;

    // Read the node records for later sorting
    for(n=0; n<nodes; n++) {

      if(ISTWIRLY(n))
        twirly();

      fread(&nlst, sizeof(_GEIdx), 1, ofp);

      // Transfer the data to the blk
      blk.nai[blk.nais++] = blk.len;
      memcpy(&blk.na[blk.len], &nlst, sizeof(_GEIdx));
      len = strlen(nlst.name);
      totalnamelen += len;
      blk.len += (sizeof(_GEIdx) - maxname) + len;

      // Store block if filled
      if(((n+1)%1024) == 0) {
        if(not store_blk()) {
          fast_printf("" fast_crlf "* Warning! There was only memory enough to read %u out of %u nodes!\a", n-1024+1, nodes);
          nodes = (Nix)(n - 1024 + 1);
          break;
        }
      }
    }

    // Store block unless just stored
    if((n%1024) != 0)
      store_blk();

    fast_printf(" " fast_crlf "");
    fast_printf("* Total length of all names: %lu." fast_crlf "", totalnamelen);

  #ifdef GOLDNODE_STATS
  }
  #endif

  // Close and delete input file
  fclose(ofp);
  remove(tmpfile);

  // At last, sort the nodes
  #ifdef GOLDNODE_STATS
  if(not make_stats)
  #endif
    sort_nodes();

  #ifdef GOLDNODE_STATS
  if(make_stats) {

    fast_printf("* Writing statistics to %s" fast_crlf, statfilename);

    ofp = fopen(statfilename, "wt");
    if(ofp) {

      fprintf(ofp, "Nodename size statistics:\n");
      calc_statistic(ofp, statistic.nodename, nodes);

      fprintf(ofp, "\nLocation size statistics:\n");
      calc_statistic(ofp, statistic.location, nodes);

      fprintf(ofp, "\nSysopname size statistics:\n");
      calc_statistic(ofp, statistic.sysopname, nodes);

      fclose(ofp);
    }
  }
  #endif
}


//  ------------------------------------------------------------------

static void check_nodelists(int force) {

  FILE *fp;
  register uint n;
  int compilen, compileu;
  Path buf, newpath, newlist, oldlist;
  char drive[GMAXDRIVE], dir[GMAXDIR], file[GMAXFILE], ext[GMAXEXT];

  // Find newest nodelists
  for(n=0; n<Nodelist->nodelists; n++) {
    *oldlist = NUL;
    _splitpath(Nodelist->nodelist[n].fn, drive, dir, file, ext);
    if((atoi(ext+1) == 999) or (ext[1] == '*')) {
      strcpy(newpath, drive);
      strcat(newpath, dir);
      strcpy(ext, ".*");
      _makepath(buf, drive, dir, file, ext);
      gdir f;

      f.open(buf);
      if(f.first()) {
        do {
          _splitpath(f.name(), drive, dir, file, ext);
          if(atoi(ext+1)) {
            strcpy(newlist, newpath);
            AddBackslash(newlist);
            strcat(newlist, f.name());
            if((*oldlist != NUL) and (FiletimeCmp(oldlist, newlist) > 0))
              strcpy(newlist, oldlist);               // Keep the newest
            strcpy(oldlist, newlist);
            strcpy(Nodelist->nodelist[n].fn, newlist);
          }
        } while(f.next());
      }
      f.close();
    }
  }

  // Get timestamps from .GXL file
  fp = fsopen(Nodelist->listindex, "rt", sh_mod);
  if (fp) {
    while(fgets(buf, sizeof(buf), fp)) {
      char* key;
      char* val=buf;
      getkeyval(&key, &val);
      key = strxcpy(newpath, strbtrim(key), sizeof(Path));
      MakePathname(key, nodepath, key);
      for(n=0; n<Nodelist->nodelists; n++) {
        if (strieql(Nodelist->nodelist[n].fn,key))
        {
          Nodelist->nodelist[n].ft = atol(val);
          break;
        }
      }
      for(n=0; n<userlists; n++) {
        if (strieql(userlist[n].fn,key))
        {
          userlist[n].ft = atol(val);
          break;
        }
      }
    }
    fclose(fp);
  }

  // Check nodelists
  for(n=0,compilen=0; n<Nodelist->nodelists; n++) {
    if(GetFiletime(Nodelist->nodelist[n].fn) - Nodelist->nodelist[n].ft > 0) {
      Nodelist->nodelist[n].fc = YES;
      compilen++;
    }
  }

  if(compilen) {
    fast_printf("* %u new nodelist file%s found." fast_crlf "", compilen, compilen==1?"":"s");
  }
  else if(Nodelist->nodelists) {
    fast_printf("* The nodelist file%s up-to-date." fast_crlf "", Nodelist->nodelists==1?" is":"s are");
  }

  // Check userlists
  for(n=0,compileu=0; n<userlists; n++) {
    if(GetFiletime(userlist[n].fn) - userlist[n].ft > 0) {
      userlist[n].fc = YES;
      compileu++;
    }
  }

  if(compileu) {
    fast_printf("* %u new userlist file%s found." fast_crlf "", compileu, compileu==1?"":"s");
  }
  else if(userlists) {
    fast_printf("* The userlist file%s up-to-date." fast_crlf "", userlists==1?" is":"s are");
  }

  if(force or compilen or compileu) {

    read_nodelists();

    // De-initialize
    farrelease(aidx);
    farrelease(iidx);
    farrelease(nidx);

    #if defined(__MSDOS__) and not defined(__386__)
    if(usedisk) {
      if(diskblk) {
        fclose(diskblk);
        remove(diskfile);
      }
    }
    #endif

    for(n=0; n<blocks; n++) {

      if(blks[n]->type == iDOS)
        farrelease(blks[n]->na);

      #if defined(__MSDOS__) and not defined(__386__)
      if(blks[n]->type == iEMS)
        emsdealloc((int)blks[n]->pos);
      if(blks[n]->type == iXMS)
        XMS.Free((int)blks[n]->pos);
      #endif

      release(blks[n]);
    }
  }
}


//  ------------------------------------------------------------------

static void fatal_error(const char* what) {

  fast_printf(what);
  fast_printf(fast_crlf);
  exit(5);
}


//  ------------------------------------------------------------------

static int do_if(char* val) {

  if(strieql(val, "OS/2") OR strieql(val, "OS2")) {
    #ifdef __OS2__
    return TRUE;
    #else
    return FALSE;
    #endif
  }
  else if(strieql(val, "NT") OR strieql(val, "W32") OR strieql(val, "WIN32")) {
    #ifdef __WIN32__
    return TRUE;
    #else
    return FALSE;
    #endif
  }
  else if(strieql(val, "386")) {
    #if defined(__MSDOS__) and defined(__386__)
    return TRUE;
    #else
    return FALSE;
    #endif
  }
  else if(strieql(val, "DOS")) {
    #ifdef __MSDOS__
    return TRUE;
    #else
    return FALSE;
    #endif
  }
  else if(strieql(val, "LINUX") or strieql(val, "UNIX")) {
    #ifdef __UNIX__
    return TRUE;
    #else
    return FALSE;
    #endif
  }
  else if(strieql(val, "INOS2")) {
    #ifdef __OS2__
    return TRUE;
    #else
    return _osmajor >= 10;
    #endif
  }
  else if(strieql(val, "FIREBIRD")) {
    return TRUE;
  }
  else if(strieql(val, "YES") OR strieql(val, "TRUE") OR strieql(val, "ON"))
    return TRUE;
  return !!atoi(val);
}


//  ------------------------------------------------------------------

static int parse_config(const char *__configfile, Addr& zoneaddr) {

  FILE* fp;
  char buf[512];
  char* ptr;
  char* key;
  word crc;
  char* value;
  char* value2;
  int _gotcond,line=0;
  static int __FAR in_if = NO;
  static int __FAR in_else = NO;
  static int __FAR cond_status = YES;

  fp = fsopen(__configfile, "rt", sh_mod);
  if(fp) {
    while(fgets((ptr=buf), sizeof(buf), fp)) {
      line++;
      // Replace TABs with SPACEs
      strchg(ptr, '\t', ' ');
      fastSkipWht(ptr);
      if(*ptr != ';' and *ptr) {
        crc=getkeyvalcrc(&key, &ptr);
        getkeyval(&value, &ptr);
        getkeyval(&value2, &ptr);

        _gotcond = YES;
        switch(crc) {
          case CRC_IF:
            if(in_if) {
              fast_printf("* %s: Misplaced IF at line %u. IF's cannot be nested." fast_crlf "", __configfile, line);
            }
            in_if = YES;
            cond_status = do_if(value);
            break;
          case CRC_ELIF:
          case CRC_ELSEIF:
            if((not in_if) or in_else) {
              fast_printf("* %s: Misplaced ELIF/ELSEIF at line %u." fast_crlf "", __configfile, line);
            }
            cond_status = do_if(value);
            break;
          case CRC_ELSE:
            if((not in_if) or in_else) {
              fast_printf("* %s: Misplaced ELSE at line %u." fast_crlf "", __configfile, line);
            }
            in_else = YES;
            cond_status ^= YES;
            break;
          case CRC_ENDIF:
            if(not in_if) {
              fast_printf("* %s: Misplaced ENDIF at line %u." fast_crlf "", __configfile, line);
            }
            in_if = in_else = NO;
            cond_status = YES;
            break;
	  default:
            _gotcond = NO;
            break;
        }

        if((not _gotcond) and cond_status) {
          switch(crc) {
            case CRC_NODEPATH:
              AddBackslash(strcpy(nodepath, value));
              break;
            case CRC_TEMPPATH:
              AddBackslash(strcpy(temppath, value));
              break;
            case CRC_ADDRESS:
            case CRC_AKA:
              if(not zoneaddr.net) {
                parse_address(value, &zoneaddr, &zoneaddr);
                zoneaddr.point = 0;
              }
              break;
            case CRC_NODELIST:
              if(Nodelist->nodelists < NUM_NODELISTS) {
                if(atoi(value2)) {
                  parse_address(value2, &Nodelist->nodezone[Nodelist->nodelists], &Nodelist->nodezone[Nodelist->nodelists]);
                  if(Nodelist->nodezone[Nodelist->nodelists].zone == 0) {
                    Nodelist->nodezone[Nodelist->nodelists].zone  = Nodelist->nodezone[Nodelist->nodelists].node;
                    Nodelist->nodezone[Nodelist->nodelists].net   = 0;
                    Nodelist->nodezone[Nodelist->nodelists].node  = 0;
                  }
                }
                else
                  Nodelist->nodezone[Nodelist->nodelists] = zoneaddr;
                Nodelist->nodezone[Nodelist->nodelists].point = 0;
                Nodelist->nodelist[Nodelist->nodelists].ft = -1;
                Nodelist->nodelist[Nodelist->nodelists].fc = NO;
                strcpy(Nodelist->nodelist[Nodelist->nodelists++].fn, value);
              }
              else {
                fast_printf("* Error: too many NODELISTs (>%d) defined!" fast_crlf, NUM_NODELISTS);
              }
              break;
            case CRC_USERLIST:
              if(userlists < NUM_USERLISTS) {
                if(atoi(value2)) {
                  parse_address(value2, &userzone[userlists], &userzone[userlists]);
                  if(userzone[userlists].zone == 0) {
                    userzone[userlists].zone  = userzone[userlists].node;
                    userzone[userlists].net   = 0;
                    userzone[userlists].node  = 0;
                  }
                }
                else
                  userzone[userlists] = zoneaddr;
                userzone[userlists].point = 0;
                userlist[userlists].ft = -1;
                userlist[userlists].fc = NO;
                strcpy(userlist[userlists++].fn, value);
              }
              else {
                fast_printf("* Error: too many USERLISTs (>%d) defined!" fast_crlf, NUM_USERLISTS);
              }
              break;
            case CRC_EXCLUDENODES:
              parse_address(value, &Nodelist->excludenode[Nodelist->excludenodes++], &zoneaddr);
              break;
            case CRC_INCLUDENODES:
              parse_address(value, &Nodelist->includenode[Nodelist->includenodes++], &zoneaddr);
              break;
            case CRC_SHARE:
            case CRC_SHAREMODE:
              if(atoi(value))
                sh_mod = atoi(value);
              else if(striinc("NO", value))
                sh_mod = 0;
              break;
            case CRC_INCLUDE:
              if(not parse_config(value,zoneaddr))     // NOTE! This is a recursive call!
                fast_printf("* Could not read configuration file '%s' !" fast_crlf "",value);
              break;
	  default:
              break;
          }
        }
      }
    }

    fclose(fp);
    return(YES);
  }
  else {
    return(NO);
  }
}


//  ------------------------------------------------------------------

static int read_config() {

  Addr zoneaddr;
  Path buf;

  if(not *configfile) {
    #if defined(__OS2__)
    if(fexist("ged2.cfg"))
      strcpy(configfile, "ged2.cfg");
    else
    #elif defined(__WIN32__)
    if(fexist("gednt.cfg"))
      strcpy(configfile, "gednt.cfg");
    else
    #endif
      strcpy(configfile, "golded.cfg");
  }

  if(access(configfile, 0)) {
    char* ptr;
    ptr = getenv("GOLDNODE");   // Look for configfilename in the environment
    if(not (ptr and *ptr))
      ptr = getenv("GOLDED");   // Look for configfilename in the environment
    if(ptr and *ptr) {
      AddBackslash(strxcpy(buf, ptr, sizeof(Path)));
      MakePathname(configfile, buf, configfile);
    }
  }

  Nodelist->nodelists = 0;
  userlists = 0;
  if(not parse_config(configfile,zoneaddr)) {
    errorlevel = 1;
    return(NO);
  }

  if(Nodelist->nodelists == 0 and userlists == 0)
    fatal_error("* Error: No NODELISTs or USERLISTs defined!");

  if(zoneaddr.net == 0)
    fatal_error("* Error: No ADDRESS or AKAs defined!");

  if(*nodepath == NUL)
    AddBackslash(strcpy(nodepath, getcwd(buf, GMAXPATH)));
  if(*temppath == NUL)
    strcpy(temppath, nodepath);

  #if defined(__MSDOS__) and not defined(__386__)
  MakePathname(diskfile, temppath, diskfile);
  #endif

  MakePathname(Nodelist->listindex, nodepath, "goldnode.gxl");
  MakePathname(Nodelist->nodeindex, nodepath, "goldnode.gxn");
  MakePathname(Nodelist->addrindex, nodepath, "goldnode.gxa");
  register uint n;
  for(n=0; n<Nodelist->nodelists; n++)
    MakePathname(Nodelist->nodelist[n].fn, nodepath, Nodelist->nodelist[n].fn);
  for(n=0; n<userlists; n++)
    MakePathname(userlist[n].fn, nodepath, userlist[n].fn);

  return(YES);
}


//  ------------------------------------------------------------------

static void run_gn(int argc, char* argv[]) {

  register int n;
  char buf[256], gversion[20];
  char* ptr;
  char* versdate = __DATE__;
  int force=NO, conditional=NO;

  // Note start time
  runtime = time(NULL);

  // Set program version string for betas
  strcpy(gversion, VR);
  if(isalpha(VR[4])) {
    if(VR[4] == 'a') {
      sprintf(gversion, "%4.4s.%c%02u%02u",
        VR, toupper(VR[4]),
        str2mon(versdate), atoi(&versdate[4])
      );
    }
    else {
      sprintf(gversion, "%4.4s.%s", VR, RV);
    }
  }

  *configfile = NUL;

  for(n=1; n<argc; n++) {
    if(strchr("-/", *argv[n])) {
      ptr = argv[n]+2;
      if(*ptr == '=')
        ptr++;
      switch(toupper(argv[n][1])) {

        case 'C':
          conditional = YES;
          break;

        case 'D':
          ignoredups = YES;
          break;

        case 'F':
          force = YES;
          break;

        case 'M':
          maxnodes = (Nix)atol(ptr);
          if(maxnodes > 65500u)
            maxnodes = 65500u;
          break;

        #if defined(__MSDOS__) and not defined(__386__)
        case 'N':
          if(toupper(*ptr) == 'D')
            usedisk = YES;
          if(toupper(*ptr) == 'E')
            useems = NO;
          if(toupper(*ptr) == 'X')
            usexms = NO;
          break;
        #endif

        case 'Q':
          quiet = YES;
          break;

        case 'S':
          maxname = atoi(ptr);
          if(maxname > MAXNODENAME)
            maxname = MAXNODENAME;
          break;

        #ifdef GOLDNODE_STATS
        case 'T':
          make_stats = true;
          break;
        #endif

        case 'U':
          fidouser = YES;
          strupr(strcpy(fidouserlst, ptr));
          break;

      }
    }
    else {
      strcpy(configfile, argv[n]);
    }
  }

  #if defined(__BORLANDC__)
  if(not quiet)
    clrscr();
  #endif

  #if defined(__OS2__)
  sprintf(buf, "GoldNODE/2 %s Nodelist Compiler.", gversion);
  #elif defined(__WIN32__)
  sprintf(buf, "GoldNODE/W32 %s Nodelist Compiler.", gversion);
  #elif defined(__MSDOS__) and defined(__386__)
  sprintf(buf, "GoldNODE/386 %s Nodelist Compiler.", gversion);
  #elif defined(__MSDOS__)
  sprintf(buf, "GoldNODE %s Nodelist Compiler.", gversion);
  #elif defined(__linux__)
  sprintf(buf, "GoldNODE/LNX %s Nodelist Compiler.", gversion);
  #else
  #error Unknown operating system!
  #endif

  fast_printf(
    // 12345678901234567890123456789012345678901234567890123456789012345678901234567890
               "\r%-43.43s"                                 "Copyright (C) 1990-98 Odinn Sorensen"
    "" fast_crlf "-------------------------------------------------------------------------------"
    "" fast_crlf "",
    buf
  );

  if(not(force or conditional)) {
    fast_printf(

      #if defined(__OS2__)
      "" fast_crlf "Commandline syntax: GN2 [-options] [configfile]"
      #elif defined(__WIN32__)
      "" fast_crlf "Commandline syntax: GNW32 [-options] [configfile]"
      #elif defined(__MSDOS__) and defined(__386__)
      "" fast_crlf "Commandline syntax: GN386 [-options] [configfile]"
      #elif defined(__MSDOS__)
      "" fast_crlf "Commandline syntax: GOLDNODE [-options] [configfile]"
      #endif

      "" fast_crlf ""
      "" fast_crlf "[-options] =   -C        Conditional compile."
      "" fast_crlf "               -F        Forced compile."
      "" fast_crlf "               -D        Remove duplicate nodes from index while compiling."

      "" fast_crlf "               -M<nodes> Max number of nodes to read (max 65500)."

      #if defined(__MSDOS__) and not defined(__386__)
      "" fast_crlf "               -ND       Enable DISK usage. (NOT RECOMMENDED!!!)"
      "" fast_crlf "               -NE       Disable EMS usage."
      "" fast_crlf "               -NX       Disable XMS usage."
      #endif

      "" fast_crlf "               -Q        Quiet compile. No screen output improves speed."
      "" fast_crlf "               -S<size>  Set max size of a name in the index."
      "" fast_crlf "               -U<file>  Create sorted FIDOUSER.LST userlist file."
      #ifdef GOLDNODE_STATS
      "" fast_crlf "               -T        Make statistics."
      #endif
      "" fast_crlf ""
      "" fast_crlf "[configfile] =           The path AND filename of GOLDED.CFG"
      "" fast_crlf "                         configuration file to read."
      "" fast_crlf ""
    );
  }
  else {

    #if defined(__MSDOS__) and not defined(__386__)
    strcpy(diskfile, "GOLDNODE.$$$");
    memdos = coreleft();
    word memxms = XMS.FreeMem();
    word memems = emsexist() ? emsfree()*16 : 0;
    fast_printf("" fast_crlf "* Memory available: DOS %lu kb, XMS %u kb, EMS %u kb." fast_crlf "", memdos/1024L, memxms, memems);
    #elif defined(__MSDOS__) and defined(__WATCOMC__) and defined(__386__)
    i86 cpu;
    long mf[12];
    cpu.r.w.ax = 0x0500;
    cpu.zeroseg();
    cpu.r.x.es = FP_SEG(mf);
    cpu.r.x.edi = FP_OFF(mf);
    cpu.genintseg(0x31);
    fast_printf("" fast_crlf "* Memory available: %lu kb." fast_crlf "", *mf/1024L);
    #endif

    Nodelist = (NodeLix*)calloc(1, sizeof(NodeLix));
    if(Nodelist) {

      fast_printf("" fast_crlf "");

      if(force) {
        fast_printf("* Forced compile." fast_crlf "");
      }

      if(read_config()) {
        if(force or conditional)
          check_nodelists(force);
      }
      else {
        fast_printf("" fast_crlf "Could not find the configuration file!" fast_crlf "");
      }
      release(Nodelist);
    }
    else {
      fast_printf("" fast_crlf "Out of memory!" fast_crlf "");
    }
  }
}


//  ------------------------------------------------------------------

int _stdc main(int argc, char *argv[]) {

  #if defined(__BORLANDC__) and defined(__MSDOS__)
  set_new_handler(0);
  #endif

  #if defined(__DJGPP__)
  setvbuf(stdout, NULL, _IONBF, 0);
  #endif

  #if defined(__BORLANDC__)
  _setcursortype(_NOCURSOR);
  #endif

  #ifdef GOLDNODE_STATS
  memset(&statistic, 0, sizeof(nl_stat));
  #endif

  run_gn(argc, argv);

  #if defined(__BORLANDC__)
  _setcursortype(_NORMALCURSOR);
  #endif

  return errorlevel;
}


//  ------------------------------------------------------------------
//  Remove exception handling in Borland C++ 4.xx

#if defined(__MSDOS__) and (__BCPLUSPLUSC__ > 0x310)

extern "C" void _ExceptInit(void) {}

new_handler _new_handler = NULL;

new_handler set_new_handler(new_handler p) {

  new_handler t = _new_handler;
  _new_handler = p;
  return t;
}

void *operator new( size_t size ) {

  void * p = 0;

  size = size ? size : 1;
  while((p = malloc(size)) == NULL && _new_handler != NULL)
    _new_handler();

  return p;
}

#endif


//  ------------------------------------------------------------------

